/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef APPLICATIONSETTING_H
#define APPLICATIONSETTING_H

#include "constant.h"
#include <QObject>
#include <QVector>

/**
 * @brief テスト設定
 */
struct TestConfiguration {
    uint32_t Mode;              //!< テスト時のモード
};

/**
 * @brief ポーリング間隔
 */
struct PollingInterval {
    uint32_t SingleMeasurement;  //!< シングル測定データ取得のポーリング間隔
    uint32_t MultiMeasurement;   //!< マルチ測定データ取得のポーリング間隔
    uint32_t Constellation;      //!< コンスタレーションデータ取得のポーリング間隔
    uint32_t DelayProfile;       //!< 遅延プロファイルデータ取得のポーリング間隔
    uint32_t KeyInput;           //!< キー(ボタン)入力状態取得間隔
    uint32_t DeviceStatus;       //!< デバイス(方位・バッテリ・ストレージ)状態取得間隔
    uint32_t Scan;               //!< スキャン結果取得のポーリング間隔
    uint32_t OpticalMeasurement; //!< 光測定のポーリング間隔
    uint32_t DCMeasurement;      //!< DC測定のポーリング間隔
    uint32_t FailureDetection;   //!< 故障状態取得のポーリング間隔
    uint32_t Compass;            //!< コンパス画面でのデバイス(方位・バッテリ・ストレージ)状態取得間隔
    uint32_t OneChannel;         //!< マルチ測定/データロガーで 1 チャンネルしか測定しない場合のポーリング間隔
};

/**
 * @brief スタブデータを格納するパス
 */
struct StubDataPath {
    QString SingleMeasurement;  //!< シングル測定でスタブ使用時に取得するデータを持つファイルのパス
    QString MultiMeasurement;   //!< マルチ測定でスタブ使用時に取得するデータを持つファイルのパス
    QString DataLogger;         //!< データロガーでスタブ使用時に参照するファイルのパス
};

/**
 * @brief 測定結果を格納するパス
 */
struct ResultDataPath {
    QString Single;             //!< シングル測定結果を格納するディレクトリのパス
    QString Multi;              //!< マルチ測定結果を格納するディレクトリのパス
    QString List;               //!< ユーザー測定結果を格納するディレクトリのパス
    QString Logger;             //!< ロガー測定結果を格納するディレクトリのパス
    QString SingleCopy;         //!< シングル測定結果をコピーするディレクトリのパス
    QString MultiCopy;          //!< マルチ測定結果をコピーするディレクトリのパス
    QString LoggerCopy;         //!< データロガー測定結果をコピーするディレクトリのパス
};

/**
 * @brief 設定ファイルを格納するパス
 */
struct SettingDataPath {
    QString UserData;                   //!< ユーザ設定データを格納するディレクトリのパス
    QString UserSettingFileName;        //!< ユーザ設定ファイルリストのファイル名
    QString UserDataUsb1;               //!< ユーザUSB1設定データを格納するディレクトリのパス
    QString UserSettingUsb1FileName;    //!< ユーザUSB1設定ファイルリストのファイル名
    QString UserDataUsb2;               //!< ユーザUSB2設定データを格納するディレクトリのパス
    QString UserSettingUsb2FileName;    //!< ユーザUSB2設定ファイルリストのファイル名
    QString Measure;                    //!< 測定設定データを格納するディレクトリのパス
    QString Logger;                     //!< データロガー設定データを格納するディレクトリのパス
    QString LoggerSettingFileName;      //!< データロガー設定ファイルリストのファイル名
    QString LoggerUsb1;                 //!< データロガーUSB1設定データを格納するディレクトリのパス
    QString LoggerSettingUsb1FileName;  //!< データロガーUSB1設定ファイルリストのファイル名
    QString LoggerUsb2;                 //!< データロガーUSB2設定データを格納するディレクトリのパス
    QString LoggerSettingUsb2FileName;  //!< データロガーUSB2設定ファイルリストのファイル名
    QString StubLogger;                 //!< スタブ用データロガー設定データを格納するディレクトリのパス
};

/**
 * @brief 一回のラッパー関数の呼び出しでコンスタレーションのデータを読み出す点数
 * @note 測定設定のコンスタレーションの全読み出し点数とは異なる
 */
struct ConstellationTermOfUse {
    uint16_t NumberOfPoints;    //!< 一回の API 呼び出しで読み込む点数
};

/**
 * @brief チャンネル定義ファイルのパス
 */
struct ChannelDefinitionPath {
    QString Data;               //!< チャンネル定義ファイルのパス
};

/**
 * @brief アニメーション設定
 */
struct AnimationSetting {
    uint32_t meterUpdateInterval;   //!< メーターの針移動間隔
    uint32_t invalidValudSpeed;    //!< 入力無効時の針移動速度倍率
};

/**
 * @brief 長押し設定
 */
struct LongPressSetting {
    uint32_t firstRepeatTime;       //!< 初回の長押し検知時間
    uint32_t lowRepeatCount;        //!< 低速押下継続回数
    uint32_t repeatIntervalLow;     //!< 長押しの押下間隔(低速)
    uint32_t repeatIntervalHigh;    //!< 長押しの押下間隔(高速)
    uint32_t delayProfileSkipIndexLow;      //!< 遅延プロファイル内での長押し(低速)時に一回の押下で進むインデックス
    uint32_t delayProfileSkipIndexHigh;     //!< 遅延プロファイル内での長押し(高速)時に一回の押下で進むインデックス
    uint32_t lowRepeatCountForFrequency;        //!< 低速押下継続回数(周波数変更用)
    uint32_t repeatIntervalLowForFrequency;     //!< 長押しの押下間隔(周波数変更用低速)
    uint32_t repeatIntervalHighForFrequency;    //!< 長押しの押下間隔(周波数変更用高速)
};

/**
 * @brief 遅延設定
 */
struct DelaySetting {
    uint32_t channelChangeDelay;    //!< チャンネル確定 遅延時間
};

/**
 * @brief ウェイクアップ設定
 */
struct WakeUpSetting {
    uint32_t waitingTimeForSub;     //!< サブ起動までの待ち時間
};

/**
 * @brief シングル測定の測定結果を表示に反映するまでの待ち時間の設定
 */
struct FixTimeSetting {
    uint32_t level;                 //!< Level
    uint32_t mer;                   //!< MER
    uint32_t ber;                   //!< BER
};

/**
 * @brief zip ファイル作成時のパスワード
 */
struct ZipPassword {
    QString Password;               //!< パスワード
};

/**
 * @brief ロック待ち時間
 */
struct WaitForLocking {
    uint32_t Meter;                 //!< メーター
    uint32_t Constellation;         //!< コンスタレーション
    uint32_t DelayProfile;          //!< 遅延プロファイル
    uint32_t PrefferedChannel;      //!< マルチ優先測定
};

/**
 * @brief 点滅間隔
 */
struct Blinking {
    uint32_t Interval;              //!< 点滅間隔
};

/**
 * @brief 測定データファイルリスト
 */
struct MeasureDataFileList {
    QString Path;                   //!< ファイルのパス
    QString FileName;               //!< ファイル名
};

/**
 * @brief 実行ファイル固有の設定ファイル管理クラス
 */
class ApplicationSetting : public QObject
{
    Q_OBJECT
public:
    /**
     * @brief コンストラクタ
     * @param settingFilePath 実行ファイル固有の設定ファイルファイルパス
     * @param parent 親オブジェクト
     */
    explicit ApplicationSetting(const QString &settingFilePath, QObject *parent = NULL);

    /**
     * @brief デストラクタ
     */
    ~ApplicationSetting();

    /**
     * @brief テスト時のモード取得
     * @return テスト時のモード
     * @note 0: ラッパー関数呼び出し 1: スタブ
     */
    ActionMode testMode(void) const
    {
        return static_cast<ActionMode>(_testConfiguration.Mode);
    }

    /**
     * @brief シングル測定データ取得のポーリング間隔取得
     * @return シングル測定データ取得のポーリング間隔
     */
    uint32_t pollingIntervalSingleMeasurement(void) const
    {
        return _pollingInterval.SingleMeasurement;
    }

    /**
     * @brief マルチ測定データ取得のポーリング間隔取得
     * @return マルチ測定データ取得のポーリング間隔
     */
    uint32_t pollingIntervalMultiMeasurement(void) const;

    /**
     * @brief コンスタレーションデータ取得のポーリング間隔取得
     * @return コンスタレーションデータ取得のポーリング間隔
     */
    uint32_t pollingIntervalConstellation(void) const
    {
        return _pollingInterval.Constellation;
    }

    /**
     * @brief 遅延プロファイルデータ取得のポーリング間隔取得
     * @return 遅延プロファイルデータ取得のポーリング間隔
     */
    uint32_t pollingIntervalDelayProfile(void) const
    {
        return _pollingInterval.DelayProfile;
    }

    /**
     * @brief キー入力取得のポーリング間隔取得
     * @return キー入力取得のポーリング間隔
     */
    uint32_t pollingIntervalKeyInput(void) const
    {
        return _pollingInterval.KeyInput;
    }

    /**
     * @brief デバイス状態取得のポーリング間隔取得
     * @return デバイス状態取得のポーリング間隔
     */
    uint32_t pollingIntervalDeviceStatus(void) const
    {
        return _pollingInterval.DeviceStatus;
    }

    /**
     * @brief スキャン結果取得のポーリング間隔取得
     * @return スキャン結果取得のポーリング間隔
     */
    uint32_t pollingIntervalScan(void) const
    {
        return _pollingInterval.Scan;
    }

    /**
     * @brief 光測定の結果取得ポーリング間隔取得
     * @return 光測定の結果取得ポーリング間隔 (QTimerのTimeOut間隔)
     */
    uint32_t pollingIntervalOpticalMeasurement(void) const
    {
        return _pollingInterval.OpticalMeasurement;
    }

    /**
     * @brief DC測定の結果取得ポーリング間隔取得
     * @return DC測定の結果取得ポーリング間隔 (QTimerのTimeOut間隔)
     */
    uint32_t pollingIntervalDCMeasurement(void) const
    {
        return _pollingInterval.DCMeasurement;
    }

    /**
     * @brief 故障状態取得のポーリング間隔取得
     * @return 故障状態取得のポーリング間隔
     */
    uint32_t pollingIntervalFailureDetection(void) const
    {
        return _pollingInterval.FailureDetection;
    }

    /**
     * @brief コンパス画面でのデバイス状態取得のポーリング間隔取得
     * @return コンパス画面でのデバイス状態取得のポーリング間隔
     */
    uint32_t pollingIntervalCompass(void) const
    {
        return _pollingInterval.Compass;
    }

    /**
     * @brief マルチ測定/データロガーで 1 チャンネルしか測定しない場合のポーリング間隔取得
     * @return マルチ測定/データロガーで 1 チャンネルしか測定しない場合のポーリング間隔
     */
    uint32_t pollingIntervalOneChannel(void) const
    {
        return _pollingInterval.OneChannel;
    }

    /**
     * @brief  シングル測定でスタブ使用時に取得するデータを持つファイルのパス取得
     * @return シングル測定でスタブ使用時に取得するデータを持つファイルのパス
     */
    QString stubDataPathSingleMeasurement(void) const
    {
        return _stubDataPath.SingleMeasurement ;
    }

    /**
     * @brief マルチ測定でスタブ使用時に取得するデータを持つファイルのパス取得
     * @return マルチ測定でスタブ使用時に取得するデータを持つファイルのパス
     */
    QString stubDataPathMultiMeasurement(void) const
    {
        return _stubDataPath.MultiMeasurement;
    }

    QString stubDataPathDataLogger(void) const
    {
        return _stubDataPath.DataLogger;
    }

    /**
     * @brief シングル測定結果を格納するパスを返す
     * @return シングル測定結果を格納するパス
     */
    QString resultDataPathSingle(void) const
    {
        return _resultDataPath.Single;
    }

    /**
     * @brief シングル測定結果をコピーするパスを返す
     * @return シングル測定結果をコピーするパス
     */
    QString resultDataCopyPathSingle(void) const
    {
        return _resultDataPath.SingleCopy;
    }

    /**
     * @brief マルチ測定結果を格納するパスを返す
     * @return マルチ測定結果を格納するパス
     */
    QString resultDataPathMulti(void) const
    {
        return _resultDataPath.Multi;
    }

    /**
     * @brief マルチ測定結果をコピーするパスを返す
     * @return マルチ測定結果をコピーするパス
     */
    QString resultDataCopyPathMulti(void) const
    {
        return _resultDataPath.MultiCopy;
    }

    /**
     * @brief ユーザー設定測定結果を格納するパスを返す
     * @return ユーザー設定測定結果を格納するパス
     */
    QString resultDataPathList(void) const
    {
        return _resultDataPath.List;
    }

    /**
     * @brief ロガー測定結果を格納するパスを返す
     * @return ロガー測定結果を格納するパス
     */
    QString resultDataPathLogger(void) const
    {
        return _resultDataPath.Logger;
    }

    /**
     * @brief ロガー測定結果をコピーするパスを返す
     * @return ロガー測定結果をコピーするパス
     */
    QString resultDataCopyPathLogger(void) const
    {
        return _resultDataPath.LoggerCopy;
    }

    /**
     * @brief ユーザー設定データを格納するパスを返す
     * @return ユーザー設定データのパス
     */
    QString settingDataPathUserData() const
    {
        return _settingDataPath.UserData;
    }

    /**
     * @brief ユーザー設定ファイルリストのファイル名を返す
     * @return ユーザー設定ファイルリストのファイル名
     */
    QString settingDataUserSettingFileListName() const
    {
        return _settingDataPath.UserSettingFileName;
    }

    /**
     * @brief USB1設定データを格納するパスを返す
     * @return USB1設定データのパス
     */
    QString settingDataPathUserDataUsb1() const
    {
        return _settingDataPath.UserDataUsb1;
    }

    /**
     * @brief USB1設定ファイルリストのファイル名を返す
     * @return USB1設定ファイルリストのファイル名
     */
    QString settingDataUserSettingUsb1FileListName() const
    {
        return _settingDataPath.UserSettingUsb1FileName;
    }

    /**
     * @brief USB2設定データを格納するパスを返す
     * @return USB2設定データのパス
     */
    QString settingDataPathUserDataUsb2() const
    {
        return _settingDataPath.UserDataUsb2;
    }

    /**
     * @brief USB2設定ファイルリストのファイル名を返す
     * @return USB2設定ファイルリストのファイル名
     */
    QString settingDataUserSettingUsb2FileListName() const
    {
        return _settingDataPath.UserSettingUsb2FileName;
    }

    /**
     * @brief 測定設定データを格納するパスを返す
     * @return 測定設定データ
     */
    QString settingDataPathMeasure(void) const
    {
        return _settingDataPath.Measure;
    }

    /**
     * @brief データロガー設定データを格納するパスを返す
     * @return データロガー設定データ
     */
    QString settingDataPathLogger(void) const
    {
        return _settingDataPath.Logger;
    }

    /**
     * @brief データロガー設定ファイルリストのファイル名を返す
     * @return データロガー設定ファイルリストのファイル名
     */
    QString settingDataLoggerSettingFileListName() const
    {
        return _settingDataPath.LoggerSettingFileName;
    }

    /**
     * @brief データロガーUSB1設定データを格納するパスを返す
     * @return データロガーUSB1設定データ
     */
    QString settingDataPathLoggerUsb1(void) const
    {
        return _settingDataPath.LoggerUsb1;
    }

    /**
     * @brief データロガーUSB1設定ファイルリストのファイル名を返す
     * @return データロガーUSB1設定ファイルリストのファイル名
     */
    QString settingDataLoggerSettingUsb1FileListName() const
    {
        return _settingDataPath.LoggerSettingUsb1FileName;
    }

    /**
     * @brief データロガーUSB2設定データを格納するパスを返す
     * @return データロガーUSB2設定データ
     */
    QString settingDataPathLoggerUsb2(void) const
    {
        return _settingDataPath.LoggerUsb2;
    }

    /**
     * @brief データロガーUSB2設定ファイルリストのファイル名を返す
     * @return データロガーUSB2設定ファイルリストのファイル名
     */
    QString settingDataLoggerSettingUsb2FileListName() const
    {
        return _settingDataPath.LoggerSettingUsb2FileName;
    }

    /**
     * @brief スタブ用データロガー設定データを格納するパスを返す
     * @return スタブ用データロガー設定データ
     */
    QString settingDataPathStubLogger(void) const
    {
        return _settingDataPath.StubLogger;
    }

    /**
     * @brief 一回のラッパー関数の呼び出しでコンスタレーションのデータを読み出す点数を返す
     * @return 一回のラッパー関数の呼び出しでコンスタレーションのデータを読み出す点数
     */
    uint32_t constellationTermOfUseNumberOfPoints(void)
    {
        return _constellationTermOfUse.NumberOfPoints;
    }

    /**
     * @brief チャンネルを定義した CSV ファイルを格納するパスを返す
     * @return チャンネルを定義した CSV ファイルを格納するパス
     */
    QString channelDefinitionData(void)
    {
        return _channelDefinition.Data;
    }

    /**
     * @brief メーターの針移動間隔を返す
     * @return 針移動時間(ms)
     */
    uint32_t meterUpdateInterval(void)
    {
        return _animationSetting.meterUpdateInterval;
    }

    /**
     * @brief 入力無効時の針移動速度倍率
     * @return 針移動速度倍率
     */
    uint32_t invalidValueMeterSpeed(void)
    {
        return _animationSetting.invalidValudSpeed;
    }

    /**
     * @brief 初回の長押し検知時間を返す
     * @return 初回の長押し検知時間(ms)
     */
    uint32_t firstRepeatTime(void)
    {
        return _longPressSetting.firstRepeatTime;
    }

    /**
     * @brief 低速押下継続回数を返す
     * @return 低速押下継続回数
     */
    uint32_t lowRepeatCount(void)
    {
        return _longPressSetting.lowRepeatCount;
    }

    /**
     * @brief 長押し時の押下間隔(低速)を返す
     * @return 長押し時の押下間隔(低速)(ms)
     */
    uint32_t repeatIntervalLow(void)
    {
        return _longPressSetting.repeatIntervalLow;
    }

    /**
     * @brief 長押し時の押下間隔(高速)を返す
     * @return 長押し時の押下間隔(高速)(ms)
     */
    uint32_t repeatIntervalHigh(void)
    {
        return _longPressSetting.repeatIntervalHigh;
    }

    /**
     * @brief 遅延プロファイル内での長押し(低速)時に一回の押下で進むインデックスを返す
     * @return 遅延プロファイル内での長押し(低速)時に一回の押下で進むインデックス
     */
    uint32_t delayProfileSkipIndexLow(void) const
    {
        return _longPressSetting.delayProfileSkipIndexLow;
    }

    /**
     * @brief 遅延プロファイル内での長押し(高速)時に一回の押下で進むインデックスを返す
     * @return 遅延プロファイル内での長押し(高速)時に一回の押下で進むインデックス
     */
    uint32_t delayProfileSkipIndexHigh(void) const
    {
        return _longPressSetting.delayProfileSkipIndexHigh;
    }

    /**
     * @brief 低速押下継続回数を返す(周波数変更用)
     * @return 低速押下継続回数
     */
    uint32_t lowRepeatCountForFrequency(void)
    {
        return _longPressSetting.lowRepeatCountForFrequency;
    }

    /**
     * @brief 長押し時の押下間隔(周波数変更用低速)を返す
     * @return 長押し時の押下間隔(周波数変更用低速)(ms)
     */
    uint32_t repeatIntervalLowForFrequency(void)
    {
        return _longPressSetting.repeatIntervalLowForFrequency;
    }

    /**
     * @brief 長押し時の押下間隔(周波数変更用高速)を返す
     * @return 長押し時の押下間隔(周波数変更用高速)(ms)
     */
    uint32_t repeatIntervalHighForFrequency(void)
    {
        return _longPressSetting.repeatIntervalHighForFrequency;
    }

    /**
     * @brief チャンネル確定遅延時間を返す
     * @return チャンネル確定遅延時間(ms)
     */
    uint32_t channelChangeDelay(void)
    {
        return _delaySetting.channelChangeDelay;
    }

    /**
     * @brief サブ起動までの待ち時間を返す
     * @return サブ起動までの待ち時間
     */
    uint32_t waitingTimeForSub(void) const
    {
        return _wakeUpSetting.waitingTimeForSub;
    }

    /**
     * @brief シングル測定の Level を表示に反映するまでの待ち時間を返す
     * @return シングル測定の Level を表示に反映するまでの待ち時間
     */
    uint32_t levelFixTime(void) const
    {
        return _fixTimeSetting.level;
    }

    /**
     * @brief シングル測定の MER を表示に反映するまでの待ち時間を返す
     * @return シングル測定の MER を表示に反映するまでの待ち時間
     */
    uint32_t merFixTime(void) const
    {
        return _fixTimeSetting.mer;
    }

    /**
     * @brief シングル測定の BER を表示に反映するまでの待ち時間を返す
     * @return シングル測定の BER を表示に反映するまでの待ち時間
     */
    uint32_t berFixTime(void) const
    {
        return _fixTimeSetting.ber;
    }

    /**
     * @brief zip ファイル作成/展開時のパスワードを返す
     * @return パスワード
     */
    QString password(void) const
    {
        return _zipPassword.Password;
    }

    /**
     * @brief シングル測定(メーター)のロック待ち時間を返す
     * @return シングル測定(メーター)のロック待ち時間
     */
    uint32_t waitingTimeForMeter(void) const
    {
        return _waitForLocking.Meter;
    }

    /**
     * @brief シングル測定(コンスタレーション)のロック待ち時間を返す
     * @return シングル測定(コンスタレーション)のロック待ち時間
     */
    uint32_t waitingTimeForConstellation(void) const
    {
        return _waitForLocking.Constellation;
    }

    /**
     * @brief シングル測定(遅延プロファイル)のロック待ち時間を返す
     * @return シングル測定(遅延プロファイル)のロック待ち時間
     */
    uint32_t waitingTimeForDelayProfile(void) const
    {
        return _waitForLocking.DelayProfile;
    }

    /**
     * @brief マルチ優先測定のロック待ち時間を返す
     * @return マルチ優先測定のロック待ち時間
     */
    uint32_t waitingTimeForPrefferedChannel(void) const
    {
        return _waitForLocking.PrefferedChannel;
    }

    /**
     * @brief 点滅間隔を返す
     * @return 点滅間隔
     */
    uint32_t blinkInterval(void) const
    {
        return _blinking.Interval;
    }

    /**
     * @brief 設定値の有効判定
     * @param invalidDataName 無効時の内容（最初に検出した設定異常箇所）
     * @return true:有効、false:無効
     */
    bool isValidSetting(QString &invalidDataName);

    /**
     * @brief 測定データファイルリストのパスを返す
     * @return 測定データファイルリストのパス
     */
    QString measureDataFileListPath(void) const
    {
        return _measureDataFileList.Path;
    }

    /**
     * @brief 測定データファイルリストのファイル名を返す
     * @return 測定データファイルリストのファイル名
     */
    QString measureDataFileListFileName(void) const
    {
        return _measureDataFileList.FileName;
    }

private:
    QString _settingFilePath;                           //!< 実行ファイル固有の設定ファイルファイルパス
    TestConfiguration _testConfiguration;               //!< テスト設定
    PollingInterval _pollingInterval;                   //!< ポーリング間隔
    StubDataPath _stubDataPath;                         //!< スタブデータパス
    ResultDataPath _resultDataPath;                     //!< 測定結果を格納するパス
    SettingDataPath _settingDataPath;                   //!< 設定ファイルを格納するパス
    ConstellationTermOfUse
    _constellationTermOfUse;     //!< 一回のラッパー関数の呼び出しでコンスタレーションのデータを読み出す点数
    ChannelDefinitionPath _channelDefinition;           //!< チャンネル定義ファイルのパス
    AnimationSetting _animationSetting;                 //!< アニメーション設定
    LongPressSetting _longPressSetting;                 //!< 長押し設定
    DelaySetting _delaySetting;                         //!< 遅延時間設定
    WakeUpSetting _wakeUpSetting;                       //!< ウェイクアップ設定
    FixTimeSetting _fixTimeSetting;                     //!< シングル測定の測定結果を表示に反映するまでの待ち時間の設定
    ZipPassword _zipPassword;                           //!< zip ファイル作成時のパスワード
    WaitForLocking _waitForLocking;                     //!< ロック待ち時間
    Blinking _blinking;                                 //!< 点滅間隔
    MeasureDataFileList _measureDataFileList;           //!< 測定データファイルリスト

    /**
     * @brief 初期化
     */
    void initialize(void);
};

#endif // APPLICATIONSETTING_H
