/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include "levelchecker.h"
#include <QElapsedTimer>

LevelChecker::LevelChecker(QQmlApplicationEngine *engine, QObject *parent)
    : QObject(parent)
    , _engine(engine)
    , _model(nullptr)
    , _viewModel(nullptr)
    , _window(nullptr)
{
    QFile file;
    file.setFileName(QString(STANDBY_LOCK_FILE_NAME));
    if (QFile::exists(QString(STANDBY_LOCK_FILE_NAME))) {
        if (!file.remove()) {
            qErrnoWarning("standby file exits.");
        }
    }
}

LC70_ERR_T LevelChecker::prepare(QGuiApplication *application)
{
    QElapsedTimer timer;
    timer.start();

    LC70_ERR_T ret = LC70_OK;
    bool connected = true;
    const QUrl url(QStringLiteral("qrc:/view/MainFrame.qml"));
    QObject::connect(_engine, &QQmlApplicationEngine::objectCreated,
    application, [url, &connected](QObject * obj, const QUrl & objUrl) {
        if (!obj && url == objUrl) {
            connected = false;
        }
    }, Qt::QueuedConnection);

    // MainFrame.qml のインスタンスが生成されなかった場合は終了する
    if (!connected) {
        return LC70_ERR;
    }

    // Model クラスの生成
    _model = new LevelCheckerModel(_engine, this);
    // ラッパー関数を呼び出すかスタブを使うか
    ret = _model->prepare(&timer);

    if (ret != LC70_OK) {
        return ret;
    }

    // MainFrame のロード
    _engine->load(url);

    // QML で作成された画面を表示し、アプリケーションを開始
    QObject *topLevel = _engine->rootObjects().value(0);
    _window = qobject_cast<QQuickWindow *>(topLevel);

    // 初期画面遷移
    _viewModel = new LevelCheckerViewModel(_engine, _model, _window, &timer, this);

    // 画面表示
    if (_window) {
        _window->show();
        _model->writeLog(QString(typeid(*this).name()),
                         QString("[START UP] the screen is displayed %1 ms").arg(timer.elapsed()));
    }

    return ret;
}

LevelChecker::~LevelChecker()
{

}
