/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include "correcter.h"
#include <QDebug>

extern "C" {
#include "liblc70.h"
}

static const QMap<QString, uint32_t> MODULATION_VALUE_MAP = {
    {OFDM,                      LC70_SETMOD_OFDM},
    {BPSK,                      LC70_SETMOD_BPSK},
    {QPSK,                      LC70_SETMOD_QPSK},
    {TC8PSK,                    LC70_SETMOD_TC8PSK},
    {THIRTY_TWO_APSK,           LC70_SETMOD_32APSK},
    {SIXTEEN_APSK,              LC70_SETMOD_16APSK},
    {EIGHT_PSK,                 LC70_SETMOD_8PSK},
    {SIXTY_FOUR_QAM,            LC70_SETMOD_64QAM},
    {TWO_HUNDRED_FIFTY_SIX_QAM, LC70_SETMOD_256QAM},
    {CW,                        LC70_SETMOD_CW},
    {FM,                        LC70_SETMOD_FM},
};

static const QMap<LC70_BAND_T, QPair<QString, QString>> BAND_MAP = {
    // Band の ID                    短い名称                                        長い名称
    {LC70_BAND_UHF,                 {QString("地デジ"),                             QString("地上デジタル")}},
    {LC70_BAND_BS,                  {QString("BS"),                                QString("BS")}},
    {LC70_BAND_CS,                  {QString("110°CS"),                            QString("110°CS")}},
    {LC70_BAND_JCSAT3,              {QString("JCSAT-3A"),                            QString("JCSAT-3A")}},
    {LC70_BAND_JCSAT4,              {QString("JCSAT-4B"),                            QString("JCSAT-4B")}},
    {LC70_BAND_SCCB,                {QString("SCC-B"),                              QString("SCC-B")}},
    {LC70_BAND_SCCC,                {QString("SCC-C"),                              QString("SCC-C")}},
    {LC70_BAND_CATV_ALL,            {QString("CATV"),                               QString("CATV")}},
    {LC70_BAND_CATV_90_470MHZ,      {QString("CATV"),                               QString("CATV")}},
    {LC70_BAND_CATV_470_1002MHZ,    {QString("CATV"),                               QString("CATV")}},
    {LC70_BAND_PILOT,               {QString("PILOT"),                              QString("PILOT")}},
    {LC70_BAND_SKYP,                {QString("スカパー！光"),    QString("スカパー！プレミアムサービス光")}},
    {LC70_BAND_LTE,                 {QString("LTE"),                                QString("LTE")}},
    {LC70_BAND_FM,                  {QString("FM"),                                 QString("FM")}},
    {LC70_BAND_CATV_UP_STREAM,      {QString("CATV上り"),                            QString("CATV上り")}}
};

static const QMap<int, QString> POLARIZATION_MAP = {
    {LC70_MEAS_SET_WAVE_R, RIGHT_HANDED_POLARIZE },
    {LC70_MEAS_SET_WAVE_L, LEFT_HANDED_POLARIZE  },
    {LC70_MEAS_SET_WAVE_H, HORIZONTALLY_POLARIZE },
    {LC70_MEAS_SET_WAVE_V, VERTICALLY_POLARIZE   }
};

static const QMap<int, QString> BROADCASTSYSTEM_MAP = {
    {LC70_BD_SYS_ISDBT,     ISDBT },
    {LC70_BD_SYS_ISDBS,     ISDBS },
    {LC70_BD_SYS_ISDBS3,    ISDBS3},
    {LC70_BD_SYS_ANNEXC,    ANNEXC},
    {LC70_BD_SYS_DVBS,      DVBS},
    {LC70_BD_SYS_DVBS2,     DVBS2},
    {LC70_BD_SYS_OPTICAST,  OPTICAST},
    {LC70_BD_SYS_LTE,       LTE},
    {LC70_BD_SYS_ANALOG,    ANALOG},
    {LC70_BD_SYS_ANNEXB,    ANNEXB},
    {LC70_BD_SYS_NONE,      ""}
};

/**
 *  @brief Band 毎の Level の範囲
 */
static const QList<MeasureLevelRange> LEVEL_RANGE = {
    // Band の種類                   最小 Level   最大 Level
    { BAND_TAB_UHF,                 20,         110 },
    { BAND_TAB_BS_R,                30,         110 },
    { BAND_TAB_BS_L,                40,         110 },
    { BAND_TAB_CS_R,                30,         110 },
    { BAND_TAB_CS_L,                40,         110 },
    { BAND_TAB_CATV_ALL,            20,         110 },
    { BAND_TAB_SKY_PERFECTV,        20,         110 },
    { BAND_TAB_CATV_UPSTREAM,       30,         110 },
    { BAND_TAB_JCSAT3A_H,           30,         110 },
    { BAND_TAB_JCSAT3A_V,           30,         110 },
    { BAND_TAB_JCSAT4B_H,           30,         110 },
    { BAND_TAB_JCSAT4B_V,           30,         110 },
    { BAND_TAB_SCCB2_H,             30,         110 },
    { BAND_TAB_SCCB2_V,             30,         110 },
    { BAND_TAB_SCCC2_H,             30,         110 },
    { BAND_TAB_SCCC2_V,             30,         110 },
    { BAND_TAB_LTE,                 20,         110 },
    { BAND_TAB_FM,                  30,         110 },
    { BAND_TAB_PILOT,               30,         110 },
    { BAND_TAB_COUNT,               -1,          -1 }
};

static QVariantList _merCnrRangeList; //!< MER/CNRの上限下限リスト

Correcter::Correcter()
{

}

QString Correcter::displayBandName(int band, bool shortName)
{
    LC70_BAND_T bandId = static_cast<LC70_BAND_T>(band);

    if (BAND_MAP.count(bandId) == 0) {
        return QString("");
    }

    if (shortName) {
        return BAND_MAP.value(bandId).first;
    } else {
        return BAND_MAP.value(bandId).second;
    }
}

int Correcter::bandIdFromDisplayBand(QString band)
{
    QMap<LC70_BAND_T, QPair<QString, QString>>::const_iterator iterator;

    for (iterator = BAND_MAP.begin(); iterator != BAND_MAP.end(); ++iterator) {
        if ((iterator->first == band) || (iterator->second == band)) {
            return static_cast<int>(iterator.key());
        }
    }

    return static_cast<int>(LC70_BAND_UNDEF);
}

QString Correcter::displayBandGroupName(int band)
{
    static const QMap<LC70_BAND_T, QString> BAND_GROUP_MAP = {
        {LC70_BAND_UHF,                 QString("地上\uff83\uff9e\uff7c\uff9e\uff80\uff99")},
        {LC70_BAND_BS,                  QString("BS")},
        {LC70_BAND_CS,                  QString("110°CS")},
        {LC70_BAND_JCSAT3,              QString("JCSAT-3A")},
        {LC70_BAND_JCSAT4,              QString("JCSAT-4B")},
        {LC70_BAND_SCCB,                QString("SCC-B")},
        {LC70_BAND_SCCC,                QString("SCC-C")},
        {LC70_BAND_CATV_ALL,            QString("CATV ALL")},
        {LC70_BAND_CATV_90_470MHZ,      QString("CATV 90-470MHz")},
        {LC70_BAND_CATV_470_1002MHZ,    QString("CCATV 470-1002MHz")},
        {LC70_BAND_PILOT,               QString("PILOT")},
        {LC70_BAND_SKYP,                QString("\uff7d\uff76\uff8a\uff9f\uff70光")},
        {LC70_BAND_LTE,                 QString("LTE")},
        {LC70_BAND_FM,                  QString("FM")},
        {LC70_BAND_CATV_UP_STREAM,      QString("CATV上り")},
    };

    LC70_BAND_T bandId = static_cast<LC70_BAND_T>(band);

    return BAND_GROUP_MAP.value(bandId, "");
}

QString Correcter::displayBroadbandSystem(unsigned int broadbandSystem)
{
    return BROADCASTSYSTEM_MAP.value(broadbandSystem, QString(""));
}

QString Correcter::convertWaveToPolarization(const int waveNumber)
{
    return POLARIZATION_MAP.value(waveNumber, QString(""));
}

int Correcter::convertPolarizationToWave(const QString &polarization)
{
    return POLARIZATION_MAP.key(polarization, 0);
}

QString Correcter::displayPolarization(const QString &polarization)
{
    static const QMap<QString, QString> polarizationMap = {
        {HORIZONTALLY_POLARIZE, "水平"},
        {VERTICALLY_POLARIZE,   "垂直"},
        {RIGHT_HANDED_POLARIZE, "右旋"},
        {LEFT_HANDED_POLARIZE,  "左旋"},
    };

    return polarizationMap.value(polarization, QString());
}

QString Correcter::displayChannelName(QString channelNumber)
{
    return channelNumber;
}

unsigned int Correcter::centerFrequency(ChannelDefinitionItem item, unsigned int modulation)
{
    // [Memo]
    //  以下のBANDは１つのチャンネルに2つ中心周波数を持っておりどちらを使うかは変調方式によって決まる
    //  CATV
    //  CATV_UHF
    //  CATV_VHF
    //  変調方式が64QAMまたは、256QAMの場合は、centerFrequency1 (QAM中心周波数)
    //  変調方式がOFDMの場合は、 centerFrequency2 (OFDM中心周波数)
    //  BANDがCATV、CATV_UHF、CATV_VHF以外の場合は、centerFrequency1を返す

    // ラッパーライブラリ V2.00 対応
    if (item.band == LC70_BAND_CATV_ALL || item.band == LC70_BAND_CATV_90_470MHZ
            || item.band == LC70_BAND_CATV_470_1002MHZ ) {
        if (modulation == LC70_SETMOD_64QAM || modulation == LC70_MEAS_SET_CATV_MOD_256QAM) {
            return item.centerFrequency1;
        }

        return item.centerFrequency2;
    }

    return item.centerFrequency1;
}

unsigned int Correcter::frequency(unsigned int centerFrequency, unsigned int localFrequency)
{
    //[Memo]
    // 周波数の値は、中心周波数 - 局発周波数の値となるため計算を実施する。
    unsigned int result = centerFrequency - localFrequency;
    return result;
}

QString Correcter::displayFrequency(int band, unsigned int frequency, bool integerOnly)
{
    // [Memo]
    //  実周波数情報の画面表示はMHzに統一
    //  局発周波数が設定されている場合は事前に中心周波数から局発周波数を引いた値にしておく。
    //  表示の際には、元の中心周波数がGHzで表現される場合は小数点2桁、そうでない場合は原則3桁で表示する。
    //  それぞれ次点の桁を四捨五入する。
    static const QMap<LC70_BAND_T, int> decimalDigitsMap = {
        {LC70_BAND_UHF, 3},                 // (MHz) 地デジ
        {LC70_BAND_BS, 2},                  // (GHz) BS
        {LC70_BAND_CS, 2},                  // (GHz) 110°CS
        {LC70_BAND_JCSAT3, 2},              // (GHz) JCSAT-3A
        {LC70_BAND_JCSAT4, 2},              // (GHz) JCSAT-4B
        {LC70_BAND_SCCB, 2},                // (GHz) SCC-B
        {LC70_BAND_SCCC, 2},                // (GHz) SCC-C
        {LC70_BAND_CATV_ALL, 3},            // (MHz) CATV ALL
        {LC70_BAND_CATV_90_470MHZ, 3},      // (MHz) CATV 90-470MHz
        {LC70_BAND_CATV_470_1002MHZ, 3},    // (MHz) CATV 470-1002MHz
        {LC70_BAND_PILOT, 2},               // (MHz) PILOT ※小数点２桁で表示する
        {LC70_BAND_SKYP, 3},                // (MHz) スカパー!プレミアム光
        {LC70_BAND_LTE, 0},                 // (MHz) LTE ※小数点なしで表示する
        {LC70_BAND_FM, 1},                  // (MHz) FM
        {LC70_BAND_CATV_UP_STREAM, 2},      // (MHz) CATV上り
    };
    LC70_BAND_T bandType = static_cast<LC70_BAND_T>(band);

    if (!integerOnly) {
        return QString("%1").arg(static_cast<float32_t>(frequency) / 1000, 0, 'f', decimalDigitsMap.value(bandType, 2));
    } else {
        return QString("%1").arg(static_cast<float32_t>(frequency) / 1000, 0, 'f', 0);
    }
}

QString Correcter::displayLocalFrequency(unsigned int localFrequency)
{
    // [Memo]
    //  局発周波数情報の画面表示はGHzに統一
    //  下２桁以降は四捨五入を実施する。
    return QString("%1").arg(static_cast<float32_t>(localFrequency) / float(1000000), 0, 'f',
                             getDecimalDigits(localFrequency));
}

QString Correcter::displayHierarchy(unsigned int layer)
{
    QString result;

    switch (layer) {
    case 0:     // レイヤーなし
        result = "";
        break;

    case 1:     // A
        result = "A";
        break;

    case 2:     // B
        result = "B";
        break;

    case 3:     // C
        result = "C";
        break;

    default:
        break;
    }

    return result;
}

QString Correcter::displayModulation(unsigned int modulation)
{
    return MODULATION_VALUE_MAP.key(modulation, "");
}

QString Correcter::modulationPrefix(const int broadcastSystem, const QString &modulation)
{
    // 放送方式がISDBTの場合の接頭句の関連Map
    static const QMap<QString, QString> prefixMap = {
        {SIXTY_FOUR_QAM, OFDM},
        {QPSK, OFDM},
    };

    // 放送方式がISDBTである場合
    if (broadcastSystem == LC70_BD_SYS_ISDBT) {
        return prefixMap.value(modulation, "");
    }

    // 放送方式がANNEXC, ANNEXB, DVBS, DVBS2の場合 変調方式の接頭句として放送方式を返す
    if (broadcastSystem == LC70_BD_SYS_ANNEXC
            || broadcastSystem == LC70_BD_SYS_ANNEXB
            || broadcastSystem == LC70_BD_SYS_DVBS
            || broadcastSystem == LC70_BD_SYS_DVBS2) {
        return displayBroadbandSystem(broadcastSystem);
    }

    // 上記の条件に当てはまらない場合 空文字を返す
    return QString("");
}

uint32_t Correcter::modulationValue(QString modulation)
{
    return MODULATION_VALUE_MAP.value(modulation, 0);
}

unsigned int Correcter::broadcastSystemValue(const QString &broadcastSystem)
{
    return BROADCASTSYSTEM_MAP.key(broadcastSystem, LC70_BD_SYS_NONE);
}

void Correcter::getModulationBitList(unsigned int modulationBit, QList<unsigned int> &list)
{
    QList<unsigned int> bitList = {
        LC70_SETMOD_OFDM,
        LC70_SETMOD_BPSK,
        LC70_SETMOD_QPSK,
        LC70_SETMOD_TC8PSK,
        LC70_SETMOD_32APSK,
        LC70_SETMOD_16APSK,
        LC70_SETMOD_8PSK,
        LC70_SETMOD_64QAM,
        LC70_SETMOD_256QAM,
        LC70_SETMOD_CW,
        LC70_SETMOD_FM,
    };

    int index = 0;
    list.clear();

    foreach (unsigned int bit, bitList) {
        if (modulationBit & bit) {
            list.append(bit);
        }

        index++;
    }
}

QString Correcter::displayPowerSupply(int dcOutVal)
{
    //    static const QMap<int, QString> powerSupplyMap = {
    //#ifdef NEW_USER_SETTING
    //        {LC70_MEAS_SET_DC_OUT_VAL_NONE,  "OFF"},
    //#else
    //        {LC70_MEAS_SET_DC_OUT_VAL_NONE,  "なし"},
    //#endif
    //        {LC70_MEAS_SET_DC_OUT_VAL_11V,   "11V"},
    //        {LC70_MEAS_SET_DC_OUT_VAL_15V,   "15V"},
    //        {LC70_MEAS_SET_DC_OUT_VAL_11V_P, "11V+P"},
    //        {LC70_MEAS_SET_DC_OUT_VAL_15V_P, "15V+P"},
    //    };

    static const QMap<int, QString> powerSupplyMap = {
    #ifdef NEW_USER_SETTING
        {LC70_MEAS_SET_DC_OUT_VAL_NONE,  "OFF"},
    #else
        {LC70_MEAS_SET_DC_OUT_VAL_NONE,  "なし"},
    #endif
        {LC70_MEAS_SET_DC_OUT_VAL_11V,   "ON"},
        {LC70_MEAS_SET_DC_OUT_VAL_15V,   "ON"},
        {LC70_MEAS_SET_DC_OUT_VAL_11V_P, "ON"},
        {LC70_MEAS_SET_DC_OUT_VAL_15V_P, "ON"},
    };

    return powerSupplyMap.value(dcOutVal, "なし");
}

bool Correcter::isSatelliteBroadCasting(int band)
{
    LC70_BAND_T bandType = static_cast<LC70_BAND_T>(band);

    if (bandType == LC70_BAND_BS
            || bandType == LC70_BAND_CS
            || bandType == LC70_BAND_JCSAT3
            || bandType == LC70_BAND_JCSAT4
            || bandType == LC70_BAND_SCCB
            || bandType == LC70_BAND_SCCC) {
        return true;
    } else {
        return false;
    }
}

bool Correcter::isDisplayLevelHierarchyPolarization(int band, unsigned int modulationValue)
{
    LC70_BAND_T bandType = static_cast<LC70_BAND_T>(band);

    if (bandType == LC70_BAND_UHF
            || bandType == LC70_BAND_BS
            || bandType == LC70_BAND_CS
            || bandType == LC70_BAND_JCSAT3
            || bandType == LC70_BAND_JCSAT4
            || bandType == LC70_BAND_SCCB
            || bandType == LC70_BAND_SCCC
            || (bandType == LC70_BAND_CATV_ALL && modulationValue == LC70_SETMOD_OFDM)
            || (bandType == LC70_BAND_CATV_90_470MHZ && modulationValue == LC70_SETMOD_OFDM)
            || (bandType == LC70_BAND_CATV_470_1002MHZ && modulationValue == LC70_SETMOD_OFDM) ) {
        return true;
    } else {
        // 衛星放送及び地デジの場合は表示する
        return false;
    }
}

void Correcter::setMerCnLimitList(const QVariantList merCnLimitList)
{
    _merCnrRangeList = merCnLimitList;
}

QPair<int, int> Correcter::merRange(const int bandId, const int broadcastSystem, const int modulation)
{
    // 閾値データのindex(LC70_MEAS_SET_THRD_IDX_xxxxの値)取得
#if 1
	// BAND CATVで ISDBT(OFDM)を選択したときに、getThresholdIndexを正しく引けない不具合を修正
	// 地デジ受信中に modulationに 64QAMが返ってくるため、CATV 64QAMと混同してしまうのが原因
    int thresholdIndex;

    if (broadcastSystem == LC70_BD_SYS_ISDBT) {
        thresholdIndex = Correcter::getThresholdIndex(bandId, broadcastSystem, LC70_SETMOD_OFDM);
    }
    else {
        thresholdIndex = Correcter::getThresholdIndex(bandId, broadcastSystem, modulation);
    }
#else
    int thresholdIndex = Correcter::getThresholdIndex(bandId, broadcastSystem, modulation);
#endif

    if (thresholdIndex < 0) {
        qDebug() << QString("there is no threshold data");
        // 閾値データの取得ができなかった際には、最小値 5, 最大値 35をMERの範囲として設定する
        return {5 , 35};
    }

    QVariantList limitList = _merCnrRangeList.at(thresholdIndex).toList();

    // 得られたMERは10倍の値が設定されているため10で割った値を使う
    int minimumValue = limitList.at(0).toInt() / 10;
    int maximumValue = limitList.at(1).toInt() / 10;

    QPair<int, int> merCnrLimits = {minimumValue, maximumValue};

    return merCnrLimits;
}

QString Correcter::displayMer(const int bandId, const int broadcastSystem, const int modulation, const float32_t mer)
{
    Q_UNUSED(bandId);
    Q_UNUSED(broadcastSystem);
    Q_UNUSED(modulation);

#if 1
    int thresholdIndex;

    if (broadcastSystem == LC70_BD_SYS_ISDBT) {
        thresholdIndex = Correcter::getThresholdIndex(bandId, broadcastSystem, LC70_SETMOD_OFDM);
    }
    else {
        thresholdIndex = Correcter::getThresholdIndex(bandId, broadcastSystem, modulation);
    }
#else
    int thresholdIndex = Correcter::getThresholdIndex(bandId, broadcastSystem, modulation);
#endif

    if (thresholdIndex < 0) {
        // 閾値情報が正しく取得できない場合は、そのままのMERの値を返す
        return QString("%1").arg(mer, 2, 'f', 1).rightJustified(2, '0');
    }

    // 閾値情報が正しく取得できた場合
    QPair<int, int> merLimits = merRange(bandId, broadcastSystem, modulation);

#if 1
    if (broadcastSystem == LC70_BD_SYS_ANNEXB || broadcastSystem == LC70_BD_SYS_ANNEXC || broadcastSystem ==LC70_BD_SYS_OPTICAST ){
               if(mer < 20){
                   return QString("<20.0");
               }
    }
    else {
        if(mer < static_cast<float32_t>(merLimits.first)){
            // 下限値を下回っている場合
            return QString(FORMAT_MINIMUM_MER).arg(static_cast<float32_t>(merLimits.first));
        }
    }
#else
    if (mer < static_cast<float32_t>(merLimits.first)) {
        // 下限値を下回っている場合
        return QString(FORMAT_MINIMUM_MER).arg(merLimits.first);
    }
#endif


    if(broadcastSystem == LC70_BD_SYS_ISDBS || broadcastSystem == LC70_BD_SYS_ISDBS3){
        if(mer > 30){
            return QString(">30.0");
        }

    }else{
        if (mer > static_cast<float32_t>(merLimits.second)) {
            // 上限値を超えている場合
            return QString(FORMAT_MAXIMUM_MER).arg(merLimits.second);
        }
    }
    // 値域内
    return QString("%1").arg(mer, 2, 'f', 1).rightJustified(2, '0');
}

QString Correcter::displayBer(const float32_t ber)
{
    return QString("%1").arg(ber, 1, 'e', 1).rightJustified(4, '0')
            .replace("e-0", "e-")    // 指数部を一桁で表示するため不要な0を削除する
            .replace("e+0", "e-");  // 0.0e+00を0.0e-0に変換する
}

bool Correcter::getLocalOscillatorList(const QVariant &modelData, const int bandId, const QString &polarization,
                                       QList<uint32_t> &localOscillatorList)
{
    static const QList<int> firstDataOnly = {2};            // BS、CSなど、局発周波数の先頭データだけを採用するケース
    static const QList<int> selectTwoData = {5, 6};       // SCC-B、SCC-Cなど、後半の2データを採用するケース
    static const QList<int> selectThreeData = {2, 5, 6};     // JCSAT3、JCSAT4など、前後の3データを採用するケース
    static const QMap<QPair<int, QString>, QPair<int, QList<int>>> localOscillatorIndexMap = {
        {{LC70_BAND_BS, RIGHT_HANDED_POLARIZE},     {LC70_MEAS_SET_LOCAL_FREQ_IDX_BS_R, firstDataOnly}},
        {{LC70_BAND_BS, LEFT_HANDED_POLARIZE},      {LC70_MEAS_SET_LOCAL_FREQ_IDX_BS_L, firstDataOnly}},
        {{LC70_BAND_CS, RIGHT_HANDED_POLARIZE},     {LC70_MEAS_SET_LOCAL_FREQ_IDX_CS_R, firstDataOnly}},
        {{LC70_BAND_CS, LEFT_HANDED_POLARIZE},      {LC70_MEAS_SET_LOCAL_FREQ_IDX_CS_L, firstDataOnly}},
        {{LC70_BAND_SCCB, HORIZONTALLY_POLARIZE},   {LC70_MEAS_SET_LOCAL_FREQ_IDX_SCCB_H, selectTwoData}},
        {{LC70_BAND_SCCB, VERTICALLY_POLARIZE},     {LC70_MEAS_SET_LOCAL_FREQ_IDX_SCCB_V, selectTwoData}},
        {{LC70_BAND_SCCC, HORIZONTALLY_POLARIZE},   {LC70_MEAS_SET_LOCAL_FREQ_IDX_SCCC_H, selectTwoData}},
        {{LC70_BAND_SCCC, VERTICALLY_POLARIZE},     {LC70_MEAS_SET_LOCAL_FREQ_IDX_SCCC_V, selectTwoData}},
        {{LC70_BAND_JCSAT3, HORIZONTALLY_POLARIZE}, {LC70_MEAS_SET_LOCAL_FREQ_IDX_JCSAT3_H, selectThreeData}},
        {{LC70_BAND_JCSAT3, VERTICALLY_POLARIZE},   {LC70_MEAS_SET_LOCAL_FREQ_IDX_JCSAT3_V, selectThreeData}},
        {{LC70_BAND_JCSAT4, HORIZONTALLY_POLARIZE}, {LC70_MEAS_SET_LOCAL_FREQ_IDX_JCSAT4_H, selectThreeData}},
        {{LC70_BAND_JCSAT4, VERTICALLY_POLARIZE},   {LC70_MEAS_SET_LOCAL_FREQ_IDX_JCSAT4_V, selectThreeData}},
        {{LC70_BAND_JCSAT3 * LC70_BAND_JCSAT4, HORIZONTALLY_POLARIZE}, {LC70_MEAS_SET_LOCAL_FREQ_IDX_JCSAT3_H, selectThreeData}},
        {{LC70_BAND_JCSAT3 * LC70_BAND_JCSAT4, VERTICALLY_POLARIZE},   {LC70_MEAS_SET_LOCAL_FREQ_IDX_JCSAT3_V, selectThreeData}},
    };

    QPair<int, QString> key = {bandId, polarization};

    if (localOscillatorIndexMap.count(key) == 0) {
        // 現在の選択に該当するデータインデックスが存在しない場合
        qDebug() << "There is no such Data";
        return false;
    }

    // 周波数に該当するデータを取得する
    QPair<int, QList<int>> value = localOscillatorIndexMap.value(key);

    foreach (int index, value.second) {
        localOscillatorList.append(modelData.toList().at(value.first).toList().at(index).toUInt());
    }

    return true;
}

QString Correcter::convertPolarizeNumToString(int polarizeNum)
{
    static const QMap<int, QString> polarizeMap = {
        {LC70_MEAS_SET_WAVE_R, RIGHT_HANDED_POLARIZE},
        {LC70_MEAS_SET_WAVE_L, LEFT_HANDED_POLARIZE},
        {LC70_MEAS_SET_WAVE_H, HORIZONTALLY_POLARIZE},
        {LC70_MEAS_SET_WAVE_V, VERTICALLY_POLARIZE}
    };

    if (polarizeMap.count(polarizeNum) == 0) {
        // 引数に該当するデータが存在しない場合
        //        qDebug() << __FUNCTION__ << "Broadcast without polarization";
        return "";
    }

    // 引数に対応するデータを取得する
    return polarizeMap.value(polarizeNum);

}

QString Correcter::polarization(const int &bandId, const int &polarizationIndex)
{
    static const QMap<QPair<int, int>, QString> polarizationMap = {
        {{LC70_BAND_BS, INDEX_RIGHT_HAND_HORIZONTAL}, RIGHT_HANDED_POLARIZE},
        {{LC70_BAND_BS, INDEX_LEFT_HAND_VERTICAL}, LEFT_HANDED_POLARIZE},
        {{LC70_BAND_CS, INDEX_RIGHT_HAND_HORIZONTAL}, RIGHT_HANDED_POLARIZE},
        {{LC70_BAND_CS, INDEX_LEFT_HAND_VERTICAL}, LEFT_HANDED_POLARIZE},
        {{LC70_BAND_JCSAT3, INDEX_RIGHT_HAND_HORIZONTAL}, HORIZONTALLY_POLARIZE},
        {{LC70_BAND_JCSAT3, INDEX_LEFT_HAND_VERTICAL}, VERTICALLY_POLARIZE},
        {{LC70_BAND_JCSAT4, INDEX_RIGHT_HAND_HORIZONTAL}, HORIZONTALLY_POLARIZE},
        {{LC70_BAND_JCSAT4, INDEX_LEFT_HAND_VERTICAL}, VERTICALLY_POLARIZE},
        {{LC70_BAND_SCCB, INDEX_RIGHT_HAND_HORIZONTAL}, HORIZONTALLY_POLARIZE},
        {{LC70_BAND_SCCB, INDEX_LEFT_HAND_VERTICAL}, VERTICALLY_POLARIZE},
        {{LC70_BAND_SCCC, INDEX_RIGHT_HAND_HORIZONTAL}, HORIZONTALLY_POLARIZE},
        {{LC70_BAND_SCCC, INDEX_LEFT_HAND_VERTICAL}, VERTICALLY_POLARIZE},
        {{LC70_BAND_JCSAT3 * LC70_BAND_JCSAT4, INDEX_RIGHT_HAND_HORIZONTAL}, HORIZONTALLY_POLARIZE},
        {{LC70_BAND_JCSAT3 * LC70_BAND_JCSAT4, INDEX_LEFT_HAND_VERTICAL}, VERTICALLY_POLARIZE},
    };

    return polarizationMap.value({bandId, polarizationIndex}, "");
}

uint8_t Correcter::broadcastSystem(const int &bandId, const int &polarizationIndex, const uint32_t &modulation)
{
    switch (bandId) {
    case LC70_BAND_UHF:
    case LC70_BAND_PILOT:
    case LC70_BAND_LTE:
        return LC70_BD_SYS_ISDBT;

    case LC70_BAND_BS: {
        if (polarizationIndex == 0) {
            return LC70_BD_SYS_ISDBS;
        } else {
            return LC70_BD_SYS_ISDBS3;
        }
    }

    case LC70_BAND_CS: {
        if (polarizationIndex == 0) {
            return LC70_BD_SYS_ISDBS;
        } else {
            return LC70_BD_SYS_ISDBS3;
        }
    }

    case LC70_BAND_CATV_ALL:
    case LC70_BAND_CATV_90_470MHZ:
    case LC70_BAND_CATV_470_1002MHZ: {
        if (modulation == LC70_SETMOD_OFDM) {
            return LC70_BD_SYS_ISDBT;
        } else if (modulation == LC70_SETMOD_64QAM) {
            return LC70_BD_SYS_ANNEXC;
        } else {
            // nothing to do.
            break;
        }
    }

    case LC70_BAND_SKYP: {
        if (modulation == LC70_SETMOD_OFDM) {
            return LC70_BD_SYS_ISDBT;
        } else if (modulation == LC70_SETMOD_256QAM) {
            return LC70_BD_SYS_OPTICAST;
        } else {
            // nothing to do.
            break;
        }
    }

    case LC70_BAND_JCSAT3:
    case LC70_BAND_JCSAT4:
    case LC70_BAND_JCSAT3 *LC70_BAND_JCSAT4:
    case LC70_BAND_SCCB:
    case LC70_BAND_SCCC:

        return LC70_BD_SYS_DVBS2;

    case LC70_BAND_FM:
    case LC70_BAND_CATV_UP_STREAM:
        break;
    }

    return 0;   // 指定なし
}

int Correcter::getDecimalDigits(const uint32_t value)
{
    if (value % 10000 != 0) {
        // 小数点以下、3桁
        return 3;
    } else if (value % 100000 != 0) {
        // 小数点以下、2桁
        return 2;
    } else  {
        // 小数点以下、1桁(整数でも10.0といった表示にするため、最低でも1を返す)
        return 1;
    }
}

bool Correcter::isModulationVisible(const int &bandId)
{
    // 変調方式 表示、非表示対応マップ
    static const QMap<LC70_BAND_T, int> modulationVisibleMap = {
        {LC70_BAND_UHF,              true},  // 地デジ
        {LC70_BAND_BS,               true},  // BS
        {LC70_BAND_CS,               true},  // 110°CS
        {LC70_BAND_JCSAT3,           true},  // JCSAT-3A
        {LC70_BAND_JCSAT4,           true},  // JCSAT-4B
        {LC70_BAND_SCCB,             true},  // SCC-B
        {LC70_BAND_SCCC,             true},  // SCC-C
        {LC70_BAND_CATV_ALL,         true},  // CATV ALL
        {LC70_BAND_CATV_90_470MHZ,   true},  // CATV 90-470MHz
        {LC70_BAND_CATV_470_1002MHZ, true},  // CATV 470-1002MHz
        {LC70_BAND_PILOT,            true},  // PILOT
        {LC70_BAND_SKYP,             true},  // スカパー!プレミアム光
        {LC70_BAND_LTE,              false}, // LTE
        {LC70_BAND_FM,               true},  // FM
        {LC70_BAND_CATV_UP_STREAM,   true},  // CATV上り
    };

    // Mapに定義されている値を返す。
    // Mapに定義されていないBANDが入って来た場合は、falseを返す
    return modulationVisibleMap.value(static_cast<LC70_BAND_T>(bandId), false);
}

uint16_t Correcter::getBroadcastSystemCondition(const int bandId, const QString &modulation) {
    static const QMap<QPair<LC70_BAND_T, QString>, uint16_t> bandModulationBdSysMap = {
        {{LC70_BAND_CATV_ALL,         OFDM},                        LC70_BD_SYS_ISDBT},
        {{LC70_BAND_CATV_ALL,         SIXTY_FOUR_QAM},              LC70_BD_SYS_ANNEXC},
        {{LC70_BAND_CATV_ALL,         TWO_HUNDRED_FIFTY_SIX_QAM},   LC70_BD_SYS_ANNEXB},
        {{LC70_BAND_CATV_90_470MHZ,   OFDM},                        LC70_BD_SYS_ISDBT},
        {{LC70_BAND_CATV_90_470MHZ,   SIXTY_FOUR_QAM},              LC70_BD_SYS_ANNEXC},
        {{LC70_BAND_CATV_90_470MHZ,   TWO_HUNDRED_FIFTY_SIX_QAM},   LC70_BD_SYS_ANNEXB},
        {{LC70_BAND_CATV_470_1002MHZ, OFDM},                        LC70_BD_SYS_ISDBT},
        {{LC70_BAND_CATV_470_1002MHZ, SIXTY_FOUR_QAM},              LC70_BD_SYS_ANNEXC},
        {{LC70_BAND_CATV_470_1002MHZ, TWO_HUNDRED_FIFTY_SIX_QAM},   LC70_BD_SYS_ANNEXB},
        {{LC70_BAND_SKYP,             OFDM},                        LC70_BD_SYS_ISDBT},
        {{LC70_BAND_SKYP,             TWO_HUNDRED_FIFTY_SIX_QAM},   LC70_BD_SYS_OPTICAST},
    };

    static const QMap<LC70_BAND_T, uint16_t> bandBdSysMap = {
        {LC70_BAND_UHF,LC70_BD_SYS_ISDBT},
        {LC70_BAND_LTE,LC70_BD_SYS_NONE},
    };

    // Band と変調方式によって決まる放送方式を判定する
    LC70_BAND_T band = static_cast<LC70_BAND_T>(bandId);
    QPair<LC70_BAND_T, QString> bandModulationKey = {band, modulation};
    if (bandModulationBdSysMap.count(bandModulationKey) != 0) {
        return bandModulationBdSysMap.value(bandModulationKey);
    }

    // Band によって決まる放送方式を判定する
    if (bandBdSysMap.count(band) != 0) {
        return bandBdSysMap.value(band);
    }

    // 放送方式は測定結果で受領するまで不明
    return LC70_BD_SYS_NONE;
}

QList<QVariant> Correcter::getDisplayChannelDetail(int bandId) {
    // BAND_IDがキー チャンネル詳細データの表示可否マップがバリューとなっている。
    // チャンネル詳細データの表示可否リストは、[周波数],[変調方式],[階層],[偏波],[局部発信周波数]の順で並んでいる。
    static const QMap <LC70_BAND_T, QList<QVariant>> channelDetailDisplayMap = {
        {LC70_BAND_UHF,              {false,  false,  false,  false, false}},
        {LC70_BAND_BS,               {false,  false,  false,  true,  true }},
        {LC70_BAND_CS,               {false,  false,  false,  true,  true }},
        {LC70_BAND_JCSAT3,           {false,  false,  false,  true,  true }},
        {LC70_BAND_JCSAT4,           {false,  false,  false,  true,  true }},
        {LC70_BAND_SCCB,             {false,  false,  false,  true,  true }},
        {LC70_BAND_SCCC,             {false,  false,  false,  true,  true }},
        {LC70_BAND_CATV_ALL,         {false,  false,  false,  false, false}},
        {LC70_BAND_CATV_90_470MHZ,   {false,  false,  false,  false, false}},
        {LC70_BAND_CATV_470_1002MHZ, {false,  false,  false,  false, false}},
        {LC70_BAND_SKYP,             {false,  false,  false,  false, false}},
        {LC70_BAND_PILOT,            {false,  false,  false,  false, false}},
        {LC70_BAND_LTE,              {false,  false,  false,  false, false}},
        {LC70_BAND_FM,               {false,  false,  false,  false, false}},
        {LC70_BAND_CATV_UP_STREAM,   {false,  false,  false,  false, false}},
    };


    return channelDetailDisplayMap.value(static_cast<LC70_BAND_T>(bandId), {false, false, false, false, false});
}

uint16_t Correcter::getModulationCondition(int bandId, uint16_t &broadcastSystem, unsigned int polatization, unsigned int modulation, QVariant setting)
{
    int index = -1;

    // BANDで決まる測定設定リストのインデックス関係マップ
    static const QMap<int, int> measurementSettingBandMap = {
        {LC70_BAND_UHF,            LC70_MEAS_SET_MANU_BDSYS_MOD_IDX_UHF},
        {LC70_BAND_PILOT,          LC70_MEAS_SET_MANU_BDSYS_MOD_IDX_PILOT},
        {LC70_BAND_LTE,            LC70_MEAS_SET_MANU_BDSYS_MOD_IDX_LTE},
        {LC70_BAND_FM,             LC70_MEAS_SET_MANU_BDSYS_MOD_IDX_FM},
        {LC70_BAND_CATV_UP_STREAM, LC70_MEAS_SET_MANU_BDSYS_MOD_IDX_CATV_UP_STREAM}
    };

    // BANDと偏波で決まる測定設定リストのインデックス関係マップ
    // アンテナ調整は、BANDが定義されていないため LC70_BAND_JCSAT3とLC70_BAND_JCSAT4を掛けた結果を用いる。
    static const QMap<QPair<int, int>, int> measurementSettingBandPolarizationMap = {
        {{LC70_BAND_BS,                        LC70_MEAS_SET_WAVE_R}, LC70_MEAS_SET_MANU_BDSYS_MOD_IDX_BS_R            },
        {{LC70_BAND_BS,                        LC70_MEAS_SET_WAVE_L}, LC70_MEAS_SET_MANU_BDSYS_MOD_IDX_BS_L            },
        {{LC70_BAND_CS,                        LC70_MEAS_SET_WAVE_R}, LC70_MEAS_SET_MANU_BDSYS_MOD_IDX_CS_R            },
        {{LC70_BAND_CS,                        LC70_MEAS_SET_WAVE_L}, LC70_MEAS_SET_MANU_BDSYS_MOD_IDX_CS_L            },
        {{LC70_BAND_JCSAT3,                    LC70_MEAS_SET_WAVE_H}, LC70_MEAS_SET_MANU_BDSYS_MOD_IDX_JCSAT3_H        },
        {{LC70_BAND_JCSAT3,                    LC70_MEAS_SET_WAVE_V}, LC70_MEAS_SET_MANU_BDSYS_MOD_IDX_JCSAT3_V        },
        {{LC70_BAND_JCSAT4,                    LC70_MEAS_SET_WAVE_H}, LC70_MEAS_SET_MANU_BDSYS_MOD_IDX_JCSAT4_H        },
        {{LC70_BAND_JCSAT4,                    LC70_MEAS_SET_WAVE_V}, LC70_MEAS_SET_MANU_BDSYS_MOD_IDX_JCSAT4_V        },
        {{LC70_BAND_JCSAT3 * LC70_BAND_JCSAT4, LC70_MEAS_SET_WAVE_H}, LC70_MEAS_SET_MANU_BDSYS_MOD_IDX_JCSAT_ANTENNA_H },
        {{LC70_BAND_JCSAT3 * LC70_BAND_JCSAT4, LC70_MEAS_SET_WAVE_V}, LC70_MEAS_SET_MANU_BDSYS_MOD_IDX_JCSAT_ANTENNA_V },
        {{LC70_BAND_SCCB,                      LC70_MEAS_SET_WAVE_H}, LC70_MEAS_SET_MANU_BDSYS_MOD_IDX_SCCB_H          },
        {{LC70_BAND_SCCB,                      LC70_MEAS_SET_WAVE_V}, LC70_MEAS_SET_MANU_BDSYS_MOD_IDX_SCCB_V          },
        {{LC70_BAND_SCCC,                      LC70_MEAS_SET_WAVE_H}, LC70_MEAS_SET_MANU_BDSYS_MOD_IDX_SCCC_H          },
        {{LC70_BAND_SCCC,                      LC70_MEAS_SET_WAVE_V}, LC70_MEAS_SET_MANU_BDSYS_MOD_IDX_SCCC_V          },
    };

    // BANDと変調方式で決まる測定設定リストのインデックスマップ
    static const QMap <QPair<int, int>, int> measurementSettingBandModulationMap = {
        {{LC70_BAND_CATV_ALL,         LC70_SETMOD_OFDM  }, LC70_MEAS_SET_MANU_BDSYS_MOD_IDX_CATV_OFDM    },
        {{LC70_BAND_CATV_ALL,         LC70_SETMOD_64QAM }, LC70_MEAS_SET_MANU_BDSYS_MOD_IDX_CATV_QAM     },
        {{LC70_BAND_CATV_ALL,         LC70_SETMOD_256QAM}, LC70_MEAS_SET_MANU_BDSYS_MOD_IDX_CATV_QAM     },
        {{LC70_BAND_CATV_90_470MHZ,   LC70_SETMOD_OFDM  }, LC70_MEAS_SET_MANU_BDSYS_MOD_IDX_CATV_OFDM    },
        {{LC70_BAND_CATV_90_470MHZ,   LC70_SETMOD_64QAM }, LC70_MEAS_SET_MANU_BDSYS_MOD_IDX_CATV_QAM     },
        {{LC70_BAND_CATV_90_470MHZ,   LC70_SETMOD_256QAM}, LC70_MEAS_SET_MANU_BDSYS_MOD_IDX_CATV_QAM     },
        {{LC70_BAND_CATV_470_1002MHZ, LC70_SETMOD_OFDM  }, LC70_MEAS_SET_MANU_BDSYS_MOD_IDX_CATV_OFDM    },
        {{LC70_BAND_CATV_470_1002MHZ, LC70_SETMOD_64QAM }, LC70_MEAS_SET_MANU_BDSYS_MOD_IDX_CATV_QAM     },
        {{LC70_BAND_CATV_470_1002MHZ, LC70_SETMOD_256QAM}, LC70_MEAS_SET_MANU_BDSYS_MOD_IDX_CATV_QAM     },
        {{LC70_BAND_SKYP,             LC70_SETMOD_OFDM  }, LC70_MEAS_SET_MANU_BDSYS_MOD_IDX_SKYP_ISDBT   },
        {{LC70_BAND_SKYP,             LC70_SETMOD_64QAM }, LC70_MEAS_SET_MANU_BDSYS_MOD_IDX_SKYP_OPTICAST},
        {{LC70_BAND_SKYP,             LC70_SETMOD_256QAM}, LC70_MEAS_SET_MANU_BDSYS_MOD_IDX_SKYP_OPTICAST},
    };

    static const int BROADBANDSYSTEM_INDEX = 0; // LC70_MEAS_SET_MANU_BDSYS_MOD_ROW_T 参照(1 番めの項目が放送方式)
    static const int MODULATION_INDEX = 1;      // LC70_MEAS_SET_MANU_BDSYS_MOD_ROW_T 参照(2 番めの項目が変調方式)

    // BANDで閾値のインデックスが決まるか
    if (measurementSettingBandMap.contains(bandId)) {
        index = measurementSettingBandMap.value(bandId);
    }

    // BANDと偏波でインデックスが決まるか
    QPair<int, int> bandPolarizationPair = {bandId, static_cast<int>(polatization)};
    if (measurementSettingBandPolarizationMap.contains(bandPolarizationPair)) {
        index = measurementSettingBandPolarizationMap.value(bandPolarizationPair);
    }

    // BANDと変調方式でインデックスが決まるか
    QPair<int, int> bandModulationPair = {bandId, static_cast<int>(modulation)};
    if (measurementSettingBandModulationMap.contains(bandModulationPair)) {
        index = measurementSettingBandModulationMap.value(bandModulationPair);
    }

    if (index < 0) {
        return -1;
    }

    QVariantList list = setting.toList()[index].toList();

    broadcastSystem = list.at(BROADBANDSYSTEM_INDEX).toInt();

    return list.at(MODULATION_INDEX).toInt();
}

int Correcter::selectCnrMer(LC70_BAND_T bandId, QVariant setting)
{
    // LC70_BAND_T の定義順に対応する LC70_MEAS_SET_CNRMER_IDX_T の enum 値を設定
    static const QMap<LC70_BAND_T, int> table = {
        { LC70_BAND_UNDEF, -1 },                                                    // LC70_BAND_UNDEF
        { LC70_BAND_UHF, LC70_MEAS_SET_CNRMER_IDX_UHF },                            // LC70_BAND_UHF
        { LC70_BAND_BS, LC70_MEAS_SET_CNRMER_IDX_BS },                              // LC70_BAND_BS
        { LC70_BAND_CS, LC70_MEAS_SET_CNRMER_IDX_CS },                              // LC70_BAND_CS
        { LC70_BAND_JCSAT3, LC70_MEAS_SET_CNRMER_IDX_JCSAT3A },                     // LC70_BAND_JCSAT3
        { LC70_BAND_JCSAT4, LC70_MEAS_SET_CNRMER_IDX_JCSAT4B },                     // LC70_BAND_JCSAT4
        { LC70_BAND_SCCB, LC70_MEAS_SET_CNRMER_IDX_SCCB2 },                         // LC70_BAND_SCCB
        { LC70_BAND_SCCC, LC70_MEAS_SET_CNRMER_IDX_SCCC2 },                         // LC70_BAND_SCCC
        { LC70_BAND_CATV_ALL, LC70_MEAS_SET_CNRMER_IDX_CATV_ALL },                  // LC70_BAND_CATV_ALL
        { LC70_BAND_CATV_90_470MHZ, LC70_MEAS_SET_CNRMER_IDX_CATV_90_470MHZ },      // LC70_BAND_CATV_90_470MHZ
        { LC70_BAND_CATV_470_1002MHZ, LC70_MEAS_SET_CNRMER_IDX_CATV_470_1002MHZ },  // LC70_BAND_CATV_470_1002MHZ
        { LC70_BAND_PILOT, LC70_MEAS_SET_CNRMER_IDX_PILOT },                        // LC70_BAND_PILOT
        { LC70_BAND_SKYP, LC70_MEAS_SET_CNRMER_IDX_SKYP },                          // LC70_BAND_SKYP
        { LC70_BAND_LTE, LC70_MEAS_SET_CNRMER_IDX_LTE },                            // LC70_BAND_LTE
        { LC70_BAND_FM, LC70_MEAS_SET_CNRMER_IDX_FM },                              // LC70_BAND_FM
        { LC70_BAND_CATV_UP_STREAM, LC70_MEAS_SET_CNRMER_IDX_CATV_UP_STREAM },      // LC70_BAND_CATV_UP_STREAM
        { LC70_BAND_END, -1 }                                                       // LC70_BAND_END
    };
    static const int SETTING_VALUE_INDEX = 2;   // LC70_MEAS_SET_CNRMER_ROW_T 参照(3 番めの項目が設定値)

    int index = table.value(bandId);
    if (index < 0) {
        return -1;
    }
    QVariantList list = setting.toList()[index].toList();

    return list.at(SETTING_VALUE_INDEX).toInt();
}

int Correcter::getThresholdIndex(int bandId, const int broadcast, const int modulation)
{
    // BANDで決まる閾値のインデックスマップ
    static const QMap<LC70_BAND_T, LC70_MEAS_SET_THRD_IDX_T> thresholdBandMap = {
        {LC70_BAND_UHF,            LC70_MEAS_SET_THRD_IDX_UHF_ISDBT},
        {LC70_BAND_PILOT,          LC70_MEAS_SET_THRD_IDX_CATV_PILOT},
        {LC70_BAND_LTE,            LC70_MEAS_SET_THRD_IDX_LTE},
        {LC70_BAND_FM,             LC70_MEAS_SET_THRD_IDX_FM},
        {LC70_BAND_CATV_UP_STREAM, LC70_MEAS_SET_THRD_IDX_CATV_UP_STREAM},
    };

    // BANDと放送方式で決まる閾値のインデックスマップ
    static const QMap<QPair<LC70_BAND_T, int>, LC70_MEAS_SET_THRD_IDX_T> thresholdBandBroadCastMap = {
        {{LC70_BAND_BS,     LC70_BD_SYS_ISDBS }, LC70_MEAS_SET_THRD_IDX_BS_ISDBS },
        {{LC70_BAND_BS,     LC70_BD_SYS_ISDBS3}, LC70_MEAS_SET_THRD_IDX_BS_ISDBS3},
        {{LC70_BAND_CS,     LC70_BD_SYS_ISDBS }, LC70_MEAS_SET_THRD_IDX_CS_ISDBS },
        {{LC70_BAND_CS,     LC70_BD_SYS_ISDBS3}, LC70_MEAS_SET_THRD_IDX_CS_ISDBS3},
        {{LC70_BAND_JCSAT3, LC70_BD_SYS_DVBS  },   LC70_MEAS_SET_THRD_IDX_JCSAT3_DVBS},
        {{LC70_BAND_JCSAT3, LC70_BD_SYS_DVBS2},  LC70_MEAS_SET_THRD_IDX_JCSAT3_DVBS2},
        {{LC70_BAND_JCSAT4, LC70_BD_SYS_DVBS},   LC70_MEAS_SET_THRD_IDX_JCSAT4_DVBS},
        {{LC70_BAND_JCSAT4, LC70_BD_SYS_DVBS2},  LC70_MEAS_SET_THRD_IDX_JCSAT4_DVBS2},
        {{LC70_BAND_SCCB,   LC70_BD_SYS_DVBS},   LC70_MEAS_SET_THRD_IDX_SCCB_DVBS},
        {{LC70_BAND_SCCB,   LC70_BD_SYS_DVBS2},  LC70_MEAS_SET_THRD_IDX_SCCB_DVBS2},
        {{LC70_BAND_SCCC,   LC70_BD_SYS_DVBS},   LC70_MEAS_SET_THRD_IDX_SCCC_DVBS},
        {{LC70_BAND_SCCC,   LC70_BD_SYS_DVBS2},  LC70_MEAS_SET_THRD_IDX_SCCC_DVBS2}
    };

    // BANDと変調方式で決まる閾値のインデックスマップ
    static const QMap<QPair<LC70_BAND_T, int>, LC70_MEAS_SET_THRD_IDX_T> thresholdBandModulationMap = {
        {{LC70_BAND_CATV_ALL,         LC70_SETMOD_OFDM   }, LC70_MEAS_SET_THRD_IDX_CATV_ISDBT_OFDM    },
        {{LC70_BAND_CATV_ALL,         LC70_SETMOD_64QAM  }, LC70_MEAS_SET_THRD_IDX_CATV_ANNEXC_64QAM   },
        {{LC70_BAND_CATV_ALL,         LC70_SETMOD_256QAM }, LC70_MEAS_SET_THRD_IDX_CATV_ANNEXC_256QAM  },
        {{LC70_BAND_CATV_90_470MHZ,   LC70_SETMOD_OFDM   }, LC70_MEAS_SET_THRD_IDX_CATV_ISDBT_OFDM    },
        {{LC70_BAND_CATV_90_470MHZ,   LC70_SETMOD_64QAM  }, LC70_MEAS_SET_THRD_IDX_CATV_ANNEXC_64QAM   },
        {{LC70_BAND_CATV_90_470MHZ,   LC70_SETMOD_256QAM }, LC70_MEAS_SET_THRD_IDX_CATV_ANNEXC_256QAM  },
        {{LC70_BAND_CATV_470_1002MHZ, LC70_SETMOD_OFDM   }, LC70_MEAS_SET_THRD_IDX_CATV_ISDBT_OFDM    },
        {{LC70_BAND_CATV_470_1002MHZ, LC70_SETMOD_64QAM  }, LC70_MEAS_SET_THRD_IDX_CATV_ANNEXC_64QAM   },
        {{LC70_BAND_CATV_470_1002MHZ, LC70_SETMOD_256QAM }, LC70_MEAS_SET_THRD_IDX_CATV_ANNEXC_256QAM  },
        {{LC70_BAND_SKYP,             LC70_SETMOD_OFDM   }, LC70_MEAS_SET_THRD_IDX_CATV_SKYP_OFDM     },
        {{LC70_BAND_SKYP,             LC70_SETMOD_256QAM }, LC70_MEAS_SET_THRD_IDX_CATV_SKYP_256QAM   }
    };

    // BANDから閾値のインデックスが決まるものか
    if (thresholdBandMap.contains(static_cast<LC70_BAND_T>(bandId))) {
        return thresholdBandMap.value(static_cast<LC70_BAND_T>(bandId));
    }

    // BANDと放送方式から閾値のインデックスが決まるものか
    QPair<LC70_BAND_T, int> bandBroadCastPair = {static_cast<LC70_BAND_T>(bandId), broadcast};
    if (thresholdBandBroadCastMap.contains(bandBroadCastPair)){
        return thresholdBandBroadCastMap.value(bandBroadCastPair);
    }

    // BANDと変調方式から閾値のインデックスが決まるものか
    QPair<LC70_BAND_T, int> bandModulationPair = {static_cast<LC70_BAND_T>(bandId), modulation};
    if (thresholdBandModulationMap.contains(bandModulationPair)) {
        return thresholdBandModulationMap.value(bandModulationPair);
    }

    // 対応する閾値データが無い場合
    qDebug("There is no corresponding threshold information");
    return -1;
}

QString Correcter::powerSupplyIcon(int powerSupplyValue)
{
    // 給電情報と設定するアイコン名のマップ
    static const QMap<int, QString> powerSupplyMap = {
        {LC70_POWER_SUPPLY_OFF,   ""},
        {LC70_POWER_SUPPLY_11V,   "icon_power_11v.png"},
        {LC70_POWER_SUPPLY_15V,   "icon_power_15v.png"},
        {LC70_POWER_SUPPLY_11V_P, "icon_power_11v_p.png"},
        {LC70_POWER_SUPPLY_15V_P, "icon_power_15v_p.png"}
    };

    return powerSupplyMap.value(powerSupplyValue, "");
}

QString Correcter::displayVoltage(int voltage)
{
    // 電圧値から表示する値を取得
    if (voltage < MINIMUM_DC_VALUE) {
        // 最小表示値 以下の場合
        return QString("%1").arg(float(MINIMUM_DC_VALUE), 2, 'f', 1).rightJustified(2, '0');
    }

    if (voltage > MAXIMUM_DC_VALUE) {
        // 最大表示値 以上の場合
        float dcMax = MAXIMUM_DC_VALUE / float(100);
        return QString(FORMAT_MAXIMUM_DC).arg(QString("%1").arg(dcMax, 2, 'f', 1).rightJustified(2, '0'));
    }

    float voltageValue = voltage / float(100);
    return QString("%1").arg(voltageValue, 2, 'f', 1).rightJustified(2, '0');
}

QString Correcter::displayOpticalValue(int opticast)
{
    if (opticast < MINIMUM_OPTICAL_VALUE) {
        // 最小表示値 以下の場合
        float opticalMin = MINIMUM_OPTICAL_VALUE / float(100);
        return QString(FORMAT_MINIMUM_OPTICAST).arg(QString("%1").arg(opticalMin, 2, 'f', 1).rightJustified(2,'0'));
    }

    if (opticast > MAXIMUM_OPTICAL_VALUE) {
        // 最大表示値 以上の場合
        float opticalMax = MAXIMUM_OPTICAL_VALUE / float(100);
        return QString(FORMAT_MAXIMUM_OPTICAST).arg(QString("%1").arg(opticalMax, 2, 'f', 1).rightJustified(2,'0'));
    }

    float opticalValue = opticast / float(100);
    if(QString("%1").arg(opticalValue, 2, 'f', 1).rightJustified(2,'0') == "-0.0"){
        return QString("0.0");
    }
    return QString("%1").arg(opticalValue, 2, 'f', 1).rightJustified(2,'0');
}

bool Correcter::displayLevel(const BandTabIndex &bandTab, const float32_t &levelValue, QString &displayValue)
{
    if (bandTab >= BAND_TAB_UHF && bandTab <= BAND_TAB_PILOT) {
        float32_t minimum = LEVEL_RANGE[bandTab].MinimumLevel;
        float32_t maximum = LEVEL_RANGE[bandTab].MaximumLevel;

        // 初期表示値(不定値)
        if (levelValue == UNDEFINED_VALUE) {
            displayValue = QString(INITIAL_LEVEL);
            return true;
        }

        // 最小値未満、最大値超過なら最小値あるいは最大値に不等号を付けて表示
        if (levelValue < minimum) {
            displayValue = QString("<%1").arg(minimum, 3, 'f', 1);
        } else if (levelValue > maximum) {
            displayValue = QString(">%1").arg(maximum);
        } else {
            // 初期値以外で 0 未満の場合
            if (levelValue < 0) {
                return false;
            }

            // 範囲内
            displayValue = QString("%1").arg(levelValue, 3, 'f', 1).rightJustified(4, '0');
        }

        return true;
    }

    return false;
}

BandTabIndex Correcter::convertBandTabIndex(const LC70_BAND_T &band, const QString &polarization)
{
    BandTabIndex index = BAND_TAB_COUNT;
    switch (band) {
    case LC70_BAND_UHF:
        index = BAND_TAB_UHF;
        break;
    case LC70_BAND_BS:
    {
        if (polarization == QString(RIGHT_HANDED_POLARIZE)) {
            index = BAND_TAB_BS_R;
        } else if (polarization == QString(LEFT_HANDED_POLARIZE)) {
            index = BAND_TAB_BS_L;
        }
        break;
    }
    case LC70_BAND_CS:
    {
        if (polarization == QString(RIGHT_HANDED_POLARIZE)) {
            index = BAND_TAB_CS_R;
        } else if (polarization == QString(LEFT_HANDED_POLARIZE)) {
            index = BAND_TAB_CS_L;
        }
        break;
    }
    case LC70_BAND_JCSAT3:
    {
        if (polarization == QString(HORIZONTALLY_POLARIZE)) {
            index = BAND_TAB_JCSAT3A_H;
        } else if (polarization == QString(VERTICALLY_POLARIZE)) {
            index = BAND_TAB_JCSAT3A_V;
        }
        break;
    }
    case LC70_BAND_JCSAT4:
    {
        if (polarization == QString(HORIZONTALLY_POLARIZE)) {
            index = BAND_TAB_JCSAT4B_H;
        } else if (polarization == QString(VERTICALLY_POLARIZE)) {
            index = BAND_TAB_JCSAT4B_V;
        }
        break;
    }
    case LC70_BAND_SCCB:
    {
        if (polarization == QString(HORIZONTALLY_POLARIZE)) {
            index = BAND_TAB_SCCB2_H;
        } else if (polarization == QString(VERTICALLY_POLARIZE)) {
            index = BAND_TAB_SCCB2_V;
        }
        break;
    }
    case LC70_BAND_SCCC:
    {
        if (polarization == QString(HORIZONTALLY_POLARIZE)) {
            index = BAND_TAB_SCCC2_H;
        } else if (polarization == QString(VERTICALLY_POLARIZE)) {
            index = BAND_TAB_SCCC2_V;
        }
        break;
    }
        break;
    case LC70_BAND_CATV_ALL:
    case LC70_BAND_CATV_90_470MHZ:
    case LC70_BAND_CATV_470_1002MHZ:
        index = BAND_TAB_CATV_ALL;
        break;
    case LC70_BAND_PILOT:
        index = BAND_TAB_PILOT;
        break;
    case LC70_BAND_SKYP:
        index = BAND_TAB_SKY_PERFECTV;
        break;
    case LC70_BAND_LTE:
        index = BAND_TAB_LTE;
        break;
    case LC70_BAND_FM:
        index = BAND_TAB_FM;
        break;
    case LC70_BAND_CATV_UP_STREAM:
        index = BAND_TAB_CATV_UPSTREAM;
        break;
    default:
        break;
    }

    return index;
}

QPair<LC70_BAND_T, int> Correcter::convertBandId(const BandTabIndex bandTabIndex)
{
    static const QMap<BandTabIndex, QPair<LC70_BAND_T, int>> bandIdMap = {
        {BAND_TAB_UHF,           {LC70_BAND_UHF            , -1}},
        {BAND_TAB_BS_R,          {LC70_BAND_BS             , LC70_MEAS_SET_WAVE_R}},
        {BAND_TAB_BS_L,          {LC70_BAND_BS             , LC70_MEAS_SET_WAVE_L}},
        {BAND_TAB_CS_R,          {LC70_BAND_CS             , LC70_MEAS_SET_WAVE_R}},
        {BAND_TAB_CS_L,          {LC70_BAND_CS             , LC70_MEAS_SET_WAVE_L}},
        {BAND_TAB_CATV_ALL,      {LC70_BAND_CATV_ALL       , -1}},
        {BAND_TAB_SKY_PERFECTV,  {LC70_BAND_SKYP           , -1}},
        {BAND_TAB_CATV_UPSTREAM, {LC70_BAND_CATV_UP_STREAM , -1}},
        {BAND_TAB_JCSAT3A_H,     {LC70_BAND_JCSAT3         , LC70_MEAS_SET_WAVE_H}},
        {BAND_TAB_JCSAT3A_V,     {LC70_BAND_JCSAT3         , LC70_MEAS_SET_WAVE_V}},
        {BAND_TAB_JCSAT4B_H,     {LC70_BAND_JCSAT4         , LC70_MEAS_SET_WAVE_H}},
        {BAND_TAB_JCSAT4B_V,     {LC70_BAND_JCSAT4         , LC70_MEAS_SET_WAVE_V}},
        {BAND_TAB_SCCB2_H,       {LC70_BAND_SCCB           , LC70_MEAS_SET_WAVE_H}},
        {BAND_TAB_SCCB2_V,       {LC70_BAND_SCCB           , LC70_MEAS_SET_WAVE_V}},
        {BAND_TAB_SCCC2_H,       {LC70_BAND_SCCC           , LC70_MEAS_SET_WAVE_H}},
        {BAND_TAB_SCCC2_V,       {LC70_BAND_SCCC           , LC70_MEAS_SET_WAVE_V}},
        {BAND_TAB_LTE,           {LC70_BAND_LTE            , -1}},
        {BAND_TAB_FM,            {LC70_BAND_FM             , -1}},
        {BAND_TAB_PILOT,         {LC70_BAND_PILOT          , -1}},
    };

    return bandIdMap.value(bandTabIndex, {LC70_BAND_UNDEF, -1});
}

bool Correcter::minimumLevel(const BandTabIndex &bandTab, float32_t &minimumLevel)
{
    if (bandTab >= BAND_TAB_UHF && bandTab <= BAND_TAB_PILOT) {
        minimumLevel = LEVEL_RANGE[bandTab].MinimumLevel;
        return true;
    }

    return false;
}

bool Correcter::maximumLevel(const BandTabIndex &bandTab, float32_t &maximumLevel)
{
    if (bandTab >= BAND_TAB_UHF && bandTab <= BAND_TAB_PILOT) {
        maximumLevel = LEVEL_RANGE[bandTab].MaximumLevel;
        return true;
    }

    return false;
}

uint32_t Correcter::selectModulationMethod(const BandTabIndex &bandTabIndex, uint8_t broadcastSystem)
{
    static const QMap<BandTabIndex, uint32_t> modulationMethod = {
        { BAND_TAB_UHF,            LC70_SETMOD_OFDM   },
        { BAND_TAB_BS_R,           LC70_SETMOD_UNDEF  },
        { BAND_TAB_BS_L,           LC70_SETMOD_UNDEF  },
        { BAND_TAB_CS_R,           LC70_SETMOD_UNDEF  },
        { BAND_TAB_CS_L,           LC70_SETMOD_UNDEF  },
        { BAND_TAB_CATV_ALL,       LC70_SETMOD_64QAM  },
        { BAND_TAB_SKY_PERFECTV,   LC70_SETMOD_256QAM },
        { BAND_TAB_CATV_UPSTREAM,  LC70_SETMOD_CW     },
        { BAND_TAB_JCSAT3A_H,      LC70_SETMOD_UNDEF  },
        { BAND_TAB_JCSAT3A_V,      LC70_SETMOD_UNDEF  },
        { BAND_TAB_JCSAT4B_H,      LC70_SETMOD_UNDEF  },
        { BAND_TAB_JCSAT4B_V,      LC70_SETMOD_UNDEF  },
        { BAND_TAB_SCCB2_H,        LC70_SETMOD_UNDEF  },
        { BAND_TAB_SCCB2_V,        LC70_SETMOD_UNDEF  },
        { BAND_TAB_SCCC2_H,        LC70_SETMOD_UNDEF  },
        { BAND_TAB_SCCC2_V,        LC70_SETMOD_UNDEF  },
        { BAND_TAB_LTE,            LC70_SETMOD_UNDEF  },
        { BAND_TAB_FM,             LC70_SETMOD_UNDEF  },
        { BAND_TAB_PILOT,          LC70_SETMOD_CW     },
    };

    // CATV ALL で放送方式が ISDBT の場合変調方式は OFDM とする
    if (bandTabIndex == BAND_TAB_CATV_ALL && broadcastSystem == LC70_BD_SYS_ISDBT) {
        return LC70_SETMOD_OFDM;
    }

    return modulationMethod.value(bandTabIndex, LC70_SETMOD_UNDEF);
}

int Correcter::displayModulationPrefixMode(const int broadcastSystem)
{
    // 放送方式から変調方式と変調方式表示レイアウトの値を取得
    static const QMap<int, int> DisplayModulationAndPrefixModeMap = {
        {LC70_BD_SYS_ISDBT,    DISPLAY_MODULATION_PREFIX_SEPARATE},
        {LC70_BD_SYS_ISDBS,    DISPLAY_MODULATION_PREFIX_TOGETHER},
        {LC70_BD_SYS_ISDBS3,   DISPLAY_MODULATION_PREFIX_TOGETHER},
        {LC70_BD_SYS_DVBS,     DISPLAY_MODULATION_PREFIX_TOGETHER},
        {LC70_BD_SYS_DVBS2,    DISPLAY_MODULATION_PREFIX_TOGETHER},
        {LC70_BD_SYS_ANNEXC,    DISPLAY_MODULATION_PREFIX_SEPARATE},
        {LC70_BD_SYS_ANNEXB,   DISPLAY_MODULATION_PREFIX_SEPARATE},
        {LC70_BD_SYS_OPTICAST, DISPLAY_MODULATION_PREFIX_TOGETHER},
    };

    return DisplayModulationAndPrefixModeMap.value(broadcastSystem, DISPLAY_MODULATION_PREFIX_TOGETHER);
}
