﻿/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef CORRECTER_H
#define CORRECTER_H

#include <QList>
#include <QString>
#include "channeldefinitionitem.h"
#include "constant.h"

extern "C" {
#include "liblc70.h"
}

/**
 * @brief 表示内容補正用の静的メソッドを定義したクラス
 */
class Correcter
{
public:
    Correcter();

    /**
     * @brief 表示する BAND の名称を返す
     * @param band BAND(LC70_BAND_T の定義値)
     * @param shortName 略称有無
     * @return 表示する BAND の名称
     */
    static QString displayBandName(int band, bool shortName = false);

    /**
     * @brief BAND の 名称から ID を返す
     * @param band 表示する BAND の名称
     * @return BAND(LC70_BAND_T の定義値)
     */
    static int bandIdFromDisplayBand(QString band);

    /**
     * @brief 表示する BAND のグループ名を返す
     * @param band BAND(LC70_BAND_T の定義地値)
     * @return 表示する BAND のグループ名
     */
    static QString displayBandGroupName(int band);
    /**
     * @brief 表示するチャンネルの名称を返す
     * @param channelNumber チャンネル名
     * @return 表示するチャンネルの名称
     */
    static QString displayChannelName(QString channelNumber);

    /**
     * @brief 引数に対応した中心周波数を取得
     * @param item チャンネル定義情報
     * @param modulation 変調方式
     * @return 引数に対応した周波数
     */
    static unsigned int centerFrequency(ChannelDefinitionItem item, unsigned int modulation);

    /**
     * @brief frequency
     * @param centerFrequency
     * @param localFrequency
     * @return
     */
    static unsigned int frequency(unsigned int centerFrequency, unsigned int localFrequency = 0);

    /**
     * @brief 周波数情報を表示する単位に変換する
     * @param band BAND種別
     * @param frequency 周波数情報 (単位はkHzを想定)
     * @param integerOnly 整数のみか否か
     * @return 表示する周波数情報
     */
    static QString displayFrequency(int band, unsigned int frequency, bool integerOnly = false);

    /**
     * @brief 局発周波数情報を表示する単位に変換する
     * @param localFrequency 局発周波数情報 (単位はkHzを想定)
     * @return 表示する局発周波数情報
     */
    static QString displayLocalFrequency(unsigned int localFrequency);
    /**
     * @brief 表示する階層を返す
     * @param layer 測定結果から取得した階層
     * @return 表示する階層
     */
    static QString displayHierarchy(unsigned int layer);

    /**
     * @brief 表示する変調方式を返す
     * @param modulation 測定結果から取得した変調方式の値
     * @return 表示する変調方式
     */
    static QString displayModulation(unsigned int modulation);

    /**
     * @brief 変調方式の前につく接頭句を返す
     * @param bandId バンドID (LC70_BAND_xxxの値)
     * @param broadcastSystem 放送方式 (LC70_BD_SYS_xxxxx の値)
     * @param modulation 変調方式
     * @return 変調方式の前につく接頭句
     */
    static QString modulationPrefix(const int broadcastSystem, const QString &modulation);

    /**
     * @brief 変調方式と接頭句をどのように表示するかのモードを取得する
     * @param broadcastSystem 放送方式 (LC70_BD_SYS_xxxxxの値)
     * @return 変調方式と接頭句をどのように表示するか
     */
    static int displayModulationPrefixMode(const int broadcastSystem);

    /**
     * @brief 変調方式の文字列に対応した値を返す
     * @param modulation 変調方式の文字列
     * @return 変調方式の値 (LC70_SETMOD_xxxの値)
     */
    static unsigned int modulationValue(QString modulation);

    /**
     * @brief 放送方式の文字列に対応した値を返す
     * @param broadcastSystem 放送方式の文字列
     * @return 放送方式の値 (LC70_BD_SYS_xxxxの値)
     */
    static unsigned int broadcastSystemValue(const QString &broadcastSystem);

    /**
     * @brief 含まれる変調方式のリストを返す
     * @param modulationBit
     * @return　変調方式の値のリスト
     */
    static void getModulationBitList(unsigned int modulationBit, QList<unsigned int> &list);

    /**
     * @brief 給電状態を返す
     * @param dcOutVal 給電状態の値
     * @return
     */
    static QString displayPowerSupply(int dcOutVal);

    /**
     * @brief 放送方式を返す
     * @param broadbandSystem
     * @return
     */
    static QString displayBroadbandSystem(unsigned int broadbandSystem);

    /**
     * @brief ライブラリ定義値から偏波面の文字列を返す
     * @param waveNumber ライブラリの定義値 LC70_MEAS_SET_WAVE_xxxの値
     * @return 偏波面の文字列
     */
    static QString convertWaveToPolarization(const int waveNumber);

    /**
     * @brief 偏波面の文字列からをライブラリ定義値返す
     * @param polarization 偏波面の文字列
     * @return ライブラリの定義値 LC70_MEAS_SET_WAVE_xxxの値
     */
    static int convertPolarizationToWave(const QString &polarization);

    /**
     * @brief 偏波を返す
     * @param polarization 偏波の値
     * @return
     */
    static QString displayPolarization(const QString &polarization);

    /**
     * @brief 衛星放送かどうかを判断する
     * @param band BAND(LC70_BAND_Tの定義値)
     * @return 衛星放送かどうか true: 衛星放送, false: 衛星放送ではない
     */
    static bool isSatelliteBroadCasting(int band);

    /**
     * @brief 階層/偏波を表示するかどうかを判断する
     * @param band BAND(LC70_BAND_Tの定義値)
     * @oparam modulationValue 変調方式
     * @return 階層/偏波を表示するかどうか  true: 表示する, false: 表示しない
     */
    static bool isDisplayLevelHierarchyPolarization(int band, unsigned int modulationValue);

    /**
     * @brief 表示するMERを返す
     * @param bandId Band の ID
     * @param broadcastSystem 放送方式
     * @param modulation 変調方式
     * @param mer MERの実値
     * @return 表示する MER
     */
    static QString displayMer(const int bandId, const int broadcastSystem, const int modulation, const float32_t mer);

    /**
     * @brief 表示するBERを返す
     * @param ber BER の実値
     * @return 表示する BER
     */
    static QString displayBer(const float32_t ber);

    /**
     * @brief 選局対象の局部発振周波数リスト取得
     * @param modelData 局部発振周波数リスト(全てのデータ)
     * @param bandId Band の ID
     * @param polarization 偏波
     * @param localOscillatorList 局部発振周波数リスト(選局対象のみ)
     * @return true: 成功、 false: 失敗
     */
    static bool getLocalOscillatorList(const QVariant &modelData, const int bandId, const QString &polarization,
                                       QList<uint32_t> &localOscillatorList);

    /**
     * @brief 引数の番号に対応した偏波面の文字列を取得
     * @param polarizeNum 偏波面の番号
     * @param polarizeString 偏波面の文字列[out]
     * @return true: 成功、 false: 失敗
     */
    static QString convertPolarizeNumToString(const int polarizeNum);

    /**
     * @brief 偏波取得
     * @param bandId Band の ID
     * @param polarizationIndex 偏波インデックス(0:右旋・水平、1:左旋、垂直)
     * @return 偏波
     */
    static QString polarization(const int &bandId, const int &polarizationIndex);

    /**
     * @brief 放送方式取得
     * @param bandId Band の ID
     * @param polarizationIndex 偏波インデックス(0:右旋・水平、1:左旋、垂直)
     * @param modulation 変調方式
     * @note 偏波がない Band の場合は 0 を指定する
     * @return
     */
    static uint8_t broadcastSystem(const int &bandId, const int &polarizationIndex = 0,
                                   const uint32_t &modulation = 0);

    /**
     * @brief 小数点桁数取得
     * @param value 数値
     * @note 整数であっても 1 を返す(1.0などの表示用)
     * @return 小数点桁数
     */
    static int getDecimalDigits(const uint32_t value);

    /**
     * @brief BAND ID から変調方式を表示か、非表示かを取得
     * @param bandId BAND ID
     * @return 変調方式を表示,非表示情報 True: 表示, False: 非表示
     */
    static bool isModulationVisible(const int &bandId);

    /**
     * @brief 放送方式(測定条件)取得
     * @param bandId Band の ID
     * @param modulation 変調方式
     * @return 放送方式(測定条件)
     */
    static uint16_t getBroadcastSystemCondition(const int bandId, const QString &modulation);

    /**
     * @brief 変調方式(測定条件)取得
     * @param bandId Band の ID
     * @param broadcastSystem 放送方式
     * @param polarization 偏波
     * @param modulation 変調方式/放送方式
     * @param setting マニュアル測定 放送方式, 変調方式選択ファイルの内容(ModelOperator 参照)
     * @return 変調方式(測定条件)
     * @note Band によって固定値を設定する必要があるため、それを判定する
     */
    static uint16_t getModulationCondition(int bandId, uint16_t &broadcastSystem, unsigned int polarization, unsigned int modulation, QVariant setting);

    /**
     * @brief CNR/MER 選択
     * @param bandId band の ID
     * @param setting CNR/MER切替設定ファイルの内容(ModelOperator 参照)
     * @return LC70_MEAS_SET_SEL_CNR(0) LC70_MEAS_SET_SEL_MER(1) LC70_MEAS_SET_SEL_UNUSE(255) ERROR(-1)
     */
    static int selectCnrMer(LC70_BAND_T bandId, QVariant setting);

    /**
    * @brief BAND_IDからチャンネル詳細情報各項目の表示可否のリストを取得する
    * @param bandId バンドID
    * @return チャンネル詳細情報各項目の表示可否リスト
    */
    static QList<QVariant> getDisplayChannelDetail(int bandId);

    /**
     * @brief getThresholdIndex 閾値Indexの取得
     * @param bandId BAND の ID
     * @param broadcast 放送方式
     * @param modulation 変調方式
     * @return 閾値のIndex(LC70_MEAS_SET_THRD_IDX_xxxの値)
     */
    static int getThresholdIndex(const int bandId, const int broadcast, const int modulation);

    /**
     * @brief MER/CNの上限下限のリストを設定
     * @param merCnLimitList MER/CN上限下限リスト
     */
    static void setMerCnLimitList(const QVariantList merCnLimitList);

    /**
     * @brief MERの値域を返す
     * @param bandId Band の ID
     * @param broadcastSystem 放送方式
     * @param modulation      変調方式
     * @return MERの値域(first:下限、seccond:上限)
     */
    static QPair<int, int> merRange(const int bandId, const int broadcastSystem, const int modulation);

    /**
     * @brief 給電アイコン名の取得
     * @param powerSupplyValue 給電設定の値 (LC70_POWER_SUPPLY_xxx の値)
     * @return 給電アイコン名
     */
    static QString powerSupplyIcon(int powerSupplyValue);

    /**
     * @brief 電圧値から画面で表示する電圧値を返す
     * @param voltage 電圧値
     * @return 表示 電圧値
     * @note DC測定画面で使用する
     */
    static QString displayVoltage(int voltage);

    /**
     * @brief 画面に表示する光パワーを返す
     * @param opticast 光パワーの値
     * @return 光パワーの表示値
     * @note 光測定画面で使用する
     */
    static QString displayOpticalValue(int opticast);

    /**
     * @brief Level の表示値を返す
     * @param bandTab Band の種類
     * @param levelValue Level の値
     * @param displayValue 表示する Level の値
     * @return Level の表示値の取得の成否
     */
    static bool displayLevel(const BandTabIndex &bandTab, const float32_t &levelValue, QString &displayValue);

    /**
     * @brief Band 定義と偏波から Band の種類を返す
     * @param band Band
     * @param polarization 偏波
     * @return Band の種類
     */
    static BandTabIndex convertBandTabIndex(const LC70_BAND_T &band, const QString &polarization);

    /**
     * @brief Band の種類から Band定義を返す
     * @param bandTabIndex Band の種類
     * @return Band と 偏波
     */
    static QPair<LC70_BAND_T, int> convertBandId(const BandTabIndex bandTabIndex);

    /**
     * @brief Band 毎の Level の最小値を返す
     * @param bandTab Band の種類
     * @param minimumLevel Band 毎の Level の最小値
     * @return Band 毎の Level の最小値の取得の成否
     */
    static bool minimumLevel(const BandTabIndex &bandTab, float32_t &minimumLevel);

    /**
     * @brief Band 毎の Level の最大値を返す
     * @param bandTab Band の種類
     * @param minimumLevel Band 毎の Level の最大値
     * @return Band 毎の Level の最大値の取得の成否
     */
    static bool maximumLevel(const BandTabIndex &bandTab, float32_t &maximumLevel);

    /**
     * @brief マニュアル測定時の変調方式を返す
     * @param bandTab Band の種類
     * @param broadcastSystem 放送方式(CATV ALL のときのみ指定)
     * @return マニュアル測定時の変調方式
     */
    static uint32_t selectModulationMethod(const BandTabIndex &bandTabIndex, uint8_t broadcastSystem = LC70_BD_SYS_NONE);
};

#endif // DISPLAYDATACORRECTER_H
