/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include "usboperation.h"
#include <QProcess>
#include <QDir>
#include <QFile>
#include <QDirIterator>
#include <sys/statvfs.h>
#include <QDebug>
#include <QElapsedTimer>

static QString _singleMeasurementPath; //!< シングル測定結果の保存パス
static QString _multiMeasurementPath;  //!< マルチ測定結果の保存パス
static QString _loggerMeasurementPath; //!< データロガー測定結果の保存パス
static QString _singleMeasurementUsbPath; //!< シングル測定結果のUSBコピー先パス
static QString _multiMeasurementUsbPath;  //!< マルチ測定結果のUSBコピー先パス
static QString _loggerMeasurementUsbPath;  //!< データロガー測定結果のUSBコピー先パス

static QString _measureDataFileListPath;  //!< 測定データファイルリストのパス
static QString _measureDataFileListFileName;  //!< 測定データファイルリストのファイル名
static constexpr const char *CSV_FILE_EXTENSION = ".csv";   //!< 設定ファイルの拡張子(CSV)

UsbOperation::UsbOperation()
{
}

bool UsbOperation::mount()
{
    // USB接続チェック TBD
    // USBをマウントする
    QProcess *process = new QProcess();
    int ret = process->execute(USB_MOUNT_COMMAND);

    if (ret != 0) {
        // プロセスエラー
        delete process;
        return false;
    }

    delete process;
    return true;
}

bool UsbOperation::umount()
{
    // USB接続チェック TBD
    // USBをアンマウントする
    QProcess *process = new QProcess();
    int ret = process->execute(USB_UMOUNT_COMMAND);

    if (ret < 0) {
        // プロセスエラー
        delete process;
        return false;
    }

    delete process;
    return true;
}

UsbErrorType UsbOperation::format()
{
    // USB接続チェック TBD
    // USBをアンマウントする
    if (!umount()) {
        // アンマウント失敗
        return UsbErrorType::FORMAT_ERROR;
    }

    // USBをフォーマットする
    QProcess *process = new QProcess();
    int ret = process->execute(USB_FORMAT_COMMAND);

    if (ret != 0) {
        // プロセスエラー
        delete process;
        return UsbErrorType::FORMAT_ERROR;
    }

    // USBをマウント
    if (!mount()) {
        // マウント失敗
        delete process;
        return UsbErrorType::FORMAT_ERROR;
    }

    process->execute(SYNC_COMMAND);
    delete process;
    return NON_ERROR;
}

qint64 UsbOperation::freeSpace(void)
{
    // 空き容量を確認
    struct statvfs statResult;

    if (0 != statvfs(USB_PATH.toUtf8().constData(), &statResult)) {
        // 取得失敗
        return false;
    }

    // 全体容量
    quint64 totalSize = static_cast<quint64>(statResult.f_frsize) * static_cast<quint64>(statResult.f_blocks);
    qDebug() << "totalSize = " << totalSize;
    // 空き容量
    quint64 freeSize = static_cast<quint64>(statResult.f_frsize) * static_cast<quint64>(statResult.f_bavail);
    qDebug() << "freeSize = " << freeSize;
    // 使用容量
    quint64 usedSize = totalSize - freeSize;
    qDebug() << "usedSize = " << usedSize;
    // 使用率
    int usedRate = 0;

    if (totalSize != 0) {
        usedRate = qRound(static_cast<double>(usedSize) / static_cast<double>(totalSize) * 100);
    }

    qDebug() << "usedRate = " << usedRate << "%";

    // 閾値チェック TBD
    return freeSize;
}

quint64 UsbOperation::getFreeSpaceSize(void)
{
    // USB接続チェック TBD
    struct statvfs statResult;

    if (0 != statvfs(USB_PATH.toUtf8().constData(), &statResult)) {
        // 取得失敗
        return -1;
    }

    // 空き容量
    quint64 freeSize = static_cast<quint64>(statResult.f_frsize) * static_cast<quint64>(statResult.f_bavail);

    return freeSize;
}

UsbErrorType UsbOperation::checkCopyable(const QStringList targetPathList, bool isFile)
{
    UsbErrorType error = UsbErrorType::NON_ERROR;

    qint64 targetTotalSize = 0;

    if (targetPathList.count() == 0) {
        return UsbErrorType::COPY_FILE_ERROR;
    }

    foreach (QString targetFilePath, targetPathList) {
        // コピー元チェック
        if (isFile) {
            error = checkExistsFile(targetFilePath);
        } else {
            error = checkExistsDirectory(targetFilePath);
        }

        if (error != UsbErrorType::NON_ERROR) {
            return error;
        }

        // コピー対象のファイルのサイズを取得する
        targetTotalSize += copyTargetSize(targetFilePath);
    }

    // USBサイズチェック
    if (freeSpace() < targetTotalSize) {
        error = UsbErrorType::SPACE_ERROR;
    }

    if (error != UsbErrorType::NON_ERROR) {
        return error;
    }

    return error;
}

qint64 UsbOperation::copyTargetSize(QString targetFilePath)
{
    // コピー対象のファイルサイズを取得する
    qint64 targetSize = 0;

    QDir targetDirectory(targetFilePath);

    // 配下のファイルのサイズを取得
    QDir::Filters fileFilter = QDir::Files | QDir::System | QDir::Hidden;

    for (QString filePath : targetDirectory.entryList(fileFilter)) {
        QFileInfo fileInfo(targetDirectory, filePath);
        targetSize += fileInfo.size();
        qDebug() << "fileInfo file name : " << fileInfo.fileName();
        qDebug() << "fileInfo file Path : " << fileInfo.filePath();
        qDebug() << "fileInfo file exist : " << fileInfo.exists();
    }

    // 配下のディレクトリのサイズを取得
    QDir::Filters directoryFilter = QDir::Dirs | QDir::NoDotAndDotDot | QDir::System | QDir::Hidden;

    for (QString childDirectoryPath : targetDirectory.entryList(directoryFilter)) {
        targetSize += copyTargetSize(targetFilePath + QDir::separator() + childDirectoryPath);
    }

    return targetSize;
}

UsbErrorType UsbOperation::copyToUsb(const UsbDataType type, const QString &originalDirPath, const bool &compress, const QString& password)
{
    UsbErrorType error = UsbErrorType::NON_ERROR;

    QStringList originalPathSplit = originalDirPath.split("/");

    // コピー先のチェック
    QString copyDirPath = getUsbTargetDirPath(type);

    // コピー先のパスに<YYYYMMDD>/<CSV or PNG>/<CSV or PNG FileName> が付く


    // 測定結果の時は /日付/拡張子 を追加
    if ( (type == MEASUREMENT_RESULT_SINGLE) ||
         (type == MEASUREMENT_RESULT_MULTI) ||
         (type == DATA_LOGGER_RESULT) ) {
        copyDirPath = copyDirPath + ("/") + originalPathSplit[originalPathSplit.count() - 3] + ("/") + originalPathSplit[originalPathSplit.count() - 2];
    }

    if (copyDirPath.isEmpty()) {
        return UsbErrorType::COPY_EXEC_ERROR;
    }

    QProcess *process = new QProcess();

    // コピー先の親フォルダがないときは作成
    QString parentCopyDirPath = getUsbTargetParentDirPath(type);

    // 測定結果の時は /日付/拡張子 を追加
    if ( (type == MEASUREMENT_RESULT_SINGLE) ||
         (type == MEASUREMENT_RESULT_MULTI) ) {
        parentCopyDirPath = parentCopyDirPath + ("/") + originalPathSplit[originalPathSplit.count() - 3] + ("/") + originalPathSplit[originalPathSplit.count() - 2];
    } else if (type == DATA_LOGGER_RESULT) {
        parentCopyDirPath = parentCopyDirPath + ("/") + originalPathSplit[originalPathSplit.count() - 2];
    }

    if (parentCopyDirPath.isEmpty()) {
        delete process;
        return UsbErrorType::COPY_EXEC_ERROR;
    }

    if (!QDir(parentCopyDirPath).exists()) {
        if (process->execute(CREATE_DIR_COMMAND + parentCopyDirPath) != 0) {
            delete process;
            return UsbErrorType::COPY_EXEC_ERROR;
        }
    }

    // コピー実施
    if (type == USER_MEASUREMENT_SETTING) {
        error = executeDirCopy(originalDirPath, copyDirPath, "", compress, password, USB_MEMORY);
    } else {
        error = executeDirCopy(originalDirPath, parentCopyDirPath, "", compress, password, USB_MEMORY);
    }

    if (error != UsbErrorType::NON_ERROR) {
        return error;
    }

    // 測定結果の時は測定データ管理ファイルもコピー
    if ( (type == MEASUREMENT_RESULT_SINGLE) ||
         (type == MEASUREMENT_RESULT_MULTI) ||
         (type == DATA_LOGGER_RESULT) ) {
        QString originalFilePath = _measureDataFileListPath + ("/") + _measureDataFileListFileName;
        error = executeDirCopy(originalFilePath, RESULT_DATA_PATH, "", compress, password, USB_MEMORY);
    }

    process->execute(SYNC_COMMAND);
    delete process;
    return error;
}


UsbErrorType UsbOperation::copyMeasureResultToUsb(const QString &source, const QString &destination, const QStringList &excludedFiles)
{
    QElapsedTimer timer;
    UsbErrorType result = UsbErrorType::NON_ERROR;

    timer.start();

    // 全てコピー
    QString command = QString("/bin/sh -c \"cp -r %1/* %2\"").arg(source).arg(destination);

    int exitCode = QProcess::execute(command);
    QProcess::execute(SYNC_COMMAND);
    qDebug() << QString("copy all : %1ms").arg(timer.elapsed());

    if (excludedFiles.count() > 0) {
        // コピー先からコピー対象外のファイルを削除
        command = QString("rm -f");
        for (int i = 0; i < excludedFiles.count(); ++i) {
            command += QString(" ");
            // コピー先のパスに付け替える
            command += replaceSourceToDestination(excludedFiles[i], destination);
        }
        qDebug() << QString("command: %1").arg(command);
        exitCode = QProcess::execute(command);
        QProcess::execute(SYNC_COMMAND);
        qDebug() << QString("remove exclude files : %1ms").arg(timer.elapsed());

        // コピー先の空のディレクトリの削除
        command = QString("find %1 -type d -empty -delete").arg(destination);
        qDebug() << QString("command: %1").arg(command);
        exitCode = QProcess::execute(command);
        QProcess::execute(SYNC_COMMAND);
        qDebug() << QString("remove empty directory : %1ms").arg(timer.elapsed());
    }

    if (exitCode == -1 || exitCode == -2) {
        result = COPY_EXEC_ERROR;
    }

    return result;
}

UsbErrorType UsbOperation::copyToBody(const UsbDataType type, const QString &copyDirPath, const bool &compress, const QString& password)
{
    QProcess *process = new QProcess();

    // コピー元のチェック
    QString originalDirPath = getUsbTargetDirPath(type);
    UsbErrorType error = UsbErrorType::NON_ERROR;
    error = checkExistsFile(originalDirPath);

    if (error != UsbErrorType::NON_ERROR) {
        return error;
    }

    // コピー先フォルダを削除・追加する
    if (copyDirPath.isEmpty()) {
        return UsbErrorType::COPY_EXEC_ERROR;
    }

    if (QDir(copyDirPath).exists()) {
        if (process->execute(DELETE_DIR_COMMAND + copyDirPath) != 0) {
            delete process;
            return UsbErrorType::COPY_EXEC_ERROR;
        }
    }

    // コピー先フォルダの親フォルダをチェック
    QStringList tmpList = copyDirPath.split("/");
    QString parentCopyDirPath = tmpList[0];

    for (int i = 1; i < tmpList.size() - 1; ++i) {
        parentCopyDirPath.append(QDir::separator() + tmpList[i]);
    }

    if (parentCopyDirPath.isEmpty()) {
        delete process;
        return UsbErrorType::COPY_EXEC_ERROR;
    }

    if (!QDir(parentCopyDirPath).exists()) {
        // 親フォルダが存在しなければ作成する
        if (process->execute(CREATE_DIR_COMMAND + parentCopyDirPath) != 0) {
            delete process;
            return UsbErrorType::COPY_EXEC_ERROR;
        }
    }

    // コピー先ディレクトリ名
    QString copyDirName = tmpList[tmpList.size() - 1];

    // フォルダコピー
    error = executeDirCopy(originalDirPath, parentCopyDirPath, copyDirName, compress, password, BODY);

    if (error != UsbErrorType::NON_ERROR) {
        return error;
    }

    process->execute(SYNC_COMMAND);
    delete process;
    return error;
}

QList<QFileInfo> UsbOperation::getUsbFileList(const UsbDataType type)
{
    // 対象フォルダチェック
    QString path = getUsbTargetDirPath(type);
    UsbErrorType error = UsbErrorType::NON_ERROR;
    error = checkExistsDirectory(path);

    if (error != UsbErrorType::NON_ERROR) {
        return QList<QFileInfo>();
    }

    // ファイル一覧を取得
    QList<QFileInfo> fileList = QList<QFileInfo>();
    // 対象ディレクトリ内のcsv,pngのファイルを検索(サブフォルダも含む)
    QDirIterator it(path, QStringList() << "*.csv", QDir::Files, QDirIterator::Subdirectories);

    while (it.hasNext()) {
        if (it.next().isEmpty()) {
            continue;
        }

        fileList.append(it.fileInfo());
    }

    return fileList;
}

UsbErrorType UsbOperation::deleteUsbData(const UsbDataType type)
{
    QString targetDirPath = getUsbTargetDirPath(type);

    // ターゲットフォルダが存在しなければ処理なし
    if (targetDirPath.isEmpty()) {
        return DELETE_ERROR;
    }

    if (!QDir(targetDirPath).exists()) {
        return DELETE_ERROR;
    }

    // ターゲットの削除・追加
    QProcess *process = new QProcess();

    if (process->execute(DELETE_DIR_COMMAND + targetDirPath) != 0) {
        delete process;
        return UsbErrorType::DELETE_ERROR;
    }

    if (process->execute(CREATE_DIR_COMMAND + targetDirPath) != 0) {
        delete process;
        return UsbErrorType::DELETE_ERROR;
    }

    process->execute(SYNC_COMMAND);
    delete process;
    return UsbErrorType::NON_ERROR;
}

UsbErrorType UsbOperation::removeMeasureResultToUsb(const QString &source, const QStringList &excludedFiles)
{
    UsbErrorType result = UsbErrorType::NON_ERROR;

    QString command;
    if (excludedFiles.count() == 0) {
        // 全て削除
        command = QString("/bin/sh -c \"rm -rf %1/*\"").arg(source);
    } else {
        // 除外するファイルを /tmp にコピー(<日付><CSV or PNG>のディレクトリからコピー)

        // 全て削除
        command = QString("/bin/sh -c \"rm -rf %1/*\"").arg(source);

        // 除外するファイルを削除元のパスにコピー
    }

    int exitCode = QProcess::execute(command);
    QProcess::execute(SYNC_COMMAND);

    if (exitCode == -1 || exitCode == -2) {
        result = DELETE_ERROR;
    }

    return result;
}

UsbErrorType UsbOperation::checkExistsDirectory(const QString &originalDirPath)
{
    // パスチェック
    if (originalDirPath.isEmpty()) {
        return UsbErrorType::COPY_FILE_ERROR;
    }

    // 存在チェック
    if (!QDir(originalDirPath).exists()) {
        return UsbErrorType::COPY_FILE_ERROR;
    }

    // ファイルチェック
    if (QDir(originalDirPath).entryList(QDir::Dirs | QDir::Files | QDir::NoDotAndDotDot).count() <= 0) {
        return UsbErrorType::COPY_FILE_ERROR;
    }

    return UsbErrorType::NON_ERROR;
}

UsbErrorType UsbOperation::checkExistsFile(const QString &originalFilePath)
{
    // パスチェック
    if (originalFilePath.isEmpty()) {
        return UsbErrorType::COPY_FILE_ERROR;
    }

    // 存在チェック
    if (!QFile(originalFilePath).exists()) {
        return UsbErrorType::COPY_FILE_ERROR;
    }

    return UsbErrorType::NON_ERROR;
}

UsbErrorType UsbOperation::checkExistsSettingFile(const QString &originalDirPath)
{
    // パスチェック
    if (originalDirPath.isEmpty()) {
        return UsbErrorType::COPY_FILE_ERROR;
    }

    // 存在チェック
    if (!QDir(originalDirPath).exists()) {
        return UsbErrorType::COPY_FILE_ERROR;
    }

    // 設定ファイル数チェック
    // リストファイルは必ず存在するため、除いて1以上
    if (QDir(originalDirPath).entryList(QDir::Dirs | QDir::Files | QDir::NoDotAndDotDot).count() <= 1) {
        return UsbErrorType::COPY_FILE_ERROR;
    }
qDebug() << QString("count ---") << QDir(originalDirPath).entryList(QDir::Dirs | QDir::Files | QDir::NoDotAndDotDot).count();
    return UsbErrorType::NON_ERROR;
}

UsbErrorType UsbOperation::executeDirCopy(const QString &originalDirPath, const QString &copyDirPath, const QString &copyDirName, const bool &compress, const QString& password, const COPY_DIRECTION &direction)
{
    // 本体⇒USBコピー実施
    QProcess *process = new QProcess();

    QString zipFileName;
    QString zipFilePath;
    QVector<QStringRef> values = originalDirPath.splitRef("/");

    if (!compress) {
        if (process->execute(COPY_DIR_COMMAND + originalDirPath + " " + copyDirPath) != 0) {
            delete process;
            return UsbErrorType::COPY_EXEC_ERROR;
        }
        delete process;
    } else {

        if (values.count() > 0) {
            zipFileName = values[values.count() - 1].toString();
            zipFileName += QString(".zip");

            for (int i = 0; i < values.count() - 1; ++i) {
                zipFilePath += values[i].toString();
                zipFilePath += QString("/");
            }
        }

        if (direction == BODY) {
            // USBパス
            QStringList originalDirName = originalDirPath.split("/");
            QString originalPath;
            for(int i = 0; i < originalDirName.size() - 1; i++) {
                originalPath += originalDirName[i] + QString("/");
            }

            // 解凍
            QString command = QString("/bin/sh -c \"cd %1; %2%3 %4\"").arg(TMP_PATH).arg(EXTRACT_ZIP_COMMAND).arg(password).arg(originalDirPath);
            qDebug() << command;
            if (process->execute(command) != 0) {
                delete process;
                return UsbErrorType::COPY_EXEC_ERROR;
            }
            delete process;

            process = new QProcess();

            // ファイル名変更 移動
            QString beforFile = TMP_PATH + ("/") + originalDirName[originalDirName.size() - 1];
            beforFile.chop(4);
            QString afterFile = copyDirPath + ("/") + copyDirName;
            command = QString("/bin/sh -c \"cd %1; %2%3 %4\"").arg(copyDirPath).arg(MV_FILE_COMMAND).arg(beforFile).arg(afterFile);
            qDebug() << command;
            if (process->execute(command) != 0) {
                delete process;
                return UsbErrorType::COPY_EXEC_ERROR;
            }

            process->execute(SYNC_COMMAND);
            delete process;
        } else if (direction == USB_MEMORY) {
            QString command = QString("/bin/sh -c \"cd %1; %2%3 %4 ./%5\"").arg(zipFilePath).arg(CREATE_ZIP_COMMAND).arg(password).arg(zipFileName).arg(values[values.count() - 1].toString());
            qDebug() << command;
            if (process->execute(command) != 0) {
                delete process;
                return UsbErrorType::COPY_EXEC_ERROR;
            }
            delete process;

            process = new QProcess();
            command = QString("/bin/sh -c \"cd %1; %2 %3 %4\"").arg(zipFilePath).arg(MV_FILE_COMMAND).arg(zipFileName).arg(copyDirPath);
            qDebug() << command;
            if (process->execute(command) != 0) {
                delete process;
                return UsbErrorType::COPY_EXEC_ERROR;
            }
            delete process;
        }
    }

    QProcess::execute(SYNC_COMMAND);
    return UsbErrorType::NON_ERROR;
}

QString UsbOperation::getUsbTargetDirPath(const UsbDataType type)
{
    QString path = "";

    if (type == UsbDataType::MEASUREMENT_RESULT_SINGLE) {
        // 測定結果(SINGLE)
        path = _singleMeasurementUsbPath;
    } else if (type == UsbDataType::MEASUREMENT_RESULT_MULTI) {
        // 測定結果(MULTI)
        path = _multiMeasurementUsbPath;
    } else if (type == UsbDataType::DATA_LOGGER_RESULT) {
        // データロガー結果
        path = _loggerMeasurementUsbPath;
    } else if (type == UsbDataType::USER_MEASUREMENT_SETTING) {
        // ユーザ測定設定
        path = USER_MEASUREMENT_PATH + ZIP_EXTENSION;
    } else if (type == UsbDataType::DATALOGGER_MEASUREMENT_SETTING) {
        // データロガー設定
        path = DATALOGGER_MEASUREMENT_PATH + ZIP_EXTENSION;
    } else {
        // nothing to do
    }

    return path;
}

QString UsbOperation::getUsbTargetParentDirPath(const UsbDataType type)
{
    QString path = "";

    if (type == UsbDataType::MEASUREMENT_RESULT_SINGLE) {
        // 測定結果(SINGLE)
        path = _singleMeasurementUsbPath;
    } else if (type == UsbDataType::MEASUREMENT_RESULT_MULTI) {
        // 測定結果(MULTI)
        path = _multiMeasurementUsbPath;
    } else if (type == UsbDataType::DATA_LOGGER_RESULT) {
        // データロガー設定
        path = _loggerMeasurementUsbPath;
    } else if (type == UsbDataType::USER_MEASUREMENT_SETTING) {
        // ユーザ測定設定
        path = USER_MEASUREMNET_SETTING_USB_PATH;   // USB メモリーのパス
    } else if (type == UsbDataType::DATALOGGER_MEASUREMENT_SETTING) {
        // データロガー設定
        path = USER_MEASUREMNET_SETTING_USB_PATH;   // USB メモリーのパス
    } else {
        // nothing to do
    }

    return path;
}

QString UsbOperation::replaceSourceToDestination(const QString &source, const QString &destination)
{
    QStringList elements = source.split("/");
    QString fileName = elements[elements.count() - 1];
    QString path;

    // ファイル名を "_" で区切った後、2 つ目の要素の左 8 文字が YYYYMMDD 形式の日付
    QStringList fileNameElements = fileName.split("_");
    if (fileNameElements.count() > 2) {
        if (fileName.contains(CSV_FILE_EXTENSION)) {
            path = destination + QString("/%1/CSV/").arg(fileNameElements[1].left(8));
        } else {
            path = destination + QString("/%1/PNG/").arg(fileNameElements[1].left(8));
        }
    }

    return path + fileName;
}

void UsbOperation::setSingleMeasurementPath(const QString &path)
{
    _singleMeasurementPath = path;
}

void UsbOperation::setMultiMeasurementPath(const QString &path)
{
    _multiMeasurementPath = path;
}

void UsbOperation::setLoggerMeasurementPath(const QString &path)
{
    _loggerMeasurementPath = path;
}

void UsbOperation::setSingleMeasurementUsbPath(const QString &path)
{
    _singleMeasurementUsbPath = path;
}

void UsbOperation::setMultiMeasurementUsbPath(const QString &path)
{
    _multiMeasurementUsbPath = path;
}

void UsbOperation::setLoggerMeasurementUsbPath(const QString &path)
{
    _loggerMeasurementUsbPath = path;
}

void UsbOperation::setMeasureDataFileListPath(const QString &path)
{
    _measureDataFileListPath = path;
}

void UsbOperation::setMeasureDataFileListFileName(const QString &filename)
{
    _measureDataFileListFileName = filename;
}
