/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef USBOPERATION_H
#define USBOPERATION_H

#include "usbdefs.h"
#include "constant.h"
#include <QObject>
#include <QFileInfo>

class UsbOperation
{
public:
    /**
     * @brief コンストラクタ
     */
    UsbOperation(void);

    /**
     * @brief コピー先の種類
     */
    enum COPY_DIRECTION {
        BODY = 0,               //!< 本体
        USB_MEMORY = 1          //!< USB メモリー
    };

    /**
     * @brief USBをマウントする
     * @return true:成功, false:失敗
     */
    static bool mount(void);
    /**
     * @brief USBをアンマウントする
     * @return true:成功, false:失敗
     */
    static bool umount(void);
    /**
     * @brief USBをフォーマットする
     * @return エラータイプ
     */
    static UsbErrorType format(void);
    /**
     * @brief usbの空き容量を取得
     * @return usbの空き容量
     */
    static qint64 freeSpace(void);

    /**
     * @brief コピー対象のファイルサイズを取得
     * @param targetFilePath
     * @return コピー対象のファイルサイズ
     */
    static qint64 copyTargetSize(const QString targetFilePath);

    /**
     * @brief コピー容量チェックを行う
     * @return エラータイプ
     */
    static UsbErrorType CheckCopyCapacity();
    /**
     * @brief コピー可能かをチェックする
     * @param targetPath コピー対象パスリスト
     * @param isFile     コピー対象がファイルかどうか true: ファイル, false: ファイル以外(ディレクトリ等)
     * @return コピー可能か True: コピー可, False: コピー不可
     */
    static UsbErrorType checkCopyable(const QStringList targetPathList, bool isFile = false);
    /**
     * @brief 空き容量を返す
     * @return 空き容量
     */
    static quint64 getFreeSpaceSize(void);
    /**
     * @brief 本体⇒USBへコピー
     * @param type データタイプ
     * @param originalPath コピー元のパス情報
     * @param compress 圧縮有無
     * @param password zip ファイルのパスワード
     * @return エラータイプ
     */
    static UsbErrorType copyToUsb(const UsbDataType type, const QString &originalDirPath, const bool &compress, const QString& password);
    /**
     * @brief 測定結果を USB メモリーにコピーする
     * @param source コピー元のパス
     * @param destination コピー先のパス
     * @param excludedFiles コピーしないファイルのパスのリスト
     * @return コピーの成否
     */
    static UsbErrorType copyMeasureResultToUsb(const QString &source, const QString &destination, const QStringList &excludedFiles);
    /**
     * @brief USB⇒本体へコピー
     * @param type データタイプ
     * @param copyPath コピー先のパス情報
     * @param compress 圧縮有無
     * @param password zip ファイルのパスワード
     * @return エラータイプ
     */
    static UsbErrorType copyToBody(const UsbDataType type, const QString &copyDirPath, const bool &compress, const QString& password);
    /**
     * @brief ファイル一覧を取得
     * @param type　読み出すデータタイプ
     * @return ファイル一覧
     */
    static QList<QFileInfo> getUsbFileList(const UsbDataType type);

    /**
     * @brief USBデータ削除
     * @param type データタイプ
     * @return エラータイプ
     */
    static UsbErrorType deleteUsbData(const UsbDataType type);
    /**
     * @brief 測定結果を削除する
     * @param source 削除するパス
     * @param excludedFiles 削除しないファイルのパスのリスト
     * @return 削除の成否
     */
    static UsbErrorType removeMeasureResultToUsb(const QString &source, const QStringList &excludedFiles);
    /**
     * @brief ディレクトリの存在チェック
     * @param originalDirPath コピー元フォルダ
     * @return エラータイプ
     */
    static UsbErrorType checkExistsDirectory(const QString &originalDirPath);

    /**
     * @brief ファイルの存在チェック
     * @param originalFilePath コピー元フォルダ
     * @return エラータイプ
     */
    static UsbErrorType checkExistsFile(const QString &originalFilePath);

    /**
     * @brief ディレクトリ内の設定ファイルの存在チェック
     * @param originalDirPath チェックフォルダ
     * @return エラータイプ
     */
    static UsbErrorType checkExistsSettingFile(const QString &originalDirPath);

    /**
     * @brief シングル測定結果の保存先パスを設定
     * @param path 保存先パス
     */
    static void setSingleMeasurementPath(const QString &path);

    /**
     * @brief マルチ測定結果の保存先パスを設定
     * @param path 保存先パス
     */
    static void setMultiMeasurementPath(const QString &path);

    /**
     * @brief ロガー測定結果の保存先パスを設定
     * @param path 保存先パス
     */
    static void setLoggerMeasurementPath(const QString &path);

    /**
     * @brief シングル測定結果のUSBコピー先パスを設定
     * @param path USBコピー先パス
     */
    static void setSingleMeasurementUsbPath(const QString &path);

    /**
     * @brief マルチ測定結果のUSBコピー先パスを設定
     * @param path USBコピー先パス
     */
    static void setMultiMeasurementUsbPath(const QString &path);

    /**
     * @brief データロガー測定結果のUSBコピー先パスを設定
     * @param path USBコピー先パス
     */
    static void setLoggerMeasurementUsbPath(const QString &path);

    /**
     * @brief 測定データファイルリストのパスを設定
     * @param path 本機コピー元パス
     */
    static void setMeasureDataFileListPath(const QString &path);

    /**
     * @brief 測定データファイルリストのファイル名を設定
     * @param path 本機コピー元パス
     */
    static void setMeasureDataFileListFileName(const QString &filename);

private:
    /**
     * @brief ディレクトリコピーの実施
     * @param originalDirPath コピー元ディレクトリパス
     * @param copyDirPath コピー先ディレクトリパス
     * @param copyDirName コピー先ディレクトリ名   本体にコピー時のみ変数使用
     * @param compress 圧縮有無
     * @param password zip ファイルのパスワード
     * @param direction コピー先(0: 本体 1: USB メモリー)
     * @return エラータイプ
     */
    static UsbErrorType executeDirCopy(const QString &originalDirPath, const QString &copyDirPath, const QString &copyDirName, const bool &compress, const QString& password, const COPY_DIRECTION &direction);

    /**
     * @brief USBメモリ内の対象ディレクトリパスを取得する
     * @param type データタイプ
     * @return USBメモリ内の対象ディレクトリパス
     */
    static QString getUsbTargetDirPath(const UsbDataType type);

    /**
     * @brief USBメモリ内の対象ディレクトリに対する親ディレクトリパスを取得する
     * @param type データタイプ
     * @return USBメモリ内の対象ディレクトリに対する親ディレクトリパス
     */
    static QString getUsbTargetParentDirPath(const UsbDataType type);

    /**
     * @brief 移動先のファイルのパスに付け替える
     * @param source 元のファイルのパス
     * @param destination 移動先のファイルのパス
     * @return 移動先のファイルのパス
     */
    static QString replaceSourceToDestination(const QString &source, const QString &destination);
};

#endif // USBOPERATION_H
