﻿/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include <QObject>
#include <QFile>
#include <QDir>
#include <QTextStream>
#include <QImage>
#include <QThread>
#include "communicator.h"
#include "correcter.h"

#include <QProcess>
#include <QDebug>
#include <QElapsedTimer>

static constexpr const char *USER_SETTING_FILE_LIST_FILENAME =
        "user_setting_file_list.csv"; // ユーザー測定設定ファイル名リスト
static constexpr const char *LOGGER_SETTING_FILE_LIST_FILENAME =
        "logger_setting_file_list.csv"; // データロガー測定設定ファイル名リスト

static QElapsedTimer timer;

Communicator::Communicator(const ActionMode &mode, SystemLog &systemLog, QObject *parent)
    : QObject(parent)
    , _mode(mode)
    , _dummy(nullptr)
    , _systemLog(systemLog)
    , _displayType(LC70_DISP_TYPE_NOT_MEAS)
    , _supplying(false)
    , _dataFileListFileName()
    , _dataFileListPath()
    , _singleMeasurementDataPath()
    , _multiMeasurementDataPath()
    , _userMeasurementDataPath()
    , _loggerMeasurementDataPath()
{
    if (_mode == ActionMode::DEVELOPMENT) {
        _dummy =  new LC70Dummy(this);
    }

#if Q_PROCESSOR_ARM
    timer.start();
    LCLK_ERR_T clockStatus = lclk_init();
    _systemLog.write(QString("ラッパー関数呼び出し"),
                     QString("%1 called %2 ms").arg(QString("lclk_init")).arg(timer.elapsed()));

    if (clockStatus != LCLK_ACTIVE) {
        _systemLog.write(QString("ラッパー関数呼び出し"),
                         QString("### ERROR lclk_init() returns %1").arg(static_cast<int>(clockStatus)));
        qDebug() << QString("### ERROR lclk_init() returns ") << static_cast<int>(clockStatus);
    }

#endif
}

Communicator::~Communicator()
{

}

LC70_ERR_T Communicator::initialize(int id)
{
    LC70_ERR_T ret = LC70_OK;

    if (_mode == ActionMode::PRODUCTION) {
#if Q_PROCESSOR_ARM
        timer.restart();
        // LC70WS アプリケーションライブラリ初期化
        ret = lc70_init((PROC_ID_T)id);
        _systemLog.write(QString("ラッパー関数呼び出し"),
                         QString("%1 id:%2(0x%3) called %4 ms").arg(QString("lc70_init")).arg(id).arg(
                             QString::number((long long)QThread::currentThreadId(), 16)).arg(timer.elapsed()));

        if (ret != LC70_OK) {
            _systemLog.write(QString("ラッパー関数呼び出し"),
                             QString("### ERROR lc70_init id = %1 ret = %2").arg(id).arg(static_cast<int>(ret)));
            return ret;
        }

#endif
    } else {
        ret = _dummy->initialize(PROC_ID_PDISPMNG);
    }

    _systemLog.write(QString("ラッパー関数呼び出し"), QString("### lc70_init id = %1").arg(id));
    return ret;
}

LC70_ERR_T Communicator::readMeasureSettingFile(LC70_FILE_ID_T fileId, void *data, bool isDefault)
{
    LC70_ERR_T ret = LC70_OK;

    if (_mode == ActionMode::PRODUCTION) {
#if Q_PROCESSOR_ARM
        int rlen = lc70_readSetFile(fileId, data, nullptr, isDefault ? TRUE : FALSE);

        if (rlen < 0) {
            ret = static_cast<LC70_ERR_T>(rlen);
        }

        if (ret < LC70_OK) {
            writeLog(QString().sprintf("### ERROR lc70_readSetFile - measure setting fileId:%d ret:%d", fileId, ret));
        }

#endif
    } else {
        _dummy->readMeasureSettingFile(fileId, data, isDefault);
    }

    return ret;
}

LC70_ERR_T Communicator::readSystemSettingFile(LC70_FILE_ID_T fileId, void *data, bool isDefault)
{
    LC70_ERR_T ret = LC70_OK;

    if (_mode == ActionMode::PRODUCTION) {
#if Q_PROCESSOR_ARM
        int rlen = lc70_readSetFile(fileId, data, nullptr, isDefault ? TRUE : FALSE);

        if (rlen < 0) {
            ret = static_cast<LC70_ERR_T>(rlen);
        }

        if (ret < LC70_OK) {
            writeLog(QString().sprintf("### ERROR lc70_readSetFile() - system setting fileId:%d ret:%d", fileId, ret));
        }

#endif
    } else {
        _dummy->readSystemSettingFile(fileId, data, isDefault);
    }

    return ret;
}

LC70_ERR_T Communicator::readUserSettingFile(const char *fileName, LC70_USER_SET_T *userSetting)
{
    // 不使用
    LC70_ERR_T ret = LC70_OK;

    if (_mode == ActionMode::PRODUCTION) {
#if Q_PROCESSOR_ARM

#endif
    } else {
        _dummy->readUserSettingFile(fileName, userSetting);
    }

    return ret;
}

LC70_ERR_T Communicator::readChannelConfigurationFile(LC70_FILE_ID_T fileId, void *data, bool isDefault)
{
    LC70_ERR_T ret = LC70_OK;

    if (_mode == ActionMode::PRODUCTION) {
#if Q_PROCESSOR_ARM
        int rlen = lc70_readSetFile(fileId, data, nullptr, isDefault ? TRUE : FALSE);

        if (rlen < 0) {
            ret = static_cast<LC70_ERR_T>(rlen);
        }

        if (ret < LC70_OK) {
            writeLog(QString().sprintf("### ERROR lc70_readSetFile() - channel configuration fileId:%d ret:%d", fileId, ret));
        }

#endif
    } else {
        _dummy->readChannelConfigurationFile(fileId, data, isDefault);
    }

    return ret;
}

LC70_ERR_T Communicator::readCalibrationSettingFile(const char *fileName,
                                                    LC70_CAL_SET_T *calibrationSetting)
{
    // 不使用
    LC70_ERR_T ret = LC70_OK;

    if (_mode == ActionMode::PRODUCTION) {
#if Q_PROCESSOR_ARM

#endif
    } else {
        _dummy->readCalibrationSettingFile(fileName, calibrationSetting);
    }

    return ret;
}

LC70_ERR_T Communicator::writeMeasureSettingFile(LC70_FILE_ID_T fileId, void *data)
{
    LC70_ERR_T ret = LC70_OK;

    if (_mode == ActionMode::PRODUCTION) {
#if Q_PROCESSOR_ARM
        int wline = lc70_writeSetFile(fileId, data, nullptr, 0);

        if (wline < 0) {
            ret = static_cast<LC70_ERR_T>(wline);
        }

        if (ret != LC70_OK) {
            writeLog(QString().sprintf("### ERROR lc70_writeSetFile() fileId:%d ret:%d", fileId, ret));
        }

        ret = lc70_ntfUpdateFile(fileId);

        if (ret != LC70_OK) {
            writeLog(QString().sprintf("### ERROR lc70_ntfUpdateFile() fileId:%d ret:%d", fileId, ret));
        }

#endif
    } else {
        _dummy->writeMeasureSettingFile(fileId, data);
    }

    return ret;
}

LC70_ERR_T Communicator::writeSystemSettingFile(LC70_FILE_ID_T fileId, void *data)
{
    LC70_ERR_T ret = LC70_OK;

    if (_mode == ActionMode::PRODUCTION) {
#if Q_PROCESSOR_ARM
        int wline = lc70_writeSetFile(fileId, data, nullptr, 0);

        if (wline < 0) {
            ret = static_cast<LC70_ERR_T>(wline);
        }

        if (ret != LC70_OK) {
            writeLog(QString().sprintf("### ERROR lc70_writeSetFile() fileId:%d ret:%d", fileId, ret));
        }

        ret = lc70_ntfUpdateFile(fileId);

        if (ret != LC70_OK) {
            writeLog(QString().sprintf("### ERROR lc70_ntfUpdateFile() fileId:%d ret:%d", fileId, ret));
        }

#endif
    } else {
        _dummy->writeSystemSettingFile(fileId, data);
    }

    return ret;
}

LC70_ERR_T Communicator::writeUserSettingFile(const char *fileName, LC70_USER_SET_T *userSetting)
{
    // 不使用
    LC70_ERR_T ret = LC70_OK;

    if (_mode == ActionMode::PRODUCTION) {
#if Q_PROCESSOR_ARM

#endif
    } else {
        _dummy->writeUserSettingFile(fileName, userSetting);
    }

    return ret;
}

LC70_ERR_T Communicator::writeChannelConfigurationFile(LC70_FILE_ID_T fileId, void *data)
{
    LC70_ERR_T ret = LC70_OK;

    if (_mode == ActionMode::PRODUCTION) {
#if Q_PROCESSOR_ARM
        int wline = lc70_writeSetFile(fileId, data, nullptr, 0);

        if (wline < 0) {
            ret = static_cast<LC70_ERR_T>(wline);
        }

        if (ret != LC70_OK) {
            writeLog(QString().sprintf("### ERROR lc70_writeSetFile() - channel configuration fileId:%d ret:%d", fileId, ret));
        }

        ret = lc70_ntfUpdateFile(fileId);

        if (ret != LC70_OK) {
            writeLog(QString().sprintf("### ERROR lc70_ntfUpdateFile() fileId:%d ret:%d", fileId, ret));
        }

#endif
    } else {
        _dummy->writeChannelConfigurationFile(fileId, data);
    }

    return ret;
}

LC70_ERR_T Communicator::writeCalibrationSettingFile(const char *fileName,
                                                     LC70_CAL_SET_T *calibrationSetting)
{
    // 不使用
    LC70_ERR_T ret = LC70_OK;

    if (_mode == ActionMode::PRODUCTION) {
#if Q_PROCESSOR_ARM

#endif
    } else {
        _dummy->writeCalibrationSettingFile(fileName, calibrationSetting);
    }

    return ret;
}

LC70_ERR_T Communicator::failureInformation(LC70_FAIL_INFO_T *failureInformation)
{
    LC70_ERR_T ret = LC70_OK;

    if (_mode == ActionMode::PRODUCTION) {
#if Q_PROCESSOR_ARM
        timer.restart();
        ret = lc70_getFailinfo(failureInformation);
        //        _systemLog.write(QString("ラッパー関数呼び出し"),
        //                         QString("%1 called %2 ms").arg(QString("lc70_getFailinfo")).arg(timer.elapsed()));

        if (ret != LC70_OK) {
            _systemLog.write(QString("ラッパー関数呼び出し"),
                             QString("### ERROR lc70_getFailinfo ret = %1").arg(static_cast<int>(ret)));
        }

#endif
    } else {
        _dummy->failureInformation(failureInformation);
    }

    return ret;
}

LC70_ERR_T Communicator::setFailureInformation(uint16_t code, uint8_t status, const uint8_t *additionalInformation)
{
    LC70_ERR_T ret = LC70_OK;

    _systemLog.write(QString("ラッパー関数呼び出し"),
                     QString("%1 code: %2 status: %3 called").arg(QString("lc70_setFailStat")).arg(code).arg(status));

    if (_mode == ActionMode::PRODUCTION) {
#if Q_PROCESSOR_ARM
        ret = lc70_setFailStat(code, status, additionalInformation);

        if (ret != LC70_OK) {
            _systemLog.write(QString("ラッパー関数呼び出し"),
                             QString("### ERROR lc70_setFailStat ret = %1").arg(static_cast<int>(ret)));
        }

#endif
    }

    return ret;
}

LC70_ERR_T Communicator::keyStatus(LC70_KEY_STAT_T *keyStatus)
{
    LC70_ERR_T ret = LC70_OK;

    if (_mode == ActionMode::PRODUCTION) {
#if Q_PROCESSOR_ARM
        //timer.restart();
        ret = lc70_getKeyStatus(keyStatus);
        //_systemLog.write(QString("ラッパー関数呼び出し"), QString("%1 called %2 ms").arg(QString("lc70_getKeyStatus")).arg(timer.elapsed()));

        if (ret != LC70_OK) {
            //            _systemLog.write(QString("ラッパー関数呼び出し"),
            //                             QString("### ERROR lc70_getKeyStatus ret = %1").arg(static_cast<int>(ret)));
        }

#endif
    } else {
        // nothing to do.
    }

    return ret;
}

LC70_ERR_T Communicator::deviceStatus(LC70_DEV_STAT_T *deviceStatus)
{
    LC70_ERR_T ret = LC70_OK;

    if (_mode == ActionMode::PRODUCTION) {
#if Q_PROCESSOR_ARM
        timer.restart();
        ret = lc70_getDeviceStatus(deviceStatus);
        //        _systemLog.write(QString("ラッパー関数呼び出し"), QString("%1 called %2 ms battery status: %3 usb type a: %4").arg(QString("lc70_getDeviceStatus")).arg(timer.elapsed()).arg(deviceStatus->ucBatt).arg(deviceStatus->usba), LOG_NOTICE);
        //        if (ret != LC70_OK) {
        //            _systemLog.write(QString("ラッパー関数呼び出し"),
        //                             QString("### ERROR lc70_getDeviceStatus ret = %1").arg(static_cast<int>(ret)), LOG_NOTICE);
        //        }
#endif
    } else {
        _dummy->deviceStatus(deviceStatus);
    }

    return ret;
}

LC70_ERR_T Communicator::setMeasuringInputSignal(LC70_INPUT_T)
{
    // 不使用
    return LC70_OK;
}

LC70_ERR_T Communicator::setPowerSupply(LC70_POWER_SUPPLY_T outputVoltage)
{
    LC70_ERR_T ret = LC70_OK;

    if (outputVoltage == LC70_POWER_SUPPLY_OFF) {
        _supplying = false;
    } else {
        _supplying = true;
    }

    if (_mode == ActionMode::PRODUCTION) {
#if Q_PROCESSOR_ARM
        timer.restart();
        ret = lc70_reqPowerSupply(outputVoltage);
        _systemLog.write(QString("ラッパー関数呼び出し"),
                         QString("%1 called %2 ms").arg(QString("lc70_reqPowerSupply")).arg(timer.elapsed()));

        if (ret != LC70_OK) {
            _systemLog.write(QString("ラッパー関数呼び出し"),
                             QString("### ERROR lc70_reqPowerSupply(%1) ret = %2").arg(static_cast<int>(outputVoltage)).arg(
                                 static_cast<int>(ret)));
        }

#endif
    } else {
        _dummy->setPowerSupply(outputVoltage);
    }

    return ret;
}

LC70_ERR_T Communicator::setDataThreshold(int32_t, LC70_DATA_THR_T *)
{
    // 不使用
    return LC70_OK;
}

LC70_ERR_T Communicator::measuringStatus(LC70_MEAS_STAT_T *measuringStatus)
{
    // 不使用(動作しない)
    LC70_ERR_T ret = LC70_OK;

    if (_mode == ActionMode::PRODUCTION) {
#if Q_PROCESSOR_ARM
        timer.restart();
        ret = lc70_getMeasStatus(measuringStatus);
        _systemLog.write(QString("ラッパー関数呼び出し"),
                         QString("%1 called %2 ms").arg(QString("lc70_getMeasStatus")).arg(timer.elapsed()));
#endif
    } else {
        _dummy->measuringStatus(measuringStatus);
    }

    return ret;
}

LC70_ERR_T Communicator::startSingleMeasuring(LC70_MEAS_SINGLE_T *measuringCondition)
{
    LC70_ERR_T ret = LC70_OK;

    if (_mode == ActionMode::PRODUCTION) {
#if Q_PROCESSOR_ARM
        QString message =
                QString("[measure condition] frequency: %1 local:%2 system: %3 band: %4 modulation: %5 condition: %6").arg(
                    measuringCondition->ulFreq).arg(
                    measuringCondition->ulLoFreq).arg(measuringCondition->usBdSys).arg(measuringCondition->usBand).arg(
                    measuringCondition->usMod).arg(measuringCondition->usCondition);
        _systemLog.write(QString("ラッパー関数呼び出し"), message, LOG_NOTICE);
        timer.restart();
        ret = lc70_startSingleMeas(_displayType, measuringCondition);
        _systemLog.write(QString("ラッパー関数呼び出し"),
                         QString("%1 called %2 ms(id=0x%3) display type:%4").arg(QString("シングル測定開始(lc70_startSingleMeas)")).arg(
                             timer.elapsed()).arg(
                             QString::number((long long)QThread::currentThreadId(), 16)).arg(_displayType), LOG_NOTICE);

        if (ret != LC70_OK) {
            _systemLog.write(QString("ラッパー関数呼び出し"),
                             QString("### ERROR lc70_startSingleMeas ret = %1").arg(static_cast<int>(ret)), LOG_NOTICE);
            qDebug() << QString("### ERROR lc70_startSingleMeas : result = ") << static_cast<int>(ret);
        }

#endif
    } else {
        // nothing to do.
    }

    return ret;
}

LC70_ERR_T Communicator::stopSingleMeasuring()
{
    LC70_ERR_T ret = LC70_OK;

    if (_mode == ActionMode::PRODUCTION) {
#if Q_PROCESSOR_ARM
        timer.restart();
        ret = lc70_stopSingleMeas(LC70_DISP_TYPE_NOT_MEAS);
        _systemLog.write(QString("ラッパー関数呼び出し"),
                         QString("%1 called %2 ms display type:%3").arg(QString("シングル測定停止(lc70_stopSingleMeas)")).arg(
                             timer.elapsed()).arg(_displayType), LOG_NOTICE);

        if (ret != LC70_OK) {
            _systemLog.write(QString("ラッパー関数呼び出し"),
                             QString("### ERROR lc70_stopSingleMeas ret = %1").arg(static_cast<int>(ret)), LOG_NOTICE);
            qDebug() << QString("### ERROR lc70_stopSingleMeas : result = ") << static_cast<int>(ret);
        }

#endif
    } else {
        // nothing to do.
    }

    return ret;
}

LC70_ERR_T Communicator::startConstellationMeasuring(LC70_CONSTA_NUM_T collectionPoints)
{
    LC70_ERR_T ret = LC70_OK;

    if (_mode == ActionMode::PRODUCTION) {
#if Q_PROCESSOR_ARM
        timer.restart();
        ret = lc70_startConstaMeas(_displayType, collectionPoints);
        _systemLog.write(QString("ラッパー関数呼び出し"),
                         QString("%1 called %2 ms(id=0x%3) display type:%4").arg(
                             QString("コンスタレーション開始(lc70_startConstaMeas)")).arg(timer.elapsed()).arg(
                             QString::number((long long)QThread::currentThreadId(), 16)).arg(_displayType), LOG_NOTICE);

        if (ret != LC70_OK) {
            _systemLog.write(QString("ラッパー関数呼び出し"),
                             QString("### ERROR lc70_startConstaMeas ret = %1").arg(static_cast<int>(ret)), LOG_NOTICE);
            qDebug() << QString("### ERROR lc70_startConstaMeas : result = ") << static_cast<int>(ret);
        }

#endif
    } else {
        // nothing to do.
    }

    return ret;
}

LC70_ERR_T Communicator::stopConstellationMeasuring(void)
{
    LC70_ERR_T ret = LC70_OK;

    if (_mode == ActionMode::PRODUCTION) {
#if Q_PROCESSOR_ARM
        timer.restart();
        ret = lc70_stopConstaMeas(LC70_DISP_TYPE_NOT_MEAS);
        _systemLog.write(QString("ラッパー関数呼び出し"),
                         QString("%1 called %2 ms display type:%3").arg(QString("コンスタレーション停止(lc70_stopConstaMeas)")).arg(
                             timer.elapsed()).arg(_displayType), LOG_NOTICE);

        if (ret != LC70_OK) {
            _systemLog.write(QString("ラッパー関数呼び出し"),
                             QString("### ERROR lc70_stopConstaMeas ret = %1").arg(static_cast<int>(ret)), LOG_NOTICE);
            qDebug() << QString("### ERROR lc70_stopConstaMeas : result = ") << static_cast<int>(ret);
        }

#endif
    } else {
        // nothing to do.
    }

    return ret;
}

LC70_ERR_T Communicator::singleMeasuringResult(int16_t constellationPoints,
                                               LC70_MEAS_SINGLE_RESULT_T *measuringResult)
{
    LC70_ERR_T ret = LC70_OK;

    if (_mode == ActionMode::PRODUCTION) {
#if Q_PROCESSOR_ARM
        timer.restart();
        ret = lc70_getSingleMeasResult(constellationPoints, measuringResult);
        _systemLog.write(QString("ラッパー関数呼び出し"),
                         QString("%1 called %2 ms(id=0x%3)").arg(QString("シングル測定結果取得(lc70_getSingleMeasResult)")).arg(
                             timer.elapsed()).arg(
                             QString::number((long long)QThread::currentThreadId(), 16)), LOG_INFO);

        if (ret != LC70_OK) {
            _systemLog.write(QString("ラッパー関数呼び出し"),
                             QString("### ERROR lc70_getSingleMeasResult ret = %1").arg(static_cast<int>(ret)), LOG_INFO);
        }

#endif
    } else {
        // nothing to do.・
    }

    return ret;
}

LC70_ERR_T Communicator::startDelayProfileMeasuring(const LC70_MEAS_SINGLE_T &measureCondition)
{
    LC70_ERR_T ret = LC70_OK;

    if (_mode == ActionMode::PRODUCTION) {
#if Q_PROCESSOR_ARM
        timer.restart();
        LC70_MEAS_SINGLE_T condition;
        memset(&condition, 0, sizeof(condition));

        condition.ulFreq = measureCondition.ulFreq;
        condition.ulLoFreq = measureCondition.ulLoFreq;

        for (int i = 0; i < STR_CHAN_LEN; ++i) {
            condition.ucStrChan[i] = measureCondition.ucStrChan[i];
        }

        condition.ucOutputEnb = measureCondition.ucOutputEnb;
        condition.powerSupply = measureCondition.powerSupply;

        condition.usBdSys = LC70_BD_SYS_ISDBT;
        condition.usBand = LC70_BAND_UHF;
        condition.usMod = LC70_SETMOD_OFDM;
        condition.usCondition = LC70_DATA_MSK_DLY_PRF;

        ret = lc70_startDlyPrfMeas(_displayType, &condition);
        _systemLog.write(QString("ラッパー関数呼び出し"),
                         QString("%1 called %2 ms(id=0x%3) display type:%4").arg(
                             QString("遅延プロファイル開始(lc70_startDlyPrfMeas)")).arg(timer.elapsed()).arg(
                             QString::number((long long)QThread::currentThreadId(), 16)).arg(_displayType), LOG_NOTICE);

        if (ret != LC70_OK) {
            _systemLog.write(QString("ラッパー関数呼び出し"),
                             QString("### ERROR lc70_startDlyPrfMeas ret = %1").arg(static_cast<int>(ret)), LOG_NOTICE);
            qDebug() << QString("### ERROR lc70_startDlyPrfMeas : result = ") << static_cast<int>(ret);
        } else {
            qDebug() << QString("start delay profile");
        }

#endif
    } else {
        // nothing to do.
    }

    return ret;
}

LC70_ERR_T Communicator::stopDelayProfileMeasuring()
{
    LC70_ERR_T ret = LC70_OK;

    if (_mode == ActionMode::PRODUCTION) {
#if Q_PROCESSOR_ARM
        timer.restart();
        ret = lc70_stopDlyPrfMeas(LC70_DISP_TYPE_NOT_MEAS);
        _systemLog.write(QString("ラッパー関数呼び出し"),
                         QString("%1 called %2 ms display type:%3").arg(QString("遅延プロファイル停止(lc70_stopDlyPrfMeas)")).arg(
                             timer.elapsed()).arg(_displayType), LOG_NOTICE);

        if (ret != LC70_OK) {
            _systemLog.write(QString("ラッパー関数呼び出し"),
                             QString("### ERROR lc70_stopDlyPrfMeas ret = %1").arg(static_cast<int>(ret)), LOG_NOTICE);
            qDebug() << QString("### ERROR lc70_stopDlyPrfMeas : result = ") << static_cast<int>(ret);
        } else {
            qDebug() << QString("stop delay profile");
        }

#endif
    } else {
        // nothing to do.
    }

    return ret;
}

LC70_ERR_T Communicator::delayProfileMeasuring(LC70_MEAS_DLY_PRF_RESULT_T *result)
{
    LC70_ERR_T ret = LC70_OK;

    if (_mode == ActionMode::PRODUCTION) {
#if Q_PROCESSOR_ARM
        timer.restart();
        ret = lc70_getDlyPrfResult(result);
        _systemLog.write(QString("ラッパー関数呼び出し"),
                         QString("%1 called %2 ms(id=0x%3)").arg(
                             QString("遅延プロファイル測定結果取得(lc70_getDlyPrfResult)")).arg(timer.elapsed()).arg(
                             QString::number((long long)QThread::currentThreadId(), 16)), LOG_INFO);

        if (ret != LC70_OK) {
            _systemLog.write(QString("ラッパー関数呼び出し"),
                             QString("### ERROR lc70_getDlyPrfResult ret = %1").arg(static_cast<int>(ret)), LOG_INFO);
        } else {
            qDebug() << QString("get delay profile");
        }

#endif
    } else {
        // nothing to do.
    }

    return ret;
}

LC70_ERR_T Communicator::startMultiMeasuring(LC70_MEAS_MULTI_T *measuringCondition)
{
    LC70_ERR_T ret = LC70_OK;

    if (_mode == ActionMode::PRODUCTION) {
#if Q_PROCESSOR_ARM
        timer.restart();
        ret = lc70_startMultiMeas(_displayType, measuringCondition);
        _systemLog.write(QString("ラッパー関数呼び出し"),
                         QString("%1 called %2 ms(id=0x%3) display type:%4").arg(QString("マルチ測定開始(lc70_startMultiMeas)")).arg(
                             timer.elapsed()).arg(
                             QString::number((long long)QThread::currentThreadId(), 16)).arg(_displayType), LOG_NOTICE);

        if (ret != LC70_OK) {
            _systemLog.write(QString("ラッパー関数呼び出し"),
                             QString("### ERROR lc70_startMultiMeas ret = %1").arg(static_cast<int>(ret)), LOG_NOTICE);
            qDebug() << QString("### ERROR lc70_startMultiMeas : result = ") << static_cast<int>(ret);
            return ret;
        }

        qDebug() << QString("start multi measuring");
#endif
    } else {
        // nothing to do.
    }

    return ret;
}

LC70_ERR_T Communicator::stopMultiMeasuring()
{
    LC70_ERR_T ret = LC70_OK;

    if (_mode == ActionMode::PRODUCTION) {
#if Q_PROCESSOR_ARM
        timer.restart();
        ret = lc70_stopMultiMeas(LC70_DISP_TYPE_NOT_MEAS);
        _systemLog.write(QString("ラッパー関数呼び出し"),
                         QString("%1 called %2 ms display type:%3").arg(QString("マルチ測定停止(lc70_stopMultiMeas)")).arg(
                             timer.elapsed()).arg(_displayType), LOG_NOTICE);

        if (ret != LC70_OK) {
            _systemLog.write(QString("ラッパー関数呼び出し"),
                             QString("### ERROR lc70_stopMultiMeas ret = %1").arg(static_cast<int>(ret)), LOG_NOTICE);
            qDebug() << QString("### ERROR lc70_stopMultiMeas : result = ") << static_cast<int>(ret);
        }

        qDebug() << QString("stop multi measuring");
#endif
    } else {
        // nothing to do.
    }

    return ret;
}

LC70_ERR_T Communicator::setMultiMeasuringChannel(uint8_t *)
{
    // 不使用
    return LC70_OK;
}

LC70_ERR_T Communicator::multiMeasuringResult(int16_t index,
                                              LC70_MEAS_MULTI_RESULT_T *measuringResult)
{
    LC70_ERR_T ret = LC70_OK;

    if (_mode == ActionMode::PRODUCTION) {
#if Q_PROCESSOR_ARM
        timer.restart();
        ret = lc70_getMultiMeasResult(index, measuringResult);
        _systemLog.write(QString("ラッパー関数呼び出し"),
                         QString("%1(index:%2) called %3 ms(id=0x%4)").arg(QString("マルチ測定結果取得(lc70_getMultiMeasResult)")).arg(
                             index).arg(
                             timer.elapsed()).arg(
                             QString::number((long long)QThread::currentThreadId(), 16)));

        if (ret != LC70_OK) {
            _systemLog.write(QString("ラッパー関数呼び出し"),
                             QString("### ERROR lc70_getMultiMeasResult ret = %1").arg(static_cast<int>(ret)), LOG_NOTICE);
        }
#endif
    } else {
        // nothing to do.
    }

    return ret;
}

LC70_ERR_T Communicator::startPrefferedChannelInMultiMeasuring(LC70_MEAS_SINGLE_T *measuringCondition)
{
    LC70_ERR_T ret = LC70_OK;

    if (_mode == ActionMode::PRODUCTION) {
#if Q_PROCESSOR_ARM
        timer.restart();
        ret = lc70_startMultiChPrioMeas(_displayType, measuringCondition);
        _systemLog.write(QString("ラッパー関数呼び出し"),
                         QString("%1 called %2 ms(id=0x%3) display type:%4").arg(
                             QString("優先測定開始(lc70_startMultiChPrioMeas)")).arg(timer.elapsed()).arg(
                             QString::number((long long)QThread::currentThreadId(), 16)).arg(_displayType), LOG_NOTICE);

        if (ret != LC70_OK) {
            _systemLog.write(QString("ラッパー関数呼び出し"),
                             QString("### ERROR lc70_startMultiChPrioMeas ret = %1").arg(static_cast<int>(ret)), LOG_NOTICE);
            qDebug() << QString("### ERROR lc70_startMultiChPrioMeas : result = ") << static_cast<int>(ret);
        }

        qDebug() << QString("start multi measure preffered channel result");
#endif
    } else {
        // nothing to do.
    }

    return ret;
}

LC70_ERR_T Communicator::stopPrefferedChannelInMultiMeasuring()
{
    LC70_ERR_T ret = LC70_OK;

    if (_mode == ActionMode::PRODUCTION) {
#if Q_PROCESSOR_ARM
        timer.restart();
        ret = lc70_stopMultiChPrioMeas(LC70_DISP_TYPE_NOT_MEAS);
        _systemLog.write(QString("ラッパー関数呼び出し"),
                         QString("%1 called %2 ms display type:%3").arg(QString("優先測定停止(lc70_stopMultiChPrioMeas)")).arg(
                             timer.elapsed()).arg(
                             _displayType), LOG_NOTICE);

        if (ret != LC70_OK) {
            _systemLog.write(QString("ラッパー関数呼び出し"),
                             QString("### ERROR lc70_stopMultiChPrioMeas ret = %1").arg(static_cast<int>(ret)), LOG_NOTICE);
            qDebug() << QString("### ERROR lc70_stopMultiChPrioMeas : result = ") << static_cast<int>(ret);
        }

        qDebug() << QString("stop multi measuring preffered channel");
#endif
    } else {
        // nothing to do.
    }

    return ret;
}

LC70_ERR_T Communicator::multiMeasuringPrefferedChannelResult(LC70_MEAS_SINGLE_RESULT_T *measuringResult)
{
    LC70_ERR_T ret = LC70_OK;

    if (_mode == ActionMode::PRODUCTION) {
#if Q_PROCESSOR_ARM
        timer.restart();
        ret = lc70_getMultiChPrioMeas(measuringResult);
        _systemLog.write(QString("ラッパー関数呼び出し"),
                         QString("%1 called %2 ms(id=0x%3)").arg(QString("優先測定結果取得(lc70_getMultiChPrioMeas)")).arg(
                             timer.elapsed()).arg(
                             QString::number((long long)QThread::currentThreadId(), 16)));

        if (ret != LC70_OK) {
            _systemLog.write(QString("ラッパー関数呼び出し"),
                             QString("### ERROR lc70_getMultiChPrioMeas ret = %1").arg(static_cast<int>(ret)), LOG_INFO);
        }

#endif
    } else {
        // nothing to do.
    }

    return ret;
}

LC70_ERR_T Communicator::systemReboot()
{
    LC70_ERR_T ret = LC70_OK;

    if (_mode == ActionMode::PRODUCTION) {
#if Q_PROCESSOR_ARM
        _systemLog.write(QString("ラッパー関数呼び出し"), QString("再起動要求(lc70_reboot)"));
        ret = lc70_reboot();

        if (ret != LC70_OK) {
            _systemLog.write(QString("ラッパー関数呼び出し"),
                             QString("### ERROR lc70_reboot ret = %1").arg(static_cast<int>(ret)), LOG_INFO);
        }
#endif
    } else {
        _dummy->systemReboot();
    }

    return ret;
}

LC70_ERR_T Communicator::systemShutdown()
{
    LC70_ERR_T ret = LC70_OK;

    if (_mode == ActionMode::PRODUCTION) {
#if Q_PROCESSOR_ARM
        _systemLog.write(QString("ラッパー関数呼び出し"), QString("シャットダウン要求(lc70_shutdown)"));
        ret = lc70_shutdown();

        if (ret != LC70_OK) {
            _systemLog.write(QString("ラッパー関数呼び出し"),
                             QString("### ERROR lc70_shutdown ret = %1").arg(static_cast<int>(ret)), LOG_INFO);
        }

#endif
    } else {
        _dummy->systemShutdown();
    }

    return ret;
}

LC70_ERR_T Communicator::systemStandBy(uint8_t mode)
{
    LC70_ERR_T ret = LC70_OK;

    if (_mode == ActionMode::PRODUCTION) {
#if Q_PROCESSOR_ARM

        // MAIN, SUB 両方の場合に実行
        if (mode == 0) {
            _systemLog.write(QString("スクリプト呼び出し(同期)"), QString("USB POWER OFF"), LOG_NOTICE);
            int exitCode = QProcess::execute(USB_POWER_OFF_SCRIPT);
            if (exitCode < 0) {
                _systemLog.write(QString("スクリプト呼び出し(同期)"),
                                 QString("### ERROR USB_POWER_OFF ret = %1").arg(static_cast<int>(exitCode)), LOG_NOTICE);
            }

        }

        _systemLog.write(QString("ラッパー関数呼び出し"),
                         QString("スタンバイ要求(lc70_standby) mode:%1").arg(mode), LOG_NOTICE);
        ret = lc70_standby(mode);

        if (ret != LC70_OK) {
            _systemLog.write(QString("ラッパー関数呼び出し"),
                             QString("### ERROR lc70_sleep ret = %1").arg(static_cast<int>(ret)), LOG_NOTICE);
        }

#endif
    } else {
        _dummy->systemSleep();
    }

    return ret;
}

LC70_ERR_T Communicator::systemWakeup(uint8_t mode)
{
    LC70_ERR_T ret = LC70_OK;

    if (_mode == ActionMode::PRODUCTION) {
#if Q_PROCESSOR_ARM

        // MAIN, SUB 両方の場合に実行
        if (mode == 0) {
            _systemLog.write(QString("スクリプト呼び出し(同期)"), QString("USB POWER ON"), LOG_NOTICE);
            int exitCode = QProcess::execute(USB_POWER_ON_SCRIPT);
            if (exitCode < 0) {
                _systemLog.write(QString("スクリプト呼び出し(同期)"),
                                 QString("### ERROR USB_POWER_ON ret = %1").arg(static_cast<int>(exitCode)), LOG_NOTICE);
            }
        }

        _systemLog.write(QString("ラッパー関数呼び出し"),
                         QString("ウェイクアップ要求(lc70_wakeup) mode:%1").arg(mode), LOG_NOTICE);

        ret = lc70_wakeup(mode);

        if (ret != LC70_OK) {
            _systemLog.write(QString("ラッパー関数呼び出し"),
                             QString("### ERROR lc70_wakeup ret = %1").arg(static_cast<int>(ret)), LOG_NOTICE);
        }

#endif
    } else {
        _dummy->systemWakeup();
    }

    return ret;
}

LC70_ERR_T Communicator::buzzer(LC70_BUZZ_T buzzerPattern, LC70_BUZZ_MODE_T buzzerMode)
{
    LC70_ERR_T ret = LC70_OK;

    if (_mode == ActionMode::PRODUCTION) {
#if Q_PROCESSOR_ARM

        // 全チャンネルスキャン完了音の場合 1 秒待つ
        if (buzzerPattern == LC70_BUZZ_ALL_CH_SCAN) {
            QThread::msleep(1000);
        }

        ret = lc70_buzzer(buzzerPattern, buzzerMode);

        if (ret != LC70_OK) {
            writeLog(QString("### ERROR lc70_buzzer() buzz : %1, buzzMode : %2").arg(static_cast<int>
                                                                                     (buzzerPattern)).arg(static_cast<int>(buzzerMode)));
        }

#endif
    } else {
        _dummy->buzzer(buzzerPattern, buzzerMode);
    }

    return ret;
}

LC70_ERR_T Communicator::fourceBuzzerVolume(LC70_BUZZ_T buzzerPattern, int buzzerVolume)
{
    LC70_ERR_T ret = LC70_OK;

    if (_mode == ActionMode::PRODUCTION) {
#if Q_PROCESSOR_ARM
        LC70_BUZZ_VOL_T buzzer = LC70_BUZZ_VOL_MUTE;

        if (buzzerVolume == LC70_BUZZ_VOL_MUTE) {
            // 音量設定 : ミュート
            buzzer = LC70_BUZZ_VOL_MUTE;
        } else if (buzzerVolume == LC70_BUZZ_VOL_MIN) {
            // 音量設定 : ボリューム小
            buzzer = LC70_BUZZ_VOL_MIN;
        } else if (buzzerVolume == LC70_BUZZ_VOL_MID) {
            // 音量設定 : ボリューム中
            buzzer = LC70_BUZZ_VOL_MID;
        } else if (buzzerVolume == LC70_BUZZ_VOL_MAX) {
            // 音量設定 : ボリューム大
            buzzer = LC70_BUZZ_VOL_MAX;
        }

        ret = lc70_forceBuzzerVolume(buzzerPattern, buzzer);

        if (ret != LC70_OK) {
            writeLog(QString("### ERROR lc70_fourceBuzzerVolume() buzz : %1, buzzVol : %2").arg(static_cast<int>
                                                                                                (buzzerPattern)).arg(static_cast<int>(buzzer)));
        }

#endif
    } else {
        _dummy->fourceBuzzerVolume(buzzerPattern, buzzerVolume);
    }

    return ret;
}

void Communicator::initDateTime()
{
    if (_mode == ActionMode::PRODUCTION) {
#if Q_PROCESSOR_ARM
        timer.restart();
        LCLK_ERR_T clockStatus = lclk_init();
        _systemLog.write(QString("ラッパー関数呼び出し"),
                         QString("%1 called %2 ms").arg(QString("lclk_init")).arg(timer.elapsed()), LOG_NOTICE);

        if (clockStatus != LCLK_ACTIVE) {
            _systemLog.write(QString("ラッパー関数呼び出し"),
                             QString("### ERROR lclk_init() returns %1").arg(static_cast<int>(clockStatus)), LOG_NOTICE);
            qDebug() << QString("### ERROR lclk_init() returns ") << static_cast<int>(clockStatus);
        }

#endif
    }
}

lclk_err_t Communicator::setDateTime(int yyyy, int mm, int dd, int week, int hour, int minutes)
{
    lclk_err_t ret = LCLK_ACTIVE;

    if (_mode == ActionMode::PRODUCTION) {
#if Q_PROCESSOR_ARM
        timer.restart();
        CRRTIME clkTime;
        clkTime.year = yyyy - 2000;
        clkTime.month = mm;
        clkTime.day = dd;
        clkTime.week = 0x01;
        clkTime.hour = hour;
        clkTime.min = minutes;
        clkTime.sec = 0x00;
        ret = lclk_set(&clkTime);
        _systemLog.write(QString("ラッパー関数呼び出し"),
                         QString("%1 called %2 ms").arg(QString("lclk_set")).arg(timer.elapsed()));

        if (ret != LCLK_ACTIVE) {
            _systemLog.write(QString("ラッパー関数呼び出し"),
                             QString("### ERROR lclk_set() returns %1").arg(static_cast<int>(ret)));
            qDebug() << QString("### ERROR lclk_set() returns ") << static_cast<int>(ret);
        }

#endif
    } else {
        int dummyret = _dummy->setDateTime(yyyy - 2000, mm, dd, week, hour, minutes);
        ret = static_cast<lclk_err_t>(dummyret);
    }

    return ret;
}

LC70_ERR_T Communicator::setAutoPowerOffTime(int offTime)
{
    LC70_ERR_T ret = LC70_OK;

    if (_mode == ActionMode::PRODUCTION) {
#if Q_PROCESSOR_ARM
        LC70_SYS_SET_POWOFF_T powerOff;
        powerOff.sysSetPowoffRow[0].ucPowerOffSelect[0] = LC70_SYS_SET_POWOFF_OFF;
        powerOff.sysSetPowoffRow[0].ucPowerOffSelect[1] = LC70_SYS_SET_POWOFF_5MIN;
        powerOff.sysSetPowoffRow[0].ucPowerOffSelect[2] = LC70_SYS_SET_POWOFF_10MIN;
        powerOff.sysSetPowoffRow[0].ucPowerOffSelect[3] = LC70_SYS_SET_POWOFF_15MIN;
        powerOff.sysSetPowoffRow[0].ucPowerOffSelect[4] = LC70_SYS_SET_POWOFF_30MIN;
        powerOff.sysSetPowoffRow[0].ucPowerOffSelect[5] = LC70_SYS_SET_POWOFF_60MIN;

        if (offTime == LC70_SYS_SET_POWOFF_OFF) {
            // すぐにOFFにする場合
            powerOff.sysSetPowoffRow[0].ucPowerOff = LC70_SYS_SET_POWOFF_OFF;
        } else if (offTime == LC70_SYS_SET_POWOFF_5MIN) {
            // POWER OFFまで5分の場合
            powerOff.sysSetPowoffRow[0].ucPowerOff = LC70_SYS_SET_POWOFF_5MIN;
        } else if (offTime == LC70_SYS_SET_POWOFF_10MIN) {
            // POWER OFFまで10分の場合
            powerOff.sysSetPowoffRow[0].ucPowerOff = LC70_SYS_SET_POWOFF_10MIN;
        } else if (offTime == LC70_SYS_SET_POWOFF_15MIN) {
            // POWER OFFまで15分の場合
            powerOff.sysSetPowoffRow[0].ucPowerOff = LC70_SYS_SET_POWOFF_15MIN;
        } else if (offTime == LC70_SYS_SET_POWOFF_30MIN) {
            // POWER OFFまで30分の場合
            powerOff.sysSetPowoffRow[0].ucPowerOff = LC70_SYS_SET_POWOFF_30MIN;
        } else if (offTime == LC70_SYS_SET_POWOFF_60MIN) {
            // POWER OFFまで60分の場合
            powerOff.sysSetPowoffRow[0].ucPowerOff = LC70_SYS_SET_POWOFF_60MIN;
        } else {
            // 上記以外の設定であった場合 初期値の15分を設定する。
            powerOff.sysSetPowoffRow[0].ucPowerOff = LC70_SYS_SET_POWOFF_OFF;
        }

        ret = writeSystemSettingFile(LC70_FILE_ID_SYS_SET_POWOFF, &powerOff);
#endif
    } else {
        ret = _dummy->setAutoPowerOffTime(offTime);
    }

    return ret;
}

LC70_ERR_T Communicator::writeMultiMeasurementData(QString fileName, uint16_t condition,
                                                   const QList<MeasureResult> &multiMeasureResult)
{
    LC70_ERR_T ret = LC70_OK;

    if (multiMeasureResult.count() == 0) {
        return LC70_ERR;
    }

    timer.restart();
    // ラッパー関数を使用しない
    QFile file(fileName);

    if (file.open(QIODevice::WriteOnly | QIODevice::Append)) {
        QTextStream outStream(&file);

        // BOMを生成する
        outStream.setGenerateByteOrderMark(true);

        QDateTime creationDateTime = QDateTime::currentDateTime();
        QVector<QStringRef> words = fileName.splitRef("/");

        if (words.count() > 0) {
            fileName = words.at(words.count() - 1).toString();
        }

        // ヘッダー
        outStream << QString("日付") << ",";
        outStream << creationDateTime.toString("yyyy/MM/dd") << "\n";
        outStream << QString("時刻") << ",";
        outStream << creationDateTime.toString("hh:mm:ss") << "\n";
        outStream << QString("測定データ名") << ",";
        outStream << fileName << "\n";

        outStream << QString("測定項目") << ",";

        if ((condition & LC70_DATA_MSK_LEVEL) == LC70_DATA_MSK_LEVEL) {
            outStream << QString("レベル");
        }

        // MER/CN の使い分け
        if ((condition & LC70_DATA_MSK_CNR) == LC70_DATA_MSK_CNR) {
            outStream << QString("+CNR");
        } else if ((condition & LC70_DATA_MSK_MER) == LC70_DATA_MSK_MER) {
            outStream << QString("+MER");
        }

        if ((condition & LC70_DATA_MSK_BER) == LC70_DATA_MSK_BER) {
            outStream << QString("+BER");
        }

        outStream <<  "\n";

        outStream << QString("チャンネル名称") << ",";

        outStream << QString("BAND") << ",";
        outStream << QString("周波数(MHz)") << ",";
        outStream << QString("レベル(dBuV)") << ",";
        outStream << QString("MER/CN(dB)") << ",";
        outStream << QString("BER") << ",";

        if (Correcter::isSatelliteBroadCasting(multiMeasureResult[0].Band.toInt())) {
            outStream << QString("Layer/偏波") << ",";
        } else {
            outStream << QString("Layer/偏波") << ",";
        }

        outStream << QString("局発周波数(GHz)") << ",";
        outStream << QString("放送規格") << ",";
        outStream << QString("変調方式") << "\n";

        for (int i = 0; i < multiMeasureResult.count(); ++i) {
            outStream << multiMeasureResult[i].Channel << ",";          // チャンネル名称
            int band = multiMeasureResult[i].Band.toInt();
            outStream << Correcter::displayBandName(band) << ",";       // 放送方式
            //outStream << static_cast<float32_t>(multiMeasureResult[i].Frequency) / 1000 << ",";


            outStream << Correcter::displayFrequency(band,multiMeasureResult[i].Frequency) << ",";


            // 測定不可の場合にマイナスの値が入っているものとする
            QString polarization = Correcter::convertWaveToPolarization(multiMeasureResult[i].Layer);
            if (multiMeasureResult[i].Level >= 0) {
                LC70_BAND_T band = static_cast<LC70_BAND_T>(multiMeasureResult[i].Band.toInt());
                BandTabIndex bandTab = Correcter::convertBandTabIndex(band, polarization);
                QString levelString;
                if (!Correcter::displayLevel(bandTab, multiMeasureResult[i].Level, levelString)) {
                }
                outStream << levelString << ",";
            } else {
                outStream << QString("---") << ",";
            }

            // PILOT, LTE, FM, CATVUPは---とする
            if ( (band == LC70_BAND_PILOT || band == LC70_BAND_LTE ||
                  band == LC70_BAND_FM || band == LC70_BAND_CATV_UP_STREAM) ) {
                outStream << QString("---") << ",";
            } else {
                // 測定不可の場合にマイナスの値が入っているものとする
                if (multiMeasureResult[i].Mer > 0) {
                    if(multiMeasureResult[i].BroadcastSystem == LC70_BD_SYS_ANNEXB || multiMeasureResult[i].BroadcastSystem == LC70_BD_SYS_ANNEXC || multiMeasureResult[i].BroadcastSystem == LC70_BD_SYS_OPTICAST){
                        if (multiMeasureResult[i].Mer < 20) {
                            outStream << QString("<20.0") << ",";
                        }else{
                            outStream << Correcter::displayMer(band, multiMeasureResult[i].BroadcastSystem, multiMeasureResult[i].Modulation, multiMeasureResult[i].Mer) << ",";
                        }
                    }else{
                        outStream << Correcter::displayMer(band, multiMeasureResult[i].BroadcastSystem, multiMeasureResult[i].Modulation, multiMeasureResult[i].Mer) << ",";
                    }
                } else {
					// サブマイコンFWの修正により、ロックしていない場合 MER/CN = 0で返ってくる。
					// CATVの場合は、他BANDと異なって <20.0と表示させる。
                    if(multiMeasureResult[i].BroadcastSystem == LC70_BD_SYS_ANNEXB || multiMeasureResult[i].BroadcastSystem == LC70_BD_SYS_ANNEXC || multiMeasureResult[i].BroadcastSystem == LC70_BD_SYS_OPTICAST){
                        outStream << QString("<20.0") << ",";
                    }else{
                        outStream << QString(MER_TIMEOUT_VALUE) << ",";
                    }
                }
            }

            // PILOT, LTE, FM, CATVUPは---とする
            if ( (band == LC70_BAND_PILOT || band == LC70_BAND_LTE ||
                  band == LC70_BAND_FM || band == LC70_BAND_CATV_UP_STREAM) ) {
                outStream << QString("---") << ",";
            } else {
                // 測定不可の場合にマイナスの値が入っているものとする

                if (multiMeasureResult[i].Ber >= 0) {
                    outStream << Correcter::displayBer(multiMeasureResult[i].Ber) << ",";
                } else {
                    outStream << QString(BER_TIMEOUT_VALUE) << ",";
                }
            }

            if (Correcter::isSatelliteBroadCasting(multiMeasureResult[i].Band.toInt())) {                   // Layer/偏波
                outStream << Correcter::displayPolarization(polarization) << ",";
            } else {
                QString layer = Correcter::displayHierarchy(multiMeasureResult[i].Layer);
                if(layer == QString("")) {
                    outStream << QString("---") << ",";
                } else {
                    outStream << layer << ",";
                }
            }

            outStream << Correcter::displayLocalFrequency(multiMeasureResult[i].LocalOscillator) << ",";    // 局発周波数(MHz)

            // 周波数(MHz)
            QString broadcastsystem = Correcter::displayBroadbandSystem(multiMeasureResult[i].BroadcastSystem); // 放送方式規格

#if 1	//20210630_dx2
            //放送規格
           // LTE, CATV上り, PILOTは BAND名と同じ表記にする
            if ( (band == LC70_BAND_PILOT || band == LC70_BAND_LTE ||
                  band == LC70_BAND_CATV_UP_STREAM || band == LC70_BAND_FM) ) {
                outStream << Correcter::displayBandName(band) << ",";
            } else {
                if (broadcastsystem == QString("")) {
                    outStream << QString("---") << ",";
                } else {
                    if(multiMeasureResult[i].BroadcastSystem == LC70_BD_SYS_ANNEXC || multiMeasureResult[i].BroadcastSystem == LC70_BD_SYS_ANNEXB){
                        broadcastsystem = "ISDBC";
                    }
                    outStream << broadcastsystem << ",";
                }
            }
#else
            if (broadcastsystem == QString("")) {
                outStream << QString("---") << ",";
            } else {
                outStream << broadcastsystem << ",";
            }
#endif

            QString modulation = Correcter::displayModulation(multiMeasureResult[i].Modulation);            // Mod
            if (modulation == QString("")) {
                outStream << QString("---") << "\n";
            } else {
                outStream << modulation << "\n";
            }
        }

        if (outStream.status() == QTextStream::WriteFailed) {
            ret = LC70_ERR;
        }
        file.close();
        QProcess::execute(SYNC_COMMAND);

        _systemLog.write(QString("ラッパー関数呼び出し"),
                         QString("write multi measurement data %1 ms").arg(timer.elapsed()));
    } else {
        ret = LC70_ERR;
    }

    // 成功していたらファイル名リストを更新
    if (ret == LC70_OK) {
        return updateMeasurementFileList(CSV, MULTI_MEASUREMENT, fileName);
    }

    return ret;
}

LC70_ERR_T Communicator::writeSingleMeasurementData(QString fileName, uint16_t condition,
                                                    const MeasureResult &singleMeasureResult)
{
    LC70_ERR_T ret = LC70_OK;

    timer.restart();
    // ラッパー関数を使用しない
    QFile file(fileName);

    if (file.open(QIODevice::WriteOnly | QIODevice::Append)) {
        QTextStream outStream(&file);

        // BOMを生成する
        outStream.setGenerateByteOrderMark(true);

        QString dateTime = singleMeasureResult.Timestamp;
        QDateTime creationDateTime = QDateTime::fromString(dateTime, "yyyy/MM/dd hh:mm:ss");

        // 日時が空ならファイル名の日時を設定
        if (dateTime.isEmpty()) {
            QStringList elements = fileName.split("_");
            if (elements.count() >= 3) {
                dateTime = elements[elements.count() - 1 - 1];
                creationDateTime = QDateTime::fromString(dateTime, "yyyyMMddhhmmss");
            } else {
                creationDateTime = QDateTime::currentDateTime();
            }
        }

        QVector<QStringRef> words = fileName.splitRef("/");

        if (words.count() > 0) {
            fileName = words.at(words.count() - 1).toString();
        }

        // ヘッダー
        outStream << QString("日付") << ",";
        outStream << creationDateTime.toString("yyyy/MM/dd") << "\n";
        outStream << QString("時刻") << ",";
        outStream << creationDateTime.toString("hh:mm:ss") << "\n";
        outStream << QString("測定データ名") << ",";
        outStream << fileName << "\n";
        outStream << QString("測定項目") << ",";

        if ((condition & LC70_DATA_MSK_LEVEL) == LC70_DATA_MSK_LEVEL) {
            outStream << QString("レベル");
        }

        // MER/CN の使い分け
        if ((condition & LC70_DATA_MSK_CNR) == LC70_DATA_MSK_CNR) {
            outStream << QString("+CNR");
        } else if ((condition & LC70_DATA_MSK_MER) == LC70_DATA_MSK_MER) {
            outStream << QString("+MER");
        }

        if ((condition & LC70_DATA_MSK_BER) == LC70_DATA_MSK_BER) {
            outStream << QString("+BER");
        }

        outStream <<  "\n";

        outStream << QString("チャンネル名称") << ",";
        outStream << QString("BAND") << ",";
        outStream << QString("周波数(MHz)") << ",";
        outStream << QString("レベル(dBuV)") << ",";
        outStream << QString("MER/CN(dB)") << ",";
        outStream << QString("BER") << ",";

        if (Correcter::isSatelliteBroadCasting(singleMeasureResult.Band.toInt())) {
            outStream << QString("Layer/偏波") << ",";
        } else {
            outStream << QString("Layer/偏波") << ",";
        }

        outStream << QString("局発周波数(GHz)") << ",";
        outStream << QString("放送規格") << ",";
        outStream << QString("変調方式") << "\n";

        int band = singleMeasureResult.Band.toInt();
        outStream << singleMeasureResult.Channel << ",";					// CH
        outStream << Correcter::displayBandName(band) << ",";				// BAND
       // outStream << static_cast<float32_t>(singleMeasureResult.Frequency) / 1000 << ",";                   // 周波数(MHz)

         outStream << Correcter::displayFrequency(band,singleMeasureResult.Frequency) << ",";

         // 測定不可の場合にマイナスの値が入っているものとする
        QString polarization = Correcter::convertWaveToPolarization(singleMeasureResult.Layer);
        if (singleMeasureResult.Level >= 0) {
            LC70_BAND_T band = static_cast<LC70_BAND_T>(singleMeasureResult.Band.toInt());
            BandTabIndex bandTab = Correcter::convertBandTabIndex(band, polarization);
            QString levelString;
            if (!Correcter::displayLevel(bandTab, singleMeasureResult.Level, levelString)) {
            }
            outStream << levelString << ",";
        } else {
            outStream << QString("---") << ",";
        }

        // PILOT, LTE, FM, CATVUPは---とする
        if ( (band == LC70_BAND_PILOT || band == LC70_BAND_LTE ||
              band == LC70_BAND_FM || band == LC70_BAND_CATV_UP_STREAM) ) {
            outStream << QString("---") << ",";
        } else {
            // 測定不可の場合にマイナスの値が入っているものとする
            if (singleMeasureResult.Mer >= 0) {
                if(singleMeasureResult.BroadcastSystem == LC70_BD_SYS_ANNEXB || singleMeasureResult.BroadcastSystem == LC70_BD_SYS_ANNEXC || singleMeasureResult.BroadcastSystem == LC70_BD_SYS_OPTICAST){
                    if (singleMeasureResult.Mer < 20) {
                        outStream << QString("<20.0") << ",";
                    }else{
                        outStream << Correcter::displayMer(band, singleMeasureResult.BroadcastSystem, singleMeasureResult.Modulation, singleMeasureResult.Mer) << ",";
                    }
                }else{
                    outStream << Correcter::displayMer(band, singleMeasureResult.BroadcastSystem, singleMeasureResult.Modulation, singleMeasureResult.Mer) << ",";

                }
            } else {
				// サブマイコンFWの修正により、ロックしていない場合 MER/CN = 0で返ってくる。
				// CATVの場合は、他BANDと異なって <20.0と表示させる。
                if(singleMeasureResult.BroadcastSystem == LC70_BD_SYS_ANNEXB || singleMeasureResult.BroadcastSystem == LC70_BD_SYS_ANNEXC || singleMeasureResult.BroadcastSystem == LC70_BD_SYS_OPTICAST){
                    outStream << QString("<20.0") << ",";
                }else{
                    outStream << QString(MER_TIMEOUT_VALUE) << ",";
                }
            }
        }

        // PILOT, LTE, FM, CATVUPは---とする
        if ( (band == LC70_BAND_PILOT || band == LC70_BAND_LTE ||
              band == LC70_BAND_FM || band == LC70_BAND_CATV_UP_STREAM) ) {
            outStream << QString("---") << ",";
        } else {
            // 測定不可の場合にマイナスの値が入っているものとする
            if (singleMeasureResult.Ber >= 0) {
                //if(Correcter::displayBer(singleMeasureResult.Ber) == "1.0e-2"){

                //ロックしていないときは 2.2e-2と表示
                if(singleMeasureResult.Locked== UNLOCKED){
                    outStream << QString(BER_TIMEOUT_VALUE) << ",";

                }else if(singleMeasureResult.Locked== LOCKED){
                    outStream << Correcter::displayBer(singleMeasureResult.Ber) << ",";
                }else{
                    outStream << QString(BER_TIMEOUT_VALUE) << ",";
                }
            } else {
                outStream << QString(BER_TIMEOUT_VALUE) << ",";
            }
        }

        if (Correcter::isSatelliteBroadCasting(singleMeasureResult.Band.toInt())) {                         // Layer/偏波
            outStream << Correcter::displayPolarization(polarization) << ",";
        } else {
            QString layer = Correcter::displayHierarchy(singleMeasureResult.Layer);
            if(layer == QString("")) {
                outStream << QString("---") << ",";
            } else {
                outStream << layer << ",";
            }
        }

        outStream << Correcter::displayLocalFrequency(singleMeasureResult.LocalOscillator) << ",";          // 局発周波数(GHz)

        QString broadcastsystem = Correcter::displayBroadbandSystem(singleMeasureResult.BroadcastSystem);   // 放送方式規格
#if 1	//20210630_dx2
        // LTE, CATV上り, PILOTは BAND名と同じ表記にする
        if ( (band == LC70_BAND_PILOT || band == LC70_BAND_LTE ||
              band == LC70_BAND_CATV_UP_STREAM || band == LC70_BAND_FM) ) {
            outStream << Correcter::displayBandName(band) << ",";
        } else{
            if (broadcastsystem == QString("")) {
                outStream << QString("---") << ",";
            } else {
                if(singleMeasureResult.BroadcastSystem == LC70_BD_SYS_ANNEXC || singleMeasureResult.BroadcastSystem == LC70_BD_SYS_ANNEXB){
                    broadcastsystem = "ISDBC";
                }
                outStream << broadcastsystem << ",";
            }
        }
#else
        if (broadcastsystem == QString("")) {
            outStream << QString("---") << ",";
        } else {
            outStream << broadcastsystem << ",";
        }
#endif
        QString modulation = Correcter::displayModulation(singleMeasureResult.Modulation);                  // Mod
        if (modulation == QString("")) {
            outStream << QString("---") << "\n";
        } else {
            outStream << modulation << "\n";
        }

        if (outStream.status() == QTextStream::WriteFailed) {
            ret = LC70_ERR;
        }

        file.close();
        QProcess::execute(SYNC_COMMAND);

        _systemLog.write(QString("ラッパー関数呼び出し"),
                         QString("write single measurement data %1 ms").arg(timer.elapsed()));
    } else {
        ret = LC70_ERR;
    }

    // 成功していたらファイル名リストを更新
    if (ret == LC70_OK) {
        return updateMeasurementFileList(CSV, SINGLE_MEASUREMENT, fileName);
    }

    return ret;
}

LC70_ERR_T Communicator::writeSingleMeasurementScreen(QString fileName, QImage &image)
{
    LC70_ERR_T ret = LC70_OK;

    // ラッパー関数を使用しない
    bool result = image.save(fileName, "PNG");

    QProcess::execute(SYNC_COMMAND);

    ret = (result) ? LC70_OK : LC70_ERR;

    // 成功していたらファイル名リストを更新
    if (ret == LC70_OK) {
        return updateMeasurementFileList(PNG, SINGLE_MEASUREMENT, fileName);
    }

    return ret;
}

LC70_ERR_T Communicator::updateMeasurementFileList(SaveFileType, MeasureType, const QString &)
{
    LC70_ERR_T ret= LC70_OK;
    // 測定データファイルが追加、削除された場合に測定種類ごとのパスからファイル一覧を取得し、測定データファイルリストを更新する。
    static const QVariantList headerList = {
        "NO",                   // No.
        "DATA_TYPE",            // データの種類(SINGLE,MULTI)
        "DATE",                 // 日付(yyyyMMdd)
        "FILE_TYPE",            // ファイルの種類(CSV,PNG)
        "FILE_NAME",            // ファイル名
        "PLACEMENT1",           // 場所1
        "PLACEMENT2",           // 場所2
    };
    // 測定データファイルリストのパスとファイル名が未設定ならエラーとする
    if (_dataFileListFileName.isEmpty() || _dataFileListPath.isEmpty()) {
        return LC70_ERR;
    }

    // 測定データファイルのパスが未設定ならエラーとする
    if (_singleMeasurementDataPath.isEmpty() || _multiMeasurementDataPath.isEmpty() ||
            _userMeasurementDataPath.isEmpty() || _loggerMeasurementDataPath.isEmpty()) {
        return LC70_ERR;
    }

    // ファイルがなければ作成する
    QDir directory;
    if (!directory.exists(_dataFileListPath)) {
        directory.mkdir(_dataFileListPath);
    }

    QList<QVariantList> resultList;

    // 測定データファイル名リストを更新
    QString path = _dataFileListPath +  QDir::separator() + _dataFileListFileName;
    static const QStringList subDirectoryList = {"PNG", "CSV"};
    QFile file(path);
    if (file.open(QIODevice::WriteOnly)) {
        // シングル測定の測定結果ファイルリスト
        retrieveMeasurementDataFile(_singleMeasurementDataPath, "SINGLE", subDirectoryList, resultList);

        // マルチ測定の測定結果ファイルリスト
        retrieveMeasurementDataFile(_multiMeasurementDataPath, "MULTI", subDirectoryList, resultList);

        // USER 設定の測定結果ファイルリスト
        retrieveMeasurementDataFile(_userMeasurementDataPath, "LIST", subDirectoryList, resultList);

        // データロガーの測定結果ファイルリスト(CSVディレクトリを持たず、日付ディレクトリ内にCSVがある構成用)
        retrieveLoggerDataFile(_loggerMeasurementDataPath, "LOGGER", "CSV", resultList);

        // ファイルヘッダ出力
        file.write(generateCsvData(headerList).toUtf8().constData());

        for (int i = 0; i < resultList.size(); ++i) {
            QVariantList outputList = resultList.at(i);
            outputList.insert(0, i + 1);

            file.write(generateCsvData(outputList).toUtf8().constData());
        }
    }
    file.close();
    QProcess::execute(SYNC_COMMAND);

    return ret;
}

QString Communicator::generateCsvData(const QVariantList &dataList)
{
    QString outputData;
    outputData.clear();
    foreach(QVariant element, dataList) {
        if (!outputData.isEmpty()) {
            outputData += ",";
        }
        outputData += element.toString();
    }
    outputData += "\n";
    return outputData;
}

LC70_ERR_T Communicator::deleteUserLogSetFile(const QString &fileName, ChannelEditMode mode)
{
    LC70_ERR_T ret = LC70_OK;

    if (_mode == ActionMode::PRODUCTION) {
#if Q_PROCESSOR_ARM
        timer.restart();
        if (mode == CHANNEL_EDIT_USER) {
            ret = lc70_deleteSetFileUser(fileName.toUtf8().constData());
            _systemLog.write(QString("ラッパー関数呼び出し"),
                             QString("%1 called %2 ms").arg(QString("lc70_deleteSetFileUser")).arg(timer.elapsed()));
        } else if (mode == CHANNEL_EDIT_USER_USB1) {
            ret = lc70_deleteSetFileUserUsb1(fileName.toUtf8().constData());
            _systemLog.write(QString("ラッパー関数呼び出し"),
                             QString("%1 called %2 ms").arg(QString("lc70_deleteSetFileUserUsb1")).arg(timer.elapsed()));
        } else if (mode == CHANNEL_EDIT_USER_USB2) {
            ret = lc70_deleteSetFileUserUsb2(fileName.toUtf8().constData());
            _systemLog.write(QString("ラッパー関数呼び出し"),
                             QString("%1 called %2 ms").arg(QString("lc70_deleteSetFileUserUsb2")).arg(timer.elapsed()));
        } else if (mode == CHANNEL_EDIT_DATA_LOGGER) {
            ret = lc70_deleteSetFileLogger(fileName.toUtf8().constData());
            _systemLog.write(QString("ラッパー関数呼び出し"),
                             QString("%1 called %2 ms").arg(QString("lc70_deleteSetFileLogger")).arg(timer.elapsed()));
        } else if (mode == CHANNEL_EDIT_DATA_LOGGER_USB1) {
            ret = lc70_deleteSetFileLoggerUsb1(fileName.toUtf8().constData());
            _systemLog.write(QString("ラッパー関数呼び出し"),
                             QString("%1 called %2 ms").arg(QString("lc70_deleteSetFileLoggerUsb1")).arg(timer.elapsed()));
        } else if (mode == CHANNEL_EDIT_DATA_LOGGER_USB2) {
            ret = lc70_deleteSetFileLoggerUsb2(fileName.toUtf8().constData());
            _systemLog.write(QString("ラッパー関数呼び出し"),
                             QString("%1 called %2 ms").arg(QString("lc70_deleteSetFileLoggerUsb2")).arg(timer.elapsed()));
        }
#endif
    } else {
        ret = _dummy->deleteUserlogSetFile(fileName.toUtf8().constData());
    }

    return ret;
}

LC70_ERR_T Communicator::allDeleteUserLogSetFile(ChannelEditMode mode)
{
    LC70_ERR_T ret = LC70_OK;

    LC70_FILE_ID_T fileId = LC70_FILE_ID_USER_SET_USER;

    if(mode == CHANNEL_EDIT_USER) {
        fileId = LC70_FILE_ID_USER_SET_USER;
    } else if(mode == CHANNEL_EDIT_USER_USB1) {
        fileId = LC70_FILE_ID_USER_SET_USER_USB1;
    } else if(mode == CHANNEL_EDIT_USER_USB2) {
        fileId = LC70_FILE_ID_USER_SET_USER_USB2;
    } else if(mode == CHANNEL_EDIT_DATA_LOGGER) {
        fileId = LC70_FILE_ID_LOGGER_SET_LOGGER;
    } else if(mode == CHANNEL_EDIT_DATA_LOGGER_USB1) {
        fileId = LC70_FILE_ID_LOGGER_SET_LOGGER_USB1;
    } else if(mode == CHANNEL_EDIT_DATA_LOGGER_USB2) {
        fileId = LC70_FILE_ID_LOGGER_SET_LOGGER_USB2;
    }

    if (_mode == ActionMode::PRODUCTION) {
#if Q_PROCESSOR_ARM
        timer.restart();
        if (mode == CHANNEL_EDIT_USER || mode == CHANNEL_EDIT_USER_USB1 || mode == CHANNEL_EDIT_USER_USB2) {
            ret = lc70_deleteSetFileUserBulk(fileId);
            _systemLog.write(QString("ラッパー関数呼び出し"),
                             QString("%1 called %2").arg(QString("lc70_deleteSetFileUserBulk")).arg(fileId));
        } else if (mode == CHANNEL_EDIT_DATA_LOGGER || mode == CHANNEL_EDIT_DATA_LOGGER_USB1 || mode == CHANNEL_EDIT_DATA_LOGGER_USB2) {
            ret = lc70_deleteSetFileLoggerBulk(fileId);
            _systemLog.write(QString("ラッパー関数呼び出し"),
                             QString("%1 called %2").arg(QString("lc70_deleteSetFileLoggerBulk")).arg(fileId));
        }
#endif
    } else {
        ret = LC70_ERR;
    }

    return ret;
}

LC70_ERR_T Communicator::readUserLogSetFile(const QString &fileName, QList<LC70_LOGGER_SET_LOGGER_ROW_T> &loggerSetList, ChannelEditMode mode)
{
    int32_t ret = LC70_OK;

    LC70_FILE_ID_T fileId = LC70_FILE_ID_USER_SET_USER;

    LC70_LOGGER_SET_LOGGER_T loggerSet;

    if(mode == CHANNEL_EDIT_USER) {
        fileId = LC70_FILE_ID_USER_SET_USER;
    } else if(mode == CHANNEL_EDIT_USER_USB1) {
        fileId = LC70_FILE_ID_USER_SET_USER_USB1;
    } else if(mode == CHANNEL_EDIT_USER_USB2) {
        fileId = LC70_FILE_ID_USER_SET_USER_USB2;
    } else if(mode == CHANNEL_EDIT_DATA_LOGGER) {
        fileId = LC70_FILE_ID_LOGGER_SET_LOGGER;
    } else if(mode == CHANNEL_EDIT_DATA_LOGGER_USB1) {
        fileId = LC70_FILE_ID_LOGGER_SET_LOGGER_USB1;
    } else if(mode == CHANNEL_EDIT_DATA_LOGGER_USB2) {
        fileId = LC70_FILE_ID_LOGGER_SET_LOGGER_USB2;
    }

    if (_mode == ActionMode::PRODUCTION) {
#if Q_PROCESSOR_ARM
        timer.restart();
        ret = lc70_readSetFile(fileId, &loggerSet, fileName.toUtf8().constData(), FALSE);
        _systemLog.write(QString("ラッパー関数呼び出し"),
                         QString("%1 called %2 ms").arg(QString("lc70_readSetFile")).arg(timer.elapsed()));

        if (ret < LC70_OK) {
            writeLog(QString("### ERROR lc70_readSetFile() - %1 ret = %2 file name: %3").arg(mode).arg(static_cast<int>
                                                                                                       (ret)).arg(fileName));
        }

#endif
    } else {
        ret = _dummy->readSetFile(fileName.toUtf8().constData(), loggerSet);
    }

    loggerSetList.clear();

    for (int i = 0; i < ret; ++i) {
        loggerSetList << loggerSet.loggerSet[i];
    }

    return static_cast<LC70_ERR_T>(ret);
}

LC70_ERR_T Communicator::writeUserLogSetFile(const QString &fileName, QList<LC70_LOGGER_SET_LOGGER_ROW_T> &loggerSetList, ChannelEditMode mode)
{
    Q_UNUSED(mode)
    LC70_ERR_T ret = LC70_OK;
    LC70_LOGGER_SET_LOGGER_T *loggerSet = new LC70_LOGGER_SET_LOGGER_T();
    memset(loggerSet, 0, sizeof(LC70_LOGGER_SET_LOGGER_T));

    int index = 0;

    LC70_FILE_ID_T fileId = LC70_FILE_ID_USER_SET_USER;

    if(mode == CHANNEL_EDIT_USER) {
        fileId = LC70_FILE_ID_USER_SET_USER;
    } else if(mode == CHANNEL_EDIT_USER_USB1) {
        fileId = LC70_FILE_ID_USER_SET_USER_USB1;
    } else if(mode == CHANNEL_EDIT_USER_USB2) {
        fileId = LC70_FILE_ID_USER_SET_USER_USB2;
    } else if(mode == CHANNEL_EDIT_DATA_LOGGER) {
        fileId = LC70_FILE_ID_LOGGER_SET_LOGGER;
    } else if(mode == CHANNEL_EDIT_DATA_LOGGER_USB1) {
        fileId = LC70_FILE_ID_LOGGER_SET_LOGGER_USB1;
    } else if(mode == CHANNEL_EDIT_DATA_LOGGER_USB2) {
        fileId = LC70_FILE_ID_LOGGER_SET_LOGGER_USB2;
    }

    foreach (const LC70_LOGGER_SET_LOGGER_ROW_T &item, loggerSetList) {
        loggerSet->loggerSet[index] = item;
        index++;
    }

    if (_mode == ActionMode::PRODUCTION) {
#if Q_PROCESSOR_ARM
        timer.restart();
        int retcode = lc70_writeSetFile(fileId, loggerSet, fileName.toUtf8().constData(), index);
        _systemLog.write(QString("ラッパー関数呼び出し"),
                         QString("%1 called %2 ms").arg(QString("lc70_writeSetFile")).arg(timer.elapsed()));

        if (retcode < 0) {
            ret = static_cast<LC70_ERR_T>(retcode);
        }

#endif
    } else {
        ret = _dummy->writeSetFile(fileName.toUtf8().constData(), *loggerSet, index);
    }

    return ret;
}

void Communicator::writeLog(const QString &message, int priority)
{
    _systemLog.write(QString("ラッパー関数呼び出し"), message, priority);
}

LC70_ERR_T Communicator::startAzimuthMeasurement()
{
    LC70_ERR_T ret = LC70_OK;

    if (_mode == ActionMode::PRODUCTION) {
#if Q_PROCESSOR_ARM
        timer.restart();
        ret = lc70_startAngleMeas();
        _systemLog.write(QString("ラッパー関数呼び出し"),
                         QString("%1 called %2 ms").arg(QString("lc70_startAngleMeas")).arg(timer.elapsed()));

        if (ret != LC70_OK) {
            _systemLog.write(QString("ラッパー関数呼び出し"),
                             QString("### ERROR lc70_startAngleMeas ret = %1").arg(static_cast<int>(ret)));
        }

#endif
    } else {
        // nothing to do.
    }

    return ret;
}

LC70_ERR_T Communicator::stopAzimuthMeasurement()
{
    LC70_ERR_T ret = LC70_OK;

    if (_mode == ActionMode::PRODUCTION) {
#if Q_PROCESSOR_ARM
        timer.restart();
        ret = lc70_stopAngleMeas();
        _systemLog.write(QString("ラッパー関数呼び出し"),
                         QString("%1 called %2 ms").arg(QString("lc70_stopAngleMeas")).arg(timer.elapsed()));

        if (ret != LC70_OK) {
            _systemLog.write(QString("ラッパー関数呼び出し"),
                             QString("### ERROR lc70_stopAngleMeas ret = %1").arg(static_cast<int>(ret)));
        }

#endif
    } else {
        // nothing to do.
    }

    return ret;
}

LC70_ERR_T Communicator::setThresholdData(QVariantList thresholdData)
{
    LC70_ERR_T retcode = LC70_OK;

    if (_mode == ActionMode::PRODUCTION) {
#if Q_PROCESSOR_ARM
        LC70_MEAS_SET_THRD_T lc70Data;

        // 現在の設定値を取得
        if (0 > readMeasureSettingFile(LC70_FILE_ID_MEAS_SET_THRD, &lc70Data, FALSE)) {
        }

        for (int i = 0; i < LC70_MEAS_SET_THRD_IDX_MAX; i++) {
            // [Memo]
            // 閾値データの登録を行う。
            // MER/CN の値については MERとCNどちらにも同値を登録しておく。
            lc70Data.measSetThrdRow[i].ucBand = thresholdData.at(i).toList()[0].toInt();
            //            lc70Data.measSetThrdRow[i].ucSpec = 0;
            //            memset(lc70Data.measSetThrdRow[i].ucDummy, 0, sizeof(lc70Data.measSetThrdRow[i].ucDummy));
            //            lc70Data.measSetThrdRow[i].ulModBit = 0;
            lc70Data.measSetThrdRow[i].level.lLow = thresholdData.at(i).toList()[1].toInt();
            lc70Data.measSetThrdRow[i].level.lHigh = thresholdData.at(i).toList()[2].toInt();
            lc70Data.measSetThrdRow[i].mer.ulLow = thresholdData.at(i).toList()[3].toInt();
            lc70Data.measSetThrdRow[i].cn.ulLow = thresholdData.at(i).toList()[3].toInt();
            lc70Data.measSetThrdRow[i].mer.ulHigh = thresholdData.at(i).toList()[4].toInt();
            lc70Data.measSetThrdRow[i].cn.ulHigh = thresholdData.at(i).toList()[4].toInt();
            lc70Data.measSetThrdRow[i].ber.low.ulMantissa = thresholdData.at(i).toList()[5].toInt();
            lc70Data.measSetThrdRow[i].ber.low.lExponent = thresholdData.at(i).toList()[6].toInt();
            lc70Data.measSetThrdRow[i].ber.high.ulMantissa = thresholdData.at(i).toList()[7].toInt();
            lc70Data.measSetThrdRow[i].ber.high.lExponent = thresholdData.at(i).toList()[8].toInt();
        }

        retcode = writeMeasureSettingFile(LC70_FILE_ID_MEAS_SET_THRD, &lc70Data);

        if (retcode < 0) {
            retcode = static_cast<LC70_ERR_T>(retcode);
        }

#endif
    } else {
        retcode = _dummy->setThresholdData(thresholdData);
    }

    return retcode;
}

LC70_ERR_T Communicator::setConstellationShowPoint(int constellationSelectNum)
{
    LC70_ERR_T retcode = LC70_OK;

    if (_mode == ActionMode::PRODUCTION) {
#if Q_PROCESSOR_ARM
        LC70_MEAS_SET_CONSTA_NUM_T lc70Data;
        // コンスタレーション表示ポイント数選択範囲を更新
        lc70Data.measSetConstaNumRow[0].ulConstaNumSelect[0] = LC70_MEAS_SET_CONSTA_NUM_1000;
        lc70Data.measSetConstaNumRow[0].ulConstaNumSelect[1] = LC70_MEAS_SET_CONSTA_NUM_2000;
        lc70Data.measSetConstaNumRow[0].ulConstaNumSelect[2] = LC70_MEAS_SET_CONSTA_NUM_4000;

        lc70Data.measSetConstaNumRow[0].ulConstaNum = constellationSelectNum;
        retcode = writeMeasureSettingFile(LC70_FILE_ID_MEAS_SET_CONSTA_NUM, &lc70Data);

        if (retcode < 0) {
            retcode = static_cast<LC70_ERR_T>(retcode);
        }

        qDebug() << __FUNCTION__ << "retcode : " << retcode;
#endif
    } else {
        retcode = _dummy->setConstellationShowPoint(constellationSelectNum);
    }

    return retcode;
}

LC70_ERR_T Communicator::setCableLossData(QVariantList cableLossList)
{
    LC70_ERR_T ret = LC70_OK;

    if (_mode == ActionMode::PRODUCTION) {
#if Q_PROCESSOR_ARM
        LC70_MEAS_SET_CABLE_LOSS_T cableLoss;

        // 10 MHz
        cableLoss.measSetCableLossRow[LC70_MEAS_SET_CABLE_LOSS_IDX_70000KHZ].ulFreq = 10000;
        cableLoss.measSetCableLossRow[LC70_MEAS_SET_CABLE_LOSS_IDX_70000KHZ].ulCorrectionValue = cableLossList.at(
                    LC70_MEAS_SET_CABLE_LOSS_IDX_70000KHZ).toInt();

        // 770 MHz
        cableLoss.measSetCableLossRow[LC70_MEAS_SET_CABLE_LOSS_IDX_770000KHZ].ulFreq = 770000;
        cableLoss.measSetCableLossRow[LC70_MEAS_SET_CABLE_LOSS_IDX_770000KHZ].ulCorrectionValue = cableLossList.at(
                    LC70_MEAS_SET_CABLE_LOSS_IDX_770000KHZ).toInt();

        // 950 MHz
        cableLoss.measSetCableLossRow[LC70_MEAS_SET_CABLE_LOSS_IDX_950000KHZ].ulFreq = 950000;
        cableLoss.measSetCableLossRow[LC70_MEAS_SET_CABLE_LOSS_IDX_950000KHZ].ulCorrectionValue = cableLossList.at(
                    LC70_MEAS_SET_CABLE_LOSS_IDX_950000KHZ).toInt();

        // 3224 MHz
        cableLoss.measSetCableLossRow[LC70_MEAS_SET_CABLE_LOSS_IDX_2600000KHZ].ulFreq = 3224000;
        cableLoss.measSetCableLossRow[LC70_MEAS_SET_CABLE_LOSS_IDX_2600000KHZ].ulCorrectionValue = cableLossList.at(
                    LC70_MEAS_SET_CABLE_LOSS_IDX_2600000KHZ).toInt();

        qDebug() << cableLoss.measSetCableLossRow[LC70_MEAS_SET_CABLE_LOSS_IDX_2600000KHZ].ulCorrectionValue;
        ret = writeMeasureSettingFile(LC70_FILE_ID_MEAS_SET_CABLE_LOSS, &cableLoss);

        if (ret > 0) {
            ret = LC70_OK;
        }

#endif
    } else {
        ret = _dummy->setLebelOffsetData(cableLossList);
    }

    return ret;
}

LC70_ERR_T Communicator::setSoundVolume(int volumeLevel)
{
    LC70_ERR_T ret = LC70_OK;

    if (_mode == ActionMode::PRODUCTION) {
#if Q_PROCESSOR_ARM
        LC70_SYS_SET_VOLUME_T soundVolume;
        soundVolume.sysSetVolumeRow[0].ucVolumeSelect[0] = LC70_SYS_SET_VOLUME_OFF;
        soundVolume.sysSetVolumeRow[0].ucVolumeSelect[1] = LC70_SYS_SET_VOLUME_MIN;
        soundVolume.sysSetVolumeRow[0].ucVolumeSelect[2] = LC70_SYS_SET_VOLUME_MID;
        soundVolume.sysSetVolumeRow[0].ucVolumeSelect[3] = LC70_SYS_SET_VOLUME_MAX;
        soundVolume.sysSetVolumeRow[0].ucVolume = volumeLevel;
        ret = writeSystemSettingFile(LC70_FILE_ID_SYS_SET_VOLUME, &soundVolume);

        if (ret > 0) {
            ret = LC70_OK;
        }

#endif
    } else {
        ret = _dummy->setSoundVolume(volumeLevel);
    }

    return ret;
}

LC70_ERR_T Communicator::setBrightness(int brightnessLevel)
{
    LC70_ERR_T ret = LC70_OK;

    if (_mode == ActionMode::PRODUCTION) {
#if Q_PROCESSOR_ARM
        LC70_SYS_SET_BRIGHTNESS_T brightness;
        brightness.sysSetBrightnessRow[0].ucBrightnessSelect[0] = LC70_BRIGHTNESS_MIN;
        brightness.sysSetBrightnessRow[0].ucBrightnessSelect[1] = LC70_BRIGHTNESS_MID;
        brightness.sysSetBrightnessRow[0].ucBrightnessSelect[2] = LC70_BRIGHTNESS_MAX;
        brightness.sysSetBrightnessRow[0].ucBrightness = brightnessLevel;

        ret = writeSystemSettingFile(LC70_FILE_ID_SYS_SET_BRIGHTNESS, &brightness);

        if (ret > 0) {
            ret = LC70_OK;
        }

#endif
    } else {
        ret = _dummy->setBrightness(brightnessLevel);
    }

    return ret;
}

LC70_ERR_T Communicator::setScreenBrightness(int brightnessLevel)
{
    LC70_ERR_T ret = LC70_OK;

    if (_mode == ActionMode::PRODUCTION) {
#if Q_PROCESSOR_ARM
        LC70_BRIGHTNESS_T brightness;

        if (brightnessLevel == LC70_BRIGHTNESS_MIN) {
            // 明るさ設定 : 暗
            brightness = LC70_BRIGHTNESS_MIN;
        } else if (brightnessLevel == LC70_BRIGHTNESS_MID) {
            // 明るさ設定 : 普
            brightness = LC70_BRIGHTNESS_MID;
        } else {
            // 明るさ設定 : 明
            brightness = LC70_BRIGHTNESS_MAX;
        }

        ret = lc70_forceBrightness(brightness);

        if (ret != LC70_OK) {
            writeLog("lc70_forceBrightness()");
        }

#endif
    } else  {
        ret = _dummy->setScreenBrightness(brightnessLevel);
    }

    return ret;
}

LC70_ERR_T Communicator::setLocalFrequency(QVariantList localFrequency)
{
    LC70_ERR_T ret = LC70_OK;

    if (_mode == ActionMode::PRODUCTION) {
#if Q_PROCESSOR_ARM
        LC70_MEAS_SET_LOCAL_FREQ_T lc70Data;

        // 現在のの設定値を取得
        if (0 > readSystemSettingFile(LC70_FILE_ID_MEAS_SET_LOCAL_FREQ, &lc70Data, FALSE)) {
        }

        // 局部発振周波数データを設定する。
        for (int i = 0; i < LC70_MEAS_SET_LOCAL_FREQ_IDX_MAX; i++) {
            lc70Data.measSetLocalFreqRow[i].ucBand = localFrequency.at(i).toList()[0].toInt();
            lc70Data.measSetLocalFreqRow[i].ucWave = localFrequency.at(i).toList()[1].toInt();
            lc70Data.measSetLocalFreqRow[i].ulLocalFreq1 = localFrequency.at(i).toList()[2].toUInt();
            lc70Data.measSetLocalFreqRow[i].ulLocalFreq2 = localFrequency.at(i).toList()[3].toUInt();
            lc70Data.measSetLocalFreqRow[i].ulLocalFreq3 = localFrequency.at(i).toList()[4].toUInt();
            lc70Data.measSetLocalFreqRow[i].ulLocalFreq4 = localFrequency.at(i).toList()[5].toUInt();
            lc70Data.measSetLocalFreqRow[i].ulLocalFreq5 = localFrequency.at(i).toList()[6].toUInt();
            lc70Data.measSetLocalFreqRow[i].ulLocalFreqUser = localFrequency.at(i).toList()[7].toUInt();
        }

        ret = writeMeasureSettingFile(LC70_FILE_ID_MEAS_SET_LOCAL_FREQ, &lc70Data);

        if (ret > 0) {
            ret = LC70_OK;
        }

#endif
    } else {
        ret = _dummy->setLocalFrequency(localFrequency);
    }

    return ret;
}

LC70_ERR_T Communicator::setIsEnableCableLoss(int isEnableCableLoss)
{
    LC70_ERR_T ret = LC70_OK;

    if (_mode == ActionMode::PRODUCTION) {
#if Q_PROCESSOR_ARM
        LC70_MEAS_SET_CABLE_LOSS_ENB_T lc70Data;
        lc70Data.measSetCableLossEnbRow[0].ucCorrectionEnbFlg = isEnableCableLoss;

        ret = writeMeasureSettingFile(LC70_FILE_ID_MEAS_SET_CABLE_LOSS_ENB, &lc70Data);

        if (ret > 0) {
            ret = LC70_OK;
        }

#endif
    } else {
        ret = _dummy->setIsEnableCableLoss(isEnableCableLoss);
    }

    return ret;
}

LC70_ERR_T Communicator::setMERCN(int mercnValue)
{
    LC70_ERR_T ret = LC70_OK;

    if (_mode == ActionMode::PRODUCTION) {
#if Q_PROCESSOR_ARM
        LC70_MEAS_SET_CNRMER_T mercn;
        mercn.measSetCnrmerRow[0].ucSelCnMerSelect[0] = LC70_MEAS_SET_SEL_CNR;
        mercn.measSetCnrmerRow[0].ucSelCnMerSelect[1] = LC70_MEAS_SET_SEL_MER;
        mercn.measSetCnrmerRow[0].ucSelCnMer = mercnValue;

        ret = writeMeasureSettingFile(LC70_FILE_ID_MEAS_SET_CNRMER, &mercn);

        if (ret > 0) {
            ret = LC70_OK;
        }

#endif
    } else {
        ret = _dummy->setMERCN(mercnValue);
    }

    return ret;
}

LC70_ERR_T Communicator::setCATVChannelFrequency(int settingNo)
{
    LC70_ERR_T ret = LC70_OK;

    if (_mode == ActionMode::PRODUCTION) {
#if Q_PROCESSOR_ARM
        LC70_MEAS_SET_CATV_FREQ_T lc70Data;

        // 現在設定中のCATVチャンネル周波数情報を取得
        if (0 > readMeasureSettingFile(LC70_FILE_ID_MEAS_SET_CATV_FREQ, &lc70Data, FALSE)) {
        }

        if (settingNo == 1) {
            // settingNoが1の場合
            // CATVチャンネル周波数の設定1の実設定情報を以下に更新する。
            // 設定1 ucUse: 1
            // 設定2 ucUse: 0
            lc70Data.measSetCatvFreqRow[LC70_MEAS_SET_CATV_FREQ_IDX_SET1_QAM].ucUse  = 1;
            lc70Data.measSetCatvFreqRow[LC70_MEAS_SET_CATV_FREQ_IDX_SET1_OFDM].ucUse = 1;
            lc70Data.measSetCatvFreqRow[LC70_MEAS_SET_CATV_FREQ_IDX_SET2_QAM].ucUse  = 0;
            lc70Data.measSetCatvFreqRow[LC70_MEAS_SET_CATV_FREQ_IDX_SET2_OFDM].ucUse = 0;
        } else if (settingNo == 2) {
            lc70Data.measSetCatvFreqRow[LC70_MEAS_SET_CATV_FREQ_IDX_SET1_QAM].ucUse  = 0;
            lc70Data.measSetCatvFreqRow[LC70_MEAS_SET_CATV_FREQ_IDX_SET1_OFDM].ucUse = 0;
            lc70Data.measSetCatvFreqRow[LC70_MEAS_SET_CATV_FREQ_IDX_SET2_QAM].ucUse  = 1;
            lc70Data.measSetCatvFreqRow[LC70_MEAS_SET_CATV_FREQ_IDX_SET2_OFDM].ucUse = 1;
        } else {
            // 何もしない
        }

        ret = writeMeasureSettingFile(LC70_FILE_ID_MEAS_SET_CATV_FREQ, &lc70Data);

        if (ret > 0) {
            ret = LC70_OK;
        }

#endif
    } else {
        ret = _dummy->setCATVChannelFrequency(settingNo);
    }

    return ret;
}

LC70_ERR_T Communicator::setRFOptical(int rfOptical)
{
    LC70_ERR_T ret = LC70_OK;

    if (_mode == ActionMode::PRODUCTION) {
#if Q_PROCESSOR_ARM
        LC70_MEAS_SET_INPUT_T inputSelect;
        inputSelect.measSetInputRow[0].ucInputSelect[0] = LC70_MEAS_SET_INPUT_RF;
        inputSelect.measSetInputRow[0].ucInputSelect[1] = LC70_MEAS_SET_INPUT_OPT;
        inputSelect.measSetInputRow[0].ucInput = rfOptical;

        ret = writeMeasureSettingFile(LC70_FILE_ID_MEAS_SET_INPUT, &inputSelect);

        if (ret > 0) {
            ret = LC70_OK;
        }

#endif
    } else {
        ret = _dummy->setMeasuringInputSignal(static_cast<LC70_INPUT_T>(rfOptical));
    }

    return ret;
}

LC70_ERR_T Communicator::startScan(LC70_MEAS_MULTI_T *condition)
{
    LC70_ERR_T ret = LC70_OK;

    if (_mode == ActionMode::PRODUCTION) {
#if Q_PROCESSOR_ARM
        timer.restart();
        ret = lc70_startScan(_displayType, condition);
        _systemLog.write(QString("ラッパー関数呼び出し"),
                         QString("%1 called %2 ms(id=0x%3)").arg(QString("スキャン開始(lc70_startScan)")).arg(
                             timer.elapsed()).arg(QString::number((long long)QThread::currentThreadId(), 16)), LOG_INFO);

        if (ret != LC70_OK) {
            _systemLog.write(QString("ラッパー関数呼び出し"),
                             QString("### ERROR lc70_startScan ret = %1").arg(static_cast<int>(ret)), LOG_INFO);
        }

#endif
    } else {
    }

    return ret;
}

LC70_ERR_T Communicator::stopScan()
{
    LC70_ERR_T ret = LC70_OK;

    if (_mode == ActionMode::PRODUCTION) {
#if Q_PROCESSOR_ARM
        timer.restart();
        ret = lc70_stopScan(_displayType);
        _systemLog.write(QString("ラッパー関数呼び出し"),
                         QString("%1 called %2 ms(id=0x%3)").arg(QString("スキャン停止(lc70_stopScan)")).arg(
                             timer.elapsed()).arg(
                             QString::number((long long)QThread::currentThreadId(), 16)), LOG_INFO);

        if (ret != LC70_OK) {
            _systemLog.write(QString("ラッパー関数呼び出し"),
                             QString("### ERROR lc70_stopScan ret = %1").arg(static_cast<int>(ret)), LOG_INFO);
        }

#endif
    } else {
    }

    return ret;
}

LC70_ERR_T Communicator::scanResult(int16_t index, LC70_MEAS_MULTI_RESULT_T &result)
{
    LC70_ERR_T ret = LC70_OK;

    if (_mode == ActionMode::PRODUCTION) {
#if Q_PROCESSOR_ARM
        timer.restart();
        ret = lc70_getScanResult(index, &result);
        _systemLog.write(QString("ラッパー関数呼び出し"),
                         QString("%1 called %2 ms(id=0x%3)").arg(QString("スキャン結果取得(lc70_getScanResult)")).arg(
                             timer.elapsed()).arg(
                             QString::number((long long)QThread::currentThreadId(), 16)), LOG_INFO);

        if (ret != LC70_OK) {
            _systemLog.write(QString("ラッパー関数呼び出し"),
                             QString("### ERROR lc70_getScanResult ret = %1").arg(static_cast<int>(ret)), LOG_INFO);
        }

#endif
    } else {
    }


    return ret;
}

LC70_ERR_T Communicator::readUserLogSetFileList(const QString &dirname,
                                                QList<LC70_LOGGER_SET_LOGGER_FLIST_ROW_T> &userlogSetFileList, ChannelEditMode mode, const QString &path, const QString &fileName)
{
    LC70_ERR_T ret = LC70_OK;

    LC70_FILE_ID_T fileId = LC70_FILE_ID_USER_SET_USER_FLIST;

    userlogSetFileList.clear();

    if(mode == CHANNEL_EDIT_USER) {
        fileId = LC70_FILE_ID_USER_SET_USER_FLIST;
    } else if(mode == CHANNEL_EDIT_USER_USB1) {
        fileId = LC70_FILE_ID_USER_SET_USER_USB1_FLIST;
    } else if(mode == CHANNEL_EDIT_USER_USB2) {
        fileId = LC70_FILE_ID_USER_SET_USER_USB2_FLIST;
    } else if(mode == CHANNEL_EDIT_DATA_LOGGER) {
        fileId = LC70_FILE_ID_LOGGER_SET_LOGGER_FLIST;
    } else if(mode == CHANNEL_EDIT_DATA_LOGGER_USB1) {
        fileId = LC70_FILE_ID_LOGGER_SET_LOGGER_USB1_FLIST;
    } else if(mode == CHANNEL_EDIT_DATA_LOGGER_USB2) {
        fileId = LC70_FILE_ID_LOGGER_SET_LOGGER_USB2_FLIST;
    }

    if (_mode == ActionMode::PRODUCTION) {
#if Q_PROCESSOR_ARM
        LC70_LOGGER_SET_LOGGER_FLIST_T fileList;
        memset(&fileList, 0, sizeof (fileList));

        QFile file(QString("%1/%2").arg(path).arg(fileName));

        // ファイルがなければ作る
        if (!file.exists()) {
            // 番号設定
            for (uint8_t i = 0; i < LC70_LOGGER_SET_LOGGER_FILE_MAX; ++i) {
                fileList.loggerSetFlist[i].ucNo = i + 1;
            }

            int result = lc70_writeSetFile(fileId, &fileList, "", LC70_LOGGER_SET_LOGGER_FILE_MAX);

            if (result < LC70_OK) {
                ret = static_cast<LC70_ERR_T>(result);
                writeLog(QString().sprintf("### ERROR lc70_writeSetFile - mode:%d fieldId:%d ret:%d", mode, fileId, ret));

                _systemLog.write(QString("ラッパー関数呼び出し"),
                                 QString("%1 called %2 ms").arg(QString("lc70_writeSetFile() - %1").arg(fileId)).arg(timer.elapsed()));
                return converErrorCode(result);
            }
        }

        // No.1〜100(最大行数に依存のファイル名が空の行が存在していることを前提とする
        int lines = lc70_readSetFile(fileId, &fileList, "", FALSE);

        if (lines < LC70_OK) {
            ret = static_cast<LC70_ERR_T>(lines);
            writeLog(QString().sprintf("### ERROR lc70_readSetFile - mode:%d fieldId:%d ret:%d", mode, fileId, ret));

            _systemLog.write(QString("ラッパー関数呼び出し"),
                             QString("%1 called %2 ms").arg(QString("lc70_readSetFile() - %1").arg(fileId)).arg(timer.elapsed()));
            return converErrorCode(lines);
        }

        for (int i = 0; i < lines; i++) {
            LC70_LOGGER_SET_LOGGER_FLIST_ROW_T *item = &(fileList.loggerSetFlist[i]);

            userlogSetFileList.append(*item);
        }

#endif
    } else {
        ret = _dummy->readSetFileList(dirname, userlogSetFileList, mode);
    }

    return ret;
}

LC70_ERR_T Communicator::writeUserLogSetFileList(const QString &dirname,
                                                 QList<LC70_LOGGER_SET_LOGGER_FLIST_ROW_T> &loggerSetFileList, ChannelEditMode mode, const QString &fileName)
{
    LC70_ERR_T ret = LC70_OK;

    LC70_FILE_ID_T fileId = LC70_FILE_ID_USER_SET_USER_FLIST;

    if(mode == CHANNEL_EDIT_USER) {
        fileId = LC70_FILE_ID_USER_SET_USER_FLIST;
    } else if(mode == CHANNEL_EDIT_USER_USB1) {
        fileId = LC70_FILE_ID_USER_SET_USER_USB1_FLIST;
    } else if(mode == CHANNEL_EDIT_USER_USB2) {
        fileId = LC70_FILE_ID_USER_SET_USER_USB2_FLIST;
    } else if(mode == CHANNEL_EDIT_DATA_LOGGER) {
        fileId = LC70_FILE_ID_LOGGER_SET_LOGGER_FLIST;
    } else if(mode == CHANNEL_EDIT_DATA_LOGGER_USB1) {
        fileId = LC70_FILE_ID_LOGGER_SET_LOGGER_USB1_FLIST;
    } else if(mode == CHANNEL_EDIT_DATA_LOGGER_USB2) {
        fileId = LC70_FILE_ID_LOGGER_SET_LOGGER_USB2_FLIST;
    }

    if (_mode == ActionMode::PRODUCTION) {
#if Q_PROCESSOR_ARM
        LC70_LOGGER_SET_LOGGER_FLIST_T filelist;
        int lines = 0;

        for (int i = 0; i < loggerSetFileList.size(); i++) {
            filelist.loggerSetFlist[i] = loggerSetFileList.at(i);
            lines++;
        }

        int retcode = lc70_writeSetFile(
                    fileId,
                    &filelist,
                    fileName.toUtf8().constData(),
                    lines);

        if (retcode < 0) {
            ret = static_cast<LC70_ERR_T>(retcode);
        }

#endif
    } else {
        ret = _dummy->writeSetFileList(dirname, loggerSetFileList, mode);
    }

    return ret;
}

LC70_ERR_T Communicator::startOpticalMeasurement(uint32_t ulWaveLength)
{
    LC70_ERR_T ret = LC70_OK;

    if (_mode == ActionMode::PRODUCTION) {
#if Q_PROCESSOR_ARM
        int retcode = lc70_startOptMeas(ulWaveLength);

        if (retcode < 0) {
            writeLog(QString("### ERROR lc70_startOptMeas() returnCode : %1").arg(retcode));
            ret = static_cast<LC70_ERR_T>(retcode);
        }

#endif
    } else {
        ret = _dummy->startOpticalMeasurement(ulWaveLength);
    }

    return ret;
}

LC70_ERR_T Communicator::stopOpticalMeasurement()
{
    LC70_ERR_T ret = LC70_OK;

    if (_mode == ActionMode::PRODUCTION) {
#if Q_PROCESSOR_ARM
        int retcode = lc70_stopOptMeas();

        if (retcode < 0) {
            writeLog(QString("### ERROR lc70_stopOptMeas() returnCode : %1").arg(retcode));
            ret = static_cast<LC70_ERR_T>(retcode);
        }

#endif
    } else {
        ret = _dummy->stopOpticalMeasurement();
    }

    return ret;
}

LC70_ERR_T Communicator::getOpticalMeasurementResult(QVariant &result)
{
    LC70_ERR_T ret = LC70_OK;

    if (_mode == ActionMode::PRODUCTION) {
#if Q_PROCESSOR_ARM
        LC70_MEAS_OPT_RESULT_T measurementData;
        int retcode = lc70_getOptMeasResult(&measurementData);

        // 取得した測定結果を設定する
        result = measurementData.lOptPow;

        if (retcode != LC70_OK) {
            writeLog(QString("### ERROR lc70_getOptMeasResult() returnCode : %1").arg(retcode));
        }

        if (retcode < 0) {
            ret = static_cast<LC70_ERR_T>(retcode);
        }

#endif
    } else {
        ret = _dummy->getOpticalMeasurementResult(result);
    }

    return ret;
}

LC70_ERR_T Communicator::startDCMeasurement()
{
    LC70_ERR_T ret = LC70_OK;

    if (_mode == ActionMode::PRODUCTION) {
#if Q_PROCESSOR_ARM
        int retcode = lc70_startDcMeas();

        if (retcode < 0) {
            writeLog(QString("### ERROR lc70_startDcMeas() returnCode : %1").arg(retcode));
            ret = static_cast<LC70_ERR_T>(retcode);
        }

#endif
    } else {
        ret = _dummy->startDCMeasurement();
    }

    return ret;
}

LC70_ERR_T Communicator::getDCMeasurementResult(QVariant &result)
{
    LC70_ERR_T ret = LC70_OK;

    if (_mode == ActionMode::PRODUCTION) {
#if Q_PROCESSOR_ARM
        LC70_MEAS_DC_RESULT_T measurementData;
        int retcode = lc70_getDcMeasResult(&measurementData);

        // 取得した測定結果を設定する
        result = measurementData.ulDcV;

        if (retcode != LC70_OK) {
            writeLog(QString("### ERROR lc70_getDcMeasResult() returnCode : %1").arg(retcode));
        }

        if (retcode < 0) {
            ret = static_cast<LC70_ERR_T>(retcode);
        }

#endif
    } else {
        ret = _dummy->getDCMeasurementResult(result);
    }

    return ret;
}

void Communicator::changeDisplayType(LC70_DISP_TYPE_T displayType)
{
    _systemLog.write(QString(typeid(*this).name()),
                     QString("Change Display Type %1 to %2").arg(convertDisplayType(_displayType)).arg(convertDisplayType(displayType)), LOG_NOTICE);
    _displayType = displayType;
}

bool Communicator::isSupplying() const
{
    return _supplying;
}

LC70_ERR_T Communicator::updateProgram(LC70_PRG_UPDATE_T *updater)
{
    LC70_ERR_T ret = LC70_ERR;

    if (_mode == ActionMode::PRODUCTION) {
#if Q_PROCESSOR_ARM
        QString message =
                QString("[PROGRAM UPATE] MAIN:%1 SUB %2").arg((char *)updater->ucMainVer).arg((char *)updater->ucSubVer);
        _systemLog.write(QString("ラッパー関数呼び出し"), message, LOG_INFO);
        ret = lc70_prgUpdate(updater);

        if (ret != LC70_OK) {
            _systemLog.write(QString("ラッパー関数呼び出し"),
                             QString("### ERROR lc70_prgUpdate ret = %1").arg(static_cast<int>(ret)), LOG_INFO);
        }

#endif
    } else {
        ret = LC70_OK;
    }

    return ret;
}

bool Communicator::deleteAllSettingFile()
{
    int32_t ret = LC70_OK;
#if Q_PROCESSOR_ARM
    QString message = QString("[DELETE ALL SETTING]");
    _systemLog.write(QString("ラッパー関数呼び出し"), message, LOG_INFO);

    ret = lc70_deleteAllSetFile();

    if (ret != LC70_OK) {
        _systemLog.write(QString("ラッパー関数呼び出し"),
                         QString("### ERROR lc70_deleteAllSetFile ret = %1").arg(static_cast<int>(ret)), LOG_INFO);
    }
#endif

    return (ret == LC70_OK) ? true : false;
}

LC70_ERR_T Communicator::notifyLowBatteryStatusChanged(bool isLowBattery)
{
    LC70_ERR_T ret = LC70_OK;

#if Q_PROCESSOR_ARM
    QString message = QString("[NOTIFY LOWBATTERY STATUS CHANGED]");
    _systemLog.write(QString("ラッパー関数呼び出し"), message, LOG_INFO);

    ret = lc70_ntfLowBattStatChg(isLowBattery ? 1 : 0);

    if (ret != LC70_OK) {
        _systemLog.write(QString("ラッパー関数呼び出し"),
                         QString("### ERROR lc70_ntfLowBattStatChg ret = %1").arg(static_cast<int>(ret)), LOG_INFO);
    }
#endif
    return ret;
}

LC70_ERR_T Communicator::measureStatus(LC70_MEAS_STAT_T *status)
{
    LC70_ERR_T ret = LC70_OK;

    if (_mode == ActionMode::PRODUCTION) {
#if Q_PROCESSOR_ARM
        timer.restart();
        ret = lc70_getMeasStatus(status);
        _systemLog.write(QString("ラッパー関数呼び出し"),
                         QString("%1 called %2 ms(id=0x%3)").arg(QString("測定状態取得(lc70_getMeasStatus)")).arg(
                             timer.elapsed()).arg(
                             QString::number((long long)QThread::currentThreadId(), 16)), LOG_NOTICE);

        if (ret != LC70_OK) {
            _systemLog.write(QString("ラッパー関数呼び出し"),
                             QString("### ERROR lc70_getMeasStatus ret = %1").arg(static_cast<int>(ret)), LOG_NOTICE);
        }

#endif
    } else {
        status->ulMeasStat = 1;
        status->usInputStat = 0;    // 光測定の場合 1 に変更する
    }

    return ret;
}

LC70_ERR_T Communicator::standByStatus(LC70_STANDBY_STAT_T *status)
{
    LC70_ERR_T ret = LC70_OK;

    if (_mode == ActionMode::PRODUCTION) {
#if Q_PROCESSOR_ARM
        timer.restart();
        ret = lc70_getStandbyStat(status);
        _systemLog.write(QString("ラッパー関数呼び出し"),
                         QString("%1 called %2 ms(id=0x%3)").arg(QString("スタンバイ状態取得(lc70_getStandbyStat)")).arg(
                             timer.elapsed()).arg(
                             QString::number((long long)QThread::currentThreadId(), 16)), LOG_NOTICE);

        if (ret != LC70_OK) {
            _systemLog.write(QString("ラッパー関数呼び出し"),
                             QString("### ERROR lc70_getStandbyStat ret = %1").arg(static_cast<int>(ret)), LOG_NOTICE);
        }

#endif
    } else {
        status->ulStandbyStat = LC70_STANDBY_STAT_STANDBY;
    }

    return ret;
}

LC70_ERR_T Communicator::wakeUpStatus(LC70_STANDBY_STAT_T *status)
{
    LC70_ERR_T ret = LC70_OK;

    if (_mode == ActionMode::PRODUCTION) {
#if Q_PROCESSOR_ARM
        timer.restart();
        ret = lc70_getStandbyStat(status);
        _systemLog.write(QString("ラッパー関数呼び出し"),
                         QString("%1 called %2 ms(id=0x%3)").arg(QString("スタンバイ状態取得(lc70_getStandbyStat)")).arg(
                             timer.elapsed()).arg(
                             QString::number((long long)QThread::currentThreadId(), 16)), LOG_NOTICE);

        if (ret != LC70_OK) {
            _systemLog.write(QString("ラッパー関数呼び出し"),
                             QString("### ERROR lc70_getStandbyStat ret = %1").arg(static_cast<int>(ret)), LOG_NOTICE);
        }

#endif
    } else {
        status->ulStandbyStat = LC70_STANDBY_STAT_WAKEUP;
    }

    return ret;
}

LC70_ERR_T Communicator::initializeFactoryDefault()
{
    LC70_ERR_T ret = LC70_OK;

    if (_mode == ActionMode::PRODUCTION) {
#if Q_PROCESSOR_ARM
        timer.restart();
        ret = lc70_initFactoryDefault();
        _systemLog.write(QString("ラッパー関数呼び出し"),
                         QString("%1 called %2 ms(id=0x%3)").arg(QString("工場出荷状態初期化要求(lc70_initFactoryDefault)")).arg(
                             timer.elapsed()).arg(
                             QString::number((long long)QThread::currentThreadId(), 16)), LOG_NOTICE);

        if (ret != LC70_OK) {
            _systemLog.write(QString("ラッパー関数呼び出し"),
                             QString("### ERROR lc70_initFactoryDefault ret = %1").arg(static_cast<int>(ret)), LOG_NOTICE);
        }

#endif
    }

    return ret;
}

LC70_ERR_T Communicator::setDeveloperMode(const uint8_t &selection)
{
    LC70_ERR_T ret = LC70_OK;

    if (_mode == ActionMode::PRODUCTION) {
#if Q_PROCESSOR_ARM
        timer.restart();
        ret = lc70_setDeveloperMode(selection);
        _systemLog.write(QString("ラッパー関数呼び出し"),
                         QString("%1 %2 called %3 ms(id=0x%4)").arg(QString("デベロッパモード設定要求(lc70_setDeveloperMode)")).arg((selection == 1) ? QString("ON") : QString("OFF")).arg(
                             timer.elapsed()).arg(
                             QString::number((long long)QThread::currentThreadId(), 16)), LOG_NOTICE);

        if (ret != LC70_OK) {
            _systemLog.write(QString("ラッパー関数呼び出し"),
                             QString("### ERROR lc70_setDeveloperMode ret = %1").arg(static_cast<int>(ret)), LOG_NOTICE);
        }

#endif
    }

    return ret;
}

LC70_ERR_T Communicator::resetSubCpu()
{
    LC70_ERR_T ret = LC70_OK;

    if (_mode == ActionMode::PRODUCTION) {
#if Q_PROCESSOR_ARM
        // サブ CPU に対する強制スタンバイ、ウェイクアップの実行
        _systemLog.write(QString("ラッパー関数呼び出し"),
                         QString("強制スタンバイ要求(lc70_force_standby)"), LOG_NOTICE);
        ret = lc70_force_standby();

        if (ret != LC70_OK) {
            _systemLog.write(QString("ラッパー関数呼び出し"),
                             QString("### ERROR lc70_force_standby ret = %1").arg(static_cast<int>(ret)), LOG_NOTICE);
            return ret;
        }

        _systemLog.write(QString("ラッパー関数呼び出し"),
                         QString("強制ウェイクアップ要求(lc70_force_wakeup)"), LOG_NOTICE);
        ret = lc70_force_wakeup();

        if (ret != LC70_OK) {
            _systemLog.write(QString("ラッパー関数呼び出し"),
                             QString("### ERROR lc70_force_wakeup ret = %1").arg(static_cast<int>(ret)), LOG_NOTICE);
            return ret;
        }
#endif
    } else {
        ret = _dummy->resetSubCpu();
    }

    return ret;
}

LC70_ERR_T Communicator::setPowerSupplyForMultiBand(const uint8_t &value)
{
    LC70_ERR_T ret = LC70_OK;

    if (value == 0) {
        _supplying = false;
    } else {
        _supplying = true;
    }

    if (_mode == ActionMode::PRODUCTION) {
#if Q_PROCESSOR_ARM
        timer.restart();
        ret = lc70_setPowerSupplyMultiMeasMixBand(value);
        _systemLog.write(QString("ラッパー関数呼び出し"),
                         QString("%1 called %2 ms").arg(QString("lc70_setPowerSupplyMultiMeasMixBand")).arg(timer.elapsed()));

        if (ret != LC70_OK) {
            _systemLog.write(QString("ラッパー関数呼び出し"),
                             QString("### ERROR lc70_setPowerSupplyMultiMeasMixBand(%1) ret = %2").arg(value).arg(
                                 static_cast<int>(ret)));
        }
#endif
    }

    return ret;
}

void Communicator::setDataFileList(const QString &dataFileListFileName, const QString &dataFileListPath)
{
    _dataFileListFileName = dataFileListFileName;
    _dataFileListPath = dataFileListPath;
}

void Communicator::setMeasurementDataPath(const QString &singleMeasurementDataPath, const QString &multiMeasurementDataPath, const QString &userMeasurementDataPath, const QString &loggerMeasurementDataPath)
{
    _singleMeasurementDataPath = singleMeasurementDataPath;
    _multiMeasurementDataPath = multiMeasurementDataPath;
    _userMeasurementDataPath = userMeasurementDataPath;
    _loggerMeasurementDataPath = loggerMeasurementDataPath;
}

LC70_ERR_T Communicator::converErrorCode(const int &result)
{
    if (result >= LC70_OK) {
        return LC70_OK;
    } else if (result < LC70_ERR_BUZZ_REPEATING) {
        // LC70_ERR_T の定義に依存するため、戻り値が追加されたら対応が必要
        return LC70_ERR;
    } else {
        return static_cast<LC70_ERR_T>(result);
    }
}

QString Communicator::convertDisplayType(const LC70_DISP_TYPE_T &displayType)
{
    QString screenName;

    switch (displayType) {
    case LC70_DISP_TYPE_NOT_MEAS:       /* 測定画面以外 */
        screenName = QString("%1(Other than measurement)").arg(displayType);
        break;

    case LC70_DISP_TYPE_METER:          /* メータ画面 */
        screenName = QString("%1(Meter)").arg(displayType);
        break;

    case LC70_DISP_TYPE_CONSTA:         /* コンスタ画面 */
        screenName = QString("%1(Constellation)").arg(displayType);
        break;

    case LC70_DISP_TYPE_DLY_PRF:        /* 遅延プロファイル画面 */
        screenName = QString("%1(Delay profile)").arg(displayType);
        break;

    case LC70_DISP_TYPE_GRAPH:          /* グラフ画面 */
        screenName = QString("%1(Graph)").arg(displayType);
        break;

    case LC70_DISP_TYPE_LIST_MULTI:     /* リスト画面(マルチ) */
        screenName = QString("%1(List - Multi)").arg(displayType);
        break;

    case LC70_DISP_TYPE_LIST_LOGGER:    /* リスト画面(データロガー) */
        screenName = QString("%1(List - DataLogger)").arg(displayType);
        break;

    default:
        break;
    }

    return screenName;
}

void Communicator::retrieveMeasurementDataFile(const QString &directoryPath, const QString &category, const QStringList &subDirectoryList, QList<QVariantList> &resultList)
{
    // 測定結果の種類を抽出
    QDir measuringResultDirectory(directoryPath);
#if 1	//20210625-dx5
    QStringList measuringResultList = measuringResultDirectory.entryList(QDir::Dirs | QDir::NoDotAndDotDot, QDir::Name | QDir::Reversed);
#else
    QStringList measuringResultList = measuringResultDirectory.entryList(QDir::Dirs | QDir::NoDotAndDotDot, QDir::Time);    
#endif
    // 日付フォルダを列挙(日付が新しいものを上位にしたいので、日付の逆順でソートしておく)
    foreach (QString measuringResultName, measuringResultList) {
        // 日付のディレクトリを抽出
        QString dateDirectoryPath = directoryPath + QDir::separator() + measuringResultName;
        QDir dateDirectory(dateDirectoryPath);

        QStringList dateList = dateDirectory.entryList(QDir::Dirs | QDir::NoDotAndDotDot);
        // 指定されているサブディレクトリの順序でリストに登録する
        foreach(QString subDirectory, subDirectoryList) {
            foreach(QString dataTypeDirectoryName, dateList) {
                if (dataTypeDirectoryName != subDirectory) {
                    continue;
                }
                // PNG, CSV ディレクトリを抽出
                QDir dataTypeDirectory(dateDirectoryPath + QDir::separator() + dataTypeDirectoryName);
                QStringList dataTypeList = dataTypeDirectory.entryList(QDir::Files | QDir::NoDotAndDotDot, QDir::Time);
                foreach (QString fileName, dataTypeList) {
                    resultList.append({category, measuringResultName, subDirectory, fileName, "", ""});
                }
            }
        }
    }
}

void Communicator::retrieveLoggerDataFile(const QString &directoryPath, const QString &category, const QString &fileType, QList<QVariantList> &resultList)
{
    // 測定結果の種類を抽出
    QDir measuringResultDirectory(directoryPath);
#if 1 // 20210707
    QStringList dateList = measuringResultDirectory.entryList(QDir::Dirs | QDir::NoDotAndDotDot, QDir::Name | QDir::Reversed);
#else
    QStringList dateList = measuringResultDirectory.entryList(QDir::Dirs | QDir::NoDotAndDotDot, QDir::Time);
#endif
    // 日付フォルダを列挙(日付が新しいものを上位にしたいので、日付の逆順でソートしておく)
    foreach(QString dataTypeDirectoryName, dateList) {
        QDir dataTypeDirectory(directoryPath + QDir::separator() + dataTypeDirectoryName);
        QStringList dataTypeList = dataTypeDirectory.entryList(QDir::Files | QDir::NoDotAndDotDot, QDir::Time);
        foreach (QString fileName, dataTypeList) {
            resultList.append({category, dataTypeDirectoryName, fileType, fileName, "", ""});
        }
    }
}

LC70_ERR_T Communicator::stopDCMeasurement()
{
    LC70_ERR_T ret = LC70_OK;

    if (_mode == ActionMode::PRODUCTION) {
#if Q_PROCESSOR_ARM
        int retcode = lc70_stopDcMeas();

        if (retcode < 0) {
            writeLog(QString("### ERROR lc70_stopDcMeas() returnCode : %1").arg(retcode));
            ret = static_cast<LC70_ERR_T>(retcode);
        }

#endif
    } else {
        ret = _dummy->stopDCMeasurement();
    }

    return ret;
}

LC70_ERR_T Communicator::getModelInformation(LC70_PRODUCT_INFO_T &result)
{
    LC70_ERR_T ret = LC70_OK;

    if (_mode == ActionMode::PRODUCTION) {
#if Q_PROCESSOR_ARM

        // ラッパー関数を用いて機種情報の取得を行う
        int retcode = lc70_getProductInfo(&result);

        if (retcode != LC70_OK) {
            writeLog(QString("### ERROR lc70_getProductInfo() result : %1").arg(retcode));
            ret = static_cast<LC70_ERR_T>(retcode);
        }

#endif
    } else {
        ret = _dummy->getModelInformation(result);
    }

    return ret;
}
