/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef COMMUNICATOR_H
#define COMMUNICATOR_H

#include <QObject>
#include <QDateTime>
#include "constant.h"
#include "systemlog.h"

extern "C" {
#include "liblc70.h"
#include "lc70ws/libclk.h"
}

#include "lc70dummy.h"

/**
 * @brief ラッパー関数とスタブを使い分けるための Adapter クラス
 */
class Communicator : public QObject
{
    Q_OBJECT
public:
    /**
     * @brief コンストラクター
     * @param mode 動作モード
     * @param systemLog システムロガー
     * @param parent 親オブジェクト
     */
    explicit Communicator(const ActionMode &mode, SystemLog &systemLog, QObject *parent = nullptr);

    /**
     * @brief デストラクター
     */
    ~Communicator();

    /**
     * @brief ラッパー関数ライブラリによる初期化
     * @id ラッパー関数を呼ぶオブジェクトの ID
     * @result 処理の成否
     */
    LC70_ERR_T initialize(int id);

    /**
     * @brief 測定設定ファイル読み込み
     * @param fileId 測定設定項目種別
     * @param data 測定設定項目に応じたデータ
     * @param isDefault デフォルトデータの指定有無
     * @return 処理結果
     */
    LC70_ERR_T readMeasureSettingFile(LC70_FILE_ID_T fileId, void *data, bool isDefault = false);

    /**
     * @brief システム設定ファイル読み込み
     * @param fileId システム設定項目種別
     * @param data システム設定項目種別に応じたデータ
     * @param isDefault デフォルトデータの指定有無
     * @return 処理結果
     */
    LC70_ERR_T readSystemSettingFile(LC70_FILE_ID_T fileId, void *data, bool isDefault = false);

    /**
     * @brief ユーザ設定ファイル読み込み
     * @param fileName ユーザ設定ファイルパス
     * @param userSetting ユーザ設定情報
     * @return 処理結果
     * @note 不使用
     */
    LC70_ERR_T readUserSettingFile(const char *fileName, LC70_USER_SET_T *userSetting);

    /**
     * @brief チャンネル定義ファイル読み込み
     * @param fileId システム設定項目種別
     * @param data システム設定項目種別に応じたデータ
     * @param isDefault デフォルトデータの指定有無
     * @return 処理結果
     */
    LC70_ERR_T readChannelConfigurationFile( LC70_FILE_ID_T fileId, void *data, bool isDefault);

    /**
     * @brief キャリブレーション設定ファイル読み込み
     * @param fileName キャリブレーション設定ファイルパス
     * @param calibrationSetting キャリブレーション設定情報
     * @return 処理結果
     * @note 不使用
     */
    LC70_ERR_T readCalibrationSettingFile(const char *fileName, LC70_CAL_SET_T *calibrationSetting);

    /**
     * @brief 測定設定ファイル書き込み
     * @param fileId 測定設定項目種別
     * @param data 測定設定情報
     * @return 処理結果
     */
    LC70_ERR_T writeMeasureSettingFile(LC70_FILE_ID_T fileId, void *data);

    /**
     * @brief システム設定ファイル書き込み
     * @param fileName システム設定項目種別
     * @param data システム設定情報
     * @return 処理結果
     */
    LC70_ERR_T writeSystemSettingFile(LC70_FILE_ID_T fileId, void *data);

    /**
     * @brief ユーザ設定ファイル書き込み
     * @param fileName ユーザ設定ファイルパス
     * @param userSetting ユーザ設定情報
     * @return 処理結果
     * @note 不使用
     */
    LC70_ERR_T writeUserSettingFile(const char *fileName, LC70_USER_SET_T *userSetting);

    /**
     * @brief チャンネル定義ファイル書き込み
     * @param fileId 設定ファイルID
     * @param data チャンネル定義情報
     * @return 処理結果
     */
    LC70_ERR_T writeChannelConfigurationFile(LC70_FILE_ID_T fileId, void *data);

    /**
     * @brief キャリブレーション設定ファイル書き込み
     * @param fileName キャリブレーション設定ファイルパス
     * @param calibrationSetting キャリブレーション設定情報
     * @return 処理結果
     * @note 不使用
     */
    LC70_ERR_T writeCalibrationSettingFile(const char *fileName, LC70_CAL_SET_T *calibrationSetting);

    /**
     * @brief 故障情報取得
     * @param failureInformation 故障情報
     * @return 処理結果
     */
    LC70_ERR_T failureInformation(LC70_FAIL_INFO_T *failureInformation);

    /**
     * @brief 故障状態通知
     * @param code 故障コード
     * @param status 故障状態
     * @param additionalInformation 追加情報
     * @return 処理結果
     */
    LC70_ERR_T setFailureInformation(uint16_t code, uint8_t status, const uint8_t *additionalInformation);

    /**
     * @brief キー(ボタン)入力状態取得
     * @param keyStatus キー入力状態
     * @return 処理結果
     */
    LC70_ERR_T keyStatus(LC70_KEY_STAT_T *keyStatus);

    /**
     * @brief デバイス(方位・バッテリ・ストレージ)状態取得
     * @param deviceStatus デバイス状態
     * @return 処理結果
     */
    LC70_ERR_T deviceStatus(LC70_DEV_STAT_T *deviceStatus);

    /**
     * @brief 測定入力信号設定
     * @param inputSignal 入力信号
     * @return 処理結果
     * @note 不使用(0.09版で削除)
     */
    LC70_ERR_T setMeasuringInputSignal(LC70_INPUT_T inputSignal);

    /**
     * @brief 給電設定
     * @param outputVoltage 出力電圧
     * @return 処理結果
     */
    LC70_ERR_T setPowerSupply(LC70_POWER_SUPPLY_T outputVoltage);

    /**
     * @brief データ有効閾値設定
     * @param blockNumber 有効閾値設定ブロック数
     * @param dataThresholdBlock 有効閾値設定ブロックの先頭アドレス
     * @return 処理結果
     * @note 不使用(0.09版で削除)
     */
    LC70_ERR_T setDataThreshold(int32_t dataThresholdBlockNumber, LC70_DATA_THR_T *dataThresholdBlock);

    /**
     * @brief 測定状態取得
     * @param measuringStatus 測定状態
     * @return 処理結果
     */
    LC70_ERR_T measuringStatus(LC70_MEAS_STAT_T *measuringStatus);

    /**
     * @brief シングルチャンネル測定開始
     * @param measuringCondition 測定条件
     * @return 処理結果
     */
    LC70_ERR_T startSingleMeasuring(LC70_MEAS_SINGLE_T *measuringCondition);

    /**
     * @brief シングルチャンネル測定停止
     * @return 処理結果
     */
    LC70_ERR_T stopSingleMeasuring(void);

    /**
     * @brief コンスタレーション測定開始
     * @param collectionPoints 収集点数
     * @return 処理結果
     */
    LC70_ERR_T startConstellationMeasuring(LC70_CONSTA_NUM_T collectionPoints);

    /**
     * @brief コンスタレーション測定停止
     * @return 処理結果
     */
    LC70_ERR_T stopConstellationMeasuring(void);

    /**
     * @brief シングルチャンネル測定結果取得
     * @param constellationPoints コンスタレーションデータの読み出し点数(0～200)
     * @param measuringResult 測定結果
     * @return 処理結果
     */
    LC70_ERR_T singleMeasuringResult(int16_t constellationPoints,
                                     LC70_MEAS_SINGLE_RESULT_T *measuringResult);

    /**
     * @brief 遅延プロファイル測定開始
     * @param measureCondition 測定条件
     * @return 処理結果
     */
    LC70_ERR_T startDelayProfileMeasuring(const LC70_MEAS_SINGLE_T &measureCondition);

    /**
     * @brief 遅延プロファイル測定停止
     * @return 処理結果
     */
    LC70_ERR_T stopDelayProfileMeasuring(void);

    /**
     * @brief 遅延プロファイル測定結果取得
     * @param result 測定結果
     * @return 処理結果
     */
    LC70_ERR_T delayProfileMeasuring(LC70_MEAS_DLY_PRF_RESULT_T *result);

    /**
     * @brief マルチチャンネル測定開始
     * @param measuringCondition 測定条件
     * @return 処理結果
     */
    LC70_ERR_T startMultiMeasuring(LC70_MEAS_MULTI_T *measuringCondition);

    /**
     * @brief マルチチャンネル測定停止(優先測定も停止)
     * @return 処理結果
     */
    LC70_ERR_T stopMultiMeasuring(void);

    /**
     * @brief マルチチャンネル測定時の測定チャンネルの設定
     * @param measureEnabled 測定要否情報の先頭アドレス
     * @return 処理結果
     * @note 不使用(0.09版で削除)
     */
    LC70_ERR_T setMultiMeasuringChannel(uint8_t *measureEnabled);

    /**
     * @brief マルチチャンネル測定結果取得
     * @param index 測定インデックス
     * @param measuringResult 測定結果
     * @return 処理結果
     */
    LC70_ERR_T multiMeasuringResult(int16_t index, LC70_MEAS_MULTI_RESULT_T *measuringResult);

    /**
     * @brief マルチチャンネル優先測定開始
     * @param measuringCondition 測定条件
     * @return 処理結果
     */
    LC70_ERR_T startPrefferedChannelInMultiMeasuring(LC70_MEAS_SINGLE_T *measuringCondition);

    /**
     * @brief マルチチャンネル優先測定停止
     * @param measuringCondition 測定条件
     * @return 処理結果
     */
    LC70_ERR_T stopPrefferedChannelInMultiMeasuring(void);

    /**
     * @brief マルチチャンネル優先測定結果取得
     * @param measuringResult 測定結果
     * @return 処理結果
     */
    LC70_ERR_T multiMeasuringPrefferedChannelResult(LC70_MEAS_SINGLE_RESULT_T *measuringResult);

    /**
     * @brief 再起動要求
     * @return 処理結果
     */
    LC70_ERR_T systemReboot(void);

    /**
     * @brief シャットダウン要求
     * @return 処理結果
     */
    LC70_ERR_T systemShutdown(void);

    /**
     * @brief スタンバイ要求
     * @param mode 0: MAIN AND SUB 1: SUB ONLY
     * @return 処理結果
     */
    LC70_ERR_T systemStandBy(uint8_t mode = 0);

    /**
     * @brief ウェイクアップ要求
     * @param mode 0: MAIN AND SUB 1: SUB ONLY
     * @return 処理結果
     */
    LC70_ERR_T systemWakeup(uint8_t mode = 0);

    /**
     * @brief ブザー制御
     * @param buzzerPattern ブザー鳴動パターン
     * @param buzzerMode ブザーモード
     * @return 処理結果
     */
    LC70_ERR_T buzzer(LC70_BUZZ_T buzzerPattern, LC70_BUZZ_MODE_T buzzerMode);

    /**
     * @brief 強制ブザー音量制御
     * @param buzzerPattern ブザーパターン
     * @param buzzerVolume ブザー音量
     * @return 処理結果
     */
    LC70_ERR_T fourceBuzzerVolume(LC70_BUZZ_T buzzerPattern, int buzzerVolume);

    /**
     * @brief initDateTime 時刻の初期化
     * @return 処理結果
     */
    void initDateTime();

    /**
     * @brief setDateTime 時刻更新
     * @param yyyy
     * @param mm
     * @param dd
     * @param week
     * @param hour
     * @param minutes
     * @return 処理結果
     */
    lclk_err_t setDateTime(int yyyy, int mm, int dd, int week, int hour, int minutes);

    /**
     * @brief オートパワーオフデータ更新
     * @param offTime 選択中のオートパワーオフデータ
     * @return 処理結果
     */
    LC70_ERR_T setAutoPowerOffTime(int offTime);

    /**
     * @brief シングルチャンネル測定結果の出力
     * @param fileName ファイルのパス
     * @param condition 測定条件
     * @param singleMeasureResult 測定結果
     * @return
     */
    LC70_ERR_T writeSingleMeasurementData(QString fileName, uint16_t condition, const MeasureResult &singleMeasureResult);

    /**
     * @brief マルチチャンネル測定結果の出力
     * @param fileName ファイルのパス
     * @param condition 測定条件
     * @param multiMeasureResult 測定結果
     * @return
     */
    LC70_ERR_T writeMultiMeasurementData(QString fileName, uint16_t condition,
                                         const QList<MeasureResult> &multiMeasureResult);

    /**
     * @brief シングル測定結果画面の出力
     * @param fileName ファイルのパス
     * @param image 画面キャプチャーデータ
     * @return
     */
    LC70_ERR_T writeSingleMeasurementScreen(QString fileName, QImage &image);

    /**
     * @brief 測定データファイル名リストを更新
     * @param fileType ファイルの種類
     * @param measureType 測定の種類
     * @param fileName ファイル名
     * @return リストの更新成否
     * @note 引数は使用していない
     */
    LC70_ERR_T updateMeasurementFileList(SaveFileType fileType = CSV, MeasureType measureType = SINGLE_MEASUREMENT, const QString &fileName = QString(""));

    /**
     * @brief ユーザ設定・データロガー設定ファイル削除
     * @param fileName 削除ファイル名
     * @param mode USER 設定かデータロガー設定か
     * @return
     */
    LC70_ERR_T deleteUserLogSetFile(const QString &fileName, ChannelEditMode mode);

    /**
     * @brief ユーザ設定・データロガー設定ファイル一括削除
     * @param mode USER 設定かデータロガー設定か
     * @return
     */
    LC70_ERR_T allDeleteUserLogSetFile(ChannelEditMode mode);

    /**
     * @brief ユーザ設定・データロガー設定ファイル読み込み
     * @param fileName 読み込みファイル名
     * @param userlogSetList ユーザ設定・データロガー設定のリスト
     * @param mode USER 設定かデータロガー設定か
     * @return
     */
    LC70_ERR_T readUserLogSetFile(const QString &fileName, QList<LC70_LOGGER_SET_LOGGER_ROW_T> &userlogSetList, ChannelEditMode mode);

    /**
     * @brief ユーザ設定・データロガー設定ファイル書き込み
     * @param fileName 書き込みファイル名
     * @param userlogSetList ユーザ設定・データロガー設定のリスト
     * @param mode USER 設定かデータロガー設定か
     * @return
     */
    LC70_ERR_T writeUserLogSetFile(const QString &fileName, QList<LC70_LOGGER_SET_LOGGER_ROW_T> &userlogSetList, ChannelEditMode mode);

    /**
     * @brief ログ出力する
     * @param message ログに出力する文字列
     * @param priority メッセージの優先度
     */
    void writeLog(const QString &message, int priority = LOG_DEBUG);

    /**
     * @brief ユーザ設定・データロガー設定ファイルリスト読み込み
     * 特定の場所から存在しているファイルリストを読み込む
     *
     * @param dirname
     * @param userlogSetFileList ユーザ設定・データロガー設定ファイルのリスト
     * @param mode USER 設定かデータロガー設定か
     * @param path 設定ファイル保存先のパス
     * @param fileName 設定ファイルリストのファイル名
     * @return
     */
    LC70_ERR_T readUserLogSetFileList(const QString &dirname,
                                      QList<LC70_LOGGER_SET_LOGGER_FLIST_ROW_T> &userlogSetFileList, ChannelEditMode mode, const QString &path, const QString &fileName);

    /**
     * @brief ユーザ設定・データロガー設定ファイルリスト書き込み
     * @param dirname
     * @param userlogSetFileList
     * @param mode USER 設定かデータロガー設定か
     * @param fileName 設定ファイルリストのファイル名
     * @return
     */
    LC70_ERR_T writeUserLogSetFileList(const QString &dirname,
                                       QList<LC70_LOGGER_SET_LOGGER_FLIST_ROW_T> &userlogSetFileList, ChannelEditMode mode, const QString &fileName);

    /**
     * @brief 方位測定を開始する
     * @return 処理結果
     * @note 開始後、デバイス状態取得で方位を取得する
     */
    LC70_ERR_T startAzimuthMeasurement(void);

    /**
     * @brief 方位測定を停止する
     * @return 処理結果
     */
    LC70_ERR_T stopAzimuthMeasurement(void);

    /**
     * @brief 閾値の設定を行う
     * @param thresholdData 閾値データリスト
     * @return 処理結果
     */
    LC70_ERR_T setThresholdData(QVariantList thresholdData);

    /**
     * @brief コンスタレーション表示ポイント数の設定を行う
     * @param constellationSelectNum 選択中コンスタレーション番号
     * @return 処理結果
     */
    LC70_ERR_T setConstellationShowPoint(int constellationSelectNum);

    /**
     * @brief ケーブル損失データの設定を行う
     * @param cableLossList ケーブル損失データリスト
     * @return 処理結果
     */
    LC70_ERR_T setCableLossData(QVariantList cableLossList);

    /**
     * @brief 音量の設定を行う
     * @param volumeLevel 音量レベル
     * @return 処理結果
     */
    LC70_ERR_T setSoundVolume(int volumeLevel);

    /**
     * @brief 明るさ設定を行う
     * @param brightnessLevel 明るさレベル
     * @return 処理結果
     */
    LC70_ERR_T setBrightness(int brightnessLevel);

    /**
     * @brief スクリーンの明るさの設定を行う
     * @param brightnessLevel 明るさレベル
     * @return 処理結果
     */
    LC70_ERR_T setScreenBrightness(int brightnessLevel);

    /**
     * @brief 局部発振周波数の設定を行う
     * @param localFrequency 局部発振周波数
     * @return 処理結果
     */
    LC70_ERR_T setLocalFrequency(QVariantList localFrequency);

    /**
     * @brief ケーブル損失有効設定を行う
     * @param isEnableCableLoss ケーブル損失が有効か 1: 有効 0:無効
     * @return 処理結果
     */
    LC70_ERR_T setIsEnableCableLoss(int isEnableCableLoss);

    /**
     * @brief MER/CN の設定を行う
     * @param mercnValue
     * @return
     */
    LC70_ERR_T setMERCN(int mercnValue);

    /**
     * @brief CATVチャンネル周波数データの設定を行う
     * @param settingNo 現在設定中のCATVチャンネル周波数データ
     * @return 処理結果
     */
    LC70_ERR_T setCATVChannelFrequency(int settingNo);

    /**
     * @brief 同軸・光の切替を行う
     * @param rfOptical
     * @return
     */
    LC70_ERR_T setRFOptical(int rfOptical);

    /**
     * @brief 有効なチャンネルのスキャンを開始する
     * @param condition
     * @return 処理結果
     */
    LC70_ERR_T startScan(LC70_MEAS_MULTI_T *condition);

    /**
     * @brief チャンネルのスキャンを停止する
     * @return 処理結果
     */
    LC70_ERR_T stopScan(void);

    /**
     * @brief スキャン結果を取得する
     * @param index
     * @param result
     * @return 処理結果
     */
    LC70_ERR_T scanResult(int16_t index, LC70_MEAS_MULTI_RESULT_T &result);

    /**
     * @brief 光測定を開始する
     * @param ulWaveLength 測定する波長
     * @return 処理結果
     */
    LC70_ERR_T startOpticalMeasurement(uint32_t ulWaveLength);

    /**
     * @brief 光測定を停止する
     * @return 処理結果
     */
    LC70_ERR_T stopOpticalMeasurement(void);

    /**
     * @brief 光測定の測定結果を取得する
     * @param result[out] 測定結果
     * @return 処理結果
     */
    LC70_ERR_T getOpticalMeasurementResult(QVariant &result);

    /**
     * @brief DC測定を開始する
     * @return 処理結果
     */
    LC70_ERR_T startDCMeasurement(void);

    /**
     * @brief DC測定を停止する
     * @return 処理結果
     */
    LC70_ERR_T stopDCMeasurement(void);

    /**
     * @brief DC測定の測定結果を取得する
     * @param result[out] 測定結果
     * @return 処理結果
     */
    LC70_ERR_T getDCMeasurementResult(QVariant &result);

    /**
     * @brief 機種情報を取得する
     * @param result[out] 機種情報
     * @return 処理結果
     */
    LC70_ERR_T getModelInformation(LC70_PRODUCT_INFO_T &result);

    /**
     * @brief 遷移先画面種別の変更
     * @param displayType 遷移先画面種別
     */
    void changeDisplayType(LC70_DISP_TYPE_T displayType);

    /**
     * @brief 給電中か否かを返す
     * @return 給電中か否か
     */
    bool isSupplying(void) const;

    /**
     * @brief プログラムのアップデートを行う
     * @param updater プログラムアップデート情報
     * @return プログラムアップデートの可否
     */
    LC70_ERR_T updateProgram(LC70_PRG_UPDATE_T *updater);

    /**
     * @brief 全設定ファイルを削除する
     * @return 削除の成否
     */
    bool deleteAllSettingFile(void);

    /**
     * @brief バッテリ低下状態変化通知
     * @param isLowBattery バッテリ低下状態が発生しているかどうか true: 発生している false:発生していない
     * @return 通知結果
     */
    LC70_ERR_T notifyLowBatteryStatusChanged(bool isLowBattery);

    /**
     * @brief 測定状態取得
     * @param status 測定状態
     * @return 測定状態取得の成否
     */
    LC70_ERR_T measureStatus(LC70_MEAS_STAT_T *status);

    /**
     * @brief スタンバイ状態取得
     * @param status スタンバイ状態
     * @return スタンバイ状態取得の成否
     */
    LC70_ERR_T standByStatus(LC70_STANDBY_STAT_T *status);

    /**
     * @brief スタンバイ状態取得(ウェイクアップ用)
     * @param status スタンバイ状態
     * @return スタンバイ状態取得の成否
     */
    LC70_ERR_T wakeUpStatus(LC70_STANDBY_STAT_T *status);

    /**
     * @brief 工場出荷状態初期化要求
     * @return 工場出荷状態初期化要求の成否
     */
    LC70_ERR_T initializeFactoryDefault(void);

    /**
     * @brief デベロッパーモード設定要求
     * @param selection 0: OFF 1: ON
     * @return デベロッパーモード設定要求の成否
     */
    LC70_ERR_T setDeveloperMode(const uint8_t &selection);

    /**
     * @brief サブ CPU の強制スタンバイ、ウェイクアップによるリセット
     * @return サブ CPU のリセット成否
     */
    LC70_ERR_T resetSubCpu(void);

    /**
     * @brief マルチ測定で複数 Band を跨って測定する場合向けの給電の設定を行う
     * @param value 0: OFF 1: ON
     * @return 給電の可否
     */
    LC70_ERR_T setPowerSupplyForMultiBand(const uint8_t &value);

    /**
     * @brief 測定データファイルのファイル名とパスを設定
     * @param dataFileListFileName 測定データファイルのファイル名
     * @param dataFileListPath 測定データファイルのパス
     */
    void setDataFileList(const QString &dataFileListFileName, const QString &dataFileListPath);

    /**
     * @brief 測定データファイルのパスを設定
     * @param singleMeasurementDataPath シングル測定の測定データファイルのパス
     * @param multiMeasurementDataPath マルチ測定の測定データファイルのパス
     * @param userMeasurementDataPath ユーザー設定の測定データファイルのパス
     * @param loggerMeasurementDataPath データロガーの測定データファイルのパス
     */
    void setMeasurementDataPath(const QString &singleMeasurementDataPath, const QString &multiMeasurementDataPath, const QString &userMeasurementDataPath, const QString &loggerMeasurementDataPath);

signals:

private:
    ActionMode _mode;                           //!< 動作モード(0: ラッパー関数呼び出し 1: スタブ)
    LC70Dummy *_dummy;                          //!< スタブ(Stub クラスを使用するため使用しない)
    SystemLog &_systemLog;                      //!< システムロガー
    LC70_DISP_TYPE_T _displayType;              //!< 遷移先画面種別
    bool _supplying;                            //!< 給電中
    QString _dataFileListFileName;              //!< 測定データファイルのリスト
    QString _dataFileListPath;                  //!< 測定データファイルのリストのパス
    QString _singleMeasurementDataPath;         //!< シングル測定の測定データファイルのパス
    QString _multiMeasurementDataPath;          //!< マルチ測定の測定データファイルのパス
    QString _userMeasurementDataPath;           //!< USER 設定の測定データファイルのパス
    QString _loggerMeasurementDataPath;         //!< データロガーの測定データファイルのパス

    /**
     * @brief エラーコードに変換する
     * @param result 設定ファイルの読み書きの戻り値
     * @return エラーコード
     * @note LC70_ERR_T の定義に依存する
     */
    LC70_ERR_T converErrorCode(const int &result);

    /**
     * @brief 遷移先画面種別に対応する画面名を返す
     * @param displayType 遷移先画面種別
     * @return 遷移先画面種別に対応する画面名
     */
    QString convertDisplayType(const LC70_DISP_TYPE_T &displayType);

    /**
     * @brief 測定データファイル名のリストを抽出する
     * @param directoryPath ディレクトリのパス
     * @param category ファイル種別を示す文字列
     * @param subDirectoryList 検索対象になるサブディレクトリのリスト
     * @param resultList 測定データファイル名のリスト
     */
    void retrieveMeasurementDataFile(const QString &directoryPath, const QString &category, const QStringList &subDirectoryList, QList<QVariantList> &resultList);

    /**
     * @brief 測定データ(Logger)のファイル名リストを抽出する
     * @param directory Pathディレクトリのパス
     * @param categoryファイル 種別を示す文字列
     * @param fileType ファイルの種類を示す文字列
     * @param resultList 測定データファイル名のリスト
     */
    void retrieveLoggerDataFile(const QString &directoryPath, const QString &category, const QString &fileType, QList<QVariantList> &resultList);

    /**
     * @brief CSVデータ生成
     * @param dataList 出力データリスト
     * @return CSVデータ
     */
    QString generateCsvData(const QVariantList &dataList);
};

#endif // COMMUNICATOR_H
