/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include <QDir>
#include <QFile>
#include <QProcess>
#include <QMap>
#include "levelcheckermodel.h"
#include "modeloperator.h"
#include "correcter.h"
#include "delayprofilelineseriesmodel.h"
#include "measureresultmodel.h"
#include "multigraphmodel.h"
#include "scanresultgroupmodel.h"
#include "scanresultmodel.h"
#include "scanlockmodel.h"
#include "bandmodel.h"
#include "displayfilesystemmodel.h"

#include <QDebug>

#define R2_10

/**
 * @brief アプリケーション固有の設定、定義ファイルの名称
 */
static const QString INI_FILE_NAME = "application_setting.ini";
static const QString MESSAGE_FILE_NAME = "messages.txt";
static const QString ERROR_MESSAGE_FILE_NAME = "errormessages.txt";
static const QString USER_DATALOGGER_SETTING_TEMPLATE = "user_datalogger_setting_template.csv";

static constexpr const char *CSV_FILE_EXTENSION = ".csv";           //!< 設定ファイルの拡張子(CSV)
static constexpr const char *BACKUP_FILE_EXTENSION = ".bak";        //!< バックアップファイルの拡張子
static constexpr const int CATV_SETTING_2 = 2;                      //!< CATV チャンネル周波数の設定 2

static constexpr const char *TEMPORARY_PATH = "/tmp";                               //!< テンポラリのパス
static constexpr const char *COPY_CANDIDATE_FILENAME = "copy_candidate.txt";        //!< データ管理画面で選択されたコピー候補のファイルパスが記録されたファイル名
static constexpr const char *COPY_DIRECTORY_CANDIDATE_FILENAME = "copy_directory_candidate.txt";   //!< データ管理画面で選択されたコピー候補のディレクトリまでのパスが記録されたファイル名
static constexpr const char *REMOVE_CANDIDATE_FILENAME = "remove_candidate.txt";    //!< データ管理画面で選択された削除候補のファイルパスが記録されたファイル名


/**
 * @brief USER データロガー設定の設定ファイルの Band ごとの先頭行のインデックス
 */
static const QMap<BandTabIndex, int> FIRST_INDEX_FOR_EACH_BAND = {
    { BAND_TAB_UHF,              0 },
    { BAND_TAB_BS_R,            40 },
    { BAND_TAB_BS_L,            52 },
    { BAND_TAB_CS_R,            64 },
    { BAND_TAB_CS_L,            77 },
    { BAND_TAB_CATV_ALL,        90 },
    { BAND_TAB_SKY_PERFECTV,   240 },
    { BAND_TAB_CATV_UPSTREAM,  353 },
    { BAND_TAB_JCSAT3A_H,      365 },
    { BAND_TAB_JCSAT3A_V,      379 },
    { BAND_TAB_JCSAT4B_H,      393 },
    { BAND_TAB_JCSAT4B_V,      409 },
    { BAND_TAB_SCCB2_H,        425 },
    { BAND_TAB_SCCB2_V,        436 },
    { BAND_TAB_SCCC2_H,        448 },
    { BAND_TAB_SCCC2_V,        460 },
    { BAND_TAB_LTE,            472 },
    { BAND_TAB_FM,             478 },
    { BAND_TAB_PILOT,          490 },
    { BAND_TAB_COUNT,          510 }
};

/**
 * @brief ログファイル名
 */
static const QString SYSTEM_LOG_FILE_NAME = "pdisp_log";

/**
 * @brief USER データロガー設定ファイル
 */
static constexpr int LIST_COLUMN = 7;               // 衛星放送以外の列数
static constexpr int FIRST_DATA_ROW = 2;            // データの先頭行(それ以前の行はヘッダー)

LevelCheckerModel::LevelCheckerModel(QQmlApplicationEngine *engine, QObject *parent)
    : QObject(parent)
    , _engine(engine)
    , _communicator(nullptr)
    , _stub(nullptr)
    , _applicationSettings(new ApplicationSetting(QCoreApplication::applicationDirPath() + "/" + INI_FILE_NAME, this))
    , _channelSettings(nullptr)
    , _systemWatcher(nullptr)
    , _measurer(nullptr)
    , _faultDetector(nullptr)
    , _modelOperator(nullptr)
    , _acquisitionTarget(SINGLE)
    , _actionMode(_applicationSettings->testMode())
    , _systemLog(SystemLog::instance(this))
    , _scanResult()
    , _isChangingMultiMeasureStyle(false)
    , _versionUpReady(false)
    , _updater()
    , _updating(false)
    , _standBy(false)
    , _wakeUpRequested(false)
    , _autoPowerOffTimer(new QTimer(this))
    , _standByMode(0)
    , _wakeUpCounter(0)
    , _scannedChannelList()
    , _scannedPolatization()
    , _elapsedTimer()
    , _displayFileSystemModel(nullptr)
    , _userDataLoggerSettingFileName()
    , _userDataLoggerSettingList()
    , _userDataLoggerEditingChannels()
    , _userDataLoggerMeasuringChannels()
    , _channelEditMode(CHANNEL_EDIT_USER)
    , _bandTabIndex(BAND_TAB_UHF)
{
    qRegisterMetaType<uint32_t>("uint32_t");
    qRegisterMetaType<uint32_t>("uint16_t");
    qRegisterMetaType<uint32_t>("float32_t");
    qRegisterMetaType<QList<QVariant> >("QList<QVariant>");
    qRegisterMetaType<LC70_MEAS_SINGLE_T>("LC70_MEAS_SINGLE_T");
    qRegisterMetaType<LC70_MEAS_MULTI_RESULT_T>("LC70_MEAS_MULTI_RESULT_T");
    qRegisterMetaType<LC70_CONSTA_NUM_T>("LC70_CONSTA_NUM_T");
    qRegisterMetaType<MeasureResult>("MeasureResult");
    qRegisterMetaType<UserSettingChannelInformation>("UserSettingChannelInformation");
    qRegisterMetaType<SingleMeasureUserSetting>("SingleMeasureUserSetting");
    qRegisterMetaType<LC70_MEAS_MULTI_T>("LC70_MEAS_MULTI_T");
    qRegisterMetaType<MultiResult>("MultiResult");
    qRegisterMetaType<QVector<QPointF> >("QVector<QPointF>");
    qRegisterMetaType<MeasureType>("MeasureType");
    qRegisterMetaType<AcquisitionTarget>("AcquisitionTarget");
    qRegisterMetaType<LC70_DISP_TYPE_T>("LC70_DISP_TYPE_T");
    qRegisterMetaType<LC70_FAIL_INFO_T>("LC70_FAIL_INFO_T");
    qRegisterMetaType<LC70_DEV_STAT_T>("LC70_DEV_STAT_T");
    qRegisterMetaType<LC70_KEY_STAT_T>("LC70_KEY_STAT_T");
    qRegisterMetaType<LC70_PRG_UPDATE_T>("LC70_PRG_UPDATE_T");
    qRegisterMetaType<BandTab>("BandTab");
    qmlRegisterType<DelayProfileLineSeriesModel>("jp.co.dxantenna.levelchecker.delayProfileLineSeriesModel", 1, 0,
            "DelayProfileLineSeriesModel");
    qmlRegisterType<MeasureResultModel>("jp.co.dxantenna.levelchecker.measureResultModel", 1, 0, "MeasureResultModel");
    qmlRegisterType<MultiGraphModel>("jp.co.dxantenna.levelchecker.multiGraphModel", 1, 0, "MultiGraphModel");
    qmlRegisterType<ScanResultGroupModel>("jp.co.dxantenna.levelchecker.scanResultGroupModel", 1, 0, "ScanResultGroupModel");
    qmlRegisterType<ScanResultModel>("jp.co.dxantenna.levelchecker.scanResultModel", 1, 0, "ScanResultModel");
    qmlRegisterType<ScanLockModel>("jp.co.dxantenna.levelchecker.scanLockModel", 1, 0, "ScanLockModel");
    qmlRegisterType<BandModel>("jp.co.dxantenna.levelchecker.BandModel", 1, 0, "BandModel");
    qmlRegisterType<DisplayFileSystemModel>("jp.co.dxantenna.levelchecker.DisplayFileSystemModel", 1, 0, "DisplayFileSystemModel");

    if (_applicationSettings == nullptr) {
        return;
    }

    // USBオペレーションにパスを設定する
    UsbOperation::setSingleMeasurementPath(_applicationSettings->resultDataPathSingle());
    UsbOperation::setMultiMeasurementPath(_applicationSettings->resultDataPathMulti());
    UsbOperation::setLoggerMeasurementPath(_applicationSettings->resultDataPathLogger());
    UsbOperation::setSingleMeasurementUsbPath(_applicationSettings->resultDataCopyPathSingle());
    UsbOperation::setMultiMeasurementUsbPath(_applicationSettings->resultDataCopyPathMulti());
    UsbOperation::setLoggerMeasurementUsbPath(_applicationSettings->resultDataCopyPathLogger());
    UsbOperation::setMeasureDataFileListPath(_applicationSettings->measureDataFileListPath());
    UsbOperation::setMeasureDataFileListFileName(_applicationSettings->measureDataFileListFileName());

    // メニューリストの初期化
    _menuList.clear();

    for (int i = 0; i < MenuListIconIndex::MAX_NUM; ++i) {
        _menuList.append("");
    }

    _systemLog.prepare(_actionMode, SYSTEM_LOG_FILE_NAME);
}

LevelCheckerModel::~LevelCheckerModel()
{
    delete _measurer;
    delete _systemWatcher;
    delete _faultDetector;
}

LC70_ERR_T LevelCheckerModel::prepare(QElapsedTimer *timer)
{
    // サブ CPU との通信を行うクラスのインスタンスを生成
    if (_communicator == nullptr) {
        _communicator = new Communicator(_actionMode, _systemLog, this);

        if (_communicator == nullptr) {
            _systemLog.write(QString(typeid(*this).name()), QString("LevelChecker is not ready. Communicator is not created."), LOG_INFO);
            return LC70_ERR;
        }
    }

    // スレッドごとにラッパー関数ライブラリを初期化
    LC70_ERR_T ret = _communicator->initialize(PROC_ID_PDISPMNG);

    if (ret != LC70_OK) {
        _systemLog.write(QString(typeid(*this).name()), QString("LevelChecker is not ready. Confirm lc70_init() function."), LOG_INFO);
        return ret;
    }

    // 測定データファイルリストのファイル名とパスを設定
    _communicator->setDataFileList(_applicationSettings->measureDataFileListFileName(), _applicationSettings->measureDataFileListPath());
    // 測定データファイルのパスを設定
    _communicator->setMeasurementDataPath(_applicationSettings->resultDataPathSingle(), _applicationSettings->resultDataPathMulti(), _applicationSettings->resultDataPathList(), _applicationSettings->resultDataPathLogger());

    writeLog(QString(typeid(*this).name()),
             QString("[START UP]main communication established %1 ms").arg(timer->elapsed()));

    // 製品版では Stub クラスを生成しない
    if (_actionMode == ActionMode::DEVELOPMENT) {

        // スタブの初期化
        if (_stub == nullptr) {
            _stub = new Stub(this);

            if (_stub == nullptr) {
                _systemLog.write(QString(typeid(*this).name()), QString("LevelChecker is not ready. Stub is not created."), LOG_INFO);
                return LC70_ERR;
            }
        }

        // 実行ファイル固有の設定がない場合はエラーにする
        if (_applicationSettings->stubDataPathMultiMeasurement().isEmpty() ||
                _applicationSettings->stubDataPathSingleMeasurement().isEmpty()) {
            _systemLog.write(QString(typeid(*this).name()),
                             QString("LevelChecker is not ready. Confirm application_setting.ini(StubDataPath) file."), LOG_INFO);
            return LC70_ERR;
        }

        bool ret = _stub->initialize(_applicationSettings->stubDataPathSingleMeasurement(),
                                     _applicationSettings->stubDataPathMultiMeasurement(),
                                     _applicationSettings->channelDefinitionData());

        if (!ret) {
            _systemLog.write(QString(typeid(*this).name()), QString("LevelChecker is not ready. Confirm stub data file."), LOG_INFO);
            return LC70_ERR;
        }
    }

    // 設定値操作クラスの生成
    if (_modelOperator == nullptr) {
        _modelOperator = new ModelOperator(_communicator, _stub, _applicationSettings);

        if (_modelOperator == nullptr) {
            _systemLog.write(QString(typeid(*this).name()), QString("LevelChecker is not ready. ModelOperator is not created."), LOG_INFO);
            return LC70_ERR;
        }
    }

    {
        QString fullpathDirname;

        if (_applicationSettings->testMode() != ActionMode::PRODUCTION) {
            fullpathDirname = _applicationSettings->stubDataPathDataLogger() + "/";
        }

        fullpathDirname += _applicationSettings->settingDataPathStubLogger();
        _modelOperator->setDataLoggerUserSettingDir(fullpathDirname);
    }

    _modelOperator->initialize(_actionMode);
    writeLog(QString(typeid(*this).name()), QString("[START UP]setting value initialized %1 ms").arg(timer->elapsed()));

    // Correcterに対して BAND毎のMER/CNの上限下限値リストを設定する
    QVariant merCnRangeList;
    getValue(VALUE_MERCNR_LIMITS, merCnRangeList);
    Correcter::setMerCnLimitList(merCnRangeList.toList());

    // 故障状態取得クラスのインスタンスを生成
    if (_faultDetector == nullptr) {
        _faultDetector = new FaultDetector(_actionMode, _communicator, _stub, _systemLog);

        if (_faultDetector == nullptr) {
            _systemLog.write(QString(typeid(*this).name()),
                             QString("LevelChecker is not ready. FaultDetector is not created."), LOG_INFO);
            return LC70_ERR;
        }
    }

    // 実行ファイル固有の設定がない場合はエラーにする
    if (_applicationSettings->pollingIntervalFailureDetection() <= 0) {
        _systemLog.write(QString(typeid(*this).name()),
                         QString("LevelChecker is not ready. Confirm application_setting.ini(PolloingInterval - FailureDetection) file."), LOG_INFO);
        return LC70_ERR;
    }

    _faultDetector->setInterval(_applicationSettings->pollingIntervalFailureDetection());
    _faultDetector->attach(new ThreadBase(_applicationSettings->pollingIntervalFailureDetection()),
                           QString(FAULT_DETECTOR_THREAD_NAME));
    connect(_faultDetector, SIGNAL(errorOccured(LC70_FAIL_INFO_T)), this, SLOT(onErrorOccured(LC70_FAIL_INFO_T)),
            Qt::QueuedConnection);

    // キー入力・デバイス状態監視クラスのインスタンスを生成
    if (_systemWatcher == nullptr) {
        _systemWatcher = new SystemWatcher(_actionMode, _communicator, _stub, _systemLog);

        if (_systemWatcher == nullptr) {
            _systemLog.write(QString(typeid(*this).name()), QString("LevelChecker is not ready. SystemWatcher is not created."), LOG_INFO);
            return LC70_ERR;
        }
    }

    // 実行ファイル固有の設定がない場合はエラーにする
    if (_applicationSettings->pollingIntervalKeyInput() <= 0 ||
            _applicationSettings->pollingIntervalDeviceStatus() <= 0) {
        _systemLog.write(QString(typeid(*this).name()),
                         QString("LevelChecker is not ready. Confirm application_setting.ini(PolloingInterval - KeyInput or DeviceStatus) file."), LOG_INFO);
        return LC70_ERR;
    }

    _systemWatcher->setInterval(_applicationSettings->pollingIntervalKeyInput(),
                                _applicationSettings->pollingIntervalDeviceStatus());
    _systemWatcher->attach(new ThreadBase(_applicationSettings->pollingIntervalKeyInput()),
                           QString(SYSTEM_WATCHER_THREAD_NAME));
    connect(_systemWatcher, SIGNAL(keyStatus(LC70_KEY_STAT_T)), this, SLOT(onKeyStatus(LC70_KEY_STAT_T)),
            Qt::QueuedConnection);
    connect(_systemWatcher, SIGNAL(deviceStatus(LC70_DEV_STAT_T)), this, SIGNAL(deviceStatus(LC70_DEV_STAT_T)),
            Qt::QueuedConnection);
    connect(_systemWatcher, SIGNAL(azimuth(uint16_t)), this, SIGNAL(azimuth(uint16_t)), Qt::QueuedConnection);
    connect(_systemWatcher, SIGNAL(notifyWakeUp()), this, SLOT(onNotifyWakeUp()), Qt::QueuedConnection);

    writeLog(QString(typeid(*this).name()), QString("[START UP]system watcher initialized %1 ms").arg(timer->elapsed()));
    // ウェイクアップカウンターを再設定
    _wakeUpCounter = _systemWatcher->wakeUpCounter();

    // ウェイクアップ状態に移行する
    writeLog(QString(typeid(*this).name()),
             QString("[LAUNCH] counter: %1").arg(_wakeUpCounter), LOG_NOTICE);

    // 測定結果取得クラスの生成
    if (_measurer == nullptr) {
        _measurer = new Measurer(_actionMode, _communicator, _stub,
                                 _applicationSettings->constellationTermOfUseNumberOfPoints(), _systemLog);

        if (_measurer == nullptr) {
            _systemLog.write(QString(typeid(*this).name()), QString("LevelChecker is not ready. Measurer is not created."), LOG_INFO);
            return LC70_ERR;
        }
    }

    // 実行ファイル固有の設定がない場合はエラーにする
    if (_applicationSettings->pollingIntervalSingleMeasurement() <= 0
            || _applicationSettings->pollingIntervalMultiMeasurement() <= 0) {
        _systemLog.write(QString(typeid(*this).name()), QString(
                             "LevelChecker is not ready. Confirm application_setting.ini"
                             "(PolloingInterval - SingleMeasurement or MultiMeasurement) file."), LOG_INFO);
        return LC70_ERR;
    }

    _measurer->setInterval(_applicationSettings->pollingIntervalSingleMeasurement(),
                           _applicationSettings->pollingIntervalDelayProfile());
    _measurer->attach(new ThreadBase(_applicationSettings->pollingIntervalSingleMeasurement()),
                      QString(MEASURER_THREAD_NAME));

    // 実行ファイル固有の設定がない場合はエラーにする
    QString applicationSettingError;

    if (!_applicationSettings->isValidSetting(applicationSettingError)) {
        _systemLog.write(QString(typeid(*this).name()),
                         QString("LevelChecker is not ready. Confirm application_setting.ini(%1) file.").arg(applicationSettingError), LOG_INFO);
        return LC70_ERR;
    }

    connect(this, SIGNAL(resumeSingleMeasuring(LC70_CONSTA_NUM_T, LC70_MEAS_SINGLE_T, uint32_t)),
            _measurer, SLOT(onResumeSingleMeasuring(LC70_CONSTA_NUM_T, LC70_MEAS_SINGLE_T, uint32_t)),
            Qt::QueuedConnection);
    connect(_measurer, SIGNAL(singleMeasureResult(MeasureResult, QList<QVariant>)), this,
            SIGNAL(singleMeasureResult(MeasureResult, QList<QVariant>)), Qt::QueuedConnection);
    connect(this, SIGNAL(singleMeasuringChangeCondition(LC70_MEAS_SINGLE_T)), _measurer,
            SLOT(onSingleMeasuringChangeCondition(LC70_MEAS_SINGLE_T)),
            Qt::QueuedConnection);
    connect(this, SIGNAL(preferredMeasuringChangeCondition(LC70_MEAS_SINGLE_T *)), _measurer,
            SLOT(onPreferredMeasuringChangeCondition(LC70_MEAS_SINGLE_T *)),
            Qt::QueuedConnection);
    connect(_measurer, SIGNAL(delayProfileResult(QVector<QPointF>)), this, SIGNAL(delayProfileResult(QVector<QPointF>)),
            Qt::QueuedConnection);

    connect(this, SIGNAL(resumeMultiMeasuring(LC70_MEAS_MULTI_T *, LC70_MEAS_SINGLE_T *, LC70_DISP_TYPE_T, uint32_t)),
            _measurer, SLOT(onResumeMultiMeasuring(LC70_MEAS_MULTI_T *, LC70_MEAS_SINGLE_T *, LC70_DISP_TYPE_T, uint32_t)),
            Qt::QueuedConnection);
    connect(_measurer, SIGNAL(multiMeasureResult(MultiResult, MeasureResult)), this, SIGNAL(multiMeasureResult(MultiResult,
            MeasureResult)),
            Qt::QueuedConnection);

    connect(_measurer, SIGNAL(notifyChannelChanging(bool)), this, SIGNAL(notifyChannelChanging(bool)),
            Qt::QueuedConnection);
    connect(_measurer, SIGNAL(scanningFinished(bool)), this, SIGNAL(scanningFinished(bool)), Qt::QueuedConnection);
    connect(_measurer, SIGNAL(notifyDelayProfileValid(bool)), this, SIGNAL(notifyDelayProfileValid(bool)));

    // チャンネルスキャンのためのConnect
    connect(this, SIGNAL(resumeScanning(LC70_MEAS_MULTI_T *, AcquisitionTarget, int)),
            _measurer, SLOT(onResumeScanningChannel(LC70_MEAS_MULTI_T *, AcquisitionTarget, int)),
            Qt::QueuedConnection);

    connect(this, SIGNAL(suspendScanning()), _measurer, SLOT(onSuspendScanningChannel()),
            Qt::QueuedConnection);

    connect(_measurer, SIGNAL(scanningChannelResult(int, LC70_MEAS_MULTI_RESULT_T)),
            this, SIGNAL(scanningChannelResult(int, LC70_MEAS_MULTI_RESULT_T)), Qt::QueuedConnection);

    // アンテナ調整
    connect(this, SIGNAL(resumeAntennaMeasuring(LC70_MEAS_MULTI_T *, uint32_t)), _measurer,
            SLOT(onResumeAntennaMeasuring(LC70_MEAS_MULTI_T *, uint32_t)), Qt::QueuedConnection);
    connect(_measurer, SIGNAL(antennaMeasureResult(MultiResult)), this, SIGNAL(antennaMeasureResult(MultiResult)),
            Qt::QueuedConnection);
    connect(this, SIGNAL(antennaMeasuringChangeCondition(LC70_MEAS_MULTI_T *)), _measurer,
            SLOT(onAntennaMeasuringChangeCondition(LC70_MEAS_MULTI_T *)),
            Qt::QueuedConnection);

    // 測定停止依頼
    connect(this, SIGNAL(requestToSuspendSingleMeasuring()), _measurer, SLOT(onSuspendSingleMeasuring()), Qt::QueuedConnection);
    connect(this, SIGNAL(requestToSuspendMultiMeasuring()), _measurer, SLOT(onSuspendMultiMeasuring()), Qt::QueuedConnection);
    connect(this, SIGNAL(requestToSuspendAntennaMeasuring()), _measurer, SLOT(onSuspendAntennaMeasuring()), Qt::QueuedConnection);

    // 測定停止通知
    connect(_measurer, SIGNAL(notifyStopMeasuring(AcquisitionTarget)), this, SLOT(onNotifyStopMeasuring(AcquisitionTarget)), Qt::QueuedConnection);

    // デバイス状態
    connect(_systemWatcher, SIGNAL(changeUsbInsertionRemoval(int)), this, SIGNAL(changeUsbInsertionRemoval(int)), Qt::DirectConnection);

    // 自動パワーオフ
    connect(_autoPowerOffTimer, SIGNAL(timeout()), this, SLOT(onAutoPowerOffTimeOut()));

    // スタンバイ
    connect(this, SIGNAL(requestToStandBy(uint8_t)), _measurer, SLOT(onRequestToStandBy(uint8_t)), Qt::DirectConnection);

    // ソフトウェアアップデート
    connect(this, SIGNAL(notifySoftwareUpdate(LC70_PRG_UPDATE_T)), _measurer, SLOT(onNotifySoftwareUpdate(LC70_PRG_UPDATE_T)), Qt::QueuedConnection);
    connect(_measurer, SIGNAL(notifySoftwareUpdateCompleted(bool)), this, SLOT(onNotifySoftwareUpdateCompleted(bool)), Qt::QueuedConnection);

    // 自動パワーオフのタイマー設定
    changePowerOffSetting();

    writeLog(QString(typeid(*this).name()),
             QString("[START UP]measure result receiver initialized %1 ms").arg(timer->elapsed()));


    if (_actionMode == PRODUCTION) {
        _systemLog.write(QString(typeid(*this).name()), QString("LevelCheckerModel prepared. ActionMode: PRODUCTION"));
    } else {
        _systemLog.write(QString(typeid(*this).name()), QString("LevelCheckerModel prepared. ActionMode: DEVELOPMENT"));
    }

    return LC70_OK;
}

QObject *LevelCheckerModel::search(const QString &objectName)
{
    if (_engine == nullptr) {
        return nullptr;
    }

    return ObjectSearcher::search(_engine, objectName);
}

ApplicationSetting *LevelCheckerModel::applicationSetting() const
{
    return _applicationSettings;
}

void LevelCheckerModel::changeAcquisitionTarget(const AcquisitionTarget &acquisitionTarget)
{
    _acquisitionTarget = acquisitionTarget;
    _measurer->changeAcquisitionTarget(acquisitionTarget);

    if (_acquisitionTarget == SINGLE) {
        writeLog(QString("シングル測定"), QString("シングル測定中"));
    } else if (_acquisitionTarget == CONSTELLATION) {
        writeLog(QString("シングル測定"), QString("コンスタレーション測定中"));
    } else if (_acquisitionTarget == DELAY_PROFILE) {
        writeLog(QString("シングル測定"), QString("遅延プロファイル測定中"));
    } else if (_acquisitionTarget == MULTI) {
        writeLog(QString("マルチ測定"), QString("マルチ測定中"));
    } else if (_acquisitionTarget == CHANNEL_SCAN) {
        writeLog(QString("スキャン"), QString("スキャン中"));
    } else if (_acquisitionTarget == ANTENNA) {
        writeLog(QString("アンテナ調整"), QString("アンテナ調整中"));
    }

    // 開発時はスタブにも測定結果取得対象を設定
    if (_applicationSettings->testMode() == DEVELOPMENT) {
        _stub->changeAcquisitionTarget(acquisitionTarget);
    }
}

AcquisitionTarget LevelCheckerModel::acquisitionTarget() const
{
    return _acquisitionTarget;
}

QString LevelCheckerModel::createDirectory(SaveFileType saveFileType, MeasureType measureType)
{
    QString path;

    if (measureType == SINGLE_MEASUREMENT) {
        path = _applicationSettings->resultDataPathSingle();
    } else if (measureType == MULTI_MEASUREMENT) {
        path = _applicationSettings->resultDataPathMulti();
    } else {
        return QString("");
    }

    QDir directory(path);

    if (!directory.exists()) {
        if (!directory.mkpath(".")) {
            return QString();
        }
    }

    // 日付のディレクトリ作成
    path = directory.path() + QString("/") + QDateTime::currentDateTime().toString("yyyyMMdd");
    directory.setPath(path);

    if (!directory.exists()) {
        if (!directory.mkpath(".")) {
            return QString();
        }
    }

    // 画像、測定データのディレクトリ作成
    if (saveFileType == SaveFileType::CSV) {
        path += QString("/CSV");
    } else if (saveFileType == SaveFileType::PNG) {
        path += QString("/PNG");
    }

    directory.setPath(path);

    if (!directory.exists()) {
        if (!directory.mkpath(".")) {
            return QString();
        }
    }

    return path + "/";
}

bool LevelCheckerModel::writeMultiMeasurementData(QString fileName, uint16_t condition,
        const QList<MeasureResult> &multiMeasureResult)
{
    LC70_ERR_T result;

    // 製品版、開発版で共通
    result = _communicator->writeMultiMeasurementData(fileName, condition, multiMeasureResult);

    return (result == LC70_OK) ? true : false;
}

bool LevelCheckerModel::writeSingleMeasurementData(QString fileName, uint16_t condition,
        const MeasureResult &singleMeasureResult)
{
    LC70_ERR_T result;

    // 製品版、開発版で共通
    result = _communicator->writeSingleMeasurementData(fileName, condition, singleMeasureResult);

    return (result == LC70_OK) ? true : false;
}

bool LevelCheckerModel::writeSingleMeasurementScreen(QString fileName, QImage &image)
{
    LC70_ERR_T result;

    // 製品版、開発版で共通(Stub クラスは使わない)
    result = _communicator->writeSingleMeasurementScreen(fileName, image);

    return (result == LC70_OK) ? true : false;
}

bool LevelCheckerModel::updateMeasurementFileList(void)
{
    LC70_ERR_T result;

    // 製品版、開発版で共通(Stub クラスは使わない)
    result = _communicator->updateMeasurementFileList();

    return (result == LC70_OK) ? true : false;
}

void LevelCheckerModel::removeMenuList(const MenuListIconIndex index)
{
    if (index >= MenuListIconIndex::MAX_NUM) {
        return;
    }

    _menuList.replace(index, "");
    emit updateMenuList(&_menuList);
}

void LevelCheckerModel::replaceMenuList(const MenuListIconIndex index, const QString &iconName)
{
    if (index >= MenuListIconIndex::MAX_NUM) {
        return;
    }

    _menuList.replace(index, iconName);

    emit updateMenuList(&_menuList);
}

void LevelCheckerModel::replaceBandIcon(const LC70_BAND_T band)
{
    // BANDアイコンの置き換えを実施する
    switch (band) {
    case LC70_BAND_UHF: // 地デジ
        replaceMenuList(MenuListIconIndex::BAND_ICON, "icon_band_chidigi.png");
        break;
    case LC70_BAND_BS:  // BS
        replaceMenuList(MenuListIconIndex::BAND_ICON, "icon_band_bs.png");
        break;
    case LC70_BAND_CS:  // CS
        replaceMenuList(MenuListIconIndex::BAND_ICON, "icon_band_cs.png");
        break;
    case LC70_BAND_BS_CS:   // BS CS統合
        replaceMenuList(MenuListIconIndex::BAND_ICON, "icon_band_bs_cs.png");
        break;
    case LC70_BAND_CATV_ALL:
        replaceMenuList(MenuListIconIndex::BAND_ICON, "icon_band_catv_all.png");
        break;
    case LC70_BAND_CATV_90_470MHZ:
        replaceMenuList(MenuListIconIndex::BAND_ICON, "icon_band_catv_90-470.png");
        break;
    case LC70_BAND_CATV_470_1002MHZ:
        replaceMenuList(MenuListIconIndex::BAND_ICON, "icon_band_catv_470-1000.png");
        break;
    case LC70_BAND_SKYP:
        replaceMenuList(MenuListIconIndex::BAND_ICON, "icon_band_catv_sky.png");
        break;
    case LC70_BAND_CATV_UP_STREAM:
        replaceMenuList(MenuListIconIndex::BAND_ICON, "icon_band_catv_up_stream.png");
        break;
    case LC70_BAND_JCSAT3:  // JCSAT3
        replaceMenuList(MenuListIconIndex::BAND_ICON, "icon_band_3a.png");
        break;
    case LC70_BAND_JCSAT4:  // JCSAT4
        replaceMenuList(MenuListIconIndex::BAND_ICON, "icon_band_4b.png");
        break;
    case LC70_BAND_SCCB:    // SCC-B
        replaceMenuList(MenuListIconIndex::BAND_ICON, "icon_band_b.png");
        break;
    case LC70_BAND_SCCC:    // SCC-C
        replaceMenuList(MenuListIconIndex::BAND_ICON, "icon_band_c.png");
        break;
    case LC70_BAND_FM:  // FM
        replaceMenuList(MenuListIconIndex::BAND_ICON, "icon_band_fm.png");
        break;
    case LC70_BAND_LTE: // LTE
        replaceMenuList(MenuListIconIndex::BAND_ICON, "icon_band_lte.png");
        break;
    case LC70_BAND_PILOT:   // PILOT
        replaceMenuList(MenuListIconIndex::BAND_ICON, "icon_band_catv_pilot.png");
        break;
    case LC70_BAND_JCSAT3 * LC70_BAND_JCSAT4: // JCSATアンテナ調整
        replaceMenuList(MenuListIconIndex::BAND_ICON, "icon_band_antenna ad .png");
        break;
    default:
        break;
    }
}

void LevelCheckerModel::clearMenuList()
{
    _menuList.clear();

    for (int i = 0; i < MenuListIconIndex::MAX_NUM; ++i) {
        _menuList.append("");
    }
}

bool LevelCheckerModel::getValue(ValueId valueId, QVariant &value)
{
    return _modelOperator->getValue(valueId, value);
}

bool LevelCheckerModel::setValue(ValueId valueId, const QVariant &value)
{
    return _modelOperator->setValue(valueId, value);
}

bool LevelCheckerModel::setInitialValue(ValueId valueId, const QVariant &value)
{
    return _modelOperator->setInitialValue(valueId, value);
}

bool LevelCheckerModel::isUpdate(ValueId valueId)
{
    return _modelOperator->isUpdate(valueId);
}

bool LevelCheckerModel::rollback(ValueId valueId)
{
    return _modelOperator->rollback(valueId);
}

bool LevelCheckerModel::commit(ValueId valueId)
{
    bool ret = _modelOperator->commit(valueId);

    // 自動パワーオフの設定時間を更新
    if (ret && valueId == VALUE_AUTOPOWEROFF) {
        changePowerOffSetting();
    }

    return ret;
}

bool LevelCheckerModel::executeInitializeSetting()
{
#ifdef R2_10
    bool ret = false;
    if (_communicator->deleteAllSettingFile()) {
        ret = _communicator->systemShutdown();
    }

    return ret;
#else
    // ユーザ設定ファイル/データロガー設定ファイルの削除
    // ファイル名取得
    // ファイル削除
    int ret;
    bool result = true;
    LC70_USERLOG_SET_FLIST_T value;
    ret = _communicator->readSystemSettingFile(LC70_FILE_ID_USER_SET_FLIST, &value);

    if (ret != LC70_OK) {
        // ユーザ測定/データロガー設定ファイルリストの取得処理にてエラーが発生した場合
        qDebug() << "get user Measure, data Logger File get Error : " << ret;
        result = false;
    }

    // 設定値の初期化
    // 設定ファイルに対して初期化を実施する。
    _modelOperator->initialize(_applicationSettings->testMode(), true);
    _modelOperator->initializeSettingFile();

    // SDカード内のデータを削除する
    QStringList deleteFilePathList = {
        applicationSetting()->resultDataPathSingle(),    // 測定結果データ(シングル)
        applicationSetting()->resultDataPathMulti(),     // 測定結果データ(マルチ)
        applicationSetting()->settingDataPathUserData(), // ユーザ設定データ
        applicationSetting()->settingDataPathMeasure(),  // 測定設定データ
        applicationSetting()->settingDataPathLogger()    // ロガー設定データ
    };

    for (int i = 0; i < deleteFilePathList.size(); i++) {
        if (!removePreservedData(deleteFilePathList.at(i), true)) {
            qDebug() << "targetFilePath : " << deleteFilePathList.at(i);
            result = false;
        }
    }

    // 内部メモリを再度更新する
    initializeUserSettingFile();

    return result;
#endif
}

bool LevelCheckerModel::fourceBuzzerVolume(int volumeLevel, LC70_BUZZ_T buzzerPattern)
{
    bool ret = false;

    if (_communicator->fourceBuzzerVolume(buzzerPattern, volumeLevel) == LC70_OK) {
        ret = true;
    }

    return ret;
}

bool LevelCheckerModel::buzzer(int buzzerPattern, int buzzerMode, bool forcedRinging)
{
    // 音量の設定値を取得する。
    QVariant value;
    int settingVolume = LC70_SYS_SET_VOLUME_OFF;

    if (getValue(VALUE_SOUNDVOLUME, value)) {
        settingVolume = value.toInt();

        // モデルから得られたデータが範囲外であった場合
        // 音量設定値に対して初期値が設定されているとして扱う
        if (settingVolume < LC70_SYS_SET_VOLUME_OFF || settingVolume > LC70_SYS_SET_VOLUME_MAX) {
            settingVolume = LC70_SYS_SET_VOLUME_OFF;
        }
    }

    if (settingVolume == LC70_SYS_SET_VOLUME_OFF) {
        // 強制鳴動不可の場合
        if (!forcedRinging) {
            // 音量設定がOFFの場合後続処理は実施しない
            return false;
        }
    }

    bool ret = false;

    LC70_BUZZ_T buzz_t = static_cast<LC70_BUZZ_T>(buzzerPattern);
    LC70_BUZZ_MODE_T buzz_mode_t = static_cast<LC70_BUZZ_MODE_T>(buzzerMode);

    LC70_ERR_T result = _communicator->buzzer(buzz_t, buzz_mode_t);

    if (result == LC70_OK) {
        ret = true;
    } else if (result == LC70_ERR_BUZZ_REPEATING) {
        ret = true;
    } else {
        // nothing to do
    }

    return ret;
}
bool LevelCheckerModel::setScreenBrightness(int brightnessLebel)
{
    bool ret = false;

    if (_communicator->setScreenBrightness(brightnessLebel) == LC70_OK) {
        ret = true;
    }

    return ret;
}

void LevelCheckerModel::onResumeSingleMeasuring(LC70_CONSTA_NUM_T constellationPoints, LC70_MEAS_SINGLE_T condition,
        uint32_t interval)
{
    // サブ CPU が起動していなければ何もしない
    if (!readyToMeasure()) {
        return;
    }

    _measurer->resume();
    emit resumeSingleMeasuring(constellationPoints, condition, interval);
}

void LevelCheckerModel::onSuspendSingleMeasuring(void)
{
    emit requestToSuspendSingleMeasuring();
}

void LevelCheckerModel::onResumeMultiMeasuring(LC70_MEAS_MULTI_T *condition, LC70_MEAS_SINGLE_T *prefferedCondition, LC70_DISP_TYPE_T displayType,
        uint32_t interval)
{
    // サブ CPU が起動していなければ何もしない
    if (!readyToMeasure()) {
        return;
    }

    // 測定が 1 チャンネルのみであれば間隔を変更する
    if (condition->sRegNum == 1) {
        interval = _applicationSettings->pollingIntervalOneChannel();
    }

    _measurer->resume();
    emit resumeMultiMeasuring(condition, prefferedCondition, displayType, interval);
}

void LevelCheckerModel::onPreferredMeasuringChangeCondition(LC70_MEAS_SINGLE_T *condition)
{
    // サブ CPU が起動していなければ何もしない
    if (!readyToMeasure()) {
        return;
    }

    emit preferredMeasuringChangeCondition(condition);
}

void LevelCheckerModel::onSuspendMultiMeasuring()
{
    // マルチ優先測定⇒優先測定の順に停止する
    emit requestToSuspendMultiMeasuring();
}

void LevelCheckerModel::onResumeAntennaMeasuring(LC70_MEAS_MULTI_T *condition, uint32_t interval)
{
    // サブ CPU が起動していなければ何もしない
    if (!readyToMeasure()) {
        return;
    }

    _measurer->resume();
    emit resumeAntennaMeasuring(condition, interval);
}

void LevelCheckerModel::onSuspendAntennaMeasuring()
{
    emit requestToSuspendAntennaMeasuring();
}

void LevelCheckerModel::onRequestToStandBy(uint8_t mode)
{
    if (mode == MAIN_AND_SUB) {
        _wakeUpCounter = _systemWatcher->wakeUpCounter();
        writeLog(QString(typeid(*this).name()), QString("STAND BY %1").arg(_wakeUpCounter));
    }

    emit requestToStandBy(mode);
}

void LevelCheckerModel::onNotifySoftwareUpdateCompleted(bool success)
{
    emit notifySoftwareUpdateCompleted(success);
    _communicator->systemReboot();
    _updating = false;
}

void LevelCheckerModel::onNotifyStopMeasuring(AcquisitionTarget target)
{
    // 測定停止を受信
    emit notifyStopMeasuring(target);
}

bool LevelCheckerModel::createZip(const QString &archiveFile, const QVector<QStringRef> &fileList, const QString &destination, const QString &password)
{
    return ZipHelper::create(archiveFile, fileList, destination, password);
}

bool LevelCheckerModel::extractZip(const QString &archiveFile, const QString &destination, const QString &password)
{
    return ZipHelper::extract(archiveFile, destination, password);
}

void LevelCheckerModel::setStandBy(const bool &standBy)
{
    _standBy = standBy;
}

QList<ChannelDefinitionItem> LevelCheckerModel::scannedChannelList() const
{
    return _scannedChannelList;
}

QString LevelCheckerModel::scannedPolatization() const
{
    return _scannedPolatization;
}

void LevelCheckerModel::setScannedChannelList(const QList<ChannelDefinitionItem> &channelList, const QString &polarization)
{
    clearScannedChannelList();

    for (int i = 0; i < channelList.count(); ++i) {
        _scannedChannelList.append(channelList[i]);
    }

    _scannedPolatization = polarization;
}

void LevelCheckerModel::clearScannedChannelList()
{
    _scannedChannelList.clear();
}

bool LevelCheckerModel::startAzimuth()
{
    if (_measurer == nullptr) {
        return false;
    }

    return _measurer->startAzimuth();
}

bool LevelCheckerModel::stopAzimuth()
{
    if (_measurer == nullptr) {
        return false;
    }

    return _measurer->stopAzimuth();
}

void LevelCheckerModel::startElapsedTimer(const QString &message)
{
    _elapsedTimer.restart();
    writeLog(message, QString("%1 ms").arg(_elapsedTimer.elapsed()), LOG_INFO);
}

void LevelCheckerModel::recordElapsedTimer(const QString &message)
{
    writeLog(message, QString("%1 ms").arg(_elapsedTimer.elapsed()), LOG_INFO);
}

void LevelCheckerModel::changeDeviceStatusInterval(bool isCompass)
{
    _systemWatcher->setInterval(_applicationSettings->pollingIntervalKeyInput(),
                                isCompass ? _applicationSettings->pollingIntervalCompass()
                                          : _applicationSettings->pollingIntervalDeviceStatus());
}

void LevelCheckerModel::initDateTime()
{
    _communicator->initDateTime();
}

bool LevelCheckerModel::notifyLowBatteryStatusChanged(bool isLowBattery)
{
    emit lowBatteryStatusChanged(isLowBattery);
    return LC70_OK == _communicator->notifyLowBatteryStatusChanged(isLowBattery);
}

bool LevelCheckerModel::readyToMeasure()
{
    if (!_modelOperator->readyToMeasure()) {
        // サブ CPU エラーにする
        LC70_FAIL_INFO_T errorInformation;
        memset(&errorInformation, 0, sizeof(LC70_FAIL_INFO_T));
        errorInformation.usCode = LC70_FAIL_SUB_ERR;
        errorInformation.ucStat = LC70_FAIL_STAT_FAIL;
        errorInformation.ucCtl = LC70_FAIL_CTL_MSGDISP;
        memcpy(&errorInformation.ucAddInfo, "No Reply Sub CPU", 16);
        emit notifyErrorOccured(errorInformation);
        return false;
    }
    return true;
}

bool LevelCheckerModel::initializeFactoryDefault()
{
    return (_communicator->initializeFactoryDefault() == LC70_OK);
}

bool LevelCheckerModel::setDeveloperMode(const uint8_t &selection)
{
    return (_communicator->setDeveloperMode(selection) == LC70_OK);
}

void LevelCheckerModel::setDisplayFileSystemModel(QFileSystemModel *model)
{
    _displayFileSystemModel = model;
}

QFileSystemModel *LevelCheckerModel::displayFileSystemModel()
{
    return _displayFileSystemModel;
}

void LevelCheckerModel::takeMeasureToFindNoView(const QString &screenName)
{
    writeLog(QString("View 作成不可"), screenName, LOG_ERR);
    // View が nullptr
    LC70_FAIL_INFO_T errorInformation;
    memset(&errorInformation, 0, sizeof(LC70_FAIL_INFO_T));
    errorInformation.usCode = LC70_FAIL_PARAM_ERR;
    errorInformation.ucStat = LC70_FAIL_STAT_FAIL;
    errorInformation.ucCtl = LC70_FAIL_CTL_MSGDISP;
    memcpy(&errorInformation.ucAddInfo, "Broken screen", 16);
    emit notifyErrorOccured(errorInformation);
}

LC70_ERR_T LevelCheckerModel::resetSubCpu()
{
    return _communicator->resetSubCpu();
}

void LevelCheckerModel::onKeyStatus(LC70_KEY_STAT_T key)
{
    // プログラムアップデート中はボタン操作を受け付けないようにする
    if (_updating) {
        return;
    }

    // ボタン状態
    static LC70_KEY_T buttonStatus = LC70_KEY_OFF;

    // 電源ボタンをの状態の変更を検知
    if (key.stat[LC70_KEY_ID_POWER] != buttonStatus) {
        if (buttonStatus == LC70_KEY_ON) {
            qDebug() << QString("POWER button released");

            if (!_standBy) {
                writeLog(QString(typeid(*this).name()), QString("STAND BY"), LOG_INFO);

                _standBy = true;

                // ウェイクアップ要求を出した画面が Home 画面に戻ったときに MAIN/SUB へのスタンバイ要求の後 SUB へのスタンバイ要求を出さないようにする
                setWakeUpRequested(false);

                // スリープ状態に移行する
                // 画面に通知
                emit notifyStandBy();
            } else {
                // 仮想環境実行時のみ実行
                if (_standBy == true && _applicationSettings->testMode() != ActionMode::PRODUCTION) {
                    onNotifyWakeUp();
                }
            }

            buttonStatus = LC70_KEY_OFF;
            return;
        } else {
            qDebug() << QString("POWER button pressed");
        }
    } else {
        bool allKeyOff = true;
        for (int i = 0; i < LC70_KEY_ID_MAX; ++i) {
            if (key.stat[i] != LC70_KEY_OFF) {
                allKeyOff = false;
                // 電源ボタン以外のボタンが押されたら自動パワーオフのタイマーをリセット
                if (_autoPowerOffTimer) {
                    if (_autoPowerOffTimer->isActive()) {
                        _autoPowerOffTimer->stop();
                    }
                    // 自動パワーオフの設定がオフならタイマーを開始しない
                    if (_autoPowerOffTimer->interval() > 0) {
                        _autoPowerOffTimer->start();
                    }
                }
                break;
            }
        }

        // オートパワーオフからの復帰判定(オートパワーオフ後に電源ボタンを押すとスタンバイ中の状態で全キーがオフのキー状態を受信する)
        if (allKeyOff && _standBy && _wakeUpCounter != _systemWatcher->wakeUpCounter()) {

            qDebug() << QString("WAITING FOR WAKING UP FROM POWER OFF %1").arg(_wakeUpCounter);

            onNotifyWakeUp();
            return;
        }

        // 電源以外のボタンは画面に通知
        emit keyStatus(key);
    }

    buttonStatus = key.stat[LC70_KEY_ID_POWER];
}

void LevelCheckerModel::onErrorOccured(LC70_FAIL_INFO_T errorInformation)
{
    // バージョンアップデートの検出
    if (errorInformation.usCode == LC70_FAIL_MSG_PRGUPDATE) {
        if (errorInformation.ucStat == LC70_FAIL_STAT_FAIL) {
            memset(&_updater, 0, sizeof(_updater));
            memcpy(&_updater.ucMainVer, errorInformation.ucAddInfo, LC70_VERSION_SIZE);
            memcpy(&_updater.ucSubVer, errorInformation.ucAddInfo + LC70_VERSION_SIZE, LC70_VERSION_SIZE);
            _versionUpReady = true;
            emit notifyVersionUpReady(_versionUpReady);
            writeLog(QString(typeid(*this).name()), QString("[UPDATE FILE DETECTED!]"), LOG_INFO);
        } else {
            _versionUpReady = false;
            emit notifyVersionUpReady(_versionUpReady);
        }
    }

    if (errorInformation.usCode == LC70_FAIL_NONE) {
        return;
    }

    // 故障情報をエラー管理に通知する
    emit notifyFailInfo(errorInformation);

    writeLog(QString(typeid(*this).name()),
             QString("[ERROR]error code: %1").arg(errorInformation.usCode));
}

void LevelCheckerModel::onNotifyWakeUp()
{
    if (_updating) {
        return;
    }

    // ウェイクアップカウンターが同じなら何もしない
    if (_wakeUpCounter == _systemWatcher->wakeUpCounter()) {
        return;
    }

    // ウェイクアップカウンターを再設定
    _wakeUpCounter = _systemWatcher->wakeUpCounter();

    // MAIN/SUB の wake up は standby が行われていたら続行
    QFile file;
    file.setFileName(QString(STANDBY_LOCK_FILE_NAME));
    if (!QFile::exists(QString(STANDBY_LOCK_FILE_NAME))) {
        writeLog(QString(typeid(*this).name()), QString("[IGNORE UNEXPECTED WAKE UP]"));
        return;
    }
    if (file.remove()) {
        qErrnoWarning("standby file exits.");
    }

    // ウェイクアップ状態に移行する
    writeLog(QString(typeid(*this).name()),
             QString("[WAKE UP] counter: %1").arg(_wakeUpCounter), LOG_NOTICE);

    _standBy = false;

    // ウェイクアップ要求
    // ウェイクアップ時は測定系の処理は何も動いていないため測定スレッドではなくメインスレッドから直接ラッパー関数を呼ぶ
    if (!_modelOperator->readyToWakeUp()) {
        // サブ CPU エラーにする
        LC70_FAIL_INFO_T errorInformation;
        memset(&errorInformation, 0, sizeof(LC70_FAIL_INFO_T));
        errorInformation.usCode = LC70_FAIL_SUB_ERR;
        errorInformation.ucStat = LC70_FAIL_STAT_FAIL;
        errorInformation.ucCtl = LC70_FAIL_CTL_MSGDISP;
        memcpy(&errorInformation.ucAddInfo, "No Reply Sub CPU", 16);
        emit notifyErrorOccured(errorInformation);
        return;
    }

    // MAIN/SUB の wake up
    if (_communicator->systemWakeup() == LC70_OK) {

    }

    // 時刻を初期化
    initDateTime();

    // 画面に通知
    emit notifyWakeUp();

    // 自動パワーオフタイマー開始
    if (_autoPowerOffTimer) {
        // 自動パワーオフの設定がオフならタイマーを開始しない
        if (_autoPowerOffTimer->interval() > 0) {
            _autoPowerOffTimer->start();
        }
    }
}

void LevelCheckerModel::onAutoPowerOffTimeOut()
{
    writeLog(QString(typeid(*this).name()), QString("自動パワーオフ検知"), LOG_INFO);

    // MainFrame に通知し、画面が動作していなければスタンバイ要求を呼び出す
    _standBy = true;
    emit notifyAutoPowerOff();
}

void LevelCheckerModel::changePowerOffSetting()
{
    QVariant value;
    _modelOperator->getValue(VALUE_AUTOPOWEROFF, value);
    bool ok = false;
    int waitingTime = value.toInt(&ok);
    if (ok) {
        if (_autoPowerOffTimer) {
            if (_autoPowerOffTimer->isActive()) {
                _autoPowerOffTimer->stop();
            }
            _autoPowerOffTimer->setInterval(waitingTime * 60 * 1000);   // 分単位の値のためミリ秒に置き換える
            // 自動パワーオフの設定がオフならタイマーを開始しない
            if (_autoPowerOffTimer->interval() > 0) {
                _autoPowerOffTimer->start();
            }
        }
    }
}

bool LevelCheckerModel::initializeUserDataLoggerSettingFile(ChannelEditMode mode)
{
    _userDataLoggerSettingList.clear();
    _userDataLoggerEditingChannels.clear();
    _userDataLoggerMeasuringChannels.clear();
    _channelEditMode = mode;
    _userDataLoggerSettingFileName.clear();

    // 設定ファイルの読み込み
    QString path = QCoreApplication::applicationDirPath() + QString("/") + USER_DATALOGGER_SETTING_TEMPLATE;

    return readUserDataLoggerSettingFile(path, QString(""), mode);
}

bool LevelCheckerModel::readUserDataLoggerSettingFile(const QString &path, const QString &fileName, ChannelEditMode mode)
{
    // USER データロガー設定ファイルの読み込み
    QFile file(path);

    // 設定ファイルを開く(開けなかったらバックアップからコピーする)
    if (!file.open(QIODevice::ReadOnly)) {
        // テンプレートファイルの場合は何もせず失敗とする
        if (path.contains(USER_DATALOGGER_SETTING_TEMPLATE)) {
            return false;
        }
        // バックバップファイルのコピー
        if (!copyFromBackupFile(path)) {
            qDebug() << QString("file open error(%1)").arg(path);
            return false;
        }

        qDebug() << QString("file open error(%1)").arg(path);
        return false;
    }

    _userDataLoggerSettingList.clear();
    _userDataLoggerEditingChannels.clear();
    _userDataLoggerMeasuringChannels.clear();

    QTextStream in(&file);
    QStringList csvLineText;
    UserDataLoggerSettingFile rowData;

    int row = 0;

    while (!in.atEnd()) {
        csvLineText = in.readLine().split(",");

        ++row;

        if (row < FIRST_DATA_ROW) {
            continue;
        }

        if (csvLineText.size() != LIST_COLUMN) {
            continue;
        }

        bool ok =false;
        int band = csvLineText[0].toInt(&ok);
        if (!ok) {
            continue;
        }
        rowData.band = static_cast<BandTabIndex>(band);         // Band
        rowData.channel = csvLineText[1];                       // チャンネル名
        uint32_t frequency = csvLineText[2].toUInt(&ok);
        if (!ok) {
            continue;
        }
        rowData.frequency = frequency;                          // 周波数
        uint8_t broadcastSystem = csvLineText[3].toUInt(&ok);
        if (!ok) {
            continue;
        }
        rowData.broadcastsystem = broadcastSystem;              // 放送方式
        uint32_t localFrequency = csvLineText[4].toUInt(&ok);
        if (!ok) {
            continue;
        }
        rowData.localFrequency = localFrequency;                // 局発周波数

        uint8_t powerSupply = csvLineText[5].toUInt(&ok);
        if (!ok) {
            continue;
        }
        rowData.powerSupply = powerSupply;                      // 給電 ON/OFF

        uint8_t mearure = csvLineText[6].toUInt(&ok);
        if (!ok) {
            continue;
        }
        rowData.measure = mearure;                              // 測定する/しない

        _userDataLoggerSettingList.append(rowData);
    }

    file.close();


    // CATV チャンネル周波数の適用(新規作成時のみ)
    if (path.contains(USER_DATALOGGER_SETTING_TEMPLATE) || mode == CHANNEL_EDIT_SELECT_DESTINATION) {
        QVariant value;
        if (getValue(VALUE_CATVCHANNEL_FREQ, value)) {
            // CATV チャンネル周波数が設定 2 の場合、一部チャンネルの周波数を付け替える
            if (value.toInt() == CATV_SETTING_2) {
                _userDataLoggerSettingList[112].frequency += 2000;  // C24
                _userDataLoggerSettingList[113].frequency += 2000;  // C25
                _userDataLoggerSettingList[114].frequency += 2000;  // C26
                _userDataLoggerSettingList[115].frequency += 2000;  // C27
            }
        }
    }

    if (_userDataLoggerSettingList.count() != USER_DATALOGGER_SETTING_FILE_ROWS) {
        qDebug() << QString("USER データロガー設定ファイル行数: %1").arg(_userDataLoggerSettingList.count());
        _userDataLoggerSettingList.clear();
        return false;
    }

    // 設定ファイル読込後は地デジを選択
    _bandTabIndex = BAND_TAB_UHF;
    selectEditingChannels(_bandTabIndex);

    _userDataLoggerSettingFileName = fileName;
    _bandTabIndex = BAND_TAB_UHF;

    return true;
}

bool LevelCheckerModel::copyFromBackupFile(const QString &path)
{
    // バックアップファイルからコピー
    QString command = QString("cp %1 %2").arg(path + QString(BACKUP_FILE_EXTENSION)).arg(path);
    int exitCode = QProcess::execute(command);

    return (exitCode != -1 && exitCode != -2);
}

#if 1
// タイマーを再設定して開始する
void LevelCheckerModel::enablePowerOffSetting()
{
    QVariant value;
    _modelOperator->getValue(VALUE_AUTOPOWEROFF, value);
    bool ok = false;
    int waitingTime = value.toInt(&ok);
    if (ok) {
        if (_autoPowerOffTimer) {
            if (_autoPowerOffTimer->isActive()) {
                _autoPowerOffTimer->stop();
            }
            _autoPowerOffTimer->setInterval(waitingTime * 60 * 1000);   // 分単位の値のためミリ秒に置き換える
            // 自動パワーオフの設定がオフならタイマーを開始しない
            if (_autoPowerOffTimer->interval() > 0) {
                _autoPowerOffTimer->start();
            }
        }
    }
}

// タイマーを止める
void LevelCheckerModel::disablePowerOffSetting()
{
    if (_autoPowerOffTimer) {
        if (_autoPowerOffTimer->isActive()) {
            _autoPowerOffTimer->stop();
        }
    }
}
#endif

bool LevelCheckerModel::importSettingFile(ChannelEditMode mode, const QString &fileName)
{
    // ファイル名が空ならテンプレートファイルから読み出す
    if (fileName.isEmpty() || fileName == QString(CSV_FILE_EXTENSION)) {
        return initializeUserDataLoggerSettingFile(mode);
    }

    // USER データロガー設定チャンネル編集モードからパスを選択
    QString path;
    if(mode == CHANNEL_EDIT_USER) {
        path = _applicationSettings->settingDataPathUserData();
    } else if(mode == CHANNEL_EDIT_USER_USB1) {
        path = _applicationSettings->settingDataPathUserDataUsb1();
    } else if(mode == CHANNEL_EDIT_USER_USB2) {
        path = _applicationSettings->settingDataPathUserDataUsb2();
    } else if(mode == CHANNEL_EDIT_DATA_LOGGER) {
        path = _applicationSettings->settingDataPathLogger();
    } else if(mode == CHANNEL_EDIT_DATA_LOGGER_USB1) {
        path = _applicationSettings->settingDataPathLoggerUsb1();
    } else if(mode == CHANNEL_EDIT_DATA_LOGGER_USB2) {
        path = _applicationSettings->settingDataPathLoggerUsb2();
    } else if(mode == CHANNEL_EDIT_SELECT_DESTINATION) {
        path = _applicationSettings->settingDataPathUserData();
    } else {
        return false;
    }

    path = path + QString("/") + fileName;

    return readUserDataLoggerSettingFile(path, fileName, mode);
}

bool LevelCheckerModel::exportSettingFile(const ChannelEditMode &mode, const QString &fileName)
{
    // USER データロガー設定ファイルの行数と異なる
    if (_userDataLoggerSettingList.count() != USER_DATALOGGER_SETTING_FILE_ROWS) {
        return false;
    }
    _channelEditMode = mode;

    // USER データロガー設定チャンネル編集モードからパスを選択
    QString path;
    if(_channelEditMode == CHANNEL_EDIT_USER) {
        path = _applicationSettings->settingDataPathUserData();
    } else if(_channelEditMode == CHANNEL_EDIT_USER_USB1) {
        path = _applicationSettings->settingDataPathUserDataUsb1();
    } else if(_channelEditMode == CHANNEL_EDIT_USER_USB2) {
        path = _applicationSettings->settingDataPathUserDataUsb2();
    } else if(_channelEditMode == CHANNEL_EDIT_DATA_LOGGER) {
        path = _applicationSettings->settingDataPathLogger();
    } else if(_channelEditMode == CHANNEL_EDIT_DATA_LOGGER_USB1) {
        path = _applicationSettings->settingDataPathLoggerUsb1();
    } else if(_channelEditMode == CHANNEL_EDIT_DATA_LOGGER_USB2) {
        path = _applicationSettings->settingDataPathLoggerUsb2();
    } else {
        path = _applicationSettings->settingDataPathUserData();
    }

    path = path + QString("/") + fileName;
    if (!fileName.contains(".csv", Qt::CaseInsensitive)) {
        path = path + QString(".csv");
    }

    {
        QFile file(path);

        if (!file.open(QIODevice::WriteOnly)) {
            return false;
        }

        QTextStream outStream(&file);

        // ヘッダー
        outStream << QString("'BandTabIndex") << QString(",");
        outStream << QString("Channel") << QString(",");
        outStream << QString("Frequency[KHz]") << QString(",");
        outStream << QString("BroadbandSystem") << QString(",");
        outStream << QString("localFrequency[KHz]") << QString(",");
        outStream << QString("powerSupply") << QString(",");
        outStream << QString("measuring") << QString("\n");

        // 全 Band 全チャンネルの測定条件
        for (int i = 0; i < _userDataLoggerSettingList.count(); ++i) {
            outStream << _userDataLoggerSettingList[i].band << QString(",");
            outStream << _userDataLoggerSettingList[i].channel << QString(",");
            outStream << _userDataLoggerSettingList[i].frequency << QString(",");
            outStream << _userDataLoggerSettingList[i].broadcastsystem << QString(",");
            outStream << _userDataLoggerSettingList[i].localFrequency << QString(",");
            outStream << _userDataLoggerSettingList[i].powerSupply << QString(",");
            outStream << _userDataLoggerSettingList[i].measure << QString("\n");
        }

        file.close();
    }

    QProcess::execute(SYNC_COMMAND);
    QString command = QString("cp %1 %2").arg(path).arg(path + QString(BACKUP_FILE_EXTENSION));
    int exitCode = QProcess::execute(command);
    QProcess::execute(SYNC_COMMAND);

    if (exitCode != -1 && exitCode != -2) {
        return true;
    } else {
        return false;
    }
}

QList<UserDataLoggerSettingFile> LevelCheckerModel::selectEditingChannels(const BandTabIndex &bandTabIndex)
{
    _userDataLoggerEditingChannels.clear();

    _bandTabIndex = bandTabIndex;
    for (int i = 0; i < _userDataLoggerSettingList.count(); ++i) {
        if (_userDataLoggerSettingList[i].band == _bandTabIndex) {
            _userDataLoggerEditingChannels.append(_userDataLoggerSettingList[i]);
        }
    }

    return _userDataLoggerEditingChannels;
}

QList<UserDataLoggerSettingFile> LevelCheckerModel::selectMeasuringChannels()
{
    _userDataLoggerMeasuringChannels.clear();

    for (int i = 0; i < _userDataLoggerSettingList.count(); ++i) {
        if (_userDataLoggerSettingList[i].measure == DO_MEASURING) {
            _userDataLoggerMeasuringChannels.append(_userDataLoggerSettingList[i]);
        }
    }

    return _userDataLoggerMeasuringChannels;
}

int LevelCheckerModel::selectMeasuringRows(const BandTabIndex &bandTabIndex)
{
    int result = 0;
    for (int i = 0; i < _userDataLoggerSettingList.count(); ++i) {
        if (_userDataLoggerSettingList[i].band == bandTabIndex &&
                _userDataLoggerSettingList[i].measure == DO_MEASURING) {
            ++result;
        }
    }

    return result;
}

BandTabIndex LevelCheckerModel::selectGroupPrefix()
{
    BandTabIndex index = BAND_TAB_UHF;
    for (int i = 0; i < _userDataLoggerSettingList.count(); ++i) {
        if (_userDataLoggerSettingList[i].measure == DO_MEASURING) {
            index = _userDataLoggerSettingList[i].band;
            break;
        }
    }

    return index;
}

void LevelCheckerModel::selectFirstValidChannel(BandTabIndex &bandTabIndex, QString &channel)
{
    bandTabIndex = BAND_TAB_UHF;
    if (_userDataLoggerSettingList.count() > 0) {
        channel = _userDataLoggerSettingList[0].channel;
    }

    for (int i = 0; i < _userDataLoggerSettingList.count(); ++i) {
        if (_userDataLoggerSettingList[i].measure == DO_MEASURING) {
            bandTabIndex = _userDataLoggerSettingList[i].band;
            channel = _userDataLoggerSettingList[i].channel;
            break;
        }
    }
}

bool LevelCheckerModel::editMeasuring(const BandTabIndex bandTabIndex, const QString &channel, const int &value)
{
    int firstIndex = FIRST_INDEX_FOR_EACH_BAND.value(bandTabIndex);
    for (int i = firstIndex; i < _userDataLoggerSettingList.count(); ++i) {
        if (_userDataLoggerSettingList[i].band != bandTabIndex) {
            break;
        }
        if (_userDataLoggerSettingList[i].channel == channel) {
            _userDataLoggerSettingList[i].measure = value;
            return true;
        }
    }

    return false;
}

void LevelCheckerModel::editPowerSupply(const BandTabIndex bandTabIndex, const int &value)
{
    // Band の給電 ON/OFF を一括更新する
    int firstIndex = FIRST_INDEX_FOR_EACH_BAND.value(bandTabIndex);
    for (int i = firstIndex; i < _userDataLoggerSettingList.count(); ++i) {
        if (_userDataLoggerSettingList[i].band != bandTabIndex) {
            break;
        }
        _userDataLoggerSettingList[i].powerSupply = value;
    }
}

bool LevelCheckerModel::editBroadcastSystem(const BandTabIndex bandTabIndex, const QString &channel, const int &value, bool scanned)
{
    int firstIndex = FIRST_INDEX_FOR_EACH_BAND.value(bandTabIndex);
    for (int i = firstIndex; i < _userDataLoggerSettingList.count(); ++i) {
        if (_userDataLoggerSettingList[i].band != bandTabIndex) {
            break;
        }
        if (_userDataLoggerSettingList[i].channel == channel) {
            // USER データロガー設定の操作により CATV ALL で値を変更するときは周波数も変更する
            if (bandTabIndex == BAND_TAB_CATV_ALL) {
                if (value != _userDataLoggerSettingList[i].broadcastsystem) {
                    if (value ==  LC70_BD_SYS_ISDBT) {
                        _userDataLoggerSettingList[i].frequency += 143;
                    } else if (value == LC70_BD_SYS_ANNEXC) {
                        // スキャン結果で放送方式が ANNEXC のときはデフォルト値のままでよいので何もしない
                        if (!scanned) {
                            _userDataLoggerSettingList[i].frequency -= 143;
                        }
                    }
                }
            }
            _userDataLoggerSettingList[i].broadcastsystem = value;
            return true;
        }
    }

    return false;
}

void LevelCheckerModel::editLocalFrequency(const BandTabIndex bandTabIndex, const int &value)
{
    // Band の局発周波数を一括更新する
    int firstIndex = FIRST_INDEX_FOR_EACH_BAND.value(bandTabIndex);
    for (int i = firstIndex; i < _userDataLoggerSettingList.count(); ++i) {
        if (_userDataLoggerSettingList[i].band != bandTabIndex) {
            break;
        }
        _userDataLoggerSettingList[i].localFrequency = value;
    }
}

bool LevelCheckerModel::editFrequency(const BandTabIndex bandTabIndex, const QString &channel, const int &value)
{
    if (bandTabIndex != BAND_TAB_CATV_UPSTREAM && bandTabIndex != BAND_TAB_FM) {
        return false;
    }

    int firstIndex = FIRST_INDEX_FOR_EACH_BAND.value(bandTabIndex);
    for (int i = firstIndex; i < _userDataLoggerSettingList.count(); ++i) {
        if (_userDataLoggerSettingList[i].band != bandTabIndex) {
            break;
        }
        if (_userDataLoggerSettingList[i].channel == channel) {
            _userDataLoggerSettingList[i].frequency = value;
            return true;
        }
    }

    return false;
}

bool LevelCheckerModel::writeCandidateFiles(const int &type, const QStringList &candidate, const QString &destination)
{
    QString fileName;

    if (type == 0) {
        // コピー
        // ディレクトリ
        fileName = TEMPORARY_PATH + QString("/") + QString(COPY_DIRECTORY_CANDIDATE_FILENAME);
        QFile directoryFile(fileName);
        if (directoryFile.open(QIODevice::WriteOnly)) {
            QTextStream outStream(&directoryFile);

            for (int i = 0; i < candidate.count(); ++i) {
                QString path = replaceSourceToDestination(candidate[i], destination);
#if 1
                QString dirpath = "";
#endif
                QStringList elements = path.split("/");
#if 1
                for (int i = 0; i < elements.count() - 1; ++i) {
                    dirpath += elements[i];
                    dirpath += QString("/");
                }
                outStream << dirpath << QString("\n");
#else
                for (int i = 0; i < elements.count() - 1; ++i) {
                    path += elements[i];
                    path += QString("/");
                }
                outStream << path << QString("\n");
#endif
            }
            directoryFile.close();
            QProcess::execute(SYNC_COMMAND);
        } else {
            return false;
        }

        // コピーするファイルのフルパス
        fileName = TEMPORARY_PATH + QString("/") + QString(COPY_CANDIDATE_FILENAME);
        QFile file(fileName);
        if (file.open(QIODevice::WriteOnly)) {
            QTextStream outStream(&file);

            for (int i = 0; i < candidate.count(); ++i) {
                outStream << candidate[i] << QString(" ") << replaceSourceToDestination(candidate[i], destination) << QString("\n");
            }
            file.close();
            QProcess::execute(SYNC_COMMAND);

            return true;
        }
    } else {
        // 削除
        fileName = TEMPORARY_PATH + QString("/") + QString(REMOVE_CANDIDATE_FILENAME);

        QFile file(fileName);
        if (file.open(QIODevice::WriteOnly)) {
            QTextStream outStream(&file);

            for (int i = 0; i < candidate.count(); ++i) {
                outStream << candidate[i] << QString("\n");
            }
            file.close();
            QProcess::execute(SYNC_COMMAND);

            return true;
        }
    }

    return false;
}

bool LevelCheckerModel::removeMeasurementFiles(const QString &path)
{
    QString command = QString("/bin/sh -c \"cat %1/%2 | xargs rm -f\"").arg(TEMPORARY_PATH).arg(REMOVE_CANDIDATE_FILENAME);

    int exitCode = QProcess::execute(command);
    QProcess::execute(SYNC_COMMAND);

    if (exitCode == -1 || exitCode == -2) {
        return false;
    }

    // 空のディレクトリを削除
    command = QString("/bin/sh -c \"find %1/ -type d | xargs rmdir -p --ignore-fail-on-non-empty\"").arg(path);

    exitCode = QProcess::execute(command);
    QProcess::execute(SYNC_COMMAND);

    if (exitCode == -1 || exitCode == -2) {
        return false;
    }

    return true;
}

bool LevelCheckerModel::copyMeasurementFiles()
{
    // ディレクトリ作成
    QString command = QString("/bin/sh -c \"cat %1/%2 | xargs mkdir -p\"").arg(TEMPORARY_PATH).arg(COPY_DIRECTORY_CANDIDATE_FILENAME);

    int exitCode = QProcess::execute(command);
    QProcess::execute(SYNC_COMMAND);

    if (exitCode == -1 || exitCode == -2) {
        return false;
    }

    // 測定結果ファイルのコピー
    command = QString("/bin/sh -c \"cat %1/%2 | xargs -n2 cp -f\"").arg(TEMPORARY_PATH).arg(COPY_CANDIDATE_FILENAME);

    exitCode = QProcess::execute(command);
    QProcess::execute(SYNC_COMMAND);

    if (exitCode == -1 || exitCode == -2) {
        return false;
    }

    // 測定データ管理ファイルコピー
    command = QString("cp -f /media/mmcblk0p3/result/data_file_list.csv /media/sda1/LC70WCS/DATA/data_file_list.csv");

    exitCode = QProcess::execute(command);
    QProcess::execute(SYNC_COMMAND);

    if (exitCode == -1 || exitCode == -2) {
        return false;
    }

    return true;
}

QString LevelCheckerModel::replaceSourceToDestination(const QString &source, const QString &destination)
{
    QStringList elements = source.split("/");
    QString fileName = elements[elements.count() - 1];
    QString path;

    // ファイル名を "/" で区切った後、5 つ目の要素が YYYYMMDD 形式の日付
    if (elements.count() > 5) {
        if (fileName.contains(CSV_FILE_EXTENSION)) {
            path = destination + QString("/%1/CSV/").arg(elements[5]);
        } else {
            path = destination + QString("/%1/PNG/").arg(elements[5]);
        }
    }

    return path + fileName;
}

bool LevelCheckerModel::importDataLoggerUserSettingList(ValueId valueId, const QString &filename)
{
    QString fullPathFilename;

    if (_applicationSettings->testMode() != ActionMode::PRODUCTION) {
        fullPathFilename = _applicationSettings->stubDataPathDataLogger() + "/";
        fullPathFilename += _applicationSettings->settingDataPathStubLogger() + "/" + filename;
    } else {
        fullPathFilename += filename;
    }


    qDebug("fullpath name : %s", fullPathFilename.toUtf8().constData());
    return _modelOperator->importDataLoggerUserSettingList(valueId, fullPathFilename);
}

bool LevelCheckerModel::exportDataLoggerUserSettingList(ValueId valueId, const QString &filename)
{
    QString fullPathFilename;

    if (_applicationSettings->testMode() != ActionMode::PRODUCTION) {
        fullPathFilename = _applicationSettings->stubDataPathDataLogger() + "/";
        fullPathFilename += _applicationSettings->settingDataPathStubLogger() + "/" + filename;
    } else {
        fullPathFilename += filename;
    }

    return _modelOperator->exportDataLoggerUserSettingList(valueId, fullPathFilename);
}

bool LevelCheckerModel::removeDataLoggerUserSettingList(ValueId valueId, const QString &filename)
{
    QString fullPathFilename;

    ChannelEditMode mode = CHANNEL_EDIT_USER;

    if(valueId == VALUE_USERSETTING_SETTINGLIST) {
        mode = CHANNEL_EDIT_USER;
    } else if(valueId == VALUE_USERSETTING_SETTINGLIST_USB1) {
        mode = CHANNEL_EDIT_USER_USB1;
    } else if(valueId == VALUE_USERSETTING_SETTINGLIST_USB2) {
        mode = CHANNEL_EDIT_USER_USB2;
    } else if(valueId == VALUE_DATALOGGERSETTING_SETTINGLIST) {
        mode = CHANNEL_EDIT_DATA_LOGGER;
    } else if(valueId == VALUE_DATALOGGERSETTING_SETTINGLIST_USB1) {
        mode = CHANNEL_EDIT_DATA_LOGGER_USB1;
    } else if(valueId == VALUE_DATALOGGERSETTING_SETTINGLIST_USB2) {
        mode = CHANNEL_EDIT_DATA_LOGGER_USB2;
    }

    if (_applicationSettings->testMode() != ActionMode::PRODUCTION) {
        fullPathFilename = _applicationSettings->stubDataPathDataLogger() + "/";
        fullPathFilename += _applicationSettings->settingDataPathStubLogger() + "/" + filename;
    } else {
        fullPathFilename += filename;
    }

    if (_communicator->deleteUserLogSetFile(fullPathFilename, mode) != LC70_OK) {
        return false;
    }

    return true;
}

bool LevelCheckerModel::renameUserDataLoggerSettingFile(ChannelEditMode mode, const QString &oldFileName, const QString &newFileName)
{
    // USER データロガー設定チャンネル編集モードからパスを選択
    QString path;
    if(mode == CHANNEL_EDIT_USER) {
        path = _applicationSettings->settingDataPathUserData();
    } else if(mode == CHANNEL_EDIT_USER_USB1) {
        path = _applicationSettings->settingDataPathUserDataUsb1();
    } else if(mode == CHANNEL_EDIT_USER_USB2) {
        path = _applicationSettings->settingDataPathUserDataUsb2();
    } else if(mode == CHANNEL_EDIT_DATA_LOGGER) {
        path = _applicationSettings->settingDataPathLogger();
    } else if(mode == CHANNEL_EDIT_DATA_LOGGER_USB1) {
        path = _applicationSettings->settingDataPathLoggerUsb1();
    } else if(mode == CHANNEL_EDIT_DATA_LOGGER_USB2) {
        path = _applicationSettings->settingDataPathLoggerUsb2();
    } else if(mode == CHANNEL_EDIT_SELECT_DESTINATION) {
        path = _applicationSettings->settingDataPathUserData();
    } else {
        return false;
    }

    // リネーム
    QString command = QString("mv %1/%2 %3/%4").arg(path).arg(oldFileName).arg(path).arg(newFileName);
    int exitCode = QProcess::execute(command);
    QProcess::execute(SYNC_COMMAND);

    if (exitCode == -1 || exitCode == -2) {
        return false;
    }

    // バックアップファイルをリネーム
    command = QString("mv %1/%2 %3/%4").arg(path).arg(oldFileName + QString(BACKUP_FILE_EXTENSION)).arg(path).arg(newFileName + QString(BACKUP_FILE_EXTENSION));
    QProcess::execute(command);
    QProcess::execute(SYNC_COMMAND);

    return true;
}

bool LevelCheckerModel::removeUserDataLoggerBackupFile(ChannelEditMode mode, const QString &fileName, bool deleteAll)
{
    // USER データロガー設定チャンネル編集モードからパスを選択
    QString path;
    if(mode == CHANNEL_EDIT_USER) {
        path = _applicationSettings->settingDataPathUserData();
    } else if(mode == CHANNEL_EDIT_USER_USB1) {
        path = _applicationSettings->settingDataPathUserDataUsb1();
    } else if(mode == CHANNEL_EDIT_USER_USB2) {
        path = _applicationSettings->settingDataPathUserDataUsb2();
    } else if(mode == CHANNEL_EDIT_DATA_LOGGER) {
        path = _applicationSettings->settingDataPathLogger();
    } else if(mode == CHANNEL_EDIT_DATA_LOGGER_USB1) {
        path = _applicationSettings->settingDataPathLoggerUsb1();
    } else if(mode == CHANNEL_EDIT_DATA_LOGGER_USB2) {
        path = _applicationSettings->settingDataPathLoggerUsb2();
    } else if(mode == CHANNEL_EDIT_SELECT_DESTINATION) {
        path = _applicationSettings->settingDataPathUserData();
    } else {
        return false;
    }

    // バックアップファイルを削除
    QString command;
    if (deleteAll) {
        command = QString("rm -f %1/*%2").arg(path).arg(QString(BACKUP_FILE_EXTENSION));
    } else {
        command = QString("rm -f %1/%2").arg(path).arg(fileName);
    }
    int exitCode = QProcess::execute(command);
    QProcess::execute(SYNC_COMMAND);

    if (exitCode == -1 || exitCode == -2) {
        return false;
    }

    return true;
}

bool LevelCheckerModel::allRemoveDataLoggerUserSettingList(ValueId valueId)
{
    QString fullPathFilename;

    ChannelEditMode mode = CHANNEL_EDIT_USER;

    if(valueId == VALUE_USERSETTING_SETTINGLIST) {
        mode = CHANNEL_EDIT_USER;
    } else if(valueId == VALUE_USERSETTING_SETTINGLIST_USB1) {
        mode = CHANNEL_EDIT_USER_USB1;
    } else if(valueId == VALUE_USERSETTING_SETTINGLIST_USB2) {
        mode = CHANNEL_EDIT_USER_USB2;
    } else if(valueId == VALUE_DATALOGGERSETTING_SETTINGLIST) {
        mode = CHANNEL_EDIT_DATA_LOGGER;
    } else if(valueId == VALUE_DATALOGGERSETTING_SETTINGLIST_USB1) {
        mode = CHANNEL_EDIT_DATA_LOGGER_USB1;
    } else if(valueId == VALUE_DATALOGGERSETTING_SETTINGLIST_USB2) {
        mode = CHANNEL_EDIT_DATA_LOGGER_USB2;
    }

    if (_communicator->allDeleteUserLogSetFile(mode) != LC70_OK) {
        return false;
    }

    return true;
}

bool LevelCheckerModel::getChannelDefinitionMap(QMap<int, QList<ChannelDefinitionItem> > &channelMap)
{
    return _modelOperator->getChannelDefinitionMap(channelMap);
}

bool LevelCheckerModel::getChannelDefinitionList(int bandId, QList<ChannelDefinitionItem> &channelList)
{
    return _modelOperator->getChannelDefinitionList(bandId, channelList);
}

bool LevelCheckerModel::getChannelDefinitionItem(int bandId, const QString &channel, ChannelDefinitionItem &channelItem)
{
    return _modelOperator->getChannelDefinitionItem(bandId, channel, channelItem);
}

bool LevelCheckerModel::setChannelDefinitionList(int bandId, QList<ChannelDefinitionItem> &channelList)
{
    return _modelOperator->setChannelDefinitionList(bandId, channelList);
}

bool LevelCheckerModel::getChannelDefinitionItemForAntennaAdjustment(int bandId, const QString& channel, ChannelDefinitionItem &channelItem)
{
    return _modelOperator->getChannelDefinitionItemForAntennaAdjustment(bandId, channel, channelItem);
}

bool LevelCheckerModel::setChannelDefinitionItem(int bandId, const QString &channel, ChannelDefinitionItem &channelItem)
{
    return _modelOperator->setChannelDefinitionItem(bandId, channel, channelItem);
}

bool LevelCheckerModel::commitAllChannelDefinitionData()
{
    return _modelOperator->commitAllChannelDefinitionData();
}

bool LevelCheckerModel::selectChannelDefinitionList(int bandId, QString polarization,
        QList<ChannelDefinitionItem> &channelList)
{
    return _modelOperator->selectChannelDefinitionList(bandId, polarization, channelList);
}

bool LevelCheckerModel::commitChannelDefinitionData(int bandId)
{
    return _modelOperator->commitChannelDefinitionData(bandId);
}

bool LevelCheckerModel::startOpticalMeasurement(uint32_t ulWaveLength)
{
    if (!readyToMeasure()) {
        return false;
    }

    return _modelOperator->startOpticalMeasurement(ulWaveLength);
}

bool LevelCheckerModel::stopOpticalMeasurement()
{
    return _modelOperator->stopOpticalMeasurement();
}

bool LevelCheckerModel::getOpticalMeasurementResult(QVariant &result)
{
    return _modelOperator->getOpticalMeasurementResult(result);
}

bool LevelCheckerModel::startDCMeasurement()
{
    if (!readyToMeasure()) {
        return false;
    }

    return _modelOperator->startDCMeasurement();
}

bool LevelCheckerModel::stopDCMeasurement()
{
    return _modelOperator->stopDCMeasurement();
}

bool LevelCheckerModel::getDCMeasurementResult(QVariant &result)
{
    return _modelOperator->getDCMeasurementResult(result);
}
void LevelCheckerModel::addDataLoggerUserSettingList(int lineno)
{
    _modelOperator->addDataLoggerUserSettingList(lineno);
}

void LevelCheckerModel::deleteDataLoggerUserSettingList(int lineno)
{
    _modelOperator->deleteDataLoggerUserSettingList(lineno);

}

#if 1
bool LevelCheckerModel::scan_bscs(QList<ChannelDefinitionItem> &requestChannelList, const AcquisitionTarget &acquisition,
                             const uint16_t &broadcastSystem, const uint16_t &modulation,
                             const bool isManual, const LC70_POWER_SUPPLY_T powerSupply)
{
    // サブ CPU が起動していなければ何もしない
    if (!readyToMeasure()) {
        return false;
    }

    int listSize = requestChannelList.size();

    static LC70_MEAS_MULTI_T condition;
    memset(&condition, 0, sizeof(LC70_MEAS_MULTI_T));

    condition.sRegNum = listSize;

    QVariant cnrMerSetting;
    getValue(VALUE_MER_CN, cnrMerSetting);

    for (int i = 0; i < listSize; i++) {
        unsigned int centerFrequency = Correcter::centerFrequency(requestChannelList[i], modulation);
        condition.single[i].ulFreq = centerFrequency;

        if(requestChannelList[i].polarization == 0) {      // 右旋
            condition.single[i].ulLoFreq = 10678000;
            condition.single[i].usBdSys = LC70_BD_SYS_ISDBS;
        }
        else if(requestChannelList[i].polarization == 1){	// 左旋
            condition.single[i].ulLoFreq = 9505000;
            condition.single[i].usBdSys = LC70_BD_SYS_ISDBS3;
        }

        condition.single[i].usBand = requestChannelList[i].band;

        if(requestChannelList[i].channelNumber =="BS-7") {
            condition.single[i].usBdSys = LC70_BD_SYS_ISDBS3;
        }
        else if(requestChannelList[i].channelNumber == "BS-17") {
            condition.single[i].usBdSys = LC70_BD_SYS_ISDBS3;
        }

        condition.single[i].usMod = 0;

        // MER/CN の使い分け
        if (Correcter::selectCnrMer(static_cast<LC70_BAND_T>(requestChannelList[i].band), cnrMerSetting) == LC70_MEAS_SET_SEL_CNR) {
            condition.single[i].usCondition = LC70_DATA_MSK_LEVEL | LC70_DATA_MSK_CNR;
        } else if (Correcter::selectCnrMer(static_cast<LC70_BAND_T>(requestChannelList[i].band), cnrMerSetting) == LC70_MEAS_SET_SEL_MER) {
            condition.single[i].usCondition = LC70_DATA_MSK_LEVEL | LC70_DATA_MSK_CNR | LC70_DATA_MSK_MER;
        } else {
            condition.single[i].usCondition = LC70_DATA_MSK_LEVEL;
        }

        // チャンネル番号文字列(NULL終端なし、8文字に足りない場合はスペースで埋める)
        memset(condition.single[i].ucStrChan, ' ', CHANNEL_NUMBER_LENGTH);

        for (int j = 0; j < CHANNEL_NUMBER_LENGTH; ++j) {
            if (j <= requestChannelList[i].channelNumber.toLocal8Bit().length() - 1) {
                condition.single[i].ucStrChan[j] = requestChannelList[i].channelNumber.toLocal8Bit()[j];
            }
        }

        int length = strnlen((char *)condition.single[i].ucStrChan, sizeof condition.single[i].ucStrChan);
        if (length < CHANNEL_NUMBER_LENGTH) {
            for (int j = length; j < CHANNEL_NUMBER_LENGTH; ++j) {
                condition.single[i].ucStrChan[j] = ' ';
            }
        }

        // 給電有効/無効設定
        if (isManual) {
            // マニュアル測定の場合 無効 : 0を設定
            condition.single[i].ucOutputEnb = 0;
        } else {
            // ユーザー/ロガー、及びユーザー/ロガー設定のSCAN時 有効 : 1を設定
            condition.single[i].ucOutputEnb = 1;
        }

        // 給電設定
        condition.single[i].powerSupply = powerSupply;
    }

    // スキャン開始
    int interval = _applicationSettings->pollingIntervalScan();
    emit resumeScanning(&condition, acquisition, interval);
    return true;
}
#endif
bool LevelCheckerModel::scan(QList<ChannelDefinitionItem> &requestChannelList, const AcquisitionTarget &acquisition,
                             const uint32_t &localFrequency, const uint16_t &broadcastSystem, const uint16_t &modulation,
                             const bool isManual, const LC70_POWER_SUPPLY_T powerSupply)
{
    // サブ CPU が起動していなければ何もしない
    if (!readyToMeasure()) {
        return false;
    }

    int listSize = requestChannelList.size();

    static LC70_MEAS_MULTI_T condition;
    memset(&condition, 0, sizeof(LC70_MEAS_MULTI_T));

    condition.sRegNum = listSize;

    QVariant cnrMerSetting;
    getValue(VALUE_MER_CN, cnrMerSetting);

    for (int i = 0; i < listSize; i++) {
        unsigned int centerFrequency = Correcter::centerFrequency(requestChannelList[i], modulation);
        condition.single[i].ulFreq = centerFrequency;
        condition.single[i].ulLoFreq = localFrequency;

        condition.single[i].usBand = requestChannelList[i].band;
        condition.single[i].usBdSys = broadcastSystem;
        condition.single[i].usMod = modulation;

        // MER/CN の使い分け
        if (Correcter::selectCnrMer(static_cast<LC70_BAND_T>(requestChannelList[i].band), cnrMerSetting) == LC70_MEAS_SET_SEL_CNR) {
            condition.single[i].usCondition = LC70_DATA_MSK_LEVEL | LC70_DATA_MSK_CNR;
        } else if (Correcter::selectCnrMer(static_cast<LC70_BAND_T>(requestChannelList[i].band), cnrMerSetting) == LC70_MEAS_SET_SEL_MER) {
            condition.single[i].usCondition = LC70_DATA_MSK_LEVEL | LC70_DATA_MSK_CNR | LC70_DATA_MSK_MER;
        } else {
            condition.single[i].usCondition = LC70_DATA_MSK_LEVEL;
        }

        // チャンネル番号文字列(NULL終端なし、8文字に足りない場合はスペースで埋める)
        memset(condition.single[i].ucStrChan, ' ', CHANNEL_NUMBER_LENGTH);

        for (int j = 0; j < CHANNEL_NUMBER_LENGTH; ++j) {
            if (j <= requestChannelList[i].channelNumber.toLocal8Bit().length() - 1) {
                condition.single[i].ucStrChan[j] = requestChannelList[i].channelNumber.toLocal8Bit()[j];
            }
        }

        int length = strnlen((char *)condition.single[i].ucStrChan, sizeof condition.single[i].ucStrChan);
        if (length < CHANNEL_NUMBER_LENGTH) {
            for (int j = length; j < CHANNEL_NUMBER_LENGTH; ++j) {
                condition.single[i].ucStrChan[j] = ' ';
            }
        }

        // 給電有効/無効設定
        if (isManual) {
            // マニュアル測定の場合 無効 : 0を設定
            condition.single[i].ucOutputEnb = 0;
        } else {
            // ユーザー/ロガー、及びユーザー/ロガー設定のSCAN時 有効 : 1を設定
            condition.single[i].ucOutputEnb = 1;
        }

        // 給電設定
        condition.single[i].powerSupply = powerSupply;
    }

    // スキャン開始
    int interval = _applicationSettings->pollingIntervalScan();
    emit resumeScanning(&condition, acquisition, interval);
    return true;
}

void LevelCheckerModel::stopChannelScan()
{
    emit suspendScanning();
}

void LevelCheckerModel::writeLog(const QString &sender, const QString &message, int priority)
{
    _systemLog.write(sender, message, priority);
}

void LevelCheckerModel::getBroadbandSystem(LC70_BAND_T bandId, QList<int> &broadbandSystemList)
{
    broadbandSystemList.clear();

    switch (bandId) {
    case LC70_BAND_UHF:
        broadbandSystemList.append(LC70_BD_SYS_ISDBT);
        break;

    case LC70_BAND_BS:
    case LC70_BAND_CS:
        broadbandSystemList.append(LC70_BD_SYS_ISDBS);
        broadbandSystemList.append(LC70_BD_SYS_ISDBS3);
        break;

    case LC70_BAND_JCSAT3:
    case LC70_BAND_JCSAT4:
    case LC70_BAND_SCCB:
    case LC70_BAND_SCCC:
        broadbandSystemList.append(LC70_BD_SYS_DVBS);
        broadbandSystemList.append(LC70_BD_SYS_DVBS2);
        break;

    // ラッパー関数ライブラリ V2.00 対応
    case LC70_BAND_CATV_ALL:
    case LC70_BAND_CATV_90_470MHZ:
    case LC70_BAND_CATV_470_1002MHZ:
        broadbandSystemList.append(LC70_BD_SYS_ISDBT);
        broadbandSystemList.append(LC70_BD_SYS_ANNEXC);
        broadbandSystemList.append(LC70_BD_SYS_ANNEXB);
        break;

    case LC70_BAND_PILOT:
        broadbandSystemList.append(LC70_BD_SYS_NONE);
        break;

    case LC70_BAND_SKYP:
        broadbandSystemList.append(LC70_BD_SYS_ISDBT);
        broadbandSystemList.append(LC70_BD_SYS_OPTICAST);
        break;

    case LC70_BAND_LTE:
        broadbandSystemList.append(LC70_BD_SYS_NONE);
        break;

    case LC70_BAND_FM:
        broadbandSystemList.append(LC70_BD_SYS_NONE);
        break;

    case LC70_BAND_CATV_UP_STREAM:
        broadbandSystemList.append(LC70_BD_SYS_NONE);
        break;

    default:
        break;
    }
}

unsigned int LevelCheckerModel::getModulationBit(LC70_BAND_T bandId, int broadbandSystem)
{
    static const QMap<LC70_BAND_T, QPair<QList<int>, unsigned int>> irregularMap = {
        {LC70_BAND_PILOT,            {{LC70_BD_SYS_NONE},  LC70_SETMOD_CW   }},
        {LC70_BAND_LTE,              {{LC70_BD_SYS_NONE},  LC70_SETMOD_UNDEF }},
        {LC70_BAND_FM,               {QList<int>(),        LC70_SETMOD_FM   }},
        {LC70_BAND_CATV_UP_STREAM,   {{LC70_BD_SYS_NONE},  LC70_SETMOD_CW   }}
    };

    if (irregularMap.count(bandId) != 0) {
        QList<int> broadcastSystemList = irregularMap.value(bandId).first;

        if (broadcastSystemList.size() == 0 || broadcastSystemList.contains(broadbandSystem)) {
            return irregularMap.value(bandId).second;
        }
    }

    unsigned int modulationBit = 0x0;

    switch (broadbandSystem) {
    case LC70_BD_SYS_ISDBT:
        modulationBit |= LC70_SETMOD_OFDM;
        break;

    case LC70_BD_SYS_ISDBS:
        modulationBit |= LC70_SETMOD_TC8PSK;
        modulationBit |= LC70_SETMOD_QPSK;
        modulationBit |= LC70_SETMOD_BPSK;
        break;

    case LC70_BD_SYS_ISDBS3:
        modulationBit |= LC70_SETMOD_8PSK;
        modulationBit |= LC70_SETMOD_QPSK;
        modulationBit |= LC70_SETMOD_BPSK;
        modulationBit |= LC70_SETMOD_16APSK;
        modulationBit |= LC70_SETMOD_32APSK;
        break;

    case LC70_BD_SYS_DVBS:
        modulationBit |= LC70_SETMOD_QPSK;
        break;

    case LC70_BD_SYS_OPTICAST:
        modulationBit |= LC70_SETMOD_256QAM;
        break;

    case LC70_BD_SYS_DVBS2:
        modulationBit |= LC70_SETMOD_8PSK;
        modulationBit |= LC70_SETMOD_QPSK;
        modulationBit |= LC70_SETMOD_16APSK;
        modulationBit |= LC70_SETMOD_32APSK;
        break;

    case LC70_BD_SYS_ANNEXC :
        modulationBit |= LC70_SETMOD_64QAM;
        modulationBit |= LC70_SETMOD_256QAM;
        break;

    case LC70_BD_SYS_ANNEXB:
        modulationBit |= LC70_SETMOD_64QAM;
        modulationBit |= LC70_SETMOD_256QAM;
        break;

    default:
        break;
    }

    return modulationBit;
}

bool LevelCheckerModel::setPowerSupply(LC70_POWER_SUPPLY_T outputVoltage)
{
    static bool powerSupplyOff = true;
    if (powerSupplyOff && outputVoltage == LC70_POWER_SUPPLY_OFF) {
        return LC70_OK;
    }

    LC70_ERR_T ret = _communicator->setPowerSupply(outputVoltage);
    if (outputVoltage == LC70_POWER_SUPPLY_OFF) {
        powerSupplyOff = true;
    } else {
        powerSupplyOff = false;
    }

    return (ret == LC70_OK);
}

LC70_ERR_T LevelCheckerModel::setPowerSupplyForMultiBand(const uint8_t &value)
{
    return _communicator->setPowerSupplyForMultiBand(value);
}

bool LevelCheckerModel::removePreservedData(const QString &targetFilePath, bool isDirectory)
{
    if (targetFilePath.isEmpty()) {
        qDebug() << "Occur targetFilePath is empty";
        return false;
    }

    if (isDirectory) {
        // 削除対象がディレクトリの場合
        if (!QDir(targetFilePath).exists()) {
            qDebug() << "Occur targetFilePath is not exist";
            return false;
        }

        // 削除対象のファイルを削除する
        if (!QDir(targetFilePath).removeRecursively()) {
            qDebug() << "Occur targetFile remove Error";
            return false;
        }

        // 削除したフォルダを再度作成する
        if (!QDir(QCoreApplication::applicationDirPath()).mkdir(targetFilePath)) {
            qDebug() << "Occur targetFile make Error";
            return false;
        }

        // 同期
        QProcess::execute(SYNC_COMMAND);
    } else {
        // 削除対象がファイルの場合
        if (!QFile(targetFilePath).exists()) {
            qDebug() << "Occur targetFilePath is not exist";
            return false;
        }

        // 削除対象のファイルを削除する
        if (!QFile(targetFilePath).remove()) {
            qDebug() << "Occur targetFile remove Error";
            return false;
        }

        if (QFileInfo(targetFilePath).dir().isEmpty(QDir::AllEntries | QDir::System | QDir::NoDotAndDotDot | QDir::Hidden)) {
            // 削除したファイルのフォルダが空の場合 フォルダも削除する
            if (!QFileInfo(targetFilePath).dir().removeRecursively()){
                qDebug() << "Occur targetDirectory remove Error";
                return false;
            }
        }

        // シングル、マルチの場合は日付フォルダとCSVフォルダを削除するため、
        // 一つ上のフォルダの空も確認
        if ((targetFilePath.contains("single", Qt::CaseInsensitive) == true) ||
            (targetFilePath.contains("multi", Qt::CaseInsensitive) == true)) {
            QStringList targetPathList = targetFilePath.split("/");
            QString parentDirPath;
            for (int i = 1; i < targetPathList.size() - 1; i++) {
                parentDirPath = parentDirPath + "/" + targetPathList[i];
            }

            if (QFileInfo(parentDirPath).dir().isEmpty(QDir::AllEntries | QDir::System | QDir::NoDotAndDotDot | QDir::Hidden)) {
                // 削除したファイルのフォルダが空の場合 フォルダも削除する
                if (!QFileInfo(parentDirPath).dir().removeRecursively()){
                    qDebug() << "Occur targetDirectory remove Error";
                    return false;
                }
            }
        }

        // 同期
        QProcess::execute(SYNC_COMMAND);
    }
    return true;
}

void LevelCheckerModel::changeSoundVolumeIcon()
{
    // モデルオペレータから音量設定を取得し、シグナルを送信する。
    QVariant soundVolume;

    if (!getValue(VALUE_SOUNDVOLUME, soundVolume)) {
        // 音量設定の取得に失敗した場合は、何もしない
        qDebug() << "ERROR: failure get soundVolume";
        return;
    }

    // 音量の取得に成功した場合は、signalを送信する。
    emit soundVolumeSetting(soundVolume.toInt());
}

ActionMode LevelCheckerModel::actionMode() const
{
    return _actionMode;
}

QList<QFileInfo> LevelCheckerModel::dataLoggerFileList(void)
{
    if (_actionMode == PRODUCTION) {
        return UsbOperation::getUsbFileList(UsbDataType::DATA_LOGGER_RESULT);
    } else {
        return _stub->dataLoggerFileList(_applicationSettings->resultDataPathLogger());
    }
}

bool LevelCheckerModel::getThresholdInitialValue(QVariant &value)
{
    return _modelOperator->getThresholdInitializeValue(value);
}

bool LevelCheckerModel::setFaultInformation(uint16_t code, uint8_t status, const uint8_t *additionalInformation)
{
    return _faultDetector->setFaultInformation(code, status, additionalInformation);
}

LC70_DEV_STAT_T LevelCheckerModel::getDeviceStatus()
{
    return _systemWatcher->getDeviceStatus();
}

void LevelCheckerModel::initializeUserSettingFile()
{
    _modelOperator->initializeUserSettingFile();
}

QList<MeasureResult> LevelCheckerModel::allScanResult()
{
    return _scanResult;
}

MeasureResult LevelCheckerModel::scanResult(int index)
{
    if (index < 0 || _scanResult.count() <= index) {
        return MeasureResult();
    }

    return _scanResult[index];
}

void LevelCheckerModel::appendScanResult(MeasureResult measureResult)
{
    _scanResult.append(measureResult);
}

void LevelCheckerModel::clearScanResult()
{
    _scanResult.clear();
}

bool LevelCheckerModel::getModelInformation(LC70_PRODUCT_INFO_T &productinfo)
{
    LC70_ERR_T error;
    error = _communicator->getModelInformation(productinfo);

    bool result = false;

    if (error == LC70_OK) {
        result = true;
    }

    return result;
}

bool LevelCheckerModel::versionUpReady() const
{
    return _versionUpReady;
}

LC70_ERR_T LevelCheckerModel::shutDown()
{
    return _communicator->systemShutdown();
}

void LevelCheckerModel::requestStandBy(uint8_t mode)
{
    if (!_modelOperator->readyToStandBy()) {
        _systemLog.write(QString(typeid(*this).name()),
                     QString("[Retry Error]"), LOG_NOTICE);
        shutDown();
    }

    _standByMode = mode;
    onRequestStandBy();
}

void LevelCheckerModel::onRequestStandBy()
{
    if (_standByMode == MAIN_AND_SUB) {
        QFile file;
        file.setFileName(QString(STANDBY_LOCK_FILE_NAME));
        if (file.open(QIODevice::WriteOnly | QIODevice::Text)) {
            file.close();
        }
    }

    if (LC70_OK == _communicator->systemStandBy(_standByMode)) {
        // nothing to do
    }
}

LC70_ERR_T LevelCheckerModel::wakeUp(uint8_t mode)
{
    if (_updating) {
        return LC70_ERR;
    }

    if (!_modelOperator->readyToWakeUp()) {
        // サブ CPU エラーにする
        LC70_FAIL_INFO_T errorInformation;
        memset(&errorInformation, 0, sizeof(LC70_FAIL_INFO_T));
        errorInformation.usCode = LC70_FAIL_SUB_ERR;
        errorInformation.ucStat = LC70_FAIL_STAT_FAIL;
        errorInformation.ucCtl = LC70_FAIL_CTL_MSGDISP;
        memcpy(&errorInformation.ucAddInfo, "No Reply Sub CPU", 16);
        emit notifyErrorOccured(errorInformation);
        return LC70_ERR;
    }

    LC70_ERR_T ret = _communicator->systemWakeup(mode);

    // MAIN, SUB  ともウェイクアップなら自動パワーオフのタイマーを開始する
    if (mode == 0) {
        if (_autoPowerOffTimer) {
            // 自動パワーオフの設定がオフならタイマーを開始しない
            if (_autoPowerOffTimer->interval() > 0) {
                _autoPowerOffTimer->start();
            }
        }
    }

    return ret;
}

bool LevelCheckerModel::wakeUpRequested() const
{
    return _wakeUpRequested;
}

void LevelCheckerModel::setWakeUpRequested(bool wakeUpRequested)
{
    _wakeUpRequested = wakeUpRequested;
}

void LevelCheckerModel::updateProgram(void)
{
    _updating = true;

    // 自動パワーオフのタイマーを止める
    if (_autoPowerOffTimer) {
        _autoPowerOffTimer->stop();
    }

    // ソフトウェア更新要求
    emit notifySoftwareUpdate(_updater);
}

void LevelCheckerModel::changeDisplayType(LC70_DISP_TYPE_T displayType)
{
    _measurer->changeDisplayType(displayType);
}

void LevelCheckerModel::getUpdateVersion(QString &version)
{
    version.sprintf("%s", _updater.ucMainVer);
}
