/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef LEVELCHECKERMODEL_H
#define LEVELCHECKERMODEL_H

#include <QObject>
#include <QQmlApplicationEngine>
#include <QQmlContext>
#include <QElapsedTimer>
#include <QTimer>
#include <QFileSystemModel>
#include "objectsearcher.h"
#include "communicator.h"
#include "stub.h"
#include "applicationsetting.h"
#include "channelsetting.h"
#include "systemwatcher.h"
#include "measurer.h"
#include "faultdetector.h"
#include "valueid.h"
#include "popupid.h"
#include "systemlog.h"
#include "usboperation.h"
#include "ziphelper.h"

class ModelOperator;

class LevelCheckerModel : public QObject
{
    Q_OBJECT
public:
    /**
     * @brief コンストラクター
     * @param engine QML のインスタンスを作成するクラスのポインター
     * @param parent 親オブジェクト
     */
    explicit LevelCheckerModel(QQmlApplicationEngine *engine, QObject *parent = nullptr);

    /**
     *  @brief デストラクター
     */
    ~LevelCheckerModel();

    /**
     * @brief 実行前の準備を行う
     * @param timer 処理時間計測用のタイマー
     * @return
     * @note ラッパー関数を呼び出すかスタブを使うかを判断する
     */
    LC70_ERR_T prepare(QElapsedTimer *timer);

    /**
     * @brief qml オブジェクト検索
     * @param engine Qml ApplicationEngine
     * @param objectName オブジェクト名
     */
    QObject *search(const QString &objectName);

    /**
     * @brief アプリケーション固有の情報クラスのインスタンスを返す
     * @return アプリケーション固有の情報クラスのインスタンス
     */
    ApplicationSetting *applicationSetting(void) const;

    /**
     * @brief 測定結果取得対象の変更
     * @param acquisitionTarget 測定結果取得対象
     */
    void changeAcquisitionTarget(const AcquisitionTarget &acquisitionTarget);

    /**
     * @brief 測定結果取得対象を返す
     * @return 測定結果取得対象
     */
    AcquisitionTarget acquisitionTarget(void) const;

    /**
     * @brief 保存先のディレクトリがなければ作成する
     * @param saveFileType PNG/CSV
     * @param measureType 測定種類
     * @return 保存先のディレクトリのパス(作成失敗時は空)
     */
    QString createDirectory(SaveFileType saveFileType, MeasureType measureType);

    /**
     * @brief シングルチャンネル測定結果の出力
     * @param fileName ファイルのパス
     * @param singleMeasureResult 測定結果
     * @param condition 測定条件
     * @return true: 出力成功 false: 出力失敗
     */
    bool writeSingleMeasurementData(QString fileName, uint16_t condition, const MeasureResult &singleMeasureResult);

    /**
     * @brief マルチチャンネル測定結果の出力
     * @param fileName ファイルのパス
     * @param condition 測定条件
     * @param multiMeasureResult 測定結果
     * @return true: 出力成功 false: 出力失敗
     */
    bool writeMultiMeasurementData(QString fileName, uint16_t condition, const QList<MeasureResult> &multiMeasureResult);

    /**
     * @brief シングル測定結果画面の出力
     * @param fileName ファイルのパス
     * @param image 画面キャプチャーデータ
     * @return true: 出力成功 false: 出力失敗
     */
    bool writeSingleMeasurementScreen(QString fileName, QImage &image);

    /**
     * @brief 測定データファイル名リストを更新
ｖ     * @return リストの更新成否
     */
    bool updateMeasurementFileList(void);

    /**
     * @brief メニューリストに追加したアイコンを削除
     * @param index　インデックス
     */
    void removeMenuList(const MenuListIconIndex index);

    /**
     * @brief 指定したインデックスのメニューリストに対してアイコンの置き換えを実施
     * @param index 置き換えを対象のインデックス
     * @param iconName 置き換えを行うアイコン名
     */
    void replaceMenuList(const MenuListIconIndex index, const QString &iconName);

    /**
     * @brief 測定する Band のアイコンを表示
     * @param band Band
     */
    void replaceBandIcon(const LC70_BAND_T band);

    /**
     * @brief メニューリストをクリアする
     */
    void clearMenuList();

    // 設定値操作用メソッド
    /**
     * @brief getValue modelOperatorで保持している値を取得する
     * @param valueId 設定値等の情報の識別ID
     * @param value 取得した値[out]
     * @return 処理の成否 (true: 成功, false: 失敗)
     */
    bool getValue(ValueId valueId, QVariant &value);

    /**
     * @brief setValue
     * @param valueId
     * @param value
     * @return
     */
    bool setValue(ValueId valueId, const QVariant &value);

    /**
     * @brief setInitialValue
     * @param valueId
     * @param value
     * @return
     */
    bool setInitialValue(ValueId valueId, const QVariant &value);

    /**
     * @brief isUpdate
     * @param valueId
     * @return
     */
    bool isUpdate(ValueId valueId);

    /**
     * @brief rollback
     * @return
     */
    bool rollback(ValueId valueId);

    /**
     * @brief commit
     * @return
     */
    bool commit(ValueId valueId);

    /**
     * @brief executeInitializeSetting
     * 初期化（工場出荷状態）に戻す処理を実行する
     *
     * @return
     */
    bool executeInitializeSetting();

    /**
     * @brief 設定に合わせて お試し鳴動を行う
     * @param volumeLevel 音量レベル
     * @param buzzerPattern ブザーパターン
     * @return 処理成否
     */
    bool fourceBuzzerVolume(int volumeLevel, LC70_BUZZ_T buzzerPattern);

    /**
     * @brief buzzer ブザーを鳴動させる
     * @param buzzerPattern ブザー鳴動パターン
     * @param buzzerMode ブザーモード
     * @param forcedRinging 強制鳴動(音量 OFF でも鳴らす)
     * @return 処理成否
     */
    bool buzzer(int buzzerPattern, int buzzerMode, bool forcedRinging = false);

    /**
     * @brief 指定された明るさで 明るさ設定を行う(お試し用)
     * @param brightnessLebel 明るさレベル
     * @return 処理結果
     */
    bool setScreenBrightness(int brightnessLebel);

    /**
     * @brief importDataLoggerUserSettingList
     * データロガー ユーザ設定リストをファイルから読み込み保持する
     *
     * @param valueId
     * @param filename
     * @return 読み込み成功 true 失敗 false
     */
    bool importDataLoggerUserSettingList(ValueId valueId, const QString &filename);

    /**
     * @brief exportDataLoggerUserSettingList
     * データロガー ユーザ設定リストをファイルに書き出す
     *
     * @param valueId
     * @param filename
     * @return 書き出し成功 true 失敗 false
     */
    bool exportDataLoggerUserSettingList(ValueId valueId, const QString &filename);

    /**
     * @brief deleteDataLoggerUserSettingList
     * データーロガー　ユーザ設定リストを削除する
     *
     * @param valueId
     * @param filename
     * @return
     */
    bool removeDataLoggerUserSettingList(ValueId valueId, const QString &filename);

    /**
     * @brief USER データロガー設定ファイルをリネームする
     * @param mode 編集モード
     * @param oldFileName 旧ファイル名
     * @param newFileName 新ファイル名
     * @return リネームの成否
     */
    bool renameUserDataLoggerSettingFile(ChannelEditMode mode, const QString &oldFileName, const QString &newFileName);

    /**
     * @brief USER データロガー設定ファイルのバックアップを削除する
     * @param mode 編集モード
     * @param fileName ファイル名
     * @param deleteAll 一括削除
     * @return 削除の成否
     */
    bool removeUserDataLoggerBackupFile(ChannelEditMode mode, const QString &fileName, bool deleteAll = false);

    /**
     * @brief deleteDataLoggerUserSettingList
     * データーロガー　ユーザ設定リストを一括削除する
     *
     * @param valueId
     * @return
     */
    bool allRemoveDataLoggerUserSettingList(ValueId valueId);

    /**
     * @brief チャンネル定義情報の MAP を取得する
     * @param channelMap[out] 読み込んだチャンネル定義情報のmap (out)
     * @return 取得の成否
     */
    bool getChannelDefinitionMap(QMap<int, QList<ChannelDefinitionItem>> &channelMap);

    /**
     * @brief 引数で指定された BAND 種別のチャンネル定義情報のリストを返す
     * @param bandId 取得したいBAND種別（LC70_BAND_T)
     * @param channelList[out] BAND種別に該当するチャンネル定義リスト(out)
     * @return 取得の成否
     */
    bool getChannelDefinitionList(int bandId, QList<ChannelDefinitionItem> &channelList);

    /**
     * @brief 引数で指定された BAND 種別 + チャネル名に合致するのチャンネル定義情報を返す
     * @param bandId 取得したいBAND種別（LC70_BAND_T)
     * @param channel 取得したいチャネル名称
     * @param channelItem[out] 合致したチャネル定義情報（out)
     * @return
     */
    bool getChannelDefinitionItem(int bandId, const QString &channel, ChannelDefinitionItem &channelItem);

    /**
     * @brief 引数で指定した BAND種別に合致するチャンネル定義リストを引数のチャンネルリスト定義情報の値に設定する。
     * @param bandId 設定したいBAND種別(LC70_BAND_T)
     * @param channelList 設定死体チャンネル定義リスト
     * @return 処理の成否 True: 成功, False: 失敗
     */
    bool setChannelDefinitionList(int bandId, QList<ChannelDefinitionItem> &channelList);

    /**
     * @brief アンテナ調整のチャンネル定義情報を返す
     * @param bandId JCSAT-3A/4B
     * @param channel 取得したいチャネル名称
     * @param channelItem[out] アンテナ調整のチャネル定義情報（out)
     * @return
     */
    bool getChannelDefinitionItemForAntennaAdjustment(int bandId, const QString &channel, ChannelDefinitionItem &channelItem);

    /**
     * @brief 引数で指定した BAND 種別 + チャンネル名に合致するチャンネル定義情報に対して引数のチャンネル定義情報の値を設定する
     * @param bandId 設定したいBAND種別 (LC70_BAND_T)
     * @param channel 設定したいチャンネル名称
     * @param channelItem 設定したいチャンネル定義
     * @return 処理の成否 True: 成功, False: 失敗
     */
    bool setChannelDefinitionItem(int bandId, const QString &channel, ChannelDefinitionItem &channelItem);

    /**
     * @brief メモリで保持しているチャンネル定義情報の全ての設定を各チャンネル定義ファイルに書き出す
     * @return 処理の成否 True: 成功, False: 失敗
     */
    bool commitAllChannelDefinitionData(void);

    /**
     * @brief 選択した偏波に属さないチャンネル定義を取り除いたリストを返す
     * @param bandId 取得したいBAND種別（LC70_BAND_T)
     * @param polarization 選択した偏波(0: 右旋 1: 左旋)
     * @param channelList 選択した偏波に属さないチャンネル定義を取り除いたリスト
     * @return 処理の成否 True: 成功, False: 失敗
     */
    bool selectChannelDefinitionList(int bandId, QString polarization, QList<ChannelDefinitionItem> &channelList);

    /**
     * @brief 引数で指定した BAND種別に合致するチャンネル定義種別の設定をチャンネル定義ファイルに書き出す
     * @param bandId 設定したいBAND種別 (LC70_BAND_T)
     * @return 処理の成否 True: 成功, False: 失敗
     */
    bool commitChannelDefinitionData(int bandId);

    /**
     * @brief 光測定を開始する
     * @param ulWaveLength 波長[nm]
     * @return 処理の成否 True: 成功, False: 失敗
     */
    bool startOpticalMeasurement(uint32_t ulWaveLength);

    /**
     * @brief 光測定を停止する
     * @return 処理の成否 True: 成功, False: 失敗
     */
    bool stopOpticalMeasurement(void);

    /**
     * @brief 光測定の結果を取得
     * @param result[out] 光測定の結果
     * @return 処理の成否 True: 成功, False: 失敗
     */
    bool getOpticalMeasurementResult(QVariant &result);

    /**
     * @brief DC測定を開始する
     * @return 処理の成否 True: 成功, False: 失敗
     */
    bool startDCMeasurement(void);

    /**
     * @brief DC測定を停止する
     * @return 処理の成否 True: 成功, False: 失敗
     */
    bool stopDCMeasurement(void);

    /**
     * @brief DC測定の結果を取得
     * @param result[out] DC測定の結果
     * @return 処理の成否 True: 成功, False: 失敗
     */
    bool getDCMeasurementResult(QVariant &result);

    /**
     * @brief データロガー新規追加処理
     * 引数で指定された行に対する設定情報を追加する
     * すでに存在している場合にはなにもしない
     *
     * @param lineno
     */
    void addDataLoggerUserSettingList(int lineno);

    /**
     * @brief データロガーリスト削除
     * @param lineno 削除対象行
     */
    void deleteDataLoggerUserSettingList(int lineno);

    /**
     * @brief 有効なチャンネル情報のスキャン開始
     * @param reqestChannelList 指定されたBANDのチャンネル定義
     * @param acquisition 測定結果取得対象
     * @param localFrequency 局発周波数
     * @param broadcastSystem 放送方式
     * @param modulation 変調方式
     * @param isManual マニュアルかどうか(true: マニュアル, false: User設定)
     * @param dcOutSetting 給電電圧(DC OUT)設定
     */
    bool scan(QList<ChannelDefinitionItem> &reqestChannelList, const AcquisitionTarget &acquisition = NOT_ACQUISITION,
              const uint32_t &localFrequency = 0, const uint16_t &broadcastSystem = 0, const uint16_t &modulation = 0,
              const bool isManual = false, const LC70_POWER_SUPPLY_T powerSupply = LC70_POWER_SUPPLY_OFF);

    /**
     * @brief アップデートファイルのバージョンを返す
     * @param version 取得したいバージョン
     */
    void getUpdateVersion(QString &version);

#if 1
    /**
     * @brief 有効なチャンネル情報のスキャン開始 (BS_CS統合モード用)
     * @param reqestChannelList 指定されたBANDのチャンネル定義
     * @param acquisition 測定結果取得対象
     * @param broadcastSystem 放送方式
     * @param modulation 変調方式
     * @param isManual マニュアルかどうか(true: マニュアル, false: User設定)
     * @param dcOutSetting 給電電圧(DC OUT)設定
     */
    bool scan_bscs(QList<ChannelDefinitionItem> &reqestChannelList, const AcquisitionTarget &acquisition = NOT_ACQUISITION,
              const uint16_t &broadcastSystem = 0, const uint16_t &modulation = 0,
              const bool isManual = false, const LC70_POWER_SUPPLY_T powerSupply = LC70_POWER_SUPPLY_OFF);
#endif

    /**
     * @brief 有効なチャンネル情報のスキャン停止
     */
    void stopChannelScan();

    /**
     * @brief ログ出力する
     * @param sender ログ出力したクラス名
     * @param message ログに出力する文字列
     * @param priority メッセージの優先度
     * @note メッセージの優先度は syslog の level と同じ
     */
    void writeLog(const QString &sender, const QString &message, int priority = LOG_DEBUG);

    /**
     * @brief 放送方式（放送規格）を返す
     * @param bandId
     * @param bandSystemList
     */
    void getBroadbandSystem(LC70_BAND_T bandId, QList<int> &broadbandSystemList);

    /**
     * @brief 放送方式で使用できる変調方式のBitを返す
     * @param bandId Band の ID
     * @param broadbandSystem 放送方式
     * @return 変調方式のBit表現
     */
    unsigned int getModulationBit(LC70_BAND_T bandId, int broadbandSystem);

    /**
     * @brief 給電設定
     * @param outputVoltage 出力電圧
     * @return 処理結果
     */
    bool setPowerSupply(LC70_POWER_SUPPLY_T outputVoltage);

    /**
     * @brief マルチ測定で複数 Band を跨って測定する場合向けの給電の設定を行う
     * @param value 0: OFF 1: ON
     * @return 給電の可否
     */
    LC70_ERR_T setPowerSupplyForMultiBand(const uint8_t &value);

    /**
     * @brief 指定されたファイルに保持されているデータを削除する
     * @param targetFilePath 削除対象ファイルのパス
     * @param isDirectory 削除対象がディレクトリかどうか
     * @return 処理結果
     */
    bool removePreservedData(const QString &targetFilePath, bool isDirecotry);

    /**
     * @brief 音量設定アイコンを変更する
     */
    void changeSoundVolumeIcon(void);

    /**
     * @brief 動作モードを返す
     * @return 動作モード
     */
    ActionMode actionMode(void) const;

    /**
     * @brief USB メモリーからデータロガーのファイル一覧を返す
     * @return USB メモリーからデータロガーのファイル一覧
     */
    QList<QFileInfo> dataLoggerFileList(void);

    /**
     * @brief 閾値設定の初期値を取得する
     * @param value 閾値設定の初期値[out]
     * @param selectedBand 選択BAND
     * @param selectedBroadcast 選択
     * @return 処理の成否
     */
    bool getThresholdInitialValue(QVariant &value);

    /**
     * @brief エラー情報の設定
     * @param code 故障コード
     * @param status 故障状態
     * @param additionalInformation 追加情報
     * @return
     */
    bool setFaultInformation(uint16_t code, uint8_t status, const uint8_t *additionalInformation);

    /**
     * @brief デバイス状態を取得する
     * @return デバイス状態
     */
    LC70_DEV_STAT_T getDeviceStatus(void);

    /**
     * @brief USER設定ファイルを初期化する
     */
    void initializeUserSettingFile(void);

    /**
     * @brief スキャン結果の一覧を返す
     * @return スキャン結果の一覧
     */
    QList<MeasureResult> allScanResult(void);

    /**
     * @brief インデックスに対応するスキャン結果を返す
     * @param index インデックス
     * @return インデックスに対応するスキャン結果
     */
    MeasureResult scanResult(int index);

    /**
     * @brief スキャン結果の追加
     * @param スキャン結果
     */
    void appendScanResult(MeasureResult measureResult);

    /**
     * @brief スキャン結果のクリアー
     */
    void clearScanResult(void);

    /**
     * @brief 機種情報の取得を行う
     * @param productinfo[out] 機種情報
     * @return 処理結果 True: 成功, False: 失敗
     */
    bool getModelInformation(LC70_PRODUCT_INFO_T &productinfo);

    /**
     * @brief バージョンアップの可否を返す
     * @return バージョンアップの可否
     */
    bool versionUpReady(void) const;

    /**
     * @brief プログラムのアップデートを行う
     */
    void updateProgram(void);

    /**
     * @brief シャットダウンする
     * @return シャットダウンの可否
     */
    LC70_ERR_T shutDown(void);

    /**
     * @brief スタンバイ要求（測定中は停止まで待ってスタンバイする）
     * @param mode 0: MAIN AND SUB 1: SUB ONLY
     */
    void requestStandBy(uint8_t mode);

    /**
     * @brief ウェイクアップ要求の呼び出し
     * @param mode 0: MAIN AND SUB 1: SUB ONLY
     * @return
     */
    LC70_ERR_T wakeUp(uint8_t mode);

    /**
     * @brief ウェイクアップ要求の有無を返す
     * @return ウェイクアップ要求の有無
     */
    bool wakeUpRequested(void) const;

    /**
     * @brief ウェイクアップ要求の有無を設定する
     * @param ウェイクアップ要求の有無
     */
    void setWakeUpRequested(bool wakeUpRequested);

    /**
     * @brief 画面種別を変更する
     * @param displayType 画面種別
     */
    void changeDisplayType(LC70_DISP_TYPE_T displayType);

    /**
     * @brief zip ファイルに圧縮
     * @param archiveFile zip ファイル名
     * @param fileList 圧縮するファイルのリスト
     * @param destination 圧縮先のパス
     * @param password パスワード
     * @return
     */
    bool createZip(const QString& archiveFile, const QVector<QStringRef> &fileList, const QString &destination, const QString &password);

    /**
     * @brief zip ファイルを展開
     * @param archiveFile zip ファイル名
     * @param destination 展開先のパス
     * @param password パスワード
     * @return
     */
    bool extractZip(const QString& archiveFile, const QString &destination, const QString &password);

    /**
     * @brief スタンバイ中か否かの設定を
     * @param standBy スタンバイ中か否か
     * @return
     */
    void setStandBy(const bool& standBy);

    /**
     * @brief スキャン後のチャンネル一覧を返す
     * @return スキャン後のチャンネル一覧
     */
    QList<ChannelDefinitionItem> scannedChannelList(void) const;

    /**
     * @brief スキャンした BAND の偏波を返す
     * @return スキャンした BAND の偏波
     */
    QString scannedPolatization(void) const;

    /**
     * @brief スキャン後のチャンネル一覧を設定する
     * @param channelList スキャン後のチャンネル一覧
     * @param polarization スキャンした BAND の偏波
     */
    void setScannedChannelList(const QList<ChannelDefinitionItem> &channelList, const QString &polarization);

    /**
     * @brief スキャン後のチャンネル一覧をクリアーする
     */
    void clearScannedChannelList(void);

    /**
     * @brief 方位測定開始
     * @return 方位測定開始の成否
     */
    bool startAzimuth(void);

    /**
     * @brief 方位測定停止
     * @return 方位測定停止の成否
     */
    bool stopAzimuth(void);

    /**
     * @brief 処理時間計測用のタイマーを開始
     * @param message 出力するメッセージ
     */
    void startElapsedTimer(const QString &message);

    /**
     * @brief 経過時間を記録
     * @param message 出力するメッセージ
     */
    void recordElapsedTimer(const QString& message);

    /**
     * @brief デバイス状態の取得間隔変更
     * @param isCompass コンパス画面かどうか
     */
    void changeDeviceStatusInterval(bool isCompass);

    /**
     * @brief 時刻の初期化
     */
    void initDateTime();

    /**
     * @brief バッテリ低下状態変化通知
     * @param isLowBattery バッテリ低下状態が発生しているかどうか true: 発生している false:発生していない
     * @return 通知結果
     */
    bool notifyLowBatteryStatusChanged(bool isLowBattery);

    /**
     * @brief 測定状態を取得し、実行可能か否かを返す
     * @note 最長10秒(20ms周期、最大500回)間取得をリトライし、リトライオーバーでサブCPUエラーとする
     * @return true: 可 false: 不可
     */
    bool readyToMeasure(void);

    /**
     * @brief 工場出荷状態初期化要求
     * @return 工場出荷状態初期化要求の成否
     */
    bool initializeFactoryDefault(void);

    /**
     * @brief デベロッパーモード設定要求
     * @param selection 0: OFF 1: ON
     * @return デベロッパーモード設定要求の成否
     */
    bool setDeveloperMode(const uint8_t &selection);

    /**
     * @brief ファイルのツリー表示用モデルを設定する
     * @return ファイルのツリー表示用モデル
     */
    void setDisplayFileSystemModel(QFileSystemModel *model);

    /**
     * @brief ファイルのツリー表示用モデルを返す
     * @return ファイルのツリー表示用モデル
     */
    QFileSystemModel *displayFileSystemModel(void);

    /**
     * @brief 画面に対応する View(qml) がロードされていない場合の対応
     * @param screenName 画面名
     */
    void takeMeasureToFindNoView(const QString &screenName);

    /**
     * @brief サブ CPU の強制スタンバイ、ウェイクアップによるリセット
     * @return サブ CPU のリセット成否
     */
    LC70_ERR_T resetSubCpu(void);

#if 1
    /**
     * @brief オートパワーオフ設定を有効化　タイマーを再設定して開始
     */
    void enablePowerOffSetting(void);

    /**
     * @brief オートパワーオフ設定を無効化　タイマーを停止
     */
    void disablePowerOffSetting(void);

#endif

    /**
     * @brief USER データロガー設定ファイルのインポート
     * @param mode USER データロガー設定編集モード
     * @param fileName USER データロガー設定ファイル名
     * @return インポートの成否(データロガー設定編集モードが未定義なら不可)
     * @note ファイル名が空なら initialzie() メソッドを呼び出す
     */
    bool importSettingFile(ChannelEditMode mode, const QString &fileName);

    /**
     * @brief バッファーの初期化(既存データを破棄し、テンプレートファイル(user_datalogger_setting_template.csv)から値を設定する)
     * @return バッファーの初期化の成否
     */
    bool initializeUserDataLoggerSettingFile(ChannelEditMode mode);

    /**
     * @brief USER データロガー設定ファイルのエクスポート
     * @param mode USER データロガー設定編集モード
     * @param fileName USER データロガー設定ファイル名(新規作成時に指定)
     * @return エクスポートの成否
     */
    bool exportSettingFile(const ChannelEditMode &mode, const QString &fileName);

    /**
     * @brief USER データロガー設定ファイルの選択された Band のチャンネルの内容を返す
     * @param bandTabIndex 選択した Band のインデックス
     * @return USER データロガー設定ファイルの選択された Band のチャンネルの内容
     */
    QList<UserDataLoggerSettingFile> selectEditingChannels(const BandTabIndex &bandTabIndex);

    /**
     * @brief USER データロガー設定ファイルの全 Band から測定するが選択されたチャンネルの内容を返す
     * @return 測定するが選択された全チャンネルの内容
     */
    QList<UserDataLoggerSettingFile> selectMeasuringChannels(void);

    /**
     * @brief USER データロガー設定ファイルの Band ごとの測定するが選択されている行数
     * @param bandTabIndex 選択した Band のインデックス
     * @return USER データロガー設定ファイルの Band ごとの測定するが選択されている行数
     */
    int selectMeasuringRows(const BandTabIndex &bandTabIndex);

    /**
     * @brief 単一の Band のチャンネルのみを測定対象としているときにどの Band を選択しているかを返す
     * @return 選択されいている Band
     */
    BandTabIndex selectGroupPrefix(void);

    /**
     * @brief 設定ファイル中の最初に測定するチャンネルの Band のタブのインデックスとチャンネル名を返す
     * @param bandTabIndex 選択した Band のタブのインデックス
     * @param channel チャンネル名
     */
    void selectFirstValidChannel(BandTabIndex &bandTabIndex, QString &channel);

    /**
     * @brief 選択した Band のチャンネルの測定する、しないを変更
     * @param bandTabIndex 選択した Band のインデックス
     * @param channel チャンネル名
     * @param value 0: 測定しない 1: 測定する
     * @return 変更の成否(登録件数の条件を超える変更は不可)
     */
    bool editMeasuring(const BandTabIndex bandTabIndex, const QString &channel, const int &value);

    /**
     * @brief 選択した Band の給電 ON、OFF を変更
     * @param bandTabIndex 選択した Band のインデックス
     * @param value 0: 給電 OFF 1: 給電 ON
     */
    void editPowerSupply(const BandTabIndex bandTabIndex, const int &value);

    /**
     * @brief 選択した Band のチャンネルの放送方式を変更
     * @param bandTabIndex 選択した Band のインデックス
     * @param channel チャンネル名
     * @param value 0: ISDBT 3: ISDBC
     * @param scanned スキャンによる変更か否か
     * @return 変更の成否(CATV ALL 以外は不可。ISDBT, ISDBC 以外の設定は不可)
     * @note ISDBT のときは ISDBC の周波数[KHz] に 143 を加えた値を周波数にする
     */
    bool editBroadcastSystem(const BandTabIndex bandTabIndex, const QString &channel, const int &value, bool scanned = false);

    /**
     * @brief 選択した Band の局発周波数[KHz]を変更
     * @param bandTabIndex 選択した Band のインデックス
     * @param value 局発周波数の値
     * @return 変更の成否(JCSAT-3A, JCSAT-4B, SCC-B, SCC-C 以外は変更不可。10678000, 11200000, 11300000 以外の設定は不可)
     * @note 表示は GHz 単位なので 1000 * 1000 で割った値になる
     */
    void editLocalFrequency(const BandTabIndex bandTabIndex, const int &value);
    /**
     * @brief 選択した Band のチャンネルの周波数を変更
     * @param bandTabIndex 選択した Band のインデックス
     * @param channel チャンネル名
     * @param value 0: 測定しない 1: 測定する
     * @return 変更の成否(CATV 上り、FM 以外は不可)
     */
    bool editFrequency(const BandTabIndex bandTabIndex, const QString &channel, const int &value);

    /**
     * @brief コピー/削除対象ファイルリストを /tmp に出力する
     * @param type 0: コピー 1: 削除
     * @param candidate コピー/削除対象ファイルのパスのリスト
     * @param destination コピー先のパス
     * @return ファイル出力の成否
     */
    bool writeCandidateFiles(const int &type, const QStringList& candidate, const QString &destination = QString(""));

    /**
     * @brief  削除対象ファイルの削除
     * @param path 以降の階層の空のディレクトリを削除するパス
     * @return 削除の成否
     * @note 事前に削除するファイルのパスを /tmp/remove_candidate.txt に出力しておくこと
     */
    bool removeMeasurementFiles(const QString &path);

    /**
     * @brief コピー対象ファイルのコピー
     * @return コピーの成否
     * @note 事前にコピーするファイルとコピー先のパスを /tmp/copy_candidate.txt に出力しておくこと
     */
    bool copyMeasurementFiles(void);

    /**
     * @brief 移動先のファイルのパスに付け替える
     * @param source 元のファイルのパス
     * @param destination 移動先のファイルのパス
     * @return 移動先のファイルのパス
     */
    QString replaceSourceToDestination(const QString &source, const QString &destination);

signals:
    /**
     * @brief ボタン入力状態を通知
     * @param keyStatus ボタン入力状態
     */
    void keyStatus(LC70_KEY_STAT_T keyStatus);

    /**
     * @brief updateMenuList
     * @param menuList 表示するメニューアイコンのリスト（左から）
     */
    void updateMenuList(QVector<QString> *menuList);

    /**
     * @brief シングル測定結果取得開始、再開依頼を通知
     * @param constellationPoints コンスタレーション収集件数
     * @param condition チャンネル別測定条件
     * @param interval 測定結果取得間隔
     */
    void resumeSingleMeasuring(LC70_CONSTA_NUM_T constellationPoints, LC70_MEAS_SINGLE_T condition, uint32_t interval);

    /**
     * @brief シングル測定結果を通知
     * @param result 測定結果
     * @param constellations コンスタレーション
     */
    void singleMeasureResult(MeasureResult result, QList<QVariant> constellations);

    /**
     * @brief 方位角を通知
     * @param azimuth 方位角
     */
    void azimuth(uint16_t azimuth);

    /**
     * @brief デバイス状態を通知
     * @param keyStatus デバイス状態
     */
    void deviceStatus(LC70_DEV_STAT_T deviceStatus);

    /**
     * @brief 音量設定を通知
     * @param volumeLevel 音量レベル
     */
    void soundVolumeSetting(int volumeLevel);

    /**
     * @brief チャンネル変更
     * @param condition チャンネル別測定条件
     */
    void singleMeasuringChangeCondition(LC70_MEAS_SINGLE_T condition);

    /**
     * @brief マルチ測定結果取得開始、再開依頼を通知
     * @param condition マルチ測定条件
     * @param prefferedCondition 優先測定条件
     * @pram displayType 遷移先画面種別
     * @param interval 測定結果取得間隔
     */
    void resumeMultiMeasuring(LC70_MEAS_MULTI_T *condition, LC70_MEAS_SINGLE_T *prefferedCondition, LC70_DISP_TYPE_T displayType, uint32_t interval);

    /**
     * @brief マルチ測定結果取得開始、再開依頼を通知
     * @param prefferedCondition 優先測定条件
     */
    void resumeMultiPrefferedMeasuring(LC70_MEAS_SINGLE_T *prefferedCondition);

    /**
     * @brief マルチチャンネル測定結果を通知
     * @param result マルチチャンネル測定結果
     * @param prefferedChannelResult マルチチャンネル優先測定結果
     */
    void multiMeasureResult(MultiResult result, MeasureResult prefferedChannelResult);

    /**
     * @brief マルチ測定優先測定チャンネル変更
     * @param condition チャンネル別測定条件
     */
    void preferredMeasuringChangeCondition(LC70_MEAS_SINGLE_T *condition);

    /**
     * @brief チャンネル情報のスキャン開始を通知
     * @param condition 指定バンドのチャンネル定義リスト
     * @param acquisition 測定結果取得対象
     * @param interval スキャン間隔
     */
    void resumeScanning(LC70_MEAS_MULTI_T *condition, AcquisitionTarget acquisition, int interval);

    /**
     * @brief チャンネルスキャン停止
     */
    void suspendScanning();

    /**
     * @brief スキャン結果を通知
     * @param index スキャン対象のチャンネルリストのインデックス
     * @param result スキャン結果
     */
    void scanningChannelResult(int index, LC70_MEAS_MULTI_RESULT_T result);

    /**
     * @brief 遅延プロファイル測定結果を通知
     * @param result 遅延プロファイル測定結果
     */
    void delayProfileResult(QVector<QPointF> result);

    /**
     * @brief チャンネル変更中を通知
     * @param changing 変更中か否か
     */
    void notifyChannelChanging(bool changing);

    /**
     * @brief スキャン終了通知
     * @param complete 完了有無
     */
    void scanningFinished(bool complete);

    /**
     * @brief アンテナ調整測定結果取得開始、再開依頼を通知
     * @param condition アンテナ調整測定条件(マルチ測定の形式)
     * @param prefferedCondition 優先測定条件
     * @param interval 測定結果取得間隔
     */
    void resumeAntennaMeasuring(LC70_MEAS_MULTI_T *condition, uint32_t interval);

    /**
     * @brief アンテナ調整測定結果を通知
     * @param result 測定結果
     */
    void antennaMeasureResult(MultiResult result);

    /**
     * @brief アンテナ調整測定チャンネル変更を通知
     * @param condition チャンネル別測定条件
     */
    void antennaMeasuringChangeCondition(LC70_MEAS_MULTI_T *condition);

    /**
     * @brief USB挿抜変更通知を送信する
     * @param usbStatus USB挿抜状態 LC70_STRAGE_Tの値
     */
    void changeUsbInsertionRemoval(int usbStatus);

    /**
     * @brief 遅延プロファイル有効状態変更通知
     * @param valid 有効状態
     */
    void notifyDelayProfileValid(bool valid);

    /**
     * @brief バージョンアップ可否の通知
     * @param versionUpReady バージョンアップ可否
     * @note 機種情報画面で受信
     */
    void notifyVersionUpReady(bool versionUpReady);

    /**
     * @brief スタンバイ通知
     */
    void notifyStandBy(void);

    /**
     * @brief ウェイクアップ通知
     */
    void notifyWakeUp(void);

    /**
     * @brief 故障情報通知
     * @param errorInformation 故障情報
     */
    void notifyFailInfo(LC70_FAIL_INFO_T errorInformation);

    /**
     * @brief 自動パワーオフ時間到来通知
     */
    void notifyAutoPowerOff(void);

    /**
     * @brief シングル測定停止依頼を通知
     */
    void requestToSuspendSingleMeasuring(void);

    /**
     * @brief マルチ測定停止依頼を通知
     */
    void requestToSuspendMultiMeasuring(void);

    /**
     * @brief アンテナ調整測定停止依頼を通知
     */
    void requestToSuspendAntennaMeasuring(void);

    /**
     * @brief スタンバイ要求
     * @param mode 0: MAIN AND SUB 1: SUB ONLY
     */
    void requestToStandBy(uint8_t mode);

    /**
     * @brief ソフトウェアアップデート要求
     * @param updater アップデート情報
     */
    void notifySoftwareUpdate(LC70_PRG_UPDATE_T updater);

    /**
     * @brief アップデートの完了を通知
     * @param success アップデート成否
     */
    void notifySoftwareUpdateCompleted(bool success);

    /**
     * @brief バッテリ低下状態変化通知
     * @param isLowBattery バッテリ低下状態が発生しているかどうか true: 発生している false:発生していない
     * @return 通知結果
     */
    bool lowBatteryStatusChanged(bool isLowBattery);

    /**
     * @brief Model から通知されたエラー情報を通知
     * @param errorInformation エラー情報
     */
    void notifyErrorOccured(LC70_FAIL_INFO_T errorInformation);

    /**
     * @brief 測定停止を通知
     * @param target 測定の種類
     */
    void notifyStopMeasuring(AcquisitionTarget target);

private:
    QQmlApplicationEngine
    *_engine;                                                    //!< QML のインスタンスを作成するクラスのポインター
    Communicator *_communicator;                                 //!< サブ CPU と連携するためのクラス
    Stub *_stub;                                                 //!< スタブクラス
    QVector<QString> _menuList;                                  //!< 表示中のメニューアイコンリスト
    ApplicationSetting *_applicationSettings;                    //!< アプリケーション固有の設定情報
    ChannelSetting *_channelSettings;                            //!< チャンネル情報
    SystemWatcher *_systemWatcher;                               //!< キー入力、デバイス状態監視クラス
    Measurer *_measurer;                                         //!< 測定結果取得クラス
    FaultDetector *_faultDetector;                               //!< 故障状態取得クラス
    ModelOperator *_modelOperator;                               //!<
    AcquisitionTarget _acquisitionTarget;                        //!< 測定結果取得対象
    ActionMode _actionMode;                                      //!< 動作モード
    SystemLog &_systemLog;                                       //!< システムロガー
    QList<MeasureResult> _scanResult;                            //!< スキャン結果
    bool _isChangingMultiMeasureStyle;                           //!< マルチ測定の表示形式変更状態
    bool _versionUpReady;                                        //!< バージョンアップ可能状態
    LC70_PRG_UPDATE_T _updater;                                  //!< バージョンアップ情報
    bool _updating;                                              //!< バージョンアップ中
    bool _standBy;                                               //!< スタンバイ中か否か
    bool _wakeUpRequested;                                       //!< ウェイクアップ要求の有無
    QTimer *_autoPowerOffTimer;                                  //!< 自動パワーオフ用のタイマー
    uint8_t _standByMode;                                        //!< スタンバイ設定(0: MAIN AND SUB 1: SUB ONLY)
    long _wakeUpCounter;                                         //!< ウェイクアップカウンター
    QList<ChannelDefinitionItem> _scannedChannelList;            //!< スキャン後のチャンネル定義情報のリスト(保存画面からシングル/マルチ測定に遷移した場合にのみ使用)
    QString _scannedPolatization;                                //!< スキャンした Band の偏波
    QElapsedTimer _elapsedTimer;                                 //!< 処理時間計測用タイマー
    QFileSystemModel *_displayFileSystemModel;                   //!< ファイルのツリー表示用モデル
    QString _userDataLoggerSettingFileName;                      //!< 選択された USER データロガー設定ファイル名
    QList<UserDataLoggerSettingFile> _userDataLoggerSettingList; //!< 選択された USER データロガー設定ファイルの内容
    QList<UserDataLoggerSettingFile> _userDataLoggerEditingChannels;    //!< 選択された USER データロガー設定ファイル中の選択された Band の内容
    QList<UserDataLoggerSettingFile> _userDataLoggerMeasuringChannels;  //!< 選択された USER データロガー設定ファイル中の測定するが選択された全チャンネル一覧
    ChannelEditMode _channelEditMode;                            //!< USER データロガー設定チャンネル編集モード
    BandTabIndex _bandTabIndex;                                  //!< USER データロガー設定で選択されている Band のタブのインデックス

private slots:
    /**
     * @brief シングル測定結果取得開始、再開依頼を受信
     * @param constellationPoints コンスタレーション収集件数
     * @param condition チャンネル別測定条件
     * @param interval 測定結果取得間隔
     * @remark attach() を行ってから依頼
     */
    void onResumeSingleMeasuring(LC70_CONSTA_NUM_T constellationPoints, LC70_MEAS_SINGLE_T condition, uint32_t interval);

    /**
     * @brief マルチ測定結果取得開始、再開依頼を受信
     * @param condition マルチ測定測定条件
     * @param prefferedCondition 優先測定測定条件
     * @pram displayType 遷移先画面種別
     * @param interval 測定結果取得間隔
     * @remark attach() を行ってから依頼
     */
    void onResumeMultiMeasuring(LC70_MEAS_MULTI_T *condition, LC70_MEAS_SINGLE_T *prefferedCondition, LC70_DISP_TYPE_T displayType, uint32_t interval);

    /**
     * @brief マルチ測定優先測定チャンネル変更を受信
     * @param condition チャンネル別測定条件
     */
    void onPreferredMeasuringChangeCondition(LC70_MEAS_SINGLE_T *condition);

    /**
     * @brief アンテナ調整測定結果取得開始、再開依頼を受信
     * @param condition アンテナ調整測定条件
     * @param interval 測定結果取得間隔
     * @remark attach() を行ってから依頼
     */
    void onResumeAntennaMeasuring(LC70_MEAS_MULTI_T *condition, uint32_t interval);

    /**
     * @brief ボタン入力状態を通知を受信
     * @param key ボタン入力状態
     * @note 電源ボタンの検知はここで行う
     */
    void onKeyStatus(LC70_KEY_STAT_T key);

    /**
     * @brief 故障状態の検知を通知を受信
     * @param errorInformation エラー情報
     */
    void onErrorOccured(LC70_FAIL_INFO_T errorInformation);

    /**
     * @brief ウェイクアップの通知を受信
     */
    void onNotifyWakeUp(void);

    /**
     * @brief 自動パワーオフまでの待ち時間のタイムアウトを受信
     */
    void onAutoPowerOffTimeOut(void);

    /**
     * @brief スタンバイ要求を受信
     */
    void onRequestStandBy(void);

    /**
     * @brief シングル測定結果取得停止依頼を受信
     * @param avoidToStop 測定停止のラッパー関数を呼ばない
     * @remark 依頼後に detach()
     */
    void onSuspendSingleMeasuring(void);

    /**
     * @brief マルチ測定停止
     */
    void onSuspendMultiMeasuring(void);

    /**
     * @brief アンテナ調整測定結果取得停止依頼を受信
     * @remark 依頼後に detach()
     */
    void onSuspendAntennaMeasuring(void);

    /**
     * @brief スタンバイ要求を受信
     * @param mode 0: MAIN AND SUB 1: SUB ONLY
     */
    void onRequestToStandBy(uint8_t mode);

    /**
     * @brief アップデートの完了を通知を受信
     * @param success アップデート成否
     */
    void onNotifySoftwareUpdateCompleted(bool success);

    /**
     * @brief 測定停止を受信
     * @param target 測定の種類
     */
    void onNotifyStopMeasuring(AcquisitionTarget target);

private:
    /**
     * @brief 自動パワーオフまでの待ち時間を設定
     */
    void changePowerOffSetting(void);

    /**
     * @brief USER データロガー設定ファイルあるいはテンプレートファイルの読み込み
     * @param path ファイルのパス
     * @param fileName ファイル名
     * @param mode 編集モード
     * @return 読み込みの成否
     */
    bool readUserDataLoggerSettingFile(const QString &path, const QString &fileName, ChannelEditMode mode);

    /**
     * @brief バックアップの設定ファイルから元ファイルにコピー
     * @param path ファイルのパス
     * @return ファイルコピーの成否
     */
    bool copyFromBackupFile(const QString &path);
};

#endif // LEVELCHECKERMODEL_H
