/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include "measureresultmodel.h"
#include "correcter.h"
#include "liblc70.h"

static const QString SYSTEM_LOG_FILE_NAME = "pdisp_log";

MeasureResultModel::MeasureResultModel(QObject *parent)
    : QAbstractListModel(parent)
    , _measureResult()
    , _hasAlldata(true)
    , _dataMax(0)
    , _dataNum(0)
{
    _roleNames[BandRole] = "band";
    _roleNames[BandNameRole] = "bandName";
    _roleNames[ChannelRole] = "channel";
    _roleNames[LevelRole] = "level";
    _roleNames[MerRole] = "mer";
    _roleNames[BerRole] = "ber";
    _roleNames[FrequencyRole] = "frequency";
    _roleNames[ModulationRole] = "modulation";
    _roleNames[LayerRole] = "layer";
    _roleNames[PolarizationRole] = "polarization";
    _roleNames[TimestampRole] = "timestamp";
    _roleNames[BroadbandSystem] = "BroadbandSystem";
    _roleNames[LocalFrequencyRole] = "LocalFrequencyRole";
}

MeasureResultModel::~MeasureResultModel()
{
    _measureResult.clear();
}

int MeasureResultModel::rowCount(const QModelIndex &parent) const
{
    if (parent.isValid()) {
        return 0;
    }

    if(!_hasAlldata){
        return _dataNum;
    }

    // ダミー行を除く
    int count = 0;
    foreach (MeasureResultRow row, _measureResult) {
        if (row.Channel != QString("---")) {
            ++count;
        }
    }

    return count;
}

QHash<int, QByteArray> MeasureResultModel::roleNames() const
{
    return _roleNames;
}

QVariant MeasureResultModel::data(const QModelIndex &index, int role) const
{
    if (!hasIndex(index.row(), index.column(), index.parent())) {
        return QVariant();
    }

    const MeasureResultRow &measureResult = _measureResult.at(index.row());

    if (role == BandRole) {
        return QVariant(measureResult.Band);
    } else if (role == BandNameRole) {
        return QVariant(measureResult.BandName);
    } else if (role == ChannelRole) {
        return QVariant(measureResult.Channel);
    } else if (role == LevelRole) {
        return QVariant(measureResult.Level);
    } else if (role == MerRole) {
        return QVariant(measureResult.Mer);
    } else if (role == BerRole) {
        return QVariant(measureResult.Ber);
    } else if (role == FrequencyRole) {
        return QVariant(measureResult.Frequency);
    } else if (role == ModulationRole) {
        return QVariant(measureResult.Modulation);
    } else if (role == LayerRole) {
        return QVariant(measureResult.Layer);
    } else if (role == PolarizationRole) {
        return QVariant(measureResult.Polarization);
    } else if (role == TimestampRole) {
        return QVariant(measureResult.Timestamp);
    } else if (role == BroadbandSystem) {
        return QVariant(measureResult.BroadcastSystem);
    } else if (role == LocalFrequencyRole) {
        return QVariant(measureResult.LocalOscillator);
    } else {
        // nothing to do
    }

    return {};
}

bool MeasureResultModel::setData(const QModelIndex &index, const QVariant &value, int role)
{
    if (!hasIndex(index.row(), index.column(), index.parent()) || !value.isValid()) {
        return false;
    }

    MeasureResultRow &measureResult = _measureResult[index.row()];

    if (role == BandRole) {
        measureResult.Band = value.toString();
    } else if (role == BandNameRole) {
        measureResult.BandName = value.toString();
    } else if (role == ChannelRole) {
        measureResult.Channel = value.toString();
    } else if (role == LevelRole) {
        measureResult.Level = value.toFloat();
    } else if (role == MerRole) {
        measureResult.Mer = value.toFloat();
    } else if (role == BerRole) {
        measureResult.Ber = value.toFloat();
    } else if (role == FrequencyRole) {
        measureResult.Frequency = value.toUInt();
    } else if (role == ModulationRole) {
        measureResult.Modulation = value.toString();
    } else if (role == LayerRole) {
        measureResult.Layer = value.toString();
    } else if (role == PolarizationRole) {
        measureResult.Polarization = value.toString();
    } else if (role == TimestampRole) {
        measureResult.Timestamp = value.toString();
    } else if (role == BroadbandSystem) {
        measureResult.BroadcastSystem = value.toString();
    }
    emit dataChanged(index, index, { role } );

    return true ;
}

void MeasureResultModel::append(const MeasureResultRow &measureResult)
{
    int pos = _measureResult.count();
    if (!_hasAlldata) {
        _dataNum++;
        if (_dataNum > _dataMax){
            _measureResult.removeFirst();
            _measureResult.append(measureResult);
            emit dataChanged(index(0), index(_dataMax - 1), { TimestampRole, BandRole, BandNameRole, ChannelRole, LevelRole, MerRole, BerRole, FrequencyRole } );
            return;
        }
    }
    QModelIndex index = QModelIndex();
    beginInsertRows(index, pos, pos);
    _measureResult.append(measureResult);
    endInsertRows();
}

void MeasureResultModel::append(const QList<MeasureResultRow> &measureResult)
{
    QModelIndex index = QModelIndex();
    beginInsertRows(index, 0, _dataMax - 1);
    _measureResult.append(measureResult);
    endInsertRows();
}

void MeasureResultModel::setPageData(const QList<MeasureResultRow> &measureResult)
{
    _measureResult.clear();
    _measureResult.append(measureResult);
    emit dataChanged(index(0), index(_dataMax - 1), { TimestampRole, BandRole, BandNameRole, ChannelRole, LevelRole, MerRole, BerRole, FrequencyRole } );
}

void MeasureResultModel::update(int row, const float32_t &level, const float32_t &mer, const float32_t &ber,const int bandId,const int broadcastSystem,const int modulation)
{
    if (row > _measureResult.count() - 1) {
        return;
    }

    MeasureResultRow &measureResult = _measureResult[row];

    // [Memo]
    // 以下の場合MER,BERは「---」を表示するために-1を設定する。
    // BANDがPILOT, LTE, FMの場合
    // BER, MERの値がnanの場合, infの場合
    LC70_BAND_T band = static_cast<LC70_BAND_T>(measureResult.Band.toInt());

    BandTabIndex bandTab = Correcter::convertBandTabIndex(band, measureResult.Polarization);
    QString levelString;
    if (!Correcter::displayLevel(bandTab, level, levelString)) {

    }

    measureResult.Level = levelString.isEmpty() ? QString(INITIAL_LEVEL) : levelString;

    if (band == LC70_BAND_PILOT || band == LC70_BAND_LTE || band == LC70_BAND_FM || band == LC70_BAND_CATV_UP_STREAM) {
        measureResult.Mer = QString::number(-1);
        measureResult.Ber = QString::number(-1);
    } else {
        if (qIsNaN(ber) || qIsInf(ber)) {
            // -1 を設定したときは ">1.0e-2" と表示する
            measureResult.Ber = QString::number(-1);
        } else {
            measureResult.Ber = QString::number(ber);
        }

        if (qIsNaN(mer) || qIsInf(ber)) {
            measureResult.Mer = QString::number(-1);
        } else {
            QString value;
            //------------------------------------------------------------------
            /*if (0 <= mer && mer < 5.0 || mer <= -2.0) {
                value = QString("-2.0");
            } else if (5.0 <= mer && mer <= 35) {
                value = QString("%1").arg(mer, 1, 'f', 1).rightJustified(2, '0');
               // value = Correcter::displayMer(band,broadcastSystem,modulation,mer);
            } else if (mer < 0 && mer > -1.1) {
                value = QString("-1.0");
            } else {
                value = QString(">35");
            }*/
            // -1 を設定したときは "---" と表示するため -1.0 になったときは -1 に付け替える
            //------------------------------------------------------------------
            QString compare = QString("%1").arg(mer, 1, 'f', 0);
            if (compare == QString("-1")  /*mer < 0 && mer > -1.1*/) {
                            value = QString("-1.0");
            }else{

               value = Correcter::displayMer(band,broadcastSystem,modulation,mer);
            }
            /*if(mer == -2.0){
                value = QString("-2.0");
            }else if(mer == -3.0){
                value = QString("-3.0");
            }*/
            if (value == QString("-1.0")) {
                measureResult.Mer = QString::number(-1);
            } else {
                measureResult.Mer = value;
            }

        }
    }

    emit dataChanged(index(row), index(row), { LevelRole, MerRole, BerRole } );
}

void MeasureResultModel::clear()
{
    int pos = _measureResult.size();
    QModelIndex index = QModelIndex();
    beginRemoveRows(index, 0, pos);
    removeRows(0, _measureResult.count());
    endRemoveRows();
    _measureResult.clear();
}

void MeasureResultModel::setDataMax(int dataMax)
{
    _dataMax = dataMax;
    _dataNum = 0;
    _hasAlldata = false;
}

void MeasureResultModel::setDataNum(int dataNum)
{
    _dataNum = dataNum;
}
