/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef MEASURERESULTMODEL_H
#define MEASURERESULTMODEL_H

#include <QAbstractListModel>
#include "constant.h"
#include "systemlog.h"

/**
 * @brief 測定結果保存データのメンバーに対応するロール
 */
enum MeasureResultRowRoles {
    BandRole = Qt::UserRole + 1,    //!< Band
    BandNameRole,                   //!< 表示 Band 名
    ChannelRole,                    //!< チャンネル
    LevelRole,                      //!< 測定入力レベル
    MerRole,                        //!< 変調誤差比
    BerRole,                        //!< ビット誤り率
    FrequencyRole,                  //!< 中心周波数
    ModulationRole,                 //!< 変調方式
    LayerRole,                      //!< 階層
    PolarizationRole,               //!< 変調方式
    TimestampRole,                  //!< 取得時間
    BroadbandSystem,                //!< 放送方式
    LocalFrequencyRole,             //!< 局部発振周波数
};

/**
 * @brief マルチチャンネル測定のリスト表示用
 */
class MeasureResultModel : public QAbstractListModel
{
    Q_OBJECT
    Q_ENUMS(MeasureResultRowRoles)
public:

    /**
     * @brief コンストラクター
     * @param parent 親オブジェクト
     */
    explicit MeasureResultModel(QObject *parent = nullptr);

    /**
     * @brief デストラクター
     */
    ~MeasureResultModel();

    /**
     * @brief 測定結果数を返す
     * @return 測定結果数
     */
    int rowCount(const QModelIndex &parent = QModelIndex()) const override;

    /**
     * @brief ロール名を返す
     * @return ロール名
     */
    QHash<int, QByteArray> roleNames() const override;

    /**
     * @brief 測定結果のデータを返す
     * @param index データの位置
     * @param role ロール
     * @return 測定データ
     */
    QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const override;

    /**
     * @brief 測定結果を設定する
     * @param index データの位置
     * @param value 値
     * @param role ロール
     * @return 処理の成否
     */
    bool setData(const QModelIndex &index, const QVariant &value, int role) override;

    /**
     * @brief 測定結果の追加
     * @param measureResult 測定結果
     */
    void append(const MeasureResultRow &measureResult);

    /**
     * @brief 測定結果の一括追加
     * @param measureResult 測定結果
     */
    void append(const QList<MeasureResultRow> &measureResult);

    /**
     * @brief 測定結果の一括切替
     * @param measureResult 測定結果
     */
    void setPageData(const QList<MeasureResultRow> &measureResult);

    /**
     * @brief 測定結果の更新
     * @param row　行
     * @param level 測定入力レベルの値
     * @param mer 変調誤差比の値
     * @param ber ビット誤り率の値
     */
    void update(int row, const float32_t &level, const float32_t &mer, const float32_t &ber,const int bandId,const int broadcastSystem,const int modulation);
    /**
     * @brief データをクリアー
     */
    void clear(void);

    /**
     * @brief 実体データの上限設定
     * @param dataMax データ数上限
     */
    void setDataMax(int dataMax);

    /**
     * @brief 全データ数の設定
     * @param dataNum データ数
     */
    void setDataNum(int dataNum);

private:
    QHash<int, QByteArray> _roleNames;      //!< ロール名
    QList<MeasureResultRow> _measureResult; //!< 測定結果リスト
    bool _hasAlldata;                       //!< 全データを実体で持つかどうか
    int _dataMax;                           //!< データ数の上限
    int _dataNum;                           //!< 実体として保持していない分も含めた全データ数
};

#endif // MEASURERESULTMODEL_H
