/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include "modeloperator.h"
#include "communicator.h"
#include "correcter.h"
#include <QThread>
#include <QDebug>

/**
 *  @brief 各チャンネルと対応する定義ファイル名
 */
static const ChannelDefinitionFile CHANNEL_DEFINITION_FILE[] = {
    { LC70_FILE_ID_CH_SET_UHF, "channel_setting_file_uhf_6mhz.csv", 2, LC70_BAND_UHF  },           //!< チャンネル定義(地デジ(帯域幅6MHz))ファイル
    { LC70_FILE_ID_CH_SET_BS, "channel_setting_file_bs_34.5mhz.csv", 3, LC70_BAND_BS},             //!< チャンネル定義(CS(帯域幅34.5MHz))ファイル
    { LC70_FILE_ID_CH_SET_CS, "channel_setting_file_cs_34.5mhz.csv", 3, LC70_BAND_CS},             //!< チャンネル定義(BS(帯域幅34.5MHz))ファイル
    { LC70_FILE_ID_CH_SET_SCCB, "channel_setting_file_sccb_27mhz.csv", 3, LC70_BAND_SCCB},         //!< チャンネル定義(SCC-B(帯域幅27MHz))ファイル
    { LC70_FILE_ID_CH_SET_SCCC, "channel_setting_file_sccc_27mhz.csv", 3, LC70_BAND_SCCC},         //!< チャンネル定義(SCC-C(帯域幅27MHz))ファイル
    { LC70_FILE_ID_CH_SET_JSAT3A, "channel_setting_file_jsat3a_27mhz.csv", 3, LC70_BAND_JCSAT3},   //!< チャンネル定義(JSAT-3A(帯域幅27MHz))ファイル
    { LC70_FILE_ID_CH_SET_JSAT4B, "channel_setting_file_jsat4b_27mhz.csv", 3, LC70_BAND_JCSAT4},   //!< チャンネル定義(JSAT-4B(帯域幅27MHz))ファイル
    // ラッパーライブラリ V2.00 対応
    { LC70_FILE_ID_CH_SET_CATV_ALL, "channel_setting_file_catv_all_6mhz.csv", 3, LC70_BAND_CATV_ALL},                           //!< チャンネル定義(CATV ALL(帯域幅6MHz))ファイル
    { LC70_FILE_ID_CH_SET_CATV_90_470MHZ, "channel_setting_file_catv_90_470mhz_6mhz.csv", 3, LC70_BAND_CATV_90_470MHZ},         //!< チャンネル定義(CATV 90-470MHz(帯域幅6MHz)ファイル
    { LC70_FILE_ID_CH_SET_CATV_470_1002MHZ, "channel_setting_file_catv_470_1002mhz_6mhz.csv", 3, LC70_BAND_CATV_470_1002MHZ},   //!< チャンネル定義(CATV 470-1002MHz(帯域幅6MHz))ファイル
    { LC70_FILE_ID_CH_SET_SKYP, "channel_setting_file_skyp_6mhz.csv", 2, LC70_BAND_SKYP},                                       //!< チャンネル定義(スカパー光(帯域幅6MHz))ファイル
    { LC70_FILE_ID_CH_SET_PILOT, "channel_setting_file_pilot.csv", 2, LC70_BAND_PILOT },                                        //!< チャンネル定義(PILOT)ファイル
    { LC70_FILE_ID_CH_SET_LTE,  "channel_setting_file_lte.csv", 3, LC70_BAND_LTE},                                              //!< チャンネル定義(LTE)ファイル
    { LC70_FILE_ID_CH_SET_FM,  "channel_setting_file_fm.csv", 2, LC70_BAND_FM},                                                 //!< チャンネル定義(FM)ファイル
    { LC70_FILE_ID_CH_SET_CATV_UP_STREAM, "channel_setting_file_catv_up_stream.csv", 2, LC70_BAND_CATV_UP_STREAM},              //!< チャンネル定義(CATV 上り)ファイル
    { LC70_FILE_ID_CH_SET_JSAT3A_ANTENNA, "channel_setting_file_jsat3a_antenna_27mhz.csv", 3, LC70_BAND_JCSAT3},                //!< チャンネル定義(アンテナ調整用 CSチャンネル JCSAT-3A(帯域幅27MHz))ファイル
    { LC70_FILE_ID_CH_SET_JSAT4B_ANTENNA, "channel_setting_file_jsat4b_antenna_27mhz.csv", 3, LC70_BAND_JCSAT4},                //!< チャンネル定義(アンテナ調整用 CSチャンネル JCSAT-4B(帯域幅27MHz))ファイル
    { LC70_FILE_ID_CH_SET_FM_DEF,  "channel_setting_file_fm_def.csv", 2, LC70_BAND_FM_DEF},                                     //!< チャンネル定義(FM DEFAULT)ファイル
    { LC70_FILE_ID_CH_SET_CATV_UP_STREAM_DEF,  "channel_setting_file_catv_up_stream_def.csv", 2, LC70_BAND_CATV_UP_STREAM_DEF}, //!< チャンネル定義(CATV 上り DEFAULT)ファイル
    { LC70_FILE_ID_CH_SET_BS_CS, "channel_setting_file_bs_cs_34.5mhz.csv", 3, LC70_BAND_BS_CS},                                 //!< チャンネル定義(BS_CS(帯域幅34.5MHz))ファイル
};

static constexpr const char *CSV_FILE_EXTENSION = ".csv";       //!< 設定ファイルの拡張子(CSV)
static constexpr int MAXIMUM_REPEAT_MEASURE_STATUS = 500;       // 測定状態取得の最大リトライ回数
static constexpr int INTERVAL_MEASURE_STATUS = 20;              // 測定状態取得の取得間隔
static constexpr int RESET_MEASURE_STATUS = 50;                 // サブ CPU リセットを行うまでのリトライ回数

const unsigned int ModelOperator::LOCAL_FREQ_UNDEFINED_DATA = 429467295;
ModelOperator::ModelOperator(Communicator *communicator, Stub *stub, ApplicationSetting *applicationSettings)
    : _communicator(communicator)
    , _stub(stub)
    , _actionMode(DEVELOPMENT)
    , _dataloggerSettingDir()
    , _applicationSettings(applicationSettings)
    , _values()
    , _channelDefinitionMap()
{
}

ModelOperator::~ModelOperator()
{
}

bool ModelOperator::initialize(ActionMode actionMode, bool defaultInit)
{
    _actionMode = actionMode;

    // チャンネル定義データの初期化を実施

    if (_actionMode == PRODUCTION) {
        initializeChannelDefinitionDataProduction(defaultInit);
        initializeProduction(defaultInit);
    } else {
        initializeChannelDefinitionDataDevelop(defaultInit);
        initializeDevelop(defaultInit);
    }

    return true;
}

bool ModelOperator::initializeSettingFile()
{
    // [Memo]
    //  デフォルトの値を読み込み
    //  読み込んだ値を設定ファイルに対して書き込む

    // システム設定ファイル
    initializeSystemSettingFile();

    // 測定設定ファイル
    initializeMeasureSettingFile();

    // ユーザー測定/データロガー設定ファイル
    initializeUserLoggerSettingFile();

    // チャンネル定義ファイル
    initializeChannelSettingFile();

    return true;
}

bool ModelOperator::initializeSystemSettingFile()
{
    // POWER OFF時間
    {
        LC70_SYS_SET_POWOFF_T lc70Data;
        _communicator->readSystemSettingFile(LC70_FILE_ID_SYS_SET_POWOFF, &lc70Data, true);
        _communicator->writeSystemSettingFile(LC70_FILE_ID_SYS_SET_POWOFF, &lc70Data);
    }

    // 音量設定
    {
        LC70_SYS_SET_VOLUME_T lc70Data;
        _communicator->readSystemSettingFile(LC70_FILE_ID_SYS_SET_VOLUME, &lc70Data, true);
        _communicator->writeSystemSettingFile(LC70_FILE_ID_SYS_SET_VOLUME, &lc70Data);
    }

    // 明るさ設定
    {
        LC70_SYS_SET_BRIGHTNESS_T lc70Data;
        _communicator->readSystemSettingFile(LC70_FILE_ID_SYS_SET_BRIGHTNESS, &lc70Data, true);
        _communicator->writeSystemSettingFile(LC70_FILE_ID_SYS_SET_BRIGHTNESS, &lc70Data);
    }

    return true;
}

bool ModelOperator::initializeMeasureSettingFile()
{
    // 局部発振周波数
    {
        LC70_MEAS_SET_LOCAL_FREQ_T lc70Data;
        _communicator->readMeasureSettingFile(LC70_FILE_ID_MEAS_SET_LOCAL_FREQ, &lc70Data, true);
        _communicator->writeMeasureSettingFile(LC70_FILE_ID_MEAS_SET_LOCAL_FREQ, &lc70Data);
    }

    // 閾値
    {
        LC70_MEAS_SET_THRD_T lc70Data;
        _communicator->readMeasureSettingFile(LC70_FILE_ID_MEAS_SET_THRD, &lc70Data, true);
        _communicator->writeMeasureSettingFile(LC70_FILE_ID_MEAS_SET_THRD, &lc70Data);
    }

    // CNR/MER切替
    {
        LC70_MEAS_SET_CNRMER_ROW_T lc70Data;
        _communicator->readMeasureSettingFile(LC70_FILE_ID_MEAS_SET_CNRMER, &lc70Data, true);
        _communicator->writeMeasureSettingFile(LC70_FILE_ID_MEAS_SET_CNRMER, &lc70Data);
    }

    // CATVチャンネル周波数
    {
        LC70_MEAS_SET_CATV_FREQ_T lc70Data;
        _communicator->readMeasureSettingFile(LC70_FILE_ID_MEAS_SET_CATV_FREQ, &lc70Data, true);
        _communicator->writeMeasureSettingFile(LC70_FILE_ID_MEAS_SET_CATV_FREQ, &lc70Data);
    }

    // ケーブル損失
    {
        LC70_MEAS_SET_CABLE_LOSS_T lc70Data;
        _communicator->readMeasureSettingFile(LC70_FILE_ID_MEAS_SET_CABLE_LOSS, &lc70Data, true);
        _communicator->writeMeasureSettingFile(LC70_FILE_ID_MEAS_SET_CABLE_LOSS, &lc70Data);
    }

    // ケーブル損失有効
    {
        LC70_MEAS_SET_CABLE_LOSS_ENB_T lc70Data;
        _communicator->readMeasureSettingFile(LC70_FILE_ID_MEAS_SET_CABLE_LOSS_ENB, &lc70Data, true);
        _communicator->writeMeasureSettingFile(LC70_FILE_ID_MEAS_SET_CABLE_LOSS_ENB, &lc70Data);
    }

    // 入力切替
    {
        LC70_MEAS_SET_INPUT_T lc70Data;
        _communicator->readMeasureSettingFile(LC70_FILE_ID_MEAS_SET_INPUT, &lc70Data, true);
        _communicator->writeMeasureSettingFile(LC70_FILE_ID_MEAS_SET_INPUT, &lc70Data);
    }

    // CATVの変調方式選択範囲
    {
        LC70_MEAS_SET_CATV_MOD_T lc70Data;
        _communicator->readMeasureSettingFile(LC70_FILE_ID_MEAS_SET_CATV_MOD, &lc70Data, true);
        _communicator->writeMeasureSettingFile(LC70_FILE_ID_MEAS_SET_CATV_MOD, &lc70Data);
    }

    // 給電電圧
    {
        LC70_MEAS_SET_DC_OUT_T lc70Data;
        _communicator->readMeasureSettingFile(LC70_FILE_ID_MEAS_SET_DC_OUT, &lc70Data, true);
        _communicator->writeMeasureSettingFile(LC70_FILE_ID_MEAS_SET_DC_OUT, &lc70Data);
    }

    // 光測定選択範囲
    {
        LC70_MEAS_SET_OPT_MEAS_T lc70Data;
        _communicator->readMeasureSettingFile(LC70_FILE_ID_MEAS_SET_OPT_MEAS, &lc70Data, true);
        _communicator->writeMeasureSettingFile(LC70_FILE_ID_MEAS_SET_OPT_MEAS, &lc70Data);
    }

    // コンスタレーション表示ポイント数選択範囲
    {
        LC70_MEAS_SET_CONSTA_NUM_T lc70Data;
        _communicator->readMeasureSettingFile(LC70_FILE_ID_MEAS_SET_CONSTA_NUM, &lc70Data, true);
        _communicator->writeMeasureSettingFile(LC70_FILE_ID_MEAS_SET_CONSTA_NUM, &lc70Data);
    }

    return true;
}

bool ModelOperator::initializeUserLoggerSettingFile()
{
    // ユーザ測定
    {
        LC70_USER_SET_USER_T lc70Data;
        _communicator->readSystemSettingFile(LC70_FILE_ID_USER_SET_USER, &lc70Data, true);
        _communicator->writeSystemSettingFile(LC70_FILE_ID_USER_SET_USER, &lc70Data);
    }

    // ユーザ測定ファイル名リスト
    {
        LC70_USER_SET_USER_FLIST_T lc70Data;
        _communicator->readSystemSettingFile(LC70_FILE_ID_USER_SET_USER_FLIST, &lc70Data, true);
        _communicator->writeSystemSettingFile(LC70_FILE_ID_USER_SET_USER_FLIST, &lc70Data);
    }

    // ユーザ測定 USB1
    {
        LC70_USER_SET_USER_T lc70Data;
        _communicator->readSystemSettingFile(LC70_FILE_ID_USER_SET_USER_USB1, &lc70Data, true);
        _communicator->writeSystemSettingFile(LC70_FILE_ID_USER_SET_USER_USB1, &lc70Data);
    }
    {
        LC70_USER_SET_USER_FLIST_T lc70Data;
        _communicator->readSystemSettingFile(LC70_FILE_ID_USER_SET_USER_USB1_FLIST, &lc70Data, true);
        _communicator->writeSystemSettingFile(LC70_FILE_ID_USER_SET_USER_USB1_FLIST, &lc70Data);
    }

    // ユーザ測定 USB2
    {
        LC70_USER_SET_USER_T lc70Data;
        _communicator->readSystemSettingFile(LC70_FILE_ID_USER_SET_USER_USB2, &lc70Data, true);
        _communicator->writeSystemSettingFile(LC70_FILE_ID_USER_SET_USER_USB2, &lc70Data);
    }
    {
        LC70_USER_SET_USER_FLIST_T lc70Data;
        _communicator->readSystemSettingFile(LC70_FILE_ID_USER_SET_USER_USB2_FLIST, &lc70Data, true);
        _communicator->writeSystemSettingFile(LC70_FILE_ID_USER_SET_USER_USB2_FLIST, &lc70Data);
    }

    // データロガー設定
    {
        LC70_LOGGER_SET_LOGGER_T lc70Data;
        _communicator->readSystemSettingFile(LC70_FILE_ID_LOGGER_SET_LOGGER, &lc70Data, true);
        _communicator->writeSystemSettingFile(LC70_FILE_ID_LOGGER_SET_LOGGER, &lc70Data);
    }

    // データロガー設定ファイル名リスト
    {
        LC70_LOGGER_SET_LOGGER_FLIST_T lc70Data;
        _communicator->readSystemSettingFile(LC70_FILE_ID_LOGGER_SET_LOGGER_FLIST, &lc70Data, true);
        _communicator->writeSystemSettingFile(LC70_FILE_ID_LOGGER_SET_LOGGER_FLIST, &lc70Data);
    }

    // データロガー設定 USB1
    {
        LC70_LOGGER_SET_LOGGER_T lc70Data;
        _communicator->readSystemSettingFile(LC70_FILE_ID_LOGGER_SET_LOGGER_USB1, &lc70Data, true);
        _communicator->writeSystemSettingFile(LC70_FILE_ID_LOGGER_SET_LOGGER_USB1, &lc70Data);
    }
    {
        LC70_LOGGER_SET_LOGGER_FLIST_T lc70Data;
        _communicator->readSystemSettingFile(LC70_FILE_ID_LOGGER_SET_LOGGER_USB1_FLIST, &lc70Data, true);
        _communicator->writeSystemSettingFile(LC70_FILE_ID_LOGGER_SET_LOGGER_USB1_FLIST, &lc70Data);
    }

    // データロガー設定 USB2
    {
        LC70_LOGGER_SET_LOGGER_T lc70Data;
        _communicator->readSystemSettingFile(LC70_FILE_ID_LOGGER_SET_LOGGER_USB2, &lc70Data, true);
        _communicator->writeSystemSettingFile(LC70_FILE_ID_LOGGER_SET_LOGGER_USB2, &lc70Data);
    }
    {
        LC70_LOGGER_SET_LOGGER_FLIST_T lc70Data;
        _communicator->readSystemSettingFile(LC70_FILE_ID_LOGGER_SET_LOGGER_USB2_FLIST, &lc70Data, true);
        _communicator->writeSystemSettingFile(LC70_FILE_ID_LOGGER_SET_LOGGER_USB2_FLIST, &lc70Data);
    }

    return true;
}

bool ModelOperator::initializeChannelSettingFile()
{
    // 地デジ
    {
        LC70_CH_SET_UHF_T lc70Data;
        _communicator->readChannelConfigurationFile(LC70_FILE_ID_CH_SET_UHF, &lc70Data, true);
        _communicator->writeChannelConfigurationFile(LC70_FILE_ID_CH_SET_UHF, &lc70Data);
    }

    // BS
    {
        LC70_CH_SET_BS_T lc70Data;
        _communicator->readChannelConfigurationFile(LC70_FILE_ID_CH_SET_BS, &lc70Data, true);
        _communicator->writeChannelConfigurationFile(LC70_FILE_ID_CH_SET_BS, &lc70Data);
    }

    // CS
    {
        LC70_CH_SET_CS_T lc70Data;
        _communicator->readChannelConfigurationFile(LC70_FILE_ID_CH_SET_CS, &lc70Data, true);
        _communicator->writeChannelConfigurationFile(LC70_FILE_ID_CH_SET_CS, &lc70Data);
    }

    // SCC-B
    {
        LC70_CH_SET_SCCB_T lc70Data;
        _communicator->readChannelConfigurationFile(LC70_FILE_ID_CH_SET_SCCB, &lc70Data, true);
        _communicator->writeChannelConfigurationFile(LC70_FILE_ID_CH_SET_SCCB, &lc70Data);
    }

    // SCC-C
    {
        LC70_CH_SET_SCCC_T lc70Data;
        _communicator->readChannelConfigurationFile(LC70_FILE_ID_CH_SET_SCCC, &lc70Data, true);
        _communicator->writeChannelConfigurationFile(LC70_FILE_ID_CH_SET_SCCC, &lc70Data);
    }

    // JSAT-3A
    {
        LC70_CH_SET_JSAT3A_T lc70Data;
        _communicator->readChannelConfigurationFile(LC70_FILE_ID_CH_SET_JSAT3A, &lc70Data, true);
        _communicator->writeChannelConfigurationFile(LC70_FILE_ID_CH_SET_JSAT3A, &lc70Data);
    }

    // JSAT-4B
    {
        LC70_CH_SET_JSAT4B_T lc70Data;
        _communicator->readChannelConfigurationFile(LC70_FILE_ID_CH_SET_JSAT4B, &lc70Data, true);
        _communicator->writeChannelConfigurationFile(LC70_FILE_ID_CH_SET_JSAT4B, &lc70Data);
    }

    // ラッパーライブラリ V2.00 対応
    // CATV_ALL
    {
        LC70_CH_SET_CATV_ALL_T lc70Data;
        _communicator->readChannelConfigurationFile(LC70_FILE_ID_CH_SET_CATV_ALL, &lc70Data, true);
        _communicator->writeChannelConfigurationFile(LC70_FILE_ID_CH_SET_CATV_ALL, &lc70Data);
    }

    // ラッパーライブラリ V2.00 対応
    {
        LC70_CH_SET_CATV_90_470MHZ_T lc70Data;
        _communicator->readChannelConfigurationFile(LC70_FILE_ID_CH_SET_CATV_90_470MHZ, &lc70Data, true);
        _communicator->writeChannelConfigurationFile(LC70_FILE_ID_CH_SET_CATV_90_470MHZ, &lc70Data);
    }

    // ラッパーライブラリ V2.00 対応
    {
        LC70_CH_SET_CATV_470_1002MHZ_T lc70Data;
        _communicator->readChannelConfigurationFile(LC70_FILE_ID_CH_SET_CATV_470_1002MHZ, &lc70Data, true);
        _communicator->writeChannelConfigurationFile(LC70_FILE_ID_CH_SET_CATV_470_1002MHZ, &lc70Data);
    }

    // スカパー
    {
        LC70_CH_SET_SKYP_T lc70Data;
        _communicator->readChannelConfigurationFile(LC70_FILE_ID_CH_SET_SKYP, &lc70Data, true);
        _communicator->writeChannelConfigurationFile(LC70_FILE_ID_CH_SET_SKYP, &lc70Data);
    }

    // PILOT
    {
        LC70_CH_SET_PILOT_T lc70Data;
        _communicator->readChannelConfigurationFile(LC70_FILE_ID_CH_SET_PILOT, &lc70Data, true);
        _communicator->writeChannelConfigurationFile(LC70_FILE_ID_CH_SET_PILOT, &lc70Data);
    }

    // LTE
    {
        LC70_CH_SET_LTE_T lc70Data;
        _communicator->readChannelConfigurationFile(LC70_FILE_ID_CH_SET_LTE, &lc70Data, true);
        _communicator->writeChannelConfigurationFile(LC70_FILE_ID_CH_SET_LTE, &lc70Data);
    }

    // FM
    {
        LC70_CH_SET_FM_T lc70Data;
        _communicator->readChannelConfigurationFile(LC70_FILE_ID_CH_SET_FM, &lc70Data, true);
        _communicator->writeChannelConfigurationFile(LC70_FILE_ID_CH_SET_FM, &lc70Data);
    }

    // CATV上り
    {
        LC70_CH_SET_CATV_UP_STREAM_T lc70Data;
        _communicator->readChannelConfigurationFile(LC70_FILE_ID_CH_SET_CATV_UP_STREAM, &lc70Data, true);
        _communicator->writeChannelConfigurationFile(LC70_FILE_ID_CH_SET_CATV_UP_STREAM, &lc70Data);
    }

    // FM Default
    {
        LC70_CH_SET_FM_DEF_T lc70Data;
        _communicator->readChannelConfigurationFile(LC70_FILE_ID_CH_SET_FM_DEF, &lc70Data, true);
        _communicator->writeChannelConfigurationFile(LC70_FILE_ID_CH_SET_FM_DEF, &lc70Data);
    }
    // CATV UPSTEAM Default
    {
        LC70_CH_SET_CATV_UP_STREAM_DEF_T lc70Data;
        _communicator->readChannelConfigurationFile(LC70_FILE_ID_CH_SET_CATV_UP_STREAM_DEF, &lc70Data, true);
        _communicator->writeChannelConfigurationFile(LC70_FILE_ID_CH_SET_CATV_UP_STREAM_DEF, &lc70Data);
    }

    // アンテナ調整 JCSAT-3A
    {
        LC70_CH_SET_JSAT3A_ANTENNA_T lc70Data;
        _communicator->readChannelConfigurationFile(LC70_FILE_ID_CH_SET_JSAT3A_ANTENNA, &lc70Data, true);
        _communicator->writeChannelConfigurationFile(LC70_FILE_ID_CH_SET_JSAT3A_ANTENNA, &lc70Data);
    }

    // アンテナ調整 JCSAT-4B
    {
        LC70_CH_SET_JSAT4B_ANTENNA_T lc70Data;
        _communicator->readChannelConfigurationFile(LC70_FILE_ID_CH_SET_JSAT4B_ANTENNA, &lc70Data, true);
        _communicator->writeChannelConfigurationFile(LC70_FILE_ID_CH_SET_JSAT4B_ANTENNA, &lc70Data);
    }

    // BS_CS統合
    {
        LC70_CH_SET_BS_CS_T lc70Data;
        _communicator->readChannelConfigurationFile(LC70_FILE_ID_CH_SET_BS_CS, &lc70Data, true);
        _communicator->writeChannelConfigurationFile(LC70_FILE_ID_CH_SET_BS_CS, &lc70Data);
    }

    return true;
}

bool ModelOperator::getValue(ValueId valueId, QVariant &value)
{
    if (0 == _values.count(valueId)) {
        value.clear();
        return false;
    }

    value = _values[valueId].getValue();
    return true;
}

bool ModelOperator::setValue(ValueId valueId, const QVariant &value)
{
    if (0 == _values.count(valueId)) {
        qDebug("set new value. valueId:%d", valueId);
    }

    if (!value.isValid()) {
        abort();
    }

    _values[valueId].setValue(value);
    return true;
}

bool ModelOperator::setInitialValue(ValueId valueId, const QVariant &value)
{
    if (0 == _values.count(valueId)) {
        qDebug("set new value. valueId:%d", valueId);
    }

    if (!value.isValid()) {
        abort();
    }

    _values[valueId].setInitialValue(value);
    return true;
}

bool ModelOperator::isUpdate(ValueId valueId)
{
    if (0 == _values.count(valueId)) {
        return false;
    }

    return _values[valueId].isUpdate();
}

bool ModelOperator::rollback(ValueId valueId)
{
    if (0 == _values.count(valueId)) {
        return false;
    }

    _values[valueId].rollback();
    return true;
}

bool ModelOperator::commit(ValueId valueId)
{
    if (0 == _values.count(valueId)) {
        return false;
    }

    bool retcode = false;

    _values[valueId].commit();

    // valueId 毎に設定値の反映処理を記述する
    QVariant value = _values[valueId].getValue();

    switch (valueId) {
    case VALUE_DATETIME: {
        QList<QVariant> data = value.toList();
        int yyyy = data[0].toInt();
        int mm = data[1].toInt();
        int dd = data[2].toInt();
        int week = data[3].toInt();
        int hour = data[4].toInt();
        int minutes = data[5].toInt();

        lclk_err_t ret = _communicator->setDateTime(yyyy, mm, dd, week, hour, minutes);
        if (ret == LCLK_I2C_ERR) {
            // I2C通信エラー発生時はリトライ
            ret = _communicator->setDateTime(yyyy, mm, dd, week, hour, minutes);
        }
        if (ret == LCLK_INACTIVE || ret == LCLK_SET_ERR || ret == LCLK_I2C_ERR) {
            retcode = true;
        }
        break;
    }

    case VALUE_AUTOPOWEROFF: {
        int powerOffTime = value.toInt();
        _communicator->setAutoPowerOffTime(powerOffTime);
        retcode = true;
        break;
    }

    case VALUE_MER_CN: {
        int mercnValue = value.toInt();
        _communicator->setMERCN(mercnValue);
        retcode = true;
        break;
    }

    case VALUE_CATVCHANNEL_FREQ: {
        int settingChannelFrequency = value.toInt();
        _communicator->setCATVChannelFrequency(settingChannelFrequency);
        retcode = true;
        break;
    }

    case VALUE_THRESHOLD: {
        // 閾値コミット処理
        QVariantList thresholdList = value.toList();
        _communicator->setThresholdData(thresholdList);
        retcode = true;
        break;
    }

    case VALUE_CONSTELLATION : {
        // コンスタレーション表示ポイント
        int constellationSelectNum = value.toInt();
        _communicator->setConstellationShowPoint(constellationSelectNum);
        retcode = true;
        break;
    }

    case VALUE_LEVEL_OFFSET: {
        // ケーブル損失コミット処理
        QVariantList cableLossList = value.toList();
        _communicator->setCableLossData(cableLossList);
        retcode = true;
        break;
    }

    case VALUE_SOUNDVOLUME: {
        int volumeLevel = value.toInt();
        _communicator->setSoundVolume(volumeLevel);
        retcode = true;
        break;
    }

    case VALUE_BRIGHTNESS: {
        // [Memo]
        //  VALUE_BRIGHTNESS コミット時
        //  設定ファイルとスクリーンの明るさを更新する。
        //   setBrightness       : 設定ファイルを更新
        //   setScreenBrightness : 実際にスクリーンの明るさを更新
        int brightnessLevel = value.toList().at(0).toInt();
        _communicator->setBrightness(brightnessLevel);
        _communicator->setScreenBrightness(brightnessLevel);
        retcode = true;
        break;
    }

    // ユーザー設定
    case VALUE_USERSETTING_FILELIST:
    case VALUE_USERSETTING_FILELIST_USB1:
    case VALUE_USERSETTING_FILELIST_USB2:
    {
        QList<LC70_LOGGER_SET_LOGGER_FLIST_ROW_T> filelist;

        foreach (const QVariant &item, value.toList()) {
            QVariantList src = item.toList();
            LC70_LOGGER_SET_LOGGER_FLIST_ROW_T dest;
            dest.ucNo = src.at(0).toInt();
            dest.ucEnbFlg = src.at(1).toInt();
            strncpy(dest.cFileName, src.at(2).toString().toUtf8().constData(), LC70_USER_USER_FNAME_STRLEN);

            filelist.append(dest);
        }

        LC70_ERR_T result = LC70_ERR;
        if (valueId == VALUE_USERSETTING_FILELIST) {
            result =_communicator->writeUserLogSetFileList(_dataloggerSettingDir, filelist, CHANNEL_EDIT_USER, _applicationSettings->settingDataUserSettingFileListName());
        } else if (valueId == VALUE_USERSETTING_FILELIST_USB1) {
            result =_communicator->writeUserLogSetFileList(_dataloggerSettingDir, filelist, CHANNEL_EDIT_USER_USB1, _applicationSettings->settingDataUserSettingUsb1FileListName());
        } else if (valueId == VALUE_USERSETTING_FILELIST_USB2) {
            result =_communicator->writeUserLogSetFileList(_dataloggerSettingDir, filelist, CHANNEL_EDIT_USER_USB2, _applicationSettings->settingDataUserSettingUsb2FileListName());
        }
        retcode = (result == LC70_OK);
        break;
    }

    case VALUE_RF_OPTICAL: {
        int rfOptical = value.toInt();
        _communicator->setRFOptical(rfOptical);
        retcode = true;
        break;
    }

    // 局部発振周波数
    case VALUE_LOCAL_FREQUENCY: {
        QVariantList localFrequencyList = value.toList();
        _communicator->setLocalFrequency(localFrequencyList);
        retcode = true;
        break;
    }

    // ケーブル損失有効
    case VALUE_LEVEL_OFFSET_ENABLE: {
        int isEnableCableLoss = value.toInt();
        _communicator->setIsEnableCableLoss(isEnableCableLoss);
        retcode = true;
        break;
    }

    // データロガー設定
    case VALUE_DATALOGGERSETTING_FILELIST:
    case VALUE_DATALOGGERSETTING_FILELIST_USB1:
    case VALUE_DATALOGGERSETTING_FILELIST_USB2:
    {
        QList<LC70_LOGGER_SET_LOGGER_FLIST_ROW_T> filelist;

        foreach (const QVariant &item, value.toList()) {
            QVariantList src = item.toList();
            LC70_LOGGER_SET_LOGGER_FLIST_ROW_T dest;
            dest.ucNo = src.at(0).toInt();
            dest.ucEnbFlg = src.at(1).toInt();
            strncpy(dest.cFileName, src.at(2).toString().toUtf8().constData(), LC70_LOGGER_LOGGER_FNAME_STRLEN);

            filelist.append(dest);
        }

        if (valueId == VALUE_DATALOGGERSETTING_FILELIST) {
            _communicator->writeUserLogSetFileList(_dataloggerSettingDir, filelist, CHANNEL_EDIT_DATA_LOGGER, _applicationSettings->settingDataLoggerSettingFileListName());
        } else if (valueId == VALUE_DATALOGGERSETTING_FILELIST_USB1) {
            _communicator->writeUserLogSetFileList(_dataloggerSettingDir, filelist, CHANNEL_EDIT_DATA_LOGGER_USB1, _applicationSettings->settingDataLoggerSettingUsb1FileListName());
        } else if (valueId == VALUE_DATALOGGERSETTING_FILELIST_USB2) {
            _communicator->writeUserLogSetFileList(_dataloggerSettingDir, filelist, CHANNEL_EDIT_DATA_LOGGER_USB2, _applicationSettings->settingDataLoggerSettingUsb2FileListName());
        }
        retcode = true;
        break;
    }
    default:
        qDebug("unknown valueId : %d", valueId);
        retcode = false;
        break;
    }

    return retcode;
}

void ModelOperator::setDataLoggerUserSettingDir(const QString &dirname)
{
    qDebug("logger dir : %s", dirname.toUtf8().constData());
    _dataloggerSettingDir = dirname;
}

void ModelOperator::initializeProduction(bool defaultInit)
{
    // VALUE_DATETIME
    {

    }
    // VALUE_AUTOPOWEROFF
    {
        LC70_SYS_SET_POWOFF_T lc70Data;

        if (0 > _communicator->readSystemSettingFile(LC70_FILE_ID_SYS_SET_POWOFF, &lc70Data, defaultInit)) {
        }

        QVariant data;
        data = lc70Data.sysSetPowoffRow[0].ucPowerOff;
        _values[VALUE_AUTOPOWEROFF].setInitialValue(data);
    }

    // VALUE_SOUNDVOLUME
    {
        LC70_SYS_SET_VOLUME_T lc70Data;

        if (0 > _communicator->readSystemSettingFile(LC70_FILE_ID_SYS_SET_VOLUME, &lc70Data, defaultInit)) {
        }

        QVariant data;
        data = lc70Data.sysSetVolumeRow[0].ucVolume;
        _values[VALUE_SOUNDVOLUME].setInitialValue(data);
    }

    // VALUE_MER_CN
    {
        LC70_MEAS_SET_CNRMER_T lc70Data;

        if (0 > _communicator->readMeasureSettingFile(LC70_FILE_ID_MEAS_SET_CNRMER, &lc70Data, defaultInit)) {
        }

        QVariantList data;

        for (int i = 0; i < LC70_MEAS_SET_CNRMER_IDX_MAX; ++i) {
            LC70_MEAS_SET_CNRMER_ROW_T row = lc70Data.measSetCnrmerRow[i];

            QVariantList value = {
                row.ucSelCnMerSelect[0],    // LC70_MEAS_SET_CNRMER_SELECT_NUM で定義されている分だけ要素数がある
                row.ucSelCnMerSelect[1],
                row.ucSelCnMer
            };
            data.insert(i, value);
        }
        _values[VALUE_MER_CN].setInitialValue(data);
    }

    // VALUE_CATVCHANNEL_FREQ
    {
        // 選択番号だけ取得
        LC70_MEAS_SET_CATV_FREQ_T lc70Data;

        if (0 > _communicator->readMeasureSettingFile(LC70_FILE_ID_MEAS_SET_CATV_FREQ, &lc70Data, defaultInit)) {
        }

        int data = -1;

        for (int i = 0; i < LC70_MEAS_SET_CATV_FREQ_IDX_MAX; ++i) {
            if (lc70Data.measSetCatvFreqRow[i].ucUse == 1) {
                data = i;
                break;
            }
        }

        // [Memo]
        //  現在CATVの値がどっちであるかを保持しておく
        //  dataの値が、想定する範囲外(0~3ではない)の場合または、設定1の場合 (0: 設定1 QAM, 1: 設定1 OFDM)は 1
        //  dataの値が設定2の場合 (2: 設定2 QAM, 3: 設定2 OFDM) は2を保持する。
        QVariant settingNo;

        if (data <= 1 || data > 3) {
            settingNo = 1;
        } else {
            settingNo = 2;
        }

        _values[VALUE_CATVCHANNEL_FREQ].setInitialValue(settingNo);
    }

    // VALUE_RF_OPTICAL
    {
        LC70_MEAS_SET_INPUT_T lc70Data;

        if (0 > _communicator->readMeasureSettingFile(LC70_FILE_ID_MEAS_SET_INPUT,  &lc70Data, defaultInit)) {
        }

        QVariant data;
        data = lc70Data.measSetInputRow[0].ucInput;
        _values[VALUE_RF_OPTICAL].setInitialValue(data);
    }

    // VALUE_THRESHOLD
    // VALUE_MERCNR_LIMITS
    {
        LC70_MEAS_SET_THRD_T lc70Data;

        if (0 > _communicator->readMeasureSettingFile(LC70_FILE_ID_MEAS_SET_THRD, &lc70Data, defaultInit)) {
        }

        QVariantList dataThreshold;
        QVariantList valueThreshold;

        QVariantList dataCnrMerLimits;
        QVariantList valueCnrMerLimits;

        for (int i = 0; i < LC70_MEAS_SET_THRD_IDX_MAX; ++i) {
            LC70_MEAS_SET_THRD_ROW_T *item = &lc70Data.measSetThrdRow[i];

            valueThreshold = {
                item->ucBand,              // バンド
                item->level.lLow,          // LEVEL  受信不可 閾値
                item->level.lHigh,         // LEVEL  受信可   閾値
                item->mer.ulLow,           // MER/CN 余裕度低 閾値
                item->mer.ulHigh,          // MER/CN 余裕度高 閾値
                item->ber.low.ulMantissa,  // BER 余裕度低 閾値 仮数部
                item->ber.low.lExponent,   // BER 余裕度低 閾値 指数部
                item->ber.high.ulMantissa, // BER 余裕度高 閾値 仮数部
                item->ber.high.lExponent   // BER 余裕度高 閾値 指数部
            };
            dataThreshold.insert(i, valueThreshold);

            valueCnrMerLimits = {
                item->mer.ulMin,           // MER/CN 余裕度最小値 閾値
                item->mer.ulMax           // MER/CN 余裕度最大値 閾値
            };

            dataCnrMerLimits.insert(i, valueCnrMerLimits);
        }

        _values[VALUE_THRESHOLD].setInitialValue(dataThreshold);
        _values[VALUE_MERCNR_LIMITS].setInitialValue(dataCnrMerLimits);
    }

    // VALUE_CONSTELLATION
    {
        LC70_MEAS_SET_CONSTA_NUM_T lc70Data;
        if (_communicator->readMeasureSettingFile(LC70_FILE_ID_MEAS_SET_CONSTA_NUM, &lc70Data, defaultInit)) {
        }

        LC70_MEAS_SET_CONSTA_NUM_ROW_T *item = &lc70Data.measSetConstaNumRow[0];

        QVariant data = item->ulConstaNum;
        _values[VALUE_CONSTELLATION].setInitialValue(data);
    }
    // VALUE_LEVEL_OFFSET
    {
        LC70_MEAS_SET_CABLE_LOSS_T lc70Data;

        if (0 > _communicator->readMeasureSettingFile(LC70_FILE_ID_MEAS_SET_CABLE_LOSS, &lc70Data, defaultInit)) {
        }

        QVariantList data;

        for (int i = 0; i < LC70_MEAS_SET_CABLE_LOSS_IDX_MAX; ++i) {
            LC70_MEAS_SET_CABLE_LOSS_ROW_T *item = &lc70Data.measSetCableLossRow[i];
            data.insert(i, item->ulCorrectionValue);
        }

        _values[VALUE_LEVEL_OFFSET].setInitialValue(data);
    }
    // VALUE_BRIGHTNESS
    {
        LC70_SYS_SET_BRIGHTNESS_T lc70Data;

        if (0 > _communicator->readMeasureSettingFile(LC70_FILE_ID_SYS_SET_BRIGHTNESS, &lc70Data, defaultInit)) {
        }

        QVariantList data;

        for (int i = 0; i < LC70_SYS_SET_BRIGHTNESS_IDX_MAX; ++i) {
            LC70_SYS_SET_BRIGHTNESS_ROW_T *item = &lc70Data.sysSetBrightnessRow[i];
            data.insert(i, item->ucBrightness);
        }

        _values[VALUE_BRIGHTNESS].setInitialValue(data);

    }

    // VALUE_USERSETTING_FILELIST
    {
        QList<LC70_LOGGER_SET_LOGGER_FLIST_ROW_T> filelist;

        QVariantList data;
        QVariantList value;

        // USER  設定ファイル名リストは取得時に読み込み
        _communicator->readUserLogSetFileList(_dataloggerSettingDir, filelist,
                                              CHANNEL_EDIT_USER,
                                              _applicationSettings->settingDataPathUserData(),
                                              _applicationSettings->settingDataUserSettingFileListName());

        int index = 0;

        foreach (const LC70_LOGGER_SET_LOGGER_FLIST_ROW_T &item, filelist) {
            value = {item.ucNo, item.ucEnbFlg, item.cFileName};
            data.insert(index, value);
            index++;
        }

        _values[VALUE_USERSETTING_FILELIST].setInitialValue(data);
    }

    // VALUE_USERSETTING_FILELIST_USB1
    {
        QList<LC70_LOGGER_SET_LOGGER_FLIST_ROW_T> filelist;

        QVariantList data;
        QVariantList value;

        // USER  設定ファイル名リストは取得時に読み込み
        _communicator->readUserLogSetFileList(_dataloggerSettingDir, filelist,
                                              CHANNEL_EDIT_USER_USB1,
                                              _applicationSettings->settingDataPathUserDataUsb1(),
                                              _applicationSettings->settingDataUserSettingUsb1FileListName());

        int index = 0;

        foreach (const LC70_LOGGER_SET_LOGGER_FLIST_ROW_T &item, filelist) {
            value = {item.ucNo, item.ucEnbFlg, item.cFileName};
            data.insert(index, value);
            index++;
        }

        _values[VALUE_USERSETTING_FILELIST_USB1].setInitialValue(data);
    }

    // VALUE_USERSETTING_FILELIST_USB2
    {
        QList<LC70_LOGGER_SET_LOGGER_FLIST_ROW_T> filelist;

        QVariantList data;
        QVariantList value;

        // USER  設定ファイル名リストは取得時に読み込み
        _communicator->readUserLogSetFileList(_dataloggerSettingDir, filelist,
                                              CHANNEL_EDIT_USER_USB2,
                                              _applicationSettings->settingDataPathUserDataUsb2(),
                                              _applicationSettings->settingDataUserSettingUsb2FileListName());

        int index = 0;

        foreach (const LC70_LOGGER_SET_LOGGER_FLIST_ROW_T &item, filelist) {
            value = {item.ucNo, item.ucEnbFlg, item.cFileName};
            data.insert(index, value);
            index++;
        }

        _values[VALUE_USERSETTING_FILELIST_USB2].setInitialValue(data);
    }

    // VALUE_LOCAL_FREQUENCY
    {
        LC70_MEAS_SET_LOCAL_FREQ_T lc70Data;

        if (0 > _communicator->readSystemSettingFile(LC70_FILE_ID_MEAS_SET_LOCAL_FREQ, &lc70Data, defaultInit)) {
        }

        QVariantList data;
        QVariantList value;

        for (int i = 0; i < LC70_MEAS_SET_LOCAL_FREQ_IDX_MAX; ++i) {
            LC70_MEAS_SET_LOCAL_FREQ_ROW_T *item = &lc70Data.measSetLocalFreqRow[i];

            value = {item->ucBand,         // バンド
                     item->ucWave,         // 偏波
                     item->ulLocalFreq1,   // 局部発振周波数1[kHz]
                     item->ulLocalFreq2,   // 局部発振周波数2[kHz]
                     item->ulLocalFreq3,   // 局部発振周波数3[kHz]
                     item->ulLocalFreq4,   // 局部発振周波数4[kHz]
                     item->ulLocalFreq5,   // 局部発振周波数5[kHz]
                     item->ulLocalFreqUser // 局部発振周波数ユーザ設定可能[kHz]
                    };
            data.insert(i, value);
        }

        _values[VALUE_LOCAL_FREQUENCY].setInitialValue(data);
    }

    // VALUE_LEVEL_OFFSET_ENABLE
    {
        LC70_MEAS_SET_CABLE_LOSS_ENB_T lc70Data;

        if (0 > _communicator->readMeasureSettingFile(LC70_FILE_ID_MEAS_SET_CABLE_LOSS_ENB, &lc70Data, defaultInit)) {
        }

        QVariant data = lc70Data.measSetCableLossEnbRow[0].ucCorrectionEnbFlg;
        _values[VALUE_LEVEL_OFFSET_ENABLE].setInitialValue(data);
    }

    // VALUE_BROADBANDSYSTEM_MODULATION
    {
        LC70_MEAS_SET_MANU_BDSYS_MOD_T lc70Data;

        if (0 > _communicator->readMeasureSettingFile(LC70_FILE_ID_MEAS_SET_MANU_BDSYS_MOD, &lc70Data, defaultInit)) {
        }

        QVariantList data;

        for (int i = 0; i < LC70_MEAS_SET_MANU_BDSYS_MOD_IDX_MAX; ++i) {
            LC70_MEAS_SET_MANU_BDSYS_MOD_ROW_T &row = lc70Data.measSetManuBdSysMod[i];

            QVariantList value = {
                row.usBdSys,
                row.ulMod
            };
            data.insert(i, value);
        }
        _values[VALUE_BROADBANDSYSTEM_MODULATION].setInitialValue(data);
    }

    // VALUE_DATALOGGERSETTING_FILELIST
    {
        QList<LC70_LOGGER_SET_LOGGER_FLIST_ROW_T> filelist;

        QVariantList data;
        QVariantList value;

        _communicator->readUserLogSetFileList(_dataloggerSettingDir, filelist,
                                              CHANNEL_EDIT_DATA_LOGGER,
                                              _applicationSettings->settingDataPathLogger(),
                                              _applicationSettings->settingDataLoggerSettingFileListName());

        int index = 0;

        foreach (const LC70_LOGGER_SET_LOGGER_FLIST_ROW_T &item, filelist) {
            value = {item.ucNo, item.ucEnbFlg, item.cFileName};
            data.insert(index, value);
            index++;
        }

        _values[VALUE_DATALOGGERSETTING_FILELIST].setInitialValue(data);
    }

    // VALUE_DATALOGGERSETTING_FILELIST_USB1
    {
        QList<LC70_LOGGER_SET_LOGGER_FLIST_ROW_T> filelist;

        QVariantList data;
        QVariantList value;

        _communicator->readUserLogSetFileList(_dataloggerSettingDir, filelist,
                                              CHANNEL_EDIT_DATA_LOGGER_USB1,
                                              _applicationSettings->settingDataPathLoggerUsb1(),
                                              _applicationSettings->settingDataLoggerSettingUsb1FileListName());

        int index = 0;

        foreach (const LC70_LOGGER_SET_LOGGER_FLIST_ROW_T &item, filelist) {
            value = {item.ucNo, item.ucEnbFlg, item.cFileName};
            data.insert(index, value);
            index++;
        }

        _values[VALUE_DATALOGGERSETTING_FILELIST_USB1].setInitialValue(data);
    }

    // VALUE_DATALOGGERSETTING_FILELIST_USB2
    {
        QList<LC70_LOGGER_SET_LOGGER_FLIST_ROW_T> filelist;

        QVariantList data;
        QVariantList value;

        _communicator->readUserLogSetFileList(_dataloggerSettingDir, filelist,
                                              CHANNEL_EDIT_DATA_LOGGER_USB2,
                                              _applicationSettings->settingDataPathLoggerUsb2(),
                                              _applicationSettings->settingDataLoggerSettingUsb2FileListName());

        int index = 0;

        foreach (const LC70_LOGGER_SET_LOGGER_FLIST_ROW_T &item, filelist) {
            value = {item.ucNo, item.ucEnbFlg, item.cFileName};
            data.insert(index, value);
            index++;
        }

        _values[VALUE_DATALOGGERSETTING_FILELIST_USB2].setInitialValue(data);
    }
}

void ModelOperator::initializeDevelop(bool defaultInit)
{
    Q_UNUSED(defaultInit);

    // VALUE_DATETIME
    {
        QList<QVariant> data = {2000, 07, 30, 14, 05, 00};
        QVariant dateTime(data);
        _values[VALUE_DATETIME].setInitialValue(data);
    }

    // VALUE_AUTOPOWEROFF
    {
        // 定義ファイル システム設定シートから
        QVariant data(15);
        _values[VALUE_AUTOPOWEROFF].setInitialValue(data);
    }

    // VALUE_SOUNDVOLUME
    {
        QVariant data(3);
        _values[VALUE_SOUNDVOLUME].setInitialValue(data);
    }

    // VALUE_MER_CN
    {
        QVariantList data;

        data.insert(0, QVariantList() << 0 << 1 << 1);
        data.insert(1, QVariantList() << 0 << 1 << 0);
        data.insert(2, QVariantList() << 0 << 1 << 0);
        data.insert(3, QVariantList() << 0 << 1 << 1);
        data.insert(4, QVariantList() << 0 << 1 << 1);
        data.insert(5, QVariantList() << 0 << 1 << 1);
        data.insert(6, QVariantList() << 0 << 1 << 1);
        data.insert(7, QVariantList() << 0 << 1 << 0);
        data.insert(8, QVariantList() << 0 << 1 << 0);
        data.insert(9, QVariantList() << 0 << 1 << 0);
        data.insert(10, QVariantList() << 0 << 1 << 0);
        data.insert(11, QVariantList() << 255 << 255 << 255);
        data.insert(12, QVariantList() << 255 << 255 << 255);
        data.insert(13, QVariantList() << 255 << 255 << 255);
        data.insert(14, QVariantList() << 255 << 255 << 255);

        _values[VALUE_MER_CN].setInitialValue(data);
    }

    // VALUE_CATVCHANNEL_FREQ
    {
        QVariant data(1);
        _values[VALUE_CATVCHANNEL_FREQ].setInitialValue(data);
    }

    // VALUE_RF_OPTICAL
    {
        QVariant data(0);
        _values[VALUE_RF_OPTICAL].setInitialValue(data);
    }

    // VALUE_OPTION_OPTICAL
    {
        QVariant data(0);
        _values[VALUE_OPTION_OPTICAL].setInitialValue(data);
    }

    // VALUE_THRESHOLD
    {
        QVariantList data;
        QVariantList value;

        // BAND毎の閾値リスト->定義ファイル.xlsx 測定設定定義シートから
        // BAND, LEVEL(受信不可, 受信可), MER(余裕度低, 余裕度高), BER(余裕度低_仮数部, 余裕度低_指数部, 余裕度高_仮数部, 余裕度高_指数部)
        value = {LC70_BAND_UHF, 250, 450, 200, 250, 200, -6, 100, -9};
        data.insert(LC70_MEAS_SET_THRD_IDX_UHF_ISDBT, value);

        // BS ISDBS
        value = {LC70_BAND_BS, 250, 500, 110, 170, 200, -6, 100, -9};
        data.insert(LC70_MEAS_SET_THRD_IDX_BS_ISDBS, value);

        // BS ISDBS3
        value = {LC70_BAND_BS, 250, 500, 110, 170, 200, -6, 100, -9};
        data.insert(LC70_MEAS_SET_THRD_IDX_BS_ISDBS3, value);

        // CS ISDBS
        value = {LC70_BAND_CS, 250, 500, 60, 110, 200, -6, 100, -9};
        data.insert(LC70_MEAS_SET_THRD_IDX_CS_ISDBS, value);

        // CS ISDBS3
        value = {LC70_BAND_CS, 250, 500, 60, 110, 200, -6, 100, -9};
        data.insert(LC70_MEAS_SET_THRD_IDX_CS_ISDBS3, value);

        // JCSAT3 DVBS
        value = {LC70_BAND_JCSAT3, 250, 500, 60, 100, 200, -6, 100, -9};
        data.insert(LC70_MEAS_SET_THRD_IDX_JCSAT3_DVBS, value);

        // JCSAT3 DVBS2
        value = {LC70_BAND_JCSAT3, 250, 500, 80, 100, 200, -6, 100, -9};
        data.insert(LC70_MEAS_SET_THRD_IDX_JCSAT3_DVBS2, value);

        // JCSAT4 DVBS
        value = {LC70_BAND_JCSAT4, 250, 500, 60, 100, 200, -6, 100, -9};
        data.insert(LC70_MEAS_SET_THRD_IDX_JCSAT4_DVBS, value);

        // JCSAT4 DVBS2
        value = {LC70_BAND_JCSAT4, 250, 500, 80, 100, 200, -6, 100, -9};
        data.insert(LC70_MEAS_SET_THRD_IDX_JCSAT4_DVBS2, value);

        // SCC-B DVBS
        value = {LC70_BAND_SCCB, 250, 500, 60, 100, 200, -6, 100, -9};
        data.insert(LC70_MEAS_SET_THRD_IDX_SCCB_DVBS, value);

        // SCC-B DVBS2
        value = {LC70_BAND_SCCB, 250, 500, 80, 100, 200, -6, 100, -9};
        data.insert(LC70_MEAS_SET_THRD_IDX_SCCB_DVBS2, value);

        // SCC-C DVBS
        value = {LC70_BAND_SCCC, 250, 500, 60, 100, 200, -6, 100, -9};
        data.insert(LC70_MEAS_SET_THRD_IDX_SCCC_DVBS, value);

        // SCC-C DVBS2
        value = {LC70_BAND_SCCC, 250, 500, 80, 100, 200, -6, 100, -9};
        data.insert(LC70_MEAS_SET_THRD_IDX_SCCC_DVBS2, value);

        // CATV ISDBT OFDM
        // ラッパーライブラリ V2.00 対応
        value = {LC70_BAND_CATV_ALL, 250, 460, 200, 250, 200, -6, 100, -9};
        data.insert(LC70_MEAS_SET_THRD_IDX_CATV_ISDBT_OFDM, value);

        // CATV ANNEXC 64QAM
        // ラッパーライブラリ V2.00 対応
        value = {LC70_BAND_CATV_ALL, 250, 490, 240, 280, 200, -6, 100, -9};
        data.insert(LC70_MEAS_SET_THRD_IDX_CATV_ANNEXC_64QAM, value);

        // CATV ANNEXC 256QAM
        // ラッパーライブラリ V2.00 対応
        value = {LC70_BAND_CATV_ALL, 250, 490, 240, 280, 200, -6, 100, -9};
        data.insert(LC70_MEAS_SET_THRD_IDX_CATV_ANNEXC_256QAM, value);

        // CATV ANNEXB 64QAM
        // ラッパーライブラリ V2.02 対応
        value = {LC70_BAND_CATV_ALL, 250, 570, 300, 340, 200, -6, 100, -9};
        data.insert(LC70_MEAS_SET_THRD_IDX_CATV_ANNEXB_64QAM, value);

        // CATV ANNEXB 256QAM
        // ラッパーライブラリ V2.02 対応
        value = {LC70_BAND_CATV_ALL, 250, 570, 300, 340, 200, -6, 100, -9};
        data.insert(LC70_MEAS_SET_THRD_IDX_CATV_ANNEXB_256QAM, value);

        // CATV PILOT
        value = {LC70_BAND_PILOT, 250, 500, 0, 0, 0, 0, 0, 0};
        data.insert(LC70_MEAS_SET_THRD_IDX_CATV_PILOT, value);

        // CATV スカパー光 OFDM
        value = {LC70_BAND_SKYP, 250, 450, 200, 250, 200, -6, 100, -9};
        data.insert(LC70_MEAS_SET_THRD_IDX_CATV_SKYP_OFDM, value);

        // CATV スカパー光 256QAM
        value = {LC70_BAND_SKYP, 250, 570, 300, 340, 200, -6, 100, -9};
        data.insert(LC70_MEAS_SET_THRD_IDX_CATV_SKYP_256QAM, value);

        // LTE
        value = {LC70_BAND_LTE, 200, 200, 0, 0, 0, 0, 0, 0};
        data.insert(LC70_MEAS_SET_THRD_IDX_LTE, value);

        // FM
        value = {LC70_BAND_FM, 600, 900, 0, 0, 0, 0, 0, 0};
        data.insert(LC70_MEAS_SET_THRD_IDX_FM, value);

        // CATV上り
        value = {LC70_BAND_CATV_UP_STREAM, 250, 500, 0, 0, 0, 0, 0, 0};
        data.insert(LC70_MEAS_SET_THRD_IDX_CATV_UP_STREAM, value);

        _values[VALUE_THRESHOLD].setInitialValue(data);
    }

    // VALUE_CONSTELLATION
    {
        QVariant data(LC70_MEAS_SET_CONSTA_NUM_2000);
        _values[VALUE_CONSTELLATION].setInitialValue(data);
    }

    // VALUE_LEVEL_OFFSET
    {
        // ケーブル損失補正
        QVariantList data;
        data = {0, 0, 0, 0};
        _values[VALUE_LEVEL_OFFSET].setInitialValue(data);
    }

    // VALUE_BRIGHTNESS
    {
        QVariantList data;

        // 定義ファイル システム設定シートから
        data.insert(0, 2);

        _values[VALUE_BRIGHTNESS].setInitialValue(data);
    }

    // VALUE_USERSETTING_FILELIST
    {
        QList<LC70_LOGGER_SET_LOGGER_FLIST_ROW_T> filelist;

        QVariantList data;
        QVariantList value;

        // USER 設定ファイル名リストは取得時に読み込み
        _communicator->readUserLogSetFileList(_dataloggerSettingDir, filelist,
                                              CHANNEL_EDIT_USER,
                                              _applicationSettings->settingDataPathUserData(),
                                              _applicationSettings->settingDataUserSettingFileListName());

        int index = 0;

        foreach (const LC70_LOGGER_SET_LOGGER_FLIST_ROW_T &item, filelist) {
            QString fileName = item.cFileName;
#if !Q_PROCESSOR_ARM
            if (fileName.contains(CSV_FILE_EXTENSION)) {
                fileName = fileName.left(fileName.length() - qstrlen(CSV_FILE_EXTENSION));
            }
#endif
            value = {item.ucNo, item.ucEnbFlg, fileName.toLocal8Bit().constData()};
            data.insert(index, value);
            index++;
        }

        _values[VALUE_USERSETTING_FILELIST].setInitialValue(data);
    }

    // VALUE_USERSETTING_FILELIST_USB1
    {
        QList<LC70_LOGGER_SET_LOGGER_FLIST_ROW_T> filelist;

        QVariantList data;
        QVariantList value;

        // USER 設定ファイル名リストは取得時に読み込み
        _communicator->readUserLogSetFileList(_dataloggerSettingDir, filelist,
                                              CHANNEL_EDIT_USER_USB1,
                                              _applicationSettings->settingDataPathUserDataUsb1(),
                                              _applicationSettings->settingDataUserSettingUsb1FileListName());

        int index = 0;

        foreach (const LC70_LOGGER_SET_LOGGER_FLIST_ROW_T &item, filelist) {
            QString fileName = item.cFileName;
#if !Q_PROCESSOR_ARM
            if (fileName.contains(CSV_FILE_EXTENSION)) {
                fileName = fileName.left(fileName.length() - qstrlen(CSV_FILE_EXTENSION));
            }
#endif
            value = {item.ucNo, item.ucEnbFlg, fileName.toLocal8Bit().constData()};
            data.insert(index, value);
            index++;
        }

        _values[VALUE_USERSETTING_FILELIST_USB1].setInitialValue(data);
    }

    // VALUE_USERSETTING_FILELIST_USB2
    {
        QList<LC70_LOGGER_SET_LOGGER_FLIST_ROW_T> filelist;

        QVariantList data;
        QVariantList value;

        // USER 設定ファイル名リストは取得時に読み込み
        _communicator->readUserLogSetFileList(_dataloggerSettingDir, filelist,
                                              CHANNEL_EDIT_USER_USB2,
                                              _applicationSettings->settingDataPathUserDataUsb2(),
                                              _applicationSettings->settingDataUserSettingUsb2FileListName());

        int index = 0;

        foreach (const LC70_LOGGER_SET_LOGGER_FLIST_ROW_T &item, filelist) {
            QString fileName = item.cFileName;
#if !Q_PROCESSOR_ARM
            if (fileName.contains(CSV_FILE_EXTENSION)) {
                fileName = fileName.left(fileName.length() - qstrlen(CSV_FILE_EXTENSION));
            }
#endif
            value = {item.ucNo, item.ucEnbFlg, fileName.toLocal8Bit().constData()};
            data.insert(index, value);
            index++;
        }

        _values[VALUE_USERSETTING_FILELIST_USB2].setInitialValue(data);
    }

    // VALUE_LOCAL_FREQUENCY
    // see LC70_MEAS_SET_LOCAL_FREQ_ROW_T
    {
        QVariantList data;
        QVariantList value;
        // BS
        value = {LC70_BAND_BS, LC70_MEAS_SET_WAVE_R, uint(10678000), LOCAL_FREQ_UNDEFINED_DATA, LOCAL_FREQ_UNDEFINED_DATA, LOCAL_FREQ_UNDEFINED_DATA, LOCAL_FREQ_UNDEFINED_DATA, LOCAL_FREQ_UNDEFINED_DATA};
        data.insert(LC70_MEAS_SET_LOCAL_FREQ_IDX_BS_R, value);
        value = {LC70_BAND_BS, LC70_MEAS_SET_WAVE_L, uint(9505000), LOCAL_FREQ_UNDEFINED_DATA, LOCAL_FREQ_UNDEFINED_DATA, LOCAL_FREQ_UNDEFINED_DATA, LOCAL_FREQ_UNDEFINED_DATA, LOCAL_FREQ_UNDEFINED_DATA};
        data.insert(LC70_MEAS_SET_LOCAL_FREQ_IDX_BS_L, value);
        // CS
        value = {LC70_BAND_CS, LC70_MEAS_SET_WAVE_R, uint(10678000), LOCAL_FREQ_UNDEFINED_DATA, LOCAL_FREQ_UNDEFINED_DATA, LOCAL_FREQ_UNDEFINED_DATA, LOCAL_FREQ_UNDEFINED_DATA, LOCAL_FREQ_UNDEFINED_DATA};
        data.insert(LC70_MEAS_SET_LOCAL_FREQ_IDX_CS_R, value);
        value = {LC70_BAND_CS, LC70_MEAS_SET_WAVE_L, uint(9505000), uint(10127000), LOCAL_FREQ_UNDEFINED_DATA, LOCAL_FREQ_UNDEFINED_DATA, LOCAL_FREQ_UNDEFINED_DATA, LOCAL_FREQ_UNDEFINED_DATA};
        data.insert(LC70_MEAS_SET_LOCAL_FREQ_IDX_BS_L, value);

        // JCSAT3
        value = {LC70_BAND_JCSAT3, LC70_MEAS_SET_WAVE_H, uint(10678000), LOCAL_FREQ_UNDEFINED_DATA, LOCAL_FREQ_UNDEFINED_DATA, uint(11200000), uint(11300000), uint(0)};
        data.insert(LC70_MEAS_SET_LOCAL_FREQ_IDX_JCSAT3_H, value);
        value = {LC70_BAND_JCSAT3, LC70_MEAS_SET_WAVE_V, uint(10678000), LOCAL_FREQ_UNDEFINED_DATA, LOCAL_FREQ_UNDEFINED_DATA, uint(11200000), uint(11300000), uint(0)};
        data.insert(LC70_MEAS_SET_LOCAL_FREQ_IDX_JCSAT3_V, value);

        // JCSAT4
        value = {LC70_BAND_JCSAT4, LC70_MEAS_SET_WAVE_H, uint(10678000), uint(10873000), uint(10990000), uint(11200000), uint(11300000), uint(0)};
        data.insert(LC70_MEAS_SET_LOCAL_FREQ_IDX_JCSAT4_H, value);
        value = {LC70_BAND_JCSAT4, LC70_MEAS_SET_WAVE_V, uint(10678000), uint(10873000), uint(10990000), uint(11200000), uint(11300000), uint(0)};
        data.insert(LC70_MEAS_SET_LOCAL_FREQ_IDX_JCSAT4_V, value);

        // SCC-B2
        value = {LC70_BAND_SCCB, LC70_MEAS_SET_WAVE_H, uint(10678000), LOCAL_FREQ_UNDEFINED_DATA, LOCAL_FREQ_UNDEFINED_DATA, uint(11200000), uint(11300000), uint(0)};
        data.insert(LC70_MEAS_SET_LOCAL_FREQ_IDX_SCCB_H, value);
        value = {LC70_BAND_SCCB, LC70_MEAS_SET_WAVE_V, uint(10678000), LOCAL_FREQ_UNDEFINED_DATA, LOCAL_FREQ_UNDEFINED_DATA, uint(11200000), uint(11300000), uint(0)};
        data.insert(LC70_MEAS_SET_LOCAL_FREQ_IDX_SCCB_V, value);

        // SCC-C2
        value = {LC70_BAND_SCCC, LC70_MEAS_SET_WAVE_H, uint(10678000), LOCAL_FREQ_UNDEFINED_DATA, LOCAL_FREQ_UNDEFINED_DATA, uint(11200000), uint(11300000), uint(0)};
        data.insert(LC70_MEAS_SET_LOCAL_FREQ_IDX_SCCC_H, value);
        value = {LC70_BAND_SCCC, LC70_MEAS_SET_WAVE_V, uint(10678000), LOCAL_FREQ_UNDEFINED_DATA, LOCAL_FREQ_UNDEFINED_DATA, uint(11200000), uint(11300000), uint(0)};
        data.insert(LC70_MEAS_SET_LOCAL_FREQ_IDX_SCCC_V, value);

        _values[VALUE_LOCAL_FREQUENCY].setInitialValue(data);
    }

    // VALUE_LEVEL_OFFSET_ENABLE
    {
        // 定義ファイル 測定設定定義シートから
        QVariant data(1);

        _values[VALUE_LEVEL_OFFSET_ENABLE].setInitialValue(data);
    }

    // VALUE_BROADBANDSYSTEM_MODULATION
    {
        QVariantList data;
        data.insert(0, QVariantList() << LC70_BD_SYS_ISDBT << LC70_SETMOD_OFDM);
        data.insert(1, QVariantList() << LC70_BD_SYS_ISDBS << LC70_SETMOD_UNDEF);
        data.insert(2, QVariantList() << LC70_BD_SYS_ISDBS3 << LC70_SETMOD_UNDEF);
        data.insert(3, QVariantList() << LC70_BD_SYS_ISDBS << LC70_SETMOD_UNDEF);
        data.insert(4, QVariantList() << LC70_BD_SYS_ISDBS3 << LC70_SETMOD_UNDEF);
        data.insert(5, QVariantList() << LC70_BD_SYS_ISDBT << LC70_SETMOD_OFDM);
        data.insert(6, QVariantList() << LC70_BD_SYS_ANNEXC << LC70_SETMOD_64QAM);
        data.insert(7, QVariantList() << LC70_BD_SYS_ISDBT << LC70_SETMOD_OFDM);
        data.insert(8, QVariantList() << LC70_BD_SYS_OPTICAST << LC70_SETMOD_256QAM);
        data.insert(9, QVariantList() << LC70_BD_SYS_NONE << LC70_SETMOD_CW);
        data.insert(10, QVariantList() << LC70_BD_SYS_DVBS2 << LC70_SETMOD_UNDEF);
        data.insert(11, QVariantList() << LC70_BD_SYS_DVBS2 << LC70_SETMOD_UNDEF);
        data.insert(12, QVariantList() << LC70_BD_SYS_DVBS2 << LC70_SETMOD_UNDEF);
        data.insert(13, QVariantList() << LC70_BD_SYS_DVBS2 << LC70_SETMOD_UNDEF);
        data.insert(14, QVariantList() << LC70_BD_SYS_DVBS2 << LC70_SETMOD_UNDEF);
        data.insert(15, QVariantList() << LC70_BD_SYS_DVBS2 << LC70_SETMOD_UNDEF);
        data.insert(16, QVariantList() << LC70_BD_SYS_DVBS2 << LC70_SETMOD_UNDEF);
        data.insert(17, QVariantList() << LC70_BD_SYS_DVBS2 << LC70_SETMOD_UNDEF);
        data.insert(18, QVariantList() << LC70_BD_SYS_DVBS2 << LC70_SETMOD_UNDEF);
        data.insert(19, QVariantList() << LC70_BD_SYS_DVBS2 << LC70_SETMOD_UNDEF);
        data.insert(20, QVariantList() << LC70_BD_SYS_NONE << LC70_SETMOD_UNDEF);
        data.insert(21, QVariantList() << LC70_BD_SYS_NONE << LC70_SETMOD_UNDEF);
        data.insert(22, QVariantList() << LC70_BD_SYS_NONE << LC70_SETMOD_UNDEF);

        _values[VALUE_BROADBANDSYSTEM_MODULATION].setInitialValue(data);
    }

    // VALUE_MERCNR_LIMITS
    {
        QVariantList data = {};
        QVariantList value = {};

        // 定義ファイル_1.0.1 測定設定定義から値を設定
        value = {50,350};
        data.insert(LC70_MEAS_SET_THRD_IDX_UHF_ISDBT, value);

        value = {50,250};
        data.insert(LC70_MEAS_SET_THRD_IDX_BS_ISDBS, value);

        value = {50, 250};
        data.insert(LC70_MEAS_SET_THRD_IDX_BS_ISDBS3, value);

        value = {50, 250};
        data.insert(LC70_MEAS_SET_THRD_IDX_CS_ISDBS, value);

        value = {50, 250};
        data.insert(LC70_MEAS_SET_THRD_IDX_CS_ISDBS3, value);

        value = {50, 250};
        data.insert(LC70_MEAS_SET_THRD_IDX_JCSAT3_DVBS, value);

        value = {50, 250};
        data.insert(LC70_MEAS_SET_THRD_IDX_JCSAT3_DVBS2, value);

        value = {50, 250};
        data.insert(LC70_MEAS_SET_THRD_IDX_JCSAT4_DVBS, value);

        value = {50, 250};
        data.insert(LC70_MEAS_SET_THRD_IDX_JCSAT4_DVBS2, value);

        value = {50, 250};
        data.insert(LC70_MEAS_SET_THRD_IDX_SCCB_DVBS, value);

        value = {50, 250};
        data.insert(LC70_MEAS_SET_THRD_IDX_SCCB_DVBS2, value);

        value = {50, 250};
        data.insert(LC70_MEAS_SET_THRD_IDX_SCCC_DVBS, value);

        value = {50, 250};
        data.insert(LC70_MEAS_SET_THRD_IDX_SCCC_DVBS2, value);

        value = {200, 400};
        data.insert(LC70_MEAS_SET_THRD_IDX_CATV_ISDBT_OFDM, value);

        value = {200, 400};
        data.insert(LC70_MEAS_SET_THRD_IDX_CATV_ANNEXC_64QAM, value);

        value = {200, 400};
        data.insert(LC70_MEAS_SET_THRD_IDX_CATV_ANNEXC_256QAM, value);

        value = {200, 400};
        data.insert(LC70_MEAS_SET_THRD_IDX_CATV_ANNEXB_64QAM, value);

        value = {200, 400};
        data.insert(LC70_MEAS_SET_THRD_IDX_CATV_ANNEXB_256QAM, value);

        value = {0, 0};
        data.insert(LC70_MEAS_SET_THRD_IDX_CATV_PILOT, value);

        value = {50, 350};
        data.insert(LC70_MEAS_SET_THRD_IDX_CATV_SKYP_OFDM, value);

        value = {50, 350};
        data.insert(LC70_MEAS_SET_THRD_IDX_CATV_SKYP_256QAM, value);

        value = {0, 0};
        data.insert(LC70_MEAS_SET_THRD_IDX_LTE, value);

        value = {0, 0};
        data.insert(LC70_MEAS_SET_THRD_IDX_FM, value);

        value = {0, 0};
        data.insert(LC70_MEAS_SET_THRD_IDX_CATV_UP_STREAM, value);

        _values[VALUE_MERCNR_LIMITS].setInitialValue(data);
    }

    // VALUE_DATALOGGERSETTING_FILELIST
    {
        QList<LC70_LOGGER_SET_LOGGER_FLIST_ROW_T> filelist;

        QVariantList data;
        QVariantList value;

        _communicator->readUserLogSetFileList(_dataloggerSettingDir, filelist,
                                              CHANNEL_EDIT_DATA_LOGGER,
                                              _applicationSettings->settingDataPathLogger(),
                                              _applicationSettings->settingDataLoggerSettingFileListName());

        int index = 0;

        foreach (const LC70_LOGGER_SET_LOGGER_FLIST_ROW_T &item, filelist) {
            QString fileName = item.cFileName;
#if !Q_PROCESSOR_ARM
            if (fileName.contains(CSV_FILE_EXTENSION)) {
                fileName = fileName.left(fileName.length() - qstrlen(CSV_FILE_EXTENSION));
            }
#endif
            value = {item.ucNo, item.ucEnbFlg, fileName.toLocal8Bit().constData()};
            data.insert(index, value);
            index++;
        }

        _values[VALUE_DATALOGGERSETTING_FILELIST].setInitialValue(data);
    }

    // VALUE_DATALOGGERSETTING_FILELIST_USB1
    {
        QList<LC70_LOGGER_SET_LOGGER_FLIST_ROW_T> filelist;

        QVariantList data;
        QVariantList value;

        _communicator->readUserLogSetFileList(_dataloggerSettingDir, filelist,
                                              CHANNEL_EDIT_DATA_LOGGER_USB1,
                                              _applicationSettings->settingDataPathLoggerUsb1(),
                                              _applicationSettings->settingDataLoggerSettingUsb1FileListName());

        int index = 0;

        foreach (const LC70_LOGGER_SET_LOGGER_FLIST_ROW_T &item, filelist) {
            QString fileName = item.cFileName;
#if !Q_PROCESSOR_ARM
            if (fileName.contains(CSV_FILE_EXTENSION)) {
                fileName = fileName.left(fileName.length() - qstrlen(CSV_FILE_EXTENSION));
            }
#endif
            value = {item.ucNo, item.ucEnbFlg, fileName.toLocal8Bit().constData()};
            data.insert(index, value);
            index++;
        }

        _values[VALUE_DATALOGGERSETTING_FILELIST_USB1].setInitialValue(data);
    }

    // VALUE_DATALOGGERSETTING_FILELIST_USB2
    {
        QList<LC70_LOGGER_SET_LOGGER_FLIST_ROW_T> filelist;

        QVariantList data;
        QVariantList value;

        _communicator->readUserLogSetFileList(_dataloggerSettingDir, filelist,
                                              CHANNEL_EDIT_DATA_LOGGER_USB2,
                                              _applicationSettings->settingDataPathLoggerUsb2(),
                                              _applicationSettings->settingDataLoggerSettingUsb2FileListName());

        int index = 0;

        foreach (const LC70_LOGGER_SET_LOGGER_FLIST_ROW_T &item, filelist) {
            QString fileName = item.cFileName;
#if !Q_PROCESSOR_ARM
            if (fileName.contains(CSV_FILE_EXTENSION)) {
                fileName = fileName.left(fileName.length() - qstrlen(CSV_FILE_EXTENSION));
            }
#endif
            value = {item.ucNo, item.ucEnbFlg, fileName.toLocal8Bit().constData()};
            data.insert(index, value);
            index++;
        }

        _values[VALUE_DATALOGGERSETTING_FILELIST_USB2].setInitialValue(data);
    }
}

void ModelOperator::initializeChannelDefinitionDataProduction(bool defaultInit)
{
    // 地デジ
    {
        LC70_CH_SET_UHF_T lc70Data;

        if (0 > _communicator->readChannelConfigurationFile(LC70_FILE_ID_CH_SET_UHF, &lc70Data, defaultInit)) {
        }

        // ChannelDefectionItemのリストに変換する
        QList<ChannelDefinitionItem> definitionItemUHF = ChannelDefinitionItemHelper::convertChannelDefinitionItem(&lc70Data,
                LC70_FILE_ID_CH_SET_UHF);

        // マップデータに格納する
        _channelDefinitionMap.insert(LC70_FILE_ID_CH_SET_UHF, definitionItemUHF);
    }

    // BS
    {
        LC70_CH_SET_BS_T lc70Data;

        if (0 > _communicator->readChannelConfigurationFile(LC70_FILE_ID_CH_SET_BS, &lc70Data, defaultInit)) {
        }

        // ChannelDefictionItemのリストに変換する
        QList<ChannelDefinitionItem> definitionItemBS = ChannelDefinitionItemHelper::convertChannelDefinitionItem(&lc70Data,
                LC70_FILE_ID_CH_SET_BS);

        // マップデータに格納する
        _channelDefinitionMap.insert(LC70_FILE_ID_CH_SET_BS, definitionItemBS);
    }

    // CS
    {
        LC70_CH_SET_CS_T lc70Data;

        if (0 > _communicator->readChannelConfigurationFile(LC70_FILE_ID_CH_SET_CS, &lc70Data, defaultInit)) {
        }

        // ChannelDefictionItemのリストに変換する
        QList<ChannelDefinitionItem> definitionItemCS = ChannelDefinitionItemHelper::convertChannelDefinitionItem(&lc70Data,
                LC70_FILE_ID_CH_SET_CS);

        // マップデータに格納する
        _channelDefinitionMap.insert(LC70_FILE_ID_CH_SET_CS, definitionItemCS);
    }

    // SCCB
    {
        LC70_CH_SET_SCCB_T lc70Data;

        if (0 > _communicator->readChannelConfigurationFile(LC70_FILE_ID_CH_SET_SCCB, &lc70Data, defaultInit)) {
        }

        // ChannelDefictionItemのリストに変換する
        QList<ChannelDefinitionItem> definitionItemSCCB = ChannelDefinitionItemHelper::convertChannelDefinitionItem(&lc70Data,
                LC70_FILE_ID_CH_SET_SCCB);

        // マップデータに格納する
        _channelDefinitionMap.insert(LC70_FILE_ID_CH_SET_SCCB, definitionItemSCCB);

    }

    // SCCC
    {
        LC70_CH_SET_SCCC_T lc70Data;

        if (0 > _communicator->readChannelConfigurationFile(LC70_FILE_ID_CH_SET_SCCC, &lc70Data, defaultInit)) {
        }

        // ChannelDefictionItemのリストに変換する
        QList<ChannelDefinitionItem> definitionItemSCCC = ChannelDefinitionItemHelper::convertChannelDefinitionItem(&lc70Data,
                LC70_FILE_ID_CH_SET_SCCC);

        // マップデータに格納する
        _channelDefinitionMap.insert(LC70_FILE_ID_CH_SET_SCCC, definitionItemSCCC);
    }

    // JSAT3A
    {
        LC70_CH_SET_JSAT3A_T lc70Data;

        if (0 > _communicator->readChannelConfigurationFile(LC70_FILE_ID_CH_SET_JSAT3A, &lc70Data, defaultInit)) {
        }

        // ChannelDefictionItemのリストに変換する
        QList<ChannelDefinitionItem> definitionItemJSAT3A = ChannelDefinitionItemHelper::convertChannelDefinitionItem(&lc70Data,
                LC70_FILE_ID_CH_SET_JSAT3A);

        // マップデータに格納する
        _channelDefinitionMap.insert(LC70_FILE_ID_CH_SET_JSAT3A, definitionItemJSAT3A);
    }

    // JSAT4B
    {
        LC70_CH_SET_JSAT4B_T lc70Data;

        if (0 > _communicator->readChannelConfigurationFile(LC70_FILE_ID_CH_SET_JSAT4B, &lc70Data, defaultInit)) {
        }

        // ChannelDefictionItemのリストに変換する
        QList<ChannelDefinitionItem> definitionItemJSAT4B = ChannelDefinitionItemHelper::convertChannelDefinitionItem(&lc70Data,
                LC70_FILE_ID_CH_SET_JSAT4B);

        // マップデータに格納する
        _channelDefinitionMap.insert(LC70_FILE_ID_CH_SET_JSAT4B, definitionItemJSAT4B);
    }

    // ラッパーライブラリ V2.00 対応
    // CATV
    {
        LC70_CH_SET_CATV_ALL_T lc70Data;

        if (0 > _communicator->readChannelConfigurationFile(LC70_FILE_ID_CH_SET_CATV_ALL, &lc70Data, defaultInit)) {
        }

        // ChannelDefictionItemのリストに変換する
        QList<ChannelDefinitionItem> definitionItemCATV = ChannelDefinitionItemHelper::convertChannelDefinitionItem(&lc70Data,
                LC70_FILE_ID_CH_SET_CATV_ALL);

        // マップデータに格納する
        _channelDefinitionMap.insert(LC70_FILE_ID_CH_SET_CATV_ALL, definitionItemCATV);
    }

    // ラッパーライブラリ V2.00 対応
    {
        LC70_CH_SET_CATV_90_470MHZ_T lc70Data;

        if (0 > _communicator->readChannelConfigurationFile(LC70_FILE_ID_CH_SET_CATV_90_470MHZ, &lc70Data, defaultInit)) {
        }

        // ChannelDefictionItemのリストに変換する
        QList<ChannelDefinitionItem> definitionItemCATV_90_470MHZ = ChannelDefinitionItemHelper::convertChannelDefinitionItem(
                    &lc70Data,
                    LC70_FILE_ID_CH_SET_CATV_90_470MHZ);

        // マップデータに格納する
        _channelDefinitionMap.insert(LC70_FILE_ID_CH_SET_CATV_90_470MHZ, definitionItemCATV_90_470MHZ);
    }

    // ラッパーライブラリ V2.00 対応
    {
        LC70_CH_SET_CATV_470_1002MHZ_T lc70Data;

        if (0 > _communicator->readChannelConfigurationFile(LC70_FILE_ID_CH_SET_CATV_470_1002MHZ, &lc70Data, defaultInit)) {
        }

        // ChannelDefictionItemのリストに変換する
        QList<ChannelDefinitionItem> definitionItemCATV_470_1002MHZ = ChannelDefinitionItemHelper::convertChannelDefinitionItem(
                    &lc70Data,
                    LC70_FILE_ID_CH_SET_CATV_470_1002MHZ);

        // マップデータに格納する
        _channelDefinitionMap.insert(LC70_FILE_ID_CH_SET_CATV_470_1002MHZ, definitionItemCATV_470_1002MHZ);
    }

    // スカパー光
    {
        LC70_CH_SET_SKYP_T lc70Data;

        if (0 > _communicator->readChannelConfigurationFile(LC70_FILE_ID_CH_SET_SKYP, &lc70Data, defaultInit)) {
        }

        // ChannelDefictionItemのリストに変換する
        QList<ChannelDefinitionItem> definitionItemSKYP = ChannelDefinitionItemHelper::convertChannelDefinitionItem(&lc70Data,
                LC70_FILE_ID_CH_SET_SKYP);

        // マップデータに格納する
        _channelDefinitionMap.insert(LC70_FILE_ID_CH_SET_SKYP, definitionItemSKYP);
    }

    // PILOT
    {
        LC70_CH_SET_PILOT_T lc70Data;

        if (0 > _communicator->readChannelConfigurationFile(LC70_FILE_ID_CH_SET_PILOT, &lc70Data, defaultInit)) {
        }

        // ChannelDefictionItemのリストに変換する
        QList<ChannelDefinitionItem> definitionItemPILOT = ChannelDefinitionItemHelper::convertChannelDefinitionItem(&lc70Data,
                LC70_FILE_ID_CH_SET_PILOT);

        // マップデータに保持する
        _channelDefinitionMap.insert(LC70_FILE_ID_CH_SET_PILOT, definitionItemPILOT);
    }

    // LTE
    {
        LC70_CH_SET_LTE_T lc70Data;

        if (0 > _communicator->readChannelConfigurationFile(LC70_FILE_ID_CH_SET_LTE, &lc70Data, defaultInit )) {

        }

        // ChannelDefictionItemのリストに変換する
        QList<ChannelDefinitionItem> definitionItemLTE = ChannelDefinitionItemHelper::convertChannelDefinitionItem(&lc70Data,
                LC70_FILE_ID_CH_SET_LTE);

        // マップデータに保持する
        _channelDefinitionMap.insert(LC70_FILE_ID_CH_SET_LTE, definitionItemLTE);
    }

    // FM
    {
        LC70_CH_SET_FM_T lc70Data;

        if (0 > _communicator->readChannelConfigurationFile(LC70_FILE_ID_CH_SET_FM, &lc70Data, defaultInit )) {
        }

        // ChannelDefictionItemのリストに変換する
        QList<ChannelDefinitionItem> definitionItemFM = ChannelDefinitionItemHelper::convertChannelDefinitionItem(&lc70Data,LC70_FILE_ID_CH_SET_FM);

        // マップデータに保持する
        _channelDefinitionMap.insert(LC70_FILE_ID_CH_SET_FM, definitionItemFM);
    }

    // CATV上り
    {
        LC70_CH_SET_CATV_UP_STREAM_T lc70Data;

        if (0 > _communicator->readChannelConfigurationFile(LC70_FILE_ID_CH_SET_CATV_UP_STREAM, &lc70Data, defaultInit)) {
        }

        // ChannelDefictionItemのリストに変換する
        QList<ChannelDefinitionItem> definitionItemCATV_STREAM = ChannelDefinitionItemHelper::convertChannelDefinitionItem(&lc70Data, LC70_FILE_ID_CH_SET_CATV_UP_STREAM);

        // マップデータに保持する
        _channelDefinitionMap.insert(LC70_FILE_ID_CH_SET_CATV_UP_STREAM, definitionItemCATV_STREAM);
    }

    // アンテナ調整(JCSAT-3A)
    {
        LC70_CH_SET_JSAT3A_ANTENNA_T lc70Data;

        if (0 > _communicator->readChannelConfigurationFile(LC70_FILE_ID_CH_SET_JSAT3A_ANTENNA, &lc70Data, defaultInit)) {
        }

        // ChannelDefictionItemのリストに変換する
        QList<ChannelDefinitionItem> definitionItemJCSAT3A = ChannelDefinitionItemHelper::convertChannelDefinitionItem(&lc70Data,LC70_FILE_ID_CH_SET_JSAT3A_ANTENNA);

        // マップデータに保持する
        _channelDefinitionMap.insert(LC70_FILE_ID_CH_SET_JSAT3A_ANTENNA, definitionItemJCSAT3A);
    }

    // アンテナ調整(JCSAT-4B)
    {
        LC70_CH_SET_JSAT4B_ANTENNA_T lc70Data;

        if (0 > _communicator->readChannelConfigurationFile(LC70_FILE_ID_CH_SET_JSAT4B_ANTENNA, &lc70Data, defaultInit)) {
        }

        // ChannelDefictionItemのリストに変換する
        QList<ChannelDefinitionItem> definitionItemJCSAT4B = ChannelDefinitionItemHelper::convertChannelDefinitionItem(&lc70Data,LC70_FILE_ID_CH_SET_JSAT4B_ANTENNA);

        // マップデータに保持する
        _channelDefinitionMap.insert(LC70_FILE_ID_CH_SET_JSAT4B_ANTENNA, definitionItemJCSAT4B);
    }

    // FM Default
    {
        LC70_CH_SET_FM_DEF_T lc70Data;

        if (0 > _communicator->readChannelConfigurationFile(LC70_FILE_ID_CH_SET_FM_DEF, &lc70Data, defaultInit )) {
        }

        // ChannelDefictionItemのリストに変換する
        QList<ChannelDefinitionItem> definitionItemFM = ChannelDefinitionItemHelper::convertChannelDefinitionItem(&lc70Data,LC70_FILE_ID_CH_SET_FM_DEF);

        // マップデータに保持する
        _channelDefinitionMap.insert(LC70_FILE_ID_CH_SET_FM_DEF, definitionItemFM);
    }

    // CATV上り
    {
        LC70_CH_SET_CATV_UP_STREAM_DEF_T lc70Data;

        if (0 > _communicator->readChannelConfigurationFile(LC70_FILE_ID_CH_SET_CATV_UP_STREAM_DEF, &lc70Data, defaultInit)) {
        }

        // ChannelDefictionItemのリストに変換する
        QList<ChannelDefinitionItem> definitionItemCATV_STREAM = ChannelDefinitionItemHelper::convertChannelDefinitionItem(&lc70Data, LC70_FILE_ID_CH_SET_CATV_UP_STREAM_DEF);

        // マップデータに保持する
        _channelDefinitionMap.insert(LC70_FILE_ID_CH_SET_CATV_UP_STREAM_DEF, definitionItemCATV_STREAM);
    }

    // BS_CS統合
    {
        LC70_CH_SET_BS_CS_T lc70Data;

        if (0 > _communicator->readChannelConfigurationFile(LC70_FILE_ID_CH_SET_BS_CS, &lc70Data, defaultInit)) {
        }

        // ChannelDefictionItemのリストに変換する
        QList<ChannelDefinitionItem> definitionItemBS_CS = ChannelDefinitionItemHelper::convertChannelDefinitionItem(&lc70Data,
                LC70_FILE_ID_CH_SET_BS_CS);

        // マップデータに格納する
        _channelDefinitionMap.insert(LC70_FILE_ID_CH_SET_BS_CS, definitionItemBS_CS);
    }


}

void ModelOperator::initializeChannelDefinitionDataDevelop(bool defaultInit)
{
    Q_UNUSED(defaultInit);
    // [Memo]
    //  for文で回すこともできるが、
    //  initializeChannelDefinitionDataProductionに表記を合わせておく
    if (_stub) {
        // _スタブクラスが設定されていない場合は、以降の処理は実施しない
    }

    // 地デジ
    {
        QList<ChannelDefinitionItem> definitionItemUHF = _stub->readChannelDefinition(CHANNEL_DEFINITION_FILE[0]);
        _channelDefinitionMap.insert(LC70_FILE_ID_CH_SET_UHF, definitionItemUHF);
    }

    // BS
    {
        QList<ChannelDefinitionItem> definitionItemBS = _stub->readChannelDefinition(CHANNEL_DEFINITION_FILE[1]);
        _channelDefinitionMap.insert(LC70_FILE_ID_CH_SET_BS, definitionItemBS);
    }

    // CS
    {
        QList<ChannelDefinitionItem> definitionItemCS = _stub->readChannelDefinition(CHANNEL_DEFINITION_FILE[2]);
        _channelDefinitionMap.insert(LC70_FILE_ID_CH_SET_CS, definitionItemCS);
    }

    // SCCB
    {
        QList<ChannelDefinitionItem> definitionItemSCCB = _stub->readChannelDefinition(CHANNEL_DEFINITION_FILE[3]);
        _channelDefinitionMap.insert(LC70_FILE_ID_CH_SET_SCCB, definitionItemSCCB);
    }

    // SCCC
    {
        QList<ChannelDefinitionItem> definitionItemSCCC = _stub->readChannelDefinition(CHANNEL_DEFINITION_FILE[4]);
        _channelDefinitionMap.insert(LC70_FILE_ID_CH_SET_SCCC, definitionItemSCCC);
    }

    // JSAT3A
    {
        QList<ChannelDefinitionItem> definitionItemJSAT3A = _stub->readChannelDefinition(CHANNEL_DEFINITION_FILE[5]);
        _channelDefinitionMap.insert(LC70_FILE_ID_CH_SET_JSAT3A, definitionItemJSAT3A);
    }

    // JSAT4B
    {
        QList<ChannelDefinitionItem> definitionItemJSAT4B = _stub->readChannelDefinition(CHANNEL_DEFINITION_FILE[6]);
        _channelDefinitionMap.insert(LC70_FILE_ID_CH_SET_JSAT4B, definitionItemJSAT4B);
    }

    // ラッパー関数ライブラリ V2.00 対応
    // CATV
    {
        QList<ChannelDefinitionItem> definitionItemCATV = _stub->readChannelDefinition(CHANNEL_DEFINITION_FILE[7]);
        _channelDefinitionMap.insert(LC70_FILE_ID_CH_SET_CATV_ALL, definitionItemCATV);
    }

    // ラッパー関数ライブラリ V2.00 対応
    {
        QList<ChannelDefinitionItem> definitionItemCATV_90_470MHZ = _stub->readChannelDefinition(CHANNEL_DEFINITION_FILE[8]);
        _channelDefinitionMap.insert(LC70_FILE_ID_CH_SET_CATV_90_470MHZ, definitionItemCATV_90_470MHZ);
    }

    // ラッパー関数ライブラリ V2.00 対応
    {
        QList<ChannelDefinitionItem> definitionItemCATV_470_1002MHZ = _stub->readChannelDefinition(CHANNEL_DEFINITION_FILE[9]);
        _channelDefinitionMap.insert(LC70_FILE_ID_CH_SET_CATV_470_1002MHZ, definitionItemCATV_470_1002MHZ);
    }

    // スカパー光
    {
        QList<ChannelDefinitionItem> definitionItemSKYP = _stub->readChannelDefinition(CHANNEL_DEFINITION_FILE[10]);
        _channelDefinitionMap.insert(LC70_FILE_ID_CH_SET_SKYP, definitionItemSKYP);
    }

    // PILOT
    {
        QList<ChannelDefinitionItem> definitionItemPILOT = _stub->readChannelDefinition(CHANNEL_DEFINITION_FILE[11]);
        _channelDefinitionMap.insert(LC70_FILE_ID_CH_SET_PILOT, definitionItemPILOT);
    }

    // LTE
    {
        QList<ChannelDefinitionItem> definitionItemLTE = _stub->readChannelDefinition(CHANNEL_DEFINITION_FILE[12]);
        _channelDefinitionMap.insert(LC70_FILE_ID_CH_SET_LTE, definitionItemLTE);
    }

    // FM
    {
        QList<ChannelDefinitionItem> definitionItemFM = _stub->readChannelDefinition(CHANNEL_DEFINITION_FILE[13]);
        _channelDefinitionMap.insert(LC70_FILE_ID_CH_SET_FM, definitionItemFM);
    }

    // CATV上り
    {
        QList<ChannelDefinitionItem> definitionItemCATVUPSTREAM = _stub->readChannelDefinition(CHANNEL_DEFINITION_FILE[14]);
        _channelDefinitionMap.insert(LC70_FILE_ID_CH_SET_CATV_UP_STREAM, definitionItemCATVUPSTREAM);
    }

    // アンテナ調整(JCSAT-3A)
    {
        QList<ChannelDefinitionItem> definitionItemJCSAT3A = _stub->readChannelDefinition(CHANNEL_DEFINITION_FILE[15]);
        _channelDefinitionMap.insert(LC70_FILE_ID_CH_SET_JSAT3A_ANTENNA, definitionItemJCSAT3A);
    }

    // アンテナ調整(JCSAT-4B)
    {
        QList<ChannelDefinitionItem> definitionItemJCSAT4B = _stub->readChannelDefinition(CHANNEL_DEFINITION_FILE[16]);
        _channelDefinitionMap.insert(LC70_FILE_ID_CH_SET_JSAT4B_ANTENNA, definitionItemJCSAT4B);
    }
}

bool ModelOperator::importDataLoggerUserSettingList(ValueId valueId, const QString &filename)
{
    QList<LC70_LOGGER_SET_LOGGER_ROW_T> dataLoggerUserSettingList;
    QString fileName = filename;

    ChannelEditMode mode = CHANNEL_EDIT_USER;

    if(valueId == VALUE_USERSETTING_SETTINGLIST) {
        mode = CHANNEL_EDIT_USER;
    } else if(valueId == VALUE_USERSETTING_SETTINGLIST_USB1) {
        mode = CHANNEL_EDIT_USER_USB1;
    } else if(valueId == VALUE_USERSETTING_SETTINGLIST_USB2) {
        mode = CHANNEL_EDIT_USER_USB2;
    } else if(valueId == VALUE_DATALOGGERSETTING_SETTINGLIST) {
        mode = CHANNEL_EDIT_DATA_LOGGER;
    } else if(valueId == VALUE_DATALOGGERSETTING_SETTINGLIST_USB1) {
        mode = CHANNEL_EDIT_DATA_LOGGER_USB1;
    } else if(valueId == VALUE_DATALOGGERSETTING_SETTINGLIST_USB2) {
        mode = CHANNEL_EDIT_DATA_LOGGER_USB2;
    }

    // 正常終了している場合の戻り値は読み取り行数
    LC70_ERR_T retcode = _communicator->readUserLogSetFile(fileName, dataLoggerUserSettingList, mode);

    QVariantList data;
    QVariantMap value;

    if (retcode >= LC70_OK) {
        int idx = 0;

        // 読み取った行の内容をリストに追加
        for (int i = 0; i < retcode; ++i) {
            LC70_LOGGER_SET_LOGGER_ROW_T row = dataLoggerUserSettingList.at(i);
#ifdef NEW_USER_SETTING
            value["index"] = static_cast<int>(row.ucNo);
#endif
            value["enable"] = static_cast<bool>(row.ucEnbFlg);
            value["band"] =  static_cast<int>(row.ucBand);
            value["broadbandSystem"] = static_cast<int>(row.ucSpec);
            value["channel"] = QString(row.cStrChNo);
            value["centerFreq"] = static_cast<unsigned int>(row.ulCenterFreq);
            value["modulationBit"] = static_cast<unsigned int>(row.ulModBit);
            value["localFreq"] = static_cast<unsigned int>(row.ulLocalFreq);
            // MER/CN の使い分け(MER ビットは必ず立っている。CNR のときは CNR ビットも立っている)
            value["MERCN"] = (row.ucGetDataFlg & LC70_DATA_MSK_CNR ? true : false);
            value["BER"] = (row.ucGetDataFlg & LC70_DATA_MSK_BER ? true : false);
            value["PowerSupply"] = static_cast<int>(row.ucDcOut);

            data.insert(idx, value);
            idx++;
            value.clear();
        }
    }

    _values[valueId].setInitialValue(data);

    return (retcode >= LC70_OK ? true : false);
}

bool ModelOperator::exportDataLoggerUserSettingList(ValueId valueId, const QString &filename)
{
    QList<LC70_LOGGER_SET_LOGGER_ROW_T> dataLoggerUserSettingList;

    ChannelDefinitionItem channelDefinitionItem;

    QVariantList data = _values[valueId].getValue().toList();

    // チャンネルでソート
    sortChannel(data);

    int size = data.size();

    // 測定するが選択されていない場合は出力しない
    bool ok = false;
    for (int i = 0; i < size; ++i) {
        QVariantMap item = data[i].toMap();
        if (item["enable"].toBool()) {
            ok = true;
            break;
        }
    }
    if (!ok) {
        return true;
    }

    QVariant cnrMerSetting;
    getValue(VALUE_MER_CN, cnrMerSetting);

    int number = 1;
    for (int i = 0; i < size; ++i) {
        QVariantMap item = data[i].toMap();

        LC70_LOGGER_SET_LOGGER_ROW_T row;
        memset(&row, 0, sizeof(LC70_LOGGER_SET_LOGGER_ROW_T));
        row.ucNo = number++;
        row.ucEnbFlg = item["enable"].toBool();
        row.ucBand = item["band"].toInt();
        row.ucSpec = item["broadbandSystem"].toInt();
        strncpy(row.cStrChNo, item["channel"].toString().toUtf8().constData(), item["channel"].toByteArray().size());
        row.ulCenterFreq = item["centerFreq"].toUInt();
        row.ulModBit = item["modulationBit"].toUInt();
        row.ulLocalFreq = item["localFreq"].toUInt();

        if (row.ucBand != LC70_BAND_PILOT && row.ucBand != LC70_BAND_LTE && row.ucBand != LC70_BAND_FM && row.ucBand != LC70_BAND_CATV_UP_STREAM) {
            if (Correcter::selectCnrMer(static_cast<LC70_BAND_T>(row.ucBand), cnrMerSetting) == LC70_MEAS_SET_SEL_CNR) {
                row.ucGetDataFlg = item["MERCN"].toBool() ? LC70_DATA_MSK_CNR : 0x0;
            } else if (Correcter::selectCnrMer(static_cast<LC70_BAND_T>(row.ucBand), cnrMerSetting) == LC70_MEAS_SET_SEL_MER) {
                row.ucGetDataFlg = item["MERCN"].toBool() ? LC70_DATA_MSK_CNR | LC70_DATA_MSK_MER : 0x0;
            }
            row.ucGetDataFlg |= item["BER"].toBool() ? 0x0004 : 0x0;
        }

        row.ucGetDataFlg |= LC70_DATA_MSK_LEVEL;
        QString powerSupply = item["PowerSupply"].toString();

        // TODO: 初回登録時、既存データ編集時で入力される値が統一されていない
//        if (powerSupply == QString("OFF") || powerSupply == QString("0")) {
//            row.ucDcOut = LC70_MEAS_SET_DC_OUT_VAL_NONE;
//        } else if (powerSupply == QString("11V") || powerSupply == QString("1")) {
//            row.ucDcOut = LC70_MEAS_SET_DC_OUT_VAL_11V;
//        } else if (powerSupply == QString("15V") || powerSupply == QString("2")) {
//            row.ucDcOut = LC70_MEAS_SET_DC_OUT_VAL_15V;
//        } else if (powerSupply == QString("11V+P") || powerSupply == QString("3")) {
//            row.ucDcOut = LC70_MEAS_SET_DC_OUT_VAL_11V_P;
//        } else if (powerSupply == QString("15V+P") || powerSupply == QString("4")) {
//            row.ucDcOut = LC70_MEAS_SET_DC_OUT_VAL_15V_P;
//        }

        getChannelDefinitionItem(row.ucBand, item["channel"].toString(), channelDefinitionItem);

        if (powerSupply == QString("OFF") || powerSupply == QString("0")) {
            row.ucDcOut = LC70_MEAS_SET_DC_OUT_VAL_NONE;
        } else {
            if ((row.ucBand == LC70_BAND_JCSAT3) && (channelDefinitionItem.polarization == LC70_MEAS_SET_WAVE_H)) {
                row.ucDcOut = LC70_MEAS_SET_DC_OUT_VAL_15V;
            } else if ((row.ucBand == LC70_BAND_JCSAT3) && (channelDefinitionItem.polarization == LC70_MEAS_SET_WAVE_V)) {
                row.ucDcOut = LC70_MEAS_SET_DC_OUT_VAL_11V;
            } else if ((row.ucBand == LC70_BAND_JCSAT4) && (channelDefinitionItem.polarization == LC70_MEAS_SET_WAVE_H)) {
                row.ucDcOut = LC70_MEAS_SET_DC_OUT_VAL_15V_P;
            } else if ((row.ucBand == LC70_BAND_JCSAT4) && (channelDefinitionItem.polarization == LC70_MEAS_SET_WAVE_V)) {
                row.ucDcOut = LC70_MEAS_SET_DC_OUT_VAL_11V_P;
            } else if ((row.ucBand == LC70_BAND_SCCB) && (channelDefinitionItem.polarization == LC70_MEAS_SET_WAVE_H)) {
                row.ucDcOut = LC70_MEAS_SET_DC_OUT_VAL_15V;
            } else if ((row.ucBand == LC70_BAND_SCCB) && (channelDefinitionItem.polarization == LC70_MEAS_SET_WAVE_V)) {
                row.ucDcOut = LC70_MEAS_SET_DC_OUT_VAL_11V;
            } else if ((row.ucBand == LC70_BAND_SCCC) && (channelDefinitionItem.polarization == LC70_MEAS_SET_WAVE_H)) {
                row.ucDcOut = LC70_MEAS_SET_DC_OUT_VAL_15V;
            } else if ((row.ucBand == LC70_BAND_SCCC) && (channelDefinitionItem.polarization == LC70_MEAS_SET_WAVE_V)) {
                row.ucDcOut = LC70_MEAS_SET_DC_OUT_VAL_11V;
            } else {
                row.ucDcOut = LC70_MEAS_SET_DC_OUT_VAL_15V;
            }
        }
        dataLoggerUserSettingList.append(row);
    }

    ChannelEditMode mode = CHANNEL_EDIT_USER;

    if(valueId == VALUE_USERSETTING_SETTINGLIST) {
        mode = CHANNEL_EDIT_USER;
    } else if(valueId == VALUE_USERSETTING_SETTINGLIST_USB1) {
        mode = CHANNEL_EDIT_USER_USB1;
    } else if(valueId == VALUE_USERSETTING_SETTINGLIST_USB2) {
        mode = CHANNEL_EDIT_USER_USB2;
    } else if(valueId == VALUE_DATALOGGERSETTING_SETTINGLIST) {
        mode = CHANNEL_EDIT_DATA_LOGGER;
    } else if(valueId == VALUE_DATALOGGERSETTING_SETTINGLIST_USB1) {
        mode = CHANNEL_EDIT_DATA_LOGGER_USB1;
    } else if(valueId == VALUE_DATALOGGERSETTING_SETTINGLIST_USB2) {
        mode = CHANNEL_EDIT_DATA_LOGGER_USB2;
    }

    LC70_ERR_T ret = _communicator->writeUserLogSetFile(filename, dataLoggerUserSettingList, mode);

    return ret == LC70_OK ? true : false;
}

void ModelOperator::addDataLoggerUserSettingList(int lineno)
{
    QVariantList data = _values[VALUE_USERSETTING_SETTINGLIST].getValue().toList();

    int listsize = data.size();

    for (int i = listsize; i <= lineno; ++i) {
        // 初期状態の情報を追加する
        QVariantMap value;

        initializeDataLoggerUserSettingList(value);

        data.append(value);
    }

    _values[VALUE_USERSETTING_SETTINGLIST].setValue(data);

}

void ModelOperator::deleteDataLoggerUserSettingList(int lineno)
{
    // 初期状態の情報で置き換える
    QVariantMap value;

    initializeDataLoggerUserSettingList(value);

    QVariantList data = _values[VALUE_USERSETTING_SETTINGLIST].getValue().toList();
    data.replace(lineno, value);

    _values[VALUE_USERSETTING_SETTINGLIST].setValue(data);

}

void ModelOperator::initializeDataLoggerUserSettingList(QVariantMap &value)
{
    value["enable"] = false;
    value["band"] =  -1;
    value["broadbandSystem"] = -1;
    value["channel"] = QString("");
    value["centerFreq"] = -1;
    value["modulationBit"] = -1;
    value["localFreq"] = 0;
    value["MERCN"] = 0;
    value["BER"] = 0;
    value["PowerSupply"] = -1;
}

void ModelOperator::sortChannel(QVariantList &channelList)
{
    // 前提
    // bandごとにソートされた後

    int currentBand = LC70_BAND_UNDEF;
    int startIndex = 0;
    // プリセットチャンネルはソート対象外
    for (int i = 0; i < LC70_CH_SET_CATV_ALL_IDX_MAX; i++) {
        QVariantMap channelData = channelList[i].toMap();
        int band = channelData["band"].toInt();
        if (channelData["enable"].toBool() && currentBand == band && i < LC70_CH_SET_CATV_ALL_IDX_MAX - 1) {
            continue;
        }
        // 以下の条件で次のBandに移行したとみなして、それまでのListをソートする
        // ・有効チャンネル内でBandが切り替わった
        // ・有効チャンネルから無効チャンネルになった
        // ・チャンネルリストの最後に到達した
        if (i - startIndex >= 2) {
            // 同じBandのチャンネルが2件以上あったらソートが必要
            QVariantList sortTargetList;
            for (int j = 0; j < i - startIndex; j++) {
                sortTargetList.append(channelList[startIndex + j].toMap());
            }
            // 並び順を判断するためにチャンネル定義リストを取得
            QList<ChannelDefinitionItem> channelDefinitionList;
            // 偏波の考慮有無に応じてバンドごとにチャンネル定義リストを作成
            if (currentBand == LC70_BAND_BS || currentBand == LC70_BAND_CS) {
                QList<ChannelDefinitionItem> channels;
                selectChannelDefinitionList(currentBand, RIGHT_HANDED_POLARIZE, channels);
                channelDefinitionList.append(channels);
                channels.clear();
                selectChannelDefinitionList(currentBand, LEFT_HANDED_POLARIZE, channels);
                channelDefinitionList.append(channels);
            } else if (currentBand == LC70_BAND_JCSAT3 || currentBand == LC70_BAND_JCSAT4 || currentBand == LC70_BAND_SCCB || currentBand == LC70_BAND_SCCC) {
                QList<ChannelDefinitionItem> channels;
                selectChannelDefinitionList(currentBand, HORIZONTALLY_POLARIZE, channels);
                channelDefinitionList.append(channels);
                channels.clear();
                selectChannelDefinitionList(currentBand, VERTICALLY_POLARIZE, channels);
                channelDefinitionList.append(channels);
            } else {
                selectChannelDefinitionList(currentBand, "", channelDefinitionList);
            }
            // チャンネル定義の先頭からで見つかった順にチャンネルリストを並び替える
            if (currentBand == LC70_BAND_FM || currentBand == LC70_BAND_CATV_UP_STREAM) {
                // FM, CATV 上りは USER データロガー設定のチャンネル順に登録されているが、チャンネル定義ファイルには
                // チャンネル名に周波数が入っているため、周波数でソートされてしまうのを避ける
                break;
            } else {
                int sortedCount = 0;
                // ソート対象の Band のチャンネルの一覧
                for (int k = 0; k < channelDefinitionList.length(); k++) {
                    // ソート対象の Band で選択されたチャンネルの一覧
                    for (int l = 0; l < sortTargetList.length(); l++) {
                        QVariantMap sortTargetData = sortTargetList[l].toMap();
                        // チャンネルが合致したら
                        if (channelDefinitionList[k].channelNumber == sortTargetData["channel"].toString()) {
                            // インデックスを更新
                            sortTargetData["index"] = startIndex + sortedCount;
                            // 合致したチャンネルを入れ替え
                            channelList[startIndex + sortedCount] = sortTargetData;
                            sortedCount++;
                            break;
                        }
                    }
                    if (startIndex + sortedCount >= i) {
                        // ソート完了なので、抜ける
                        break;
                    }
                }
            }
        }
        if (!channelData["enable"].toBool()) {
            // 残りは全て無効チャンネルなので、ここで終了
            break;
        }
        // 現在のバンドとその開始インデックスを保持しておく
        currentBand = band;
        startIndex = i;
    }
}

bool ModelOperator::getChannelDefinitionMap(QMap<int, QList<ChannelDefinitionItem> > &channelDefinitionMap)
{
    if (_channelDefinitionMap.isEmpty()) {
        // チャンネル定義マップが空の場合Falseを返す
        return false;
    }

    channelDefinitionMap = _channelDefinitionMap;
    return true;
}

bool ModelOperator::getChannelDefinitionList(int bandId, QList<ChannelDefinitionItem> &channelList)
{
    if (_channelDefinitionMap.isEmpty()) {
        // チャンネル定義マップが空の場合Falseを返す
        return false;
    }

    for (int i = 0; i < BAND_TOTAL; ++i) {
        //各チャンネルと対応する定義ファイル名から対応する IDを検索する
        if (bandId != LC70_BAND_UNDEF && bandId != LC70_BAND_END && bandId == CHANNEL_DEFINITION_FILE[i].band) {
            channelList = _channelDefinitionMap.value(CHANNEL_DEFINITION_FILE[i].Id);
            break;
        }
    }

    if (channelList.isEmpty()) {
        // リストデータが空の場合、Falseを返す
        return false;
    }

    return true;
}

bool ModelOperator::getChannelDefinitionItem(int bandId, const QString &channel, ChannelDefinitionItem &channelItem)
{
    QList<ChannelDefinitionItem> channelDefectionList;

    if (!getChannelDefinitionList(bandId, channelDefectionList)) {
        return false;
    }

    for (int i = 0; i < channelDefectionList.length(); ++i) {
        if (channelDefectionList[i].channelNumber == channel) {
            channelItem = channelDefectionList[i];
            return true;
        }
    }

    return false;
}

bool ModelOperator::setChannelDefinitionList(int bandId, QList<ChannelDefinitionItem> &channelList)
{
    if (_channelDefinitionMap.isEmpty()) {
        // チャンネル定義マップが空の場合、Falseを返す。
        return false;
    }

    // 変更対象データのキーを検索する
    int targetDataKey = -1;

    for (int i = 0; i < BAND_TOTAL; ++i) {
        //各チャンネルと対応する定義ファイル名から対応する IDを検索する
        if (bandId != LC70_BAND_UNDEF && bandId != LC70_BAND_END && bandId == CHANNEL_DEFINITION_FILE[i].band) {
            targetDataKey = CHANNEL_DEFINITION_FILE[i].Id;
        }
    }

    if (targetDataKey == -1) {
        // 変更対象データのキーが見つからない場合は、Falseを返す
        return false;
    }

    // QMapのinsert関数を用いて変更対象データの置き換えを実施する。
    _channelDefinitionMap.insert(targetDataKey, channelList);

    return true;
}

bool ModelOperator::getChannelDefinitionItemForAntennaAdjustment(int bandId, const QString &channel, ChannelDefinitionItem &channelItem)
{
    // アンテナ調整(JCSAT-3A/4B)のチャンネル定義を取得
    QList<ChannelDefinitionItem> channelList;
    if (bandId == LC70_BAND_JCSAT3) {
        channelList = _channelDefinitionMap.value(CHANNEL_DEFINITION_FILE[15].Id);;
    } else if (bandId == LC70_BAND_JCSAT4) {
        channelList = _channelDefinitionMap.value(CHANNEL_DEFINITION_FILE[16].Id);;
    } else {
        return false;
    }

    if (channelList.isEmpty()) {
        return false;
    }

    // チャンネルを引き当てる
    for (int i = 0; i < channelList.length(); ++i) {
        if (channelList[i].channelNumber == channel) {
            channelItem = channelList[i];
            return true;
        }
    }

    return true;
}

bool ModelOperator::setChannelDefinitionItem(int bandId, const QString &channel, ChannelDefinitionItem &channelItem)
{
    if (_channelDefinitionMap.isEmpty()) {
        // チャンネル定義マップが空の場合、Falseを返す
    }

    // 変更を行う チャンネルリスト情報を取得する。
    QList<ChannelDefinitionItem> targetChannelDefectionList;

    if (!getChannelDefinitionList(bandId, targetChannelDefectionList)) {
        // 変更を行うチャンネルリスト情報の取得に失敗した場合、Falseを返す
        return false;
    }

    // 変更対象のリストインデックスを検索する
    int targetDataIndex = -1;

    for (int i = 0; i < targetChannelDefectionList.length(); ++i) {
        if (targetChannelDefectionList[i].channelNumber == channel) {
            targetDataIndex = i;
        }
    }

    if (targetDataIndex == -1) {
        // 変更対象のリストインデックスが見つからない場合は、Falseを返す。
    }

    // replace関数を用いてデータの置き換えを実施対象データの置き換えを実施する。
    targetChannelDefectionList.replace(targetDataIndex, channelItem);

    // Mapに格納しているリストデータの更新を実施する
    if (!setChannelDefinitionList(bandId, targetChannelDefectionList)) {
        // Mapに格納しているリストデータの更新に失敗した場合Falseを返す。
        return false;
    }

    return true;
}

bool ModelOperator::commitAllChannelDefinitionData()
{
    if (_channelDefinitionMap.isEmpty()) {
        // チャンネル定義情報マップが空の場合、Falseを返す
        return false;
    }

    // 地デジ
    {
        QList<ChannelDefinitionItem> targetChannelDefectionList;
        targetChannelDefectionList = _channelDefinitionMap.value(LC70_FILE_ID_CH_SET_UHF);

        if (targetChannelDefectionList.isEmpty()) {
            // 対応するチャンネル定義リストが空の場合、Falseを返す
            return false;
        }

        LC70_CH_SET_UHF_T lc70Data;

        // ラッパー関数で用いるデータ型に変換を実施する
        for (int i = 0; i < targetChannelDefectionList.length(); ++i) {

            // チャンネル番号文字列
            strncpy(lc70Data.chSetRow[i].cStrChNo,
                    static_cast<ChannelDefinitionItem>(targetChannelDefectionList.at(i)).channelNumber.toLocal8Bit().constData(),
                    LC70_CH_SET_CH_STRLEN);

            // 中心周波数
            lc70Data.chSetRow[i].ulCentFreq = static_cast<ChannelDefinitionItem>(targetChannelDefectionList.at(i)).centerFrequency1;
        }

        // 定義ファイルに保存を行う
        if (LC70_OK != _communicator->writeChannelConfigurationFile(LC70_FILE_ID_CH_SET_UHF, &lc70Data)) {
            // 処理結果が正常終了ではない場合はfalseを返し後続処理は実施しない
            return false;
        }
    }

    // BS
    {
        QList<ChannelDefinitionItem> targetChannelDefectionList;
        targetChannelDefectionList = _channelDefinitionMap.value(LC70_FILE_ID_CH_SET_BS);

        if (targetChannelDefectionList.isEmpty()) {
            // 対応するチャンネル定義リストが空の場合、Falseを返す
            return false;
        }

        LC70_CH_SET_BS_T lc70Data;

        // ラッパー関数で用いるデータ型に変換を実施する
        for (int i = 0; i < targetChannelDefectionList.length(); ++i) {

            // チャンネル番号文字列
            strncpy(lc70Data.chSetRow[i].cStrChNo,
                    static_cast<ChannelDefinitionItem>(targetChannelDefectionList.at(i)).channelNumber.toLocal8Bit().constData(),
                    LC70_CH_SET_CH_STRLEN);

            // 中心周波数
            lc70Data.chSetRow[i].ulCentFreq = static_cast<ChannelDefinitionItem>(targetChannelDefectionList.at(i)).centerFrequency1;

            // 偏波番号
            lc70Data.chSetRow[i].ucWave = static_cast<ChannelDefinitionItem>(targetChannelDefectionList.at(i)).polarization;
        }

        // 定義ファイルに保存を行う。
        if (LC70_OK != _communicator->writeChannelConfigurationFile(LC70_FILE_ID_CH_SET_BS, &lc70Data)) {
            // 処理結果が正常終了ではない場合はfalseを返し後続処理は実施しない
            return false;
        }
    }

    // CS
    {
        QList<ChannelDefinitionItem> targetChannelDefectionList;
        targetChannelDefectionList = _channelDefinitionMap.value(LC70_FILE_ID_CH_SET_CS);

        if (targetChannelDefectionList.isEmpty()) {
            // 対応するチャンネル定義リストが空の場合、Falseを返す
            return false;
        }

        LC70_CH_SET_CS_T lc70Data;

        // ラッパー関数で用いるデータ型に変換を実施する
        for (int i = 0; i < targetChannelDefectionList.length(); ++i) {

            // チャンネル番号文字列
            strncpy(lc70Data.chSetRow[i].cStrChNo,
                    static_cast<ChannelDefinitionItem>(targetChannelDefectionList.at(i)).channelNumber.toLocal8Bit().constData(),
                    LC70_CH_SET_CH_STRLEN);

            // 中心周波数[kHz]
            lc70Data.chSetRow[i].ulCentFreq = static_cast<ChannelDefinitionItem>(targetChannelDefectionList.at(i)).centerFrequency1;

            // 偏波番号
            lc70Data.chSetRow[i].ucWave = static_cast<ChannelDefinitionItem>(targetChannelDefectionList.at(i)).polarization;

        }

        // 定義ファイルに保存を行う
        if (LC70_OK != _communicator->writeChannelConfigurationFile(LC70_FILE_ID_CH_SET_CS, &lc70Data)) {
            // 処理結果が正常終了ではない場合はfalseを返し後続処理は実施しない
            return false;
        }
    }

// SCCB
    {
        QList<ChannelDefinitionItem> targetChannelDefectionList;
        targetChannelDefectionList = _channelDefinitionMap.value(LC70_FILE_ID_CH_SET_SCCB);

        if (targetChannelDefectionList.isEmpty()) {
            // 対応するチャンネル定義リストが空の場合、Falseを返す
            return false;
        }

        LC70_CH_SET_SCCB_T lc70Data;

        // ラッパー関数で用いるデータ型に変換を実施する
        for (int i = 0; i < targetChannelDefectionList.length(); ++i) {

            // チャンネル番号文字列
            strncpy(lc70Data.chSetRow[i].cStrChNo,
                    static_cast<ChannelDefinitionItem>(targetChannelDefectionList.at(i)).channelNumber.toLocal8Bit().constData(),
                    LC70_CH_SET_CH_STRLEN);

            // 中心周波数[kHz]
            lc70Data.chSetRow[i].ulCentFreq = static_cast<ChannelDefinitionItem>(targetChannelDefectionList.at(i)).centerFrequency1;

            // 偏波番号
            lc70Data.chSetRow[i].ucWave = static_cast<ChannelDefinitionItem>(targetChannelDefectionList.at(i)).polarization;
        }

        // 定義ファイルに保存を行う
        if (LC70_OK != _communicator->writeChannelConfigurationFile(LC70_FILE_ID_CH_SET_SCCB, &lc70Data)) {
            // 処理結果が正常終了ではない場合はfalseを返し後続処理は実施しない
            return false;
        }
    }

    // SCCC
    {
        QList<ChannelDefinitionItem> targetChannelDefectionList;
        targetChannelDefectionList = _channelDefinitionMap.value(LC70_FILE_ID_CH_SET_SCCC);

        if (targetChannelDefectionList.isEmpty()) {
            // 対応するチャンネル定義リストが空の場合、Falseを返す
            return false;
        }

        LC70_CH_SET_SCCC_T lc70Data;

        // ラッパー関数で用いるデータ型に変換を実施する
        for (int i = 0; i < targetChannelDefectionList.length(); ++i) {

            // チャンネル番号文字列
            strncpy(lc70Data.chSetRow[i].cStrChNo,
                    static_cast<ChannelDefinitionItem>(targetChannelDefectionList.at(i)).channelNumber.toLocal8Bit().constData(),
                    LC70_CH_SET_CH_STRLEN);

            // 中心周波数[kHz]
            lc70Data.chSetRow[i].ulCentFreq = static_cast<ChannelDefinitionItem>(targetChannelDefectionList.at(i)).centerFrequency1;

            // 偏波番号
            lc70Data.chSetRow[i].ucWave = static_cast<ChannelDefinitionItem>(targetChannelDefectionList.at(i)).polarization;
        }

        // 定義ファイルに保存を行う
        if (LC70_OK != _communicator->writeChannelConfigurationFile(LC70_FILE_ID_CH_SET_SCCC, &lc70Data)) {
            // 処理結果が正常終了ではない場合はfalseを返し後続処理は実施しない
            return false;
        }
    }

    // JSAT-3A
    {
        QList<ChannelDefinitionItem> targetChannelDefectionList;
        targetChannelDefectionList = _channelDefinitionMap.value(LC70_FILE_ID_CH_SET_JSAT3A);

        if (targetChannelDefectionList.isEmpty()) {
            // 対応するチャンネル定義リストが空の場合、Falseを返す
            return false;
        }

        LC70_CH_SET_JSAT3A_T lc70Data;

        // ラッパー関数で用いるデータ型に変換を実施する
        for (int i = 0; i < targetChannelDefectionList.length(); ++i) {

            // チャンネル番号文字列
            strncpy(lc70Data.chSetRow[i].cStrChNo,
                    static_cast<ChannelDefinitionItem>(targetChannelDefectionList.at(i)).channelNumber.toLocal8Bit().constData(),
                    LC70_CH_SET_CH_STRLEN);

            // 中心周波数[kHz]
            lc70Data.chSetRow[i].ulCentFreq = static_cast<ChannelDefinitionItem>(targetChannelDefectionList.at(i)).centerFrequency1;

            // 偏波番号
            lc70Data.chSetRow[i].ucWave = static_cast<ChannelDefinitionItem>(targetChannelDefectionList.at(i)).polarization;
        }

        // 定義ファイルに保存を行う
        if (LC70_OK != _communicator->writeChannelConfigurationFile(LC70_FILE_ID_CH_SET_JSAT3A, &lc70Data)) {
            // 処理結果が正常終了ではない場合はfalseを返し後続処理は実施しない
            return false;
        }

    }

    // JSAT4B
    {
        QList<ChannelDefinitionItem> targetChannelDefectionList;
        targetChannelDefectionList = _channelDefinitionMap.value(LC70_FILE_ID_CH_SET_JSAT4B);

        if (targetChannelDefectionList.isEmpty()) {
            // 対応するチャンネル定義リストが空の場合、Falseを返す
            return false;
        }

        LC70_CH_SET_JSAT4B_T lc70Data;

        // ラッパー関数で用いるデータ型に変換を実施する
        for (int i = 0; i < targetChannelDefectionList.length(); ++i) {

            // チャンネル番号文字列
            strncpy(lc70Data.chSetRow[i].cStrChNo,
                    static_cast<ChannelDefinitionItem>(targetChannelDefectionList.at(i)).channelNumber.toLocal8Bit().constData(),
                    LC70_CH_SET_CH_STRLEN);
            // 中心周波数[kHz]
            lc70Data.chSetRow[i].ulCentFreq = static_cast<ChannelDefinitionItem>(targetChannelDefectionList.at(i)).centerFrequency1;

            // 偏波番号
            lc70Data.chSetRow[i].ucWave = static_cast<ChannelDefinitionItem>(targetChannelDefectionList.at(i)).polarization;
        }

        // 定義ファイルに保存を行う
        if (LC70_OK != _communicator->writeChannelConfigurationFile(LC70_FILE_ID_CH_SET_JSAT4B, &lc70Data)) {
            // 処理結果が正常終了ではない場合はfalseを返し後続処理は実施しない
            return false;
        }
    }

    // ラッパー関数ライブラリ V2.00 対応
    // CATV_ALL
    {
        QList<ChannelDefinitionItem> targetChannelDefectionList;
        targetChannelDefectionList = _channelDefinitionMap.value(LC70_FILE_ID_CH_SET_CATV_ALL);

        if (targetChannelDefectionList.isEmpty()) {
            // 対応するチャンネル定義リストが空の場合、Falseを返す
            return false;
        }

        LC70_CH_SET_CATV_ALL_T lc70Data;

        // ラッパー関数で用いるデータ型に変換を実施する
        for (int i = 0; i < targetChannelDefectionList.length(); ++i) {

            // チャンネル番号文字列
            strncpy(lc70Data.chSetRow[i].cStrChNo,
                    static_cast<ChannelDefinitionItem>(targetChannelDefectionList.at(i)).channelNumber.toLocal8Bit().constData(),
                    LC70_CH_SET_CH_STRLEN);

            // QAM 中心周波数[kHz]
            lc70Data.chSetRow[i].ulCentFreqQam = static_cast<ChannelDefinitionItem>(targetChannelDefectionList.at(
                    i)).centerFrequency1;

            // OFDM中心周波数[kHz]
            lc70Data.chSetRow[i].ulCentFreqOfdm = static_cast<ChannelDefinitionItem>(targetChannelDefectionList.at(
                    i)).centerFrequency2;
        }

        // 定義ファイルに保存を行う
        if (LC70_OK != _communicator->writeChannelConfigurationFile(LC70_FILE_ID_CH_SET_CATV_ALL, &lc70Data)) {
            // 処理結果が正常終了ではない場合はfalseを返し後続処理は実施しない
            return false;
        }
    }

    // ラッパー関数ライブラリ V2.00 対応
    // CATV_90_470MHz
    {
        QList<ChannelDefinitionItem> targetChannelDefectionList;
        targetChannelDefectionList = _channelDefinitionMap.value(LC70_FILE_ID_CH_SET_CATV_90_470MHZ);

        if (targetChannelDefectionList.isEmpty()) {
            // 対応するチャンネル定義リストが空の場合、Falseを返す
            return false;
        }

        LC70_CH_SET_CATV_ALL_T lc70Data;

        // ラッパー関数で用いるデータ型に変換を実施する
        for (int i = 0; i < targetChannelDefectionList.length(); ++i) {

            // チャンネル番号文字列
            strncpy(lc70Data.chSetRow[i].cStrChNo,
                    static_cast<ChannelDefinitionItem>(targetChannelDefectionList.at(i)).channelNumber.toLocal8Bit().constData(),
                    LC70_CH_SET_CH_STRLEN);

            // QAM 中心周波数[kHz]
            lc70Data.chSetRow[i].ulCentFreqQam = static_cast<ChannelDefinitionItem>(targetChannelDefectionList.at(
                    i)).centerFrequency1;

            // OFDM中心周波数[kHz]
            lc70Data.chSetRow[i].ulCentFreqOfdm = static_cast<ChannelDefinitionItem>(targetChannelDefectionList.at(
                    i)).centerFrequency2;
        }

        // 定義ファイルに保存を行う
        if (LC70_OK != _communicator->writeChannelConfigurationFile(LC70_FILE_ID_CH_SET_CATV_90_470MHZ, &lc70Data)) {
            // 処理結果が正常終了ではない場合はfalseを返し後続処理は実施しない
            return false;
        }
    }

    // ラッパー関数ライブラリ V2.00 対応
    // CATV_470_1002MHZ
    {
        QList<ChannelDefinitionItem> targetChannelDefectionList;
        targetChannelDefectionList = _channelDefinitionMap.value(LC70_FILE_ID_CH_SET_CATV_470_1002MHZ);

        if (targetChannelDefectionList.isEmpty()) {
            // 対応するチャンネル定義リストが空の場合、Falseを返す
            return false;
        }

        LC70_CH_SET_CATV_470_1002MHZ_T lc70Data;

        // ラッパー関数で用いるデータ型に変換を実施する
        for (int i = 0; i < targetChannelDefectionList.length(); ++i) {

            // チャンネル番号文字列
            strncpy(lc70Data.chSetRow[i].cStrChNo,
                    static_cast<ChannelDefinitionItem>(targetChannelDefectionList.at(i)).channelNumber.toLocal8Bit().constData(),
                    LC70_CH_SET_CH_STRLEN);

            // QAM 中心周波数[kHz]
            lc70Data.chSetRow[i].ulCentFreqQam = static_cast<ChannelDefinitionItem>(targetChannelDefectionList.at(
                    i)).centerFrequency1;

            // OFDM中心周波数[kHz]
            lc70Data.chSetRow[i].ulCentFreqOfdm = static_cast<ChannelDefinitionItem>(targetChannelDefectionList.at(
                    i)).centerFrequency2;
        }

        // 定義ファイルに保存を行う
        if (LC70_OK != _communicator->writeChannelConfigurationFile(LC70_FILE_ID_CH_SET_CATV_470_1002MHZ, &lc70Data)) {
            // 処理結果が正常終了ではない場合はfalseを返し後続処理は実施しない
            return false;
        }
    }

    // スカパー光
    {
        QList<ChannelDefinitionItem> targetChannelDefectionList;
        targetChannelDefectionList = _channelDefinitionMap.value(LC70_FILE_ID_CH_SET_SKYP);

        if (targetChannelDefectionList.isEmpty()) {
            // 対応するチャンネル定義リストが空の場合、Falseを返す
            return false;
        }

        LC70_CH_SET_SKYP_T lc70Data;

        // ラッパー関数で用いるデータ型に変換を実施する
        for (int i = 0; i < targetChannelDefectionList.length(); ++i) {

            // チャンネル番号文字列
            strncpy(lc70Data.chSetRow[i].cStrChNo,
                    static_cast<ChannelDefinitionItem>(targetChannelDefectionList.at(i)).channelNumber.toLocal8Bit().constData(),
                    LC70_CH_SET_CH_STRLEN);

            // 中心周波数[kHz]
            lc70Data.chSetRow[i].ulCentFreq = static_cast<ChannelDefinitionItem>(targetChannelDefectionList.at(i)).centerFrequency1;
        }

        // 定義ファイルに保存を行う
        if (LC70_OK != _communicator->writeChannelConfigurationFile(LC70_FILE_ID_CH_SET_SKYP, &lc70Data)) {
            // 処理結果が正常終了ではない場合はfalseを返し後続処理は実施しない
            return false;
        }
    }

    // PILOT
    {
        QList<ChannelDefinitionItem> targetChannelDefectionList;
        targetChannelDefectionList = _channelDefinitionMap.value(LC70_FILE_ID_CH_SET_PILOT);

        if (targetChannelDefectionList.isEmpty()) {
            // 対応するチャンネル定義リストが空の場合、Falseを返す
            return false;
        }

        LC70_CH_SET_PILOT_T lc70Data;

        // ラッパー関数で用いるデータ型に変換を実施する
        for (int i = 0; i < targetChannelDefectionList.length(); ++i) {
            // 中心周波数[kHz]
            lc70Data.chSetRow[i].ulCentFreq = static_cast<ChannelDefinitionItem>(targetChannelDefectionList.at(i)).centerFrequency1;
        }

        // 定義ファイルに保存を行う
        if (LC70_OK != _communicator->writeChannelConfigurationFile(LC70_FILE_ID_CH_SET_PILOT, &lc70Data)) {
            // 処理結果が正常終了ではない場合はfalseを返し後続処理は実施しない
            return false;
        }
    }

    // LTE
    {
        QList<ChannelDefinitionItem> targetChannelDefectionList;
        targetChannelDefectionList = _channelDefinitionMap.value(LC70_FILE_ID_CH_SET_LTE);

        if (targetChannelDefectionList.isEmpty()) {
            // 対応するチャンネル定義リストが空の場合、Falseを返す
            return false;
        }

        LC70_CH_SET_LTE_T lc70Data;

        // ラッパー関数で用いるデータ型に変換を実施する
        for (int i = 0; i < targetChannelDefectionList.length(); ++i) {

            // チャンネル番号文字列
            strncpy(lc70Data.chSetRow[i].cStrChNo,
                    static_cast<ChannelDefinitionItem>(targetChannelDefectionList.at(i)).channelNumber.toLocal8Bit().constData(),
                    LC70_CH_SET_CH_STRLEN);

            // 中心周波数[kHz]
            lc70Data.chSetRow[i].ulCentFreq = static_cast<ChannelDefinitionItem>(targetChannelDefectionList.at(i)).centerFrequency1;

            // 帯域幅[kHz]
            lc70Data.chSetRow[i].ulBandWide = static_cast<ChannelDefinitionItem>(targetChannelDefectionList.at(i)).bandWidth;
        }

        // 定義ファイルに保存を行う
        if (LC70_OK != _communicator->writeChannelConfigurationFile(LC70_FILE_ID_CH_SET_LTE, &lc70Data)) {
            // 処理結果が正常終了ではない場合はfalseを返し後続処理は実施しない
            return false;
        }

        return true;
    }

    // FM
    {
        QList<ChannelDefinitionItem> targetChannelDefectionList;
        targetChannelDefectionList = _channelDefinitionMap.value(LC70_FILE_ID_CH_SET_FM);

        if (targetChannelDefectionList.isEmpty()) {
            // 対応するチャンネル定義リストが空の場合、Falseを返す
            return false;
        }

        LC70_CH_SET_FM_T lc70Data;

        // ラッパー関数で用いるデータ型に変換を実施する
        for (int i = 0; i < targetChannelDefectionList.length(); ++i) {
            // 中心周波数[kHz]
            lc70Data.chSetRow[i].ulCentFreq = static_cast<ChannelDefinitionItem>(targetChannelDefectionList.at(i)).centerFrequency1;
        }

        // 定義ファイルに保存を行う
        if (LC70_OK != _communicator->writeChannelConfigurationFile(LC70_FILE_ID_CH_SET_FM, &lc70Data)) {
            // 処理結果が正常終了ではない場合はfalseを返し後続処理は実施しない
            return false;
        }
    }

    // CATV上り
    {
        QList<ChannelDefinitionItem> targetChannelDefectionList;
        targetChannelDefectionList = _channelDefinitionMap.value(LC70_FILE_ID_CH_SET_CATV_UP_STREAM);

        if (targetChannelDefectionList.isEmpty()) {
            // 対応するチャンネル定義リストが空の場合、Falseを返す
            return false;
        }

        LC70_CH_SET_CATV_UP_STREAM_T lc70Data;

        // ラッパー関数で用いるデータ型に変換を実施する
        for (int i = 0; i < targetChannelDefectionList.length(); ++i) {
            // 中心周波数[kHz]
            lc70Data.chSetRow[i].ulCentFreq = static_cast<ChannelDefinitionItem>(targetChannelDefectionList.at(i)).centerFrequency1;
        }

        // 定義ファイルに保存を行う
        if (LC70_OK != _communicator->writeChannelConfigurationFile(LC70_FILE_ID_CH_SET_CATV_UP_STREAM, &lc70Data)) {
            // 処理結果が正常終了ではない場合はfalseを返し後続処理は実施しない
            return false;
        }
    }

    // アンテナ調整のチャンネル定義は更新しないため割愛
}

bool ModelOperator::commitChannelDefinitionData(int bandId)
{
    if (_channelDefinitionMap.isEmpty()) {
        // チャンネル定義情報マップが空の場合、Falseを返す
        return false;
    }

    // 地デジ
    if (bandId == LC70_BAND_UHF) {
        QList<ChannelDefinitionItem> targetChannelDefectionList;
        targetChannelDefectionList = _channelDefinitionMap.value(LC70_FILE_ID_CH_SET_UHF);

        if (targetChannelDefectionList.isEmpty()) {
            // 対応するチャンネル定義リストが空の場合、Falseを返す
            return false;
        }

        LC70_CH_SET_UHF_T lc70Data;

        // ラッパー関数で用いるデータ型に変換を実施する
        for (int i = 0; i < targetChannelDefectionList.length(); ++i) {

            // チャンネル番号文字列
            strncpy(lc70Data.chSetRow[i].cStrChNo,
                    static_cast<ChannelDefinitionItem>(targetChannelDefectionList.at(i)).channelNumber.toLocal8Bit().constData(),
                    LC70_CH_SET_CH_STRLEN);

            // 中心周波数
            lc70Data.chSetRow[i].ulCentFreq = static_cast<ChannelDefinitionItem>(targetChannelDefectionList.at(i)).centerFrequency1;
        }

        // 定義ファイルに保存を行う
        if (LC70_OK != _communicator->writeChannelConfigurationFile(LC70_FILE_ID_CH_SET_UHF, &lc70Data)) {
            // 処理結果が正常終了ではない場合はfalseを返し後続処理は実施しない
            return false;
        }

        // 正常終了の場合trueを返す
        return true;
    }

    // BS
    if (bandId == LC70_BAND_BS) {
        QList<ChannelDefinitionItem> targetChannelDefectionList;
        targetChannelDefectionList = _channelDefinitionMap.value(LC70_FILE_ID_CH_SET_BS);

        if (targetChannelDefectionList.isEmpty()) {
            // 対応するチャンネル定義リストが空の場合、Falseを返す
            return false;
        }

        LC70_CH_SET_BS_T lc70Data;

        // ラッパー関数で用いるデータ型に変換を実施する
        for (int i = 0; i < targetChannelDefectionList.length(); ++i) {

            // チャンネル番号文字列
            strncpy(lc70Data.chSetRow[i].cStrChNo,
                    static_cast<ChannelDefinitionItem>(targetChannelDefectionList.at(i)).channelNumber.toLocal8Bit().constData(),
                    LC70_CH_SET_CH_STRLEN);

            // 中心周波数
            lc70Data.chSetRow[i].ulCentFreq = static_cast<ChannelDefinitionItem>(targetChannelDefectionList.at(i)).centerFrequency1;

            // 偏波番号
            lc70Data.chSetRow[i].ucWave = static_cast<ChannelDefinitionItem>(targetChannelDefectionList.at(i)).polarization;
        }

        // 定義ファイルに保存を行う。
        if (LC70_OK != _communicator->writeChannelConfigurationFile(LC70_FILE_ID_CH_SET_BS, &lc70Data)) {
            // 処理結果が正常終了ではない場合はfalseを返し後続処理は実施しない
            return false;
        }

        // 正常終了の場合trueを返す
        return true;
    }

    // CS
    if (bandId == LC70_BAND_CS) {
        QList<ChannelDefinitionItem> targetChannelDefectionList;
        targetChannelDefectionList = _channelDefinitionMap.value(LC70_FILE_ID_CH_SET_CS);

        if (targetChannelDefectionList.isEmpty()) {
            // 対応するチャンネル定義リストが空の場合、Falseを返す
            return false;
        }

        LC70_CH_SET_CS_T lc70Data;

        // ラッパー関数で用いるデータ型に変換を実施する
        for (int i = 0; i < targetChannelDefectionList.length(); ++i) {

            // チャンネル番号文字列
            strncpy(lc70Data.chSetRow[i].cStrChNo,
                    static_cast<ChannelDefinitionItem>(targetChannelDefectionList.at(i)).channelNumber.toLocal8Bit().constData(),
                    LC70_CH_SET_CH_STRLEN);

            // 中心周波数[kHz]
            lc70Data.chSetRow[i].ulCentFreq = static_cast<ChannelDefinitionItem>(targetChannelDefectionList.at(i)).centerFrequency1;

            // 偏波番号
            lc70Data.chSetRow[i].ucWave = static_cast<ChannelDefinitionItem>(targetChannelDefectionList.at(i)).polarization;

        }

        // 定義ファイルに保存を行う
        if (LC70_OK != _communicator->writeChannelConfigurationFile(LC70_FILE_ID_CH_SET_CS, &lc70Data)) {
            // 処理結果が正常終了ではない場合はfalseを返し後続処理は実施しない
            return false;
        }

        // 正常終了の場合trueを返す
        return true;
    }

    // SCCB
    if (bandId == LC70_BAND_SCCB) {
        QList<ChannelDefinitionItem> targetChannelDefectionList;
        targetChannelDefectionList = _channelDefinitionMap.value(LC70_FILE_ID_CH_SET_SCCB);

        if (targetChannelDefectionList.isEmpty()) {
            // 対応するチャンネル定義リストが空の場合、Falseを返す
            return false;
        }

        LC70_CH_SET_SCCB_T lc70Data;

        // ラッパー関数で用いるデータ型に変換を実施する
        for (int i = 0; i < targetChannelDefectionList.length(); ++i) {

            // チャンネル番号文字列
            strncpy(lc70Data.chSetRow[i].cStrChNo,
                    static_cast<ChannelDefinitionItem>(targetChannelDefectionList.at(i)).channelNumber.toLocal8Bit().constData(),
                    LC70_CH_SET_CH_STRLEN);

            // 中心周波数[kHz]
            lc70Data.chSetRow[i].ulCentFreq = static_cast<ChannelDefinitionItem>(targetChannelDefectionList.at(i)).centerFrequency1;

            // 偏波番号
            lc70Data.chSetRow[i].ucWave = static_cast<ChannelDefinitionItem>(targetChannelDefectionList.at(i)).polarization;
        }

        // 定義ファイルに保存を行う
        if (LC70_OK != _communicator->writeChannelConfigurationFile(LC70_FILE_ID_CH_SET_SCCB, &lc70Data)) {
            // 処理結果が正常終了ではない場合はfalseを返し後続処理は実施しない
            return false;
        }

        // 正常終了の場合trueを返す
        return true;
    }

    // SCCC
    if (bandId == LC70_BAND_SCCC) {
        QList<ChannelDefinitionItem> targetChannelDefectionList;
        targetChannelDefectionList = _channelDefinitionMap.value(LC70_FILE_ID_CH_SET_SCCC);

        if (targetChannelDefectionList.isEmpty()) {
            // 対応するチャンネル定義リストが空の場合、Falseを返す
            return false;
        }

        LC70_CH_SET_SCCC_T lc70Data;

        // ラッパー関数で用いるデータ型に変換を実施する
        for (int i = 0; i < targetChannelDefectionList.length(); ++i) {

            // チャンネル番号文字列
            strncpy(lc70Data.chSetRow[i].cStrChNo,
                    static_cast<ChannelDefinitionItem>(targetChannelDefectionList.at(i)).channelNumber.toLocal8Bit().constData(),
                    LC70_CH_SET_CH_STRLEN);

            // 中心周波数[kHz]
            lc70Data.chSetRow[i].ulCentFreq = static_cast<ChannelDefinitionItem>(targetChannelDefectionList.at(i)).centerFrequency1;

            // 偏波番号
            lc70Data.chSetRow[i].ucWave = static_cast<ChannelDefinitionItem>(targetChannelDefectionList.at(i)).polarization;
        }

        // 定義ファイルに保存を行う
        if (LC70_OK != _communicator->writeChannelConfigurationFile(LC70_FILE_ID_CH_SET_SCCC, &lc70Data)) {
            // 処理結果が正常終了ではない場合はfalseを返し後続処理は実施しない
            return false;
        }

        // 正常終了の場合trueを返す
        return true;
    }

    // JSAT-3A
    if (bandId == LC70_BAND_JCSAT3) {
        QList<ChannelDefinitionItem> targetChannelDefectionList;
        targetChannelDefectionList = _channelDefinitionMap.value(LC70_FILE_ID_CH_SET_JSAT3A);

        if (targetChannelDefectionList.isEmpty()) {
            // 対応するチャンネル定義リストが空の場合、Falseを返す
            return false;
        }

        LC70_CH_SET_JSAT3A_T lc70Data;

        // ラッパー関数で用いるデータ型に変換を実施する
        for (int i = 0; i < targetChannelDefectionList.length(); ++i) {

            // チャンネル番号文字列
            strncpy(lc70Data.chSetRow[i].cStrChNo,
                    static_cast<ChannelDefinitionItem>(targetChannelDefectionList.at(i)).channelNumber.toLocal8Bit().constData(),
                    LC70_CH_SET_CH_STRLEN);

            // 中心周波数[kHz]
            lc70Data.chSetRow[i].ulCentFreq = static_cast<ChannelDefinitionItem>(targetChannelDefectionList.at(i)).centerFrequency1;

            // 偏波番号
            lc70Data.chSetRow[i].ucWave = static_cast<ChannelDefinitionItem>(targetChannelDefectionList.at(i)).polarization;
        }

        // 定義ファイルに保存を行う
        if (LC70_OK != _communicator->writeChannelConfigurationFile(LC70_FILE_ID_CH_SET_JSAT3A, &lc70Data)) {
            // 処理結果が正常終了ではない場合はfalseを返し後続処理は実施しない
            return false;
        }

        // 正常終了の場合trueを返す
        return true;
    }

    // JSAT4B
    if (bandId == LC70_BAND_JCSAT4) {
        QList<ChannelDefinitionItem> targetChannelDefectionList;
        targetChannelDefectionList = _channelDefinitionMap.value(LC70_FILE_ID_CH_SET_JSAT4B);

        if (targetChannelDefectionList.isEmpty()) {
            // 対応するチャンネル定義リストが空の場合、Falseを返す
            return false;
        }

        LC70_CH_SET_JSAT4B_T lc70Data;

        // ラッパー関数で用いるデータ型に変換を実施する
        for (int i = 0; i < targetChannelDefectionList.length(); ++i) {

            // チャンネル番号文字列
            strncpy(lc70Data.chSetRow[i].cStrChNo,
                    static_cast<ChannelDefinitionItem>(targetChannelDefectionList.at(i)).channelNumber.toLocal8Bit().constData(),
                    LC70_CH_SET_CH_STRLEN);
            // 中心周波数[kHz]
            lc70Data.chSetRow[i].ulCentFreq = static_cast<ChannelDefinitionItem>(targetChannelDefectionList.at(i)).centerFrequency1;

            // 偏波番号
            lc70Data.chSetRow[i].ucWave = static_cast<ChannelDefinitionItem>(targetChannelDefectionList.at(i)).polarization;
        }

        // 定義ファイルに保存を行う
        if (LC70_OK != _communicator->writeChannelConfigurationFile(LC70_FILE_ID_CH_SET_JSAT4B, &lc70Data)) {
            // 処理結果が正常終了ではない場合はfalseを返し後続処理は実施しない
            return false;
        }

        // 正常終了の場合trueを返す
        return true;
    }

    // ラッパー関数ライブラリ V2.00 対応
    // CATV_ALL
    if (bandId == LC70_BAND_CATV_ALL) {
        QList<ChannelDefinitionItem> targetChannelDefectionList;
        targetChannelDefectionList = _channelDefinitionMap.value(LC70_FILE_ID_CH_SET_CATV_ALL);

        if (targetChannelDefectionList.isEmpty()) {
            // 対応するチャンネル定義リストが空の場合、Falseを返す
            return false;
        }

        LC70_CH_SET_CATV_ALL_T lc70Data;

        // ラッパー関数で用いるデータ型に変換を実施する
        for (int i = 0; i < targetChannelDefectionList.length(); ++i) {

            // チャンネル番号文字列
            strncpy(lc70Data.chSetRow[i].cStrChNo,
                    static_cast<ChannelDefinitionItem>(targetChannelDefectionList.at(i)).channelNumber.toLocal8Bit().constData(),
                    LC70_CH_SET_CH_STRLEN);

            // QAM 中心周波数[kHz]
            lc70Data.chSetRow[i].ulCentFreqQam = static_cast<ChannelDefinitionItem>(targetChannelDefectionList.at(
                    i)).centerFrequency1;

            // OFDM中心周波数[kHz]
            lc70Data.chSetRow[i].ulCentFreqOfdm = static_cast<ChannelDefinitionItem>(targetChannelDefectionList.at(
                    i)).centerFrequency2;
        }

        // 定義ファイルに保存を行う
        if (LC70_OK != _communicator->writeChannelConfigurationFile(LC70_FILE_ID_CH_SET_CATV_ALL, &lc70Data)) {
            // 処理結果が正常終了ではない場合はfalseを返し後続処理は実施しない
            return false;
        }

        // 正常終了の場合trueを返す
        return true;
    }

    // ラッパー関数ライブラリ V2.00 対応
    // CATV_90_470MHZ
    if (bandId == LC70_BAND_CATV_90_470MHZ) {
        QList<ChannelDefinitionItem> targetChannelDefectionList;
        targetChannelDefectionList = _channelDefinitionMap.value(LC70_FILE_ID_CH_SET_CATV_90_470MHZ);

        if (targetChannelDefectionList.isEmpty()) {
            // 対応するチャンネル定義リストが空の場合、Falseを返す
            return false;
        }

        LC70_CH_SET_CATV_90_470MHZ_T lc70Data;

        // ラッパー関数で用いるデータ型に変換を実施する
        for (int i = 0; i < targetChannelDefectionList.length(); ++i) {

            // チャンネル番号文字列
            strncpy(lc70Data.chSetRow[i].cStrChNo,
                    static_cast<ChannelDefinitionItem>(targetChannelDefectionList.at(i)).channelNumber.toLocal8Bit().constData(),
                    LC70_CH_SET_CH_STRLEN);

            // QAM 中心周波数[kHz]
            lc70Data.chSetRow[i].ulCentFreqQam = static_cast<ChannelDefinitionItem>(targetChannelDefectionList.at(
                    i)).centerFrequency1;

            // OFDM中心周波数[kHz]
            lc70Data.chSetRow[i].ulCentFreqOfdm = static_cast<ChannelDefinitionItem>(targetChannelDefectionList.at(
                    i)).centerFrequency2;
        }

        // 定義ファイルに保存を行う
        if (LC70_OK != _communicator->writeChannelConfigurationFile(LC70_FILE_ID_CH_SET_CATV_90_470MHZ, &lc70Data)) {
            // 処理結果が正常終了ではない場合はfalseを返し後続処理は実施しない
            return false;
        }

        // 正常終了の場合trueを返す
        return true;
    }

    // ラッパー関数ライブラリ V2.00 対応
    // CATV_470_1002MHZ
    if (bandId == LC70_BAND_CATV_470_1002MHZ) {
        QList<ChannelDefinitionItem> targetChannelDefectionList;
        targetChannelDefectionList = _channelDefinitionMap.value(LC70_FILE_ID_CH_SET_CATV_470_1002MHZ);

        if (targetChannelDefectionList.isEmpty()) {
            // 対応するチャンネル定義リストが空の場合、Falseを返す
            return false;
        }

        LC70_CH_SET_CATV_470_1002MHZ_T lc70Data;

        // ラッパー関数で用いるデータ型に変換を実施する
        for (int i = 0; i < targetChannelDefectionList.length(); ++i) {

            // チャンネル番号文字列
            strncpy(lc70Data.chSetRow[i].cStrChNo,
                    static_cast<ChannelDefinitionItem>(targetChannelDefectionList.at(i)).channelNumber.toLocal8Bit().constData(),
                    LC70_CH_SET_CH_STRLEN);

            // QAM 中心周波数[kHz]
            lc70Data.chSetRow[i].ulCentFreqQam = static_cast<ChannelDefinitionItem>(targetChannelDefectionList.at(
                    i)).centerFrequency1;

            // OFDM中心周波数[kHz]
            lc70Data.chSetRow[i].ulCentFreqOfdm = static_cast<ChannelDefinitionItem>(targetChannelDefectionList.at(
                    i)).centerFrequency2;
        }

        // 定義ファイルに保存を行う
        if (LC70_OK != _communicator->writeChannelConfigurationFile(LC70_FILE_ID_CH_SET_CATV_470_1002MHZ, &lc70Data)) {
            // 処理結果が正常終了ではない場合はfalseを返し後続処理は実施しない
            return false;
        }

        // 正常終了の場合trueを返す
        return true;
    }

    // スカパー光
    if (bandId == LC70_BAND_SKYP) {
        QList<ChannelDefinitionItem> targetChannelDefectionList;
        targetChannelDefectionList = _channelDefinitionMap.value(LC70_FILE_ID_CH_SET_SKYP);

        if (targetChannelDefectionList.isEmpty()) {
            // 対応するチャンネル定義リストが空の場合、Falseを返す
            return false;
        }

        LC70_CH_SET_SKYP_T lc70Data;

        // ラッパー関数で用いるデータ型に変換を実施する
        for (int i = 0; i < targetChannelDefectionList.length(); ++i) {

            // チャンネル番号文字列
            strncpy(lc70Data.chSetRow[i].cStrChNo,
                    static_cast<ChannelDefinitionItem>(targetChannelDefectionList.at(i)).channelNumber.toLocal8Bit().constData(),
                    LC70_CH_SET_CH_STRLEN);

            // 中心周波数[kHz]
            lc70Data.chSetRow[i].ulCentFreq = static_cast<ChannelDefinitionItem>(targetChannelDefectionList.at(i)).centerFrequency1;
        }

        // 定義ファイルに保存を行う
        if (LC70_OK != _communicator->writeChannelConfigurationFile(LC70_FILE_ID_CH_SET_SKYP, &lc70Data)) {
            // 処理結果が正常終了ではない場合はfalseを返し後続処理は実施しない
            return false;
        }

        // 正常終了の場合trueを返す
        return true;
    }

    // PILOT
    if (bandId == LC70_BAND_PILOT) {
        QList<ChannelDefinitionItem> targetChannelDefectionList;
        targetChannelDefectionList = _channelDefinitionMap.value(LC70_FILE_ID_CH_SET_PILOT);

        if (targetChannelDefectionList.isEmpty()) {
            // 対応するチャンネル定義リストが空の場合、Falseを返す
            return false;
        }

        LC70_CH_SET_PILOT_T lc70Data;

        // ラッパー関数で用いるデータ型に変換を実施する
        for (int i = 0; i < targetChannelDefectionList.length(); ++i) {
            // 中心周波数[kHz]
            lc70Data.chSetRow[i].ulCentFreq = static_cast<ChannelDefinitionItem>(targetChannelDefectionList.at(i)).centerFrequency1;
        }

        // 定義ファイルに保存を行う
        if (LC70_OK != _communicator->writeChannelConfigurationFile(LC70_FILE_ID_CH_SET_PILOT, &lc70Data)) {
            // 処理結果が正常終了ではない場合はfalseを返し後続処理は実施しない
            return false;
        }

        // 正常終了の場合trueを返す
        return true;
    }

    // LTE
    if (bandId == LC70_BAND_LTE) {
        QList<ChannelDefinitionItem> targetChannelDefectionList;
        targetChannelDefectionList = _channelDefinitionMap.value(LC70_FILE_ID_CH_SET_LTE);

        if (targetChannelDefectionList.isEmpty()) {
            // 対応するチャンネル定義リストが空の場合、Falseを返す
            return false;
        }

        LC70_CH_SET_LTE_T lc70Data;

        // ラッパー関数で用いるデータ型に変換を実施する
        for (int i = 0; i < targetChannelDefectionList.length(); ++i) {

            // チャンネル番号文字列
            strncpy(lc70Data.chSetRow[i].cStrChNo,
                    static_cast<ChannelDefinitionItem>(targetChannelDefectionList.at(i)).channelNumber.toLocal8Bit().constData(),
                    LC70_CH_SET_CH_STRLEN);

            // 中心周波数[kHz]
            lc70Data.chSetRow[i].ulCentFreq = static_cast<ChannelDefinitionItem>(targetChannelDefectionList.at(i)).centerFrequency1;

            // 帯域幅[kHz]
            lc70Data.chSetRow[i].ulBandWide = static_cast<ChannelDefinitionItem>(targetChannelDefectionList.at(i)).bandWidth;
        }

        // 定義ファイルに保存を行う
        if (LC70_OK != _communicator->writeChannelConfigurationFile(LC70_FILE_ID_CH_SET_LTE, &lc70Data)) {
            // 処理結果が正常終了ではない場合はfalseを返し後続処理は実施しない
            return false;
        }

        // 正常終了の場合trueを返す
        return true;
    }

    // FM
    if (bandId == LC70_BAND_FM) {
        QList<ChannelDefinitionItem> targetChannelDefectionList;
        targetChannelDefectionList = _channelDefinitionMap.value(LC70_FILE_ID_CH_SET_FM);

        if (targetChannelDefectionList.isEmpty()) {
            // 対応するチャンネル定義リストが空の場合、Falseを返す
            return false;
        }

        LC70_CH_SET_FM_T lc70Data;

        // ラッパー関数で用いるデータ型に変換を実施する
        for (int i = 0; i < targetChannelDefectionList.length(); ++i) {
            // 中心周波数[kHz]
            lc70Data.chSetRow[i].ulCentFreq = static_cast<ChannelDefinitionItem>(targetChannelDefectionList.at(i)).centerFrequency1;
        }

        // 定義ファイルに保存を行う
        if (LC70_OK != _communicator->writeChannelConfigurationFile(LC70_FILE_ID_CH_SET_FM, &lc70Data)) {
            // 処理結果が正常終了ではない場合はfalseを返し後続処理は実施しない
            return false;
        }

        // 正常終了の場合trueを返す
        return true;
    }

    // CATV上り
    if (bandId == LC70_BAND_CATV_UP_STREAM) {
        QList<ChannelDefinitionItem> targetChannelDefectionList;
        targetChannelDefectionList = _channelDefinitionMap.value(LC70_FILE_ID_CH_SET_CATV_UP_STREAM);

        if (targetChannelDefectionList.isEmpty()) {
            // 対応するチャンネル定義リストが空の場合、Falseを返す
            return false;
        }

        LC70_CH_SET_CATV_UP_STREAM_T lc70Data;

        // ラッパー関数で用いるデータ型に変換を実施する
        for (int i = 0; i < targetChannelDefectionList.length(); ++i) {
            // 中心周波数[kHz]
            lc70Data.chSetRow[i].ulCentFreq = static_cast<ChannelDefinitionItem>(targetChannelDefectionList.at(i)).centerFrequency1;
        }

        // 定義ファイルに保存を行う
        if (LC70_OK != _communicator->writeChannelConfigurationFile(LC70_FILE_ID_CH_SET_CATV_UP_STREAM, &lc70Data)) {
            // 処理結果が正常終了ではない場合はfalseを返し後続処理は実施しない
            return false;
        }

        // 正常終了の場合trueを返す
        return true;
    }

    // アンテナ調整用のチャンネル定義ファイルは更新しないため割愛

    // 対応するデータがない場合はfalseを返す
    return false;
}

bool ModelOperator::startOpticalMeasurement(uint32_t waveLength)
{
    bool retcode = false;

    // 光測定を開始する
    if (LC70_OK == _communicator->startOpticalMeasurement(waveLength)) {
        retcode = true;
    }

    return retcode;
}

bool ModelOperator::stopOpticalMeasurement()
{
    bool retcode = false;

    // 光測定を停止する
    if (LC70_OK == _communicator->stopOpticalMeasurement()) {
        retcode = true;
    }

    return retcode;
}

bool ModelOperator::getOpticalMeasurementResult(QVariant &result)
{
    bool retcode = false;

    // 光測定の測定結果を取得する
    if (LC70_OK == _communicator->getOpticalMeasurementResult(result)) {
        retcode = true;
    }

    return retcode;
}

bool ModelOperator::startDCMeasurement()
{
    bool retcode = false;

    // DC測定を開始する
    if (LC70_OK == _communicator->startDCMeasurement()) {
        retcode = true;
    }

    return retcode;
}

bool ModelOperator::stopDCMeasurement()
{
    bool retcode = false;

    // DC測定を停止する
    if (LC70_OK == _communicator->stopDCMeasurement()) {
        retcode = true;
    }

    return retcode;
}

bool ModelOperator::getDCMeasurementResult(QVariant &result)
{
    bool retcode = false;

    // DC測定の測定結果を取得する
    if (LC70_OK == _communicator->getDCMeasurementResult(result)) {
        retcode = true;
    }

    return retcode;
}

bool ModelOperator::getThresholdInitializeValue(QVariant &value)
{
    LC70_MEAS_SET_THRD_T lc70Data;

    if (0 > _communicator->readMeasureSettingFile(LC70_FILE_ID_MEAS_SET_THRD, &lc70Data, true)) {
        return false;
    }

    QVariantList rowData;
    QVariantList thrdData;

    for (int i = 0; i < LC70_MEAS_SET_THRD_IDX_MAX; ++i) {
        LC70_MEAS_SET_THRD_ROW_T *item = &lc70Data.measSetThrdRow[i];

        rowData = {item->ucBand,              // バンド
                   item->level.lLow,          // LEVEL  受信不可 閾値
                   item->level.lHigh,         // LEVEL  受信可   閾値
                   item->mer.ulLow,           // MER/CN 余裕度低 閾値
                   item->mer.ulHigh,          // MER/CN 余裕度高 閾値
                   item->ber.low.ulMantissa,  // BER 余裕度低 閾値 仮数部
                   item->ber.low.lExponent,   // BER 余裕度低 閾値 指数部
                   item->ber.high.ulMantissa, // BER 余裕度高 閾値 仮数部
                   item->ber.high.lExponent   // BER 余裕度高 閾値 指数部
                  };
        thrdData.insert(i, rowData);
    }

    value = thrdData;
    return true;
}

void ModelOperator::initializeUserSettingFile()
{
    // ユーザー設定とデータロガー設定は分離(ここは影響しないはず)
    // VALUE_USERSETTING_FILELIST
    {
        QList<LC70_LOGGER_SET_LOGGER_FLIST_ROW_T> filelist;

        QVariantList data;
        QVariantList value;

        // USER  設定ファイル名リストは取得時に読み込み
        _communicator->readUserLogSetFileList(_dataloggerSettingDir, filelist,
                                              CHANNEL_EDIT_USER,
                                              _applicationSettings->settingDataPathUserData(),
                                              _applicationSettings->settingDataUserSettingFileListName());

        int index = 0;

        foreach (const LC70_LOGGER_SET_LOGGER_FLIST_ROW_T &item, filelist) {
            value = {item.ucNo, item.ucEnbFlg, item.cFileName};
            data.insert(index, value);
            index++;
        }

        _values[VALUE_USERSETTING_FILELIST].setInitialValue(data);
    }

    // VALUE_USERSETTING_FILELIST_USB1
    {
        QList<LC70_LOGGER_SET_LOGGER_FLIST_ROW_T> filelist;

        QVariantList data;
        QVariantList value;

        // USER  設定ファイル名リストは取得時に読み込み
        _communicator->readUserLogSetFileList(_dataloggerSettingDir, filelist,
                                              CHANNEL_EDIT_USER_USB1,
                                              _applicationSettings->settingDataPathUserDataUsb1(),
                                              _applicationSettings->settingDataUserSettingUsb1FileListName());

        int index = 0;

        foreach (const LC70_LOGGER_SET_LOGGER_FLIST_ROW_T &item, filelist) {
            value = {item.ucNo, item.ucEnbFlg, item.cFileName};
            data.insert(index, value);
            index++;
        }

        _values[VALUE_USERSETTING_FILELIST_USB1].setInitialValue(data);
    }

    // VALUE_USERSETTING_FILELIST_USB2
    {
        QList<LC70_LOGGER_SET_LOGGER_FLIST_ROW_T> filelist;

        QVariantList data;
        QVariantList value;

        // USER  設定ファイル名リストは取得時に読み込み
        _communicator->readUserLogSetFileList(_dataloggerSettingDir, filelist,
                                              CHANNEL_EDIT_USER_USB2,
                                              _applicationSettings->settingDataPathUserDataUsb2(),
                                              _applicationSettings->settingDataUserSettingUsb2FileListName());

        int index = 0;

        foreach (const LC70_LOGGER_SET_LOGGER_FLIST_ROW_T &item, filelist) {
            value = {item.ucNo, item.ucEnbFlg, item.cFileName};
            data.insert(index, value);
            index++;
        }

        _values[VALUE_USERSETTING_FILELIST_USB2].setInitialValue(data);
    }

    // VALUE_DATALOGGERSETTING_FILELIST
    {
        QList<LC70_LOGGER_SET_LOGGER_FLIST_ROW_T> filelist;

        QVariantList data;
        QVariantList value;

        _communicator->readUserLogSetFileList(_dataloggerSettingDir, filelist,
                                              CHANNEL_EDIT_DATA_LOGGER,
                                              _applicationSettings->settingDataPathLogger(),
                                              _applicationSettings->settingDataLoggerSettingFileListName());

        int index = 0;

        foreach (const LC70_LOGGER_SET_LOGGER_FLIST_ROW_T &item, filelist) {
            value = {item.ucNo, item.ucEnbFlg, item.cFileName};
            data.insert(index, value);
            index++;
        }

        _values[VALUE_DATALOGGERSETTING_FILELIST].setInitialValue(data);
    }

    // VALUE_DATALOGGERSETTING_FILELIST_USB1
    {
        QList<LC70_LOGGER_SET_LOGGER_FLIST_ROW_T> filelist;

        QVariantList data;
        QVariantList value;

        _communicator->readUserLogSetFileList(_dataloggerSettingDir, filelist,
                                              CHANNEL_EDIT_DATA_LOGGER_USB1,
                                              _applicationSettings->settingDataPathLoggerUsb1(),
                                              _applicationSettings->settingDataLoggerSettingUsb1FileListName());

        int index = 0;

        foreach (const LC70_LOGGER_SET_LOGGER_FLIST_ROW_T &item, filelist) {
            value = {item.ucNo, item.ucEnbFlg, item.cFileName};
            data.insert(index, value);
            index++;
        }

        _values[VALUE_DATALOGGERSETTING_FILELIST_USB1].setInitialValue(data);
    }

    // VALUE_DATALOGGERSETTING_FILELIST_USB2
    {
        QList<LC70_LOGGER_SET_LOGGER_FLIST_ROW_T> filelist;

        QVariantList data;
        QVariantList value;

        _communicator->readUserLogSetFileList(_dataloggerSettingDir, filelist,
                                              CHANNEL_EDIT_DATA_LOGGER_USB2,
                                              _applicationSettings->settingDataPathLoggerUsb2(),
                                              _applicationSettings->settingDataLoggerSettingUsb2FileListName());

        int index = 0;

        foreach (const LC70_LOGGER_SET_LOGGER_FLIST_ROW_T &item, filelist) {
            value = {item.ucNo, item.ucEnbFlg, item.cFileName};
            data.insert(index, value);
            index++;
        }

        _values[VALUE_DATALOGGERSETTING_FILELIST_USB2].setInitialValue(data);
    }
}

bool ModelOperator::readyToMeasure()
{
    bool result = false;
    static int count = 0;

    while (!result) {
        if (count == MAXIMUM_REPEAT_MEASURE_STATUS) {
            break;
        }
        LC70_MEAS_STAT_T status;
        memset(&status, 0, sizeof(LC70_MEAS_STAT_T));
        LC70_ERR_T ret = _communicator->measureStatus(&status);

        if (ret == LC70_OK) {
            if (status.ulMeasStat == LC70_MEAS_STAT_ENB &&
                    (status.usInputStat == LC70_INPUT_RF || status.usInputStat == LC70_INPUT_OPT)) {
                result = true;
                count = 0;
                break;
            }
        }
        ++count;

        // 一定回数測定状態取得のエラーが続けばサブ CPU をリセットする
        if (count == RESET_MEASURE_STATUS) {
            _communicator->resetSubCpu();
        }

        QThread::msleep(INTERVAL_MEASURE_STATUS);
    }

    return result;
}

bool ModelOperator::readyToStandBy()
{
    bool result = false;
    static int count = 0;

    while (!result) {
        if (count == MAXIMUM_REPEAT_MEASURE_STATUS) {
            break;
        }
        LC70_STANDBY_STAT_T status;
        memset(&status, 0, sizeof(LC70_STANDBY_STAT_T));
        LC70_ERR_T ret = _communicator->standByStatus(&status);

        if (ret == LC70_OK) {
            if (status.ulStandbyStat == LC70_STANDBY_STAT_STANDBY ||
                    status.ulStandbyStat == LC70_STANDBY_STAT_WAKEUP) {
                result = true;
                break;
            }
        }
        ++count;

        QThread::msleep(INTERVAL_MEASURE_STATUS);
    }

    return result;
}

bool ModelOperator::readyToWakeUp()
{
    bool result = false;
    static int count = 0;

    while (!result) {
        if (count == MAXIMUM_REPEAT_MEASURE_STATUS) {
            break;
        }
        LC70_STANDBY_STAT_T status;
        memset(&status, 0, sizeof(LC70_STANDBY_STAT_T));
        LC70_ERR_T ret = _communicator->wakeUpStatus(&status);

        if (ret == LC70_OK) {
            if (status.ulStandbyStat == LC70_STANDBY_STAT_WAKEUP ||
                    status.ulStandbyStat == LC70_STANDBY_STAT_STANDBY) {
                result = true;
                break;
            }
        }
        ++count;

        QThread::msleep(INTERVAL_MEASURE_STATUS);
    }

    return result;
}

bool ModelOperator::selectChannelDefinitionList(int bandId, QString polarization, QList<ChannelDefinitionItem> &channelList)
{
    QMap<QPair<LC70_BAND_T, QString>, int> polarizationValueMap = {
        {{LC70_BAND_UHF,    ""},                    -1},
        {{LC70_BAND_BS,     LEFT_HANDED_POLARIZE},  LC70_MEAS_SET_WAVE_L},
        {{LC70_BAND_BS,     RIGHT_HANDED_POLARIZE}, LC70_MEAS_SET_WAVE_R},
        {{LC70_BAND_CS,     LEFT_HANDED_POLARIZE},  LC70_MEAS_SET_WAVE_L},
        {{LC70_BAND_CS,     RIGHT_HANDED_POLARIZE}, LC70_MEAS_SET_WAVE_R},
        {{LC70_BAND_JCSAT3, HORIZONTALLY_POLARIZE}, LC70_MEAS_SET_WAVE_H},
        {{LC70_BAND_JCSAT3, VERTICALLY_POLARIZE},   LC70_MEAS_SET_WAVE_V},
        {{LC70_BAND_JCSAT4, HORIZONTALLY_POLARIZE}, LC70_MEAS_SET_WAVE_H},
        {{LC70_BAND_JCSAT4, VERTICALLY_POLARIZE},   LC70_MEAS_SET_WAVE_V},
        {{LC70_BAND_SCCB,   HORIZONTALLY_POLARIZE}, LC70_MEAS_SET_WAVE_H},
        {{LC70_BAND_SCCB,   VERTICALLY_POLARIZE},   LC70_MEAS_SET_WAVE_V},
        {{LC70_BAND_SCCC,   HORIZONTALLY_POLARIZE}, LC70_MEAS_SET_WAVE_H},
        {{LC70_BAND_SCCC,   VERTICALLY_POLARIZE},   LC70_MEAS_SET_WAVE_V},
        {{LC70_BAND_CATV_ALL, ""},                  -1},
        {{LC70_BAND_CATV_90_470MHZ, ""},            -1},
        {{LC70_BAND_CATV_470_1002MHZ, ""},          -1},
        {{LC70_BAND_PILOT,  ""},                    -1},
        {{LC70_BAND_SKYP,  ""},                     -1},
        {{LC70_BAND_LTE,  ""},                      -1},
        {{LC70_BAND_FM,  ""},                       -1},
        {{LC70_BAND_CATV_UP_STREAM, ""},            -1},
        {{LC70_BAND_FM_DEF,  ""},                   -1},
        {{LC70_BAND_CATV_UP_STREAM_DEF, ""},        -1},
        {{LC70_BAND_BS_CS, ""},                     -1},
    };

    QPair<LC70_BAND_T, QString> key(static_cast<LC70_BAND_T>(bandId), polarization);

    // KeyがMapに登録されていない場合は後続処理は実施しない
    if (polarizationValueMap.count(key) == 0) {
        return false;
    }

    QList<ChannelDefinitionItem> list;
    bool ret = getChannelDefinitionList(bandId, list);

    if (!ret) {
        return ret;
    }

    // 偏波面が無いBANDの場合はチャンネル定義リスト全てを返す
    if (polarizationValueMap.value(key) == -1) {
        // FM, CATV UP上りの場合は有効なチャンネルのみを抽出したリストを返す
        if ((bandId == LC70_BAND_FM_DEF) || (bandId == LC70_BAND_CATV_UP_STREAM_DEF)) {
            for (int i = 0; i < list.count(); ++i) {
                if (list[i].centerFrequency1 != 0) {
                    channelList.append(list[i]);
                }
            }
        } else {
            channelList = list;
        }
    } else {
        // 偏波面があるBANDの場合は対応するデータを抽出したリストを返す
        for (int i = 0; i < list.count(); ++i) {
            if (list[i].polarization == polarizationValueMap.value(key)) {
                channelList.append(list[i]);
            }
        }
    }

    return ret;
}
