/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include <QVariant>
#include "valueid.h"
#include "valueelement.h"

#include "helper/channeldefinitionhelper.h"
#include "constant.h"
#include "applicationsetting.h"
#include "stub.h"

#ifndef MODELFACADE_H
#define MODELFACADE_H

class Communicator;

/**
 * @brief The ModelOperator class
 *
 * 設定値や機器状態などの情報を保持し
 * 要求に応じて、更新や反映を行う
 *
 * 値は全てIDで区別し操作する
 *
 */
class ModelOperator
{
public:
    ModelOperator(Communicator *communicator, Stub *stub, ApplicationSetting *applicationSettings);
    virtual ~ModelOperator();

    /**
     * @brief initialize全ての設定値の初期設定を行う
     * @param defaultInit デフォルト値を読み込む場合には true を指定する
     * @return 処理結果
     */
    bool initialize(ActionMode actionMode, bool defaltInit = false);

    /**
     * @brief 設定ファイルに対してデフォルト値を設定する
     * @return 処理結果
     */
    bool initializeSettingFile(void);

    /**
     * @brief システム設定ファイルに対してデフォルト値を設定する
     * @return 処理結果
     */
    bool initializeSystemSettingFile(void);

    /**
     * @brief 測定設定ファイルに対してデフォルト値を設定する
     * @return 処理結果
     */
    bool initializeMeasureSettingFile(void);

    /**
     * @brief ユーザ/ロガー設定ファイルに対してデフォルト値を設定する
     * @return 処理結果
     */
    bool initializeUserLoggerSettingFile(void);

    /**
     * @brief チャンネル定義ファイルに対してデフォルト値を設定する
     * @return 処理結果
     */
    bool initializeChannelSettingFile(void);

    /**
     * @brief getValue
     * 設定値を取得する
     *
     * @param valueId 設定値の識別ID
     * @param value 設定値
     * @return 処理の成否
     */
    bool getValue(ValueId valueId, QVariant &value);

    /**
     * @brief setValue
     * 設定値を設定する
     * 設定した値は commit() の実行で確定データとなる
     * rollback() の実行で元の値に戻る
     *
     * @param valueId
     * @param value
     * @return
     */
    bool setValue(ValueId valueId, const QVariant &value);

    /**
     * @brief setValue
     * 設定値を初期設定する
     * 設定した値は commit() の実行で確定データとなる
     * rollback() の実行で元の値に戻る
     *
     * @param valueId
     * @param value
     * @return
     */
    bool setInitialValue(ValueId valueId, const QVariant &value);

    /**
     * @brief isUpdate
     * 指定した設定を更新したかを返す
     *
     * @param ValueId
     * @return true
     * @return false
     */
    bool isUpdate(ValueId valueId);

    /**
     * @brief rollback
     * @return
     */
    bool rollback(ValueId valueId);

    /**
     * @brief commit
     * @return
     */
    bool commit(ValueId valueId);

    /**
     * @brief データロガー設定のディレクトリ名を設定する
     * @param dirname
     */
    void setDataLoggerUserSettingDir(const QString &dirname);

    /**
     * @brief importDataLoggerUserSettingList
     * @param filename
     * @return
     */
    bool importDataLoggerUserSettingList(ValueId valueId, const QString &filename);

    /**
     * @brief exportDataLoggerUserSettingList
     * @param filename
     * @return
     */
    bool exportDataLoggerUserSettingList(ValueId valueId, const QString &filename);

    /**
     * @brief addDataLoggerUserSettingList
     * @param lineno    追加したい行番号
     */
    void addDataLoggerUserSettingList(int lineno);

    /**
     * @brief deleteDataLoggerUserSettingList
     * @param lineno
     */
    void deleteDataLoggerUserSettingList(int lineno);

    /**
     * @brief チャンネル定義情報のマップを返す
     * @param channelDefinitionMap[out] 読み込んだチャンネル定義情報のmap
     * @return 取得の成否 True: 成功, False: 失敗
     */
    bool getChannelDefinitionMap(QMap<int, QList<ChannelDefinitionItem>> &channelDefinitionMap);

    /**
     * @brief 引数で指定された BAND 種別に合致するのチャンネル定義のリストを返す
     * @param bandId 取得したいBAND種別（LC70_BAND_T)
     * @param channelList[out] BAND種別に該当するチャンネル定義リスト(out)
     * @return 取得の成否 True: 成功, False: 失敗
     */
    bool getChannelDefinitionList(int bandId, QList<ChannelDefinitionItem> &channelList);

    /**
     * @brief 引数で指定された BAND 種別 + チャネル名に合致するチャンネル定義情報を返す
     * @param bandId ファイルIDとチャンネル定義した CSVファイルパス
     * @param channel レベルチェッカー(LC70WS)_アプリケーションライブラリ仕様書
     * @param channelItem[out] 合致したチャンネル定義情報
     * @return 取得の成否 True: 成功, False:失敗
     */
    bool getChannelDefinitionItem(int bandId, const QString &channel, ChannelDefinitionItem &channelItem);

    /**
     * @brief 引数で指定した BAND種別に合致するチャンネル定義リストを引数のチャンネルリスト定義情報の値に設定する。
     * @param bandId 設定したいBAND種別(LC70_BAND_T)
     * @param channelList 設定死体チャンネル定義リスト
     * @return 処理の成否 True: 成功, False: 失敗
     */
    bool setChannelDefinitionList(int bandId, QList<ChannelDefinitionItem> &channelList);

    /**
     * @brief アンテナ調整のチャンネル定義情報を返す
     * @param bandId JCSAT-3A/4B
     * @param channel 取得したいチャネル名称
     * @param channelItem[out] アンテナ調整のチャネル定義情報（out)
     * @return
     */
    bool getChannelDefinitionItemForAntennaAdjustment(int bandId, const QString &channel, ChannelDefinitionItem &channelItem);

    /**
     * @brief 引数で指定した BAND 種別 + チャンネル名に合致するチャンネル定義情報に対して引数のチャンネル定義情報の値を設定する
     * @param bandId 設定したいBAND種別 (LC70_BAND_T)
     * @param channel 設定したいチャンネル名称
     * @param channelItem 設定したいチャンネル定義
     * @return 処理の成否 True: 成功, False: 失敗
     *
     * @note 1度実行する毎にsetChannelDefinitionListが呼び出されてしまうので、
     *       1BANDで複数の設定を更新する場合は、この関数を何度も呼び出すのではなく、
     *       setChannelDefinitionListでデータを更新するようにしてください
     */
    bool setChannelDefinitionItem(int bandId, const QString &channel, ChannelDefinitionItem &channelItem);

    /**
     * @brief メモリで保持しているチャンネル定義情報の全ての設定を各チャンネル定義ファイルに書き出す
     * @return 処理の成否 True: 成功, False: 失敗
     */
    bool commitAllChannelDefinitionData(void);

    /**
     * @brief 引数で指定した BAND種別に合致するチャンネル定義種別の設定をチャンネル定義ファイルに書き出す
     * @param bandId
     * @return 処理の成否 True: 成功, False: 失敗
     */
    bool commitChannelDefinitionData(int bandId);

    /**
     * @brief  光測定を開始する
     * @param  waveLength 測定する波長[nm]
     * @return 処理の成否 True: 成功, False: 失敗
     */
    bool startOpticalMeasurement(uint32_t waveLength);

    /**
     * @brief 光測定を停止する
     * @return 処理の成否 True: 成功, False: 失敗
     */
    bool stopOpticalMeasurement(void);

    /**
     * @brief 光測定の測定結果を受信する
     * @param result[out] 光測定の測定結果
     * @return 処理の成否 True: 成功, False: 失敗
     */
    bool getOpticalMeasurementResult(QVariant &result);

    /**
     * @brief DC測定を開始する
     * @return 処理の成否 True: 成功, False:失敗
     */
    bool startDCMeasurement(void);

    /**
     * @brief DC測定を停止する
     * @return 処理の成否 True: 成功, False:失敗
     */
    bool stopDCMeasurement(void);

    /**
     * @brief DC測定の測定結果を受信する
     * @param result
     * @return
     */
    bool getDCMeasurementResult(QVariant &result);

    /**
     * @brief getInitializeValue
     *        設定値の初期値を取得する
     * @param valueId 設定値の識別ID
     * @param value 設定値
     * @return 処理の成否
     */
    bool getThresholdInitializeValue(QVariant &value);

    /**
     * @brief シングル測定を停止する
     */
    void stopSingleMeasurement(void);

    /**
     * @brief ユーザ設定ファイルを初期化する
     */
    void initializeUserSettingFile(void);

    /**
     * @brief 測定状態を取得し、実行可能か否かを返す
     * @note 最長10秒(20ms周期、最大500回)間取得をリトライし、リトライオーバーでサブCPUエラーとする
     * @return true: 可 false: 不可
     */
    bool readyToMeasure(void);

    /**
     * @brief スタンバイ状態を取得し、スタンバイ実行可能か否かを返す
     * @note 最長10秒(20ms周期、最大500回)間取得をリトライし、リトライオーバーでサブCPUエラーとする
     * @return true: 可 false: 不可
     */
    bool readyToStandBy(void);

    /**
     * @brief スタンバイ状態を取得し、ウェイクアップ実行可能か否かを返す
     * @note 最長10秒(20ms周期、最大500回)間取得をリトライし、リトライオーバーでサブCPUエラーとする
     * @return true: 可 false: 不可
     */
    bool readyToWakeUp(void);

    /**
     * @brief 選択した偏波に属さないチャンネル定義を取り除いたリストを返す
     * @param bandId 取得したいBAND種別（LC70_BAND_T)
     * @param polarization 選択した偏波(0: 右旋 1: 左旋)
     * @param channelList 選択した偏波に属さないチャンネル定義を取り除いたリスト
     * @return 処理の成否 True: 成功, False: 失敗
     */
    bool selectChannelDefinitionList(int bandId, QString polarization, QList<ChannelDefinitionItem> &channelList);

private:
    Communicator *_communicator;
    Stub *_stub;                                      //!< スタブクラス(Develop実行用)
    ActionMode _actionMode;
    QString _dataloggerSettingDir;
    ApplicationSetting *_applicationSettings;               //!< アプリケーション固有の設定情報
    static const unsigned int LOCAL_FREQ_UNDEFINED_DATA;    //!< 局部発振周波数 未定義データ値

    /**
    * @brief 設定値ID単位で設定値を保持する
    */
    QMap<ValueId, ValueElement> _values;

    /**
     * @brief チャンネル定義データを保持したマップ
     */
    QMap<int, QList<ChannelDefinitionItem>> _channelDefinitionMap;

    /**
     * @brief チャンネル定義データの初期化を実施する (実機確認時用)
     * @param defaultInit デフォルト読み込みの場合 true を指定
     */
    void initializeChannelDefinitionDataProduction(bool defaultInit);

    /**
     * @brief チャンネル定義データの初期化を実施する (Ubuntu確認時用)
     * @param defaultInit デフォルト読み込みの場合 true を指定
     */
    void initializeChannelDefinitionDataDevelop(bool defaultInit);

    /**
     * @brief initializeProduction
     * @param defaultInit デフォルト値読み込みの場合 true を指定
     */
    void initializeProduction(bool defaultInit);

    /**
     * @brief initializeDevelop
     * @param defaultInit デフォルト値読み込みの場合 true を指定
     */
    void initializeDevelop(bool defaultInit);

    /**
     * @brief DataLoggerUserSettingListを初期化する
     * @param value 値
     */
    void initializeDataLoggerUserSettingList(QVariantMap &value);

    /**
     * @brief Channelでソート
     * @param valueId USER データロガー設定のリスト
     */
    void sortChannel(QVariantList &channelList);
};

#endif // MODELFACADE_H
