/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include "multigraphmodel.h"

MultiGraphModel::MultiGraphModel(QObject *parent)
    : QAbstractListModel(parent)
    , _barGraphData()
{
    _roleNames[MultiGraphLevelRole] = "level";
    _roleNames[MultiGraphColorIndexRole] = "colorIndex";
    _roleNames[MultiGraphSelectionRole] = "selection";
    _roleNames[MultiGraphCurrentMeasuringRole] = "currentMeasuring";
}

MultiGraphModel::~MultiGraphModel()
{
    _barGraphData.clear();
}

int MultiGraphModel::rowCount(const QModelIndex &parent) const
{
    if (parent.isValid()) {
        return 0;
    }

    return _barGraphData.size();
}

QHash<int, QByteArray> MultiGraphModel::roleNames() const
{
    return _roleNames;
}

QVariant MultiGraphModel::data(const QModelIndex &index, int role) const
{
    if (!hasIndex(index.row(), index.column(), index.parent())) {
        return QVariant();
    }

    const BarGraphData &measureResult = _barGraphData.at(index.row());

    if (role == MultiGraphLevelRole) {
        return QVariant(measureResult.Level);
    } else if (role == MultiGraphColorIndexRole) {
        return QVariant(measureResult.ColorIndex);
    } else if (role == MultiGraphSelectionRole) {
        return QVariant(measureResult.Selection);
    } else if (role == MultiGraphCurrentMeasuringRole) {
        return QVariant(measureResult.CurrentMeasuring);
    }

    return {};
}

bool MultiGraphModel::setData(const QModelIndex &index, const QVariant &value, int role)
{
    if (!hasIndex(index.row(), index.column(), index.parent()) || !value.isValid()) {
        return false;
    }

    BarGraphData &barGraphData = _barGraphData[index.row()];

    if (role == MultiGraphLevelRole) {
        barGraphData.Level = value.toFloat();
    } else if (role == MultiGraphColorIndexRole) {
        barGraphData.ColorIndex = value.toInt();
    } else if (role == MultiGraphSelectionRole) {
        barGraphData.Selection = value.toBool();
    } else if (role == MultiGraphCurrentMeasuringRole) {
        barGraphData.CurrentMeasuring = value.toBool();
    }

    emit dataChanged(index, index, { role } );

    return true ;
}

void MultiGraphModel::append(const BarGraphData &barGraphData)
{
    int position = _barGraphData.size();
    QModelIndex modelIndex = QModelIndex();
    beginInsertRows(modelIndex, position, position);
    _barGraphData.append(barGraphData);
    endInsertRows();
}

void MultiGraphModel::update(int row, const int &currentIndex, const float32_t &level, const int &color)
{
    if (row < 0 || row >= _barGraphData.count()) {
        return;
    }

    if (currentIndex < 0 || currentIndex >= _barGraphData.count()) {
        return;
    }

    _barGraphData[row].Level = level;
    _barGraphData[row].ColorIndex = color;

    if (row != currentIndex) {
        _barGraphData[row].CurrentMeasuring = false;
        emit dataChanged(index(row), index(row), { MultiGraphLevelRole, MultiGraphColorIndexRole, MultiGraphSelectionRole, MultiGraphCurrentMeasuringRole } );

        _barGraphData[currentIndex].CurrentMeasuring = true;

        emit dataChanged(index(currentIndex), index(currentIndex), { MultiGraphLevelRole, MultiGraphColorIndexRole, MultiGraphSelectionRole, MultiGraphCurrentMeasuringRole } );
    } else {
        _barGraphData[row].CurrentMeasuring = true;
        emit dataChanged(index(row), index(row), { MultiGraphLevelRole, MultiGraphColorIndexRole, MultiGraphSelectionRole, MultiGraphCurrentMeasuringRole } );
    }
}

void MultiGraphModel::changeSelection(int currentRow, int nextRow, bool isInitialize)
{
    if (currentRow < 0 || nextRow < 0) {
        return;
    }

    if (currentRow >= _barGraphData.count() || nextRow >= _barGraphData.count()) {
        return;
    }

    // 初期化時フラグがtrueの場合、白線の初期化を行う
    if (isInitialize) {
        InitializeCurrentPole(false);
    }

    _barGraphData[currentRow].Selection = false;
    _barGraphData[nextRow].Selection = true;

    // 優先測定中のチャンネルの青線、スキャン中の白線の表示を更新
    if (isInitialize) {
        // 初期化フラグがtrueの場合は白線に対してもデータ変更通知を送信する
        emit dataChanged(index(0), index(_barGraphData.count() - 1), { MultiGraphSelectionRole, MultiGraphCurrentMeasuringRole } );
    }else {
        // 初期化フラグがfalseの場合は、青線に対してのみのデータ変更通知を送信する。
        emit dataChanged(index(0), index(_barGraphData.count() - 1), { MultiGraphSelectionRole } );
    }
}

void MultiGraphModel::InitializeCurrentPole(bool isUpdate, int firstIndex)
{
    // 白線、青線のクリアー
    for (int i = 0; i < _barGraphData.count(); ++i) {
        _barGraphData[i].CurrentMeasuring = false;
        _barGraphData[i].Selection = false;
    }

    // 白線、青線を先頭に戻す
    _barGraphData[firstIndex].CurrentMeasuring = true;
    _barGraphData[firstIndex].Selection = true;

    if (isUpdate) {
        emit dataChanged(index(0), index(_barGraphData.count() - 1), { MultiGraphCurrentMeasuringRole, MultiGraphSelectionRole } );
    }
}

void MultiGraphModel::clear()
{
    // 白線、青線のクリアー
    for (int i = 0; i < _barGraphData.count(); ++i) {
        _barGraphData[i].Level = 0;

        _barGraphData[i].CurrentMeasuring = false;
        _barGraphData[i].Selection = false;
    }

    emit dataChanged(index(0), index(_barGraphData.count() - 1), { MultiGraphLevelRole, MultiGraphCurrentMeasuringRole, MultiGraphSelectionRole } );

    _barGraphData.clear();
}
