/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef MULTIGRAPHMODEL_H
#define MULTIGRAPHMODEL_H

#include <QAbstractListModel>
#include "constant.h"

/**
 * @brief マルチ測定のグラフ表示に対応するロール
 * @note リストのロールと値が重複しないようにする(Qt::UserRole + 11 まで使用している)
 */
enum MultiGraphRoles {
    MultiGraphLevelRole = Qt::UserRole + 20,    //!< 測定入力レベル
    MultiGraphColorIndexRole,                   //!< 棒グラフの色のインデックス
    MultiGraphSelectionRole,                    //!< 優先測定で本チャンネルを選択中か否か
    MultiGraphCurrentMeasuringRole              //!< 本チャンネルが現在測定中か否か
};

/**
 * @brief マルチ測定のグラフ表示用
 */
class MultiGraphModel : public QAbstractListModel
{
    Q_OBJECT
    Q_ENUMS(MultiGraphRoles)
public:
    /**
     * @brief コンストラクター
     * @param parent 親オブジェクト
     */
    explicit MultiGraphModel(QObject *parent = nullptr);

    /**
     * @brief デストラクター
     */
    ~MultiGraphModel();

    /**
     * @brief 測定結果数を返す
     * @return 測定結果数
     */
    int rowCount(const QModelIndex &parent = QModelIndex()) const override;

    /**
     * @brief ロール名を返す
     * @return ロール名
     */
    QHash<int, QByteArray> roleNames() const override;

    /**
     * @brief 測定結果のデータを返す
     * @param index データの位置
     * @param role ロール
     * @return 測定データ
     */
    QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const override;

    /**
     * @brief 測定結果を設定する
     * @param index データの位置
     * @param value 値
     * @param role ロール
     * @return 処理の成否
     */
    bool setData(const QModelIndex &index, const QVariant &value, int role) override;

    /**
     * @brief 棒グラフ用の測定結果の追加
     * @param barGraphData 測定結果
     */
    void append(const BarGraphData &barGraphData);

    /**
     * @brief 棒グラフ用の測定結果の更新
     * @param row　行
     * @param level 測定入力レベルの値
     * @param color 棒グラフの色
     */
    void update(int row, const int &currentIndex, const float32_t &level, const int &color);

    /**
     * @brief 選択行の更新
     * @param previousRow 前の行
     * @param currentRow 現在の行
     * @param isInitialize 初期化かどうか
     */
    void changeSelection(int previousRow, int currentRow, bool isInitialize = false);

    /**
     * @brief 表示中の白線を先頭に戻す
     * @param isUpdate 関数内で表示の更新を実施するか (true: 更新を実施, false: 更新は実施しない)
     * @param firstIndex 先頭のインデックス
     */
    void InitializeCurrentPole(bool isUpdate, int firstIndex = 0);

    /**
     * @brief データをクリアー
     */
    void clear(void);

private:
    QHash<int, QByteArray> _roleNames;      //!< ロール名
    QList<BarGraphData> _barGraphData;      //!< 棒グラフのデータ
};

#endif // MULTIGRAPHMODEL_H
