/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include "faultdetector.h"

FaultDetector::FaultDetector(const ActionMode &actionMode, Communicator *communicator, Stub *stub, SystemLog &systemLog,
                             QObject *parent)
    : ParallelTask(systemLog, parent)
    , _actionMode(actionMode)
    , _communicator(communicator)
    , _stub(stub)
    , _interval(100)
{
    if (actionMode == DEVELOPMENT) {
        connect(_stub, SIGNAL(failure(LC70_FAIL_INFO_T *)), this, SLOT(onFailureInformation(LC70_FAIL_INFO_T *)), Qt::QueuedConnection);
    }
}

FaultDetector::~FaultDetector()
{
    suspend();
    _systemLog.write(QString(typeid(*this).name()), QString("### fault detection thread is suspended."));
}

void FaultDetector::work()
{
    // スレッド開始後初回のみ実行
    if (!_initialized) {

        // スレッドごとにラッパー関数ライブラリを初期化
        if (_communicator != nullptr) {
            _communicator->initialize(
                5);    // 5 は ラッパー関数ライブラリーが認識するスレッド ID として割り振られた値
        }

        _initialized = true;
    }

    if (_actionMode == PRODUCTION) {
        LC70_FAIL_INFO_T errorInformation;   // エラー情報
        memset(&errorInformation, 0, sizeof(errorInformation));

        LC70_ERR_T ret = _communicator->failureInformation(&errorInformation);

        if (ret == LC70_OK) {
            emit errorOccured(errorInformation);
        }
    }
}

void FaultDetector::setInterval(const uint32_t &interval)
{
    _interval = interval;
}

bool FaultDetector::setFaultInformation(uint16_t code, uint8_t status, const uint8_t *additionalInformation)
{
    LC70_ERR_T ret = _communicator->setFailureInformation(code, status, additionalInformation);
    return ret == LC70_OK;
}

void FaultDetector::onFailureInformation(LC70_FAIL_INFO_T *failureInformation)
{
    // スタブから受信
    emit errorOccured(*failureInformation);
}
