/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef FAULTDETECTOR_H
#define FAULTDETECTOR_H

#include "paralleltask.h"
#include "communicator.h"
#include "stub.h"

class FaultDetector : public ParallelTask
{
    Q_OBJECT
public:
    /**
     * @brief コンストラクター
     * @param actionMode 動作モード
     * @param communicator サブ CPU と連携するためのクラスのポインター
     * @param stub スタブクラスのポインター
     * @param systemLog システムロガー
     * @param parent 親オブジェクト
     * @note communicator と stub は動作モードに基づきどちらか一方を使用する
     */
    explicit FaultDetector(const ActionMode &actionMode, Communicator *communicator, Stub *stub,
                           SystemLog &systemLog, QObject *parent = nullptr);

    /**
     * @brief デストラクター
     */
    ~FaultDetector();

    /**
     * @brief スレッドで実行する処理
     */
    virtual void work() override;

    /**
    * @brief 故障状態を取得する間隔
    * @param interval 故障状態取得間隔
    */
    void setInterval(const uint32_t &interval);

    /**
     * @brief エラー情報の設定
     * @param code 故障コード
     * @param status 故障状態
     * @param additionalInformation 追加情報
     * @return
     */
    bool setFaultInformation(uint16_t code, uint8_t status, const uint8_t *additionalInformation);

signals:
    /**
     * @brief 故障状態の検知を通知
     * @param errorInformation エラー情報
     */
    void errorOccured(LC70_FAIL_INFO_T errorInformation);

private:
    ActionMode _actionMode;                         //!< 動作モード
    Communicator *_communicator;                    //!< サブ CPU と連携するためのクラス
    Stub *_stub;                                    //!< スタブクラス
    uint32_t _interval;                             //!< 故障状態取得間隔

private slots:
    /**
     * @brief 故障情報を受信
     * @param failureInformation 故障情報
     */
    void onFailureInformation(LC70_FAIL_INFO_T *failureInformation);
};

#endif // FAULTDETECTOR_H
