/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include "measurer.h"
#include <QDebug>

static constexpr int DELAY_PROFILE_VALID = 2;   // 遅延プロファイルデータ有効(全ブロック収集完了)
static constexpr int MAXIMUM_REPEAT_STANDBY_STATUS = 500;       // スタンバイ状態取得の最大リトライ回数
static constexpr int INTERVAL_STANDBY_STATUS = 20;              // スタンバイ状態取得の取得間隔

Measurer::Measurer(const ActionMode &actionMode, Communicator *communicator, Stub *stub, const uint16_t numberOfPoints,
                   SystemLog &systemLog, QObject *parent)
    : ParallelTask(systemLog, parent)
    , _actionMode(actionMode)
    , _communicator(communicator)
    , _stub(stub)
    , _acquisitionTarget(NOT_ACQUISITION)
    , _singleMeasuringCondition()
    , _constellationPoints(LC70_CONSTA_1000)
    , _numberOfPoints(numberOfPoints)
    , _preferredMeasuringCondition()
    , _multiMeasuringChannels(0)
    , _channelScanIndex(0)
    , _isSingleMeasuringExecutable(false)
    , _isConstellationExecutable(false)
    , _isDelayProfileExecutable(false)
    , _isMultiMeasuringExecutable(false)
    , _isPreferredMeasuringExecutable(false)
    , _isScanningExecutable(false)
    , _displayType(LC70_DISP_TYPE_NOT_MEAS)
    , _singleMeasurementInterval(200)
    , _delayProfileInterval(200)
    , _repeatCount(_delayProfileInterval / _singleMeasurementInterval)
    , _previousMeasuringIndex(0)
    , _multiMeasuringIndex(0)
    , _isSingleMeasureingValid(false)
    , _delayProfileStatus(DATA_NOTHING)
    , _channelChanging(false)
{

}

Measurer::~Measurer()
{
    suspend();
    _systemLog.write(QString(typeid(*this).name()), QString("### measuring thread is suspended."));
}

void Measurer::work()
{
    // チャンネル変更中は抜ける
    if (_channelChanging) {
        return;
    }

    // スレッド開始後初回のみ実行
    if (!_initialized) {

        // スレッドごとにラッパー関数ライブラリを初期化
        if (_communicator != nullptr) {
            _communicator->initialize(
                16);    // 16 は ラッパー関数ライブラリーが認識するスレッド ID として割り振られた値
        }

        _initialized = true;
    }

    static uint32_t count = 1;

    // 測定、スキャン実行時以外は測定対象を NOT_ACQUISITION とし、測定結果の取得を行わないようにする
    switch (_acquisitionTarget) {
    case SINGLE:
        // シングル測定
        measureSingle();
        break;

    case CONSTELLATION:
        // コンスタレーション
        constellation();
        break;

    case DELAY_PROFILE: {
        // シングル測定
        measureSingle();

        if (count == _repeatCount) {
            // 遅延プロファイル
            delayProfile();

            count = 1;
            return;
        }

        ++count;
    }
    break;

    case MULTI:
        // マルチ測定
        measureMulti();
        break;

    case CHANNEL_SCAN:
        // チャンネルスキャン
        scanChannel();
        break;

    case ANTENNA:
        // アンテナ調整
        measureAntenna();
        break;

    default:
        break;
    }
}

void Measurer::changeAcquisitionTarget(const AcquisitionTarget &acquisitionTarget)
{
    _systemLog.write(QString(typeid(*this).name()), QString("[Enter] %1 %2").arg(__FUNCTION__).arg(acquisitionTarget));

    if (acquisitionTarget == NOT_ACQUISITION) {
        _acquisitionTarget = acquisitionTarget;
        _systemLog.write(QString(typeid(*this).name()), QString("[Leave] %1 NO TARGET").arg(__FUNCTION__));
        return;
    }

    // シングルから
    if (_acquisitionTarget == SINGLE) {
        if (acquisitionTarget == CONSTELLATION) {           // コンスタレーション
            qDebug() << QString("[Enter Constellation] %1").arg(__FUNCTION__);

            // コンスタレーション開始
            if (!startConstellationMeasuring()) {
                // nothing to do
            }

        } else if (acquisitionTarget == DELAY_PROFILE) {    // 遅延プロファイル
            qDebug() << QString("[Enter Delay Profile] %1").arg(__FUNCTION__);

            // 遅延プロファイル開始
            if (!startDelayProfileMeasuring()) {
                // nothing to do
            }
        }
    }

    // コンスタレーションから
    if (_acquisitionTarget == CONSTELLATION) {
        if (acquisitionTarget == SINGLE) {                  // シングル
            qDebug() << QString("[Enter Single Measurement] %1").arg(__FUNCTION__);

            // コンスタレーション停止
            if (!stopConstellationMeasuring()) {
                return;
            }
        } else if (acquisitionTarget == DELAY_PROFILE) {    // 遅延プロファイル
            qDebug() << QString("[Enter Delay Profile] %1").arg(__FUNCTION__);

            // コンスタレーション停止
            if (!stopConstellationMeasuring()) {
                return;
            }

            // 遅延プロファイル開始
            if (!startDelayProfileMeasuring()) {
                return;
            }
        }
    }

    // 遅延プロファイルから
    if (_acquisitionTarget == DELAY_PROFILE) {
        if (acquisitionTarget == SINGLE) {                  // シングル
            qDebug() << QString("[Enter Single Measurement] %1").arg(__FUNCTION__);

            // 遅延プロファイル停止
            if (!stopDelayProfileMeasuring())  {
                return;
            }
        } else if (acquisitionTarget == CONSTELLATION) {    // コンスタレーション
            qDebug() << QString("[Enter Constellation] %1").arg(__FUNCTION__);

            // 遅延プロファイル停止
            if (!stopDelayProfileMeasuring()) {
                return;
            }

            // コンスタレーション開始
            if (!startConstellationMeasuring()) {
                return;
            }
        }
    }

    // スキャンは対象を変更するのみとする(開始、終了は呼び出し側で行う)

    _acquisitionTarget = acquisitionTarget;

    _systemLog.write(QString(typeid(*this).name()), QString("[Leave] %1").arg(__FUNCTION__));
}

void Measurer::setInterval(const uint32_t &singleMeasurementInterval, const uint32_t &delayProfileInterval)
{
    _singleMeasurementInterval = singleMeasurementInterval;
    _delayProfileInterval = delayProfileInterval;

    // キー入力を何回行えばデバイス状態取得までの時間に到達するか
    if (_delayProfileInterval > 0 && _singleMeasurementInterval > 0
            && _delayProfileInterval >= _singleMeasurementInterval) {
        _repeatCount = _delayProfileInterval / _singleMeasurementInterval;
    }

    if (_repeatCount == 0) {
        _repeatCount = 1;
    }
}

void Measurer::onResumeSingleMeasuring(LC70_CONSTA_NUM_T constellationPoints,
                                       LC70_MEAS_SINGLE_T condition,
                                       uint32_t interval)
{
    _systemLog.write(QString(typeid(*this).name()), QString("[Enter] %1").arg(__FUNCTION__));

    _singleMeasuringCondition = condition;
    _constellationPoints = constellationPoints;

    this->usingThread()->setInterval(interval);
    resume();
    _systemLog.write(QString(typeid(*this).name()), QString("single measuring thread start. %1").arg(__FUNCTION__));

    // シングル測定(コンスタレーション、遅延プロファイル)はチャンネル変更処理(onSingleMeasuringChangeCondition)で既に開始している

    _systemLog.write(QString(typeid(*this).name()), QString("[Leave] %1").arg(__FUNCTION__));
}

void Measurer::onSuspendSingleMeasuring(void)
{
    _systemLog.write(QString(typeid(*this).name()), QString("[Enter] %1").arg(__FUNCTION__));

    // 遷移先画面種別を測定画面以外に変更
    changeDisplayType(LC70_DISP_TYPE_NOT_MEAS);

    LC70_ERR_T ret = LC70_ERR;

    switch (_acquisitionTarget) {
    case SINGLE:
    case CONSTELLATION:

        // シングル測定停止
        if (stopSingleMeasuring()) {
            // nothing to do
        }

        break;

    case DELAY_PROFILE:

        // 遅延プロファイル停止
        if (!stopDelayProfileMeasuring()) {
            // nothing to do
        }

        // シングル測定停止
        if (!stopSingleMeasuring()) {
            // nothing to do
        }

        break;

    default:
        break;
    }

    if (ret != LC70_OK) {
        qDebug() << QString("stop measuring ret = %1").arg(static_cast<int>(ret));
    }

    // 測定対象なしに設定
    _acquisitionTarget = NOT_ACQUISITION;

    // シングル測定の停止を通知
    emit notifyStopMeasuring(SINGLE);

    _systemLog.write(QString(typeid(*this).name()), QString("[Leave] %1").arg(__FUNCTION__));
}

void Measurer::onSingleMeasuringChangeCondition(LC70_MEAS_SINGLE_T condition)
{
    _channelChanging = true;

    // チャンネルの変更
    _singleMeasuringCondition = condition;

    switch (_acquisitionTarget) {
    case SINGLE:
        _systemLog.write(QString(typeid(*this).name()), QString("[Enter Single Measurement] %1").arg(__FUNCTION__));

        // チャンネル変更中を通知
        emit notifyChannelChanging(true);

        // シングル測定を停止せず、条件を変更して再度開始を呼び出すため、内部的な認識を「停止状態」に変更しておく
        _isSingleMeasuringExecutable = false;

        changeDisplayType(LC70_DISP_TYPE_METER);

        // シングル測定開始
        if (!startSingleMeasuring(_acquisitionTarget)) {
            // nothing to do
        }

        // チャンネル変更済を通知
        emit notifyChannelChanging(false);

        break;

    case CONSTELLATION:
        _systemLog.write(QString(typeid(*this).name()), QString("[Enter Constellation] %1").arg(__FUNCTION__));

        // チャンネル変更中を通知
        emit notifyChannelChanging(true);

        // シングル測定を停止せず、条件を変更して再度開始を呼び出すため、内部的な認識を「停止状態」に変更しておく
        _isSingleMeasuringExecutable = false;

        changeDisplayType(LC70_DISP_TYPE_CONSTA);

        // シングル測定開始
        if (!startSingleMeasuring(_acquisitionTarget)) {
            emit notifyChannelChanging(false);
            _channelChanging = false;
            return;
        }

        // コンスタレーション開始(コンスタレーション開始後のチャンネル変更では開始を呼ばないようにする)
        if (!_isConstellationExecutable) {
            if (!startConstellationMeasuring()) {
                emit notifyChannelChanging(false);
                _channelChanging = false;
                return;
            }
        }

        // チャンネル変更済を通知
        emit notifyChannelChanging(false);

        break;

    case DELAY_PROFILE:
        _systemLog.write(QString(typeid(*this).name()), QString("[Enter Delay Profile] %1").arg(__FUNCTION__));

        // チャンネル変更中を通知
        emit notifyChannelChanging(true);

        // 遅延プロファイル測定を停止せず、条件を変更して再度開始を呼び出すため、内部的な認識を「停止状態」に変更しておく
        _isDelayProfileExecutable = false;

        // シングル測定を停止せず、条件を変更して再度開始を呼び出すため、内部的な認識を「停止状態」に変更しておく
        _isSingleMeasuringExecutable = false;

        changeDisplayType(LC70_DISP_TYPE_DLY_PRF);

        // シングル測定開始
        if (!startSingleMeasuring(_acquisitionTarget)) {
            emit notifyChannelChanging(false);
            _channelChanging = false;
            return;
        }

        // 遅延プロファイル開始
        if (!startDelayProfileMeasuring()) {
            emit notifyChannelChanging(false);
            _channelChanging = false;
            return;
        }

        // チャンネル変更済を通知
        emit notifyChannelChanging(false);

        break;

    default:
        break;
    }

    _channelChanging = false;

    _systemLog.write(QString(typeid(*this).name()), QString("[Leave] %1").arg(__FUNCTION__));
}

void Measurer::onPreferredMeasuringChangeCondition(LC70_MEAS_SINGLE_T *condition)
{
    _systemLog.write(QString(typeid(*this).name()),
                     QString("[Enter] %1 Channel: %2").arg(__FUNCTION__).arg(QString::fromLocal8Bit((
                                 char *)condition->ucStrChan)));

    _channelChanging = true;

    // チャンネル変更中を通知
    emit notifyChannelChanging(true);

    // 優先測定を停止せず、条件を変更して再度開始を呼び出すため、内部的な認識を「停止状態」に変更しておく
    _isPreferredMeasuringExecutable = false;

    // 優先測定開始
    if (!startPrefferedMeasuring(condition)) {
        // nothing to do
    }

    // チャンネル変更済を通知
    emit notifyChannelChanging(false);

    _channelChanging = false;

    _systemLog.write(QString(typeid(*this).name()), QString("[Leave] %1").arg(__FUNCTION__));
}

void Measurer::onAntennaMeasuringChangeCondition(LC70_MEAS_MULTI_T *condition)
{
    _systemLog.write(QString(typeid(*this).name()),
                     QString("[Enter] %1 Channel: %2").arg(__FUNCTION__).arg(QString::fromLocal8Bit((
                                 char *)condition->single[0].ucStrChan)), LOG_NOTICE);

    _channelChanging = true;

    // チャンネル変更中を通知
    emit notifyChannelChanging(true);

    // 測定中の測定条件変更
    if (_isMultiMeasuringExecutable) {
        // マルチチャンネル測定停止
        stopMultiMeasuring();
    }

    // 測定するインデックスのクリアー
    _multiMeasuringIndex = 0;

    // 遷移先画面種別をリスト画面(データロガー)に変更
    changeDisplayType(LC70_DISP_TYPE_ANTENNA_ADJUST);

    // マルチ測定開始
    if (!startMultiMeasuring(condition)) {
        emit notifyChannelChanging(false);
        _channelChanging = false;
        return;
    }

    // マルチ測定のチャンネル数
    _multiMeasuringChannels = condition->sRegNum;

    // チャンネル変更済を通知
    emit notifyChannelChanging(false);

    _channelChanging = false;

    _systemLog.write(QString(typeid(*this).name()), QString("[Leave] %1").arg(__FUNCTION__), LOG_NOTICE);
}

void Measurer::onResumeMultiMeasuring(LC70_MEAS_MULTI_T *condition, LC70_MEAS_SINGLE_T *prefferedCondition, LC70_DISP_TYPE_T displayType,
                                      uint32_t interval)
{
    _systemLog.write(QString(typeid(*this).name()), QString("[Enter] %1").arg(__FUNCTION__), LOG_INFO);

    this->usingThread()->setInterval(interval);
    resume();
    _systemLog.write(QString(typeid(*this).name()), QString("multi measuring thread start. %1").arg(__FUNCTION__));

    // 測定するインデックスのクリアー
    _multiMeasuringIndex = 0;

    // マルチ測定を対象
    changeAcquisitionTarget(MULTI);

    // チャンネル変更中を通知
    emit notifyChannelChanging(true);

    // 一つ前の測定中インデックスをクリアー
    _previousMeasuringIndex = 0;

    // 遷移先画面種別を指定
    changeDisplayType(displayType);

    // マルチ測定開始
    if (!startMultiMeasuring(condition)) {
        emit notifyChannelChanging(false);
        return;
    }

    // 優先測定開始
    if (!startPrefferedMeasuring(prefferedCondition)) {
        emit notifyChannelChanging(false);
        return;
    }

    // チャンネル変更済を通知
    emit notifyChannelChanging(false);

    _systemLog.write(QString(typeid(*this).name()), QString("[Leave] %1").arg(__FUNCTION__));
}

void Measurer::onSuspendMultiMeasuring(void)
{
    _systemLog.write(QString(typeid(*this).name()), QString("[Enter] %1").arg(__FUNCTION__));

    // 遷移先画面種別を測定画面以外に変更
    changeDisplayType(LC70_DISP_TYPE_NOT_MEAS);

    // マルチチャンネル優先測定停止
    if (!stopPrefferedMeasuring()) {
        return;
    }

    // マルチチャンネル測定停止
    if (!stopMultiMeasuring()) {
        return;
    }

    // 測定対象なしに設定
    _acquisitionTarget = NOT_ACQUISITION;

    // マルチ測定の停止を通知
    emit notifyStopMeasuring(MULTI);

    _systemLog.write(QString(typeid(*this).name()), QString("[Leave] %1").arg(__FUNCTION__));
}

void Measurer::onResumeScanningChannel(LC70_MEAS_MULTI_T *condition,
                                       AcquisitionTarget acquisition, int interval)
{
    _systemLog.write(QString(typeid(*this).name()), QString("[Enter] %1").arg(__FUNCTION__));

    LC70_ERR_T ret = LC70_ERR;

    if (acquisition == DELAY_PROFILE) {
        // 遅延プロファイル測定停止
        if (!stopDelayProfileMeasuring()) {
            // nothing to do
        }
    }

    if ((acquisition == SINGLE) || (acquisition == CONSTELLATION) || (acquisition == DELAY_PROFILE)) {
        // シングル測定停止
        if (!stopSingleMeasuring()) {
            return;
        }
    } else if (acquisition == MULTI) {
        // マルチチャンネル優先測定停止
        if (!stopPrefferedMeasuring()) {
            return;
        }

        // マルチチャンネル測定停止
        if (!stopMultiMeasuring()) {
            return;
        }
    }

    _acquisitionTarget = CHANNEL_SCAN;
    _multiMeasuringChannels = condition->sRegNum;

    // チャンネルスキャン開始
    _isScanningFinished = false;
    _channelScanIndex = 0;
    _previousMeasuringIndex = 0;

    ret = _communicator->startScan(condition);

    if (ret == LC70_OK) {
        _isScanningExecutable = true;
    } else {
        _systemLog.write(QString(typeid(*this).name()), QString("Failed to start scanning."));
        return;
    }

    // スタブ制御
    if (_actionMode == DEVELOPMENT) {
        _stub->resetScanning();
    }

    this->usingThread()->setInterval(interval);
    resume();

    _systemLog.write(QString(typeid(*this).name()), QString("[Leave] %1").arg(__FUNCTION__));
}

void Measurer::onSuspendScanningChannel(void)
{
    _systemLog.write(QString(typeid(*this).name()), QString("[Enter] %1").arg(__FUNCTION__));

    LC70_ERR_T ret = LC70_ERR;

    // チャンネルスキャン停止
    ret = _communicator->stopScan();

    if (ret == LC70_OK) {
        _isScanningFinished = true;
        _channelScanIndex = 0;

        _isScanningExecutable = false;
    } else {
        _systemLog.write(QString(typeid(*this).name()), QString("Failed to stop scanning."));
        return;
    }

    // 測定対象なしに設定
    _acquisitionTarget = NOT_ACQUISITION;

    emit scanningFinished(false);

    _systemLog.write(QString(typeid(*this).name()), QString("[Leave] %1").arg(__FUNCTION__));
}

void Measurer::onResumeAntennaMeasuring(LC70_MEAS_MULTI_T *condition, uint32_t interval)
{
    _systemLog.write(QString(typeid(*this).name()), QString("[Enter] %1").arg(__FUNCTION__), LOG_INFO);

    this->usingThread()->setInterval(interval);
    resume();
    _systemLog.write(QString(typeid(*this).name()), QString("antenna measuring thread start. %1").arg(__FUNCTION__));

    // 測定するインデックスのクリアー
    _multiMeasuringIndex = 0;

    // アンテナ調整を対象
    changeAcquisitionTarget(ANTENNA);

    // チャンネル変更中を通知
    emit notifyChannelChanging(true);

    // 一つ前の測定中インデックスをクリアー
    _previousMeasuringIndex = 0;

    // 遷移先画面種別をJCSAT アンテナ調整(メータ)に変更
    changeDisplayType(LC70_DISP_TYPE_ANTENNA_ADJUST);

    // マルチ測定開始
    if (!startMultiMeasuring(condition)) {
        emit notifyChannelChanging(false);
        return;
    }

    // チャンネル変更済を通知
    emit notifyChannelChanging(false);

    _systemLog.write(QString(typeid(*this).name()), QString("[Leave] %1").arg(__FUNCTION__), LOG_INFO);
}

void Measurer::onSuspendAntennaMeasuring()
{
    _systemLog.write(QString(typeid(*this).name()), QString("[Enter] %1").arg(__FUNCTION__), LOG_INFO);

    LC70_ERR_T ret = LC70_ERR;

    if (!_isMultiMeasuringExecutable) {
        return;
    }

    // 測定対象なしに設定
    _acquisitionTarget = NOT_ACQUISITION;

    // アンテナ調整測定停止
    ret = _communicator->stopMultiMeasuring();

    // 遷移先画面種別を測定画面以外に変更
    changeDisplayType(LC70_DISP_TYPE_NOT_MEAS);

    if (ret == LC70_OK) {
        _isMultiMeasuringExecutable = false;
    } else {
        _systemLog.write(QString(typeid(*this).name()), QString("Failed to stop antenna measuring."));
        return;
    }

    // アンテナ調整の停止を通知
    emit notifyStopMeasuring(ANTENNA);

    _systemLog.write(QString(typeid(*this).name()), QString("[Leave] %1").arg(__FUNCTION__), LOG_INFO);
}

void Measurer::onRequestToStandBy(uint8_t mode)
{
    // 測定中であれば止める
    switch (_acquisitionTarget) {
    case SINGLE:
    case CONSTELLATION:
    case DELAY_PROFILE:
        onSuspendSingleMeasuring();
        break;
    case MULTI:
        onSuspendMultiMeasuring();
        break;
    case CHANNEL_SCAN:
        onSuspendScanningChannel();
        break;
    case ANTENNA:
        onSuspendAntennaMeasuring();
        break;
    default:
        break;
    }

    bool result = false;
    static int count = 0;
    LC70_STANDBY_STAT_T status;

    while (!result) {
        if (count == MAXIMUM_REPEAT_STANDBY_STATUS) {
            break;
        }
        memset(&status, 0, sizeof(LC70_STANDBY_STAT_T));
        LC70_ERR_T ret = _communicator->standByStatus(&status);

        if (ret == LC70_OK) {
            if (status.ulStandbyStat == LC70_STANDBY_STAT_STANDBY ||
                    status.ulStandbyStat == LC70_STANDBY_STAT_WAKEUP) {
                result = true;
                break;
            }
        }
        ++count;

        QThread::msleep(INTERVAL_STANDBY_STATUS);
    }

    if (!result) {
        _systemLog.write(QString(typeid(*this).name()),
                         QString("[Retry Error] count: %1 status:%2").arg(count).arg(status.ulStandbyStat), LOG_NOTICE);
        //_communicator->systemShutdown();
        return;
    }

    if (mode == MAIN_AND_SUB) {
        QFile file;
        file.setFileName(QString(STANDBY_LOCK_FILE_NAME));
        if (file.open(QIODevice::WriteOnly | QIODevice::Text)) {
            file.close();
        }
    }

    _communicator->systemStandBy(mode);

    if (mode == MAIN_AND_SUB) {
        // スリープコマンド実行
        if (!QProcess::startDetached("/usr/local/bin/sleep.sh")) {
            _systemLog.write(QString(typeid(*this).name()), QString("### ERROR failed to stand by."), LOG_INFO);
        }
    }
}

bool Measurer::startAzimuth()
{
    // 方位測定開始
    LC70_ERR_T ret = _communicator->startAzimuthMeasurement();

    return (ret == LC70_OK) ? true : false;
}

bool Measurer::stopAzimuth()
{
    // 方位測定停止
    LC70_ERR_T ret = _communicator->stopAzimuthMeasurement();

    return (ret == LC70_OK) ? true : false;
}

void Measurer::changeDisplayType(LC70_DISP_TYPE_T displayType)
{
    _displayType = displayType;
    _communicator->changeDisplayType(_displayType);
}

void Measurer::scanChannel()
{
    // スキャン測定開始に成功していなければ測定結果取得を呼ばない
    if (!_isScanningExecutable) {
        return;
    }

    // スキャンが完了していれば処理しない
    if (_isScanningFinished) {
        return;
    }

    LC70_MEAS_MULTI_RESULT_T result;
    memset(&result, 0x00, sizeof(result));

    if (_actionMode == PRODUCTION) {
        // マルチチャンネル測定結果取得
        int16_t i = static_cast<int16_t>(_channelScanIndex);

        if (_communicator->scanResult(i, result) != LC70_OK) {
            return;
        }

        // 指定と異なるインデックスの測定結果が返ってきている
        if (_channelScanIndex != result.ucReadIdx) {
            _systemLog.write(QString(typeid(*this).name()),
                             QString("read index(%1) is different from scan index(%2) ").arg(result.ucReadIdx).arg(_channelScanIndex));
            _previousMeasuringIndex = result.ucCurrIdx;
            return;
        }

        // 読み出しインデックスが測定中インデックスより小さかったら次のチャンネルを測定する
        if (result.ucCurrIdx <= result.ucReadIdx) {
            // 測定中インデックスが一周して読み出しインデックスより小さくなった場合は次のチャンネルを測定し、そうでなければ同じインデックスで測定を繰り返す
            if (_previousMeasuringIndex <= result.ucCurrIdx) {
                _previousMeasuringIndex = result.ucCurrIdx;
                return;
            }
        }

        _previousMeasuringIndex = result.ucCurrIdx;

        // 収集ステータスで Level が立っていなければ再度同じインデックスで測定結果を取得する
        if ((result.usStat & LC70_DATA_MSK_LEVEL) != LC70_DATA_MSK_LEVEL) {
            return;
        }

    } else {
        qDebug() << QString("scan index: %1").arg(_channelScanIndex);

        // スタブでは全チャンネルスキャンした時点で終了とする
        _stub->channelScanResult(_channelScanIndex, &result, _multiMeasuringChannels);

        // ラッパー関数ライブラリ V2.00 対応
//        result.ucLock = (_channelScanIndex % 2 == 0) ? LOCKED :
//                        UNLOCKED; // スキャン用に偶数インデックスを有効とする
        result.ucLock = (_channelScanIndex == 3 || _channelScanIndex == 17 || _channelScanIndex == 39) ? LOCKED :
                        UNLOCKED; // スキャン用に 3 番めのインデックスのみを有効とする
    }

    // 測定結果を通知(有効なチャンネルかどうかは受信側で判断する)
    emit scanningChannelResult(_channelScanIndex, result);

    // チャンネル数分取得できたか
    _channelScanIndex++;

    // 測定完了の確認
    if (result.ucCurrIdx == 0xFF || _channelScanIndex == _multiMeasuringChannels) {

        _systemLog.write(QString(typeid(*this).name()), QString("all scanned %1 channel(s)").arg(_channelScanIndex), LOG_INFO);

        LC70_ERR_T ret = LC70_ERR;

        // チャンネルスキャン停止
        ret = _communicator->stopScan();

        if (ret == LC70_OK) {
            _isScanningFinished = true;
            _channelScanIndex = 0;

            _isScanningExecutable = false;
        } else {
            _systemLog.write(QString(typeid(*this).name()), QString("Failed to stop scanning."));
            return;
        }

        // 測定対象なしに設定
        _acquisitionTarget = NOT_ACQUISITION;

        emit scanningFinished(true);
    }
}

void Measurer::measureAntenna()
{
    // マルチ測定開始に成功していなければ測定結果取得を呼ばない
    if (!_isMultiMeasuringExecutable) {
        return;
    }

    LC70_MEAS_MULTI_RESULT_T result;
    memset(&result, 0x00, sizeof(result));

    LC70_ERR_T multiMeasuringResult = LC70_ERR;

    if (_actionMode == PRODUCTION) {
        // マルチチャンネル測定結果取得
        multiMeasuringResult = _communicator->multiMeasuringResult(_multiMeasuringIndex, &result);

        // 次のチャンネルを測定するか判断
        isMultiMeasuringResultSuitable(result, multiMeasuringResult);

    } else {
        multiMeasuringResult = _stub->multiMeasuringResult(_multiMeasuringIndex, &result);

        if (result.ucCurrIdx >= _multiMeasuringChannels) {
            result.ucCurrIdx = 0;
        }

        ++_multiMeasuringIndex;
    }

    if (_multiMeasuringIndex >= _multiMeasuringChannels) {
        _multiMeasuringIndex = 0;
    }

    // マルチ測定結果を通知
    MultiResult measureResult;
    measureResult.received = (multiMeasuringResult == LC70_OK) ? true : false;
    measureResult.locked = result.ucLock;
    measureResult.status = result.usStat;
    measureResult.readIndex = result.ucReadIdx;
    measureResult.currentMeasuringIndex = result.ucCurrIdx;
    measureResult.result.Status = result.usStat;
    measureResult.result.Level = static_cast<float32_t>(result.ulLevel) / 10;   // ラッパーライブラリ V2.00 対応
    measureResult.result.Mer = static_cast<float32_t>(result.ulCnr) /
                               100;      // ラッパーライブラリ V2.00 対応(2020.11.24 MER は 1 / 100)
    measureResult.result.Cnr = static_cast<float32_t>(result.ulCnr) /
                               100;      // ラッパーライブラリ V2.00 対応(2020.11.24 MER は 1 / 100)
    if (result.ulBerTotalBitNum > 0) {
        measureResult.result.Ber = static_cast<float32_t>(result.ulBerErrBitNum) /
                                   static_cast<float32_t>(result.ulBerTotalBitNum); // ラッパーライブラリ V2.00 対応
    }
    measureResult.result.BroadcastSystem = result.usBdSys;
    measureResult.result.Modulation = result.ulModBitPtrn;
    measureResult.result.Layer = result.ucLayer;
    measureResult.result.GuardInterval = result.ucGi;

    emit antennaMeasureResult(measureResult);
}

QString Measurer::convertAcqusitionTarget(const AcquisitionTarget &acquisitionTarget)
{
    QString targetName;

    switch (acquisitionTarget) {
    case NOT_ACQUISITION:   //!< 取得なし
        targetName = QString("No target");
        break;

    case SINGLE:            //!< シングル測定
        targetName = QString("Single measurement");
        break;

    case CONSTELLATION:     //!< コンスタレーション
        targetName = QString("Constellation");
        break;

    case DELAY_PROFILE:     //!< 遅延プロファイル
        targetName = QString("Delay profile");
        break;

    case MULTI:             //!< マルチ測定
        targetName = QString("Multi measurement");
        break;

    case CHANNEL_SCAN:      //!< チャンネルスキャン
        targetName = QString("Scanning");
        break;

    case ANTENNA:           //!< アンテナ調整
        targetName = QString("Antenna adjustment");
        break;

    default:
        break;
    }

    return targetName;
}

void Measurer::measureSingle()
{
    LC70_MEAS_SINGLE_RESULT_T result;
    memset(&result, 0x00, sizeof(result));

    // 測定開始に成功していなければ測定結果取得を呼ばない
    if (!_isSingleMeasuringExecutable) {
        _systemLog.write(QString(typeid(*this).name()), QString("%1 single measuring not work").arg(__FUNCTION__));
        return;
    }

    MeasureResult measureResult;
    QList<QVariant> constellations;
    LC70_ERR_T ret = LC70_ERR;

    if (_actionMode == PRODUCTION) {
        ret = _communicator->singleMeasuringResult(0, &result);
    } else {
        ret = _stub->singleMeasuringResult(0, &result);
    }

    measureResult.Received = (ret == LC70_OK) ? true : false;

    // シングル測定の有効状態を設定
    _isSingleMeasureingValid = result.ucLock == LOCKED;

    // シングル測定結果を通知
    measureResult.Timestamp = QDateTime::currentDateTime().toString("yyyy/MM/dd hh:mm:ss");
    measureResult.Locked = result.ucLock;                               // ラッパーライブラリ V2.00 対応
    measureResult.Status = result.usStat;
    measureResult.Level = static_cast<float32_t>(result.ulLevel) / 10;   // ラッパーライブラリ V2.00 対応
    measureResult.Mer = static_cast<float32_t>(result.ulCnr) /
                        100;      // ラッパーライブラリ V2.00 対応(2020.11.24 MER は 1 / 100)
    measureResult.Cnr = static_cast<float32_t>(result.ulCnr) /
                        100;      // ラッパーライブラリ V2.00 対応(2020.11.24 MER は 1 / 100)
    if (result.ulBerTotalBitNum > 0) {
        measureResult.Ber = static_cast<float32_t>(result.ulBerErrBitNum) /
                            static_cast<float32_t>(result.ulBerTotalBitNum); // ラッパーライブラリ V2.00 対応
    }
    measureResult.BroadcastSystem = result.usBdSys;
    measureResult.Modulation = result.ulModBitPtrn;
    measureResult.Layer = result.ucLayer;
    measureResult.GuardInterval = result.ucGi;                           // ラッパーライブラリ V2.01 対応

    emit singleMeasureResult(measureResult, constellations);
}

void Measurer::constellation()
{
    LC70_MEAS_SINGLE_RESULT_T result;
    memset(&result, 0x00, sizeof(result));

    // 測定開始、コンスタレーション開始に成功していなければ測定結果取得を呼ばない
    if (!_isSingleMeasuringExecutable || !_isConstellationExecutable) {
        _systemLog.write(QString(typeid(*this).name()), QString("%1 constellation not work").arg(__FUNCTION__));
        return;
    }

    LC70_ERR_T ret = LC70_ERR;

    if (_actionMode == PRODUCTION) {
        ret = _communicator->singleMeasuringResult(_numberOfPoints, &result);
    } else {
        // コンスタレーションの読み出し点数の設定値に基づいたデータ数を取得
        ret = _stub->singleMeasuringResult(_numberOfPoints, &result);
    }

    QList<QVariant> constellations;

    // コンスタレーションの反映
    for (int i = 0; i < result.sConstaNum; ++i) {
        QPoint point;
        point.setX(result.consta[i].x);
        point.setY(result.consta[i].y);
        constellations.append(point);
    }

    // シングル測定結果を通知
    MeasureResult measureResult;
    measureResult.Received = (ret == LC70_OK) ? true : false;
    measureResult.Locked = result.ucLock;
    measureResult.Status = result.usStat;
    measureResult.Level = static_cast<float32_t>(result.ulLevel) / 10;   // ラッパーライブラリ V2.00 対応
    measureResult.Mer = static_cast<float32_t>(result.ulCnr) /
                        100;      // ラッパーライブラリ V2.00 対応(2020.11.24 MER は 1 / 100)
    measureResult.Cnr = static_cast<float32_t>(result.ulCnr) /
                        100;      // ラッパーライブラリ V2.00 対応(2020.11.24 MER は 1 / 100)
    if (result.ulBerTotalBitNum > 0) {
        measureResult.Ber = static_cast<float32_t>(result.ulBerErrBitNum) /
                            static_cast<float32_t>(result.ulBerTotalBitNum); // ラッパーライブラリ V2.00 対応
    }
    measureResult.BroadcastSystem = result.usBdSys;
    measureResult.Modulation = result.ulModBitPtrn;
    measureResult.Layer = result.ucLayer;

    emit singleMeasureResult(measureResult, constellations);
}

void Measurer::delayProfile()
{
    // 測定開始、遅延プロファイル開始に成功していなければ測定結果取得を呼ばない
    if (!_isSingleMeasuringExecutable || !_isDelayProfileExecutable) {
        _systemLog.write(QString(typeid(*this).name()), QString("%1 delay profile not work").arg(__FUNCTION__));
        return;
    }

    LC70_MEAS_DLY_PRF_RESULT_T result;
    memset(&result, 0x00, sizeof(result));

    LC70_ERR_T funcResult;

    if (_actionMode == PRODUCTION) {
        funcResult = _communicator->delayProfileMeasuring(&result);
    } else {
        funcResult = _stub->delayProfileMeasuring(&result);
    }

    if (funcResult == LC70_OK) {
        _delayProfileStatus = result.usStat;
        checkDelayProfileStatus();
    }

    if (!_isSingleMeasureingValid || (_delayProfileStatus != DATA_VALID)) {
        return;
    }

    QVector<QPointF> delayProfile;

    double x = MINIMUM_DELAY_PROFILE_TIME + DELAY_PROFILE_INTERVAL;

    for (int i = 0; i < LC70_DLY_PRF_NUM; ++i) {
        QPointF point(x, static_cast<double>(result.ucDlyPrfData[i]));
        delayProfile.append(point);
        x += DELAY_PROFILE_INTERVAL;
    }

    _systemLog.write(QString(typeid(*this).name()),
                     QString("%1 result: %2 points").arg(__FUNCTION__).arg(delayProfile.count()));

    emit delayProfileResult(delayProfile);
}

void Measurer::checkDelayProfileStatus()
{
    if (_acquisitionTarget != DELAY_PROFILE) {
        return;
    }

    if (!_isSingleMeasureingValid) {
        // シングル測定の結果が取れていない場合は無効
        emit notifyDelayProfileValid(false);
        return;
    }

    if (_delayProfileStatus == DATA_NOTHING) {
        // 遅延プロファイルデータなし
        emit notifyDelayProfileValid(false);
    } else if (_delayProfileStatus == DATA_VALID) {
        emit notifyDelayProfileValid(true);
    } else {
        // nothing to do
    }
}

void Measurer::measureMulti()
{
    // マルチ測定開始、マルチ測定優先測定開始に成功していなければ測定結果取得を呼ばない
    if (!_isMultiMeasuringExecutable || !_isPreferredMeasuringExecutable) {
        return;
    }

    LC70_MEAS_MULTI_RESULT_T result;
    memset(&result, 0x00, sizeof(result));
    LC70_MEAS_SINGLE_RESULT_T prefferedChannel;
    memset(&prefferedChannel, 0x00, sizeof(prefferedChannel));

    LC70_ERR_T multiMeasuringResult = LC70_ERR;
    LC70_ERR_T prefferedMeasuringResult = LC70_ERR;

    if (_actionMode == PRODUCTION) {
        // マルチチャンネル優先測定結果取得
        prefferedMeasuringResult = _communicator->multiMeasuringPrefferedChannelResult(&prefferedChannel);

        // マルチチャンネル測定結果取得
        multiMeasuringResult = _communicator->multiMeasuringResult(_multiMeasuringIndex, &result);

        // 次のチャンネルを測定するか判断
        isMultiMeasuringResultSuitable(result, multiMeasuringResult);

    } else {
        multiMeasuringResult = _stub->multiMeasuringResult(_multiMeasuringIndex, &result);

        if (result.ucCurrIdx >= _multiMeasuringChannels) {
            result.ucCurrIdx = 0;
        }

        prefferedMeasuringResult = _stub->multiMeasuringPrefferedChannelResult(&prefferedChannel);
        ++_multiMeasuringIndex;
    }

    if (_multiMeasuringIndex >= _multiMeasuringChannels) {
        _multiMeasuringIndex = 0;
    }

    // マルチ測定結果を通知
    MultiResult measureResult;
    measureResult.received = (multiMeasuringResult == LC70_OK) ? true : false;

    if (measureResult.received) {
        measureResult.locked = result.ucLock;
        measureResult.status = result.usStat;
        measureResult.readIndex = result.ucReadIdx;
        measureResult.currentMeasuringIndex = result.ucCurrIdx;
        measureResult.result.Locked = result.ucLock;
        measureResult.result.Status = result.usStat;
        measureResult.result.Level = static_cast<float32_t>(result.ulLevel) / 10;   // ラッパーライブラリ V2.00 対応
        measureResult.result.Mer = static_cast<float32_t>(result.ulCnr) /
                                   100;      // ラッパーライブラリ V2.00 対応(2020.11.24 MER は 1 / 100)
        measureResult.result.Cnr = static_cast<float32_t>(result.ulCnr) /
                                   100;      // ラッパーライブラリ V2.00 対応(2020.11.24 MER は 1 / 100)
        if (result.ulBerTotalBitNum > 0) {
            measureResult.result.Ber = static_cast<float32_t>(result.ulBerErrBitNum) /
                                       static_cast<float32_t>(result.ulBerTotalBitNum); // ラッパーライブラリ V2.00 対応
        }
        measureResult.result.BroadcastSystem = result.usBdSys;
        measureResult.result.Modulation = result.ulModBitPtrn;
        measureResult.result.Layer = result.ucLayer;
    }

    // 優先測定結果
    MeasureResult prefferedChannelMeasureResult;
    prefferedChannelMeasureResult.Received = (prefferedMeasuringResult == LC70_OK) ? true : false;
    prefferedChannelMeasureResult.Locked = prefferedChannel.ucLock;
    prefferedChannelMeasureResult.Status = prefferedChannel.usStat;
    prefferedChannelMeasureResult.Level = static_cast<float32_t>(prefferedChannel.ulLevel) /
                                          10;    // ラッパーライブラリ V2.00 対応
    prefferedChannelMeasureResult.Mer = static_cast<float32_t>(prefferedChannel.ulCnr) /
                                        100;       // ラッパーライブラリ V2.00 対応
    prefferedChannelMeasureResult.Cnr = static_cast<float32_t>(prefferedChannel.ulCnr) /
                                        100;       // ラッパーライブラリ V2.00 対応
    if (prefferedChannel.ulBerTotalBitNum > 0) {
        prefferedChannelMeasureResult.Ber = static_cast<float32_t>
                                            (prefferedChannel.ulBerErrBitNum) /   // ラッパーライブラリ V2.00 対応
                                            static_cast<float32_t>(prefferedChannel.ulBerTotalBitNum);
    }
    prefferedChannelMeasureResult.BroadcastSystem = prefferedChannel.usBdSys;
    prefferedChannelMeasureResult.Modulation = prefferedChannel.ulModBitPtrn;
    prefferedChannelMeasureResult.Layer = prefferedChannel.ucLayer;

    emit multiMeasureResult(measureResult, prefferedChannelMeasureResult);
}

bool Measurer::stopSingleMeasurement()
{
    if (!stopSingleMeasuring()) {
        return false;
    }

    return true;
}

bool Measurer::startSingleMeasuring(const AcquisitionTarget &target)
{
    if (_isSingleMeasuringExecutable) {
        return true;
    }

    // 測定条件にコンスタレーションを追加
    if (target == CONSTELLATION) {
        _singleMeasuringCondition.usCondition |= LC70_DATA_MSK_CONSTA;
    } else {
        _singleMeasuringCondition.usCondition &= ~LC70_DATA_MSK_CONSTA;
    }

    // シングル測定開始
    LC70_ERR_T ret = _communicator->startSingleMeasuring(&_singleMeasuringCondition);

    if (ret != LC70_OK) {
        _systemLog.write(QString(typeid(*this).name()), QString("Failed to start single measuring."));
        return false;
    }

    _isSingleMeasuringExecutable = true;
    if ((_singleMeasuringCondition.usCondition & LC70_DATA_MSK_CONSTA) == LC70_DATA_MSK_CONSTA) {
        // 測定条件にコンスタを含んでいる場合は、コンスタ測定も実行中にする
        _isConstellationExecutable = true;
    }
    return true;
}

bool Measurer::stopSingleMeasuring()
{
    if (!_isSingleMeasuringExecutable) {
        return true;
    }

    // 次回リトライできなくなるのを防ぐため、成否に関係なくフラグは落としておく
    _isSingleMeasuringExecutable = false;
    // シングル測定停止時にはコンスタ測定も停止する
    _isConstellationExecutable = false;

    LC70_ERR_T ret = _communicator->stopSingleMeasuring();

    if (ret != LC70_OK) {
        _systemLog.write(QString(typeid(*this).name()), QString("Failed to stop single measuring."));
        return false;
    }

    return true;
}

bool Measurer::startConstellationMeasuring()
{
    if (_isConstellationExecutable) {
        return true;
    }

    // コンスタレーション開始
    LC70_ERR_T ret = _communicator->startConstellationMeasuring(_constellationPoints);

    if (ret != LC70_OK) {
        _systemLog.write(QString(typeid(*this).name()), QString("Failed to start constellation."));
        return false;
    }

    _isConstellationExecutable = true;
    return true;
}

bool Measurer::stopConstellationMeasuring()
{
    if (!_isConstellationExecutable) {
        return true;
    }

    // 次回リトライできなくなるのを防ぐため、成否に関係なくフラグは落としておく
    _isConstellationExecutable = false;

    LC70_ERR_T ret = _communicator->stopConstellationMeasuring();

    if (ret != LC70_OK) {
        _systemLog.write(QString(typeid(*this).name()), QString("Failed to stop constellation."));
        return false;
    }

    return true;
}

bool Measurer::startDelayProfileMeasuring()
{
    if (_isDelayProfileExecutable) {
        return true;
    }

    // 測定条件からコンスタレーションを除く
    _singleMeasuringCondition.usCondition &= ~LC70_DATA_MSK_CONSTA;

    // 遅延プロファイル開始
    LC70_ERR_T ret = _communicator->startDelayProfileMeasuring(_singleMeasuringCondition);

    if (ret != LC70_OK) {
        _systemLog.write(QString(typeid(*this).name()), QString("Failed to start delay profile."));
        return false;
    }

    _isDelayProfileExecutable = true;

    return true;
}

bool Measurer::stopDelayProfileMeasuring()
{
    if (!_isDelayProfileExecutable) {
        return true;
    }

    // 次回リトライできなくなるのを防ぐため、成否に関係なくフラグは落としておく
    _isDelayProfileExecutable = false;

    // 遅延プロファイル停止
    LC70_ERR_T ret = _communicator->stopDelayProfileMeasuring();

    if (ret != LC70_OK) {
        _systemLog.write(QString(typeid(*this).name()), QString("Failed to stop delay profile."));
        return false;
    }

    return true;
}

bool Measurer::startMultiMeasuring(LC70_MEAS_MULTI_T *condition)
{
    if (_isMultiMeasuringExecutable) {
        return true;
    }

    LC70_ERR_T ret = _communicator->startMultiMeasuring(condition);

    if (ret != LC70_OK) {
        _systemLog.write(QString(typeid(*this).name()), QString("Failed to start multi measuring."));
        return false;
    }

    // マルチ測定のチャンネル数
    _multiMeasuringChannels = condition->sRegNum;

    _isMultiMeasuringExecutable = true;

    return true;
}

bool Measurer::stopMultiMeasuring()
{
    if (!_isMultiMeasuringExecutable) {
        return true;
    }

    // 次回リトライできなくなるのを防ぐため、成否に関係なくフラグは落としておく
    _isMultiMeasuringExecutable = false;

    // マルチチャンネル測定停止
    LC70_ERR_T ret = _communicator->stopMultiMeasuring();

    if (ret != LC70_OK) {
        _systemLog.write(QString(typeid(*this).name()), QString("Failed to stop multi measuring."), LOG_INFO);
        return false;
    }

    return true;
}

bool Measurer::startPrefferedMeasuring(LC70_MEAS_SINGLE_T *prefferedCondition)
{
    if (_isPreferredMeasuringExecutable) {
        return true;
    }

    // 優先測定開始
    LC70_ERR_T ret = _communicator->startPrefferedChannelInMultiMeasuring(prefferedCondition);

    if (ret != LC70_OK) {
        _systemLog.write(QString(typeid(*this).name()), QString("Failed to start preffered measuring."));
        return false;
    }

    _isPreferredMeasuringExecutable = true;
    return true;
}

bool Measurer::stopPrefferedMeasuring()
{
    if (!_isPreferredMeasuringExecutable) {
        return true;
    }

    // 次回リトライできなくなるのを防ぐため、成否に関係なくフラグは落としておく
    _isPreferredMeasuringExecutable = false;

    LC70_ERR_T ret = _communicator->stopPrefferedChannelInMultiMeasuring();

    if (ret != LC70_OK) {
        _systemLog.write(QString(typeid(*this).name()), QString("Failed to stop preffered measuring."));
        // 次回リトライできなくなるのを防ぐため、成否に関係なくフラグは落としておく
        _isMultiMeasuringExecutable = false;
        return false;
    }

    return true;
}

void Measurer::isMultiMeasuringResultSuitable(const LC70_MEAS_MULTI_RESULT_T& result, LC70_ERR_T& multiMeasuringResult)
{
    // 読み出しインデックスが測定中インデックスより小さかったら次のチャンネルを測定する
    // ユーザー設定で 1 チャンネルしかない場合はインデックスが 0 のまま測定を繰り返すようにする
    if (multiMeasuringResult == LC70_OK &&
            (result.ucCurrIdx > result.ucReadIdx ||
            (result.ucCurrIdx == result.ucReadIdx && _multiMeasuringChannels == 1))
            ) {
        // 収集ステータスが Level を返していれば続行
        if ((result.usStat & LC70_DATA_MSK_LEVEL) == LC70_DATA_MSK_LEVEL) {
            ++_multiMeasuringIndex;
        } else {
            // 測定できていないとみなす
            multiMeasuringResult = LC70_ERR;
        }

        _previousMeasuringIndex = result.ucCurrIdx;
    } else {
        // 測定中インデックスが一周して読み出しインデックスより小さくなった場合は次のチャンネルを測定し、そうでなければ同じインデックスで測定を繰り返す
        if (_previousMeasuringIndex > result.ucCurrIdx) {
            // 収集ステータスが Level を返していれば続行
            if ((result.usStat & LC70_DATA_MSK_LEVEL) == LC70_DATA_MSK_LEVEL) {
                ++_multiMeasuringIndex;
            } else {
                // 測定できていないとみなす
                multiMeasuringResult = LC70_ERR;
            }
        } else {
            _systemLog.write(QString(typeid(*this).name()),
                             QString("%1 Failed to get multi measure result index = %2 ret = %3 status = %4").arg(__FUNCTION__).arg(
                                 _multiMeasuringIndex).arg(
                                 multiMeasuringResult).arg(
                                 result.usStat));
            // 測定できていないとみなす
            multiMeasuringResult = LC70_ERR;
        }

        _previousMeasuringIndex = result.ucCurrIdx;
    }
}

void Measurer::onNotifySoftwareUpdate(LC70_PRG_UPDATE_T updater)
{
    LC70_ERR_T ret = _communicator->updateProgram(&updater);

    if (ret == LC70_OK) {
        emit notifySoftwareUpdateCompleted(true);
    } else {
        emit notifySoftwareUpdateCompleted(false);
    }
}
