/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef MEASURER_H
#define MEASURER_H

#include "paralleltask.h"
#include "communicator.h"
#include "stub.h"

/**
 * @brief スタンバイ時のパラメーター
 */
static constexpr long MAIN_AND_SUB = 0;     // MAIN, SUB CPU ともスタンバイ
static constexpr long SUB_ONLY = 1;         // SUB CPU のみスタンバイ

class Measurer : public ParallelTask
{
    Q_OBJECT
public:
    /**
     * @brief コンストラクター
     * @param actionMode 動作モード
     * @param communicator サブ CPU と連携するためのクラスのポインター
     * @param stub スタブクラスのポインター
     * @param numberOfPoints 一回の API 呼び出しで取得するコンスタレーションのデータ点数
     * @param systemLog システムロガー
     * @param parent 親オブジェクト
     * @note communicator と stub は動作モードに基づきどちらか一方を使用する
     */
    explicit Measurer(const ActionMode &actionMode, Communicator *communicator, Stub *stub, const uint16_t numberOfPoints,
                      SystemLog &systemLog, QObject *parent = nullptr);

    /**
     * @brief デストラクター
     */
    ~Measurer();

    /**
     * @brief スレッドで実行する処理
     */
    virtual void work() override;

    /**
     * @brief 測定結果取得対象の変更
     * @param acquisitionTarget 測定結果取得対象
     */
    void changeAcquisitionTarget(const AcquisitionTarget &acquisitionTarget);

    /**
     * @brief シングル測定、遅延プロファイルを取得する間隔
     * @param singleMeasurementInterval シングル測定取得間隔
     * @param delayProfileInterval 遅延プロファイル取得間隔
     */
    void setInterval(const uint32_t &singleMeasurementInterval, const uint32_t &delayProfileInterval);

    /**
     * @brief シングル測定停止
     * @return true: 成功、false: 失敗
     */
    bool stopSingleMeasurement(void);

    /**
     * @brief 遷移先画面種別の変更を受信
     * @param displayType 遷移先画面種別
     */
    void changeDisplayType(LC70_DISP_TYPE_T displayType);

    /**
     * @brief 方位測定開始
     * @return 方位測定開始の成否
     */
    bool startAzimuth(void);

    /**
     * @brief 方位測定停止
     * @return 方位測定停止の成否
     */
    bool stopAzimuth(void);

signals:
    /**
     * @brief シングル測定結果を通知
     * @param result 測定結果
     * @param constellations コンスタレーション
     */
    void singleMeasureResult(MeasureResult result, QList<QVariant> constellations);

    /**
     * @brief マルチチャンネル測定結果を通知
     * @param result マルチチャンネル測定結果
     * @param prefferedChannelResult マルチチャンネル優先測定結果
     */
    void multiMeasureResult(MultiResult result, MeasureResult prefferedChannelResult);

    /**
     * @brief 有効チャンネル情報の結果を通知
     * @param index スキャン対象のチャンネルリストのインデックス
     * @param result　スキャン結果
     */
    void scanningChannelResult(int index, LC70_MEAS_MULTI_RESULT_T result);

    /**
     * @brief 遅延プロファイル測定結果を通知
     * @param result 遅延プロファイル測定結果
     */
    void delayProfileResult(QVector<QPointF> result);

    /**
     * @brief アンテナ調整チャンネル測定結果を通知
     * @param result アンテナ調整測定結果
     */
    void antennaMeasureResult(MultiResult result);

    /**
     * @brief チャンネル変更中を通知
     * @param changing 変更中か否か
     */
    void notifyChannelChanging(bool changing);

    /**
     * @brief スキャン終了通知
     * @param complete 完了有無
     */
    void scanningFinished(bool complete);

    /**
     * @brief 遅延プロファイル有効状態変更通知
     * @param valid 有効状態
     */
    void notifyDelayProfileValid(bool valid);

    /**
     * @brief ウェイクアップの通知
     */
    void notifyWakeUp(void);

    /**
     * @brief アップデートの完了を通知
     * @param success アップデート成否
     */
    void notifySoftwareUpdateCompleted(bool success);

    /**
     * @brief 測定停止を通知
     * @param target 測定の種類
     */
    void notifyStopMeasuring(AcquisitionTarget target);

private slots:
    /**
     * @brief シングル測定再開依頼を受信
     * @param condition 測定条件
     * @param interval 測定結果取得間隔
     */
    void onResumeSingleMeasuring(LC70_CONSTA_NUM_T constellationPoints,
                                 LC70_MEAS_SINGLE_T condition, uint32_t interval);

    /**
     * @brief シングル測定チャンネル変更
     * @param displayType 測定開始後に表示する画面
     * @param condition チャンネル別測定条件
     * @note 測定条件のうち、Level, MER, BER は呼び出し側で設定しておくこと
     */
    void onSingleMeasuringChangeCondition(LC70_MEAS_SINGLE_T condition);

    /**
     * @brief マルチ測定優先測定チャンネル変更
     * @param condition チャンネル別測定条件
     */
    void onPreferredMeasuringChangeCondition(LC70_MEAS_SINGLE_T *condition);

    /**
     * @brief アンテナ調整測定チャンネル変更
     * @param condition チャンネル別測定条件
     */
    void onAntennaMeasuringChangeCondition(LC70_MEAS_MULTI_T *condition);

    /**
     * @brief マルチ測定再開依頼を受信
     * @param condition マルチ測定測定条件
     * @param prefferedCondition マルチ優先測定条件
     * @pram displayType 遷移先画面種別
     * @param interval 測定結果取得間隔
     */
    void onResumeMultiMeasuring(LC70_MEAS_MULTI_T *condition, LC70_MEAS_SINGLE_T *prefferedCondition, LC70_DISP_TYPE_T displayType, uint32_t interval);

    /**
     * @brief 有効なチャンネル情報のスキャン開始
     * @param condition 指定バンドのチャンネル定義リスト
     * @param acquisition 測定結果取得対象
     * @param interval スキャン間隔
     */
    void onResumeScanningChannel(LC70_MEAS_MULTI_T *condition, AcquisitionTarget acquisition, int interval);

    /**
     * @brief チャンネル情報のスキャン停止
     */
    void onSuspendScanningChannel(void);

    /**
     * @brief アンテナ調整再開依頼を受信
     * @param condition アンテナ調整測定条件
     * @param interval 測定結果取得間隔
     */
    void onResumeAntennaMeasuring(LC70_MEAS_MULTI_T *condition, uint32_t interval);

    /**
     * @brief シングル測定停止依頼を受信
     */
    void onSuspendSingleMeasuring(void);

    /**
     * @brief マルチ測定停止依頼を受信
     */
    void onSuspendMultiMeasuring(void);

    /**
     * @brief アンテナ調整測定停止依頼を受信
     */
    void onSuspendAntennaMeasuring(void);

    /**
     * @brief スタンバイ要求を受信
     * @param mode 0: MAIN AND SUB 1: SUB ONLY
     */
    void onRequestToStandBy(uint8_t mode);

    /**
     * @brief ソフトウェアアップデート要求を受信
     * @param updater アップデート情報
     */
    void onNotifySoftwareUpdate(LC70_PRG_UPDATE_T updater);

private:
    ActionMode _actionMode;                         //!< 動作モード
    Communicator *_communicator;                    //!< サブ CPU と連携するためのクラス
    Stub *_stub;                                    //!< スタブクラス
    AcquisitionTarget _acquisitionTarget;           //!< 測定結果取得対象
    LC70_MEAS_SINGLE_T _singleMeasuringCondition;   //!< シングル測定測定条件
    LC70_CONSTA_NUM_T _constellationPoints;         //!< コンスタレーション収集件数
    uint16_t _numberOfPoints;                       //!< 一回の API 呼び出しで取得するコンスタレーションのデータ点数
    LC70_MEAS_SINGLE_T _preferredMeasuringCondition;    //!< マルチ測定優先測定測定条件
    uint16_t _multiMeasuringChannels;               //!< マルチ測定時のチャンネル数
    int _channelScanIndex;                          //!< チャンネルスキャン実行回数
    bool _isSingleMeasuringExecutable;              //!< シングル測定の実行可否
    bool _isConstellationExecutable;                //!< コンスタレーションの実行可否
    bool _isDelayProfileExecutable;                 //!< 遅延プロファイルの実行可否
    bool _isMultiMeasuringExecutable;               //!< マルチ測定の実行可否
    bool _isPreferredMeasuringExecutable;           //!< マルチ測定優先測定の実行可否
    bool _isScanningExecutable;                     //!< スキャンの実行可否
    bool _isScanningFinished;                       //!< スキャン完了判定
    LC70_DISP_TYPE_T _displayType;                  //!< 遷移先画面種別
    uint32_t _singleMeasurementInterval;            //!< シングル測定取得間隔
    uint32_t _delayProfileInterval;                 //!< 遅延プロファイル取得間隔
    uint32_t _repeatCount;                          //!< シングル測定を何回繰り返せば遅延プロファイルを取得する間隔に到達するか(遅延プロファイル取得の方がシングル測定より頻度が低いことが前提)
    uint8_t _previousMeasuringIndex;                //!< 一つ前のマルチ測定中のインデックス
    int16_t _multiMeasuringIndex;                   //!< マルチ測定で測定するインデックス
    bool _isSingleMeasureingValid;                  //!< シングル測定結果有効状態
    uint16_t _delayProfileStatus;                   //!< 遅延プロファイルの収集ステータス
    bool _channelChanging;                          //!< チャンネル変更中

    /**
     * @brief シングル測定結果を取得
     */
    void measureSingle(void);

    /**
     * @brief コンスタレーションを取得
     */
    void constellation(void);

    /**
     * @brief 遅延プロファイルを取得
     */
    void delayProfile(void);

    /**
     * @brief マルチ測定結果を取得
     */
    void measureMulti(void);

    /**
     * @brief チャンネルスキャン結果を取得
     */
    void scanChannel(void);

    /**
     * @brief アンテナ調整測定結果を取得
     */
    void measureAntenna(void);

    /**
     * @brief 測定結果取得対象の enum 値に対応する文字列を返す
     * @param acquisitionTarget 測定結果取得対象
     * @return 測定結果取得対象の enum 値に対応する文字列
     */
    QString convertAcqusitionTarget(const AcquisitionTarget &acquisitionTarget);

    /**
     * @brief 遅延プロファイル状態チェック
     */
    void checkDelayProfileStatus(void);

    /**
     * @brief シングル測定開始
     * @param target 測定結果取得対象
     * @return true:成功、false:失敗
     */
    bool startSingleMeasuring(const AcquisitionTarget &target);

    /**
     * @brief シングル測定停止
     * @return true:成功、false:失敗
     */
    bool stopSingleMeasuring(void);

    /**
     * @brief コンスタレーション測定開始
     * @return true:成功、false:失敗
     */
    bool startConstellationMeasuring(void);

    /**
     * @brief コンスタレーション測定停止
     * @return true:成功、false:失敗
     */
    bool stopConstellationMeasuring(void);

    /**
     * @brief 遅延プロファイル測定開始
     * @return true:成功、false:失敗
     */
    bool startDelayProfileMeasuring(void);

    /**
     * @brief 遅延プロファイル測定停止
     * @return true:成功、false:失敗
     */
    bool stopDelayProfileMeasuring(void);

    /**
     * @brief マルチ測定開始
     * @param condition 測定条件
     * @return true:成功、false:失敗
     */
    bool startMultiMeasuring(LC70_MEAS_MULTI_T *condition);

    /**
     * @brief マルチ測定停止
     * @return true:成功、false:失敗
     */
    bool stopMultiMeasuring(void);

    /**
     * @brief マルチ優先測定開始
     * @param prefferedCondition 測定条件
     * @return true:成功、false:失敗
     */
    bool startPrefferedMeasuring(LC70_MEAS_SINGLE_T *prefferedCondition);

    /**
     * @brief マルチ優先測定停止
     * @return true:成功、false:失敗
     */
    bool stopPrefferedMeasuring(void);

    /**
     * @brief 受領したマルチ測定結果から次のチャンネルの測定を行ってよいか判断
     * @param result[in] マルチ測定結果
     * @param multiMeasuringResult[out] マルチ測定結果取得のラッパー関数の戻り値
     */
    void isMultiMeasuringResultSuitable(const LC70_MEAS_MULTI_RESULT_T& result, LC70_ERR_T& multiMeasuringResult);
};

#endif // MEASURER_H
