/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include "systemwatcher.h"

SystemWatcher::SystemWatcher(const ActionMode &actionMode, Communicator *communicator, Stub *stub,
                             SystemLog &systemLog, QObject *parent)
    : ParallelTask(systemLog, parent)
    , _actionMode(actionMode)
    , _communicator(communicator)
    , _stub(stub)
    , _keyInput(nullptr)
    , _devices()
    , _keyStatusInterval(100)
    , _deviceStatusInterval(500)
    , _repeatCount(_deviceStatusInterval / _keyStatusInterval)
    , _wakeUpCounter(0)
    , _previousWakeUpCounter(-1)
{
    if (actionMode == DEVELOPMENT) {
        connect(_stub, SIGNAL(keyStatus(LC70_KEY_STAT_T *)), this, SLOT(onKeyStatus(LC70_KEY_STAT_T *)), Qt::QueuedConnection);
        connect(_stub, SIGNAL(machineStatus(LC70_DEV_STAT_T *)), this, SLOT(onMachineStatus(LC70_DEV_STAT_T *)),
                Qt::QueuedConnection);
    }
}

SystemWatcher::~SystemWatcher()
{
    suspend();
    _systemLog.write(QString(typeid(*this).name()), QString("### device monitoring thread is suspended."));
}

void SystemWatcher::work()
{
    // スレッド開始後初回のみ実行
    if (!_initialized) {

        // スレッドごとにラッパー関数ライブラリを初期化
        if (_communicator != nullptr) {
            _communicator->initialize(
                14);    // 14 は ラッパー関数ライブラリーが認識するスレッド ID として割り振られた値
        }

        _initialized = true;
    }

    if (_actionMode == PRODUCTION) {
        static uint32_t count = 1;

        static LC70_KEY_STAT_T buttons;

        for (int i = 0; i < LC70_KEY_ID_MAX; ++i) {
            buttons.stat[i] = LC70_KEY_OFF;
        }

        LC70_ERR_T ret = _communicator->keyStatus(&buttons);

        if (ret == LC70_OK) {
            emit keyStatus(buttons);
        }

        if (count >= _repeatCount) {
            static LC70_DEV_STAT_T devices;
            memset(&devices, 0, sizeof(LC70_DEV_STAT_T));

            LC70_ERR_T ret = _communicator->deviceStatus(&devices);


            if (ret == LC70_OK) {
                // USB装着状態を送信する
                if (_devices.usba != devices.usba) {
                    emit changeUsbInsertionRemoval(static_cast<int>(devices.usba));
                }

                emit deviceStatus(devices);
                emit azimuth(devices.usAngle);
                _devices = devices;
            }

            count = 1;
            return;
        }

        ++count;

        // ウェイクアップカウンターの確認
        FILE* fp = fopen("/sys/power/wakeup_count", "r");
        if (fp == nullptr) {
            return;
        }

        char buffer[WAKE_UP_COUNTER_LENGTH] = { '\0' };
        fgets(buffer, WAKE_UP_COUNTER_LENGTH, fp);
        fclose(fp);
        _wakeUpCounter = atol(buffer);

        // ウェイクアップカウンターの値が変わっていたらスタンバイから復帰
        if (_previousWakeUpCounter >= 0 && _wakeUpCounter > _previousWakeUpCounter) {
            _systemLog.write(QString(typeid(*this).name()), QString("WAKE UP!! count:%1").arg(_wakeUpCounter), LOG_INFO);

            // ウェイクアップを通知
            emit notifyWakeUp();
        }

        // スタンバイ時のカウンターの変化を避ける
        if (_wakeUpCounter > _previousWakeUpCounter) {
            _previousWakeUpCounter = _wakeUpCounter + 1;
        }
    }
}

void SystemWatcher::setInterval(const uint32_t &keyStatusInterval, const uint32_t &deviceStatusInterval)
{
    _keyStatusInterval = keyStatusInterval;
    _deviceStatusInterval = deviceStatusInterval;

    // キー入力を何回行えばデバイス状態取得までの時間に到達するか
    if (_deviceStatusInterval > 0 && _keyStatusInterval > 0 && _deviceStatusInterval >= _keyStatusInterval) {
        _repeatCount = _deviceStatusInterval / _keyStatusInterval;
    }

    if (_repeatCount == 0) {
        _repeatCount = 1;
    }
}

void SystemWatcher::onKeyStatus(LC70_KEY_STAT_T *keyInput)
{
    _keyInput = keyInput;
    emit keyStatus(*_keyInput);
}

void SystemWatcher::onMachineStatus(LC70_DEV_STAT_T *devices)
{
    _devices = *devices;
    emit deviceStatus(_devices);

    emit azimuth(devices->usAngle);
}

LC70_DEV_STAT_T SystemWatcher::getDeviceStatus()
{
    return _devices;
}

long SystemWatcher::wakeUpCounter() const
{
    return _wakeUpCounter;
}
