/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef SYSTEMWATCHER_H
#define SYSTEMWATCHER_H

#include "paralleltask.h"
#include "communicator.h"
#include "stub.h"

class SystemWatcher : public ParallelTask
{
    Q_OBJECT
public:
    /**
     * @brief コンストラクター
     * @param actionMode 動作モード
     * @param communicator サブ CPU と連携するためのクラスのポインター
     * @param stub スタブクラスのポインター
     * @param systemLog システムロガー
     * @param parent 親オブジェクト
     * @note communicator と stub は動作モードに基づきどちらか一方を使用する
     */
    explicit SystemWatcher(const ActionMode &actionMode, Communicator *communicator, Stub *stub, SystemLog &systemLog,
                           QObject *parent = nullptr);

    /**
     * @brief デストラクター
     */
    ~SystemWatcher();

    /**
     * @brief スレッドで実行する処理
     */
    virtual void work() override;

    /**
     * @brief キー入力、デバイス状態を取得する間隔
     * @param keyStatusInterval キー入力取得間隔
     * @param deviceStatusInterval デバイス状態取得間隔
     */
    void setInterval(const uint32_t &keyStatusInterval, const uint32_t &deviceStatusInterval);

    /**
     * @brief デバイス状態を取得する
     * @param deviceStatus デバイス状態
     */
    LC70_DEV_STAT_T getDeviceStatus(void);

    /**
     * @brief ウェイアクアップカウンターの値を返す
     * @return ウェイアクアップカウンターの値
     */
    long wakeUpCounter(void) const;

signals:
    /**
     * @brief ボタン入力状態を通知
     * @param keyStatus ボタン入力状態
     */
    void keyStatus(LC70_KEY_STAT_T keyStatus);

    /**
     * @brief デバイス状態を通知
     * @param keyStatus デバイス状態
     */
    void deviceStatus(LC70_DEV_STAT_T deviceStatus);

    /**
     * @brief 方位角を通知
     * @param azimuth 方位角
     */
    void azimuth(uint16_t azimuth);

    /**
     * @brief USB挿抜変更通知を送信する
     * @param usbStatus USB挿抜状態 LC70_STRAGE_T値
     */
    void changeUsbInsertionRemoval(int usbStatus);

    /**
     * @brief ウェイクアップの通知
     */
    void notifyWakeUp(void);

private:
    ActionMode _actionMode;                     //!< 動作モード
    Communicator *_communicator;                //!< サブ CPU と連携するためのクラス
    Stub *_stub;                                //!< スタブクラス
    LC70_KEY_STAT_T *_keyInput;                 //!< キー入力状態
    LC70_DEV_STAT_T _devices;                   //!< デバイス状態
    uint32_t _keyStatusInterval;                //!< キー入力取得間隔
    uint32_t _deviceStatusInterval;             //!< デバイス状態取得間隔
    uint32_t _repeatCount;                      //!< キー入力取得を何回繰り返せばデバイス状態を取得する間隔に到達するか(デバイス状態取得の方がキー入力より頻度が低いことが前提)
    long _wakeUpCounter;                        //!< ウェイアクアップカウンターの値
    long _previousWakeUpCounter;                //!< 一つ前のウェイクアップカウンターの値

private slots:
    /**
     * @brief スタブからのボタン入力状態を受信
     * @param keyStatus ボタン入力状態
     * @note スタブ利用時は ButtonSimulator からの入力を受信して返すのみ(定周期での動作ではない)
     */
    void onKeyStatus(LC70_KEY_STAT_T *keyInput);

    /**
     * @brief スタブからのデバイス状態を受信
     * @param deviceStatus デバイス状態
     * @note スタブ利用時は ButtonSimulator からの入力を受信して返すのみ(定周期での動作ではない)
     */
    void onMachineStatus(LC70_DEV_STAT_T *deviceStatus);

};

#endif // SYSTEMWATCHER_H
