/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef LC70DUMMY_H
#define LC70DUMMY_H

#include <QObject>
#include "constant.h"

extern "C" {
#include "liblc70.h"
}

class LC70Dummy : public QObject
{
    Q_OBJECT
public:
    explicit LC70Dummy(QObject *parent = nullptr);

    /**
     * @brief 初期化
     * @param processId プロセスID
     * @result 処理結果
     */
    LC70_ERR_T initialize(PROC_ID_T processId);

    /**
     * @brief 測定設定ファイル読み込み
     * @param fileId 測定設定項目種別
     * @param data 測定設定項目に応じたデータ
     * @param isDefault デフォルトデータの指定有無
     * @result 処理結果
     */
    LC70_ERR_T readMeasureSettingFile(LC70_FILE_ID_T fileId, void *data, bool isDefault = false);

    /**
     * @brief システム設定ファイル読み込み
     * @param fileId システム設定項目種別
     * @param data システム設定項目種別に応じたデータ
     * @param isDefault デフォルトデータの指定有無
     * @result 処理結果
     */
    LC70_ERR_T readSystemSettingFile(LC70_FILE_ID_T fileId, void *data, bool isDefault = false);

    /**
     * @brief ユーザ設定ファイル読み込み
     * @param fileName ユーザ設定ファイルパス
     * @param userSetting ユーザ設定情報
     * @result 処理結果
     */
    LC70_ERR_T readUserSettingFile(const char *fileName, LC70_USER_SET_T *userSetting);

    /**
     * @brief チャンネル定義ファイル読み込み
     * @param fileName チャンネル定義ファイルパス
     * @param channelConfiguration チャンネル定義情報
     * @result 処理結果
     */
    LC70_ERR_T readChannelConfigurationFile(LC70_FILE_ID_T fileId, void *data, bool isDefault = false);

    /**
     * @brief キャリブレーション設定ファイル読み込み
     * @param fileName キャリブレーション設定ファイルパス
     * @param calibrationSetting キャリブレーション設定情報
     * @result 処理結果
     */
    LC70_ERR_T readCalibrationSettingFile(const char *fileName, LC70_CAL_SET_T *calibrationSetting);

    /**
     * @brief 測定設定ファイル書き込み
     * @param fileId 測定設定項目種別
     * @param data 測定設定情報
     * @result 処理結果
     */
    LC70_ERR_T writeMeasureSettingFile(LC70_FILE_ID_T fileId, void *data);
    /**
     * @brief システム設定ファイル書き込み
     * @param fileId 設定ファイルID
     * @param data システム設定情報
     * @result 処理結果
     */
    LC70_ERR_T writeSystemSettingFile(LC70_FILE_ID_T fileId, void *data);
    /**
     * @brief ユーザ設定ファイル書き込み
     * @param fileName ユーザ設定ファイルパス
     * @param userSetting ユーザ設定情報
     * @result 処理結果
     */
    LC70_ERR_T writeUserSettingFile(const char *fileName, LC70_USER_SET_T *userSetting);

    /**
     * @brief チャンネル定義ファイル書き込み
     * @param fileId 設定ファイルID
     * @param data チャンネル定義情報
     * @result 処理結果
     */
    LC70_ERR_T writeChannelConfigurationFile(LC70_FILE_ID_T fileId, void *data);

    /**
     * @brief キャリブレーション設定ファイル書き込み
     * @param fileName キャリブレーション設定ファイルパス
     * @param calibrationSetting キャリブレーション設定情報
     * @result 処理結果
     */
    LC70_ERR_T writeCalibrationSettingFile(const char *fileName, LC70_CAL_SET_T *calibrationSetting);

    /**
     * @brief 故障情報取得
     * @param failureInformation 故障情報
     * @result 処理結果
     */
    LC70_ERR_T failureInformation(LC70_FAIL_INFO_T *failureInformation);

    /**
     * @brief キー(ボタン)入力状態取得
     * @param keyStatus キー入力状態
     * @result 処理結果
     */
    LC70_ERR_T keyStatus(LC70_KEY_STAT_T *keyStatus);

    /**
     * @brief デバイス(方位・バッテリ・ストレージ)状態取得
     * @param deviceStatus デバイス状態
     * @result 処理結果
     */
    LC70_ERR_T deviceStatus(LC70_DEV_STAT_T *deviceStatus);

    /**
     * @brief 測定入力信号設定
     * @param inputSignal 入力信号
     * @result 処理結果
     */
    LC70_ERR_T setMeasuringInputSignal(LC70_INPUT_T inputSignal);

    /**
     * @brief 給電設定
     * @param outputVoltage 出力電圧
     * @result 処理結果
     */
    LC70_ERR_T setPowerSupply(LC70_POWER_SUPPLY_T outputVoltage);

    /**
     * @brief データ有効閾値設定
     * @param blockNumber 有効閾値設定ブロック数
     * @param dataThresholdBlock 有効閾値設定ブロックの先頭アドレス
     * @result 処理結果
     */
    LC70_ERR_T setDataThreshold(int32_t dataThresholdBlockNumber, LC70_DATA_THR_T *dataThresholdBlock);

    /**
     * @brief 測定状態取得
     * @param measuringStatus 測定状態
     * @result 処理結果
     */
    LC70_ERR_T measuringStatus(LC70_MEAS_STAT_T *measuringStatus);

    /**
     * @brief シングルチャンネル測定開始
     * @param measuringCondition 測定条件
     * @result 処理結果
     */
    LC70_ERR_T startSingleMeasuring(LC70_MEAS_SINGLE_T *measuringCondition);

    /**
     * @brief シングルチャンネル測定停止
     * @result 処理結果
     */
    LC70_ERR_T stopSingleMeasuring(void);

    /**
     * @brief コンスタレーション測定開始
     * @param collectionPoints 収集点数
     * @result 処理結果
     */
    LC70_ERR_T startConstellationMeasuring(LC70_CONSTA_NUM_T collectionPoints);

    /**
     * @brief コンスタレーション測定停止
     * @result 処理結果
     */
    LC70_ERR_T stopConstellationMeasuring(void);

    /**
     * @brief シングルチャンネル測定結果取得
     * @param constellationPoints コンスタレーションデータの読み出し点数(0～200)
     * @param measuringResult 測定結果
     * @result 処理結果
     */
    LC70_ERR_T singleMeasuringResult(int16_t constellationPoints,
                                     LC70_MEAS_SINGLE_RESULT_T *measuringResult);

    /**
     * @brief 遅延プロファイル測定開始
     * @result 処理結果
     */
    LC70_ERR_T startDelayProfileMeasuring(void);

    /**
     * @brief 遅延プロファイル測定停止
     * @result 処理結果
     */
    LC70_ERR_T stopDelayProfileMeasuring(void);

    /**
     * @brief 遅延プロファイル測定結果取得
     * @result 処理結果
     */
    LC70_ERR_T delayProfileMeasuring(void);

    /**
     * @brief マルチチャンネル測定開始
     * @param measuringCondition 測定条件
     * @result 処理結果
     */
    LC70_ERR_T startMultiMeasuring(LC70_MEAS_MULTI_T *measuringCondition);

    /**
     * @brief マルチチャンネル測定停止
     * @result 処理結果
     */
    LC70_ERR_T stopMultiMeasuring(void);

    /**
     * @brief マルチチャンネル測定開始
     * @param measureEnabled 測定要否情報の先頭アドレス
     * @result 処理結果
     */
    LC70_ERR_T setMultiMeasuringChannel(uint8_t *measureEnabled);

    /**
     * @brief マルチチャンネル測定結果取得
     * @param index 測定インデックス
     * @param measuringResult 測定結果
     * @result 処理結果
     */
    LC70_ERR_T multiMeasuringResult(int16_t index, LC70_MEAS_MULTI_RESULT_T *measuringResult);

    /**
     * @brief マルチチャンネルフォーカスチャンネル優先測定
     * @param mode モード
     * @param index 選択チャンネルのインデックス
     * @param measuringCondition 測定条件
     * @result 処理結果
     */
    LC70_ERR_T multiMeasuringFocus(LC70_FMODE_T mode, int16_t index, uint16_t measuringCondition);

    /**
     * @brief 再起動要求
     * @result 処理結果
     */
    LC70_ERR_T systemReboot(void);

    /**
     * @brief シャットダウン要求
     * @result 処理結果
     */
    LC70_ERR_T systemShutdown(void);

    /**
     * @brief スリープ要求
     * @result 処理結果
     */
    LC70_ERR_T systemSleep(void);

    /**
     * @brief ウェイクアップ要求
     * @result 処理結果
     */
    LC70_ERR_T systemWakeup(void);

    /**
     * @brief ブザー制御
     * @param buzzerPattern ブザー鳴動パターン
     * @param buzzerMode ブザーモード
     * @result 処理結果
     */
    LC70_ERR_T buzzer(LC70_BUZZ_T buzzerPattern, LC70_BUZZ_MODE_T buzzerMode);

    /**
     * @brief 強制ブザー音量制御
     * @param buzzerPattern ブザーパターン
     * @param buzzerVolume ブザー音量
     * @result 処理結果
     */
    LC70_ERR_T fourceBuzzerVolume(LC70_BUZZ_T buzzerPattern, int buzzerVolume);

    /**
     * @brief setDateTime 現在時刻設定
     * @param yyyy
     * @param mm
     * @param dd
     * @param week
     * @param hour
     * @param minutes
     * @return
     */
    int setDateTime(int yyyy, int mm, int dd, int week, int hour, int minutes);

    /**
     * @brief オートパワーオフ時間を設定する
     * @param offTime 設定したオートパワーオフ時間
     * @return 処理結果
     */
    LC70_ERR_T setAutoPowerOffTime(int offTime);

    /**
     * @brief deleteUserlogSetFile
     * @param fileName
     * @return
     */
    LC70_ERR_T deleteUserlogSetFile(const char *fileName);

    /**
     * @brief readSetFile
     * @param fileName
     * @param userlogSet
     * @return 読み込んだ行数あるいは LC70_ERR_T
     */
    int readSetFile(const char *fileName, LC70_LOGGER_SET_LOGGER_T &userlogSet);

    /**
     * @brief writeSetFile
     * @param fileName
     * @param userlogSet
     * @param lines
     * @return
     */
    LC70_ERR_T writeSetFile(const char *fileName, const LC70_LOGGER_SET_LOGGER_T &userlogSet, int lines);

    /**
     * @brief readSetFileList
     * @param dirname
     * @param userlogSetFileList
     * @param mode
     * @return
     */
    LC70_ERR_T readSetFileList(const QString &dirname, QList<LC70_LOGGER_SET_LOGGER_FLIST_ROW_T> &userlogSetFileList, ChannelEditMode mode);

    /**
     * @brief writeSetFileList
     * @param dirname
     * @param userlogSetFileList
     * @param mode
     * @return
     */
    LC70_ERR_T writeSetFileList(const QString &dirname, QList<LC70_LOGGER_SET_LOGGER_FLIST_ROW_T> &userlogSetFileList, ChannelEditMode mode);

    /**
     * @brief 閾値設定制御
     * @param thresholdData 閾値データ
     * @return 処理結果
     */
    LC70_ERR_T setThresholdData(QVariantList thresholdData);

    /**
     * @brief コンスタレーション表示ポイント設定制御
     * @param constellationSelectNum 選択中コンスタレーション番号
     * @return 処理結果
     */
    LC70_ERR_T setConstellationShowPoint(int constellationSelectNum);

    /**
     * @brief ケーブル補正データ設定制御
     * @param levelOffsetData ケーブル補正データリスト
     * @return 処理結果
     */
    LC70_ERR_T setLebelOffsetData(QVariantList levelOffsetDataList);

    /**
     * @brief 音量設定制御
     * @param soundVolume 音量設定値
     * @return 処理結果
     */
    LC70_ERR_T setSoundVolume(int soundVolume);

    /**
     * @brief 明るさ設定制御
     * @param brightness 明るさ設定値
     * @return 処理結果
     */
    LC70_ERR_T setBrightness(int brightness);

    /**
     * @brief 明るさ設定制御
     * @param brightness 明るさ設定値
     * @return 処理結果
     */
    LC70_ERR_T setScreenBrightness(int brightness);
    /**
     * @brief 局部発振周波数設定制御
     * @param localFrequency 局部発振周波数
     * @return 処理結果
     */
    LC70_ERR_T setLocalFrequency(QVariantList localFrequency);

    /**
     * @brief ケーブル損失有効設定制御
     * @param isEnableCableLoss ケーブル損失有効かどうか 0:無効 1:有効
     * @return 処理結果
     */
    LC70_ERR_T setIsEnableCableLoss(int isEnableCableLoss);

    /**
     * @brief setMERCN
     * @param mercnValue
     * @return
     */
    LC70_ERR_T setMERCN(int mercnValue);

    /**
     * @brief CATVチャンネル周波数の設定制御
     * @param settingNo 設定するCATVチャンネル周波数 設定番号
     * @return 処理結果
     */
    LC70_ERR_T setCATVChannelFrequency(int settingNo);

    /**
     * @brief 光測定開始制御
     * @param ulWaveLength 測定する波長
     * @return 処理結果
     */
    LC70_ERR_T startOpticalMeasurement(uint32_t ulWaveLength);

    /**
     * @brief 光測定停止制御
     * @return 処理結果
     */
    LC70_ERR_T stopOpticalMeasurement(void);

    /**
     * @brief 光測定の測定結果の取得
     * @param result[out] 測定結果
     * @return 処理結果
     */
    LC70_ERR_T getOpticalMeasurementResult(QVariant &result);

    /**
     * @brief DC測定開始制御
     * @return 処理結果
     */
    LC70_ERR_T startDCMeasurement(void);

    /**
     * @brief DC測定停止制御
     * @return 処理結果
     */
    LC70_ERR_T stopDCMeasurement(void);

    /**
     * @brief DC測定の測定結果の取得
     * @param result[out] 測定結果
     * @return 処理結果
     */
    LC70_ERR_T getDCMeasurementResult(QVariant &result);

    /**
     * @brief 機種情報の取得
     * @param result[out] 機種情報
     * @return 処理結果
     */
    LC70_ERR_T getModelInformation(LC70_PRODUCT_INFO_T &result);

    /**
     * @brief サブ CPU の強制スタンバイ、ウェイクアップによるリセット
     * @return サブ CPU のリセット成否
     */
    LC70_ERR_T resetSubCpu(void);

signals:

private:
    static int testValue;
};

#endif // LC70DUMMY_H
