/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include "stub.h"
#include <QFile>
#include <QImage>
#include <QDebug>
#include "channeldefinitionitem.h"

static constexpr const char *SINGLE_MEASUREMENT_DATA_FILENAME = "single.csv";
static constexpr const char *CONSTELLATION_DATA_FILENAME = "constellation.csv";
static constexpr const char *MULTI_MEASUREMENT_DATA_FILENAME = "multi.csv";
static constexpr const char *PREFFERED_CHANNEL_DATA_FILENAME = "preffered.csv";
static constexpr const char *DELAY_PROFILE_DATA_FILENAME = "delayprofile.csv";

/**
 *  @brief 各チャンネルと対応する定義ファイル名
 */

static const ChannelDefinitionFile CHANNEL_DEFINITION_FILE[] = {
    { LC70_FILE_ID_CH_SET_UHF,    "channel_setting_file_uhf_6mhz.csv",     2, LC70_BAND_UHF  },        //!< チャンネル定義(地デジ(帯域幅6MHz))ファイル
    { LC70_FILE_ID_CH_SET_BS,     "channel_setting_file_bs_34.5mhz.csv",   3, LC70_BAND_BS},           //!< チャンネル定義(CS(帯域幅34.5MHz))ファイル
    { LC70_FILE_ID_CH_SET_CS,     "channel_setting_file_cs_34.5mhz.csv",   3, LC70_BAND_CS},           //!< チャンネル定義(BS(帯域幅34.5MHz))ファイル
    { LC70_FILE_ID_CH_SET_SCCB,   "channel_setting_file_sccb_27mhz.csv",   3, LC70_BAND_SCCB},         //!< チャンネル定義(SCC-B(帯域幅27MHz))ファイル
    { LC70_FILE_ID_CH_SET_SCCC,   "channel_setting_file_sccc_27mhz.csv",   3, LC70_BAND_SCCC},         //!< チャンネル定義(SCC-C(帯域幅27MHz))ファイル
    { LC70_FILE_ID_CH_SET_JSAT3A, "channel_setting_file_jsat3a_27mhz.csv", 3, LC70_BAND_JCSAT3},       //!< チャンネル定義(JSAT-3A(帯域幅27MHz))ファイル
    { LC70_FILE_ID_CH_SET_JSAT4B, "channel_setting_file_jsat4b_27mhz.csv", 3, LC70_BAND_JCSAT4},       //!< チャンネル定義(JSAT-4B(帯域幅27MHz))ファイル
    // ラッパー関数ライブラリ V2.00 対応
    { LC70_FILE_ID_CH_SET_CATV_ALL,         "channel_setting_file_catv_all_6mhz.csv",         3, LC70_BAND_CATV_ALL},         //!< チャンネル定義(CATV ALL(帯域幅6MHz))ファイル
    { LC70_FILE_ID_CH_SET_CATV_90_470MHZ,   "channel_setting_file_catv_90_470mhz_6mhz.csv",   3, LC70_BAND_CATV_90_470MHZ},   //!< チャンネル定義(CATV 90-470MHz(帯域幅6MHz))ファイル
    { LC70_FILE_ID_CH_SET_CATV_470_1002MHZ, "channel_setting_file_catv_470_1002mhz_6mhz.csv", 3, LC70_BAND_CATV_470_1002MHZ}, //!< チャンネル定義(CATV 470-1002MHz(帯域幅6MHz))ファイル
    { LC70_FILE_ID_CH_SET_SKYP,             "channel_setting_file_skyp_6mhz.csv",             2, LC70_BAND_SKYP},             //!< チャンネル定義(スカパー光(帯域幅6MHz))ファイル
    { LC70_FILE_ID_CH_SET_PILOT,            "channel_setting_file_pilot.csv",                 2, LC70_BAND_PILOT },           //!< チャンネル定義(PILOT)ファイル
    { LC70_FILE_ID_CH_SET_LTE,              "channel_setting_file_lte.csv",                   3, LC70_BAND_LTE},              //!< チャンネル定義(LTE)ファイル
    { LC70_FILE_ID_CH_SET_FM,               "channel_setting_file_fm.csv",                    2, LC70_BAND_FM},               //!< チャンネル定義(FM)ファイル
    { LC70_FILE_ID_CH_SET_CATV_UP_STREAM,   "channel_setting_file_catv_up_stream.csv",        2, LC70_BAND_CATV_UP_STREAM}    //!< チャンネル定義(CATV上り)）ファイル
};

static constexpr const char *IGNORE_ELEMENT =
    "CH_NO";                              //!< CSV ファイル中の無視する要素

Stub::Stub(QObject *parent)
    : QObject(parent)
    , _adaptor(nullptr)
    , _connection(QDBusConnection::sessionBus())
    , _acquisitionTarget(NOT_ACQUISITION)
    , _channelDefinitionFilePath()
    , _singleMeasurementResult()
    , _constellationResult()
    , _multiMeasurementResult()
    , _prefferedChannelResult()
    , _delayProfileResult()
    , _scanIndex(0)
{
    ChannelDefinitionItem channel = {-1, "", 1, 2, 3, 4 };
    // D-Bus のサービス名("jp.co.dxantenna.levelchecker")を登録
    _connection.registerService("jp.co.dxantenna.levelchecker");
    // D-Bus のオブジェクト名("/")を登録
    _connection.registerObject("/", this);
    // D-Bus 経由のメソッド呼び出しに対応するスロットを接続
    _connection.connect("jp.co.dxantenna", "/", "jp.co.dxantenna.levelchecker", "buttonStatus", this,
                        SLOT(buttonStatus(int)));
    _connection.connect("jp.co.dxantenna", "/", "jp.co.dxantenna.levelchecker", "deviceStatus", this,
                        SLOT(deviceStatus(int, int, int, int, int)));
    _connection.connect("jp.co.dxantenna", "/", "jp.co.dxantenna.levelchecker", "failureInformation", this,
                        SLOT(failureInformation(int, int, int, int, QString)));

    _adaptor = new LevelcheckerAdaptor(this);
}

Stub::~Stub()
{
    _connection.disconnect("jp.co.dxantenna", "/", "jp.co.dxantenna.levelchecker", "buttonStatus", this,
                           SLOT(buttonStatus(int)));
    _connection.disconnect("jp.co.dxantenna", "/", "jp.co.dxantenna.levelchecker", "deviceStatus", this,
                        SLOT(deviceStatus(int, int, int, int, int)));
    _connection.disconnect("jp.co.dxantenna", "/", "jp.co.dxantenna.levelchecker", "failureInformation", this,
                        SLOT(failureInformation(int, int, int, int, QString)));
}

bool Stub::initialize(const QString &singleMeasurementDataPath, const QString &multiMeasurementDataPath,
                      const QString &channelDefinitionFilePath)
{
    // チャンネルを定義した CSV ファイルを配置したパス
    _channelDefinitionFilePath = channelDefinitionFilePath;

    // シングルチャンネル測定
    QFile single(singleMeasurementDataPath + QString("/") + SINGLE_MEASUREMENT_DATA_FILENAME);

    if (!single.open(QIODevice::ReadOnly | QIODevice::Text)) {
        return false;
    }

    _singleMeasurementResult.clear();
    QTextStream singleStream(&single);

    while (!singleStream.atEnd()) {
        QString line = singleStream.readLine();
        bool ok[8] = { false, false, false, false, false, false, false, false };

        QVector<QStringRef> values = line.splitRef(",");

        if (values.count() != 8) {
            continue;
        }

        if (values[0].contains("#")) {
            continue;
        }

        MeasurementResult result;
        result.locked = LOCKED;
        result.status = LC70_DATA_MSK_LEVEL | LC70_DATA_MSK_BER | LC70_DATA_MSK_MER | LC70_DATA_MSK_CONSTA |
                        LC70_DATA_MSK_DLY_PRF;
        result.level = values[0].toFloat(&ok[0]);
        result.bitErrorCount = values[1].toFloat(&ok[1]);
        result.totalCount = values[2].toFloat(&ok[2]);
        result.mer = values[3].toFloat(&ok[3]);
        result.cnr = values[4].toFloat(&ok[4]);
        result.system = values[5].toUInt(&ok[5]);
        result.layer = values[6].toUInt(&ok[6]);
        result.modulation = values[7].toUInt(&ok[7]);

        if (!ok[0] || !ok[1] || !ok[2] || !ok[3] || !ok[4] || !ok[5] || !ok[6] || !ok[7]) {
            single.close();
            return false;
        }

        _singleMeasurementResult.append(result);
    }

    single.close();

    // コンスタレーション
    QFile constellation(singleMeasurementDataPath + QString("/") + CONSTELLATION_DATA_FILENAME);

    if (!constellation.open(QIODevice::ReadOnly | QIODevice::Text)) {
        return false;
    }

    _constellationResult.clear();
    QTextStream constellationStream(&constellation);

    while (!constellationStream.atEnd()) {
        QString line = constellationStream.readLine();
        bool ok[2] = { false, false };
        QVector<QStringRef> values = line.splitRef(",");

        if (values.count() != 2) {
            continue;
        }

        if (values[0].contains("#")) {
            continue;
        }

        QPoint result;
        result.setX(values[0].toInt(&ok[0]));
        result.setY(values[1].toInt(&ok[1]));

        if (!ok[0] || !ok[1]) {
            constellation.close();
            return false;
        }

        _constellationResult.append(result);
    }

    constellation.close();

    // 遅延プロファイル
    QFile delayProfile(singleMeasurementDataPath + QString("/") + DELAY_PROFILE_DATA_FILENAME);

    if (!delayProfile.open(QIODevice::ReadOnly | QIODevice::Text)) {
        return false;
    }

    _delayProfileResult.clear();
    QTextStream delayProfileStream(&delayProfile);

    while (!delayProfileStream.atEnd()) {
        QString line = delayProfileStream.readLine();
        bool ok = false;
        QVector<QStringRef> values = line.splitRef(",");

        if (values.count() != 2048) {
            continue;
        }

        if (values[0].contains("#")) {
            continue;
        }

        QList<QPointF> pointList;
        double x = MINIMUM_DELAY_PROFILE_TIME + DELAY_PROFILE_INTERVAL;

        foreach (QStringRef value, values) {
            float dB = value.toFloat(&ok);

            if (!ok) {
                delayProfile.close();
                return false;
            }

            pointList.append(QPointF(x, dB));
            x += DELAY_PROFILE_INTERVAL;
        }

        _delayProfileResult.append(pointList);
    }

    delayProfile.close();

    // マルチチャンネル測定
    QFile multi(multiMeasurementDataPath + QString("/") + MULTI_MEASUREMENT_DATA_FILENAME);

    if (!multi.open(QIODevice::ReadOnly | QIODevice::Text)) {
        return false;
    }

    _multiMeasurementResult.clear();
    QTextStream multiStream(&multi);

    // スタブデータ(CSVファイル)の列順は、ロック状態, 収集ステータス, 読出データインデックス, カレント測定中インデックス, level, mer, cnr, ビットエラー数, 総データ数 とする
    while (!multiStream.atEnd()) {
        QString line = multiStream.readLine();
        bool ok[9] = { false, false, false, false, false, false, false, false, false };
        QVector<QStringRef> values = line.splitRef(",");

        if (values.count() != 9) {
            continue;
        }

        if (values[0].contains("#")) {
            continue;
        }

        MultiMeasurementResult result;
        result.locked = values[0].toUInt(&ok[0]);
        result.status = values[1].toUInt(&ok[1]);
        result.readIndex = values[2].toUInt(&ok[2]);
        result.currentMeasuringIndex = values[3].toUInt(&ok[3]);
        result.level = values[4].toFloat(&ok[4]);
        result.mer = values[5].toFloat(&ok[5]);
        result.cnr = values[6].toFloat(&ok[6]);
        result.bitErrorCount = values[7].toUInt(&ok[7]);
        result.totalCount = values[8].toUInt(&ok[8]);

        if (!ok[0] || !ok[1] || !ok[2] || !ok[3] || !ok[4] || !ok[5] || !ok[6] || !ok[7] || !ok[8] ) {
            multi.close();
            return false;
        }

        _multiMeasurementResult.append(result);
    }

    multi.close();

    // マルチチャンネル優先測定
    QFile prefferedChannel(multiMeasurementDataPath + QString("/") + PREFFERED_CHANNEL_DATA_FILENAME);

    if (!prefferedChannel.open(QIODevice::ReadOnly | QIODevice::Text)) {
        return false;
    }

    _prefferedChannelResult.clear();
    QTextStream prefferedChannelStream(&prefferedChannel);

    while (!prefferedChannelStream.atEnd()) {
        QString line = prefferedChannelStream.readLine();
        bool ok[8] = { false, false, false, false, false, false, false, false };

        QVector<QStringRef> values = line.splitRef(",");

        if (values.count() != 8) {
            continue;
        }

        if (values[0].contains("#")) {
            continue;
        }

        MeasurementResult result;
        result.locked = LOCKED;
        result.status = LC70_DATA_MSK_LEVEL | LC70_DATA_MSK_BER | LC70_DATA_MSK_MER;
        result.level = values[0].toFloat(&ok[0]);
        result.bitErrorCount = values[1].toUInt(&ok[1]);
        result.totalCount = values[2].toUInt(&ok[2]);
        result.mer = values[3].toFloat(&ok[3]);
        result.mer = values[4].toFloat(&ok[4]);
        result.system = values[5].toUInt(&ok[5]);
        result.layer = values[6].toUInt(&ok[6]);
        result.modulation = values[7].toUInt(&ok[7]);

        if (!ok[0] || !ok[1] || !ok[2] || !ok[3] || !ok[4] || !ok[5] || !ok[6] || !ok[7]) {
            prefferedChannel.close();
            return false;
        }

        _prefferedChannelResult.append(result);
    }

    prefferedChannel.close();

    return true;
}

void Stub::changeAcquisitionTarget(const AcquisitionTarget &acquisitionTarget)
{
    _acquisitionTarget = acquisitionTarget;
}

LC70_ERR_T Stub::singleMeasuringResult(int16_t constellationPoints, LC70_MEAS_SINGLE_RESULT_T *measuringResult)
{
    if (_singleMeasurementResult.count() == 0) {
        return LC70_ERR;
    }

    if (_acquisitionTarget == SINGLE || _acquisitionTarget == CONSTELLATION || _acquisitionTarget == DELAY_PROFILE) {
        static int singleMeasurementIndex = 0;
        static int constellationIndex = 0;

        // シングル測定結果
        measuringResult->ulLevel = _singleMeasurementResult[singleMeasurementIndex].level;
        measuringResult->ulBerErrBitNum = _singleMeasurementResult[singleMeasurementIndex].bitErrorCount;
        measuringResult->ulBerTotalBitNum = _singleMeasurementResult[singleMeasurementIndex].totalCount;
        measuringResult->ulCnr = _singleMeasurementResult[singleMeasurementIndex].cnr;
        measuringResult->ucLock = _singleMeasurementResult[singleMeasurementIndex].locked;
        measuringResult->usStat = _singleMeasurementResult[singleMeasurementIndex].status;
        measuringResult->ucLayer = _singleMeasurementResult[singleMeasurementIndex].layer;
        measuringResult->usBdSys = _singleMeasurementResult[singleMeasurementIndex].system;
        measuringResult->ulModBitPtrn = _singleMeasurementResult[singleMeasurementIndex].modulation;
        measuringResult->ucGi = 0;

        ++singleMeasurementIndex;

        if (singleMeasurementIndex > _singleMeasurementResult.count() - 1) {
            singleMeasurementIndex = 0;
        }

        // シングル測定の場合はこの時点で終わり
        if (_acquisitionTarget == SINGLE) {
            return LC70_OK;
        }

        // コンスタレーション
        if (_constellationResult.count() == 0) {
            return LC70_OK;
        }

        // マイナスか最大読み出し点数より大きい
        if (constellationPoints < 0 || constellationPoints > LC70_MAX_CONSTA_RD_NUM) {
            return LC70_ERR;
        }

        measuringResult->sConstaNum = constellationPoints;

        // 次に読み出すインデックスの計算用(スタブ固有の処理)
        int additions = 0;

        for (int i = 0; i < constellationPoints; ++i) {
            if (constellationIndex + i > _constellationResult.count() - 1) {
                constellationIndex = 0;
                additions = 0;
            }

            QPoint point = _constellationResult[constellationIndex + i].toPoint();
            measuringResult->consta[i].x = point.x();
            measuringResult->consta[i].y = point.y();
            ++additions;
        }

        // 次に読み出す位置を設定
        constellationIndex += additions;

        if (constellationIndex > _constellationResult.count() - 1) {
            constellationIndex = 0;
        }
    }

    return LC70_OK;
}

LC70_ERR_T Stub::delayProfileMeasuring(LC70_MEAS_DLY_PRF_RESULT_T *result)
{
    static int delayProfileIndex = 0;

    if (_acquisitionTarget == DELAY_PROFILE) {
        if (_delayProfileResult.count() == 0) {
            return LC70_ERR;
        }

        result->usStat = 2;

        for (int i = 0; i < DELAY_PROFILE_DATA_COUNT; ++i) {
            result->ucDlyPrfData[i] = static_cast<uint8_t>(_delayProfileResult[delayProfileIndex][i].y());
        }
    }

    ++delayProfileIndex;

    if (delayProfileIndex > _delayProfileResult.count() - 1) {
        delayProfileIndex = 0;
    }

    return LC70_OK;
}

LC70_ERR_T Stub::multiMeasuringResult(int16_t measuringIndex, LC70_MEAS_MULTI_RESULT_T *measuringResult)
{
    // スタブは 240 行(40 チャンネルが 6 回分)
    static int count = 0;
    static int additiononalIndex = 0;

    if (_multiMeasurementResult.count() == 0) {
        return LC70_ERR;
    }

    if (additiononalIndex + measuringIndex >= 240) {
        additiononalIndex = 0;
    }

    memset(measuringResult, 0, sizeof(LC70_MEAS_MULTI_RESULT_T));
    measuringResult->ucLock = _multiMeasurementResult[additiononalIndex + measuringIndex].locked;   // ロック状態
    measuringResult->usStat =
        _multiMeasurementResult[additiononalIndex + measuringIndex].status;                         // 収集ステータス
    measuringResult->ucReadIdx =
        _multiMeasurementResult[additiononalIndex +
                                                  measuringIndex].readIndex;                        // 読出データインデックス
    measuringResult->ucCurrIdx =
        _multiMeasurementResult[additiononalIndex +
                                                  measuringIndex].currentMeasuringIndex;            // カレント測定中インデックス
    measuringResult->usBdSys = MES_DF_SETMOD_OFDM;  // 放送方式規格
    measuringResult->ulModBitPtrn = LC70_SETMOD_OFDM;   // 変調方式
    measuringResult->ulLevel = _multiMeasurementResult[additiononalIndex + measuringIndex].level;   // LEVEL
    measuringResult->ulCnr = _multiMeasurementResult[additiononalIndex + measuringIndex].cnr;       // CNR
    measuringResult->ulBerErrBitNum = _multiMeasurementResult[additiononalIndex +
                                                        measuringIndex].bitErrorCount;    // エラー数
    measuringResult->ulBerTotalBitNum = _multiMeasurementResult[additiononalIndex +
                                                          measuringIndex].totalCount;     // 総データ数

    ++count;

    if (count == 40) {
        count = 0;
        additiononalIndex += 40;

        if (additiononalIndex > 200) {
            additiononalIndex = 0;
        }
    }

    return LC70_OK;
}

LC70_ERR_T Stub::multiMeasuringPrefferedChannelResult(LC70_MEAS_SINGLE_RESULT_T *measuringResult)
{
    static int index = 0;

    if (_prefferedChannelResult.count() == 0) {
        return LC70_ERR;
    }

    // マルチチャンネル優先測定結果
    measuringResult->ulLevel = _singleMeasurementResult[index].level;
    measuringResult->ulBerErrBitNum = _singleMeasurementResult[index].bitErrorCount;
    measuringResult->ulBerTotalBitNum = _singleMeasurementResult[index].totalCount;
    measuringResult->ulCnr = _singleMeasurementResult[index].cnr;
    measuringResult->ucLock = _singleMeasurementResult[index].locked;
    measuringResult->usStat = _singleMeasurementResult[index].status;
    measuringResult->ucLayer = _singleMeasurementResult[index].layer;
    measuringResult->usBdSys = _singleMeasurementResult[index].system;
    measuringResult->ulModBitPtrn = _singleMeasurementResult[index].modulation;

    ++index;

    if (index > _prefferedChannelResult.count() - 1) {
        index = 0;
    }

    return LC70_OK;
}

LC70_ERR_T Stub::channelScanResult(int16_t measuringIndex, LC70_MEAS_MULTI_RESULT_T *measuringResult, int total)
{
    static int additiononalIndex = 0;

    if (_multiMeasurementResult.count() == 0) {
        return LC70_ERR;
    }

    memset(measuringResult, 0, sizeof(LC70_MEAS_MULTI_RESULT_T));
    measuringResult->ucLock = _multiMeasurementResult[additiononalIndex + measuringIndex].locked;   // ロック状態
    measuringResult->usStat =
        _multiMeasurementResult[additiononalIndex + measuringIndex].status;                         // 収集ステータス
    measuringResult->ucReadIdx =
        _multiMeasurementResult[additiononalIndex +
                                                  measuringIndex].readIndex;                        // 読出データインデックス
    measuringResult->ucCurrIdx =
        _multiMeasurementResult[additiononalIndex +
                                                  measuringIndex].currentMeasuringIndex;            // カレント測定中インデックス
    measuringResult->ulLevel = _multiMeasurementResult[additiononalIndex + measuringIndex].level;   // LEVEL
    measuringResult->ulCnr = _multiMeasurementResult[additiononalIndex + measuringIndex].cnr;       // CNR
    measuringResult->ulBerErrBitNum = _multiMeasurementResult[additiononalIndex +
                                                        measuringIndex].bitErrorCount;    // エラー数
    measuringResult->ulBerTotalBitNum = _multiMeasurementResult[additiononalIndex +
                                                          measuringIndex].totalCount;     // 総データ数

    ++_scanIndex;

    if (_scanIndex == total) {
        measuringResult->ucCurrIdx = 0xFF;
        _scanIndex = 0;
    }

    return LC70_OK;
}

void Stub::buttonStatus(int status)
{
    static LC70_KEY_STAT_T buttons;

    for (int i = 0; i < LC70_KEY_ID_MAX; ++i) {
        // ボタン入力状態をクリアー
        buttons.stat[i] = LC70_KEY_OFF;
    }

    for (int i = 0; i < LC70_KEY_ID_MAX; ++i) {
        if ((status & (1 << i)) == LC70_KEY_ON) {
            // 受信したボタン入力状態を反映
            buttons.stat[i] = LC70_KEY_ON;
        }
    }

    emit keyStatus(&buttons);
}

void Stub::deviceStatus(int battery, int powerSupply, int usb, int sd, int azimuth)
{
    static LC70_DEV_STAT_T devices;

    devices.ucBatt = static_cast<uint8_t>(battery);
    devices.usbc = static_cast<LC70_USBC_T>(powerSupply);
    devices.usba = static_cast<LC70_STRAGE_T>(usb);
    devices.sd = static_cast<LC70_STRAGE_T>(sd);
    devices.usAngle = azimuth;

    emit machineStatus(&devices);
}

void Stub::failureInformation(int code, int status, int action, int wait, QString information)
{
    if (code == 0) {
        return;
    }
    static LC70_FAIL_INFO_T failureInformation;
    memset(&failureInformation, 0, sizeof(LC70_FAIL_INFO_T));
    failureInformation.usCode = code;
    failureInformation.ucStat = status;
    failureInformation.ucCtl = action;
    failureInformation.usWait = wait;
    for (int i = 0; i < information.length(); ++i) {
        if (i == LC70_FAIL_ADD_INFO_NUM) {
            break;
        }
        failureInformation.ucAddInfo[i] = information.toLocal8Bit().constData()[i];
    }

    emit failure(&failureInformation);
}

void Stub::resetScanning()
{
    _scanIndex = 0;
}

QList<ChannelDefinitionItem> Stub::readChannelDefinition(const ChannelDefinitionFile &definition)
{
    QList<ChannelDefinitionItem> channelDefinitionList;
    QFile file(_channelDefinitionFilePath + QString("/") + definition.FileName);

    if (!file.open(QIODevice::ReadOnly)) {
        qDebug() << file.errorString();
        return channelDefinitionList;
    }

    QTextStream stream(&file);
    int count = 0;
    void *data = nullptr;
    LC70_CH_SET_UHF_T uhf;                          // 地デジ
    LC70_CH_SET_BS_T bs;                            // BS
    LC70_CH_SET_CS_T cs;                            // CS
    LC70_CH_SET_SCCB_T sccb;                        // SCCB
    LC70_CH_SET_SCCC_T sccc;                        // SCCC
    LC70_CH_SET_JSAT3A_T jsat3a;                    // JSAT3A
    LC70_CH_SET_JSAT4B_T jsat4b;                    // JSAT4B
    LC70_CH_SET_CATV_ALL_T catvAll;                 // CATV_ALL(ラッパー関数ライブラリ V2.00 対応)
    LC70_CH_SET_CATV_90_470MHZ_T catv90_470Mhz;     // CAT_90_470MHZ(ラッパー関数ライブラリ V2.00 対応)
    LC70_CH_SET_CATV_470_1002MHZ_T catv470_1002MHz; // CATV_470_1002MHZ(ラッパー関数ライブラリ V2.00 対応)
    LC70_CH_SET_SKYP_T skyp;                        // スカパープレミアム光
    LC70_CH_SET_PILOT_T pilot;                      // PILOT
    LC70_CH_SET_LTE_T lte;                          // LTE
    LC70_CH_SET_FM_T fm;                            // FM
    LC70_CH_SET_CATV_UP_STREAM_T catv_up_steam;     // CATV上り

    // CSV ファイルを読み込んで、ラッパー関数を使った場合と同じチャンネル定義の構造体に代入する
    while (!stream.atEnd()) {
        QString line = stream.readLine();
        QVector<QStringRef> values = line.splitRef(",");

        // 列の数と異なる行はスキップする
        if (values.count() != definition.Count) {
            continue;
        }

        // ヘッダー行はスキップする
        if (line.contains(QString(IGNORE_ELEMENT))) {
            continue;
        }

        bool ok = false;

        switch (definition.Id) {
        case LC70_FILE_ID_CH_SET_UHF: {     // 地デジ

            data = &uhf;

            uint32_t frequency = values[1].toInt(&ok);      // 中心周波数

            if (!ok) {
                break;
            }

            strncpy(uhf.chSetRow[count].cStrChNo, values[0].toString().toLocal8Bit().constData(), LC70_CH_SET_CH_STRLEN);
            uhf.chSetRow[count].ulCentFreq = frequency;
            ++count;

            break;
        }

        case LC70_FILE_ID_CH_SET_BS: {      // BS

            data = &bs;

            uint8_t polarization = values[0].toInt(&ok);    // 偏波

            if (!ok) {
                break;
            }

            uint32_t frequency = values[2].toInt(&ok);      // 中心周波数

            if (!ok) {
                break;
            }

            bs.chSetRow[count].ucWave = polarization;
            strncpy(bs.chSetRow[count].cStrChNo, values[1].toString().toLocal8Bit().constData(), LC70_CH_SET_CH_STRLEN);
            bs.chSetRow[count].ulCentFreq = frequency;
            ++count;

            break;
        }

        case LC70_FILE_ID_CH_SET_CS: { //CS
            data = &cs;

            uint8_t polarization = values[0].toInt(&ok);   // 偏波

            if (!ok) {
                break;
            }

            uint32_t frequency = values[2].toInt(&ok);      // 中心周波数

            if (!ok) {
                break;
            }

            cs.chSetRow[count].ucWave = polarization;
            strncpy(cs.chSetRow[count].cStrChNo, values[1].toString().toLocal8Bit().constData(), LC70_CH_SET_CH_STRLEN);
            cs.chSetRow[count].ulCentFreq = frequency;
            ++count;

            break;

        }

        case LC70_FILE_ID_CH_SET_SCCB: {      // SCC-B

            data = &sccb;

            uint8_t polarization = values[0].toInt(&ok);    // 偏波

            if (!ok) {
                break;
            }

            uint32_t frequency = values[2].toInt(&ok);      // 中心周波数

            if (!ok) {
                break;
            }

            sccb.chSetRow[count].ucWave = polarization;
            strncpy(sccb.chSetRow[count].cStrChNo, values[1].toString().toLocal8Bit().constData(), LC70_CH_SET_CH_STRLEN);
            sccb.chSetRow[count].ulCentFreq = frequency;
            ++count;

            break;
        }

        case LC70_FILE_ID_CH_SET_SCCC: {      // SCC-C

            data = &sccc;

            uint8_t polarization = values[0].toInt(&ok);    // 偏波

            if (!ok) {
                break;
            }

            uint32_t frequency = values[2].toInt(&ok);      // 中心周波数

            if (!ok) {
                break;
            }

            sccc.chSetRow[count].ucWave = polarization;
            strncpy(sccc.chSetRow[count].cStrChNo, values[1].toString().toLocal8Bit().constData(), LC70_CH_SET_CH_STRLEN);
            sccc.chSetRow[count].ulCentFreq = frequency;
            ++count;

            break;
        }

        case LC70_FILE_ID_CH_SET_JSAT3A: {      // JSAT-3A

            data = &jsat3a;

            uint8_t polarization = values[0].toInt(&ok);    // 偏波

            if (!ok) {
                break;
            }

            uint32_t frequency = values[2].toInt(&ok);      // 中心周波数

            if (!ok) {
                break;
            }

            jsat3a.chSetRow[count].ucWave = polarization;
            strncpy(jsat3a.chSetRow[count].cStrChNo, values[1].toString().toLocal8Bit().constData(), LC70_CH_SET_CH_STRLEN);
            jsat3a.chSetRow[count].ulCentFreq = frequency;
            ++count;

            break;
        }

        case LC70_FILE_ID_CH_SET_JSAT4B: {      // JSAT-4B

            data = &jsat4b;

            uint8_t polarization = values[0].toInt(&ok);    // 偏波

            if (!ok) {
                break;
            }

            uint32_t frequency = values[2].toInt(&ok);      // 中心周波数

            if (!ok) {
                break;
            }

            jsat4b.chSetRow[count].ucWave = polarization;
            strncpy(jsat4b.chSetRow[count].cStrChNo, values[1].toString().toLocal8Bit().constData(), LC70_CH_SET_CH_STRLEN);
            jsat4b.chSetRow[count].ulCentFreq = frequency;
            ++count;

            break;
        }

        // ラッパー関数ライブラリ V2.00 対応
        case LC70_FILE_ID_CH_SET_CATV_ALL: {      // CATV
            data = &catvAll;

            uint32_t frequency = values[1].toInt(&ok);      // 中心周波数

            if (!ok) {
                break;
            }

            uint32_t frequency2 = values[2].toInt(&ok);     // 中心周波数2

            if (!ok) {
                break;
            }


            strncpy(catvAll.chSetRow[count].cStrChNo, values[0].toString().toLocal8Bit().constData(), LC70_CH_SET_CH_STRLEN);
            catvAll.chSetRow[count].ulCentFreqQam = frequency;
            catvAll.chSetRow[count].ulCentFreqOfdm = frequency2;
            ++count;

            break;
        }

        // ラッパー関数ライブラリ V2.00 対応
        case LC70_FILE_ID_CH_SET_CATV_90_470MHZ: {      // CATV 90-470MHz
            data = &catv90_470Mhz;

            uint32_t frequency = values[1].toInt(&ok);      // 中心周波数

            if (!ok) {
                break;
            }

            uint32_t frequency2 = values[2].toInt(&ok);     // 中心周波数2

            if (!ok) {
                break;
            }


            strncpy(catv90_470Mhz.chSetRow[count].cStrChNo, values[0].toString().toLocal8Bit().constData(), LC70_CH_SET_CH_STRLEN);
            catv90_470Mhz.chSetRow[count].ulCentFreqQam = frequency;
            catv90_470Mhz.chSetRow[count].ulCentFreqOfdm = frequency2;
            ++count;

            break;
        }

        // ラッパー関数ライブラリ V2.00 対応
        case LC70_FILE_ID_CH_SET_CATV_470_1002MHZ: {      // CATV 470-1002MHz
            data = &catv470_1002MHz;

            uint32_t frequency = values[1].toInt(&ok);      // 中心周波数

            if (!ok) {
                break;
            }

            uint32_t frequency2 = values[2].toInt(&ok);     // 中心周波数2

            if (!ok) {
                break;
            }


            strncpy(catv470_1002MHz.chSetRow[count].cStrChNo, values[0].toString().toLocal8Bit().constData(),
                    LC70_CH_SET_CH_STRLEN);
            catv470_1002MHz.chSetRow[count].ulCentFreqQam = frequency;
            catv470_1002MHz.chSetRow[count].ulCentFreqOfdm = frequency2;
            ++count;

            break;
        }

        case LC70_FILE_ID_CH_SET_SKYP: {      // スカパー光
            data = &skyp;

            uint32_t frequency = values[1].toInt(&ok);      // 中心周波数

            if (!ok) {
                break;
            }


            strncpy(skyp.chSetRow[count].cStrChNo, values[0].toString().toLocal8Bit().constData(), LC70_CH_SET_CH_STRLEN);
            skyp.chSetRow[count].ulCentFreq = frequency;
            ++count;

            break;
        }


        case LC70_FILE_ID_CH_SET_PILOT: {      //  PILOT
            data = &pilot;

            uint32_t frequency = values[1].toInt(&ok);      // 中心周波数

            if (!ok) {
                break;
            }

            strncpy(pilot.chSetRow[count].cStrChNo, values[0].toString().toLocal8Bit().constData(), LC70_CH_SET_CH_STRLEN);
            pilot.chSetRow[count].ulCentFreq = frequency;
            ++count;

            break;
        }

        case LC70_FILE_ID_CH_SET_LTE: {      // LTE
            data = &lte;

            uint32_t frequency = values[1].toInt(&ok);      // 中心周波数

            if (!ok) {
                break;
            }

            uint32_t bandwidth = values[2].toInt(&ok);      // 帯域幅

            if (!ok) {
                break;
            }

            strncpy(lte.chSetRow[count].cStrChNo, values[0].toString().toLocal8Bit().constData(), LC70_CH_SET_CH_STRLEN);
            lte.chSetRow[count].ulCentFreq = frequency;
            lte.chSetRow[count].ulBandWide = bandwidth;
            ++count;

            break;
        }

        case LC70_FILE_ID_CH_SET_FM: {                      // FM
            data = &fm;

            uint32_t frequency = values[1].toInt(&ok);      // 中心周波数

            if (!ok) {
                break;
            }


            strncpy(fm.chSetRow[count].cStrChNo, values[0].toString().toLocal8Bit().constData(), LC70_CH_SET_CH_STRLEN);
            fm.chSetRow[count].ulCentFreq = frequency;
            ++count;

            break;
        }

        case LC70_FILE_ID_CH_SET_CATV_UP_STREAM : {          // CATV上り
            data = &catv_up_steam;

            uint32_t frequency = values[1].toInt(&ok);        // 中心周波数

            if(!ok) {
                break;
            }

            memset(fm.chSetRow[count].cStrChNo, 0, LC70_CH_SET_CH_STRLEN);
            for (int i = 0; i < LC70_CH_SET_CH_STRLEN; ++i) {
                fm.chSetRow[count].cStrChNo[i] = values[0].toString().toLocal8Bit().constData()[i];
            }
            //strncpy(fm.chSetRow[count].cStrChNo, values[0].toString().toLocal8Bit().constData(), LC70_CH_SET_CH_STRLEN);
            catv_up_steam.chSetRow[count].ulCentFreq = frequency;
            ++count;

            break;
        }
        default:
            break;
        }
    }

    // BAND ごとのチャンネル定義の構造体を ChannelDefinitionItem 構造体に変換する
    return ChannelDefinitionItemHelper::convertChannelDefinitionItem(data, definition.Id);
}

QList<QFileInfo> Stub::dataLoggerFileList(const QString &path)
{
    QDir directory(path);

    if (!directory.exists()) {
        return QList<QFileInfo>();
    }

    // ファイル一覧を取得
    QList<QFileInfo> fileList = QList<QFileInfo>();
    // 拡張子 csv のファイルを検索(サブフォルダも含む)
    QDirIterator it(path, QStringList() << "*.csv", QDir::Files, QDirIterator::Subdirectories);

    while (it.hasNext()) {
        if (it.next().isEmpty()) {
            continue;
        }

        fileList.append(it.fileInfo());
    }

    return fileList;
}
