/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef STUB_H
#define STUB_H

#include <QObject>
#include <QMap>
#include "constant.h"
#include <QtDBus/QDBusConnection>
#include "./dbus/levelcheckeradaptor.h"
#include "lc70dummy.h"
#include "channeldefinitionitem.h"
#include "helper/channeldefinitionhelper.h"
/**
 *  @brief チャンネルごとの ID と定義ファイル名の構造体
 */
struct ChannelDefinitionFile {
    uint32_t Id;        //!< ファイル ID
    QString FileName;   //!< チャンネル定義ファイル名
    int Count;          //!< CSV の項目数
    int band;           //!< BAND種別
};

/**
 * @brief スタブクラス(ダミーの測定データを使う API, ButtonSimulator を使う API の場合にこのクラスを使う)
 */
class Stub : public QObject
{
    Q_OBJECT
public:
    /**
     * @brief コンストラクター
     * @param parent 親オブジェクト
     */
    explicit Stub(QObject *parent = nullptr);

    /**
     * @brief デストラクター
     */
    ~Stub();

    /**
     * @brief スタブの初期化
     * @param singleMeasurementDataPath スタブ用シングル測定結果ファイルのパス
     * @param multiMeasurementDataPath スタブ用マルチ測定結果ファイルのパス
     * @param channelDefinitionFilePath チャンネルを定義した CSV ファイルを配置したパス
     * @return 処理の成否
     */
    bool initialize(const QString &singleMeasurementDataPath, const QString &multiMeasurementDataPath,
                    const QString &channelDefinitionFilePath);

    /**
     * @brief 測定結果取得対象の変更
     * @param acquisitionTarget 測定結果取得対象
     */
    void changeAcquisitionTarget(const AcquisitionTarget &acquisitionTarget);

    /**
     * @brief シングルチャンネル測定結果取得
     * @param constellationPoints コンスタレーションデータの読み出し点数(0～200)
     * @param measuringResult 測定結果
     * @result 処理結果
     */
    LC70_ERR_T singleMeasuringResult(int16_t constellationPoints,
                                     LC70_MEAS_SINGLE_RESULT_T *measuringResult);

    /**
     * @brief 遅延プロファイル測定結果取得
     * @param result 測定結果
     * @return 処理結果
     */
    LC70_ERR_T delayProfileMeasuring(LC70_MEAS_DLY_PRF_RESULT_T *result);

    /**
     * @brief マルチチャンネル測定結果取得
     * @param measuringIndex 測定インデックス(0〜119)
     * @param measuringResult 測定結果
     * @return 処理結果
     */
    LC70_ERR_T multiMeasuringResult(int16_t measuringIndex, LC70_MEAS_MULTI_RESULT_T *measuringResult);

    /**
     * @brief マルチチャンネル優先測定結果取得
     * @param measuringResult 測定結果
     * @result 処理結果
     */
    LC70_ERR_T multiMeasuringPrefferedChannelResult(LC70_MEAS_SINGLE_RESULT_T *measuringResult);

    /**
     * @brief データロガーチャンネルスキャン結果取得
     * @param measuringIndex 測定インデックス(0〜119)
     * @param measuringResult 測定結果
     * @param total 測定件数
     * @return
     */
    LC70_ERR_T channelScanResult(int16_t measuringIndex, LC70_MEAS_MULTI_RESULT_T *measuringResult, int total);

    /**
     * @brief スキャン処理のリセット
     */
    void resetScanning(void);

    /**
     * @brief チャンネルを定義した CSV ファイルを読み込んでチャンネル定義のリストを返す
     * @param definition ファイル ID とチャンネルを定義した CSV ファイルのパス
     * @note レベルチェッカー(LC70WS)_アプリケーションライブラリ仕様書 表 5 2 fileId データ型対応表 を参照
     * @return チャンネルを定義した CSV ファイルを読み込んでチャンネル定義のリスト
     */
    QList<ChannelDefinitionItem> readChannelDefinition(const ChannelDefinitionFile &definition);

    /**
     * @brief データロガーの出力ファイルの一覧を取得する
     * @param path データロガーの出力ファイルを配置しているパス
     * @return データロガーの出力ファイルの一覧
     */
    QList<QFileInfo> dataLoggerFileList(const QString &path);

signals:
    /**
     * @brief ボタン入力状態を通知
     * @param keyStatus ボタン入力状態
     */
    void keyStatus(LC70_KEY_STAT_T *keyStatus);

    /**
     * @brief デバイス状態を通知
     * @param deviceStatus デバイス状態
     */
    void machineStatus(LC70_DEV_STAT_T *deviceStatus);

    /**
     * @brief 故障情報を通知
     * @param failureInformation 故障情報
     */
    void failure(LC70_FAIL_INFO_T *failureInformation);

private:
    LevelcheckerAdaptor *_adaptor;          //!< ButtonSimulator と D-Bus で接続するためのアダプター
    QDBusConnection _connection;            //!< D-Bus 接続用クラス
    AcquisitionTarget _acquisitionTarget;   //!< 測定結果取得対象
    QString _channelDefinitionFilePath;     //!< チャンネルを定義した CSV ファイルを配置したパス

    /**
     * @brief シングルチャンネル測定結果を格納する構造体
     */
    struct MeasurementResult {
        float32_t level;                    //!< 測定入力レベル
        uint32_t bitErrorCount;             //!< ビット誤り数
        uint32_t totalCount;                //!< 総データ数
        float32_t mer;                      //!< 誤差変調比
        float32_t cnr;                      //!< CMR
        uint16_t system;                    //!< 放送方式
        uint8_t layer;                      //!< 64QAMのレイヤ
        uint32_t modulation;                //!< 変調方式
        uint8_t locked;                     //!< ロック状態 0: アンロック 1: ロック(ラッパー関数ライブラリ V2.00 対応)
        uint16_t status;                    //!< 収集ステータス

        MeasurementResult() : level(0.0f), bitErrorCount(0), totalCount(0), mer(0.0f), cnr(0.0f), system(0), layer(0),
            modulation(0), locked(0), status(0) {}
    };

    /**
     * @brief マルチチャンネル測定結果を格納する構造体
     */
    struct MultiMeasurementResult {
        uint8_t locked;                     //!< ロック状態 0: アンロック 1: ロック(ラッパー関数ライブラリ V2.00 対応)
        uint16_t status;                    //!< 収集ステータス
        uint8_t readIndex;                  //!< 読出データインデックス
        uint8_t currentMeasuringIndex;      //!< カレント測定中インデックス
        float32_t level;                    //!< 測定入力レベル
        float32_t mer;                      //!< 誤差変調比
        float32_t cnr;                      //!< CNR(ラッパー関数ライブラリ V2.00 対応)
        uint32_t bitErrorCount;             //!< ビット誤り数
        uint32_t totalCount;                //!< 総データ数
    };

    QList<MeasurementResult> _singleMeasurementResult;                  //!< シングルチャンネル測定データ
    QList<QVariant> _constellationResult;                               //!< コンスタレーションデータ
    QList<MultiMeasurementResult> _multiMeasurementResult;              //!< マルチチャンネル測定データ
    QList<MeasurementResult> _prefferedChannelResult;                   //!< マルチチャンネル優先測定データ
    QList<QList<QPointF> > _delayProfileResult;                         //!< 遅延プロファイルデータ
    int _scanIndex;                                                     //!< スキャンするインデックス

private slots:
    /**
     * @brief 各ボタンの状態を受信
     * @param status 各ボタン状態(ボタンの ID に対応するビットが ON になった値が来ることとする)
     */
    void buttonStatus(int status);

    /**
     * @brief 各デバイスの状態を受信
     * @param battery バッテリー
     * @param powerSupply 給電
     * @param usb USB
     * @param sd SD カード
     * @param azimuth 方位角
     */
    void deviceStatus(int battery, int powerSupply, int usb, int sd, int azimuth);

    /**
     * @brief 故障情報を受信
     * @param code 故障コード
     * @param status 故障状態
     * @param action 故障時の動作
     * @param wait 再表示までの待機時間
     * @param information 追加情報
     */
    void failureInformation(int code, int status, int action, int wait, QString information);
};

#endif // STUB_H
