/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef PARALLELTASK_H
#define PARALLELTASK_H

#include <QObject>
#include "threadbase.h"
#include "constant.h"
#include "systemlog.h"

/**
 * @brief 非同期処理を行うベースクラス
 * @note attach() で外部で生成されたスレッドに移動させて動作させる
 * @attention スレッドの生成、破棄は外部で行うものとする
 */
class ParallelTask : public QObject
{
    Q_OBJECT
public:
    /**
     * @brief コンストラクタ
     * @param systemLog システムロガー
     * @param parent 親オブジェクト
     */
    explicit ParallelTask(SystemLog &systemLog, QObject *parent = nullptr);

    /**
     * @brief デストラクタ
     */
    virtual ~ParallelTask();

    /**
     * @brief スレッドと連携
     * @param thread 連携するスレッド
     * @param name オブジェクト名（psコマンドで表示されるスレッド名）
     * @return 処理結果
     * @note スレッドを開始する
     */
    bool attach(ThreadBase *thread, const QString &name);

    /**
     * @brief スレッドとの連携を解除
     * @note 呼び出し前に attach() していることが前提条件
     * @return 常にtrue
     */
    bool detach(void);

    /**
     * @brief 開始、再開処理
     * @param priority スレッドの優先度
     * @return 処理結果(true:正常終了, false:異常終了)
     */
    virtual bool resume(void);

    /**
     * @brief 中断処理
     * @return 処理結果(true:正常終了, false:異常終了)
     */
    virtual bool suspend(void);

    /**
     * @brief スレッドの優先度を設定
     * @param priority スレッドの優先度
     */
    void setPriority(const QThread::Priority &priority = QThread::InheritPriority);

    /**
     * @brief 連携しているスレッドを返す
     * @note QObject::thread()と区別するため違う名前にする
     * @return 連携しているスレッド
     */
    ThreadBase *usingThread(void);

    /**
     * @brief 非同期処理の実行中か否かを返す
     * @return 非同期処理の実行中か否か
     * @note 単にスレッドが動いているだけという状態と識別するため
     */
    bool isWorking(void) const;

signals:
    /**
     * @brief タイムアウトの通知
     */
    void timeout(void);

protected:
    SystemLog &_systemLog;  //!< システムロガー
    bool _initialized;      //!< 初期化実行有無

protected slots:
    /**
     * @brief 非同期処理の実装(サブクラスでオーバーライドすること)
     */
    virtual void work(void) = 0;

private:
    ThreadBase *_thread;    //!< 処理を実行するスレッド
    bool _working;          //!< 実行中
};

#endif // PARALLELTASK_H
