/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 2.12
import QtQuick.Window 2.12
import QtQuick.Controls 2.12

import "header"
import "menubar"
import "popup"
import "measurement/single"
import "measurement/multi"

import jp.co.dxantenna.levelchecker.delayProfileLineSeriesModel 1.0
import jp.co.dxantenna.levelchecker.DisplayFileSystemModel 1.0
import "parts"

Window {
    id: mainFrame
    objectName: "MainFrame"
    visible: true
    width: 800
    height: 480
    flags: Qt.FramelessWindowHint

    // ViewModel と連携するデータ
    property var viewModel: ""                          // 開く画面の viewModel
    property string qmlFile: ""                         // 開く画面の qml ファイルのパス
    property string defaultFontName: defaultFont.name   // デフォルトのフォント
    property bool showActivityIndicator: false          // アップデート中のインジケーター表示
    property var  userSettingFileList: []
    signal loaded()
    signal menuBarSelected(int type, int rank, int index)
    property bool checkingopt: true

    readonly property QtObject screenSize : QtObject {
        property int headerHeight: 52       // ヘッダーエリアの高さ
    }

    // デフォルトフォントの設定
    FontLoader { id: defaultFont; objectName: "FontLoader"; name: "UD Shin Go Csm M"; source: "qrc:/resource/font/UDShinGoCsm-Medium.ttf"; }

    // 定数
    QtObject {
        id: constant
        readonly property int screenTypeSingleMeasurement: 0    // シングル測定画面
        readonly property int screenTypeMultiMeasurement: 1     // マルチ測定画面
        readonly property int screenTypeOther: 2                // その他の画面
    }

    // 背景画像
    Image {
        id: background
        objectName: "Background"
        anchors.fill: parent
        source: "qrc:/resource/png/back.png"
    }

    // メニューリスト
    MenuList {
        id: menulist
        anchors { top: parent.top; left: parent.left; leftMargin: 15 }
        visible: false
    }

    // 機器状態
    MachineStatus {
        id: machinestatus
        x: 385
        y: 7
    }

    // メニューバーの表示内容
    MenuBarModel {
        id: menuBarModel
        objectName: "MenuBarModel"
        checkopt: checkingopt
    }

    // データ管理表示用ファイルリスト
    DisplayFileSystemModel {
        id: displayFilesyStemModel
        objectName: "DisplayFileSystemModel"
    }

    // コンテンツ画面表示用ローダー
    Loader {
        id: loader
        objectName: "Loader"
        width: parent.width
        height: parent.height - screenSize.headerHeight
        anchors.left: parent.left
        anchors.bottom: parent.bottom
        onLoaded: {
            //console.log("load " + source)
        }
    }

    // シングル測定(画面遷移の高速化のためロードしない)
    SingleMeasurement {
        id: singleMeasurement
        objectName: "SingleMeasurement"
        width: parent.width
        height: parent.height - screenSize.headerHeight
        anchors.left: parent.left
        anchors.top: parent.top
        anchors.topMargin: screenSize.headerHeight
        visible: false
    }

    // マルチ測定(画面遷移の高速化のためロードしない)
    MultiMeasurement {
        id: multiMeasurement
        objectName: "MultiMeasurement"
        width: parent.width
        height: parent.height - screenSize.headerHeight
        anchors.left: parent.left
        anchors.top: parent.top
        anchors.topMargin: screenSize.headerHeight
        visible: false
    }

    // メニューバー表示時の背景
    Rectangle {
        id: menubarBackground
        anchors.fill: parent
        color: "#000000"
        opacity: 0.65
        visible: false
    }

    // メニューバー
    MenuBar {
        id: menuBar1
        x: 0
        y: screenSize.headerHeight
        rank: 0
        items: menuBarModel.selectModel(type, rank, index)
    }

    MenuBar {
        id: menuBar2
        x: 200
        y: screenSize.headerHeight
        rank: 1
        items: menuBarModel.selectModel(type, rank, index)
    }

    MenuBar {
        id: menuBar3
        x: 400
        y: screenSize.headerHeight
        rank: 2
        items: menuBarModel.selectModel(type, rank, index)
    }

    MenuBar {
        id: menuBar4
        x: 600
        y: screenSize.headerHeight
        rank: 3
        items: menuBarModel.selectModel(type, rank, index)
    }

    // エラー > ポップアップの上に表示するポップアップ > 通常のポップアップの順に上に表示する
    // 通常のポップアップ
    BasicPopup {
        id: basicPopup
        z:1 // 最下位

        // アップデート中
        ActivityIndicator {
            id: busyIndicator
            width: 102
            height: 102
            x: (parent.width - width) / 2
            y: (parent.height - height) / 2 + 50
            visible: mainFrame.showActivityIndicator
            onVisibleChanged: {
                if (visible) {
                    running = true
                } else {
                    running = false
                }
            }
        }
    }

    // ポップアップの上に表示するポップアップ
    StrongPopup {
        id: strongPopup
        z:2 // 中間
    }

    // ポップアップの上に表示するポップアップ
    LowBatteryPopup {
        id: lowBatteryPopup
        z:3 // エラーポップアップの次の優先度
    }

    // エラー表示用のポップアップ
    ErrorPopup {
        id: errorPopup
        z:4 // 最上位
    }

    // 画面遷移時に MainFrameViewModel から呼び出される
    function load(screenType)
    {
        viewModel = attributes.viewModel
        qmlFile = attributes.qmlPath
        if (screenType === constant.screenTypeSingleMeasurement) {
            // シングル測定画面
            multiMeasurement.visible = false
            singleMeasurement.visible = true
            loader.visible = false
            loader.source = ""
        } else if (screenType === constant.screenTypeMultiMeasurement) {
            // マルチ測定画面
            singleMeasurement.visible = false
            multiMeasurement.visible = true
            loader.visible = false
            loader.source = ""
        } else if (screenType === constant.screenTypeOther){
            // その他の画面
            loader.source = qmlFile
            loader.visible = true
            singleMeasurement.visible = false
            multiMeasurement.visible = false
        }
    }

    // ファイルリストを設定する
    function setFileList(pFileList)
    {
        menuBarModel.setMenuBarContents(pFileList)
    }

    // ファイルリストをクリアする
    function clearFileList()
    {
        menuBarModel.clearMenuBarContents()
    }

    // メニューバーを開く
    function openMenuBar(type, rank, index)
    {
        menubarBackground.visible = true

        menuBar1.type = type
        menuBar2.type = type
        menuBar3.type = type
        menuBar4.type = type

        // 階層ごとに開く
        switch (rank) {
        case 0:
            menuBar1.jumpToPositionAtIndex(index)
            menuBar1.active = true
            menuBar1.visible = true
            break;
        case 1:
            menuBar1.active =false
            menuBar1.pressed = false

            menuBar2.jumpToPositionAtIndex(index)
            menuBar2.active = true
            menuBar2.visible = true
            break;
        case 2:
            menuBar2.active =false
            menuBar2.pressed = false

            menuBar3.jumpToPositionAtIndex(index)
            menuBar3.active = true
            menuBar3.visible = true
            break;
        case 3:
            menuBar3.active =false
            menuBar3.pressed = false

            menuBar4.jumpToPositionAtIndex(index)
            menuBar4.active = true
            menuBar4.visible = true
            break;
        default:
            break;
        }
    }

    // メニューバーを閉じる
    function closeMenuBar(rank)
    {
        // 階層ごとに閉じる
        switch (rank) {
        case 0:
            menuBar1.focus = false
            menuBar1.visible = false
            menuBar1.pressed = false
            menuBar1.currentIndex = 0
            menubarBackground.visible = false
            break;
        case 1:
            menuBar2.focus = false
            menuBar2.visible = false
            menuBar2.pressed = false
            menuBar2.currentIndex = 0

            menuBar1.focus = true
            menuBar1.active = true
            menuBar1.pressed = false
            break;
        case 2:
            menuBar3.focus = false
            menuBar3.visible = false
            menuBar3.pressed = false
            menuBar3.currentIndex = 0

            menuBar2.focus = true
            menuBar2.active = true
            menuBar2.pressed = false
            break;
        case 3:
            menuBar4.focus = false
            menuBar4.visible = false
            menuBar4.pressed = false
            menuBar4.currentIndex = 0

            menuBar3.focus = true
            menuBar3.active = true
            menuBar3.pressed = false
            break;
        default:
            break;
        }
    }

    // 全てのメニューバーを閉じる
    function closeAllMenuBar()
    {
        menuBar4.focus = false
        menuBar4.active = false
        menuBar4.visible = false
        menuBar4.pressed = false
        menuBar4.currentIndex = 0
        menuBar3.focus = false
        menuBar3.active = false
        menuBar3.visible = false
        menuBar3.pressed = false
        menuBar3.currentIndex = 0
        menuBar2.focus = false
        menuBar2.active = false
        menuBar2.visible = false
        menuBar2.pressed = false
        menuBar2.currentIndex = 0
        menuBar1.focus = false
        menuBar1.active = false
        menuBar1.visible = false
        menuBar1.pressed = false
        menuBar1.currentIndex = 0

        menubarBackground.visible = false
    }

    // 階層を指定してメニューバーのカーソルを上に移動
    function moveUpCursor(rank)
    {
        switch (rank) {
        case 0:
            menuBar1.moveUp()
            break;
        case 1:
            menuBar2.moveUp()
            break;
        case 2:
            menuBar3.moveUp()
            break;
        case 3:
            menuBar4.moveUp()
            break;
        default:
            break;
        }
    }

    // 階層を指定してメニューバーのカーソルを下に移動
    function moveDownCursor(rank)
    {
        switch (rank) {
        case 0:
            menuBar1.moveDown()
            break;
        case 1:
            menuBar2.moveDown()
            break;
        case 2:
            menuBar3.moveDown()
            break;
        case 3:
            menuBar4.moveDown()
            break;
        default:
            break;
        }
    }

    function cursorPressed(rank)
    {
        switch (rank) {
        case 0:
            menuBar1.pressed = true
            menuBar2.pressed = false
            menuBar3.pressed = false
            menuBar4.pressed = false
            break;
        case 1:
            menuBar1.pressed = false
            menuBar2.pressed = true
            menuBar3.pressed = false
            menuBar4.pressed = false
            break;
        case 2:
            menuBar1.pressed = false
            menuBar2.pressed = false
            menuBar3.pressed = true
            menuBar4.pressed = false
            break;
        case 3:
            menuBar1.pressed = false
            menuBar2.pressed = false
            menuBar3.pressed = false
            menuBar4.pressed = true
            break;
        default:
            break;
        }
    }

    // ポップアップを開く
    function openPopup(layoutType, popupMessage, leftButtonCaption, rightButtonCaption, centerButtonCaption)
    {
        basicPopup.layoutType = layoutType
        basicPopup.message = popupMessage

        // その他
        basicPopup.leftButtonMessage = leftButtonCaption
        basicPopup.rightButtonMessage = rightButtonCaption
        basicPopup.centerButtonMessage = centerButtonCaption;
        basicPopup.open()
    }

    // ポップアップの上に表示するポップアップを開く
    function openStrongPopup(popupMessage, leftButtonCaption, rightButtonCaption)
    {
        strongPopup.message = popupMessage
        strongPopup.leftButtonMessage = leftButtonCaption
        strongPopup.rightButtonMessage = rightButtonCaption

        strongPopup.open()
    }

    // バッテリ低下状態中のポップアップを開く
    function openLowBatteryPopup()
    {
        lowBatteryPopup.isOverlay = basicPopup.opened
        lowBatteryPopup.open()
    }

    // エラー用のポップアップを表示
    function openErrorPopup(popupMessage, buttonCaption)
    {
        errorPopup.message = popupMessage
        errorPopup.buttonMessage = buttonCaption
        errorPopup.isOverlay = basicPopup.opened || lowBatteryPopup.opened

        errorPopup.open()
    }
}
