/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 2.12
import QtQuick.Controls 2.12
import QtQuick.Layouts 1.12
import "./"

// メニューバーの一階層を表示
Item {
    id: root
    width: (active) ? itemWidth + margin : itemWidth
    height: titleHeight
    visible: false

    readonly property int titleHeight: 30                    // タイトルの高さ
    readonly property int itemWidth: 185                     // 項目の幅
    readonly property int itemHeight: 60                     // 項目の高さ
    readonly property int maximumDisplayItems: 6             // 最大表示数(7 は見切れる)
    readonly property int maximumHeight: 425                 // 最大の高さ(画面全体の高さからヘッダー部分の高さを引いた値)一番下の白縁が消えるので5小さくしておく
    readonly property int iconWidth:62                       // 項目中のアイコンの幅
    readonly property int iconHeight: 48                     // 項目中のアイコンの高さ
    readonly property int margin: 0                          // アクティブ時のマージン

    property int type: 0                                     // メニューバーの種類(0: 測定 1: 未定)
    property int rank : 0                                    // 開いている階層
    property int index: 0                                    // 開いている階層が持つ表示内容のインデックス
    property bool active: false                              // 現在選択中の階層か否か
    property bool pressed: false                             // 押された状態か
    property alias items: listView.model                     // 表示項目
    property string noname: "未設定"
    onItemsChanged: {
        // 表示項目数が変わればメニューバーの高さも変える
        if (items.count <  maximumDisplayItems) {
            root.height = items.count * itemHeight + titleHeight
        } else {
            root.height = maximumHeight
        }
    }
    property alias currentIndex: listView.currentIndex

    // メニューバー全体
    Rectangle {
        id: floor
        anchors.fill: parent
        color: "transparent"

        // タイトルの背景
        Rectangle {
            id: titleBackground
            width: root.itemWidth
            height: root.titleHeight
            anchors { top: parent.top; left: parent.left; }
            color: "#31333A"
            // タイトル
            Label {
                id: title
                anchors.fill: parent
                color: "#E6E6E6"
                horizontalAlignment: Qt.AlignHCenter
                verticalAlignment: Qt.AlignVCenter
                text: items.title
                font.pixelSize: 14
                font.bold: true
                font.family: defaultFontName
            }
        }
        Item {
            id: listViewArea
            anchors {
                top: titleBackground.bottom;
                bottom: parent.bottom;
                left: parent.left;
            }
            clip: true

            width:  root.itemWidth
            height: root.height

            // リスト
            ListView {
                id: listView
                anchors {
                    top:    parent.top
                    bottom: parent.bottom
                    left:   parent.left
                    right:  parent.right
                }

                currentIndex: 0
                delegate: menuBarItem
                cacheBuffer: itemHeight * count
            }
        }
    }

    // 表示項目
    Component {
        id: menuBarItem
        Rectangle {
            id: rectangle
            width: itemWidth
            height: itemHeight
            color: (index != listView.currentIndex) ? "#41444D" : (pressed) ? "#80BFFF" : "#265B93"
            clip: false

            // カーソル画像
            Image {
                id: cursorImage
                width: sourceSize.width
                height: sourceSize.height
                source: cursor
                visible: (active && index === listView.currentIndex && !pressed)
                x: 0
                y: 0

                // アイコン
                Image {
                    id: iconImageInner
                    width: (icon === "") ? 0 : iconWidth
                    height: iconHeight
                    anchors {
                        top: parent.top
                        topMargin: 8
                        left: parent.left
                        leftMargin: 8
                    }
                    source: icon
                    onSourceChanged: {
                        //console.log("icon:" + icon)
                    }
                }
            }

            // アイコン
            Image {
                id: iconImage
                width: (icon === "") ? 0 : iconWidth
                height: iconHeight
                anchors {
                    top: parent.top
                    topMargin: 8
                    left: parent.left
                    leftMargin: 8
                }
                source: icon
                visible: !cursorImage.visible
            }

            // 項目名
            Label {
                id: label
                width: itemWidth - iconWidth
                height: itemHeight
                anchors.top: parent.top
                anchors.left: iconImage.right
                anchors.leftMargin: (icon !== "") ? 0 : 20
                font.pixelSize:16  //14
                color: (index === currentIndex) ? "#E6E6E6" : "#E6E6E6"
                horizontalAlignment: (icon !== "") ? Qt.AlignHCenter : Qt.AlignLeft
                verticalAlignment: Qt.AlignVCenter
                text: name
                font.family: defaultFontName
            }

            // ファイル番号
            Label {
                id: fileNoLabel
                width: 35
                height: itemHeight
                anchors {
                    top: parent.top
                    left: parent.left
                    leftMargin: 8
                }
                text: fileNo
                font.pixelSize: 15
                color: "#E6E6E6"
                horizontalAlignment: Qt.AlignLeft
                verticalAlignment: Qt.AlignVCenter
                font.family: defaultFontName
                visible: (type == 1) ? true : false
            }

            // ファイル名
            Label {
                id: fileNameLabel
                width: itemWidth - fileNoLabel.width
                height: itemHeight
                anchors.top: parent.top
                anchors.left: fileNoLabel.right
                anchors.leftMargin: 0
                font.pixelSize: 15
                color: "#E6E6E6"
                horizontalAlignment: Qt.AlignLeft
                verticalAlignment: Qt.AlignVCenter
                text:(fileName === "") ? noname : fileName
                font.family: defaultFontName
                visible: (type == 1) ? true : false
                elide: "ElideRight"
            }
        }
    }

    // カーソルを上に移動
    function moveUp()
    {
        pressed = false
        listView.currentIndex--
        if (listView.currentIndex < 0) {
            listView.currentIndex = listView.count - 1
        }
        mainFrame.menuBarSelected(type, rank, listView.currentIndex)
    }

    // カーソルを下に移動
    function moveDown()
    {
        pressed = false
        listView.currentIndex++
        if (listView.currentIndex > listView.count - 1) {
            listView.currentIndex = 0
        }
        mainFrame.menuBarSelected(type, rank, listView.currentIndex)
    }

    // カーソルを指定した位置に移動
    function jumpToPositionAtIndex(index)
    {
        root.index = index
        listView.positionViewAtIndex(root.currentIndex, ListView.Visible)
    }
}
