/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 2.12
import QtQuick.Window 2.12
import QtQuick.Controls 2.12

Item {
    id: root
    property string displayBand:""                       // 表示中のBAND
    property var frequency: ""                           // 周波数の値
    property string displayFrequencyUnit: "MHz"             // 表示周波数の単位
    property string modulation: ""                          // 変調方式
    property string modulationPrefix: ""                    // 変調方式の接頭句
    property string hierarchy: ""                           // 階層
    property string polarization: ""                        // 偏波
    property string localFrequency: ""                      // 局発周波数
    property var displayChannelDetail: []                // チャンネル詳細情報の表示可否リスト [周波数],[変調方式],[階層],[偏波],[局部発信周波数]の順で並んでいる。

    readonly property int areaHeight        : 20         // 各表示領域の高さ (1行表示の場合)
    readonly property int area2LineHeight   : 38         // 各表示領域の高さ (2行表示の場合)
    readonly property int areaWidth         : 175        // 各表示領域の幅
    readonly property int contentsFontSize  : 18         // 表示内容の文字の高さ
    readonly property string contentsColor  : "#E6E6E6"  // 表示内容の色
    readonly property int colonPoint        : 60         // コロンを表示するx軸

    property int displayModulationPrefixMode: 0             // 変調方式表示モード : 接頭句と変調方式をどのように表示するかを決める
    readonly property int modulationDisplayModeTogether : 0 // 変調方式表示モード : 接頭句 変調方式を１つのラベルで纏めて表示するモード
    readonly property int modulationDisplayModeSeparate : 1 // 変調方式表示モード : 接頭句 変調方式を個別のラベルで分けて表示する

    onModulationChanged: {
        if (root.displayModulationPrefixMode == root.modulationDisplayModeTogether) {
            // 変調方式表示モードが１つのラベルに纏めて表示する場合
            if (root.modulation == "" || root.modulationPrefix == "") {
                valueMode.text = root.modulation
            }else {
                valueMode.text = root.modulationPrefix + " " + root.modulation
            }
            valueModePrefix.text = ""
        } else if (root.displayModulationPrefixMode == root.modulationDisplayModeSeparate) {
            // ・変調方式の表示モードが個別のラベルに分けて表示する場合
            valueMode.text = root.modulation
        } else {
            // nothing to do
        }
    }

    onModulationPrefixChanged: {
        if (root.displayModulationPrefixMode == root.modulationDisplayModeTogether) {
            // 変調方式表示モードが１つのラベルに纏めて表示する場合
            if (root.modulation == "" || root.modulationPrefix == "") {
                valueMode.text = root.modulation
            }else {
                valueMode.text = root.modulationPrefix + " " + root.modulation
            }
            valueModePrefix.text = ""
        } else if (root.displayModulationPrefixMode == root.modulationDisplayModeSeparate) {
            // ・変調方式の表示モードが個別のラベルに分けて表示する場合
            valueModePrefix.text = root.modulationPrefix
        } else {
            // nothing to do
        }
    }

    // 部品の高さと幅を決定しておく
    height: 100
    width:  175

    // 周波数表示エリア
    Rectangle {
        id : showFrequencyArea
        width:  root.areaWidth
        height: visible ? root.areaHeight : 0
        color: "transparent"
        visible: (root.displayChannelDetail[0] === true) ? true : false

        // 周波数ラベル
        Label {
            id: labelFrequency
            width:  45
            anchors {
                top: parent.top
                bottom: parent.bottom
                left: parent.left
            }
            font.family: defaultFontName
            font.pixelSize: root.contentsFontSize
            verticalAlignment: Text.AlignVCenter
            color: root.contentsColor
            text: "周波数"
        }

        // コロン
        Label {
            id: frequencyColon
            width: 5
            height: labelFrequency.height
            x: 60
            anchors {
                top: parent.top
            }
            font.family: defaultFontName
            font.pixelSize: root.contentsFontSize
            verticalAlignment: Text.AlignVCenter
            color: root.contentsColor
            text: ":"
        }

        // 周波数値
        Label {
            id: valueFrequency
            width: 65
            height: labelFrequency.height
            anchors {
                top: labelFrequency.top
                left: frequencyColon.right
                leftMargin: 5
            }
            font.family: defaultFontName
            font.pixelSize: root.contentsFontSize
            horizontalAlignment: Text.AlignLeft
            verticalAlignment: Text.AlignVCenter
            color: root.contentsColor
            text: root.frequency
        }

        // 周波数単位
        Label {
            id: unitFrequency
            width: 30
            height: labelFrequency.height
            anchors {
                top: labelFrequency.top
                left: valueFrequency.right
            }
            font.family: defaultFontName
            font.pixelSize: root.contentsFontSize
            horizontalAlignment: Text.AlignLeft
            verticalAlignment: Text.AlignVCenter
            color: root.contentsColor
            text: root.displayFrequencyUnit
        }
    }

    // 変調方式表示エリア
    Rectangle {
        id: showModelArea
        width:  root.areaWidth
        height: visible ? valueModePrefix.visible ? root.area2LineHeight : root.areaHeight : 0
        color: "transparent"
        anchors.top: showFrequencyArea.bottom
        visible: (root.displayChannelDetail[1]=== true) ? true : false

        // 変調ラベル
        Label {
            id: labelMode
            width  : 58
            height : root.areaHeight
            anchors {
                top: parent.top
                left: parent.left
            }
            font.family: defaultFontName
            font.pixelSize: root.contentsFontSize
            verticalAlignment: Text.AlignVCenter
            color: root.contentsColor
            text: "変調方式"
        }

        // コロン
        Label {
            id: modeColon
            width: 5
            height: labelMode.height
            x: 60
            anchors {
                top: parent.top
            }
            font.family: defaultFontName
            font.pixelSize: root.contentsFontSize
            verticalAlignment: Text.AlignVCenter
            color: root.contentsColor
            text: ":"
        }

        // 変調方式 接頭句
        Label {
            id: valueModePrefix
            width: 86
            height: visible ? labelMode.height : 0
            anchors {
                top: labelMode.top
                left: modeColon.right
                leftMargin: 5
            }
            font.family: defaultFontName
            font.pixelSize: root.contentsFontSize
            horizontalAlignment: Text.AlignLeft
            verticalAlignment: Text.AlignVCenter
            color:root.contentsColor
            text: ""
            visible: (text != "" && valueMode.text !== "")
        }
        // 変調方式設定値
        Label {
            id: valueMode
            width: 86
            height: visible ? labelMode.height : 0
            anchors {
                top: valueModePrefix.bottom
                left: modeColon.right
                leftMargin: 5
            }
            font.family: defaultFontName
            font.pixelSize: root.contentsFontSize
            horizontalAlignment: Text.AlignLeft
            verticalAlignment: Text.AlignVCenter
            color: root.contentsColor
            text:  ""
            visible: (text != "")
        }
    }

    // 階層表示エリア
    Rectangle {
        id : showhierarchyArea
        width:  root.areaWidth
        height: visible ? root.areaHeight : 0
        color: "transparent"
        anchors.top: showModelArea.bottom
        visible: (root.displayChannelDetail[2] === true) ? true : false

        // 階層ラベル
        Label {
            id: labelhierarchy
            width: 45
            anchors {
                top: parent.top
                bottom: parent.bottom
                left: parent.left
            }
            font.family: defaultFontName
            font.pixelSize: contentsFontSize
            verticalAlignment: Text.AlignVCenter
            color: root.contentsColor
            text: "階層"
        }

        // コロン
        Label {
            id: hierarchyColon
            width: 5
            height: labelhierarchy.height
            x: 60
            anchors {
                top: parent.top
            }
            font.family: defaultFontName
            font.pixelSize: root.contentsFontSize
            verticalAlignment: Text.AlignVCenter
            color: root.contentsColor
            text: ":"
        }

        // 階層
        Label {
            id: valuehierarchy
            width:  40
            height: labelhierarchy.height
            anchors {
                top: labelhierarchy.top
                left: hierarchyColon.right
                leftMargin: 5
            }
            font.family: defaultFontName
            font.pixelSize: root.contentsFontSize
            horizontalAlignment: Text.AlignLeft
            verticalAlignment: Text.AlignVCenter
            color: root.contentsColor
            text:  root.hierarchy
        }
    }

    // 偏波表示エリア
    Rectangle {
        id : showPolarizationArea
        width: root.areaWidth
        height: visible ? root.areaHeight : 0
        color: "transparent"
        anchors.top: showhierarchyArea.bottom
        visible: (root.displayChannelDetail[3] === true) ? true : false

        // 偏波ラベル
        Label {
            id: labelPolarization
            width: 45
            anchors {
                top: parent.top
                bottom: parent.bottom
                left: parent.left
            }
            font.family: defaultFontName
            font.pixelSize: root.contentsFontSize
            verticalAlignment: Text.AlignVCenter
            color: root.contentsColor
            text: "偏波"
        }

        // コロン
        Label {
            id: polarizationColon
            width: 5
            height: labelPolarization.height
            x: 45
            anchors {
                top: parent.top
            }
            font.family: defaultFontName
            font.pixelSize: root.contentsFontSize
            verticalAlignment: Text.AlignVCenter
            color: root.contentsColor
            text: ":"
        }

        // 偏波表示ラベル
        Label {
            id: valuePolarization
            width:  40
            height: labelPolarization.height
            anchors {
                top: labelPolarization.top
                left: polarizationColon.right
                leftMargin:5
            }
            horizontalAlignment: Text.AlignLeft
            verticalAlignment: Text.AlignVCenter
            color: root.contentsColor
            font.family: defaultFontName
            font.pixelSize: root.contentsFontSize
            text: root.polarization
        }
    }

    // 局発周波数
    Rectangle {
        id: showLocalFrequencyArea
        width: root.areaWidth
        height: visible ? root.areaHeight : 0
        color: "transparent"
        anchors.top: showPolarizationArea.bottom
        visible: (root.displayChannelDetail[4] === true) ? true : false


        // 局発周波数ラベル
        Label {
            id: labelLocalFrequency
            width: 45
            anchors {
                top: parent.top
                bottom: parent.bottom
                left: parent.left
            }
            font.family: defaultFontName
            font.pixelSize: root.contentsFontSize
            verticalAlignment: Text.AlignVCenter
            color: root.contentsColor
            text: "局発"
        }

        // コロン
        Label {
            id: localFrequencyColon
            width: 5
            height: labelLocalFrequency.height
            x: 45
            anchors {
                top: parent.top
            }
            font.family: defaultFontName
            font.pixelSize: root.contentsFontSize
            verticalAlignment: Text.AlignVCenter
            color: root.contentsColor
            text: ":"
        }

        // 局発周波数値
        Label {
            id: valueLocalFrequency
            width: 78
            height: labelLocalFrequency.height
            anchors {
                top: labelLocalFrequency.top
                left: localFrequencyColon.right
                leftMargin: 5
            }
            font.family: defaultFontName
            font.pixelSize: root.contentsFontSize
            horizontalAlignment: Text.AlignLeft
            verticalAlignment: Text.AlignVCenter
            color: root.contentsColor
            text: root.localFrequency
        }

        // 局発周波数単位
        Label {
            id: unitLocalFrequency
            width: 30
            height: labelLocalFrequency.height
            anchors {
                top: labelLocalFrequency.top
                left: valueLocalFrequency.right
            }
            font.family: defaultFontName
            font.pixelSize: root.contentsFontSize
            horizontalAlignment: Text.AlignLeft
            verticalAlignment: Text.AlignVCenter
            color: root.contentsColor
            text: "GHz"
        }
    }
}

