/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 2.12
import QtQuick.Controls 2.12

Item {
    id: root
    width: compassImage.width
    height: width
    visible: isEnabledCompass

    property int azimuth: 0                     // 最新のコンパス角度
    property int currentAngle: 0                // 現在のコンパス角度
    property int beforeAngle: 0                 // アニメーション開始時点のコンパス角度
    property int afterAngle: 0                  // アニメーション終了時点のコンパス角度
    property int waitingAngle: 0                // 適用保留中のコンパス角度（アニメーション終了後に即適用）
    property bool hasWaitingData: false         // 適用保留中のデータがあるかどうか
    property bool isEnabledCompass: true        // 有効な測定結果を受信できているかどうか
    property bool isErrorOccurred: false        // 測定エラーが発生しているかどうか
    onAzimuthChanged: {
        if (moveCompass.running) {
            // アニメーション中なので完了後に反映する
            root.hasWaitingData = true
            root.waitingAngle = root.azimuth
        } else {
            // アニメーション中ではないので即反映する
            root.afterAngle = root.azimuth
            startAnimation()
        }
    }
    NumberAnimation {
        id: moveCompass
        target: root
        property: "currentAngle"
        from: root.beforeAngle       // 針の移動開始角度
        to: root.afterAngle          // 針の移動終了角度
        duration: 250
        onFinished: {
            // 移動後の針の位置を、次回の開始位置として記憶する
            root.beforeAngle = root.afterAngle

            // 回転方向を補正していた場合、角度をもとの値に戻しておく
            if (root.beforeAngle < 0) {
                root.beforeAngle = root.beforeAngle + 360
            } else if (root.beforeAngle > 360) {
                root.beforeAngle = root.beforeAngle - 360
            } else {
                // nothing to do
            }

            if (root.hasWaitingData) {
                // 針移動を保留していた場合は反映する
                root.afterAngle = root.waitingAngle
                root.hasWaitingData = false
                startAnimation()
            }
        }
    }
    function startAnimation()
    {
        if (root.afterAngle > 360) {
            // 入力値が異常である場合は表示に反映しない
            return
        }

        // 現在角度から最新角度に変更する際、真北近辺で 0 から 360 への変化がおこる
        // これを単純に指定すると、コンパスが1回転してしまう
        // 移動量が少ない方向への回転にできるよう、正転・逆転を考える
        if (Math.abs(root.afterAngle - root.beforeAngle) > 180 ) {
            // 半円以上の移動をしようとしているため、回転方向を逆転させる
            if (root.beforeAngle < root.afterAngle) {
                root.afterAngle = -(360 - root.afterAngle)
            } else {
                root.afterAngle = 360 + root.afterAngle
            }
        }

        moveCompass.start()
    }

    Image {
        id: directionArrow
        y: -28
        anchors.horizontalCenterOffset: 0
        anchors {
            bottom: compassImage.top
            bottomMargin: -213
            horizontalCenter: compassImage.horizontalCenter
        }
        source: "qrc:/resource/png/compass_frame.png"
        visible: isEnabledCompass && !isErrorOccurred
    }

    Item {
        id: compassImage
        // コンパス画像の左右の長さが異なっており、上部に方位角を示す矢印がついている
        // 回転中心を整えるため、左右に余白を加える。
        // 上部に円直径 251px に加え 23pxの余白（矢印などの領域）があるため、
        // 左右に加える余白は、既にある 2px の余白を引いた 21px * 2(左右) = 42px
        // 円直径が 251px なので 251 + 42 = 293px
        width: 293
        // 画像の中心点を整えるために補正しているため、縦幅は横幅と同じになる
        height: width
        anchors {
            bottom: parent.bottom
            horizontalCenter: parent.horizontalCenter
        }

        Image {
            id: circleImage
            anchors.horizontalCenterOffset: 0
            anchors.topMargin: 13
            anchors {
                top: parent.top
                horizontalCenter: parent.horizontalCenter
            }

            source: "qrc:/resource/png/compass.png"
        }

        transformOrigin: Item.Center

        rotation: -root.currentAngle
        Label {
            id: northLabel
            anchors {
                top: compassImage.top
                topMargin: 45
                horizontalCenter: compassImage.horizontalCenter
            }
            width: 30
            height: 30
            text: "北"
            font.pixelSize: 22
            color: "white"
            horizontalAlignment: Text.AlignHCenter
            verticalAlignment: Text.AlignVCenter
            rotation: -compassImage.rotation
        }
        Label {
            id: westLabel
            anchors {
                left: compassImage.left
                leftMargin: northLabel.anchors.topMargin
                verticalCenter: compassImage.verticalCenter
            }
            width: northLabel.width
            height: northLabel.height
            text: "西"
            font.pixelSize: northLabel.font.pixelSize
            color: northLabel.color
            horizontalAlignment: northLabel.horizontalAlignment
            verticalAlignment: northLabel.verticalAlignment
            rotation: northLabel.rotation
        }
        Label {
            id: southLabel
            anchors {
                bottom: compassImage.bottom
                bottomMargin: northLabel.anchors.topMargin
                horizontalCenter: compassImage.horizontalCenter
            }
            width: northLabel.width
            height: northLabel.height
            text: "南"
            font.pixelSize: northLabel.font.pixelSize
            color: northLabel.color
            horizontalAlignment: northLabel.horizontalAlignment
            verticalAlignment: northLabel.verticalAlignment
            rotation: northLabel.rotation
        }
        Label {
            id: eastLabel
            anchors {
                right: compassImage.right
                rightMargin: northLabel.anchors.topMargin
                verticalCenter: compassImage.verticalCenter
            }
            width: northLabel.width
            height: northLabel.height
            text: "東"
            font.pixelSize: northLabel.font.pixelSize
            color: northLabel.color
            horizontalAlignment: northLabel.horizontalAlignment
            verticalAlignment: northLabel.verticalAlignment
            rotation: northLabel.rotation
        }

        Label {
            id: degrees0
            anchors {
                top: compassImage.top
                topMargin: -10
                horizontalCenter: compassImage.horizontalCenter
            }
            width: degrees30.width
            height: degrees30.height
            text: "0"
            font.pixelSize: degrees30.font.pixelSize
            color: degrees30.color
            horizontalAlignment: degrees30.horizontalAlignment
            verticalAlignment: degrees30.verticalAlignment
            rotation: degrees30.rotation
        }
        Label {
            id: degrees30
            anchors {
                top: compassImage.top
                topMargin: 10
                right: compassImage.right
                rightMargin: 35
            }
            width: 80
            height: 20
            text: "30"
            font.pixelSize: 13
            color: northLabel.color
            horizontalAlignment: Text.AlignHCenter
            verticalAlignment: Text.AlignVCenter
            rotation: -compassImage.rotation
        }
        Label {
            id: degrees60
            anchors {
                top: compassImage.top
                topMargin: 60
                right: compassImage.right
                rightMargin: -20
            }
            width: degrees30.width
            height: degrees30.height
            text: "60"
            font.pixelSize: degrees30.font.pixelSize
            color: degrees30.color
            horizontalAlignment: degrees30.horizontalAlignment
            verticalAlignment: degrees30.verticalAlignment
            rotation: degrees30.rotation
        }
        Label {
            id: degrees90
            anchors {
                top: compassImage.top
                topMargin: 138
                right: compassImage.right
                rightMargin: -40
            }
            width: degrees30.width
            height: degrees30.height
            text: "90"
            font.pixelSize: degrees30.font.pixelSize
            color: degrees30.color
            horizontalAlignment: degrees30.horizontalAlignment
            verticalAlignment: degrees30.verticalAlignment
            rotation: degrees30.rotation
        }
        Label {
            id: degrees120
            anchors {
                top: compassImage.top
                topMargin: 210
                right: compassImage.right
                rightMargin: degrees60.anchors.rightMargin
            }
            width: degrees30.width
            height: degrees30.height
            text: "120"
            font.pixelSize: degrees30.font.pixelSize
            color: degrees30.color
            horizontalAlignment: degrees30.horizontalAlignment
            verticalAlignment: degrees30.verticalAlignment
            rotation: degrees30.rotation
        }
        Label {
            id: degrees150
            anchors {
                top: compassImage.top
                topMargin: 265
                right: compassImage.right
                rightMargin: degrees30.anchors.rightMargin
            }
            width: degrees30.width
            height: degrees30.height
            text: "150"
            font.pixelSize: degrees30.font.pixelSize
            color: degrees30.color
            horizontalAlignment: degrees30.horizontalAlignment
            verticalAlignment: degrees30.verticalAlignment
            rotation: degrees30.rotation
        }
        Label {
            id: degrees180
            anchors {
                top: compassImage.bottom
                topMargin: -5
                horizontalCenter: compassImage.horizontalCenter
            }
            width: degrees30.width
            height: degrees30.height
            text: "180"
            font.pixelSize: degrees30.font.pixelSize
            color: degrees30.color
            horizontalAlignment: degrees30.horizontalAlignment
            verticalAlignment: degrees30.verticalAlignment
            rotation: degrees30.rotation
        }
        Label {
            id: degrees210
            anchors {
                top: compassImage.top
                topMargin: degrees150.anchors.topMargin
                left: compassImage.left
                leftMargin: degrees150.anchors.rightMargin
            }
            width: degrees30.width
            height: degrees30.height
            text: "210"
            font.pixelSize: degrees30.font.pixelSize
            color: degrees30.color
            horizontalAlignment: degrees30.horizontalAlignment
            verticalAlignment: degrees30.verticalAlignment
            rotation: degrees30.rotation
        }
        Label {
            id: degrees240
            anchors {
                top: compassImage.top
                topMargin: degrees120.anchors.topMargin
                left: compassImage.left
                leftMargin: degrees120.anchors.rightMargin
            }
            width: degrees30.width
            height: degrees30.height
            text: "240"
            font.pixelSize: degrees30.font.pixelSize
            color: degrees30.color
            horizontalAlignment: degrees30.horizontalAlignment
            verticalAlignment: degrees30.verticalAlignment
            rotation: degrees30.rotation
        }
        Label {
            id: degrees270
            anchors {
                top: compassImage.top
                topMargin: degrees90.anchors.topMargin
                left: compassImage.left
                leftMargin: degrees90.anchors.rightMargin -3
            }
            width: degrees30.width
            height: degrees30.height
            text: "270"
            font.pixelSize: degrees30.font.pixelSize
            color: degrees30.color
            horizontalAlignment: degrees30.horizontalAlignment
            verticalAlignment: degrees30.verticalAlignment
            rotation: degrees30.rotation
        }
        Label {
            id: degrees300
            anchors {
                top: compassImage.top
                topMargin: degrees60.anchors.topMargin
                left: compassImage.left
                leftMargin: degrees60.anchors.rightMargin
            }
            width: degrees30.width
            height: degrees30.height
            text: "300"
            font.pixelSize: degrees30.font.pixelSize
            color: degrees30.color
            horizontalAlignment: degrees30.horizontalAlignment
            verticalAlignment: degrees30.verticalAlignment
            rotation: degrees30.rotation
        }
        Label {
            id: degrees330
            anchors {
                top: compassImage.top
                topMargin: degrees30.anchors.topMargin
                left: compassImage.left
                leftMargin: degrees30.anchors.rightMargin
            }
            width: degrees30.width
            height: degrees30.height
            text: "330"
            font.pixelSize: degrees30.font.pixelSize
            color: degrees30.color
            horizontalAlignment: degrees30.horizontalAlignment
            verticalAlignment: degrees30.verticalAlignment
            rotation: degrees30.rotation
        }
    }

    Rectangle {
        width: 210
        height: 32
        visible: isErrorOccurred
        anchors {
            horizontalCenter: root.horizontalCenter
            verticalCenter: root.verticalCenter
        }
        radius: 6
        color: "#41444d"
        border.color: "#333333"

        Label {
            text: "エラーが発生しています。"
            color: "white"
            font.pixelSize: 16
            anchors {
                horizontalCenter: parent.horizontalCenter
                verticalCenter: parent.verticalCenter
            }
            horizontalAlignment: Text.AlignLeft
        }
    }

}
