/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include "dataloggermeasurementlistviewmodel.h"
#include <QProcess>
#include <QElapsedTimer>
#include "mainframeviewmodel.h"

static constexpr int BUTTON_COLUMN_NUMBER = 1;  //!< グラフ、リストの列
static constexpr int BUTTON_ROW_NUMBER = 2;     //!< グラフ、リストの行
static constexpr int LIST_ROWS = 10;             //!< リストの行数
static constexpr int LIST_MAX = 99999;          //!< リストの最大行数

#if 1
static constexpr int SUBSLEEP_INTERVAL = 15*60*1000;    //サブCPUをスリープさせる _intervalTimeしきい値　15分
#endif

static constexpr const char *CSV_FILE_EXTENSION = ".csv";       //!< 測定データファイルの拡張子(CSV)

static constexpr int LTE_BAND_WIDTH = 10000;                        // LTEの帯域幅
static constexpr int CATV_ALL_CENTER_FREQUENCY_ISDBT_SUFFIX = 143;  // CATVの周波数末尾、ISDBTの特徴
static constexpr int CATV_ALL_CENTER_FREQUENCY_MOD = 1000;          // CATVの周波数末尾を取り出す際に周波数を割る値
                                                                    // static_cast<uint32_t>(pow(10, QString::number(CATV_ALL_CENTER_FREQUENCY_ISDBT_SUFFIX).length() -1))

DataLoggerMeasurementListViewModel::DataLoggerMeasurementListViewModel(ScreenId screenId, LevelCheckerModel *&model,
        QObject *parent)
    : AbstractViewModel(screenId, model, parent)
    , _list(nullptr)
    , _channelList()
    , _currentChannelIndex(0)
    , _condition()
    , _preferredCondition()
    , _previousPreferredCondition()
    , _measureResultModel(nullptr)
    , _multiMeasureResult()
    , _isFileOpen(false)
    , _totalPage(1)
    , _currentPage(1)
    , _levelThreshold(0.0f)
    , _userSettings()
    , _isChannelChanging(false)
    , _thresholdSet(false)
    , _broadCastSystem(LC70_BD_SYS_NONE)
    , _resultListNum(0)
    , _interval(INTERVAL_0)
    , _intervalTime(0)
    , _cycle(0)
    , _receiveCount(0)
    , _isSetEnddatetime(false)
    , _end_yyyy(0)
    , _end_mm(0)
    , _end_dd(0)
    , _end_HH(0)
    , _end_MM(0)
    , _measuringCondition(0)
    , _cnrMerSetting()
    , _isTopOfFile(true)
    , _outPutFileName("")
    , _isUsbRemove(false)
    , _isLowBattery(false)
    , _isWaitRestart(false)
    , _suspended(true)
    , _failureCode(0)
    , _actionInFailure(LC70_FAIL_CTL_NONE)
    , _intervalTimer(new QTimer())
    , _wakeUpTimer(new QTimer())
    , _hasPowerSupplyChannel(false)
    , _cancelPowerSupply(false)
    , _powerSupplied(false)
{
    setObjectName("dataLoggerMeasurementListViewModel");
    if (_intervalTimer) {
        _intervalTimer->setSingleShot(true);
    }
    if (_wakeUpTimer) {
        _wakeUpTimer->setSingleShot(true);
    }
    connect(_intervalTimer, SIGNAL(timeout()), this, SLOT(emitResume()));
    connect(_wakeUpTimer, SIGNAL(timeout()), this, SLOT(onTimeOutForWakingUp()));
}

DataLoggerMeasurementListViewModel::~DataLoggerMeasurementListViewModel()
{
    disconnect(_intervalTimer, SIGNAL(timeout()), this, SLOT(emitResume()));
    disconnect(_wakeUpTimer, SIGNAL(timeout()), this, SLOT(onTimeOutForWakingUp()));
}

void DataLoggerMeasurementListViewModel::resume()
{
    AbstractViewModel::resume();

    if (_channelList.count() == 0) {
        qDebug() << QString("No Channels");
        return;
    }

    int intervalValue = _takeOver["getInterval"].toInt();
    _interval = INTERVAL_0;
    _isSetEnddatetime = _takeOver["enddatetimeSet"].toBool();

    int cyclevalue = _takeOver["getCycle"].toInt();
    _cycle = 0;

    // 画面共有情報に対して削除要求を送信
    emit removeAttribute("enddatetimeSet");

    if (_isSetEnddatetime) {
        _end_yyyy = _takeOver["enddatetimeyyyy"].toInt();
        _end_mm = _takeOver["enddatetimemm"].toInt();
        _end_dd = _takeOver["enddatetimedd"].toInt();
        _end_HH = _takeOver["enddatetimeHH"].toInt();
        _end_MM = _takeOver["enddatetimeMM"].toInt();
    }

    if ((0 <= intervalValue) && (intervalValue <= 4)) {
        _interval = static_cast<IntervalValue>(intervalValue);
    }

    switch (_interval) {
    case INTERVAL_0:
        _intervalTime = 0;
        break;

    case INTERVAL_10:
        _intervalTime = 5 * 60 * 1000;
        break;

    case INTERVAL_20:
        _intervalTime = 15 * 60 * 1000;
        break;

    case INTERVAL_40:
        _intervalTime = 30 * 60 * 1000;
        break;

    case INTERVAL_60:
        _intervalTime = 60 * 60 * 1000;
        break;

    default:
        _intervalTime = 0;
        break;
    }

    if ( (0 <= cyclevalue) && (cyclevalue <=9) ) {
        _cycle = (cyclevalue + 1) * 100;
    }
    qDebug() << "_intervalTime:" << _intervalTime;
    qDebug() << "_cycle:" << _cycle;

    //  画面遷移時に実行する
    //  CN/MERの選択状態をqmlに送信する
    int valueMerCN = Correcter::selectCnrMer(static_cast<LC70_BAND_T>(_channelList[_currentChannelIndex].band), _cnrMerSetting);

    if (_view) {
        _view->setProperty("selectCnMer", valueMerCN);
    }

    // チャンネル変更
    changeChannel();

    if (_channelList.count() > 0) {
        emitResume();

#if 1
        // オートパワーオフ設定を無効化
        qDebug() << QString("オートパワーオフ設定を無効化");
        _model->disablePowerOffSetting();
#endif

    }
}

void DataLoggerMeasurementListViewModel::emitResume()
{
    // 時刻の初期化
    _model->initDateTime();

    if (_channelList.count() == 0) {
        // チャンネルリストがクリアされている場合は測定開始しない
        return;
    }

    // suspendMultiMeasuring で解除されたため、再設定
    _model->changeAcquisitionTarget(MULTI);
    _model->changeDisplayType(LC70_DISP_TYPE_LIST_LOGGER);

    // // チャンネル変更
    // changeChannel();

    // 給電するチャンネルを持っていたら給電中とする
    if (_hasPowerSupplyChannel) {
        // 給電中の判定
        MainFrameViewModel *view = dynamic_cast<MainFrameViewModel *>(_mainFrame);
        if (view) {
            _powerSupplied = (view->batteryLevel() > BATTERY_LEVEL2 || view->batteryStatus() >= BATTERY_STATUS_CHARGING) ? true : false;
            if (!_powerSupplied) {
                // ポップアップを表示
                emit requestToShowPopup(POPUP_DISABLE_POWER_SUPPLY);
            }
        }
    }

    // スレッド開始時、先頭の給電情報に給電アイコンを更新する
    if (_userSettings.count() > 0) {
        updatePowerSupplyIcon(0);
    }

    // Band アイコンの差し替え
    if (_channelList.count() > 0) {
        _model->replaceBandIcon(static_cast<LC70_BAND_T>(_channelList[0].band));
    }

#if 1
    // サブ CPU ウェイクアップ
    // サブに対するウェイクアップ要求が行われていなければ行う
    if (!_model->wakeUpRequested()) {
        qDebug() << QString("%1 [DATA LOGGER] Warning:SUB CPU WAKE UP 測定開始直前にサブCPUがウェイクアップしていない場合のファイルセーフ").arg(QDateTime::currentDateTime().toString("hh:mm:ss"));
        _model->setWakeUpRequested(true);
        _model->wakeUp(1);
    }
#endif
    // スレッドを開始
    emit resumeMultiMeasuring(&_condition,
                              &_preferredCondition,
                              LC70_DISP_TYPE_LIST_LOGGER,
                              _model->applicationSetting()->pollingIntervalMultiMeasurement());

    _suspended = false;

    // 測定開始したので開始待ちのフラグを落とす
    _isWaitRestart = false;
}

#if 1
void DataLoggerMeasurementListViewModel::setRestartTimer(bool isWakeup)
#else
void DataLoggerMeasurementListViewModel::setRestartTimer(void)
#endif
{
    if (_intervalTimer == nullptr || _wakeUpTimer == nullptr) {
        return;
    }

    // タイマー停止
    if (_intervalTimer->isActive()) {
        _intervalTimer->stop();
    }

    if (_wakeUpTimer->isActive()) {
        _wakeUpTimer->stop();
    }

#if 1
    // 測定開始間隔の設定
    _intervalTimer->setSingleShot(true);
    _intervalTimer->setInterval(_intervalTime);
    // ウェイアクアップは測定開始間隔の 1 分前
    if (isWakeup) {
        _wakeUpTimer->setSingleShot(true);
        _wakeUpTimer->setInterval(_intervalTime - (60 * 1000));
    }

    // タイマー開始
    _intervalTimer->start();
    if (isWakeup) {
        _wakeUpTimer->start();
    }
#else
    // 測定開始間隔の設定
    _intervalTimer->setSingleShot(true);
    _intervalTimer->setInterval(_intervalTime);
    // ウェイアクアップは測定開始間隔の 1 分前
    _wakeUpTimer->setSingleShot(true);
    _wakeUpTimer->setInterval(_intervalTime - (60 * 1000));

    // タイマー開始
    _intervalTimer->start();
    _wakeUpTimer->start();
#endif
}


void DataLoggerMeasurementListViewModel::emitSuspend(void)
{
    _suspended = true;
    _powerSupplied = false;

    // 給電アイコンをクリアー
    _model->replaceMenuList(MenuListIconIndex::POWER_SUPPLY_ICON, "");

    // スレッドを停止
    emit requestToSuspendMultiMeasuring();
}

void DataLoggerMeasurementListViewModel::suspend()
{
    // 給電状態をクリアする(lc70_setPowerSupplyMultiMeasMixBand() の呼び出しは不要）
    _model->removeMenuList(MenuListIconIndex::POWER_SUPPLY_ICON);
    _powerSupplied = false;

    // タイマーを停止
    if (_intervalTimer->isActive()) {
        _intervalTimer->stop();
    }

    if (_wakeUpTimer->isActive()) {
        _wakeUpTimer->stop();
    }

    // スレッドを停止
    emitSuspend();

    AbstractViewModel::suspend();

    // QML オブジェクトへのポインターをクリアー
    _list = nullptr;
    _measureResultModel = nullptr;

    // 優先測定条件のクリアー
    memset(&_preferredCondition, 0x00, sizeof(LC70_MEAS_SINGLE_T));
    memset(&_previousPreferredCondition, 0x00, sizeof(LC70_MEAS_SINGLE_T));

    // マルチ測定条件のクリアー
    for (int i = 0; i < _channelList.count(); ++i) {
        _condition.sRegNum = 0;
        memset(&_condition.single[i], 0x00, sizeof(_condition.single[i]));
    }

    // チャンネル情報のクリアー
    _channelList.clear();

    // ユーザー設定のクリアー
    _userSettings.clear();

    // 測定結果ファイルの測定項目をクリアー
    _measuringCondition = 0;

    if (_isFileOpen) {
        _outputfile.close();
        QProcess::execute(SYNC_COMMAND);
    }

    _isSetEnddatetime = false;

    _outputfile.setFileName("");
    _isFileOpen = false;
    _isUsbRemove = false;
    _isLowBattery = false;
    _hasPowerSupplyChannel = false;
    _cancelPowerSupply = false;
}

void DataLoggerMeasurementListViewModel::createConnection(QObject *qml, QObject *mainFrame)
{
    AbstractViewModel::createConnection(qml, mainFrame);

    connect(this, SIGNAL(requestSetTakeOver(QVariant, QVariant)), _view, SLOT(backup(QVariant, QVariant)));
    connect(this, SIGNAL(removeAttribute(QString)), _mainFrame, SLOT(onRemoveAttribute(QString)));
    connect(this, SIGNAL(resumeMultiMeasuring(LC70_MEAS_MULTI_T *, LC70_MEAS_SINGLE_T *, LC70_DISP_TYPE_T, uint32_t)),_model, SLOT(onResumeMultiMeasuring(LC70_MEAS_MULTI_T *, LC70_MEAS_SINGLE_T *, LC70_DISP_TYPE_T, uint32_t)));
    connect(_model, SIGNAL(multiMeasureResult(MultiResult, MeasureResult)),this, SLOT(onMultiMeasureResult(MultiResult, MeasureResult)));
    connect(_model, SIGNAL(notifyChannelChanging(bool)), this, SLOT(onNotifyChannelChanging(bool)));
    connect(this, SIGNAL(changeCondition(LC70_MEAS_SINGLE_T *)),_model, SIGNAL(preferredMeasuringChangeCondition(LC70_MEAS_SINGLE_T *)));
    connect(this, SIGNAL(scrollPage(QVariant)), _view, SLOT(onScrollPage(QVariant)));
    connect(_mainFrame, SIGNAL(notifyPowerSupplyError(uint16_t, uint8_t, QString)), this, SLOT(detectPowerSupplyError(uint16_t, uint8_t, QString)));
    connect(_model, SIGNAL(changeUsbInsertionRemoval(int)), this, SLOT(onChangeUsbInsertionRemoval(int)));
    connect(this, SIGNAL(requestToSuspendMultiMeasuring()), _model, SLOT(onSuspendMultiMeasuring()));
    connect(_model, SIGNAL(notifyStopMeasuring(AcquisitionTarget)), this, SLOT(onNotifyStopMeasuring(AcquisitionTarget)));

    // マルチ測定結果の取得に設定
    _model->changeAcquisitionTarget(MULTI);
}

void DataLoggerMeasurementListViewModel::activate()
{
    // 給電可・不可の判定
    MainFrameViewModel *view = dynamic_cast<MainFrameViewModel *>(_mainFrame);
    if (view) {
        _allowPowerSupply = (view->batteryLevel() > BATTERY_LEVEL2) ? true : false;
    }

    // 各画面
    if (_list == nullptr) {
        _list = _model->search("list");
    }
    // 測定結果リスト(MeasureResultModel の実体(objectName:measureResultModel)を QML から参照)
    _measureResultModel = dynamic_cast<MeasureResultModel *>(_model->search("measureResultModel"));
    _measureResultModel->setDataMax(LIST_ROWS);

    // QML ファイルのプロパティの変更(MultiMeasurement.qml を参照)
    // 画面ロード時は初期値に設定する
    if (_view) {
        _view->setProperty("level", INITIAL_LEVEL);
    }
    if (_view) {
        _view->setProperty("mer", INITIAL_MER);
    }
    if (_view) {
        _view->setProperty("ber", INITIAL_BER);
    }


    QVariant valueopt;
    int rfOptical = 0;
    _model->getValue(VALUE_RF_OPTICAL, valueopt);
    rfOptical = valueopt.toInt();
    if(rfOptical == 1){
        _model->replaceMenuList(MenuListIconIndex::PREAMP_ICON, "icon_preamp.png");
    }

    // CNR/MER 切替設定の取得
    _cnrMerSetting.clear();
    _model->getValue(VALUE_MER_CN, _cnrMerSetting);

    // ユーザー情報の反映
    if (!createUserSettingChannelList(_userSettings)) {
        return;
    }

    if (_channelList.count() == 0) {
        return;
    }

    if (_view) {
        // ユーザー設定名を反映
        _view->setProperty("userMode", _takeOver["selectedUserListFile"].toString());
    }

    _currentChannelIndex = 0;

    // 測定結果の初期化
    _multiMeasureResult.clear();

    // リストページ数設定
    _totalPage = 1;

    _currentPage = 1;
    if (_view) {
        _view->setProperty("totalPage", _totalPage);        // 総ページ
    }
    if (_view) {
        _view->setProperty("currentPage", _currentPage);    // 現在のページ
    }
    _model->changeDisplayType(LC70_DISP_TYPE_LIST_LOGGER);
}

void DataLoggerMeasurementListViewModel::deactivate()
{
    AbstractViewModel::deactivate();
    disconnect(this, SIGNAL(requestSetTakeOver(QVariant, QVariant)), _view, SLOT(backup(QVariant, QVariant)));
    disconnect(this, SIGNAL(removeAttribute(QString)), _mainFrame, SLOT(onRemoveAttribute(QString)));
    disconnect(this, SIGNAL(resumeMultiMeasuring(LC70_MEAS_MULTI_T *, LC70_MEAS_SINGLE_T *, LC70_DISP_TYPE_T, uint32_t)),_model, SLOT(onResumeMultiMeasuring(LC70_MEAS_MULTI_T *, LC70_MEAS_SINGLE_T *, LC70_DISP_TYPE_T, uint32_t)));
    disconnect(_model, SIGNAL(multiMeasureResult(MultiResult, MeasureResult)),this, SLOT(onMultiMeasureResult(MultiResult, MeasureResult)));
    disconnect(_model, SIGNAL(notifyChannelChanging(bool)), this, SLOT(onNotifyChannelChanging(bool)));
    disconnect(this, SIGNAL(changeCondition(LC70_MEAS_SINGLE_T *)),_model, SIGNAL(preferredMeasuringChangeCondition(LC70_MEAS_SINGLE_T *)));
    disconnect(this, SIGNAL(scrollPage(QVariant)), _view, SLOT(onScrollPage(QVariant)));
    disconnect(_mainFrame, SIGNAL(notifyPowerSupplyError(uint16_t, uint8_t, QString)), this, SLOT(detectPowerSupplyError(uint16_t, uint8_t, QString)));
    disconnect(_model, SIGNAL(changeUsbInsertionRemoval(int)), this, SLOT(onChangeUsbInsertionRemoval(int)));
    disconnect(this, SIGNAL(requestToSuspendMultiMeasuring()), _model, SLOT(onSuspendMultiMeasuring()));
    disconnect(_model, SIGNAL(notifyStopMeasuring(AcquisitionTarget)), this, SLOT(onNotifyStopMeasuring(AcquisitionTarget)));

    _totalPage = 1;
    _currentPage = 1;
    if (_view) {
        _view->setProperty("totalPage", _totalPage);
    }
    if (_view) {
        _view->setProperty("currentPage", _currentPage);
    }
    _resultListNum = 0;
#if 1
    // データロガー測定停止時にかをリセット　リセットしないと次の測定条件次第測定チャンネル数が少ないでデータロガー測定が行われなくなる。
    _receiveCount = 0;
#endif
    _outPutFileName = "";
    _isTopOfFile = true;
    _failureCode = 0;
    if (_isFileOpen) {
        fileClose();
    }
    clear();
}

void DataLoggerMeasurementListViewModel::readyToStandby()
{
    fileClose();

    // 測定データファイル名リストを更新
    _model->updateMeasurementFileList();
}

void DataLoggerMeasurementListViewModel::onKeyStatus(ButtonId id)
{
    Q_UNUSED(id);
    if (id == BUTTON_POWER) {
        if (_isFileOpen) {
            fileClose();
        }
        emitSuspend();
#if 0
// 給電デバッグ用
    } else if (id == BUTTON_SCAN) {
        dynamic_cast<MainFrameViewModel *>(_mainFrame)->test();
// 給電デバッグ用
#endif
    } else {
        // 停止中でなければポップアップで確認
        if (id == BUTTON_ESCAPE) {
            emit requestToShowPopup(POPUP_MEASUREMENT_CANCEL);

#if 0
//#if 1
            // サブ CPU ウェイクアップ
            qDebug() << QString("%1 [DATA LOGGER]SUB CPU WAKE UP").arg(QDateTime::currentDateTime().toString("hh:mm:ss"));

            // サブに対するウェイクアップ要求が行われていなければ行う
            if (!_model->wakeUpRequested()) {
                _model->setWakeUpRequested(true);
                _model->wakeUp(1);
            }
#else
            if (_suspended) {
            }
#endif
        }
#if 1
//        AbstractViewModel::onKeyStatus(id);
#else
        AbstractViewModel::onKeyStatus(id);
#endif
    }
}

void DataLoggerMeasurementListViewModel::onAccept(PopupId popupId, bool okOnly)
{
    Q_UNUSED(okOnly);

    AbstractViewModel::onAccept(popupId, okOnly);

    if (popupId == POPUP_POWER_SUPPLY_ERROR_USER_MEASURING) {
        // ブザーを止める
        _model->buzzer(LC70_BUZZ_OFF, LC70_BUZZ_MODE_STOP);

        // ファイルを閉じて測定データファイル名リストを更新
        fileClose();

        // 前の画面に戻る
        back();
    }

    if (popupId == POPUP_MEASUREMENT_CANCEL) {
        if (_isFileOpen) {
            fileClose();
        }

#if 1
        // オートパワーオフ設定を有効化
        qDebug() << QString("オートパワーオフ設定を有効化");
        _model->enablePowerOffSetting();
#endif
        emitSuspend();

#if 1
        // サブ CPU ウェイクアップ
        qDebug() << QString("%1 [DATA LOGGER]SUB CPU WAKE UP").arg(QDateTime::currentDateTime().toString("hh:mm:ss"));

        // サブに対するウェイクアップ要求が行われていなければ行う
        if (!_model->wakeUpRequested()) {
            _model->setWakeUpRequested(true);
            _model->wakeUp(1);
        }
#endif

        // 本画面を消去し、データロガー画面に遷移する
        //   HOME で中断しても
#ifdef NEW_USER_SETTING
        emit requestDirect(SCREEN_USER_DATALOGGER_SETTING, true);
#else
        emit requestDirect(SCREEN_USER_DATA_LOGGER_MENU, true);
#endif
        return;
    }

    if (popupId == POPUP_DATA_LOGGER_USB_REMOVE_ERROR) {
        emit back();
        return;
    }

    if (popupId == POPUP_FAILURE_NO_USB_CONFIRMATION) {
        emit back();
        return;
    }
}

void DataLoggerMeasurementListViewModel::onLowBatteryStatusChanged(bool isLowBattery)
{
    // バッテリー低下でなければ何もしない
    if (!isLowBattery) {
        _isLowBattery = false;
        return;
    }

    // バッテリー 10% 以下で既に給電を停止していたら何もしない
    if (_cancelPowerSupply) {
        return;
    }

    // バッテリー低下により MainFrame でシャットダウン処理が呼び出されている
    _isLowBattery = true;
    _cancelPowerSupply = true;
    // 給電していたら止める
    _model->setPowerSupplyForMultiBand(LC70_POWER_SUPPLY_OFF);
    _model->removeMenuList(MenuListIconIndex::POWER_SUPPLY_ICON);
    _powerSupplied = false;

    // 測定を止める
    emitSuspend();

    // 測定条件で給電 OFF に設定
    changeChannel();

    // 測定再開
    if (!_isWaitRestart) {
        emitResume();
    }
}

void DataLoggerMeasurementListViewModel::onMultiMeasureResult(MultiResult result, MeasureResult prefferedChannelResult)
{
    Q_UNUSED(prefferedChannelResult);

    if (_view == nullptr) {
        return;
    }

    // チャンネル変更中に届いた測定結果は表示しない
    if (_isChannelChanging) {
        return;
    }

    // チャンネル情報がなければ表示しない
    if (_channelList.count() < 1) {
        emitSuspend();
        return;
    }

    if (result.readIndex >= _channelList.count()) {
        return;
    }

    // 測定結果を取得していなければ無視する
    if (!result.received) {
        qDebug("%s Multi Result append skip. received[%s]", Q_FUNC_INFO, result.received ? "true" : "false" );
        return;
    }

    if (_view) {
        // 測定したBAND情報を画面に送信する
        _view->setProperty("band", Correcter::displayBandName(_channelList[result.readIndex].band));
    }

    // Band アイコンの差し替え
    _model->replaceBandIcon(static_cast<LC70_BAND_T>(_channelList[result.currentMeasuringIndex].band));

    // 参照データのインデックスを取得
    int targetIndex = result.readIndex + 1;

    if (targetIndex >= _userSettings.count()) {
        // ユーザ設定のデータ数を超えている場合最初の要素を参照する
        targetIndex = 0;
    }

    // 測定したUSER設定に対応した給電アイコンを表示する
    updatePowerSupplyIcon(targetIndex);

    if (_view) {
        // チャンネル詳細データの表示可否を通知
        _view->setProperty("displayChannelDetail", Correcter::getDisplayChannelDetail(_channelList[result.readIndex].band));
    }

    // 変調方式
    QString modulation = Correcter::displayModulation(result.result.Modulation);
    QString modulationPrefix = Correcter::modulationPrefix(result.result.BroadcastSystem, modulation);
    int displayModulationPrefixMode = Correcter::displayModulationPrefixMode(result.result.BroadcastSystem);

    if (_channelList[result.readIndex].band == LC70_BAND_PILOT || _channelList[result.readIndex].band == LC70_BAND_CATV_UP_STREAM) {
        if (_view) {
        // 測定BANDがPILOTの場合、変調方式にCWを固定で表示する
            _view->setProperty("displayModulationPrefixMode", DISPLAY_MODULATION_PREFIX_TOGETHER);
        }
        if (_view) {
            _view->setProperty("modulation", CW);
        }
        if (_view) {
            _view->setProperty("modulationPrefix", QString(""));
        }
    } else if (_channelList[result.readIndex].band == LC70_BAND_FM) {
        if (_view) {
            // 測定BANDがFMの場合、変調方式にFMを固定で表示する
            _view->setProperty("displayModulationPrefixMode", DISPLAY_MODULATION_PREFIX_TOGETHER);
        }
        if (_view) {
            _view->setProperty("modulation", FM);
        }
        if (_view) {
            _view->setProperty("modulationPrefix", QString(""));
        }
    } else {
        if (_view) {
            _view->setProperty("displayModulationPrefixMode", displayModulationPrefixMode);
        }
        if (_view) {
            _view->setProperty("modulation", modulation);
        }
        if (_view) {
            _view->setProperty("modulationPrefix", modulationPrefix);
        }
    }
    // 階層/偏波
    // 放送方式が ISDBT のみ有効
    if (result.result.BroadcastSystem == LC70_BD_SYS_ISDBT) {
        _view->setProperty("hierarchy", Correcter::displayHierarchy(result.result.Layer));
    } else {
        _view->setProperty("polarization", Correcter::displayPolarization(
                               Correcter::convertWaveToPolarization(_channelList[result.readIndex].polarization)));
    }

    // マルチチャンネル測定結果
    if (result.readIndex < _channelList.count()) {
        int index = result.readIndex;

        // 中心周波数と局発周波数から実周波数を求める
#if 1
        unsigned int frequency = Correcter::frequency(_userSettings[index].ulCenterFreq, _userSettings[index].ulLocalFreq);
#else
        unsigned int centerFrequency = Correcter::centerFrequency(_channelList[index], result.result.Modulation);
        unsigned int frequency = Correcter::frequency(centerFrequency, _userSettings[index].ulLocalFreq);
#endif
        int band = _channelList[index].band;
        QString merValue;
        QString berValue;

        int valueMerCN = Correcter::selectCnrMer(static_cast<LC70_BAND_T>(band), _cnrMerSetting);

        if (result.locked == LOCKED) {
            if (((result.status & LC70_DATA_MSK_CNR) == LC70_DATA_MSK_CNR && valueMerCN == LC70_MEAS_SET_SEL_CNR)
            || ((result.status & LC70_DATA_MSK_MER) == LC70_DATA_MSK_MER && valueMerCN == LC70_MEAS_SET_SEL_MER)) {
                merValue = Correcter::displayMer(band, _userSettings[result.readIndex].ucSpec, _userSettings[result.readIndex].ulModBit, result.result.Mer);
            } else {
                merValue = QString(MER_TIMEOUT_VALUE);
            }

            if ((result.status & LC70_DATA_MSK_BER) == LC70_DATA_MSK_BER){
                berValue = Correcter::displayBer(result.result.Ber);
            } else {
                berValue = QString(BER_TIMEOUT_VALUE);
            }
        } else {
            if(result.result.BroadcastSystem == LC70_BD_SYS_ANNEXB || result.result.BroadcastSystem == LC70_BD_SYS_ANNEXC || result.result.BroadcastSystem == LC70_BD_SYS_OPTICAST){
                merValue = QString("<20.0");
            }else{
                merValue = QString(MER_TIMEOUT_VALUE);
            }
            berValue = QString(BER_TIMEOUT_VALUE);
        }

        MeasureResultRow dispData;
        dispData.Timestamp = QDateTime::currentDateTime().toString("yy.MM.dd hh:mm:ss");
        dispData.Band = Correcter::displayBandName(band);

        // FM, PILOT, CATV 上りはチャンネル情報に周波数を表示する
        dispData.Channel = _channelList[index].channelNumber;

        QString polarize = Correcter::convertPolarizeNumToString(_channelList.at(index).polarization);
        BandTabIndex bandTab = Correcter::convertBandTabIndex(static_cast<LC70_BAND_T>(band), polarize);
        QString levelString;
        if (!Correcter::displayLevel(bandTab, result.result.Level, levelString)) {
            qDebug() << QString("displayLevel() failed");
        }

        dispData.Level = levelString;
        dispData.Mer = merValue;
        dispData.Ber = berValue;
        dispData.Frequency = Correcter::displayFrequency(band, frequency);

        // 測定結果一覧に追加表示
        _measureResultModel->append(dispData);
        if (_view) {
            _view->setProperty("currentListIndex", _measureResultModel->rowCount() - 1);
        }
        if (_view) {
            _view->setProperty("currentBaseIndex", _measureResultModel->rowCount() < LIST_ROWS ? 0 : _measureResultModel->rowCount() - LIST_ROWS);
        }

        if (_view) {
            // 左の項目表示に結果を反映
            _view->setProperty("channel", dispData.Channel);
        }
        if (_view) {
            if (_view) {
                _view->setProperty("level", levelString);
            }
        }
        if (_view) {
            _view->setProperty("frequency", dispData.Frequency);
        }

        if (dispData.Mer == "-1") {

            if (_view) {
                _view->setProperty("mer", MER_TIMEOUT_VALUE);
            }
        } else {
            if (_view) {
                QString mer = Correcter::displayMer(band, _userSettings[result.readIndex].ucSpec, _userSettings[result.readIndex].ulModBit, result.result.Mer);
                // displayMer だと 閾値未満は "<5" と返ってくるため　MER_TIMEOUT_VALUE に差し替える
                // toDouble() は ">35" が返ってきた場合に識別できないので使わない
                /*if (mer.contains("<")) {
                    mer = QString(MER_TIMEOUT_VALUE);
                }*/
                _view->setProperty("mer", mer);
            }
        }

        if (dispData.Ber == "-1") {
            if (_view) {
                _view->setProperty("ber", BER_TIMEOUT_VALUE);
            }
        } else {
            if (_view) {
                _view->setProperty("ber", Correcter::displayBer(result.result.Ber));
            }
        }

        if (_view) {
            _view->setProperty("isSatelliteBroadCasting",
                               Correcter::isSatelliteBroadCasting(_channelList[result.readIndex].band));
        }
        if (_view) {
            _view->setProperty("localFrequency", Correcter::displayLocalFrequency(_userSettings[result.readIndex].ulLocalFreq));
        }
        if (_view) {
            // 階層
            _view->setProperty("hierarchy", Correcter::displayHierarchy(result.result.Layer));
        }

        if (_view) {
            // 変調方式 表示するかどうかを画面に送信する
            _view->setProperty("isModulationVisible", Correcter::isModulationVisible(_channelList[result.readIndex].band));
        }

        // CN/MERの選択状態をqmlに送信する
        if (_view) {
            if ( (_channelList[result.readIndex].band == LC70_BAND_UHF) ||
                 (_channelList[result.readIndex].band == LC70_BAND_CATV_ALL) ||
                 (_channelList[result.readIndex].band == LC70_BAND_CATV_90_470MHZ) ||
                 (_channelList[result.readIndex].band == LC70_BAND_CATV_470_1002MHZ) ||
                 (_channelList[result.readIndex].band == LC70_BAND_SKYP) ){
                // MER
                _view->setProperty("selectCnMer", 1);
            } else {
                // CN
                _view->setProperty("selectCnMer", 0);
            }
        }

        // CSV 出力用
        MeasureResult measureResult;
        measureResult.Timestamp = dispData.Timestamp;
        measureResult.Band = QString::number(band);
        measureResult.Channel = _channelList[index].channelNumber;
        measureResult.Level = result.result.Level;
        measureResult.Frequency = frequency;
        measureResult.BroadcastSystem = result.result.BroadcastSystem;
        measureResult.Modulation = result.result.Modulation;
        if (Correcter::isSatelliteBroadCasting(band)) {
            measureResult.Layer = _channelList[index].polarization;
        }else {
            measureResult.Layer = result.result.Layer;
        }
        measureResult.LocalOscillator = _userSettings[result.readIndex].ulLocalFreq;

        // MER
        if (dispData.Mer == "-1") {
            measureResult.Mer = UNDEFINED_VALUE;
        } else {
            measureResult.Mer = result.result.Mer;
        }

        // BER
        if (dispData.Ber == "-1") {
            measureResult.Ber = UNDEFINED_VALUE;
        } else {
            measureResult.Ber = result.result.Ber;
        }

        _multiMeasureResult.append(measureResult);

        _resultListNum++;

        // リストの総ページ数設定
        _totalPage = _resultListNum / LIST_ROWS;

        if (_resultListNum % LIST_ROWS > 0) {
            _totalPage += 1;
        }

        //  最後のページ表示
        _currentPage = _totalPage;
        emit scrollPage(_currentPage);

        if (_view) {
            _view->setProperty("totalPage", _totalPage);
        }
        if (_view) {
            _view->setProperty("currentPage", _currentPage);
        }
    }

    fileoutput();

    // 測定間隔が設定されている場合は測定対象の全チャンネルの測定結果を受信したら停止
    if (_intervalTime > 0) {
        _receiveCount++;

#if 1
        if (_receiveCount >= _channelList.count()) {
#else
        if (_receiveCount == _channelList.count()) {
#endif
            _receiveCount = 0;
            _model->setWakeUpRequested(true);

            // file は open したままで.
            emitSuspend();
            // 測定の再開待ちを開始
            _isWaitRestart = true;
        }
    }

    if (_resultListNum >= (_channelList.count() * _cycle) ) {
        if (_isFileOpen) {
            // 指定サイクル数到達のため停止
            fileClose();
        }
#if 1
        // オートパワーオフ設定を有効化
        qDebug() << QString("オートパワーオフ設定を有効化");
        _model->enablePowerOffSetting();
#endif
        // 測定完了をTakeOverに設定
        emit requestSetTakeOver("completeDataLoggerMeasurement", true);

#ifdef NEW_USER_SETTING
        emit requestDirect(SCREEN_USER_DATALOGGER_SETTING, true);
#else
        emit requestDirect(SCREEN_USER_DATA_LOGGER_MENU, true);
#endif
    }
}

void DataLoggerMeasurementListViewModel::fileoutput()
{
    // fileが開いていない場合のみfileOpenを実施する
    if (!_isFileOpen){
        fileOpen();
    }

    if (_isFileOpen) {
        QTextStream outStream(&_outputfile);

        // BOMを生成する
        outStream.setGenerateByteOrderMark(true);

        if (_isTopOfFile) {
            QString fileName = _outputfile.fileName();

            QDateTime creationDateTime = QDateTime::fromString(QString("20") + _multiMeasureResult[0].Timestamp,
                                         "yyyy.MM.dd hh:mm:ss");
            QVector<QStringRef> words = fileName.splitRef("/");

            if (words.count() > 0) {
                fileName = words.at(words.count() - 1).toString();
            }

            // ヘッダー
            outStream << QString("日付") << ",";
            outStream << creationDateTime.toString("yyyy/MM/dd") << "\n";
            outStream << QString("時刻") << ",";
            outStream << creationDateTime.toString("hh:mm:ss") << "\n";
            outStream << QString("測定データ名") << ",";
            outStream << fileName << "\n";

            outStream << QString("測定項目") << ",";

            if ((_measuringCondition & LC70_DATA_MSK_LEVEL) == LC70_DATA_MSK_LEVEL) {
                outStream << QString("レベル");
            }

            // MER/CN の使い分け
            if ((_measuringCondition & LC70_DATA_MSK_CNR) == LC70_DATA_MSK_CNR) {
                outStream << QString("+CNR");
            } else if ((_measuringCondition & LC70_DATA_MSK_MER) == LC70_DATA_MSK_MER) {
                outStream << QString("+MER");
            }

            if ((_measuringCondition & LC70_DATA_MSK_BER) == LC70_DATA_MSK_BER) {
                outStream << QString("+BER");
            }

            outStream <<  "\n";

            outStream << QString("日時") << ",";
            outStream << QString("チャンネル名称") << ",";
            outStream << QString("BAND") << ",";
            outStream << QString("周波数(MHz)") << ",";
            outStream << QString("レベル(dBuV)") << ",";
            outStream << QString("MER/CN(dB)") << ",";
            outStream << QString("BER") << ",";
            outStream << QString("Layer/偏波") << ",";
            outStream << QString("局発周波数(GHz)") << ",";
            outStream << QString("放送規格") << ",";
            outStream << QString("変調方式") << "\n";
            _isTopOfFile = false;
        }

        for (int i = 0; i < _multiMeasureResult.count(); ++i) {

            outStream << _multiMeasureResult[i].Timestamp << ",";

            // FM, PILOT, CATV 上りはチャンネル情報に周波数を出力する
            outStream << _multiMeasureResult[i].Channel << ",";
            int band = _multiMeasureResult[i].Band.toInt();
            outStream << Correcter::displayBandName(band) << ",";                                                                 // 放送方式
            //outStream << static_cast<float32_t>(_multiMeasureResult[i].Frequency) / 1000 << ",";                                  // 周波数
            outStream << Correcter::displayFrequency(band,_multiMeasureResult[i].Frequency) << ",";

            ChannelDefinitionItem channelDefinitionItem;
            QString levelString;
            QString channelString = _multiMeasureResult[i].Channel;
            // CATV上り, FMはチャンネルの代わりに周波数を用いてgetChannelDefinitionItemを行う
            if (band == LC70_BAND_CATV_UP_STREAM || band == LC70_BAND_FM) {
                channelString = Correcter::displayFrequency(band, _multiMeasureResult[i].Frequency);
            }
            if (_model->getChannelDefinitionItem(band, channelString, channelDefinitionItem)) {
                QString polarize = Correcter::convertPolarizeNumToString(channelDefinitionItem.polarization);
                BandTabIndex bandTab = Correcter::convertBandTabIndex(static_cast<LC70_BAND_T>(band), polarize);
                if (!Correcter::displayLevel(bandTab, _multiMeasureResult[i].Level, levelString)) {
                    qDebug() << QString("displayLevel() failed");
                }
            }
            outStream << levelString << ",";        // LEVEL

            // PILOT, LTE, FM, CATVUPは---とする
            if ( (band == LC70_BAND_PILOT || band == LC70_BAND_LTE ||
                  band == LC70_BAND_FM || band == LC70_BAND_CATV_UP_STREAM) ) {
                outStream << QString("---") << ",";
            } else {
                // 測定不可の場合にマイナスの値が入っているものとする
                if (_multiMeasureResult[i].Mer >= 0) {
                    if(_multiMeasureResult[i].BroadcastSystem == LC70_BD_SYS_ANNEXB || _multiMeasureResult[i].BroadcastSystem == LC70_BD_SYS_ANNEXC || _multiMeasureResult[i].BroadcastSystem == LC70_BD_SYS_OPTICAST){
                        if (_multiMeasureResult[i].Mer < 20) {
                            outStream << QString("<20.0") << ",";
                        }else{

                            outStream << Correcter::displayMer(band, _multiMeasureResult[i].BroadcastSystem, _multiMeasureResult[i].Modulation, _multiMeasureResult[i].Mer) << ",";
                        }
                    }else{
                       outStream << Correcter::displayMer(band, _multiMeasureResult[i].BroadcastSystem, _multiMeasureResult[i].Modulation, _multiMeasureResult[i].Mer) << ",";

                    }
                } else {
                    outStream << QString(MER_TIMEOUT_VALUE) << ",";
                }
            }

            // PILOT, LTE, FM, CATVUPは---とする
            if ( (band == LC70_BAND_PILOT || band == LC70_BAND_LTE ||
                  band == LC70_BAND_FM || band == LC70_BAND_CATV_UP_STREAM) ) {
                outStream << QString("---") << ",";
            } else {
                // 測定不可の場合にマイナスの値が入っているものとする
                if (_multiMeasureResult[i].Ber >= 0) {
                    outStream << Correcter::displayBer(_multiMeasureResult[i].Ber) << ",";
                } else {
                    outStream << QString(BER_TIMEOUT_VALUE) << ",";
                }
            }

            QString polarization = Correcter::convertWaveToPolarization(_multiMeasureResult[i].Layer);
            if (Correcter::isSatelliteBroadCasting(_multiMeasureResult[i].Band.toInt())) {                         // Layer/偏波
                outStream << Correcter::displayPolarization(polarization) << ",";
            } else {
                QString layer = Correcter::displayHierarchy(_multiMeasureResult[i].Layer);
                if(layer == QString("")) {
                    outStream << QString("---") << ",";
                } else {
                    outStream << layer << ",";
                }
            }

            outStream << Correcter::displayLocalFrequency(_multiMeasureResult[i].LocalOscillator) << ",";          // 局発周波数(GHz)

            QString broadcastsystem = Correcter::displayBroadbandSystem(_multiMeasureResult[i].BroadcastSystem);   // 放送方式規格

#if 1	//20210630_dx2
            if ( (band == LC70_BAND_PILOT || band == LC70_BAND_LTE ||
                  band == LC70_BAND_CATV_UP_STREAM || band == LC70_BAND_FM) ) {
                outStream << Correcter::displayBandName(band) << ",";
            } else{
                if (broadcastsystem == QString("")) {
                    outStream << QString("---") << ",";
                } else {
                    if(_multiMeasureResult[i].BroadcastSystem == LC70_BD_SYS_ANNEXC || _multiMeasureResult[i].BroadcastSystem == LC70_BD_SYS_ANNEXB){
                        broadcastsystem = "ISDBC";
                    }
                    outStream << broadcastsystem << ",";
                }
            }
#else
            if (broadcastsystem == QString("")) {
                outStream << QString("---") << ",";
            } else {
                outStream << broadcastsystem << ",";
            }
#endif

            QString modulation = Correcter::displayModulation(_multiMeasureResult[i].Modulation);                  // Mod
            if (modulation == QString("")) {
                outStream << QString("---") << "\n";
            } else {
                outStream << modulation << "\n";
            }
        }

        // ファイル出力しクリア
        _multiMeasureResult.clear();
        if (_isFileOpen) {
            if(_measureResultModel->rowCount() % 10 == 0){
                // 負荷軽減のため、10件に1回だけ同期する
                QProcess::execute(SYNC_COMMAND);
            }
            if (outStream.status() == QTextStream::WriteFailed) {
                fileClose();
                emitSuspend();
                emit requestToShowPopup(POPUP_WRITE_ERROR);
            }
        }
    }
}

void DataLoggerMeasurementListViewModel::fileOpen()
{
    prepareDataLoggerMeasure();

#if 1
    // 書き出しファイル名が空の場合にOpen処理を実施する
    if (_outputfile.fileName() == "") {
#else
    // 書き出しファイル名が空の場合、１度もUSBを抜いていない場合にOpen処理を実施する
    if (_outputfile.fileName() == "" && !_isUsbRemove ) {
#endif
        // 日時と拡張子を補完
        _outPutFileName += QString("_") + QDateTime::currentDateTime().toString("yyyyMMddhhmmss") + CSV_FILE_EXTENSION;
        _outputfile.setFileName(_outPutFileName);
        _isFileOpen = _outputfile.open(QIODevice::WriteOnly | QIODevice::Append);
    }
}

void DataLoggerMeasurementListViewModel::fileClose()
{
    if (_isFileOpen) {
        _outputfile.close();
        QProcess::execute(SYNC_COMMAND);
    }

    // 測定データファイル名リストを更新
    _model->updateMeasurementFileList();

    _outputfile.setFileName("");
    _isFileOpen = false;
}

void DataLoggerMeasurementListViewModel::onNotifyChannelChanging(bool changing)
{
    _isChannelChanging = changing;

    if (_preferredCondition.ulFreq != _previousPreferredCondition.ulFreq ||
            _preferredCondition.ulLoFreq != _previousPreferredCondition.ulLoFreq ||
            _preferredCondition.usBdSys != _previousPreferredCondition.usBdSys ||
            _preferredCondition.usBand != _previousPreferredCondition.usBand ||
            _preferredCondition.usMod != _previousPreferredCondition.usMod ||
            _preferredCondition.usCondition != _previousPreferredCondition.usCondition ||
            QString::fromLocal8Bit((char *)_preferredCondition.ucStrChan) != QString::fromLocal8Bit((
                        char *)_previousPreferredCondition.ucStrChan) ||
            _preferredCondition.ucOutputEnb != _previousPreferredCondition.ucOutputEnb) {
        emit changeCondition(&_preferredCondition);
        _previousPreferredCondition = _preferredCondition;
    }
}

void DataLoggerMeasurementListViewModel::changeChannel()
{
    if (_channelList.count() < 1) {
        return;
    }

    if (_view) {
        _view->setProperty("modulation", Correcter::displayModulation(_userSettings[_currentChannelIndex].ulModBit));
    }
    // 中央周波数と局発周波数から周波数を求める
    unsigned int localFrequency = _userSettings.at(_currentChannelIndex).ulLocalFreq;

    // 変調方式から対象の中央周波数を取得する
    unsigned int modulationValue = _userSettings[_currentChannelIndex].ulModBit;
    unsigned int centerFrequency = Correcter::centerFrequency(_channelList[_currentChannelIndex], modulationValue);
    unsigned int frequency = Correcter::frequency(centerFrequency, localFrequency);

    if (_view) {
        // 画面にチャンネル情報を反映
        _view->setProperty("band", Correcter::displayBandName(_channelList[_currentChannelIndex].band));
    }
    if (_view) {
        _view->setProperty("frequency", Correcter::displayFrequency(_channelList[_currentChannelIndex].band, frequency));
    }
    int valueMerCN = Correcter::selectCnrMer(static_cast<LC70_BAND_T>(_channelList[_currentChannelIndex].band), _cnrMerSetting);

    if (_view) {
        _view->setProperty("selectCnMer", valueMerCN);
    }

    if (_view) {
        // FM, PILOT, CATV 上りはチャンネル情報に周波数を表示する
        _view->setProperty("channel", Correcter::displayChannelName(_channelList[_currentChannelIndex].channelNumber));
    }

    // 局発周波数
    // KHz => GHz
    if (_view) {
        _view->setProperty("localFrequency", Correcter::displayLocalFrequency(localFrequency));
    }
    // 階層
    if (_view) {
        _view->setProperty("hierarchy", Correcter::displayHierarchy(_channelList[_currentChannelIndex].band));
    }
    // 変調方式 表示するかどうかを画面に送信する
    if (_view) {
        _view->setProperty("isModulationVisible", Correcter::isModulationVisible(_channelList[_currentChannelIndex].band));
    }
    // チャンネル詳細データの表示可否を通知
    if (_view) {
        _view->setProperty("displayChannelDetail", Correcter::getDisplayChannelDetail(_channelList[_currentChannelIndex].band));
    }
    if (_channelList[_currentChannelIndex].band == LC70_BAND_PILOT || _channelList[_currentChannelIndex].band == LC70_BAND_CATV_UP_STREAM) {
        // 測定BANDがPILOTの場合 変調方式CWを固定値を表示
        if (_view) {
            _view->setProperty("displayModulationPrefixMode", DISPLAY_MODULATION_PREFIX_TOGETHER);
        }
        if (_view) {
            _view->setProperty("modulation", CW);
        }
        if (_view) {
            _view->setProperty("modulationPrefix", QString(""));
        }
    } else if (_channelList[_currentChannelIndex].band == LC70_BAND_FM) {
        // 測定BANDがPILOTの場合 変調方式FMの固定値を表示
        if (_view) {
            _view->setProperty("displayModulationPrefixMode", DISPLAY_MODULATION_PREFIX_TOGETHER);
        }
        if (_view) {
            _view->setProperty("modulation", FM);
        }
        if (_view) {
            _view->setProperty("modulationPrefix", QString(""));
        }
    } else {
        if (_view) {
            _view->setProperty("displayModulationPrefixMode", DISPLAY_MODULATION_PREFIX_TOGETHER);
        }
        if (_view) {
            _view->setProperty("modulation", QString(""));
        }
        if (_view) {
            _view->setProperty("modulationPrefix", QString(""));
        }
    }

    // 優先測定測定条件
    setPreferredPMeasuringCondition();

    // マルチ測定測定条件
    setUserMultiMeasuringCondition();

    _thresholdSet = false;
    _levelThreshold = 0;
}

void DataLoggerMeasurementListViewModel::setUserMultiMeasuringCondition(void)
{
    memset(&_condition, 0x00, sizeof(LC70_MEAS_MULTI_T));

    int setChannelNum = _userSettings.count();

    // 最大設定チャンネル数チェック
    // （登録時に上限チェックは実施されているが、テストデータで意識せずに上限を超えていた場合など問題になるため）
    if (setChannelNum > LC70_MAX_REG_CH) {
        setChannelNum = LC70_MAX_REG_CH;
    }


    // 登録件数
    _condition.sRegNum = setChannelNum;

    _measuringCondition = 0;
    _hasPowerSupplyChannel = false;
    BatteryLevel batteryLevel = BATTERY_LEVEL_UNKNOWN;
    BatteryStatus batteryStatus = BATTERY_STATUS_UNKNOWN;
    MainFrameViewModel *mainFrame = dynamic_cast<MainFrameViewModel *>(_mainFrame);
    if (mainFrame) {
        batteryLevel = static_cast<BatteryLevel>(mainFrame->batteryLevel());
        batteryStatus = static_cast<BatteryStatus>(mainFrame->batteryStatus());
    }

    for (int i = 0; i < setChannelNum; i++) {
        // 変調方式
        unsigned int modulationValue = _userSettings.at(i).ulModBit;
        // 中心周波数
        unsigned int centerFrequency = Correcter::centerFrequency(_channelList[i], modulationValue);

        _condition.single[i].ulFreq = centerFrequency;
        _condition.single[i].usBand = _userSettings[i].ucBand;

        // チャンネル番号文字列(NULL終端なし、8文字に足りない場合はスペースで埋める)
        memset(_condition.single[i].ucStrChan, ' ', CHANNEL_NUMBER_LENGTH);

        for (unsigned int j = 0; j < CHANNEL_NUMBER_LENGTH; ++j) {
            if (j <= strlen(_userSettings[i].cStrChNo) - 1) {
                _condition.single[i].ucStrChan[j] = _userSettings[i].cStrChNo[j];
            }
        }

        int length = strnlen((char *)_condition.single[i].ucStrChan, sizeof _condition.single[i].ucStrChan);
        if (length < CHANNEL_NUMBER_LENGTH) {
            for (int j = length; j < CHANNEL_NUMBER_LENGTH; ++j) {
                _condition.single[i].ucStrChan[j] = ' ';
            }
        }

        // 局発周波数
        _condition.single[i].ulLoFreq = _userSettings[i].ulLocalFreq;

        // 放送方式
        _condition.single[i].usBdSys = _userSettings[i].ucSpec;

        // 変調方式
        _condition.single[i].usMod = _userSettings[i].ulModBit;

        // 測定対象
        // グラフとリストで異なる
        // LEVELは必ず立てる
        if(_userSettings[i].ucBand == LC70_BAND_PILOT
                || _userSettings[i].ucBand == LC70_BAND_LTE
                || _userSettings[i].ucBand == LC70_BAND_FM
                || _userSettings[i].ucBand == LC70_BAND_CATV_UP_STREAM){
            _condition.single[i].usCondition = LC70_DATA_MSK_LEVEL;
        } else {
            _condition.single[i].usCondition = LC70_DATA_MSK_LEVEL;
            // MER/CN の使い分け(CNR ビットは必ず立てる。MER のときは MER も立てる)
            if ((_userSettings[i].ucGetDataFlg & LC70_DATA_MSK_CNR) == LC70_DATA_MSK_CNR) {
                if (Correcter::selectCnrMer(static_cast<LC70_BAND_T>(_userSettings[i].ucBand), _cnrMerSetting) == LC70_MEAS_SET_SEL_CNR) {
                    _condition.single[i].usCondition |= LC70_DATA_MSK_CNR;
                } else if (Correcter::selectCnrMer(static_cast<LC70_BAND_T>(_userSettings[i].ucBand), _cnrMerSetting) == LC70_MEAS_SET_SEL_MER) {
                    _condition.single[i].usCondition |= (LC70_DATA_MSK_CNR | LC70_DATA_MSK_MER);
                }else {
                    // nothing to do
                }
            }
            if ((_userSettings[i].ucGetDataFlg & LC70_DATA_MSK_BER) == LC70_DATA_MSK_BER) {
                _condition.single[i].usCondition |= (LC70_DATA_MSK_BER);
            }
        }
        // 給電有効 有効(ユーザー/ロガー、及びユーザー/ロガー設定のSCAN時)
        _condition.single[i].ucOutputEnb = 1;
        //_condition.single[i].powerSupply = _isLowBattery ? LC70_POWER_SUPPLY_OFF : static_cast<LC70_POWER_SUPPLY_T>(_userSettings[i].ucDcOut);
        _condition.single[i].powerSupply = (batteryLevel > BATTERY_LEVEL2 || batteryStatus >= BATTERY_STATUS_CHARGING) ? static_cast<LC70_POWER_SUPPLY_T>(_userSettings[i].ucDcOut) : LC70_POWER_SUPPLY_OFF;

        // 給電するチャンネルを持つか
        if (static_cast<LC70_POWER_SUPPLY_T>(_userSettings[i].ucDcOut) > LC70_POWER_SUPPLY_OFF) {
            _hasPowerSupplyChannel = true;
        }
    }

    // 給電するチャンネルがなければ給電キャンセルはしていないこととする
    if (!_hasPowerSupplyChannel) {
        _cancelPowerSupply = false;
    } else {
        // 給電するチャンネルを持ち、バッテリーが 10% 以下なら給電キャンセルしたとみなす
        _cancelPowerSupply = (batteryLevel > BATTERY_LEVEL2 || batteryStatus >= BATTERY_STATUS_CHARGING) ? false : true;
    }
}

void DataLoggerMeasurementListViewModel::setPreferredPMeasuringCondition()
{
    memset(&_preferredCondition, 0x00, sizeof(LC70_MEAS_SINGLE_T));

    // 中央周波数と局発周波数から実周波数を求める
    unsigned int localFrequency = _userSettings.at(_currentChannelIndex).ulLocalFreq;

    // 変調方式から対象の中央周波数情報を取得する
    unsigned int modulationValue = _userSettings.at(_currentChannelIndex).ulModBit;
    unsigned int centerFrequency = Correcter::centerFrequency(_channelList[_currentChannelIndex], modulationValue);

    // 中心周波数
    _preferredCondition.ulFreq = centerFrequency;

    // チャンネル情報設定
    _preferredCondition.ulFreq = centerFrequency; // 中心周波数
    _preferredCondition.usBand = _channelList[_currentChannelIndex].band; // BAND

    // チャンネル番号文字列(NULL終端なし、8文字に足りない場合はスペースで埋める)
    memset(_preferredCondition.ucStrChan, ' ', CHANNEL_NUMBER_LENGTH);

    for (int i = 0; i < CHANNEL_NUMBER_LENGTH; ++i) {
        if (i <= _channelList[_currentChannelIndex].channelNumber.toLocal8Bit().length() - 1) {
            _preferredCondition.ucStrChan[i] = _channelList[_currentChannelIndex].channelNumber.toLocal8Bit()[i];
        }
    }

    int length = strnlen((char *)_preferredCondition.ucStrChan, sizeof _preferredCondition.ucStrChan);
    if (length < CHANNEL_NUMBER_LENGTH) {
        for (int j = length; j < CHANNEL_NUMBER_LENGTH; ++j) {
            _preferredCondition.ucStrChan[j] = ' ';
        }
    }

    // 局発周波数の連携
    _preferredCondition.ulLoFreq = localFrequency;

    // 放送方式の連携
    _preferredCondition.usBdSys = _userSettings[_currentChannelIndex].ucSpec;
    // 変調方式
    _preferredCondition.usMod = _userSettings[_currentChannelIndex].ulModBit;

    // 測定条件の連携
    // MER/CN の使い分け(CNR ビットは必ず立てる。MER のときは MER も立てる)
    if (_preferredCondition.usBand == LC70_BAND_PILOT
            || _preferredCondition.usBand == LC70_BAND_LTE
            || _preferredCondition.usBand == LC70_BAND_FM
            || _preferredCondition.usBand == LC70_BAND_CATV_UP_STREAM) {
        _preferredCondition.usCondition = LC70_DATA_MSK_LEVEL;
    } else {
        _preferredCondition.usCondition = LC70_DATA_MSK_LEVEL;
        if ((_userSettings[_currentChannelIndex].ucGetDataFlg & LC70_DATA_MSK_CNR) == LC70_DATA_MSK_CNR) {
            if (Correcter::selectCnrMer(static_cast<LC70_BAND_T>(_userSettings[_currentChannelIndex].ucBand), _cnrMerSetting) == LC70_MEAS_SET_SEL_CNR) {
                _preferredCondition.usCondition |= LC70_DATA_MSK_CNR;
            } else if (Correcter::selectCnrMer(static_cast<LC70_BAND_T>(_userSettings[_currentChannelIndex].ucBand), _cnrMerSetting) == LC70_MEAS_SET_SEL_MER) {
                _preferredCondition.usCondition |= (LC70_DATA_MSK_CNR | LC70_DATA_MSK_MER);
            }else {
                // nothing to do
            }
        }
        if ((_userSettings[_currentChannelIndex].ucGetDataFlg & LC70_DATA_MSK_BER) == LC70_DATA_MSK_BER) {
            _preferredCondition.usCondition |= (LC70_DATA_MSK_BER);
        }
    }

    // 給電有効/無効 0:無効(マニュアル測定) 1:有効(ユーザー/ロガー、及びユーザー/ロガー設定のSCAN時)
    _preferredCondition.ucOutputEnb = 1;

    // 給電設定
    _preferredCondition.powerSupply = _isLowBattery ? LC70_POWER_SUPPLY_OFF : static_cast<LC70_POWER_SUPPLY_T>(_userSettings[_currentChannelIndex].ucDcOut);

    // チャンネル変更中の操作を禁止
    if (_isChannelChanging) {
        return;
    }

    // 測定条件を変更
    _previousPreferredCondition = _preferredCondition;
}

void DataLoggerMeasurementListViewModel::clear()
{
    // マルチチャンネル測定結果のクリアー
    for (int i = 0; i < _resultListNum; ++i) {
        // リスト表示内容の更新
        _measureResultModel->update(i, 0, 0, 0,0,0,10);
    }

    // 共通表示部分のクリアー
    // 優先測定測定 入力レベル
    double value = 0.0f;
    if (_view) {
        _view->setProperty("level", QString("%1").arg(value, 3, 'f', 1).rightJustified(4, '0'));
    }
    // 優先測定 MER
    if (_view) {
        _view->setProperty("mer", QString("%1").arg(value, 3, 'f', 1).rightJustified(4, '0'));
    }

    // 優先測定 BER
    if (_view) {
        _view->setProperty("ber", QString("%1").arg(value, 1, 'e', 1).rightJustified(4, '0'));
    }

    // 優先測定 周波数
    if (_view) {
        _view->setProperty("frequency", QString(""));
    }

    // 優先測定 変調方式
    if (_view) {
        _view->setProperty("modulation", QString(""));
    }

    // 優先測定 階層
    if (_view) {
        _view->setProperty("hierarchy", QString(""));
    }

    // 優先測定 局発周波数
    if (_view) {
        _view->setProperty("localFrequency", QString("%1").arg(value, 3, 'f', 3).rightJustified(6, '0'));
    }
}

void DataLoggerMeasurementListViewModel::detectPowerSupplyError(uint16_t code, uint8_t action, QString message)
{
    _failureCode = code;
    _actionInFailure = action;

#if 1
    // 給電を停止する
    _model->setPowerSupplyForMultiBand(LC70_POWER_SUPPLY_OFF);
    _model->removeMenuList(MenuListIconIndex::POWER_SUPPLY_ICON);
    _powerSupplied = false;
#endif

    if (_actionInFailure == LC70_FAIL_CTL_MSGDISP_CLR) {
        uint8_t addInfo[LC70_FAIL_ADD_INFO_NUM];
        memset(addInfo, 0x00, LC70_FAIL_ADD_INFO_NUM);
        // _model->setFaultInformation(_failureCode, LC70_FAIL_STAT_NORMAL, addInfo); 故障復旧処理不要
        _failureCode = 0;
        _actionInFailure = LC70_FAIL_CTL_NONE;
    }

    _model->removeMenuList(MenuListIconIndex::POWER_SUPPLY_ICON);

#if 0
    // 測定を停止する
    _suspended = true;
    // スレッドを停止
    emit requestToSuspendMultiMeasuring();
#endif

    _model->buzzer(LC70_BUZZ_ERR, LC70_BUZZ_MODE_ONESHOT, true);

    emit requestToShowPopupWithMessage(POPUP_POWER_SUPPLY_ERROR_USER_MEASURING, message);
}

void DataLoggerMeasurementListViewModel::onTimeOutForWakingUp()
{
    // サブ CPU ウェイクアップ
    qDebug() << QString("%1 [DATA LOGGER]SUB CPU WAKE UP").arg(QDateTime::currentDateTime().toString("hh:mm:ss"));

    // サブに対するウェイクアップ要求が行われていなければ行う
    if (!_model->wakeUpRequested()) {
        _model->setWakeUpRequested(true);
        _model->wakeUp(1);
    }
}

void DataLoggerMeasurementListViewModel::onNotifyStopMeasuring(AcquisitionTarget target)
{
    if (target != MULTI) {
        return;
    }

#if 1
#if 1
    // マルチ測定の停止を受信
    if (_isWaitRestart) {
        bool isStandbyRequest = false;
        if (_intervalTime >= SUBSLEEP_INTERVAL) {
            isStandbyRequest = true;
        }
        // サブ CPU スタンバイ
        if (_model->wakeUpRequested() && isStandbyRequest) {
            qDebug() << QString("%1 [DATA LOGGER]SUB CPU STANDBY").arg(QDateTime::currentDateTime().toString("hh:mm:ss"));
            _model->setWakeUpRequested(false);
            _model->requestStandBy(1);
        }

        // 再開 1 分前のタイマーを追加し、タイムアウトでサブ CPU ウェイクアップしてから測定再開
        if (_intervalTime > 0)  {
            setRestartTimer(isStandbyRequest);
        }
    }
#else
    // マルチ測定の停止を受信
    if (_isWaitRestart && (_intervalTime >= SUBSLEEP_INTERVAL)) {
        qDebug() << QString("%1 [DATA LOGGER]SUB CPU STANDBY").arg(QDateTime::currentDateTime().toString("hh:mm:ss"));
        // サブ CPU スタンバイ
        if (_model->wakeUpRequested()) {
            _model->setWakeUpRequested(false);
            _model->requestStandBy(1);
        }

        // 再開 1 分前のタイマーを追加し、タイムアウトでサブ CPU ウェイクアップしてから測定再開
        if (_intervalTime > 0)  {
            setRestartTimer();
        }
    }
#endif
#else
    // マルチ測定の停止を受信
    if (_isWaitRestart) {
        qDebug() << QString("%1 [DATA LOGGER]SUB CPU STANDBY").arg(QDateTime::currentDateTime().toString("hh:mm:ss"));
        // サブ CPU スタンバイ
        if (_model->wakeUpRequested()) {
            _model->setWakeUpRequested(false);
            _model->requestStandBy(1);
        }

        // 再開 1 分前のタイマーを追加し、タイムアウトでサブ CPU ウェイクアップしてから測定再開
        if (_intervalTime > 0)  {
            setRestartTimer();
        }
    }
#endif
}

void DataLoggerMeasurementListViewModel::updatePowerSupplyIcon(int targetIndex)
{
    // 給電中以外は何もしない
    if (!_powerSupplied) {
        return;
    }
    int powerSupply = static_cast<LC70_POWER_SUPPLY_T>(_userSettings[targetIndex].ucDcOut);
qDebug() << QString("powerSupplyIcon --- ") << Correcter::powerSupplyIcon(powerSupply);
    _model->replaceMenuList(MenuListIconIndex::POWER_SUPPLY_ICON, Correcter::powerSupplyIcon(powerSupply));
}


void DataLoggerMeasurementListViewModel::prepareDataLoggerMeasure()
{
    // この時点では設定ファイル名の一部のみとし、初回の測定結果の出力時にファイル名に日付と拡張子を補完する
//#if 1	//20210625_dx5
//    QString prefix = "_" + QDateTime::currentDateTime().toString("yyyyMMddhhmmss");
//#else
//    QString prefix = "_" + QDateTime::currentDateTime().toString("yyyyMMddhhmm");
//#endif
//    QString fileName = _takeOver["selectedUserListFile"].toString() + prefix + CSV_FILE_EXTENSION;
    QString fileName = _takeOver["selectedUserListFile"].toString();

    QString path = _model->applicationSetting()->resultDataPathLogger();

    QDir directory(path);

    if (!directory.exists()) {
        if (!directory.mkpath(".")) {
            return;
        }
    }

    // 日付のディレクトリ作成
    path = directory.path() + QString("/") + QDateTime::currentDateTime().toString("yyyyMMdd");
    directory.setPath(path);

    if (!directory.exists()) {
        if (!directory.mkpath(".")) {
            return;
        }
    }

    _outPutFileName = path + "/" + fileName;
}

void DataLoggerMeasurementListViewModel::onChangeUsbInsertionRemoval(int usbStatus)
{
#if 0
    // 再度 USBを抜き差ししてもダイアログは表示しない
    if (usbStatus == LC70_STRAGE_NONE && !_isUsbRemove) {
        fileClose();
        // 測定を停止
        emitSuspend();
        // USBが抜かれたことを知らせるダイアログを表示する
        emit requestToShowPopup(POPUP_DATA_LOGGER_USB_REMOVE_ERROR);
        _isUsbRemove = true;
    }
#endif
}

void DataLoggerMeasurementListViewModel::recover()
{
    if (_list == nullptr) {
        _list = _model->search("list");
    }
}

void DataLoggerMeasurementListViewModel::changePowerSupplyStatus(bool allowPowerSupply)
{
    if (allowPowerSupply) {
        if (!_cancelPowerSupply) {
            // 給電可で給電キャンセルしてなければ何もしない
            return;
        } else {
            // 給電可で給電キャンセルしていれば給電を再開

            // 給電しているチャンネルがなければ何もしない
            if (_hasPowerSupplyChannel) {
                // 測定を止める
                emitSuspend();

                // 測定条件で給電 ON に設定
                changeChannel();

                // 測定再開
                if (!_isWaitRestart) {
                    emitResume();
                    _powerSupplied = true;
                }
            }

            // 給電キャンセルなし
            _cancelPowerSupply = false;
        }
    } else {
        if (_cancelPowerSupply) {
            // 給電不可のとき既に給電停止していたら何もしない
            return;
        } else {
            // バッテリー状態の変化に伴う給電の停止
            _cancelPowerSupply = true;

            // 給電しているチャンネルがなければ何もしない
            if (_hasPowerSupplyChannel) {
                // 給電を止める
                _model->setPowerSupplyForMultiBand(LC70_POWER_SUPPLY_OFF);
                _model->removeMenuList(MenuListIconIndex::POWER_SUPPLY_ICON);
                _powerSupplied = false;

                // 測定を止める
                emitSuspend();

                // 測定条件で給電 OFF に設定
                changeChannel();

                // 測定再開
                if (!_isWaitRestart) {
                    emitResume();
                }

                // ポップアップを表示
                emit requestToShowPopup(POPUP_DISABLE_POWER_SUPPLY);
            }
        }
    }
}

bool DataLoggerMeasurementListViewModel::createUserSettingChannelList(QVector<LC70_LOGGER_SET_LOGGER_ROW_T> &userSettingList)
{
    // 未定義はtrue扱い
    static const QMap<LC70_BAND_T, bool> merBerMap = {
        {LC70_BAND_PILOT,              false},
        {LC70_BAND_LTE,                false},
        {LC70_BAND_FM,                 false},
        {LC70_BAND_CATV_UP_STREAM,     false},
    };

    // 未定義はLC70_BD_SYS_NONE扱い
    static const QMap<LC70_BAND_T, int> modulationMap = {
        {LC70_BAND_PILOT,              LC70_SETMOD_CW},
        {LC70_BAND_LTE,                LC70_SETMOD_UNDEF},
        {LC70_BAND_FM,                 LC70_SETMOD_FM},
        {LC70_BAND_CATV_UP_STREAM,     LC70_SETMOD_CW},
    };

    // 画面間共有情報から読み込み対象の情報を取得
    ChannelEditMode mode = static_cast<ChannelEditMode>(_takeOver["channelEditMode"].toInt());
    QString fileName = _takeOver["userSettingFileName"].toString();
    if (!fileName.contains(CSV_FILE_EXTENSION)) {
        fileName += QString(CSV_FILE_EXTENSION);
    }

    if (!_model->importSettingFile(mode, fileName)) {
        qDebug() << __PRETTY_FUNCTION__ << ", importSettingFile Error. mode: " << mode << " , fileName: " << fileName;
        return false;
    }

    // 読み込んだファイルの内容を取得
    QList<UserDataLoggerSettingFile> settingFileList = _model->selectMeasuringChannels();

    userSettingList.clear();
    _channelList.clear();

    QVariant broadBandModulationSetting;
    _model->getValue(VALUE_BROADBANDSYSTEM_MODULATION, broadBandModulationSetting);

    for (int i = 0; i < settingFileList.size(); ++i ) {
        LC70_BAND_T bandId = Correcter::convertBandId(settingFileList.at(i).band).first;
        int polarization = Correcter::convertBandId(settingFileList.at(i).band).second;

        if (bandId == LC70_BAND_UNDEF) {
            qDebug() << __PRETTY_FUNCTION__ << ", convertBandId Error. band: " << Correcter::convertBandId(settingFileList.at(i).band);
            continue;
        }
        LC70_LOGGER_SET_LOGGER_ROW_T row;
        memset(&row, 0, sizeof(LC70_LOGGER_SET_LOGGER_ROW_T));
        row.ucEnbFlg = 1;
        row.ucNo = i;
        row.ucBand = bandId;
        row.ucSpec = settingFileList.at(i).broadcastsystem;
        strncpy(row.cStrChNo, settingFileList.at(i).channel.toUtf8().constData(), sizeof(row.cStrChNo));
        row.ulCenterFreq = settingFileList.at(i).frequency;
        row.ulLocalFreq = settingFileList.at(i).localFrequency;
        row.ucGetDataFlg = merBerMap.value(bandId, true) ? LC70_DATA_MSK_CNR : LC70_DATA_MSK_NONE;
        if (bandId == LC70_BAND_UHF || bandId == LC70_BAND_CATV_ALL || bandId == LC70_BAND_SKYP) {
            row.ucGetDataFlg |= LC70_DATA_MSK_MER;
        }
        row.ucGetDataFlg |= merBerMap.value(bandId, true) ? LC70_DATA_MSK_BER : LC70_DATA_MSK_NONE;
        row.ucDcOut = settingFileList.at(i).powerSupply;

        ChannelDefinitionItem channelDefinitionItem;
        channelDefinitionItem.band = bandId;
        channelDefinitionItem.channelNumber = settingFileList.at(i).channel;
        channelDefinitionItem.polarization = polarization;
        channelDefinitionItem.valid = true;

        if (bandId == LC70_BAND_CATV_ALL) {
            // CATV ALL は周波数の末尾の特徴から中心周波数などの情報を特定する
            uint32_t localFrequencySuffix = settingFileList.at(i).frequency % CATV_ALL_CENTER_FREQUENCY_MOD;

            if (localFrequencySuffix == CATV_ALL_CENTER_FREQUENCY_ISDBT_SUFFIX) {
                channelDefinitionItem.centerFrequency1 = settingFileList.at(i).frequency - CATV_ALL_CENTER_FREQUENCY_ISDBT_SUFFIX;
                channelDefinitionItem.centerFrequency2 = settingFileList.at(i).frequency;
                row.ulModBit = modulationMap.value(bandId, LC70_SETMOD_OFDM);
            } else {
                channelDefinitionItem.centerFrequency1 = settingFileList.at(i).frequency;
                channelDefinitionItem.centerFrequency2 = settingFileList.at(i).frequency + CATV_ALL_CENTER_FREQUENCY_ISDBT_SUFFIX;
                row.ulModBit = modulationMap.value(bandId, LC70_SETMOD_64QAM);
            }
        } else {
            channelDefinitionItem.centerFrequency1 = settingFileList.at(i).frequency;
            channelDefinitionItem.centerFrequency2 = -1;

//            QList<int> broadbandSystemList;
//            _model->getBroadbandSystem(bandId, broadbandSystemList);
//            if (broadbandSystemList.isEmpty()) {
//                continue;
//            }
//            int broadbandSystem = broadbandSystemList.at(0);
//            unsigned int modulationBit = _model->getModulationBit(bandId, broadbandSystem);

            // Band 毎の変調方式の特定
            unsigned int modulation = Correcter::selectModulationMethod(settingFileList[i].band);

            // 放送方式、変調方式の設定(定義ファイル 1.0.1 対応)
            uint16_t broadcastSystem = settingFileList[i].broadcastsystem;
            uint32_t modulationBit = Correcter::getModulationCondition(channelDefinitionItem.band, broadcastSystem, channelDefinitionItem.polarization, modulation, broadBandModulationSetting);

            row.ulModBit = modulationBit;
        }

        if (bandId == LC70_BAND_LTE) {
            // LTEは帯域幅を設定する
            channelDefinitionItem.bandWidth = LTE_BAND_WIDTH;
        } else {
            channelDefinitionItem.bandWidth = -1;
        }

        // ユーザー設定追加
        userSettingList.append(static_cast<LC70_LOGGER_SET_LOGGER_ROW_T>(row));
        // チャンネル定義追加
        _channelList.append(channelDefinitionItem);
    }

    // 作成に成功したチャンネル情報が1件もなければエラーにする
    return !_channelList.isEmpty();
}
