/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef DATALOGGERMEASUREMENTLISTVIEWMODEL_H
#define DATALOGGERMEASUREMENTLISTVIEWMODEL_H

#include "abstractviewmodel.h"
#include "buttonoperation.h"
#include "correcter.h"
#include "measureresultmodel.h"

/**
 * @brief データ取得結果表示 ViewModel
 */
class DataLoggerMeasurementListViewModel : public AbstractViewModel
{
    Q_OBJECT
public:
    /**
     * @brief コンストラクター
     * @param screenId 画面 ID
     * @param model Model クラスのポインター
     * @param parent 親オブジェクト
     */
    explicit DataLoggerMeasurementListViewModel(ScreenId screenId, LevelCheckerModel *&model, QObject *parent = nullptr);

    /**
     * @brief デストラクタ
     */
    virtual ~DataLoggerMeasurementListViewModel();

    /**
     * @brief 画面を開くときに呼び出すスレッドの開始、再開処理
     */
    virtual void resume(void) override;

    /**
     * @brief 画面を閉じるときに呼び出すスレッドの停止処理
     */
    virtual void suspend(void) override;

    /**
     * @brief 画面を開くときに呼び出す qml との接続処理
     * @param qml qml のオブジェクトのポインター
     * @param mainFrame MainFrameViewModel のポインター
     */
    virtual void createConnection(QObject *qml, QObject *mainFrame) override;

    /**
     * @brief 画面Active 時の処理
     *
     * 画面表示後データ収集開始
     */
    virtual void activate() override;

    /**
     * @brief 画面を閉じるときに呼び出す表示な用の保存処理
     */
    virtual void deactivate(void) override;

    /**
     * @brief シャットダウン直前の処理
     */
    virtual void readyToStandby(void) override;

public slots:
    /**
     * @brief USB挿抜変更通知を受信する
     * @param usbStatus USB挿抜状態 LC70_STRAGE_Tの値
     */
    void onChangeUsbInsertionRemoval(int usbStatus);

signals:
    /**
     * @brief 測定結果取得開始、再開依頼を通知
     * @param condition マルチ測定条件
     * @param prefferedCondition 優先測定条件
     * @pram displayType 遷移先画面種別
     * @param interval 測定結果取得間隔
     */
    void resumeMultiMeasuring(LC70_MEAS_MULTI_T *condition, LC70_MEAS_SINGLE_T *prefferedCondition, LC70_DISP_TYPE_T displayType, uint32_t interval);

    /**
     * @brief チャンネル変更
     * @param condition チャンネル別測定条件
     */
    void changeCondition(LC70_MEAS_SINGLE_T *condition);

    /**
     * @brief リストのページを移動
     * @param page ページ
     */
    void scrollPage(QVariant page);

    /**
     * @brief 引き継ぎデータの削除要求
     * @param key 引き継ぎデータのキー
     */
    void removeAttribute(QString key);

    /**
     * @brief マルチ測定停止依頼
     */
    void requestToSuspendMultiMeasuring(void);

    /**
     * @brief 画面共通情報設定要求
     * @param key 画面共通情報キー
     * @param value 画面共通情報データ
     */
    void requestSetTakeOver(QVariant, QVariant);

protected:
    /**
     * @brief 必要に応じて View を指すポインターを付け替えた後の復帰処理を行う
     */
    virtual void recover(void) override;

    /**
     * @brief 給電処理の変更
     * @param allowPowerSupply バッテリー 10% 以下で false, 11 % 以上で true
     * @note _allowPowerSupply の値を参照して給電処理を変更する
     */
    virtual void changePowerSupplyStatus(bool allowPowerSupply);

protected slots:
    /**
     * @brief ボタン押下の状態を受信
     * @param id ボタン ID
     */
    virtual void onKeyStatus(ButtonId id) override;

    /**
     * @brief ポップアップから受諾の通知を受信
     * @param okOnly OK のみの場合 true
     */
    virtual void onAccept(PopupId popupId, bool okOnly) override;

    /**
     * @brief バッテリ低下状態変化通知
     * @param isLowBattery バッテリ低下状態かどうか
     */
    virtual void onLowBatteryStatusChanged(bool isLowBattery) override;

private slots:
    /**
     * @brief マルチ測定結果(優先測定結果を含む)を受信
     * @param result 測定結果
     * @param prefferedChannelResult マルチチャンネル優先測定結果
     */
    void onMultiMeasureResult(MultiResult result, MeasureResult prefferedChannelResult);

    /**
     * @brief チャンネル変更中通知を受信
     * @param changing 変更中か否か
     */
    void onNotifyChannelChanging(bool changing);

    /**
     * @brief 再開発行
     */
    void emitResume(void);

    /**
     * @brief 給電異常検知
     * @param code 故障コード
     * @param action 故障時の動作
     * @param message 表示メッセージ
     */
    void detectPowerSupplyError(uint16_t code, uint8_t action, QString message);

    /**
     * @brief ウェイクアップ用タイマーのタイムアウトを受信
     */
    void onTimeOutForWakingUp(void);

    /**
     * @brief 測定停止を受信
     * @param target 測定の種類
     */
    void onNotifyStopMeasuring(AcquisitionTarget target);

private:
    /**
     * @brief ファイルの準備等 データロガー測定実施位のため準備を行う
     */
    void prepareDataLoggerMeasure(void);

    /**
     * @brief チャンネル変更
     */
    void changeChannel(void);

    /**
     * @brief ユーザ設定によるマルチ測定条件の設定
     */
    void setUserMultiMeasuringCondition(void);

    /**
     * @brief 優先測定条件の設定
     */
    void setPreferredPMeasuringCondition(void);

    /**
     * @brief 表示クリアー
     */
    void clear(void);

    /**
     * @brief ファイル出力
     */
    void fileoutput();

    /**
     * @brief ファイル出力開始処理
     */
    void fileOpen();

    /**
     * @brief ファイル出力終了処理
     */
    void fileClose();

    /**
     * @brief リスタートセット
     */
//    void setRestartTimer(void);
    void setRestartTimer(bool isWakeup);

    /**
     * @brief 中断発行
     */
    void emitSuspend(void);

    /**
     * @brief 給電アイコンの更新を行う
     * @param targetIndex 参照データのインデックス
     */
    void updatePowerSupplyIcon(int targetIndex);

    /**
     * @brief ユーザー設定チャンネルリスト作成
     * @param userSettingList ユーザー設定
     * @return true：成功、false：失敗
     * @note AbstractMeasurementViewModel と同じ定義
     */
    bool createUserSettingChannelList(QVector<LC70_LOGGER_SET_LOGGER_ROW_T> &userSettingList);

    // 操作対象のエリア定義
    enum IntervalValue {
        INTERVAL_0  = 0,    // 連続
        INTERVAL_10 = 1,    // 10分
        INTERVAL_20 = 2,    // 20分
        INTERVAL_40 = 3,    // 40分
        INTERVAL_60 = 4,    // 60分
    };

    QObject *_list;                                 //!< リストの QML
    QList<ChannelDefinitionItem> _channelList;      //!< チャンネル定義情報のリスト
    int _currentChannelIndex;                       //!< チャンネル定義情報のリストのインデックス
    LC70_MEAS_MULTI_T _condition;                   //!< マルチ測定条件
    LC70_MEAS_SINGLE_T _preferredCondition;         //!< 優先測定条件
    LC70_MEAS_SINGLE_T _previousPreferredCondition; //!< 前回の優先測定条件
    MeasureResultModel *_measureResultModel;        //!< 測定結果リストのモデル(View の実体を参照)
    QList<MeasureResult> _multiMeasureResult;       //!< マルチチャンネル測定結果
    QFile _outputfile;                              //!< 書き出しファイル
    bool _isFileOpen;                               //!< 書き出しファイルが既に開いているか
    int _totalPage;                                 //!< リストのページ数
    int _currentPage;                               //!< リストの現在のページ
    double _levelThreshold;                         //!< 測定入力レベルの閾値
    QVector<LC70_LOGGER_SET_LOGGER_ROW_T> _userSettings;   //!< ユーザー設定の設定値
    bool _isChannelChanging;                        //!< チャンネル変更中
    bool _thresholdSet;                             //!< 閾値設定済みか否か
    uint16_t _broadCastSystem;                      //!< 放送方式
    int _resultListNum;                             //!< リストの行数
    IntervalValue _interval;                        //!< 結果の書き出し間隔選択番号
    int _intervalTime;                              //!< 結果の書き出し間隔
    int _cycle;                                     //!< 結果の書き出しサイクル
    int _receiveCount;                              //!< 結果受取回数
    bool _isSetEnddatetime;                         //!< 書き出し終了時間をセットしているか
    int _end_yyyy;                                  //!< 書き出し終了時間(年)
    int _end_mm;                                    //!< 書き出し終了時間(月)
    int _end_dd;                                    //!< 書き出し終了時間(日)
    int _end_HH;                                    //!< 書き出し終了時間(時間)
    int _end_MM;                                    //!< 書き出し終了時間(分)
    uint16_t _measuringCondition;                   //!< 測定結果ファイル出力時の測定項目
    QVariant _cnrMerSetting;                        //!< CNR/MER 切替設定
    bool _isTopOfFile;                              //!< 書き込み箇所がファイルの先頭かどうか
    QString _outPutFileName;                        //!< 書き出しファイル名
    bool _isUsbRemove;                              //!< USBが抜かれたかどうか
    bool _isLowBattery;                             //!< バッテリ低下状態かどうか
    bool _isWaitRestart;                            //!< 測定間隔設定により、測定停止後に再開を待っている状態かどうか
    bool _suspended;                                //!< 測定停止中か否か
    uint16_t _failureCode;                          //!< 故障コード
    uint8_t _actionInFailure;                       //!< 故障時の動作
    QTimer *_intervalTimer;                         //!< 停止間隔用タイマー
    QTimer *_wakeUpTimer;                           //!< ウェイアクアップ用タイマー
    bool _hasPowerSupplyChannel;                    //!< 給電するチャンネルを持つか否か
    bool _cancelPowerSupply;                        //!< 給電キャンセル(ローバッテリー時)
    bool _powerSupplied;                            //!< 給電中か否か
};

#endif // DATALOGGERMEASUREMENTLISTVIEWMODEL_H
