/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include "dataloggersettingviewmodel.h"

const int DataLoggerSettingViewModel::BUTTON_COLUMN_NUMBER = 2;
const int DataLoggerSettingViewModel::BUTTON_ROW_NUMBER    = 3;

QVector<QVector<QString>> dataloggerSettingButtonData = {
    { {"m1"}, {""}, {""} },
    { {"m4"}, {"m5"}, {""} },
};

QVector<QVector<QString>> readData1DataloggerSettingButtonData = {
    { {"m1"}, {"m2"}, {""} },
    { {"m4"}, {"m5"}, {""} },
};

QVector<QVector<QString>> readData2DataloggerSettingButtonData = {
    { {"m1"}, {""}, {"m3"} },
    { {"m4"}, {"m5"}, {""} },
};

QVector<QVector<QString>> allDataDataloggerSettingButtonData = {
    { {"m1"}, {"m2"}, {"m3"} },
    { {"m4"}, {"m5"}, {""} },
};

DataLoggerSettingViewModel::DataLoggerSettingViewModel(ScreenId screenId, LevelCheckerModel *&model, QObject *parent)
    : AbstractViewModel(screenId, model, parent)
    , _buttonOperationMode(DATALOGGER_SETTING)
{
    setObjectName("DataLoggerSettingViewModel");
    _buttonOperation.setButtonName(BUTTON_COLUMN_NUMBER, BUTTON_ROW_NUMBER, dataloggerSettingButtonData);
    _buttonOperation.setActiveObject(dataloggerSettingButtonData[0][0]);
    _activeButton = dataloggerSettingButtonData[0][0];
}

DataLoggerSettingViewModel::~DataLoggerSettingViewModel()
{

}

void DataLoggerSettingViewModel::createConnection(QObject *qml, QObject *mainFrame)
{
    AbstractViewModel::createConnection(qml, mainFrame);
}

void DataLoggerSettingViewModel::activate()
{
    AbstractViewModel::activate();

    if (nullptr == _view) {
        qErrnoWarning("_view is null");
        return;

    }

    // サブに対するウェイクアップ要求が行われていなければ行う
    if (!_model->wakeUpRequested()) {
        _model->setWakeUpRequested(true);
        _model->wakeUp(1);
    }

    connect(this, SIGNAL(requestSetTakeOver(QVariant, QVariant)), _view, SLOT(backup(QVariant, QVariant)));
    connect(this, SIGNAL(removeAttribute(QString)), _mainFrame, SLOT(onRemoveAttribute(QString)));

    // ヘッダメニュー表示
    _model->replaceMenuList(MenuListIconIndex::FUNCTION_ICON, "icon_menu_logger.png");

    // 取り込みデータの有無でボタン操作・表示を設定
    buttonOperationVisibleSetting();
}

void DataLoggerSettingViewModel::deactivate()
{
    AbstractViewModel::deactivate();
    disconnect(this, SIGNAL(requestSetTakeOver(QVariant, QVariant)), _view, SLOT(backup(QVariant, QVariant)));
    disconnect(this, SIGNAL(removeAttribute(QString)), _mainFrame, SLOT(onRemoveAttribute(QString)));

}

void DataLoggerSettingViewModel::setTakeOver()
{
    if (_activeButton == "m1") {
        emit requestSetTakeOver("userDataloggerFunction", CHANNEL_EDIT_DATA_LOGGER);
    } else if (_activeButton == "m2") {
        emit requestSetTakeOver("userDataloggerFunction", CHANNEL_EDIT_DATA_LOGGER_USB1);
    } else if (_activeButton == "m3") {
        emit requestSetTakeOver("userDataloggerFunction", CHANNEL_EDIT_DATA_LOGGER_USB2);
    }
}

void DataLoggerSettingViewModel::onRequest(int condition)
{
    emit request(_screenId, condition);
}

void DataLoggerSettingViewModel::moveCursor(ButtonId buttonId)
{
    if ((buttonId == BUTTON_RIGHT) || (buttonId == BUTTON_LEFT) || (buttonId == BUTTON_UP) || (buttonId == BUTTON_DOWN)) {
        // UP, DOWN なし

        QString nextBtn;
        bool ret = _buttonOperation.nextActiveObject(buttonId, nextBtn);

        if (ret) {

            // inactive
            QObject *target = _view->findChild<QObject *>(_activeButton);

            if (target) {
                target->setProperty("active", false);
            }

            // active
            target = _view->findChild<QObject *>(nextBtn);

            if (target) {
                target->setProperty("active", true);
            }

            _activeButton = nextBtn;
            _buttonOperation.setActiveObject(_activeButton);
        }
    }
}

void DataLoggerSettingViewModel::setBoolProperty(QString objectName, QString propertyName, bool value)
{
    QObject *target = nullptr;
    if (_view) {
        target = _view->findChild<QObject *>(objectName);
    }
    if (target) {
        target->setProperty(propertyName.toUtf8().constData(), value);
    }
}

void DataLoggerSettingViewModel::onKeyStatus(ButtonId id)
{
    if (id == BUTTON_ENTER) {
        QObject *target = _view->findChild<QObject *>(_activeButton);
        target->setProperty("press", false);

        if (isErrorUsbStatus()) {
            return;
        }

        if (_activeButton == "m4") {            // 本体⇛USB　一括コピー
            emit requestToShowPopup(POPUP_COPY_ALL_TO_USB_USER_SETTING_DATA_CONFIRMATION);
        } else if (_activeButton == "m5") {     // USB⇛本体　一括コピー
            emit requestToShowPopup(POPUP_COPY_ALL_TO_BODY_SELECT_USER_SETTING_DATA_CONFIRMATION);
        } else {
            int button_index = _activeButton.mid(4).toInt() - 1; // ”func" 文字列を削除して数字を取り出す
            onRequest(button_index);
        }
    } else if (id == BUTTON_ESCAPE || id == BUTTON_HOME) {
        emit back();
    } else if (id == BUTTON_USER) {
#ifdef NEW_USER_SETTING
        emit requestDirect(SCREEN_USER_SETTING, true);
#else
        emit requestDirect(SCREEN_USER_DATA_LOGGER_MENU, true);
#endif
    } else {
        moveCursor(id);
    }
}

void DataLoggerSettingViewModel::onAccept(PopupId popupId, bool /*okOnly*/)
{
    if (popupId == POPUP_COPY_ALL_TO_USB_USER_SETTING_DATA_CONFIRMATION) {
        // USB メモリーが未装着ならボタンが押せたとしてもポップアップを表示して戻る
        if (_model->getDeviceStatus().usba != LC70_STRAGE_FORMAT) {
            emit requestToShowPopup(POPUP_FAILURE_NO_USB_CONFIRMATION);
            return;
        }
        // コピー(本体⇒USB)
        copyUserMeasurementSettingDataToUsb();
    } else {
        // nothing to do
    }
}

void DataLoggerSettingViewModel::onSelect(PopupId popupId, int index)
{
    if (popupId == POPUP_COPY_ALL_TO_BODY_SELECT_USER_SETTING_DATA_CONFIRMATION) {
        // コピー(USB⇒本体)
        copyUserMeasurementSettingDataToBody(index);

        // 取り込みデータの有無でボタン操作・表示を設定
        buttonOperationVisibleSetting();

        return;
    }
}

void DataLoggerSettingViewModel::onReject(PopupId popupId)
{
    Q_UNUSED(popupId)
}

void DataLoggerSettingViewModel::onButtonPress(ButtonId id)
{
    if (id == BUTTON_ENTER) {
        QObject *target = _view->findChild<QObject *>(_activeButton);
        target->setProperty("press", true);
    }
}

void DataLoggerSettingViewModel::copyUserMeasurementSettingDataToUsb()
{
    UsbErrorType error = UsbErrorType::NON_ERROR;

    QString copyTarget = _model->applicationSetting()->settingDataPathLogger();

    QStringList copyTargetPathList = {};

    error = UsbOperation::checkExistsDirectory(copyTarget);

    if (error == UsbErrorType::NON_ERROR) {
        copyTargetPathList.append(copyTarget);
    }

    // データロガーのファイル判定


    if (copyTargetPathList.isEmpty()) {
        // コピー対象ファイルが無い場合は以降の処理は実施しない
        return;
    }

    error = UsbErrorType::NON_ERROR;

    // コピー可能な状態かのチェックを行う
    error = UsbOperation::checkCopyable(copyTargetPathList);

    if (error == UsbErrorType::COPY_FILE_ERROR) {
        // コピー元ファイルがない
        emit requestToShowPopup(POPUP_NOTHING_COPY_FILE);
        return;
    } else if (error == UsbErrorType::SPACE_ERROR) {
        // USBの空き容量が足りない
        emit requestToShowPopup(POPUP_USB_NO_SPACE);
        return;
    } else {
        // nothing to do
    }
    error = UsbOperation::copyToUsb(UsbDataType::
                                    DATALOGGER_MEASUREMENT_SETTING, copyTarget, true, _model->applicationSetting()->password());

    if (error == UsbErrorType::COPY_EXEC_ERROR) {
        // コピー失敗
        emit requestToShowPopup(POPUP_FAILURE_COPY_EXECUTE);
    }
}

void DataLoggerSettingViewModel::copyUserMeasurementSettingDataToBody(int index)
{
    UsbErrorType error = UsbErrorType::NON_ERROR;

    // USBから本体へフォルダコピー
    if(index == 0) {
        error = UsbOperation::copyToBody(UsbDataType::DATALOGGER_MEASUREMENT_SETTING,
                                         _model->applicationSetting()->settingDataPathLoggerUsb1(), true, _model->applicationSetting()->password());
    } else if(index == 1) {
        error = UsbOperation::copyToBody(UsbDataType::DATALOGGER_MEASUREMENT_SETTING,
                                         _model->applicationSetting()->settingDataPathLoggerUsb2(), true, _model->applicationSetting()->password());
    }

    if (error == UsbErrorType::COPY_EXEC_ERROR) {
        // コピー失敗
        emit requestToShowPopup(POPUP_FAILURE_COPY_EXECUTE);
    } else if (error == UsbErrorType::COPY_FILE_ERROR) {
        // コピー元ファイルがない
        emit requestToShowPopup(POPUP_NOTHING_COPY_FILE);
    } else {
        // コピーしたファイルを読み込み直す
        _model->initializeUserSettingFile();
    }
}

bool DataLoggerSettingViewModel::isErrorUsbStatus()
{
    QMap<QString, QList<LC70_STRAGE_T>> enableUsbStatusMap = {
        {"m4", {LC70_STRAGE_FORMAT}},
        {"m5", {LC70_STRAGE_FORMAT}},
    };

    if (enableUsbStatusMap.count(_activeButton) == 0) {
        // USBに無関係のボタン
        return false;
    }

    if (enableUsbStatusMap.value(_activeButton).contains(_model->getDeviceStatus().usba)) {
        // 実行可能なUSB状態になっている
        return false;
    }

    // ボタンごとに定義している、押下可能なUSB状態になっていなければエラー表示
    emit requestToShowPopup(POPUP_FAILURE_NO_USB_CONFIRMATION); // USB未装着エラー
    return true;
}

void DataLoggerSettingViewModel::buttonOperationVisibleSetting()
{
    bool readData1Mode = false;
    bool readData2Mode = false;
    UsbErrorType error = UsbErrorType::NON_ERROR;
    QString copyTarget = _model->applicationSetting()->settingDataPathLoggerUsb1();
    error = UsbOperation::checkExistsSettingFile(copyTarget);
    if (error == UsbErrorType::NON_ERROR) {
        if (_view) {
            _view->setProperty("readData1", true);
            readData1Mode = true;
        }
    } else {
        if (_view) {
            _view->setProperty("readData1", false);
            readData1Mode = false;
        }
    }
    error = UsbErrorType::NON_ERROR;
    copyTarget = _model->applicationSetting()->settingDataPathLoggerUsb2();
    error = UsbOperation::checkExistsSettingFile(copyTarget);
    if (error == UsbErrorType::NON_ERROR) {
        if (_view) {
            _view->setProperty("readData2", true);
            readData2Mode = true;
        }
    } else {
        if (_view) {
            _view->setProperty("readData2", false);
            readData2Mode = false;
        }
    }

    // 選択中のアイコンをリセット
    setBoolProperty(_activeButton, "active", false);

    if ( (readData1Mode == false) && (readData2Mode == false) ){
        _buttonOperation.setButtonName(BUTTON_COLUMN_NUMBER, BUTTON_ROW_NUMBER, dataloggerSettingButtonData);
        _buttonOperation.setActiveObject(dataloggerSettingButtonData[0][0]);
        _activeButton = dataloggerSettingButtonData[0][0];
    } else if ( (readData1Mode == true) && (readData2Mode == false) ){
        _buttonOperation.setButtonName(BUTTON_COLUMN_NUMBER, BUTTON_ROW_NUMBER, readData1DataloggerSettingButtonData);
        _buttonOperation.setActiveObject(readData1DataloggerSettingButtonData[0][0]);
        _activeButton = readData1DataloggerSettingButtonData[0][0];
    } else if ( (readData1Mode == false) && (readData2Mode == true) ){
        _buttonOperation.setButtonName(BUTTON_COLUMN_NUMBER, BUTTON_ROW_NUMBER, readData2DataloggerSettingButtonData);
        _buttonOperation.setActiveObject(readData2DataloggerSettingButtonData[0][0]);
        _activeButton = readData2DataloggerSettingButtonData[0][0];
    } else if ( (readData1Mode == true) && (readData2Mode == true) ){
        _buttonOperation.setButtonName(BUTTON_COLUMN_NUMBER, BUTTON_ROW_NUMBER, allDataDataloggerSettingButtonData);
        _buttonOperation.setActiveObject(allDataDataloggerSettingButtonData[0][0]);
        _activeButton = allDataDataloggerSettingButtonData[0][0];
    }

    // 選択中のアイコンの設定
    setBoolProperty(_activeButton, "active", true);
}
