/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include "dataloggerdatalistviewmodel.h"
#include "usboperation.h"

const static QMap<ButtonId, int> LIST_INDEX_UPDATE_MAP = {
    {BUTTON_UP, -1},
    {BUTTON_DOWN, 1}
};

static const int LIST_ROWS = 6;
static const QString EXTENSION_CSV = "csv";

DataloggerDataListViewModel::DataloggerDataListViewModel(ScreenId screenId, LevelCheckerModel *&model,
        QObject *parent)
    : AbstractViewModel(screenId, model, parent)
    , _listData(new MeasurementResultFileComponentList())
    , _listIndex(0)
    , _totalPage(1)
    , _currentPage(1)
{
    setObjectName("dataloggerDataListViewModel");
}

DataloggerDataListViewModel::~DataloggerDataListViewModel()
{

}

void DataloggerDataListViewModel::activate()
{
    if (_view == nullptr) {
        qErrnoWarning("_view is null");
        return;
    }

    // コネクションの生成
    connect(this, SIGNAL(requestSetTakeOver(QVariant, QVariant)), _view, SLOT(backup(QVariant, QVariant)));
    connect(this, SIGNAL(scrollPage(QVariant)), _view, SLOT(onScrollPage(QVariant)));

    // データロガーデータ取得結果の取得
    QList<QFileInfo> fileList;
    fileList = _model->dataLoggerFileList();

    // 作成日でソートを行う
    QList<QFileInfo> sortedFileList = sortCreateDate(fileList);

    QList<MeasurementResultFileListItems> items;

    int index = 0;
    for (index = 0; index < sortedFileList.size(); ++index) {
        items.append(MeasurementResultFileListItems(index + 1, sortedFileList[index].fileName(), sortedFileList[index].absoluteFilePath(),
                     sortedFileList[index].completeSuffix()));
    }

    // ダミー行の追加
    int surplus = items.count() % LIST_ROWS;

    if (surplus != 0) {
        // ページを満たすためのダミー行の追加
        surplus = LIST_ROWS - surplus;
        for (int i = 0; i < surplus; ++i) {
            items.append(MeasurementResultFileListItems(index + 1, "", ""));
            ++index;
        }
    }

    // データが1件もなければ全ての行に対してダミーを追加する
    if (items.count() == 0) {
        for (int i = 0; i < LIST_ROWS; ++i) {
            items.append(MeasurementResultFileListItems(index + 1, "", ""));
            ++index;
        }
    }

    _listData->setListData(items);

    if (_listData->getFileName(_listIndex).isEmpty()) {
        // カーソルが当たっているインデックスが空の場合
        // リストの先頭にカーソルを当てる
        _listIndex = 0;
    }

    // リストの総ページ数設定
    _totalPage = _listData->rowCount() / LIST_ROWS;

    if (_listData->rowCount() % LIST_ROWS > 0) {
        _totalPage += 1;
    }

    // 画面へ反映
    if (_view) {
        _view->setProperty("totalPage", _totalPage);
    }
    if (_view) {
        _view->setProperty("listModel", QVariant::fromValue(_listData.data()));
    }
    if (_view) {
        _view->setProperty("currentIndex", QVariant::fromValue(_listIndex));
    }

    // 現在のページを求める
    calcCurrentPage();
}

void DataloggerDataListViewModel::suspend()
{
    // コネクションの破棄
    disconnect(this, SIGNAL(requestSetTakeOver(QVariant, QVariant)), _view, SLOT(backup(QVariant, QVariant)));
    disconnect(this, SIGNAL(scrollPage(QVariant)), _view, SLOT(onScrollPage(QVariant)));

    // ローカル変数を初期化する
    _totalPage = 1;
    _currentPage = 1;
    AbstractViewModel::suspend();
}

void DataloggerDataListViewModel::setTakeOver()
{
    if (_listData->rowCount() == 0 || (_listData->rowCount() - 1 < _listIndex)) {
        return;
    }

    if (_listData->getExtension(_listIndex) == EXTENSION_CSV) {
        emit requestSetTakeOver("DataLoggerResultDataCsv", _listData->getFilePath(_listIndex));
    } else {
        // nothing to do
    }
}

void DataloggerDataListViewModel::onKeyStatus(ButtonId id)
{
    AbstractViewModel::onKeyStatus(id);

    if (id == BUTTON_ENTER) {
        if (_listData->rowCount() == 0 || _listIndex < 0) {
            return;
        }

        if (_listData->rowCount() - 1 < _listIndex) {
            return;
        }

        QString fileName = _listData->getFileName(_listIndex);

        if (!_listData->getFileName(_listIndex).isEmpty()) {
            onRequest(0);
        } else {
            // データなし
        }
    } else if (id == BUTTON_USER) {
#ifdef NEW_USER_SETTING
        emit requestDirect(SCREEN_USER_SETTING, true);
#else
        emit requestDirect(SCREEN_USER_DATA_LOGGER_MENU, true);
#endif
    } else {
        moveCursor(id);
    }
}

void DataloggerDataListViewModel::onRequest(int condition)
{
    emit request(_screenId, condition);
}

void DataloggerDataListViewModel::moveCursor(ButtonId buttonId)
{
    if ((buttonId == BUTTON_UP) || (buttonId == BUTTON_DOWN)) {
        if (_listData->rowCount() == 0) {
            return;
        }

        // リスト選択行の更新
        _listIndex += LIST_INDEX_UPDATE_MAP.value(buttonId, 0);

        // 上下限値のチェック
        if (_listIndex < 0) {
            _listIndex = _listData->rowCount() - 1;
        } else if (_listIndex == _listData->rowCount()) {
            _listIndex = 0;
        } else {
            // nothing to do
        }

        // 現在のページを求める
        calcCurrentPage();

        if (_view) {
            _view->setProperty("currentIndex", QVariant::fromValue(_listIndex));
        }
    }
}

QList<QFileInfo> DataloggerDataListViewModel::sortCreateDate(QList<QFileInfo> dataList)
{
    // 作成日でソートを実施
    int i = 0;
    for (i = 0; i < dataList.size() - 1; ++i) {
        int maxIndex = i;
        for (int j = i + 1; j < dataList.size(); ++j) {
            if(dataList[maxIndex].metadataChangeTime() < dataList[j].metadataChangeTime()) {
                maxIndex = j;
            }
        }
        if (i != maxIndex) {
            // swap処理を実施
            dataList.swap(i, maxIndex);
        }
    }

    return dataList;
}

void DataloggerDataListViewModel::calcCurrentPage()
{
    int page = (_listIndex / LIST_ROWS) + 1;

    if (page != _currentPage) {
        _currentPage = page;
        emit scrollPage(QVariant::fromValue(_currentPage));
    }
}
