/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include "dataloggerdetailsdataviewmodel.h"

static constexpr int LIST_COLUMN = 11;              // 衛星放送以外の列数
static constexpr int FIRST_DATA_ROW = 6;            // データの先頭行(それ以前の行はヘッダー)
static constexpr int LIST_ROWS = 10;                 // リスト1ページ当たりの表示件数

DataLoggerDetailsDataViewModel::DataLoggerDetailsDataViewModel(ScreenId screenId, LevelCheckerModel *&model,
        QObject *parent)
    : AbstractViewModel(screenId, model, parent)
    , _list(nullptr)
    , _measureResultModel(nullptr)
    , _totalPage(1)
    , _currentPage(1)
    , _currentRowIndex(0)
    , _band(LC70_BAND_UNDEF)
    , _channel()
    , _level()
    , _mer()
    , _ber()
    , _frequency()
    , _mode()
    , _layer()
    , _polarization()
    , _localFrequency()
    , _listData()
    , _listDataNum(0)
{
    setObjectName("dataLoggerDetailsDataViewModel");
}

DataLoggerDetailsDataViewModel::~DataLoggerDetailsDataViewModel()
{
}

void DataLoggerDetailsDataViewModel::suspend()
{
    AbstractViewModel::suspend();

    // QML オブジェクトへのポインターをクリアー
    _list = nullptr;
    _measureResultModel = nullptr;
}

void DataLoggerDetailsDataViewModel::createConnection(QObject *qml, QObject *mainFrame)
{
    AbstractViewModel::createConnection(qml, mainFrame);
    connect(this, SIGNAL(scrollPage(QVariant)), _view, SLOT(onScrollPage(QVariant)));
}

void DataLoggerDetailsDataViewModel::activate()
{
    AbstractViewModel::activate();

    // 各画面
    _list = _model->search("detailsList");

    // 測定結果リスト(MeasureResultModel の実体(objectName:measureResultModel)を QML から参照)
    _measureResultModel = dynamic_cast<MeasureResultModel *>(_model->search("dataLoggerMeasureResultModel"));

    // 画面間共有情報からcsvファイル情報を取得し読み込み
    QString filePath = _takeOver.value("DataLoggerResultDataCsv").toString();
    QFile file(filePath);

    if (!file.open(QIODevice::ReadOnly)) {
        qErrnoWarning("file open error");
        return;
    }

    _currentRowIndex = 0;

    QTextStream in(&file);
    QStringList csvLineText = {};
    MeasureResultRow item;
    _listData = QList<MeasureResultRow>();

    int row = 0;

    while (!in.atEnd()) {
        csvLineText = in.readLine().split(",");

        ++row;

        if (row < FIRST_DATA_ROW) {
            continue;
        }

        if (csvLineText.size() != LIST_COLUMN) {
            continue;
        }

        item.Timestamp = csvLineText[0];
        item.Channel = csvLineText[1];
        item.Band = csvLineText[2];
//        item.Frequency = Correcter::displayFrequency(Correcter::bandIdFromDisplayBand(csvLineText[2]), csvLineText[3].toUInt());
        item.Frequency = csvLineText[3];
        item.Level = csvLineText[4];
        item.Mer = csvLineText[5];
        item.Ber = csvLineText[6];
        item.Layer = csvLineText[7];
        item.Polarization = csvLineText[7];
        item.LocalOscillator = csvLineText[8];
        item.BroadcastSystem = csvLineText[9];
        item.Modulation = csvLineText[10];

        _listData.append(item);
    }

    file.close();

    _listDataNum = _listData.count();
    while((_listData.count() % LIST_ROWS) != 0) {
        MeasureResultRow dummy;
        _listData.append(dummy);
    }
    _measureResultModel->setDataMax(LIST_ROWS);
    _measureResultModel->setDataNum(_listDataNum);
    _measureResultModel->append(_listData.mid(0, LIST_ROWS));
    if (_list) {
        _list->setProperty("baseIndex", 0);
    }

    // リストの総ページ数設定
    _totalPage = _listDataNum / LIST_ROWS;

    if (_listDataNum % LIST_ROWS > 0) {
        _totalPage += 1;
    }

    _currentPage = 1;
    if (_view) {
        _view->setProperty("totalPage", _totalPage);        // 総ページ
    }
    if (_view) {
        _view->setProperty("currentPage", _currentPage);    // 現在のページ
    }

    // データ名表示
    if (_view) {
        QStringList filename = filePath.split("/");
        _view->setProperty("fileName", filename[filename.count() - 1]);
    }

    changeRow();
}

void DataLoggerDetailsDataViewModel::deactivate()
{
    _band = LC70_BAND_UNDEF;
    _channel.clear();
    _level.clear();
    _mer.clear();
    _ber.clear();
    _frequency.clear();
    _mode.clear();
    _layer.clear();
    _polarization.clear();
    _localFrequency.clear();

    AbstractViewModel::deactivate();
}

void DataLoggerDetailsDataViewModel::recover()
{
    _list = _model->search("detailsList");
}

void DataLoggerDetailsDataViewModel::onKeyStatus(ButtonId id)
{
    AbstractViewModel::onKeyStatus(id);

    if ((id == BUTTON_CHANNEL_UP) || (id == ButtonId::BUTTON_DOWN)) {
        _currentRowIndex = nextRow();

        if (_currentRowIndex < 0) {
            return;
        }
        updatePage();

        // チャンネル変更
        changeRow();

    } else if ((id == BUTTON_CHANNEL_DOWN) || (id == ButtonId::BUTTON_UP)) {
        _currentRowIndex = previousRow();

        if (_currentRowIndex < 0) {
            return;
        }
        updatePage();

        // チャンネル変更
        changeRow();

    } else {
        // nothing to do
    }
}

int DataLoggerDetailsDataViewModel::nextRow()
{
    ++_currentRowIndex;

    if (_currentRowIndex > _listDataNum - 1) {
        _currentRowIndex = 0;
    }

    return _currentRowIndex;
}

int DataLoggerDetailsDataViewModel::previousRow()
{
    --_currentRowIndex;

    if (_currentRowIndex < 0) {
        _currentRowIndex = _listDataNum - 1;
    }

    return _currentRowIndex;
}

void DataLoggerDetailsDataViewModel::changeRow()
{
    QMap<QString, QString> polarizationMap = {
        { "右旋", "R-Turn" },
        { "左旋", "L-Turn" },
        { "水平", "horizontal" },
        { "垂直", "vertical" },
    };
    QModelIndex index = _measureResultModel->index(_currentRowIndex % LIST_ROWS);
    _band = Correcter::bandIdFromDisplayBand(_measureResultModel->data(index, BandRole).toString());
    _channel = _measureResultModel->data(index, ChannelRole).toString();
    _level = _measureResultModel->data(index, LevelRole).toString();
    _mer = _measureResultModel->data(index, MerRole).toString();
    _ber = _measureResultModel->data(index, BerRole).toString();
    _mode = _measureResultModel->data(index, ModulationRole).toString();
    _layer = _measureResultModel->data(index, LayerRole).toString();
    _polarization = polarizationMap.value(_measureResultModel->data(index, PolarizationRole).toString());
    _frequency = _measureResultModel->data(index, FrequencyRole).toString();
    _localFrequency = _measureResultModel->data(index, LocalFrequencyRole).toString();

    // 画面にチャンネル情報を反映
    if (_view) {
        _view->setProperty("band", Correcter::displayBandName(_band));
    }
    if (_view) {
        _view->setProperty("channel", Correcter::displayChannelName(_channel));
    }
    if (_view) {
        _view->setProperty("frequency", _frequency);
    }
    if (_view) {
        _view->setProperty("level", _level);
    }
    if (_view) {
        _view->setProperty("mer", _mer);
    }
    if (_view) {
        _view->setProperty("ber", _ber);
    }
    if (_view) {
        _view->setProperty("polarization", _polarization);
    }
    if (_view) {
        _view->setProperty("localFrequency", _localFrequency);
    }
    QString modulation = Correcter::displayModulation(_mode.toUInt());
    QString modulationPrefix = Correcter::modulationPrefix(_measureResultModel->data(index, BroadbandSystem).toUInt(), modulation);
    int displayModulationPrefixMode = Correcter::displayModulationPrefixMode(_measureResultModel->data(index, BroadbandSystem).toUInt());

    if (_band == LC70_BAND_PILOT || _band == LC70_BAND_CATV_UP_STREAM) {
        // 選択しているリストのBANDがPILOTの場合、変調方式CWの固定値を表示
        if (_view) {
            _view->setProperty("displayModulationPrefixMode", DISPLAY_MODULATION_PREFIX_TOGETHER);
        }
        if (_view) {
            _view->setProperty("modulation", CW);
        }
        if (_view) {
            _view->setProperty("modulationPrefix", QString(""));
        }
    }else if (_band == LC70_BAND_FM) {
        // 選択しているリストのBANDがFMの場合、変調方式FMの固定値を表示
        if (_view) {
            _view->setProperty("displayModulationPrefixMode", DISPLAY_MODULATION_PREFIX_TOGETHER);
        }
        if (_view) {
            _view->setProperty("modulation", FM);
        }
        if (_view) {
            _view->setProperty("modulationPrefix", QString(""));
        }
    } else {
        if (_view) {
            _view->setProperty("displayModulationPrefixMode", displayModulationPrefixMode);
        }
        if (_view) {
            _view->setProperty("modulation", modulation);
        }
        if (_view) {
            _view->setProperty("modulationPrefix", modulationPrefix);
        }
    }

    if (_view) {
        _view->setProperty("isSatelliteBroadCasting", Correcter::isSatelliteBroadCasting(_band));
    }
    if (_view) {
        _view->setProperty("hierarchy", Correcter::displayHierarchy(_layer.toUInt()));
    }
    if (_view) {
        _view->setProperty("polarization", Correcter::displayPolarization(_polarization));
    }
    if (_view) {
        _view->setProperty("isModulationVisible",Correcter::isModulationVisible(_band));
    }
    // チャンネル詳細データの表示可否を通知
    if (_view) {
        _view->setProperty("displayChannelDetail", Correcter::getDisplayChannelDetail(_band));
    }
    // 選択中のチャンネルのインデックス
    if (_list) {
        _list->setProperty("currentChannelIndex", _currentRowIndex);
    }

    // CN/MERの選択状態をqmlに送信する
    if (_view) {
        if ( (_band == LC70_BAND_UHF) || (_band == LC70_BAND_CATV_ALL) ||
             (_band == LC70_BAND_CATV_90_470MHZ) || (_band == LC70_BAND_CATV_470_1002MHZ) ||
             (_band == LC70_BAND_SKYP) ){
            // MER
            _view->setProperty("selectCnMer", 1);
        } else {
            // CN
            _view->setProperty("selectCnMer", 0);
        }
    }
}

void DataLoggerDetailsDataViewModel::updatePage()
{
    int startIndex = (_currentPage - 1) * LIST_ROWS;
    int endIndex = (_currentPage * LIST_ROWS) - 1;
    if (_currentRowIndex < startIndex || _currentRowIndex > endIndex) {
        _currentPage = (_currentRowIndex / LIST_ROWS) + 1;
        int baseIndex = (_currentPage - 1) * LIST_ROWS;
        _measureResultModel->setPageData(_listData.mid(baseIndex, baseIndex + LIST_ROWS > _listData.count() ? -1 : LIST_ROWS));
        if (_list) {
            _list->setProperty("baseIndex", baseIndex);
        }
        emit scrollPage(_currentPage);
    }
}

void DataLoggerDetailsDataViewModel::updateRow()
{
    int startIndex = (_currentPage - 1) * LIST_ROWS;
    int endIndex = (_currentPage * LIST_ROWS) - 1;
    if (_currentRowIndex < startIndex || _currentRowIndex > endIndex) {
        _currentRowIndex = startIndex + (_currentRowIndex % LIST_ROWS);
    }
    while(_currentRowIndex >= _listDataNum) {
        _currentRowIndex = previousRow();
    }
}
