/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef DATALOGGERDETAILSDATAVIEWMODEL_H
#define DATALOGGERDETAILSDATAVIEWMODEL_H

#include "abstractviewmodel.h"
#include "buttonoperation.h"
#include "correcter.h"
#include "measureresultmodel.h"

class DataLoggerDetailsDataViewModel : public AbstractViewModel
{
    Q_OBJECT
public:
    /**
     * @brief コンストラクター
     * @param screenId 画面 ID
     * @param model Model クラスのポインター
     * @param parent 親オブジェクト
     */
    explicit DataLoggerDetailsDataViewModel(ScreenId screenId, LevelCheckerModel *&model, QObject *parent = nullptr);

    /**
     * @brief デストラクタ
     */
    virtual ~DataLoggerDetailsDataViewModel();

    /**
     * @brief 画面を閉じるときに呼び出すスレッドの停止処理
     */
    virtual void suspend(void) override;

    /**
     * @brief 画面を開くときに呼び出す qml との接続処理
     * @param qml qml のオブジェクトのポインター
     * @param mainFrame MainFrameViewModel のポインター
     */
    virtual void createConnection(QObject *qml, QObject *mainFrame) override;

    /**
     * @brief 画面Active 時の処理
     *
     * 画面表示後データ収集開始
     */
    virtual void activate() override;

    /**
     * @brief 画面を閉じるときに呼び出す表示な用の保存処理
     */
    virtual void deactivate(void) override;

signals:
    /**
     * @brief 測定結果取得開始、再開依頼を通知
     * @param condition マルチ測定条件
     * @param prefferedCondition 優先測定条件
     * @pram displayType 遷移先画面種別
     * @param interval 測定結果取得間隔
     */
    void resumeMultiMeasuring(LC70_MEAS_MULTI_T *condition, LC70_MEAS_SINGLE_T *prefferedCondition, LC70_DISP_TYPE_T displayType, uint32_t interval);

    /**
     * @brief 測定結果取得停止依頼を通知
     */
    void suspendMultiMeasuring(void);

    /**
     * @brief チャンネル変更
     * @param condition チャンネル別測定条件
     */
    void changeCondition(LC70_MEAS_SINGLE_T *condition);

    /**
     * @brief リストのページを移動
     * @param page ページ
     */
    void scrollPage(QVariant page);

protected:
    /**
     * @brief 必要に応じて View を指すポインターを付け替えた後の復帰処理を行う
     */
    virtual void recover(void) override;

protected slots:
    /**
     * @brief ボタン押下の状態を受信
     * @param id ボタン ID
     */
    virtual void onKeyStatus(ButtonId id) override;

private:
    QObject *_list;                             //!< リストの QML
    MeasureResultModel *_measureResultModel;    //!< 測定結果リストのモデル(View の実体を参照)
    int _totalPage;                             //!< リストのページ数
    int _currentPage;                           //!< リストの現在のページ
    int _currentRowIndex;                       //!< 選択行のインデックス
    int _band;                                  //!< Band
    QString _channel;                           //!< チャンネル
    QString _level;                             //!< LEVEL
    QString _mer;                               //!< 変調誤差比
    QString _ber;                               //!< ビット誤り率
    QString _frequency;                             //!< 周波数
    QString _mode;                              //!< 変調方式
    QString _layer;                             //!< 階層
    QString _polarization;                      //!< 偏波
    QString _localFrequency;                    //!< 局発周波数
    QList<MeasureResultRow> _listData;          //!< リストデータ
    int _listDataNum;                           //!< リストデータ数

    /**
     * @brief 次の有効な行のインデックスを返す
     * @return 次の有効な行のインデックス
     */
    int nextRow(void);

    /**
     * @brief 一つ前の有効な行のインデックスを返す
     * @return 一つ前の有効な行のインデックス
     */
    int previousRow(void);

    /**
     * @brief 選択行変更
     */
    void changeRow(void);

    /**
     * @brief 表示ページ更新
     */
    void updatePage(void);

    /**
     * @brief 選択行更新
     */
    void updateRow(void);
};

#endif // DATALOGGERDETAILSDATAVIEWMODEL_H
