/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include "datamanagementmenuviewmodel.h"
#include <QTimer>
#include "usboperation.h"

// メニューマップ(測定データ、データロガー、ユーザ設定、USBフォーマット)　フォーマット済み USBが接続中
QVector<QVector<QString>> dataManagementMenuButtonDataWithFormattedUsb = {
    {{"m0"}, {"m1"}, {"m3"}},
};

const int DataManagementMenuViewModel::BUTTON_COLUMN_NUMBER = 1;
const int DataManagementMenuViewModel::BUTTON_ROW_NUMBER = 3;

DataManagementMenuViewModel::DataManagementMenuViewModel(ScreenId screenId, LevelCheckerModel *&model, QObject *parent)
    : AbstractViewModel(screenId, model, parent)
{
    setObjectName("dataManagementMenuViewModel");
    _buttonOperation.setButtonName(BUTTON_COLUMN_NUMBER, BUTTON_ROW_NUMBER, dataManagementMenuButtonDataWithFormattedUsb);
    _buttonOperation.setActiveObject(dataManagementMenuButtonDataWithFormattedUsb[0][0]);
    _activeButton = dataManagementMenuButtonDataWithFormattedUsb[0][0];
}

DataManagementMenuViewModel::~DataManagementMenuViewModel()
{

}

void DataManagementMenuViewModel::activate()
{
    if (_view == nullptr) {
        qErrnoWarning("_view is null");
        return;
    }

    // ヘッダメニュー表示
    _model->replaceMenuList(MenuListIconIndex::FUNCTION_ICON, "icon_menu_data.png");

    // 画面表示時の初期選択アイコンを設定
    setBoolProperty(_activeButton, "active", true);
}

void DataManagementMenuViewModel::onKeyStatus(ButtonId id)
{
    AbstractViewModel::onKeyStatus(id);

    if (id == BUTTON_ENTER) {
        QObject *target = nullptr;
        if (_view) {
            target = _view->findChild<QObject *>(_activeButton);
        }
        if (target) {
            target->setProperty("press", false);
        }
        if (isErrorUsbStatus()) {
            return;
        }

        if (_activeButton == "m3") {
            emit requestToShowPopup(POPUP_USB_FORMAT_CONFIRMATION); // USBフォーマット実行確認
        } else {
            int button_index = _activeButton.mid(1).toInt();
            onRequest(button_index);
        }
    } else if (id == BUTTON_USER) {
#ifdef NEW_USER_SETTING
        emit requestDirect(SCREEN_USER_SETTING, true);
#else
        emit requestDirect(SCREEN_USER_DATA_LOGGER_MENU, true);
#endif
    } else {
        moveCursor(id);
    }
}

void DataManagementMenuViewModel::onButtonPress(ButtonId id)
{
    if (id == BUTTON_ENTER) {
        QObject *target = nullptr;
        if (_view) {
            target = _view->findChild<QObject *>(_activeButton);
        }
        if (target) {
            target->setProperty("press", true);
        }
    }
}

void DataManagementMenuViewModel::onAccept(PopupId popupId, bool okOnly)
{
    AbstractViewModel::onAccept(popupId, okOnly);

    if (popupId == POPUP_USB_FORMAT_CONFIRMATION) {
        // USBのフォーマットを実施
        emit requestToShowPopup(POPUP_USB_FORMAT_RUNNING);
        QTimer::singleShot(300, this, SLOT(startUsbFormat()));
    }
}

void DataManagementMenuViewModel::onReject(PopupId popupId)
{
    Q_UNUSED(popupId)
}

void DataManagementMenuViewModel::onRequest(int condition)
{
    emit request(_screenId, condition);
}

void DataManagementMenuViewModel::moveCursor(ButtonId buttonId)
{
    if ((buttonId == BUTTON_RIGHT) || (buttonId == BUTTON_LEFT)) {

        QString nextButton;
        bool ret = _buttonOperation.nextActiveObject(buttonId, nextButton);

        if (ret) {

            // inactive
            setBoolProperty(_activeButton, "active", false);

            // active
            setBoolProperty(nextButton, "active", true);

            _activeButton = nextButton;
            _buttonOperation.setActiveObject(_activeButton);
        }
    }
}

void DataManagementMenuViewModel::startUsbFormat()
{
    if (UsbOperation::format() != UsbErrorType::NON_ERROR) {
        // エラー
        emit requestPopupClose();
        QTimer::singleShot(100, this, SLOT(displayFormatErrorPopup()));
        return;
    }

    // フォーマット完了
    emit requestPopupClose();
}

void DataManagementMenuViewModel::displayFormatErrorPopup()
{
    emit requestToShowPopup(POPUP_FAILURE_FORMAT_EXECUTE);
}

bool DataManagementMenuViewModel::isErrorUsbStatus()
{
    QMap<QString, QList<LC70_STRAGE_T>> enableUsbStatusMap = {
        {"m3", {LC70_STRAGE_FORMAT, LC70_STRAGE_UNFORMAT}},
    };

    if (enableUsbStatusMap.count(_activeButton) == 0) {
        // USBに無関係のボタン
        return false;
    }

    if (enableUsbStatusMap.value(_activeButton).contains(_model->getDeviceStatus().usba)) {
        // 実行可能なUSB状態になっている
        return false;
    }

    // ボタンごとに定義している、押下可能なUSB状態になっていなければエラー表示
    setBoolProperty(_activeButton, "press", false);
    emit requestToShowPopup(POPUP_FAILURE_NO_USB_CONFIRMATION); // USB未装着エラー
    return true;
}

void DataManagementMenuViewModel::setBoolProperty(QString objectName, QString propertyName, bool value)
{
    QObject *target = nullptr;
    if (_view) {
        target = _view->findChild<QObject *>(objectName);
    }
    if (target) {
        target->setProperty(propertyName.toUtf8().constData(), value);
    }

}
