/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include "measurementdatamenuviewmodel.h"
#include "usboperation.h"

// メニューマップ(測定データ表示、USBへコピー削除) USB接続中
QVector<QVector<QString>> measurementDataMenuButtonDataWithUsb = {
    {{"m0"}, {"m1"}},
};

const int MeasurementDataMenuViewModel::BUTTON_COLUMN_NUMBER = 1;
const int MeasurementDataMenuViewModel::BUTTON_ROW_NUMBER = 2;

MeasurementDataMenuViewModel::MeasurementDataMenuViewModel(ScreenId screenId, LevelCheckerModel *&model,
        QObject *parent)
    : AbstractViewModel(screenId, model, parent)
{
    setObjectName("measurementDataMenuViewModel");
    _buttonOperation.setButtonName(BUTTON_COLUMN_NUMBER, BUTTON_ROW_NUMBER, measurementDataMenuButtonDataWithUsb);
    _buttonOperation.setActiveObject(measurementDataMenuButtonDataWithUsb[0][0]);
    _activeButton = measurementDataMenuButtonDataWithUsb[0][0];
}

MeasurementDataMenuViewModel::~MeasurementDataMenuViewModel()
{

}

void MeasurementDataMenuViewModel::activate()
{
    connect(this, SIGNAL(requestSetTakeOver(QVariant, QVariant)), _view, SLOT(backup(QVariant, QVariant)));

    if (_view == nullptr) {
        qErrnoWarning("_view is null");
        return;
    }
    _model->replaceMenuList(MenuListIconIndex::FUNCTION_ICON, "icon_menu_data.png");

    // 先頭のボタンをアクティブにする
    resetToInitialButtonState(measurementDataMenuButtonDataWithUsb[0][0], _activeButton, _buttonOperation);
}

void MeasurementDataMenuViewModel::deactivate()
{
    AbstractViewModel::deactivate();

    disconnect(this, SIGNAL(requestSetTakeOver(QVariant, QVariant)), _view, SLOT(backup(QVariant, QVariant)));
}

void MeasurementDataMenuViewModel::onKeyStatus(ButtonId id)
{
    AbstractViewModel::onKeyStatus(id);

    if (id == BUTTON_ENTER) {
        QObject *target = nullptr;
        if (_view) {
            target = _view->findChild<QObject *>(_activeButton);
        }
        if (target) {
            target->setProperty("press", false);
        }
//        if (isErrorUsbStatus()) {
//            return;
//        }
        int button_index = _activeButton.mid(1).toInt();
        if (_activeButton == "m1") {
            onRequest(button_index);
        } else {
            int button_index = _activeButton.mid(1).toInt();
            emit requestSetTakeOver("selecteMenu", 1);
            onRequest(button_index);
        }
    } else if (id == BUTTON_USER) {
#ifdef NEW_USER_SETTING
        emit requestDirect(SCREEN_USER_SETTING, true);
#else
        emit requestDirect(SCREEN_USER_DATA_LOGGER_MENU, true);
#endif
    } else {
        moveCursor(id);
    }
}

void MeasurementDataMenuViewModel::onAccept(PopupId popupId, bool okOnly)
{
    AbstractViewModel::onAccept(popupId, okOnly);

    if (popupId == POPUP_DELETE_ALL_MEASUREMENT_CONFIRMATION) {
        // 測定データの一括削除
        removeAllMeasurementResultData();
    } else {
        // nothing to do
    }
}

void MeasurementDataMenuViewModel::onReject(PopupId popupId)
{
    Q_UNUSED(popupId)
}

void MeasurementDataMenuViewModel::onRequest(int condition)
{
    emit request(_screenId, condition);
}

void MeasurementDataMenuViewModel::moveCursor(ButtonId buttonId)
{
    if ((buttonId == BUTTON_RIGHT) || (buttonId == BUTTON_LEFT)) {

        QString nextButton;
        bool ret = _buttonOperation.nextActiveObject(buttonId, nextButton);

        if (ret) {

            // inactive
            setBoolProperty(_activeButton, "active", false);

            // active
            setBoolProperty(nextButton, "active", true);

            _activeButton = nextButton;
            _buttonOperation.setActiveObject(_activeButton);
        }
    }
}

void MeasurementDataMenuViewModel::onButtonPress(ButtonId id)
{
    if (id == BUTTON_ENTER) {
        QObject *target = nullptr;
        if (_view) {
            target = _view->findChild<QObject *>(_activeButton);
        }
        if (target) {
            target->setProperty("press", true);
        }
    }
}

void MeasurementDataMenuViewModel::removeAllMeasurementResultData()
{
    QStringList pathList = {
        _model->applicationSetting()->resultDataPathSingle(),
        _model->applicationSetting()->resultDataPathMulti()
    };

    for (int i = 0; i < pathList.size(); ++i) {
        qDebug() << "measuring Path List.at " << i << pathList.at(i);
        _model->removePreservedData(pathList.at(i), true);
    }
}

void MeasurementDataMenuViewModel::copyMeasurementResultData()
{
    QStringList pathList = {
        _model->applicationSetting()->resultDataPathSingle(),
        _model->applicationSetting()->resultDataPathMulti()
    };

    QList<UsbDataType> typeList = {
        UsbDataType::MEASUREMENT_RESULT_SINGLE,
        UsbDataType::MEASUREMENT_RESULT_MULTI
    };

    UsbErrorType error = UsbErrorType::NON_ERROR;

    // コピー処理を行うパスを決める
    QStringList copyTargetPath = {};

    for (int i = 0; i < pathList.size(); ++i) {
        error = UsbOperation::checkExistsDirectory(pathList.at(i));

        if (error == UsbErrorType::NON_ERROR) {
            copyTargetPath.append(pathList.at(i));
        }
    }

    if (copyTargetPath.isEmpty()) {
        // コピー対象ファイルが無い場合は以降の処理は実施しない
        return;
    }

    error = UsbErrorType::NON_ERROR;

    // コピー可能な状態かのチェックを行う
    error = UsbOperation::checkCopyable(copyTargetPath);

    if (error == UsbErrorType::COPY_FILE_ERROR) {
        // コピー元ファイルなし
        emit requestToShowPopup(POPUP_NOTHING_COPY_FILE);
        return;
    } else if (error == UsbErrorType::SPACE_ERROR) {
        // USBの空き容量が足りない
        emit requestToShowPopup(POPUP_USB_NO_SPACE);
        return;
    } else {
        // nothing to do
    }

    // コピー処理を実施
    for (int i = 0; i < copyTargetPath.size(); ++i) {
        error = UsbOperation::copyToUsb(typeList[i], pathList[i], false, _model->applicationSetting()->password());

        if (error == UsbErrorType::COPY_EXEC_ERROR) {
            // コピー実施エラー
            emit requestToShowPopup(POPUP_FAILURE_COPY_EXECUTE);
            return;
        }
    }
}

bool MeasurementDataMenuViewModel::isErrorUsbStatus()
{
    QMap<QString, QList<LC70_STRAGE_T>> enableUsbStatusMap = {
        {"m1", {LC70_STRAGE_FORMAT}},
    };

    if (enableUsbStatusMap.count(_activeButton) == 0) {
        // USBに無関係のボタン
        return false;
    }

    if (enableUsbStatusMap.value(_activeButton).contains(_model->getDeviceStatus().usba)) {
        // 実行可能なUSB状態になっている
        return false;
    }

    // ボタンごとに定義している、押下可能なUSB状態になっていなければエラー表示
    setBoolProperty(_activeButton, "press", false);
    emit requestToShowPopup(POPUP_FAILURE_NO_USB_CONFIRMATION); // USB未装着エラー
    return true;
}

void MeasurementDataMenuViewModel::setBoolProperty(QString objectName, QString propertyName, bool value)
{
    QObject *target = nullptr;
    if (_view) {
        target = _view->findChild<QObject *>(objectName);
    }
    if (target) {
        target->setProperty(propertyName.toUtf8().constData(), value);
    }
}
