/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include "measurementresultcopydeleteviewmodel.h"

const static QMap<ButtonId, int> LIST_INDEX_UPDATE_MAP = {
    {BUTTON_UP, -1},
    {BUTTON_DOWN, 1}
};

static const int LIST_ROWS = 5;

static constexpr const char *SINGLE_BUTTON = "m1";       //!< シングル測定(qml ファイルの objectName プロパティと合わせておく)
static constexpr const char *MULTI_BUTTON = "m2";        //!< マルチ測定(qml ファイルの objectName プロパティと合わせておく)
static constexpr const char *DATALOGGER_BUTTON = "m3";   //!< データロガー(qml ファイルの objectName プロパティと合わせておく)
static constexpr const char *LIST_BUTTON = "m4";         //!< リスト(qml ファイルの objectName プロパティと合わせておく)

static constexpr int MEASUREMENT_INDEX_NONE = 0;             //!< ボタン選択なし
static constexpr int MEASUREMENT_INDEX_SINGLE = 1;           //!< シングル測定選択
static constexpr int MEASUREMENT_INDEX_MULTI = 2;            //!< マルチ測定選択
static constexpr int MEASUREMENT_INDEX_DATALOGGER = 3;       //!< データロガー選択

static constexpr const char *SELECT_BUTTON = "selectButton"; //!< 全選択 / 解除(qml ファイルの objectName プロパティと合わせておく)
static constexpr const char *COPY_BUTTON = "copyButton";     //!< コピー(qml ファイルの objectName プロパティと合わせておく)
static constexpr const char *DELETE_BUTTON = "deleteButton"; //!< 削除(qml ファイルの objectName プロパティと合わせておく)

static constexpr int ACTION_INDEX_NONE = 0;            //!< ボタン選択なし
static constexpr int ACTION_INDEX_SELECT = 1;          //!< 全選択 / 解除選択
static constexpr int ACTION_INDEX_COPY = 2;            //!< コピー選択
static constexpr int ACTION_INDEX_DELETE = 3;          //!< 削除選択

static constexpr int COPY = 0;                          //!< コピー
static constexpr int REMOVE = 1;                        //!< 削除

MeasurementResultCopyDeleteViewModel::MeasurementResultCopyDeleteViewModel(ScreenId screenId, LevelCheckerModel *&model, QObject *parent)
    : AbstractViewModel(screenId, model, parent)
    , _listData(new MeasurementResultOperationList())
    , _listIndex(0)
    , _totalPage(1)
    , _currentPage(1)
    , _selectArea(MEASUREMENT_AREA)
    , _currentMeasurementIndex(MEASUREMENT_INDEX_SINGLE)
    , _currentActionIndex(ACTION_INDEX_SELECT)
    , _editItemList()
    , _temporaryFileName()
{
    setObjectName("MeasurementResultCopyDelete");
}

MeasurementResultCopyDeleteViewModel::~MeasurementResultCopyDeleteViewModel()
{
}

void MeasurementResultCopyDeleteViewModel::createConnection(QObject *qml, QObject *mainFrame)
{
    AbstractViewModel::createConnection(qml, mainFrame);

    connect(this, SIGNAL(showIndicator(bool)), mainFrame, SLOT(onShowIndicator(bool)));
    connect(this, SIGNAL(executeCopy(int)), this, SLOT(onExecuteOperation(int)), Qt::QueuedConnection);
    connect(this, SIGNAL(removeMeasurement(void)), this, SLOT(onRemoveMeasurementResultData(void)), Qt::QueuedConnection);
}

void MeasurementResultCopyDeleteViewModel::activate()
{
    if (_view == nullptr) {
        qErrnoWarning("_view is null");
        return;
    }

    // コネクション
    connect(this, SIGNAL(scrollPage(QVariant)), _view, SLOT(onScrollPage(QVariant)));

    // 画面表示時は測定選択から
    _selectArea = MEASUREMENT_AREA;
    _currentMeasurementIndex = MEASUREMENT_INDEX_SINGLE;
    if (_view) {
        _view->setProperty("activeArea", _selectArea);
        _view->setProperty("currentMeasurementIndex", _currentMeasurementIndex);
    }

    // ファイルリスト更新
    updateFileList();
}

void MeasurementResultCopyDeleteViewModel::deactivate()
{
    AbstractViewModel::deactivate();

    disconnect(this, SIGNAL(showIndicator(bool)), _mainFrame, SLOT(onShowIndicator(bool)));
    disconnect(this, SIGNAL(executeCopy(int)), this, SLOT(onExecuteOperation(int)));
    disconnect(this, SIGNAL(removeMeasurement(void)), this, SLOT(onRemoveMeasurementResultData(void)));

    // ローカル変数を初期値戻す
    _listIndex = 0;
    _totalPage = 1;
    _currentPage = 1;
}

void MeasurementResultCopyDeleteViewModel::suspend()
{
    // コネクションの破棄
    disconnect(this, SIGNAL(scrollPage(QVariant)), _view, SLOT(onScrollPage(QVariant)));

    // ローカル変数を初期化する
    _listIndex = 0;
    _totalPage = 1;
    _currentPage = 1;

    AbstractViewModel::suspend();
}

void MeasurementResultCopyDeleteViewModel::onKeyStatus(ButtonId id)
{
    AbstractViewModel::onKeyStatus(id);

    if (id == BUTTON_USER) {
#ifdef NEW_USER_SETTING
        emit requestDirect(SCREEN_USER_SETTING, true);
#else
        emit requestDirect(SCREEN_USER_DATA_LOGGER_MENU, true);
#endif
    }

    switch (_selectArea) {
    case MEASUREMENT_AREA:
        keyStatusMeasurementArea(id);
        break;
    case FILELIST_AREA:
        keyStatusFileListArea(id);
        break;
    case ACTION_AREA:
        keyStatusActionArea(id);
        break;
    }
}

void MeasurementResultCopyDeleteViewModel::pressAllSelectCancelButton()
{
    // 未選択な項目がないかチェックを行う
    bool existsUnSelect = false;
    for (int index = 0; index < _listData->rowCount(); ++index) {
        if (_listData->getFilePath(index).isEmpty()) {
            break;
        }
        if (_listData->getIsSelected(index) == false) {
            existsUnSelect = true;
            break;
        }
    }

    if (existsUnSelect) {
        // 未選択な項目が有る場合は、全選択を行う
        for (int index = 0; index < _listData->rowCount(); ++index) {
            if (_listData->getFilePath(index).isEmpty()) {
                break;
            }
            _listData->setIsSelected(index, true);
        }
    } else {
        // 未選択な項目が無い場合、全解除を行う
        for (int index = 0; index < _listData->rowCount(); ++index) {
            if (_listData->getFilePath(index).isEmpty()) {
                break;
            }
            _listData->setIsSelected(index, false);
        }
    }
}

void MeasurementResultCopyDeleteViewModel::updateFileList()
{
    QStringList dirList;
    dirList.clear();

    if(_currentMeasurementIndex == MEASUREMENT_INDEX_SINGLE) {
        dirList.append(_model->applicationSetting()->resultDataPathSingle());
    } else if(_currentMeasurementIndex == MEASUREMENT_INDEX_MULTI) {
        dirList.append(_model->applicationSetting()->resultDataPathMulti());
    } else if(_currentMeasurementIndex == MEASUREMENT_INDEX_DATALOGGER) {
        dirList.append(_model->applicationSetting()->resultDataPathLogger());
    }

    QList<MeasurementResultOperationListItems> items = {};

    // 作成日でソートを実施したリスト表示用のアイテムを取得
    QList<QFileInfo> dataList = displayDataList(dirList);

    int index = 1;
    for (int i = 0; i < dataList.size(); ++i) {
        QString userDataName;
        QString dataFormat;
        // 画面表示用のデータ名と日付に分割したものを用意
        if(_currentMeasurementIndex == MEASUREMENT_INDEX_DATALOGGER) {
            // ファイル名
            QStringList dataNameSplitList = dataList[i].fileName().split("_");
            for (int len = 0; len < dataNameSplitList.size() - 1; ++len) {
                if(len != 0) {
                    userDataName = userDataName + QString("_");
                }
                userDataName = userDataName + dataNameSplitList[len];
            }
            userDataName = userDataName + QString(".") + dataList[i].completeSuffix();
            // 日時
            QString data = dataNameSplitList[dataNameSplitList.size() - 1];
#if 1	//20210625_dx5
            dataFormat = data.mid(0, 4) + ("/") + data.mid(4, 2) + ("/") + data.mid(6, 2) + ("　")
                        + data.mid(8, 2) + (":") + data.mid(10, 2) + (":") + data.mid(12, 2);
#else
            dataFormat = data.mid(0, 4) + ("/") + data.mid(4, 2) + ("/") + data.mid(6, 2) + ("　") + data.mid(8, 2) + (":") + data.mid(10, 2);
#endif
        } else {
            // ファイル名
            QStringList dataNameSplitList = dataList[i].fileName().split("_");
            for (int len = 0; len < dataNameSplitList.size() - 2; ++len) {
                if(len != 0) {
                    userDataName = userDataName + QString("_");
                }
                userDataName = userDataName + dataNameSplitList[len];
            }
            userDataName = userDataName + QString(".") + dataList[i].completeSuffix();
            // 日時
            QString data = dataNameSplitList[dataNameSplitList.size() - 2];
#if 1	//20210625_dx5            
            dataFormat = data.mid(0, 4) + ("/") + data.mid(4, 2) + ("/") + data.mid(6, 2) + ("　")
                        + data.mid(8, 2) + (":") + data.mid(10, 2) + (":") + data.mid(12, 2);
#else
            dataFormat = data.mid(0, 4) + ("/") + data.mid(4, 2) + ("/") + data.mid(6, 2) + ("　") + data.mid(8, 2) + (":") + data.mid(10, 2);
#endif
        }

        items.append(MeasurementResultOperationListItems(index, dataList[i].fileName(), dataList[i].absoluteFilePath(), false
                                                         ,userDataName, dataFormat));
        ++index;
    }

    // ダミー行の追加
    int surplus = items.count() % LIST_ROWS;

    if (surplus != 0) {
        // ページを満たすためのダミー行の追加
        surplus = LIST_ROWS - surplus;
        for (int i = 0; i < surplus; ++i) {
            items.append(MeasurementResultOperationListItems(index,"",""));
            ++index;
        }
    }

    // データが1件もなければ全ての行に対してダミーを追加する
    if (items.count() == 0) {
        for (int i = 0; i < LIST_ROWS; ++i) {
            items.append(MeasurementResultOperationListItems(index, "", ""));
            ++index;
        }
    }

    _listData->setListData(items);

    if (_listData->getFileName(_listIndex).isEmpty()) {
        // カーソルが当たっているインデックスが空の場合
        // リストの先頭にカーソルを当てる
        _listIndex = 0;
    }

    // リストの総ページ数設定
    _totalPage = _listData->rowCount() / LIST_ROWS;

    if (_listData->rowCount() % LIST_ROWS > 0) {
        _totalPage += 1;
    }

    // 画面へ反映
    if (_view) {
        _view->setProperty("totalPage", _totalPage);
        _view->setProperty("listModel", QVariant::fromValue(_listData));
        _view->setProperty("currentIndex", QVariant::fromValue(_listIndex));
    }

    // 現在のページを求める
    calcCurrentPage();
}

QStringList MeasurementResultCopyDeleteViewModel::getTargetPathList()
{
    // 処理を行う対象パスのリストを取得
    QStringList targetPathList;
    for (int index = 0; index < _listData->rowCount(); ++index) {
        if (!(_listData->getFilePath(index).isEmpty()) && _listData->getIsSelected(index)){
            targetPathList.append(_listData->getFilePath(index));
        }
    }

    return targetPathList;
}

QList<QFileInfo> MeasurementResultCopyDeleteViewModel::displayDataList(QStringList dirList)
{
    // リストに表示するファイル拡張子の設定
//    QStringList extensionFilters = {"*.csv", "*.png"};

//    // ソート対象のファイルを取得
//    QList<QFileInfo> fileInfoList = {};

//    int i = 0;
//    while (i < dirList.size()) {
//        if (QDir(dirList[i]).exists()) {
//            QDirIterator it(dirList[i], extensionFilters, QDir::Files, QDirIterator::Subdirectories);

//            while(it.hasNext()) {
//                if (it.next().isEmpty()) {
//                    continue;
//                }
//                fileInfoList.append(it.fileInfo());
//            }
//        }
//        ++i;
//    }

//    // ファイル生成日でソートを実施する
//    for (i = 0; i < fileInfoList.size() - 1; ++i) {
//        int maxIndex = i;
//        for (int j = i + 1; j < fileInfoList.size(); ++j) {
//            if(fileInfoList[maxIndex].metadataChangeTime() < fileInfoList[j].metadataChangeTime()) {
//                maxIndex = j;
//            }
//        }
//        if (i != maxIndex) {
//            // swap処理を実施
//            fileInfoList.swap(i, maxIndex);
//        }
//    }

    // 対象のファイルを取得
    QList<QFileInfo> fileInfoList = {};

    QString index;
    if(_currentMeasurementIndex == MEASUREMENT_INDEX_SINGLE) {
        index = ("single");
    } else if(_currentMeasurementIndex == MEASUREMENT_INDEX_MULTI) {
        index = ("multi");
    } else if(_currentMeasurementIndex == MEASUREMENT_INDEX_DATALOGGER) {
        index = ("logger");
    }

    QString filePath = _model->applicationSetting()->measureDataFileListPath()
                        + ("/") + _model->applicationSetting()->measureDataFileListFileName();
    QFile file(filePath);

    if (!file.open(QIODevice::ReadOnly)) {
        qErrnoWarning("file open error");
        return fileInfoList;
    }

    QTextStream in(&file);
    QStringList csvLineText = {};
    QFileInfo fileinfo;
    QString toUpLow;
    int row = 0;
    while (!in.atEnd()) {
        csvLineText = in.readLine().split(",");

        ++row;

        toUpLow = csvLineText[1];
        if(toUpLow.toLower() == index) {
            if(_currentMeasurementIndex == MEASUREMENT_INDEX_DATALOGGER) {
                fileinfo = _model->applicationSetting()->measureDataFileListPath() + ("/")
                            + toUpLow.toLower() + ("/")
                            + csvLineText[2] + ("/")
                            + csvLineText[4];
            } else {

                fileinfo = _model->applicationSetting()->measureDataFileListPath() + ("/")
                            + toUpLow.toLower() + ("/")
                            + csvLineText[2] + ("/")
                            + csvLineText[3] + ("/")
                            + csvLineText[4];
            }
            fileInfoList.append(fileinfo);
        }
    }

    return fileInfoList;
}

void MeasurementResultCopyDeleteViewModel::calcCurrentPage()
{
    int page = (_listIndex / LIST_ROWS) + 1;

    if (page != _currentPage) {
        _currentPage = page;
        emit scrollPage(QVariant::fromValue(_currentPage));
    }
}

void MeasurementResultCopyDeleteViewModel::keyStatusMeasurementArea(ButtonId id)
{
    if ( (id == BUTTON_DOWN) || (id == BUTTON_ENTER) ){
        moveArea(FILELIST_AREA);
    } else if( (id == BUTTON_LEFT) || (id == BUTTON_RIGHT) ) {
        moveCursorMeasurementArea(id);
        // ファイルリスト更新
        updateFileList();
    }
}

void MeasurementResultCopyDeleteViewModel::moveCursorMeasurementArea(ButtonId id)
{
    qDebug() << QString("moveCursorMeasurementArea: %1").arg(id == BUTTON_LEFT ? "LEFT" : "RIGHT");
    int nextAction = _currentMeasurementIndex;
    do {
        nextAction = nextAction + (id == BUTTON_LEFT ? -1 : 1);
        qDebug() << QString("moveCursorMeasurementArea: next=%1").arg(nextAction);
        if (nextAction < MEASUREMENT_INDEX_SINGLE) {
            nextAction = MEASUREMENT_INDEX_DATALOGGER;
        }
        if (nextAction > MEASUREMENT_INDEX_DATALOGGER) {
            nextAction = MEASUREMENT_INDEX_SINGLE;
        }
        if ((nextAction == MEASUREMENT_INDEX_SINGLE)
                || (nextAction == MEASUREMENT_INDEX_MULTI)
                || (nextAction == MEASUREMENT_INDEX_DATALOGGER)) {
            qDebug() << QString("moveCursorMeasurementArea: %1 -> %2").arg(_currentMeasurementIndex).arg(nextAction);
            _currentMeasurementIndex = nextAction;
            if (_view) {
                _view->setProperty("currentMeasurementIndex", _currentMeasurementIndex);
            }
            return;
        }
    } while(nextAction != _currentMeasurementIndex);
}

void MeasurementResultCopyDeleteViewModel::keyStatusFileListArea(ButtonId id)
{
    if ((id == BUTTON_UP) || (id == BUTTON_DOWN)) {
        if(moveCursorFileListArea(id) == true) {
            moveArea(MEASUREMENT_AREA);
        }
    } else if (id == BUTTON_RIGHT) {
        moveArea(ACTION_AREA);
    } else if (id == BUTTON_ENTER) {
        // カーソルが当たっているリスト項目の選択の有無を変更する
        if (!(_listData->getFilePath(_listIndex).isEmpty())) {
            _listData->setIsSelected(_listIndex, !(_listData->getIsSelected(_listIndex)));
        }
    }
}

bool MeasurementResultCopyDeleteViewModel::moveCursorFileListArea(ButtonId id)
{
    if ((id == BUTTON_UP) || (id == BUTTON_DOWN)) {
        if (_listData->rowCount() == 0) {
            return false;
        }

        // リスト選択行の更新
        _listIndex += LIST_INDEX_UPDATE_MAP.value(id, 0);

        // 上下限値のチェック
        if (_listIndex < 0) {
//            _listIndex = _listData->rowCount() - 1;
            return true;
        } else if (_listData->getFileName(_listIndex).isEmpty()) {
            // カーソルが当たっているインデックスが空の場合
            // リストの先頭にカーソルを当てる
            _listIndex = 0;
        } else {
            // nothing to do
        }

        // 現在のページを求める
        calcCurrentPage();

        if (_view) {
            _view->setProperty("currentIndex", QVariant::fromValue(_listIndex));
        }
    }
    return false;
}

void MeasurementResultCopyDeleteViewModel::keyStatusActionArea(ButtonId id)
{
    if ((id == BUTTON_UP) || (id == BUTTON_DOWN)) {
        moveCursorActionArea(id);
    } else if (id == BUTTON_LEFT) {
        moveArea(FILELIST_AREA);
    } else if (id == BUTTON_ENTER) {
        if(_currentActionIndex == ACTION_INDEX_SELECT) {
            // 全選択/解除ボタン押下時
            pressAllSelectCancelButton();
        } else if(_currentActionIndex == ACTION_INDEX_COPY) {
            // コピーボタン押下時
            pressCopyButton();
        } else if(_currentActionIndex == ACTION_INDEX_DELETE) {
            // 削除ボタン押下時
            pressDeleteButton();
        }
    }
}

void MeasurementResultCopyDeleteViewModel::moveCursorActionArea(ButtonId id)
{
    qDebug() << QString("moveCursorActionArea: %1").arg(id == BUTTON_UP ? "UP" : "DOWN");
    int nextAction = _currentActionIndex;
    do {
        nextAction = nextAction + (id == BUTTON_UP ? -1 : 1);
        qDebug() << QString("moveCursorActionArea: next=%1").arg(nextAction);
        if (nextAction < ACTION_INDEX_SELECT) {
            nextAction = ACTION_INDEX_DELETE;
        }
        if (nextAction > ACTION_INDEX_DELETE) {
            nextAction = ACTION_INDEX_SELECT;
        }
        if ((nextAction == ACTION_INDEX_SELECT)
                || (nextAction == ACTION_INDEX_COPY)
                || (nextAction == ACTION_INDEX_DELETE)) {
            qDebug() << QString("moveCursorActionArea: %1 -> %2").arg(_currentActionIndex).arg(nextAction);
            _currentActionIndex = nextAction;
            if (_view) {
                _view->setProperty("currentActionIndex", _currentActionIndex);
            }
            return;
        }
    } while(nextAction != _currentActionIndex);
}

void MeasurementResultCopyDeleteViewModel::moveArea(SelectArea destination)
{
    if (_selectArea == destination) {
        return;
    }

    // _selectArea変更前処理
    if(_selectArea == MEASUREMENT_AREA) {
        // 測定選択からファイルリストに移動するときは必ずファイルリストの先頭から
        _listIndex = 0;
    }

    // _selectArea変更
    _selectArea = destination;
    if (_view) {
        _view->setProperty("activeArea", _selectArea);
    }

    // _selectArea変更後処理
    if(_selectArea == MEASUREMENT_AREA) {
        if (_view) {
            _view->setProperty("currentMeasurementIndex", _currentMeasurementIndex);
        }
    } else if(_selectArea == FILELIST_AREA) {
        if (_view) {
            _view->setProperty("currentIndex", QVariant::fromValue(_listIndex));
        }

        // 測定結果がない時は MEASUREMENT_AREA に移動
        if (_listData->getFileName(_listIndex).isEmpty()) {
            moveArea(MEASUREMENT_AREA);
        }
    } else if(_selectArea == ACTION_AREA) {
        _currentActionIndex = ACTION_INDEX_SELECT;
        if (_view) {
            _view->setProperty("currentActionIndex", _currentActionIndex);
        }
    }
}

void MeasurementResultCopyDeleteViewModel::pressCopyButton()
{
    // USB メモリーが未装着ならボタンが押せたとしてもポップアップを表示して戻る
    if (_model->getDeviceStatus().usba != LC70_STRAGE_FORMAT) {
        emit requestToShowPopup(POPUP_FAILURE_NO_USB_CONFIRMATION);
        return;
    }

    // コピー確認ダイアログを表示
    if(MeasurementResultCopyDeleteViewModel::getTargetPathList().isEmpty()) {
        // 処理対象のパスが空の場合 ダイアログは表示しない
        return;
    }

    emit requestToShowPopup(POPUP_CONFIRM_COPY_SELECTED_FILE);
}

void MeasurementResultCopyDeleteViewModel::pressDeleteButton()
{
    if (MeasurementResultCopyDeleteViewModel::getTargetPathList().isEmpty()) {
        // 処理対象のパスが空の場合 ダイアログは表示しない
        return;
    }

    // 削除確認ダイアログを表示
    emit requestToShowPopup(POPUP_CONFIRM_DELETE_SELECTED_FILE);
}

void MeasurementResultCopyDeleteViewModel::onAccept(PopupId popupId, bool okOnly)
{
    AbstractViewModel::onAccept(popupId, okOnly);

    Q_UNUSED(okOnly);
    if (popupId == POPUP_CONFIRM_DELETE_SELECTED_FILE) {
        // 削除中ダイアログを表示
        emit requestToShowPopup(POPUP_CONFIRM_DELETE);

        // 溜まったイベントの処理
        qApp->processEvents();

        // 削除処理通知
        emit removeMeasurement();
    } else if (popupId == POPUP_CONFIRM_COPY_SELECTED_FILE) {
        // コピー中ダイアログを表示
        emit requestToShowPopup(POPUP_CONFIRM_COPY);

        // 溜まったイベントの処理
        qApp->processEvents();

        // コピー処理通知
        emit executeCopy(ConfirmDataSaveDialogSelection::Copy);
    }
}

void MeasurementResultCopyDeleteViewModel::onSelect(PopupId popupId, int index)
{
//    if (popupId == POPUP_CONFIRM_COPY_SELECTED_FILE) {
//        // コピー中ダイアログを表示
//        emit requestToShowPopup(POPUP_CONFIRM_COPY);

//        // 溜まったイベントの処理
//        qApp->processEvents();

//        // コピー処理通知
//        emit executeCopy(index);
//    }
}

void MeasurementResultCopyDeleteViewModel::onReject(PopupId popupId)
{
    Q_UNUSED(popupId);
}

void MeasurementResultCopyDeleteViewModel::onExecuteOperation(int selectedButtonIndex)
{
    // 処理を行う対象のパスのリストを取得
    QStringList pathList = getTargetPathList();
    if (pathList.isEmpty()) {
        // 対象パスのリストが空の場合後続処理は実施しない
        // ポップアップを閉じる
        emit requestPopupClose();
        return;
    }
    if (selectedButtonIndex == ConfirmDataSaveDialogSelection::Copy) {
        // USB メモリーが未装着ならボタンが押せたとしてもポップアップを表示して戻る
        if (_model->getDeviceStatus().usba != LC70_STRAGE_FORMAT) {
            emit requestToShowPopup(POPUP_FAILURE_NO_USB_CONFIRMATION);
            return;
        }

        // コピーのみを実施
        copyFile();
    }
//    else if (selectedButtonIndex == ConfirmDataSaveDialogSelection::CopyAndDelete) {
//        // USB メモリーが未装着ならボタンが押せたとしてもポップアップを表示して戻る
//        if (_model->getDeviceStatus().usba != LC70_STRAGE_FORMAT) {
//            emit requestToShowPopup(POPUP_FAILURE_NO_USB_CONFIRMATION);
//            return;
//        }
//        // コピー後、コピー元のファイルを削除する
//        if (!copyFile()) {
//            // コピーに失敗したら削除しない
//            return;
//        }

//        // 削除中ダイアログを表示
//        emit requestToShowPopup(POPUP_CONFIRM_DELETE);

//        // 溜まったイベントの処理
//        qApp->processEvents();

//        // 削除
//        emit removeMeasurement();

//        // 溜まったイベントの処理
//        qApp->processEvents();
//    }

    // ポップアップを閉じる
    emit requestPopupClose();
}

void MeasurementResultCopyDeleteViewModel::copyMeasurementResultData(QStringList targetPathList)
{
    UsbErrorType error = UsbErrorType::NON_ERROR;

    // コピー処理を行うパスを決める
    QStringList copyTargetPath = {};
    for (int index = 0; index < targetPathList.size(); ++index) {
        error = UsbOperation::checkExistsFile(targetPathList.at(index));

        if (error == UsbErrorType::NON_ERROR) {
            copyTargetPath.append(targetPathList.at(index));
        }
    }

    if (copyTargetPath.isEmpty()) {
        // コピー対象ファイルが無い場合は、以降の処理は実施しない
        return;
    }

    error = UsbErrorType::NON_ERROR;

    // コピー可能な状態化のチェックを行う
    error = UsbOperation::checkCopyable(copyTargetPath, true);

    if (error == UsbErrorType::COPY_FILE_ERROR) {
        // コピー元ファイルなし
        emit requestToShowPopup(POPUP_NOTHING_COPY_FILE);
        return;
    } else if (error == UsbErrorType::SPACE_ERROR) {
        // USBの空き容量が足りない
        emit requestToShowPopup(POPUP_USB_NO_SPACE);
        return;
    }

    // コピー処理を実施
    for (int i = 0; i < copyTargetPath.size(); ++i) {

        UsbDataType fileType;
        if (copyTargetPath.at(i).contains("/single/", Qt::CaseInsensitive)){
            fileType = UsbDataType::MEASUREMENT_RESULT_SINGLE;
        } else if (copyTargetPath.at(i).contains("/multi/", Qt::CaseInsensitive)) {
            fileType = UsbDataType::MEASUREMENT_RESULT_MULTI;
        } else {
            fileType = UsbDataType::DATA_LOGGER_RESULT;
        }

        error = UsbOperation::copyToUsb(fileType, targetPathList[i], false, _model->applicationSetting()->password());

        if (error == UsbErrorType::COPY_EXEC_ERROR) {
            // コピー実施エラー
            emit requestToShowPopup(POPUP_FAILURE_COPY_EXECUTE);
            return;
        }
    }
}

bool MeasurementResultCopyDeleteViewModel::copyFile()
{
    QElapsedTimer timer;
    timer.start();

    UsbErrorType error = UsbErrorType::NON_ERROR;
    // 処理を行う対象のパスのリストを取得
    QStringList target = getTargetPathList();
    if (target.isEmpty()){
        // 選択リストが空の場合後続処理は実施しない
        return false;
    }

    error = UsbErrorType::NON_ERROR;

    // コピー可能な状態かチェックを行う
    error = UsbOperation::checkCopyable(target, true);

    if (error == UsbErrorType::COPY_FILE_ERROR) {
        // コピー元ファイルなし
        emit requestToShowPopup(POPUP_NOTHING_COPY_FILE);
        return false;
    } else if (error == UsbErrorType::SPACE_ERROR) {
        // USBの空き容量が足りない
        emit requestToShowPopup(POPUP_USB_NO_SPACE);
        return false;
    }

    // コピーする測定対象の選定
    QString source;
    QString destination;
    if (target[0].contains("/single/", Qt::CaseInsensitive)) {
        source = _model->applicationSetting()->resultDataPathSingle();
        destination = _model->applicationSetting()->resultDataCopyPathSingle();
    } else if (target[0].contains("/multi/", Qt::CaseInsensitive)) {
        source = _model->applicationSetting()->resultDataPathMulti();
        destination = _model->applicationSetting()->resultDataCopyPathMulti();
    } else {
        source = _model->applicationSetting()->resultDataPathLogger();
        destination = _model->applicationSetting()->resultDataCopyPathLogger();
    }

    // 削除対象ファイルのパスのリストを /tmp にファイル出力
    if (!_model->writeCandidateFiles(COPY, target, destination)) {
        // コピー実施エラー
        emit requestToShowPopup(POPUP_FAILURE_COPY_EXECUTE);
        return false;
    }

    // 選択されたファイルを USB メモリーにコピー
    qDebug() << QString("copy ready: %1 ms").arg(timer.elapsed());
    if (!_model->copyMeasurementFiles()) {
        // コピー実施エラー
        emit requestToShowPopup(POPUP_FAILURE_COPY_EXECUTE);
        return false;
    }
    qDebug() << QString("copied: %1 ms").arg(timer.elapsed());

    return true;
}

void MeasurementResultCopyDeleteViewModel::removeFile()
{
    QElapsedTimer timer;
    timer.start();

    QStringList target = getTargetPathList();
    if (target.isEmpty()){
        // 選択リストが空の場合後続処理は実施しない
        return;
    }

    // 削除対象ファイルのパスのリストを /tmp にファイル出力
    if (!_model->writeCandidateFiles(REMOVE, target)) {
        // 削除失敗ポップアップを表示する
        emit requestToShowPopup(POPUP_FAILURE_DELETE_EXECUTE);
    }

    // 空のディレクトリを削除するときの最上位の階層までのパス
    QString source;
    if (target[0].contains("/single/", Qt::CaseInsensitive)) {
        source = _model->applicationSetting()->resultDataPathSingle();
    } else if (target[0].contains("/multi/", Qt::CaseInsensitive)) {
        source = _model->applicationSetting()->resultDataPathMulti();
    } else {
        source = _model->applicationSetting()->resultDataPathLogger();
    }

    // 選択されたファイルをローカルから削除
    qDebug() << QString("remove ready: %1 ms").arg(timer.elapsed());
    if (!_model->removeMeasurementFiles(source)) {
        // 削除失敗ポップアップを表示する
        emit requestToShowPopup(POPUP_FAILURE_DELETE_EXECUTE);
    }
    qDebug() << QString("removed: %1 ms").arg(timer.elapsed());
}

QStringList MeasurementResultCopyDeleteViewModel::excludedFileList()
{
    // 処理対象外のパスのリストを取得
    QStringList targetPathList;
    for (int index = 0; index < _listData->rowCount(); ++index) {
        if (!_listData->getFilePath(index).isEmpty() && !_listData->getIsSelected(index)){
            targetPathList.append(_listData->getFilePath(index));
        }
    }

    return targetPathList;
}

void MeasurementResultCopyDeleteViewModel::onRemoveMeasurementResultData()
{
    QStringList targetPathList = MeasurementResultCopyDeleteViewModel::getTargetPathList();

    if (targetPathList.isEmpty()){
        // 選択リストが空の場合後続処理は実施しない
        // ポップアップを閉じる
        emit requestPopupClose();
        return;
    }

    // ファイルの削除
    removeFile();

    // ファイル名リスト更新時は、ディレクトリの中を確認して更新しているため、
    // 複数削除でも一回で反映される
    _model->updateMeasurementFileList();

    // ファイルリスト更新
    updateFileList();

    // リストの先頭にカーソルを当てる
    _listIndex = 0;

    // ポップアップを閉じる
    emit requestPopupClose();
}

