/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef MEASUREMENTDATARESULTCOPYDELETEVIEWMODEL_H
#define MEASUREMENTDATARESULTCOPYDELETEVIEWMODEL_H

#include "abstractviewmodel.h"
#include "measurementresultoperationlist.h"
#include "buttonoperation.h"

enum ConfirmDataSaveDialogSelection {
    Copy = 0,
    CopyAndDelete,
};

/**
 * @brief 測定データ操作画面のベースクラス
 */
class MeasurementResultCopyDeleteViewModel : public AbstractViewModel
{
    Q_OBJECT
signals:
    /**
     * @brief リストのページを移動
     */
    void scrollPage(QVariant);

    /**
     * @brief アップデート中のインジケーターの表示を通知
     * @param show 表示/非表示
     */
    void showIndicator(bool show);

    /**
     * @brief コピー処理を通知
     * @param selectedButtonIndex 選択したボタンindex
     */
    void executeCopy(int selectedButtonIndex);

    /**
     * @brief 測定データの削除を通知
     */
    void removeMeasurement(void);

public:
    /**
     * @brief コンストラクタ
     * @param ScreenId 画面 ID
     * @param model Model クラスのポインター
     * @param objectName 紐付けを行うQMLのObject名
     * @param parent 親オブジェクト
     */
    explicit MeasurementResultCopyDeleteViewModel(ScreenId ScreenId, LevelCheckerModel *&model, QObject *parent = nullptr);

    /**
     * @brief デストラクタ
     */
    virtual ~MeasurementResultCopyDeleteViewModel();

    /**
     * @brief 画面を開くときに呼び出す qml との接続処理
     * @param qml qml のオブジェクトのポインター
     * @param mainFrame MainFrameViewModel のポインター
     */
    virtual void createConnection(QObject *qml, QObject *mainFrame) override;

    /**
     * @brief 画面Active時の処理
     */
    virtual void activate(void) override;

    /**
     * @brief 画面Deactive時の処理
     */
    virtual void deactivate(void) override;

    /**
     * @brief 画面を閉じるときに呼び出すスレッド停止処理
     */
    virtual void suspend(void) override;

protected slots:
    /**
     * @brief ポプアップからの受諾の通知を受信
     * @param PopupId ポップアップのID
     * @param okonly okOnly OKのみの場合 true
     */
    virtual void onAccept(PopupId PopupId, bool okonly) override;

    /**
     * @brief ポップアップから選択の通知を受信
     * @param popupId ポップアップID
     * @param index 選択されたインデックス
     */
    virtual void onSelect(PopupId popupId, int index) override;

    /**
     * @brief ポップアップから拒否通知を受信
     * @param popupId ポップアップID
     */
    virtual void onReject(PopupId popupId) override;

protected:
    /**
     * @brief ボタン押下時イベント受信時の処理
     * @param id ボタンid
     */
    virtual void onKeyStatus(ButtonId id) override;

    /**
     * @brief 全選択/解除ボタン押下時の処理
     */
    void pressAllSelectCancelButton(void);

    /**
     * @brief ファイルリストの更新
     */
    void updateFileList(void);

    /**
     * @brief 処理対象のパスのリストを取得
     * @return 処理対象のパスリスト
     */
    QStringList getTargetPathList(void);

    /**
     * @brief コピー/削除対象外のファイルのパスのリストを返す
     * @return コピー/削除対象外のファイルのパスのリスト
     */
    QStringList excludedFileList(void);

    QSharedPointer<MeasurementResultOperationList> _listData; //!< 測定データリスト
    int _listIndex;                                           //!< 選択中のインデックス
    int _totalPage;                                           //!< リスト全ページ数
    int _currentPage;                                         //!< 選択中のページ数

private slots:
    /**
     * @brief 処理の実施を行う
     * @param selectedButtonIndex 選択したボタンindex
     */
    void onExecuteOperation(int selectedButtonIndex);

    /**
     * @brief 測定データの削除を実施
     * @param targetPathList 処理対象リスト
     */
    void onRemoveMeasurementResultData(void);

private:
    /**
     * @brief 操作対象のエリア定義
     */
    enum SelectArea {
        MEASUREMENT_AREA = 1,    // 測定選択リスト
        FILELIST_AREA  = 2,      // ファイルリスト
        ACTION_AREA  = 3,        // 動作リスト
    };

    SelectArea _selectArea;         //!< 選択している操作対象エリア
    QVariantList _editItemList;     //!< 編集項目のリスト
    QString _temporaryFileName;     //!< 仮のファイル名
    int _currentMeasurementIndex;   //!< 選択中の測定選択ボタン
    int _currentActionIndex;        //!< 選択中の動作ボタン

    /**
     * @brief リスト表示用のデータリストを取得
     * @param dirList リスト表示用の対象のファイルのディレクトリ名のリスト
     * @return リスト表示用のデータリスト
     * @note データリストは作成日でソートした状態となっている
     */
    QList<QFileInfo> displayDataList(QStringList dirList);

    /**
     * @brief 選択しているページを求める
     */
    void calcCurrentPage(void);

    /**
     * @brief 測定選択リストエリアでのボタン操作
     * @param id ボタン ID
     */
    void keyStatusMeasurementArea(ButtonId id);

    /**
     * @brief 測定選択リストエリアでのカーソル移動
     * @param id ボタン ID
     */
    void moveCursorMeasurementArea(ButtonId id);

    /**
     * @brief ファイルリストエリアでのボタン操作
     * @param id ボタン ID
     */
    void keyStatusFileListArea(ButtonId id);

    /**
     * @brief ファイルリストエリアでのカーソル移動
     * @param id ボタン ID
      * @return true:エリア移動する
     *         false:エリア移動しない
     */
    bool moveCursorFileListArea(ButtonId id);

    /**
     * @brief 動作ボタンエリアでのボタン操作
     * @param id ボタン ID
     */
    void keyStatusActionArea(ButtonId id);

    /**
     * @brief 動作ボタンエリアでのカーソル移動
     * @param id ボタン ID
     */
    void moveCursorActionArea(ButtonId id);

    /**
     * @brief エリア移動
     * @param destination 移動先エリア
     */
    void moveArea(SelectArea destination);

    /**
     * @brief コピー
     */
    void pressCopyButton();

    /**
     * @brief 削除
     */
    void pressDeleteButton();

    /**
     * @brief 測定データのコピーを実施
     * @param targetPathList 処理対象リスト
     */
    void copyMeasurementResultData(const QStringList targetPathList);

    /**
     * @brief 選択されたファイルをコピー
     * @return コピーの成否
     */
    bool copyFile(void);

    /**
     * @brief 選択されたファイルを削除
     */
    void removeFile(void);
};

#endif // MEASUREMENTDATARESULTCOPYDELETEVIEWMODEL_H
