/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef MEASUREMENTRESULTCSVCOMPONENTLIST_H
#define MEASUREMENTRESULTCSVCOMPONENTLIST_H

#include <QAbstractListModel>

/**
 * @brief 測定結果データ(CSV)項目
 */
enum MeasurementResultCsvColumn {
    Channel = 0,                   //!< チャンネル名称
    BandName,                      //!< 放送方式(BAND名)
    Frequency,                     //!< 周波数
    Level,                         //!< 測定入力レベル
    Mer,                           //!< 変調誤差比
    Ber,                           //!< ビット誤り率
    Layer,                         //!< 階層、偏波面
    LocalOscillator,               //!< 局発周波数
    BroadcastSystem,               //!< 放送方式規格
    Modulation,                    //!< 変調方式
    ColumnMax,                     //!< 項目数
};

/**
 * @brief リストアイテム
 */
struct MeasureResultCsvItems {
    QString band;               //!< BAND 種別
    QString channel;            //!< チャンネル番号
    QString frequency;          //!< 周波数
    QString localOscillator;    //!< 局発周波数
    QString broadcastSystem;    //!< 放送方式
    QString modulation;         //!< 変調方式
    QString layer;              //!< 階層、偏波面
    QString level;              //!< 測定入力レベル
    QString mer;                //!< 変調誤差比
    QString ber;                //!< ビット誤り率
    MeasureResultCsvItems(QString paramBand = "", QString paramChannel = "", QString paramFrequency = "",
                          QString paramLocalOscillator = "", QString paramBroadcastSystem = "", QString paramModulation = "",
                          QString paramLayer = "", QString paramLevel = "", QString paramMer = "", QString paramBer = "")
    {
        band            =   paramBand;
        channel         =   paramChannel;
        frequency       =   paramFrequency;
        localOscillator =   paramLocalOscillator;
        broadcastSystem =   paramBroadcastSystem;
        modulation      =   paramModulation;
        layer           =   paramLayer;
        level           =   paramLevel;
        mer             =   paramMer;
        ber             =   paramBer;
    }
};

/**
 * @brief 測定結果データ(CSV)のリストクラス
 */
class MeasurementResultCsvComponentList : public QAbstractListModel
{
    Q_OBJECT
    Q_ENUMS(MeasurementResultCsvRoles)
public:
    /**
     * @brief 測定結果データ(CSV)用ロール定義
     */
    enum MeasurementResultCsvRoles {
        BandRole = Qt::UserRole + 60,   //!< BAND 種別
        ChannelRole,                    //!< チャンネル番号
        FrequencyRole,                  //!< 周波数
        LocalOscillatorRole,            //!< 局発周波数
        BroadcastSystemRole,            //!< 放送方式
        ModulationRole,                 //!< 変調方式
        LayerRole,                      //!< 階層、偏波面
        LevelRole,                      //!< 測定入力レベル
        MerRole,                        //!< 変調誤差比
        BerRole,                        //!< ビット誤り率
    };

    /**
     * @brief コンストラクタ
     * @param parent 親オブジェクト
     */
    MeasurementResultCsvComponentList(QObject *parent = nullptr);

    /**
     * @brief デストラクター
     */
    ~MeasurementResultCsvComponentList();
    /**
     * @brief 測定結果数を返す
     * @return 測定結果数
     */
    int rowCount(const QModelIndex &parent = QModelIndex()) const override;

    /**
     * @brief Role情報取得
     * @return ロール名
     */
    QHash<int, QByteArray> roleNames() const override;

    /**
     * @brief 測定データ取得
     * @param index データの位置
     * @param role ロール
     * @return 測定データ
     */
    virtual QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const override;

    /**
     * @brief リストデータを設定する
     * @param items リストデータ
     */
    void setListData(QList<MeasureResultCsvItems> items);

    void setBand(int listIndex, const QString &band);
    QString getBand(int listIndex);

    void setChannel(int listIndex, const QString &channel);
    QString getChannel(int listIndex);

    void setFrequency(int listIndex, const QString &frequency);
    QString getFrequency(int listIndex);

    void setLocalOscillator(int listIndex, const QString &localOscillator);
    QString getLocalOscillator(int listIndex);

    void setBroadcastSystem(int listIndex, const QString &broadcastSystem);
    QString getBroadcastSystem(int listIndex);

    void setModulation(int listIndex, const QString &modulation);
    QString getModulation(int listIndex);

    void setLayer(int listIndex, const QString &layer);
    QString getLayer(int listIndex);

    void setLevel(int listIndex, const QString &level);
    QString getLevel(int listIndex);

    void setMer(int listIndex, const QString &mer);
    QString getMer(int listIndex);

    void setBer(int listIndex, const QString &ber);
    QString getBer(int listIndex);

private:
    QHash<int, QByteArray> _roles;         //!< ロールデータ
    QList<MeasureResultCsvItems> _data;    //!< リストデータ
};

#endif // MEASUREMENTRESULTCSVCOMPONENTLIST_H
