/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include "measurementresultcsvdataviewmodel.h"
#include "correcter.h"

const static QMap<ButtonId, int> LIST_INDEX_UPDATE_MAP = {
    {BUTTON_UP, -1},
    {BUTTON_DOWN, 1}
};

static const QString EMPTY_VALUE = "---";
static const int MINIMUM_LIST_SIZE = 10;
static const int HEADER_SIZE = 5;

MeasurementResultCsvDataViewModel::MeasurementResultCsvDataViewModel(ScreenId screenId, LevelCheckerModel *&model,
        QObject *parent)
    : AbstractViewModel(screenId, model, parent),
      _buttonOperation(),
      _listData(new MeasurementResultCsvComponentList()),
      _listIndex(0),
      _totalPage(1),
      _currentPage(1),
      _list(nullptr)
{
    setObjectName("measurementResultCsvDataViewModel");
}

MeasurementResultCsvDataViewModel::~MeasurementResultCsvDataViewModel()
{

}

void MeasurementResultCsvDataViewModel::activate()
{
    // コネクション
    connect(this, SIGNAL(scrollPage(QVariant)), _view, SLOT(onScrollPage(QVariant)));

    // 画面間共有情報からcsvファイル情報を取得し読み込み
    QString filePath = _takeOver.value("MeasurementResultDataCsv").toString();
    QFile file(filePath);

    if (!file.open(QIODevice::ReadOnly)) {
        qErrnoWarning("file open error");
        return;
    }

    QTextStream in(&file);
    QStringList csvLineText = {};
    QList<MeasureResultCsvItems> items;

    int currentLine = 0;

    int displayBandID = LC70_BAND_UNDEF;

    while (!in.atEnd()) {
        csvLineText = in.readLine().split(",");
        currentLine++;

        // [Memo]
        //  測定結果ファイルのヘッダー部(5行目まで)は、
        //  画面表示項目には必要のないデータであるため読み飛ばす。
        if (currentLine <= HEADER_SIZE) {
            continue;
        }

        if (csvLineText.size() != MeasurementResultCsvColumn::ColumnMax) {
            continue;
        }

        // 表示BANDを覚える
        if (static_cast<LC70_BAND_T>(displayBandID) == LC70_BAND_UNDEF) {
            displayBandID = Correcter::bandIdFromDisplayBand(csvLineText[MeasurementResultCsvColumn::BandName]);
        }

        // Level の値が閾値より低い値(<20 など)の場合はそのまま出力する
        bool ok = false;
        QString level;
        csvLineText[MeasurementResultCsvColumn::Level].toDouble(&ok);
        if (ok) {
            level = QString("%1").arg(csvLineText[MeasurementResultCsvColumn::Level].toDouble() , 4, 'f', 1);
        } else {
            level = csvLineText[MeasurementResultCsvColumn::Level];
        }

        QString mer;               // Ber
        QString ber;               // Mer

        QString mervalue = csvLineText[MeasurementResultCsvColumn::Mer];
        QString bervalue = csvLineText[MeasurementResultCsvColumn::Ber];

        // MER の値が <5.0 や >35 だった場合はそのまま出力する
        mervalue.toDouble(&ok);
        if (ok) {
            mer = QString("%1").arg(mervalue.toDouble(), 4, 'f', 1);
        } else {
            mer = mervalue;
        }

        // BER は出力されている値が対数なのでそのまま出力する
        if (bervalue == QString("---")) {
            ber = QString("---");
        } else {
            ber = bervalue;
        }

        QString modulation = csvLineText[MeasurementResultCsvColumn::Modulation];

        if(modulation == QString("---")) {
            modulation = QString("");
        }

        QString hierarchyPolarization = csvLineText[MeasurementResultCsvColumn::Layer];

        if(hierarchyPolarization == QString("---")) {
            hierarchyPolarization = QString("");
        }

        items.append(MeasureResultCsvItems(csvLineText[MeasurementResultCsvColumn::BandName],
                                           csvLineText[MeasurementResultCsvColumn::Channel],
                                           csvLineText[MeasurementResultCsvColumn::Frequency],
                                           csvLineText[MeasurementResultCsvColumn::LocalOscillator],
                                           csvLineText[MeasurementResultCsvColumn::BroadcastSystem],
                                           modulation,
                                           hierarchyPolarization,
                                           level,
                                           mer,
                                           ber));
    }

    file.close();

    // リスト行が最低数に達するまで空データを追加する
    while (items.size() < MINIMUM_LIST_SIZE) {
        items.append(MeasureResultCsvItems("", "", "", "", "", "", "", "", "", ""));
    }

    // リスト行で割り切れない場合ダミー行を追加し、ページ送りが不自然にならないようにする
    int dummyRowCount = items.size() % MINIMUM_LIST_SIZE;
    if (dummyRowCount > 0) {
        dummyRowCount = MINIMUM_LIST_SIZE - dummyRowCount;
        for (int i = 0; i < dummyRowCount; ++i) {
            items.append(MeasureResultCsvItems("", "", "", "", "", "", "", "", "", ""));
        }
    }

    // リストデータ設定
    _listData->setListData(items);

    // リスト情報
    if (_view) {
        _view->setProperty("listModel", QVariant::fromValue(_listData));
    }
    if (_view) {
        _view->setProperty("currentIndex", QVariant::fromValue(_listIndex));
    }
    // 変調方式 表示するかどうかを画面に送信する
    if (_view) {
        _view->setProperty("isModulationVisible",Correcter::isModulationVisible(displayBandID));
    }
    // データ名表示
    if (_view) {
        QStringList filename = filePath.split("/");
        _view->setProperty("fileName", filename[filename.count() - 1]);
    }
    // 画面情報の更新
    updateDisplayInfo();
    // リストボタン状態
    if (_view) {
        _view->setProperty("listButtonActiveState", QVariant::fromValue(true));
    }
}

void MeasurementResultCsvDataViewModel::deactivate()
{
    AbstractViewModel::deactivate();

    // コネクションの破棄
    disconnect(this, SIGNAL(scrollPage(QVariant)), _view, SLOT(onScrollPage(QVariant)));

    // 表示していたリストのインデックスを初期化する
    _listIndex = 0;
    _totalPage = 1;
    _currentPage = 1;
}

void MeasurementResultCsvDataViewModel::onKeyStatus(ButtonId id)
{
    // Home, Escape ボタンはベースクラスで対応
    AbstractViewModel::onKeyStatus(id);

    if (id == BUTTON_USER) {
#ifdef NEW_USER_SETTING
        emit requestDirect(SCREEN_USER_SETTING, true);
#else
        emit requestDirect(SCREEN_USER_DATA_LOGGER_MENU, true);
#endif
    }

    moveCursor(id);
}

void MeasurementResultCsvDataViewModel::moveCursor(ButtonId buttonId)
{
    if ((buttonId == BUTTON_UP) || (buttonId == BUTTON_DOWN)) {
        if (_listData->rowCount() == 0) {
            return;
        }

        // リスト選択行の更新
        _listIndex += LIST_INDEX_UPDATE_MAP.value(buttonId, 0);

        // 上下限値のチェック：上下限値を超えたら循環させる
        if (_listIndex < 0) {
            _listIndex = _listData->rowCount() - 1;
            while (_listData->getBand(_listIndex) == "") {
                _listIndex--;
            }
//        } else if (_listIndex == _listData->rowCount()) {
        } else if (_listData->getBand(_listIndex) == "") {
            // カーソルが当たっているインデックスが空の場合
            // リストの先頭にカーソルを当てる
            _listIndex = 0;
        } else {
            // nothing to do
        }

        if (_view) {
            _view->setProperty("currentIndex", QVariant::fromValue(_listIndex));
        }
        updateDisplayInfo();
    }
}

void MeasurementResultCsvDataViewModel::updateDisplayInfo()
{
    bool isSatelliteBroadCasting = Correcter::isSatelliteBroadCasting(Correcter::bandIdFromDisplayBand(_listData->getBand(_listIndex)));
    QString band = _listData->getBand(_listIndex);
    if (_view) {
        _view->setProperty("band", band);
    }
    if (_view) {
        _view->setProperty("bandId", Correcter::bandIdFromDisplayBand(band));
    }
    if (_view) {
        _view->setProperty("isSatelliteBroadCasting", isSatelliteBroadCasting);
    }
    if (_view) {
        _view->setProperty("channel", QVariant::fromValue(Correcter::displayChannelName(_listData->getChannel(_listIndex))));
    }
    if (_view) {
        _view->setProperty("level", QVariant::fromValue(_listData->getLevel(_listIndex)));
    }
    if (_view) {
        _view->setProperty("mer", QVariant::fromValue(_listData->getMer(_listIndex)));
    }
    if (_view) {
        _view->setProperty("ber", QVariant::fromValue(_listData->getBer(_listIndex)));
    }
    if (_view) {
        _view->setProperty("frequency", QVariant::fromValue(_listData->getFrequency(_listIndex)));
    }
    if (band == Correcter::displayBandName(LC70_BAND_FM)) {
        // FMの場合は固定でFMを設定
        if (_view) {
            _view->setProperty("displayModulationPrefixMode", DISPLAY_MODULATION_PREFIX_TOGETHER);
        }
        if (_view) {
            _view->setProperty("modulation", FM);
        }
        if (_view) {
            _view->setProperty("modulationPrefix", QString(""));
        }
    } else if (band == Correcter::displayBandName(LC70_BAND_PILOT) || band == Correcter::displayBandName(LC70_BAND_CATV_UP_STREAM)) {
        // "PILOT"か"CATV上る"の場合は固定でCWを設定
        if (_view) {
            _view->setProperty("displayModulationPrefixMode", DISPLAY_MODULATION_PREFIX_TOGETHER);
        }
        if (_view) {
            _view->setProperty("modulation", CW);
        }
        if (_view) {
            _view->setProperty("modulationPrefix", QString(""));
        }
    }else {
        if (_view) {
            _view->setProperty("displayModulationPrefixMode", Correcter::displayModulationPrefixMode(Correcter::broadcastSystemValue(_listData->getBroadcastSystem(_listIndex))));
        }
        if (_view) {
            _view->setProperty("modulation", QVariant::fromValue(_listData->getModulation(_listIndex)));
        }
        if (_view) {
            _view->setProperty("modulationPrefix", Correcter::modulationPrefix(Correcter::broadcastSystemValue(_listData->getBroadcastSystem(_listIndex)), _listData->getModulation(_listIndex)));
        }
    }
    if (isSatelliteBroadCasting) {
        if (_view) {
            _view->setProperty("hierarchy", "0");
        }
        if (_view) {
            _view->setProperty("polarization", QVariant::fromValue(_listData->getLayer(_listIndex)));
        }
    } else {
        if (_view) {
            _view->setProperty("hierarchy", QVariant::fromValue(_listData->getLayer(_listIndex)));
        }
        if (_view) {
            _view->setProperty("polarization", "0");
        }
    }
    if (_view) {
        _view->setProperty("localFrequency", QVariant::fromValue(_listData->getLocalOscillator(_listIndex)));
    }

    // チャンネル詳細データの表示可否を通知
    if (_view) {
        _view->setProperty("displayChannelDetail", Correcter::getDisplayChannelDetail(Correcter::bandIdFromDisplayBand(band)));
    }

    // リストの総ページ数設定
    _totalPage = _listData->rowCount() / MINIMUM_LIST_SIZE;

    if (_listData->rowCount() % MINIMUM_LIST_SIZE > 0) {
        _totalPage += 1;
    }

    // 現在のページを求める
    calcCurrentPage();
}

void MeasurementResultCsvDataViewModel::calcCurrentPage()
{
    int page = (_listIndex / MINIMUM_LIST_SIZE) + 1;

    if (page != _currentPage) {
        _currentPage = page;
        emit scrollPage(QVariant::fromValue(_currentPage));
    }
}
