﻿/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include "measurementresultdatalistviewmodel.h"

const static QMap<ButtonId, int> LIST_INDEX_UPDATE_MAP = {
    {BUTTON_UP, -1},
    {BUTTON_DOWN, 1}
};

static const int MEASURE_RESULT_PNG = 0;
static const int MEASURE_RESULT_CSV = 1;
static const int MEASURE_RESULT_DATALOGGER = 2;
static const int LIST_ROWS = 5;
static const QString EXTENSION_PNG = "png";
static const QString EXTENSION_CSV = "csv";

static constexpr const char *SINGLE_BUTTON = "m1";       //!< シングル測定(qml ファイルの objectName プロパティと合わせておく)
static constexpr const char *MULTI_BUTTON = "m2";        //!< マルチ測定(qml ファイルの objectName プロパティと合わせておく)
static constexpr const char *DATALOGGER_BUTTON = "m3";   //!< データロガー(qml ファイルの objectName プロパティと合わせておく)

static constexpr int MEASUREMENT_INDEX_NONE = 0;             //!< ボタン選択なし
static constexpr int MEASUREMENT_INDEX_SINGLE = 1;           //!< シングル測定選択
static constexpr int MEASUREMENT_INDEX_MULTI = 2;            //!< マルチ測定選択
static constexpr int MEASUREMENT_INDEX_DATALOGGER = 3;       //!< データロガー選択

MeasurementResultDataListViewModel::MeasurementResultDataListViewModel(ScreenId screenId, LevelCheckerModel *&model,
        QObject *parent)
    : AbstractViewModel(screenId, model, parent)
    , _listData(new MeasurementResultFileComponentList())
    , _listIndex(0)
    , _totalPage(1)
    , _currentPage(1)
    , _selectArea(MEASUREMENT_AREA)
    , _currentMeasurementIndex(MEASUREMENT_INDEX_SINGLE)
    , _editItemList()
    , _temporaryFileName()
{
    setObjectName("measurementResultDataListViewModel");
}

MeasurementResultDataListViewModel::~MeasurementResultDataListViewModel()
{

}

void MeasurementResultDataListViewModel::activate()
{
    if (_view == nullptr) {
        qErrnoWarning("_view is null");
        return;
    }

    // コネクション
    connect(this, SIGNAL(requestSetTakeOver(QVariant, QVariant)), _view, SLOT(backup(QVariant, QVariant)));
    connect(this, SIGNAL(requestGetTakeOver()), _view, SLOT(getTakeOver()));
    connect(this, SIGNAL(scrollPage(QVariant)), _view, SLOT(onScrollPage(QVariant)));

    emit requestGetTakeOver();

    // データ管理から開いたかどうか
    QObject *target = nullptr;
    int selectMenu = 0;
    if (_view) {
        target = _view->findChild<QObject *>("takeOverData");
    }
    if (target) {
        QVariant value = target->property("selecteMenu");
        selectMenu = value.toInt();
    }

    if (selectMenu == 1) {
        // データ管理から開いた時は測定選択から
        _selectArea = MEASUREMENT_AREA;
        _currentMeasurementIndex = MEASUREMENT_INDEX_SINGLE;
        _listIndex = 0;
        _totalPage = 1;
        _currentPage = 1;
    }
    emit requestSetTakeOver("selecteMenu", 0);

    // 画面反映
    if (_view) {
        _view->setProperty("activeArea", _selectArea);
        _view->setProperty("currentMeasurementIndex", _currentMeasurementIndex);
    }

    // ファイルリスト更新
    updateFileList();

    emit scrollPage(QVariant::fromValue(_currentPage));
}

void MeasurementResultDataListViewModel::suspend()
{
    // コネクションの破棄
    disconnect(this, SIGNAL(requestSetTakeOver(QVariant, QVariant)), _view, SLOT(backup(QVariant, QVariant)));
    disconnect(this, SIGNAL(scrollPage(QVariant)), _view, SLOT(onScrollPage(QVariant)));

    AbstractViewModel::suspend();
}
void MeasurementResultDataListViewModel::setTakeOver()
{
    if (_listData->rowCount() == 0 || (_listData->rowCount() - 1 < _listIndex)) {
        return;
    }

    if (_listData->getExtension(_listIndex) == EXTENSION_PNG) {
        emit requestSetTakeOver("MeasurementResultDataPng", _listData->getFilePath(_listIndex));
    } else if (_listData->getExtension(_listIndex) == EXTENSION_CSV) {
        if(_currentMeasurementIndex == MEASUREMENT_INDEX_DATALOGGER) {
            emit requestSetTakeOver("DataLoggerResultDataCsv", _listData->getFilePath(_listIndex));
        } else {
            emit requestSetTakeOver("MeasurementResultDataCsv", _listData->getFilePath(_listIndex));
        }
    } else {
        // nothing to do
    }
}

void MeasurementResultDataListViewModel::onKeyStatus(ButtonId id)
{
    AbstractViewModel::onKeyStatus(id);

    if (id == BUTTON_USER) {
#ifdef NEW_USER_SETTING
        emit requestDirect(SCREEN_USER_SETTING, true);
#else
        emit requestDirect(SCREEN_USER_DATA_LOGGER_MENU, true);
#endif
    }

    switch (_selectArea) {
    case MEASUREMENT_AREA:
        keyStatusMeasurementArea(id);
        break;
    case FILELIST_AREA:
        keyStatusFileListArea(id);
        break;
    case ACTION_AREA:
        keyStatusActionArea(id);
        break;
    }
}

void MeasurementResultDataListViewModel::onRequest(int condition)
{
    emit request(_screenId, condition);
}

void MeasurementResultDataListViewModel::updateFileList()
{
    // 測定ファイルの取得
    QStringList dirList;
    dirList.clear();

    if(_currentMeasurementIndex == MEASUREMENT_INDEX_SINGLE) {
        dirList.append(_model->applicationSetting()->resultDataPathSingle());
    } else if(_currentMeasurementIndex == MEASUREMENT_INDEX_MULTI) {
        dirList.append(_model->applicationSetting()->resultDataPathMulti());
    } else if(_currentMeasurementIndex == MEASUREMENT_INDEX_DATALOGGER) {
        dirList.append(_model->applicationSetting()->resultDataPathLogger());
    }

    QList<MeasurementResultFileListItems> items = {};

    // 作成日でソートを実施したリスト表示用のアイテムを取得
    QList<QFileInfo> dataList = displayDataList(dirList);

    // ファイルリストを用意
    int index = 1;
    for (int i = 0; i < dataList.size(); ++i) {
        QString userDataName;
        QString dataFormat;
        // 画面表示用のデータ名と日時に分割したものを用意
        if(_currentMeasurementIndex == MEASUREMENT_INDEX_DATALOGGER) {
            // ファイル名
            QStringList dataNameSplitList = dataList[i].fileName().split("_");
            for (int len = 0; len < dataNameSplitList.size() - 1; ++len) {
                if(len != 0) {
                    userDataName = userDataName + QString("_");
                }
                userDataName = userDataName + dataNameSplitList[len];
            }
            userDataName = userDataName + QString(".") + dataList[i].completeSuffix();
            // 日時
            QString data = dataNameSplitList[dataNameSplitList.size() - 1];
#if 1	//20210625_dx5
            dataFormat = data.mid(0, 4) + ("/") + data.mid(4, 2) + ("/") + data.mid(6, 2) + ("　")
                        + data.mid(8, 2) + (":") + data.mid(10, 2) + (":") + data.mid(12, 2);
#else
            dataFormat = data.mid(0, 4) + ("/") + data.mid(4, 2) + ("/") + data.mid(6, 2) + ("　") + data.mid(8, 2) + (":") + data.mid(10, 2);
#endif
        } else {
            // ファイル名
            QStringList dataNameSplitList = dataList[i].fileName().split("_");
            for (int len = 0; len < dataNameSplitList.size() - 2; ++len) {
                if(len != 0) {
                    userDataName = userDataName + QString("_");
                }
                userDataName = userDataName + dataNameSplitList[len];
            }
            userDataName = userDataName + QString(".") + dataList[i].completeSuffix();
            // 日時
            QString data = dataNameSplitList[dataNameSplitList.size() - 2];
#if 1	//20210625_dx5
            dataFormat = data.mid(0, 4) + ("/") + data.mid(4, 2) + ("/") + data.mid(6, 2) + ("　")
                        + data.mid(8, 2) + (":") + data.mid(10, 2) + (":") + data.mid(12, 2);

#else
            dataFormat = data.mid(0, 4) + ("/") + data.mid(4, 2) + ("/") + data.mid(6, 2) + ("　") + data.mid(8, 2) + (":") + data.mid(10, 2);
#endif
        }
        items.append(MeasurementResultFileListItems(index, dataList[i].fileName(), dataList[i].absoluteFilePath(), dataList[i].completeSuffix()
                                                    ,userDataName, dataFormat));
        ++index;
    }

    // ダミー行の追加
    int surplus = items.count() % LIST_ROWS;

    if (surplus != 0) {
        // ページを満たすためのダミー行の追加
        surplus = LIST_ROWS - surplus;
        for (int i = 0; i < surplus; ++i) {
            items.append(MeasurementResultFileListItems(index, "", "", ""));
            ++index;
        }
    }

    // データが1件もなければ全ての行に対してダミーを追加する
    if (items.count() == 0) {
        for (int i = 0; i < LIST_ROWS; ++i) {
            items.append(MeasurementResultFileListItems(index, "", ""));
            ++index;
        }
    }

    _listData->setListData(items);

    if (_listData->getFileName(_listIndex).isEmpty()) {
        // カーソルが当たっているインデックスが空の場合
        // リストの先頭にカーソルを当てる
        _listIndex = 0;
    }

    // リストの総ページ数設定
    _totalPage = _listData->rowCount() / LIST_ROWS;

    if (_listData->rowCount() % LIST_ROWS > 0) {
        _totalPage += 1;
    }

    // 画面へ反映
    if (_view) {
        _view->setProperty("totalPage", _totalPage);
        _view->setProperty("listModel", QVariant::fromValue(_listData));
        _view->setProperty("currentIndex", QVariant::fromValue(_listIndex));
    }

    // 現在のページを求める
    calcCurrentPage();
}

QList<QFileInfo> MeasurementResultDataListViewModel::displayDataList(QStringList dirList)
{
    // リストに表示するファイル拡張子の設定
//    QStringList extensionFilters = {"*.csv", "*.png"};

//    // ソート対象のファイルを取得
//    QList<QFileInfo> fileInfoList = {};

//    int i = 0;
//    while (i < dirList.size()) {
//        if (QDir(dirList[i]).exists()) {
//            QDirIterator it(dirList[i], extensionFilters, QDir::Files, QDirIterator::Subdirectories);

//            while(it.hasNext()) {
//                if (it.next().isEmpty()) {
//                    continue;
//                }
//                fileInfoList.append(it.fileInfo());
//            }
//        }
//        ++i;
//    }

//    for (i = 0; i < fileInfoList.size() - 1; ++i) {
//        int maxIndex = i;
//        for (int j = i + 1; j < fileInfoList.size(); ++j) {
//            if(fileInfoList[maxIndex].metadataChangeTime() < fileInfoList[j].metadataChangeTime()) {
//                maxIndex = j;
//            }
//        }
//        if (i != maxIndex) {
//            // swap処理を実施
//            fileInfoList.swap(i, maxIndex);
//        }
//    }

    // 対象のファイルを取得
    QList<QFileInfo> fileInfoList = {};

    QString index;
    if(_currentMeasurementIndex == MEASUREMENT_INDEX_SINGLE) {
        index = ("single");
    } else if(_currentMeasurementIndex == MEASUREMENT_INDEX_MULTI) {
        index = ("multi");
    } else if(_currentMeasurementIndex == MEASUREMENT_INDEX_DATALOGGER) {
        index = ("logger");
    }

    QString filePath = _model->applicationSetting()->measureDataFileListPath()
                        + ("/") + _model->applicationSetting()->measureDataFileListFileName();
    QFile file(filePath);

    if (!file.open(QIODevice::ReadOnly)) {
        qErrnoWarning("file open error");
        return fileInfoList;
    }

    QTextStream in(&file);
    QStringList csvLineText = {};
    QFileInfo fileinfo;
    QString toUpLow;
    int row = 0;
    while (!in.atEnd()) {
        csvLineText = in.readLine().split(",");

        ++row;

        toUpLow = csvLineText[1];
        if(toUpLow.toLower() == index) {
            if(_currentMeasurementIndex == MEASUREMENT_INDEX_DATALOGGER) {
                fileinfo = _model->applicationSetting()->measureDataFileListPath() + ("/")
                            + toUpLow.toLower() + ("/")
                            + csvLineText[2] + ("/")
                            + csvLineText[4];
            } else {

                fileinfo = _model->applicationSetting()->measureDataFileListPath() + ("/")
                            + toUpLow.toLower() + ("/")
                            + csvLineText[2] + ("/")
                            + csvLineText[3] + ("/")
                            + csvLineText[4];
            }
            fileInfoList.append(fileinfo);
        }
    }

    return fileInfoList;
}

void MeasurementResultDataListViewModel::calcCurrentPage()
{
    int page = (_listIndex / LIST_ROWS) + 1;

    if (page != _currentPage) {
        _currentPage = page;
        emit scrollPage(QVariant::fromValue(_currentPage));
    }
}

void MeasurementResultDataListViewModel::keyStatusMeasurementArea(ButtonId id)
{
    if ( (id == BUTTON_DOWN) || (id == BUTTON_ENTER) ){
        moveArea(FILELIST_AREA);
    } else if( (id == BUTTON_LEFT) || (id == BUTTON_RIGHT) ) {
        moveCursorMeasurementArea(id);
        // ファイルリスト更新
        updateFileList();
    }
}

void MeasurementResultDataListViewModel::moveCursorMeasurementArea(ButtonId id)
{
    qDebug() << QString("moveCursorMeasurementArea: %1").arg(id == BUTTON_LEFT ? "LEFT" : "RIGHT");
    int nextAction = _currentMeasurementIndex;
    do {
        nextAction = nextAction + (id == BUTTON_LEFT ? -1 : 1);
        qDebug() << QString("moveCursorMeasurementArea: next=%1").arg(nextAction);
        if (nextAction < MEASUREMENT_INDEX_SINGLE) {
            nextAction = MEASUREMENT_INDEX_DATALOGGER;
        }
        if (nextAction > MEASUREMENT_INDEX_DATALOGGER) {
            nextAction = MEASUREMENT_INDEX_SINGLE;
        }
        if ((nextAction == MEASUREMENT_INDEX_SINGLE)
                || (nextAction == MEASUREMENT_INDEX_MULTI)
                || (nextAction == MEASUREMENT_INDEX_DATALOGGER)) {
            qDebug() << QString("moveCursorMeasurementArea: %1 -> %2").arg(_currentMeasurementIndex).arg(nextAction);
            _currentMeasurementIndex = nextAction;
            if (_view) {
                _view->setProperty("currentMeasurementIndex", _currentMeasurementIndex);
            }
            return;
        }
    } while(nextAction != _currentMeasurementIndex);
}

void MeasurementResultDataListViewModel::keyStatusFileListArea(ButtonId id)
{
    if ((id == BUTTON_UP) || (id == BUTTON_DOWN)) {
        if(moveCursorFileListArea(id) == true) {
            moveArea(MEASUREMENT_AREA);
        }
    } else if (id == BUTTON_ENTER) {
        if (_listData->rowCount() == 0 || _listIndex < 0) {
            return;
        }

        if (_listData->rowCount() - 1 < _listIndex) {
            return;
        }
        if (_listData->getExtension(_listIndex) == EXTENSION_PNG) {
            onRequest(MEASURE_RESULT_PNG);
        } else if (_listData->getExtension(_listIndex) == EXTENSION_CSV) {
            if(_currentMeasurementIndex == MEASUREMENT_INDEX_DATALOGGER) {
                onRequest(MEASURE_RESULT_DATALOGGER);
            } else {
                onRequest(MEASURE_RESULT_CSV);
            }
        } else {
            // データなし
        }
    }
}

bool MeasurementResultDataListViewModel::moveCursorFileListArea(ButtonId id)
{
    if ((id == BUTTON_UP) || (id == BUTTON_DOWN)) {
        if (_listData->rowCount() == 0) {
            return false;
        }

        // リスト選択行の更新
        _listIndex += LIST_INDEX_UPDATE_MAP.value(id, 0);

        // 上下限値のチェック
        if (_listIndex < 0) {
//            _listIndex = _listData->rowCount() - 1;
            return true;
        } else if (_listData->getFileName(_listIndex).isEmpty()) {
            // カーソルが当たっているインデックスが空の場合
            // リストの先頭にカーソルを当てる
            _listIndex = 0;
        } else {
            // nothing to do
        }

        // 現在のページを求める
        calcCurrentPage();

        if (_view) {
            _view->setProperty("currentIndex", QVariant::fromValue(_listIndex));
        }
    }
    return false;
}

void MeasurementResultDataListViewModel::keyStatusActionArea(ButtonId id)
{


}

void MeasurementResultDataListViewModel::moveCursorActionArea(ButtonId id)
{


}

void MeasurementResultDataListViewModel::moveArea(SelectArea destination)
{
    if (_selectArea == destination) {
        return;
    }

    // _selectArea変更前処理
    if(_selectArea == MEASUREMENT_AREA) {
        // 測定選択からファイルリストに移動するときは必ずファイルリストの先頭から
        _listIndex = 0;
    }

    // _selectArea変更
    _selectArea = destination;
    if (_view) {
        _view->setProperty("activeArea", _selectArea);
    }

    // _selectArea変更後処理
    if(_selectArea == MEASUREMENT_AREA) {
        if (_view) {
            _view->setProperty("currentMeasurementIndex", _currentMeasurementIndex);
        }
    } else if(_selectArea == FILELIST_AREA) {
        if (_view) {
            _view->setProperty("currentIndex", QVariant::fromValue(_listIndex));
        }

        // 測定結果がない時は MEASUREMENT_AREA に移動
        if (_listData->getFileName(_listIndex).isEmpty()) {
            moveArea(MEASUREMENT_AREA);
        }
    }
}
