﻿/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef MEASUREMENTRESULTDATALISTVIEWMODEL_H
#define MEASUREMENTRESULTDATALISTVIEWMODEL_H

#include "abstractviewmodel.h"
#include "measurementresultfilecomponentlist.h"
#include "buttonoperation.h"

/**
 * @brief 測定結果データ一覧表示 ViewModel
 */
class MeasurementResultDataListViewModel : public AbstractViewModel
{
    Q_OBJECT
public:
    /**
     * @brief コンストラクター
     * @param screenId 画面 ID
     * @param model Model クラスのポインター
     * @param parent 親オブジェクト
     */
    explicit MeasurementResultDataListViewModel(ScreenId screenId, LevelCheckerModel *&model, QObject *parent = nullptr);

    /**
     * @brief デストラクタ
     */
    virtual ~MeasurementResultDataListViewModel();

    /**
     * @brief 画面Active 時の処理
     */
    virtual void activate(void) override;

    /**
     * @brief 画面を閉じるときに呼び出すスレッド停止処理
     */
    virtual void suspend(void) override;

    /**
     * @brief 遷移先画面で参照する遷移前画面のデータ設定
     */
    virtual void setTakeOver() override;

signals:
    void requestSetTakeOver(QVariant, QVariant);
    void requestGetTakeOver();

    /**
     * @brief リストのページを移動
     */
    void scrollPage(QVariant);
protected:
    /**
     * @brief ボタン押下時イベント受信時の処理
     * @param id ボタンID
     */
    virtual void onKeyStatus(ButtonId id) override;

private slots:
    /**
     * @brief 画面遷移要求の受信時処理
     * @param condition 画面IDリストのインデックス
     */
    virtual void onRequest(int condition) override;

private:
    /**
     * @brief 操作対象のエリア定義
     */
    enum SelectArea {
        MEASUREMENT_AREA = 1,    // 測定選択リスト
        FILELIST_AREA  = 2,      // ファイルリスト
        ACTION_AREA  = 3,        // 動作リスト
    };

    QSharedPointer<MeasurementResultFileComponentList> _listData;   //!< リストデータ
    int _listIndex;                                                 //!< リストのインデックス
    int _totalPage;                                                 //!< リストの全ページ数
    int _currentPage;                                               //!< 選択中のページ数

    SelectArea _selectArea;                                         //!< 選択している操作対象エリア
    QVariantList _editItemList;                                     //!< 編集項目のリスト
    QString _temporaryFileName;                                     //!< 仮のファイル名
    int _currentMeasurementIndex;                                   //!< 選択中の測定選択ボタン

    /**
     * @brief リスト表示用のデータリストを取得
     * @param dirList リスト表示用の対象のファイルのディレクトリ名のリスト
     * @return リスト表示用のデータリスト
     * @note データリストは作成日でソートした状態となっている
     */
    QList<QFileInfo> displayDataList(QStringList dirList);

    /**
     * @brief 選択しているページを求める
     */
    void calcCurrentPage(void);

    /**
     * @brief 測定選択リストエリアでのボタン操作
     * @param id ボタン ID
     */
    void keyStatusMeasurementArea(ButtonId id);

    /**
     * @brief 測定選択リストエリアでのカーソル移動
     * @param id ボタン ID
     */
    void moveCursorMeasurementArea(ButtonId id);

    /**
     * @brief ファイルリストエリアでのボタン操作
     * @param id ボタン ID
     */
    void keyStatusFileListArea(ButtonId id);

    /**
     * @brief ファイルリストエリアでのカーソル移動
     * @param id ボタン ID
      * @return true:エリア移動する
     *         false:エリア移動しない
     */
    bool moveCursorFileListArea(ButtonId id);

    /**
     * @brief 動作ボタンエリアでのボタン操作
     * @param id ボタン ID
     */
    void keyStatusActionArea(ButtonId id);

    /**
     * @brief 動作ボタンエリアでのカーソル移動
     * @param id ボタン ID
     */
    void moveCursorActionArea(ButtonId id);

    /**
     * @brief エリア移動
     * @param destination 移動先エリア
     */
    void moveArea(SelectArea destination);

    /**
     * @brief ファイルリストの更新
     */
    void updateFileList();

};

#endif // MEASUREMENTRESULTDATALISTVIEWMODEL_H
