/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include "measurementresultoperationviewmodelbase.h"

const static QMap<ButtonId, int> LIST_INDEX_UPDATE_MAP = {
    {BUTTON_UP, -1},
    {BUTTON_DOWN, 1}
};

static QVector<QVector<QString>> displayButtonData = {
    {"m1"},
    {"m2"}
};

static const int LIST_ROWS = 6;

static const int OPERATION_BUTTON_COLUMN = 2;
static const int OPERATION_BUTTON_ROW = 1;

MeasurementResultOperationViewModelBase::MeasurementResultOperationViewModelBase(ScreenId screenId, LevelCheckerModel *&model, QString objectName, QObject *parent)
    : AbstractViewModel(screenId, model, parent)
    , _listData(new MeasurementResultOperationList())
    , _listIndex(0)
    , _cursoredButtonArea(false)
    , _activeButton("m1")
    , _totalPage(1)
    , _currentPage(1)
{
    setObjectName(objectName);
    _buttonOperation.setButtonName(OPERATION_BUTTON_COLUMN, OPERATION_BUTTON_ROW, displayButtonData);
    _activeButton = displayButtonData[0][0];
}

MeasurementResultOperationViewModelBase::~MeasurementResultOperationViewModelBase()
{
}

void MeasurementResultOperationViewModelBase::activate()
{
    // リストデータの設定を行う
    updateListData();

    // 画面遷移時は、常にリストにカーソルが当たっているようにし、先頭のボタンを活性にする
    _cursoredButtonArea = false;
    _activeButton = displayButtonData[0][0];
    _buttonOperation.setActiveObject(_activeButton);

    if (_view) {
        _view->setProperty("cursoredButtonArea", _cursoredButtonArea);
    }
}

void MeasurementResultOperationViewModelBase::deactivate()
{
    AbstractViewModel::deactivate();

    // ローカル変数を初期値戻す
    _listIndex = 0;
    _activeButton = displayButtonData[0][0];
    _cursoredButtonArea = false;
}

void MeasurementResultOperationViewModelBase::suspend()
{
    // ローカル変数を初期化する
    _totalPage = 1;
    _currentPage = 1;

    AbstractViewModel::suspend();
}


void MeasurementResultOperationViewModelBase::onKeyStatus(ButtonId id)
{
    AbstractViewModel::onKeyStatus(id);
    if (id == BUTTON_ENTER) {
        QObject *target = _view->findChild<QObject *>(_activeButton);
        target->setProperty("press", false);
        pressEnterButton();
    } else if (id == BUTTON_USER) {
#ifdef NEW_USER_SETTING
        emit requestDirect(SCREEN_USER_SETTING, true);
#else
        emit requestDirect(SCREEN_USER_DATA_LOGGER_MENU, true);
#endif
    } else {
        moveCursor(id);
    }
}

void MeasurementResultOperationViewModelBase::pressEnterButton()
{
    if(_cursoredButtonArea) {
        if (_activeButton == displayButtonData[0][0]) {
            // 全選択/解除ボタン押下時
            pressAllSelectCancelButton();
        } else if (_activeButton == displayButtonData[1][0]) {
            // 処理ボタン押下時
            pressExecuteButton();
        } else {
            // nothing to do
        }
    } else {
        // カーソルが当たっているリスト項目の選択の有無を変更する
        if (!(_listData->getFilePath(_listIndex).isEmpty())) {
            _listData->setIsSelected(_listIndex, !(_listData->getIsSelected(_listIndex)));
        }
    }
}

void MeasurementResultOperationViewModelBase::pressAllSelectCancelButton()
{
    // 未選択な項目がないかチェックを行う
    bool existsUnSelect = false;
    for (int index = 0; index < _listData->rowCount(); ++index) {
        if (_listData->getFilePath(index).isEmpty()) {
            break;
        }
        if (_listData->getIsSelected(index) == false) {
            existsUnSelect = true;
            break;
        }
    }

    if (existsUnSelect) {
        // 未選択な項目が有る場合は、全選択を行う
        for (int index = 0; index < _listData->rowCount(); ++index) {
            if (_listData->getFilePath(index).isEmpty()) {
                break;
            }
            _listData->setIsSelected(index, true);
        }
    } else {
        // 未選択な項目が無い場合、全解除を行う
        for (int index = 0; index < _listData->rowCount(); ++index) {
            if (_listData->getFilePath(index).isEmpty()) {
                break;
            }
            _listData->setIsSelected(index, false);
        }
    }
}

void MeasurementResultOperationViewModelBase::pressExecuteButton()
{
    // 処理はoverride先で記述
}

void MeasurementResultOperationViewModelBase::moveCursor(ButtonId buttonId)
{
    if ((buttonId == BUTTON_UP) || (buttonId == BUTTON_DOWN)) {
        if (_cursoredButtonArea) {
            // ボタンエリアを選択している場合
            QString nextButton;
            bool ret = _buttonOperation.nextActiveObject(buttonId, nextButton);

            if (ret) {
                QObject *parts = _model->search("DataManagerOperationBase");

                QObject *target = parts->findChild<QObject *>(_activeButton);
                if (target) {
                    target->setProperty("active", false);
                }

                target = parts->findChild<QObject *>(nextButton);
                if (target) {
                    target->setProperty("active", true);
                }
                _activeButton = nextButton;
                _buttonOperation.setActiveObject(_activeButton);
            }
        } else {
            // リストエリアを選択している場合
            if (_listData->rowCount() == 0) {
                return;
            }

            // リスト選択行の更新
            _listIndex += LIST_INDEX_UPDATE_MAP.value(buttonId, 0);

            // 上下限値のチェック
            if (_listIndex < 0) {
                _listIndex = _listData->rowCount() - 1;
            } else if (_listIndex == _listData->rowCount()) {
                _listIndex = 0;
            } else {
                // nothing to do
            }

            // 現在のページを求める
            calcCurrentPage();

            _view->setProperty("currentIndex", QVariant::fromValue(_listIndex));
        }
    } else if ((buttonId == BUTTON_LEFT) || (buttonId == BUTTON_RIGHT)) {
        _cursoredButtonArea = !_cursoredButtonArea;
        QObject *parts = _model->search("DataManagerOperationBase");
        QObject *target = parts->findChild<QObject *>(_activeButton);
        if (target) {
            if (_cursoredButtonArea) {
                target->setProperty("active", true);
                _buttonOperation.setActiveObject(_activeButton);
            } else {
                target->setProperty("active", false);
                _buttonOperation.setActiveObject("");
            }
        }
        _view->setProperty("cursoredButtonArea", _cursoredButtonArea);
    } else {
        // nothing to do
    }
}

void MeasurementResultOperationViewModelBase::onButtonPress(ButtonId id)
{
    if (id == BUTTON_ENTER) {
        QObject *target = _view->findChild<QObject *>(_activeButton);
        target->setProperty("press", true);
    }
}

void MeasurementResultOperationViewModelBase::updateListData()
{
    QStringList dirList;
    dirList.clear();
    dirList.append(_model->applicationSetting()->resultDataPathSingle());
    dirList.append(_model->applicationSetting()->resultDataPathMulti());
    QList<MeasurementResultOperationListItems> items = {};

    // 作成日でソートを実施したリスト表示用のアイテムを取得
    QList<QFileInfo> dataList = displayDataList(dirList);

    int index = 1;
    for (int i = 0; i < dataList.size(); ++i) {
        items.append(MeasurementResultOperationListItems(index, dataList[i].fileName(), dataList[i].absoluteFilePath()));
        ++index;
    }

    // ダミー行の追加
    int surplus = items.count() % LIST_ROWS;

    if (surplus != 0) {
        // ページを満たすためのダミー行の追加
        surplus = LIST_ROWS - surplus;
        for (int i = 0; i < surplus; ++i) {
            items.append(MeasurementResultOperationListItems(index,"",""));
            ++index;
        }
    }

    // データが1件もなければ全ての行に対してダミーを追加する
    if (items.count() == 0) {
        for (int i = 0; i < LIST_ROWS; ++i) {
            items.append(MeasurementResultOperationListItems(index, "", ""));
            ++index;
        }
    }

    _listData->setListData(items);

    // リストの総ページ数設定
    _totalPage = _listData->rowCount() / LIST_ROWS;

    if (_listData->rowCount() % LIST_ROWS > 0) {
        _totalPage += 1;
    }

    // 更新時、リストの先頭を選択している状態にする
    _listIndex = 0;
    _currentPage = 1;

    // 画面へ反映
    _view->setProperty("totalPage", _totalPage);
    _view->setProperty("currentPage", _currentPage);
    _view->setProperty("listModel", QVariant::fromValue(_listData));
    _view->setProperty("currentIndex", QVariant::fromValue(_listIndex));
}

QStringList MeasurementResultOperationViewModelBase::getTargetPathList()
{
    // 処理を行う対象パスのリストを取得
    QStringList targetPathList;
    for (int index = 0; index < _listData->rowCount(); ++index) {
        if (!(_listData->getFilePath(index).isEmpty()) && _listData->getIsSelected(index)){
            targetPathList.append(_listData->getFilePath(index));
        }
    }

    return targetPathList;
}

QList<QFileInfo> MeasurementResultOperationViewModelBase::displayDataList(QStringList dirList)
{
    // リストに表示するファイル拡張子の設定
    QStringList extensionFilters = {"*.csv", "*.png"};

    // ソート対象のファイルを取得
    QList<QFileInfo> fileInfoList = {};

    int i = 0;
    while (i < dirList.size()) {
        if (QDir(dirList[i]).exists()) {
            QDirIterator it(dirList[i], extensionFilters, QDir::Files, QDirIterator::Subdirectories);

            while(it.hasNext()) {
                if (it.next().isEmpty()) {
                    continue;
                }
                fileInfoList.append(it.fileInfo());
            }
        }
        ++i;
    }

    // ファイル生成日でソートを実施する
    for (i = 0; i < fileInfoList.size() - 1; ++i) {
        int maxIndex = i;
        for (int j = i + 1; j < fileInfoList.size(); ++j) {
            if(fileInfoList[maxIndex].metadataChangeTime() < fileInfoList[j].metadataChangeTime()) {
                maxIndex = j;
            }
        }
        if (i != maxIndex) {
            // swap処理を実施
            fileInfoList.swap(i, maxIndex);
        }
    }

    return fileInfoList;
}

void MeasurementResultOperationViewModelBase::calcCurrentPage()
{
    int page = (_listIndex / LIST_ROWS) + 1;

    if (page != _currentPage) {
        _currentPage = page;
        emit scrollPage(QVariant::fromValue(_currentPage));
    }
}
