/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include "usermeasurementsettingmenuviewmodel.h"
#include "usboperation.h"

// メニューマップ(ユーザ測定設定を表示, コピー（本体⇒USB）, コピー（USB⇒本体）, 一括削除) USB接続中
QVector<QVector<QString>> userMeasurementSettingMenuButtonDataWithUsb = {
    {{"m0"}, {"m1"}, {"m2"}, {"m3"}},
};

const int UserMeasurementSettingMenuViewModel::BUTTON_COLUMN_NUMBER = 1;
const int UserMeasurementSettingMenuViewModel::BUTTON_ROW_NUMBER = 4;

UserMeasurementSettingMenuViewModel::UserMeasurementSettingMenuViewModel(ScreenId screenId, LevelCheckerModel *&model,
        QObject *parent)
    : AbstractViewModel(screenId, model, parent)
{
    setObjectName("userMeasurementSettingMenuViewModel");
    _buttonOperation.setButtonName(BUTTON_COLUMN_NUMBER, BUTTON_ROW_NUMBER, userMeasurementSettingMenuButtonDataWithUsb);
    _buttonOperation.setActiveObject(userMeasurementSettingMenuButtonDataWithUsb[0][0]);
    _activeButton = userMeasurementSettingMenuButtonDataWithUsb[0][0];
}

UserMeasurementSettingMenuViewModel::~UserMeasurementSettingMenuViewModel()
{
}

void UserMeasurementSettingMenuViewModel::activate()
{
    if (_view == nullptr) {
        qErrnoWarning("_view is null");
        return;
    }

    // コネクション
    connect(this, SIGNAL(requestSetTakeOver(QVariant, QVariant)), _view, SLOT(backup(QVariant, QVariant)));

    // 画面共有情報のクリア
    emit removeAttribute("dataloggerFunction");

    // 画面表示時の初期選択アイコンを設定
    setBoolProperty(_activeButton, "active", true);
}

void UserMeasurementSettingMenuViewModel::setTakeOver()
{
    if (_activeButton == "m0") {
        emit requestSetTakeOver("selectedUserSetting", 1);  // see DataloggerViewModel::DataloggerEditType
    }
}

void UserMeasurementSettingMenuViewModel::onKeyStatus(ButtonId id)
{
    AbstractViewModel::onKeyStatus(id);

    if (id == BUTTON_ENTER) {
        QObject *target = nullptr;
        if (_view) {
            target = _view->findChild<QObject *>(_activeButton);
        }
        if (target) {
            target->setProperty("press", false);
        }

        if (isErrorUsbStatus()) {
            return;
        }
        if (_activeButton == "m1") {
            emit requestToShowPopup(POPUP_COPY_ALL_TO_USB_USER_SETTING_DATA_CONFIRMATION); // コピー（本体⇒USB）
        } else if (_activeButton == "m2") {
            emit requestToShowPopup(POPUP_COPY_ALL_TO_BODY_USER_SETTING_DATA_CONFIRMATION); // コピー（USB⇒本体）
        } else if (_activeButton == "m3") {
            emit requestToShowPopup(POPUP_DELETE_ALL_USER_SETTING_DATA_CONFIRMATION); // 一括削除
        } else {
            // ユーザ設定(ユーザ設定表示へ遷移)
            emit requestSetTakeOver("dataloggerFunction", 5);
            int button_index = _activeButton.mid(1).toInt();
            onRequest(button_index);
        }
    } else if (id == BUTTON_USER) {
#ifdef NEW_USER_SETTING
        emit requestDirect(SCREEN_USER_SETTING, true);
#else
        emit requestDirect(SCREEN_USER_DATA_LOGGER_MENU, true);
#endif
    } else {
        moveCursor(id);
    }
}

void UserMeasurementSettingMenuViewModel::onAccept(PopupId popupId, bool okOnly)
{
    AbstractViewModel::onAccept(popupId, okOnly);

    if (popupId == POPUP_COPY_ALL_TO_USB_USER_SETTING_DATA_CONFIRMATION) {
        // コピー(本体⇒USB)
        copyUserMeasurementSettingDataToUsb();
    } else if (popupId == POPUP_COPY_ALL_TO_BODY_USER_SETTING_DATA_CONFIRMATION) {
        // コピー(USB⇒本体)
        copyUserMeasurementSettingDataToBody();
    } else if (popupId == POPUP_DELETE_ALL_USER_SETTING_DATA_CONFIRMATION) {
        // 一括削除
        removeAllUserMeasurementSettingData();
    } else {
        // nothing to do
    }
}

void UserMeasurementSettingMenuViewModel::onReject(PopupId popupId)
{
    Q_UNUSED(popupId)
}

void UserMeasurementSettingMenuViewModel::onRequest(int condition)
{
    emit request(_screenId, condition);
}

void UserMeasurementSettingMenuViewModel::moveCursor(ButtonId buttonId)
{
    if ((buttonId == BUTTON_RIGHT) || (buttonId == BUTTON_LEFT)) {

        QString nextButton;
        bool ret = _buttonOperation.nextActiveObject(buttonId, nextButton);

        if (ret) {
            // inactive
            setBoolProperty(_activeButton, "active", false);

            // active
            setBoolProperty(nextButton, "active", true);

            _activeButton = nextButton;
            _buttonOperation.setActiveObject(_activeButton);
        }
    }
}

void UserMeasurementSettingMenuViewModel::onButtonPress(ButtonId id)
{
    if (id == BUTTON_ENTER) {
        QObject *target = nullptr;
        if (_view) {
            target = _view->findChild<QObject *>(_activeButton);
        }
        if (target) {
            target->setProperty("press", true);
        }
    }
}

void UserMeasurementSettingMenuViewModel::removeAllUserMeasurementSettingData()
{
    QString path = _model->applicationSetting()->settingDataPathUserData();

    if (path.isEmpty()) {
        return;
    }

    if (!QDir(path).exists()) {
        return;
    }

    // 対象フォルダを削除・再作成
    QDir(path).removeRecursively();
    QDir(QCoreApplication::applicationDirPath()).mkdir(_model->applicationSetting()->settingDataPathUserData());

    // 内部メモリを再度更新する
    _model->initializeUserSettingFile();
}

void UserMeasurementSettingMenuViewModel::copyUserMeasurementSettingDataToUsb()
{
    UsbErrorType error = UsbErrorType::NON_ERROR;

    QString copyTarget = _model->applicationSetting()->settingDataPathUserData();

    QStringList copyTargetPathList = {};

    error = UsbOperation::checkExistsDirectory(copyTarget);

    if (error == UsbErrorType::NON_ERROR) {
        copyTargetPathList.append(copyTarget);
    }

    if (copyTargetPathList.isEmpty()) {
        // コピー対象ファイルが無い場合は以降の処理は実施しない
        return;
    }

    error = UsbErrorType::NON_ERROR;

    // コピー可能な状態かのチェックを行う
    error = UsbOperation::checkCopyable(copyTargetPathList);

    if (error == UsbErrorType::COPY_FILE_ERROR) {
        // コピー元ファイルがない
        emit requestToShowPopup(POPUP_NOTHING_COPY_FILE);
        return;
    } else if (error == UsbErrorType::SPACE_ERROR) {
        // USBの空き容量が足りない
        emit requestToShowPopup(POPUP_USB_NO_SPACE);
        return;
    } else {
        // nothing to do
    }

    error = UsbOperation::copyToUsb(UsbDataType::
                                    USER_MEASUREMENT_SETTING, copyTarget, true, _model->applicationSetting()->password());

    if (error == UsbErrorType::COPY_EXEC_ERROR) {
        // コピー失敗
        emit requestToShowPopup(POPUP_FAILURE_COPY_EXECUTE);
    }
}

void UserMeasurementSettingMenuViewModel::copyUserMeasurementSettingDataToBody()
{
    UsbErrorType error = UsbErrorType::NON_ERROR;
    // USBから本体へフォルダコピー
//    error = UsbOperation::copyToBody(UsbDataType::USER_MEASUREMENT_SETTING,
//                                     _model->applicationSetting()->settingDataPathUserData(), true, _model->applicationSetting()->password());

    if (error == UsbErrorType::COPY_EXEC_ERROR) {
        // コピー失敗
        emit requestToShowPopup(POPUP_FAILURE_COPY_EXECUTE);
    } else if (error == UsbErrorType::COPY_FILE_ERROR) {
        // コピー元ファイルがない
        emit requestToShowPopup(POPUP_NOTHING_COPY_FILE);
    } else {
        // コピーしたファイルを読み込み直す
        _model->initializeUserSettingFile();
    }
}

void UserMeasurementSettingMenuViewModel::copyDataLoggerMeasurementSettingDataToUsb()
{
    UsbErrorType error = UsbErrorType::NON_ERROR;

    QString copyTarget = _model->applicationSetting()->settingDataPathLogger();

    QStringList copyTargetPathList = {};

    error = UsbOperation::checkExistsDirectory(copyTarget);

    if (error == UsbErrorType::NON_ERROR) {
        copyTargetPathList.append(copyTarget);
    }

    if (copyTargetPathList.isEmpty()) {
        // コピー対象ファイルが無い場合は以降の処理は実施しない
        return;
    }

    error = UsbErrorType::NON_ERROR;

    // コピー可能な状態かのチェックを行う
    error = UsbOperation::checkCopyable(copyTargetPathList);

    if (error == UsbErrorType::COPY_FILE_ERROR) {
        // コピー元ファイルがない
        emit requestToShowPopup(POPUP_NOTHING_COPY_FILE);
        return;
    } else if (error == UsbErrorType::SPACE_ERROR) {
        // USBの空き容量が足りない
        emit requestToShowPopup(POPUP_USB_NO_SPACE);
        return;
    } else {
        // nothing to do
    }

    error = UsbOperation::copyToUsb(UsbDataType::
                                    DATALOGGER_MEASUREMENT_SETTING, copyTarget, true, _model->applicationSetting()->password());

    if (error == UsbErrorType::COPY_EXEC_ERROR) {
        // コピー失敗
        emit requestToShowPopup(POPUP_FAILURE_COPY_EXECUTE);
    }
}

bool UserMeasurementSettingMenuViewModel::isErrorUsbStatus()
{
    QMap<QString, QList<LC70_STRAGE_T>> enableUsbStatusMap = {
        {"m1", {LC70_STRAGE_FORMAT}},
        {"m2", {LC70_STRAGE_FORMAT}},
    };

    if (enableUsbStatusMap.count(_activeButton) == 0) {
        // USBに無関係のボタン
        return false;
    }

    if (enableUsbStatusMap.value(_activeButton).contains(_model->getDeviceStatus().usba)) {
        // 実行可能なUSB状態になっている
        return false;
    }

    // ボタンごとに定義している、押下可能なUSB状態になっていなければエラー表示
    setBoolProperty(_activeButton, "press", false);
    emit requestToShowPopup(POPUP_FAILURE_NO_USB_CONFIRMATION); // USB未装着エラー
    return true;
}

void UserMeasurementSettingMenuViewModel::setBoolProperty(QString objectName, QString propertyName, bool value)
{
    QObject *target = nullptr;
    if (_view) {
        target = _view->findChild<QObject *>(objectName);
    }
    if (target) {
        target->setProperty(propertyName.toUtf8().constData(), value);
    }
}
