/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include "abstractviewmodel.h"
#include "mainframeviewmodel.h"
#include "buttonoperation.h"
#include <QDebug>

//#define UNEXPECTED_ERROR_POPUP;                           // 静電異常発生時ポップアップ表示
static constexpr int NULL_POINTER_DETECTION_TIMING = 10;    // _view が nullptr に鳴ったことを検出するタイマーの間隔[ms]

AbstractViewModel::AbstractViewModel(ScreenId screenId, LevelCheckerModel *&model, QObject *parent)
    : QObject(parent)
    , _view(nullptr)
    , _qmlFile()
    , _model(model)
    , _screenId(screenId)
    , _lastPressedButtonId(BUTTON_UNKNOWN)
    , _mainFrame(nullptr)
    , _takeOver()
    , _menuBarType(0)
    , _menuBarRank(0)
    , _pressDownState(PRESS_DOWN_LEVEL_0)
    , _useFrequencyInput(false)
    , _reactToLongPress(false)
    , _allowPowerSupply(false)
    , _pressedButtonId(BUTTON_UNKNOWN)
    , _acceptButtonInput(true)
    , _pressAndHoldTimer(new QTimer())
    , _pressAndHoldTimerInterval(0)
    , _pressCount(0)
    , _inAction(false)
    , _nullCheckTimer(new QTimer())
{
    // マップの初期化を実施
    _menuBarIndexMap = {
        {FIRST_FLOOR, 0}, {SECOND_FLOOR, 0}, {THIRD_FLOOR, 0}, {FOURTH_FLOOR, 0}
    };

    _nullCheckTimer->setInterval(NULL_POINTER_DETECTION_TIMING);
    _nullCheckTimer->setSingleShot(true);
}

AbstractViewModel::~AbstractViewModel()
{

}

void AbstractViewModel::initialize()
{

}

void AbstractViewModel::finalize()
{

}

void AbstractViewModel::resume()
{
    connect(_model, SIGNAL(keyStatus(LC70_KEY_STAT_T)), this, SLOT(receiveKeyStatus(LC70_KEY_STAT_T)),
            Qt::QueuedConnection);
    connect(_pressAndHoldTimer, SIGNAL(timeout()), this, SLOT(detectPressAndHold()));
}

void AbstractViewModel::suspend()
{
    // 非動作中
    _inAction = false;

    disconnect(_model, SIGNAL(keyStatus(LC70_KEY_STAT_T)), this, SLOT(receiveKeyStatus(LC70_KEY_STAT_T)));
    disconnect(_pressAndHoldTimer, SIGNAL(timeout()), this, SLOT(detectPressAndHold()));


    if(isMenuBarVisible()) {
        // 表示しているメニューバーを閉じる。
        closeAllMenuBar();
    }
}

void AbstractViewModel::standBy()
{
    // Home 画面でのみオーバーライドして呼び出す
}

void AbstractViewModel::createConnection(QObject *qml, QObject *mainFrame)
{
    _view = qml;
    _mainFrame = mainFrame;

    if (_view == nullptr) {
//        emit requestToShowPopup(POPUP_UNEXPECTED_ERROR);
        return;
    }

    // qml を指すポインターの NULL チェック
    connect(_nullCheckTimer, &QTimer::timeout, [this]() {
        if (_view == nullptr) {
#ifdef UNEXPECTED_ERROR_POPUP
            // 予期しないエラーのポップアップ通知
            emit requestToShowPopup(POPUP_UNEXPECTED_ERROR);
#else
            // ポップアップを表示せず、再度 MainFrame にある画面のオブジェクトにポインターを付け直す
            if (this->objectName() == QString("singleMeasurementViewModel")) {
                QObject *view = _model->search(QString(
                                           MAIN_FRAME_QML_OBJECT_NAME));
                if (view) {
                    _view = view->findChild<QObject *>(SINGLE_MEASUREMENT_QML_OBJECT_NAME);
                }
            } else if (this->objectName() == QString("multiMeasurementViewModel")) {
                QObject *view = _model->search(QString(
                                           MAIN_FRAME_QML_OBJECT_NAME));
                if (view) {
                    _view = view->findChild<QObject *>(MULTI_MEASUREMENT_QML_OBJECT_NAME);
                }
            } else {
                _view = _model->search(this->objectName());
                //emit requestToReload(_qmlFile);
            }
            if (_view) {
                recover();  // 必要に応じて復帰処理を行う
                _model->writeLog(_view->objectName(), QString("RESET view pointer!"), LOG_NOTICE);
            }
#endif
            return;
        }
        _nullCheckTimer->start();
    });
    _nullCheckTimer->start();
\
    connect(_view, SIGNAL(request(int)), this, SLOT(onRequest(int)));
    connect(this, SIGNAL(request(ScreenId, int)), _mainFrame, SIGNAL(request(ScreenId, int)));
    connect(this, SIGNAL(requestDirect(ScreenId, bool)), _mainFrame, SIGNAL(requestDirect(ScreenId, bool)));
    connect(this, SIGNAL(requestInHistory(ScreenId)), _mainFrame, SIGNAL(requestInHistory(ScreenId)));
    connect(this, SIGNAL(requestWithChangeHistory(ScreenId, QList<ScreenId>, bool)), _mainFrame,
            SIGNAL(requestWithChangeHistory(ScreenId, QList<ScreenId>, bool)));
    connect(this, SIGNAL(back()), _mainFrame, SIGNAL(back()));
    connect(this, SIGNAL(buttonClicked(ButtonId)), _mainFrame, SLOT(onButtonClicked(ButtonId)));
    connect(this, SIGNAL(requestToShowPopup(PopupId)), _mainFrame, SLOT(onRequestToShowPopup(PopupId)));
    connect(this, SIGNAL(requestToShowPopupScan(PopupId,int)), _mainFrame, SLOT(onRequestToShowPopupScan(PopupId, int)));
    connect(this, SIGNAL(requestToShowPopupWithMessage(PopupId, QString)), _mainFrame, SLOT(onRequestToShowPopupWithMessage(PopupId, QString)));
    connect(this, SIGNAL(requestToShowLowBatteryPopup()), _mainFrame, SLOT(onRequestToShowLowBatteryPopup()));
    connect(this, SIGNAL(requestPopupClose()), _mainFrame, SLOT(onRequestPopupClose()));
    connect(this, SIGNAL(requestToShowMenuBar(QVariant, QVariant, QVariant)), _mainFrame, SIGNAL(requestToShowMenuBar(QVariant, QVariant, QVariant)));
    connect(this, SIGNAL(requestToCloseMenuBar(QVariant)), _mainFrame, SIGNAL(requestToCloseMenuBar(QVariant)));
    connect(this, SIGNAL(requestToCloseAllMenuBar()), _mainFrame, SIGNAL(requestToCloseAllMenuBar()));
    connect(this, SIGNAL(notifyChangeInAction(bool)), _mainFrame, SLOT(onNotifyChangeInAction(bool)));
    connect(_mainFrame, SIGNAL(accept(PopupId, bool)), this, SLOT(onAccept(PopupId, bool)));
    connect(_mainFrame, SIGNAL(reject(PopupId)), this, SLOT(onReject(PopupId)));
    connect(_mainFrame, SIGNAL(rejectScanResult()), this, SLOT(onRejectScanResult()));
    connect(_mainFrame, SIGNAL(select(PopupId, int)), this, SLOT(onSelect(PopupId, int)));
    connect(_mainFrame, SIGNAL(notifyMenuBarSelected(int, int, int)), this, SLOT(onNotifyMenuBarSelected(int, int, int)));
    connect(_model, SIGNAL(lowBatteryStatusChanged(bool)), this, SLOT(onLowBatteryStatusChanged(bool)));
    connect(this, SIGNAL(requestToReload(QString)), _mainFrame, SLOT(onRequestToReload(QString)));
    connect(_mainFrame, SIGNAL(allowPowerSupplyChanged(bool)), this, SLOT(onAllowPowerSupplyChanged(bool)));
}

void AbstractViewModel::activate()
{
    qDebug() << Q_FUNC_INFO;
}

void AbstractViewModel::deactivate()
{
    disconnect(_view, SIGNAL(request(int)), this, SLOT(onRequest(int)));
    disconnect(this, SIGNAL(request(ScreenId, int)), _mainFrame, SIGNAL(request(ScreenId, int)));
    disconnect(this, SIGNAL(requestDirect(ScreenId, bool)), _mainFrame, SIGNAL(requestDirect(ScreenId, bool)));
    disconnect(this, SIGNAL(requestInHistory(ScreenId)), _mainFrame, SIGNAL(requestInHistory(ScreenId)));
    disconnect(this, SIGNAL(requestWithChangeHistory(ScreenId, QList<ScreenId>, bool)), _mainFrame,
               SIGNAL(requestWithChangeHistory(ScreenId, QList<ScreenId>, bool)));
    disconnect(this, SIGNAL(back()), _mainFrame, SIGNAL(back()));
    disconnect(this, SIGNAL(buttonClicked(ButtonId)), _mainFrame, SLOT(onButtonClicked(ButtonId)));
    disconnect(this, SIGNAL(requestToShowPopup(PopupId)), _mainFrame, SLOT(onRequestToShowPopup(PopupId)));
    disconnect(this, SIGNAL(requestToShowPopupScan(PopupId, int)), _mainFrame, SLOT(onRequestToShowPopupScan(PopupId, int)));
    disconnect(this, SIGNAL(requestToShowPopupWithMessage(PopupId, QString)), _mainFrame, SLOT(onRequestToShowPopupWithMessage(PopupId, QString)));
    disconnect(this, SIGNAL(requestToShowLowBatteryPopup()), _mainFrame, SLOT(onRequestToShowLowBatteryPopup()));
    disconnect(this, SIGNAL(requestPopupClose()), _mainFrame, SLOT(onRequestPopupClose()));
    disconnect(this, SIGNAL(requestToShowMenuBar(QVariant, QVariant, QVariant)), _mainFrame, SIGNAL(requestToShowMenuBar(QVariant, QVariant, QVariant)));
    disconnect(this, SIGNAL(requestToCloseMenuBar(QVariant)), _mainFrame, SIGNAL(requestToCloseMenuBar(QVariant)));
    disconnect(this, SIGNAL(requestToCloseAllMenuBar()), _mainFrame, SIGNAL(requestToCloseAllMenuBar()));
    disconnect(this, SIGNAL(notifyChangeInAction(bool)), _mainFrame, SLOT(onNotifyChangeInAction(bool)));
    disconnect(_mainFrame, SIGNAL(accept(PopupId, bool)), this, SLOT(onAccept(PopupId, bool)));
    disconnect(_mainFrame, SIGNAL(reject(PopupId)), this, SLOT(onReject(PopupId)));
    disconnect(_mainFrame, SIGNAL(rejectScanResult()), this, SLOT(onRejectScanResult()));
    disconnect(_mainFrame, SIGNAL(select(PopupId, int)), this, SLOT(onSelect(PopupId, int)));
    disconnect(_mainFrame, SIGNAL(notifyMenuBarSelected(int, int, int)), this, SLOT(onNotifyMenuBarSelected(int, int, int)));
    disconnect(_model, SIGNAL(lowBatteryStatusChanged(bool)), this, SLOT(onLowBatteryStatusChanged(bool)));
    disconnect(this, SIGNAL(requestToReload(QString)), _mainFrame, SLOT(onRequestToReload(QString)));
    disconnect(_mainFrame, SIGNAL(allowPowerSupplyChanged(bool)), this, SLOT(onAllowPowerSupplyChanged(bool)));

    _nullCheckTimer->stop();

    _takeOver.clear();

    // 長押し検出タイマー停止
    stopPressAndHoldTimer();
    _pressedButtonId = BUTTON_UNKNOWN;
    _useFrequencyInput = false;
    _reactToLongPress = false;
}

void AbstractViewModel::setTakeOver()
{

}

void AbstractViewModel::setAcceptButtonInput(bool accept)
{
    _acceptButtonInput = accept;
    if(accept){
        connect(_model, SIGNAL(keyStatus(LC70_KEY_STAT_T)), this, SLOT(receiveKeyStatus(LC70_KEY_STAT_T)),
                Qt::QueuedConnection);
    }else{
        disconnect(_model, SIGNAL(keyStatus(LC70_KEY_STAT_T)), this, SLOT(receiveKeyStatus(LC70_KEY_STAT_T)));
    }
}

void AbstractViewModel::readyToStandby()
{

}

void AbstractViewModel::takeOverReference(const QVariantMap &takeOver)
{
    _takeOver = takeOver;
}

bool AbstractViewModel::inAction() const
{
    return _inAction;
}

void AbstractViewModel::setQmlFile(const QString &qmlFile)
{
    _qmlFile = qmlFile;
}

void AbstractViewModel::setButtonActive(QObject *screen, const QString &buttonName, bool active)
{
    if (screen == nullptr) {
        return;
    }

    QObject *target = screen->findChild<QObject *>(buttonName);

    if (target) {
        target->setProperty("active", active);
    }
}

void AbstractViewModel::setButtonVisible(QObject *screen, const QString &buttonName, bool visible)
{
    if (screen == nullptr) {
        return;
    }

    QObject *target = screen->findChild<QObject *>(buttonName);

    if (target) {
        target->setProperty("visible", visible);
    }
}

void AbstractViewModel::onRequest(int condition)
{
    emit request(_screenId, condition);
}

void AbstractViewModel::receiveKeyStatus(LC70_KEY_STAT_T status)
{
    QString buttonState;
    for (int i = 0; i < LC70_KEY_ID_MAX; ++i) {
        buttonState += QString::number(static_cast<int>(status.stat[i]));
    }
    //_model->writeLog(QString(typeid(*this).name()), QString("RECEIVE BUTTON STATUS:%1").arg(buttonState));

    // ボタン入力の受付禁止なら何もしない
    if (!_acceptButtonInput) {
        return;
    }

    // ボタン状態
    static LC70_KEY_T buttonStatus[LC70_KEY_ID_MAX] = {
        LC70_KEY_OFF, LC70_KEY_OFF, LC70_KEY_OFF, LC70_KEY_OFF,
        LC70_KEY_OFF, LC70_KEY_OFF, LC70_KEY_OFF, LC70_KEY_OFF,
        LC70_KEY_OFF, LC70_KEY_OFF, LC70_KEY_OFF, LC70_KEY_OFF,
        LC70_KEY_OFF, LC70_KEY_OFF, LC70_KEY_OFF, LC70_KEY_OFF,
        LC70_KEY_OFF
    };

    // 前回取得したボタン状態と異なるボタンのリスト
    QList<int> buttons;

    for (int i = 0; i < LC70_KEY_ID_MAX; ++i) {
        // 前回取得したボタン状態と異なるボタンを検知
        if (status.stat[i] != buttonStatus[i]) {
            buttons.append(i);
        }
    }

    // 電源、給電ボタンを優先
    if (buttons.contains(LC70_KEY_ID_POWER)) {
        manipulateButton(LC70_KEY_ID_POWER, status);
    } else if (buttons.contains(LC70_KEY_ID_POWER_SUPPLY)) {
        manipulateButton(LC70_KEY_ID_POWER_SUPPLY, status);
    } else {
        // 電源、給電以外では ID がいちばん大きい前回と状態が異なるボタンを対象とする
        for (int i = buttons.count() - 1; i >= 0; --i) {
            if (buttons[i] == LC70_KEY_ID_POWER ||
                    buttons[i] == LC70_KEY_ID_POWER_SUPPLY) {
                continue;
            }
            manipulateButton(static_cast<LC70_KEYID_T>(buttons[i]), status);
            break;
        }
    }

    // 取得したボタン状態を保持
    for (int i = 0; i < LC70_KEY_ID_MAX; ++i) {
        buttonStatus[i] = status.stat[i];
    }
}

void AbstractViewModel::onAccept(PopupId popupId, bool okOnly)
{
    // 異常(_view が nullptr になる)を検知したときのポップアップを閉じたら前の画面に戻る
    Q_UNUSED(okOnly)
    if (popupId == POPUP_UNEXPECTED_ERROR) {
        emit requestToCloseAllMenuBar();
        emit back();
    }
}

void AbstractViewModel::onReject(PopupId popupId)
{
    Q_UNUSED(popupId)
    qDebug() << Q_FUNC_INFO;
}

void AbstractViewModel::onRejectScanResult()
{
    qDebug() << Q_FUNC_INFO;
}

void AbstractViewModel::onButtonPress(ButtonId id)
{
    Q_UNUSED(id)
}


void AbstractViewModel::onSelect(PopupId popupId, int index)
{
    Q_UNUSED(popupId)
    Q_UNUSED(index)
}

void AbstractViewModel::onLowBatteryStatusChanged(bool isLowBattery)
{
    _allowPowerSupply = (isLowBattery) ? false : true;
}

void AbstractViewModel::changePowerSupplyStatus(bool allowPowerSupply)
{
    Q_UNUSED(allowPowerSupply)
}

void AbstractViewModel::onAllowPowerSupplyChanged(bool allowPowerSupply)
{
    // 給電可/不可 が変化したときに仮想関数を呼び出す
    if (allowPowerSupply != _allowPowerSupply) {
        // 給電可/不可設定
        _allowPowerSupply = allowPowerSupply;

        changePowerSupplyStatus(allowPowerSupply);
    }
}

void AbstractViewModel::onNotifyMenuBarSelected(int type, int rank, int index)
{
    Q_UNUSED(type)
    Q_UNUSED(rank)
    Q_UNUSED(index)
}

void AbstractViewModel::manipulateButton(const LC70_KEYID_T &button, const LC70_KEY_STAT_T status)
{
    // ボタンIDの変換
    if (BUTTON_UNKNOWN == _pressedButtonId) {
        if (LC70_KEY_ON == status.stat[button]) {
            _pressedButtonId = static_cast<ButtonId>(button);
            QTime time = QTime::currentTime();
            static QTime press_before = QTime::currentTime();
            //QString message = QString("Button %1 pressed").arg(static_cast<int>(button));
            press_before = time;
            //_model->writeLog(QString(typeid(*this).name()), message);
            onButtonPress(_pressedButtonId);
            _lastPressedButtonId = _pressedButtonId;  // 最後に押下したIDを保持しておく
            // 長押し検出タイマー開始
            startPressAndHoldTimer();
        }
    } else {
        // Release された時に呼び出しを行う
        if (LC70_KEY_OFF == status.stat[_pressedButtonId]) {
            // 長押し検出タイマー停止
            stopPressAndHoldTimer();
            onKeyStatus(_pressedButtonId);
            // MainFrame に通知
            QTime time = QTime::currentTime();
            static QTime release_before = QTime::currentTime();
            //QString message = QString("Button %1 released").arg(static_cast<int>(button));
            release_before = time;
            //_model->writeLog(QString(typeid(*this).name()), message);
            emit buttonClicked(_pressedButtonId);
            _pressedButtonId = BUTTON_UNKNOWN;
        } else {
            // 押下状態が継続
            //QString message = QString("Button %1 pressed...").arg(static_cast<int>(button));
            //_model->writeLog(QString(typeid(*this).name()), message);
        }
    }
}

void AbstractViewModel::onKeyStatus(ButtonId id)
{
    if (!_acceptButtonInput) {
        return;
    }

    if (id == BUTTON_HOME) {
        qDebug() << QString("[DETECT] Home Button");
        emit requestToCloseAllMenuBar();
        emit requestDirect(SCREEN_HOME, true);
    } else if (id == BUTTON_ESCAPE) {
        emit requestToCloseAllMenuBar();
        emit back();
    } else {
        // nothing to do.
    }
}

void AbstractViewModel::initializeMenuBarInformation()
{
    // メニューバーに関わる情報を初期化する
    _menuBarRank = 0;
    _menuBarType = 0;
    _menuBarIndexMap = {
        {FIRST_FLOOR, 0}, {SECOND_FLOOR, 0}, {THIRD_FLOOR, 0}, {FOURTH_FLOOR, 0}
    };
}

void AbstractViewModel::changeInAction(const bool &inAction)
{
    // 動作中か否かの変更を通知
    if (_inAction != inAction) {
        _inAction = inAction;
        emit notifyChangeInAction(_inAction);
    }
}

void AbstractViewModel::recover()
{
    connect(_view, SIGNAL(request(int)), this, SLOT(onRequest(int)));
}

void AbstractViewModel::startPressAndHoldTimer()
{
    // 長押し対象キーのリスト
    static const QList<ButtonId> targetButtonList = {
        BUTTON_MENU,            // Menu
        BUTTON_BAND,            // Band
        BUTTON_UP,              // 上
        BUTTON_DOWN,            // 下
        BUTTON_RIGHT,           // 右
        BUTTON_LEFT,            // 左
        BUTTON_CHANNEL_DOWN,    // チャンネル Down
        BUTTON_CHANNEL_UP,      // チャンネル Up
    };

    MainFrameViewModel *object = static_cast<MainFrameViewModel *>(_mainFrame);

    if (object->buttonProhibition(_pressedButtonId)) {
        // 画面定義で無効に設定されているキーは、長押し検出タイマーを作動させない
        return;
    }

    // 押下しているボタンが長押し対象であるかチェック
    foreach (ButtonId id, targetButtonList) {
        if (_pressedButtonId == id) {

            // Menu, Band ボタンの場合、長押し反応させない場合は無視する
            if (id == BUTTON_MENU || id == BUTTON_BAND) {
                if (!_reactToLongPress) {
                    return;
                }
            }

            stopPressAndHoldTimer();

            _pressAndHoldTimerInterval = _model->applicationSetting()->firstRepeatTime();
            _pressCount = 0;

            // 長押し検出タイマー開始
            if (_pressAndHoldTimerInterval != 0) {
                _pressAndHoldTimer->start(_pressAndHoldTimerInterval);
            }

            return;
        }
    }
}

void AbstractViewModel::stopPressAndHoldTimer()
{
    if (!_pressAndHoldTimer->isActive()) {
        // タイマーが稼働していなかったら何もしない
        return;
    }

    _pressAndHoldTimer->stop();
    _pressAndHoldTimerInterval = 0;
    _pressCount = 0;
    _pressDownState = PRESS_DOWN_LEVEL_0;
}

void AbstractViewModel::detectPressAndHold()
{
    uint32_t count = _useFrequencyInput ? _model->applicationSetting()->lowRepeatCountForFrequency() : _model->applicationSetting()->lowRepeatCount();
    if (_pressCount < count) {
        // 長押し検知〜指定回数まで
        ++_pressCount;
        // 長押し間隔(低速)
        _pressAndHoldTimerInterval = _useFrequencyInput ? _model->applicationSetting()->repeatIntervalLowForFrequency() : _model->applicationSetting()->repeatIntervalLow();
        _pressDownState = PRESS_DOWN_LEVEL_1;
    } else {
        // 指定回数以降
        // 長押し間隔(高速)
        _pressAndHoldTimerInterval = _useFrequencyInput ? _model->applicationSetting()->repeatIntervalHighForFrequency() : _model->applicationSetting()->repeatIntervalHigh();
        _pressDownState = PRESS_DOWN_LEVEL_2;
    }

    // ボタンを押下する
    onKeyStatus(_pressedButtonId);
    //QString message = QString("Button %1 press and hold").arg(static_cast<int>(_pressedButtonId));
    //_model->writeLog(QString(typeid(*this).name()), message);
    emit buttonClicked(_pressedButtonId);

    // 次回の押下タイミングまでのタイマーを再設定
    _pressAndHoldTimer->start(_pressAndHoldTimerInterval);
}

bool AbstractViewModel::isMenuBarVisible()
{
    return _menuBarVisible;
}

void AbstractViewModel::setMenuBarVisible(bool isMenuBarVisible)
{
    _menuBarVisible = isMenuBarVisible;
}

void AbstractViewModel::closeCurrentMenuBar()
{
    emit requestToCloseMenuBar(_menuBarRank);
}

void AbstractViewModel::closeAllMenuBar()
{
    emit requestToCloseAllMenuBar();
    setMenuBarVisible(false);
    initializeMenuBarInformation();
}

bool AbstractViewModel::isKeyHolding(ButtonId id)
{
    return _pressAndHoldTimer->isActive() && (_pressedButtonId == id);
}

bool AbstractViewModel::isLowBattery()
{
    MainFrameViewModel *mainFrame = static_cast<MainFrameViewModel *>(_mainFrame);
    return mainFrame->isLowBattery();
}

BandTabIndex AbstractViewModel::identifyBandTab(const LC70_BAND_T &band, const QString &channel)
{
    BandTabIndex index = BAND_TAB_COUNT;
    switch (band) {
    case LC70_BAND_UHF:
        index = BAND_TAB_UHF;
        break;
    case LC70_BAND_BS:
    case LC70_BAND_CS:
    case LC70_BAND_BS_CS:
    {
        QList<ChannelDefinitionItem> channels;
        // 右旋
        _model->selectChannelDefinitionList(band, RIGHT_HANDED_POLARIZE, channels);
        for (int i = 0; i < channels.count(); ++i) {
            if (channels[i].channelNumber == channel) {
                if (band == LC70_BAND_BS) {
                    index = BAND_TAB_BS_R;
                } else if (band == LC70_BAND_CS) {
                    index = BAND_TAB_CS_R;
                }
                break;
            }
        }

        // 左旋
        channels.clear();
        _model->selectChannelDefinitionList(band, LEFT_HANDED_POLARIZE, channels);
        for (int i = 0; i < channels.count(); ++i) {
            if (channels[i].channelNumber == channel) {
                if (band == LC70_BAND_BS) {
                    index = BAND_TAB_BS_L;
                } else if (band == LC70_BAND_CS) {
                    index = BAND_TAB_CS_L;
                }
                break;
            }
        }
    }
        break;
    case LC70_BAND_JCSAT3:
    case LC70_BAND_JCSAT4:
    case LC70_BAND_SCCB:
    case LC70_BAND_SCCC:
    {
        QList<ChannelDefinitionItem> channels;
        // 水平
        _model->selectChannelDefinitionList(band, HORIZONTALLY_POLARIZE, channels);
        for (int i = 0; i < channels.count(); ++i) {
            if (channels[i].channelNumber == channel) {
                if (band == LC70_BAND_JCSAT3) {
                    index = BAND_TAB_JCSAT3A_H;
                } else if (band == LC70_BAND_JCSAT4) {
                    index = BAND_TAB_JCSAT4B_H;
                } else if (band == LC70_BAND_SCCB) {
                    index = BAND_TAB_SCCB2_H;
                } else if (band == LC70_BAND_SCCC) {
                    index = BAND_TAB_SCCC2_H;
                }
                break;
            }
        }

        // 垂直
        channels.clear();
        _model->selectChannelDefinitionList(band, VERTICALLY_POLARIZE, channels);
        for (int i = 0; i < channels.count(); ++i) {
            if (channels[i].channelNumber == channel) {
                if (band == LC70_BAND_JCSAT3) {
                    index = BAND_TAB_JCSAT3A_V;
                } else if (band == LC70_BAND_JCSAT4) {
                    index = BAND_TAB_JCSAT4B_V;
                } else if (band == LC70_BAND_SCCB) {
                    index = BAND_TAB_SCCB2_V;
                } else if (band == LC70_BAND_SCCC) {
                    index = BAND_TAB_SCCC2_V;
                }
                break;
            }
        }
    }
        break;
    case LC70_BAND_CATV_ALL:
    case LC70_BAND_CATV_90_470MHZ:
    case LC70_BAND_CATV_470_1002MHZ:
        index = BAND_TAB_CATV_ALL;
        break;
    case LC70_BAND_PILOT:
        index = BAND_TAB_PILOT;
        break;
    case LC70_BAND_SKYP:
        index = BAND_TAB_SKY_PERFECTV;
        break;
    case LC70_BAND_LTE:
        index = BAND_TAB_LTE;
        break;
    case LC70_BAND_FM:
        index = BAND_TAB_FM;
        break;
    case LC70_BAND_CATV_UP_STREAM:
        index = BAND_TAB_CATV_UPSTREAM;
        break;
    default:
        break;
    }

    return index;
}

int AbstractViewModel::batteryStatus() const
{
    MainFrameViewModel *mainFrame = static_cast<MainFrameViewModel *>(_mainFrame);
    return mainFrame->batteryStatus();
}

int AbstractViewModel::batteryLevel() const
{
    MainFrameViewModel *mainFrame = static_cast<MainFrameViewModel *>(_mainFrame);
    return mainFrame->batteryLevel();
}

int AbstractViewModel::countFilesRecursive(QString dirName)
{
    int count = 0;
    QDirIterator it(dirName, QDir::Files, QDirIterator::Subdirectories);
    while(it.hasNext()) {
        if (it.next() > 0) {
            count++;
        }
    }

    return count;
}

bool AbstractViewModel::allowPowerSupply() const
{
    return _allowPowerSupply;
}

void AbstractViewModel::resetToInitialButtonState(const QString &defaultButtonName, QString &activeButtonName, ButtonOperation &buttonOperation)
{
    if (defaultButtonName.isEmpty()) {
        // アクティブ化するボタンの名称が指定されていない場合は何もしない
        qDebug() << "defaultButtonName is Empty";
        return;
    }
    if (_view == nullptr) {
        // Viewにアクセスできない場合は何もしない
        qDebug() << "view is null";
        return;
    }

    QObject *target = nullptr;

    if (activeButtonName != defaultButtonName) {
        // アクティブ化したいボタンと、現在のアクティブボタンが異なる場合、非アクティブにしておく
        target = _view->findChild<QObject *>(activeButtonName);
        if (target) {
            target->setProperty("active", false);
            target->setProperty("press", false);
        } else {
            qDebug() << "target button not found. button name is " << activeButtonName << ".";
        }
        activeButtonName = defaultButtonName;
    }

    // ボタンをアクティブにする
    target = _view->findChild<QObject *>(activeButtonName);
    if (target) {
        target->setProperty("active", true);
        target->setProperty("press", false);
        buttonOperation.setActiveObject(activeButtonName);
    } else {
        qDebug() << "target button not found. button name is " << activeButtonName << ".(initial state)";
    }
}
