/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef ABSTRACTVIEWMODEL_H
#define ABSTRACTVIEWMODEL_H

#include <QObject>
#include "screenid.h"
#include "buttonid.h"
#include "popupid.h"
#include "levelcheckermodel.h"

class ButtonOperation;

/**
 * @brief ViewModel のベースクラス
 */
class AbstractViewModel : public QObject
{
    Q_OBJECT
public:
    /**
     * @brief コンストラクター
     * @param screenId 画面 ID
     * @param model Model クラスのポインター
     * @param parent 親オブジェクト
     */
    explicit AbstractViewModel(ScreenId screenId, LevelCheckerModel *&model, QObject *parent = nullptr);

    /**
     * @brief デストラクター
     */
    virtual ~AbstractViewModel();

    /**
     * @brief インスタンスを生成するときに呼び出す初期化処理
     */
    virtual void initialize(void);

    /**
     * @brief インスタンスを破棄するときに呼び出す終了処理
     */
    virtual void finalize(void);

    /**
     * @brief 画面を開くときに呼び出すスレッドの開始、再開処理
     */
    virtual void resume(void);

    /**
     * @brief 画面を閉じるときに呼び出すスレッドの停止処理
     */
    virtual void suspend(void);

    /**
     * @brief サブ CPU のスタンバイ
     */
    virtual void standBy(void);

    /**
     * @brief 画面を開くときに呼び出す qml との接続処理
     * @param qml qml のオブジェクトのポインター
     * @param mainFrame MainFrameViewModel のポインター
     */
    virtual void createConnection(QObject *qml, QObject *mainFrame);

    /**
     * @brief 画面を開くときに呼び出す表示内容の復元処理
     */
    virtual void activate(void);

    /**
     * @brief 画面を閉じるときに呼び出す表示な用の保存処理
     */
    virtual void deactivate(void);

    /**
     * @brief 遷移先画面で参照する遷移前画面のデータ設定
     */
    virtual void setTakeOver();

    /**
     * @brief ボタン入力を受け付けるか否かの設定
     * @param accept true: ボタン入力を受け付ける false: ボタン入力を受け付けない
     */
    virtual void setAcceptButtonInput(bool accept);

    /**
     * @brief シャットダウン直前の処理
     */
    virtual void readyToStandby(void);

    /**
     * @brief MainFrame が持つ QML 入力情報の参照
     * @param takeOver QML で選択した情報(測定種類, BAND, 局発周波数など)
     */
    void takeOverReference(const QVariantMap &takeOver);

    /**
     * @brief 動作中か否かを返す
     * @return 動作中か否か
     */
    bool inAction(void) const;

    /**
     * @brief qml ファイルのパスを設定
     * @param qmlFile qml ファイルのパス
     */
    void setQmlFile(const QString& qmlFile);

    /**
     * @brief 画面上のボタンのアクティブ/非アクティブを設定する
     * @param screen 画面(QML)
     * @param buttonName ボタン名
     * @param active アクティブ/非アクティブ
     */
    static void setButtonActive(QObject *screen, const QString &buttonName, bool active);

    /**
     * @brief 画面上のボタンの表示/非表示を設定する
     * @param screen 画面(QML)
     * @param buttonName ボタン名
     * @param active 表示/非表示
     */
    static void setButtonVisible(QObject *screen, const QString &buttonName, bool visible);

signals:
    /**
     * @brief 画面遷移要求
     * @param screenId 現在表示している画面の画面 ID
     * @param condition 遷移先を特定する情報(ボタンのインデックスなど)
     */
    void request(ScreenId currentScreenId, int condition = -1);

    /**
     * @brief 画面遷移要求
     * @param nextScreenId 次に遷移する画面 ID
     * @param clearHistory 履歴削除
     * @note 次の遷移したい画面を直接指定する
     */
    void requestDirect(ScreenId nextScreenId, bool clearHistory);

    /**
     * @brief 画面遷移要求
     * @param nextScreenId 次に遷移する画面 ID
     * @note 画面遷移履歴の中から次の遷移したい画面を直接指定する
     */
    void requestInHistory(ScreenId nextScreenId);

    /**
     * @brief 画面遷移要求
     * @param nextScreenId   次に遷移する画面 ID
     * @param history        画面遷移履歴 指定した値に画面遷移履歴を書き換える
     * @param requestTransit ユーザ/データロガー設定の測定画面からメニューバーを押下し、同一の測定画面に遷移した時のみfalseとする
     *                       falseの場合、画面遷移の実施せずに遷移履歴のみを変更する
     * @note 次の遷移したい画面を直接指定し、その画面に至る履歴を変更する
     */
    void requestWithChangeHistory(ScreenId nextScreenId, QList<ScreenId> history, bool requestTransit = true);

    /**
     * @brief 一つ前の画面に戻る
     * @note 現在 Home 画面にいる場合は無効
     */
    void back(void);

    /**
     * @brief ボタンクリックを MainFrame に通知
     * @param buttonId ボタン ID
     */
    void buttonClicked(ButtonId buttonId);

    /**
     * @brief ポップアップを開くよう要求
     * @param popupId ポップアップ ID
     */
    void requestToShowPopup(PopupId popupId);

    /**
     * @brief ポップアップを開くよう要求
     * @param popupId ポップアップ ID
     * @param detailId 詳細 ID
     */
    void requestToShowPopupScan(PopupId popupId, int detailId);

    /**
     * @brief 指定した内容でポップアップを開くよう要求
     * @param popupId ポップアップ ID
     * @param message メッセージ内容
     */
    void requestToShowPopupWithMessage(PopupId popupId, QString message);

    /**
     * @brief バッテリ低下状態ポップアップを開くよう要求
     */
    void requestToShowLowBatteryPopup();

    /**
     * @brief ポップアップのClose要求を通知
     */
    void requestPopupClose(void);

    /**
     * @brief メニューバーを開くよう要求
     * @param type メニューバーの種類(0: 測定 1: USER 2: 簡易スペアナ)
     * @param rank 階層
     * @param index メニューバーに表示する内容を指示するインデックス(MenuBarModel.qml 参照)
     */
    void requestToShowMenuBar(QVariant type, QVariant rank, QVariant index);

    /**
     * @brief メニューバーを閉じるよう要求
     * @param rank 階層
     */
    void requestToCloseMenuBar(QVariant rank);

    /**
     * @brief 全てのメニューバーを閉じるよう要求
     */
    void requestToCloseAllMenuBar(void);

    /**
     * @brief ファイルリストを設定する
     * @param file ファイルリスト
     */
    void requestToSetFileList(QVariant fileList);

    /**
     * @brief ファイルリストをクリアする
     */
    void requestToClearFileList();

    /**
     * @brief 引き継ぎデータの変更要求
     * @param key 引き継ぎデータのキー
     * @param value 引き継ぎデータの値
     */
    void changeAttribute(QString key, QVariant value);

    /**
     * @brief 引き継ぎデータの削除要求
     * @param key 引き継ぎデータのキー
     */
    void removeAttribute(QString key);

    /**
     * @brief 動作中か否かの変更を通知
     * @param inAction 動作中か否か
     */
    void notifyChangeInAction(bool inAction);

    /**
     * @brief View の再ロード依頼
     * @param value qml ファイルパス
     * @note 未使用
     */
    void requestToReload(QString qmlFile);

protected:
    /**
     * @brief 押下ボタンの通知を受けた時に行う処理
     * @param id 押下ボタンのID
     */
    virtual void onKeyStatus(ButtonId id);

    virtual void onButtonPress(ButtonId id);

    /**
     * @brief メニューバーに関わる情報を初期化する
     */
    virtual void initializeMenuBarInformation(void);

    /**
     * @brief 動作中か否かを変更する
     * @param inAction 動作中か否か
     */
    virtual void changeInAction(const bool &inAction);

    /**
     * @brief 必要に応じてView を指すポインターを付け替えた後の復帰処理を行う
     */
    virtual void recover(void);

    /**
     * @brief メニューバー表示ステータスの取得
     * @return メニューバー表示ステータス (True: 表示中, False: 非表示)
     */
    bool isMenuBarVisible(void);

    /**
     * @brief メニューバー表示ステータスの更新
     * @param isMenuBarVisible 設定するステータス (True: 表示中, False 非表示中)
     */
    void setMenuBarVisible(bool isMenuBarVisible);

    /**
     * @brief 現在操作中のメニューバーを閉じる
     */
    void closeCurrentMenuBar(void);

    /**
     * @brief 表示中の全てのメニューバーを閉じる
     */
    void closeAllMenuBar(void);

    /**
     * @brief ボタン長押し中かどうかを取得
     * @param id 押下ボタンのID
     * @return　ボタン長押し中かどうか
     */
    bool isKeyHolding(ButtonId id);

    /**
     * @brief バッテリ低下状態取得
     * @return バッテリ低下状態かどうか
     */
    bool isLowBattery();

    /**
     * @brief Band ID とチャンネル名に該当する Band タブのインデックスを返す
     * @param band Band
     * @param channel チャンネル名
     * @return 該当する Band タブのインデックス
     */
    BandTabIndex identifyBandTab(const LC70_BAND_T &band, const QString &channel);

    /**
     * @brief バッテリー充電状態を返す
     * @return バッテリー充電状態
     */
    int batteryStatus(void) const;

    /**
     * @brief バッテリーレベルを返す
     * @return バッテリーレベル
     */
    int batteryLevel(void) const;

    /**
     * @brief ファイル数の再帰的な取得
     * @param dirName ディレクトリ名
     */
    int countFilesRecursive(QString dirName);

    /**
     * @brief 給電可/不可を返す
     * @return 給電可/不可
     */
    bool allowPowerSupply(void) const;

    /**
     * @brief ボタン選択状態のリセット
     * @param defaultButtonName 初期選択したいボタンの名称
     * @param activeButtonName 最後に選択していたボタンの名称
     * @param buttonOperation ボタン操作オブジェクト
     */
    void resetToInitialButtonState(const QString &defaultButtonName, QString &activeButtonName, ButtonOperation &buttonOperation);

    /**
     * @brief 長押し状態
     */
    enum PRESS_DOWN_BUTTON_STATE {
        PRESS_DOWN_LEVEL_0 = 0,
        PRESS_DOWN_LEVEL_1,
        PRESS_DOWN_LEVEL_2,
    };

    QObject *_view;                     //!< qml ファイルをロードした実体
    QString _qmlFile;                   //!< qml ファイルのパス
    LevelCheckerModel *_model;          //!< Model クラスのポインター
    ScreenId _screenId;                 //!< 画面 ID
    ButtonId _lastPressedButtonId;      //!< 最後に受け付けたボタン ID
    QObject *_mainFrame;                //!< MainFrame オブジェクト(MainFrame 以外の画面で使用する)
    QVariantMap _takeOver;              //!< 前画面までに QML で設定した引き継ぎ情報
    int _menuBarType;                   //!< メニューバーの種類
    int _menuBarRank;                   //!< メニューバーの階層
    QMap<int, int> _menuBarIndexMap;    //!< 現在の階層の内容を示すインデックス
    PRESS_DOWN_BUTTON_STATE _pressDownState;    //!< 長押し状態
    bool _useFrequencyInput;            //!< 周波数入力有無
    bool _reactToLongPress;             //!< Menu, Band ボタンの長押し対応
    bool _allowPowerSupply;             //!< 給電可/不可

protected slots:
    /**
     * @brief QML からの画面遷移要求を受信
     * @param condition 遷移先を特定する情報(ボタンのインデックスなど)
     */
    virtual void onRequest(int condition);

    /**
     * @brief キー通知を受信する
     * @param status ボタンの押下状態
     */
    void receiveKeyStatus(LC70_KEY_STAT_T status);

    /**
     * @brief ポップアップから受諾の通知を受信
     * @param okOnly OK のみの場合 true
     */
    virtual void onAccept(PopupId popupId, bool okOnly);

    /**
     * @brief ポップアップから拒否の通知を受信
     */
    virtual void onReject(PopupId popupId);

    /**
     * @brief スキャン結果ポップアップから拒否の通知を受信
     */
    virtual void onRejectScanResult();

    /**
     * @brief ポップアップから選択の通知を受信
     * @param index 選択されたインデックス
     */
    virtual void onSelect(PopupId popupId, int index);

    /**
     * @brief メニューバーの選択内容を受信
     * @param type メニューバーの種類
     * @param rank 階層
     * @param index 選択されたリスト中のインデックス
     * @note ここでの index は ListView の currentIndex であり、表示内容を指示するインデックスではない
     */
    virtual void onNotifyMenuBarSelected(int type, int rank, int index);

    /**
     * @brief バッテリ低下状態変化通知(5% 以下)
     * @param isLowBattery バッテリ低下状態かどうか
     */
    virtual void onLowBatteryStatusChanged(bool isLowBattery);

    /**
     * @brief 給電処理の変更
     * @param allowPowerSupply バッテリー 10% 以下で false, 11 % 以上で true
     * @note _allowPowerSupply の値を参照して給電処理を変更する
     */
    virtual void changePowerSupplyStatus(bool allowPowerSupply);

private:
    ButtonId _pressedButtonId;
    bool _acceptButtonInput;            //!< ボタン入力の受付許可
    QTimer *_pressAndHoldTimer;         //!< ボタン長押し検出タイマー
    uint _pressAndHoldTimerInterval;    //!< ボタン長押しタイマー値
    uint _pressCount;                   //!< 長押し中のpress発火回数
    bool _inAction;                     //!< 動作中か否か
    bool _menuBarVisible;               //!< メニューバー表示ステータス
    QTimer *_nullCheckTimer;            //!< qml 画面を指すポインターが nullptr になったかを定期的にチェックするためのタイマー

    /**
     * @brief ボタンの操作
     * @param button ボタン ID
     * @param status 各ボタンの ON/OFF
     */
    void manipulateButton(const LC70_KEYID_T &button, const LC70_KEY_STAT_T status);

    /**
     * @brief 長押しタイマー開始
     */
    void startPressAndHoldTimer(void);

    /**
     * @brief 長押しタイマー停止
     */
    void stopPressAndHoldTimer(void);

private slots:
    /**
     * @brief 長押し検出タイマータイムアップ
     */
    void detectPressAndHold(void);

    /**
     * @brief 給電可能状態の変化通知を受信
     * @param allowPowerSupply バッテリー 10% 以下で false, 11 % 以上で true
     */
    void onAllowPowerSupplyChanged(bool allowPowerSupply);
};

#endif // ABSTRACTVIEWMODEL_H
