/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include <QDebug>
#include <QTimer>
#include "errormanager.h"
#include "errormanagerdef.h"

ErrorManager::ErrorManager(LevelCheckerModel *&model, QObject *parent)
    : QObject(parent)
    , _model(model)
    , _errorPopup(nullptr)
    , _pressedButtonId(BUTTON_UNKNOWN)
    , _pressAndHoldTimer(new QTimer())
    , _pressAndHoldTimerInterval(0)
    , _pressCount(0)
    , _failList()
    , _displayFailCode()
    , _displayProcessType(ProcessType::INVALID)
    , _isWaitReNotification(false)
    , _isEnabledCompass(true)
{
    connect(_model, SIGNAL(keyStatus(LC70_KEY_STAT_T)), this, SLOT(onKeyStatus(LC70_KEY_STAT_T)));
    connect(_pressAndHoldTimer, SIGNAL(timeout()), this, SLOT(detectPressAndHold()));
    connect(this, SIGNAL(notifyChangeErrorStatus()), this, SLOT(onNotifyChangeErrorStatus()));
}

ErrorManager::~ErrorManager()
{
    disconnect(_model, SIGNAL(keyStatus(LC70_KEY_STAT_T)), this, SLOT(onKeyStatus(LC70_KEY_STAT_T)));
    disconnect(_pressAndHoldTimer, SIGNAL(timeout()), this, SLOT(detectPressAndHold()));
    disconnect(this, SIGNAL(notifyChangeErrorStatus()), this, SLOT(onNotifyChangeErrorStatus()));
}

bool ErrorManager::isOccurring(uint16_t failCode) {
    for (int i = 0; i < _failList.size(); ++i) {
        if (_failList[i].code == failCode) {
            // 既に発生を認識している内容
            return true;
        }
    }

    // 未発生
    return false;
}

void ErrorManager::receiveFailInfo(LC70_FAIL_INFO_T failInformation)
{
    // 新規故障であるか判断
    if (!isNewNotification(failInformation)) {
        return;
    }

    switch (failInformation.ucCtl) {
    case LC70_FAIL_CTL_MSGDISP:     // メッセージ表示
        showPopup(failInformation.usCode, failInformation.ucCtl);
        break;
    case LC70_FAIL_CTL_MSGDISP_CLR: // メッセージ表示(メッセージを閉じる時に復旧)
        showPopup(failInformation.usCode, failInformation.ucCtl);
        break;
//    case LC70_FAIL_CTL_REBOOT:      // 再起動
//    case LC70_FAIL_CTL_NONE:        // なし（継続）
//    case LC70_FAIL_CTL_SHUTDOWN:    // シャットダウン
//    case LC70_FAIL_CTL_UPDATE:      // アップデート
    default:
        break;
    }
}

bool ErrorManager::isNewNotification(const LC70_FAIL_INFO_T failInformation)
{
    // 発生通知か、復旧通知かを判断
    if (failInformation.ucStat == LC70_FAIL_STAT_FAIL) {
        if (isOccurring(failInformation.usCode)) {
            // 既に発生を認識している内容
            return false;
        }

    } else if (failInformation.ucStat == LC70_FAIL_STAT_NORMAL) {
        if (!isOccurring(failInformation.usCode)) {
            // 既に復旧を認識している内容
            return false;
        }

        for (int i = 0; i < _failList.size(); ++i) {
            if (_failList[i].code == failInformation.usCode) {
                // 復旧したため情報を削除する
                _failList.removeAt(i);
                emit notifyChangeErrorStatus();
                break;
            }
        }
        return false;

    } else {
        // 不明な通知
        return false;
    }

    if (_displayFailCode != LC70_FAIL_NONE) {
        // 現在エラーポップアップを表示中であるため、新しく発生した故障との優先順位を比較する
        if (!isHighPriority(failInformation.usCode)) {
            // 優先順位が低いため、現在の故障表示を解除するまで無視する
            return false;
        }

        // 優先順位が高い故障が発生したため、表示を切り替える
        // 一旦現在の故障は非発生状態にする
        // 優先順位が高い故障の表示を解除した際に、まだ発生状態であればその時改めて情報を採用する
        for (int i = 0; i < _failList.size(); ++i) {
            if (_failList.at(i).code == _displayFailCode) {
                _failList.removeAt(i);
                _displayFailCode = LC70_FAIL_NONE;
                break;
            }
        }
    }

    // 新規通知であるため、情報を新規追加する
    char addInfo[LC70_FAIL_ADD_INFO_NUM];
    memset(addInfo, 0x00, LC70_FAIL_ADD_INFO_NUM);
    for (int i = 0; i < LC70_FAIL_ADD_INFO_NUM; ++i) {
        if (failInformation.ucAddInfo == 0x00) {
            break;
        }
        addInfo[i] = failInformation.ucAddInfo[i];
    }
    _failList.append(errorPopupInfo(failInformation.usCode,
                                    failInformation.ucCtl,
                                    failInformation.usWait,
                                    QString(addInfo)));

    emit notifyChangeErrorStatus();
    return true;
}

void ErrorManager::showPopup(const uint16_t failCode, uint8_t action)
{
    // 故障コードに応じたメッセージを表示する(10種類)
    // 再表示の待機時間満了という条件でも表示する
    // 引数は故障コードにしておき、指定の故障コードが現在発生していなかったら
    // 表示要求は無視するようにしてみる
    // そのため待機時間はポップアップ閉じた際にシングルショットで対応
    static const QMap<uint16_t, MessageInfo> messageInfoMap = {
        {LC70_FAIL_FILE_ERR,                {MSG_FILE_ERR,          true,  ProcessType::CLOSE_POPUP}},
        {LC70_FAIL_MODEL_GAUGE_ERR,         {MSG_MODEL_GAUGE_ERR,   true,  ProcessType::CLOSE_POPUP}},
        {LC70_FAIL_DCDC_ERR,                {MSG_DCDC_ERR,          true,  ProcessType::NOTIFY_POWOER_SUPPLY_ERROR}},
        {LC70_FAIL_SUB_ERR,                 {MSG_SUB_ERR,           true,  ProcessType::CLOSE_POPUP}},
        {LC70_FAIL_PARAM_ERR,               {MSG_PARAM_ERR,         true,  ProcessType::CLOSE_POPUP}},
        {LC70_FAIL_MSG_SD_FULL,             {MSG_SD_FULL,           false, ProcessType::CLOSE_POPUP}},
        {LC70_FAIL_MSG_RTC_ERR,             {MSG_RTC_ERR,           false, ProcessType::TRANSIT_HOME}},
        {LC70_FAIL_MSG_OVER_CURR_ERR,       {MSG_OVER_CURR_ERR,     false, ProcessType::NOTIFY_POWOER_SUPPLY_ERROR}},
        {LC70_FAIL_MSG_DCDC_TEMP_ERR,       {MSG_DCDC_TEMP_ERR,     false, ProcessType::NOTIFY_POWOER_SUPPLY_ERROR}},
        {LC70_FAIL_USBC_PD_ERR,             {MSG_USBC_PD_ERR,       false, ProcessType::CLOSE_POPUP}},
        {LC70_FAIL_MEAS_PARAM_ERR,          {MSG_MEAS_PARAM_ERR,    true,  ProcessType::CLOSE_POPUP}},
        {LC70_FAIL_MSG_PRGUPDATE_COMPLATE,  {MSG_UPDATE_SUCCESS,    false, ProcessType::CLOSE_POPUP}},
        {LC70_FAIL_MSG_PRGUPDATE_ERR,       {MSG_UPDATE_ERR,        true,  ProcessType::CLOSE_POPUP}},
    };

    if (!isOccurring(failCode)) {
        return;
    }

    if (messageInfoMap.count(failCode) == 0) {
        return;
    }

    MessageInfo messageInfo = messageInfoMap.value(failCode);
    QString message = messageInfo.message;
    if (messageInfo.hasAddInfo) {
        message += getAddInfo(failCode);
    }

    if (messageInfo.processType == ProcessType::NOTIFY_POWOER_SUPPLY_ERROR) {
        // 給電異常については、マニュアル、USER測定で異なる挙動が必要であるため、
        // 発生を画面に通知し、ハンドリングはまかせる
        emit notifyPowerSupplyError(failCode, action, message);
        return;
    }

    _displayFailCode = failCode;
    _displayProcessType = messageInfo.processType;

    // メッセージ表示
    emit requestToOpenErrorPopup(message, QString("戻る"));

    _errorPopup = _model->search(QString("ErrorPopup"));
    connect(this, SIGNAL(requestToClose()), _errorPopup, SLOT(closePopup()));
}

QString ErrorManager::getAddInfo(const uint16_t failCode)
{
    for (int i = 0; i < _failList.size(); ++i) {
        if (_failList[i].code == failCode) {
            return _failList[i].addInfo;
        }
    }

    return QString("");
}

bool ErrorManager::enabledCompass()
{
    return _isEnabledCompass;
}

void ErrorManager::checkEnableCompass()
{
    // コンパスを非表示にする必要がある故障
    static const QList<uint16_t> magnetErrorList = {
        LC70_FAIL_DEV_MAG_ERR,      /* 磁気センサ異常 */
        LC70_FAIL_DEV_MAG_I2C_ERR,  /* I2C通信エラー(for 磁気センサ） */
    };

    bool isError = false;
    for (int i = 0; i < _failList.size(); ++i) {
        if (magnetErrorList.contains(_failList.at(i).code)) {
            // コンパス異常発生中
            isError = true;
            break;
        }
    }

    if (_isEnabledCompass == !isError) {
        // 現状と変化なければ何もしない
        return;
    }

    _isEnabledCompass = !isError;
    emit notifyChangeCompassEnable(_isEnabledCompass);
}

void ErrorManager::onKeyStatus(LC70_KEY_STAT_T keyStatus)
{
    // ポップアップが開いていない場合のボタンの受信は無視する
    if (_displayFailCode == LC70_FAIL_NONE) {
        return;
    }

    // ボタン状態
    static LC70_KEY_T buttonStatus[LC70_KEY_ID_MAX] = {
        LC70_KEY_OFF, LC70_KEY_OFF, LC70_KEY_OFF, LC70_KEY_OFF,
        LC70_KEY_OFF, LC70_KEY_OFF, LC70_KEY_OFF, LC70_KEY_OFF,
        LC70_KEY_OFF, LC70_KEY_OFF, LC70_KEY_OFF, LC70_KEY_OFF,
        LC70_KEY_OFF, LC70_KEY_OFF, LC70_KEY_OFF, LC70_KEY_OFF,
        LC70_KEY_OFF
    };

    // 前回取得したボタン状態と異なるボタンのリスト
    QList<int> buttons;

    for (int i = 0; i < LC70_KEY_ID_MAX; ++i) {
        // 前回取得したボタン状態と異なるボタンを検知
        if (keyStatus.stat[i] != buttonStatus[i]) {
            buttons.append(i);
        }
    }

    // 電源、給電ボタンを優先
    if (buttons.contains(LC70_KEY_ID_POWER)) {
        manipulateButton(LC70_KEY_ID_POWER, keyStatus);
    } else if (buttons.contains(LC70_KEY_ID_POWER_SUPPLY)) {
        manipulateButton(LC70_KEY_ID_POWER_SUPPLY, keyStatus);
    } else {
        // 電源、給電以外では ID がいちばん大きい前回と状態が異なるボタンを対象とする
        for (int i = buttons.count() - 1; i >= 0; --i) {
            if (buttons[i] == LC70_KEY_ID_POWER ||
                    buttons[i] == LC70_KEY_ID_POWER_SUPPLY) {
                continue;
            }

            manipulateButton(static_cast<LC70_KEYID_T>(buttons[i]), keyStatus);
            break;
        }
    }

    // 取得したボタン状態を保持
    for (int i = 0; i < LC70_KEY_ID_MAX; ++i) {
        buttonStatus[i] = keyStatus.stat[i];
    }
}

void ErrorManager::manipulateButton(const LC70_KEYID_T &button, const LC70_KEY_STAT_T status)
{
    static ButtonId id = BUTTON_UNKNOWN;

    // ボタンIDの変換
    if (BUTTON_UNKNOWN == id) {
        if (LC70_KEY_ON == status.stat[button]) {
            id = static_cast<ButtonId>(button);
            QString message = QString("Button %1 pressed").arg(static_cast<int>(button));
            _model->writeLog(QString(typeid(*this).name()), message);
            // 長押し検出タイマー開始
            _pressedButtonId = id;
            startPressAndHoldTimer();
        }
    } else {
        // Release された時に呼び出しを行う
        if (LC70_KEY_OFF == status.stat[id]) {
            // 長押し検出タイマー停止
            stopPressAndHoldTimer();
            QString message = QString("Button %1 released").arg(static_cast<int>(button));
            _model->writeLog(QString(typeid(*this).name()), message);
            inputButton(id);
            id = BUTTON_UNKNOWN;
        } else {
            // 押下状態が継続
            QString message = QString("Button %1 pressed...").arg(static_cast<int>(button));
            _model->writeLog(QString(typeid(*this).name()), message);
        }
    }
}

void ErrorManager::startPressAndHoldTimer()
{
    // 長押し対象キーのリスト
    static const QList<ButtonId> normalTargetList = {BUTTON_RIGHT, BUTTON_LEFT};

    // 押下しているボタンが長押し対象であるかチェック
    foreach (ButtonId id, normalTargetList) {
        if (_pressedButtonId == id) {
            stopPressAndHoldTimer();

            _pressAndHoldTimerInterval = _model->applicationSetting()->firstRepeatTime();
            _pressCount = 0;

            // 長押し検出タイマー開始
            if (_pressAndHoldTimerInterval != 0) {
                _pressAndHoldTimer->start(_pressAndHoldTimerInterval);
            }

            return;
        }
    }
}

void ErrorManager::stopPressAndHoldTimer()
{
    if (!_pressAndHoldTimer->isActive()) {
        // タイマーが稼働していなかったら何もしない
        return;
    }

    _pressAndHoldTimer->stop();
    _pressAndHoldTimerInterval = 0;
    _pressCount = 0;
}

void ErrorManager::detectPressAndHold()
{
    if (_pressCount < _model->applicationSetting()->lowRepeatCount()) {
        // 長押し検知〜指定回数まで
        ++_pressCount;
        // 長押し間隔(低速)
        _pressAndHoldTimerInterval = _model->applicationSetting()->repeatIntervalLow();
    } else {
        // 指定回数以降
        // 長押し間隔(高速)
        _pressAndHoldTimerInterval = _model->applicationSetting()->repeatIntervalHigh();
    }

    // ボタンを押下する
    QString message = QString("Button %1 released").arg(static_cast<int>(_pressedButtonId));
    _model->writeLog(QString(typeid(*this).name()), message);
    inputButton(_pressedButtonId);

    // 次回の押下タイミングまでのタイマーを再設定
    _pressAndHoldTimer->start(_pressAndHoldTimerInterval);
}

void ErrorManager::pressButton(ButtonId buttonId)
{
    if (buttonId != BUTTON_ENTER) {
        return;
    }

    propertyChange("m1", "keyPressed", true);
    propertyChange("m1", "press", true);
}

void ErrorManager::inputButton(ButtonId buttonId)
{
    propertyChange("m1", "press", false);

    switch (buttonId) {
    case BUTTON_ENTER:
        enterKey();
        break;

    case BUTTON_ESCAPE:
        escapeKey();
        break;

    default:
        break;
    }
}

void ErrorManager::enterKey()
{
    propertyChange("m1", "keyPressed", false);
    onReject();
}

void ErrorManager::escapeKey()
{
    closeAction();
}

void ErrorManager::propertyChange(QString buttonName, QString propertyName, QVariant value)
{
    QObject *target = nullptr;
    if (_errorPopup) {
        target = _errorPopup->findChild<QObject *>(buttonName);
    }

    if (target) {
        target->setProperty(propertyName.toUtf8().constData(), value);
    }
}

void ErrorManager::close()
{
    // ポップアップを閉じるよう QML に通知
    emit requestToClose();
    disconnect(this, SIGNAL(requestToClose()), _errorPopup, SLOT(closePopup()));

    emit closed();

    // 再表示待機時間が設定されている場合、指定時間後に再表示する
    if (isOccurring(_displayFailCode)) {
        // エラーポップアップの再表示
        setReNotificationTimer(_displayFailCode);
    }

    // 復旧通知が必要な場合は、故障状態を通知する
    if (isRecoveryProc(_displayFailCode)) {
        uint8_t addInfo[LC70_FAIL_ADD_INFO_NUM];
        memset(addInfo, 0x00, LC70_FAIL_ADD_INFO_NUM);
        //_model->setFaultInformation(_displayFailCode, LC70_FAIL_STAT_NORMAL, addInfo);     故障復旧処理不要
    }

    _displayFailCode = LC70_FAIL_NONE;
}

void ErrorManager::onReject()
{
    closeAction();
}

void ErrorManager::closeAction()
{
    // 表示中のポップアップを閉じる
    close();

    switch (_displayProcessType) {
    case ProcessType::TRANSIT_HOME:
        // ホーム画面に遷移する
        if (_controller != nullptr && _controller->currentScreenId() != SCREEN_HOME) {
            emit requestDirect(SCREEN_HOME, true);
        }
        break;

    default:
        break;
    }
}

void ErrorManager::setReNotificationTimer(const uint16_t failCode)
{
    for (int i = 0; i < _failList.size(); ++i) {
        if (_failList[i].code == failCode) {

            if (_failList[i].interval == 0) {
                // 再表示しない設定であるため終了
                return;
            }
            // 再表示までの時間を設定
            _failList[i].reminingTime = _failList[i].interval;
            break;

        }
    }

    if (!_isWaitReNotification) {
        QTimer::singleShot(1000, this, SLOT(checkReNotification()));
        _isWaitReNotification = true;
    }
}

void ErrorManager::checkReNotification() {
    bool isSetTimer = false;
    for (int i = 0; i < _failList.size(); ++i) {
        if (_failList[i].reminingTime == 0) {
            continue;
        }

        // 再表示までの時間をカウントダウン
        --(_failList[i].reminingTime);
        if (_failList[i].reminingTime != 0) {
            isSetTimer = true;
            continue;
        }

        // カウント0で再表示
        if (_displayFailCode != LC70_FAIL_NONE) {
            // 現在別のポップアップを表示中であるため、それを閉じるまで再表示しない
            // 次のカウントで再チェックするため、カウントを延長する
            ++(_failList[i].reminingTime);
            isSetTimer = true;
        } else {
            // 再表示する
            showPopup(_failList[i].code, _failList[i].reaction);
        }
    }

    if (isSetTimer) {
        QTimer::singleShot(1000, this, SLOT(checkReNotification()));
    } else {
        // 再表示待機中のメッセージがなくなった
        _isWaitReNotification = false;
    }
}

bool ErrorManager::isOccuredPowerSupplyError(void)
{
    static const QList<uint16_t> powerSupplyErrorList = {
        LC70_FAIL_DCDC_ERR,
        LC70_FAIL_MSG_OVER_CURR_ERR,
        LC70_FAIL_MSG_DCDC_TEMP_ERR,
    };

    foreach (uint16_t code, powerSupplyErrorList) {
        if (isOccurring(code)) {
            return true;
        }
    }

    return false;
}

void ErrorManager::onNotifyChangeErrorStatus()
{
    // コンパス非表示対象のエラーを確認する
    checkEnableCompass();
}

void ErrorManager::onRequestToCloseErrorPopup(bool clearErrorList)
{
    if (clearErrorList) {
        _failList.clear();
    }

    emit requestToClose();
}

void ErrorManager::attachController(ScreenTransitionController *controller)
{
    _controller = controller;
}

bool ErrorManager::isHighPriority(uint16_t code) {
    static const QMap<uint16_t, int> priorityMap = {
        {LC70_FAIL_NONE,                    -1},
        {LC70_FAIL_FILE_ERR,                1},
        {LC70_FAIL_DEV_MAG_ERR,             1},
        {LC70_FAIL_DEV_MAG_I2C_ERR,         1},
        {LC70_FAIL_MODEL_GAUGE_ERR,         1},
        {LC70_FAIL_DCDC_ERR,                1},
        {LC70_FAIL_SUB_ERR,                 1},
        {LC70_FAIL_PARAM_ERR,               1},
        {LC70_FAIL_MSG_SD_FULL,             1},
        {LC70_FAIL_MSG_RTC_ERR,             1},
        {LC70_FAIL_MSG_OVER_CURR_ERR,       1},
        {LC70_FAIL_MSG_DCDC_TEMP_ERR,       1},
        {LC70_FAIL_USBC_PD_ERR,             1},
        {LC70_FAIL_MEAS_PARAM_ERR,          1},
        {LC70_FAIL_MSG_PRGUPDATE,           1},
        {LC70_FAIL_MSG_PRGUPDATE_COMPLATE,  1},
        {LC70_FAIL_MSG_PRGUPDATE_ERR,       1},
    };

    if (priorityMap.value(_displayFailCode, -1) < priorityMap.value(code, -1)) {
        return true;
    }

    return false;
}

bool ErrorManager::isRecoveryProc(uint16_t code)
{
    for (int i = 0; i < _failList.size(); ++i) {
        if (_failList[i].code == code) {
            return _failList[i].reaction == LC70_FAIL_CTL_MSGDISP_CLR;
        }
    }
    return false;
}
