/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef ERRORMANAGER_H
#define ERRORMANAGER_H

#include <QObject>
#include "levelcheckermodel.h"
#include "screenid.h"
#include "buttonoperation.h"
#include "screentransitioncontroller.h"

/**
 * @brief エラー管理
 */
class ErrorManager : public QObject
{
    Q_OBJECT

    /**
     * @brief 故障情報
     */
    struct errorPopupInfo {
        uint16_t code;              //!< 故障コード
        uint8_t reaction;           //!< 故障時の動作
        uint16_t interval;          //!< メッセージ再表示間隔(s)
        QString addInfo;            //!< 追加情報
        uint16_t reminingTime;      //!< 再表示までの残り時間(s)

        errorPopupInfo(uint16_t paramCode = 0, uint8_t paramReaction = 0, uint16_t paramInterval = 0, QString paramAddInfo = "", uint16_t paramReminingTime = 0) {
            code = paramCode;
            reaction = paramReaction;
            interval = paramInterval;
            addInfo = paramAddInfo;
            reminingTime = paramReminingTime;
        }
    };

    /**
     * @brief ポップアップクローズ時の処理種別
     */
    enum class ProcessType : int {
        INVALID = 0,                //!< 無効値
        CLOSE_POPUP,                //!< ポップアップを閉じるだけ
        TRANSIT_HOME,               //!< ポップアップを閉じてからHOMEに遷移する
        NOTIFY_POWOER_SUPPLY_ERROR, //!< ポップアップを閉じてから給電異常を通知する
    };

    /**
     * @brief メッセージ情報
     */
    struct MessageInfo {
        QString message;            //!< メッセージ内容
        bool hasAddInfo;            //!< 追加情報有無
        ProcessType processType;    //!< クローズ時の処理
    };

public:
    /**
     * @brief コンストラクター
     * @param model Model クラスのポインター
     * @param parent 親オブジェクト
     */
    ErrorManager(LevelCheckerModel *&model, QObject *parent = nullptr);

    /**
     * @brief デストラクター
     */
    ~ErrorManager();

    /**
     * @brief エラー管理と画面遷移コントローラの接続
     * @param controller 画面遷移コントローラ
     */
    void attachController(ScreenTransitionController *controller);

    /**
     * @brief 給電異常発生状態確認
     * @return true: 発生中、false: 発生していない
     */
    bool isOccuredPowerSupplyError(void);

    /**
     * @brief コンパス有効状態取得
     * @return true：有効、false：無効
     */
    bool enabledCompass(void);

signals:
    /**
     * @brief View にエラーポップアップを表示するよう依頼
     * @param message ポップアップに表示するメッセージ
     * @param buttonCaption ボタンのキャプション
     */
    void requestToOpenErrorPopup(QVariant message, QVariant buttonCaption);

    /**
     * @brief ポップアップを閉じるよう通知
     */
    void requestToClose(void);

    /**
     * @brief ポップアップが閉じたことを通知
     */
    void closed(void);

    /**
     * @brief 給電異常を通知
     * @code 故障コード
     * @action 故障時の動作
     * @param message エラーメッセージ
     */
    void notifyPowerSupplyError(uint16_t code, uint8_t action, QString message);

    /**
     * @brief コンパス有効状態変化通知
     * @param enabled true：有効に変化、false：無効に変化
     */
    void notifyChangeCompassEnable(bool enabled);

    /**
     * @brief エラー状態変化通知
     */
    void notifyChangeErrorStatus(void);

    /**
     * @brief 画面遷移要求
     * @param nextScreenId 次に遷移する画面 ID
     * @param clearHistory 履歴削除
     * @note 次の遷移したい画面を直接指定する
     */
    void requestDirect(ScreenId nextScreenId, bool clearHistory);

public slots:
    /**
     * @brief 故障情報受信
     * @param failInformation 故障情報
     */
    void receiveFailInfo(LC70_FAIL_INFO_T failInformation);

private:
    LevelCheckerModel *_model;                  //!< Model クラスのポインター
    QObject *_errorPopup;                       //!< ErrorPopup.qml
    ButtonId _pressedButtonId;                  //!< 押下したボタンのID
    QTimer *_pressAndHoldTimer;                 //!< ボタン長押し検出タイマー
    uint _pressAndHoldTimerInterval;            //!< ボタン長押しタイマー値
    uint _pressCount;                           //!< 長押し中のpress発火回数
    QList<errorPopupInfo> _failList;            //!< 故障情報リスト
    uint16_t _displayFailCode;                  //!< ポップアップ表示中の故障コード
    ProcessType _displayProcessType;            //!< 表示中のポップアップの処理種別
    bool _isWaitReNotification;                 //!< 再表示待機中フラグ
    bool _isEnabledCompass;                     //!< コンパス有効フラグ
    ScreenTransitionController *_controller;    //!< 画面遷移コントローラー

    /**
     * @brief ポップアップを閉じる
     */
    void close(void);

    /**
     * @brief 新規故障判定
     * @param failInformation 故障情報
     * @return true:新規故障、false:新規故障ではない
     */
    bool isNewNotification(const LC70_FAIL_INFO_T failInformation);

    /**
     * @brief 発生中の故障であるか判定
     * @param failCode 故障コード
     * @return true:発生中の故障、false:未発生の故障
     */
    bool isOccurring(uint16_t failCode);

    /**
     * @brief ポップアップ表示
     * @param failCode 故障コード
     * @param action 故障時の動作
     *
     */
    void showPopup(const uint16_t failCode, uint8_t action);

    /**
     * @brief 追加情報取得
     * @param failCode 故障コード
     * @return 追加情報
     */
    QString getAddInfo(const uint16_t failCode);

    /**
     * @brief ボタンの操作
     * @param button ボタン ID
     * @param status 各ボタンの ON/OFF
     */
    void manipulateButton(const LC70_KEYID_T &button, const LC70_KEY_STAT_T status);

    /**
     * @brief ボタン入力の対応
     * @param buttonId ボタン ID
     */
    void inputButton(ButtonId buttonId);

    /**
     * @brief エンターキー処理
     */
    void enterKey();

    /**
     * @brief ESCAPEキー処理
     */
    void escapeKey(void);

    /**
     * @brief 長押しタイマー開始
     */
    void startPressAndHoldTimer(void);

    /**
     * @brief 長押しタイマー停止
     */
    void stopPressAndHoldTimer(void);

    /**
     * @brief 再表示タイマー開始
     * @param failCode 故障コード
     */
    void setReNotificationTimer(const uint16_t failCode);

    /**
     * @brief ボタン押下の対応
     * @param buttonId ボタン ID
     */
    void pressButton(ButtonId buttonId);

    /**
     * @brief プロパティ変更
     * @param buttonName   オブジェクト名
     * @param propertyName プロパティ名
     * @param value        プロパティ値
     */
    void propertyChange(QString buttonName, QString propertyName, QVariant value);

    /**
     * @brief ポップアップを閉じる
     */
    void closeAction();

    /**
     * @brief コンパス有効状態チェック
     */
    void checkEnableCompass(void);

    /**
     * @brief より優先度の高い故障であるか判定する
     * @param code 故障ID
     * @return true：優先度の高い故障、false：現在表示中の故障の方がが優先度が高い
     * @note 優先度が同じであれば先勝ち
     */
    bool isHighPriority(uint16_t code);

    /**
     * @brief 故障メッセージ表示後の復旧通知が必要かどうか
     * @param code 故障ID
     * @return 復旧通知の要否 true: 必要, false: 不要
     */
    bool isRecoveryProc(uint16_t code);

private slots:
    /**
     * @brief 押下ボタンの通知を受けた時に行う処理
     * @param keyStatus ボタンの押下状態
     */
    void onKeyStatus(LC70_KEY_STAT_T keyStatus);

    /**
     * @brief 長押し検出タイマータイムアップ
     */
    void detectPressAndHold(void);

    /**
     * @brief 再表示チェック
     */
    void checkReNotification(void);

    /**
     * @brief Rejectボタン押下通知
     */
    void onReject(void);

    /**
     * @brief エラー状態変化通知受信
     */
    void onNotifyChangeErrorStatus(void);

    /**
     * @brief エラーポップアップを閉じる通知を受信
     * @param force エラーリストのクリアー有無
     */
    void onRequestToCloseErrorPopup(bool clearErrorList = true);
};

#endif // ERRORMANAGER_H
