/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef POPUPMANAGER_H
#define POPUPMANAGER_H

#include <QObject>
#include <QVariant>
#include "popupid.h"
#include "levelcheckermodel.h"
#include "buttonoperation.h"
#include "scanresultmodel.h"
#include "scanlockmodel.h"

/**
 * @brief ポップアップの表示管理
 */
class PopupManager : public QObject
{
    Q_OBJECT
public:
    /**
     * @brief コンストラクター
     * @param model Model クラスのポインター
     * @param parent 親オブジェクト
     */
    explicit PopupManager(LevelCheckerModel *&model, QObject *parent = nullptr);

    /**
     *  @brief デストラクター
     */
    ~PopupManager();

    /**
     * @brief ポップアップを開く
     * @param popupId ポップアップ ID
     * @param focedDisplay 強制的に開くか否か
     * @return
     */
    bool open(PopupId popupId, bool focedDisplay);

    /**
     * @brief Scanターゲット設定
     * @param popupId ポップアップ ID
     * @param detailId  ID
     * @return
     */
    bool openScan(PopupId popupId, int detailId, bool focedDisplay);


    /**
     * @brief 指定した内容でポップアップを開く
     * @param popupId ポップアップ ID
     * @param message 表示内容
     * @param focedDisplay 強制的に開くか否か
     * @return
     */
    bool openWithMessage(PopupId popupId, QString message, bool focedDisplay);

    /**
     * @brief バッテリ低下状態ポップアップを開く
     * @return
     */
    bool openLowBattery();

    /**
     * @brief ポップアップを閉じる
     */
    void close(void);

    /**
     * @brief ポップアップが開いているか否かを返す
     * @return ポップアップが開いていれば true 閉じていれば false
     */
    bool isOpend(void) const;

signals:
    /**
     * @brief View にポップアップを表示するよう依頼
     * @param layoutType レイアウトタイプ
     * @param message ポップアップに表示するメッセージ
     * @param leftButtonMessage 左ボタンのメッセージ
     * @param rightButtonMessage 右ボタンのメッセージ
     * @param centerButtonMessage 中央ボタンのメッセージ
     */
    void requestToOpenPopup(QVariant layoutType, QVariant message, QVariant leftButtonMessage, QVariant rightButtonMessage, QVariant centerButtonMessage);

    /**
     * @brief View にポップアップの上に表示するポップアップを表示するよう依頼
     * @param message ポップアップに表示するメッセージ
     * @param leftButtonCaption 左ボタンのキャプション
     * @param rightButtonCaption 右ボタンのキャプション
     */
    void requestToOpenStrongPopup(QVariant message, QVariant leftButtonCaption, QVariant rightButtonCaption);

    /**
     * @brief View にバッテリ低下状態中のポップアップを表示するよう依頼
     */
    void requestToOpenLowBatteryPopup();

    /**
     * @brief View にエラーポップアップを表示するよう依頼
     * @param message ポップアップに表示するメッセージ
     * @param buttonCaption ボタンのキャプション
     */
    void requestToOpenErrorPopup(QVariant message, QVariant buttonCaption);

    /**
     * @brief ポップアップからの受諾を通知
     * @param popupId ポップアップの種類
     * @param okOnly OK のみの場合 true
     */
    void accept(PopupId popupId, bool okOnly);

    /**
     * @brief ポップアップからの拒否を通知
     * @param popupId ポップアップの種類
     */
    void reject(PopupId popupId);

    /**
     * @brief スキャン結果ポップアップからの拒否を通知
     */
    void rejectScanResult();

    /**
     * @brief ポップアップからの選択を通知
     * @param popupId ポップアップの種類
     * @param index 選択されたインデックス
     */
    void select(PopupId popupId, int index);

    /**
     * @brief ポップアップを閉じるよう通知
     */
    void requestToClose(void);

    /**
     * @brief ストロングポップアップを閉じるよう通知
     */
    void requestToCloseStrong(void);

    /**
     * @brief バッテリ低下状態ポップアップを閉じるよう通知
     */
    void requestToCloseLowBattery(void);

    /**
     * @brief ポップアップが閉じたことを通知
     */
    void closed(void);

    /**
     * @brief 入力されたファイル名を通知
     * @param popupId ポップアップの種類
     * @param saveLocation 保存先
     * @param fileName 入力されたファイル名
     */
    void inputFileName(PopupId popupId, SaveLocation saveLocation, QString fileName);

    /**
     * @brief 強制的に TextInput の active focus を設定するよう通知
     */
    void notifyActiveFocus(void);

    /**
     * @brief スキャンの実行を依頼
     * @note 呼び出し画面でスキャン処理(開始、結果取得、中止)を実施
     */
    void requestToScan(void);

    /**
     * @brief スキャン中止を依頼
     */
    void requestToStopScanning(void);

    /**
     * @brief ポップアップ表示中に入力されたボタンのボタン ID を通知
     * @param popupId ポップアップ ID
     * @param buttonId ボタン ID
     * @ボタン入力が画面操作を伴う場合にポップアップの呼び出し元の画面に通知するために使う
     */
    void notifyButtonInput(PopupId popupId, ButtonId buttonId);

    /**
     * @brief 上スクロールを依頼
     */
    void requestScrollUp(void);

    /**
     * @brief 下スクロールを依頼
     */
    void requestScrollDown(void);

    /**
     * @brief スキャン結果の行数を通知
     * @param rowCount 行数
     */
    void setScanResultRowCount(QVariant rowCount);

    /**
     * @brief 測定データ取得中止を依頼
     */
    void requestToStopAcquring(void);

private:
    LevelCheckerModel *_model;                  //!< Model クラスのポインター
    QObject *_basicPopup;                       //!< BasicPopup.qml
    QObject *_strongPopup;                      //!< StrongPopup.qml
    bool _popupAlereadyOpened;                  //!< ポップアップが既に表示されている
    bool _strongPopupAlreadyOpened;             //!< ポップアップの上に表示するポップアップが既に表示されている
    bool _isOpened;                             //!< ポップアップが開いているか否か
    ButtonOperation *_buttonOperation;          //!< ボタン操作クラス
    QString _activeButton;                      //!< アクティブなボタン名
    QVector<QVector<QString>> _threeButtonData; //!< threeButtonPopup のボタン ID
    QVector<QVector<QString>> _twoButtonData;   //!< twoButtonPopup のボタン ID
    QVector<QVector<QString>> _oneButtonData;   //!< oneButtonPopup のボタン ID
    PopupId _openPopupId;
    LC70_BAND_T _scanBandId;                    //! < スキャンバンド ID
    QList<LC70_BAND_T> _scanList;               //! < スキャンリスト
    QVector<QVector<QString>> _keyboardButtonData;
    QVector<QVector<QString>> _keyboardPattrnAlphanumericUpperLower;
    QObject *_fileNameInput;                    //!< ファイル名入力エレメント
    QString _fileName;                          //!< ファイル名
    SaveLocation _saveLocation;                 //!< 保存先
    ButtonId _pressedButtonId;                  //!< 押下したボタンのID
    QTimer *_pressAndHoldTimer;                 //!< ボタン長押し検出タイマー
    uint _pressAndHoldTimerInterval;            //!< ボタン長押しタイマー値
    uint _pressCount;                           //!< 長押し中のpress発火回数
    ScanLockModel *_scanLockModel;          //!< スキャン結果モデル
    ScanResultModel *_scanResultModel;          //!< スキャン結果モデル
    QMap<LC70_BAND_T, QList<ChannelDefinitionItem>> _catvChannelDefMap; //!< CATVチャンネル定義Map
    bool _noValidScanResult;                    //!< 有効なスキャン結果の有無
    QObject *_lowBatteryPopup;                  //!< LowBatteryPopup.qml
    bool _isRejectScanResult;                      //!< スキャン結果でESCボタンを押下された

    // [memo] BasicPopup.qmlの定義値と連動しています。
    //        更新時は注意！！！！！！
    /**
     * @brief ポップアップのレイアウトの種類
     */
    enum PopupLayoutType {
        NORMAL_LAYOUT    = 1,               //!< メッセージ, ボタン2つのレイアウト
        ONE_BUTTON_LAYOUT,                  //!< メッセージ, ボタン1つのレイアウト
        PROGRESSBAR_LAYOUT,                 //!< メッセージ, プログレスバー, ボタン1つのレイアウト
        KEYBORAD_LAYOUT,                    //!< キーボードのレイアウト
        MESSAGE_LAYOUT,                     //!< メッセージのみ、ボタンなしのレイアウト
        POWER_SUPPLY_SELECT_LAYOUT,         //!< 給電選択用のレイアウト
        SCAN_RESULT_LAYOUT,                 //!< スキャン結果レイアウト
        THREE_BUTTON_LAYOUT,                //!< メッセージ, ボタン3つのレイアウト
        PROGRESSBAR_LABELBUTTON_LAYOUT,     //!< メッセージ, プログレスバー, ボタン1つのレイアウト
        CONFIRM_UPDATE_LAYOUT,              //!< アップデート確認のレイアウト
        UPDATE_LAYOUT,                      //!< アップデートのレイアウト
    };

    enum KEYBORAD_KIND {
        KEYBORAD_ALPHANUMERIC_UPPER_LOWER,  //!< 半角英数大文字小文字
    };
    KEYBORAD_KIND   _keyboardPattern;

    /**
     * @brief ボタン押下の対応
     * @param buttonId ボタン ID
     */
    void pressButton(ButtonId buttonId);

    /**
     * @brief ボタン入力の対応
     * @param buttonId ボタン ID
     */
    void inputButton(ButtonId buttonId);

    /**
     * @brief エンターキー処理
     */
    void enterKey();

    /**
     * @brief ESCAPEキー処理
     */
    void escapeKey(void);

    /**
     * @brief プロパティ変更
     * @param buttonName   オブジェクト名
     * @param propertyName プロパティ名
     * @param value        プロパティ値
     */
    void propertyChange(QString buttonName, QString propertyName, QVariant value);

    /**
     * @brief BasicPopupのconnect
     */
    void connectBasicPopup();
    /**
     * @brief BasicPopupのdisconnect
     */
    void disconnectBasicPopup();
    /**
     * @brief ダイアログ 3 ボタン時の_buttonOperation 設定
     * @param isRightButtonActive 右ボタンをアクティブにするかどうか
     */
    void setThreeButton(bool isRightButtonActive = false);
    /**
     * @brief ダイアログ２ボタン時の_buttonOperation 設定
     * @param isRightButtonActive 右ボタンをアクティブにするかどうか
     *        false : 左ボタンをアクティブ, true: 右ボタンをアクティブ
     * @param visible 可視/不可視
     */
    void setTwoButton(bool isRightButtonActive = false, bool visible = true);
    /**
     * @brief ダイアログ１ボタン時の_buttonOperation 設定
     */
    void setOneButton();
    /**
     * @brief
     */
    void setKeyboardInit();

    /**
     * @brief ファイル名の入力
     */
    void setFileName(void);

    /**
     * @brief ボタンの操作
     * @param button ボタン ID
     * @param status 各ボタンの ON/OFF
     */
    void manipulateButton(const LC70_KEYID_T &button, const LC70_KEY_STAT_T status);

    /**
     * @brief 長押しタイマー開始
     */
    void startPressAndHoldTimer(void);

    /**
     * @brief 長押しタイマー停止
     */
    void stopPressAndHoldTimer(void);

    /**
     * @brief ポップアップ表示イベントの発火
     * @param layout レイアウト種別
     * @param message 表示メッセージ
     * @param leftCaption 左ボタンテキスト
     * @param rightCaption 右ボタンテキスト
     * @param centerCaption 中央ボタンテキスト
     */
    void emitRequestToOpenPopup(PopupLayoutType layout, const QString message, const QString leftCaption = "", const QString rightCaption = "", const QString centerCaption = "");

    /**
     * @brief ポップアップ表示へのイベント接続
     * @param isSelect onSelectを使うかどうか（使わない場合はonAccept）
     */
    void createPopupConnection(const bool isSelect = false);

    /**
     * @brief 強ポップアップ表示イベントの発火
     * @param message 表示メッセージ
     * @param leftCaption 左ボタンテキスト
     * @param rightCaption 右ボタンテキスト
     */
    void emitRequestToOpenStrongPopup(const QString message, const QString leftCaption = "", const QString rightCaption = "");

    /**
     * @brief 強ポップアップ表示へのイベント接続
     */
    void createStrongPopupConnection();

    /**
     * @brief バッテリ低下状態ポップアップ表示イベントの発火
     */
    void emitRequestToOpenLowBatteryPopup();

    /**
     * @brief バッテリ低下状態ポップアップ表示へのイベント接続
     */
    void createLowBatteryPopupConnection();

    /**
     * @brief ボタン１つのポップアップを設定する
     * @param message 表示メッセージ
     * @param buttonCaption ボタンテキスト
     */
    void setupOneButtonPopup(const QString message, const QString buttonCaption);

    /**
     * @brief はい/いいえボタンのポップアップを設定する
     * @param message 表示メッセージ
     */
    void setupYesNoButtonPopup(const QString message, const bool isDefaultNo = false);

    /**
     * @brief キーボードダイアログの保存ボタンの有効可否チェック
     */
    void checkValidKeyboardSaveButton(void);

private slots:
    /**
     * @brief 押下ボタンの通知を受けた時に行う処理
     * @param keyStatus ボタンの押下状態
     */
    void onKeyStatus(LC70_KEY_STAT_T keyStatus);

    /**
     * @brief ポップアップから受諾の通知を受信
     * @param okOnly OK のみの場合 true
     * @note 通常のポップアップのみ対象
     */
    void onAccept(bool okOnly);

    /**
     * @brief ポップアップから拒否の通知を受信
     * @note 通常のポップアップのみ対象
     */
    void onReject(void);

    /**
     * @brief ポップアップから選択の通知を受信
     * @param index 選択されたインデックス
     * @note 通常のポップアップのみ対象
     */
    void onSelect(int index);

    /**
     * @brief プログレスバーの進捗の値を通知
     * @param value 進捗の値
     */
    void onProgress(double value);

    /**
     * @brief プログレステキストを通知
     * @param current 現在、ロックされたチャンネルまたはスキャン番号
     * @param flag ロックの有無
     */
    void onProgressText(QString current, bool flag);

    /**
     * @brief スキャン終了を受信
     * @param complete 完了有無
     */
    void onScanningFinished(bool complete);

    /**
     * @brief 長押し検出タイマータイムアップ
     */
    void detectPressAndHold(void);

    /**
     * @brief スキャン結果データ作成
     */
    void createScanResultModel(void);

    /**
     * @brief スキャン結果データを行末まで補填
     * @param resultData スキャン結果データ
     * @param groupData 1グループのデータ
     * @param elementCount 要素数
     * @param rowCount 行数
     * @return 補填したデータ数
     */
    int padingScanResultData(ScanResultData &resultData, ScanResultGroupData &groupData, const int elementCount, const int rowCount);

    /**
     * @brief 行色の取得
     * @param rowCount 行数
     * @return 色の種類
     */
    RowColorType getRowColorType(const int rowCount);

    /**
     * @brief スキャン結果行数の取得
     * @param elementCount 要素数
     * @return 行数
     */
    int getScanResultRowCount(const int elementCount);

    /**
     * @brief スキャン結果グループタイトル設定
     * @param resultData スキャン結果データ
     * @param title グループタイトル
     * @param rowCount 行数
     * @param elementCount 要素数
     */
    void setScanResultGroupTitle(ScanResultData &resultData, const QString &title, const int rowCount, int &elementCount);

    /**
     * @brief スキャングループ名を取得
     * @param band Band の ID
     * @param channel チャンネル番号
     * @return グループ名
     */
    QString getScanGroupName(int band, QString channel);

    /**
     * @brief 測定データ取得終了を受信
     * @param complete 完了有無
     */
    void onAcquiringFinished(bool complete);

    /**
     * @brief バッテリ低下状態変化通知
     * @param isLowBattery バッテリ低下状態かどうか
     */
    void onLowBatteryStatusChanged(bool isLowBattery);
};

#endif // POPUPMANAGER_H
