/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include "screentransitioncontroller.h"

#include <QDebug>

ScreenTransitionController::ScreenTransitionController(QObject *parent)
    : QObject(parent)
    , _screenInformations(new ScreenInformations(this))
    , _history(QList<ScreenId>())
    , _buttonProhibition(new ButtonProhibition(this))
    , _historyIndex(0)
    , _attributes(nullptr)
{

}

ScreenTransitionController::~ScreenTransitionController()
{

}

void ScreenTransitionController::prepare(Attributes *&attributes)
{
    _attributes = attributes;
}

void ScreenTransitionController::onRequest(ScreenId currentScreenId, int condition)
{
    ScreenInformation currentInformation = _screenInformations->screen(currentScreenId);

    if (currentInformation.next.length() == 1) {
        // 次の遷移先が一画面しかない場合
        ScreenInformation nextInformation = _screenInformations->screen(currentInformation.next[0]);

        // 現在の画面を履歴に追加
        if (nextInformation.id != SCREEN_MAINFRAME) {
            _history.append(nextInformation.id);
            _historyIndex = _history.count() - 1;
        }

        ButtonProhibitionData buttonProhibitionData = _buttonProhibition->buttonProhibition(nextInformation.prohibitionId);
        emit transit(nextInformation.id, nextInformation.screenName, nextInformation.objectName, nextInformation.qml,
                     buttonProhibitionData);

        _screenId = nextInformation.id;
    } else {
        // 次の遷移先が複数ある場合

        // 測定画面の選択
        if (currentScreenId == SCREEN_LOCAL_OSCILLATOR_SETTING
                || currentScreenId == SCREEN_MODULATION)  {
            selectMeasurementScreen(currentScreenId,currentInformation);
            return;
        }

        // ボタンのインデックスが遷移先を決める画面の場合
        // Home 画面などパラメーター condition が次の遷移先を示すインデックスになっている(遷移元の画面の画面遷移ボタンと同期させる)
        ScreenInformation nextInformation = _screenInformations->screen(currentInformation.next[condition]);

        // 現在の画面を履歴に追加
        _history.append(nextInformation.id);
        _historyIndex = _history.count() - 1;

        ButtonProhibitionData buttonProhibitionData = _buttonProhibition->buttonProhibition(nextInformation.prohibitionId);
        emit transit(nextInformation.id, nextInformation.screenName, nextInformation.objectName, nextInformation.qml,
                     buttonProhibitionData);
        _screenId = nextInformation.id;
    }
}

void ScreenTransitionController::onRequestDirect(ScreenId nextScreenId, bool clearHistory)
{
    ScreenInformation nextInformation = _screenInformations->screen(nextScreenId);

    if (clearHistory) {
        //  履歴削除フラグがtrueの場合
        //  現在積まれている履歴を削除し、ホームのみの履歴を追加
        _history.clear();
        _history.append(SCREEN_HOME);
        _historyIndex = 0;
    }

    // 画面遷移履歴を追加
    if (_history.last() != nextScreenId) {
        _history.append(nextScreenId);
        ++_historyIndex;
    }

    ButtonProhibitionData buttonProhibitionData = _buttonProhibition->buttonProhibition(nextInformation.prohibitionId);
    emit transit(nextInformation.id, nextInformation.screenName, nextInformation.objectName, nextInformation.qml,
                 buttonProhibitionData);
    _screenId = nextInformation.id;
}

void ScreenTransitionController::onRequestInHistory(ScreenId nextScreenId)
{
    int index = -1;

    // 履歴の中から画面 ID と合致するインデックスを特定
    for (int i = _history.count() - 1; i >= 0; --i) {
        if (_history[i] != nextScreenId) {
            continue;
        }

        index = i;
        break;
    }

    // 見つからなければ何もしない
    if (index < 0) {
        return;
    }

    // 履歴の編集
    for (int i = _history.count() - 1; i > index; --i) {
        _history.pop_back();
        --_historyIndex;
    }

    ScreenId screenId = _history[_historyIndex];

    ScreenInformation nextInformation = _screenInformations->screen(screenId);

    ButtonProhibitionData buttonProhibitionData = _buttonProhibition->buttonProhibition(nextInformation.prohibitionId);
    emit transit(nextInformation.id, nextInformation.screenName, nextInformation.objectName, nextInformation.qml,
                 buttonProhibitionData);
    _screenId = nextInformation.id;
}

void ScreenTransitionController::onRequestWithChangeHistory(ScreenId nextScreenId, QList<ScreenId> history,
        bool requestTransit)
{
    _history.clear();

    // 画面遷移履歴の変更
    foreach (ScreenId screenId, history) {
        _history.append(screenId);
    }

    // 戻る場合のため積んでおく(本来は変調方式や偏波、局初周波数が積まれているが、測定画面から戻るときは BscreenIdand 画面になるので、仮の画面として測定画面自体を積んでおく)
    _history.append(nextScreenId);
    _historyIndex = _history.count() - 1;

    //  ユーザ/データロガー設定の測定画面からメニューバーを経由し、
    //  同一の測定画面に遷移する場合以外は、Trueとなる前提
    // requestTransitの値がTrueの場合のみ画面遷移を行う
    if (requestTransit) {
        ScreenInformation nextInformation = _screenInformations->screen(nextScreenId);

        ButtonProhibitionData buttonProhibitionData = _buttonProhibition->buttonProhibition(nextInformation.prohibitionId);
        emit transit(nextInformation.id, nextInformation.screenName, nextInformation.objectName, nextInformation.qml,
                     buttonProhibitionData);
        _screenId = nextInformation.id;
    }
}

void ScreenTransitionController::onBack()
{
    if (_historyIndex - 1 < 0) {
        return;
    }

    // 現在の画面を履歴から消去し、一つ前の画面 ID を取得する
    _history.pop_back();
    --_historyIndex;

    ScreenId screenId = _history[_historyIndex];

    ScreenInformation nextInformation = _screenInformations->screen(screenId);

    ButtonProhibitionData buttonProhibitionData = _buttonProhibition->buttonProhibition(nextInformation.prohibitionId);
    emit transit(nextInformation.id, nextInformation.screenName, nextInformation.objectName, nextInformation.qml,
                 buttonProhibitionData);
    _screenId = nextInformation.id;
}

void ScreenTransitionController::selectMeasurementScreen(const ScreenId screenId, const ScreenInformation &currentInformation)
{
    int targetScreen = -1;

    // 履歴からシングル/マルチ測定を探す
    // シングル測定, マルチ測定,　その他
    QVariantMap map = _attributes->takeOver();
    QString selectedMeasurement = map["selectedMeasurement"].toString();

    if(screenId == SCREEN_LOCAL_OSCILLATOR_SETTING) {
        // 局発周波数選択画面
        if (selectedMeasurement == "SingleMeasurement") {
            // シングル測定を選択している場合はシングル測定に遷移
            targetScreen = 0;
        } else if (selectedMeasurement == "MultiMeasurement") {
            // マルチ測定を選択している場合はマルチ測定に遷移
            targetScreen = 1;
        } else if (selectedMeasurement == "AntennaAdjustment") {
            // Band でアンテナ調整を選択している場合はアンテナ調整に遷移
            targetScreen = 2;
        } else {
            // 未設定の場合
            // ファイル選択画面に遷移
            targetScreen = 3;
        }
    } else if (screenId == SCREEN_MODULATION) {
        // ScreenInformations クラスの次の画面リストに定義したインデックスを指定する(シングル測定、マルチ測定の順に定義しておく)
        if (selectedMeasurement == "SingleMeasurement") {
            // シングル測定を選択している場合はシングル測定に遷移
            targetScreen = 0;
        } else if (selectedMeasurement == "MultiMeasurement") {
            // マルチ測定を選択している場合はマルチ測定に遷移
            targetScreen = 1;
        } else {
            // 未設定なら閾値選択
            targetScreen = 2;
        }
    } else {
        // nothing to do
    }

    // 次の画面が見つからない
    if (targetScreen < 0) {
        qDebug() << QString("Next screen is not found.");
        return;
    }

    // 画面遷移履歴に追加
    if (_history.last() != currentInformation.next[targetScreen]) {
        _history.append(currentInformation.next[targetScreen]);
        ++_historyIndex;
    }

    // 次の画面の情報
    ScreenInformation nextInformation = _screenInformations->screen(currentInformation.next[targetScreen]);
    ButtonProhibitionData buttonProhibitionData = _buttonProhibition->buttonProhibition(nextInformation.prohibitionId);
    emit transit(nextInformation.id, nextInformation.screenName, nextInformation.objectName, nextInformation.qml,
                 buttonProhibitionData);
    _screenId = nextInformation.id;
}

ScreenId ScreenTransitionController::currentScreenId(void)
{
    return _screenId;
}
