/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef SCREENTRANSITIONCONTROLLER_H
#define SCREENTRANSITIONCONTROLLER_H

#include <QObject>
#include "screeninformations.h"
#include "buttonprohibition.h"
#include "attributes.h"

/**
 * @brief 画面遷移を管理するクラス
 */
class ScreenTransitionController : public QObject
{
    Q_OBJECT
public:
    /**
     * @brief コンストラクタ
     * @param parent 親オブジェクト
     */
    explicit ScreenTransitionController(QObject *parent = nullptr);

    /**
     * @brief デストラクタ
     */
    ~ScreenTransitionController();

    /**
     * @brief View - ViewModel 間の連携データを設定する
     * @param attributes View - ViewModel 間の連携データ
     */
    void prepare(Attributes *&attributes);

    /**
     * @brief 現在の画面IDを取得
     * @return 画面ID
     */
    ScreenId currentScreenId(void);

signals:
    /**
     * @brief 画面遷移実行通知
     * @param nextId 次の画面 ID
     * @param screenName 画面名
     * @param objectName QML のオブジェクト名
     * @param qmlFileName QML ファイル名
     * @param buttonProhibitionData ボタン禁則情報
     */
    void transit(ScreenId nextId, QString screenName, QString objectName, QString qmlFileName,
                 ButtonProhibitionData buttonProhibitionData);

private slots:
    /**
     * @brief 画面遷移要求の受信
     * @param currentScreenId 現在表示している画面の画面 ID
     * @param condition 遷移先を特定する情報(ボタンのインデックスなど)
     */
    void onRequest(ScreenId currentScreenId, int condition);

    /**
     * @brief 画面遷移要求の受信(直接指定)
     * @param clearHistory 履歴削除
     * @param currentScreenId 遷移したい画面 ID
     */
    void onRequestDirect(ScreenId nextScreenId, bool clearHistory);

    /**
     * @brief 画面遷移要求の受信(履歴にある画面を指定)
     * @param nextScreenId 遷移したい画面 ID
     * @note 履歴になければ遷移しない
     */
    void onRequestInHistory(ScreenId nextScreenId);

    /**
     * @brief 画面遷移要求の受信(履歴の書き換え)
     * @param nextScreenId   次に遷移する画面 ID
     * @param hisotory       シングル測定からマルチ測定あるいはその逆の場合のみ使用する
     * @param requestTransit ユーザ/データロガーの測定画面からホームから遷移してきた場合の同一測定画面に遷移する際に使用
     *                       falseの場合は、履歴のみを変更し、画面遷移の実施は行わない
     */
    void onRequestWithChangeHistory(ScreenId nextScreenId, QList<ScreenId> history, bool requestTransit = true);

    /**
     * @brief 一つ前の画面に戻る要求の受信
     */
    void onBack(void);

private:
    ScreenInformations *_screenInformations;    //!< 全画面情報
    QList<ScreenId> _history;                   //!< 画面遷移履歴
    ButtonProhibition *_buttonProhibition;      //!< ボタン禁則情報
    int32_t _historyIndex;                      //!< 現在表示している画面の階層(0 は Home 画面)
    ScreenId _screenId;                         //!< 現在の画面 ID
    Attributes *_attributes;                    //!< View - ViewModel 間の連携データ

    /**
     * @brief シングル/マルチ測定画面への遷移を選択
     * @param screenId スクリーンID
     * @param currentInformation 現在の画面の情報
     */
    void selectMeasurementScreen(const ScreenId screenId,const ScreenInformation &currentInformation);
};

#endif // SCREENTRANSITIONCONTROLLER_H
