/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include "viewmodelfactory.h"
#include "mainframeviewmodel.h"
#include "viewmodel/home/homeviewmodel.h"
#include "viewmodel/measurement/bandviewmodel.h"
#include "viewmodel/measurement/polarizationviewmodel.h"
#include "viewmodel/measurement/localoscillatorsettingviewmodel.h"
#include "viewmodel/measurement/modulationviewmodel.h"
#include "viewmodel/measurement/multi/multimeasurementviewmodel.h"
#include "viewmodel/measurement/single/singlemeasurementviewmodel.h"
#include "viewmodel/other/dcmeasurementviewmodel.h"
#include "viewmodel/optical_setting/wavelengthselectviewmodel.h"
#include "viewmodel/optical_setting/opticalmeasurementviewmodel.h"
#include "viewmodel/system_setting/systemsettingmenuviewmodel.h"
#include "viewmodel/system_setting/autopoweroffsettingviewmodel.h"
#include "viewmodel/system_setting/datetimesettingviewmodel.h"
#include "viewmodel/system_setting/volumesettingviewmodel.h"
#include "viewmodel/system_setting/initializationviewmodel.h"
#include "viewmodel/system_setting/brightnesssettingviewmodel.h"
#include "viewmodel/measurement_setting/catvchannelfrequencyviewmodel.h"
#include "viewmodel/measurement_setting/measurementsettingmenuviewmodel.h"
#include "viewmodel/measurement_setting/switchmercnviewmodel.h"
#include "viewmodel/measurement_setting/switchopticalcoaxialcableviewmodel.h"
#include "viewmodel/measurement_setting/thresholdviewmodel.h"
#include "viewmodel/measurement_setting/thresholdsettingviewmodel.h"
#include "viewmodel/measurement_setting/constellationsettingviewmodel.h"
#include "viewmodel/measurement_setting/leveloffsetviewmodel.h"
#include "viewmodel/measurement_setting/localoscillatorviewmodel.h"
#include "viewmodel/user/userdataloggermenuviewmodel.h"
#include "viewmodel/user/fileselectionviewmodel.h"
#include "viewmodel/user/manualautoselectionviewmodel.h"
#include "viewmodel/data_logger/dataloggermeasurementlistviewmodel.h"
#include "viewmodel/measurement_setting/broadcastsettingviewmodel.h"
#include "viewmodel/data_management/datamanagementmenuviewmodel.h"
#include "viewmodel/data_management/measurementdatamenuviewmodel.h"
#include "viewmodel/data_management/dataloggermenuviewmodel.h"
#include "viewmodel/data_management/usermeasurementsettingmenuviewmodel.h"
#include "viewmodel/data_management/measurementresultdatalistviewmodel.h"
#include "viewmodel/data_management/dataloggerdatalistviewmodel.h"
#include "viewmodel/data_management/measurementresultpngdataviewmodel.h"
#include "viewmodel/data_management/measurementresultcsvdataviewmodel.h"
#include "viewmodel/measurement/antennaadjustment.h"
#include "viewmodel/data_management/dataloggerdetailsdataviewmodel.h"
#include "viewmodel/measurement/catvviewmodel.h"
#include "viewmodel/system_setting/modelinformationviewmodel.h"
#include "viewmodel/data_management/measurementresultcopyviewmodel.h"
#include "viewmodel/data_management/measurementresultdeletionviewmodel.h"
#include "viewmodel/data_management/measurementresultcopydeleteviewmodel.h"
#include "viewmodel/other/compassviewmodel.h"
#include "viewmodel/other/otherviewmodel.h"
#include "viewmodel/system_setting/batteryinformationviewmodel.h"
#include "viewmodel/system_setting/manualviewmodel.h"
#ifdef NEW_USER_SETTING
#include "viewmodel/user/usersettingviewmodel.h"
#include "viewmodel/user/userdataloggersettingviewmodel.h"
#include "viewmodel/user/usermeasurementselectionviewmodel.h"
#include "viewmodel/data_logger/dataloggersettingviewmodel.h"

#endif

static ViewModelFactory *instance_ = nullptr;

ViewModelFactory &ViewModelFactory::instance(QObject *parent)
{
    if (instance_ == nullptr) {
        instance_ = new ViewModelFactory(parent);
    }

    return *instance_;
}

ViewModelFactory::~ViewModelFactory()
{
    foreach (AbstractViewModel *viewModel, _viewModels) {
        if (viewModel == nullptr) {
            continue;
        }

        delete viewModel;
    }

    _viewModels.clear();
}

AbstractViewModel *ViewModelFactory::specify(const ScreenId &screenId)
{
    // 各 ViewModel のインスタンスは事前に生成しておく
    if (SCREEN_MAINFRAME <= screenId && screenId < SCREEN_ID_COUNT) {
        return _viewModels[screenId];
    } else {
        return nullptr;
    }
}

void ViewModelFactory::prepare(QObject *parent, LevelCheckerModel *model)
{
    // 親オブジェクトあるいは Model が空か、既に ViewModel を生成済みなら何もしない
    if (parent == nullptr || model == nullptr || _viewModels.count() > 0) {
        return;
    }

    this->setParent(parent);
    _model = model;

    // enum 値の順に追加することで ScreenId とインデックスを同期させる
    _viewModels.append(new MainFrameViewModel(SCREEN_MAINFRAME, _model, this)); //0
    _viewModels.append(new HomeViewModel(SCREEN_HOME, _model, this)); //1
    _viewModels.append(new BandViewModel(SCREEN_BAND, _model, this)); //2
    // 他の画面の ViewModel を追加する
    _viewModels.append(new LocalOscillatorSettingViewModel(SCREEN_LOCAL_OSCILLATOR_SETTING, _model, this)); //3
    _viewModels.append(new ModulationViewModel(SCREEN_MODULATION, _model, this)); //4
    _viewModels.append(nullptr); //5
    _viewModels.append(new UserDataLoggerMenuViewModel(SCREEN_USER_DATA_LOGGER_MENU, _model, this)); //6
    _viewModels.append(nullptr); //7
    _viewModels.append(nullptr); //8
#ifdef NEW_USER_SETTING
    _viewModels.append(new UserSettingViewModel(SCREEN_USER_SETTING, _model, this)); //9
#else
    _viewModels.append(nullptr); //9
#endif
    _viewModels.append(new ManualAutoSelectionViewModel(SCREEN_MANUAL_AUTO_SELECTION, _model, this)); //10
    _viewModels.append(nullptr); //11
    _viewModels.append(nullptr); //12
    _viewModels.append(nullptr); //13
    _viewModels.append(new FileSelectionViewModel(SCREEN_FILE_SELECTION, _model, this)); //14
    _viewModels.append(new SingleMeasurementViewModel(SCREEN_SINGLE_MEASUREMENT, _model, this)); //15
    _viewModels.append(nullptr); //16(不使用)
    _viewModels.append(nullptr); //17(不使用)
    _viewModels.append(new MultiMeasurementViewModel(SCREEN_MULTI_MEASUREMENT, _model, this)); //18
    _viewModels.append(nullptr); //19(不使用)
    _viewModels.append(nullptr); //20
    _viewModels.append(nullptr); //21
    _viewModels.append(nullptr); //22
    _viewModels.append(nullptr); //23
    _viewModels.append(nullptr); //24
    _viewModels.append(new DCMeasurementViewModel(SCREEN_DC_MEASUREMENT, _model, this)); //25
    _viewModels.append(new WaveLengthSelectViewModel(SCREEN_WAVELENGTH_SELECT, _model, this)); //26
    _viewModels.append(new OpticalMeasurementViewModel(SCREEN_OPTICAL_MEASUREMENT, _model, this)); //27
    // システム設定
    _viewModels.append(new SystemSettingMenuViewModel(SCREEN_SYSTEM_SETTING_MENU, _model, this)); //28
    _viewModels.append(new AutoPowerOffSettingViewModel(SCREEN_AUTO_POWER_OFF_SETTING, _model, this)); //29
    _viewModels.append(new DateTimeSettingViewModel(SCREEN_DATE_TIME_SETTING, _model, this)); //30
    _viewModels.append(new VolumeSettingViewModel(SCREEN_VOLUME_SETTING, _model, this)); //31
    _viewModels.append(new InitializationViewModel(SCREEN_INITIALIZATION, _model, this)); //32
    // 測定設定
    _viewModels.append(new MeasurementSettingMenuViewModel(SCREEN_MEASUREMENT_SETTING_MENU, _model, this)); //33
    _viewModels.append(new LevelOffsetViewModel(SCREEN_CABLE_OFFSET, _model, this)); //34
    _viewModels.append(new LocalOscillatorViewModel(SCREEN_LOCAL_OSCILLATOR, _model, this)); //35
    _viewModels.append(new ThresholdViewModel(SCREEN_THRESHOLD, _model, this)); //36
    _viewModels.append(new ThresholdSettingViewModel(SCREEN_THRESHOLD_SETTING, _model, this)); //37
    _viewModels.append(new SwitchMERCNViewModel(SCREEN_SWITCH_MER_CN, _model, this)); //38
    _viewModels.append(new CATVChannelFrequencyViewModel(SCREEN_CATV_CHANNEL_FREQUENCY, _model, this)); //39
    _viewModels.append(new ConstellationSettingViewModel(SCREEN_CONSTELLATION_SETTING, _model, this)); //40
    _viewModels.append(new SwitchOpticalCoaxialCableViewModel(SCREEN_SWITCH_OPTICAL_COAXIAL_CABLE, _model, this)); //41

    _viewModels.append(nullptr); //42
    _viewModels.append(nullptr); //43
    _viewModels.append(nullptr); //44
    _viewModels.append(new DataLoggerMeasurementListViewModel(SCREEN_DATALOGGER_MEASUREMENT_LIST, _model, this)); //45
    _viewModels.append(new DataManagementMenuViewModel(SCREEN_DATA_MANAGEMENT_MENU, _model, this)); //46
    _viewModels.append(new MeasurementDataMenuViewModel(SCREEN_MEASUREMENT_DATA_MENU, _model, this)); //47
    _viewModels.append(nullptr); //48
    _viewModels.append(nullptr); //49
    _viewModels.append(nullptr); //50
    _viewModels.append(nullptr); //51
    _viewModels.append(new DataLoggerMenuViewModel(SCREEN_DATA_LOGGER_MENU, _model, this)); //52
    _viewModels.append(nullptr); //53
    _viewModels.append(new PolarizationViewModel(SCREEN_POLARIZATION, _model, this)); //54
    _viewModels.append(nullptr); //55
    _viewModels.append(new brightnessSettingViewModel(SCREEN_BRIGHTNESS_SETTING, _model, this)); //56
    _viewModels.append(new BroadcastSettingViewModel(SCREEN_BROADCAST_SETTING, _model, this)); //57
    _viewModels.append(new UserMeasurementSettingMenuViewModel(SCREEN_USER_MEASUREMENT_SETTING_MENU, _model, this)); //58
    _viewModels.append(new MeasurementResultDataListViewModel(SCREEN_MEASUREMENT_RESURT_DATA_LIST, _model, this)); //59
    _viewModels.append(new DataloggerDataListViewModel(SCREEN_DATA_LOGGER_LIST, _model, this)); //60
    _viewModels.append(new MeasurementResultPngDataViewModel(SCREEN_MEASUREMENT_RESURT_DATA_PNG, _model, this)); //61
    _viewModels.append(new MeasurementResultCsvDataViewModel(SCREEN_MEASUREMENT_RESURT_DATA_CSV, _model, this)); //62
    _viewModels.append(new DataLoggerDetailsDataViewModel(SCREEN_DATA_LOGGER_DETAILS_DATA, _model, this)); //63
    _viewModels.append(new AntennaAdjustment(SCREEN_ANTENNA_ADJUSTMENT, _model, this)); //64
    _viewModels.append(new CATVViewModel(SCREEN_CATV_MENU, _model, this)); //65
    _viewModels.append(new ModelInformationViewModel(SCREEN_MODEL_INFORMATION, _model, this)); //66
    _viewModels.append(new MeasurementResultCopyViewModel(SCREEN_MEASUREMENT_RESULT_DATA_COPY, _model, this)); // 67
    _viewModels.append(new MeasurementResultDeletionViewModel(SCREEN_MEASUREMENT_RESULT_DATA_DELETION, _model, this)); // 68
    _viewModels.append(new MeasurementResultCopyDeleteViewModel(SCREEN_MEASUREMENT_RESULT_DATA_COPY_DELETION, _model, this)); // 69
    _viewModels.append(new CompassViewModel(SCREEN_COMPASS, _model, this)); // 70

#ifdef NEW_USER_SETTING
    _viewModels.append(new UserDataLoggerSettingViewModel(SCREEN_USER_DATALOGGER_SETTING, _model, this)); // 71
    _viewModels.append(new UserMeasurementSelectionViewModel(SCREEN_USER_MEASUREMENT_SELECTION, _model, this)); // 72
    _viewModels.append(new DataLoggerSettingViewModel(SCREEN_DATALOGGER_SETTING, _model, this)); // 73
#endif
    _viewModels.append(new OtherViewModel(SCREEN_OTHER, _model, this)); // 74
    _viewModels.append(new BatteryInformationViewModel(SCREEN_BATTERY_INFO,_model,this));//75
    _viewModels.append(new Manualviewmodel(SCREEN_MANUAL, _model, this));
}

ViewModelFactory::ViewModelFactory(QObject *parent)
    : QObject(parent)
    , _viewModels()
    , _model(nullptr)
{

}
