/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include "homeviewmodel.h"

QVector<QVector<QString>> defaultHomeButtonData = {
    {{"m1"}, {"m6"}, {"m5"}, {"m3"}},
    {{"m2"}, {"m8"}, {"m9"}, {"m7"}},
};

QVector<QVector<QString>> selectedCoaxialCableHomeButtonData = {
    {{"m1"}, {"m6"}, {"m5"}, {"m3"}},
    {{"m2"}, {"m8"}, {"m9"}, {"m7"}},
};

QVector<QVector<QString>> developerModeButtonData = {
    {{"developerOn"}, {"developerOff"}}
};

const int HomeViewModel::BUTTON_COLUMN_NUMBER = 2;
const int HomeViewModel::BUTTON_ROW_NUMBER = 4;

static constexpr int SINGLE_MEASUREMENT_INDEX =
    0;  // シングル測定選択時の遷移先インデックス(ScreenInformations クラスの Home 画面からの遷移先定義に依存)
static constexpr int OTHER_MENU_INDEX = 2;              // その他選択時の遷移先インデックス(ScreenInformations クラスの Home 画面からの遷移先定義に依存)
static constexpr int OPTICAL_MEASUREMENT_INDEX = 4;     // 光測定選択時の遷移先インデックス(ScreenInformations クラスの Home 画面からの遷移先定義に依存)
static constexpr int MULTI_MEASUREMENT_INDEX =
    5;  // マルチ測定選択時の遷移先インデックス(ScreenInformations クラスの Home 画面からの遷移先定義に依存)
static constexpr int USER_MEASUREMENT_INDEX = 1;        // ユーザー設定選択時の遷移先インデックス(ScreenInformations クラスの Home 画面からの遷移先定義に依存)
static constexpr int SYSTEM_SETTING_INDEX = 8;          // システム設定選択時の遷移先インデックス(ScreenInformations クラスの Home 画面からの遷移先定義に依存)

static constexpr int DEVELOPER_ON = 0;
static constexpr int DEVELOPER_OFF = 1;
static constexpr int DEVELOPER_ON_SELECTION = 1;
static constexpr int DEVELOPER_OFF_SELECTION = 0;

static constexpr int SEQUENCE_START = 0;
static constexpr int SEQUENCE_SUCCESS = 10;
static constexpr int NEXT_SEQUENCE_BUTTON[] = {
    BUTTON_MENU,
    BUTTON_MENU,
    BUTTON_BAND,
    BUTTON_BAND,
    BUTTON_HOME,
    BUTTON_HOME,
    BUTTON_CHANNEL_DOWN,
    BUTTON_CHANNEL_UP,
    BUTTON_SCREEN_SAVE,
    BUTTON_DATA_SAVE
};

HomeViewModel::HomeViewModel(ScreenId screenId, LevelCheckerModel *&model, QObject *parent)
    : AbstractViewModel(screenId, model, parent)
    , _isDeveloperMode(false)
    , _developerOnOff(DEVELOPER_ON)
    , _developerSequence(SEQUENCE_START)
{
    setObjectName("homeViewModel");
    _buttonOperation.setButtonName(BUTTON_COLUMN_NUMBER, BUTTON_ROW_NUMBER, defaultHomeButtonData);
    _buttonOperation.setActiveObject(defaultHomeButtonData[0][0]);
    _activeButton = defaultHomeButtonData[0][0];
}

HomeViewModel::~HomeViewModel()
{

}

void HomeViewModel::activate()
{
    if (nullptr == _view) {
        qErrnoWarning("_view is null");
        return;
    }

    connect(this, SIGNAL(requestSetTakeOver(QVariant, QVariant)), _view, SLOT(backup(QVariant, QVariant)));
    connect(this, SIGNAL(requestStandBy()), this, SLOT(onRequestStandBy()), Qt::QueuedConnection);

    // デベロッパーモード表示の無効化
    _isDeveloperMode = false;
    if (_view) {
        _view->setProperty("isDeveloperMode", _isDeveloperMode);
    }
    _developerSequence = SEQUENCE_START;

    // 時刻の初期化
    _model->initDateTime();

    // メニューアイコンのリセット
    _model->clearMenuList();

    // 設定値を取得しメニューのアイコンを更新する
    _model->changeSoundVolumeIcon();

    // 同軸・光のメニューアイコンの表示
    QVariant value;
    int rfOptical = 0;
    _model->getValue(VALUE_RF_OPTICAL, value);
    rfOptical = value.toInt();

    switch (rfOptical) {
    case 0:
        _model->replaceMenuList(MenuListIconIndex::RF_OPTICAL_ICON, "icon_hikari_rf.png");
        break;

    case 1:
        _model->replaceMenuList(MenuListIconIndex::RF_OPTICAL_ICON, "icon_hikari_opt.png");
        break;

    default:
        break;
    }

    // デフォルトボタン以外がアクティブであれば、非アクティブに戻す
    resetToInitialButtonState(defaultHomeButtonData[0][0], _activeButton, _buttonOperation);
}

void HomeViewModel::deactivate()
{
    AbstractViewModel::deactivate();
    disconnect(this, SIGNAL(requestSetTakeOver(QVariant, QVariant)), _view, SLOT(backup(QVariant, QVariant)));
    disconnect(this, SIGNAL(requestStandBy()), this, SLOT(onRequestStandBy()));

}

void HomeViewModel::standBy()
{
    emit requestStandBy();
}

void HomeViewModel::onRequestStandBy()
{
    // ウェイクアップ要求を行っていればスタンバイ要求を行う
    if (_model->wakeUpRequested()) {
        _model->setWakeUpRequested(false);
        _model->requestStandBy(1);
    }
}

void HomeViewModel::setTakeOver()
{
    if (_activeButton == "m8") {
        emit requestSetTakeOver("transitionSourceScreen", "MeasureSetting");
        emit requestSetTakeOver("selectedMeasurement", "");
    } else {
        emit requestSetTakeOver("transitionSourceScreen", "");

        if (_activeButton == "m1") {
            emit requestSetTakeOver("selectedMeasurement", "SingleMeasurement");
        } else if (_activeButton == "m6") {
            emit requestSetTakeOver("selectedMeasurement", "MultiMeasurement");
        }
    }
}

void HomeViewModel::onRequest(int condition)
{
    // シングル・マルチ測定・ユーザー設定・光測定・システム設定を選択していたならウェイクアップ要求
    // その他はコンパスはサブ CPU スタンバイ、DC/光測定がサブ CPU ウェイクアップが必要なため、個別対応する
    if (condition == SINGLE_MEASUREMENT_INDEX ||
            condition == MULTI_MEASUREMENT_INDEX ||
            condition == OPTICAL_MEASUREMENT_INDEX ||
            condition == USER_MEASUREMENT_INDEX ||
            condition == SYSTEM_SETTING_INDEX) {
        _model->setWakeUpRequested(true);
        _model->wakeUp(1);
    } else if (condition == OTHER_MENU_INDEX) {
        _model->setWakeUpRequested(false);
    }

    emit request(_screenId, condition); // 0 origin のため
    return;
}

void HomeViewModel::moveCursor(ButtonId id)
{
    if ((id == BUTTON_UP) || (id == BUTTON_DOWN) || (id == BUTTON_RIGHT) || (id == BUTTON_LEFT)) {

        QString nextBtn;
        bool ret = _buttonOperation.nextActiveObject(id, nextBtn);

        if (ret) {

            // inactive
            QObject *target = nullptr;
            if (_view) {
                target = _view->findChild<QObject *>(_activeButton);
            }
            if (target) {
                target->setProperty("active", false);
            }

            // active
            target = nullptr;
            if (_view) {
                target = _view->findChild<QObject *>(nextBtn);
            }
            if (target) {
                target->setProperty("active", true);
            }

            _activeButton = nextBtn;
            _buttonOperation.setActiveObject(_activeButton);
        }
    }
}

void HomeViewModel::onKeyStatus(ButtonId id)
{
    if (_isDeveloperMode){
        if (id == BUTTON_ENTER) {
            if (_developerOnOff == DEVELOPER_ON) {
                _model->setDeveloperMode(DEVELOPER_ON_SELECTION);
            } else if (_developerOnOff == DEVELOPER_OFF) {
                _model->setDeveloperMode(DEVELOPER_OFF_SELECTION);
            } else {
                qDebug() << QString("UNDEFINED DEVELOPER SELECTION");
                return;
            }
            changeDeveloperMode(false);
        } else if (id == BUTTON_ESCAPE) {
            changeDeveloperMode(false);
        } else if (id == BUTTON_LEFT || id == BUTTON_RIGHT) {
            if (_view) {
                QObject *item = _view->findChild<QObject *>(developerModeButtonData[0][_developerOnOff]);
                if (item) {
                    item->setProperty("active", false);
                }
                _developerOnOff = _developerOnOff == DEVELOPER_ON ? DEVELOPER_OFF : DEVELOPER_ON;
                item = nullptr;
                if (_view) {
                    item = _view->findChild<QObject *>(developerModeButtonData[0][_developerOnOff]);
                }
                if (item) {
                    item->setProperty("active", true);
                }
                _buttonOperation.setActiveObject(developerModeButtonData[0][_developerOnOff]);
            }
        }
    } else {
        if (_developerSequence == SEQUENCE_SUCCESS) {
            if (id == BUTTON_ENTER) {
                changeDeveloperMode(true);
                return;
            }
        } else if (_developerSequence >= SEQUENCE_START && _developerSequence < SEQUENCE_SUCCESS){
            if (id == NEXT_SEQUENCE_BUTTON[_developerSequence]) {
                _developerSequence++;
                return;
            }
        }
        _developerSequence = SEQUENCE_START;
        if (id == BUTTON_ENTER) {
            int button_index = _activeButton.mid(1).toInt() - 1; // 0 orign のため
            onRequest(button_index);
        } else if (id == BUTTON_USER) {
            onRequest(1); // ユーザー・データロガー画面へ遷移する
        } else {
            moveCursor(id);
        }
    }
}

void HomeViewModel::onButtonPress(ButtonId id)
{
    if (id == BUTTON_ENTER) {
        if (_isDeveloperMode) {
            if (_view) {
                QObject *item = _view->findChild<QObject *>(developerModeButtonData[0][_developerOnOff]);
                if (item) {
#if 1	//20210625_dx5
                    item->setProperty("activebg", "combtn_p.png");
#else
                    item->setProperty("activebg", "constellationSetting_btn_p.png");
#endif
                }
            }
        } else if (_developerSequence != SEQUENCE_SUCCESS) {
            if (_view) {
                QObject *target = _view->findChild<QObject *>(_activeButton);
                if (target) {
                    target->setProperty("activebg", "menubtn_p.png");
                }
            }
        }
    }
}

void HomeViewModel::changeDeveloperMode(bool isDeveloperMode)
{
    _isDeveloperMode = isDeveloperMode;
    if (_view) {
        _view->setProperty("isDeveloperMode", _isDeveloperMode);
    }
    _developerSequence = SEQUENCE_START;

    if (_isDeveloperMode) {
        _developerOnOff = DEVELOPER_ON;
        _buttonOperation.setButtonName(1, 2, developerModeButtonData);
        QObject *item = nullptr;
        if (_view) {
            item = _view->findChild<QObject *>(developerModeButtonData[0][DEVELOPER_ON]);
            if (item) {
                item->setProperty("active", true);
#if 1	//20210625_dx5
                item->setProperty("activebg", "combtn_sel.png");
#else
                item->setProperty("activebg", "constellationSetting_btn_r.png");
#endif
            }
        }
        item = nullptr;
        if (_view) {
            item = _view->findChild<QObject *>(developerModeButtonData[0][DEVELOPER_OFF]);
            if (item) {
                item->setProperty("active", false);
#if 1	//20210625_dx5
                item->setProperty("activebg", "combtn_sel.png");
#else
                item->setProperty("activebg", "constellationSetting_btn_l.png");
#endif
            }
        }
        _buttonOperation.setActiveObject(developerModeButtonData[0][DEVELOPER_ON]);
    } else {
        QVariant value;
        _model->getValue(VALUE_RF_OPTICAL, value);
        int rfOptical = value.toInt();
        _buttonOperation.setButtonName(BUTTON_COLUMN_NUMBER, BUTTON_ROW_NUMBER, rfOptical == 0 ? selectedCoaxialCableHomeButtonData : defaultHomeButtonData);
        _buttonOperation.setActiveObject(_activeButton);
    }
}
