/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include "levelcheckerviewmodel.h"
#include <QQmlContext>

LevelCheckerViewModel::LevelCheckerViewModel(QQmlApplicationEngine *engine, LevelCheckerModel *model,
        QQuickWindow *window, QElapsedTimer *timer, QObject *parent)
    : QObject(parent)
    , _engine(engine)
    , _model(model)
    , _factory(ViewModelFactory::instance(this))
    , _controller()
    , _mainFrame(nullptr)
    , _popupManager(nullptr)
    , _errorManager(nullptr)
    , _window(window)
{
    create(timer);
}

LevelCheckerViewModel::~LevelCheckerViewModel()
{
    disconnect(_model, SIGNAL(notifyErrorOccured(LC70_FAIL_INFO_T)), this, SLOT(onNotifyErrorOccured(LC70_FAIL_INFO_T)));
}

void LevelCheckerViewModel::create(QElapsedTimer *timer)
{
    // ファクトリーで全画面の　ViewModel を生成する
    _factory.prepare(this, _model);
    _model->writeLog(QString(typeid(*this).name()),
                     QString("[START UP] all viewmodel created %1 ms").arg(timer->elapsed()));

    // 画面遷移コントローラーの生成
    _controller = new ScreenTransitionController(this);
    _model->writeLog(QString(typeid(*this).name()),
                     QString("[START UP] screen controller created %1 ms").arg(timer->elapsed()));

    // ポップアップ表示管理の生成
    _popupManager = new PopupManager(_model, this);
    _model->writeLog(QString(typeid(*this).name()),
                     QString("[START UP] popup manager created %1 ms").arg(timer->elapsed()));

    // エラー管理の生成
    _errorManager = new ErrorManager(_model, this);
    _errorManager->attachController(_controller);

    connect(_model, SIGNAL(notifyErrorOccured(LC70_FAIL_INFO_T)), this, SLOT(onNotifyErrorOccured(LC70_FAIL_INFO_T)));
    _model->writeLog(QString(typeid(*this).name()),
                     QString("[START UP] error manager created %1 ms").arg(timer->elapsed()));

    // MainFrameViewModel の取得
    _mainFrame = dynamic_cast<MainFrameViewModel *>(_factory.specify(SCREEN_MAINFRAME));

    // MainFrameViewModel に連携データを公開
    _engine->rootContext()->setContextProperty("attributes", _mainFrame->attributes());

    // 画面遷移コントローラー、 ViewModel のファクトリー、ポップアップ表示管理と ViewModel の接続
    _mainFrame->attach(_controller, &_factory, _popupManager, _errorManager, _window);
    _model->writeLog(QString(typeid(*this).name()), QString("[START UP] mainframe created %1 ms").arg(timer->elapsed()));
}

void LevelCheckerViewModel::onNotifyErrorOccured(LC70_FAIL_INFO_T errorInformation)
{
    _errorManager->receiveFailInfo(errorInformation);
}
