/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include "mainframeviewmodel.h"
#include <QDebug>

MainFrameViewModel::MainFrameViewModel(ScreenId screenId, LevelCheckerModel *&model, QObject *parent)
    : AbstractViewModel(screenId, model, parent)
    , _window(nullptr)
    , _controller(nullptr)
    , _history()
    , _currentScreen(nullptr)
    , _pastScreen(nullptr)
    , _attributes(new Attributes(this))
    , _popupManager(nullptr)
    , _errorManager(nullptr)
    , _currentScreenButtonProhibition()
    , _menuList(nullptr)
    , _machineStatus(nullptr)
    , _menuBarModel(nullptr)
    , _waitingForPowerOff(false)
    , _measurementImage()
    , _shutdownTimer(new QTimer(this))
    , _isLowBattery(false)
    , _loader(nullptr)
    , _fileList()
    , _batteryStatus(0)
    , _batteryLevel(0)
    , _oldBatteryLevel(0)
    , _test(false)
{

}

MainFrameViewModel::~MainFrameViewModel()
{
    disconnect(this, SIGNAL(request(ScreenId, int)), _controller, SLOT(onRequest(ScreenId, int)));
    disconnect(this, SIGNAL(requestDirect(ScreenId, bool)), _controller, SLOT(onRequestDirect(ScreenId, bool)));
    disconnect(this, SIGNAL(requestInHistory(ScreenId)), _controller, SLOT(onRequestInHistory(ScreenId)));
    disconnect(this, SIGNAL(requestWithChangeHistory(ScreenId, QList<ScreenId>, bool)), _controller,
               SLOT(onRequestWithChangeHistory(ScreenId, QList<ScreenId>, bool)));
    disconnect(this, SIGNAL(back()), _controller, SLOT(onBack()));
    disconnect(this, SIGNAL(requestToLoad(QVariant)), _view, SLOT(load(QVariant)));
    disconnect(this, SIGNAL(requestToOpenPopup(QVariant, QVariant, QVariant, QVariant, QVariant)), _view,
               SLOT(openPopup(QVariant, QVariant, QVariant, QVariant, QVariant)));
    disconnect(this, SIGNAL(requestToOpenStrongPopup(QVariant, QVariant, QVariant)), _view,
               SLOT(openStrongPopup(QVariant, QVariant, QVariant)));
    disconnect(this, SIGNAL(requestToOpenLowBatteryPopup()), _view, SLOT(openLowBatteryPopup()));
    disconnect(this, SIGNAL(requestToOpenErrorPopup(QVariant, QVariant)), _view, SLOT(openErrorPopup(QVariant, QVariant)));
    disconnect(this, SIGNAL(requestToShowMenuBar(QVariant, QVariant, QVariant)), _view, SLOT(openMenuBar(QVariant, QVariant, QVariant)));
    disconnect(this, SIGNAL(requestToSetFileList(QVariant)), _view, SLOT(setFileList(QVariant)));
    disconnect(this, SIGNAL(requestToClearFileList()), _view, SLOT(clearFileList()));
    disconnect(this, SIGNAL(requestToCloseMenuBar(QVariant)), _view, SLOT(closeMenuBar(QVariant)));
    disconnect(this, SIGNAL(requestToCloseAllMenuBar()), _view, SLOT(closeAllMenuBar()));
    disconnect(_view, SIGNAL(loaded()), this, SLOT(onLoaded()));
    disconnect(_view, SIGNAL(menuBarSelected(int, int, int)), this, SIGNAL(notifyMenuBarSelected(int, int, int)));
    disconnect(this, SIGNAL(cursorPressed(QVariant)), _view, SLOT(cursorPressed(QVariant)));
    disconnect(this, SIGNAL(cursorMoveUp(QVariant)), _view, SLOT(moveUpCursor(QVariant)));
    disconnect(this, SIGNAL(cursorMoveDown(QVariant)), _view, SLOT(moveDownCursor(QVariant)));
    disconnect(this, SIGNAL(acquiringFinished(bool)), _popupManager, SLOT(onAcquiringFinished(bool)));
    disconnect(_controller, SIGNAL(transit(ScreenId, QString, QString, QString, ButtonProhibitionData)), this,
               SLOT(onTransit(ScreenId, QString, QString, QString, ButtonProhibitionData)));
    disconnect(_popupManager, SIGNAL(requestToOpenPopup(QVariant, QVariant, QVariant, QVariant, QVariant)), this,
               SIGNAL(requestToOpenPopup(QVariant, QVariant, QVariant, QVariant, QVariant)));
    disconnect(_popupManager, SIGNAL(requestToOpenStrongPopup(QVariant, QVariant, QVariant)), this,
               SIGNAL(requestToOpenStrongPopup(QVariant, QVariant, QVariant)));
    disconnect(_popupManager, SIGNAL(requestToOpenLowBatteryPopup()), this, SIGNAL(requestToOpenLowBatteryPopup()));
    disconnect(_popupManager, SIGNAL(requestToOpenErrorPopup(QVariant, QVariant)), this,
               SIGNAL(requestToOpenErrorPopup(QVariant, QVariant)));
    disconnect(_popupManager, SIGNAL(closed()), this, SLOT(onPopupClosed()));
    disconnect(_popupManager, SIGNAL(accept(PopupId, bool)), this, SIGNAL(accept(PopupId, bool)));
    disconnect(_popupManager, SIGNAL(reject(PopupId)), this, SIGNAL(reject(PopupId)));
    disconnect(_popupManager, SIGNAL(rejectScanResult()), this, SIGNAL(rejectScanResult()));
    disconnect(_popupManager, SIGNAL(select(PopupId, int)), this, SIGNAL(select(PopupId, int)));
    disconnect(_popupManager, SIGNAL(notifyButtonInput(PopupId, ButtonId)), this, SIGNAL(notifyButtonInput(PopupId,
               ButtonId)));
    disconnect(_popupManager, SIGNAL(inputFileName(PopupId, SaveLocation, QString)), this, SIGNAL(inputFileName(PopupId,
               SaveLocation,
               QString)));
    disconnect(_popupManager, SIGNAL(requestToScan(void)), this, SIGNAL(requestToScan()));
    disconnect(_popupManager, SIGNAL(requestToStopScanning(void)), this, SIGNAL(requestToStopScanning()));
    disconnect(_popupManager, SIGNAL(requestToStopAcquring()), this, SIGNAL(requestToStopAcquring()));
    disconnect(this, SIGNAL(progress(double)), _popupManager, SLOT(onProgress(double)));
    disconnect(this, SIGNAL(progressText(QString, bool)), _popupManager, SLOT(onProgressText(QString, bool)));
    disconnect(_model, SIGNAL(updateMenuList(QVector<QString> *)), this, SLOT(onUpdateMenuList(QVector<QString> *)));
    disconnect(_model, SIGNAL(scanningFinished(bool)), _popupManager, SLOT(onScanningFinished(bool)));
    disconnect(_model, SIGNAL(notifyStandBy()), this, SLOT(onNotifyStandBy()));
    disconnect(_model, SIGNAL(notifyWakeUp()), this, SLOT(onNotifyWakeUp()));
    disconnect(_model, SIGNAL(notifyFailInfo(LC70_FAIL_INFO_T)), _errorManager, SLOT(receiveFailInfo(LC70_FAIL_INFO_T)));
    disconnect(_model, SIGNAL(notifyAutoPowerOff()), this, SLOT(onNotifyAutoPowerOff()));
    disconnect(_errorManager, SIGNAL(requestToOpenErrorPopup(QVariant, QVariant)), this, SLOT(onNotifyErrorPopup(QVariant, QVariant)));
    disconnect(_errorManager, SIGNAL(notifyPowerSupplyError(uint16_t, uint8_t, QString)), this, SIGNAL(notifyPowerSupplyError(uint16_t, uint8_t, QString)));
    disconnect(_errorManager, SIGNAL(closed()), this, SLOT(onPopupClosed()));
    disconnect(_errorManager, SIGNAL(notifyChangeCompassEnable(bool)), this, SLOT(onNotifyChangeCompassEnable(bool)));
    disconnect(_errorManager, SIGNAL(requestDirect(ScreenId, bool)), this, SIGNAL(requestDirect(ScreenId, bool)));
    disconnect(this, SIGNAL(requestToCloseErrorPopup(bool)), _errorManager, SLOT(onRequestToCloseErrorPopup(bool)));
    disconnect(this, SIGNAL(requestToStandBy(uint8_t)), _model, SLOT(onRequestToStandBy(uint8_t)));
    disconnect(_model, SIGNAL(deviceStatus(LC70_DEV_STAT_T)), this, SLOT(onDeviceStatus(LC70_DEV_STAT_T)));
    disconnect(_model, SIGNAL(azimuth(uint16_t)), this, SLOT(onAzimuth(uint16_t)));
    disconnect(_model, SIGNAL(soundVolumeSetting(int)), this, SLOT(onChangeSoundVolumeImage(int)));
}

void MainFrameViewModel::attach(ScreenTransitionController *controller, ViewModelFactory *factory,
                                PopupManager *popupManager, ErrorManager *errorManager, QQuickWindow *window)
{
    if (controller == nullptr) {
        return;
    }

    _controller = controller;
    _controller->prepare(_attributes);

    _factory = factory;
    _popupManager = popupManager;
    _errorManager = errorManager;
    _window = window;

    // qml ファイルの objectName プロパティの値
    _view = _model->search(QString(
                               MAIN_FRAME_QML_OBJECT_NAME));
    _menuList = _model->search(QString(MENU_LIST_QML_OBJECT_NAME));
    _machineStatus = _model->search(QString(MACHINE_STATUS_QML_OBJECT_NAME));
    _menuBarModel = _model->search(QString(MENU_BAR_MODEL_QML_OBJECT_NAME));
    _loader = _model->search(QString(LOADER_QML_OBJECT_NAME));

    // QML に定義しているオブジェクトをモデルに渡す
    _model->setDisplayFileSystemModel(_view->findChild<QFileSystemModel *>(QString("DisplayFileSystemModel")));

    connect(this, SIGNAL(request(ScreenId, int)), _controller, SLOT(onRequest(ScreenId, int)));
    connect(this, SIGNAL(requestDirect(ScreenId, bool)), _controller, SLOT(onRequestDirect(ScreenId, bool)));
    connect(this, SIGNAL(requestInHistory(ScreenId)), _controller, SLOT(onRequestInHistory(ScreenId)));
    connect(this, SIGNAL(requestWithChangeHistory(ScreenId, QList<ScreenId>, bool)), _controller,
            SLOT(onRequestWithChangeHistory(ScreenId, QList<ScreenId>, bool)));
    connect(this, SIGNAL(back()), _controller, SLOT(onBack()));
    connect(this, SIGNAL(requestToLoad(QVariant)), _view, SLOT(load(QVariant)));
    connect(this, SIGNAL(requestToOpenPopup(QVariant, QVariant, QVariant, QVariant, QVariant)), _view,
            SLOT(openPopup(QVariant, QVariant, QVariant, QVariant, QVariant)));
    connect(this, SIGNAL(requestToOpenStrongPopup(QVariant, QVariant, QVariant)), _view,
            SLOT(openStrongPopup(QVariant, QVariant, QVariant)));
    connect(this, SIGNAL(requestToOpenLowBatteryPopup()), _view, SLOT(openLowBatteryPopup()));
    connect(this, SIGNAL(requestToOpenErrorPopup(QVariant, QVariant)), _view, SLOT(openErrorPopup(QVariant, QVariant)));
    connect(this, SIGNAL(requestToShowMenuBar(QVariant, QVariant, QVariant)), _view, SLOT(openMenuBar(QVariant, QVariant, QVariant)));
    connect(this, SIGNAL(requestToSetFileList(QVariant)), _view, SLOT(setFileList(QVariant)));
    connect(this, SIGNAL(requestToClearFileList()), _view, SLOT(clearFileList()));
    connect(this, SIGNAL(requestToCloseMenuBar(QVariant)), _view, SLOT(closeMenuBar(QVariant)));
    connect(this, SIGNAL(requestToCloseAllMenuBar()), _view, SLOT(closeAllMenuBar()));
    connect(_view, SIGNAL(loaded()), this, SLOT(onLoaded()));
    connect(_view, SIGNAL(menuBarSelected(int, int, int)), this, SIGNAL(notifyMenuBarSelected(int, int, int)));
    connect(this, SIGNAL(cursorPressed(QVariant)), _view, SLOT(cursorPressed(QVariant)));
    connect(this, SIGNAL(cursorMoveUp(QVariant)), _view, SLOT(moveUpCursor(QVariant)));
    connect(this, SIGNAL(cursorMoveDown(QVariant)), _view, SLOT(moveDownCursor(QVariant)));
    connect(this, SIGNAL(acquiringFinished(bool)), _popupManager, SLOT(onAcquiringFinished(bool)));
    connect(_controller, SIGNAL(transit(ScreenId, QString, QString, QString, ButtonProhibitionData)), this,
            SLOT(onTransit(ScreenId, QString, QString, QString, ButtonProhibitionData)));
    connect(_popupManager, SIGNAL(requestToOpenPopup(QVariant, QVariant, QVariant, QVariant, QVariant)), this,
            SIGNAL(requestToOpenPopup(QVariant, QVariant, QVariant, QVariant, QVariant)));
    connect(_popupManager, SIGNAL(requestToOpenStrongPopup(QVariant, QVariant, QVariant)), this,
            SIGNAL(requestToOpenStrongPopup(QVariant, QVariant, QVariant)));
    connect(_popupManager, SIGNAL(requestToOpenErrorPopup(QVariant, QVariant)), this,
            SIGNAL(requestToOpenErrorPopup(QVariant, QVariant)));
    connect(_popupManager, SIGNAL(requestToOpenLowBatteryPopup()), this, SIGNAL(requestToOpenLowBatteryPopup()));
    connect(_popupManager, SIGNAL(closed()), this, SLOT(onPopupClosed()));
    connect(_popupManager, SIGNAL(accept(PopupId, bool)), this, SIGNAL(accept(PopupId, bool)));
    connect(_popupManager, SIGNAL(reject(PopupId)), this, SIGNAL(reject(PopupId)));
    connect(_popupManager, SIGNAL(rejectScanResult()), this, SIGNAL(rejectScanResult()));
    connect(_popupManager, SIGNAL(select(PopupId, int)), this, SIGNAL(select(PopupId, int)));
    connect(_popupManager, SIGNAL(notifyButtonInput(PopupId, ButtonId)), this, SIGNAL(notifyButtonInput(PopupId,
            ButtonId)));
    connect(_popupManager, SIGNAL(inputFileName(PopupId, SaveLocation, QString)), this, SIGNAL(inputFileName(PopupId,
            SaveLocation,
            QString)));
    connect(_popupManager, SIGNAL(requestToScan(void)), this, SIGNAL(requestToScan()));
    connect(_popupManager, SIGNAL(requestToStopScanning(void)), this, SIGNAL(requestToStopScanning()));
    connect(_popupManager, SIGNAL(requestToStopAcquring()), this, SIGNAL(requestToStopAcquring()));
    connect(this, SIGNAL(progress(double)), _popupManager, SLOT(onProgress(double)));
    connect(this, SIGNAL(progressText(QString, bool)), _popupManager, SLOT(onProgressText(QString, bool)));
    connect(_model, SIGNAL(updateMenuList(QVector<QString> *)), this, SLOT(onUpdateMenuList(QVector<QString> *)));
    connect(_model, SIGNAL(scanningFinished(bool)), _popupManager, SLOT(onScanningFinished(bool)));
    connect(_model, SIGNAL(notifyStandBy()), this, SLOT(onNotifyStandBy()));
    connect(_model, SIGNAL(notifyWakeUp()), this, SLOT(onNotifyWakeUp()));
    connect(_model, SIGNAL(notifyFailInfo(LC70_FAIL_INFO_T)), _errorManager, SLOT(receiveFailInfo(LC70_FAIL_INFO_T)));
    connect(_model, SIGNAL(notifyAutoPowerOff()), this, SLOT(onNotifyAutoPowerOff()));
    connect(_errorManager, SIGNAL(closed()), this, SLOT(onPopupClosed()));
    connect(_errorManager, SIGNAL(requestToOpenErrorPopup(QVariant, QVariant)), this, SLOT(onNotifyErrorPopup(QVariant, QVariant)));
    connect(_errorManager, SIGNAL(notifyPowerSupplyError(uint16_t, uint8_t, QString)), this, SIGNAL(notifyPowerSupplyError(uint16_t, uint8_t, QString)));
    connect(_errorManager, SIGNAL(notifyChangeCompassEnable(bool)), this, SIGNAL(notifyChangeCompassEnable(bool)));
    connect(_errorManager, SIGNAL(requestDirect(ScreenId, bool)), this, SIGNAL(requestDirect(ScreenId, bool)));
    connect(this, SIGNAL(requestToCloseErrorPopup(bool)), _errorManager, SLOT(onRequestToCloseErrorPopup(bool)));
    connect(this, SIGNAL(requestToStandBy(uint8_t)), _model, SLOT(onRequestToStandBy(uint8_t)));
    connect(_shutdownTimer, SIGNAL(timeout()), this, SLOT(onShutdown()));

    // ラッパー関数経由で取得する値
    // デバイス状態
    connect(_model, SIGNAL(deviceStatus(LC70_DEV_STAT_T)), this, SLOT(onDeviceStatus(LC70_DEV_STAT_T)),
            Qt::QueuedConnection);
    // 方位角
    connect(_model, SIGNAL(azimuth(uint16_t)), this, SLOT(onAzimuth(uint16_t)), Qt::QueuedConnection);

    // 音量設定
    connect(_model, SIGNAL(soundVolumeSetting(int)), this, SLOT(onChangeSoundVolumeImage(int)));

    // 起動直後にバッテリ低下状態変化を通知する（初回はバッテリ低下未発生）
    _model->notifyLowBatteryStatusChanged(false);

    // 測定停止通知
    connect(_model, SIGNAL(notifyStopMeasuring(AcquisitionTarget)), this, SLOT(onNotifyStopMeasuring(AcquisitionTarget)));

    // Home 画面を表示
    prepare();
}

void MainFrameViewModel::prepare(void)
{
    // ホーム画面を表示
    emit request(_screenId);

    _menuList->setProperty("visible", true);

    // 起動時にサブのスタンバイ要求
    emit requestToStandBy(1);
}

void MainFrameViewModel::onTransit(ScreenId nextId, QString screenName, QString objectName, QString qmlFileName,
                                   ButtonProhibitionData buttonProhibitionData)
{
    _view = _model->search(QString(
                               MAIN_FRAME_QML_OBJECT_NAME));

    if (_view == nullptr) {
        _model->takeMeasureToFindNoView(MAIN_FRAME_QML_OBJECT_NAME);
        return;
    }

    // 自動パワーオフ待ちをクリアー
    _waitingForPowerOff = false;

    // ViewModelオブジェクトを特定
    QObject *viewModel = _factory->specify(nextId);

    // ViewModel が特定できなければ画面遷移しない
    if (!viewModel) {
        return;
    }

    // 今まで開いていた画面
    _pastScreen = _currentScreen;

    // これから開く画面
    _currentScreen = dynamic_cast<AbstractViewModel *>(viewModel);

    if (_pastScreen) {
        qDebug() << QString("Past:") << _pastScreen->objectName();
    } else {
        qDebug() << QString("Past: nullptr");
    }

    qDebug() << QString("Current:") << _currentScreen->objectName();

    // ホーム画面に遷移する場合に引き継ぎデータをクリアーする
    if (nextId == SCREEN_HOME) {
        _attributes->clearTakeOver();
        qDebug() << QString("clear take over.");
    }

    // 今まで開いていた画面の引き継ぎデータを設定する
    _attributes->setTakeOverData();

    // これから開く画面の情報を設定する
    _attributes->setQmlPath(qmlFileName);
    _attributes->setViewModel(QVariant::fromValue(_currentScreen));

    // これから開く画面のボタン禁則情報
    _currentScreenButtonProhibition = buttonProhibitionData;

    // コンテンツ画面の変更を要求する
    // シングル測定、マルチ測定は起動の高速化のためロードせず予め画面を作成し、非表示にしているのを表示する
    if (nextId == SCREEN_SINGLE_MEASUREMENT) {
        emit requestToLoad(QVariant(SCREEN_TYPE_SINGLE_MEASUREMENT));
    } else if (nextId == SCREEN_MULTI_MEASUREMENT) {
        emit requestToLoad(QVariant(SCREEN_TYPE_MULTI_MEASUREMENT));
    } else {
        emit requestToLoad(QVariant(SCREEN_TYPE_OTHER));
    }

    _model->writeLog(QString("画面遷移："), screenName);

    // 開く画面と対応する QML の連携
    QObject *view = nullptr;
    if (nextId == SCREEN_SINGLE_MEASUREMENT) {
        view = _view->findChild<QObject *>(QString("SingleMeasurement"));
    } else if (nextId == SCREEN_MULTI_MEASUREMENT) {
        view = _view->findChild<QObject *>(QString("MultiMeasurement"));
    } else {
        view = _model->search(objectName);
    }

    // qml ファイルがロードされなかった、あるいは表示できなかった
    if (view == nullptr) {
        _model->takeMeasureToFindNoView(screenName);
        return;
    }

    // qml ファイル名を設定
    _currentScreen->setQmlFile(qmlFileName);

    // 引き継ぎ情報を渡す
    _currentScreen->takeOverReference(_attributes->takeOver());

    // オブジェクト間の signla 接続
    _currentScreen->createConnection(view, this);

    // 開く画面の前回設定値の復元
    _currentScreen->activate();

    // ファイルリスト生成
    if (_pastScreen) {
        if (((_pastScreen->objectName() != "singleMeasurementViewModel") &&
            (_pastScreen->objectName() != "multiMeasurementViewModel") &&
            (_pastScreen->objectName() != "antennaAdjustmentViewModel")) &&
            ((_currentScreen->objectName() == "singleMeasurementViewModel") ||
             (_currentScreen->objectName() == "multiMeasurementViewModel") ||
             (_currentScreen->objectName() == "antennaAdjustmentViewModel")))
        {
            QVariant fileValue;
            int cnt = 0;
            bool ret = _model->getValue(VALUE_USERSETTING_FILELIST, fileValue);
            if (ret) {
                QVariantList contents;
                QVariantList vFileList = fileValue.toList();
                _fileList.clear();
                contents.clear();
                for (QVariantList::iterator it = vFileList.begin(); it != vFileList.end(); ++it, ++cnt) {
                    // ファイル番号
                    int fileNo = cnt + 1;
                    // ファイル名
                    QString fileName = (*it).toList()[2].toString();
                    _fileList.push_back(fileName);
                    // 名前
                    QString name = QString("");
                    // ノート
                    QString note = QString("");
                    // アイコン
                    QString icon = QString("");
                    // カーソル
                    QString cursor = "qrc:/resource/png/menubar_cur.png";
//                    if (cnt == 0) {
//                        if (fileName == "") {
//                            cursor = "qrc:/resource/png/menubar_cur.png";
//                        } else {
//                            cursor = "qrc:/resource/png/menubar_cur.png";
//                        }
//                    } else if (cnt == vFileList.count() - 1)  {
//                        if (fileName == "") {
//                            cursor = "qrc:/resource/png/menubar_cur.png";
//                        } else {
//                            cursor = "qrc:/resource/png/menubar_cur.png";
//                        }
//                    } else {
//                        if (fileName == "") {
//                            cursor = "qrc:/resource/png/menubar_cur.png";
//                        } else {
//                            cursor = "qrc:/resource/png/menubar_cur.png";
//                        }
//                    }
                    QVariantMap map;
                    map.insert("fileNo", fileNo);
                    map.insert("fileName", fileName);
                    map.insert("name", name);
                    map.insert("note", note);
                    map.insert("icon", icon);
                    map.insert("cursor", cursor);
                    contents.push_back(map);
                }
                emit requestToSetFileList(contents);
            }
        }
    }

    // 開く画面の ViewModel に対するスレッドの開始・再開、
    _currentScreen->resume();

    // 閉じる画面の ViewModel に対するスレッドの停止
    if (_pastScreen) {
        _pastScreen->suspend();
    }

    // 閉じる画面の ViewModel に対する現在設定内容の保存
    if (_pastScreen) {
        _pastScreen->deactivate();
        if (((_currentScreen->objectName() != "singleMeasurementViewModel") &&
             (_currentScreen->objectName() != "multiMeasurementViewModel") &&
             (_currentScreen->objectName() != "antennaAdjustmentViewModel")) &&
            ((_pastScreen->objectName() == "singleMeasurementViewModel") ||
             (_pastScreen->objectName() == "multiMeasurementViewModel") ||
             (_pastScreen->objectName() == "antennaAdjustmentViewModel"))) {
            _fileList.clear();
            emit requestToClearFileList();
        }
    }

    // 測定系の画面以外ならスタンバイ実行(測定系の画面は測定停止を受信後スタンバイ実行)
    if (_pastScreen != nullptr && _currentScreen != nullptr) {
        if (_pastScreen->objectName() != QString("singleMeasurementViewModel") &&
                _pastScreen->objectName() != QString("multiMeasurementViewModel") &&
                _pastScreen->objectName() != QString("antennaAdjustmentViewModel") &&
                _pastScreen->objectName() != QString("dataLoggerMeasurementListViewModel")) {
            _currentScreen->standBy();
        }
    }
}

void MainFrameViewModel::onLoaded()
{
    qDebug() << Q_FUNC_INFO;
}

void MainFrameViewModel::onButtonClicked(ButtonId buttonId)
{
    if (buttonId == BUTTON_UNKNOWN) {
        return;
    }

    if (_currentScreenButtonProhibition.prohibition[buttonId] == false) {
        //return;
    }
}

void MainFrameViewModel::onRequestToShowPopup(PopupId popupId)
{
    _currentScreen->setAcceptButtonInput(false);
    _popupManager->open(popupId, false);
}

void MainFrameViewModel::onRequestToShowPopupScan(PopupId popupId, int detailId)
{
    qDebug() << "onRequestToShowPopupScan() detailId: " << detailId;
    _currentScreen->setAcceptButtonInput(false);
    _popupManager->openScan(popupId, detailId, false);
}

void MainFrameViewModel::onRequestToShowPopupWithMessage(PopupId popupId, QString message)
{
    _currentScreen->setAcceptButtonInput(false);
    _popupManager->openWithMessage(popupId, message, false);
}

void MainFrameViewModel::onRequestToShowLowBatteryPopup()
{
    _currentScreen->setAcceptButtonInput(false);
    _popupManager->openLowBattery();
}

void MainFrameViewModel::onRequestPopupClose()
{
    if (!_popupManager->isOpend()) {
        // ポップアップが表示されていなければ処理なし
        return;
    }

    // ポップアップを閉じる
    _popupManager->close();
}

void MainFrameViewModel::onPopupClosed()
{
    _currentScreen->setAcceptButtonInput(true);
}

void MainFrameViewModel::onRequestToCapture()
{
    // 既に保存されているQImageを削除する
    _measurementImage = QImage();
    if (_window) {
        _measurementImage = _window->grabWindow();
    }
}

void MainFrameViewModel::onRequestToSaveImage(MeasureType measureType, SaveLocation saveLocation, QString fileName)
{
    Q_UNUSED(saveLocation)

    // イメージが登録されてなければ後続処理は実施しない
    if (!_measurementImage.isNull()) {

        QString path;
        if (measureType == SINGLE_MEASUREMENT) {
            path = _model->createDirectory(SaveFileType::PNG, SINGLE_MEASUREMENT);
        } else if (measureType == MULTI_MEASUREMENT) {
            path = _model->createDirectory(SaveFileType::PNG, MULTI_MEASUREMENT);
        } else {
            return;
        }

        if (!path.isEmpty()) {
            if (!_model->writeSingleMeasurementScreen(path + fileName, _measurementImage)) {
                _model->writeLog(QString("メインフレーム"), QString("save error fileName: %1").arg(path + fileName));
                onRequestToShowPopup(POPUP_WRITE_ERROR);
            }
        }
    }
    _measurementImage = QImage();
}

void MainFrameViewModel::onRequestToClearCaptureImage()
{
    _measurementImage = QImage();
}

void MainFrameViewModel::onRequestToSaveData(SaveLocation saveLocation, QString fileName,
        uint16_t condition, MeasureResult singleMeasureResult)
{
    Q_UNUSED(saveLocation)

    // 測定データの出力
    QString path = _model->createDirectory(SaveFileType::CSV, SINGLE_MEASUREMENT);

    if (!path.isEmpty()) {
        // ファイルに保存
        if (!_model->writeSingleMeasurementData(path + fileName, condition, singleMeasureResult)) {
            qDebug() << "save error: " << path + fileName;
            onRequestToShowPopup(POPUP_WRITE_ERROR);
        }
    }
}

void MainFrameViewModel::onRequestToSaveDataMulti(SaveLocation saveLocation, MeasureType measureType, QString fileName,
        uint16_t condition, QList<MeasureResult> multiMeasureResult)
{
    Q_UNUSED(saveLocation)

    // 測定データの出力
    QString path = _model->createDirectory(SaveFileType::CSV, MULTI_MEASUREMENT);

    if (!path.isEmpty()) {
        // ファイルに保存
        if (!_model->writeMultiMeasurementData(path + fileName, condition, multiMeasureResult)) {
            qDebug() << "save error: " << path + fileName;
            onRequestToShowPopup(POPUP_WRITE_ERROR);
        }

        emit notifySaveEnd();
    }
}

void MainFrameViewModel::onAzimuth(uint16_t azimuth)
{
    // 方位角を受信
    emit notifyAzimuth(azimuth);
}

void MainFrameViewModel::onDeviceStatus(LC70_DEV_STAT_T deviceStatus)
{
    static uint8_t battery = 0xFF;
    static uint8_t batteryVol = 0xFF;

    battery = deviceStatus.ucBatt;
    _model->writeLog(QString("デバイス状態取得"), QString("バッテリー状態： %1").arg(deviceStatus.ucBatt));

    _batteryStatus = BATTERY_STATUS_UNKNOWN;
    switch(battery & LC70_BATT_STS_MSK) {   // バッテリー状態
    case LC70_BATT_STS_MSK_UNCHG:   // 未充電
        _batteryStatus =  BATTERY_STATUS_UNCHARGED;
        break;
    case LC70_BATT_STS_MSK_CHGING:  // 充電中
        _batteryStatus =  BATTERY_STATUS_CHARGING;
        // 充電中を検知した時点でシャットダウン待ちならポップアップを閉じてシャットダウンを止める
        if (_shutdownTimer->isActive()) {
            _shutdownTimer->stop();
            _popupManager->close();
        }
        break;
    case LC70_BATT_STS_MSK_FULL:    // 満充電
        _batteryStatus =  BATTERY_STATUS_CHARGED;
        break;
    case LC70_BATT_STS_MSK_INDEF:   // 不定値
    default:
        break;
    }
    //qDebug() << QString("[Battery Status]:%1").arg(_batteryStatus);

    _machineStatus->setProperty("batteryStatus", _batteryStatus);

    battery = deviceStatus.ucBatt;
    _oldBatteryLevel = _batteryLevel;
    _batteryLevel = BATTERY_LEVEL_UNKNOWN;
    switch(battery & LC70_BATT_LVL_MSK) {   // バッテリー残量
    case LC70_BATT_LVL_MSK_RED0:    // 残量0-5%
        _batteryLevel = BATTERY_LEVEL1;
        break;
    case LC70_BATT_LVL_MSK_RED1:    // 残量6-10%
        _batteryLevel = BATTERY_LEVEL2;
        break;
    case LC70_BATT_LVL_MSK_GREEN1:  // 残量11-30%
        _batteryLevel = BATTERY_LEVEL3;
        break;
    case LC70_BATT_LVL_MSK_GREEN2:  // 残量31-50%
        _batteryLevel = BATTERY_LEVEL4;
        break;
    case LC70_BATT_LVL_MSK_GREEN3:  // 残量51-70%
        _batteryLevel = BATTERY_LEVEL5;
        break;
    case LC70_BATT_LVL_MSK_GREEN4:  // 残量71-100%
        _batteryLevel = BATTERY_LEVEL6;
        break;
    case LC70_BATT_LVL_MSK_RSV:     // 予約
    case LC70_BATT_LVL_MSK_ERR:     // 不定値
    default:
        break;
    }
    //qDebug() << QString("[Battery Level]:%1").arg(_batteryLevel);

    _machineStatus->setProperty("batteryLevel", _batteryLevel);

    // バッテリー残量値
    if (batteryVol != deviceStatus.ucBatv) {
        batteryVol = deviceStatus.ucBatv;
        _model->writeLog(QString("デバイス状態取得"), QString("バッテリー残量値： %1").arg(deviceStatus.ucBatv));

        _machineStatus->setProperty("batteryVol", batteryVol);
        //qDebug() << QString("batteryVol --- ") << batteryVol;
    }

    // バッテリー使用可能時間
    emit notifyBatteryCharge(deviceStatus.ucBatt, deviceStatus.ucBatempH, deviceStatus.ucBatempM);

    // 給電状態 USB type-c
    if (deviceStatus.usbc == LC70_USBC_CONNECT) {
        _machineStatus->setProperty("usbTypeCVisible", false);
    } else if (deviceStatus.usbc == LC70_USBC_POWERED) {
        _machineStatus->setProperty("usbTypeCVisible", true);
    } else if (deviceStatus.usbc == LC70_USBC_DISCONNECT) {
        _machineStatus->setProperty("usbTypeCVisible", false);
    } else {
        _machineStatus->setProperty("usbTypeCVisible", false);
    }

    // USB type-A
    if (deviceStatus.usba == LC70_STRAGE_NONE) {
        _machineStatus->setProperty("usb2Visible", false);
    } else if (deviceStatus.usba == LC70_STRAGE_NON_SUPPORT_FORMAT) {
        _machineStatus->setProperty("usb2Visible", false);
    } else if (deviceStatus.usba == LC70_STRAGE_FORMAT) {
        _machineStatus->setProperty("usb2Visible", true);
    } else if (deviceStatus.usba == LC70_STRAGE_UNFORMAT) {
        _machineStatus->setProperty("usb2Visible", false);
    } else if (deviceStatus.usba == LC70_STRAGE_UNACQ) {
        _machineStatus->setProperty("usb2Visible", false);
    } else {
        _machineStatus->setProperty("usb2Visible", false);
    }

    // 充電中ではなくバッテリー残量が 0 - 5 % ならポップアップを表示して 10 秒後にシャットダウンする
    if (_batteryStatus <= BATTERY_STATUS_UNCHARGED && _batteryLevel == BATTERY_LEVEL1 && !_shutdownTimer->isActive()) {
        // ポップアップを一旦全て閉じる
        emit requestToCloseErrorPopup(true);
        onRequestPopupClose();

        // バッテリー不足でシャットダウンすることをポップアップで表示
        _currentScreen->setAcceptButtonInput(false);
        _popupManager->openLowBattery();

        // 10 秒後にシャットダウンするためのタイマー開始
        _shutdownTimer->setInterval(10000);
        _shutdownTimer->start();
    }

    // バッテリ低下状態に変化があった場合は、サブCPUと画面に通知
    if (_oldBatteryLevel != _batteryLevel) {
        if (_batteryLevel <= BATTERY_LEVEL1) {
            // AC アダプターが装着されていなければローバッテリーとする
            _isLowBattery = (_batteryStatus <= BATTERY_STATUS_UNCHARGED);
            _model->notifyLowBatteryStatusChanged(_isLowBattery);
        } else {
            _isLowBattery = false;
            _model->notifyLowBatteryStatusChanged(_isLowBattery);
        }
    }
    // 給電可能かどうかは allowPowerSupplyChanged signal で判定する
    // 給電可能か画面に通知(バッテリー 10% 以下で false, 11 % 以上で true)
    // emit allowPowerSupplyChanged(_batteryLevel > BATTERY_LEVEL2);
    if (_batteryLevel <= BATTERY_LEVEL2) {
        if (_batteryStatus == BATTERY_STATUS_CHARGING) {
            emit allowPowerSupplyChanged(true);
        } else {
            emit allowPowerSupplyChanged(false);
        }
    } else {
        emit allowPowerSupplyChanged(true);
    }
}

void MainFrameViewModel::onUpdateMenuList(QVector<QString> *menuList)
{
    // menu1
    updateMenu(0, menuList);

    // menu2
    updateMenu(1, menuList);

    // menu3
    updateMenu(2, menuList);

    // menu4
    updateMenu(3, menuList);

    // menu5
    updateMenu(4, menuList);

    //menu6
    updateMenu(5, menuList);


}

void MainFrameViewModel::onChangeAttribute(QString key, QVariant value)
{
    _attributes->change(key, value);
}

void MainFrameViewModel::onRemoveAttribute(QString key)
{
    _attributes->remove(key);
}

void MainFrameViewModel::onNotifyStandBy()
{
    // 画面が動作中でなければスタンバイ実行
    if (!_currentScreen->inAction()) {
        // 画面非表示
        _view->setProperty("visible", false);

        // エラーポップアップを閉じる
        emit requestToCloseErrorPopup(true);

        // ポップアップは閉じる
        onRequestPopupClose();

        // スタンバイ前の処理の呼び出し
        _currentScreen->readyToStandby();

        // Home 画面に遷移しておく
        emit requestDirect(SCREEN_HOME, true);

        _waitingForPowerOff = false;

        // スタンバイ(MAIN, SUB)
        emit requestToStandBy(0);
    } else {
        _model->setStandBy(false);
        _waitingForPowerOff = true;
    }
}

void MainFrameViewModel::onNotifyWakeUp()
{
    // Home 画面の復帰
    _currentScreen->createConnection(_model->search(QString("Home")), this);
    _currentScreen->activate();
    _currentScreen->resume();

    // シャットダウン待ちの状態でスタンバイして復帰したとき
    if (_shutdownTimer->isActive()) {
        // バッテリー不足でシャットダウンすることをポップアップで表示
        _currentScreen->setAcceptButtonInput(false);
        _popupManager->openLowBattery();
    }

    _view->setProperty("visible", true);
}

void MainFrameViewModel::updateMenu(int index, QVector<QString> *&menuList)
{
    QString menuIconName;

    if (menuList->size() <= index) {
        menuIconName.clear();
    } else {
        menuIconName = "qrc:/resource/png/" + menuList->at(index);
    }

    static QVector<const char *> propertyNames = QVector<const char *>() << "optical" << "functionType" << "band" << "scan"
            << "powerSupply" << "preamp";

    _menuList->setProperty(propertyNames[index], menuIconName);
}

void MainFrameViewModel::onChangeSoundVolumeImage(int value)
{
    QMap<int, QString> volumeIconMap = {
        {static_cast<int>(LC70_BUZZ_VOL_MUTE), "qrc:/resource/png/icon_sound_mute.png"},
        {static_cast<int>(LC70_BUZZ_VOL_MIN),  "qrc:/resource/png/icon_sound_1.png"},
        {static_cast<int>(LC70_BUZZ_VOL_MID),  "qrc:/resource/png/icon_sound_2.png"},
        {static_cast<int>(LC70_BUZZ_VOL_MAX),  "qrc:/resource/png/icon_sound_3.png"}
    };

    QString setImage = volumeIconMap.value(value, "qrc:/resource/png/icon_sound_mute.png");
    _machineStatus->setProperty("soundVolume", setImage);
}

bool MainFrameViewModel::buttonProhibition(ButtonId buttonId)
{
    return _currentScreenButtonProhibition.prohibition[buttonId];
}

void MainFrameViewModel::onNotifyErrorPopup(QVariant message, QVariant buttonCaption)
{
    _currentScreen->setAcceptButtonInput(false);
    emit requestToOpenErrorPopup(message, buttonCaption);
}

void MainFrameViewModel::onNotifyChangeInAction(bool inAction)
{
    // スタンバイ待ちで非動作中になればスタンバイ実行
    // データロガー実行中は常に実行中とする
    if (!inAction) {
        if (_waitingForPowerOff) {

            _model->writeLog(QString(typeid(*this).name()), QString("### Turn off the power. Screen is not running"), LOG_INFO);

            // スタンバイ実行
            onNotifyStandBy();
        }
    }
}

void MainFrameViewModel::onNotifyAutoPowerOff()
{
    // 自動パワーオフの時間になったが動作中であれば処理待ちとする
    if (_currentScreen->inAction()) {
        _waitingForPowerOff = true;
        _model->writeLog(QString(typeid(*this).name()), QString("### Do not turn off the power. Screen is running."), LOG_INFO);
        return;
    }

    // スタンバイ実行
    onNotifyStandBy();
}

void MainFrameViewModel::onShowIndicator(bool show)
{
    _view->setProperty("showActivityIndicator", show);
}

void MainFrameViewModel::onShutdown()
{
    // この時点で充電中状態に復帰していたらシャットダウンしない
    if (_batteryStatus == BATTERY_STATUS_CHARGING) {
        return;
    }

    // シャットダウン前の処理の呼び出し
    _currentScreen->readyToStandby();

    if(_model->shutDown() != LC70_OK) {
        // シャットダウンできなかったらリトライ
        _shutdownTimer->start();
    }
}

void MainFrameViewModel::onRequestToReload(QString qmlFile)
{
    // 静電気により現在表示している画面へのポインターが消えたとき qml のロードを行う
    // ただし、現在未使用
    if (qmlFile.isEmpty()) {
        return;
    }
    if (_loader) {
        _loader->setProperty("active", false);
        _loader->setProperty("source", qmlFile);
        _loader->setProperty("active", true);
    }
}

void MainFrameViewModel::onNotifyStopMeasuring(AcquisitionTarget target)
{
    Q_UNUSED(target)
    // 前の画面を閉じた後に行う処理(現状 Home 画面に戻ったときにサブを止めるのみ)
    _currentScreen->standBy();
}

bool MainFrameViewModel::isEnabledCompass(void)
{
    return _errorManager->enabledCompass();
}

QObject *MainFrameViewModel::singleMeasurementView()
{
    QObject *view = nullptr;
    if (_view) {
        view = _view->findChild<QObject *>(QString("SingleMeasurement"));
    }

    return view;
}

QObject *MainFrameViewModel::multiMeasurementView()
{
    QObject *view = nullptr;
    if (_view) {
        view = _view->findChild<QObject *>(QString("MultiMeasurement"));
    }

    return view;
}

int MainFrameViewModel::batteryStatus() const
{
    return _batteryStatus;
}

int MainFrameViewModel::batteryLevel() const
{
    return _batteryLevel;
}

void MainFrameViewModel::menuoptchange(bool checkopt){
    _view->setProperty("checkingopt",checkopt);

}

void MainFrameViewModel::test()
{
//    LC70_DEV_STAT_T deviceStatus;

//    _debug = true;
//    // 給電デバッグ用
//    _test = !_test;

//    // 給電デバッグ用
//    if (_debug) {
//        deviceStatus.ucBatt = 0xFF;
//        deviceStatus.ucBatt &= LC70_BATT_STS_MSK;
//        deviceStatus.ucBatt &= LC70_BATT_LVL_MSK;
//        if (_test) {
//            deviceStatus.ucBatt |= LC70_BATT_STS_MSK_UNCHG;
//        } else {
//            deviceStatus.ucBatt |= LC70_BATT_STS_MSK_CHGING;
//        }
//        deviceStatus.ucBatt |= LC70_BATT_LVL_MSK_RED0;

//        onDeviceStatus(deviceStatus);
//    }
}
