/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef MAINFRAME_H
#define MAINFRAME_H

#include "abstractviewmodel.h"
#include "constant.h"
#include "screentransitioncontroller.h"
#include "viewmodelfactory.h"
#include "attributes.h"
#include "popupmanager.h"
#include "errormanager.h"
#include <QQuickWindow>
#include <QTimer>

/**
 * @brief 全画面表示するウィンドウの ViewModel
 * @note AbstractViewModel を継承しているがコンテンツ画面とは異なる固有のメソッドを持つ
 */
class MainFrameViewModel : public AbstractViewModel
{
    Q_OBJECT
    Q_PROPERTY(Attributes *attributes READ attributes WRITE setAttributes)
public:
    /**
     * @brief コンストラクター
     * @param screenId 画面 ID
     * @param model Model クラスのポインター
     * @param parent 親オブジェクト
     */
    explicit MainFrameViewModel(ScreenId screenId, LevelCheckerModel *&model, QObject *parent = nullptr);

    /**
     * @brief コンストラクター
     * @param screenId 画面 ID
     * @param model Model クラスのポインター
     * @param controller 画面遷移コントローラー
     * @param parent 親オブジェクト
     */
    explicit MainFrameViewModel(ScreenId screenId, LevelCheckerModel *&model, ScreenTransitionController *_controller,
                                QObject *parent = nullptr);

    /**
     * @brief デストラクター
     */
    ~MainFrameViewModel();

    /**
     * @brief qml と ViewModel の接続
     * @param controller 画面遷移コントローラーのポインター
     * @param factory ViewModel のファクトリーの参照
     * @param popupManager ポップアップ表示管理クラスのポインター
     * @param errorManager エラー管理クラスのポインター
     * @param window アプリケーションを表示するウィンドウ
     */
    void attach(ScreenTransitionController *controller, ViewModelFactory *factory, PopupManager *popupManager, ErrorManager *errorManager,
                QQuickWindow *window);

    Attributes *attributes(void) const
    {
        return _attributes;
    }

    void setAttributes(Attributes *qmlInformation)
    {
        _attributes = qmlInformation;
    }

    /**
     * @brief 操作可能ボタンかどうか判定する
     * @param buttonId ボタンID
     * @return true:操作可能、false:操作不可能
     */
    bool buttonProhibition(ButtonId buttonId);

    /**
     * @brief コンパス有効状態取得
     * @return true：有効、false：無効
     */
    bool isEnabledCompass(void);

    /**
     * @brief バッテリ低下状態取得
     * @return バッテリ低下状態かどうか
     */
    bool isLowBattery() {
        return _isLowBattery;
    }

    /**
     * @brief シングル測定の画面へのポインターを返す
     * @return シングル測定の画面へのポインター
     */
    QObject *singleMeasurementView(void);

    /**
     * @brief マルチ測定の画面へのポインターを返す
     * @return マルチ測定の画面へのポインター
     */
    QObject *multiMeasurementView(void);

    /**
     * @brief ファイルリスト取得
     * @return ファイルリスト
     */
    QVariantList getUserFileList() {
        return _fileList;
    }

    /**
     * @brief バッテリー充電状態を返す
     * @return バッテリー充電状態
     */
    int batteryStatus(void) const;

    /**
     * @brief バッテリーレベルを返す
     * @return バッテリーレベル
     */
    int batteryLevel(void) const;

    void menuoptchange(bool checkopt);

    /**
     * @brief 給電デバッグ用
     */
    void test(void);

signals:
    /**
     * @brief View にコンテンツ画面をロードするよう依頼
     * @param screenType 0: シングル測定 1: マルチ測定 2: その他
     */
    void requestToLoad(QVariant screenType);

    /**
     * @brief View にポップアップを表示するよう依頼
     * @param layoutType ポップアップのレイアウト
     * @param message ポップアップに表示するメッセージ
     * @param leftButtonCaption 左ボタンのキャプション
     * @param rightButtonCaption 右ボタンのキャプション
     * @param centerButtonCaption 中央ボタンのキャプション
     */
    void requestToOpenPopup(QVariant layoutType, QVariant message, QVariant leftButtonCaption, QVariant rightButtonCaption, QVariant centerButtonCaption);

    /**
     * @brief View にポップアップの上に表示するポップアップを表示するよう依頼
     * @param message ポップアップに表示するメッセージ
     * @param leftButtonCaption 左ボタンのキャプション
     * @param rightButtonCaption 右ボタンのキャプション
     */
    void requestToOpenStrongPopup(QVariant message, QVariant leftButtonCaption, QVariant rightButtonCaption);

    /**
     * @brief View にバッテリ低下状態中のポップアップを表示するよう依頼
     */
    void requestToOpenLowBatteryPopup();

    /**
     * @brief View にエラーポップアップを表示するよう依頼
     * @param message ポップアップに表示するメッセージ
     * @param buttonCaption ボタンのキャプション
     */
    void requestToOpenErrorPopup(QVariant message, QVariant buttonCaption);

    /**
     * @brief エラーポップアップを閉じるよう通知
     * @param force エラーリストのクリアー有無
     */
    void requestToCloseErrorPopup(bool clearErrorList = true);

    /**
     * @brief ポップアップからの受諾を通知
     * @param okOnly OK のみの場合 true
     */
    void accept(PopupId popupId, bool okOnly);

    /**
     * @brief ポップアップからの拒否を通知
     */
    void reject(PopupId popupId);

    /**
     * @brief スキャン結果ポップアップからの拒否を通知
     */
    void rejectScanResult();

    /**
     * @brief ポップアップからの選択を通知
     * @param index 選択されたインデックス
     */
    void select(PopupId popupId, int index);

    /**
     * @brief ポップアップ表示中に入力されたボタンのボタン ID を通知
     * @param popupId ポップアップ ID
     * @param buttonId ボタン ID
     * @ボタン入力が画面操作を伴う場合にポップアップの呼び出し元の画面に通知するために使う
     */
    void notifyButtonInput(PopupId popupId, ButtonId buttonId);

    /**
     * @brief 方位角を通知
     * @param azimuth 方位角
     */
    void notifyAzimuth(uint16_t azimuth);

    /**
     * @brief メニューバーの選択内容を通知
     * @param type メニューバーの種類
     * @param rank 階層
     * @param index 選択されたリスト中のインデックス
     * @note ここでの index は ListView の currentIndex であり、表示内容を指示するインデックスではない
     */
    void notifyMenuBarSelected(int type, int rank, int index);

    /**
     * @brief バッテリー使用可能時間を通知
     * @param azimuth 方位角
     */
    void notifyBatteryCharge(uint8_t battery, uint8_t batempH, uint8_t batempM);

    /**
     * @brief メニューバーのカースルのあるボタンを押した状態にする
     * @param rank 階層
     */
    void cursorPressed(QVariant rank);

    /**
     * @brief メニューバーのカーソルを上に移動
     * @param rank 階層
     */
    void cursorMoveUp(QVariant rank);

    /**
     * @brief メニューバーのカーソルを下に移動
     * @param rank 階層
     */
    void cursorMoveDown(QVariant rank);

    /**
     * @brief 入力されたファイル名を通知
     * @param popupId ポップアップの種類
     * @param saveLocation 保存先
     * @param fileName 入力されたファイル名
     */
    void inputFileName(PopupId popupId, SaveLocation saveLocation, QString fileName);

    /**
     * @brief ファイル保存完了を通知
     */
    void notifySaveEnd();

    /**
     * @brief スキャンの実行を通知
     * @note 呼び出し画面でスキャン処理(開始、結果取得、中止)を実施
     */
    void requestToScan(void);

    /**
     * @brief スキャン中止を通知
     */
    void requestToStopScanning(void);

    /**
     * @brief プログレスバーの進捗の値を通知
     * @param value 進捗の値
     */
    void progress(double value);

    /**
     * @brief プログレステキストを通知
     * @param current 現在、ロックされたチャンネルまたはスキャン番号
     * @param flag ロックの有無
     */
    void progressText(QString current, bool flag);

    /**
     * @brief 給電異常を通知
     * @code 故障コード
     * @action 故障時の動作
     * @param message エラーメッセージ
     */
    void notifyPowerSupplyError(uint16_t code, uint8_t action, QString message);

    /**
     * @brief コンパス有効状態変化通知
     * @param enabled true：有効に変化、false：無効に変化
     */
    void notifyChangeCompassEnable(bool enabled);

    /**
     * @brief 測定データ取得中止を依頼
     */
    void requestToStopAcquring(void);

    /**
     * @brief 測定データ取得終了通知
     * @param complete 完了有無
     */
    void acquiringFinished(bool complete);

    /**
     * @brief スタンバイ要求
     * @param mode 0: MAIN AND SUB 1: SUB ONLY
     */
    void requestToStandBy(uint8_t mode);

    /**
     * @brief 給電可能状態の変化通知
     * @param isLowBattery バッテリー 10% 以下で false, 11 % 以上で true
     */
    void allowPowerSupplyChanged(bool isLowBattery);

private:
    QQuickWindow *_window;                                  //!< アプリケーション表示ウィンドウ
    ScreenTransitionController *_controller;                //!< 画面遷移コントローラー
    ViewModelFactory *_factory;                             //!< ViewModel のファクトリー
    QList<ScreenId> _history;                               //!< 画面遷移の履歴
    AbstractViewModel *_currentScreen;                      //!< 現在表示しているコンテンツ画面
    AbstractViewModel *_pastScreen;                         //!< 今まで表示していたコンテンツ画面
    Attributes *_attributes;                                //!< qml と連携する情報
    PopupManager *_popupManager;                            //!< ポップアップ表示管理
    ErrorManager *_errorManager;                            //!< エラー管理
    ButtonProhibitionData _currentScreenButtonProhibition;  //!< 現在表示している画面のボタン禁則情報
    QObject *_menuList;                                     //!< メニューリストを表す qml 部品
    QObject *_machineStatus;                                //!< 機器状態を表す qml 部品
    QObject *_menuBarModel;                                 //!< メニューバーの各階層のデータ
    bool _waitingForPowerOff;                               //!< 自動パワーオフ待ちか否か
    QImage _measurementImage;                               //!< 測定画面のイメージデータ
    QTimer *_shutdownTimer;                                 //!< シャットダウン待ちタイマー
    bool _isLowBattery;                                     //!< バッテリ低下状態かどうか
    QObject *_loader;                                       //!< MainFrame 内の Loader
    QVariantList _fileList;                                 //!< ファイルリスト
    int _batteryStatus;                                     //!< バッテリー状態
    int _batteryLevel;                                      //!< バッテリーレベル
    int _oldBatteryLevel;                                   //!< 以前のバッテリーレベル
    bool _test;                                             //!< 給電デバッグ用

    /**
     * @brief Home 画面表示
     */
    void prepare(void);

private slots:
    /**
     * @brief 画面遷移実行要求受信
     * @param nextId 次に表示する画面の画面 ID
     * @param screenName 画面名
     * @param objectName QML のオブジェクト名
     * @param qmlFileName 次に表示する画面の QML ファイル名
     * @param buttonProhibitionData ボタン禁則情報
     */
    void onTransit(ScreenId nextId, QString screenName, QString objectName, QString qmlFileName,
                   ButtonProhibitionData buttonProhibitionData);

    /**
     * @brief qml からロードされたことを受信
     */
    void onLoaded(void);

    /**
     * @brief ボタンクリックを受信
     * @param buttonId ボタン ID
     */
    void onButtonClicked(ButtonId buttonId);

    /**
     * @brief ポップアップを開く要求を受信
     * @param popupId ポップアップ ID
     */
    void onRequestToShowPopup(PopupId popupId);

    /**
     * @brief ポップアップを開く要求を受信
     * @param popupId ポップアップ ID
     * @param detailId 詳細 ID
     */
    void onRequestToShowPopupScan(PopupId popupId, int detailId);

    /**
     * @brief 指定した内容でポップアップを開く要求を受信
     * @param popupId ポップアップ ID
     * @param message メッセージ内容
     */
    void onRequestToShowPopupWithMessage(PopupId popupId, QString message);

    /**
     * @brief バッテリ低下状態ポップアップを開く要求を受信
     */
    void onRequestToShowLowBatteryPopup();

    /**
     * @brief ポップアップのClose要求を受信
     */
    void onRequestPopupClose(void);

    /**
     * @brief ポップアップが閉じたことを受信
     */
    void onPopupClosed(void);

    /**
     * @brief 測定画面のキャプチャー依頼を受信
     */
    void onRequestToCapture(void);

    /**
     * @brief 測定画面のキャプチャーの保存依頼を受信
     * @param measureType 測定種類
     * @param saveLocation 保存先
     * @param fileName ファイル名
     */
    void onRequestToSaveImage(MeasureType measureType, SaveLocation saveLocation, QString fileName);

    /**
     * @brief キャプチャーイメージの消去依頼を受信
     */
    void onRequestToClearCaptureImage(void);

    /**
     * @brief 測定データの保存依頼を受信(シングルチャンネル測定)
     * @param saveLocation 保存先
     * @param fileName ファイル名
     * @param condition 測定条件
     * @param singleMeasureResult 測定結果
     */
    void onRequestToSaveData(SaveLocation saveLocation, QString fileName, uint16_t condition,
                             MeasureResult singleMeasureResult);

    /**
     * @brief 測定データの保存依頼を受信(マルチチャンネル測定)
     * @param saveLocation 保存先
     * @param measureType 測定の種類
     * @param fileName ファイル名
     * @param condition 測定条件
     * @param multiMeasureResult 測定結果
     */
    void onRequestToSaveDataMulti(SaveLocation saveLocation, MeasureType measureType, QString fileName, uint16_t condition,
                                  QList<MeasureResult> multiMeasureResult);

    /**
     * @brief 方位角を受信
     * @param azimuth 方位角
     */
    void onAzimuth(uint16_t azimuth);

    /**
     * @brief デバイス状態を受信
     * @param deviceStatus デバイス状態
     */
    void onDeviceStatus(LC70_DEV_STAT_T deviceStatus);

    /**
     * @brief 音量画像更新通知を受信
     * @param volumeLevel 音量レベル
     */
    void onChangeSoundVolumeImage(int volumeLevel);

    /**
     * @brief updateMenuList
     *
     * メニューアイコンを更新する
     * 表示できるのは 5 つまで
     *
     * @param menuList 表示するメニューアイコン名のリスト
     */
    void onUpdateMenuList(QVector<QString> *menulist);

    /**
     * @brief 引き継ぎデータの変更要求を受信
     * @param key 引き継ぎデータのキー
     * @param value 引き継ぎデータの値
     */
    void onChangeAttribute(QString key, QVariant value);

    /**
     * @brief 引き継ぎデータの削除要求を受信
     * @param key 引き継ぎデータのキー
     */
    void onRemoveAttribute(QString key);

    /**
     * @brief スタンバイ通知を受信
     */
    void onNotifyStandBy(void);

    /**
     * @brief ウェイクアップ通知を受信
     */
    void onNotifyWakeUp(void);

    /**
     * @brief エラーポップアップ表示要求を受信
     * @param message メッセージ内容
     * @param buttonCaption ボタン名
     */
    void onNotifyErrorPopup(QVariant message, QVariant buttonCaption);

    /**
     * @brief 動作中か否かの変更を受信
     * @param inAction 動作中か否か
     */
    void onNotifyChangeInAction(bool inAction);

    /**
     * @brief 自動パワーオフ時間到来通知を受信
     */
    void onNotifyAutoPowerOff(void);

    /**
     * @brief アップデート中のインジケーターの表示通知を受信
     * @param show 表示/非表示
     */
    void onShowIndicator(bool show);

    /**
     * @brief シャットダウン要求を受信
     */
    void onShutdown();

    /**
     * @brief View の再ロード依頼を受信
     * @param value qml ファイルパス
     * @note 未使用
     */
    void onRequestToReload(QString qmlFile);

    /**
     * @brief 測定停止を受信
     * @param target 測定の種類
     */
    void onNotifyStopMeasuring(AcquisitionTarget target);

private:

    /**
     * @brief updateMenu
     *
     * index で指定されたメニューアイコンを更新する
     *
     * @param index 更新するアイコン名称の添字
     * @param menuList 表示するメニューアイコン名のリスト
     */
    void updateMenu(int index, QVector<QString> *&menuList);
};

#endif // MAINFRAME_H
