/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include "abstractmeasurementviewmodel.h"
#include "mainframeviewmodel.h"
#include "correcter.h"

static const QString TAKEOVER_NAME_MEASUREMENT = "selectedMeasurement";
static const QString TAKEOVER_NAME_BAND = "selectedBand";
static const QString TAKEOVER_NAME_MODULATION = "selectedModulation";
static const QString TAKEOVER_NAME_POLARIZATION = "selectedPolarization";
static const QString TAKEOVER_NAME_LOCALOSCILLATOR = "selectedLocalOscillator";

static constexpr int LTE_BAND_WIDTH = 10000;                        // LTEの帯域幅
static constexpr int CATV_ALL_CENTER_FREQUENCY_ISDBT_SUFFIX = 143;  // CATVの周波数末尾、ISDBTの特徴
static constexpr int CATV_ALL_CENTER_FREQUENCY_MOD = 1000;          // CATVの周波数末尾を取り出す際に周波数を割る値
                                                                    // static_cast<uint32_t>(pow(10, QString::number(CATV_ALL_CENTER_FREQUENCY_ISDBT_SUFFIX).length() -1))

static constexpr const char *CSV_FILE_EXTENSION = ".csv";       //!< 測定データファイルの拡張子(CSV)

static const int CONTENTS_INDEX_END = -1;        // メニュー表示終了
static const int CONTENTS_INDEX_POLA_LR = 0;     // 偏波(BS/CS)
static const int CONTENTS_INDEX_POLA_VH = 1;     // 偏波(その他)
static const int CONTENTS_INDEX_MOD = 2;         // 変調方式(スカパー光以外)
static const int CONTENTS_INDEX_FREQ_JCSAT = 0;  // 局発周波数(JCSAT)
static const int CONTENTS_INDEX_FREQ_SCC = 1;    // 局発周波数(SCC)

AbstractMeasurementViewModel::AbstractMeasurementViewModel(ScreenId screenId, LevelCheckerModel *&model, QObject *parent)
    : AbstractViewModel(screenId, model, parent)
    , _channelList()
    , _currentChannelIndex(0)
    , _localFrequencyList()
    , _changeHistory()
    , _powerSupply(LC70_POWER_SUPPLY_OFF)
    , _selectedFileName()
    , _hasMultiBandsInUserSetting(false)
    , _cancelPowerSupply(false)
    , _isScanning(false)
    , _powerSupplied(false)
{

}

AbstractMeasurementViewModel::~AbstractMeasurementViewModel()
{

}

void AbstractMeasurementViewModel::createConnection(QObject *qml, QObject *mainFrame)
{
    AbstractViewModel::createConnection(qml, mainFrame);
    connect(this, SIGNAL(cursorMoveUp(QVariant)), mainFrame, SIGNAL(cursorMoveUp(QVariant)));
    connect(this, SIGNAL(cursorMoveDown(QVariant)), mainFrame, SIGNAL(cursorMoveDown(QVariant)));
}

void AbstractMeasurementViewModel::activate()
{
    // 給電可・不可の判定
    MainFrameViewModel *view = dynamic_cast<MainFrameViewModel *>(_mainFrame);
    if (view) {
        // バッテリー残量が 11% 以上か充電中、満充電なら給電可
        _allowPowerSupply = (view->batteryLevel() > BATTERY_LEVEL2 || view->batteryStatus() >= BATTERY_STATUS_CHARGING) ? true : false;
    }
}

void AbstractMeasurementViewModel::deactivate()
{
    AbstractViewModel::deactivate();
    disconnect(this, SIGNAL(cursorMoveUp(QVariant)), _mainFrame, SIGNAL(cursorMoveUp(QVariant)));
    disconnect(this, SIGNAL(cursorMoveDown(QVariant)), _mainFrame, SIGNAL(cursorMoveDown(QVariant)));

    _hasMultiBandsInUserSetting = false;
    _cancelPowerSupply = false;
    _powerSupplied = false;
}

void AbstractMeasurementViewModel::onNotifyMenuBarSelected(int type, int rank, int index)
{
    _menuBarType = type;
    _menuBarRank = rank;
    _menuBarIndexMap.insert(rank, index);
}

void AbstractMeasurementViewModel::onLowBatteryStatusChanged(bool isLowBattery)
{
    if (isLowBattery) {
        if (_powerSupply != LC70_POWER_SUPPLY_OFF) {
            // 給電中にバッテリ低下状態になったら、給電を停止
            _powerSupply = LC70_POWER_SUPPLY_OFF;

            // マルチ測定の複数 Band を測定するときとその他で使い分ける
            if (_hasMultiBandsInUserSetting) {
                _model->setPowerSupplyForMultiBand(_powerSupply);
            } else {
                _model->setPowerSupply(_powerSupply);
            }
            // 給電中ではない
            _powerSupplied = false;

            _model->removeMenuList(MenuListIconIndex::POWER_SUPPLY_ICON);
        }

        // スキャンを止める
        if (_isScanning) {
            onRequestToStopScanning();
        }

        emit requestToShowLowBatteryPopup();
    }
}

void AbstractMeasurementViewModel::onRequestToStopScanning()
{

}

void AbstractMeasurementViewModel::selectNextMenuBar(const int &rank, const int &index, bool permissionOfChange)
{
    // 各階層の選択に基づいて保持している設定値を変更し、次の階層を決定する
    int contentsIndex = CONTENTS_INDEX_END;
    if (rank == FIRST_FLOOR) {
        contentsIndex = selectFirstFloor(index);
    } else if (rank == SECOND_FLOOR) {
        contentsIndex = selectSecondFloor(index, permissionOfChange);
    } else if (rank == THIRD_FLOOR) {
        contentsIndex = selectThirdFloor(index, permissionOfChange);
    } else if (rank == FOURTH_FLOOR) {
        contentsIndex = selectFourthFloor(index, permissionOfChange);
    }
    // 次の階層がない場合はここで終わり
    if (contentsIndex < 0) {
        return;
    }
    // 次の階層を表示する
    _menuBarRank = rank + 1;
    _menuBarIndexMap.insert(_menuBarRank, 0);
    emit requestToShowMenuBar(QVariant(_menuBarType), QVariant(_menuBarRank), QVariant(contentsIndex));
}

void AbstractMeasurementViewModel::menuBarOperation(ButtonId buttonId)
{
    switch (buttonId) {
    case BUTTON_UP:
        emit cursorMoveUp(_menuBarRank);
        break;

    case BUTTON_DOWN:
        emit cursorMoveDown(_menuBarRank);
        break;

    case BUTTON_RIGHT: {
        // 1〜3 階層目は次の階層へ移動
        if (MINIMUM_RANK <= _menuBarRank && _menuBarRank < MAXIMUM_RANK) {
            // 選択した階層のインデックスから次のリストの内容を選択して次の階層を開く
            selectNextMenuBar(_menuBarRank, _menuBarIndexMap.value(_menuBarRank));
        }

        break;
    }

    case BUTTON_LEFT: {
        // 2〜4 階層目は前の階層へ移動
        if (MINIMUM_RANK  < _menuBarRank && _menuBarRank <= MAXIMUM_RANK) {
            // 画面遷移履歴を削除し、前の階層に戻る
            closeCurrentMenuBar();
            _changeHistory.removeAt(_menuBarRank);
            --_menuBarRank;
        }

        break;
    }

    case BUTTON_ENTER: {
        if (MINIMUM_RANK <= _menuBarRank && _menuBarRank <= MAXIMUM_RANK) {
            // 選択した階層のインデックスから次のリストの内容を選択して次の階層を開く
            selectNextMenuBar(_menuBarRank, _menuBarIndexMap.value(_menuBarRank), true);
        }

        break;
    }

    case BUTTON_ESCAPE: {
        if (MINIMUM_RANK < _menuBarRank && _menuBarRank <= MAXIMUM_RANK) {
            // 画面遷移履歴を削除し、前の階層に戻る
            closeCurrentMenuBar();
            _changeHistory.removeAt(_menuBarRank);
            --_menuBarRank;

        } else if (_menuBarRank == MINIMUM_RANK) {
            closeAllMenuBar();
        }

        break;
    }
    case BUTTON_HOME: {
        // 表示しているメニューバーを全て閉じる
        closeAllMenuBar();

        // ホーム画面へ遷移する
        emit requestDirect(SCREEN_HOME, true);
        break;
    }

    case BUTTON_USER: {
        // 表示しているメニューバーを全て閉じる
        closeAllMenuBar();

        // ユーザデータロガー画面へ遷移する
#ifdef NEW_USER_SETTING
        emit requestDirect(SCREEN_USER_SETTING, true);
#else
        emit requestDirect(SCREEN_USER_DATA_LOGGER_MENU, true);
#endif
        break;
    }
    default:
        break;
    }
}

void AbstractMeasurementViewModel::changeByMenuBar()
{

}

void AbstractMeasurementViewModel::changeMenuIcon()
{
    // 測定設定(機能)アイコンを更新
    changeMeasurementIcon();
    // 選択BANDのアイコンを更新
    changeBandIcon();
}

bool AbstractMeasurementViewModel::isValidChannel(int channleIndex)
{
    return _channelList[channleIndex].valid;
}

int AbstractMeasurementViewModel::nextChannel()
{
    for (int i = _currentChannelIndex + 1; i < _channelList.count(); ++i) {
        if (!isValidChannel(i)) {
            // 無効なチャンネルは対象外
            continue;
        }
        return i;
    }
    // 見つからなかったら先頭から一番最初に見つかった有効なチャンネルのインデックスを返す
    for (int i = 0; i < _channelList.count(); ++i) {
        if (!isValidChannel(i)) {
            continue;
        }
        return i;
    }
    // 全チャンネル無効
    return -1;
}

int AbstractMeasurementViewModel::previousChannel()
{
    for (int i = _currentChannelIndex - 1; i >= 0; --i) {
        if (!isValidChannel(i)) {
            continue;
        }
        return i;
    }
    // 見つからなかったら末尾から一番最初に見つかった有効なチャンネルのインデックスを返す
    for (int i = _channelList.count() - 1; i >= 0; --i) {
        if (!isValidChannel(i)) {
            continue;
        }
        return i;
    }
    // 全チャンネル無効
    return -1;
}

void AbstractMeasurementViewModel::addSelectBandIcon(int bandId)
{
    // BANDアイコンの置き換えを実施する
    switch (bandId) {
    case LC70_BAND_UHF:  // 地デジ
        _model->replaceMenuList(MenuListIconIndex::BAND_ICON, "icon_band_chidigi.png");
        break;

    case LC70_BAND_JCSAT3:  // JCSAT3
        _model->replaceMenuList(MenuListIconIndex::BAND_ICON, "icon_band_3a.png");
        break;

    case LC70_BAND_SCCB:  // SCC-B
        _model->replaceMenuList(MenuListIconIndex::BAND_ICON, "icon_band_b.png");
        break;

    case LC70_BAND_FM:  // FM
        _model->replaceMenuList(MenuListIconIndex::BAND_ICON, "icon_band_fm.png");
        break;

    case LC70_BAND_BS:  // BS
        _model->replaceMenuList(MenuListIconIndex::BAND_ICON, "icon_band_bs.png");
        break;

    case LC70_BAND_JCSAT4:  // JCSAT4
        _model->replaceMenuList(MenuListIconIndex::BAND_ICON, "icon_band_4b.png");
        break;

    case LC70_BAND_SCCC:  // SCC-C
        _model->replaceMenuList(MenuListIconIndex::BAND_ICON, "icon_band_c.png");
        break;

    case LC70_BAND_CS: // CS
        _model->replaceMenuList(MenuListIconIndex::BAND_ICON, "icon_band_cs.png");
        break;

    case LC70_BAND_JCSAT3 * LC70_BAND_JCSAT4: // JCSATアンテナ調整
        _model->replaceMenuList(MenuListIconIndex::BAND_ICON, "icon_band_antenna ad .png");
        break;

    case LC70_BAND_LTE: // LTE
        _model->replaceMenuList(MenuListIconIndex::BAND_ICON, "icon_band_lte.png");
        break;

    case LC70_BAND_CATV_UP_STREAM: // CATV 上り
        _model->replaceMenuList(MenuListIconIndex::BAND_ICON, "icon_band_catv_up_stream.png");
        break;

    default:
        break;
    }
}

void AbstractMeasurementViewModel::resetScreen()
{

}

int AbstractMeasurementViewModel::selectFirstFloor(const int &index)
{
    if (_menuBarType  == 0) {
        _changeHistory.clear();
        updateTakeOver(TAKEOVER_NAME_MEASUREMENT, QString(index == 1 ? MULTI_MEASURE_TYPE_NAME : SINGLE_MEASURE_TYPE_NAME), true);
        _changeHistory.append(SCREEN_HOME);
    }
    else if (_menuBarType == 1) {
        _changeHistory.clear();
        _changeHistory.append(SCREEN_HOME);
        MainFrameViewModel *mainFrame = static_cast<MainFrameViewModel *>(_mainFrame);
        QVariantList vFileList = mainFrame->getUserFileList();
        if (index < vFileList.count()) {
            QString filename = vFileList[index].toString();
            if ("" == filename) {
                return CONTENTS_INDEX_END;
            } else {
                _selectedFileName = filename + CSV_FILE_EXTENSION;
                return index;
            }
        } else {
            return CONTENTS_INDEX_END;
        }
    }
    return index;
}

int AbstractMeasurementViewModel::selectSecondFloor(const int &index, bool permissionOfChange)
{
    int band;
    if (_menuBarType == 0) {
        if (_takeOver["selectedMeasurement"] == QString(SINGLE_MEASURE_TYPE_NAME)) {
            band = getBandFromMenubarIndexSingle(index);
            QVariant valueopt;
                int rfOptical = 0;
                _model->getValue(VALUE_RF_OPTICAL, valueopt);
                rfOptical = valueopt.toInt();
                if(rfOptical == 1){

                    switch (index) {
                    case MENUBAR_UHF_OPT:
                    case MENUBAR_PILOT_OPT:
                    case MENUBAR_LTE_OPT:
                    case MENUBAR_FM_OPT:
                    case MENUBAR_BSCS_OPT:
                    case MENUBAR_CATV_UP_STREAM_OPT:
                        if (permissionOfChange) {
                            updateTakeOver(TAKEOVER_NAME_BAND, band, true);
                            removeTakeOver(TAKEOVER_NAME_MODULATION);
                            removeTakeOver(TAKEOVER_NAME_POLARIZATION);
                            removeTakeOver(TAKEOVER_NAME_LOCALOSCILLATOR);
                            changeByMenuBar();
                        }
                        return CONTENTS_INDEX_END;
                    case MENUBAR_SKYPERFECTV_OPTICAL_OPT:
                        if (permissionOfChange) {
                            updateTakeOver(TAKEOVER_NAME_BAND, band, true);
                            updateTakeOver(TAKEOVER_NAME_MODULATION, QString(TWO_HUNDRED_FIFTY_SIX_QAM), true);
                            removeTakeOver(TAKEOVER_NAME_POLARIZATION);
                            removeTakeOver(TAKEOVER_NAME_LOCALOSCILLATOR);
                            changeByMenuBar();
                        }
                        return CONTENTS_INDEX_END;
                    case MENUBAR_BS_OPT:
                    case MENUBAR_CS_OPT:
                        updateTakeOver(TAKEOVER_NAME_BAND, band);
                        return CONTENTS_INDEX_POLA_LR;
                    case MENUBAR_JCSAT3A_OPT:
                    case MENUBAR_JCSAT4B_OPT:
                    case MENUBAR_SCCB_OPT:
                    case MENUBAR_SCCC_OPT:
                        updateTakeOver(TAKEOVER_NAME_BAND, band);
                        return CONTENTS_INDEX_POLA_VH;
                    case MENUBAR_CATV_ALL_OPT:
                    case MENUBAR_CATV_90_470_OPT:
                    case MENUBAR_CATV_470_1002_OPT:
                        updateTakeOver(TAKEOVER_NAME_BAND, band);
                        return CONTENTS_INDEX_MOD;
                    default:
                        return CONTENTS_INDEX_END;
                    }

                }else{
                    _view->setProperty("optcheck",false);
                    switch (index) {
                    case MENUBAR_UHF_SINGLE:
                    case MENUBAR_PILOT_SINGLE:
                    case MENUBAR_LTE_SINGLE:
                    case MENUBAR_FM_SINGLE:
                    case MENUBAR_BSCS_SINGLE:
                    case MENUBAR_CATV_UP_STREAM_SINGLE:
                        if (permissionOfChange) {
                            updateTakeOver(TAKEOVER_NAME_BAND, band, true);
                            removeTakeOver(TAKEOVER_NAME_MODULATION);
                            removeTakeOver(TAKEOVER_NAME_POLARIZATION);
                            removeTakeOver(TAKEOVER_NAME_LOCALOSCILLATOR);
                            changeByMenuBar();
                        }
                        return CONTENTS_INDEX_END;
                    case MENUBAR_SKYPERFECTV_OPTICAL_SINGLE:
                        if (permissionOfChange) {
                            updateTakeOver(TAKEOVER_NAME_BAND, band, true);
                            updateTakeOver(TAKEOVER_NAME_MODULATION, QString(TWO_HUNDRED_FIFTY_SIX_QAM), true);
                            removeTakeOver(TAKEOVER_NAME_POLARIZATION);
                            removeTakeOver(TAKEOVER_NAME_LOCALOSCILLATOR);
                            changeByMenuBar();
                        }
                        return CONTENTS_INDEX_END;
                    case MENUBAR_BS_SINGLE:
                    case MENUBAR_CS_SINGLE:
                        updateTakeOver(TAKEOVER_NAME_BAND, band);
                        return CONTENTS_INDEX_POLA_LR;
                    case MENUBAR_JCSAT3A_SINGLE:
                    case MENUBAR_JCSAT4B_SINGLE:
                    case MENUBAR_ANTENNA_SINGLE:
                    case MENUBAR_SCCB_SINGLE:
                    case MENUBAR_SCCC_SINGLE:
                        updateTakeOver(TAKEOVER_NAME_BAND, band);
                        return CONTENTS_INDEX_POLA_VH;
                    case MENUBAR_CATV_ALL_SINGLE:
                    case MENUBAR_CATV_90_470_SINGLE:
                    case MENUBAR_CATV_470_1002_SINGLE:
                        updateTakeOver(TAKEOVER_NAME_BAND, band);
                        return CONTENTS_INDEX_MOD;
                    default:
                        return CONTENTS_INDEX_END;
                    }
                }
        } else if (_takeOver["selectedMeasurement"] == QString(MULTI_MEASURE_TYPE_NAME)) {
            band = getBandFromMenubarIndexMulti(index);
            switch (index) {
            case MENUBAR_UHF_MULTI:
            case MENUBAR_PILOT_MULTI:
            case MENUBAR_LTE_MULTI:
            case MENUBAR_FM_MULTI:
            case MENUBAR_BSCS_MULTI:
            case MENUBAR_CATV_UP_STREAM_MULTI:
                if (permissionOfChange) {
                    updateTakeOver(TAKEOVER_NAME_BAND, band, true);
                    removeTakeOver(TAKEOVER_NAME_MODULATION);
                    removeTakeOver(TAKEOVER_NAME_POLARIZATION);
                    removeTakeOver(TAKEOVER_NAME_LOCALOSCILLATOR);
                    changeByMenuBar();
                }
                return CONTENTS_INDEX_END;
            case MENUBAR_SKYPERFECTV_OPTICAL_MULTI:
                if (permissionOfChange) {
                    updateTakeOver(TAKEOVER_NAME_BAND, band, true);
                    updateTakeOver(TAKEOVER_NAME_MODULATION, QString(TWO_HUNDRED_FIFTY_SIX_QAM), true);
                    removeTakeOver(TAKEOVER_NAME_POLARIZATION);
                    removeTakeOver(TAKEOVER_NAME_LOCALOSCILLATOR);
                    changeByMenuBar();
                }
                return CONTENTS_INDEX_END;
            case MENUBAR_BS_MULTI:
            case MENUBAR_CS_MULTI:            
                updateTakeOver(TAKEOVER_NAME_BAND, band);
                return CONTENTS_INDEX_POLA_LR;
            case MENUBAR_JCSAT3A_MULTI:
            case MENUBAR_JCSAT4B_MULTI:
            case MENUBAR_SCCB_MULTI:
            case MENUBAR_SCCC_MULTI:
                updateTakeOver(TAKEOVER_NAME_BAND, band);
                return CONTENTS_INDEX_POLA_VH;
            case MENUBAR_CATV_ALL_MULTI:
            case MENUBAR_CATV_90_470_MULTI:
            case MENUBAR_CATV_470_1002_MULTI:
                updateTakeOver(TAKEOVER_NAME_BAND, band);
                return CONTENTS_INDEX_MOD;
            default:
                return CONTENTS_INDEX_END;
            }
        }
        return CONTENTS_INDEX_END;
    } else if (_menuBarType == 1) {
        if (permissionOfChange) {
            updateTakeOver(TAKEOVER_NAME_BAND, LC70_BAND_UNDEF);
            updateTakeOver(TAKEOVER_NAME_MEASUREMENT, QString(index == 1 ? MULTI_MEASURE_TYPE_NAME : SINGLE_MEASURE_TYPE_NAME), true);

            updateTakeOver("channelEditMode", CHANNEL_EDIT_USER, true);
            updateTakeOver("userSettingFileName", _selectedFileName, true);
            changeByMenuBar();
        }
        return CONTENTS_INDEX_END;
    }
    return CONTENTS_INDEX_END;
}

int AbstractMeasurementViewModel::selectThirdFloor(const int &index, bool permissionOfChange)
{
    bool ok = false;
    int bandId = _takeOver["selectedBand"].toInt(&ok);
    if (!ok) {
        return CONTENTS_INDEX_END;
    }

    QVariant values;
    _localFrequencyList.clear();
    _model->getValue(VALUE_LOCAL_FREQUENCY, values);

    QString polarization = Correcter::polarization(bandId, index);

    if (!polarization.isEmpty()) {
        updateTakeOver(TAKEOVER_NAME_POLARIZATION, polarization);
    }

    switch (bandId) {
    case LC70_BAND_BS:
    case LC70_BAND_CS: {
        if(permissionOfChange) {
            // 偏波面の選択と共に局発周波数情報を更新する
            QList<uint32_t> localFrequency = {};
            if (Correcter::getLocalOscillatorList(values, bandId, _takeOver[TAKEOVER_NAME_POLARIZATION].toString(), localFrequency)) {
                updateTakeOver(TAKEOVER_NAME_LOCALOSCILLATOR, localFrequency.first());
            }
            updateTakeOver(TAKEOVER_NAME_BAND, _takeOver[TAKEOVER_NAME_BAND], true);
            updateTakeOver(TAKEOVER_NAME_POLARIZATION, _takeOver[TAKEOVER_NAME_POLARIZATION], true);
            updateTakeOver(TAKEOVER_NAME_LOCALOSCILLATOR, _takeOver[TAKEOVER_NAME_LOCALOSCILLATOR], true);
            removeTakeOver(TAKEOVER_NAME_MODULATION);
            changeByMenuBar();
        }
        return CONTENTS_INDEX_END;
    }
    case LC70_BAND_JCSAT3:
    case LC70_BAND_JCSAT4:
    case LC70_BAND_JCSAT3 * LC70_BAND_JCSAT4:
        Correcter::getLocalOscillatorList(values, bandId, _takeOver[TAKEOVER_NAME_POLARIZATION].toString(), _localFrequencyList);
        return CONTENTS_INDEX_FREQ_JCSAT;
    case LC70_BAND_SCCB:
    case LC70_BAND_SCCC:
        Correcter::getLocalOscillatorList(values, bandId, _takeOver[TAKEOVER_NAME_POLARIZATION].toString(), _localFrequencyList);
        return CONTENTS_INDEX_FREQ_SCC;
    case LC70_BAND_CATV_ALL:
    case LC70_BAND_CATV_90_470MHZ:
    case LC70_BAND_CATV_470_1002MHZ:
        if (permissionOfChange) {
            if (index == 0) {
                updateTakeOver(TAKEOVER_NAME_MODULATION, QString(OFDM), true);
            } else if (index == 1) {
                updateTakeOver(TAKEOVER_NAME_MODULATION, QString(SIXTY_FOUR_QAM), true);
            } else {
                return CONTENTS_INDEX_END;
            }
            updateTakeOver(TAKEOVER_NAME_BAND, _takeOver[TAKEOVER_NAME_BAND], true);
            removeTakeOver(TAKEOVER_NAME_POLARIZATION);
            removeTakeOver(TAKEOVER_NAME_LOCALOSCILLATOR);
            changeByMenuBar();
        }
        return CONTENTS_INDEX_END;
    default:
        return CONTENTS_INDEX_END;
    }
}

int AbstractMeasurementViewModel::selectFourthFloor(const int &index, bool permissionOfChange)
{
    bool ok = false;
    int bandId = _takeOver["selectedBand"].toInt(&ok);
    if (!ok) {
        return CONTENTS_INDEX_END;
    }

    QString localOscillator;
    switch (bandId) {
    case LC70_BAND_JCSAT3:
    case LC70_BAND_JCSAT4:
    case LC70_BAND_JCSAT3 * LC70_BAND_JCSAT4: {
        if (index == 0) {
            localOscillator = QString("10678000");
        } else if (index == 1) {
            localOscillator = QString("11200000");
        } else if (index == 2) {
            localOscillator = QString("11300000");
        } else {
            return CONTENTS_INDEX_END;
        }
        break;
    }
    case LC70_BAND_SCCB:
    case LC70_BAND_SCCC: {
        if (index == 0) {
            localOscillator = QString("11200000");
        } else if (index == 1) {
            localOscillator = QString("11300000");
        } else {
            return CONTENTS_INDEX_END;
        }
        break;
    }
    default:
        return CONTENTS_INDEX_END;
    }
    if (permissionOfChange) {
        updateTakeOver(TAKEOVER_NAME_BAND, _takeOver[TAKEOVER_NAME_BAND], true);
        updateTakeOver(TAKEOVER_NAME_POLARIZATION, _takeOver[TAKEOVER_NAME_POLARIZATION], true);
        updateTakeOver(TAKEOVER_NAME_LOCALOSCILLATOR, localOscillator, true);
        removeTakeOver(TAKEOVER_NAME_MODULATION);
        changeByMenuBar();
    }
    return CONTENTS_INDEX_END;
}

void AbstractMeasurementViewModel::changeMeasurementIcon()
{
    // 測定設定(機能)のアイコンを更新
    if (_takeOver["selectedMeasurement"] == QString(SINGLE_MEASURE_TYPE_NAME)) {
        // シングル測定設定の場合
        _model->replaceMenuList(MenuListIconIndex::FUNCTION_ICON, "icon_menu_single.png");
    } else if (_takeOver["selectedMeasurement"] == QString(MULTI_MEASURE_TYPE_NAME)) {
        // マルチ測定設定の場合
        _model->replaceMenuList(MenuListIconIndex::FUNCTION_ICON, "icon_menu_multi.png");
    }
}

void AbstractMeasurementViewModel::changeBandIcon()
{
    // 選択BANDのアイコンを更新
    if (_takeOver["selectedBand"] ==  LC70_BAND_UHF) {
        // 地デジの場合
        _model->replaceMenuList(MenuListIconIndex::BAND_ICON, "icon_band_chidigi.png");
    } else if (_takeOver["selectedBand"] == LC70_BAND_BS) {
        // BSの場合
        _model->replaceMenuList(MenuListIconIndex::BAND_ICON, "icon_band_bs.png");
    } else if (_takeOver["selectedBand"] == LC70_BAND_CS) {
        // CSの場合
        _model->replaceMenuList(MenuListIconIndex::BAND_ICON, "icon_band_cs.png");
    } else if (_takeOver["selectedBand"] == LC70_BAND_JCSAT3) {
        // JCSAT3の場合
        _model->replaceMenuList(MenuListIconIndex::BAND_ICON, "icon_band_3a.png");
    } else if (_takeOver["selectedBand"] == LC70_BAND_JCSAT4) {
        // JCSAT4の場合
        _model->replaceMenuList(MenuListIconIndex::BAND_ICON, "icon_band_4b.png");
    } else if (_takeOver["selectedBand"] == LC70_BAND_JCSAT3 * LC70_BAND_JCSAT4) {
        // JCSATアンテナ調整の場合
        _model->replaceMenuList(MenuListIconIndex::BAND_ICON, "icon_band_antenna ad .png");
    } else if (_takeOver["selectedBand"] == LC70_BAND_SCCB) {
        // SCC-Bの場合
        _model->replaceMenuList(MenuListIconIndex::BAND_ICON, "icon_band_b.png");
    } else if (_takeOver["selectedBand"] == LC70_BAND_SCCC) {
        // SCC-Cの場合
        _model->replaceMenuList(MenuListIconIndex::BAND_ICON, "icon_band_c.png");
    } else if (_takeOver["selectedBand"] == LC70_BAND_CATV_ALL) {   // ラッパーライブラリ V2.00 対応
        // CATVの場合
        _model->replaceMenuList(MenuListIconIndex::BAND_ICON, "icon_band_catv_all.png");
    } else if (_takeOver["selectedBand"] == LC70_BAND_CATV_90_470MHZ) { // ラッパーライブラリ V2.00 対応
        // CATV_VHFの場合

        _model->replaceMenuList(MenuListIconIndex::BAND_ICON, "icon_band_catv_90-470.png");
    } else if (_takeOver["selectedBand"] == LC70_BAND_CATV_470_1002MHZ) {   // ラッパーライブラリ V2.00 対応
        // CATV_UHFの場合
        _model->replaceMenuList(MenuListIconIndex::BAND_ICON, "icon_band_catv_470-1000.png");
    } else if (_takeOver["selectedBand"] == LC70_BAND_PILOT) {
        // PILOTの場合
        _model->replaceMenuList(MenuListIconIndex::BAND_ICON, "icon_band_catv_pilot.png");
    } else if (_takeOver["selectedBand"] == LC70_BAND_SKYP) {
        // スカパー光の場合
        _model->replaceMenuList(MenuListIconIndex::BAND_ICON, "icon_band_catv_sky.png");
    } else if (_takeOver["selectedBand"] == LC70_BAND_LTE) {
        // LTEの場合
        _model->replaceMenuList(MenuListIconIndex::BAND_ICON, "icon_band_lte.png");
    } else if (_takeOver["selectedBand"] == LC70_BAND_FM) {
        // FMの場合
        _model->replaceMenuList(MenuListIconIndex::BAND_ICON, "icon_band_fm.png");
    } else if (_takeOver["selectedBand"] == LC70_BAND_CATV_UP_STREAM){
        // CATV上りの場合
        _model->replaceMenuList(MenuListIconIndex::BAND_ICON, "icon_band_catv_up_stream.png");
    } else {
        // 何もしない
    }
}

void AbstractMeasurementViewModel::updateTakeOver(const QString name, const QVariant value, bool withEvent)
{
    _takeOver[name] = value;
    if (withEvent) {
        emit changeAttribute(name, value);
    }
}

void AbstractMeasurementViewModel::removeTakeOver(const QString name)
{
    emit removeAttribute(name);
    _takeOver.remove(name);
}

int AbstractMeasurementViewModel::getBandFromMenubarIndexSingle(const int &index)
{

    QVariant valueopt;
    int rfOptical = 0;
    _model->getValue(VALUE_RF_OPTICAL, valueopt);
    rfOptical = valueopt.toInt();
    if(rfOptical == 1){

        switch (index) {
        case MENUBAR_UHF_OPT:
            return LC70_BAND_UHF;
        case MENUBAR_PILOT_OPT:
            return LC70_BAND_PILOT;
        case MENUBAR_LTE_OPT:
            return LC70_BAND_LTE;
        case MENUBAR_BS_OPT:
            return LC70_BAND_BS;
        case MENUBAR_CS_OPT:
            return LC70_BAND_CS;
        case MENUBAR_JCSAT3A_OPT:
            return LC70_BAND_JCSAT3;
        case MENUBAR_JCSAT4B_OPT:
            return LC70_BAND_JCSAT4;
        case MENUBAR_SCCB_OPT:
            return LC70_BAND_SCCB;
        case MENUBAR_SCCC_OPT:
            return LC70_BAND_SCCC;
        case MENUBAR_CATV_ALL_OPT:
            return LC70_BAND_CATV_ALL;
        case MENUBAR_CATV_90_470_OPT:
            return LC70_BAND_CATV_90_470MHZ;
        case MENUBAR_CATV_470_1002_OPT:
            return LC70_BAND_CATV_470_1002MHZ;
        case MENUBAR_SKYPERFECTV_OPTICAL_OPT:
            return LC70_BAND_SKYP;
        case MENUBAR_FM_OPT:
            return LC70_BAND_FM;
        case MENUBAR_CATV_UP_STREAM_OPT:
            return LC70_BAND_CATV_UP_STREAM;
        case MENUBAR_BSCS_OPT:
            return LC70_BAND_BS_CS;
        default:
            return LC70_BAND_UNDEF;
        }

    }else{

        switch (index) {
        case MENUBAR_UHF_SINGLE:
            return LC70_BAND_UHF;
        case MENUBAR_PILOT_SINGLE:
            return LC70_BAND_PILOT;
        case MENUBAR_LTE_SINGLE:
            return LC70_BAND_LTE;
        case MENUBAR_BS_SINGLE:
            return LC70_BAND_BS;
        case MENUBAR_CS_SINGLE:
            return LC70_BAND_CS;
        case MENUBAR_JCSAT3A_SINGLE:
            return LC70_BAND_JCSAT3;
        case MENUBAR_JCSAT4B_SINGLE:
            return LC70_BAND_JCSAT4;
        case MENUBAR_ANTENNA_SINGLE:
            return LC70_BAND_JCSAT3 * LC70_BAND_JCSAT4;
        case MENUBAR_SCCB_SINGLE:
            return LC70_BAND_SCCB;
        case MENUBAR_SCCC_SINGLE:
            return LC70_BAND_SCCC;
        case MENUBAR_CATV_ALL_SINGLE:
            return LC70_BAND_CATV_ALL;
        case MENUBAR_CATV_90_470_SINGLE:
            return LC70_BAND_CATV_90_470MHZ;
        case MENUBAR_CATV_470_1002_SINGLE:
            return LC70_BAND_CATV_470_1002MHZ;
        case MENUBAR_SKYPERFECTV_OPTICAL_SINGLE:
            return LC70_BAND_SKYP;
        case MENUBAR_FM_SINGLE:
            return LC70_BAND_FM;
        case MENUBAR_CATV_UP_STREAM_SINGLE:
            return LC70_BAND_CATV_UP_STREAM;
        case MENUBAR_BSCS_SINGLE:
            return LC70_BAND_BS_CS;
        default:
            return LC70_BAND_UNDEF;
        }
    }
}

int AbstractMeasurementViewModel::getBandFromMenubarIndexMulti(const int &index)
{
    switch (index) {
    case MENUBAR_UHF_MULTI:
        return LC70_BAND_UHF;
    case MENUBAR_PILOT_MULTI:
        return LC70_BAND_PILOT;
    case MENUBAR_LTE_MULTI:
        return LC70_BAND_LTE;
    case MENUBAR_BS_MULTI:
        return LC70_BAND_BS;
    case MENUBAR_CS_MULTI:
        return LC70_BAND_CS;
    case MENUBAR_JCSAT3A_MULTI:
        return LC70_BAND_JCSAT3;
    case MENUBAR_JCSAT4B_MULTI:
        return LC70_BAND_JCSAT4;
    case MENUBAR_SCCB_MULTI:
        return LC70_BAND_SCCB;
    case MENUBAR_SCCC_MULTI:
        return LC70_BAND_SCCC;
    case MENUBAR_CATV_ALL_MULTI:
        return LC70_BAND_CATV_ALL;
    case MENUBAR_CATV_90_470_MULTI:
        return LC70_BAND_CATV_90_470MHZ;
    case MENUBAR_CATV_470_1002_MULTI:
        return LC70_BAND_CATV_470_1002MHZ;
    case MENUBAR_SKYPERFECTV_OPTICAL_MULTI:
        return LC70_BAND_SKYP;
    case MENUBAR_FM_MULTI:
        return LC70_BAND_FM;
    case MENUBAR_CATV_UP_STREAM_MULTI:
        return LC70_BAND_CATV_UP_STREAM;
    case MENUBAR_BSCS_MULTI:
        return LC70_BAND_BS_CS;
    default:
        return LC70_BAND_UNDEF;
    }
}

bool AbstractMeasurementViewModel::createUserSettingChannelList(QVector<LC70_LOGGER_SET_LOGGER_ROW_T> &userSettingList)
{
    // 未定義はtrue扱い
    static const QMap<LC70_BAND_T, bool> merBerMap = {
        {LC70_BAND_PILOT,              false},
        {LC70_BAND_LTE,                false},
        {LC70_BAND_FM,                 false},
        {LC70_BAND_CATV_UP_STREAM,     false},
    };

    // 未定義はLC70_BD_SYS_NONE扱い
    static const QMap<LC70_BAND_T, int> modulationMap = {
        {LC70_BAND_PILOT,              LC70_SETMOD_CW},
        {LC70_BAND_LTE,                LC70_SETMOD_UNDEF},
        {LC70_BAND_FM,                 LC70_SETMOD_FM},
        {LC70_BAND_CATV_UP_STREAM,     LC70_SETMOD_CW},
    };

    // 画面間共有情報から読み込み対象の情報を取得
    ChannelEditMode mode = static_cast<ChannelEditMode>(_takeOver["channelEditMode"].toInt());
    QString fileName = _takeOver["userSettingFileName"].toString();
    if (!fileName.contains(CSV_FILE_EXTENSION)) {
        fileName += QString(CSV_FILE_EXTENSION);
    }

    if (!_model->importSettingFile(mode, fileName)) {
        qDebug() << __PRETTY_FUNCTION__ << ", importSettingFile Error. mode: " << mode << " , fileName: " << fileName;
        return false;
    }

    // 読み込んだファイルの内容を取得
    QList<UserDataLoggerSettingFile> settingFileList = _model->selectMeasuringChannels();

    userSettingList.clear();
    _channelList.clear();

    QVariant broadBandModulationSetting;
    _model->getValue(VALUE_BROADBANDSYSTEM_MODULATION, broadBandModulationSetting);

    for (int i = 0; i < settingFileList.size(); ++i ) {
        LC70_BAND_T bandId = Correcter::convertBandId(settingFileList.at(i).band).first;
        int polarization = Correcter::convertBandId(settingFileList.at(i).band).second;

        if (bandId == LC70_BAND_UNDEF) {
            qDebug() << __PRETTY_FUNCTION__ << ", convertBandId Error. band: " << Correcter::convertBandId(settingFileList.at(i).band);
            continue;
        }
        LC70_LOGGER_SET_LOGGER_ROW_T row;
        memset(&row, 0, sizeof(LC70_LOGGER_SET_LOGGER_ROW_T));
        row.ucEnbFlg = 1;
        row.ucNo = i;
        row.ucBand = bandId;
        row.ucSpec = settingFileList.at(i).broadcastsystem;
        strncpy(row.cStrChNo, settingFileList.at(i).channel.toUtf8().constData(), sizeof(row.cStrChNo));
#if 1
        row.ulCenterFreq = settingFileList.at(i).frequency; // 実周波数をそのまま送ること(局発周波数を引かない)
#else
        row.ulCenterFreq = settingFileList.at(i).frequency - settingFileList.at(i).localFrequency;
#endif
        row.ulLocalFreq = settingFileList.at(i).localFrequency;
        row.ucGetDataFlg = merBerMap.value(bandId, true) ? LC70_DATA_MSK_CNR : LC70_DATA_MSK_NONE;
        if (bandId == LC70_BAND_UHF || bandId == LC70_BAND_CATV_ALL || bandId == LC70_BAND_SKYP) {
            row.ucGetDataFlg |= LC70_DATA_MSK_MER;
        }

        row.ucGetDataFlg |= merBerMap.value(bandId, true) ? LC70_DATA_MSK_BER : LC70_DATA_MSK_NONE;
        row.ucDcOut = settingFileList.at(i).powerSupply;

        ChannelDefinitionItem channelDefinitionItem;
        channelDefinitionItem.band = bandId;
        channelDefinitionItem.channelNumber = settingFileList.at(i).channel;
        channelDefinitionItem.polarization = polarization;
        channelDefinitionItem.valid = true;

        if (bandId == LC70_BAND_CATV_ALL) {
            // CATV ALL は周波数の末尾の特徴から中心周波数などの情報を特定する
            uint32_t localFrequencySuffix = settingFileList.at(i).frequency % CATV_ALL_CENTER_FREQUENCY_MOD;

            if (localFrequencySuffix == CATV_ALL_CENTER_FREQUENCY_ISDBT_SUFFIX) {
                channelDefinitionItem.centerFrequency1 = settingFileList.at(i).frequency - CATV_ALL_CENTER_FREQUENCY_ISDBT_SUFFIX;
                channelDefinitionItem.centerFrequency2 = settingFileList.at(i).frequency;
                row.ulModBit = modulationMap.value(bandId, LC70_SETMOD_OFDM);
            } else {
                channelDefinitionItem.centerFrequency1 = settingFileList.at(i).frequency;
                channelDefinitionItem.centerFrequency2 = settingFileList.at(i).frequency + CATV_ALL_CENTER_FREQUENCY_ISDBT_SUFFIX;
                row.ulModBit = modulationMap.value(bandId, LC70_SETMOD_64QAM);
            }
        } else {
            channelDefinitionItem.centerFrequency1 = settingFileList.at(i).frequency;
            channelDefinitionItem.centerFrequency2 = -1;

            //            QList<int> broadbandSystemList;
            //            _model->getBroadbandSystem(bandId, broadbandSystemList);
            //            if (broadbandSystemList.isEmpty()) {
            //                continue;
            //            }
            //            int broadbandSystem = broadbandSystemList.at(0);
            //            unsigned int modulationBit = _model->getModulationBit(bandId, broadbandSystem);

            // Band 毎の変調方式の特定
            unsigned int modulation = Correcter::selectModulationMethod(settingFileList[i].band);

            // 放送方式、変調方式の設定(定義ファイル 1.0.1 対応)
            uint16_t broadcastSystem = settingFileList[i].broadcastsystem;
            uint32_t modulationBit = Correcter::getModulationCondition(channelDefinitionItem.band, broadcastSystem, channelDefinitionItem.polarization, modulation, broadBandModulationSetting);

            row.ulModBit = modulationBit;
        }

        if (bandId == LC70_BAND_LTE) {
            // LTEは帯域幅を設定する
            channelDefinitionItem.bandWidth = LTE_BAND_WIDTH;
        } else {
            channelDefinitionItem.bandWidth = -1;
        }

        // ユーザー設定追加
        userSettingList.append(static_cast<LC70_LOGGER_SET_LOGGER_ROW_T>(row));
        // チャンネル定義追加
        _channelList.append(channelDefinitionItem);
    }

    // 作成に成功したチャンネル情報が1件もなければエラーにする
    return !_channelList.isEmpty();
}

void AbstractMeasurementViewModel::abortUserSettingMeasurement()
{
    updateTakeOver("fromUserSetting", false, true);
    updateTakeOver("channelEditMode", "", true);
    updateTakeOver("userSettingFileName", "", true);
    emit requestDirect(SCREEN_HOME, true);
}

void AbstractMeasurementViewModel::changePowerSupplyStatus(bool allowPowerSupply)
{
    // 給電可であれば何もしない
    if (allowPowerSupply) {
        _cancelPowerSupply = false;
        return;
    }

    // 既に給電停止していたら何もしない
    if (_cancelPowerSupply) {
        return;
    }

    // バッテリー状態の変化に伴う給電の停止
    _cancelPowerSupply = true;

    if (_powerSupplied) {
        // 給電中にバッテリ低下状態になったら、給電を停止
        _powerSupply = LC70_POWER_SUPPLY_OFF;

        // マルチ測定の複数 Band を測定するときとその他で使い分ける
        if (_hasMultiBandsInUserSetting) {
            _model->setPowerSupplyForMultiBand(_powerSupply);
        } else {
            _model->setPowerSupply(_powerSupply);
        }

        // ポップアップを表示
        emit requestToShowPopup(POPUP_DISABLE_POWER_SUPPLY);
    }

    _model->removeMenuList(MenuListIconIndex::POWER_SUPPLY_ICON);

    // 給電中ではない
    _powerSupplied = false;

    // スキャンを止める(ローバッテリーのときは onLowBatteryStatusChanged( でスキャンを止めている)
    if (_isScanning) {
        onRequestToStopScanning();
    }
}
