/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef ABSTRACTMEASUREMENTVIEWMODEL_H
#define ABSTRACTMEASUREMENTVIEWMODEL_H

#include "abstractviewmodel.h"
#include "mainframeviewmodel.h"

#define NO_FILE_NAME " "

/**
 * @brief 測定画面 ViewModel のベースクラス
 */
class AbstractMeasurementViewModel : public AbstractViewModel
{
    Q_OBJECT
public:
    /**
     * @brief コンストラクター
     * @param screenId 画面 ID
     * @param model Model クラスのポインター
     * @param parent 親オブジェクト
     */
    explicit AbstractMeasurementViewModel(ScreenId screenId, LevelCheckerModel *&model, QObject *parent = nullptr);

    /**
     * @brief デストラクタ
     */
    virtual ~AbstractMeasurementViewModel();

    /**
     * @brief 画面を開くときに呼び出す qml との接続処理
     * @param qml qml のオブジェクトのポインター
     * @param mainFrame MainFrameViewModel のポインター
     */
    virtual void createConnection(QObject *qml, QObject *mainFrame) override;

    /**
     * @brief 画面を開くときに呼び出す表示内容の復元処理
     */
    virtual void activate(void) override;

    /**
     * @brief 画面を閉じるときに呼び出す表示な用の保存処理
     */
    virtual void deactivate(void) override;

signals:
    /**
     * @brief メニューバーのカーソルを上に移動
     * @param rank 階層
     */
    void cursorMoveUp(QVariant rank);

    /**
     * @brief メニューバーのカーソルを下に移動
     * @param rank 階層
     */
    void cursorMoveDown(QVariant rank);

protected slots:
    /**
     * @brief メニューバーの選択内容を受信
     * @param type メニューバーの種類
     * @param rank 階層
     * @param index 選択されたリスト中のインデックス
     * @note ここでの index は ListView の currentIndex であり、表示内容を指示するインデックスではない
     */
    virtual void onNotifyMenuBarSelected(int type, int rank, int index) override;

    /**
     * @brief バッテリ低下状態変化通知
     * @param isLowBattery バッテリ低下状態かどうか
     */
    virtual void onLowBatteryStatusChanged(bool isLowBattery) override;

    /**
     * @brief スキャン中止依頼を受信
     */
    virtual void onRequestToStopScanning(void);

protected:
    /**
     * @brief 次の階層のメニューバーを開く
     * @param rank 現在の階層
     * @param index 現在の階層の選択したインデックス
     * @param permissionOfChange 変更許可
     */
    void selectNextMenuBar(const int &rank, const int &index, bool permissionOfChange = false);

    /**
     * @brief メニューバーの操作
     * @param buttonId ボタン ID
     */
    void menuBarOperation(ButtonId buttonId);

    /**
     * @brief メニューバーによる設定変更
     * @note スレッドは止めずに変更した設定で表示し直す
     */
    virtual void changeByMenuBar(void);

    /**
     * @brief メニューアイコンの変更を実施する
     */
    void changeMenuIcon(void);

    /**
     * @brief 有効なチャンネルであるか判定する
     * @param channleIndex チャンネルインデックス
     * @return true:有効、false:無効
     */
    virtual bool isValidChannel(int channleIndex);

    /**
     * @brief 次の有効なチャンネルのインデックスを返す
     * @return 次の有効なチャンネルのインデックス
     */
    int nextChannel(void);

    /**
     * @brief 一つ前の有効なチャンネルのインデックスを返す
     * @return 一つ前の有効なチャンネルのインデックス
     */
    int previousChannel(void);

    /**
     * @brief 選択した BAND のアイコンを追加する
     * @param bandId Band ID
     */
    void addSelectBandIcon(int bandId);

    /**
     * @brief 初期状態の画面表示に戻す
     */
    virtual void resetScreen(void);

    /**
     * @brief ユーザー設定チャンネルリスト作成
     * @param userSettingList ユーザー設定
     * @return true：成功、false：失敗
     */
    bool createUserSettingChannelList(QVector<LC70_LOGGER_SET_LOGGER_ROW_T> &userSettingList);

    /**
     * @brief ユーザー設定測定失敗時処理
     */
    void abortUserSettingMeasurement(void);

    /**
     * @brief 給電処理の変更
     * @param allowPowerSupply バッテリー 10% 以下で false, 11 % 以上で true
     * @note _allowPowerSupply の値を参照して給電処理を変更する
     */
    virtual void changePowerSupplyStatus(bool allowPowerSupply);

    QList<ChannelDefinitionItem> _channelList;      //!< チャンネル定義情報のリスト
    int _currentChannelIndex;                       //!< チャンネル定義情報のリストのインデックス
    QList<uint32_t> _localFrequencyList;            //!< 局発周波数リスト
    QList<ScreenId> _changeHistory;                 //!< 変更する画面遷移履歴
    LC70_POWER_SUPPLY_T _powerSupply;               //!< 給電状態
    QString _selectedFileName;                      //!< メニューで選択したファイル名
    bool _hasMultiBandsInUserSetting;               //!< USER 設定で複数の Band を跨るか否か
    bool _cancelPowerSupply;                        //!< 給電キャンセル(ローバッテリー時)
    bool _isScanning;                               //!< SCAN中フラグ
    bool _powerSupplied;                            //!< 給電中か否か

private:
    /**
     * @brief 第 １ 階層の選択
     * @param index 第 １ 階層の選択したインデックス
     * @return 次に表示する階層の内容を示すインデックス(なければ -1)
     */
    int selectFirstFloor(const int &index);

    /**
     * @brief 第 2 階層の選択
     * @param index 第 2 階層の選択したインデックス
     * @param permissionOfChange 変更許可
     * @return 次に表示する階層の内容を示すインデックス(なければ -1)
     */
    int selectSecondFloor(const int &index, bool permissionOfChange = false);

    /**
     * @brief 第 3 階層の選択
     * @param index 第 3 階層の選択したインデックス
     * @param permissionOfChange 変更許可
     * @return 次に表示する階層の内容を示すインデックス(なければ -1)
     */
    int selectThirdFloor(const int &index, bool permissionOfChange = false);

    /**
     * @brief 第 4 階層の選択
     * @param index 第 4 階層の選択したインデックス
     * @param permissionOfChange 変更許可
     * @return 次に表示する階層の内容を示すインデックス(なければ -1)
     */
    int selectFourthFloor(const int &index, bool permissionOfChange = false);

    /**
     * @brief 測定設定のアイコンを更新
     */
    void changeMeasurementIcon(void);

    /**
     * @brief 選択BANDのアイコンを更新
     */
    void changeBandIcon(void);

    /**
     * @brief 画面引き継ぎデータの更新
     * @param name データ名
     * @param value データ値
     * @param withEvent 変化イベントを発火するかどうか
     */
    void updateTakeOver(const QString name, const QVariant value, bool withEvent = false);

    /**
     * @brief 画面引き継ぎデータの更新
     * @param name データ名
     */
    void removeTakeOver(const QString name);

    /**
     * @brief サイドメニューのインデックスからバンド種別を取得（シングル測定）
     * @param index インデックス
     * @return バンド種別（LC70_BAND_T）
     */
    int getBandFromMenubarIndexSingle(const int &index);

    /**
     * @brief サイドメニューのインデックスからバンド種別を取得（マルチ測定）
     * @param index インデックス
     * @return バンド種別（LC70_BAND_T）
     */
    int getBandFromMenubarIndexMulti(const int &index);
};

#endif // ABSTRACTMEASUREMENTVIEWMODEL_H
