/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include "antennaadjustment.h"
#include "mainframeviewmodel.h"
#include <QElapsedTimer>

// 水平(JCSAT-3A と JCSAT-4B で共通なチャンネル 定義ファイル_1.0.8 参照)
static constexpr const char *TARGET_CHANNELS_HORIZONTAL[] = {"JD2", "JD4", "JD6", "JD8", "JD10", "JD12", "JD14", "JD16"};
// 垂直(JCSAT-3A と JCSAT-4B で共通なチャンネル 定義ファイル_1.0.8 参照)
static constexpr const char *TARGET_CHANNELS_VERTICAL[] = {"JD1", "JD3", "JD5", "JD7", "JD9", "JD11", "JD13", "JD15"};

static constexpr float32_t LEVEL_NEEDLE_OFFSET = -128.0;        //!< 測定入力レベルの針の位置のオフセット
static constexpr float32_t LEVEL_NEEDLE_STEP = 2.51;          //!< 測定入力レベルの針の移動量

static constexpr float MINIMUM_NEEDLE_POSITION = -128.0;        //!< 針の位置の最小値
static constexpr float MAXIMUM_NEEDLE_POSITION = 122.0;         //!< 針の位置の最大値

static constexpr int POWER_SUPPLY_INDEX =
    3;        //!< 給電 を表示するメニューリストのインデックス

static constexpr int NUMBER_OF_CHANNELS =
    2;    //!< JCSAT-3A と JCSAT-4B の同じチャンネルを測定するので数は 2

static constexpr const char *IMAGE_BUTTON_NAME =
    "imageButton"; //!< 画像保存ボタン名(AntennaAdjustment.qml 参照)

static constexpr const char *PNG_FILE_EXTENSION = ".png";       //!< 測定画面の拡張子(PNG)

static QElapsedTimer timer;

AntennaAdjustment::AntennaAdjustment(ScreenId screenId, LevelCheckerModel *&model, QObject *parent)
    : AbstractMeasurementViewModel(screenId, model, parent)
    , _condition()
    , _previousCondition()
    , _broadBandSystem(LC70_BD_SYS_DVBS)
    , _isChannelChanging(false)
    , _thresholdSetJCSAT3A(false)
    , _thresholdSetJCSAT4B(false)
    , _bandIndex(JCSAT3A_INDEX)
    , _isLoaded(false)
    , _cnrMerSetting()
    , _alreadySuspended(false)
    , _powerSupplyType(POWER_SUPPLY_COUNT)
    , _isMeasuring(false)
    , _actionInFailure(LC70_FAIL_CTL_NONE)
    , _jcsat3aLevel(UNDEFINED_VALUE)
    , _newestJcsat3aLevel(UNDEFINED_VALUE)
    , _jcsat4bLevel(UNDEFINED_VALUE)
    , _newestJcsat4bLevel(UNDEFINED_VALUE)
    , _isUpdateMeter(false)
    , _meterUpdateInterval(0)
    , _invalidValueMeterSpeed(0)
{
    setObjectName("antennaAdjustmentViewModel");
}

AntennaAdjustment::~AntennaAdjustment()
{

}

void AntennaAdjustment::resume()
{
    qDebug() << QString("[Enter] AntennaAdjustmentViewModel::resume()");
    AbstractViewModel::resume();

    // CNR/MER 切替設定の取得
    _cnrMerSetting.clear();
    _model->getValue(VALUE_MER_CN, _cnrMerSetting);

    // 画面遷移時に実行する
    // CN/MERの選択状態をqmlに送信する
    int valueMerCN = Correcter::selectCnrMer(LC70_BAND_JCSAT3, _cnrMerSetting);

    // 得られた値が範囲外であった場合(0,1 以外の値であった場合)
    // 初期値のMERを選択する
    if (valueMerCN != LC70_MEAS_SET_SEL_CNR && valueMerCN != LC70_MEAS_SET_SEL_CNR) {
        valueMerCN = LC70_MEAS_SET_SEL_MER;
    }

    if (_view) {
        _view->setProperty("selectCnMer", valueMerCN);
    }
    // チャンネル変更
    changeChannel();

    // 画面ロード後にする
    _isLoaded = true;

    qDebug() << QString("[Leave] AntennaAdjustmentViewModel::resume()");
}

void AntennaAdjustment::suspend()
{
    qDebug() << QString("[Enter] AntennaAdjustmentViewModel::suspend()");

    // スレッドを停止(既に停止していたら呼ばない。遷移先画面種別を測定画面以外に変更)
    if (!_alreadySuspended) {
        emit requestToSuspendAntennaMeasuring();
    }
    _alreadySuspended = false;

    AbstractViewModel::suspend();

    // 給電を停止
    _powerSupply = LC70_POWER_SUPPLY_OFF;
    _model->removeMenuList(MenuListIconIndex::POWER_SUPPLY_ICON);
    _powerSupplyType = POWER_SUPPLY_COUNT;

    // メニューバーをクリアー
    for (int i = _menuBarRank; i <= MINIMUM_RANK; --i) {
        if (MINIMUM_RANK < i && i <= MAXIMUM_RANK) {
            // 前の階層に戻る
            closeCurrentMenuBar();

            // 画面遷移履歴の削除
            _changeHistory.removeAt(_menuBarRank);

            --_menuBarRank;
        } else if (i == MINIMUM_RANK) {
            closeAllMenuBar();
            break;
        }
    }

    // 測定条件のクリアー
    for (int i = 0; i < _channelList.count(); ++i) {
        _condition.sRegNum = 0;
        _previousCondition.sRegNum = 0;
        memset(&_condition.single[i], 0x00, sizeof(_condition.single[i]));
        memset(&_previousCondition.single[i], 0x00, sizeof(_previousCondition.single[i]));
    }

    // JCSAT-3A を指す
    _bandIndex = JCSAT3A_INDEX;

    // チャンネル情報のクリアー
    _channelList.clear();

    // 局発周波数のリストをクリアー
    _localFrequencyList.clear();

    // 画面ロード前にする
    _isLoaded = false;

    // 測定中にない
    _isMeasuring = false;

    // レベルメーターの設置値をクリアー
    _jcsat3aLevel = 0;
    _newestJcsat3aLevel = 0;
    _jcsat4bLevel = 0;
    _newestJcsat4bLevel = 0;
    _isUpdateMeter = false;

    qDebug() << QString("[Leave] AntennaAdjustmentViewModel::suspend()");
    _model->writeLog(QString("アンテナ調整"), QString("アンテナ調整終了"));
}

void AntennaAdjustment::createConnection(QObject *qml, QObject *mainFrame)
{
    AbstractMeasurementViewModel::createConnection(qml, mainFrame);

    connect(this, SIGNAL(requestSetTakeOver(QVariant, QVariant)), _view, SLOT(backup(QVariant, QVariant)));
    connect(this, SIGNAL(resumeAntennaMeasuring(LC70_MEAS_MULTI_T *, uint32_t)), _model,
            SLOT(onResumeAntennaMeasuring(LC70_MEAS_MULTI_T *, uint32_t)));
    connect(_model, SIGNAL(antennaMeasureResult(MultiResult)), this,
            SLOT(onAntennaMeasureResult(MultiResult)));
    connect(_model, SIGNAL(notifyChannelChanging(bool)), this, SLOT(onNotifyChannelChanging(bool)));
    connect(mainFrame, SIGNAL(notifyPowerSupplyError(uint16_t, uint8_t, QString)), this, SLOT(detectPowerSupplyError(uint16_t, uint8_t, QString)));
    connect(this, SIGNAL(changeCondition(LC70_MEAS_MULTI_T *)), _model,
            SIGNAL(antennaMeasuringChangeCondition(LC70_MEAS_MULTI_T *)));
    connect(this, SIGNAL(changeAttribute(QString, QVariant)), mainFrame, SLOT(onChangeAttribute(QString, QVariant)));
    connect(this, SIGNAL(cursorPressed(QVariant)), mainFrame, SIGNAL(cursorPressed(QVariant)));
    connect(this, SIGNAL(removeAttribute(QString)), mainFrame, SLOT(onRemoveAttribute(QString)), Qt::DirectConnection);
    connect(this, SIGNAL(requestToCapture()), mainFrame, SLOT(onRequestToCapture()));
    connect(this, SIGNAL(requestToSaveImage(MeasureType, SaveLocation, QString)), mainFrame, SLOT(onRequestToSaveImage(MeasureType, SaveLocation, QString)));
    connect(mainFrame, SIGNAL(inputFileName(PopupId, SaveLocation, QString)), this, SLOT(onInputFileName(PopupId,SaveLocation, QString)));
    connect(this, SIGNAL(requestToSuspendAntennaMeasuring()), _model, SLOT(onSuspendAntennaMeasuring()));
    connect(this, SIGNAL(notifyLevelThreshold(QVariant, QVariant, QVariant)), _view, SLOT(updateLevelThreshold(QVariant, QVariant, QVariant)));
    connect(this, SIGNAL(notifyJcsat3aLevelMeter(QVariant,QVariant)), _view, SLOT(updateJcsat3aLevelMeterValue(QVariant, QVariant)));
    connect(this, SIGNAL(notifyJcsat4bLevelMeter(QVariant,QVariant)), _view, SLOT(updateJcsat4bLevelMeterValue(QVariant, QVariant)));
    connect(this, SIGNAL(requestToClearCaptureImage()), mainFrame, SLOT(onRequestToClearCaptureImage()));
}

void AntennaAdjustment::activate()
{
    AbstractMeasurementViewModel::activate();

    // 画面遷移時は、メニューバーを閉じた状態にする
    setMenuBarVisible(false);

    // QML ファイルのプロパティの変更(AntennaMeasurement.qml を参照)
    // 画面ロード時は初期値に設定する
    clear();

    // 複数の Band を跨る
    _hasMultiBandsInUserSetting = true;

    // JCSAT のチャンネル情報設定(JCSAT-3A と JCSAT-4B で共通の JD-1〜16 を使用する)
    if (_takeOver["selectedPolarization"].toString() == HORIZONTALLY_POLARIZE) {
        int count = sizeof(TARGET_CHANNELS_HORIZONTAL) / sizeof(TARGET_CHANNELS_HORIZONTAL[0]);

        for (int i = 0; i < count; ++i) {
            ChannelDefinitionItem channel;

            if (_model->getChannelDefinitionItemForAntennaAdjustment(LC70_BAND_JCSAT3, QString(TARGET_CHANNELS_HORIZONTAL[i]), channel)) {
                _channelList.append(channel);
            }
        }
    } else if (_takeOver["selectedPolarization"].toString() == VERTICALLY_POLARIZE) {
        int count = sizeof(TARGET_CHANNELS_VERTICAL) / sizeof(TARGET_CHANNELS_VERTICAL[0]);

        for (int i = 0; i < count; ++i) {
            ChannelDefinitionItem channel;

            if (_model->getChannelDefinitionItemForAntennaAdjustment(LC70_BAND_JCSAT3, QString(TARGET_CHANNELS_VERTICAL[i]), channel)) {
                _channelList.append(channel);
            }
        }
    }

    // 画面遷移用にシングル測定を経由してきたことにする
    emit changeAttribute(QString("selectedMeasurement"), QString(SINGLE_MEASURE_TYPE_NAME));

    _currentChannelIndex = 0;

    // スキャン
    _model->replaceMenuList(MenuListIconIndex::SCAN_ICON, "");

    // 給電
    _model->replaceMenuList(MenuListIconIndex::POWER_SUPPLY_ICON, "");
    _powerSupply = LC70_POWER_SUPPLY_OFF;

    // 閾値表示(画面ロード時は放送方式 DVBS で閾値を取得)
    _bandIndex = JCSAT3A_INDEX;
    refreshThreshold(LC70_BAND_JCSAT3);
    _bandIndex = JCSAT4B_INDEX;
    refreshThreshold(LC70_BAND_JCSAT4);
    _bandIndex = JCSAT3A_INDEX;

    // アニメーション設定
    // メーターの更新周期を取得
    _meterUpdateInterval = _model->applicationSetting()->meterUpdateInterval();
    // 入力無効時のメーター針移動倍率を取得
    _invalidValueMeterSpeed = _model->applicationSetting()->invalidValueMeterSpeed();

    // アンテナ調整ではスキャンがないので false 固定
    _isScanning = false;
}

void AntennaAdjustment::deactivate()
{
    AbstractMeasurementViewModel::deactivate();

    disconnect(this, SIGNAL(requestSetTakeOver(QVariant, QVariant)), _view, SLOT(backup(QVariant, QVariant)));
    disconnect(this, SIGNAL(resumeAntennaMeasuring(LC70_MEAS_MULTI_T *, uint32_t)), _model,
               SLOT(onResumeAntennaMeasuring(LC70_MEAS_MULTI_T *, uint32_t)));
    disconnect(_model, SIGNAL(antennaMeasureResult(MultiResult)), this,
               SLOT(onAntennaMeasureResult(MultiResult)));
    disconnect(_model, SIGNAL(notifyChannelChanging(bool)), this, SLOT(onNotifyChannelChanging(bool)));
    disconnect(this, SIGNAL(changeCondition(LC70_MEAS_MULTI_T *)), _model,
            SIGNAL(antennaMeasuringChangeCondition(LC70_MEAS_MULTI_T *)));
    disconnect(_mainFrame, SIGNAL(notifyPowerSupplyError(uint16_t, uint8_t, QString)), this, SLOT(detectPowerSupplyError(uint16_t, uint8_t, QString)));
    disconnect(this, SIGNAL(changeAttribute(QString, QVariant)), _mainFrame, SLOT(onChangeAttribute(QString, QVariant)));
    disconnect(this, SIGNAL(cursorPressed(QVariant)), _mainFrame, SIGNAL(cursorPressed(QVariant)));
    disconnect(this, SIGNAL(removeAttribute(QString)), _mainFrame, SLOT(onRemoveAttribute(QString)));
    disconnect(this, SIGNAL(requestToCapture()), _mainFrame, SLOT(onRequestToCapture()));
    disconnect(this, SIGNAL(requestToSaveImage(MeasureType, SaveLocation, QString)), _mainFrame, SLOT(onRequestToSaveImage(MeasureType, SaveLocation, QString)));
    disconnect(_mainFrame, SIGNAL(inputFileName(PopupId, SaveLocation, QString)), this, SLOT(onInputFileName(PopupId,SaveLocation, QString)));
    disconnect(this, SIGNAL(requestToSuspendAntennaMeasuring()), _model, SLOT(onSuspendAntennaMeasuring()));
    disconnect(this, SIGNAL(notifyLevelThreshold(QVariant, QVariant, QVariant)), _view, SLOT(updateLevelThreshold(QVariant, QVariant, QVariant)));
    disconnect(this, SIGNAL(notifyJcsat3aLevelMeter(QVariant,QVariant)), _view, SLOT(updateJcsat3aLevelMeterValue(QVariant, QVariant)));
    disconnect(this, SIGNAL(notifyJcsat4bLevelMeter(QVariant,QVariant)), _view, SLOT(updateJcsat4bLevelMeterValue(QVariant, QVariant)));
    disconnect(this, SIGNAL(requestToClearCaptureImage()), _mainFrame, SLOT(onRequestToClearCaptureImage()));

    // メニューバーの情報をクリア
    setMenuBarVisible(false);

    // 測定中ではない
    if (_view) {
        _view->setProperty("jcsat3aTarget", false);
    }
    if (_view) {
        _view->setProperty("jcsat4bTarget", false);
    }

    // 給電
    _model->removeMenuList(MenuListIconIndex::POWER_SUPPLY_ICON);
    _powerSupply = LC70_POWER_SUPPLY_OFF;

    // スキャン
    _model->removeMenuList(MenuListIconIndex::SCAN_ICON);
}

void AntennaAdjustment::onKeyStatus(ButtonId id)
{
    // メニューバー表示中
    if (isMenuBarVisible()) {
        menuBarOperation(id);
        return;
    }

    // Home, Escape ボタン(偏波から遷移した場合を除く)はベースクラスで対応
    if (id == BUTTON_HOME) {
        // スレッドを停止
        emit requestToSuspendAntennaMeasuring();

        AbstractViewModel::onKeyStatus(id);
    } else if (id == BUTTON_CHANNEL_UP) {
        _currentChannelIndex = nextChannel();

        if (_currentChannelIndex < 0) {
            return;
        }

        // チャンネル変更
        qDebug() << QString("channel up\n");
        changeChannel();
    } else if (id == BUTTON_CHANNEL_DOWN) {
        _currentChannelIndex = previousChannel();

        if (_currentChannelIndex < 0) {
            return;
        }

        // チャンネル変更
        qDebug() << QString("channel down\n");
        changeChannel();
    } else if (id == BUTTON_SCREEN_SAVE) {
        QString path = _model->applicationSetting()->resultDataPathSingle();
        int fileCount = countFilesRecursive(path);
        if (fileCount >= 1000) {
            emit requestToShowPopup(POPUP_DATA_SINGLE_MAX_SAVE_FILE);
            return;
        }

        // SCREENボタン押下時の画像を保持する
        emit requestToCapture();
        setBoolProperty(IMAGE_BUTTON_NAME, "pressed", false);

        emit requestToShowPopup(POPUP_INPUT_FILENAME);
    } else if (id == BUTTON_POWER_SUPPLY) {
        if (_powerSupply == LC70_POWER_SUPPLY_OFF) {
            // バッテリーレベルが 5% 以下で未充電あるいは不定値のときはシャットダウン
            if (isLowBattery() && batteryStatus() <= BATTERY_STATUS_UNCHARGED) {
                // バッテリ低下状態中は給電開始させない
                emit requestToShowLowBatteryPopup();
                return;
            }

            // バッテリーレベルが 10% 以下なら給電不可
            if (batteryLevel() <= BATTERY_LEVEL2 && batteryStatus() <= BATTERY_STATUS_UNCHARGED) {
                emit requestToShowPopup(POPUP_DISABLE_POWER_SUPPLY);
                return;
            }

            // 給電確認
            // JCSAT, SCC は垂直11v水平15v, 15v固定, 給電しないを選択
            emit requestToShowPopup(POPUP_POWER_SUPPLY_SELECT3);
        } else {
            // ヘッダーアイコン設定
            _model->removeMenuList(MenuListIconIndex::POWER_SUPPLY_ICON);

            _powerSupply = LC70_POWER_SUPPLY_OFF;
            _powerSupplyType = POWER_SUPPLY_COUNT;

            // 給電中ではない
            _powerSupplied = false;

            // 測定再開
            _bandIndex = JCSAT3A_INDEX;
            changeChannel();
        }
    } else if (id == BUTTON_ESCAPE) {
        emit requestInHistory(SCREEN_BAND);
    } else if (id == BUTTON_USER) {
#ifdef NEW_USER_SETTING
        emit requestDirect(SCREEN_USER_SETTING, true);
#else
        emit requestDirect(SCREEN_USER_DATA_LOGGER_MENU, true);
#endif
    } else if (id == BUTTON_MENU) {
        if (isMenuBarVisible()) {
            return;
        }

        // メニューバーの設定を初期設定にする
        AbstractViewModel::initializeMenuBarInformation();

        // メニューバーの表示
        _menuBarType = 1;
        emit requestToShowMenuBar(_menuBarType, _menuBarRank, _menuBarIndexMap.value(_menuBarRank));
        setMenuBarVisible(true);
    } else if (id == BUTTON_BAND) {
        if (isMenuBarVisible()) {
            return;
        }

        // メニューバーの表示
        emit requestToShowMenuBar(_menuBarType, _menuBarRank, _menuBarIndexMap.value(_menuBarRank));
        _menuBarRank = 1;  // 第 2 階層(Band)
        emit requestToShowMenuBar(_menuBarType, _menuBarRank, _menuBarIndexMap.value(_menuBarRank));
        setMenuBarVisible(true);

        // 測定設定をシングル測定
        _takeOver["selectedMeasurement"] = QString(SINGLE_MEASURE_TYPE_NAME);
        emit changeAttribute(QString("selectedMeasurement"), SINGLE_MEASURE_TYPE_NAME);

        // 画面遷移履歴の追加
        _changeHistory.clear();
        _changeHistory.append(SCREEN_HOME);
    } else if (id == BUTTON_SCAN) {
        // アンテナ調整はスキャン不可
        emit requestToShowPopupWithMessage(POPUP_SCAN_NOT_APPLICABLE, QString());
    }
}

void AntennaAdjustment::onAccept(PopupId popupId, bool okOnly)
{
    Q_UNUSED(okOnly);

    AbstractViewModel::onAccept(popupId, okOnly);

    if (popupId == POPUP_POWER_SUPPLY_ERROR_MANUAL_MEASURING) {
        // ブザーを止める
        _model->buzzer(LC70_BUZZ_OFF, LC70_BUZZ_MODE_STOP);

        // 測定再開
//        emit resumeAntennaMeasuring(&_condition,
//                                    _model->applicationSetting()->pollingIntervalMultiMeasurement());
    }
}

void AntennaAdjustment::onSelect(PopupId popupId, int index)
{
    // 給電しますかに「はい」を選択した場合
    Q_UNUSED(popupId);

    if (index == POWER_SUPPLY_COUNT) {
        _powerSupplyType = POWER_SUPPLY_COUNT;
        return;
    }

    _bandIndex = JCSAT3A_INDEX;

    if (index == FIXED_15V) {       // 15v固定 給電
        _powerSupply = LC70_POWER_SUPPLY_15V;
        _powerSupplyType = FIXED_15V;

        _model->replaceMenuList(MenuListIconIndex::POWER_SUPPLY_ICON, Correcter::powerSupplyIcon(_powerSupply));
    } else if (index == H15V11) {   // 水平15v/垂直11v 給電
        _powerSupplyType = H15V11;

        // 偏波による分岐
        if (_takeOver["selectedPolarization"].toString() == HORIZONTALLY_POLARIZE) {
            // パルスありなしは Band により切り替える
            if (_bandIndex == JCSAT3A_INDEX) {
                _powerSupply = LC70_POWER_SUPPLY_15V;
                _model->replaceMenuList(MenuListIconIndex::POWER_SUPPLY_ICON, Correcter::powerSupplyIcon(_powerSupply));
            } else if (_bandIndex == JCSAT4B_INDEX) {
                _powerSupply = LC70_POWER_SUPPLY_15V_P;
                _model->replaceMenuList(MenuListIconIndex::POWER_SUPPLY_ICON, Correcter::powerSupplyIcon(_powerSupply));
            }
        } else if (_takeOver["selectedPolarization"].toString() == VERTICALLY_POLARIZE) {
            if (_bandIndex == JCSAT3A_INDEX) {
                _powerSupply = LC70_POWER_SUPPLY_11V;
                _model->replaceMenuList(MenuListIconIndex::POWER_SUPPLY_ICON, Correcter::powerSupplyIcon(_powerSupply));
            } else if (_bandIndex == JCSAT4B_INDEX) {
                _powerSupply = LC70_POWER_SUPPLY_11V_P;
                _model->replaceMenuList(MenuListIconIndex::POWER_SUPPLY_ICON, Correcter::powerSupplyIcon(_powerSupply));
            }
        }
    }

    // 測定再開(本メソッドで指定した給電条件を使う)
    changeChannel();

    // 給電中
    _powerSupplied = true;
}

void AntennaAdjustment::onReject(PopupId popupId)
{
    if (popupId == POPUP_SELECT_SAVE_DATA) {
        // nothing to do.
    }
    if (popupId == POPUP_INPUT_FILENAME) {
        // 保存するために保持した各データをクリアする
        emit requestToClearCaptureImage();
    }
}

void AntennaAdjustment::onButtonPress(ButtonId id)
{
    // メニューバー表示中
    if (isMenuBarVisible()) {
        if (id == BUTTON_ENTER) {
            if (_menuBarType == 1) {
                MainFrameViewModel *mainFrame = static_cast<MainFrameViewModel *>(_mainFrame);
                QVariantList vFileList = mainFrame->getUserFileList();
                if (vFileList[_menuBarIndexMap.value(_menuBarRank)].toString().isEmpty()) {
                    return;
                }
            }
            emit cursorPressed(_menuBarRank);
        }

        return;
    }

    if (id == BUTTON_SCREEN_SAVE) {
        setBoolProperty(IMAGE_BUTTON_NAME, "pressed", true);
    }
}

void AntennaAdjustment::onLowBatteryStatusChanged(bool isLowBattery)
{
    LC70_POWER_SUPPLY_T before = _powerSupply;
    AbstractMeasurementViewModel::onLowBatteryStatusChanged(isLowBattery);
    if (before != LC70_POWER_SUPPLY_OFF && _powerSupply == LC70_POWER_SUPPLY_OFF) {
        // バッテリ低下により給電が停止した場合は、給電種別をクリアする
        _powerSupplyType = POWER_SUPPLY_COUNT;
        // 測定再開
        _bandIndex = JCSAT3A_INDEX;
        changeChannel();
    }
}

void AntennaAdjustment::onRequestToStopScanning()
{
    // nothind to do.
}

void AntennaAdjustment::onAntennaMeasureResult(MultiResult result)
{
    // チャンネル変更中に届いた測定結果は表示しない
    if (_isChannelChanging) {
        return;
    }

    // 無効なインデックスの場合
    if (_currentChannelIndex < 0) {
        _model->writeLog(QString("アンテナ調整"), QString("%1 all invalid channels").arg(__FUNCTION__));
        return;
    }

    // チャンネル情報がなければ表示しない
    if (_channelList.count() < 1) {
        return;
    }

    // チャンネルが無効な場合
    if (!_channelList[_currentChannelIndex].valid) {
        _model->writeLog(QString("アンテナ調整"),
                         QString("%1 invalid channels(%2)").arg(__FUNCTION__).arg(_currentChannelIndex));
        return;
    }

    _bandIndex = result.readIndex;

    if (_bandIndex == JCSAT3A_INDEX) {
        // JCSAT3の場合
        if (_view) {
            _view->setProperty("jcsat3aTarget", true);
        }
        if (_view) {
            _view->setProperty("jcsat4bTarget", false);
        }
    } else if (_bandIndex == JCSAT4B_INDEX) {
        // JCSAT4の場合
        if (_view) {
            _view->setProperty("jcsat3aTarget", false);
        }
        if (_view) {
            _view->setProperty("jcsat4bTarget", true);
        }
    }

    // 現在のチャンネルから偏波面の取得を行う
    QString polarize = Correcter::convertWaveToPolarization(_channelList.at(_currentChannelIndex).polarization);

    // 給電が水平15/垂直11Vの場合は、バンドと偏波で給電アイコンを更新する
    if (_powerSupplyType == H15V11) {
        if (_bandIndex == JCSAT3A_INDEX) {
            // 偏波による分岐
            if (polarize == HORIZONTALLY_POLARIZE) {
                _powerSupply = LC70_POWER_SUPPLY_15V;
            } else if (polarize == VERTICALLY_POLARIZE) {
                _powerSupply = LC70_POWER_SUPPLY_11V;
            }
        } else if (_bandIndex == JCSAT4B_INDEX) {
            // 偏波による分岐
            if (polarize == HORIZONTALLY_POLARIZE) {
                _powerSupply = LC70_POWER_SUPPLY_15V_P;
            } else if (polarize == VERTICALLY_POLARIZE) {
                _powerSupply = LC70_POWER_SUPPLY_11V_P;
            }
        }
        if (_powerSupplied) {
            _model->replaceMenuList(MenuListIconIndex::POWER_SUPPLY_ICON, Correcter::powerSupplyIcon(_powerSupply));
        } else {
            _powerSupply = LC70_POWER_SUPPLY_OFF;
        }
    } else {
        _powerSupply = LC70_POWER_SUPPLY_OFF;
    }

    // 閾値の反映(同じチャンネルで異なる Band の測定のため、インデックスが 0 なら JCSAT-3A で、 1 なら JCSAT-4B とする)
    if (!_thresholdSetJCSAT3A && result.readIndex == JCSAT3A_INDEX) {
        _broadBandSystem = result.result.BroadcastSystem;
        refreshThreshold(LC70_BAND_JCSAT3);
    }

    if (!_thresholdSetJCSAT4B && result.readIndex == JCSAT4B_INDEX) {
        _broadBandSystem = result.result.BroadcastSystem;
        refreshThreshold(LC70_BAND_JCSAT4);
    }

    static int count = 0;
    quint64 totalTime = 0;
    timer.restart();

    if (_bandIndex == JCSAT3A_INDEX || _bandIndex == JCSAT4B_INDEX) {
        // 測定入力レベル
        if ((result.status & LC70_DATA_MSK_LEVEL) == LC70_DATA_MSK_LEVEL) {
            if (_bandIndex == JCSAT3A_INDEX) {
                // JCSAT-3Aの最新のLEVEL値を取得
                _newestJcsat3aLevel = result.result.Level;
            } else {
                // JCSAT-4Bの最新のLEVEL値を取得
                _newestJcsat4bLevel = result.result.Level;
            }

            if (!_isUpdateMeter) {
                // メーター更新を開始
                onUpdateMeter();
            }
        }

        // カレントインデックスと読み出しインデックスが同じ場合は不定値になるので、MER/CN, BER を更新せずに前の値を残す
        if (result.currentMeasuringIndex != result.readIndex) {
            // 変調誤差比
            QString merValue = QString("---");
            if (result.locked == LOCKED
                    && (result.status & LC70_DATA_MSK_CNR) == LC70_DATA_MSK_CNR
                    && !qIsNaN(result.result.Mer)
                    && !qIsInf(result.result.Mer)) {
                if (_bandIndex == JCSAT3A_INDEX) {
                    merValue = Correcter::displayMer(static_cast<int>(LC70_BAND_JCSAT3), static_cast<int>(_broadBandSystem), static_cast<int>(result.result.Modulation), result.result.Mer);
                } else {
                    merValue = Correcter::displayMer(static_cast<int>(LC70_BAND_JCSAT4), static_cast<int>(_broadBandSystem), static_cast<int>(result.result.Modulation), result.result.Mer);
                }
            } else {
                merValue = QString(MER_TIMEOUT_VALUE);
            }
            // displayMer だと 閾値未満は "<5" と返ってくるため　MER_TIMEOUT_VALUE に差し替える
            // toDouble() は ">35" が返ってきた場合に識別できないので使わない
            if (merValue.contains("<")) {
                merValue = QString(MER_TIMEOUT_VALUE);
            }
            _view->setProperty("mer", merValue);

            if (_view) {
                _view->setProperty(_bandIndex == JCSAT3A_INDEX ? "jcsat3aMer" : "jcsat4bMer", merValue);
            }

            // ビット誤り率
            QString berValue = QString("---");
            if (result.locked == LOCKED
                    && (result.status & LC70_DATA_MSK_BER) == LC70_DATA_MSK_BER
                    && !qIsNaN(result.result.Ber)
                    && !qIsInf(result.result.Ber)) {
                berValue = Correcter::displayBer(result.result.Ber);
            } else {
                berValue = QString(BER_TIMEOUT_VALUE);
            }
            if (_view) {
                _view->setProperty(_bandIndex == JCSAT3A_INDEX ? "jcsat3aBer" : "jcsat4bBer", berValue);
            }
        }
    }

    // 周波数
    unsigned centerFrequency = 0;
    unsigned int localFrequency = _takeOver["selectedLocalOscillator"].toUInt();

    if (_bandIndex == JCSAT3A_INDEX) {
        centerFrequency = Correcter::centerFrequency(_channelList[_currentChannelIndex], result.result.Modulation);
        if (_view) {
            _view->setProperty("frequency", Correcter::displayFrequency(LC70_BAND_JCSAT3, Correcter::frequency(centerFrequency,
                               localFrequency), true));
        }
    } else if (_bandIndex == JCSAT4B_INDEX) {
        ChannelDefinitionItem channel;

        if (_takeOver["selectedPolarization"].toString() == HORIZONTALLY_POLARIZE) {
            if (_model->getChannelDefinitionItemForAntennaAdjustment(LC70_BAND_JCSAT4, QString(TARGET_CHANNELS_HORIZONTAL[_currentChannelIndex]),
                                                 channel)) {
            }
        } else if (_takeOver["selectedPolarization"].toString() == VERTICALLY_POLARIZE) {
            if (_model->getChannelDefinitionItemForAntennaAdjustment(LC70_BAND_JCSAT4, QString(TARGET_CHANNELS_VERTICAL[_currentChannelIndex]),
                                                 channel)) {
            }
        }

        centerFrequency = Correcter::centerFrequency(channel, result.result.Modulation);
        if (_view) {
            _view->setProperty("frequency", Correcter::displayFrequency(LC70_BAND_JCSAT4, Correcter::frequency(centerFrequency,
                               localFrequency), true));
        }
    }

    totalTime += timer.elapsed();
    ++count;

    if (count == 10) {
        _model->writeLog(QString("アンテナ調整"), QString("%1 %2 ms(average)").arg(__FUNCTION__).arg(totalTime / 10));
        count = 0;
        totalTime = 0;
    }
}

void AntennaAdjustment::onUpdateMeter()
{
    // JCSAT-3A側のメーター更新
    QString jcsat3aLevelString;
    float32_t jcsat3aLevelNeedle;
    updateLevelMeter(JCSAT3A_INDEX, jcsat3aLevelString, jcsat3aLevelNeedle);
    emit notifyJcsat3aLevelMeter(jcsat3aLevelString, jcsat3aLevelNeedle);

    // JCSAT-4B側のメーター更新
    QString jcsat4bLevelString;
    float32_t jcsat4bLevelNeedle;
    updateLevelMeter(JCSAT4B_INDEX, jcsat4bLevelString, jcsat4bLevelNeedle);
    emit notifyJcsat4bLevelMeter(jcsat4bLevelString, jcsat4bLevelNeedle);

    if ((_jcsat3aLevel != _newestJcsat3aLevel) || (_jcsat4bLevel != _newestJcsat4bLevel)) {
        _isUpdateMeter = true;
        QTimer::singleShot((_meterUpdateInterval), this, SLOT(onUpdateMeter()));
    } else {
        // メーター更新反映完了
        _isUpdateMeter = false;
    }
}

void AntennaAdjustment::updateLevelMeter(const BandIndex bandIndex, QString &levelString, float32_t &levelNeedle) {
    // 現在値と最新値との差の大きさによって、針の移動量を決定するためのリスト
    static const QList<QPair<float32_t, float32_t>> levelResolutionList = {
        {20.0f, 16.0f},  // 例：差が20以上ある場合、針を動かす量は8ずつにする
        {10.0f, 8.0f},
        {5.0f,  2.0f},
        {3.0f,  1.2f},
        {2.0f,  0.8f},
        {1.0f,  0.4f},  // 例：差が1〜2の場合、針を動かす量は0.2ずつにする
        {0.0f,  0.2f},  // 例：それ未満は 0.1ずつにする
    };

    float32_t newestLevel;
    float32_t* displayLevel;
    LC70_BAND_T band;

    if (bandIndex == JCSAT3A_INDEX) {
        band = LC70_BAND_JCSAT3;
        newestLevel = _newestJcsat3aLevel;
        displayLevel = &_jcsat3aLevel;
    } else {
        band = LC70_BAND_JCSAT4;
        newestLevel = _newestJcsat4bLevel;
        displayLevel = &_jcsat4bLevel;
    }
    float32_t levelValue;
    bool isUndefinedValue = static_cast<int>(newestLevel) == UNDEFINED_VALUE;
    float32_t newestValue = isUndefinedValue ? 0 : newestLevel;

    if (newestValue != *displayLevel ) {
        float32_t difference = newestValue - *displayLevel;
        if (isUndefinedValue && (_invalidValueMeterSpeed == 0)) {
            // 無効時にアニメーションしない設定であれば、最新の値を採用する
            levelValue = newestValue;
        } else if (qIsNaN(difference) || qIsInf(difference)) {
            // 値の差がない場合は最新の値を採用する
            levelValue = newestValue;
        } else if ((0 < difference && difference < 1.0) ||
                   (-1.0 < difference && difference < 0)) {
            // 一目盛り分(±1.0 dBuV) のときは最新の値を採用する
            levelValue = newestValue;
            //_model->writeLog(QString("シングル測定"), QString("[NO ANIMATION] LEVEL %1 - %2 difference:%3").arg(newestValue).arg(_level).arg(difference), LOG_NOTICE);
        } else {
            // 差の大きさによって、一度に針を動かす量を決定する
            float32_t resolution = levelResolutionList.last().second;

            // 無効値が入力された場合用の特殊処理
            if (isUndefinedValue) {
                // 針の移動速度を、有効時と変え、倍率をかける
                resolution *= static_cast<float32_t>(_invalidValueMeterSpeed);
            }
            for (int i = 0 ; i < levelResolutionList.size(); ++i) {
                if (levelResolutionList.at(i).first < qAbs(difference)) {
                    resolution = levelResolutionList.at(i).second;
                    break;
                }
            }

            if (qAbs(difference) > resolution) {
                if (difference < 0) {
                    levelValue = *displayLevel - resolution;
                } else {
                    levelValue = *displayLevel + resolution;
                }
            } else {
                // 差が予定している針の移動量を下回る場合は、小さい方の値を採用する
                if (difference < 0) {
                    levelValue = *displayLevel - difference;
                } else {
                    levelValue = *displayLevel + difference;
                }
            }
            //_model->writeLog(QString("シングル測定"), QString("[ANIMATION] LEVEL %1 - %2 difference:%3").arg(newestValue).arg(_level).arg(difference), LOG_NOTICE);
        }
    } else {
        levelValue = newestValue;
    }

    *displayLevel = levelValue;

    // 現在のチャンネルから偏波面の取得を行う
    if (_channelList.count() == 0) {
        return;
    }
    QString polarize = _takeOver["selectedPolarization"].toString();
    BandTabIndex bandTab = Correcter::convertBandTabIndex(band, polarize);
    float32_t minimumLevel = 0;

    // 表示クリアー時のときは初期値設定
    if (isUndefinedValue) {
        levelValue = UNDEFINED_VALUE;
    }
    if (!Correcter::displayLevel(bandTab, levelValue, levelString)) {
        qDebug() << QString("displayLevel() failed");
    }

    if (!Correcter::minimumLevel(bandTab, minimumLevel)) {
        qDebug() << QString("minimumLevel() failed");
    }

    // 最大値は針の最大値から
    float32_t maximumLevel;
    if (!Correcter::maximumLevel(bandTab, maximumLevel)) {
        maximumLevel = MAXIMUM_LEVEL;
    }
#if 1	//20210705_dx
    if (levelValue > minimumLevel && levelValue <= MAXIMUM_LEVEL) {
#else
    if (levelValue > minimumLevel && levelValue <= maximumLevel) {
#endif
        levelNeedle = LEVEL_NEEDLE_OFFSET + ((levelValue - minimumLevel) * LEVEL_NEEDLE_STEP);
        if(minimumLevel == 30){
            levelNeedle = levelNeedle + (minimumLevel-6);
        }else if(minimumLevel == 40){
            levelNeedle = levelNeedle + (minimumLevel+11);
        }
#if 1	//20210705_dx
    } else if (levelValue > MAXIMUM_LEVEL) {
        levelNeedle = LEVEL_NEEDLE_OFFSET + ((MAXIMUM_LEVEL - 20) * LEVEL_NEEDLE_STEP);
#else
    } else if (levelValue > maximumLevel) {
        levelNeedle = LEVEL_NEEDLE_OFFSET + ((MAXIMUM_LEVEL - minimumLevel) * LEVEL_NEEDLE_STEP);
#endif
    } else {
        if (minimumLevel == 30 || minimumLevel == 40) {
            // 20 はメーターの目盛りの最小値
            if (levelValue > 20) {
                levelNeedle = LEVEL_NEEDLE_OFFSET + ((levelValue - 20) * LEVEL_NEEDLE_STEP);
            } else {
                levelNeedle = MINIMUM_NEEDLE_POSITION;
            }
        } else {
            levelNeedle = MAXIMUM_NEEDLE_POSITION;
        }
    }
}

void AntennaAdjustment::onNotifyChannelChanging(bool changing)
{
    qDebug() << __FUNCTION__ << changing;
    _isChannelChanging = changing;

    // 異なる Band でチャンネルの切り替えのみ
    if (_condition.single[0].ulFreq != _previousCondition.single[0].ulFreq ||
            QString::fromLocal8Bit((char *)_condition.single[0].ucStrChan) != QString::fromLocal8Bit((
                        char *)_previousCondition.single[0].ucStrChan)) {
        qDebug() << QString("[Change condition again]");

        _previousCondition = _condition;
    }
}

void AntennaAdjustment::changeChannel()
{
    if (_channelList.count() < 1) {
        return;
    }

    _model->writeLog(QString("アンテナ調整"), QString("[Enter] %1").arg(__FUNCTION__));
    timer.start();

    // 画面表示をクリアー
    clear();

    // 中心周波数と局発周波数の情報から周波数情報を取得する
    unsigned int localFrequency = _takeOver["selectedLocalOscillator"].toUInt();

    // 画面にチャンネル情報を反映(周波数は測定結果から出力)
    if (_view) {
        _view->setProperty("band", Correcter::displayBandName(_channelList[_currentChannelIndex].band));
    }
    if (_view) {
        _view->setProperty("channel", Correcter::displayChannelName(_channelList[_currentChannelIndex].channelNumber));
    }
    if (_view) {
        _view->setProperty("polarization", Correcter::displayPolarization(_takeOver["selectedPolarization"].toString()));
    }

    // 局発周波数の設定を行う
    // kHz => GHz
    if (_view) {
        _view->setProperty("localFrequency", Correcter::displayLocalFrequency(localFrequency));
    }

    memset(&_condition, 0x00, sizeof(LC70_MEAS_MULTI_T));

    // 登録件数(JCSAT-3A と JCSAT-4B の 2 件)
    _condition.sRegNum = NUMBER_OF_CHANNELS;

    for (int i = 0; i < _condition.sRegNum; ++i) {
        _condition.single[i].ulFreq = Correcter::centerFrequency(_channelList[_currentChannelIndex], 0);

        // BAND
        if (i == 0) {
            _condition.single[i].usBand = LC70_BAND_JCSAT3;
        } else if (i == 1) {
            _condition.single[i].usBand = LC70_BAND_JCSAT4;
            ChannelDefinitionItem channel;

            if (_takeOver["selectedPolarization"].toString() == HORIZONTALLY_POLARIZE) {
                if (_model->getChannelDefinitionItemForAntennaAdjustment(LC70_BAND_JCSAT4, QString(TARGET_CHANNELS_HORIZONTAL[_currentChannelIndex]),
                                                     channel)) {
                }
            } else if (_takeOver["selectedPolarization"].toString() == VERTICALLY_POLARIZE) {
                if (_model->getChannelDefinitionItemForAntennaAdjustment(LC70_BAND_JCSAT4, QString(TARGET_CHANNELS_VERTICAL[_currentChannelIndex]),
                                                     channel)) {
                }
            }

            _condition.single[i].ulFreq = Correcter::centerFrequency(channel, 0);
        }

        // チャンネル番号文字列(NULL終端なし、8文字に足りない場合はスペースで埋める)
        memset(_condition.single[i].ucStrChan, ' ', CHANNEL_NUMBER_LENGTH);

        for (int j = 0; j < CHANNEL_NUMBER_LENGTH; ++j) {
            // 同じチャンネル番号に揃える
            if (j <= _channelList[_currentChannelIndex].channelNumber.toLocal8Bit().length() - 1) {
                _condition.single[i].ucStrChan[j] = _channelList[_currentChannelIndex].channelNumber.toLocal8Bit()[j];
            }
        }

        int length = strnlen((char *)_condition.single[i].ucStrChan, sizeof _condition.single[i].ucStrChan);
        if (length < CHANNEL_NUMBER_LENGTH) {
            for (int j = length; j < CHANNEL_NUMBER_LENGTH; ++j) {
                _condition.single[i].ucStrChan[j] = ' ';
            }
        }

        // 局発周波数の連携
        unsigned int localFrequency = _takeOver["selectedLocalOscillator"].toUInt();
        _condition.single[i].ulLoFreq = localFrequency;

        // 放送方式の連携
        _condition.single[i].usBdSys = LC70_BD_SYS_DVBS2;

        // 変調方式
        _condition.single[i].usMod = LC70_SETMOD_UNDEF;

        // 測定条件
        _condition.single[i].usCondition = LC70_DATA_MSK_LEVEL | LC70_DATA_MSK_CNR |
                                           LC70_DATA_MSK_BER;

        // 給電あり(マニュアルだが、給電操作を測定プロセス以下で行う)
        _condition.single[i].ucOutputEnb = 1;

        // 給電設定
        switch (_powerSupplyType) {
        case POWER_SUPPLY_COUNT:
            _condition.single[i].powerSupply = LC70_POWER_SUPPLY_OFF;
            break;
        case FIXED_15V:
            _condition.single[i].powerSupply = LC70_POWER_SUPPLY_15V;
            break;
        case H15V11: {
            // 現在のチャンネルから偏波面の取得を行う
            QString polarize = Correcter::convertWaveToPolarization(_channelList.at(_currentChannelIndex).polarization);

            // 偏波による分岐
            if (polarize == HORIZONTALLY_POLARIZE) {

                if (i == JCSAT3A_INDEX) {
                    _condition.single[i].powerSupply = LC70_POWER_SUPPLY_15V;
                } else if (i == JCSAT4B_INDEX) {
                    _condition.single[i].powerSupply = LC70_POWER_SUPPLY_15V_P;
                }
            } else if (polarize == VERTICALLY_POLARIZE) {

                if (i == JCSAT3A_INDEX) {
                    _condition.single[i].powerSupply = LC70_POWER_SUPPLY_11V;
                } else if (i == JCSAT4B_INDEX) {
                    _condition.single[i].powerSupply = LC70_POWER_SUPPLY_11V_P;
                }
            } else {
                // nothing to do.
            }
            break;
        }
        default:
            _condition.single[i].powerSupply = LC70_POWER_SUPPLY_OFF;
            break;
        }
    }

    // チャンネル変更中の操作を禁止
    if (_isChannelChanging) {
        return;
    }

    // 閾値の設定をクリアー
    _thresholdSetJCSAT3A = false;
    _thresholdSetJCSAT4B = false;

    if (!_isMeasuring) {
        // 測定中でなければスレッドを開始
        emit resumeAntennaMeasuring(&_condition,
                                    _model->applicationSetting()->pollingIntervalMultiMeasurement());
        _isMeasuring = true;
    } else {
        _isChannelChanging = true;  // 先に変更しておく
        clear(true);                // 測定結果をクリアー

        // 測定中ではない
        if (_view) {
            _view->setProperty("jcsat3aTarget", false);
        }
        if (_view) {
            _view->setProperty("jcsat4bTarget", false);
        }

        // 測定条件変更
        emit changeCondition(&_condition);
        qDebug() << QString("[CALL] AntennaAdjustment::changeChannel()");
    }

    _previousCondition = _condition;
    _model->writeLog(QString("アンテナ調整"), QString("[Leave] %1 %2 ms").arg(__FUNCTION__).arg(timer.elapsed()));
}

void AntennaAdjustment::clear(bool measurementValuleOnly)
{
    _newestJcsat3aLevel = -1;
    _newestJcsat4bLevel = -1;
    if (!_isUpdateMeter) {
        onUpdateMeter();
    }

    if (_view) {
        _view->setProperty("jcsat3aLevel", INITIAL_LEVEL);
    }
    if (_view) {
        _view->setProperty("jcsat3aLevelNeedlePosition", MINIMUM_NEEDLE_POSITION);
    }
    if (_view) {
        _view->setProperty("jcsat3aLevelRedZoneRatio", 0);
    }
    if (_view) {
        _view->setProperty("jcsat3aMer", INITIAL_MER);
    }
    if (_view) {
        _view->setProperty("jcsat3aBer", INITIAL_BER);
    }
    if (_view) {
        _view->setProperty("jcsat4bLevel", INITIAL_LEVEL);
    }
    if (_view) {
        _view->setProperty("jcsat4bLevelNeedlePosition", MINIMUM_NEEDLE_POSITION);
    }
    if (_view) {
        _view->setProperty("jcsat4bLevelRedZoneRatio", 0);
    }
    if (_view) {
        _view->setProperty("jcsat4bMer", INITIAL_MER);
    }
    if (_view) {
        _view->setProperty("jcsat4bBer", INITIAL_BER);
    }

    if (!measurementValuleOnly) {
        if (_view) {
            _view->setProperty("band", "");
        }
        if (_view) {
            _view->setProperty("channel", "");
        }
        if (_view) {
            _view->setProperty("polarization", "");
        }
        if (_view) {
            _view->setProperty("localFrequency", "");
        }
    }
}

void AntennaAdjustment::refreshThreshold(const int &bandId)
{
    qDebug() << QString("[Enter] %1 arg: %2").arg(__FUNCTION__).arg(bandId);

    // 閾値の取得
    QVariant thresholdSetting;     // 実態は QVariantMap
    _model->getValue(VALUE_THRESHOLD, thresholdSetting);

    QVariantList thresholds;

    switch (bandId) {
    case LC70_BAND_JCSAT3:
        if (_broadBandSystem == LC70_BD_SYS_DVBS) {
            // 放送方式 DVBS
            thresholds = thresholdSetting.toList().at(LC70_MEAS_SET_THRD_IDX_JCSAT3_DVBS).toList();
        } else {
            // 放送方式 DVBS2
            thresholds = thresholdSetting.toList().at(LC70_MEAS_SET_THRD_IDX_JCSAT3_DVBS2).toList();
        }

        break;

    case LC70_BAND_JCSAT4:
        if (_broadBandSystem == LC70_BD_SYS_DVBS) {
            // 放送方式 DVBS
            thresholds = thresholdSetting.toList().at(LC70_MEAS_SET_THRD_IDX_JCSAT4_DVBS).toList();
        } else {
            // 放送方式 DVBS2
            thresholds = thresholdSetting.toList().at(LC70_MEAS_SET_THRD_IDX_JCSAT4_DVBS2).toList();
        }

        break;

    default:
        return;
    }

    // 要素数が不足している場合
    if (thresholds.count() < THRESHOLD_ITEM_COUNT) {
        return;
    }

    // LEVEL の閾値(受信可の値)
    double levelLow = thresholds[1].toDouble() / 10;
    double levelHigh = thresholds[2].toDouble() / 10;

    double levelRedZone = 0;
    double levelYellowZone = 0;

    if (0 <= (levelLow - MINIMUM_LEVEL) / (MAXIMUM_LEVEL - MINIMUM_LEVEL)
            && (levelLow - MINIMUM_LEVEL) / (MAXIMUM_LEVEL - MINIMUM_LEVEL) <= 1.0) {
        levelRedZone = (levelLow - MINIMUM_LEVEL) / (MAXIMUM_LEVEL - MINIMUM_LEVEL);
    }

    if (0 <= (levelHigh - levelLow) / (MAXIMUM_LEVEL - MINIMUM_LEVEL)
            && (levelHigh - levelLow) / (MAXIMUM_LEVEL - MINIMUM_LEVEL) <= 1.0) {
        levelYellowZone = (levelHigh - levelLow) / (MAXIMUM_LEVEL - MINIMUM_LEVEL);
    }

    // Level の閾値を通知
    emit notifyLevelThreshold(levelRedZone, levelYellowZone, (bandId == LC70_BAND_JCSAT3) ? JCSAT3A_INDEX : JCSAT4B_INDEX);

    if (_bandIndex == JCSAT3A_INDEX) {
        _thresholdSetJCSAT3A = true;
    } else if (_bandIndex == JCSAT4B_INDEX) {
        _thresholdSetJCSAT4B = true;
    }

    qDebug() << QString("[Leave]") << __FUNCTION__;
}

void AntennaAdjustment::changeByMenuBar()
{
    _model->writeLog(QString("アンテナ調整"), QString("[ENTER] %1").arg(__FUNCTION__));

    int barType = _menuBarType;
    int barIndex = _menuBarIndexMap[0];

    // メニューバーを閉じる
    closeAllMenuBar();

    // メニューバーの情報をクリア
    setMenuBarVisible(false);
    initializeMenuBarInformation();

    _model->changeAcquisitionTarget(ANTENNA);

    // Menuアイコンの更新を実施
    changeMenuIcon();
    if (barType == 1) {
        _model->replaceMenuList(MenuListIconIndex::FUNCTION_ICON, "icon_menu_user.png");
    }

    // 給電を停止
    _powerSupply = LC70_POWER_SUPPLY_OFF;
    _model->removeMenuList(MenuListIconIndex::POWER_SUPPLY_ICON);

    if (_takeOver["selectedBand"].toInt() == LC70_BAND_JCSAT3 * LC70_BAND_JCSAT4) {
        // アンテナ調整を選択している場合は測定設定にアンテナ調整を設定
        _takeOver["selectedMeasurement"] = ANTENNA_MEASURE_TYPE_NAME;
    }

    // シングル測定を選択している場合
    if (_takeOver["selectedMeasurement"].toString() == QString(SINGLE_MEASURE_TYPE_NAME)) {
        if (barType == 0) {
            // ユーザ設定からの遷移判定を初期値に戻す
            _takeOver["formUserSetting"] = false;

            // スレッド停止(遷移先画面種別を測定画面以外に変更)
            emit requestToSuspendAntennaMeasuring();
            _alreadySuspended = true;           // 終了処理のときにアンテナ測定の停止処理を呼ばないようにする

            AbstractViewModel::suspend();

            // 画面共有データの送信
            // [Memo] 測定情報、BAND情報は必ずデータが設定されている前提
            //        変調方式、偏波面、局発周波数は不要な場合は、未設定
            emit requestSetTakeOver("selectedMeasurement", _takeOver["selectedMeasurement"]);
            emit requestSetTakeOver("selectedBand", _takeOver["selectedBand"]);
            emit requestSetTakeOver("fromUserSetting", false);

            // 変調方式、偏波面、局発周波数はnullではない場合に送信
            if (!_takeOver["selectedModulation"].isNull()) {
                emit requestSetTakeOver("selectedModulation", _takeOver["selectedModulation"]);
            }

            if (!_takeOver["selectedPolarization"].isNull()) {
                emit requestSetTakeOver("selectedPolarization", _takeOver["selectedPolarization"]);
            }

            if (!_takeOver["selectedLocalOscillator"].isNull()) {
                emit requestSetTakeOver("selectedLocalOscillator", _takeOver["selectedLocalOscillator"]);
            }
        } else if (barType == 1) {
            // ユーザ設定からの遷移判定を初期値に戻す
            _takeOver["fromUserSetting"] = false;

            // スレッド停止(遷移先画面種別を測定画面以外に変更)
            emit requestToSuspendAntennaMeasuring();
            _alreadySuspended = true;           // 終了処理のときにアンテナ測定の停止処理を呼ばないようにする

            AbstractViewModel::suspend();

            // 一時停止
            MainFrameViewModel *mainFrame = static_cast<MainFrameViewModel *>(_mainFrame);
            QVariantList vFileList = mainFrame->getUserFileList();
            QVariant vFileName = vFileList[barIndex];
            QString sFileName = vFileName.toString();
            _model->importDataLoggerUserSettingList(VALUE_USERSETTING_SETTINGLIST, sFileName);
            emit requestSetTakeOver("fromUserSetting", true);
            emit requestSetTakeOver("selectedUserListFile", sFileName);
        }

        // 画面遷移
        QList<ScreenId> history = QList<ScreenId>() << SCREEN_HOME << SCREEN_BAND;
        emit requestWithChangeHistory(SCREEN_SINGLE_MEASUREMENT, history);
        return;
    }

    // マルチ測定を選択している場合
    if (_takeOver["selectedMeasurement"].toString() == QString(MULTI_MEASURE_TYPE_NAME)) {

        // ユーザー設定からの遷移判定を初期値に戻す
        _takeOver["fromUserSetting"] = false;

        // スレッド停止(遷移先画面種別を測定画面以外に変更)
        emit requestToSuspendAntennaMeasuring();
        _alreadySuspended = true;           // 終了処理のときにアンテナ測定の停止処理を呼ばないようにする

        AbstractViewModel::suspend();

        // 画面共有データの送信
        // [Memo] 測定情報、BAND情報は必ずデータが設定されている前提
        //        変調方式、偏波面、局発周波数は不要な場合は、未設定
        emit requestSetTakeOver("selectedMeasurement", _takeOver["selectedMeasurement"]);
        emit requestSetTakeOver("selectedBand", _takeOver["selectedBand"]);
        emit requestSetTakeOver("fromUserSetting", false);

        if (barType == 0) {
            // 変調方式、偏波面、局発周波数はnullではない場合に送信
            if (!_takeOver["selectedModulation"].isNull()) {
                emit requestSetTakeOver("selectedModulation", _takeOver["selectedModulation"]);
            }

            if (!_takeOver["selectedPolarization"].isNull()) {
                emit requestSetTakeOver("selectedPolarization", _takeOver["selectedPolarization"]);
            }

            if (!_takeOver["selectedLocalOscillator"].isNull()) {
                emit requestSetTakeOver("selectedLocalOscillator", _takeOver["selectedLocalOscillator"]);
            }
        } else if (barType == 1) {
            MainFrameViewModel *mainFrame = static_cast<MainFrameViewModel *>(_mainFrame);
            QVariantList vFileList = mainFrame->getUserFileList();
            QVariant vFileName = vFileList[barIndex];
            QString sFileName = vFileName.toString();
            _model->importDataLoggerUserSettingList(VALUE_USERSETTING_SETTINGLIST, sFileName);
            emit requestSetTakeOver("fromUserSetting", true);
            emit requestSetTakeOver("selectedUserListFile", sFileName);
        }

        // 画面遷移
        QList<ScreenId> history = QList<ScreenId>() << SCREEN_HOME << SCREEN_BAND;
        emit requestWithChangeHistory(SCREEN_MULTI_MEASUREMENT, history);
        return;
    }

    // 画面遷移履歴のクリアー
    _changeHistory.clear();

    // マルチ測定条件のクリアー
    for (int i = 0; i < _channelList.count(); ++i) {
        _condition.sRegNum = 0;
        memset(&_condition.single[i], 0x00, sizeof(_condition.single[i]));
        memset(&_previousCondition.single[i], 0x00, sizeof(_previousCondition.single[i]));
    }

    // チャンネル情報のクリアー
    _channelList.clear();

    // 局発周波数のリストをクリアー
    _localFrequencyList.clear();

    // 給電をヘッダーから取り除く
    _model->removeMenuList(MenuListIconIndex::POWER_SUPPLY_ICON);

    // スキャンをヘッダーから取り除く
    _model->removeMenuList(MenuListIconIndex::SCAN_ICON);

    // 初期値に戻す
    activate();

    // チャンネル変更(測定開始)
    changeChannel();

    _model->writeLog(QString("アンテナ調整"), QString("[LEAVE] %1").arg(__FUNCTION__));
}

void AntennaAdjustment::detectPowerSupplyError(uint16_t code, uint8_t action, QString message)
{
    _failureCode = code;
    _actionInFailure = action;
    // マニュアル測定だが、実際は測定条件で給電設定を行うので、給電要求で停止はしない
    _powerSupply = LC70_POWER_SUPPLY_OFF;

    _model->removeMenuList(MenuListIconIndex::POWER_SUPPLY_ICON);

    if (_actionInFailure == LC70_FAIL_CTL_MSGDISP_CLR) {
        uint8_t addInfo[LC70_FAIL_ADD_INFO_NUM];
        memset(addInfo, 0x00, LC70_FAIL_ADD_INFO_NUM);
        //_model->setFaultInformation(_failureCode, LC70_FAIL_STAT_NORMAL, addInfo);     故障復旧処理不要
        _failureCode = 0;
        _actionInFailure = LC70_FAIL_CTL_NONE;
    }
#if 1
    _powerSupplyType = POWER_SUPPLY_COUNT;
    _bandIndex = JCSAT3A_INDEX;
    changeChannel();
#else
    // 測定停止
    emit requestToSuspendAntennaMeasuring();
#endif

    _model->buzzer(LC70_BUZZ_ERR, LC70_BUZZ_MODE_ONESHOT, true);

    emit requestToShowPopupWithMessage(POPUP_POWER_SUPPLY_ERROR_MANUAL_MEASURING, message);
}

void AntennaAdjustment::onNotifyStopMeasuring(AcquisitionTarget target)
{
    if (target != ANTENNA) {
        return;
    }

    // アンテナ調整の停止を受信

}

void AntennaAdjustment::setBoolProperty(QString objectName, QString propertyName, bool value)
{
    QObject *target = nullptr;
    if (_view) {
        target = _view->findChild<QObject *>(objectName);
    }
    if (target) {
        // 押下前の画像に更新
        target->setProperty(propertyName.toUtf8().constData(), value);
    }
}

void AntennaAdjustment::onInputFileName(PopupId popupId, SaveLocation saveLocation, QString fileName)
{
    Q_UNUSED(popupId);

    QString prefix = "_" + QDateTime::currentDateTime().toString("yyyyMMddhhmmss") + "_" + _takeOver["selectedBand"].toString();
    fileName = fileName + prefix + PNG_FILE_EXTENSION;

    // キャプチャーした一時データを保存する
    emit requestToSaveImage(SINGLE_MEASUREMENT, saveLocation, fileName);
}
