/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef ANTENNAADJUSTMENT_H
#define ANTENNAADJUSTMENT_H

#include "abstractmeasurementviewmodel.h"
#include "correcter.h"

class AntennaAdjustment : public AbstractMeasurementViewModel
{
    Q_OBJECT
public:
    /**
     * @brief コンストラクター
     * @param screenId 画面 ID
     * @param model Model クラスのポインター
     * @param parent 親オブジェクト
     */
    explicit AntennaAdjustment(ScreenId screenId, LevelCheckerModel *&model, QObject *parent = nullptr);

    /**
     * @brief デストラクタ
     */
    ~AntennaAdjustment();

    /**
     * @brief 画面を開くときに呼び出すスレッドの開始、再開処理
     */
    virtual void resume(void) override;

    /**
     * @brief 画面を閉じるときに呼び出すスレッドの停止処理
     */
    virtual void suspend(void) override;

    /**
     * @brief 画面を開くときに呼び出す qml との接続処理
     * @param qml qml のオブジェクトのポインター
     * @param mainFrame MainFrameViewModel のポインター
     */
    virtual void createConnection(QObject *qml, QObject *mainFrame) override;

    /**
     * @brief 画面を開くときに呼び出す表示内容の復元処理
     */
    virtual void activate(void) override;

    /**
     * @brief 画面を閉じるときに呼び出す表示な用の保存処理
     */
    virtual void deactivate(void) override;

    /**
     * @brief メニューバーによる設定変更
     * @note スレッドは止めずに変更した設定で表示し直す
     */
    void changeByMenuBar(void) override;

signals:
    /**
     * @brief 測定結果取得開始、再開依頼を通知
     * @param condition アンテナ調整測定条件(マルチ測定の形式)
     * @param prefferedCondition 優先測定条件
     * @param interval 測定結果取得間隔
     */
    void resumeAntennaMeasuring(LC70_MEAS_MULTI_T *condition, uint32_t interval);

    /**
     * @brief チャンネル変更
     * @param condition チャンネル別測定条件
     */
    void changeCondition(LC70_MEAS_MULTI_T *condition);

    /**
     * @brief メニューバーのカーソルのあるボタンを押した状態にする
     * @param rank 階層
     */
    void cursorPressed(QVariant rank);

    /**
     * @brief 画面共通情報設定要求
     * @param key 画面共通情報キー
     * @param value 画面共通情報データ
     */
    void requestSetTakeOver(QVariant key, QVariant value);

    /**
     * @brief 画面のキャプチャー依頼
     */
    void requestToCapture(void);

    /**
     * @brief キャプチャーした画像の保存依頼
     * @param measureType 測定種類
     * @param saveLocation 保存先
     * @param fileName ファイル名
     */
    void requestToSaveImage(MeasureType measureType, SaveLocation saveLocation, QString fileName);

    /**
     * @brief アンテナ調整測定停止依頼
     */
    void requestToSuspendAntennaMeasuring(void);

    /**
     * @brief Level の閾値の通知
     * @param red 受信不可の割合
     * @param yellow 受信可の割合
     * @param selection 0: JCSAT-3A 1: JCSAT-4B
     */
    void notifyLevelThreshold(QVariant red, QVariant yellow, QVariant selection);

    /**
     * @brief LEVEL メーター更新(JCSAT3A)
     * @param level LEVEL の値
     * @param levelNeedlePosition LEVEL メーターの針の角度
     */
    void notifyJcsat3aLevelMeter(QVariant level, QVariant levelNeedlePosition);

    /**
     * @brief LEVEL メーター更新(JCSAT4B)
     * @param level LEVEL の値
     * @param levelNeedlePosition LEVEL メーターの針の角度
     */
    void notifyJcsat4bLevelMeter(QVariant level, QVariant levelNeedlePosition);

    /**
     * @brief キャプチャーした画像の削除依頼
     */
    void requestToClearCaptureImage(void);

protected slots:
    /**
     * @brief ボタン押下の状態を受信
     * @param id ボタン ID
     */
    virtual void onKeyStatus(ButtonId id) override;

    /**
     * @brief ポップアップから受諾の通知を受信
     * @param okOnly OK のみの場合 true
     */
    virtual void onAccept(PopupId popupId, bool okOnly) override;

    /**
     * @brief ポップアップから選択の通知を受信
     * @param index 選択されたインデックス
     */
    virtual void onSelect(PopupId popupId, int index) override;

    /**
     * @brief ポップアップから拒否の通知を受信
     */
    virtual void onReject(PopupId popupId) override;

    /**
     * @brief ボタン押下を受信
     * @param id ボタン ID
     */
    virtual void onButtonPress(ButtonId id) override;

    /**
     * @brief バッテリ低下状態変化通知
     * @param isLowBattery バッテリ低下状態かどうか
     */
    void onLowBatteryStatusChanged(bool isLowBattery) override;

    /**
     * @brief スキャン中止依頼を受信
     */
    virtual void onRequestToStopScanning(void) override;

private slots:
    /**
     * @brief アンテナ調整測定結果を受信
     * @param result 測定結果
     */
    void onAntennaMeasureResult(MultiResult result);

    /**
     * @brief チャンネル変更中通知を受信
     * @param changing 変更中か否か
     */
    void onNotifyChannelChanging(bool changing);

    /**
     * @brief 給電異常検知
     * @param code 故障コード
     * @param action 故障時の動作
     * @param message 表示メッセージ
     */
    void detectPowerSupplyError(uint16_t code, uint8_t action, QString message);

    /**
     * @brief 測定停止を受信
     * @param target 測定の種類
     */
    void onNotifyStopMeasuring(AcquisitionTarget target);

    /**
     * @brief メーター表示更新(アニメーション用)
     */
    void onUpdateMeter(void);

private:
    /**
     * @brief Band ごとのインデックス
     */
    enum BandIndex {
        JCSAT3A_INDEX = 0,                      //!< JCSAT-3A
        JCSAT4B_INDEX                           //!< JCSAT-4B
    };

    LC70_MEAS_MULTI_T _condition;               //!< マルチ測定条件(アンテナ調整用)
    LC70_MEAS_MULTI_T _previousCondition;       //!< 前回のマルチ測定条件(アンテナ調整用)
    uint16_t _broadBandSystem;                  //!< 放送方式
    bool _isChannelChanging;                    //!< チャンネル変更中
    bool _thresholdSetJCSAT3A;                  //!< JCSAT-3A の閾値設定済みか否か
    bool _thresholdSetJCSAT4B;                  //!< JCSAT-4B の閾値設定済みか否か
    uint8_t _bandIndex;                         //!< 現在計測している Band を識別するためのインデックス
    bool _isLoaded;                             //!< 画面ロード後か否か
    QVariant _cnrMerSetting;                    //!< CNR/MER 切替設定
    bool _alreadySuspended;                     //!< 既に停止しているか否か
    PowerSupplyType _powerSupplyType;           //!< 給電種類
    bool _isMeasuring;                          //!< 測定中か否か
    uint16_t _failureCode;                      //!< 故障コード
    uint8_t _actionInFailure;                   //!< 故障時の動作
    float32_t _jcsat3aLevel;                    //!< 表示している値(LEVEL)(JCSAT3A)
    float32_t _newestJcsat3aLevel;              //!< 最新の値(LEVEL)(JCSAT3A)
    float32_t _jcsat4bLevel;                    //!< 表示している値(LEVEL)(JCSAT4b)
    float32_t _newestJcsat4bLevel;              //!< 最新の値(LEVEL)(JCSAT4B)
    bool _isUpdateMeter;                        //!< メーター更新稼働中フラグ
    int _meterUpdateInterval;                   //!< メーター更新周期
    int _invalidValueMeterSpeed;                //!< 入力無効時のメーター針移動倍率

    /**
     * @brief チャンネル変更
     */
    void changeChannel();

    /**
     * @brief 表示クリア
     * @parm measurementValuleOnly 測定結果のみ
     */
    void clear(bool measurementValuleOnly = false);

    /**
     * @brief 閾値の反映
     * @param bandId Band の ID
     */
    void refreshThreshold(const int &bandId);

    /**
     * @brief プロパティに Bool 値を設定
     * @param objectName オブジェクト名
     * @param propertyName プロパティ名
     * @param value 設定する Bool 値
     */
    void setBoolProperty(QString objectName, QString propertyName, bool value);

    /**
     * @brief LEVEL メーター更新
     * @param bandIndex Band Index
     * @param levelString LEVELの表示値
     * @param levelNeedle LEVELメーターの針の角度
     */
    void updateLevelMeter(const BandIndex bandIndex, QString &levelString, float32_t &levelNeedle);

private slots:
    /**
     * @brief 入力されたファイル名を受信
     * @param popupId ポップアップの種類
     * @param saveLocation 保存先
     * @param fileName 入力されたファイル名
     */
    void onInputFileName(PopupId popupId, SaveLocation saveLocation, QString fileName);
};

#endif // ANTENNAADJUSTMENT_H
