/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include "bandviewmodel.h"

QVector<QVector<QString>> singleMeasureBandButtonData = {
    { {"m1"},  {"m2"}, {"m3"},  {"m4"},  {"m5"}},
    { {""},  {"m7"},   {"m8"},  {"m9"},  {""}  },
    { {"m6"}, {"m12"},   {"m13"}, {"m14"}, {""}  }
};

QVector<QVector<QString>> multiMeasureBandButtonData = {
    { {"m1"},  {"m2"}, {"m3"}, {"m4"},  {"m5"}},
    { {""},  {"m7"},   {"m8"}, {"m9"},  {""}},
    { {"m6"}, {"m12"},   {""},   {"m14"}, {""}}
};

QVector<QVector<QString>> opticalMeasureBandButtonData = {
    { {"m1"},  {"m2"}, {"m3"},  {"m4"},  {"m5"}},
    { {""},  {"m7"},   {"m8"},  {"m9"},  {""}  },
    { {"m6"}, {"m12"},   {""}, {"m14"}, {""}  }
};


const int BandViewModel::BUTTON_COLUMN_NUMBER = 3;
const int BandViewModel::BUTTON_ROW_NUMBER    = 5;


BandViewModel::BandViewModel(ScreenId screenId, LevelCheckerModel *&model, QObject *parent)
    : AbstractViewModel(screenId, model, parent)
    , _selectedMeasurement("")
{
    setObjectName("bandViewModel");
}

BandViewModel::~BandViewModel()
{

}

void BandViewModel::activate()
{
    qDebug() << Q_FUNC_INFO << "activate";
    connect(this, SIGNAL(requestSetTakeOver(QVariant, QVariant)), _view, SLOT(backup(QVariant, QVariant)));
    connect(this, SIGNAL(requestGetTakeOver()), _view, SLOT(getTakeOver()));
    connect(this, SIGNAL(removeAttribute(QString)), _mainFrame, SLOT(onRemoveAttribute(QString)));
    emit requestGetTakeOver();

    QObject *target = nullptr;
    if (_view) {
        target = _view->findChild<QObject *>("takeOverData");
    }
    if (target) {
        QVariant measurement = target->property("selectedMeasurement");

        // アンテナ調整の場合はシングル測定に読み替える。
        if (measurement == QString("AntennaAdjustment")) {
            measurement = QString(SINGLE_MEASURE_TYPE_NAME);
            //  アンテナ調整からEscapeボタンを押下し、BAND画面に戻って来た場合
            //  測定選択情報もシングルに更新をして置かなければ、
            //  次測定画面に遷移する際に不整合が発生するため、画面共有情報をシングル測定に設定する。
            emit requestSetTakeOver("selectedMeasurement", QString(SINGLE_MEASURE_TYPE_NAME));
        }

        if (_selectedMeasurement != measurement.toString()) {
            // シングル測定では、アンテナ調整は選択できない。
            // 測定設定が異なる場合は、左上のボタンをActiveに設定する。
            _activeButton.clear();
        }

        _selectedMeasurement = measurement.toString();
    }

    // 選択した測定に合わせてボタンの設定を更新する
    setButtonSetting();

    // スクリーン名を更新する
    setScreenName();

    // ※シングル測定の場合は、JCSAT アンテナ調整のボタンを使用不可で表示する
    // ※光・同軸種別が光の場合は、スカパープレミアムグループの３つのボタンと
    //   その他グループの３つのボタンは使用不可（トーンダウン）で表示する
    showHeaderArea();

    // BAND選択画面以降に設定する画面共有情報(選択BAND、偏波面、変調方式、局発周波数)情報をクリアする
    emit removeAttribute("selectedBand");
    emit removeAttribute("selectedModulation");
    emit removeAttribute("selectedLocalOscillator");
    emit removeAttribute("selectedPolarization");
}

void BandViewModel::deactivate()
{
    AbstractViewModel::deactivate();

    disconnect(this, SIGNAL(requestSetTakeOver(QVariant, QVariant)), _view, SLOT(backup(QVariant, QVariant)));
    disconnect(this, SIGNAL(requestGetTakeOver()), _view, SLOT(getTakeOver()));
    disconnect(this, SIGNAL(removeAttribute(QString)), _mainFrame, SLOT(onRemoveAttribute(QString)));
}

void BandViewModel::setTakeOver()
{
    // BAND は LC70_BAND_T のメンバーを値とする
    if (_activeButton == "m2") {
        emit requestSetTakeOver("selectedBand", LC70_BAND_BS);
    } else if (_activeButton == "m7") {
        emit requestSetTakeOver("selectedBand", LC70_BAND_CS);
    } else if (_activeButton == "m4") {
        emit requestSetTakeOver("selectedBand", LC70_BAND_SCCB);
    } else if (_activeButton == "m9") {
        emit requestSetTakeOver("selectedBand", LC70_BAND_SCCC);
    } else if (_activeButton == "m3") {
        emit requestSetTakeOver("selectedBand", LC70_BAND_JCSAT3);
    } else if (_activeButton == "m8") {
        emit requestSetTakeOver("selectedBand", LC70_BAND_JCSAT4);
    } else if (_activeButton == "m1") {
        emit requestSetTakeOver("selectedBand", LC70_BAND_UHF);
    } else if (_activeButton == "m13") {
        emit requestSetTakeOver("selectedBand", LC70_BAND_JCSAT3 * LC70_BAND_JCSAT4);
    } else if (_activeButton == "m14") {
        emit requestSetTakeOver("selectedBand", LC70_BAND_LTE);
    } else if (_activeButton == "m5") {
        emit requestSetTakeOver("selectedBand", LC70_BAND_FM);
    } else if (_activeButton == "m12") {
        emit requestSetTakeOver("selectedBand", LC70_BAND_BS_CS);


    } else {}
}

void BandViewModel::moveCursor(ButtonId buttonId)
{
    if ((buttonId == BUTTON_UP) || (buttonId == BUTTON_DOWN) || (buttonId == BUTTON_RIGHT) || (buttonId == BUTTON_LEFT)) {

        QString nextButton;
        bool ret = _buttonOperation.nextActiveObject(buttonId, nextButton);

        if (ret) {

            // inactive
            QObject *target = nullptr;
            if (_view) {
                target = _view->findChild<QObject *>(_activeButton);
            }
            if (target) {
                target->setProperty("active", false);
            }

            // active
            target = nullptr;
            if (_view) {
                target = _view->findChild<QObject *>(nextButton);
            }
            if (target) {
                target->setProperty("active", true);
            }

            _activeButton = nextButton;
            _buttonOperation.setActiveObject(_activeButton);
        }
    }
}

void BandViewModel::onKeyStatus(ButtonId id)
{
    // Home, Escape ボタンはベースクラスで対応
    AbstractViewModel::onKeyStatus(id);

    if (id == BUTTON_ENTER) {
        enterKey();
    } else if (id == BUTTON_USER) {
#ifdef NEW_USER_SETTING
        emit requestDirect(SCREEN_USER_SETTING, true);
#else
        emit requestDirect(SCREEN_USER_DATA_LOGGER_MENU, true);
#endif
    } else {
        moveCursor(id);
    }
}

void BandViewModel::onButtonPress(ButtonId id)
{
    if (id == BUTTON_ENTER) {
        QObject *target = nullptr;
        if (_view) {
            target = _view->findChild<QObject *>(_activeButton);
        }
        if (target) {
            target->setProperty("press", true);
        }
    }
}

void BandViewModel::enterKey()
{
    int buttonIndex = _activeButton.mid(1).toInt();


    // 選択したBANDアイコンを追加する
    addSelectBandIcon(buttonIndex);

    setTakeOver();

    switch (buttonIndex) {
    case 2:     // BS
    case 7:    // N-SAT110
        onRequest(0);     // 偏波面選択画面 : 右旋、左旋
        break;

    case 3:     // JCSAT3
    case 4:     // SCC-B
    case 8:     // JCSAT4
    case 9:     // SCC-C
        onRequest(0);     // 偏波面選択画面 : 水平、垂直
        break;

    case 13:    // JCSAT ｱﾝﾃﾅ調整
        // アンテナ調整に遷移させるため選択された測定を変更する
        emit requestSetTakeOver("selectedMeasurement", "AntennaAdjustment");
        onRequest(0);     // 偏波面選択画面 : 水平、垂直
        break;

    case 6:     // CATV
        onRequest(1);     // CATV メニュー
        break;

    case 1:     // 地デジ
    case 14:    // LTE
    case 5:     // FM
        if (_selectedMeasurement == "SingleMeasurement") {
            onRequest(2);   // シングルチャンネル
        } else if (_selectedMeasurement == "MultiMeasurement") {
            onRequest(3);   // マルチチャンネル
        }
        break;

    case 12:  // BS/CS統合
        if (_selectedMeasurement == "SingleMeasurement") {
            onRequest(2);   // シングルチャンネル
        } else if (_selectedMeasurement == "MultiMeasurement") {
            onRequest(3);   // マルチチャンネル
        }
        break;

    default:
        break;
    }
}

void BandViewModel::onRequest(int condition)
{
    if (condition == 0 || condition == 1) {
        emit requestSetTakeOver("autoSettingPolarization", false);
    }
    emit request(_screenId, condition);
    return;
}

void BandViewModel::showHeaderArea()
{
    if (_selectedMeasurement == "SingleMeasurement") {
        _model->replaceMenuList(MenuListIconIndex::FUNCTION_ICON, "icon_menu_single.png");
    } else if (_selectedMeasurement == "MultiMeasurement") {
        _model->replaceMenuList(MenuListIconIndex::FUNCTION_ICON, "icon_menu_multi.png");
    }

    // BAND選択画面遷移時は、必ずヘッダーのBANDアイコンをクリアする
    _model->removeMenuList(MenuListIconIndex::BAND_ICON);
}

void BandViewModel::setScreenName()
{
    QObject *target = nullptr;
    if (_view) {
        target = _view->findChild<QObject *>("screenName");
    }
    if (_selectedMeasurement == "SingleMeasurement") {
        if (target) {
            target->setProperty("name", "シングル測定");
        }
    } else if (_selectedMeasurement == "MultiMeasurement") {
        if (target) {
            target->setProperty("name", "マルチ測定");
        }
    }
    //  Home → USER データロガー経由
    // else if (_selectedMeasurement == "UserSetting") {}
    else {}
}

void BandViewModel::setButtonSetting()
{
    QObject *target = nullptr;

    QVariant valueopt;
    int rfOptical = 0;
    _model->getValue(VALUE_RF_OPTICAL, valueopt);
    rfOptical = valueopt.toInt();


    if (_selectedMeasurement == "SingleMeasurement") {
        // シングル測定選択時
        if(rfOptical == 1){
            _buttonOperation.setButtonName(BUTTON_COLUMN_NUMBER, BUTTON_ROW_NUMBER, opticalMeasureBandButtonData);
            if (_view) {
                target = _view->findChild<QObject *>("m13");
            }
            if(target) {
                    target->setProperty("disable",true);
            }

        }else{
            _buttonOperation.setButtonName(BUTTON_COLUMN_NUMBER, BUTTON_ROW_NUMBER, singleMeasureBandButtonData);
            if (_view) {
                target = _view->findChild<QObject *>("m13");
            }

            if (target) {
                target->setProperty("visible", true);
                target->setProperty("disable", false);
            }

        }

        // シングル測定選択時は、アンテナ調整ボタン、FMボタンを活性にする。


        target = nullptr;
        if (_view) {
            target = _view->findChild<QObject *>("m5");
        }

        if (target) {
            target->setProperty("disable", false);
        }

    } else if (_selectedMeasurement == "MultiMeasurement") {
        // マルチ測定選択時
        _buttonOperation.setButtonName(BUTTON_COLUMN_NUMBER, BUTTON_ROW_NUMBER, multiMeasureBandButtonData);

        // マルチ測定では、アンテナ調整ボタンは非活性、FMボタンを活性にする。
        if (_view) {
            target = _view->findChild<QObject *>("m13");
        }

        if (target) {
            target->setProperty("visible", false);
            target->setProperty("disable", true);
        }

        target = nullptr;
        if (_view) {
            target = _view->findChild<QObject *>("m5");
        }
        if (target) {
            target->setProperty("disable", false);
        }
    }

    // 画面遷移時は、先頭のボタンをアクティブにする
    QString defaultButtonName;
    if (_selectedMeasurement == "SingleMeasurement") {
        defaultButtonName = singleMeasureBandButtonData[0][0];
    } else {
        defaultButtonName = multiMeasureBandButtonData[0][0];
    }

    resetToInitialButtonState(defaultButtonName, _activeButton, _buttonOperation);
}

void BandViewModel::addSelectBandIcon(int selectButtonIndex)
{
    // BANDアイコンの置き換えを実施する
    switch (selectButtonIndex) {
    case 1:  // 地デジ
        _model->replaceMenuList(MenuListIconIndex::BAND_ICON, "icon_band_chidigi.png");
        break;

    case 3:  // JCSAT3
        _model->replaceMenuList(MenuListIconIndex::BAND_ICON, "icon_band_3a.png");
        break;

    case 4:  // SCC-B
        _model->replaceMenuList(MenuListIconIndex::BAND_ICON, "icon_band_b.png");
        break;

    case 5:  // FM
        _model->replaceMenuList(MenuListIconIndex::BAND_ICON, "icon_band_fm.png");
        break;

    case 2:  // BS
        _model->replaceMenuList(MenuListIconIndex::BAND_ICON, "icon_band_bs.png");
        break;

    case 8:  // JCSAT4
        _model->replaceMenuList(MenuListIconIndex::BAND_ICON, "icon_band_4b.png");
        break;

    case 9:  // SCC-C
        _model->replaceMenuList(MenuListIconIndex::BAND_ICON, "icon_band_c.png");
        break;

    case 7:  // CS
        _model->replaceMenuList(MenuListIconIndex::BAND_ICON, "icon_band_cs.png");
        break;

    case 13: // JCSATアンテナ調整
        _model->replaceMenuList(MenuListIconIndex::BAND_ICON, "icon_band_antenna ad .png");
        break;

    case 14: // LTE
        _model->replaceMenuList(MenuListIconIndex::BAND_ICON, "icon_band_lte.png");
        break;

    case 12: // BS CS統合
        _model->replaceMenuList(MenuListIconIndex::BAND_ICON, "icon_band_bs.png");
        break;

    default:
        break;
    }
}
