/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include "catvviewmodel.h"
#include "mainframeviewmodel.h"
#include "correcter.h"

// マニュアルシングル測定から遷移した場合のボタン配置
static QVector<QVector<QString>> BUTTON_DATA_MANUAL_SINGLE = {
    { {"m1"}, {"m2"}, {"m3"} },
    { {"m4"}, {"m5"}, {"m6"} }
};

// マニュアルマルチ測定から遷移した場合のボタン配置
static QVector<QVector<QString>> BUTTON_DATA_MANUAL_MULTI = {
    {{"m1"},{"m2"},{"m3"}},
    {{"m4"},{"m5"},{"m6"}}
};

// 閾値設定画面から遷移した場合のボタン配置
static QVector<QVector<QString>> BUTTON_DATA_THRESHOLD_SETTING = {
    { {"m1"}, {""},  {""}   },
    { {"m4"}, {"m5"},{"m6"} }
};

// 自動設定画面から遷移した場合のボタン配置
static QVector<QVector<QString>> BUTTON_DATA_AUTO_SETTING = {
    {{"m1"}, {"m2"},{"m3"}},
    {{"m4"}, {""}, {""}},
};

const int CATVViewModel::BUTTON_COLUMN_NUMBER = 2;
const int CATVViewModel::BUTTON_ROW_NUMBER = 3;

CATVViewModel::CATVViewModel(ScreenId screenId, LevelCheckerModel *&model, QObject *parent)
    : AbstractViewModel(screenId, model, parent)
    , _selectedMeasurement()
    , _fromAutoSetting(false)
    , _channelList()
    , _broadBandModulationSetting()
    , _powerSupply(LC70_POWER_SUPPLY_OFF)
    , _failureCode(0)
    , _actionInFailure(LC70_FAIL_CTL_NONE)
{
    setObjectName("catvViewModel");
}

CATVViewModel::~CATVViewModel()
{

}

void CATVViewModel::activate()
{
    connect(this, SIGNAL(requestSetTakeOver(QVariant, QVariant)), _view, SLOT(backup(QVariant, QVariant)));
    connect(this, SIGNAL(removeAttribute(QString)), _mainFrame, SLOT(onRemoveAttribute(QString)));
    connect(_mainFrame, SIGNAL(requestToScan()), this, SLOT(onRequestToScan()));
    connect(_mainFrame, SIGNAL(requestToStopScanning()), this, SLOT(onRequestToStopScanning()));
    connect(_mainFrame, SIGNAL(notifyPowerSupplyError(uint16_t, uint8_t, QString)), this, SLOT(detectPowerSupplyError(uint16_t, uint8_t, QString)));
    connect(_model, SIGNAL(scanningFinished(bool)), this, SLOT(onScanningFinished(bool)));
    connect(_model, SIGNAL(scanningChannelResult(int, LC70_MEAS_MULTI_RESULT_T)), this, SLOT(onScanningChannelResult(int,
            LC70_MEAS_MULTI_RESULT_T)));
    connect(this, SIGNAL(progress(double)), _mainFrame, SIGNAL(progress(double)));
    connect(this, SIGNAL(requestToReScan()), this, SLOT(onRequestToReScan()), Qt::QueuedConnection);
    connect(this, SIGNAL(requestToShowRegistScanResultPopup()), this, SLOT(onRequestToShowRegistScanResultPopup()), Qt::QueuedConnection);

    // マニュアル測定 放送方式, 変調方式選択の取得
    _broadBandModulationSetting.clear();
    _model->getValue(VALUE_BROADBANDSYSTEM_MODULATION, _broadBandModulationSetting);

    // ユーザー設定自動以外からユーザー設定自動に切り替えた場合は前に選択したボタンをクリアー
    if (_fromAutoSetting != _takeOver["autoSettingPolarization"].toBool()) {
        _activeButton.clear();
    }
    _fromAutoSetting = _takeOver["autoSettingPolarization"].toBool();

    if (_takeOver["selectedMeasurement"].toString().isEmpty() && _fromAutoSetting != true) {
        // 測定設定のBAND画面から遷移してきた場合
        // 選択できないボタンがactiveボタンとして選択されていた場合は、
        // activeボタンをクリアし、先頭のボタンを選択するようにする。
        if (_activeButton == "m2" || _activeButton == "m3") {
            _activeButton.clear();
        }
    }

    // シングル/マルチ測定を切り替えた場合は前に選択したボタンをクリアー
    if (_takeOver["selectedMeasurement"].isValid()
            && _selectedMeasurement != _takeOver["selectedMeasurement"].toString()) {
        _activeButton.clear();
    }

    _selectedMeasurement = _takeOver["selectedMeasurement"].toString();

    // [Memo]
    // CATVチャンネル周波数画面に遷移して来た際には、BANDアイコンをクリアする
    _model->removeMenuList(MenuListIconIndex::BAND_ICON);

    // 選択した測定に合わせてボタンの設定を更新する
    setButtonSetting();
}

void CATVViewModel::setButtonSetting()
{
    QObject *target = nullptr;

    if (!_selectedMeasurement.isEmpty()) {
        if (_selectedMeasurement == SINGLE_MEASURE_TYPE_NAME) {
            // シングル測定から遷移して来た場合
            if (_view) {
                _view->setProperty("transitionMode", SINGLE_MEASUREMENT_MODE);
            }
            // ボタン操作情報を更新
            _buttonOperation.setButtonName(BUTTON_COLUMN_NUMBER, BUTTON_ROW_NUMBER, BUTTON_DATA_MANUAL_SINGLE);
        } else if (_selectedMeasurement == MULTI_MEASURE_TYPE_NAME) {
            // マルチ測定から遷移して来た場合
            if (_view) {
                _view->setProperty("transitionMode", MULTI_MEASUREMENT_MODE);
            }
            // ボタン操作情報を更新
            _buttonOperation.setButtonName(BUTTON_COLUMN_NUMBER, BUTTON_ROW_NUMBER, BUTTON_DATA_MANUAL_MULTI);
        } else {
            // nothing to do
        }
    } else {
        if (_fromAutoSetting) {
            // 自動設定から遷移してきた場合
            if (_view) {
                _view->setProperty("transitionMode", AUTO_SETTING_MODE);
            }
            // ボタン操作情報を更新
            _buttonOperation.setButtonName(BUTTON_COLUMN_NUMBER, BUTTON_ROW_NUMBER, BUTTON_DATA_AUTO_SETTING);
        } else {
            // 閾値設定画面から遷移してきた場合
            if (_view) {
                _view->setProperty("transitionMode", THRESHOLD_SETTING_MODE);
            }
            // ボタン操作情報を更新
            _buttonOperation.setButtonName(BUTTON_COLUMN_NUMBER, BUTTON_ROW_NUMBER, BUTTON_DATA_THRESHOLD_SETTING);
        }
    }

    // 先頭のボタンをアクティブにする
    resetToInitialButtonState("m1", _activeButton, _buttonOperation);
}

void CATVViewModel::deactivate()
{
    AbstractViewModel::deactivate();

    disconnect(this, SIGNAL(requestSetTakeOver(QVariant, QVariant)), _view, SLOT(backup(QVariant, QVariant)));
    disconnect(this, SIGNAL(removeAttribute(QString)), _mainFrame, SLOT(onRemoveAttribute(QString)));
    disconnect(this, SIGNAL(requestSetTakeOver(QVariant, QVariant)), _view, SLOT(backup(QVariant, QVariant)));
    disconnect(this, SIGNAL(removeAttribute(QString)), _mainFrame, SLOT(onRemoveAttribute(QString)));
    disconnect(_mainFrame, SIGNAL(requestToScan()), this, SLOT(onRequestToScan()));
    disconnect(_mainFrame, SIGNAL(requestToStopScanning()), this, SLOT(onRequestToStopScanning()));
    disconnect(_mainFrame, SIGNAL(notifyPowerSupplyError(uint16_t, uint8_t, QString)), this, SLOT(detectPowerSupplyError(uint16_t, uint8_t, QString)));
    disconnect(_model, SIGNAL(scanningFinished(bool)), this, SLOT(onScanningFinished(bool)));
    disconnect(_model, SIGNAL(scanningChannelResult(int, LC70_MEAS_MULTI_RESULT_T)), this, SLOT(onScanningChannelResult(int,
               LC70_MEAS_MULTI_RESULT_T)));
    disconnect(this, SIGNAL(progress(double)), _mainFrame, SIGNAL(progress(double)));
    disconnect(this, SIGNAL(requestToReScan()), this, SLOT(onRequestToReScan()));
    disconnect(this, SIGNAL(requestToShowRegistScanResultPopup()), this, SLOT(onRequestToShowRegistScanResultPopup()));
}

void CATVViewModel::suspend()
{
    // 給電を停止
    _powerSupply = LC70_POWER_SUPPLY_OFF;
    _model->setPowerSupply(_powerSupply);
    _model->removeMenuList(MenuListIconIndex::POWER_SUPPLY_ICON);

    _failureCode = 0;

    AbstractViewModel::suspend();
}

void CATVViewModel::setTakeOver()
{
    emit requestSetTakeOver("selectedBand", getActiveBandId());
}

void CATVViewModel::onKeyStatus(ButtonId id)
{
    // Home, Escape ボタンはベースクラスで対応
    AbstractViewModel::onKeyStatus(id);

    if (id == BUTTON_ENTER) {
        enterKey();
    } else if (id == BUTTON_USER) {
#ifdef NEW_USER_SETTING
        emit requestDirect(SCREEN_USER_SETTING, true);
#else
        emit requestDirect(SCREEN_USER_DATA_LOGGER_MENU, true);
#endif
    } else {
        moveCursor(id);
    }
}

void CATVViewModel::onButtonPress(ButtonId id)
{
    if (id == BUTTON_ENTER) {
        QObject *target = nullptr;
        if (_view) {
            target = _view->findChild<QObject *>(_activeButton);
        }
        if (target) {
            target->setProperty("activebg", "catv_bandbtn_p.png");
        }
    }
}

void CATVViewModel::onRequest(int condition)
{
    emit request(_screenId, condition);
}

void CATVViewModel::moveCursor(ButtonId buttonId)
{
    if ((buttonId == BUTTON_RIGHT) || (buttonId == BUTTON_LEFT) || (buttonId == BUTTON_UP) || (buttonId == BUTTON_DOWN)) {

        QString nextButton;
        bool ret = _buttonOperation.nextActiveObject(buttonId, nextButton);

        if (ret) {

            // inactive
            QObject *target = nullptr;
            if (_view) {
                target = _view->findChild<QObject *>(_activeButton);
            }
            if (target) {
                target->setProperty("active", false);
            }

            // active
            target = nullptr;
            if (_view) {
                target = _view->findChild<QObject *>(nextButton);
            }
            if (target) {
                target->setProperty("active", true);
            }

            _activeButton = nextButton;
            _buttonOperation.setActiveObject(_activeButton);
        }
    }
}

void CATVViewModel::enterKey()
{
    // [Memo]
    //  USER設定、自動設定から遷移してきた場合は、画面遷移は実施せず、
    //  給電確認を行う
    int buttonIndex = _activeButton.mid(1).toInt();

    if (_fromAutoSetting) {
        if (isLowBattery()) {
            // バッテリ低下状態中は給電開始させない
            onRequestScanConfirm();
            return;
        }
        emit requestToShowPopup(POPUP_POWER_SUPPLY_SELECT2);
    } else {

        // [Memo]
        //  USER設定 自動設定の場合はBANDアイコンは更新しない
        // 選択したBANDアイコンを追加する
        addSelectBandIcon(buttonIndex);

        switch (buttonIndex) {
        case 1:
        case 2:
        case 3:
            onRequest(0);
            break;
        case 4: // スカパー光
            if (_selectedMeasurement.isEmpty()) {
                // 測定設定からCATV BAND画面に遷移した場合は、変調方式選択画面に遷移する。
//                onRequest(0); // 変調方式選択画面
                // 変調方式を256QAMに自動選択して測定設定画面に遷移する。
                emit requestSetTakeOver("selectedModulation", TWO_HUNDRED_FIFTY_SIX_QAM);
                onRequest(3); // 閾値設定画面

            } else {
                // 測定設定以外からCATV BAND選択画面に遷移した場合は、変調方式を256QAMに自動選択して測定設定画面に遷移する。
                emit requestSetTakeOver("selectedModulation", TWO_HUNDRED_FIFTY_SIX_QAM);
                if (_selectedMeasurement == SINGLE_MEASURE_TYPE_NAME) {
                    onRequest(1);   // シングルチャンネル
                } else if (_selectedMeasurement == MULTI_MEASURE_TYPE_NAME) {
                    onRequest(2);   // マルチチャンネル
                }
            }
            break;
        case 5: // PILOT
            if (_selectedMeasurement.isEmpty()) {
                // 測定設定からCATV BAND画面に遷移した場合は、閾値設定画面に遷移する。
                onRequest(3); // 閾値設定画面
            } else {
                // 測定設定以外からCATV BAND選択画面に遷移した場合は、測定設定画面に遷移する。
                if (_selectedMeasurement == SINGLE_MEASURE_TYPE_NAME) {
                    onRequest(1);   // シングルチャンネル
                } else if (_selectedMeasurement == MULTI_MEASURE_TYPE_NAME) {
                    onRequest(2);   // マルチチャンネル
                }
            }
            break;
        case 6: // CATV上り
            if (_selectedMeasurement.isEmpty()) {
                // 測定設定からCATV BAND画面に遷移した場合は、閾値設定画面に遷移する
                onRequest(3); // 閾値設定画面
            } else {
                // 測定設定以外からCATV BAND選択画面に遷移した場合は、測定設定画面に遷移する
                if (_selectedMeasurement == SINGLE_MEASURE_TYPE_NAME) {
                    onRequest(1); // シングルチャンネル
                } else if (_selectedMeasurement == MULTI_MEASURE_TYPE_NAME) {
                        onRequest(2); // マルチチャンネル
                } else {
                    // nothing to do
                }
            }
            break;
        default:
            break;

        }
    }
}

void CATVViewModel::onAccept(PopupId popupId, bool okOnly)
{
    Q_UNUSED(okOnly);

    AbstractViewModel::onAccept(popupId, okOnly);

    if (popupId == POPUP_POWER_SUPPLY_SELECT2) {
        // 給電選択で 15vを選択
        _powerSupply = LC70_POWER_SUPPLY_15V;
        _model->setPowerSupply(_powerSupply);
        _model->replaceMenuList(MenuListIconIndex::POWER_SUPPLY_ICON, Correcter::powerSupplyIcon(_powerSupply));

        // onReject 経路でのスキャン開始は処理を一度切る必要があった
        // 見えるタイミングがことなるため onReject と同じ処理とする
        QTimer::singleShot(100, this, SLOT(onRequestScanConfirm()));
    } else if (popupId == POPUP_POWER_SUPPLY_ERROR_MANUAL_MEASURING) {
        // nothing to do.
    } else if (popupId == POPUP_CONFIRM_REGIST_SCAN_RESULT) {
        // 保存先選択画面へ遷移
#ifdef NEW_USER_SETTING
        // 保存先選択で USER データロガー設定を呼び出す
        emit requestSetTakeOver("userDataloggerFunction", CHANNEL_EDIT_SELECT_DESTINATION);
        emit requestDirect(SCREEN_USER_DATALOGGER_SETTING, false);
#else
        onRequest(4);
#endif
        return;
    } else {
        // nothing to do
    }
}

void CATVViewModel::onReject(PopupId popupId)
{
    Q_UNUSED(popupId);

    if (popupId == POPUP_POWER_SUPPLY_SELECT2) {
        QTimer::singleShot(100, this, SLOT(onRequestScanConfirm()));
    }

    if (_fromAutoSetting) {
        // ボタン押下の状態を元に戻す
        QObject *target = nullptr;
        if (_view) {
            target = _view->findChild<QObject *>(_activeButton);
        }

        if (target) {
            if (_activeButton == "m1") {
                target->setProperty("activebg", "catv_bandbtn_dr.png");
            } else if (_activeButton == "m2") {
                target->setProperty("activebg", "catv_bandbtn_dlr.png");
            } else if (_activeButton == "m3") {
                target->setProperty("activebg", "catv_bandbtn_l.png");
            } else if (_activeButton == "m4") {
                target->setProperty("activebg", "catv_bandbtn_ur.png");
            } else if (_activeButton == "m5") {
                target->setProperty("activebg", "catv_bandbtn_ulr.png");
            } else if (_activeButton == "m6") {
                target->setProperty("activebg", "catv_bandbtn_l.png");
            }
        }

        // 給電中だった場合は給電OFFにする
        if (_powerSupply != LC70_POWER_SUPPLY_OFF) {
            _powerSupply = LC70_POWER_SUPPLY_OFF;
            _model->setPowerSupply(_powerSupply);
            _model->replaceMenuList(MenuListIconIndex::POWER_SUPPLY_ICON, Correcter::powerSupplyIcon(_powerSupply));
        }

        // スキャン結果のアイコンを消去
        _model->removeMenuList(MenuListIconIndex::SCAN_ICON);

    }
}

void CATVViewModel::onSelect(PopupId popupId, int index)
{
    if (popupId == POPUP_SCAN_RESULT) {
        // スキャン結果を採用する
        selectReactionScanResult(index);
        return;
    }
}

void CATVViewModel::onLowBatteryStatusChanged(bool isLowBattery)
{
    if (isLowBattery && _powerSupply != LC70_POWER_SUPPLY_OFF) {
        // 給電を停止する
        _powerSupply = LC70_POWER_SUPPLY_OFF;
        _model->setPowerSupply(_powerSupply);
        _model->removeMenuList(MenuListIconIndex::POWER_SUPPLY_ICON);
        emit requestToShowLowBatteryPopup();
    }
}

int CATVViewModel::getActiveBandId()
{
    LC70_BAND_T bandId = LC70_BAND_UNDEF;

    // BAND は LC70_BAND_Tのメンバーを値とする
    if (_activeButton == "m1") {
        bandId = LC70_BAND_CATV_ALL;
    } else if (_activeButton == "m2") {
        bandId = LC70_BAND_CATV_90_470MHZ;
    } else if (_activeButton == "m3") {
        bandId = LC70_BAND_CATV_470_1002MHZ;
    } else if (_activeButton == "m4") {
        bandId = LC70_BAND_SKYP;
    } else if (_activeButton == "m5") {
        bandId = LC70_BAND_PILOT;
    } else if (_activeButton == "m6") {
        bandId = LC70_BAND_CATV_UP_STREAM;
    } else {}

    return bandId;
}


void CATVViewModel::addSelectBandIcon(int selectButtonIndex)
{
    // BANDアイコンの置き換えを実施する
    switch (selectButtonIndex) {
    case 1:  // CATV ALL
        _model->replaceMenuList(MenuListIconIndex::BAND_ICON, "icon_band_catv_all.png");
        break;

    case 2:  // CATV 90-470
        _model->replaceMenuList(MenuListIconIndex::BAND_ICON, "icon_band_catv_90-470.png");
        break;

    case 3:  // CATV 470-1002
        _model->replaceMenuList(MenuListIconIndex::BAND_ICON, "icon_band_catv_470-1000.png");
        break;

    case 4:  // スカパー光
        _model->replaceMenuList(MenuListIconIndex::BAND_ICON, "icon_band_catv_sky.png");
        break;

    case 5:  // PILOT
        _model->replaceMenuList(MenuListIconIndex::BAND_ICON, "icon_band_catv_pilot.png");
        break;
    case 6:  // CATV上り
        _model->replaceMenuList(MenuListIconIndex::BAND_ICON, "icon_band_catv_up_stream.png");
    default:
        break;
    }
}

void CATVViewModel::onRequestScanConfirm()
{
    emit requestToShowPopup(POPUP_SCAN_CONFIRMATION);
}

void CATVViewModel::onRequestToScan()
{
    // 指定BANDのチャンネルデータを取得
    _model->getChannelDefinitionList(getActiveBandId(), _channelList);

    // 保持したスキャン結果をクリアー
    _model->clearScanResult();

    // 局発周波数
    // CATVは局発周波数を設定しないため固定値0を設定する
    uint32_t localFrequency = 0;

    // 変調方式
    QString selectedModulation =  getScanningModualtion();
    unsigned int modulationValue = Correcter::modulationValue(selectedModulation);
    QString modulation = Correcter::displayModulation(modulationValue);
    uint16_t bdSys = Correcter::getBroadcastSystemCondition(getActiveBandId(), modulation);

    // 放送方式、変調方式の設定(定義ファイル 1.0.1 対応)
    uint16_t mod = Correcter::getModulationCondition(getActiveBandId(), bdSys, 0, modulationValue, _broadBandModulationSetting);

    // スキャン実行を通知
    _model->writeLog(QString("USER・データロガー設定  自動  CATV BAND選択画面"),
                     QString("スキャン開始"));
    _model->scan(_channelList, AcquisitionTarget::NOT_ACQUISITION, localFrequency, bdSys, mod, false, _powerSupply);
}

void CATVViewModel::onRequestToStopScanning()
{
    // スキャン中止
    _model->stopChannelScan();
    _model->writeLog(QString("USER・データロガー設定  自動  CATV BAND選択画面"),
                     QString("スキャン中止"));
}

void CATVViewModel::onScanningChannelResult(int index, LC70_MEAS_MULTI_RESULT_T result)
{
    // ラッパー関数ライブラリ V2.00 対応
    if (result.ucLock == UNLOCKED) {
        // 無効判定
        _channelList[index].valid = false;
    }

    // SCAN時設定した変調方式がOFDMかつ、
    // スキャン結果がOFDM以外の変調方式の場合
    // スキャン結果として登録する変調方式をOFDMに設定する
    int registModulation = result.ulModBitPtrn;
    if (getScanningModualtion() == OFDM && result.ulModBitPtrn != LC70_SETMOD_OFDM) {
        registModulation = LC70_SETMOD_OFDM;
    }

    // スキャン結果の保持
    if (result.ucLock == LOCKED) {
        MeasureResult scanResult;
        scanResult.Received = true;
        scanResult.Band = QString::number(_channelList[index].band);
        scanResult.Channel = _channelList[index].channelNumber;
        scanResult.Status = result.usStat;
        scanResult.Level = result.ulLevel;
        scanResult.Mer = 0;
        scanResult.Ber = 0;
        scanResult.BroadcastSystem = result.usBdSys;
        scanResult.Modulation = registModulation;
        scanResult.Layer = result.ucLayer;
        scanResult.Frequency = -1;
        scanResult.LocalOscillator = 0;
        _model->appendScanResult(scanResult);

        // スキャン時 有効チャンネルが見つかった場合 ブザーを1度鳴動させる
        _model->buzzer(LC70_BUZZ_VOL_CHG, LC70_BUZZ_MODE_ONESHOT);
    }

    // プログレスバーの進捗を設定
    double value = static_cast<double>(index + 1) / static_cast<double>(_channelList.count()) * 100;
    emit progress(value);
}

void CATVViewModel::onScanningFinished(bool complete)
{
    _model->writeLog(QString("USER・データロガー設定  自動  CATV BAND選択画面"),
                     QString("スキャン終了"));

    // 給電中だった場合は、一旦給電OFFにする
    if (_powerSupply != LC70_POWER_SUPPLY_OFF) {
        // 再スキャン時の給電有無を判断するため、_powerSupplyは更新しない
        _model->setPowerSupply(LC70_POWER_SUPPLY_OFF);
        _model->replaceMenuList(MenuListIconIndex::POWER_SUPPLY_ICON, Correcter::powerSupplyIcon(LC70_POWER_SUPPLY_OFF));
    }

    if (complete) {
        // スキャン完了時 ブザーを1度鳴動させる
        _model->buzzer(LC70_BUZZ_ALL_CH_SCAN, LC70_BUZZ_MODE_ONESHOT);

        emit requestToShowPopup(POPUP_SCAN_RESULT);
    } else {
#ifdef NEW_USER_SETTING
        emit requestDirect(SCREEN_USER_SETTING, true);
#else
        emit requestDirect(SCREEN_USER_DATA_LOGGER_MENU, true);
#endif
        _model->removeMenuList(MenuListIconIndex::SCAN_ICON);
    }
}

QString CATVViewModel::getScanningModualtion()
{
    QMap<LC70_BAND_T, QString> scanningModualtionMap = {
        {LC70_BAND_CATV_ALL,         OFDM},
        {LC70_BAND_CATV_90_470MHZ,   OFDM},
        {LC70_BAND_CATV_470_1002MHZ, OFDM},
        {LC70_BAND_SKYP,             OFDM},
        {LC70_BAND_PILOT,            CW  },
    };

    int bandId = getActiveBandId();

    return scanningModualtionMap.value(static_cast<LC70_BAND_T>(bandId), OFDM);
}

void CATVViewModel::selectReactionScanResult(const int index) {
    if (index == 0) {
        // スキャンアイコン表示
        _model->replaceMenuList(MenuListIconIndex::SCAN_ICON, "icon_scan_n.png");

        // 保存先確認ダイアログを表示する
        emit requestToShowRegistScanResultPopup();

    } else {
        // 再スキャン
        emit requestToReScan();
    }
}

void CATVViewModel::onRequestToReScan() {
    // スキャン中ポップアップを表示
    emit requestToShowPopup(POPUP_SCANNING);

    // スキャン開始時に給電が選択されていた場合は、再スキャン中も給電する
    if (_powerSupply != LC70_POWER_SUPPLY_OFF) {
        _model->setPowerSupply(_powerSupply);
        _model->replaceMenuList(MenuListIconIndex::POWER_SUPPLY_ICON, Correcter::powerSupplyIcon(_powerSupply));
    }

    // 再スキャン
    onRequestToScan();
}

void CATVViewModel::detectPowerSupplyError(uint16_t code, uint8_t action, QString message)
{
    _failureCode = code;
    _actionInFailure = action;
    // 給電を停止する
    _powerSupply = LC70_POWER_SUPPLY_OFF;
    _model->setPowerSupply(_powerSupply);
    _model->removeMenuList(MenuListIconIndex::POWER_SUPPLY_ICON);

    if (_actionInFailure == LC70_FAIL_CTL_MSGDISP_CLR) {
        uint8_t addInfo[LC70_FAIL_ADD_INFO_NUM];
        memset(addInfo, 0x00, LC70_FAIL_ADD_INFO_NUM);
        //_model->setFaultInformation(_failureCode, LC70_FAIL_STAT_NORMAL, addInfo);     故障復旧処理不要
        _failureCode = 0;
        _actionInFailure = LC70_FAIL_CTL_NONE;
    }

    _model->buzzer(LC70_BUZZ_ERR, LC70_BUZZ_MODE_ONESHOT, true);

    emit requestToShowPopupWithMessage(POPUP_POWER_SUPPLY_ERROR_MANUAL_MEASURING, message);
}

void CATVViewModel::onRequestToShowRegistScanResultPopup() {
    emit requestToShowPopup(POPUP_CONFIRM_REGIST_SCAN_RESULT);
}
