/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef CATVVIEWMODEL_H
#define CATVVIEWMODEL_H

#include "abstractviewmodel.h"
#include "buttonid.h"
#include "buttonoperation.h"

// 画面遷移モード
enum SCREEN_TRANSITION_MODE {
    SINGLE_MEASUREMENT_MODE = 0,    // シングル測定
    MULTI_MEASUREMENT_MODE,         // マルチ測定
    THRESHOLD_SETTING_MODE,         // 閾値設定
    AUTO_SETTING_MODE               // 自動設定
};

class CATVViewModel : public AbstractViewModel
{
    Q_OBJECT
public:
    /**
     * @brief コンストラクター
     * @param screenId 画面 ID
     * @param model Model クラスのポインター
     * @param parent 親オブジェクト
     */
    explicit CATVViewModel(ScreenId screenId, LevelCheckerModel *&model, QObject *parent = nullptr);

    /**
     * @brief デストラクタ
     */
    virtual ~CATVViewModel();

    /**
     * @brief 画面を開くときに呼び出す表示内容の復元処理
     */
    virtual void activate(void) override;

    /**
     * @brief 画面を閉じるときに呼び出す表示な用の保存処理
     */
    virtual void deactivate(void) override;

    /**
     * @brief 画面を閉じる時に呼び出すスレッド停止の処理
     */
    virtual void suspend(void) override;

    /**
     * @brief 遷移先画面で参照する遷移前画面のデータ設定
     */
    virtual void setTakeOver() override;

signals:
    void requestSetTakeOver(QVariant, QVariant);

    /**
     * @brief プログレスバーの進捗の値を通知
     * @param value 進捗の値
     */
    void progress(double value);

    /**
     * @brief 再スキャン要求
     */
    void requestToReScan(void);

    /**
     * @brief スキャン結果保存確認ダイアログを表示する
     * @note コンテキストを切るためにシグナル/スロットで実装する
     */
    void requestToShowRegistScanResultPopup(void);

protected:
    /**
     * @brief 押下ボタンを受信
     * @param id 押下ボタンのID
     */
    virtual void onKeyStatus(ButtonId id) override;

    virtual void onButtonPress(ButtonId id) override;

protected slots:
    /**
     * @brief ポップアップから承諾の通知を受信
     * @param popupId 承諾通知を送ったポップアップID
     * @param okOnly  OK のみの場合 true
     */
    virtual void onAccept(PopupId popupId, bool okOnly) override;

    /**
     * @brief onReject ポップアップから拒否の通知を受信
     * @param popupId  拒否通知を送ったポップアップID
     */
    virtual void onReject(PopupId popupId) override;

    /**
     * @brief ポップアップから選択の通知を受信
     * @param index 選択されたインデックス
     */
    virtual void onSelect(PopupId popupId, int index) override;

    /**
     * @brief バッテリ低下状態変化通知
     * @param isLowBattery バッテリ低下状態かどうか
     */
    virtual void onLowBatteryStatusChanged(bool isLowBattery) override;

private slots:
    virtual void onRequest(int condition) override;

    /**
     * @brief スキャン結果を受信
     * @param index チャンネル番号
     * @param result スキャン結果
     */
    void onScanningChannelResult(int index, LC70_MEAS_MULTI_RESULT_T result);

    /**
     * @brief スキャン確認ダイアログを表示する
     */
    void onRequestScanConfirm();

    /**
     * @brief スキャン実行依頼を受信
     * @note 呼び出し画面でスキャン処理(開始、結果取得、中止)を実施
     */
    void onRequestToScan(void);

    /**
     * @brief スキャン中止依頼を受信
     */
    void onRequestToStopScanning(void);

    /**
     * @brief スキャン終了通知を受信
     * @param complete 完了成否
     */
    void onScanningFinished(bool complete);

    /**
     * @brief 再スキャン要求受信
     */
    void onRequestToReScan(void);

    /**
     * @brief 給電異常検知
     * @param code 故障コード
     * @param action 故障時の動作
     * @param message 表示メッセージ
     */
    void detectPowerSupplyError(uint16_t code, uint8_t action, QString message);

    /**
     * @brief スキャン結果保存確認ダイアログを表示する
     * @note コンテキストを１度切るためにシグナル/スロットで実装
     */
    void onRequestToShowRegistScanResultPopup(void);

private:
    /**
     * @brief カーソル移動
     * @param buttonId 押下ボタンのID
     */
    void moveCursor(ButtonId buttonId);

    /**
     * @brief エンターキー処理
     */
    void enterKey();

    /**
     * @brief 選択したBANDのアイコンを追加する
     * @param selectButtonIndex 選択したボタンインデックス
     */
    void addSelectBandIcon(int selectButtonIndex);

    /**
     * @brief ボタン設定を更新する
     */
    void setButtonSetting(void);

    /**
     * @brief Activeボタンに対応するBANDIDを取得する
     * @return Activeボタンに対応したバンドID
     */
    int getActiveBandId(void);

    /**
     * @brief スキャン時の変調方式を取得する
     * @return 変調方式の値
     */
    QString getScanningModualtion(void);

    /**
     * @brief スキャン結果後の動作選択
     * @param index 選択結果
     */
    void selectReactionScanResult(const int index);

private:
    QString         _selectedMeasurement;
    bool            _fromAutoSetting;
    ButtonOperation _buttonOperation;
    QString         _activeButton;
    QList<ChannelDefinitionItem> _channelList;
    QVariant _broadBandModulationSetting;           //!< マニュアル測定 放送方式, 変調方式選択
    LC70_POWER_SUPPLY_T _powerSupply;               //!< 給電状態
    uint16_t _failureCode;                          //!< 故障コード
    uint8_t _actionInFailure;                       //!< 故障時の動作
    static const int BUTTON_COLUMN_NUMBER;
    static const int BUTTON_ROW_NUMBER;
};

#endif // CATVVIEWMODEL_H
