/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef LOCALOSCILLATORSETTINGVIEWMODEL_H
#define LOCALOSCILLATORSETTINGVIEWMODEL_H

#include "abstractviewmodel.h"
#include "buttonid.h"
#include "buttonoperation.h"

/**
 * @brief 局部発振周波数選択 ViewModel
 */
class LocalOscillatorSettingViewModel : public AbstractViewModel
{
    Q_OBJECT
public:
    /**
     * @brief コンストラクター
     * @param screenId 画面 ID
     * @param model Model クラスのポインター
     * @param parent 親オブジェクト
     */
    explicit LocalOscillatorSettingViewModel(ScreenId screenId, LevelCheckerModel *&model, QObject *parent = nullptr);

    /**
     * @brief デストラクタ
     */
    virtual ~LocalOscillatorSettingViewModel();

    /**
     * @brief 画面を開くときに呼び出す表示内容の復元処理
     */
    virtual void activate(void) override;

    /**
     * @brief 画面を開くときに呼び出す表示内容の復元処理
     */
    virtual void deactivate(void) override;

    /**
     * @brief 画面を閉じるときに呼び出すスレッドの停止処理
     */
    virtual void suspend(void) override;

    /**
     * @brief 遷移先画面で参照する遷移前画面のデータ設定
     */
    virtual void setTakeOver() override;

signals:
    void requestSetTakeOver(QVariant, QVariant);
    void requestGetTakeOver();

    /**
     * @brief プログレスバーの進捗の値を通知
     * @param value 進捗の値
     */
    void progress(double value);

    /**
     * @brief 再スキャン要求
     */
    void requestToReScan(void);

    /**
     * @brief スキャン結果保存確認ダイアログを表示する
     * @note コンテキストを切るためにシグナル/スロットで実装する
     */
    void requestToShowRegistScanResultPopup(void);

protected slots:
    /**
     * @brief 押下ボタンを受信
     * @param id 押下ボタンのID
     */
    virtual void onKeyStatus(ButtonId id) override;

    /**
     * @brief ボタン押下時の処理
     * @param id 押下したボタン
     */
    virtual void onButtonPress(ButtonId id) override;

    /**
     * @brief ポップアップから選択の通知を受信
     * @param PopupId ポップアップのID
     * @param index 選択されたインデックス
     */
    virtual void onSelect(PopupId PopupId, int index) override;

    /**
     * @brief ポップアップから肯定の通知を受信
     * @param popupId ポップアップのID
     * @param okOnly okボタンのみかどうか
     */
    virtual void onAccept(PopupId popupId, bool okOnly) override;

    /**
     * @brief ポップアップから拒否の通知を受信
     * @param PopupId ポップアップのID
     */
    virtual void onReject(PopupId PopupId) override;

    /**
     * @brief 画面遷移要求受信時処理
     * @param condition 状態
     */
    virtual void onRequest(int condition) override;

    /**
     * @brief バッテリ低下状態変化通知
     * @param isLowBattery バッテリ低下状態かどうか
     */
    virtual void onLowBatteryStatusChanged(bool isLowBattery) override;

    /**
     * @brief 必要に応じてView を指すポインターを付け替えた後の復帰処理を行う
     */
    virtual void recover(void) override;

private slots:

    /**
     * @brief スキャン結果を受信
     * @param index スキャン番号
     * @param result スキャン結果
     */
    void onScanningChannelResult(int index, LC70_MEAS_MULTI_RESULT_T result);

    /**
     * @brief スキャン確認ダイアログを表示する
     */
    void onRequestScanConfirm(void);

    /**
     * @brief スキャンの実行依頼を受信
     * @note 呼び出し画面でスキャン処理(開始、結果取得、中止)を実施
     */
    void onRequestToScan(void);

    /**
     * @brief スキャン中止依頼を受信
     */
    void onRequestToStopScanning(void);

    /**
     * @brief スキャン終了通知を受信
     * @param complete 完了の成否
     */
    void onScanningFinished(bool complete);

    /**
     * @brief 再スキャン要求受信
     */
    void onRequestToReScan(void);

    /**
     * @brief 給電異常検知
     * @param code 故障コード
     * @param action 故障時の動作
     * @param message 表示メッセージ
     */
    void detectPowerSupplyError(uint16_t code, uint8_t action, QString message);

    /**
     * @brief スキャン結果保存確認ダイアログを表示する
     * @note コンテキストを１度切るためにシグナル/スロットで実装
     */
    void onRequestToShowRegistScanResultPopup(void);

private:
    /**
     * @brief カーソル移動
     * @param buttonId 押下ボタンのID
     */
    void moveCursor(ButtonId buttonId);
    /**
     * @brief エンターキー処理
     */
    void enterKey();
    /**
     * @brief ボタンメッセージ設定
     */
    void setButtonMessage();
    /**
     * @brief プロパティ変更
     * @param buttonName   オブジェクト名
     * @param propertyName プロパティ名
     * @param value        プロパティ値
     */
    void propertyChange(QString buttonName, QString propertyName, QString value);
    /**
     * @brief ボタンレイアウト設定
     * @param buttonCount ボタンの数
     */
    void layoutSetting(int buttonCount);

    /**
     * @brief 局発周波数リストを取得するL
     * @param localOscillatorList[out] 局発周波数リスト
     * @return 処理結果
     */
    bool getLocalOscillatorList(QList<uint32_t> &localOscillatorList);

    /**
     * @brief 小数桁数取得
     * @param value 値
     * @return 値を表示する際の小数桁数(最低でも1)
     */
    int getDecimalDigits(const uint value);

    /**
     * @brief スキャン結果後の動作選択
     * @param index 選択結果
     */
    void selectReactionScanResult(const int index);

    LC70_BAND_T     _selectedBand;
    QString         _selectedPolarization;
    ButtonOperation _buttonOperation;
    QString         _activeButton;
    QList<uint32_t> _localOscillatorList;
    QMap<QString, uint> _valueMap;
    static const int BUTTON_COLUMN_NUMBER1;
    static const int BUTTON_COLUMN_NUMBER2;
    static const int BUTTON_ROW_NUMBER1;
    static const int BUTTON_ROW_NUMBER3;
    static const int EDIT_LOCAL_FREQUENCY_INDEX;
    QList<ChannelDefinitionItem> _channelList;
    bool _fromAutoSetting;
    QVariant _broadBandModulationSetting;       //!< マニュアル測定 放送方式, 変調方式選択
    LC70_POWER_SUPPLY_T _powerSupply;           //!< 給電状態
    uint16_t _failureCode;                      //!< 故障コード
    uint8_t _actionInFailure;                       //!< 故障時の動作
};

#endif // LOCALOSCILLATORSETTINGVIEWMODEL_H
