/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include "modulationviewmodel.h"

// ボタンレイアウト (シングル, マルチ測定から遷移した場合 BANDスカパー, PILOT以外)
QVector<QVector<QString>> modulationButtonData = {
    { {"m1"}, {"m2"}, }
};
const int ModulationViewModel::BUTTON_COLUMN_NUMBER = 1;
const int ModulationViewModel::BUTTON_ROW_NUMBER    = 2;

// ボタンレイアウト (BAND: スカパー光)
QVector<QVector<QString>> skypModulationButtonData = {
    { {"m1"}, {"m3"} }
};
const int ModulationViewModel::SKYP_BUTTON_COLUMN_NUMBER = 1;
const int ModulationViewModel::SKYP_BUTTON_ROW_NUMBER    = 2;

// ボタンレイアウト (測定設定インジケータ画面から遷移した場合 BAND: スカパー光, PILOT以外)
QVector<QVector<QString>> settingThresholdModulationButtonData = {
    { {"m1"}, {"m2"}, {"m3"} }
};
const int ModulationViewModel::SETTING_THRESHOLD_BUTTON_COLUMN_NUMBER = 1;
const int ModulationViewModel::SETTING_THRESHOLD_BUTTON_ROW_NUMBER    = 3;

ModulationViewModel::ModulationViewModel(ScreenId screenId, LevelCheckerModel *&model, QObject *parent)
    : AbstractViewModel(screenId, model, parent)
    , _transitionSourceScreen("")
    , _isSkyp(false)
    , _isThresholdSetting(false)
{
}

ModulationViewModel::~ModulationViewModel()
{

}

void ModulationViewModel::activate(void)
{
    connect(this, SIGNAL(requestSetTakeOver(QVariant, QVariant)), _view, SLOT(backup(QVariant, QVariant)));
    connect(this, SIGNAL(requestGetTakeOver()), _view, SLOT(getTakeOver()));
    emit requestGetTakeOver();

    QObject *target = nullptr;
    if (_view) {
        target = _view->findChild<QObject *>("takeOverData");
    }

    if (target) {
        _transitionSourceScreen = target->property("transitionSourceScreen").toString();
    }

    QString selectedMeasurement = "";
    // 画面共有情報の測定設定にデータが設定されているかを調べ
    if (_takeOver["selectedMeasurement"].isValid()) {
        selectedMeasurement = _takeOver["selectedMeasurement"].toString();
    }

    if (selectedMeasurement.isEmpty()) {
        _isThresholdSetting = true;
    } else {
        _isThresholdSetting = false;
    }
    if (_view) {
        _view->setProperty("isThresholdSetting", QVariant::fromValue(_isThresholdSetting));
    }
    _isSkyp = _takeOver.value("selectedBand").toInt() == static_cast<int>(LC70_BAND_SKYP);
    if (_view) {
        _view->setProperty("isSkyp", QVariant::fromValue(_isSkyp));
    }
    if (_isSkyp) {
        _buttonOperation.setButtonName(SKYP_BUTTON_COLUMN_NUMBER, SKYP_BUTTON_ROW_NUMBER, skypModulationButtonData);
        _activeButton = skypModulationButtonData[0][0];
    } else {
        if (_isThresholdSetting) {
            // BAND: スカパー光以外, 測定設定>インジケータ画面から遷移した場合
            _buttonOperation.setButtonName(SETTING_THRESHOLD_BUTTON_COLUMN_NUMBER, SETTING_THRESHOLD_BUTTON_ROW_NUMBER, settingThresholdModulationButtonData);
            _activeButton = settingThresholdModulationButtonData[0][0];
        } else {
            // BAND: スカパー光以外, 測定画面から遷移した場合
            _buttonOperation.setButtonName(BUTTON_COLUMN_NUMBER, BUTTON_ROW_NUMBER, modulationButtonData);
            _activeButton = modulationButtonData[0][0];
        }
    }

    target = nullptr;
    if (_view) {
        target = _view->findChild<QObject *>(_activeButton);
    }
    if (target == nullptr) {
        qDebug() << "undefined objectName " + _activeButton;
        return;
    }

    target->setProperty("active", true);
    _buttonOperation.setActiveObject(_activeButton);
}

void ModulationViewModel::deactivate()
{
    AbstractViewModel::deactivate();
    this->disconnect();
    _isSkyp = false;
    _isThresholdSetting = false;
}

void ModulationViewModel::setTakeOver()
{
    if (_activeButton == "m1") {
        emit requestSetTakeOver("selectedModulation", OFDM);
    } else if (_activeButton == "m2") {
        emit requestSetTakeOver("selectedModulation", SIXTY_FOUR_QAM);
    } else if (_activeButton == "m3") {
        emit requestSetTakeOver("selectedModulation", TWO_HUNDRED_FIFTY_SIX_QAM);
    }
}

void ModulationViewModel::moveCursor(ButtonId buttonId)
{
    if ((buttonId == BUTTON_UP) || (buttonId == BUTTON_DOWN) || (buttonId == BUTTON_RIGHT) || (buttonId == BUTTON_LEFT)) {

        QString nextButton;
        bool ret = _buttonOperation.nextActiveObject(buttonId, nextButton);

        if (ret) {

            // inactive
            QObject *target = nullptr;
            if (_view) {
                target = _view->findChild<QObject *>(_activeButton);
            }
            if (target) {
                target->setProperty("active", false);
            }

            // active
            target = nullptr;
            if (_view) {
                target = _view->findChild<QObject *>(nextButton);
            }
            if (target) {
                target->setProperty("active", true);
            }

            _activeButton = nextButton;
            _buttonOperation.setActiveObject(_activeButton);
        }
    }
}

void ModulationViewModel::onKeyStatus(ButtonId id)
{
    // Home, Escape ボタンはベースクラスで対応
    AbstractViewModel::onKeyStatus(id);

    if (id == BUTTON_ENTER) {
        enterKey();
    } else if (id == BUTTON_USER) {
#ifdef NEW_USER_SETTING
        emit requestDirect(SCREEN_USER_SETTING, true);
#else
        emit requestDirect(SCREEN_USER_DATA_LOGGER_MENU, true);
#endif
    } else {
        moveCursor(id);
    }
}

void ModulationViewModel::onButtonPress(ButtonId id)
{
    if (id == BUTTON_ENTER) {
        QObject *target = nullptr;
        if (_view) {
            target = _view->findChild<QObject *>(_activeButton);
        }
        if (target) {
            if (_isSkyp) {
                target->setProperty("activebg", "combtn_p.png");
            } else {
                target->setProperty("activebg", "combtn_p.png");
            }
        }
    }

    return;
}

void ModulationViewModel::enterKey()
{
    if (_transitionSourceScreen == "MeasureSetting") {
        // 測定設定から変調方式設定画面に遷移した場合、閾値設定画面に遷移する。
        onRequest(2);
        return;
    }

    // 測定設定MeasureSetting以外から変調方式設定画面に遷移した場合、チャンネル測定画面に遷移する。
    onRequest(0);
}

void ModulationViewModel::onRequest(int condition)
{
    emit request(_screenId, condition);
    return;
}
