/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include "multimeasurementviewmodel.h"
#include "mainframeviewmodel.h"
#include <QElapsedTimer>

static constexpr int BUTTON_COLUMN_NUMBER = 1;                                  //!< グラフ、リストの列
static constexpr int BUTTON_ROW_NUMBER = 2;                                     //!< グラフ、リストの行

QVector<QVector<QString>> multiMeasurementButtonData = { { "graphButton", "listButton" } };  //!< ボタン名称の種類

static constexpr int GRAPH_BUTTON_INDEX = 0;    //!< グラフボタンのインデックス
static constexpr int LIST_BUTTON_INDEX = 1;     //!< リストボタンのインデックス

static constexpr const char *IMAGE_BUTTON_NAME =
        "imageButton"; //!< 画像保存ボタン名(MultiMeasurement.qml 参照)
static constexpr const char *DATA_BUTTON_NAME =
        "dataButton";   //!< データ保存ボタン名(MultiMeasurement.qml 参照)
static constexpr const char *MEASUREMENT_RESULT_DISPLAY =
        "measurementResultDisplay";  //!< 測定結果表示中(MultiMeasurement.qml 参照)

static constexpr const char *CSV_FILE_EXTENSION = ".csv";       //!< 測定データファイルの拡張子(CSV)
static constexpr const char *PNG_FILE_EXTENSION = ".png";       //!< 測定画面の拡張子(PNG)

static constexpr int LIST_ROWS = 6;             //!< リストの行数

static constexpr int SCAN_INDEX = 2;                //!< SCAN を表示するメニューリストのインデックス
static constexpr int POWER_SUPPLY_INDEX =
        3;        //!< 給電 を表示するメニューリストのインデックス

static constexpr int FM_CHANNELS = 12;                  //!< FM チャンネル数

static constexpr int USER_CHANNELS_LEVEL1 = 13;         //!< ユーザー設定のチャンネル数(13 ch 以内)
static constexpr int USER_CHANNELS_LEVEL2 =
        60;         //!< ユーザー設定のチャンネル数(14 ch 以上 60 ch 以内)
static constexpr int USER_CHANNELS_LEVEL3 =
        120;        //!< ユーザー設定のチャンネル数(61 ch 以上　120 ch まで)

static constexpr const char *UNLOCK_STRING = "UNLOCK";              //!< アンロック時の放送方式、変調方式、階層の表示文字列

// MultiMeasurementGraph.qml のインデックスと連動させる
enum BAND_CATEGORY_INDEX {
    BAND_CATEGORY_UHF = 0,              // 地デジ
    BAND_CATEGORY_BS_RIGHT,             // BS(右旋)
    BAND_CATEGORY_BS_LEFT,              // BS(左旋)
    BAND_CATEGORY_CS_RIGHT,             // 110° CS(右旋)
    BAND_CATEGORY_CS_LEFT,              // 110° CS(左旋)
    BAND_CATEGORY_SCCB_HORIZONTAL,      // SCC-B(水平)
    BAND_CATEGORY_SCCB_VERTICAL,        // SCC-B(垂直)
    BAND_CATEGORY_SCCC_HORIZONTAL,      // SCC-C(水平)
    BAND_CATEGORY_SCCC_VERTICAL,        // SCC-C(垂直)
    BAND_CATEGORY_JCSAT3A_VERTICAL,     // JCSAT-3A(垂直)
    BAND_CATEGORY_JCSAT3A_HORIZONTAL,   // JCSAT-3A(水平)
    BAND_CATEGORY_JCSAT4B_VERTICAL,     // JCSAT-4B(垂直)
    BAND_CATEGORY_JCSAT4B_HORIZONTAL,   // JCSAT-4B(水平)
    BAND_CATEGORY_CATV_ALL,             // CATV ALL
    BAND_CATEGORY_CATV_90_470,          // CATV 90-470 MHz
    BAND_CATEGORY_CATV_470_1002,        // CATV 470-1002 MHz
    BAND_CATEGORY_PILOT,                // PILOT
    BAND_CATEGORY_SKY_PERFETV,          // スカパー光
    BAND_CATEGORY_LTE,                  // LTE
    BAND_CATEGORY_FM,                   // FM
    BAND_CATEGORY_BS_CS,                // BS_CS統合
    BAND_CATEGORY_CATV_UPSTREAM,        // CATV 上り
    BAND_CATEGORY_USER1,                // ユーザー設定 1
    BAND_CATEGORY_USER2,                // ユーザー設定 2
    BAND_CATEGORY_USER3,                // ユーザー設定 3
    BAND_CATEGORY_USER4                 // ユーザー設定 4
};

/**
 * @brief 棒グラフの種類
 */
enum GRAPH_TYPE {
    GRAPH_TYPE_1 = 0,                   // 13 本
    GRAPH_TYPE_2,                       // 60 本
    GRAPH_TYPE_3,                       // 120 本
    GRAPH_TYPE_4                        // 160 本
};

// MultiMeasurementFixedGraph.qml の背景画像定義のインデックスと連動させる
enum FIXED_GRAPH_CATEGORY_INDEX {
    FIXED_GRAPH_CATEGORY_UHF = 0,              // 地デジ
    FIXED_GRAPH_CATEGORY_BS_RIGHT,             // BS(右旋)
    FIXED_GRAPH_CATEGORY_BS_LEFT,              // BS(左旋)
    FIXED_GRAPH_CATEGORY_CS_RIGHT,             // 110° CS(右旋)
    FIXED_GRAPH_CATEGORY_CS_LEFT,              // 110° CS(左旋)
    FIXED_GRAPH_CATEGORY_CATV_ALL,             // CATV ALL
    FIXED_GRAPH_CATEGORY_CATV_90_470,          // CATV 90-470 MHz
    FIXED_GRAPH_CATEGORY_CATV_470_1002,        // CATV 470-1002 MHz
    FIXED_GRAPH_CATEGORY_SKY_PERFETV,          // スカパー光
    FIXED_GRAPH_CATEGORY_PILOT,                // PILOT
    FIXED_GRAPH_CATEGORY_CATV_UPSTREAM,        // CATV 上り
    FIXED_GRAPH_CATEGORY_JCSAT3A_VERTICAL,     // JCSAT-3A(垂直)
    FIXED_GRAPH_CATEGORY_JCSAT3A_HORIZONTAL,   // JCSAT-3A(水平)
    FIXED_GRAPH_CATEGORY_JCSAT4B_VERTICAL,     // JCSAT-4B(垂直)
    FIXED_GRAPH_CATEGORY_JCSAT4B_HORIZONTAL,   // JCSAT-4B(水平)
    FIXED_GRAPH_CATEGORY_SCCB_VERTICAL,        // SCC-B(垂直)
    FIXED_GRAPH_CATEGORY_SCCB_HORIZONTAL,      // SCC-B(水平)
    FIXED_GRAPH_CATEGORY_SCCC_VERTICAL,        // SCC-C(垂直)
    FIXED_GRAPH_CATEGORY_SCCC_HORIZONTAL,      // SCC-C(水平)
    FIXED_GRAPH_CATEGORY_LTE,                  // LTE
    FIXED_GRAPH_CATEGORY_FM,                   // FM
    FIXED_GRAPH_CATEGORY_BS_CS,                // BS CS統合
};

/**
 *  @brief Band タブごとの Band ID の定義
 */
static const QMap<int, LC70_BAND_T> TAB_BAND_MAP = {
    {BAND_TAB_UHF,              LC70_BAND_UHF},
    {BAND_TAB_BS_R,             LC70_BAND_BS},
    {BAND_TAB_BS_L,             LC70_BAND_BS},
    {BAND_TAB_CS_R,             LC70_BAND_CS},
    {BAND_TAB_CS_L,             LC70_BAND_CS},
    {BAND_TAB_CATV_ALL,         LC70_BAND_CATV_ALL},
    {BAND_TAB_SKY_PERFECTV,     LC70_BAND_SKYP},
    {BAND_TAB_CATV_UPSTREAM,    LC70_BAND_CATV_UP_STREAM},
    {BAND_TAB_JCSAT3A_H,        LC70_BAND_JCSAT3},
    {BAND_TAB_JCSAT3A_V,        LC70_BAND_JCSAT3},
    {BAND_TAB_JCSAT4B_H,        LC70_BAND_JCSAT4},
    {BAND_TAB_JCSAT4B_V,        LC70_BAND_JCSAT4},
    {BAND_TAB_SCCB2_H,          LC70_BAND_SCCB},
    {BAND_TAB_SCCB2_V,          LC70_BAND_SCCB},
    {BAND_TAB_SCCC2_H,          LC70_BAND_SCCC},
    {BAND_TAB_SCCC2_V,          LC70_BAND_SCCC},
    {BAND_TAB_LTE,              LC70_BAND_LTE},
    {BAND_TAB_FM,               LC70_BAND_FM},
    {BAND_TAB_PILOT,            LC70_BAND_PILOT},
    {BAND_TAB_COUNT,            LC70_BAND_UNDEF},
};

static QElapsedTimer timer;

MultiMeasurementViewModel::MultiMeasurementViewModel(ScreenId screenId, LevelCheckerModel *&model, QObject *parent)
    : AbstractMeasurementViewModel(screenId, model, parent)
    , _buttonOperation(new ButtonOperation(this))
    , _activeButton()
    , _selectedButtonIndex(GRAPH_BUTTON_INDEX)
    , _fixedGraph(nullptr)
    , _list(nullptr)
    , _preservedChannelList()
    , _saveFileType(SAVE_FILE_COUNT)
    , _condition()
    , _preferredCondition()
    , _previousPreferredCondition()
    , _measureResultModel(nullptr)
    , _multiGraphModel(nullptr)
    , _multiMeasureResult()
    , _totalPage(1)
    , _currentPage(1)
    , _userSettings()
    , _isChannelChanging(false)
    , _fromUserSetting(false)
    , _previousBandId(LC70_BAND_UNDEF)
    , _barType(0)
    , _broadBandSystem(LC70_BD_SYS_NONE)
    , _modulation(LC70_SETMOD_UNDEF)
    , _bandId(LC70_BAND_UNDEF)
    , _alreadySuspended(false)
    , _measuringCondition(0)
    , _confirmeChangeChannelTimer(new QTimer())
    , _cnrMerSetting()
    , _broadBandModulationSetting()
    , _holdMeasurementData()
    , _previousSelectedIndex(GRAPH_BUTTON_INDEX)
    , _dataSave2State(DataSave2None)
    , _channelListForGraph()
    , _failureCode(0)
    , _actionInFailure(LC70_FAIL_CTL_NONE)
    , _autoScroll(false)
    , _lockWaitingTimer(new QTimer())
    , _waitforLockingTimeOut(false)
    , _confirmedValue(false)
    ,_bkMenuBarType(0)
    , _currentMeasuringIndex(0)
    , _startSaveData(false)
{
    setObjectName("multiMeasurementViewModel");
    _buttonOperation->setButtonName(BUTTON_COLUMN_NUMBER, BUTTON_ROW_NUMBER, multiMeasurementButtonData);
    _buttonOperation->setActiveObject(multiMeasurementButtonData[0][0]);
    _activeButton = multiMeasurementButtonData[0][0];

    qmlRegisterType<MeasureResultModel>("jp.co.dxantenna.levelchecker.measureResultModel", 1, 0, "MeasureResultModel");
    qmlRegisterType<MultiGraphModel>("jp.co.dxantenna.levelchecker.multiGraphModel", 1, 0, "MultiGraphModel");

    // 優先測定のロック待ちの時間を設定
    _lockWaitingTimer->setInterval(_model->applicationSetting()->waitingTimeForPrefferedChannel());
    _lockWaitingTimer->setSingleShot(true);
    connect(_lockWaitingTimer, &QTimer::timeout, [this]() {
        // ロック待ち時間切れ
        _waitforLockingTimeOut = true;

        // 値確定
        _confirmedValue = true;

        // 点滅を止める
        if (_view) {
            _view->setProperty("blinkInterval", 0);
        }
    });
}

MultiMeasurementViewModel::~MultiMeasurementViewModel()
{

}

void MultiMeasurementViewModel::resume()
{
    if (_view == nullptr) {
        MainFrameViewModel* view = dynamic_cast<MainFrameViewModel *>(_mainFrame);
        if (view) {
            _view = view->multiMeasurementView();
        }
        if (_view == nullptr) {
            return;
        }
    }
    AbstractViewModel::resume();

    _confirmeChangeChannelTimer->setSingleShot(true);
    connect(_confirmeChangeChannelTimer, SIGNAL(timeout()), this, SLOT(onConfirmedChangeChannelTimeout()));

    // [Memo]
    // 画面遷移時に実行する
    // CN/MERの選択状態をqmlに送信する
    int valueMerCN = Correcter::selectCnrMer(static_cast<LC70_BAND_T>(_bandId), _cnrMerSetting);

    if (_view) {
        _view->setProperty("selectCnMer", valueMerCN);
    }

    // チャンネル変更(測定開始)
    changeChannel(true, false);

    // スレッドを開始
    // USER 設定のマルチ測定であれば遷移先画面種別はデータロガーリストにする
    if (_fromUserSetting && _hasMultiBandsInUserSetting) {
        emit resumeMultiMeasuring(&_condition, &_preferredCondition, LC70_DISP_TYPE_LIST_LOGGER,
                                  _model->applicationSetting()->pollingIntervalMultiMeasurement());
    } else {
        emit resumeMultiMeasuring(&_condition, &_preferredCondition, LC70_DISP_TYPE_GRAPH,
                                  _model->applicationSetting()->pollingIntervalMultiMeasurement());
    }
}

void MultiMeasurementViewModel::suspend()
{
    disconnect(_confirmeChangeChannelTimer, SIGNAL(timeout()), this, SLOT(onConfirmedChangeChannelTimeout()));

    // 給電を停止
    _powerSupply = LC70_POWER_SUPPLY_OFF;
    _model->setPowerSupply(_powerSupply);
    _model->removeMenuList(MenuListIconIndex::POWER_SUPPLY_ICON);

    // シングル測定に遷移する場合は既に一時停止してからシングル測定で測定を開始しているため、測定対象なしにならないようスレッドを停止しない
    if (!_alreadySuspended) {
        // スレッドを停止(遷移先画面種別測定画面以外に変更)
        emit requestToSuspendMultiMeasuring();

        AbstractViewModel::suspend();
    }

    // グラフ表示に戻す
    _selectedButtonIndex = GRAPH_BUTTON_INDEX;

    if (_fixedGraph) {
        _fixedGraph->setProperty("visible", true);
    }

    if (_list) {
        _list->setProperty("visible", false);

        // リストの説明用のアイコンを元に戻す
        _list->setProperty("storing", false);
    }

    // グラフのインデックスを先頭に移動する
    if (_multiGraphModel) {
        _multiGraphModel->changeSelection(_currentChannelIndex, 0);
    }

    // QML オブジェクトへのポインターをクリアー
    _fixedGraph = nullptr;
    _list = nullptr;
    _measureResultModel = nullptr;
    _multiGraphModel = nullptr;

    // 優先測定条件のクリアー
    memset(&_preferredCondition, 0x00, sizeof(LC70_MEAS_SINGLE_T));
    memset(&_previousPreferredCondition, 0x00, sizeof(LC70_MEAS_SINGLE_T));

    // マルチ測定条件のクリアー
    for (int i = 0; i < _channelList.count(); ++i) {
        _condition.sRegNum = 0;
        memset(&_condition.single[i], 0x00, sizeof(_condition.single[i]));
    }

    // チャンネル情報のクリアー
    _channelList.clear();
    _preservedChannelList.clear();
    _channelListForGraph.clear();

    // 局初周波数のリストをクリアー
    _localFrequencyList.clear();

    // ユーザー設定のクリアー
    _userSettings.clear();

    // 測定結果ファイルの測定項目をクリアー
    _measuringCondition = 0;

    // ユーザー設定からの遷移判定を初期値に戻す
    _fromUserSetting = false;

    // 保持している測定データのクリア
    _holdMeasurementData.clear();

    _broadBandSystem = LC70_BD_SYS_NONE;
    _modulation = LC70_SETMOD_UNDEF;
    _isScanning = false;

    // 測定データ保存の動作状態
    _dataSave2State = DataSave2None;

    // 故障コードのクリアー
    _failureCode = 0;

    // ロック待ち時間のクリアー
    _waitforLockingTimeOut = false;

    // 値未確定
    _confirmedValue = false;

    // USER 設定で複数の Band を跨るか否かのクリアー
    _hasMultiBandsInUserSetting = false;

    // チャンネル定義情報のリストのインデックスのクリアー
    _currentChannelIndex = 0;

    if (_view) {
        _view->setProperty("currentChannelIndex", -1);
        _view->setProperty("scanChannelIndex", 0);
        _view->setProperty("currentBandIndex", 0);
        _view->setProperty("currentPage", 1);
        _view->setProperty("totalPage", 1);
        _view->setProperty("autoScroll", 0);
        _view->setProperty("selectGraphList", 0);
        _view->setProperty("graphIndex", false);
        _view->setProperty("hasMultiBandsInUserSetting", false);
    }
    _model->writeLog(QString("マルチ測定"), QString("マルチ測定終了"));
}

void MultiMeasurementViewModel::createConnection(QObject *qml, QObject *mainFrame)
{
    AbstractMeasurementViewModel::createConnection(qml, mainFrame);
    connect(this, SIGNAL(requestSetTakeOver(QVariant, QVariant)), _view, SLOT(backup(QVariant, QVariant)));
    connect(this, SIGNAL(resumeMultiMeasuring(LC70_MEAS_MULTI_T *, LC70_MEAS_SINGLE_T *, LC70_DISP_TYPE_T, uint32_t)), _model,
            SLOT(onResumeMultiMeasuring(LC70_MEAS_MULTI_T *, LC70_MEAS_SINGLE_T *, LC70_DISP_TYPE_T, uint32_t)));
    connect(_model, SIGNAL(multiMeasureResult(MultiResult, MeasureResult)), this, SLOT(onMultiMeasureResult(MultiResult,
                                                                                                            MeasureResult)));
    connect(_model, SIGNAL(notifyChannelChanging(bool)), this, SLOT(onNotifyChannelChanging(bool)));
    connect(_model, SIGNAL(scanningChannelResult(int, LC70_MEAS_MULTI_RESULT_T)), this, SLOT(onScanningChannelResult(int,
                                                                                                                     LC70_MEAS_MULTI_RESULT_T)));
    connect(_model, SIGNAL(scanningFinished(bool)), this, SLOT(onScanningFinished(bool)));
    connect(this, SIGNAL(changeCondition(LC70_MEAS_SINGLE_T *)), _model,
            SLOT(onPreferredMeasuringChangeCondition(LC70_MEAS_SINGLE_T *)));
    connect(this, SIGNAL(requestToSaveDataMulti(SaveLocation, MeasureType, QString, uint16_t, QList<MeasureResult>)), mainFrame,
            SLOT(onRequestToSaveDataMulti(SaveLocation, MeasureType, QString, uint16_t, QList<MeasureResult>)));
    connect(this, SIGNAL(requestToCapture()), mainFrame, SLOT(onRequestToCapture()));
    connect(this, SIGNAL(requestToSaveImage(MeasureType, SaveLocation, QString)), mainFrame, SLOT(onRequestToSaveImage(MeasureType, SaveLocation, QString)));
    connect(this, SIGNAL(requestToClearCaptureImage()), mainFrame, SLOT(onRequestToClearCaptureImage()));
    connect(mainFrame, SIGNAL(inputFileName(PopupId, SaveLocation, QString)), this, SLOT(onInputFileName(PopupId,
                                                                                                         SaveLocation, QString)));
    connect(mainFrame, SIGNAL(requestToScan()), this, SLOT(onRequestToScan()));
    connect(mainFrame, SIGNAL(requestToStopScanning()), this, SLOT(onRequestToStopScanning()));
    connect(mainFrame, SIGNAL(requestToStopAcquring()), this, SLOT(onRequestToStopAcquiring()));
    connect(mainFrame, SIGNAL(notifyButtonInput(PopupId, ButtonId)), this, SLOT(onNotifyButtonInput(PopupId, ButtonId)));
    connect(this, SIGNAL(cursorPressed(QVariant)), mainFrame, SIGNAL(cursorPressed(QVariant)));
    connect(this, SIGNAL(scrollPage(QVariant)), _view, SLOT(onScrollPage(QVariant)));
    connect(this, SIGNAL(progress(double)), mainFrame, SIGNAL(progress(double)));
    connect(this, SIGNAL(progressText(QString, bool)), mainFrame, SIGNAL(progressText(QString, bool)));
    connect(this, SIGNAL(changeAttribute(QString, QVariant)), mainFrame, SLOT(onChangeAttribute(QString, QVariant)));
    connect(this, SIGNAL(removeAttribute(QString)), mainFrame, SLOT(onRemoveAttribute(QString)));
    connect(mainFrame, SIGNAL(notifyPowerSupplyError(uint16_t, uint8_t, QString)), this, SLOT(detectPowerSupplyError(uint16_t, uint8_t, QString)));
    connect(this, SIGNAL(requestToReScan()), this, SLOT(onRequestToReScan()), Qt::QueuedConnection);
    connect(this, SIGNAL(requestToShowRegistScanResultPopup()), this, SLOT(onRequestToShowRegistScanResultPopup()), Qt::QueuedConnection);
    connect(this, SIGNAL(acquiringFinished(bool)), mainFrame, SIGNAL(acquiringFinished(bool)));
    connect(this, SIGNAL(requestToSuspendMultiMeasuring()), _model, SLOT(onSuspendMultiMeasuring()));
    connect(mainFrame, SIGNAL(notifySaveEnd()), this, SLOT(onNotifySaveEnd()));

    // マルチ測定結果の取得に設定
    _model->writeLog(QString("マルチ測定"), QString("マルチ測定中"));
    _model->changeAcquisitionTarget(MULTI);
    _model->writeLog(QString("マルチ測定"), QString("グラフ表示"));
}

void MultiMeasurementViewModel::activate()
{
    AbstractMeasurementViewModel::activate();

    // 一時停止していない
    _alreadySuspended = false;

    // 各画面
    if (_view) {
        _fixedGraph = _view->findChild<QObject *>(QString("fixedGraph"));
        _list = _view->findChild<QObject *>(QString("list"));
    }

    if (_fixedGraph == nullptr || _list == nullptr) {
        return;
    }

    // グラフ用の測定結果のモデル(MultiGraphModel の実体(objectName:multiGraphModel)を QML から参照)
    if (_view) {
        _multiGraphModel = dynamic_cast<MultiGraphModel *>(_view->findChild<QObject *>(QString("multiGraphModel")));
    }

    // リスト用の測定結果のモデル(MeasureResultModel の実体(objectName:measureResultModel)を QML から参照)
    if (_view) {
        _measureResultModel = dynamic_cast<MeasureResultModel *>(_view->findChild<QObject *>(QString("measureResultModel")));
    }

    _autoScroll = false;

    // チャンネルリストをクリアする
    _channelList.clear();
    _channelListForGraph.clear();

    MainFrameViewModel *mainFrame = static_cast<MainFrameViewModel *>(_mainFrame);

    //光モード時プリアンプアイコン表示
    QVariant valueopt;
    int rfOptical = 0;
    _model->getValue(VALUE_RF_OPTICAL, valueopt);
    rfOptical = valueopt.toInt();
    if(rfOptical == 1){
        mainFrame->menuoptchange(false);
        _model->replaceMenuList(MenuListIconIndex::PREAMP_ICON, "icon_preamp.png");
    } else {
        mainFrame->menuoptchange(true);
    }

    // 画面遷移時は、メニューバーを閉じた状態にする
    setMenuBarVisible(false);

    // QML ファイルのプロパティの変更(MultiMeasurement.qml を参照)
    // 画面ロード時は初期値に設定する
    if (_view) {
        _view->setProperty("level", INITIAL_LEVEL);
        _view->setProperty("mer", INITIAL_MER);
        _view->setProperty("ber", INITIAL_BER);
        _view->setProperty("scanChannelIndex", 0);
        _view->setProperty("modulation", INITIAL_MODULATION);
        _view->setProperty("modulationPrefix", QString(""));
    }

    // 測定中のインデックスのクリアー
    _currentMeasuringIndex = 0;

    bool ok = false;
    _bandId = _takeOver["selectedBand"].toUInt(&ok);

    // CNR/MER 切替設定の取得
    _cnrMerSetting.clear();
    _model->getValue(VALUE_MER_CN, _cnrMerSetting);

    // マニュアル測定 放送方式, 変調方式選択の取得
    _broadBandModulationSetting.clear();
    _model->getValue(VALUE_BROADBANDSYSTEM_MODULATION, _broadBandModulationSetting);

    _currentChannelIndex = 0;
    if (_fixedGraph) {
        _fixedGraph->setProperty("visible", true);
    }

    bool scanned = false;

    // ユーザー情報の反映
    if (_takeOver["fromUserSetting"].toBool()) {
        _fromUserSetting = true;

        if (!createUserSettingChannelList(_userSettings)) {
            // 前回の測定結果をクリアするため、無効値を設定する
            clear();
            _view->setProperty("isSatelliteBroadCasting", Correcter::isSatelliteBroadCasting(LC70_BAND_UHF));
            _view->setProperty("channel", "");
            _view->setProperty("isModulationVisible", Correcter::isModulationVisible(LC70_BAND_UHF));
            _view->setProperty("displayChannelDetail", Correcter::getDisplayChannelDetail(LC70_BAND_UHF));
            emit requestToShowPopup(POPUP_FAILURE_READ_USER_SETTING_FILE);
            return;
        }

        // USER 測定画面から遷移した場合は _takeOver に Band 跨ぎを判定するキー("hasMultiBandsInUserSetting")があるが、
        // メニューバーから移動したときにはないので、設定されているチャンネルから Band 跨ぎを判断する
        BandTabIndex userSettingBandTab =  BAND_TAB_COUNT;
        _hasMultiBandsInUserSetting = false;
        for (int i = 0; i < _channelList.size(); ++i) {
            BandTabIndex bandTabIndex = Correcter::convertBandTabIndex(static_cast<LC70_BAND_T>(_channelList.at(i).band), Correcter::convertPolarizeNumToString(_channelList.at(i).polarization));
            if (userSettingBandTab != bandTabIndex) {
                if (userSettingBandTab != BAND_TAB_COUNT) {
                    _hasMultiBandsInUserSetting = true;
                    break;
                }
                userSettingBandTab = bandTabIndex;
            }
        }
        if (_view) {
            _view->setProperty("hasMultiBandsInUserSetting", _hasMultiBandsInUserSetting);
        }

        if (!_hasMultiBandsInUserSetting) {
            // グラフ用にチャンネル定義リストを保持する
            if (_channelList[0].band == LC70_BAND_FM) {
                _model->selectChannelDefinitionList(LC70_BAND_FM_DEF, Correcter::convertWaveToPolarization(_channelList[0].polarization), _channelListForGraph);
                complementChannelDefinition(LC70_BAND_FM_DEF);
            } else if (_channelList[0].band == LC70_BAND_CATV_UP_STREAM) {
                _model->selectChannelDefinitionList(LC70_BAND_CATV_UP_STREAM_DEF, Correcter::convertWaveToPolarization(_channelList[0].polarization), _channelListForGraph);
                complementChannelDefinition(LC70_BAND_CATV_UP_STREAM_DEF);
            } else {
                _model->selectChannelDefinitionList(_channelList[0].band, Correcter::convertWaveToPolarization(_channelList[0].polarization), _channelListForGraph);
            }
        }

        _bandId = static_cast<int>(_channelList[_currentChannelIndex].band);

        _takeOver["selectedPolarization"] = Correcter::convertWaveToPolarization(_channelList[_currentChannelIndex].polarization);

        // ヘッダーに Band を表示
        addSelectBandIcon(_bandId);

        // ユーザー設定名を反映
        if (_view) {
            _view->setProperty("userMode", _takeOver["selectedUserListFile"].toString());
        }
    } else {
        // ユーザー設定ではない
        _fromUserSetting = false;

        // ユーザー設定名をクリアー
        if (_view) {
            _view->setProperty("userMode", QString());
        }
        QString polarization = "";

        if (_takeOver.contains("selectedPolarization")) {
            polarization = _takeOver["selectedPolarization"].toString();
        }

        // スキャン結果があればその内容を反映する(保存画面から戻ってきた場合に使用するため、Band などの測定条件は同一であることを前提とする)
        QList<ChannelDefinitionItem> scannedChannelList;

        // 保存先選択画面で保存されていたらスキャン結果を読み取る
        //if (_takeOver.contains("saveScanData") && _takeOver["saveScanData"].toBool()) {
            scannedChannelList = _model->scannedChannelList();
        //}

        if (scannedChannelList.count() > 0 && polarization == _model->scannedPolatization()) {
            // スキャン結果は有効なチャンネルのみ保持している(シングル測定とは異なる)
            for (int i = 0; i < scannedChannelList.count(); ++i) {
                _channelList.append(scannedChannelList[i]);
            }
            scanned = true;
            // ユーザー設定名を反映
            if (_view) {
                if(_takeOver["selectedUserListFile"].toString() == "") {
                    _view->setProperty("userMode", NO_FILE_NAME);
                } else {
                    _view->setProperty("userMode", _takeOver["selectedUserListFile"].toString());
                }
            }
            // ユーザー設定名をクリア
            emit removeAttribute("selectedUserListFile");
        } else {

            // FMの場合は FM DEF とし、CATV上りの場合は CATV DEFにする
            if (_bandId == LC70_BAND_FM) {
                _bandId = LC70_BAND_FM_DEF;
            }

            if (_bandId == LC70_BAND_CATV_UP_STREAM) {
                _bandId = LC70_BAND_CATV_UP_STREAM_DEF;
            }

            // チャンネル定義リストを取得する
            _model->selectChannelDefinitionList(_bandId, polarization, _channelList);
        }
        // グラフ用にチャンネル定義リストを保持する
        _model->selectChannelDefinitionList(_bandId, polarization, _channelListForGraph);

        // 地デジの場合は OFDM とする
        if (_bandId == LC70_BAND_UHF) {
            _takeOver["selectedModulation"] = QString("OFDM");
        }

        _preservedChannelList.clear();
        
        // マルチ測定の場合、_channelListは SCANで有効なチャンネルのみのリストになりうる
        for (int i = 0; i < _channelList.count(); ++i) {
            _preservedChannelList.append(_channelList[i]);
        }
    }

    if (_channelList.count() > 0) {
        _previousBandId = _channelList[0].band;
    }

    // 保存していたスキャン後の有効チャンネルを破棄
    _model->clearScannedChannelList();

    _barType = 0;

    // FM DEFの場合は FM に戻す CATV上りの場合は CATV に戻す
    if (_bandId == LC70_BAND_FM_DEF) {
        _bandId = LC70_BAND_FM;
    }

    if (_bandId == LC70_BAND_CATV_UP_STREAM_DEF) {
        _bandId = LC70_BAND_CATV_UP_STREAM;
    }

    if (_view) {
        // X 軸の表示のためどの Band かは指定しておく
        _view->setProperty("currentBandIndex", selectXAsixCategory(_bandId,
                                                                   Correcter::convertWaveToPolarization(_channelList[_currentChannelIndex].polarization)));
    }
    // 測定結果の初期化
    _multiMeasureResult.clear();

    // 局発周波数 空の場合(局発周波数選択画面を経由していない場合)は 0 とする
    if (_takeOver["selectedLocalOscillator"].toString().isEmpty()) {
        _takeOver["selectedLocalOscillator"] = "0";
    }

    // グラフ表示用モデルの変更
    changeMultiGraphModel();

    // リスト表示用モデルの変更
    changeMultiListModel();

    // リストの総ページ数設定
    if (_measureResultModel) {
        _totalPage = _measureResultModel->rowCount() / LIST_ROWS;
    }
    if (_measureResultModel) {
        if (_measureResultModel->rowCount() % LIST_ROWS > 0) {
            _totalPage += 1;
        }
    }

    _currentPage = 1;
    if (_view) {
        _view->setProperty("totalPage", _totalPage);        // 総ページ
        _view->setProperty("currentPage", _currentPage);    // 現在のページ

#if 1
        // MENUボタンでUSER設定を呼び出すときにリストが先頭にならない不具合対応
        emit scrollPage(_currentPage);
#endif
    }

    // USER 設定で複数の Band を跨る場合はグラフを使用しない
    if (_hasMultiBandsInUserSetting) {
        _model->changeDisplayType(LC70_DISP_TYPE_LIST_LOGGER);

        if (_fixedGraph) {
            _fixedGraph->setProperty("visible", false);
        }
        if (_list) {
            _list->setProperty("visible", true);
        }

        // ユーザー設定で選択した内容を測定条件に反映
        setUserMultiMeasuringCondition(false);

        setBoolProperty(IMAGE_BUTTON_NAME, "visible", false);
        setBoolProperty(IMAGE_BUTTON_NAME, "enable", false);
        if (_view) {
            _view->setProperty("selectGraphList", 1);
        }
        _selectedButtonIndex = 1;

        setButtonVisible(_view, multiMeasurementButtonData[0][0], false);
    } else {
        // 画面を開いたときは常にグラフから
        _model->changeDisplayType(LC70_DISP_TYPE_GRAPH);
        if (_view) {
            setButtonActive(_view, _activeButton, false);
        }
        int currentChannelIndex = 0;
        selectGraphIndex(0, currentChannelIndex);

        if (_multiGraphModel) {
            // グラフ表示時、白線が残ってしまうことを防ぐため白線の位置を初期化する
            _multiGraphModel->InitializeCurrentPole(true, currentChannelIndex);
        }

        setBoolProperty(IMAGE_BUTTON_NAME, "visible", true);
        setBoolProperty(IMAGE_BUTTON_NAME, "enable", true);
        if (_view) {
            _view->setProperty("selectGraphList", 0);
        }
        _selectedButtonIndex = 0;
        setButtonVisible(_view, multiMeasurementButtonData[0][0], true);
        // グラフを表示するのでリストを非表示
        if (_list) {
            _list->setProperty("visible", false);
        }
    }

    // 画面を開いたときは常に測定結果表示中ラベルは非表示
    setBoolProperty(MEASUREMENT_RESULT_DISPLAY, "visible", false);

    _activeButton = multiMeasurementButtonData[0][_selectedButtonIndex];
    if (_view) {
        setButtonActive(_view, _activeButton, true);
    }
    _buttonOperation->setActiveObject(_activeButton);

    // USER 設定で Band を跨る場合(リストのみ表示)以外
    if (!_hasMultiBandsInUserSetting) {
        // SAVE１ ボタン表示
        setBoolProperty(IMAGE_BUTTON_NAME, "visible", true);
        setBoolProperty(IMAGE_BUTTON_NAME, "enable", true);
    }

    // スキャンアイコン表示
    if (!scanned) {
        // なし
        _model->replaceMenuList(MenuListIconIndex::SCAN_ICON, "");
    } else {
        // あり
        _model->replaceMenuList(MenuListIconIndex::SCAN_ICON, "icon_scan_n.png");
    }

    // モデル側で保持しているScan結果をクリア
    _model->clearScanResult();

    // 給電
    _model->replaceMenuList(MenuListIconIndex::POWER_SUPPLY_ICON, "");
    _powerSupply = LC70_POWER_SUPPLY_OFF;
}

void MultiMeasurementViewModel::deactivate()
{
    AbstractMeasurementViewModel::deactivate();

    disconnect(this, SIGNAL(resumeMultiMeasuring(LC70_MEAS_MULTI_T *, LC70_MEAS_SINGLE_T *, LC70_DISP_TYPE_T, uint32_t)), _model,
               SLOT(onResumeMultiMeasuring(LC70_MEAS_MULTI_T *, LC70_MEAS_SINGLE_T *, LC70_DISP_TYPE_T, uint32_t)));
    disconnect(_model, SIGNAL(multiMeasureResult(MultiResult, MeasureResult)), this, SLOT(onMultiMeasureResult(MultiResult,
                                                                                                               MeasureResult)));
    disconnect(_model, SIGNAL(notifyChannelChanging(bool)), this, SLOT(onNotifyChannelChanging(bool)));
    disconnect(_model, SIGNAL(scanningChannelResult(int, LC70_MEAS_MULTI_RESULT_T)), this, SLOT(onScanningChannelResult(int,
                                                                                                                        LC70_MEAS_MULTI_RESULT_T)));
    disconnect(_model, SIGNAL(scanningFinished(bool)), this, SLOT(onScanningFinished(bool)));
    disconnect(this, SIGNAL(changeCondition(LC70_MEAS_SINGLE_T *)), _model,
               SIGNAL(preferredMeasuringChangeCondition(LC70_MEAS_SINGLE_T *)));
    disconnect(this, SIGNAL(requestToSaveDataMulti(SaveLocation, MeasureType, QString, uint16_t, QList<MeasureResult>)), _mainFrame,
               SLOT(onRequestToSaveDataMulti(SaveLocation, MeasureType, QString, uint16_t, QList<MeasureResult>)));
    disconnect(this, SIGNAL(requestToCapture()), _mainFrame, SLOT(onRequestToCapture()));
    disconnect(this, SIGNAL(requestToSaveImage(MeasureType, SaveLocation, QString)), _mainFrame, SLOT(onRequestToSaveImage(MeasureType, SaveLocation, QString)));
    disconnect(this, SIGNAL(requestToClearCaptureImage()), _mainFrame, SLOT(onRequestToClearCaptureImage()));
    disconnect(_mainFrame, SIGNAL(inputFileName(PopupId, SaveLocation, QString)), this, SLOT(onInputFileName(PopupId,
                                                                                                             SaveLocation, QString)));
    disconnect(_mainFrame, SIGNAL(requestToScan()), this, SLOT(onRequestToScan()));
    disconnect(_mainFrame, SIGNAL(requestToStopScanning()), this, SLOT(onRequestToStopScanning()));
    disconnect(_mainFrame, SIGNAL(requestToStopAcquring()), this, SLOT(onRequestToStopAcquiring()));
    disconnect(_mainFrame, SIGNAL(notifyButtonInput(PopupId, ButtonId)), this, SLOT(onNotifyButtonInput(PopupId,
                                                                                                        ButtonId)));
    disconnect(this, SIGNAL(cursorPressed(QVariant)), _mainFrame, SIGNAL(cursorPressed(QVariant)));
    disconnect(this, SIGNAL(scrollPage(QVariant)), _view, SLOT(onScrollPage(QVariant)));
    disconnect(this, SIGNAL(progress(double)), _mainFrame, SIGNAL(progress(double)));
    disconnect(this, SIGNAL(progressText(QString, bool)), _mainFrame, SIGNAL(progressText(QString, bool)));
    disconnect(this, SIGNAL(changeAttribute(QString, QVariant)), _mainFrame, SLOT(onChangeAttribute(QString, QVariant)));
    disconnect(this, SIGNAL(removeAttribute(QString)), _mainFrame, SLOT(onRemoveAttribute(QString)));
    disconnect(_mainFrame, SIGNAL(notifyPowerSupplyError(uint16_t, uint8_t, QString)), this, SLOT(detectPowerSupplyError(uint16_t, uint8_t, QString)));
    disconnect(this, SIGNAL(requestToReScan()), this, SLOT(onRequestToReScan()));
    disconnect(this, SIGNAL(requestToShowRegistScanResultPopup()), this, SLOT(onRequestToShowRegistScanResultPopup()));
    disconnect(this, SIGNAL(acquiringFinished(bool)), _mainFrame, SIGNAL(acquiringFinished(bool)));
    disconnect(this, SIGNAL(requestToSuspendMultiMeasuring()), _model, SLOT(onSuspendMultiMeasuring()));
    disconnect(_mainFrame, SIGNAL(notifySaveEnd()), this, SLOT(onNotifySaveEnd()));

    // メニューバーの情報をクリア
    setMenuBarVisible(false);

    // 給電
    _model->removeMenuList(MenuListIconIndex::POWER_SUPPLY_ICON);

    // スキャン
    _model->removeMenuList(MenuListIconIndex::SCAN_ICON);

    // ユーザー設定名をクリアー
    if (_view) {
        _view->setProperty("userMode", QString());
    }

    // SAVE1, 2 ボタン表示を更新
    setBoolProperty(IMAGE_BUTTON_NAME, "pressed", false);
    setBoolProperty(DATA_BUTTON_NAME, "pressed", false);
}

void MultiMeasurementViewModel::onKeyStatus(ButtonId id)
{
    // 測定データ保存内容確認中
    if (_dataSave2State != DataSave2None) {
        if (id != BUTTON_DATA_SAVE && id != BUTTON_CHANNEL_DOWN  && id != BUTTON_CHANNEL_UP && id != BUTTON_ESCAPE &&
                id != BUTTON_UP  && id != BUTTON_DOWN) {
            return;
        }
    }

    // メニューバー表示中
    if (isMenuBarVisible()) {
        menuBarOperation(id);
        return;
    }

    if (id == BUTTON_HOME) {
        // スレッドを停止
        emit requestToSuspendMultiMeasuring();

        AbstractViewModel::onKeyStatus(id);
    } else if (id == BUTTON_ENTER) {
        int index = -1;
        if (_view) {
            QObject *target = _view->findChild<QObject *>(_activeButton);

            if (target) {
                bool ok = false;
                // 押下前の画像に更新
                target->setProperty("pressed", false);
                index = target->property("index").toInt(&ok);

                if (!ok) {
                    return;
                }
            }
        }

        if (index < 0) {
            return;
        }
        if (_selectedButtonIndex == GRAPH_BUTTON_INDEX) {
                     _autoScroll = false;
                     if (_view) {
                         _view->setProperty("autoScroll", _autoScroll);
                     }

                 }


        // リストの自動スクロール切り替え
        if (_selectedButtonIndex == LIST_BUTTON_INDEX) {
            _autoScroll = !_autoScroll;
            if (_view) {
                _view->setProperty("autoScroll", _autoScroll);
            }
        }

        // グラフ/リスト表示切り替え
        if(_selectedButtonIndex == index) {
            return;
        }
        onRequest(index);
    } else if (id == BUTTON_SCAN) {
        // ユーザー設定から来た場合はスキャン不可
        if (!_fromUserSetting) {
            // FM, PILOT, LTE, CATV 上りはスキャン不可
            if (_bandId == LC70_BAND_UNDEF || _bandId == LC70_BAND_FM || _bandId == LC70_BAND_PILOT || _bandId == LC70_BAND_LTE || _bandId == LC70_BAND_CATV_UP_STREAM) {
                emit requestToShowPopupWithMessage(POPUP_SCAN_NOT_APPLICABLE, QString());            return;
                return;
            }
            int bandId = _takeOver["selectedBand"].toInt();
            emit requestToShowPopupScan(POPUP_SCAN_CONFIRMATION, bandId);
        }
    } else if (id == BUTTON_DATA_SAVE) {
        saveMeasuringMData();
    } else if (id == BUTTON_SCREEN_SAVE) {
        // リストのときは使用しない
        if (_selectedButtonIndex == GRAPH_BUTTON_INDEX) {

            QString path = _model->applicationSetting()->resultDataPathMulti();
            int fileCount = countFilesRecursive(path);
            if (fileCount >= 1000) {
                emit requestToShowPopup(POPUP_DATA_MULTI_MAX_SAVE_FILE);
                return;
            }

            // 画像保存
            _saveFileType = SaveFileType::PNG;

            //SCREENボタン押下時の画像を保持する
            emit requestToCapture();
            setBoolProperty(IMAGE_BUTTON_NAME, "pressed", false);

            emit requestToShowPopup(POPUP_INPUT_FILENAME);
        }
    } else if ( (id == BUTTON_CHANNEL_UP) || (id == BUTTON_UP) ){
        int previousChannelIndex = 0;
        selectGraphIndex(_currentChannelIndex, previousChannelIndex);

        if (_selectedButtonIndex == GRAPH_BUTTON_INDEX) {
            _currentChannelIndex = nextChannel();
        } else if (_selectedButtonIndex == LIST_BUTTON_INDEX) {
            _currentChannelIndex = previousChannel();
        }

        if (_currentChannelIndex < 0) {
            return;
        }

        int page = (_currentChannelIndex / LIST_ROWS) + 1;

        if (page != _currentPage) {
            _currentPage = page;
            emit scrollPage(_currentPage);
        }

        // グラフのインデックスを特定する
        int currentChannelIndex = 0;
        selectGraphIndex(_currentChannelIndex, currentChannelIndex);

        // チャンネル変更
        if (_multiGraphModel) {
            _multiGraphModel->changeSelection(previousChannelIndex, currentChannelIndex);
        }
        changeChannel(false, true);

        // グラフの実装変更後以下のチャンネル変更確定処理は使用していない
        //        // チャンネル変更確定タイマーを開始する
        //        startConfirmedChangeChannelTimer();
    } else if ( (id == BUTTON_CHANNEL_DOWN) || (id == BUTTON_DOWN) ){
        int previousChannelIndex = 0;
        selectGraphIndex(_currentChannelIndex, previousChannelIndex);

        if (_selectedButtonIndex == GRAPH_BUTTON_INDEX) {
            _currentChannelIndex = previousChannel();
        } else if (_selectedButtonIndex == LIST_BUTTON_INDEX) {
            _currentChannelIndex = nextChannel();
        }

        if (_currentChannelIndex < 0) {
            return;
        }

        int page = (_currentChannelIndex / LIST_ROWS) + 1;

        if (page != _currentPage) {
            _currentPage = page;
            emit scrollPage(_currentPage);
        }

        // グラフのインデックスを特定する
        int currentChannelIndex = 0;
        selectGraphIndex(_currentChannelIndex, currentChannelIndex);

        // チャンネル変更
        if (_multiGraphModel) {
            _multiGraphModel->changeSelection(previousChannelIndex, currentChannelIndex);
        }
        changeChannel(false, true);

        // グラフの実装変更後以下のチャンネル変更確定処理は使用していない
        //        // チャンネル変更確定タイマーを開始する
        //        startConfirmedChangeChannelTimer();
    } else if (id == BUTTON_MENU) {
        if (isMenuBarVisible()) {
            return;
        }

        // メニューバーの設定を初期設定とする
        AbstractViewModel::initializeMenuBarInformation();

        // メニューバーの表示
        _menuBarType = 1;
        emit requestToShowMenuBar(_menuBarType, _menuBarRank, _menuBarIndexMap.value(_menuBarRank));
        setMenuBarVisible(true);
    } else if (id == BUTTON_BAND) {
        if (isMenuBarVisible()) {
            return;
        }

        // メニューバーの表示
        emit requestToShowMenuBar(_menuBarType, _menuBarRank, _menuBarIndexMap.value(_menuBarRank));
        emit cursorMoveDown(_menuBarRank);
        // 第 2 階層(Band)
        selectNextMenuBar(_menuBarRank, _menuBarIndexMap.value(_menuBarRank), true);
        setMenuBarVisible(true);

        // 測定設定をシングル測定
        _takeOver["selectedMeasurement"] = QString(MULTI_MEASURE_TYPE_NAME);
        emit changeAttribute(QString("selectedMeasurement"), MULTI_MEASURE_TYPE_NAME);

        // 画面遷移履歴を追加
        _changeHistory.clear();
        _changeHistory.append(SCREEN_HOME);

    } else if (id == BUTTON_USER) {
        // ユーザーデータロガー
#ifdef NEW_USER_SETTING
        emit requestDirect(SCREEN_USER_SETTING, true);
#else
        emit requestDirect(SCREEN_USER_DATA_LOGGER_MENU, true);
#endif
    } else if (id == BUTTON_POWER_SUPPLY) {
        // USER 設定で複数 Band を跨って測定する場合
        if (_fromUserSetting && _hasMultiBandsInUserSetting) {
            // 複数 Band 向け給電切り替え
            switchPowerSupplyForMultiBand();
        } else {
            // 給電設定切り替え
            switchPowerSupply();
        }
    } else if (id == BUTTON_ESCAPE) {
        _model->removeMenuList(MenuListIconIndex::PREAMP_ICON);
        if (!_fromUserSetting) {
            // 画面共有情報の更新を実施
            emit changeAttribute("fromUserSetting", false);

            // ユーザ設定からの遷移以外はBAND選択画面に遷移する
            emit requestInHistory(SCREEN_BAND);
        } else {
            // 画面共有情報の更新を実施
            emit changeAttribute("fromUserSetting", false);

            //自動スクロールをオンにしていると、測定再開時に先頭チャンネルから結果表示しない不具合を修正
            clear();

            // ユーザ設定からの遷移の場合は、ユーザ設定画面に遷移する
            AbstractViewModel::onKeyStatus(id);
        }
    } else {
        moveCursor(id);
    }
}

void MultiMeasurementViewModel::onButtonPress(ButtonId id)
{
    // 測定データ保存内容確認中
    if (_dataSave2State != DataSave2None) {
        return;
    }

    // メニューバー表示中
    if (isMenuBarVisible()) {
        if (id == BUTTON_ENTER) {
            if (_menuBarType == 1) {
                MainFrameViewModel *mainFrame = static_cast<MainFrameViewModel *>(_mainFrame);
                QVariantList vFileList = mainFrame->getUserFileList();
                if (vFileList[_menuBarIndexMap.value(_menuBarRank)].toString().isEmpty()) {
                    return;
                }
            }
            emit cursorPressed(_menuBarRank);
        }

        return;
    }

    if (id == BUTTON_ENTER) {
        setBoolProperty(_activeButton, "pressed", true);
    } else if (id == BUTTON_DATA_SAVE) {
        setBoolProperty(DATA_BUTTON_NAME, "pressed", true);
        if (_view) {
            setButtonActive(_view, _activeButton, false);
        }
    } else if (id == BUTTON_SCREEN_SAVE) {
        if (!_fromUserSetting && _selectedButtonIndex == GRAPH_BUTTON_INDEX) {
            setBoolProperty(IMAGE_BUTTON_NAME, "pressed", true);
        }
    }
}

void MultiMeasurementViewModel::onAccept(PopupId popupId, bool okOnly)
{
    Q_UNUSED(okOnly);

    AbstractViewModel::onAccept(popupId, okOnly);

    if (popupId == POPUP_POWER_SUPPLY_SELECT2) {
        // USER 設定で複数 Band を跨って測定する場合
        if (_fromUserSetting && _hasMultiBandsInUserSetting) {
            _model->setPowerSupplyForMultiBand(1);
            _powerSupply = _condition.single[_currentMeasuringIndex].powerSupply;
            _model->replaceMenuList(MenuListIconIndex::POWER_SUPPLY_ICON, Correcter::powerSupplyIcon(_powerSupply));
        } else {
            // 給電選択で 15v を選択
            _powerSupply = LC70_POWER_SUPPLY_15V;
            _model->setPowerSupply(_powerSupply);
            _model->replaceMenuList(MenuListIconIndex::POWER_SUPPLY_ICON, Correcter::powerSupplyIcon(_powerSupply));
        }
        // 給電中
        _powerSupplied = true;

        return;
    } else if (popupId == POPUP_POWER_SUPPLY_ERROR_MANUAL_MEASURING) {
        // ブザーを止める
        _model->buzzer(LC70_BUZZ_OFF, LC70_BUZZ_MODE_STOP);

#if 0
        // 測定再開
        // USER 設定のマルチ測定であれば遷移先画面種別はリストにする
        LC70_DISP_TYPE_T displayType = LC70_DISP_TYPE_GRAPH;
        if (_selectedButtonIndex == LIST_BUTTON_INDEX) {
            displayType = LC70_DISP_TYPE_LIST_MULTI;
        }
        emit resumeMultiMeasuring(&_condition, &_preferredCondition, displayType,
                                  _model->applicationSetting()->pollingIntervalMultiMeasurement());
#endif
    } else if (popupId == POPUP_SELECT_SAVE_DATA) {
        // 保存先選択ダイアログで"ユーザ設定に保存"を選択した場合
        emit requestSetTakeOver("powerSupply", _powerSupply);
        emit requestSetTakeOver("dataloggerFunction", 6);
        emit requestSetTakeOver("selectedMeasurement", "MultiMeasurement");
        emit requestDirect(SCREEN_FILE_SELECTION, false);
        return;
    } else if (popupId == POPUP_SCAN_RESULT) {
        // SCAN結果保存確認ダイアログで"はい"を選択した場合
#if 1
        // オートパワーオフ設定を有効化
        _model->enablePowerOffSetting();
#endif

        emit requestSetTakeOver("powerSupply", _powerSupply);
        emit requestSetTakeOver("dataloggerFunction", 6);
#ifdef NEW_USER_SETTING
        // 保存先選択で USER データロガー設定を呼び出す
        emit requestSetTakeOver("userDataloggerFunction", CHANNEL_EDIT_SELECT_DESTINATION);
        emit requestDirect(SCREEN_USER_DATALOGGER_SETTING, false);
#else
        emit requestDirect(SCREEN_FILE_SELECTION, false);
#endif
        return;
    } else if (popupId == POPUP_POWER_SUPPLY_ERROR_USER_MEASURING) {
        // 測定再開
        LC70_DISP_TYPE_T displayType = LC70_DISP_TYPE_GRAPH;
        // 遷移先画面種別を元に戻す
        if (_selectedButtonIndex == LIST_BUTTON_INDEX) {
            displayType = LC70_DISP_TYPE_LIST_MULTI;
            // USER 設定のマルチ測定で複数 Band を測定する場合はデータロガー
            if (_fromUserSetting && _hasMultiBandsInUserSetting) {
                displayType = LC70_DISP_TYPE_LIST_LOGGER;
            }
        }

        emit resumeMultiMeasuring(&_condition, &_preferredCondition, displayType,
                                  _model->applicationSetting()->pollingIntervalMultiMeasurement());

    } else if (popupId == POPUP_CONFIRM_ACQUIRE_MEASURE_DATA) {

        QString path = _model->applicationSetting()->resultDataPathMulti();
        int fileCount = countFilesRecursive(path);
        if (fileCount >= 1000) {
            emit requestToShowPopup(POPUP_DATA_MULTI_MAX_SAVE_FILE);
            return;
        }
#if 1
        // オートパワーオフ設定を無効化
        qDebug() << QString("オートパワーオフ設定を無効化");
        _model->disablePowerOffSetting();
#endif
        _startSaveData = true;

        // 保存する測定データを取得
        acquireMeasuringData();
        // 測定結果表示中ラベルを表示
        setBoolProperty(MEASUREMENT_RESULT_DISPLAY, "visible", true);
    } else if(popupId == POPUP_MEASURE_COMPLETE) {
        // 保存しますか？　→　はい
        // キーボードを開いて保存
#if 1
        // オートパワーオフ設定を有効化
        qDebug() << QString("オートパワーオフ設定を有効化");
        _model->enablePowerOffSetting();
#endif
        saveMeasuringMData();
    } else if (popupId == POPUP_FAILURE_READ_USER_SETTING_FILE) {
        // USER設定による測定開始前に戻る
        abortUserSettingMeasurement();
    } else if (popupId == POPUP_DATA_MULTI_MAX_SAVE_FILE) {
        setBoolProperty(IMAGE_BUTTON_NAME, "pressed", false);
        setBoolProperty(DATA_BUTTON_NAME, "pressed", false);
    } else {
        // nothing to do
    }
}

void MultiMeasurementViewModel::onReject(PopupId popupId)
{
    if (popupId == POPUP_SCAN_RESULT) {
#if 1
        // オートパワーオフ設定を有効化
        _model->enablePowerOffSetting();
#endif
        _model->clearScannedChannelList();

        if (_view) {
            _view->setProperty("userMode", NO_FILE_NAME);
        }

        // スキャン後処理を実施
        startMeasuringAfterScan();
    } else if (popupId == POPUP_INPUT_FILENAME) {
        // 保存するために保持した各データをクリアする
        emit requestToClearCaptureImage();
        // もとの機能に戻す
        returnToMeasurement();
    } else if (popupId == POPUP_SELECT_SAVE_DATA){
        // 保持している測定データをクリアする
        _holdMeasurementData.clear();

    } else if (popupId == POPUP_CONFIRM_ACQUIRE_MEASURE_DATA) {
        // 選択表示を戻す
        setBoolProperty(DATA_BUTTON_NAME, "pressed", false);
        if (_view) {
            setButtonActive(_view, _activeButton, true);
        }
    } else if (popupId == POPUP_SAVE_MEASUREMENT_DATA) {
#if 1
        // オートパワーオフ設定を有効化
        qDebug() << QString("オートパワーオフ設定を有効化");
        _model->enablePowerOffSetting();
#endif
        // もとの機能に戻す
        returnToMeasurement();
        // USER 設定の場合測定を再開しておく
        if (_fromUserSetting && _hasMultiBandsInUserSetting) {
            // スレッドを開始
            emit resumeMultiMeasuring(&_condition, &_preferredCondition, LC70_DISP_TYPE_LIST_LOGGER,
                                      _model->applicationSetting()->pollingIntervalMultiMeasurement());
        }
    } else if(popupId == POPUP_MEASURE_COMPLETE) {
        // 保存しますか？　→　いいえ
#if 1
        // オートパワーオフ設定を有効化
        qDebug() << QString("オートパワーオフ設定を有効化");
        _model->enablePowerOffSetting();
#endif
        // もとの機能に戻す
        returnToMeasurement();
        // USER 設定の場合測定を再開しておく
        if (_fromUserSetting && _hasMultiBandsInUserSetting) {
            // スレッドを開始
            emit resumeMultiMeasuring(&_condition, &_preferredCondition, LC70_DISP_TYPE_LIST_LOGGER,
                                      _model->applicationSetting()->pollingIntervalMultiMeasurement());
        }
    } else if (popupId == POPUP_FAILURE_READ_USER_SETTING_FILE) {
        // USER設定による測定開始前に戻る
        abortUserSettingMeasurement();
    } else if (popupId == POPUP_MEASURE_COMPLETE) {
        // USER 設定の場合測定を再開しておく
        if (_fromUserSetting && _hasMultiBandsInUserSetting) {
            // スレッドを開始
            emit resumeMultiMeasuring(&_condition, &_preferredCondition, LC70_DISP_TYPE_LIST_LOGGER,
                                      _model->applicationSetting()->pollingIntervalMultiMeasurement());
        }
    } else {
        // nothing to do
    }
}

void MultiMeasurementViewModel::onRejectScanResult()
{

    // スキャンアイコンなし
    _model->removeMenuList(MenuListIconIndex::SCAN_ICON);

    // スキャン後のチャンネル一覧をクリアー
    _model->clearScannedChannelList();

    // スキャン結果をクリアー
    _model->clearScanResult();

    // いったん測定を停止(遷移先画面種別測定画面以外に変更)
    emit requestToSuspendMultiMeasuring();

    // 表示クリアー
    clear();

    // 測定を開始
    onScanningFinished(false);
}

void MultiMeasurementViewModel::onRequest(int condition)
{
    if (_view == nullptr || _fixedGraph == nullptr || _list == nullptr) {
        return;
    }

    // 測定結果の初期化
    _multiMeasureResult.clear();
    changeMultiListModel();

    // データの不整合を避けるためグラフ、リストの表示変更の場合はいったん測定を停止(遷移先画面種別測定画面以外に変更)
    emit requestToSuspendMultiMeasuring();

    // USER 設定で Band を跨る場合はリストの表示のみ
    if (_hasMultiBandsInUserSetting) {
        // マルチチャンネル測定結果のクリアー
        for (int i = 0; i < _channelList.count(); ++i) {
            // リスト表示内容の更新
            if (_measureResultModel) {
                _measureResultModel->update(i, -1, -1, -1,0,0,10);
            }
        }

        if (_multiGraphModel) {
            // グラフを更新
            _multiGraphModel->clear();
        }

        // リストを先頭に移動
        _currentPage = 1;
        if (_view) {
            _view->setProperty("currentPage", _currentPage);    // 現在のページ
            emit scrollPage(_currentPage);
        }
        if (_view) {
            // 測定中のチャンネルのインデックスを 0 にする
            _view->setProperty("scanChannelIndex", 0);
        }
        if (_view) {
            // 測定中のチャンネルのインデックスを 0 にする
            _view->setProperty("currentChannelIndex", 0);
        }

        // 測定を再開
        emit resumeMultiMeasuring(&_condition, &_preferredCondition, LC70_DISP_TYPE_LIST_LOGGER,
                                  _model->applicationSetting()->pollingIntervalMultiMeasurement());
        return;
    }

    // 表示クリア
    clear();

    _selectedButtonIndex = condition;

    LC70_DISP_TYPE_T displayType = LC70_DISP_TYPE_GRAPH;

    // グラフとリストの表示選択
    switch (condition) {
    case GRAPH_BUTTON_INDEX: {    // グラフ
        _model->writeLog(QString("マルチ測定"), QString("グラフ表示"));

        if (_fixedGraph) {
            _fixedGraph->setProperty("visible", true);
        }

        if (_list) {
            _list->setProperty("visible", false);
        }
        _isChannelChanging = true;

        // チャンネルを先頭に戻す
        _currentChannelIndex = 0;
        changeChannel(true);

        changeMultiGraphModel();

        int currentChannelIndex = 0;
        selectGraphIndex(0, currentChannelIndex);

        if (_multiGraphModel) {
            // グラフ表示時、白線が残ってしまうことを防ぐため白線の位置を初期化する
            _multiGraphModel->InitializeCurrentPole(true, currentChannelIndex);
        }
        if (_fromUserSetting) {
            // ユーザー設定で選択した内容を測定条件に反映
            setUserMultiMeasuringCondition(true);
        } else {
            // 通常操作の選択内容を測定条件に反映
            setMultiMeasuringCondition(true);
        }

        setBoolProperty(IMAGE_BUTTON_NAME, "visible", true);
        setBoolProperty(IMAGE_BUTTON_NAME, "enable", true);
        if (_view) {
            _view->setProperty("selectGraphList", 0);
        }

        break;
    }
    case LIST_BUTTON_INDEX: {    // リスト
        _model->writeLog(QString("マルチ測定"), QString("リスト表示"));
        if (_fixedGraph) {
            _fixedGraph->setProperty("visible", false);
        }
        if (_list) {
            _list->setProperty("visible", true);
        }
        _isChannelChanging = true;

        // チャンネルを先頭に戻す
        _currentChannelIndex = 0;
        changeChannel(true);

        displayType = LC70_DISP_TYPE_LIST_MULTI;

        if (_fromUserSetting) {
            // ユーザー設定で選択した内容を測定条件に反映
            setUserMultiMeasuringCondition(false);
        } else {
            // 通常操作の選択内容を測定条件に反映
            setMultiMeasuringCondition(false);
        }

        setBoolProperty(IMAGE_BUTTON_NAME, "visible", false);
        setBoolProperty(IMAGE_BUTTON_NAME, "enable", false);
        if (_view) {
            _view->setProperty("selectGraphList", 1);
        }

        break;
    }

    default:
        break;
    }

    // 測定を再開
    emit resumeMultiMeasuring(&_condition, &_preferredCondition, displayType,
                              _model->applicationSetting()->pollingIntervalMultiMeasurement());

    // マルチ測定は点滅なしのため以下コメントアウト
    //    // 優先測定のロック待ちタイマーを開始
    //    // 点滅時間設定
    //    if (_view) {
    //        _view->setProperty("blinkInterval", _model->applicationSetting()->blinkInterval());
    //    }

    // ロック待ち時間切れをリセット
    _waitforLockingTimeOut = false;

    // 値未確定
    _confirmedValue = false;

    _lockWaitingTimer->setSingleShot(true);
    if (_lockWaitingTimer->isActive()) {
        _lockWaitingTimer->stop();
    }

    // ロック待ちタイマー開始
    _lockWaitingTimer->start();
}

void MultiMeasurementViewModel::onMultiMeasureResult(MultiResult result, MeasureResult)
{
    if (_view == nullptr || _fixedGraph == nullptr || _list == nullptr) {
        return;
    }

    // チャンネル変更中か、チャンネル確定までのタイマーが動作中であれば届いた測定結果は表示しない
    if (_isChannelChanging || _confirmeChangeChannelTimer->isActive()) {
        return;
    }

    // 無効なインデックスの場合
    if (_currentChannelIndex < 0) {
        _model->writeLog(QString("マルチ測定"), QString("%1 all invalid channels").arg(__FUNCTION__));
        return;
    }

    // チャンネル情報がなければ表示しない
    if (_channelList.count() < 1) {
        return;
    }

    // 測定結果に不正なインデックスが設定されている場合は表示しない
    if (result.readIndex >= _channelList.count() ||
            result.currentMeasuringIndex >= _channelList.count()) {
        return;
    }

#if 1
    // SCAN中は測定結果を表示しない
    if (_isScanning) {
        return;
    }
#endif

    static int errorCount = 0;

    if (_startSaveData) {
        errorCount = 0;
        _startSaveData = false;
    }

    if (!result.received) {
        ++errorCount;
    } else {
        errorCount = 0;
    }

    // 一定回数エラーが続いたらサブ CPU をリセットする
    if (errorCount == REPEAT_TIMES_TO_RESET) {
        errorCount = 0;
        _model->resetSubCpu();

        // 画面起動時の状態に戻す
        resetScreen();
        return;
    }

    if (result.locked == LOCKED) {

        // ロック待ちのタイマーを止める
        if (_lockWaitingTimer->isActive()) {
            _lockWaitingTimer->stop();
        }
        // 点滅を止める
        if (_view) {
            _view->setProperty("blinkInterval", 0);
        }

        // 値確定
        _confirmedValue = true;

        bool isUpdateMerCnrRange = false;
        // 指定した変調方式と異なる変調方式が来た場合
        if (result.result.Modulation != _condition.single[result.readIndex].usMod) {
            _condition.single[result.readIndex].usMod = result.result.Modulation;
            isUpdateMerCnrRange = true;
        }

        // MER/CNRの範囲更新
        if (_fromUserSetting) {
            checkMerCnrRange(
                        _userSettings[result.readIndex].ucBand,
                    _userSettings[result.readIndex].ucSpec,
                    _userSettings[result.readIndex].ulModBit,
                    isUpdateMerCnrRange);
        }else {
            checkMerCnrRange(
                        _channelList[result.readIndex].band,
                    result.result.BroadcastSystem,
                    result.result.Modulation,
                    isUpdateMerCnrRange);
        }
    }

    static int count = 0;
    quint64 totalTime = 0;
    timer.restart();

    // インデックスが 0 だったら初期値に戻す
    if (result.readIndex == 0) {
        _previousBandId = _channelList[result.readIndex].band;
        _barType = 0;
    }

    // マルチ測定結果の受信内容
    if (result.readIndex < _channelList.count()) {
        _model->writeLog(QString("マルチ測定"),
                         QString("測定結果 Channel:%1 Status:%2 Current:%3 Read:%4 Level:%5").arg(
                             _channelList[result.readIndex].channelNumber).arg(
                    result.status).arg(
                    result.currentMeasuringIndex).arg(
                    result.readIndex).arg(result.result.Level), LOG_INFO);
    } else {
        _model->writeLog(QString("マルチ測定"),
                         QString("[ERROR] INVALID READ INDEX: %1(CHANNEL LIST CONTAINS %2 CHANNEL(S)").arg(
                             result.readIndex).arg(_channelList.count()), LOG_INFO);
        return;
    }

    // 測定結果を取得していなければ無視する
    if (!result.received) {
        return;
    }

    // FM, CATV 上りの場合、無効なチャンネルは無視する
    if (_channelList[result.readIndex].band == LC70_BAND_FM ||
            _channelList[result.readIndex].band == LC70_BAND_CATV_UP_STREAM) {
        if (!_channelList[result.readIndex].valid) {
            result.received = false;
            result.locked = UNLOCKED;
            result.result.Level = -1;
        }
    }

    // 測定中のチャンネルのインデックス(読み出しの一つ後に設定)
    _currentMeasuringIndex = result.currentMeasuringIndex;

    if (_currentMeasuringIndex >= _channelList.count()) {
        _currentMeasuringIndex = 0;
    }

    if (_view) {
        _view->setProperty("scanChannelIndex", _currentMeasuringIndex);
    }

    // Band アイコンの差し替え
    _model->replaceBandIcon(static_cast<LC70_BAND_T>(_channelList[result.currentMeasuringIndex].band));

    // USER 設定で複数の Band を測定する場合は各チャンネルの給電設定を反映
    if (_fromUserSetting && _hasMultiBandsInUserSetting) {
        if (_powerSupply != LC70_POWER_SUPPLY_OFF) {
            _powerSupply = static_cast<LC70_POWER_SUPPLY_T>(_condition.single[_currentMeasuringIndex].powerSupply);
            _model->replaceMenuList(MenuListIconIndex::POWER_SUPPLY_ICON, Correcter::powerSupplyIcon(_powerSupply));
        }
    }

    // マルチチャンネル測定結果
    // 収集ステータスの該当ビットが立っていれば更新する
    int color = 0;

    if ((result.status & LC70_DATA_MSK_LEVEL) == LC70_DATA_MSK_LEVEL) {

        // BAND が変わるたびに交互にバーの色を変える
        if (_channelList[result.readIndex].band != _previousBandId) {
            _previousBandId = _channelList[result.readIndex].band;
            _barType = (_barType == 0) ? 1 : 0;
        }

        // バーの色を設定
#if 1
        if ((result.locked == LOCKED) && (result.result.Mer > 5.0)) {
#else
        if (result.locked == LOCKED) {
#endif
            color = (_barType == 0) ? 1 : 3;
        } else {
            if (_channelList[result.readIndex].band == LC70_BAND_LTE ||
                    _channelList[result.readIndex].band == LC70_BAND_FM ||
                    _channelList[result.readIndex].band == LC70_BAND_PILOT ||
                    _channelList[result.readIndex].band == LC70_BAND_CATV_UP_STREAM) {
                // LTE, FM, PILOT, CATV 上りは一色のみ
                color = (_barType == 0) ? 1 : 3;
            } else {
                color = (_barType == 0) ? 0 : 2;
            }
        }

        _multiMeasureResult[result.readIndex].Received = result.received;
        _multiMeasureResult[result.readIndex].Locked = result.locked;
        _multiMeasureResult[result.readIndex].Status = result.status;

        _multiMeasureResult[result.readIndex].BroadcastSystem = result.result.BroadcastSystem;
        _multiMeasureResult[result.readIndex].Modulation = result.result.Modulation;
        _multiMeasureResult[result.readIndex].Layer = result.result.Layer;
        if (_dataSave2State == DataSave2Retrieving) {
            // ログ収集中はLockedの値を残しておく
            _multiMeasureResult[result.readIndex].Locked = result.locked;
        }

        // 衛星系は偏波
//        if (Correcter::isSatelliteBroadCasting(_bandId)) {
//            _multiMeasureResult[result.readIndex].Layer = _channelList[_currentChannelIndex].polarization;
//        }
        if (Correcter::isSatelliteBroadCasting(_multiMeasureResult[result.readIndex].Band.toInt())) {
            _multiMeasureResult[result.readIndex].Layer = _channelList[result.readIndex].polarization;
        }
        _multiMeasureResult[result.readIndex].Level = (result.received) ? result.result.Level : -1;
        if (_multiMeasureResult[result.readIndex].Level > MAXIMUM_LEVEL) {
            _multiMeasureResult[result.readIndex].Level = MAXIMUM_LEVEL;
        }

        int valueMerCN = Correcter::selectCnrMer(static_cast<LC70_BAND_T>(_channelList[result.readIndex].band), _cnrMerSetting);

        _model->writeLog(QString("マルチ測定"),
                         QString("MER VALUE:%1 MER/CN:%2 CONDITION:%3 LOCKED:%4").arg(result.result.Mer).arg(valueMerCN).arg(result.status).arg(result.locked), LOG_INFO);

        if ((result.locked == LOCKED) && result.received) {
            if (isnanf(result.result.Mer) || isinff(result.result.Mer)) {
                _multiMeasureResult[result.readIndex].Mer = UNDEFINED_VALUE;
            } else {
                if (((result.status & LC70_DATA_MSK_CNR) == LC70_DATA_MSK_CNR && valueMerCN == LC70_MEAS_SET_SEL_CNR) ||
                        ((result.status & LC70_DATA_MSK_MER) == LC70_DATA_MSK_MER && valueMerCN == LC70_MEAS_SET_SEL_MER)) {
                    _multiMeasureResult[result.readIndex].Mer = result.result.Mer;
                } else {
                    // nothing to do.


                    _multiMeasureResult[result.readIndex].Mer = UNLOCKED_VALUE;
                }
            }
        } else {
            if(_multiMeasureResult[result.readIndex].BroadcastSystem == LC70_BD_SYS_ANNEXB || _multiMeasureResult[result.readIndex].BroadcastSystem == LC70_BD_SYS_ANNEXC || _multiMeasureResult[result.readIndex].BroadcastSystem == LC70_BD_SYS_OPTICAST ){
                _multiMeasureResult[result.readIndex].Mer = -3;
            }else{
                // ファイル出力時に "<5.0" と出力する
                _multiMeasureResult[result.readIndex].Mer = UNLOCKED_VALUE;
            }
        }

        _model->writeLog(QString("マルチ測定"),
                         QString("BER VALUE:%1 CONDITION:%2 LOCKED:%3").arg(result.result.Ber).arg(result.status).arg(result.locked), LOG_INFO);

        if (result.locked == LOCKED) {
            if (isnanf(result.result.Ber) || isinff(result.result.Ber)) {
                _multiMeasureResult[result.readIndex].Ber = UNDEFINED_VALUE;
            } else {
                if ((result.status & LC70_DATA_MSK_BER) == LC70_DATA_MSK_BER) {
                    _multiMeasureResult[result.readIndex].Ber = result.result.Ber;
                } else {
                    // nothing to do.
                     //BERのステータスbitが立たない場合に2.2e-2を表示

                    _multiMeasureResult[result.readIndex].Ber = UNLOCKED_VALUE;
                }
            }
        } else {
            // ファイル出力時に "1.0e-2" と出力する
            _multiMeasureResult[result.readIndex].Ber = UNLOCKED_VALUE;
        }

        // リスト表示内容の更新
        if (_selectedButtonIndex == LIST_BUTTON_INDEX) {
            // リスト更新
            if (_measureResultModel) {
                _measureResultModel->update(result.readIndex, result.result.Level, _multiMeasureResult[result.readIndex].Mer,
                        _multiMeasureResult[result.readIndex].Ber,_bandId,_multiMeasureResult[result.readIndex].BroadcastSystem,_multiMeasureResult[result.readIndex].Modulation);
            }
        } else {
            // グラフのインデックスを特定する
            int readIndex = 0;
            int currentIndex = 0;
            float32_t graphvalue = 0;
             float32_t minimumLevel_bscs = 0; //BSCS

            float32_t minimumLevel=0;
            selectGraphIndex(result.readIndex, readIndex);
            selectGraphIndex(result.currentMeasuringIndex, currentIndex);
            LC70_BAND_T band = static_cast<LC70_BAND_T>(_channelList.at(_currentChannelIndex).band);
            QString polarize = Correcter::convertPolarizeNumToString(_channelList.at(_currentChannelIndex).polarization);
            BandTabIndex bandTab = Correcter::convertBandTabIndex(band, polarize);
            Correcter::minimumLevel(bandTab,minimumLevel);



            LC70_BAND_T bandbscs = static_cast<LC70_BAND_T>(_channelList.at(result.readIndex).band);
            QString polarizebscs = Correcter::convertPolarizeNumToString(_channelList.at(result.readIndex).polarization);
            BandTabIndex bandTabbscs = Correcter::convertBandTabIndex(bandbscs, polarizebscs);
            Correcter::minimumLevel(bandTabbscs,minimumLevel_bscs);



            if (_multiGraphModel) {

                if(_bandId == LC70_BAND_BS_CS){
                    if(result.result.Level<=minimumLevel_bscs){
                        if(_channelList[result.readIndex].polarization == 0) {      // 右旋

                            graphvalue = 29;
                            _multiGraphModel->update(readIndex, currentIndex, graphvalue, color);

                        }
                        else if(_channelList[result.readIndex].polarization == 1) {	// 左旋
                            graphvalue = 39;
                            _multiGraphModel->update(readIndex, currentIndex, graphvalue, color);

                        }

                    }else if(result.result.Level>=120){

                        graphvalue = 120;
                        _multiGraphModel->update(readIndex, currentIndex, graphvalue, color);

                    }else{

                        // グラフ更新

                        _multiGraphModel->update(readIndex, currentIndex, result.result.Level, color);
                    }

                }else{


                    if(result.result.Level<=minimumLevel){
                        /*if(_channelList[readIndex].polarization == 0) {      // 右旋
                                         graphvalue = 29;
                                          _multiGraphModel->update(readIndex, currentIndex, graphvalue, color);

                                     }
                                     else if(_channelList[readIndex].polarization == 1) {	// 左旋
                                         graphvalue = 39;
                                          _multiGraphModel->update(readIndex, currentIndex, graphvalue, color);


                            _multiGraphModel->update(readIndex, currentIndex, graphvalue, color);

                        }


                        //graphvalue = 29;

                    }else{*/
                        graphvalue = minimumLevel;
                        // 20以下の時は1メモリ分高くグラフを表示する。また30以下、40以下の時は1メモリ分高くなっていたので、低くするために-1する
                        if(minimumLevel == 20){
                            graphvalue += 0.5;
                        }else{
                            graphvalue = minimumLevel;
                            graphvalue--;
                        }

                        _multiGraphModel->update(readIndex, currentIndex, graphvalue, color);


                    }else if(result.result.Level>=120){

                        graphvalue = 120;
                        _multiGraphModel->update(readIndex, currentIndex, graphvalue, color);

                    }else{

                        // グラフ更新

                        _multiGraphModel->update(readIndex, currentIndex, result.result.Level, color);
                    }
                }
            }
            _model->writeLog(QString("マルチ測定"),
                             QString("グラフ更新 index:%1 %2 ms(average)").arg(result.readIndex).arg(timer.elapsed()), LOG_INFO);
        }

        // 保存用データ取得中に最後のチャンネルまで表示し終えたら測定完了のポップアップを表示する
        if (_dataSave2State == DataSave2Retrieving) {
            double value = static_cast<double>(result.readIndex + 1) / static_cast<double>(_channelList.count()) * 100;
            emit progress(value);
            if (result.readIndex == 0) {
                // 先頭チャンネルが測定できたら表示を更新する
                holdMeasurementData();
                changeChannel();
                _holdMeasurementData.clear();
            }
            if (_holdMeasurementData.isEmpty() && result.readIndex == _channelList.count() - 1) {
                holdMeasurementData();

                // いったん測定を停止(遷移先画面種別測定画面以外に変更)
                emit requestToSuspendMultiMeasuring();

                emit acquiringFinished(true);

                emit requestToShowPopupWithMessage(POPUP_MEASURE_COMPLETE, QString("%1ch分 測定完了しました\n保存しますか？").arg(_channelList.count()));
            }
        }
    }

    // マルチ測定結果から現在選択されているチャンネルの結果を表示する
    if (result.readIndex == _currentChannelIndex) {
        displayPrefferedChannel(_currentChannelIndex);
    }

    if (_view) {
        _view->setProperty("scanChannelIndex", _currentMeasuringIndex);
    }

    totalTime += timer.elapsed();
    ++count;

    if (count == 10) {
        _model->writeLog(QString("マルチ測定"), QString("%1 %2 ms(average)").arg(__FUNCTION__).arg(totalTime / 10));
        count = 0;
        totalTime = 0;
    }
}

void MultiMeasurementViewModel::onInputFileName(PopupId popupId, SaveLocation saveLocation, QString fileName)
{
    Q_UNUSED(popupId);

    QString prefix;

    if (saveLocation == SAVE_MEASUREMENT_DATA) {
#if 1	//20210625_dx5
        prefix = "_" + QDateTime::currentDateTime().toString("yyyyMMddhhmmss")
#else
        prefix = "_" + QDateTime::currentDateTime().toString("yyyyMMddhhmm")
#endif
                + "_" + QString::number(_channelList.at(_currentChannelIndex).band);
    } else {
        prefix = "_" + QString::number(_channelList.at(_currentChannelIndex).band);
    }

    switch (_saveFileType) {
    // 測定データ
    case CSV: {
        fileName = fileName + prefix + CSV_FILE_EXTENSION;
        MeasureType measureType = MULTI_MEASUREMENT;
        if (_fromUserSetting && _hasMultiBandsInUserSetting) {
            measureType = LIST_MEASUREMNET;
        }
        emit requestToSaveDataMulti(saveLocation, measureType, fileName, _measuringCondition, _holdMeasurementData);
        break;
    }

        // 画像
    case PNG:
        fileName = fileName + prefix + PNG_FILE_EXTENSION;
        // キャプチャーした一時データを保存する
        emit requestToSaveImage(MULTI_MEASUREMENT, saveLocation, fileName);
        break;

    default:
        break;
    }

    _saveFileType = SaveFileType::SAVE_FILE_COUNT;

    // USER 設定の場合測定を再開しておく
    if (_fromUserSetting && _hasMultiBandsInUserSetting) {
        // スレッドを開始
        emit resumeMultiMeasuring(&_condition, &_preferredCondition, LC70_DISP_TYPE_LIST_LOGGER,
                                  _model->applicationSetting()->pollingIntervalMultiMeasurement());
    }
}

void MultiMeasurementViewModel::onNotifySaveEnd()
{
    // もとの機能に戻す
    returnToMeasurement();
}

void MultiMeasurementViewModel::onNotifyChannelChanging(bool changing)
{
    qDebug() << __FUNCTION__ << changing;
    _isChannelChanging = changing;

    if (_preferredCondition.ulFreq != _previousPreferredCondition.ulFreq ||
            _preferredCondition.ulLoFreq != _previousPreferredCondition.ulLoFreq ||
            _preferredCondition.usBdSys != _previousPreferredCondition.usBdSys ||
            _preferredCondition.usBand != _previousPreferredCondition.usBand ||
            _preferredCondition.usMod != _previousPreferredCondition.usMod ||
            _preferredCondition.usCondition != _previousPreferredCondition.usCondition ||
            QString::fromLocal8Bit((char *)_preferredCondition.ucStrChan) != QString::fromLocal8Bit((
                                                                                                        char *)_previousPreferredCondition.ucStrChan) ||
            _preferredCondition.ucOutputEnb != _previousPreferredCondition.ucOutputEnb ||
            _preferredCondition.powerSupply != _previousPreferredCondition.powerSupply) {
        qDebug() << QString("[Change condition again]");
        emit changeCondition(&_preferredCondition);
        _previousPreferredCondition = _preferredCondition;
        return;
    }

    // チャンネル変更後
    if (!changing) {
        // マルチ測定は点滅なしのため以下コメントアウト
        //        // 点滅時間設定
        //        if (_view) {
        //            _view->setProperty("blinkInterval", _model->applicationSetting()->blinkInterval());
        //        }

        // ロック待ち時間切れをリセット
        _waitforLockingTimeOut = false;

        // 値未確定
        _confirmedValue = false;

        _lockWaitingTimer->setSingleShot(true);
        if (_lockWaitingTimer->isActive()) {
            _lockWaitingTimer->stop();
        }

        // ロック待ちタイマー開始
        _lockWaitingTimer->start();
    }
}

void MultiMeasurementViewModel::onRequestToScan()
{
    // ユーザー設定は再スキャン不可
    if (_fromUserSetting) {
        return;
    }

#if 1
    //チャンネル定義リストを取得する
    QString polarization = "";

    if (_takeOver.contains("selectedPolarization")) {
        polarization = _takeOver["selectedPolarization"].toString();
    }
#endif

    // 保持したスキャン結果をクリアー
    _model->clearScanResult();

    // チャンネル情報を元に戻す
    _channelList.clear();

#if 1
    // チャンネル定義リストを取得する
    // マルチ測定では、activate時に _preservedChannelListを再構築しない場合があるため、チャンネル定義リストから取得するように修正
    _model->selectChannelDefinitionList(_bandId, polarization, _channelList);
#else
    for (int i = 0; i < _preservedChannelList.count(); ++i) {
        _channelList.append(_preservedChannelList[i]);
    }
#endif

    // 局発周波数
    uint32_t localFrequency = getLocalFrequency();

    // 変調方式
    unsigned int modulationValue = getModulationValue();
    QString modulation = Correcter::displayModulation(modulationValue);
    uint16_t bdSys = Correcter::getBroadcastSystemCondition(_bandId, modulation);

    // 放送方式、変調方式の設定(定義ファイル 1.0.1 対応)
    uint16_t mod = Correcter::getModulationCondition(_bandId, bdSys, _channelList.at(_currentChannelIndex).polarization, modulationValue, _broadBandModulationSetting);

#if 1
    // オートパワーオフ設定を無効化
    _model->disablePowerOffSetting();
#endif

    // 表示クリア
    clear();

    // スキャンアイコンなし
    _model->removeMenuList(MenuListIconIndex::SCAN_ICON);

    // スキャンのときは測定画面以外にする
    _model->changeDisplayType(LC70_DISP_TYPE_NOT_MEAS);

    // スキャン開始
    _isScanning = true;

#if 1
    //    _isScanning = true;
    if (_bandId == LC70_BAND_BS_CS) {
        _model->scan_bscs(_channelList, AcquisitionTarget::MULTI, bdSys, mod, true, _powerSupply);
    }
    else {
        _model->scan(_channelList, AcquisitionTarget::MULTI, localFrequency, bdSys, mod, true, _powerSupply);
    }
#else
    _model->scan(_channelList, AcquisitionTarget::MULTI, localFrequency, bdSys, mod, true, _powerSupply);
#endif

    // スキャン開始
    _model->writeLog(QString("マルチ測定"),
                     QString("スキャン開始 変調方式:%1 局発周波数:%2").arg(modulation).arg(localFrequency));
}

void MultiMeasurementViewModel::onRequestToStopScanning()
{
    // ユーザー設定は再スキャン不可
    if (_fromUserSetting) {
        return;
    }

    // スキャン中止
    _model->stopChannelScan();
    _isScanning = false;
    _model->writeLog(QString("マルチ測定"), QString("スキャン中止"));

    // スキャン結果を破棄
    _model->clearScanResult();

    // 全チャンネル再取得
    refreshAllChannels();

    // 遷移先画面種別を元に戻す
    switch (_selectedButtonIndex) {
    case GRAPH_BUTTON_INDEX:    // グラフ
        _model->changeDisplayType(LC70_DISP_TYPE_GRAPH);
        break;

    case LIST_BUTTON_INDEX:     // リスト
        _model->changeDisplayType(LC70_DISP_TYPE_LIST_MULTI);
        break;

    default:
        break;
    }

    // FM の場合、最初にスキャンしているとチャンネル表示していないので呼び出す
    if (_bandId == LC70_BAND_FM) {
        changeChannel();
    }
}

void MultiMeasurementViewModel::onScanningChannelResult(int index, LC70_MEAS_MULTI_RESULT_T result)
{
//    static int count = 0;

    // チャンネルがない場合はスキャン終了
    if (_channelList.count() == 0) {
        onScanningFinished(true);
        return;
    }


    if ((result.ucLock == UNLOCKED) || (result.ulCnr <= 500)) {

        // 無効判定
        _channelList[index].valid = false;
    }

    // SCAN時設定した変調方式がOFDMかつ、
    // スキャン結果がOFDM以外の変調方式の場合
    // スキャン結果として登録する変調方式をOFDMに設定する
    int registModulation = result.ulModBitPtrn;
    if(getModulationValue() == LC70_SETMOD_OFDM && result.ulModBitPtrn != LC70_SETMOD_OFDM) {
        registModulation = LC70_SETMOD_OFDM;
    }

    int lockFlag = false;
    // スキャン結果の保持(ロックされていて MER が 5.0 以上なら有効とする
    if (result.ucLock == LOCKED && result.ulCnr > 500) {
        MeasureResult scanResult;
        scanResult.Received = true;
        scanResult.Band = QString::number(_channelList[index].band);
        scanResult.Channel = _channelList[index].channelNumber;
        scanResult.Status = result.usStat;
        scanResult.Level = result.ulLevel;
        scanResult.Mer = 0;
        scanResult.Ber = 0;
        scanResult.BroadcastSystem = result.usBdSys;
        scanResult.Modulation = registModulation;
        scanResult.Layer = result.ucLayer;
        scanResult.Frequency = -1;
        scanResult.LocalOscillator = getLocalFrequency();
        _model->appendScanResult(scanResult);

        // スキャン時 有効チャンネルが見つかった場合 ブザーを1度鳴動させる
#if 1
        _model->buzzer(LC70_BUZZ_1CH_SCAN, LC70_BUZZ_MODE_ONESHOT);
#else
        _model->buzzer(LC70_BUZZ_VOL_CHG, LC70_BUZZ_MODE_ONESHOT);
#endif
        lockFlag = true;
    }

    emit progressText(_channelList[index].channelNumber, lockFlag);
    // プログレスバーの進捗を設定
    qDebug() << QString("total: %1 current: %2").arg(_channelList.count()).arg(index);
    double value = static_cast<double>(index + 1) / static_cast<double>(_channelList.count()) * 100;
    emit progress(value);

// 給電デバッグ用
//    ++count;
//    if (count == 1) {
//        dynamic_cast<MainFrameViewModel *>(_mainFrame)->test();
//    } else if (count == 4) {
//        dynamic_cast<MainFrameViewModel *>(_mainFrame)->test();
//        count = 0;
//    }
}

void MultiMeasurementViewModel::onScanningFinished(bool complete)
{
    _model->writeLog(QString("マルチ測定"), QString("スキャン終了"));

    _isScanning = false;
    QList<MeasureResult> result = _model->allScanResult();

    // 全て無効なら全チャンネルを再取得
    if (result.count() == 0) {
        refreshAllChannels();

        // 優先設定の測定条件を設定
        setPreferredMeasuringCondition();
    } else {
        // 有効チャンネルのみに更新する
        QList<ChannelDefinitionItem> channelList;

        for (int i = 0; i < _channelList.count(); ++i) {
            if (_channelList[i].valid) {
                channelList.append(_channelList[i]);
            }
        }

        _channelList.clear();

        for (int i = 0; i < channelList.count(); ++i) {
            _channelList.append(channelList[i]);
        }
    }

    // デバッグ用出力
    for (int i = 0; i < result.count(); ++i) {
        QString log =
                QString("有効チャンネル Channel:%1 Status:%2 Broadcast:%3 Modulation:%4 Local:%5 Level:%6 MER:%7 BER:%8").arg(
                    result[i].Channel).arg(result[i].Status).arg(result[i].BroadcastSystem).arg(result[i].Modulation).arg(
                    result[i].LocalOscillator).arg(result[i].Level).arg(result[i].Mer).arg(result[i].Ber);
        _model->writeLog(QString("マルチ測定"), log, LOG_INFO);
    }

    // スキャン後の優先測定チャンネルのインデックスの反映
    QString channelName;

    for (int i = 0; i < STR_CHAN_LEN; ++i) {
        channelName += static_cast<unsigned char>(_preferredCondition.ucStrChan[i]);
    }

    // 見つからなければ最初に戻す
    _currentChannelIndex = 0;

    for (int i = 0; i < _channelList.count(); ++i) {
        if (_channelList[i].channelNumber == channelName.trimmed()) {
            _currentChannelIndex = i;
        }
    }

    changeMultiGraphModel();
    changeMultiListModel();

    switch (_selectedButtonIndex) {
    case GRAPH_BUTTON_INDEX: {

        // 通常操作の選択内容を測定条件に反映
        _model->changeDisplayType(LC70_DISP_TYPE_GRAPH);
        setMultiMeasuringCondition(true);
        break;
    }

    case LIST_BUTTON_INDEX: {

        // リストの総ページ数設定
        if (_measureResultModel) {
            _totalPage = _measureResultModel->rowCount() / LIST_ROWS;
        }

        if (_measureResultModel) {
            if (_measureResultModel->rowCount() % LIST_ROWS > 0) {
                _totalPage += 1;
            }
        }
        _currentPage = 1;
        if (_view) {
            _view->setProperty("totalPage", _totalPage);        // 総ページ
            _view->setProperty("currentPage", _currentPage);    // 現在のページ
        }

        // 通常操作の選択内容を測定条件に反映
        _model->changeDisplayType(LC70_DISP_TYPE_LIST_MULTI);
        setMultiMeasuringCondition(false);
        break;
    }

    default:
        return;
    }

    if (complete) {
        // スキャン完了時 ブザーを1度鳴動させる
        _model->buzzer(LC70_BUZZ_ALL_CH_SCAN, LC70_BUZZ_MODE_ONESHOT);
        emit requestToShowPopup(POPUP_SCAN_RESULT);

        // スキャンアイコンを表示する
        _model->replaceMenuList(MenuListIconIndex::SCAN_ICON, "icon_scan_n.png");

        int previousChannelIndex = 0;
        selectGraphIndex(_currentChannelIndex, previousChannelIndex);

        // 有効なチャンネルがあればその先頭に移動
        // 有効なチャンネルの先頭に設定
        for (int i = 0; i < _channelList.count(); ++i) {
            if (_channelList[i].valid) {
                _currentChannelIndex = i;
                break;
            }
        }

        // スキャン後の有効なチャンネル一覧を保存
        _model->setScannedChannelList(_channelList, _takeOver["selectedPolarization"].toString());

        // 局初周波数を引き継ぐ
        emit requestSetTakeOver("selectedLocalOscillator", _takeOver["selectedLocalOscillator"]);

        int currentChannelIndex = 0;
        selectGraphIndex(_currentChannelIndex, currentChannelIndex);

        if (_multiGraphModel) {
            // チャンネル変更
            _multiGraphModel->changeSelection(previousChannelIndex, currentChannelIndex, true);
        }

        // ポップアップを閉じた後に測定再開
    } else {

#if 1
        // オートパワーオフ設定を有効化
        _model->enablePowerOffSetting();
#endif

        _model->removeMenuList(MenuListIconIndex::SCAN_ICON);

        // キャンセルした場合は、ここで測定を再開する
        // スキャン後処理を実施
        startMeasuringAfterScan();
    }
}

void MultiMeasurementViewModel::onSelect(PopupId popupId, int index)
{
    if (popupId == POPUP_SCAN_RESULT) {
        // スキャン結果を採用する
        selectReactionScanResult(index);
        return;
    }

    // 現在のチャンネルから偏波面の取得を行う
    QString polarize = Correcter::convertWaveToPolarization(_channelList.at(_currentChannelIndex).polarization);

    if (index == FIXED_15V) {       // 15v固定 給電
        _powerSupply = LC70_POWER_SUPPLY_15V;
        _model->setPowerSupply(_powerSupply);
        _model->replaceMenuList(MenuListIconIndex::POWER_SUPPLY_ICON, Correcter::powerSupplyIcon(_powerSupply));
        _powerSupplied = true;
    } else if (index == H15V11) {   // 水平15v/垂直11v 給電
        if (_bandId == LC70_BAND_JCSAT4) {
            // 偏波による分岐
            if (polarize == HORIZONTALLY_POLARIZE) {
                _powerSupply = LC70_POWER_SUPPLY_15V_P;
                _model->setPowerSupply(_powerSupply);
                _model->replaceMenuList(MenuListIconIndex::POWER_SUPPLY_ICON, Correcter::powerSupplyIcon(_powerSupply));
            } else if (polarize == VERTICALLY_POLARIZE) {
                _powerSupply = LC70_POWER_SUPPLY_11V_P;
                _model->setPowerSupply(_powerSupply);
                _model->replaceMenuList(MenuListIconIndex::POWER_SUPPLY_ICON, Correcter::powerSupplyIcon(_powerSupply));
            }
        } else if (_bandId == LC70_BAND_JCSAT3 ||
                   _bandId == LC70_BAND_SCCB ||
                   _bandId == LC70_BAND_SCCC) {

            // 偏波による分岐
            if (polarize == HORIZONTALLY_POLARIZE) {
                _powerSupply = LC70_POWER_SUPPLY_15V;
                _model->setPowerSupply(_powerSupply);
                _model->replaceMenuList(MenuListIconIndex::POWER_SUPPLY_ICON, Correcter::powerSupplyIcon(_powerSupply));
            } else if (polarize == VERTICALLY_POLARIZE) {
                _powerSupply = LC70_POWER_SUPPLY_11V;
                _model->setPowerSupply(_powerSupply);
                _model->replaceMenuList(MenuListIconIndex::POWER_SUPPLY_ICON, Correcter::powerSupplyIcon(_powerSupply));
            }
        } else {
            _powerSupply = LC70_POWER_SUPPLY_15V;
            _model->setPowerSupply(_powerSupply);
            _model->replaceMenuList(MenuListIconIndex::POWER_SUPPLY_ICON, Correcter::powerSupplyIcon(_powerSupply));
        }
        _powerSupplied = true;
    }
}

void MultiMeasurementViewModel::onNotifyButtonInput(PopupId popupId, ButtonId buttonId)
{
    Q_UNUSED(popupId);
    // ポップアップ表示中に入力されたボタンの処理
    onKeyStatus(buttonId);
}

void MultiMeasurementViewModel::moveCursor(ButtonId buttonId)
{
    // USER 設定で Band を跨る場合はグラフ、リストの選択不可(リストのみ表示)
    if (_hasMultiBandsInUserSetting) {
        return;
    }

    if ((buttonId == BUTTON_RIGHT) || (buttonId == BUTTON_LEFT)) {

        QString nextButton;
        bool ret = _buttonOperation->nextActiveObject(buttonId, nextButton);

        if (ret) {
            if (_view) {
                // inactive
                setButtonActive(_view, _activeButton, false);

                // active
                setButtonActive(_view, nextButton, true);
            }

            _activeButton = nextButton;
            _buttonOperation->setActiveObject(_activeButton);
        }
    }
}

void MultiMeasurementViewModel::changeChannel(bool isLaunched, bool noBandIconChange)
{
    if (_view == nullptr) {
        return;
    }

    if (_channelList.count() < 1) {
        return;
    }

    _model->writeLog(QString("マルチ測定"), QString("[Enter] %1").arg(__FUNCTION__));
    timer.start();

    // Band 設定
    if (!_fromUserSetting) {
        if (_view) {
            _view->setProperty("currentBandIndex", selectXAsixCategory(_bandId,
                                                                       Correcter::convertWaveToPolarization(_channelList[_currentChannelIndex].polarization)));
        }
    } else {
        if (_view) {
            _view->setProperty("currentBandIndex", selectXAsixCategory(static_cast<LC70_BAND_T>(_userSettings[_currentChannelIndex].ucBand),
                                                                       Correcter::convertWaveToPolarization(_channelList[_currentChannelIndex].polarization)));
        }
    }

    // 局発周波数、中心周波数の値から周波数の取得
    unsigned int localFrequency = getLocalFrequency();
    // 変調方式から対象の周波数を取得する
    unsigned int modulationValue = Correcter::modulationValue(Correcter::displayModulation(getModulationValue()));

    unsigned int centerFrequency = Correcter::centerFrequency(_channelList[_currentChannelIndex], modulationValue);
    unsigned int frequency = Correcter::frequency(centerFrequency, localFrequency);

    // 画面にチャンネル情報を反映する
    if (_view) {
        _view->setProperty("band", Correcter::displayBandName(_channelList[_currentChannelIndex].band));
        _view->setProperty("isSatelliteBroadCasting",
                           Correcter::isSatelliteBroadCasting(_channelList[_currentChannelIndex].band));
        _view->setProperty("channel", Correcter::displayChannelName(_channelList[_currentChannelIndex].channelNumber));
        _view->setProperty("frequency", Correcter::displayFrequency(_channelList[_currentChannelIndex].band, frequency));
        _view->setProperty("polarization", Correcter::displayPolarization(Correcter::convertWaveToPolarization(
                                                                              _channelList[_currentChannelIndex].polarization)));
        _view->setProperty("isModulationVisible", Correcter::isModulationVisible(_channelList[_currentChannelIndex].band));

        int valueMerCN = Correcter::selectCnrMer(static_cast<LC70_BAND_T>(_channelList[_currentChannelIndex].band), _cnrMerSetting);
        _view->setProperty("selectCnMer", valueMerCN);

        // 局発周波数の設定
        // kHz => GHzに値変換を実施
        _view->setProperty("localFrequency", Correcter::displayLocalFrequency(localFrequency));

        // 選択中のチャンネルのインデックス
        _view->setProperty("currentChannelIndex", _currentChannelIndex);

        // チャンネル詳細データの表示可否を通知
        _view->setProperty("displayChannelDetail", Correcter::getDisplayChannelDetail(_channelList[_currentChannelIndex].band));
    }

    // 上下ボタンによるチャンネル変更時はアイコンを変更しない
    if (!noBandIconChange) {
        // Band アイコンの差し替え
        _model->replaceBandIcon(static_cast<LC70_BAND_T>(_channelList[_currentChannelIndex].band));
    }

    // マルチ測定の測定条件
    if (_fromUserSetting) {
        // 優先測定測定条件
        setUserPreferredMeasuringCondition();
        if (_selectedButtonIndex == GRAPH_BUTTON_INDEX) {
            setUserMultiMeasuringCondition(true);
        } else if (_selectedButtonIndex == LIST_BUTTON_INDEX) {
            setUserMultiMeasuringCondition(false);
        }else {
            // nothing to do
        }
    } else {
        // 優先測定測定条件
        setPreferredMeasuringCondition();
        if (_selectedButtonIndex == GRAPH_BUTTON_INDEX) {
            setMultiMeasuringCondition(true);   // グラフ表示の設定
        } else if (_selectedButtonIndex == LIST_BUTTON_INDEX) {
            setMultiMeasuringCondition(false);  // リスト表示の設定
        } else {
            // nothing to do
        }
    }

    if (_view) {
        // 優先測定 LEVEL
        _view->setProperty("level", INITIAL_LEVEL);

        // 優先測定 MER
        _view->setProperty("mer", INITIAL_MER);

        // 優先測定 BER
        _view->setProperty("ber", INITIAL_BER);
    }
    if (_dataSave2State == DataSave2Retrieving && !_holdMeasurementData.isEmpty()) {
        // ログ収集結果表示中は、収集したデータを表示
        MeasureResult displayChannelData = _holdMeasurementData[_currentChannelIndex];

        if (_view) {
            LC70_BAND_T band = static_cast<LC70_BAND_T>(_channelList.at(_currentChannelIndex).band);
            QString polarize = Correcter::convertPolarizeNumToString(_channelList.at(_currentChannelIndex).polarization);
            BandTabIndex bandTab = Correcter::convertBandTabIndex(band, polarize);
            QString levelString;
            if (!Correcter::displayLevel(bandTab, displayChannelData.Level, levelString)) {
                qDebug() << QString("displayLevel() failed");
            }

            if (_view) {
                _view->setProperty("level", levelString);
            }
        }
        if (displayChannelData.Mer != UNDEFINED_VALUE) {
            if (_view) {
#if 1	//20210625_dx4
                if(displayChannelData.Mer == -3){
                    _view->setProperty("mer","<20.0");

                }else{
                 _view->setProperty("mer", Correcter::displayMer(_channelList.at(_currentChannelIndex).band, displayChannelData.BroadcastSystem, displayChannelData.Modulation, displayChannelData.Mer));
                }
#else
                _view->setProperty("mer", Correcter::displayMer(_channelList.at(_currentChannelIndex).band, displayChannelData.BroadcastSystem, displayChannelData.Modulation, displayChannelData.Mer));
#endif
            }
        }
        if (displayChannelData.Ber != UNDEFINED_VALUE) {
            if (_view) {
#if 1	//20210625_dx4
                if(displayChannelData.Ber  == UNLOCKED_VALUE){
                    _view->setProperty("ber", "2.2e-2");

                }else{
                    _view->setProperty("ber", Correcter::displayBer(displayChannelData.Ber));
                }
#else
                _view->setProperty("ber", Correcter::displayBer(displayChannelData.Ber));
#endif
            }
        }

        if (_channelList[_currentChannelIndex].band == LC70_BAND_FM) {
            if (_view) {
                // 測定BANDがFMの場合、変調方式FMの固定値を表示
                _view->setProperty("displayModulationPrefixMode", DISPLAY_MODULATION_PREFIX_TOGETHER);
                _view->setProperty("modulation", FM);
                _view->setProperty("modulationPrefix", QString(""));
            }
        } else if (_channelList[_currentChannelIndex].band == LC70_BAND_PILOT || _channelList[_currentChannelIndex].band == LC70_BAND_CATV_UP_STREAM) {
            if (_view) {
                // 測定BANDが"PILOT", "CATV上る"の場合、編著方式CWの固定値を表示
                _view->setProperty("displayModulationPrefixMode", DISPLAY_MODULATION_PREFIX_TOGETHER);
                _view->setProperty("modulation", CW);
                _view->setProperty("modulationPrefix", QString(""));
            }
        } else {
            // 優先測定 変調方式
            QString modulation = Correcter::displayModulation(displayChannelData.Modulation);
            // 変調方式の接頭句
            QString modulationPrefix = Correcter::modulationPrefix(displayChannelData.BroadcastSystem, modulation);
            // 変調方式と変調方式の接頭句をどのように表示するかを取得する
            int displayModulationPrefixMode = Correcter::displayModulationPrefixMode(displayChannelData.BroadcastSystem);

            if (displayChannelData.Locked == LOCKED) {
                if (_view) {
                    _view->setProperty("displayModulationPrefixMode", displayModulationPrefixMode);
                    _view->setProperty("modulation", modulation);
                    _view->setProperty("modulationPrefix", modulationPrefix);
                }
            } else {
                if (_view) {
                    _view->setProperty("displayModulationPrefixMode", DISPLAY_MODULATION_PREFIX_TOGETHER);
                    _view->setProperty("modulation", QString(INITIAL_MODULATION));
                    _view->setProperty("modulationPrefix", QString(""));
                }
            }
        }
    }

    // マルチ測定の該当チャンネルの測定結果から表示
    displayPrefferedChannel(_currentChannelIndex);

    _model->writeLog(QString("マルチ測定"), QString("[Leave] %1 %2 ms").arg(__FUNCTION__).arg(timer.elapsed()));

    if (_dataSave2State == DataSave2Retrieving) {
        // ログ収集中は優先測定を開始しない
        return;
    }

    // 画面起動時以外に通知(マルチ測定開始の後に優先測定を開始するため)
    _previousPreferredCondition = _preferredCondition;
    if (!isLaunched) {
        // 測定条件の変更を通知
        emit changeCondition(&_preferredCondition);
    }
}

void MultiMeasurementViewModel::setMultiMeasuringCondition(bool displayGraph)
{
    qDebug() << QString("[Enter]") << __FUNCTION__;
    memset(&_condition, 0x00, sizeof(LC70_MEAS_MULTI_T));

    // 登録件数
    _condition.sRegNum = _channelList.count();
    unsigned int modulationValue = getModulationValue();

    _measuringCondition = 0;

    for (int i = 0; i < _channelList.count(); ++i) {
        // 中心周波数
        unsigned int centerFrequency = Correcter::centerFrequency(_channelList[i], modulationValue);
        _condition.single[i].ulFreq = centerFrequency;

        // Band
        _condition.single[i].usBand = _channelList[i].band;

        // チャンネル番号文字列(NULL終端なし、8文字に足りない場合はスペースで埋める)
        memset(_condition.single[i].ucStrChan, ' ', CHANNEL_NUMBER_LENGTH);

        for (int j = 0; j < CHANNEL_NUMBER_LENGTH; ++j) {
            if (j <= _channelList[_currentChannelIndex].channelNumber.toLocal8Bit().length() - 1) {
                _condition.single[i].ucStrChan[j] = _channelList[i].channelNumber.toLocal8Bit()[j];
            }
        }

        int length = strnlen((char *)_condition.single[i].ucStrChan, sizeof _condition.single[i].ucStrChan);
        if (length < CHANNEL_NUMBER_LENGTH) {
            for (int j = length; j < CHANNEL_NUMBER_LENGTH; ++j) {
                _condition.single[i].ucStrChan[j] = ' ';
            }
        }

        // 局発周波数の連携
        unsigned int localFrequency = getLocalFrequency();
        if (_bandId == LC70_BAND_BS_CS) {
            if(_channelList[i].polarization == 0) {      // 右旋
                localFrequency = 10678000;
            }
            else if(_channelList[i].polarization == 1) {	// 左旋
                localFrequency = 9505000;
            }
        }

        _condition.single[i].ulLoFreq = localFrequency;

        // 放送方式、変調方式の設定(定義ファイル 1.0.1 対応)
        if (_bandId == LC70_BAND_BS_CS) {
            _condition.single[i].usMod = Correcter::getModulationCondition(_channelList[i].band, _condition.single[i].usBdSys, _channelList[i].polarization, modulationValue, _broadBandModulationSetting);
        }
        else {
            _condition.single[i].usMod = Correcter::getModulationCondition(_bandId, _condition.single[i].usBdSys, _channelList[i].polarization, modulationValue, _broadBandModulationSetting);
        }

        //BS-7, BS-17の場合、usBdSysを ISDB-S3に設定する
        if ((_bandId == LC70_BAND_BS) || (_bandId == LC70_BAND_CS) || (_bandId == LC70_BAND_BS_CS)) {
            if(_channelList[i].channelNumber == "BS-7") {
                _condition.single[i].usBdSys = LC70_BD_SYS_ISDBS3;
            }
            else if(_channelList[i].channelNumber == "BS-17") {
                _condition.single[i].usBdSys = LC70_BD_SYS_ISDBS3;
            }
        }
        // グラフとリストで異なる
        if (displayGraph) {
#if 1
            // BANDがLTE, PILOT, FMの場合, CATV上るの場合はLEVELのみを測定する
            if (_channelList[i].band == LC70_BAND_PILOT
                    || _channelList[i].band == LC70_BAND_LTE
                    || _channelList[i].band == LC70_BAND_FM
                    || _channelList[i].band == LC70_BAND_CATV_UP_STREAM) {
                _condition.single[i].usCondition = LC70_DATA_MSK_LEVEL;
            } else {
                // MER/CN の使い分け(CNR ビットは必ず立てる。MER のときは MER も立てる)
                if (Correcter::selectCnrMer(static_cast<LC70_BAND_T>(_channelList[_currentChannelIndex].band), _cnrMerSetting) == LC70_MEAS_SET_SEL_CNR) {
                    _condition.single[i].usCondition = LC70_DATA_MSK_LEVEL | LC70_DATA_MSK_CNR; // 測定条件の連携
                } else if (Correcter::selectCnrMer(static_cast<LC70_BAND_T>(_channelList[_currentChannelIndex].band), _cnrMerSetting) == LC70_MEAS_SET_SEL_MER) {
                    _condition.single[i].usCondition = LC70_DATA_MSK_LEVEL | LC70_DATA_MSK_CNR | LC70_DATA_MSK_MER; // 測定条件の連携
                } else {
                    // nothing to do
                }
            }
            _measuringCondition |= _condition.single[i].usCondition;
#else
            _condition.single[i].usCondition = LC70_DATA_MSK_LEVEL;  // 測定条件の連携
            _measuringCondition = LC70_DATA_MSK_LEVEL;
#endif
        } else {
            // BANDがLTE, PILOT, FMの場合, CATV上るの場合はLEVELのみを測定する
            if (_channelList[i].band == LC70_BAND_PILOT
                    || _channelList[i].band == LC70_BAND_LTE
                    || _channelList[i].band == LC70_BAND_FM
                    || _channelList[i].band == LC70_BAND_CATV_UP_STREAM) {
                _condition.single[i].usCondition = LC70_DATA_MSK_LEVEL;
            } else {
                // MER/CN の使い分け(CNR ビットは必ず立てる。MER のときは MER も立てる)
                if (Correcter::selectCnrMer(static_cast<LC70_BAND_T>(_channelList[_currentChannelIndex].band), _cnrMerSetting) == LC70_MEAS_SET_SEL_CNR) {
                    _condition.single[i].usCondition = LC70_DATA_MSK_LEVEL | LC70_DATA_MSK_CNR | LC70_DATA_MSK_BER; // 測定条件の連携
                } else if (Correcter::selectCnrMer(static_cast<LC70_BAND_T>(_channelList[_currentChannelIndex].band), _cnrMerSetting) == LC70_MEAS_SET_SEL_MER) {
                    _condition.single[i].usCondition = LC70_DATA_MSK_LEVEL | LC70_DATA_MSK_CNR | LC70_DATA_MSK_BER | LC70_DATA_MSK_MER; // 測定条件の連携
                } else {
                    // nothing to do
                }
            }
            _measuringCondition |= _condition.single[i].usCondition;
        }

        // 給電有効/無効 0:無効(マニュアル測定、スキャン、ユーザー) 1:有効(データロガーのみ)
        _condition.single[i].ucOutputEnb = 0;

        _model->writeLog(QString("マルチ測定"),
                         QString("channel %1 band:%2 frequency:%3 system:%4 modulation:%5 condition:%6").arg(
                             QString("%1").arg((char *)_condition.single[i].ucStrChan)).arg(_condition.single[i].usBand).arg(
                             _condition.single[i].ulFreq).arg(_condition.single[i].usBdSys).arg(_condition.single[i].usMod).arg(
                             _condition.single[i].usCondition), LOG_INFO);
    }

    qDebug() << QString("[Leave]") << __FUNCTION__;
}

void MultiMeasurementViewModel::setUserMultiMeasuringCondition(bool displayGraph)
{
    qDebug() << QString("[Enter]") << __FUNCTION__;
    memset(&_condition, 0x00, sizeof(LC70_MEAS_MULTI_T));

    // 登録件数
    _condition.sRegNum = _userSettings.count();

    for (int i = 0; i < _userSettings.count(); i++) {
        // 中心周波数
        _condition.single[i].ulFreq = _userSettings[i].ulCenterFreq;
        // Band
        _condition.single[i].usBand = _userSettings[i].ucBand;

        // チャンネル番号文字列(NULL終端なし、8文字に足りない場合はスペースで埋める)
        memset(_condition.single[i].ucStrChan, ' ', CHANNEL_NUMBER_LENGTH);

        for (unsigned int j = 0; j < CHANNEL_NUMBER_LENGTH; ++j) {
            if (j <= strlen(_userSettings[i].cStrChNo) - 1) {
                _condition.single[i].ucStrChan[j] = _userSettings[i].cStrChNo[j];
            }
        }

        int length = strnlen((char *)_condition.single[i].ucStrChan, sizeof _condition.single[i].ucStrChan);
        if (length < CHANNEL_NUMBER_LENGTH) {
            for (int j = length; j < CHANNEL_NUMBER_LENGTH; ++j) {
                _condition.single[i].ucStrChan[j] = ' ';
            }
        }

        // 局発周波数
        _condition.single[i].ulLoFreq = _userSettings[i].ulLocalFreq;

        // 放送方式
        _condition.single[i].usBdSys = _userSettings[i].ucSpec;
        //BS-7, BS-17の場合、usBdSysを ISDB-S3に設定する
        if ((_userSettings[i].ucBand == LC70_BAND_BS) || (_userSettings[i].ucBand == LC70_BAND_CS)) {
            //            QString test = QString::fromLocal8Bit(_userSettings[i].cStrChNo, LC70_CH_SET_CH_STRLEN).trimmed();
            if(QString::fromLocal8Bit(_userSettings[i].cStrChNo).trimmed() == "BS-7") {
                _condition.single[i].usBdSys = LC70_BD_SYS_ISDBS3;
            }
            else if(QString::fromLocal8Bit(_userSettings[i].cStrChNo).trimmed() == "BS-17") {
                _condition.single[i].usBdSys = LC70_BD_SYS_ISDBS3;
            }
        }

        // 変調方式
        _condition.single[i].usMod = _userSettings[i].ulModBit;

        // 測定対象
        // グラフとリストで異なる
        if (displayGraph) {
            _condition.single[i].usCondition = LC70_DATA_MSK_LEVEL;  // 測定条件の連携
            
#if 1
            // MER/CN の使い分け(CNR ビットは必ず立てる。MER のときは MER も立てる)
            if ((_userSettings[i].ucGetDataFlg & LC70_DATA_MSK_CNR) == LC70_DATA_MSK_CNR) {
                if (Correcter::selectCnrMer(static_cast<LC70_BAND_T>(_channelList[i].band), _cnrMerSetting) == LC70_MEAS_SET_SEL_CNR) {
                    _condition.single[i].usCondition |= LC70_DATA_MSK_CNR;
                } else if (Correcter::selectCnrMer(static_cast<LC70_BAND_T>(_channelList[i].band), _cnrMerSetting) == LC70_MEAS_SET_SEL_MER) {
                    _condition.single[i].usCondition |= (LC70_DATA_MSK_CNR | LC70_DATA_MSK_MER);
                }else {
                    // nothing to do
                }
            }
#endif
        } else {
            // LEVELは必ず立てる
            _condition.single[i].usCondition = LC70_DATA_MSK_LEVEL;
            // MER/CN の使い分け(CNR ビットは必ず立てる。MER のときは MER も立てる)
            if ((_userSettings[i].ucGetDataFlg & LC70_DATA_MSK_CNR) == LC70_DATA_MSK_CNR) {
                if (Correcter::selectCnrMer(static_cast<LC70_BAND_T>(_channelList[i].band), _cnrMerSetting) == LC70_MEAS_SET_SEL_CNR) {
                    _condition.single[i].usCondition |= LC70_DATA_MSK_CNR;
                } else if (Correcter::selectCnrMer(static_cast<LC70_BAND_T>(_channelList[i].band), _cnrMerSetting) == LC70_MEAS_SET_SEL_MER) {
                    _condition.single[i].usCondition |= (LC70_DATA_MSK_CNR | LC70_DATA_MSK_MER);
                }else {
                    // nothing to do
                }
            }
            if ((_userSettings[i].ucGetDataFlg & LC70_DATA_MSK_BER) == LC70_DATA_MSK_BER) {
                _condition.single[i].usCondition |= (LC70_DATA_MSK_BER);
            }
        }
        // 給電有効/無効 0:無効(マニュアル測定、スキャン、ユーザー) 1:有効(データロガーのみ)
        _condition.single[i].ucOutputEnb = 0;

        // 給電設定
        if (displayGraph) {
            _condition.single[i].powerSupply = static_cast<LC70_POWER_SUPPLY_T>(_userSettings[i].ucDcOut);
        } else {
            // リストの場合は各 Band に応じた給電電圧を設定しておく
            LC70_BAND_T band = static_cast<LC70_BAND_T>(_condition.single[i].usBand);
            if (band == LC70_BAND_JCSAT3 || band == LC70_BAND_SCCB || band == LC70_BAND_SCCC) {
                // 偏波による分岐
                QString polarize = Correcter::convertWaveToPolarization(_channelList[i].polarization);
                if (polarize == HORIZONTALLY_POLARIZE) {
                    _condition.single[i].powerSupply = LC70_POWER_SUPPLY_15V;
                } else if (polarize == VERTICALLY_POLARIZE) {
                    _condition.single[i].powerSupply = LC70_POWER_SUPPLY_11V;
                }
            } else if (band == LC70_BAND_JCSAT4) {
                // 偏波による分岐
                QString polarize = Correcter::convertWaveToPolarization(_channelList[i].polarization);
                if (polarize == HORIZONTALLY_POLARIZE) {
                    _condition.single[i].powerSupply = LC70_POWER_SUPPLY_15V_P;
                } else if (polarize == VERTICALLY_POLARIZE) {
                    _condition.single[i].powerSupply = LC70_POWER_SUPPLY_11V_P;
                }
            } else {
                // 15V
                _condition.single[i].powerSupply = LC70_POWER_SUPPLY_15V;
            }
        }
    }

    qDebug() << QString("[Leave]") << __FUNCTION__;
}

void MultiMeasurementViewModel::setPreferredMeasuringCondition()
{
    memset(&_preferredCondition, 0x00, sizeof(LC70_MEAS_SINGLE_T));

    // チャンネル情報設定

    // 中央周波数と局発周波数から周波数を取得
    unsigned int localFrequency = getLocalFrequency();

    unsigned int modulationValue = getModulationValue();

    unsigned int centerFrequency = Correcter::centerFrequency(_channelList[_currentChannelIndex], modulationValue);
    // 中心周波数
    _preferredCondition.ulFreq = centerFrequency;

    // 実放送周波数
    _preferredCondition.usBand = _channelList[_currentChannelIndex].band; // BAND

    // チャンネル番号文字列(NULL終端なし、8文字に足りない場合はスペースで埋める)
    memset(_preferredCondition.ucStrChan, ' ', CHANNEL_NUMBER_LENGTH);

    for (int i = 0; i < CHANNEL_NUMBER_LENGTH; ++i) {
        if (i <= _channelList[_currentChannelIndex].channelNumber.toLocal8Bit().length() - 1) {
            _preferredCondition.ucStrChan[i] = _channelList[_currentChannelIndex].channelNumber.toLocal8Bit()[i];
        }
    }

    int length = strnlen((char *)_preferredCondition.ucStrChan, sizeof _preferredCondition.ucStrChan);
    if (length < CHANNEL_NUMBER_LENGTH) {
        for (int j = length; j < CHANNEL_NUMBER_LENGTH; ++j) {
            _preferredCondition.ucStrChan[j] = ' ';
        }
    }

    // 局発周波数との連携
    _preferredCondition.ulLoFreq = localFrequency;

    // 放送方式、変調方式の設定(定義ファイル 1.0.1 対応)
    if (_bandId == LC70_BAND_BS_CS) {
        _preferredCondition.usMod = Correcter::getModulationCondition(_channelList[_currentChannelIndex].band, _preferredCondition.usBdSys, _channelList[_currentChannelIndex].polarization, modulationValue, _broadBandModulationSetting);
    }
    else {
        _preferredCondition.usMod = Correcter::getModulationCondition(_bandId, _preferredCondition.usBdSys, _channelList[_currentChannelIndex].polarization, modulationValue, _broadBandModulationSetting);
    }

    // 測定条件の連携
    // MER/CN の使い分け(CNR ビットは必ず立てる。MER のときは MER も立てる)
    if (_preferredCondition.usBand == LC70_BAND_PILOT
            || _preferredCondition.usBand == LC70_BAND_LTE
            || _preferredCondition.usBand == LC70_BAND_FM
            || _preferredCondition.usBand == LC70_BAND_CATV_UP_STREAM) {
        _preferredCondition.usCondition = LC70_DATA_MSK_LEVEL;
    } else {
        if (Correcter::selectCnrMer(static_cast<LC70_BAND_T>(_channelList[_currentChannelIndex].band), _cnrMerSetting) == LC70_MEAS_SET_SEL_CNR) {
            _preferredCondition.usCondition = LC70_DATA_MSK_LEVEL | LC70_DATA_MSK_CNR | LC70_DATA_MSK_BER;
        } else if (Correcter::selectCnrMer(static_cast<LC70_BAND_T>(_channelList[_currentChannelIndex].band), _cnrMerSetting) == LC70_MEAS_SET_SEL_MER) {
            _preferredCondition.usCondition = LC70_DATA_MSK_LEVEL | LC70_DATA_MSK_CNR | LC70_DATA_MSK_BER | LC70_DATA_MSK_MER;
        }
    }

    // 給電有効/無効 0:無効(マニュアル測定、スキャン、ユーザー) 1:有効(データロガーのみ)
    _preferredCondition.ucOutputEnb = 0;

    // チャンネル変更中の操作を禁止
    if (_isChannelChanging) {
        return;
    }

    _model->writeLog(QString("マルチ測定"),
                     QString("優先測定チャンネル変更 band:%1 frequency:%2 system:%3 modulation:%4 condition:%5").arg(
                         _preferredCondition.usBand).arg(
                         _preferredCondition.ulFreq).arg(_preferredCondition.usBdSys).arg(_preferredCondition.usMod).arg(
                         _preferredCondition.usCondition));
}

void MultiMeasurementViewModel::setUserPreferredMeasuringCondition()
{

    memset(&_preferredCondition, 0x00, sizeof(LC70_MEAS_SINGLE_T));

    // 中央周波数と局発周波数から周波数を取得
    unsigned int localFrequency = getLocalFrequency();

    unsigned int modulationValue = getModulationValue();
    unsigned int centerFrequency = Correcter::centerFrequency(_channelList[_currentChannelIndex], modulationValue);

    // 中心周波数
#if 1
    _preferredCondition.ulFreq = centerFrequency;
#else
    _preferredCondition.ulFreq = centerFrequency - localFrequency;
#endif
    // 実放送周波数
    _preferredCondition.usBand = _channelList[_currentChannelIndex].band; // BAND

    // チャンネル番号文字列(NULL終端なし、8文字に足りない場合はスペースで埋める)
    memset(_preferredCondition.ucStrChan, ' ', CHANNEL_NUMBER_LENGTH);

    for (int i = 0; i < CHANNEL_NUMBER_LENGTH; ++i) {
        if (i <= _channelList[_currentChannelIndex].channelNumber.toLocal8Bit().length() - 1) {
            _preferredCondition.ucStrChan[i] = _channelList[_currentChannelIndex].channelNumber.toLocal8Bit()[i];
        }
    }

    int length = strnlen((char *)_preferredCondition.ucStrChan, sizeof _preferredCondition.ucStrChan);
    if (length < CHANNEL_NUMBER_LENGTH) {
        for (int j = length; j < CHANNEL_NUMBER_LENGTH; ++j) {
            _preferredCondition.ucStrChan[j] = ' ';
        }
    }

    // 局発周波数との連携
    _preferredCondition.ulLoFreq = localFrequency;

    _preferredCondition.usBdSys = _userSettings[_currentChannelIndex].ucSpec;
    _preferredCondition.usMod = _userSettings[_currentChannelIndex].ulModBit;

    // 測定条件の連携
    // MER/CN の使い分け(CNR ビットは必ず立てる。MER のときは MER も立てる)
    // LEVELは必ず立てる
    if(_preferredCondition.usBand == LC70_BAND_PILOT
            || _preferredCondition.usBand == LC70_BAND_LTE
            || _preferredCondition.usBand == LC70_BAND_FM
            || _preferredCondition.usBand == LC70_BAND_CATV_UP_STREAM) {
        _preferredCondition.usCondition = LC70_DATA_MSK_LEVEL;
    }else {
        _preferredCondition.usCondition = LC70_DATA_MSK_LEVEL;
        // MER/CN の使い分け(CNR ビットは必ず立てる。MER のときは MER も立てる)
        if ((_userSettings[_currentChannelIndex].ucGetDataFlg & LC70_DATA_MSK_CNR) == LC70_DATA_MSK_CNR) {
            if (Correcter::selectCnrMer(static_cast<LC70_BAND_T>(_channelList[_currentChannelIndex].band), _cnrMerSetting) == LC70_MEAS_SET_SEL_CNR) {
                _preferredCondition.usCondition |= LC70_DATA_MSK_CNR;
            } else if (Correcter::selectCnrMer(static_cast<LC70_BAND_T>(_channelList[_currentChannelIndex].band), _cnrMerSetting) == LC70_MEAS_SET_SEL_MER) {
                _preferredCondition.usCondition |= (LC70_DATA_MSK_CNR | LC70_DATA_MSK_MER);
            }else {
                // nothing to do
            }
        }
        if ((_userSettings[_currentChannelIndex].ucGetDataFlg & LC70_DATA_MSK_BER) == LC70_DATA_MSK_BER) {
            _preferredCondition.usCondition |= (LC70_DATA_MSK_BER);
        }
    }

    // 給電有効/無効 0:無効(マニュアル測定、スキャン、ユーザー) 1:有効(データロガーのみ)
    _preferredCondition.ucOutputEnb = 0;

    // 給電設定
    _preferredCondition.powerSupply = static_cast<LC70_POWER_SUPPLY_T>(_userSettings[_currentChannelIndex].ucDcOut);

    // チャンネル変更中の操作を禁止
    if (_isChannelChanging) {
        return;
    }

    _model->writeLog(QString("マルチ測定"),
                     QString("優先測定チャンネル変更 band:%1 frequency:%2 system:%3 modulation:%4 condition:%5").arg(
                         _preferredCondition.usBand).arg(
                         _preferredCondition.ulFreq).arg(_preferredCondition.usBdSys).arg(_preferredCondition.usMod).arg(
                         _preferredCondition.usCondition));
}

void MultiMeasurementViewModel::clear()
{
    // マルチチャンネル測定結果のクリアー
    for (int i = 0; i < _channelList.count(); ++i) {
        // リスト表示内容の更新
        if (_measureResultModel) {
            _measureResultModel->update(i, -1, -1, -1,0,0,10);
        }
    }

    if (_multiGraphModel) {
        // グラフを更新
        _multiGraphModel->clear();
    }

    // リストを先頭に移動
    _currentPage = 1;
    if (_view) {
        _view->setProperty("currentPage", _currentPage);    // 現在のページ
        emit scrollPage(_currentPage);
    }
    if (_view) {
        // 測定中のチャンネルのインデックスを 0 にする
        _view->setProperty("scanChannelIndex", 0);
    }
    if (_view) {
        // 測定中のチャンネルのインデックスを 0 にする
        _view->setProperty("currentChannelIndex", 0);
    }
    double value = 0.0f;
    if (_view) {
        // 共通表示部分のクリアー
        // 優先測定測定 入力レベル
        _view->setProperty("level", INITIAL_LEVEL);
    }
    if (_view) {
        // 優先測定 MER
        _view->setProperty("mer", INITIAL_MER);
    }
    if (_view) {
        // 優先測定 BER
        _view->setProperty("ber", INITIAL_BER);
    }
    if (_view) {
        // 優先測定 周波数
        _view->setProperty("frequency", QString(""));
    }
    if (_view) {
        // 優先測定 変調方式
        _view->setProperty("displayModulationPrefixMode", DISPLAY_MODULATION_PREFIX_TOGETHER);
    }
    if (_view) {
        _view->setProperty("modulation", QString(INITIAL_MODULATION));
    }
    if (_view) {
        _view->setProperty("modulationPrefix", QString(""));
    }

    if (_view) {
        // 優先測定 階層
        _view->setProperty("hierarchy", QString(""));
    }
    if (_view) {
        // 優先測定 局発周波数
        _view->setProperty("localFrequency", QString("%1").arg(value, 3, 'f', 3).rightJustified(6, '0'));
    }
}

void MultiMeasurementViewModel::changeByMenuBar()
{
    _model->writeLog(QString("マルチ測定"), QString("[ENTER] %1").arg(__FUNCTION__));

#if 1
    _bkMenuBarType = _menuBarType;
#else
    int barType = _menuBarType;
#endif
    int barIndex = _menuBarIndexMap[0];

    // メニューバーを閉じる
    closeAllMenuBar();

    // Menuアイコンの更新を実施
    changeMenuIcon();
#if 1
    if (_bkMenuBarType == 1) {
#else
    if (barType == 1) {
#endif
        _model->replaceMenuList(MenuListIconIndex::FUNCTION_ICON, "icon_menu_user.png");
    }

    // アンテナ調整を選択している場合
    if (_takeOver["selectedBand"].toInt() == LC70_BAND_JCSAT3 * LC70_BAND_JCSAT4) {

        // 測定設定にアンテナ調整を追加
        _takeOver["selectedMeasurement"] = ANTENNA_MEASURE_TYPE_NAME;

        // ユーザ設定からの遷移判定を初期値に戻す
        _takeOver["fromUserSetting"] = false;
        _fromUserSetting = false;

        // スレッドを停止(遷移先画面種別測定画面以外に変更)
        emit requestToSuspendMultiMeasuring();

        AbstractViewModel::suspend();

        // 一時停止
        _alreadySuspended = true;

        // 画面共有データの送信
        // [Memo] 測定情報、BAND情報は必ずデータが設定されている前提
        //        変調方式、偏波面、局発周波数は不要な場合は、未設定
        emit requestSetTakeOver("selectedMeasurement", _takeOver["selectedMeasurement"]);
        emit requestSetTakeOver("selectedBand", _takeOver["selectedBand"]);
        emit requestSetTakeOver("fromUserSetting", false);

        // 変調方式、偏波面、局発周波数はnullではない場合に送信
        if (!_takeOver["selectedModulation"].isNull()) {
            emit requestSetTakeOver("selectedModulation", _takeOver["selectedModulation"]);
        }

        if (!_takeOver["selectedPolarization"].isNull()) {
            emit requestSetTakeOver("selectedPolarization", _takeOver["selectedPolarization"]);
        }

        if (!_takeOver["selectedLocalOscillator"].isNull()) {
            emit requestSetTakeOver("selectedLocalOscillator", _takeOver["selectedLocalOscillator"]);
        }

        // 画面遷移
        QList<ScreenId> history = QList<ScreenId>() << SCREEN_HOME << SCREEN_BAND;
        emit requestWithChangeHistory(SCREEN_ANTENNA_ADJUSTMENT, history);
        return;
    }

    // シングル測定を選択している場合
    if (_takeOver["selectedMeasurement"].toString() == QString(SINGLE_MEASURE_TYPE_NAME)) {

#if 1
        if (_bkMenuBarType == 0) {
#else
        if (barType == 0) {
#endif
            // 測定設定にアンテナ調整を追加
            _takeOver["selectedMeasurement"] = ANTENNA_MEASURE_TYPE_NAME;

            // ユーザ設定からの遷移判定を初期値に戻す
            _takeOver["formUserSetting"] = false;
            _fromUserSetting = false;

            // スレッドを停止(遷移先画面種別測定画面以外に変更)
            emit requestToSuspendMultiMeasuring();

            AbstractViewModel::suspend();

            // 一時停止済み
            _alreadySuspended = true;

            // 画面共有データの送信
            // [Memo] 測定情報、BAND情報は必ずデータが設定されている前提
            //        変調方式、偏波面、局発周波数は不要な場合は、未設定
            emit requestSetTakeOver("selectedMeasurement", _takeOver["selectedMeasurement"]);
            emit requestSetTakeOver("selectedBand", _takeOver["selectedBand"]);
            emit requestSetTakeOver("fromUserSetting", false);

            // 変調方式、偏波面、局発周波数はnullではない場合に送信
            if (!_takeOver["selectedModulation"].isNull()) {
                emit requestSetTakeOver("selectedModulation", _takeOver["selectedModulation"]);
            }

            if (!_takeOver["selectedPolarization"].isNull()) {
                emit requestSetTakeOver("selectedPolarization", _takeOver["selectedPolarization"]);
            }

            if (!_takeOver["selectedLocalOscillator"].isNull()) {
                emit requestSetTakeOver("selectedLocalOscillator", _takeOver["selectedLocalOscillator"]);
            }
#if 1
        } else if (_bkMenuBarType == 1) {
#else
        } else if (barType == 1) {
#endif
            // ユーザ設定からの遷移判定を初期値に戻す
            _takeOver["fromUserSetting"] = false;
            _fromUserSetting = false;

            // スレッドを停止(遷移先画面種別測定画面以外に変更)
            emit requestToSuspendMultiMeasuring();

            AbstractViewModel::suspend();

            // 一時停止
            _alreadySuspended = true;
            MainFrameViewModel *mainFrame = static_cast<MainFrameViewModel *>(_mainFrame);
            QVariantList vFileList = mainFrame->getUserFileList();
            QVariant vFileName = vFileList[barIndex];
            QString sFileName = vFileName.toString();
            _model->importDataLoggerUserSettingList(VALUE_USERSETTING_SETTINGLIST, sFileName);
            emit requestSetTakeOver("fromUserSetting", true);
            emit requestSetTakeOver("selectedUserListFile", sFileName);
        }

        // 画面遷移
        QList<ScreenId> history = QList<ScreenId>() << SCREEN_HOME << SCREEN_BAND;
        emit requestWithChangeHistory(SCREEN_SINGLE_MEASUREMENT, history);
        return;
    }

    // ユーザ設定からの遷移判定がTrueの場合
#if 1
    if (_fromUserSetting || _bkMenuBarType == 1) {
#else
    if (_fromUserSetting || barType == 1) {
#endif
        // ユーザ設定からの遷移判定を初期値に戻す
        _takeOver["fromUserSetting"] = false;
        emit changeAttribute(QString("fromUserSetting"), false);
        // ユーザー設定名をクリア
        _takeOver["selectedUserListFile"] = "";
        emit removeAttribute("selectedUserListFile");
        if (_view) {
            _view->setProperty("userMode", "");
        }

        _fromUserSetting = false;
        _hasMultiBandsInUserSetting = false;

        // [Memo]
        // 画面の遷移履歴のみが変化する特別な画面遷移処理となるため、 requestWithChangeHistoryを行うが画面遷移は実施しない。

        // 画面遷移を実施し、遷移履歴を更新する
        QList<ScreenId> history = QList<ScreenId>() << SCREEN_HOME << SCREEN_BAND;
        emit requestWithChangeHistory(SCREEN_MULTI_MEASUREMENT, history, false);

        // Menuボタン押下によるユーザ設定の場合
#if 1
        if (_bkMenuBarType == 1) {
#else
        if (barType == 1) {
#endif
            MainFrameViewModel *mainFrame = static_cast<MainFrameViewModel *>(_mainFrame);
            QVariantList vFileList = mainFrame->getUserFileList();
            QVariant vFileName = vFileList[barIndex];
            QString sFileName = vFileName.toString();
            _model->importDataLoggerUserSettingList(VALUE_USERSETTING_SETTINGLIST, sFileName);
            _takeOver["fromUserSetting"] = true;
            _takeOver["selectedUserListFile"] = sFileName;
        }
    }

    // 画面を初期状態に戻す
    resetScreen();

    _model->writeLog(QString("マルチ測定"), QString("[LEAVE] %1").arg(__FUNCTION__));
}

void MultiMeasurementViewModel::resetScreen()
{
    // 給電を停止
    _powerSupply = LC70_POWER_SUPPLY_OFF;
    _model->setPowerSupply(_powerSupply);
    _model->removeMenuList(MenuListIconIndex::POWER_SUPPLY_ICON);

    // 画面遷移履歴のクリアー
    _changeHistory.clear();

    // 優先測定条件のクリアー
    memset(&_preferredCondition, 0x00, sizeof(LC70_MEAS_SINGLE_T));
    memset(&_previousPreferredCondition, 0x00, sizeof(LC70_MEAS_SINGLE_T));

    // マルチ測定条件のクリアー
    for (int i = 0; i < _channelList.count(); ++i) {
        _condition.sRegNum = 0;
        memset(&_condition.single[i], 0x00, sizeof(_condition.single[i]));
    }

    // チャンネル情報のクリアー
    _channelList.clear();

    // 局初周波数のリストをクリアー
    _localFrequencyList.clear();

    // 保持している測定データをクリア
    _holdMeasurementData.clear();

    // ユーザー設定のクリアー
    _userSettings.clear();

    // ユーザー設定からの遷移判定を初期値に戻す
    _fromUserSetting = false;

    // 給電をヘッダーから取り除く
    _model->removeMenuList(MenuListIconIndex::POWER_SUPPLY_ICON);

    // スキャンをヘッダーから取り除く
    _model->removeMenuList(MenuListIconIndex::SCAN_ICON);

    // ユーザー設定名をクリアー
    if (_view) {
        _view->setProperty("userMode", QString());
    }

    // 現在グラフを表示中でなければ、グラフに戻す必要がある
    int beforeMode = _selectedButtonIndex;

    // 初期値に戻す
    activate();

#if 1
    if ((beforeMode != GRAPH_BUTTON_INDEX) && (_bkMenuBarType != 1)) {
#else
    if (beforeMode != GRAPH_BUTTON_INDEX) {
#endif
        // グラフ以外を表示しているので、表示形式をグラフに変更する
        _selectedButtonIndex = beforeMode;
        onRequest(GRAPH_BUTTON_INDEX);
    } else {
        // 現在グラフを表示しているので、測定を停止し、条件を再設定してから再開する
        // スレッドを停止(遷移先画面種別測定画面以外に変更)
        emit requestToSuspendMultiMeasuring();

        // チャンネル変更(測定開始)
        changeChannel(true);

        // スレッドを開始
        // USER 設定のマルチ測定であれば遷移先画面種別はデータロガーリストにする
        if (_fromUserSetting && _hasMultiBandsInUserSetting) {
            _model->changeDisplayType(LC70_DISP_TYPE_LIST_MULTI);
            emit resumeMultiMeasuring(&_condition, &_preferredCondition, LC70_DISP_TYPE_LIST_LOGGER,
                                      _model->applicationSetting()->pollingIntervalMultiMeasurement());
        } else {
            _model->changeDisplayType(LC70_DISP_TYPE_GRAPH);
            emit resumeMultiMeasuring(&_condition, &_preferredCondition, LC70_DISP_TYPE_GRAPH,
                                      _model->applicationSetting()->pollingIntervalMultiMeasurement());
        }
    }
}

void MultiMeasurementViewModel::recover()
{
    if (_view) {
        _fixedGraph = _view->findChild<QObject *>(QString("fixedGraph"));
        _list = _view->findChild<QObject *>(QString("list"));
        _multiGraphModel = dynamic_cast<MultiGraphModel *>(_view->findChild<QObject *>(QString("multiGraphModel")));
        _measureResultModel = dynamic_cast<MeasureResultModel *>(_view->findChild<QObject *>(QString("measureResultModel")));
    }
}

uint32_t MultiMeasurementViewModel::selectXAsixCategory(int bandId, QString polarization)
{
    switch (bandId) {
    case LC70_BAND_UHF:
        return BAND_CATEGORY_UHF;

    case LC70_BAND_BS: {
        if (polarization == QString(RIGHT_HANDED_POLARIZE)) {
            return BAND_CATEGORY_BS_RIGHT;
        } else {
            return BAND_CATEGORY_BS_LEFT;
        }
    }

    case LC70_BAND_CS: {
        if (polarization == QString(RIGHT_HANDED_POLARIZE)) {
            return BAND_CATEGORY_CS_RIGHT;
        } else {
            return BAND_CATEGORY_CS_LEFT;
        }
    }

    case LC70_BAND_SCCB: {
        if (polarization == QString(HORIZONTALLY_POLARIZE)) {
            return BAND_CATEGORY_SCCB_HORIZONTAL;
        } else {
            return BAND_CATEGORY_SCCB_VERTICAL;
        }
    }

    case LC70_BAND_SCCC: {
        if (polarization == QString(HORIZONTALLY_POLARIZE)) {
            return BAND_CATEGORY_SCCC_HORIZONTAL;
        } else {
            return BAND_CATEGORY_SCCC_VERTICAL;
        }
    }

    case LC70_BAND_JCSAT3: {
        if (polarization == QString(VERTICALLY_POLARIZE)) {
            return BAND_CATEGORY_JCSAT3A_VERTICAL;
        } else {
            return BAND_CATEGORY_JCSAT3A_HORIZONTAL;
        }
    }

    case LC70_BAND_JCSAT4: {
        if (polarization == QString(VERTICALLY_POLARIZE)) {
            return BAND_CATEGORY_JCSAT4B_VERTICAL;
        } else {
            return BAND_CATEGORY_JCSAT4B_VERTICAL;
        }
    }

    case LC70_BAND_CATV_ALL:  {
        return BAND_CATEGORY_CATV_ALL;
    }

    case LC70_BAND_CATV_90_470MHZ: {
        return BAND_CATEGORY_CATV_90_470;
    }

    case LC70_BAND_CATV_470_1002MHZ: {
        return BAND_CATEGORY_CATV_470_1002;
    }

    case LC70_BAND_PILOT: {
        return BAND_CATEGORY_PILOT;
    }

    case LC70_BAND_SKYP: {
        return BAND_CATEGORY_SKY_PERFETV;
    }

    case LC70_BAND_LTE: {
        return BAND_CATEGORY_LTE;
    }

    case LC70_BAND_FM: {
        return BAND_CATEGORY_FM;
    }

    case LC70_BAND_CATV_UP_STREAM: {
        return BAND_CATEGORY_CATV_UPSTREAM;
    }

    case LC70_BAND_BS_CS:
        return BAND_CATEGORY_BS_CS;
        //          return 21;
    default:
        break;
    }

    return bandId;
}

void MultiMeasurementViewModel::startConfirmedChangeChannelTimer()
{
    // タイマーを開始する
    // 既にタイマーが開始している場合は、再度タイマーを開始する
    _confirmeChangeChannelTimer->stop();
    _confirmeChangeChannelTimer->start(_model->applicationSetting()->channelChangeDelay());
}

void MultiMeasurementViewModel::onConfirmedChangeChannelTimeout()
{
    _previousPreferredCondition = _preferredCondition;
    emit changeCondition(&_preferredCondition);

}

void MultiMeasurementViewModel::setBoolProperty(QString objectName, QString propertyName, bool value)
{
    if (_view) {
        QObject *target = _view->findChild<QObject *>(objectName);
        if (target) {
            // 押下前の画像に更新
            target->setProperty(propertyName.toUtf8().constData(), value);
        }
    }
}

void MultiMeasurementViewModel::changeMultiListModel()
{
    unsigned int modulationValue = getModulationValue();
    //    unsigned int localFrequency = getLocalFrequency();

    if (_measureResultModel) {
        _measureResultModel->clear();
    }
    if (_channelList.count() == 0) {
        return;
    }

    for (int i = 0; i < _channelList.count(); ++i) {
        if (_channelList[i].band != LC70_BAND_FM && _channelList[i].band != LC70_BAND_CATV_UP_STREAM) {
            if (!_channelList[i].valid) {
                continue;
            }
        }

        unsigned int localFrequency = getLocalFrequency();
        if (_bandId == LC70_BAND_BS_CS) {
            if(_channelList[i].polarization == 0) {      // 右旋
                localFrequency = 10678000;
            }
            else if(_channelList[i].polarization == 1) {	// 左旋
                localFrequency = 9505000;
            }
        }

        // 中心周波数、局発周波数から周波数の値を取得
        unsigned int centerFrequency = Correcter::centerFrequency(_channelList[i], modulationValue);
        // USER 設定の CATV ALL  の場合は　USER 設定で選択された放送方式から中心周波数を特定する
        if (_fromUserSetting) {
            if (_channelList[i].band == LC70_BAND_CATV_ALL) {
                if (_userSettings.count() == _channelList.count()) {
                    centerFrequency = _userSettings[i].ulCenterFreq;
                }
            }
            localFrequency = _userSettings[i].ulLocalFreq;
        }
        unsigned int frequency = Correcter::frequency(centerFrequency, localFrequency);

        // リスト表示用
        MeasureResultRow result;
        result.Band = QString::number(_channelList[i].band);
        result.BandName = Correcter::displayBandName(_channelList[i].band);

        result.Channel = _channelList[i].channelNumber;

        // LEVEL, MER, BER の初期表示は "---"
        result.Level = QString("---");
        result.Mer = QString("---");
        result.Ber = QString("---");

        // 周波数
        result.Frequency = Correcter::displayFrequency(_channelList[i].band, frequency);

        // 偏波
        result.Polarization = Correcter::convertPolarizeNumToString(_channelList[i].polarization);

        if (_measureResultModel) {
            _measureResultModel->append(result);
        }

        // CSV 出力用
        MeasureResult measureResult;
        measureResult.Band = QString::number(_channelList[i].band);
        measureResult.Channel = _channelList[i].channelNumber;
        measureResult.LocalOscillator = localFrequency;
        measureResult.Frequency = frequency;

        // まだ測定ができていないチャンネルに対しては、BER,MERを 「---」と表示するため不定値を設定する
        measureResult.Ber = UNDEFINED_VALUE;
        measureResult.Mer = UNDEFINED_VALUE;
        measureResult.Level = UNDEFINED_VALUE;

        if (Correcter::isSatelliteBroadCasting(_channelList[i].band)) {
            // BANDが衛星放送の場合 偏波面を予め設定する
            measureResult.Layer = _channelList[i].polarization;
        }

        _multiMeasureResult.append(measureResult);
    }

    int surplus = _channelList.count() % LIST_ROWS;
    surplus = LIST_ROWS - surplus;      // リスト 1 ページの行数に満たない場合に追加する行

    // 測定している行を先頭に表示するためダミー行は 5 行になる
    surplus = LIST_ROWS - 1;

    // 1 ページを満たすためのダミー行の追加(CSV にはダミー行は不要）
    for (int i = 0; i < surplus; ++i) {
        MeasureResultRow dummy;
        // LEVEL, MER, BER の初期表示は "---"
        dummy.Channel = QString("---");
        dummy.Level = QString("---");
        dummy.Mer = QString::number(-1);
        dummy.Ber = QString::number(-1);
        dummy.Frequency = QString("---");

        if (_measureResultModel) {
            _measureResultModel->append(dummy);
        }
    }
}

void MultiMeasurementViewModel::changeMultiGraphModel()
{
    // USER 設定で複数の Band を跨る場合はグラフを使用しない
    if (_hasMultiBandsInUserSetting) {
        return;
    }

    if (_multiGraphModel) {
        // グラフ表示用
        _multiGraphModel->clear();
    }

    // グラフの背景画像を設定
    setGraphBackground(_bandId, _takeOver["selectedPolarization"].toString());

    if (_channelList.count() == 0) {
        return;
    }

    for (int i = 0; i < _channelListForGraph.count(); ++i) {
        BarGraphData barGraphData;
        barGraphData.Level = 0.0f;
        barGraphData.ColorIndex = 0;

        if (i == 0) {
            barGraphData.CurrentMeasuring = true;
            barGraphData.Selection = true;
        } else {
            barGraphData.CurrentMeasuring = false;
            barGraphData.Selection = false;
        }

        if (_multiGraphModel) {
            _multiGraphModel->append(barGraphData);
        }
    }

    if (_multiGraphModel) {
        // 優先測定のチャンネル
        _multiGraphModel->changeSelection(0, _currentChannelIndex, true);
    }
}

QList<QString> MultiMeasurementViewModel::displayChannelList(int graphMode, QList<QString> allChannelList)
{
    QList<QString> displayChannelList = {};
    // リストに表示するチャンネルのインデックスのマップ
    QMap<int, QList<int>> displayChannelIndexMap = {
        {GRAPH_TYPE_1,  {0, 2,  4,  6, 9, 11, 13}},
        {GRAPH_TYPE_2,  {0, 9, 19, 29, 39, 49, 59}},
        {GRAPH_TYPE_3,  {0, 19, 39, 59, 79, 99, 119}},
        {GRAPH_TYPE_4,  {0, 19, 39, 59, 79, 99, 119, 139, 159}}
    };

    if (!displayChannelIndexMap.contains(graphMode)) {
        // 対応するモードが定義されていなければ空のリストを返す
        return QList<QString>();
    }

    QList<int> displayIndexList = displayChannelIndexMap.value(graphMode);

    for (int i = 0; i < displayIndexList.size(); ++i) {
        if (allChannelList.size() <= displayIndexList.at(i)) {
            // 表示するインデックスがチャンネルリストのサイズを超えた場合
            // ループを中断する
            break;
        } else {
            displayChannelList.append(allChannelList.at(displayIndexList.at(i)));
        }
    }

    // 埋まらなかったdisplayChannelListに対し空白を追加する
    if (displayIndexList.size() > displayChannelList.size()) {
        for (int i = displayChannelList.size(); i < displayIndexList.size() ; ++i) {
            displayChannelList.append("");
        }
    }

    return displayChannelList;
}

unsigned int MultiMeasurementViewModel::getLocalFrequency()
{
    unsigned int localFreqency = 0;

    if (_fromUserSetting) {
        localFreqency = _userSettings.at(_currentChannelIndex).ulLocalFreq;
    } else {
        if (_bandId == LC70_BAND_BS_CS) {
            if(_channelList[_currentChannelIndex].polarization == 0) {      // 右旋
                localFreqency = 10678000;
            }
            else if(_channelList[_currentChannelIndex].polarization == 1) {	// 左旋
                localFreqency = 9505000;
            }
        }
        else {
            localFreqency = _takeOver["selectedLocalOscillator"].toUInt();
        }
    }

    return localFreqency;
}

unsigned int MultiMeasurementViewModel::getModulationValue()
{
    unsigned int modulationValue;

    if (_fromUserSetting) {
        modulationValue = _userSettings.at(_currentChannelIndex).ulModBit;
    } else {
        modulationValue = Correcter::modulationValue(_takeOver["selectedModulation"].toString());
    }

    return modulationValue;
}

void MultiMeasurementViewModel::detectPowerSupplyError(uint16_t code, uint8_t action, QString message)
{
    _failureCode = code;
    _actionInFailure = action;

#if 1
    // スキャン実行中の場合、スキャンを停止
    if(_isScanning == true) {
        onRequestToStopScanning();
        //        onScanningFinished(false);	//onRequestToStopScanningから呼び出されるため 不要
    }
#endif

    if (_takeOver["fromUserSetting"].toBool()) {
        // マニュアル測定： 給電を停止する
        _powerSupply = LC70_POWER_SUPPLY_OFF;

#if 1
        if (_hasMultiBandsInUserSetting == true) {
            _model->setPowerSupplyForMultiBand(_powerSupply);
        }
        else {
            _model->setPowerSupply(_powerSupply);
        }
#else
        _model->setPowerSupply(_powerSupply);
#endif
        _model->removeMenuList(MenuListIconIndex::POWER_SUPPLY_ICON);

        if (_actionInFailure == LC70_FAIL_CTL_MSGDISP_CLR) {
            uint8_t addInfo[LC70_FAIL_ADD_INFO_NUM];
            memset(addInfo, 0x00, LC70_FAIL_ADD_INFO_NUM);
            //_model->setFaultInformation(_failureCode, LC70_FAIL_STAT_NORMAL, addInfo);     故障復旧処理不要
            _failureCode = 0;
            _actionInFailure = LC70_FAIL_CTL_NONE;
        }
#if 0
        // 測定停止
        emit requestToSuspendMultiMeasuring();
#endif

        _model->buzzer(LC70_BUZZ_ERR, LC70_BUZZ_MODE_ONESHOT, true);

        emit requestToShowPopupWithMessage(POPUP_POWER_SUPPLY_ERROR_USER_MEASURING, message);
    } else {
        // マニュアル測定： 給電を停止する
        _powerSupply = LC70_POWER_SUPPLY_OFF;
        _model->setPowerSupply(_powerSupply);
        _model->removeMenuList(MenuListIconIndex::POWER_SUPPLY_ICON);

        if (_actionInFailure == LC70_FAIL_CTL_MSGDISP_CLR) {
            uint8_t addInfo[LC70_FAIL_ADD_INFO_NUM];
            memset(addInfo, 0x00, LC70_FAIL_ADD_INFO_NUM);
            //_model->setFaultInformation(_failureCode, LC70_FAIL_STAT_NORMAL, addInfo);     故障復旧処理不要
            _failureCode = 0;
            _actionInFailure = LC70_FAIL_CTL_NONE;
        }
#if 0
        // 測定停止
        emit requestToSuspendMultiMeasuring();
#endif

        _model->buzzer(LC70_BUZZ_ERR, LC70_BUZZ_MODE_ONESHOT, true);

        emit requestToShowPopupWithMessage(POPUP_POWER_SUPPLY_ERROR_MANUAL_MEASURING, message);
    }
}

void MultiMeasurementViewModel::selectReactionScanResult(const int index) {
    if (index == 0) {
        // SCANアイコンを表示する
        _model->replaceMenuList(MenuListIconIndex::SCAN_ICON, "icon_scan_n.png");
    }
}

void MultiMeasurementViewModel::onRequestToShowRegistScanResultPopup() {
    emit requestToShowPopup(POPUP_CONFIRM_REGIST_SCAN_RESULT);
}

void MultiMeasurementViewModel::startMeasuringAfterScan()
{
    // チャンネル変更(resumeMultiMeasuringで測定を開始する)
    changeChannel(true);

    // マルチ測定結果の取得に設定
    _model->changeAcquisitionTarget(MULTI);

    LC70_DISP_TYPE_T displayType = LC70_DISP_TYPE_GRAPH;
    if (_selectedButtonIndex == GRAPH_BUTTON_INDEX) {
        qDebug() << "初期化している";
        int currentChannelIndex = 0;
        selectGraphIndex(0, currentChannelIndex);

        if (_multiGraphModel) {
            // グラフ表示時、白線が残ってしまうことを防ぐため白線の位置を初期化する
            _multiGraphModel->InitializeCurrentPole(true, currentChannelIndex);
        }
    } else {
        displayType = LC70_DISP_TYPE_LIST_MULTI;
    }

    // スレッドを開始
    emit resumeMultiMeasuring(&_condition, &_preferredCondition, displayType,
                              _model->applicationSetting()->pollingIntervalMultiMeasurement());
}

void MultiMeasurementViewModel::onRequestToReScan() {
    // スキャン中ポップアップを表示
    emit requestToShowPopup(POPUP_SCANNING);

    // 再スキャン
    onRequestToScan();
}

void MultiMeasurementViewModel::checkMerCnrRange(int bandId,int bdSys, int modulation, bool forceUpdate)
{
    bool isUpdateMerCnrRange = false;
    // USER設定から遷移して来た場合は、USER設定から値の更新を行う
    if(_previousBandId != bandId) {
        isUpdateMerCnrRange = true;
    }

    if (_broadBandSystem != bdSys) {
        _broadBandSystem = bdSys;
        isUpdateMerCnrRange = true;
    }

    if (_modulation != modulation) {
        _modulation = modulation;
        isUpdateMerCnrRange = true;
    }

    if (forceUpdate || isUpdateMerCnrRange) {
        updateMerCnrRange(bandId, _broadBandSystem, _modulation);
    }
}

void MultiMeasurementViewModel::updateMerCnrRange(const int bandId, const int broadBandSystem, const int modulation)
{
    // MERの変化単位の更新
    QPair<int, int> merRange = Correcter::merRange(bandId, broadBandSystem, modulation);

    // MERの最大値, 最小値を送信する
    if (_view) {
        _view->setProperty("minimumMer", merRange.first);
    }
    if (_view) {
        _view->setProperty("maximumMer", merRange.second);
    }
}

void MultiMeasurementViewModel::onRequestToStopAcquiring()
{
#if 1
    // オートパワーオフ設定を有効化
    qDebug() << QString("オートパワーオフ設定を有効化");
    _model->enablePowerOffSetting();
#endif
    // もとの機能に戻す
    returnToMeasurement();
}

void MultiMeasurementViewModel::onNotifyStopMeasuring(AcquisitionTarget target)
{
    if (target != MULTI) {
        return;
    }

    // マルチ測定の停止を受信
}

void MultiMeasurementViewModel::holdMeasurementData()
{
    // 保持しているデータをクリアする
    _holdMeasurementData.clear();

    // 現在の測定結果を保持する
    
    //レイヤーと偏波を分ける
    for (int i = 0; i < _multiMeasureResult.count(); ++i) {
         _holdMeasurementData.append(_multiMeasureResult[i]);

		// 衛星の場合は 偏波情報に _channellist.polarlizationで上書きする
        if (Correcter::isSatelliteBroadCasting(_multiMeasureResult[i].Band.toInt())) {
            _holdMeasurementData[i].Layer = _channelList[i].polarization;
        }
    }
}

void MultiMeasurementViewModel::setGraphBackground(int bandId, QString polarization)
{
    int index = FIXED_GRAPH_CATEGORY_UHF;
    polarization = _takeOver["selectedPolarization"].toString();

    switch (bandId) {
    case LC70_BAND_UHF:
        index = FIXED_GRAPH_CATEGORY_UHF;
        break;
    case LC70_BAND_BS: {
        if (polarization == QString(RIGHT_HANDED_POLARIZE)) {
            index = FIXED_GRAPH_CATEGORY_BS_RIGHT;
        } else {
            index = FIXED_GRAPH_CATEGORY_BS_LEFT;
        }
        break;
    }
    case LC70_BAND_CS: {
        if (polarization == QString(RIGHT_HANDED_POLARIZE)) {
            index = FIXED_GRAPH_CATEGORY_CS_RIGHT;
        } else {
            index = FIXED_GRAPH_CATEGORY_CS_LEFT;
        }
        break;
    }
    case LC70_BAND_JCSAT3: {
        if (polarization == QString(VERTICALLY_POLARIZE)) {
            index = FIXED_GRAPH_CATEGORY_JCSAT3A_VERTICAL;
        } else {
            index = FIXED_GRAPH_CATEGORY_JCSAT3A_HORIZONTAL;
        }
        break;
    }
    case LC70_BAND_JCSAT4: {
        if (polarization == QString(VERTICALLY_POLARIZE)) {
            index = FIXED_GRAPH_CATEGORY_JCSAT4B_VERTICAL;
        } else {
            index = FIXED_GRAPH_CATEGORY_JCSAT4B_HORIZONTAL;
        }
        break;
    }
    case LC70_BAND_SCCB: {
        if (polarization == QString(VERTICALLY_POLARIZE)) {
            index = FIXED_GRAPH_CATEGORY_SCCB_VERTICAL;
        } else {
            index = FIXED_GRAPH_CATEGORY_SCCB_HORIZONTAL;
        }
        break;
    }
    case LC70_BAND_SCCC: {
        if (polarization == QString(VERTICALLY_POLARIZE)) {
            index = FIXED_GRAPH_CATEGORY_SCCC_VERTICAL;
        } else {
            index = FIXED_GRAPH_CATEGORY_SCCC_HORIZONTAL;
        }
        break;
    }
    case LC70_BAND_CATV_ALL:
        index = FIXED_GRAPH_CATEGORY_CATV_ALL;
        break;
    case LC70_BAND_CATV_90_470MHZ:
        index = FIXED_GRAPH_CATEGORY_CATV_90_470;
        break;
    case LC70_BAND_CATV_470_1002MHZ:
        index = FIXED_GRAPH_CATEGORY_CATV_470_1002;
        break;
    case LC70_BAND_CATV_UP_STREAM:
        index = FIXED_GRAPH_CATEGORY_CATV_UPSTREAM;
        break;
    case LC70_BAND_PILOT:
        index = FIXED_GRAPH_CATEGORY_PILOT;
        break;
    case LC70_BAND_SKYP:
        index = FIXED_GRAPH_CATEGORY_SKY_PERFETV;
        break;
    case LC70_BAND_LTE:
        index = FIXED_GRAPH_CATEGORY_LTE;
        break;
    case LC70_BAND_FM:
        index = FIXED_GRAPH_CATEGORY_FM;
        break;
    case LC70_BAND_BS_CS:
        index = FIXED_GRAPH_CATEGORY_BS_CS;
        break;
    default:
        break;
    }

    // グラフ枠の画像を設定
    if (_view) {
        _view->setProperty("graphIndex", index);
    }
}

void MultiMeasurementViewModel::saveMeasuringMData()
{
    switch (_dataSave2State) {
    case DataSave2None:
        // 測定データを取得するか確認
        emit requestToShowPopup(POPUP_CONFIRM_ACQUIRE_MEASURE_DATA);
        break;
    case DataSave2Retrieving:
        _saveFileType = SaveFileType::CSV;
        // 測定データを保存
        _dataSave2State = DataSave2Storing;
        emit requestToShowPopup(POPUP_SAVE_MEASUREMENT_DATA);
        break;
    default:
        break;
    }
}

void MultiMeasurementViewModel::acquireMeasuringData()
{
    _dataSave2State = DataSave2Retrieving;
    if (_list) {
        _list->setProperty("storing", true);
    }
    _previousSelectedIndex = _selectedButtonIndex;

    // 測定データ取得中はリスト表示
    changeButton(LIST_BUTTON_INDEX);
    onRequest(LIST_BUTTON_INDEX);

    // 測定データ取得中ポップアップを表示
    emit requestToShowPopup(POPUP_MEASURING);
}

void MultiMeasurementViewModel::changeButton(int buttonIndex)
{
    if (buttonIndex == LIST_BUTTON_INDEX) {
        if (_view) {
            // inactive
            setButtonActive(_view, multiMeasurementButtonData[0][0], false);

            // active
            setButtonActive(_view, multiMeasurementButtonData[0][1], true);
        }

        _activeButton = multiMeasurementButtonData[0][1];
    } else {
        if (_view) {
            // inactive
            setButtonActive(_view, multiMeasurementButtonData[0][1], false);

            // active
            setButtonActive(_view, multiMeasurementButtonData[0][0], true);
        }

        _activeButton = multiMeasurementButtonData[0][0];
    }
    _buttonOperation->setActiveObject(_activeButton);
}

void MultiMeasurementViewModel::returnToMeasurement()
{
    if (_dataSave2State == DataSave2None) {
        // データ測定していなかった場合は何もしない
        return;
    }
    // もとの機能に戻す
    _dataSave2State = DataSave2None;
    _holdMeasurementData.clear();
    if (_list) {
        _list->setProperty("storing", false);
    }
    setBoolProperty(DATA_BUTTON_NAME, "pressed", false);
    setBoolProperty(MEASUREMENT_RESULT_DISPLAY, "visible", false);

    changeButton(_previousSelectedIndex);
    onRequest(_previousSelectedIndex);
}

void MultiMeasurementViewModel::refreshAllChannels()
{
    // 全チャンネル再取得
#if 1
    //bandId = _takeOver["selectedPolarization"].toString() になると思われるが既存実装への影響を懸念してBSCSのみ別処理とする。
#endif
    int bandId = _channelList[_currentChannelIndex].band;
    _channelList.clear();
    QString polarization = "";

    if (_takeOver.contains("selectedPolarization")) {
        polarization = _takeOver["selectedPolarization"].toString();
    }

#if 1
    // 画面共有情報からBandを取得
    int selectedBand = _takeOver["selectedBand"].toInt();

    if (selectedBand == LC70_BAND_BS_CS) {
        _model->selectChannelDefinitionList(LC70_BAND_BS_CS, polarization, _channelList);
    }
    else {
        _model->selectChannelDefinitionList(bandId, polarization, _channelList);
    }
#else
    _model->selectChannelDefinitionList(bandId, polarization, _channelList);
#endif

    _currentChannelIndex = 0;
}

void MultiMeasurementViewModel::selectGraphIndex(const int &originalIndex, int &replacedIndex)
{
    if (originalIndex < 0) {
        return;
    }

    if (_channelList.count() == 0 || _channelListForGraph.count() == 0) {
        return;
    }

    for (int i = 0; i < _channelListForGraph.count(); ++i) {
        if (_channelList[originalIndex].channelNumber == _channelListForGraph[i].channelNumber) {
            replacedIndex = i;
            break;
        }
    }
}

void MultiMeasurementViewModel::setChannelsFromPreset(QVariant values, const LC70_BAND_T &band)
{
    QVariantList valueList = values.toList();
    if (valueList.count() != (LC70_CH_SET_CATV_ALL_IDX_MAX + LC70_LOGGER_SET_LOGGER_SAVE_FM_NUM + LC70_LOGGER_SET_LOGGER_SAVE_UP_STREAM_NUM)) {
        qDebug() << QString("valueList.count(): %1 LC70_LOGGER_SET_LOGGER_IDX_MAX: %2").arg(valueList.count()).arg(LC70_LOGGER_SET_LOGGER_IDX_MAX);
        return;
    }

    int start = (band == LC70_BAND_FM) ? LC70_CH_SET_CATV_ALL_IDX_MAX : LC70_CH_SET_CATV_ALL_IDX_MAX + LC70_LOGGER_SET_LOGGER_SAVE_FM_NUM;
    int end = (band == LC70_BAND_FM) ? LC70_CH_SET_CATV_ALL_IDX_MAX + LC70_LOGGER_SET_LOGGER_SAVE_FM_NUM : (LC70_CH_SET_CATV_ALL_IDX_MAX + LC70_LOGGER_SET_LOGGER_SAVE_FM_NUM + LC70_LOGGER_SET_LOGGER_SAVE_UP_STREAM_NUM);

    QString bandName = (band == LC70_BAND_FM) ? QString("FM") : QString("UL");

    // Band のプリセット全チャンネルを対象とする
    int count = 1;
    for (int i = start; i < end; ++i) {
        QVariantMap item = valueList[i].toMap();

        // チャンネル定義ファイルからチャンネルの情報を取得し測定チャンネルリストに追加
        ChannelDefinitionItem channelDefinitionItem;
        if (_model->getChannelDefinitionItem(band, item["channel"].toString(), channelDefinitionItem)) {

            channelDefinitionItem.valid = item["enable"].toBool();

            // 有効な行のみ
            if (channelDefinitionItem.valid) {
                // 周波数を USER 設定で入力した値に付け替える
                channelDefinitionItem.centerFrequency1 = item["centerFreq"].toUInt();

                // 周波数が入っているのを USER 設定のチャンネル名に付け替え
                channelDefinitionItem.channelNumber = QString("%1 %2").arg(bandName).arg(count);

                _channelList.append(channelDefinitionItem);

                // USER 設定ファイルの 1 チャンネルの内容を追加
                setUserSettings(item, channelDefinitionItem);
            }
        }

        // グラフ用にチャンネル定義リストを保持する
        _channelListForGraph.append(channelDefinitionItem);

        ++count;
    }
}

void MultiMeasurementViewModel::setChannelsFromSetting(QVariant values, const LC70_BAND_T &band)
{
    QVariantList valueList = values.toList();
    if (valueList.count() != (LC70_CH_SET_CATV_ALL_IDX_MAX + LC70_LOGGER_SET_LOGGER_SAVE_FM_NUM + LC70_LOGGER_SET_LOGGER_SAVE_UP_STREAM_NUM)) {
        qDebug() << QString("valueList.count(): %1 LC70_LOGGER_SET_LOGGER_IDX_MAX: %2").arg(valueList.count()).arg(LC70_LOGGER_SET_LOGGER_IDX_MAX);
        return;
    }

    // プリセットの 24 件を除いた 150 件が対象
    int count = 0;
    foreach (QVariant value, valueList) {
        QVariantMap item = value.toMap();

        // 150 行目以降はプリセットの定義なので抜ける
        if (++count == LC70_CH_SET_CATV_ALL_IDX_MAX) {
            break;
        }

        // 無効な行が見つかったら以降も無効な行なので抜ける
        if (!item["enable"].toBool()) {
            break;
        }

        // チャンネル定義ファイルからチャンネルの情報を取得し測定チャンネルリストに追加
        ChannelDefinitionItem channelDefinitionItem;
        if (_model->getChannelDefinitionItem(band, item["channel"].toString(), channelDefinitionItem)) {
            _channelList.append(channelDefinitionItem);
        }

        // USER 設定ファイルの 1 チャンネルの内容を追加
        setUserSettings(item, channelDefinitionItem);
    }
    // グラフ用にチャンネル定義リストを保持する
    _model->selectChannelDefinitionList(_channelList[0].band, Correcter::convertWaveToPolarization(_channelList[0].polarization), _channelListForGraph);
}

void MultiMeasurementViewModel::setChannelsFromSetting(QVariant values)
{
    // USER 設定で複数の Band を跨ぐ場合にチャンネル情報を設定
    QVariantList valueList = values.toList();
    if (valueList.count() != (LC70_CH_SET_CATV_ALL_IDX_MAX + LC70_LOGGER_SET_LOGGER_SAVE_FM_NUM + LC70_LOGGER_SET_LOGGER_SAVE_UP_STREAM_NUM)) {
        qDebug() << QString("valueList.count(): %1 LC70_LOGGER_SET_LOGGER_IDX_MAX: %2").arg(valueList.count()).arg(LC70_LOGGER_SET_LOGGER_IDX_MAX);
        return;
    }

    // プリセットの 24 件を除いた 150 件が対象
    int count = 0;
    foreach (QVariant value, valueList) {
        QVariantMap item = value.toMap();

        // 150 行目以降はプリセットの定義なので抜ける
        if (++count == LC70_CH_SET_CATV_ALL_IDX_MAX) {
            break;
        }

        // 無効な行が見つかったら以降も無効な行なので抜ける
        if (!item["enable"].toBool()) {
            break;
        }

        LC70_BAND_T band = static_cast<LC70_BAND_T>(item["band"].toUInt());

        //        // FM/CATV 上りは Band を付け替えてプリセットの値を読み取る
        //        if (band == LC70_BAND_FM) {
        //            band = LC70_BAND_FM_DEF;
        //        } else if (band == LC70_BAND_CATV_UP_STREAM) {
        //            band = LC70_BAND_CATV_UP_STREAM_DEF;
        //        }

        // チャンネル定義ファイルからチャンネルの情報を取得し測定チャンネルリストに追加
        ChannelDefinitionItem channelDefinitionItem;
        QString channel = item["channel"].toString();
        if (band == LC70_BAND_FM) {
            channel = QString("%1").arg(item["centerFreq"].toFloat() / 1000, 2, 'f', 1);    // 小数点第 1 位
        } else if (band == LC70_BAND_CATV_UP_STREAM) {
            channel = QString("%1").arg(item["centerFreq"].toFloat() / 1000, 2, 'f', 2);    // 小数点第 2 位
        }

        if (_model->getChannelDefinitionItem(band, channel, channelDefinitionItem)) {
            //            // FM/CATV 上りは USER 設定で入力した周波数に差し替える
            //            if (band == LC70_BAND_FM || band == LC70_BAND_CATV_UP_STREAM) {
            //                channelDefinitionItem.centerFrequency1 = item["centerFreq"].toInt();
            //                channelDefinitionItem.centerFrequency2 = 0;
            //            }
            // FM/CATV 上りは USER 設定で入力した周波数に該当するチャンネルのチャンネル名を USER 設定でのチャンネル名に差し替える(プリセットのチャンネル定義ではない)
            if (band == LC70_BAND_FM) {
                int fmIndex = 1;
                for (int i = LC70_CH_SET_CATV_ALL_IDX_MAX; i < LC70_CH_SET_CATV_ALL_IDX_MAX + LC70_USER_SET_USER_SAVE_FM_NUM; ++i) {
                    if (values.toList()[i].toMap()["channel"].toString() == item["channel"].toString()) {
                        // プリセットの定義の周波数が定義されたインデックスからチャンネル名を組み立てる
                        channelDefinitionItem.channelNumber = QString("FM %1").arg(fmIndex);
                        break;
                    }
                    ++fmIndex;
                }
                channelDefinitionItem.centerFrequency1 = item["centerFreq"].toInt();
                channelDefinitionItem.centerFrequency2 = 0;
            } else if (band == LC70_BAND_CATV_UP_STREAM) {
                int catvUpstreamIndex = 1;
                for (int i = LC70_CH_SET_CATV_ALL_IDX_MAX + LC70_USER_SET_USER_SAVE_FM_NUM; i < LC70_CH_SET_CATV_ALL_IDX_MAX + LC70_USER_SET_USER_SAVE_FM_NUM + LC70_USER_SET_USER_SAVE_UP_STREAM_NUM; ++i) {
                    if (values.toList()[i].toMap()["channel"].toString() == item["channel"].toString()) {
                        // プリセットの定義の周波数が定義されたインデックスからチャンネル名を組み立てる
                        channelDefinitionItem.channelNumber = QString("UL %1").arg(catvUpstreamIndex);
                        break;
                    }
                    ++catvUpstreamIndex;
                }
                channelDefinitionItem.centerFrequency1 = item["centerFreq"].toInt();
                channelDefinitionItem.centerFrequency2 = 0;
            }

            _channelList.append(channelDefinitionItem);
        }

        // USER 設定ファイルの 1 チャンネルの内容を追加
        setUserSettings(item, channelDefinitionItem);
    }
}

void MultiMeasurementViewModel::setUserSettings(const QVariantMap &item, const ChannelDefinitionItem &channelDefinitionItem)
{
    LC70_LOGGER_SET_LOGGER_ROW_T row;
    memset(&row, 0, sizeof(LC70_LOGGER_SET_LOGGER_ROW_T));
    row.ucEnbFlg = item["enable"].toBool();
    row.ucNo = item["ucNo"].toInt();
    row.ucBand = item["band"].toInt();
    row.ucSpec = item["broadbandSystem"].toInt();
    strncpy(row.cStrChNo, item["channel"].toString().toUtf8().constData(), item["channel"].toByteArray().size());
    row.ulCenterFreq = Correcter::centerFrequency(channelDefinitionItem, item["modulationBit"].toUInt());
    row.ulModBit = item["modulationBit"].toUInt();
    row.ulLocalFreq = item["localFreq"].toUInt();
    row.ucGetDataFlg = item["MERCN"].toBool() ? 0x0002 : 0x0;
    row.ucGetDataFlg |= item["BER"].toBool() ? 0x0004 : 0x0;
    row.ucDcOut = item["PowerSupply"].toInt();

    qDebug() << QString("[USER SETTING] Band:%1 Channel: %2 BroadbandSystem: %3 Modulation: %4 CenterFrequency: %5 LocalFrequency: %6 PowerSupply: %7 FLAG: %8").arg(row.ucBand).arg(item["channel"].toString()).arg(row.ucSpec).arg(row.ulModBit).arg(row.ulCenterFreq).arg(row.ulLocalFreq).arg(row.ucDcOut).arg(row.ucGetDataFlg);
    _userSettings.append(static_cast<LC70_LOGGER_SET_LOGGER_ROW_T>(row)); // _channelList では不足する項目を補う
}

bool MultiMeasurementViewModel::isValidPresetChannel(const unsigned int &frequency, const QVariant &values)
{
    // 既に測定チャンネルリストにあれば無効とする
    for (int i = 0; i < _channelList.count(); ++i) {
        // CATV ALL ではないので centerFrequency1 しか使わない
        if (_channelList[i].valid && static_cast<unsigned int>(_channelList[i].centerFrequency1) == frequency) {
            return false;
        }
    }

    int count = 0;
    foreach (QVariant value, values.toList()) {
        QVariantMap item = value.toMap();

        // 150 行目以降はプリセットの定義なので抜ける
        if (++count == LC70_CH_SET_CATV_ALL_IDX_MAX) {
            break;
        }

        // 設定内容にあるチャンネルと合致
        if (item["centerFreq"].toUInt() == frequency) {
            return true;
        }
    }

    return false;
}

void MultiMeasurementViewModel::switchPowerSupply()
{
    if (_powerSupply == LC70_POWER_SUPPLY_OFF) {
        // バッテリーレベルが 5% 以下で未充電あるいは不定値のときはシャットダウン
        if (isLowBattery() && batteryStatus() <= BATTERY_STATUS_UNCHARGED) {
            // バッテリ低下状態中は給電開始させない
            emit requestToShowLowBatteryPopup();
            return;
        }

        // バッテリーレベルが 10% 以下なら給電不可
        if (batteryLevel() <= BATTERY_LEVEL2 && batteryStatus() <= BATTERY_STATUS_UNCHARGED) {
            emit requestToShowPopup(POPUP_DISABLE_POWER_SUPPLY);
            return;
        }

        // 給電確認
        if (_bandId == LC70_BAND_JCSAT3 || _bandId == LC70_BAND_JCSAT4 || _bandId == LC70_BAND_SCCB
                || _bandId == LC70_BAND_SCCC) {
            // JCSAT, SCC は垂直11v水平15v, 15v固定, 給電しないを選択
            emit requestToShowPopup(POPUP_POWER_SUPPLY_SELECT3);
        } else {
            // 給電しますか?
            emit requestToShowPopup(POPUP_POWER_SUPPLY_SELECT2);
        }
    } else {
        // 給電停止
        _powerSupply = LC70_POWER_SUPPLY_OFF;
        _model->setPowerSupply(_powerSupply);
        // ヘッダーアイコン設定
        _model->removeMenuList(MenuListIconIndex::POWER_SUPPLY_ICON);
        // 給電中ではない
        _powerSupplied = false;
    }
}

void MultiMeasurementViewModel::switchPowerSupplyForMultiBand()
{
    if (_powerSupply == LC70_POWER_SUPPLY_OFF) {
        // バッテリーレベルが 5% 以下で未充電あるいは不定値のときはシャットダウン
        if (isLowBattery() && batteryStatus() <= BATTERY_STATUS_UNCHARGED) {
            // バッテリ低下状態中は給電開始させない
            emit requestToShowLowBatteryPopup();
            return;
        }

        // バッテリーレベルが 10% 以下なら給電不可
        if (batteryLevel() <= BATTERY_LEVEL2 && batteryStatus() <= BATTERY_STATUS_UNCHARGED) {
            emit requestToShowPopup(POPUP_DISABLE_POWER_SUPPLY);
            return;
        }

        emit requestToShowPopup(POPUP_POWER_SUPPLY_SELECT2);
    } else {
        // 給電停止
        _powerSupply = LC70_POWER_SUPPLY_OFF;
        _model->setPowerSupplyForMultiBand(0);
        _model->removeMenuList(MenuListIconIndex::POWER_SUPPLY_ICON);
        // 給電中ではない
        _powerSupplied = false;
    }
}

void MultiMeasurementViewModel::displayPrefferedChannel(int index)
{
    if (index < 0 || index > _multiMeasureResult.count() || _multiMeasureResult.count() == 0) {
        return;
    }

    // 優先測定の結果は使わずマルチ測定の測定結果から表示する(優先測定の測定条件は判定に使用する)

    // ログ収集中は優先測定結果を更新しない
    if (_dataSave2State == DataSave2None) {
        // 優先測定測定 入力レベル
        // ラッパー関数の戻り値が LC70_OK で、収集ステータスの該当ビットが立っており、読み出しインデックスがチャンネルリストの現在のインデックスと合致していれば更新する
        if ((_multiMeasureResult[index].Status & LC70_DATA_MSK_LEVEL) == LC70_DATA_MSK_LEVEL && _multiMeasureResult[index].Received) {
            LC70_BAND_T band = static_cast<LC70_BAND_T>(_channelList.at(_currentChannelIndex).band);
            QString polarize = Correcter::convertPolarizeNumToString(_channelList.at(_currentChannelIndex).polarization);
            BandTabIndex bandTab = Correcter::convertBandTabIndex(band, polarize);
            QString levelString;

            if (!Correcter::displayLevel(bandTab, _multiMeasureResult[index].Level, levelString)) {
                qDebug() << QString("displayLevel() failed");
            }

            if (_view) {
                _view->setProperty("level", levelString);
            }
        }

        // 読み出しインデックスがチャンネルリストの現在のインデックスと合致していれば
        // ロックされているかつ、BANDがPILOT, LTE, FM 以外であれば、値更新、
        // それ以外の場合は "---" 表示
        if (_multiMeasureResult[index].Locked == LOCKED) {
            // 優先測定 MER
            // 収集ステータスの該当ビットが立っていれば更新する
            if (((_multiMeasureResult[index].Status & LC70_DATA_MSK_CNR) == LC70_DATA_MSK_CNR ||
                 (_multiMeasureResult[index].Status & LC70_DATA_MSK_MER) == LC70_DATA_MSK_MER) && _multiMeasureResult[index].Received) {
                if (_preferredCondition.usBand == LC70_BAND_PILOT
                        || _preferredCondition.usBand == LC70_BAND_LTE
                        || _preferredCondition.usBand == LC70_BAND_FM
                        || _preferredCondition.usBand == LC70_BAND_CATV_UP_STREAM) {
                    if (_view) {
                        _view->setProperty("mer", QString("---"));
                    }
                } else {
                    if (qIsNaN(_multiMeasureResult[index].Mer) || qIsInf(_multiMeasureResult[index].Mer)) {
                        if (_view) {
                            // MERの値がnanかinfであった場合 "---"で表示する
                            _view->setProperty("mer", QString("---"));
                        }
                    } else {
                        if (_view) {
                            _view->setProperty("mer", Correcter::displayMer(_preferredCondition.usBand, _preferredCondition.usBdSys, _preferredCondition.usMod, _multiMeasureResult[index].Mer));
                        }
                    }
                }
            } else {
                if (_view) {
                    // 取得データフラグの MER/CN のビットが立っていない場合 MER には -2.0 が入っている
                    if (_multiMeasureResult[index].Mer <= -2.0) {
                        if (_preferredCondition.usBdSys == LC70_BD_SYS_ANNEXB || _preferredCondition.usBdSys == LC70_BD_SYS_ANNEXC || _preferredCondition.usBdSys == LC70_BD_SYS_OPTICAST) {
                            _view->setProperty("mer", QString("<20.0"));
                        } else {
                            _view->setProperty("mer", QString(MER_TIMEOUT_VALUE));
                        }
                    } else {
                        _view->setProperty("mer", QString("---"));
                    }
                }
            }
        } else {
            if (_view) {
                //_view->setProperty("mer", QString("---"));
                // ロック待ち時間のタイムアウト後あるいは値確定後
                //if (_waitforLockingTimeOut || _confirmedValue) {

                // 何も取得していない
                if (_multiMeasureResult[index].Status == 0) {
                    _view->setProperty("mer", QString("---"));
                } else {
                    if (_preferredCondition.usBdSys == LC70_BD_SYS_ANNEXB || _preferredCondition.usBdSys == LC70_BD_SYS_ANNEXC || _preferredCondition.usBdSys == LC70_BD_SYS_OPTICAST){
                        _view->setProperty("mer", QString("<20.0"));
                    } else {
                        _view->setProperty("mer", QString(MER_TIMEOUT_VALUE));
                    }
                }
               // _view->setProperty("mer", QString(MER_TIMEOUT_VALUE));
                //}
            }
        }

        // ロックされていなければ、 "---"を表示
        if (_multiMeasureResult[index].Locked == LOCKED) {
            // 優先測定 BER
            // 収集ステータスの該当ビットが立っていれば更新する
            if ((_multiMeasureResult[index].Status & LC70_DATA_MSK_BER) == LC70_DATA_MSK_BER && _multiMeasureResult[index].Received) {
                // bandがPILOT, LET, FMの場合は "---"を表示
                if (_preferredCondition.usBand == LC70_BAND_PILOT
                        || _preferredCondition.usBand == LC70_BAND_LTE
                        || _preferredCondition.usBand == LC70_BAND_FM
                        || _preferredCondition.usBand == LC70_BAND_CATV_UP_STREAM) {
                    if (_view) {
                        _view->setProperty("ber", QString("---"));
                    }
                } else {
                    // 値がnan、infの場合 "---"を表示
                    if (qIsNaN(_multiMeasureResult[index].Ber) || qIsInf(_multiMeasureResult[index].Ber)) {
                        if (_view) {
                            _view->setProperty("ber", QString("---"));
                        }
                    } else {
                        if (_view) {
                            _view->setProperty("ber", QString("%1").arg(Correcter::displayBer(_multiMeasureResult[index].Ber)));
                        }
                    }
                }
            } else {
                if (_view) {
                    // 取得データフラグの BER のビットが立っていない場合 BER には -2.0 が入っている
                    if (_multiMeasureResult[index].Ber <= -2.0) {
                        _view->setProperty("ber", QString(BER_TIMEOUT_VALUE));
                    } else {
                        _view->setProperty("ber", QString("---"));
                    }
                }
            }
        } else {
            if (_view) {
                //_view->setProperty("ber", QString("---"));
                // ロック待ち時間のタイムアウト後あるいは値確定後
                //if (_waitforLockingTimeOut || _confirmedValue) {
                //}

                // 何も取得していない
                if (_multiMeasureResult[index].Status == 0) {
                    _view->setProperty("ber", QString("---"));
                } else {
                    _view->setProperty("ber", QString(BER_TIMEOUT_VALUE));
                }
            }
        }

        if (_channelList[_currentChannelIndex].band == LC70_BAND_FM) {
            if (_view) {
                // 測定BANDがFMの場合、変調方式FMの固定値を表示
                _view->setProperty("displayModulationPrefixMode", DISPLAY_MODULATION_PREFIX_TOGETHER);
                _view->setProperty("modulation", FM);
                _view->setProperty("modulationPrefix", QString(""));
            }
        } else if (_channelList[_currentChannelIndex].band == LC70_BAND_PILOT || _channelList[_currentChannelIndex].band == LC70_BAND_CATV_UP_STREAM) {
            if (_view) {
                // 測定BANDが"PILOT", "CATV上る"の場合、編著方式CWの固定値を表示
                _view->setProperty("displayModulationPrefixMode", DISPLAY_MODULATION_PREFIX_TOGETHER);
                _view->setProperty("modulation", CW);
                _view->setProperty("modulationPrefix", QString(""));
            }
        } else {
            // 優先測定 変調方式
            QString modulation = Correcter::displayModulation(_multiMeasureResult[index].Modulation);
            if (modulation.isEmpty()) {
                modulation = QString(INITIAL_MODULATION);
            }
            // 変調方式の接頭句
            QString modulationPrefix = Correcter::modulationPrefix(_multiMeasureResult[index].BroadcastSystem, modulation);
            // 変調方式と変調方式の接頭句をどのように表示するかを取得する
            int displayModulationPrefixMode = Correcter::displayModulationPrefixMode(_multiMeasureResult[index].BroadcastSystem);


            if (_multiMeasureResult[index].Locked == LOCKED) {
                if (_view) {
                    _view->setProperty("displayModulationPrefixMode", displayModulationPrefixMode);
                    _view->setProperty("modulation", modulation);
                    _view->setProperty("modulationPrefix", modulationPrefix);
                }
            } else {
                if (_view) {
                    // 変調方式未確定時に UNLOCK と表示
                    _view->setProperty("displayModulationPrefixMode", DISPLAY_MODULATION_PREFIX_TOGETHER);
                    _view->setProperty("modulation", QString(INITIAL_MODULATION));
                    _view->setProperty("modulationPrefix", QString(""));
                }
            }
        }

        // 優先測定 階層
        // 放送方式が ISDBT のみ有効
        if (_multiMeasureResult[index].BroadcastSystem == LC70_BD_SYS_ISDBT) {
            if (_multiMeasureResult[index].Locked == LOCKED) {
                if (_view) {
                    _view->setProperty("hierarchy", Correcter::displayHierarchy(_multiMeasureResult[index].Layer));
                }
            } else {
                if (_view) {
                    _view->setProperty("hierarchy", QString(""));
                }
            }
        } else {
            if (_view) {
                _view->setProperty("hierarchy", QString(""));
            }
        }
    }
}

void MultiMeasurementViewModel::complementChannelDefinition(const LC70_BAND_T band)
{
    if (band == LC70_BAND_CATV_UP_STREAM_DEF) {
        // UL 8 〜 12 を追加
        for (int i = 7; i < 12; ++i) {
            ChannelDefinitionItem channelDefinitionItem;
            channelDefinitionItem.band = LC70_BAND_CATV_UP_STREAM;
            channelDefinitionItem.channelNumber = QString("UL %1").arg(i + 1);
            channelDefinitionItem.centerFrequency1 = 10000;
            channelDefinitionItem.centerFrequency2 = -1;
            channelDefinitionItem.polarization = -1;
            channelDefinitionItem.bandWidth = -1;
            channelDefinitionItem.valid = true;

            _channelListForGraph.append(channelDefinitionItem);
        }
    } else if (band == LC70_BAND_FM_DEF) {
        // FM 9 〜 12 を追加
        for (int i = 8; i < 12; ++i) {
            ChannelDefinitionItem channelDefinitionItem;
            channelDefinitionItem.band = LC70_BAND_FM;
            channelDefinitionItem.channelNumber = QString("FM %1").arg(i + 1);
            channelDefinitionItem.centerFrequency1 = 70000;
            channelDefinitionItem.centerFrequency2 = -1;
            channelDefinitionItem.polarization = -1;
            channelDefinitionItem.bandWidth = -1;
            channelDefinitionItem.valid = true;

            _channelListForGraph.append(channelDefinitionItem);
        }
    }
}
