/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef MultiMeasurementVIEWMODEL_H
#define MultiMeasurementVIEWMODEL_H

#include "../abstractmeasurementviewmodel.h"
#include "buttonoperation.h"
#include "correcter.h"
#include "measureresultmodel.h"
#include "multigraphmodel.h"

/**
 * @brief マルチチャンネル測定グラフ ViewModel
 */
class MultiMeasurementViewModel : public AbstractMeasurementViewModel
{
    Q_OBJECT
public:
    /**
     * @brief コンストラクター
     * @param screenId 画面 ID
     * @param model Model クラスのポインター
     * @param parent 親オブジェクト
     */
    explicit MultiMeasurementViewModel(ScreenId screenId, LevelCheckerModel *&model, QObject *parent = nullptr);

    /**
     * @brief デストラクタ
     */
    virtual ~MultiMeasurementViewModel();

    /**
     * @brief 画面を開くときに呼び出すスレッドの開始、再開処理
     */
    virtual void resume(void) override;

    /**
     * @brief 画面を閉じるときに呼び出すスレッドの停止処理
     */
    virtual void suspend(void) override;

    /**
     * @brief 画面を開くときに呼び出す qml との接続処理
     * @param qml qml のオブジェクトのポインター
     * @param mainFrame MainFrameViewModel のポインター
     */
    virtual void createConnection(QObject *qml, QObject *mainFrame) override;

    /**
     * @brief 画面を開くときに呼び出す表示内容の復元処理
     */
    virtual void activate(void) override;

    /**
     * @brief 画面を閉じるときに呼び出す表示な用の保存処理
     */
    virtual void deactivate(void) override;

    /**
     * @brief メニューバーによる設定変更
     * @note スレッドは止めずに変更した設定で表示し直す
     */
    void changeByMenuBar(void) override;

signals:

    /**
     * @brief 画面共通情報設定要求
     * @param key 画面共通情報キー
     * @param value 画面共通情報データ
     */
    void requestSetTakeOver(QVariant key, QVariant value);

    /**
     * @brief 測定結果取得開始、再開依頼を通知
     * @param condition マルチ測定条件
     * @param prefferedCondition 優先測定条件
     * @pram displayType 遷移先画面種別
     * @param interval 測定結果取得間隔
     */
    void resumeMultiMeasuring(LC70_MEAS_MULTI_T *condition, LC70_MEAS_SINGLE_T *prefferedCondition, LC70_DISP_TYPE_T displayType, uint32_t interval);

    /**
     * @brief チャンネル変更
     * @param condition チャンネル別測定条件
     */
    void changeCondition(LC70_MEAS_SINGLE_T *condition);

    /**
     * @brief 画面のキャプチャー依頼
     */
    void requestToCapture(void);

    /**
     * @brief キャプチャーした画像の保存依頼
     * @param measureType 測定種類
     * @param saveLocation 保存先
     * @param fileName ファイル名
     */
    void requestToSaveImage(MeasureType measureType, SaveLocation saveLocation, QString fileName);

    /**
     * @brief キャプチャーした画像の削除依頼
     */
    void requestToClearCaptureImage(void);

    /**
     * @brief データ保存を依頼
     * @param saveLocation 保存先
     * @param measureType 測定の種類
     * @param fileName ファイル名
     * @param condition 測定条件
     * @param singleMeasureResult 測定結果
     */
    void requestToSaveDataMulti(SaveLocation saveLocation, MeasureType measureType, QString fileName, uint16_t condition,
                                QList<MeasureResult> singleMeasureResult);

    /**
     * @brief メニューバーのカーソルのあるボタンを押した状態にする
     * @param rank 階層
     */
    void cursorPressed(QVariant rank);

    /**
     * @brief リストのページを移動
     * @param page ページ
     */
    void scrollPage(QVariant page);

    /**
     * @brief プログレスバーの進捗の値を通知
     * @param value 進捗の値
     */
    void progress(double value);

    /**
     * @brief プログレスを通知
     * @param current 現在、ロックされたチャンネル
     * @param total チャンネルの総合値
     * @param flag ロックされた
     */
    void progressText(QString current, bool flag);

    /**
     * @brief 再スキャン要求
     */
    void requestToReScan(void);

    /**
     * @brief スキャン結果保存確認ダイアログを表示する
     * @note コンテキストを切るためにシグナル/スロットで実装する
     */
    void requestToShowRegistScanResultPopup(void);

    /**
     * @brief 測定データ取得終了通知
     * @param complete 完了有無
     */
    void acquiringFinished(bool complete);

    /**
     * @brief マルチ測定停止依頼
     */
    void requestToSuspendMultiMeasuring(void);

protected:
    /**
     * @brief 初期状態の画面表示に戻す
     */
    virtual void resetScreen(void) override;

    /**
     * @brief 必要に応じて View を指すポインターを付け替えた後の復帰処理を行う
     */
    virtual void recover(void) override;

protected slots:
    /**
     * @brief ボタン押下の状態を受信
     * @param id ボタン ID
     */
    virtual void onKeyStatus(ButtonId id) override;

    /**
     * @brief ボタン押下を受信
     * @param id ボタン ID
     */
    virtual void onButtonPress(ButtonId id) override;

    /**
     * @brief ポップアップから受諾の通知を受信
     * @param okOnly OK のみの場合 true
     */
    virtual void onAccept(PopupId popupId, bool okOnly) override;

    /**
     * @brief ポップアップから拒否通知を受信
     * @param popupId 表示中ポップアップのID
     */
    virtual void onReject(PopupId popupId) override;

    /**
     * @brief スキャン結果ポップアップから拒否通知を受信
     */
    virtual void onRejectScanResult() override;

    /**
     * @brief ポップアップから選択の通知を受信
     * @param index 選択されたインデックス
     */
    virtual void onSelect(PopupId popupId, int index) override;

private slots:
    /**
     * @brief 機能変更
     * @param 選択機能
     */
    virtual void onRequest(int condition) override;

    /**
     * @brief マルチ測定結果(優先測定結果を含む)を受信
     * @param result 測定結果
     * @param prefferedChannelResult マルチチャンネル優先測定結果
     */
    void onMultiMeasureResult(MultiResult result, MeasureResult prefferedChannelResult);

    /**
     * @brief 入力されたファイル名を受信
     * @param popupId ポップアップの種類
     * @param saveLocation 保存先
     * @param fileName 入力されたファイル名
     */
    void onInputFileName(PopupId popupId, SaveLocation saveLocation, QString fileName);

    /**
     * @brief ファイル保存完了を受信
     */
    void onNotifySaveEnd();

    /**
     * @brief チャンネル変更中通知を受信
     * @param changing 変更中か否か
     */
    void onNotifyChannelChanging(bool changing);

    /**
     * @brief スキャンの実行依頼を受信
     * @note 呼び出し画面でスキャン処理(開始、結果取得、中止)を実施
     */
    void onRequestToScan(void);

    /**
     * @brief スキャン中止依頼を受信
     */
    virtual void onRequestToStopScanning(void) override;

    /**
     * @brief スキャン結果を受信
     * @param index スキャン対象のチャンネルリストのインデックス
     * @param result スキャン結果
     */
    void onScanningChannelResult(int index, LC70_MEAS_MULTI_RESULT_T result);

    /**
     * @brief スキャン終了通知を受信
     * @param complete 完了有無
     */
    void onScanningFinished(bool complete);

    /**
     * @brief ポップアップ表示中に入力されたボタンのボタン ID を受信
     * @param popupId ポップアップ ID
     * @param buttonId ボタン ID
     */
    void onNotifyButtonInput(PopupId popupId, ButtonId buttonId);

    /**
     * @brief チャンネル確定遅延タイマー タイムアウト受信時処理
     */
    void onConfirmedChangeChannelTimeout(void);

    /**
     * @brief 給電異常検知
     * @param code 故障コード
     * @param action 故障時の動作
     * @param message 表示メッセージ
     */
    void detectPowerSupplyError(uint16_t code, uint8_t action, QString message);

    /**
     * @brief 再スキャン要求受信
     */
    void onRequestToReScan(void);

    /**
     * @brief スキャン結果保存確認ダイアログを表示する
     * @note コンテキストを１度切るためにシグナル/スロットで実装
     */
    void onRequestToShowRegistScanResultPopup(void);

    /**
     * @brief 測定データ取得中止依頼を受信
     */
    void onRequestToStopAcquiring(void);

    /**
     * @brief 測定停止を受信
     * @param target 測定の種類
     */
    void onNotifyStopMeasuring(AcquisitionTarget target);

private:

    /**
     * @brief 測定データ保存の動作状態
     */
    enum DataSave2State {
        DataSave2None = 0,                      //!< 動作なし
        DataSave2Retrieving,                    //!< データ取得
        DataSave2Storing,                       //!< データ保存
    };

    ButtonOperation *_buttonOperation;          //!< ボタン操作クラス
    QString _activeButton;                      //!< アクティブなボタンの objectName
    int _selectedButtonIndex;                   //!< 選択機能
    QObject *_fixedGraph;                       //!< グラフ(枠を画像表示)の QML
    QObject *_list;                             //!< リストの QML
    QList<ChannelDefinitionItem>
    _preservedChannelList; //!< 保持しておくチャンネル定義情報のリスト(ユーザー設定を除く)
    SaveFileType _saveFileType;                 //!< 保存するファイルの種類
    LC70_MEAS_MULTI_T _condition;               //!< マルチ測定条件
    LC70_MEAS_SINGLE_T _preferredCondition;     //!< 優先測定条件
    LC70_MEAS_SINGLE_T _previousPreferredCondition; //!< 前回の優先測定条件
    MeasureResultModel *_measureResultModel;    //!< リスト用の測定結果のモデル(View の実体を参照)
    MultiGraphModel *_multiGraphModel;          //!< グラフ用の測定結果のモデル(View の実体を参照)
    QList<MeasureResult> _multiMeasureResult;   //!< マルチチャンネル測定結果(リスト)
    int _totalPage;                             //!< リストのページ数
    int _currentPage;                           //!< リストの現在のページ
    QVector<LC70_LOGGER_SET_LOGGER_ROW_T> _userSettings;   //!< ユーザー設定の設定値
    bool _isChannelChanging;            //!< チャンネル変更中
    bool _fromUserSetting;                      //!< ユーザー設定からの遷移
    int _previousBandId;                        //!< 前回表示したチャンネルの BAND
    int _barType = 0;                           //!< 0: 色 1 のバー 1: 色 2 のバー
    uint16_t _broadBandSystem;                  //!< 放送方式 閾値の更新に使用
    uint16_t _modulation;                       //!< 変調方式 閾値の更新に使用
    int _bandId;                                //!< Band の ID
    bool _alreadySuspended;                     //!< 既に一時停止しているか否か(マルチ測定に遷移する場合に true にする)
    uint16_t _measuringCondition;               //!< 測定結果ファイル出力時の測定項目
    QTimer *_confirmeChangeChannelTimer;        //!< チャンネル確定遅延タイマー
    QVariant _cnrMerSetting;                    //!< CNR/MER 切替設定
    QVariant _broadBandModulationSetting;       //!< マニュアル測定 放送方式, 変調方式選択
    QList<MeasureResult> _holdMeasurementData;  //!< 保存する測定データ (DATAボタン押下時の測定データ)
    int _previousSelectedIndex;                 //!< 保存処理前に選択していた機能
    DataSave2State _dataSave2State;             //!< 測定データ保存の動作状態
    QList<ChannelDefinitionItem> _channelListForGraph;  //!< グラフ用のチャンネル定義情報のリスト
    uint16_t _failureCode;                      //!< 故障コード
    uint8_t _actionInFailure;                   //!< 故障時の動作
    bool _autoScroll;                           //!< リストの自動スクロール有無
    QTimer *_lockWaitingTimer;                  //!< ロック待ちのタイマー
    bool _waitforLockingTimeOut;                //!< ロック待ち時間満了か否か
    bool _confirmedValue;                       //!< 確定した値か否か(ロックの取得あるいは待ち時間満了で確定とする)
    int _bkMenuBarType;							//!< MENU呼び出しか否か
    unsigned char _currentMeasuringIndex;       //!< 現在測定中のインデックス
    bool _startSaveData;                        //!< SAVE2 開始

    /**
     * @brief 選択ボタンの移動
     * @param buttonId ボタン ID
     */
    void moveCursor(ButtonId buttonId);

    /**
     * @brief チャンネル変更
     * @param isLaunched 画面ロード直後は true で changeChannel() 呼び出し後に false
     * @param noBandIconChange Band アイコンの変更有無(上下ボタンによる変更時は Band アイコンを変更しない)
     * @note CHANNEL_UP_BUTTON, CHANNEL_DOWN_BUTTONの押下時は、連打対策のためtimerのtimeoutを受けた時点でチャンネル変更を確定する
     */
    void changeChannel(bool isLaunched = false, bool noBandIconChange = true);

    /**
     * @brief マルチ測定条件の設定
     * @param displayGraph グラフ表示
     */
    void setMultiMeasuringCondition(bool displayGraph = true);

    /**
     * @brief ユーザ設定によるマルチ測定条件の設定
     * @param displayGraph グラフ表示中
     */
    void setUserMultiMeasuringCondition(bool displayGraph = true);

    /**
     * @brief 優先測定条件の設定
     */
    void setPreferredMeasuringCondition(void);

    /**
     * @brief ユーザ設定による優先測定条件の設定
     */
    void setUserPreferredMeasuringCondition(void);

    /**
     * @brief 表示クリアー
     */
    void clear(void);

    /**
     * @brief Band ID と偏波の組み合わせにより決まるチャンネルのカテゴリーのインデックスを返す
     * @param bandId Band ID
     * @param polarization 偏波の種類
     * @return Band ID と偏波の組み合わせにより決まるチャンネルのカテゴリーのインデックス
     * @note MultiMeasurementGraph.qml 参照
     */
    uint32_t selectXAsixCategory(int bandId, QString polarization);

    /**
     * @brief チャンネル変更確定タイマーを開始する
     */
    void startConfirmedChangeChannelTimer(void);

    /**
     * @brief プロパティに Bool 値を設定
     * @param objectName オブジェクト名
     * @param propertyName プロパティ名
     * @param value 設定する Bool 値
     */
    void setBoolProperty(QString objectName, QString propertyName, bool value);

    /**
     * @brief グラフ表示用モデルの変更
     */
    void changeMultiGraphModel(void);

    /**
     * @brief リスト表示用モデルの変更
     */
    void changeMultiListModel(void);

    /**
     * @brief                   グラフのx座標に表示するチャンネルのインデックスを取得
     * @param graphMode         グラフモード
     * @param allChannelList    グラフに表示する全てのチャンネルリスト
     * @return x座標に表示するチャンネル名リスト
     */
    QList<QString> displayChannelList(int graphMode, QList<QString> allChannelList);

    /**
     * @brief 局発周波数の値を取得
     * @return 局発周波数の値
     */
    unsigned int getLocalFrequency(void);

    /**
     * @brief 変調方式の値を取得
     * @return 変調方式の値
     */
    unsigned int getModulationValue(void);

    /**
     * @brief スキャン結果後の動作選択
     * @param index 選択結果
     */
    void selectReactionScanResult(const int index);

    /**
     * @brief スキャン後の測定開始処理
     */
    void startMeasuringAfterScan(void);

    /**
     * @brief MER/CNRの範囲更新チェック
     * @param bandId      バンドID
     * @param bdSys       放送方式
     * @param modulation  変調方式
     * @param forceUpdate 強制的に更新するかどうか
     */
    void checkMerCnrRange(int bandId, int bdSys, int modulation, bool forceUpdate);

    /**
     * @brief MER/CNRの範囲を更新する(最大値/最小値の更新)
     * @param bandId 測定するBAND (LC70_BAND_xxxの値)
     * @param broadBandSystem 放送方式 (LC70_BD_SYS_xxxの値)
     * @param modulation 変調方式 (LC70_SETMOD_xxxxの値)
     */
    void updateMerCnrRange(const int bandId, const int BroadbandSystem, const int modulation);

    /**
     * @brief 関数実行時の測定データを保持する
     * @note DATAボタンが押下されたときに使用する
     */
    void holdMeasurementData(void);

    /**
     * @brief グラフの背景画像を設定する
     * @param bandId Band Id
     * @param polarization 偏波
     */
    void setGraphBackground(int bandId, QString polarization);

    /**
     * @brief 測定データの保存
     */
    void saveMeasuringMData(void);

    /**
     * @brief 測定データの取得
     */
    void acquireMeasuringData(void);

    /**
     * @brief 選択機能に対応するボタンをアクティブにする
     * @param buttonIndex 選択機能のボタンのインデックス
     */
    void changeButton(int buttonIndex);

    /**
     * @brief 測定に戻す
     */
    void returnToMeasurement(void);

    /**
     * @brief 全チャンネルを再取得する
     */
    void refreshAllChannels(void);

    /**
     * @brief 測定結果のインデックスからグラフ表示用のインデックスを得る
     * @param originalIndex 測定したインデックス
     * @param replacedIndex[out] グラフに表示するインデックス
     */
    void selectGraphIndex(const int &originalIndex, int &replacedIndex);

    /**
     * @brief USER 設定ファイルのプリセットから測定するチャンネルを取得
     * @param values USER 設定ファイルの内容
     * @param band Band Id
     * @note FM, CATV 上りのみ
     */
    void setChannelsFromPreset(QVariant values, const LC70_BAND_T &band);

    /**
     * @brief USER 設定ファイルから測定するチャンネルを取得
     * @param values USER 設定ファイルの内容
     * @param band Band Id
     * @note FM, CATV 上り以外の Band
     */
    void setChannelsFromSetting(QVariant values, const LC70_BAND_T &band);

    /**
     * @brief USER 設定ファイルから測定するチャンネルを取得
     * @param values USER 設定ファイルの内容
     * @note USER 設定で複数の Band を跨る場合に使用
     */
    void setChannelsFromSetting(QVariant values);

    /**
     * @brief USER 設定ファイルの内容を保持
     * @param item USER 設定ファイル中の 1 チャンネルの定義
     * @param channelDefinitionItem チャンネル定義
     */
    void setUserSettings(const QVariantMap &item, const ChannelDefinitionItem &channelDefinitionItem);

    /**
     * @brief 設定ファイルに登録したプリセットのチャンネルか否かを返す
     * @param frequency チャンネル名に入った周波数
     * @param values USER 設定ファイルの内容
     * @return 登録したプリセットのチャンネルか否か
     * @note 不使用
     */
    bool isValidPresetChannel(const unsigned int &frequency, const QVariant &values);

    /**
     * @brief 給電切り替え
     */
    void switchPowerSupply(void);

    /**
     * @brief 複数 Band 向け給電切り替え
     */
    void switchPowerSupplyForMultiBand(void);

    /**
     * @brief 優先測定の表示
     * @param index チャンネルのインデックス
     */
    void displayPrefferedChannel(int index);

    /**
     * @brief CATV 上りと FM のプリセットのチャンネル定義で不足しているチャンネルを補う
     * @param band Band
     * @note CATV 上りと FM のみ
     */
    void complementChannelDefinition(const LC70_BAND_T band);
};

#endif // MultiMeasurementVIEWMODEL_H
