/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include "polarizationviewmodel.h"
#include "mainframeviewmodel.h"
#include "correcter.h"

QVector<QVector<QString>> polarizationButtonData = {
    { {"m1"}, {"m2"} }
};
const int PolarizationViewModel::BUTTON_COLUMN_NUMBER = 1;
const int PolarizationViewModel::BUTTON_ROW_NUMBER    = 2;

static constexpr int UNIQUE_LOCAL_FREQUENCY = 1;
static constexpr int SCREEN_INDEX_LOCAL_OSCILLATOR_SETTING = 0;
static constexpr int SCREEN_INDEX_SINGLE_MEASUREMENT = 2;
static constexpr int SCREEN_INDEX_MULTI_MEASUREMENT = 3;

PolarizationViewModel::PolarizationViewModel(ScreenId screenId, LevelCheckerModel *&model, QObject *parent)
    : AbstractViewModel(screenId, model, parent)
    , _fromAutoSetting(false)
    , _selectedMeasurement()
    , _channelList()
    , _bandId()
    , _broadBandModulationSetting()
    , _powerSupply(LC70_POWER_SUPPLY_OFF)
    , _failureCode(0)
    , _actionInFailure(LC70_FAIL_CTL_NONE)
{
    _buttonOperation.setButtonName(BUTTON_COLUMN_NUMBER, BUTTON_ROW_NUMBER, polarizationButtonData);

    _activeButton = polarizationButtonData[0][0];
    _buttonOperation.setActiveObject(_activeButton);
}

PolarizationViewModel::~PolarizationViewModel()
{

}

void PolarizationViewModel::setTakeOver()
{
    QString polarization = selectedPolarization();

    if (polarization.isEmpty()) {
        return;
    }

    emit requestSetTakeOver("selectedPolarization", polarization);
}

QString PolarizationViewModel::selectedPolarization()
{
    static const QMap<QString, int> indexConvertMap  = {
        {"m1", INDEX_RIGHT_HAND_HORIZONTAL},
        {"m2", INDEX_LEFT_HAND_VERTICAL },
    };

    if (indexConvertMap.count(_activeButton) == 0) {
        return "";
    }

    return Correcter::polarization(_bandId, indexConvertMap.value(_activeButton));
}

void PolarizationViewModel::activate(void)
{
    connect(this, SIGNAL(requestSetTakeOver(QVariant, QVariant)), _view, SLOT(backup(QVariant, QVariant)));
    connect(this, SIGNAL(requestGetTakeOver()), _view, SLOT(getTakeOver()));
    connect(_mainFrame, SIGNAL(requestToScan()), this, SLOT(onRequestToScan()));
    connect(_mainFrame, SIGNAL(requestToStopScanning()), this, SLOT(onRequestToStopScanning()));
    connect(_mainFrame, SIGNAL(notifyPowerSupplyError(uint16_t, uint8_t, QString)), this, SLOT(detectPowerSupplyError(uint16_t, uint8_t, QString)));
    connect(_model, SIGNAL(scanningFinished(bool)), this, SLOT(onScanningFinished(bool)));
    connect(_model, SIGNAL(scanningChannelResult(int, LC70_MEAS_MULTI_RESULT_T)), this, SLOT(onScanningChannelResult(int,
            LC70_MEAS_MULTI_RESULT_T)));
    connect(this, SIGNAL(progress(double)), _mainFrame, SIGNAL(progress(double)));
    connect(this, SIGNAL(requestToReScan()), this, SLOT(onRequestToReScan()), Qt::QueuedConnection);
    connect(this, SIGNAL(requestToShowRegistScanResultPopup()), this, SLOT(onRequestToShowRegistScanResultPopup()), Qt::QueuedConnection);

    emit requestGetTakeOver();

    // マニュアル測定 放送方式, 変調方式選択の取得
    _broadBandModulationSetting.clear();
    _model->getValue(VALUE_BROADBANDSYSTEM_MODULATION, _broadBandModulationSetting);

    QObject *target = nullptr;
    if (_view) {
        target = _view->findChild<QObject *>("takeOverData");
    }

    if (target) {
        QVariant value = target->property("selectedMeasurement");
        _selectedMeasurement = value.toString();

        value = target->property("selectedBand");
        bool  ok = false;

        int band = value.toInt(&ok);

        if (!ok) {
            return;
        }

        _bandId = band;
    }

    _fromAutoSetting = _takeOver["autoSettingPolarization"].toBool();

    // 先頭のボタンをアクティブにする
    resetToInitialButtonState(polarizationButtonData[0][0], _activeButton, _buttonOperation);

    setButtonMessage();
}

void PolarizationViewModel::deactivate()
{
    AbstractViewModel::deactivate();

    disconnect(this, SIGNAL(requestSetTakeOver(QVariant, QVariant)), _view, SLOT(backup(QVariant, QVariant)));
    disconnect(this, SIGNAL(requestGetTakeOver()), _view, SLOT(getTakeOver()));
    disconnect(_mainFrame, SIGNAL(requestToScan()), this, SLOT(onRequestToScan()));
    disconnect(_mainFrame, SIGNAL(requestToStopScanning()), this, SLOT(onRequestToStopScanning()));
    disconnect(_mainFrame, SIGNAL(notifyPowerSupplyError(uint16_t, uint8_t, QString)), this, SLOT(detectPowerSupplyError(uint16_t, uint8_t, QString)));
    disconnect(_model, SIGNAL(scanningFinished(bool)), this, SLOT(onScanningFinished(bool)));
    disconnect(_model, SIGNAL(scanningChannelResult(int, LC70_MEAS_MULTI_RESULT_T)), this, SLOT(onScanningChannelResult(int,
               LC70_MEAS_MULTI_RESULT_T)));
    disconnect(this, SIGNAL(progress(double)), _mainFrame, SIGNAL(progress(double)));
    disconnect(this, SIGNAL(requestToReScan()), this, SLOT(onRequestToReScan()));
    disconnect(this, SIGNAL(requestToShowRegistScanResultPopup()), this, SLOT(onRequestToShowRegistScanResultPopup()));
}

void PolarizationViewModel::suspend()
{
    // 給電を停止
    _powerSupply = LC70_POWER_SUPPLY_OFF;
    _model->setPowerSupply(_powerSupply);
    _model->removeMenuList(MenuListIconIndex::POWER_SUPPLY_ICON);

    _failureCode = 0;

    AbstractViewModel::suspend();
}

void PolarizationViewModel::moveCursor(ButtonId buttonId)
{
    if ((buttonId == BUTTON_RIGHT) || (buttonId == BUTTON_LEFT)) {

        QString nextButton;
        bool ret = _buttonOperation.nextActiveObject(buttonId, nextButton);

        if (ret) {

            // inactive
            QObject *target = nullptr;
            if (_view) {
                target = _view->findChild<QObject *>(_activeButton);
            }

            if (target) {
                target->setProperty("active", false);
            }

            // active
            target = nullptr;
            if (_view) {
                target = _view->findChild<QObject *>(nextButton);
            }
            if (target) {
                target->setProperty("active", true);
            }

            _activeButton = nextButton;
            _buttonOperation.setActiveObject(_activeButton);
        }
    }
}

void PolarizationViewModel::onKeyStatus(ButtonId id)
{
    // Home, Escape ボタンはベースクラスで対応
    AbstractViewModel::onKeyStatus(id);

    if (id == BUTTON_ENTER) {
        enterKey();
    } else if (id == BUTTON_USER) {
#ifdef NEW_USER_SETTING
        emit requestDirect(SCREEN_USER_SETTING, true);
#else
        emit requestDirect(SCREEN_USER_DATA_LOGGER_MENU, true);
#endif
    } else {
        moveCursor(id);
    }
}

void PolarizationViewModel::onButtonPress(ButtonId id)
{
    if (id == BUTTON_ENTER) {
        QObject *target = nullptr;
        if (_view) {
            target = _view->findChild<QObject *>(_activeButton);
        }
        if (target) {
            target->setProperty("activebg", "combtn_p.png");
        }
    }
}

void PolarizationViewModel::enterKey()
{
    if (_fromAutoSetting) {
        // 選択したBANDによって処理は変化する
        // BS, CSの場合は、給電確認ダイアログを表示
        // JCSAT3A, JCSAT4B, SCC-B2, SCC-C2の場合は、局発周波数選択画面へ遷移する
        if (_bandId == LC70_BAND_BS || _bandId == LC70_BAND_CS) {
            if (isLowBattery()) {
                // バッテリ低下状態中は給電開始させない
                onRequestScanConfirm();
                return;
            }
            emit requestToShowPopup(POPUP_POWER_SUPPLY_SELECT2);
        } else if (_bandId == LC70_BAND_JCSAT3 || _bandId == LC70_BAND_JCSAT4
                   || _bandId == LC70_BAND_SCCB || _bandId == LC70_BAND_SCCC){
            // 画面遷移
            int requestIndex = nextScreenIndex();
            onRequest(requestIndex);;
        } else {
            // nothing to do
        }
    } else {
        // 画面遷移
        int requestIndex = nextScreenIndex();

        onRequest(requestIndex);
    }
}

int PolarizationViewModel::nextScreenIndex()
{
    // 局発周波数の選択肢が単一である場合、わざわざ選択させない
    QVariant data;

    if (_model->getValue(VALUE_LOCAL_FREQUENCY, data)) {
        QList<uint32_t> localFreqList;

        if (Correcter::getLocalOscillatorList(data, _bandId, selectedPolarization(), localFreqList)) {
            if (localFreqList.size() == UNIQUE_LOCAL_FREQUENCY) {
                // 局発周波数が単一
                emit requestSetTakeOver("selectedLocalOscillator", localFreqList.first());

                QString measurementType = _takeOver["selectedMeasurement"].toString();

                if (measurementType == SINGLE_MEASURE_TYPE_NAME) {
                    return SCREEN_INDEX_SINGLE_MEASUREMENT;
                } else {
                    return SCREEN_INDEX_MULTI_MEASUREMENT;
                }
            }
        }
    }

    return SCREEN_INDEX_LOCAL_OSCILLATOR_SETTING;
}

void PolarizationViewModel::onReject(PopupId popupId)
{
    if (popupId == POPUP_POWER_SUPPLY_SELECT2) {
        QTimer::singleShot(100, this, SLOT(onRequestScanConfirm()));
    }

    // ボタン押下の状態を元に戻す
    QObject *target = nullptr;
    if (_view) {
        target = _view->findChild<QObject *>(_activeButton);
    }

    if (target) {
        if (_activeButton == "m1") {
            target->setProperty("activebg", "modulationbtn_r.png");
        } else if (_activeButton == "m2") {
            target->setProperty("activebg", "modulationbtn_l.png");
        }
    }

    // 給電中だった場合は給電OFFにする
    if (_powerSupply != LC70_POWER_SUPPLY_OFF) {
        _powerSupply = LC70_POWER_SUPPLY_OFF;
        _model->setPowerSupply(_powerSupply);
        _model->replaceMenuList(MenuListIconIndex::POWER_SUPPLY_ICON, Correcter::powerSupplyIcon(_powerSupply));
    }

    // スキャン結果のアイコンを消去
    _model->removeMenuList(MenuListIconIndex::SCAN_ICON);

}

void PolarizationViewModel::onAccept(PopupId popupId, bool okOnly)
{
    Q_UNUSED(okOnly);

    AbstractViewModel::onAccept(popupId, okOnly);

    if (popupId == POPUP_POWER_SUPPLY_SELECT2) {
        // 給電選択で 15v を選択
        _powerSupply = LC70_POWER_SUPPLY_15V;
        _model->setPowerSupply(_powerSupply);
        _model->replaceMenuList(MenuListIconIndex::POWER_SUPPLY_ICON, Correcter::powerSupplyIcon(_powerSupply));

        // スキャン開始
        QTimer::singleShot(100, this, SLOT(onRequestScanConfirm()));
    } else if (popupId == POPUP_POWER_SUPPLY_ERROR_MANUAL_MEASURING) {
        // nothing to do.
    } else if (popupId == POPUP_CONFIRM_REGIST_SCAN_RESULT) {
        // 保存先選択画面へ遷移
#ifdef NEW_USER_SETTING
        // 保存先選択で USER データロガー設定を呼び出す
        emit requestSetTakeOver("userDataloggerFunction", CHANNEL_EDIT_SELECT_DESTINATION);
        emit requestDirect(SCREEN_USER_DATALOGGER_SETTING, false);
#else
        onRequest(1);
#endif
    } else {
        // nothing to do
    }
}

void PolarizationViewModel::onSelect(PopupId popupId, int index)
{
    if (popupId == POPUP_SCAN_RESULT) {
        // スキャン結果を採用する
        selectReactionScanResult(index);
        return;
    }
}

void PolarizationViewModel::onLowBatteryStatusChanged(bool isLowBattery)
{
    if (isLowBattery && _powerSupply != LC70_POWER_SUPPLY_OFF) {
        // 給電を停止する
        _powerSupply = LC70_POWER_SUPPLY_OFF;
        _model->setPowerSupply(_powerSupply);
        _model->removeMenuList(MenuListIconIndex::POWER_SUPPLY_ICON);
        emit requestToShowLowBatteryPopup();
    }
}

void PolarizationViewModel::onScanningChannelResult(int index, LC70_MEAS_MULTI_RESULT_T result)
{
    // ラッパー関数ライブラリ V2.00 対応
    if (result.ucLock == UNLOCKED) {
        // 無効判定
        _channelList[index].valid = false;
    }

    // スキャン結果の保持
    if (result.ucLock == LOCKED) {
        MeasureResult scanResult;
        scanResult.Received = true;
        scanResult.Band = QString::number(_channelList[index].band);
        scanResult.Channel = _channelList[index].channelNumber;
        scanResult.Status = result.usStat;
        scanResult.Level = result.ulLevel;
        scanResult.Mer = 0;
        scanResult.Ber = 0;
        scanResult.BroadcastSystem = result.usBdSys;
        scanResult.Modulation = result.ulModBitPtrn;
        scanResult.Layer = result.ucLayer;
        scanResult.Frequency = -1;
        scanResult.LocalOscillator = 0;
        _model->appendScanResult(scanResult);

        // スキャン結果で有効チャンネルが見つかった場合 ブザーを1度鳴動させる
        _model->buzzer(LC70_BUZZ_VOL_CHG, LC70_BUZZ_MODE_ONESHOT);
    }

    // プログレスバーの進捗を設定
    double value = static_cast<double>(index + 1) / static_cast<double>(_channelList.count()) * 100;
    emit progress(value);
}

void PolarizationViewModel::onRequestScanConfirm(void)
{
    emit requestToShowPopup(POPUP_SCAN_CONFIRMATION);
}

void PolarizationViewModel::onRequest(int condition)
{
    emit request(_screenId, condition);
    return;
}

void PolarizationViewModel::setButtonMessage()
{
    if ((_bandId == LC70_BAND_BS) || (_bandId == LC70_BAND_CS)) {
        // 右旋、左旋
        QObject *target = nullptr;
        if (_view) {
            target = _view->findChild<QObject *>("m1");
        }
        if (target) {
            target->setProperty("message", Correcter::displayPolarization(RIGHT_HANDED_POLARIZE));
        }

        target = nullptr;
        if (_view) {
            target = _view->findChild<QObject *>("m2");
        }
        if (target) {
            target->setProperty("message", Correcter::displayPolarization(LEFT_HANDED_POLARIZE));
        }
    } else {
        // 水平、垂直
        QObject *target = nullptr;
        if (_view) {
            target = _view->findChild<QObject *>("m1");
        }
        if (target) {
            target->setProperty("message", Correcter::displayPolarization(HORIZONTALLY_POLARIZE));
        }

        target = nullptr;
        if (_view) {
            target = _view->findChild<QObject *>("m2");
        }
        if (target) {
            target->setProperty("message", Correcter::displayPolarization(VERTICALLY_POLARIZE));
        }
    }
}

void PolarizationViewModel::onRequestToScan()
{
    // 指定BANDのチャンネルデータを取得
    _model->getChannelDefinitionList(_bandId, _channelList);

    // 保持したスキャン結果をクリアー
    _model->clearScanResult();

    // 局発周波数
    QString polarization = selectedPolarization();

    QVariant value;
    _model->getValue(VALUE_LOCAL_FREQUENCY, value);
    QList<uint32_t> localOscillatorList;
    Correcter::getLocalOscillatorList(value, _bandId, polarization, localOscillatorList);
    if (localOscillatorList.isEmpty()) {
        return;
    }
    uint32_t localFrequency = localOscillatorList.first();

    // 変調方式
    // 偏波面選択画面でSCANを行うチャンネルの変調方式は"定義なし"の固定値
    unsigned int modulationValue = LC70_SETMOD_UNDEF;
    QString modulation = Correcter::displayModulation(modulationValue);
    uint16_t bdSys = Correcter::getBroadcastSystemCondition(_bandId, modulation);

    // 放送方式、変調方式の設定(定義ファイル 1.0.1 対応)
    uint16_t mod = Correcter::getModulationCondition(_bandId, bdSys, Correcter::convertPolarizationToWave(polarization), modulationValue, _broadBandModulationSetting);

    // スキャン実行を通知
    _model->writeLog(QString("USER・データロガー設定  自動  偏波面選択画面"),
                     QString("スキャン開始"));

    _model->scan(_channelList, AcquisitionTarget::NOT_ACQUISITION, localFrequency, bdSys, mod, false, _powerSupply);
}

void PolarizationViewModel::onRequestToStopScanning()
{
    // スキャン中止
    _model->writeLog(QString("USER・データロガー設定  自動  偏波面選択画面"),
                     QString("スキャン中止"));
    _model->stopChannelScan();
}

void PolarizationViewModel::onScanningFinished(bool complete)
{
    _model->writeLog(QString("USER・データロガー設定  自動  偏波面選択画面"),
                     QString("スキャン終了"));

    // 給電中だった場合は、一旦給電OFFにする
    if (_powerSupply != LC70_POWER_SUPPLY_OFF) {
        // 再スキャン時の給電有無を判断するため、_powerSupplyは更新しない
        _model->setPowerSupply(LC70_POWER_SUPPLY_OFF);
        _model->replaceMenuList(MenuListIconIndex::POWER_SUPPLY_ICON, Correcter::powerSupplyIcon(LC70_POWER_SUPPLY_OFF));
    }

    if (complete) {
        // スキャン完了時 ブザーを1度鳴動させる
        _model->buzzer(LC70_BUZZ_ALL_CH_SCAN, LC70_BUZZ_MODE_ONESHOT);

        emit requestToShowPopup(POPUP_SCAN_RESULT);
    } else {
        _model->removeMenuList(MenuListIconIndex::SCAN_ICON);
#ifdef NEW_USER_SETTING
        emit requestDirect(SCREEN_USER_SETTING, true);
#else
        emit requestDirect(SCREEN_USER_DATA_LOGGER_MENU, true);
#endif
    }
}

void PolarizationViewModel::selectReactionScanResult(const int index) {
    if (index == 0) {
        // スキャンアイコン表示
        _model->replaceMenuList(MenuListIconIndex::SCAN_ICON, "icon_scan_n.png");
        emit requestToShowRegistScanResultPopup();
    } else {
        // 再スキャン
        emit requestToReScan();
    }
}

void PolarizationViewModel::onRequestToReScan() {
    // スキャン中ポップアップを表示
    emit requestToShowPopup(POPUP_SCANNING);

    // スキャン開始時に給電が選択されていた場合は、再スキャン中も給電する
    if (_powerSupply != LC70_POWER_SUPPLY_OFF) {
        _model->setPowerSupply(_powerSupply);
        _model->replaceMenuList(MenuListIconIndex::POWER_SUPPLY_ICON, Correcter::powerSupplyIcon(_powerSupply));
    }

    // 再スキャン
    onRequestToScan();
}

void PolarizationViewModel::detectPowerSupplyError(uint16_t code, uint8_t action, QString message)
{
    _failureCode = code;
    _actionInFailure = action;
    // 給電を停止する
    _failureCode = code;
    _powerSupply = LC70_POWER_SUPPLY_OFF;
    _model->setPowerSupply(_powerSupply);
    _model->removeMenuList(MenuListIconIndex::POWER_SUPPLY_ICON);

    if (_actionInFailure == LC70_FAIL_CTL_MSGDISP_CLR) {
        uint8_t addInfo[LC70_FAIL_ADD_INFO_NUM];
        memset(addInfo, 0x00, LC70_FAIL_ADD_INFO_NUM);
        //_model->setFaultInformation(_failureCode, LC70_FAIL_STAT_NORMAL, addInfo);     故障復旧処理不要
        _failureCode = 0;
        _actionInFailure = LC70_FAIL_CTL_NONE;
    }

    _model->buzzer(LC70_BUZZ_ERR, LC70_BUZZ_MODE_ONESHOT, true);

    emit requestToShowPopupWithMessage(POPUP_POWER_SUPPLY_ERROR_MANUAL_MEASURING, message);
}

void PolarizationViewModel::onRequestToShowRegistScanResultPopup() {
    emit requestToShowPopup(POPUP_CONFIRM_REGIST_SCAN_RESULT);
}
