/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include <QDateTime>
#include "singlemeasurementviewmodel.h"
#include "mainframeviewmodel.h"

#include <QDebug>
#include <QElapsedTimer>

static constexpr float32_t LEVEL_NEEDLE_OFFSET = -128.0;            //!< 測定入力レベルの針の位置のオフセット
static constexpr float32_t LEVEL_NEEDLE_STEP = 2.51;                //!< 測定入力レベルの針の移動量
static constexpr float32_t BER_NEEDLE_STEP = -3.0;                  //!< ビット誤り率の針の移動量

static constexpr float32_t LEVEL_METER_MAXMUM_POSITION = 122.0;     //!< レベルメータの最小値
static constexpr float32_t LEVEL_METER_MINMUM_POSITION = -128.0;    //!< レベルメータの最大値

#if 1	//210628_dx2
static constexpr float32_t BER_METER_MINIMUM_POSITION = 2.2e-2;     //!< ビット誤り率メータの最小値
#else
static constexpr float32_t BER_METER_MINIMUM_POSITION = 1.0e-2;     //!< ビット誤り率メータの最小値
#endif

static constexpr int32_t INITIAL_LEVEL_NEEDLE_POSITION = -128;      //!< 測定入力レベルの針の位置の初期値
static constexpr uint32_t INITIAL_MER_NEEDLE_POSITION = 0;          //!< 変調誤差比の針の位置の初期値
static constexpr uint32_t INITIAL_BER_NEEDLE_POSITION = 2;          //!< ビット誤り率の針の位置の初期値

static constexpr int32_t DELAY_PROFILE_ZERO_INDEX = 512;            //!< 遅延プロファイル 0μs のインデックス
static constexpr int32_t DELAY_PROFILE_XAXIS_LENGTH = 509;          //!< 遅延プロファイルの X 軸の長さ(DelayProfile.qml 参照)
static constexpr const char *DELAY_TIME_INITIAL_VALUE = "0.0";      //!< 遅延時間の初期値
static constexpr const char *LEVEL_DEFFERENCE_INITIAL_VALUE = "0";  //!< レベル差の初期値

static constexpr const char *UNLOCK_STRING = "UNLOCK";              //!< アンロック時の放送方式、変調方式、階層の表示文字列

static QElapsedTimer timer;
static QElapsedTimer measuringTimer;

// ボタン名
static const QString BUTTON_NAME_SINGLE = "singleButton";
static const QString BUTTON_NAME_CONSTELLATION = "constellationButton";
static const QString BUTTON_NAME_DELAY_PROFILE = "delayProfileButton";

QVector<QVector<QString>> SINGLE_MEASUREMENT_BUTTON_DATA = {
    {{BUTTON_NAME_SINGLE}, {BUTTON_NAME_CONSTELLATION}, {BUTTON_NAME_DELAY_PROFILE}},
    {{BUTTON_NAME_SINGLE}, {BUTTON_NAME_CONSTELLATION}},
    {{BUTTON_NAME_SINGLE}},
};

QVector<QVector<QString>> SINGLE_MEASUREMENT_BUTTON_DATA_LIMITED = {
    {{BUTTON_NAME_SINGLE}, {BUTTON_NAME_CONSTELLATION}},
    {{BUTTON_NAME_SINGLE}},
};

static constexpr int BUTTON_COLUMN_NUMBER =
        1;                  //!< 測定、コンスタレーション、遅延プロファイルの列
static constexpr int BUTTON_ROW_NUMBER_NORMAL =
        3;                     //!< 測定、コンスタレーション、遅延プロファイルの行
static constexpr int BUTTON_ROW_NUMBER_LIMITED1 =
        2;                     //!< 測定、コンスタレーション、遅延プロファイルの行
static constexpr int BUTTON_ROW_NUMBER_LIMITED2 =
        1;                     //!< 測定、コンスタレーション、遅延プロファイルの行

static constexpr const char *IMAGE_BUTTON_NAME =
        "imageButton"; //!< 画像保存ボタン名(SingleMeasurement.qml 参照)
static constexpr const char *DATA_BUTTON_NAME =
        "dataButton";   //!< データ保存ボタン名(SingleMeasurement.qml 参照)

static constexpr const char *CSV_FILE_EXTENSION = ".csv";       //!< 測定データファイルの拡張子(CSV)
static constexpr const char *PNG_FILE_EXTENSION = ".png";       //!< 測定画面の拡張子(PNG)

static constexpr int MEASUREMENT_BUTTON_INDEX = 0;              //!< 測定ボタンのインデックス
static constexpr int CONSTELLATION_BUTTON_INDEX =
        1;            //!< コンスタレーションボタンのインデックス
static constexpr int DELAY_PROFILE_BUTTON_INDEX =
        2;            //!< 遅延プロファイルボタンのインデックス

static const double BER_METER_STEP =
        0.1666666666666667;    //!< 指数が 1 増えるときのBER のメーターの割合

static constexpr int SCAN_INDEX = 2;                //!< SCAN を表示するメニューリストのインデックス
static constexpr int POWER_SUPPLY_INDEX =
        3;        //!< 給電 を表示するメニューリストのインデックス

static const float32_t BER_METER_INVALID_VALUE = 1.0e-2;        //!< BER メーターで無効値時に使用する値

static const int MER_STEP_VALUE = 5;  //!< MERメータの１ステップあたりの変化量

SingleMeasurementViewModel::SingleMeasurementViewModel(ScreenId screenId, LevelCheckerModel *&model, QObject *parent)
    : AbstractMeasurementViewModel(screenId, model, parent)
    , _buttonOperation(new ButtonOperation(this))
    , _activeButton()
    , _meter(nullptr)
    , _constellation(nullptr)
    , _delayProfile(nullptr)
    , _captureFileName()
    , _level(0.0f)
    , _mer(0.0f)
    , _ber(BER_METER_MINIMUM_POSITION)
    , _limitContellationPoints(LC70_CONSTA_1000)
    , _selectedButtonIndex(MEASUREMENT_BUTTON_INDEX)
    , _buttonPattern(NormalPattern)
    , _preservedChannelList()
    , _condition()
    , _previousCondition()
    , _modulation(LC70_SETMOD_UNDEF)
    , _layer(0)
    , _timeStamp()
    , _saveFileType(SAVE_FILE_COUNT)
    , _lineSeries(nullptr)
    , _isConstellationDataClear(false)
    , _userSettings()
    , _isChannelChanging(false)
    , _fromUserSetting(false)
    , _thresholdSet(false)
    , _broadcastSystem(LC70_BD_SYS_NONE)
    , _measurementButtons()
    , _alreadySuspended(false)
    , _confirmedChangeChannelTimer(new QTimer())
    , _levelReceivableThreshold(0.0)
    , _mercnMerginHighThreshold(0.0)
    , _berMarginHighThreshold(0.0)
    , _isDelayProfileValid(false)
    , _delayTimeIndex(0)
    , _cursoredGraph(false)
    , _merRange()
    , _cnrMerSetting()
    , _broadBandModulationSetting()
    , _newestLevel(UNDEFINED_VALUE)
    , _newestMer(UNDEFINED_VALUE)
    , _newestBer(UNDEFINED_VALUE)
    , _isUpdateMeter(false)
    , _meterUpdateInterval(0)
    , _invalidValueMeterSpeed(0)
    , _levelFixTimer(new QTimer())
    , _merFixTimer(new QTimer())
    , _berFixTimer(new QTimer())
    , _displayLevel(false)
    , _displayMer(false)
    , _displayBer(false)
    , _holdMeasurementData()
    , _combobox(nullptr)
    , _iscomboboxVisible(false)
    , _comboboxState(Normal)
    , _modulationInMeasureResult(LC70_SETMOD_UNDEF)
    , _pressedChannelButtonId(BUTTON_UNKNOWN)
    , _buzzerState(LC70_BUZZ_OFF)
    , _failureCode(0)
    , _actionInFailure(LC70_FAIL_CTL_NONE)
    , _powerSupplyErrorOccured(false)
    , _lockWaitingTimer(new QTimer())
    , _waitforLockingTimeOut(false)
    , _confirmedValue(false)
    , _locked(UNLOCKED)
{
    setObjectName("singleMeasurementViewModel");

    // Level を表示するまでの待ち時間を設定
    _levelFixTimer->setInterval(_model->applicationSetting()->levelFixTime());
    _levelFixTimer->setSingleShot(true);
    connect(_levelFixTimer, &QTimer::timeout, [this]() {
        _displayLevel = true;
        _model->writeLog(QString("シングル測定"), QString("測定結果から Level の表示を更新するまでの待ち時間(%1 ms)経過").arg(_levelFixTimer->interval()), LOG_INFO);
    });

    // MER を表示するまでの待ち時間を設定
    _merFixTimer->setInterval(_model->applicationSetting()->merFixTime());
    _merFixTimer->setSingleShot(true);
    connect(_merFixTimer, &QTimer::timeout, [this]() {
        _displayMer = true;
        _model->writeLog(QString("シングル測定"), QString("測定結果から MER の表示を更新するまでの待ち時間(%1 ms)経過").arg(_merFixTimer->interval()), LOG_INFO);
    });

    // BER を表示するまでの待ち時間を設定
    _berFixTimer->setInterval(_model->applicationSetting()->berFixTime());
    _berFixTimer->setSingleShot(true);
    connect(_berFixTimer, &QTimer::timeout, [this]() {
        _displayBer = true;
        _model->writeLog(QString("シングル測定"), QString("測定結果から BER の表示を更新するまでの待ち時間(%1 ms)経過").arg(_berFixTimer->interval()), LOG_INFO);
    });

    // メーターのロック待ちの時間を設定
    _lockWaitingTimer->setInterval(_model->applicationSetting()->waitingTimeForMeter());
    _lockWaitingTimer->setSingleShot(true);
    connect(_lockWaitingTimer, &QTimer::timeout, [this]() {
        // ロック待ち時間切れ
        _waitforLockingTimeOut = true;
        _confirmedValue = true; // 値確定

        // 点滅を止める
        if (_view) {
            _view->setProperty("blinkInterval", 0);
            _view->setProperty("confirmedValue", _confirmedValue);
        }
    });
}

SingleMeasurementViewModel::~SingleMeasurementViewModel()
{

}

void SingleMeasurementViewModel::resume()
{
    qDebug() << QString("[Enter] SingleMeasurementViewModel::resume()");
    AbstractViewModel::resume();

    // チャンネル変更ボタンの長押し判定用変数をクリアしておく
    _pressedChannelButtonId = BUTTON_UNKNOWN;
    _confirmedChangeChannelTimer->setSingleShot(true);
    connect(_confirmedChangeChannelTimer, SIGNAL(timeout()), this, SLOT(onConfirmedChangeChannelTimeout()));

    // スレッドを開始(方位角)
    emit resumeSingleMeasuring(_limitContellationPoints, _condition,
                               _model->applicationSetting()->pollingIntervalSingleMeasurement());

    if (_channelList.isEmpty()) {
        return;
    }
    // [Memo]
    // 画面遷移時に実行する
    // CN/MERの選択状態をqmlに送信する
    int valueMerCN = Correcter::selectCnrMer(static_cast<LC70_BAND_T>(_channelList[_currentChannelIndex].band), _cnrMerSetting);

    // 得られた値が範囲外であった場合(0,1 以外の値であった場合)
    // 初期値のMERを選択する
    if (valueMerCN != LC70_MEAS_SET_SEL_CNR && valueMerCN != LC70_MEAS_SET_SEL_CNR) {
        valueMerCN = LC70_MEAS_SET_SEL_MER;
    }

    if (_view) {
        _view->setProperty("selectCnMer", valueMerCN);
    }
    _thresholdSet = false;

    // チャンネル変更(測定開始)
    changeChannel();

    qDebug() << QString("[Leave] SingleMeasurementViewModel::resume()");
}

void SingleMeasurementViewModel::suspend()
{
    qDebug() << QString("[Enter] SingleMeasurementViewModel::suspend()");

    disconnect(_confirmedChangeChannelTimer, SIGNAL(timeout()), this, SLOT(onConfirmedChangeChannelTimeout()));

    // ブザーが鳴動している場合はブザーを終了する
    stopBuzzer();
    _buzzerState = LC70_BUZZ_OFF;

    // 給電を停止
    _powerSupply = LC70_POWER_SUPPLY_OFF;
    _model->setPowerSupply(_powerSupply);
    _model->removeMenuList(MenuListIconIndex::POWER_SUPPLY_ICON);

    // メーターに戻しておく
    if (_view) {
        QObject *target = _view->findChild<QObject *>(_activeButton);
        if (target) {
            // 押下前の画像に更新
            target->setProperty("pressed", false);
            target->setProperty("active", false);
        }
        // ユーザー設定名を画面遷移時に戻しておく
        _view->setProperty("userMode", QString(""));
    }

    // シングル測定の状態に戻しておく
    resetSettings(MEASUREMENT_BUTTON_INDEX);
    changeSettingToSingleMeasurement(true);

    // マルチ測定に遷移する場合は既に一時停止してからマルチ測定で測定を開始しているため、測定対象なしにならないようスレッドを停止しない
    if (!_alreadySuspended) {
        // スレッドを停止(遷移先画面種別は測定画面以外に変更される)
        emit requestToSuspendSingleMeasuring();

        AbstractViewModel::suspend();
    }

    // 測定条件のクリアー
    memset(&_condition, 0x00, sizeof(LC70_MEAS_SINGLE_T));
    memset(&_previousCondition, 0x00, sizeof(LC70_MEAS_SINGLE_T));

    // チャンネル情報のクリアー
    _channelList.clear();
    _preservedChannelList.clear();

    // 局発周波数のリストをクリアー
    _localFrequencyList.clear();

    // ユーザー設定のクリアー
    _userSettings.clear();

    // ユーザー設定からの遷移判定を初期値に戻す
    _fromUserSetting = false;

    // グラフからカーソルを外す
    _cursoredGraph = false;
    _delayTimeIndex = 0;
    if (_delayProfile) {
        _delayProfile->setProperty("cursoredGraph", _cursoredGraph);
    }

    // チャンネル変更中をクリアー
    _isChannelChanging = false;

    // 保持している測定データのクリア
    clearHoldMeasurementData();

    _level = 0;
    _mer = 0;
    _ber = BER_METER_MINIMUM_POSITION;
    _newestLevel = 0;
    _newestMer = 0;
    _newestBer = BER_METER_MINIMUM_POSITION;
    _isUpdateMeter = false;
    _broadcastSystem = LC70_BD_SYS_NONE;
    _modulation = LC70_SETMOD_UNDEF;
    _layer = 0;
    _timeStamp.clear();
    _isScanning = false;
    _modulationInMeasureResult= LC70_SETMOD_UNDEF;
    _failureCode = 0;
    _actionInFailure = LC70_FAIL_CTL_NONE;
    _powerSupplyErrorOccured = false;
    _waitforLockingTimeOut = false;
    _confirmedValue = false;
    _locked = UNLOCKED;
    // コンボボックス編集状態初期状態に戻す
    _iscomboboxVisible = false;
    _comboboxState = Normal;
    if (_view) {
        _view->setProperty("editFrequencyCombobox", false);
        _view->setProperty("frequencyComboboxVisible", false);
        if (_combobox) {
            _combobox->setProperty("value", "");
        }
    }

    // QML オブジェクトへのポインターをクリアー
    _meter = nullptr;
    _constellation = nullptr;
    _delayProfile = nullptr;
    _lineSeries = nullptr;
    _combobox = nullptr;

    qDebug() << QString("[Leave] SingleMeasurementViewModel::suspend()");
    _model->writeLog(QString("シングル測定"), QString("シングル測定終了"));
}

void SingleMeasurementViewModel::createConnection(QObject *qml, QObject *mainFrame)
{
    AbstractMeasurementViewModel::createConnection(qml, mainFrame);

    connect(this, SIGNAL(requestSetTakeOver(QVariant, QVariant)), _view, SLOT(backup(QVariant, QVariant)));
    connect(this, SIGNAL(resumeSingleMeasuring(LC70_CONSTA_NUM_T, LC70_MEAS_SINGLE_T, uint32_t)), _model,
            SLOT(onResumeSingleMeasuring(LC70_CONSTA_NUM_T, LC70_MEAS_SINGLE_T, uint32_t)));
    connect(_model, SIGNAL(singleMeasureResult(MeasureResult, QList<QVariant>)), this,
            SLOT(onSingleMeasureResult(MeasureResult, QList<QVariant>)));
    connect(_model, SIGNAL(delayProfileResult(QVector<QPointF>)), this, SLOT(onDelayProfileResult(QVector<QPointF>)));
    connect(_model, SIGNAL(notifyChannelChanging(bool)), this, SLOT(onNotifyChannelChanging(bool)));
    connect(_model, SIGNAL(scanningChannelResult(int, LC70_MEAS_MULTI_RESULT_T)), this, SLOT(onScanningChannelResult(int,
                                                                                                                     LC70_MEAS_MULTI_RESULT_T)));
    connect(_model, SIGNAL(scanningFinished(bool)), this, SLOT(onScanningFinished(bool)));
    connect(this, SIGNAL(changeCondition(LC70_MEAS_SINGLE_T)), _model,
            SIGNAL(singleMeasuringChangeCondition(LC70_MEAS_SINGLE_T)));
    connect(this, SIGNAL(requestToCapture()), mainFrame, SLOT(onRequestToCapture()));
    connect(this, SIGNAL(requestToSaveImage(MeasureType, SaveLocation, QString)), mainFrame, SLOT(onRequestToSaveImage(MeasureType, SaveLocation, QString)));
    connect(this, SIGNAL(requestToClearCaptureImage()), mainFrame, SLOT(onRequestToClearCaptureImage()));
    connect(this, SIGNAL(requestToSaveData(SaveLocation, QString, uint16_t, MeasureResult)), mainFrame,
            SLOT(onRequestToSaveData(SaveLocation, QString, uint16_t, MeasureResult)));
    connect(mainFrame, SIGNAL(inputFileName(PopupId, SaveLocation, QString)), this, SLOT(onInputFileName(PopupId,SaveLocation, QString)));
    connect(mainFrame, SIGNAL(requestToScan()), this, SLOT(onRequestToScan()));
    connect(mainFrame, SIGNAL(requestToStopScanning()), this, SLOT(onRequestToStopScanning()));
    connect(mainFrame, SIGNAL(notifyButtonInput(PopupId, ButtonId)), this, SLOT(onNotifyButtonInput(PopupId, ButtonId)));
    connect(mainFrame, SIGNAL(notifyPowerSupplyError(uint16_t, uint8_t, QString)), this, SLOT(detectPowerSupplyError(uint16_t, uint8_t, QString)));
    connect(this, SIGNAL(cursorPressed(QVariant)), mainFrame, SIGNAL(cursorPressed(QVariant)));
    connect(this, SIGNAL(progress(double)), mainFrame, SIGNAL(progress(double)));
    connect(this, SIGNAL(progressText(QString, bool)), mainFrame, SIGNAL(progressText(QString, bool)));
    connect(this, SIGNAL(changeAttribute(QString, QVariant)), mainFrame, SLOT(onChangeAttribute(QString, QVariant)),
            Qt::DirectConnection);
    connect(this, SIGNAL(removeAttribute(QString)), mainFrame, SLOT(onRemoveAttribute(QString)), Qt::DirectConnection);
    connect(_model, SIGNAL(notifyDelayProfileValid(bool)), this, SLOT(onNotifyDelayProfileValid(bool)));
    connect(this, SIGNAL(requestToReScan()), this, SLOT(onRequestToReScan()), Qt::QueuedConnection);
    connect(this, SIGNAL(notifyLevelMeter(QVariant,QVariant)), _view, SLOT(updateLevelMeterValue(QVariant, QVariant)));
    connect(this, SIGNAL(notifyMerMeter(QVariant,QVariant)), _view, SLOT(updateMerMeterValue(QVariant, QVariant)));
    connect(this, SIGNAL(notifyBerMeter(QVariant,QVariant)), _view, SLOT(updateBerMeterValue(QVariant, QVariant)));
    connect(this, SIGNAL(notifyLevelThreshold(QVariant,QVariant)), _view, SLOT(updateLevelThreshold(QVariant, QVariant)));
    connect(this, SIGNAL(notifyMerThreshold(QVariant,QVariant)), _view, SLOT(updateMerThreshold(QVariant, QVariant)));
    connect(this, SIGNAL(notifyBerThreshold(QVariant,QVariant)), _view, SLOT(updateBerThreshold(QVariant, QVariant)));
    connect(this, SIGNAL(requestToShowRegistScanResultPopup()), this, SLOT(onRequestToShowRegistScanResultPopup()), Qt::QueuedConnection);
    connect(this, SIGNAL(requestToSuspendSingleMeasuring()), _model, SLOT(onSuspendSingleMeasuring()));

    // 各画面
    if (_view) {
        _meter = _view->findChild<QObject *>("meter");
        _constellation = _view->findChild<QObject *>("constellation");
        _delayProfile = _view->findChild<QObject *>("delayProfile");
    }

    // コンボボックス
    if (_meter) {
        QObject * levelGauge = _meter->findChild<QObject *>("level");
        if (levelGauge) {
            if (_combobox == nullptr) {
                _combobox = levelGauge->findChild<QObject *>("frequencyCombobox");
            }
        }
    }

    // シングル測定結果の取得に設定
    _model->changeAcquisitionTarget(SINGLE);
    _model->writeLog(QString("シングル測定"), QString("メーター表示"));
}

void SingleMeasurementViewModel::activate()
{
    AbstractMeasurementViewModel::activate();

    if (_view == nullptr) {
        return;
    }

    // 一時停止していない
    _alreadySuspended = false;

    // チャンネル変更中をクリアー
    _isChannelChanging = false;

    // 遅延プロファイル測定結果
    if (_view) {
        _lineSeries = dynamic_cast<DelayProfileLineSeriesModel *>(_view->findChild<QObject *>(QString("delayProfileLineSeries")));
    }

    // 画面遷移時は、メニューバーを閉じた状態にする
    setMenuBarVisible(false);

    // QML ファイルのプロパティの変更(SingleMeasurement.qml を参照)
    // 画面ロード時は初期値に設定する
    if (_view) {
        _view->setProperty("level", INITIAL_LEVEL);
        _view->setProperty("levelNeedlePosition", INITIAL_LEVEL_NEEDLE_POSITION);
        _view->setProperty("mer", INITIAL_MER);
        _view->setProperty("merNeedlePosition", INITIAL_MER_NEEDLE_POSITION);
        _view->setProperty("ber", INITIAL_BER);
        _view->setProperty("berNeedlePosition", INITIAL_BER_NEEDLE_POSITION);
    }

    qDebug() << QString("selectedMeasurement:") << _takeOver["selectedMeasurement"].toString();
    qDebug() << QString("selectedBand:") << _takeOver["selectedBand"].toString();
    qDebug() << QString("selectedModulation:") << _takeOver["selectedModulation"].toString();
    qDebug() << QString("selectedLocalOscillator:") << _takeOver["selectedLocalOscillator"].toString();
    qDebug() << QString("selectedPolarization:") << _takeOver["selectedPolarization"].toString();

    // CNR/MER 切替設定の取得
    _cnrMerSetting.clear();
    _model->getValue(VALUE_MER_CN, _cnrMerSetting);

    // マニュアル測定 放送方式, 変調方式選択の取得
    _broadBandModulationSetting.clear();
    _model->getValue(VALUE_BROADBANDSYSTEM_MODULATION, _broadBandModulationSetting);


    MainFrameViewModel *mainFrame = static_cast<MainFrameViewModel *>(_mainFrame);
    QVariant valueopt;
    int rfOptical = 0;
    _model->getValue(VALUE_RF_OPTICAL, valueopt);
    rfOptical = valueopt.toInt();
    if(rfOptical == 1){
        mainFrame->menuoptchange(false);
        _model->replaceMenuList(MenuListIconIndex::PREAMP_ICON, "icon_preamp.png");
    }else{
        mainFrame->menuoptchange(true);
    }


    _currentChannelIndex = 0;
    bool scanned = false;
    LC70_BAND_T band = LC70_BAND_UNDEF;

    if (_takeOver["fromUserSetting"].toBool()) {
        // ユーザー設定の反映
        _fromUserSetting = true;

        if (!createUserSettingChannelList(_userSettings)) {
            if (_view) {
                _buttonPattern = NormalPattern;
                _measurementButtons = SINGLE_MEASUREMENT_BUTTON_DATA;
                _buttonOperation->setButtonName(BUTTON_COLUMN_NUMBER, BUTTON_ROW_NUMBER_NORMAL, _measurementButtons);
                _view->setProperty("buttonPattern", 0);

                // 前回の測定結果をクリアするため、無効値を設定する
                emit notifyLevelMeter(INITIAL_LEVEL, INITIAL_LEVEL_NEEDLE_POSITION);
                emit notifyMerMeter(INITIAL_MER, INITIAL_MER_NEEDLE_POSITION);
                emit notifyBerMeter(INITIAL_BER, INITIAL_BER_NEEDLE_POSITION);

                _view->setProperty("band", Correcter::displayBandName(LC70_BAND_UHF));
                _view->setProperty("selectedModulation", "");
                _view->setProperty("isSatelliteBroadCasting", Correcter::isSatelliteBroadCasting(LC70_BAND_UHF));
                _view->setProperty("isDisplaylevelHierarchyPolarization",
                                   Correcter::isDisplayLevelHierarchyPolarization(LC70_BAND_UHF, LC70_SETMOD_UNDEF));
                _view->setProperty("channel", "");
                _view->setProperty("frequency", "");
                _view->setProperty("polarization", "");
                _view->setProperty("isModulationVisible", Correcter::isModulationVisible(LC70_BAND_UHF));
                _view->setProperty("selectCnMer", Correcter::selectCnrMer(LC70_BAND_UHF, _cnrMerSetting));
            }
            emit requestToShowPopup(POPUP_FAILURE_READ_USER_SETTING_FILE);
            return;
        }

        // ヘッダーに Band を表示
        addSelectBandIcon(band);

        // ユーザー設定名を反映
        if (_view) {
            _view->setProperty("userMode", _takeOver["selectedUserListFile"].toString());
        }
    } else {
        // 画面共有情報からBandを取得
        int bandId = _takeOver["selectedBand"].toInt();

        if (bandId == LC70_BAND_FM) {
            bandId = LC70_BAND_FM_DEF;
        }

        if (bandId == LC70_BAND_CATV_UP_STREAM) {
            bandId = LC70_BAND_CATV_UP_STREAM_DEF;
        }

        // ユーザー設定の反映
        _fromUserSetting = false;

        QString polarization = "";

        if (_takeOver.contains("selectedPolarization")) {
            polarization = _takeOver["selectedPolarization"].toString();
        }
        _model->selectChannelDefinitionList(bandId, polarization, _channelList);

        // スキャン結果があればその内容を反映する(保存画面から戻ってきた場合に使用するため、Band などの測定条件は同一であることを前提とする)
        QList<ChannelDefinitionItem> scannedChannelList;

        // 保存先選択画面で保存されていたらスキャン結果を読み取る
        //if (_takeOver.contains("saveScanData") && _takeOver["saveScanData"].toBool()) {
            scannedChannelList = _model->scannedChannelList();
        //}

        if (scannedChannelList.count() > 0 &&
                scannedChannelList.count() == _channelList.count() &&
                polarization == _model->scannedPolatization()) {

            _currentChannelIndex = -1;

            for (int i = 0; i < scannedChannelList.count(); ++i) {
                _channelList[i].band = scannedChannelList[i].band;
                _channelList[i].bandWidth = scannedChannelList[i].bandWidth;
                _channelList[i].centerFrequency1 = scannedChannelList[i].centerFrequency1;
                _channelList[i].centerFrequency2 = scannedChannelList[i].centerFrequency2;
                _channelList[i].channelNumber= scannedChannelList[i].channelNumber;
                _channelList[i].polarization = scannedChannelList[i].polarization;
                _channelList[i].valid = scannedChannelList[i].valid;
                if (_currentChannelIndex < 0 && _channelList[i].valid) {
                    _currentChannelIndex = i;
                }
            }
            scanned = true;
            if (_view) {
                // ユーザー設定名を反映
                if(_takeOver["selectedUserListFile"].toString() == "") {
                    _view->setProperty("userMode", NO_FILE_NAME);
                } else {
                    _view->setProperty("userMode", _takeOver["selectedUserListFile"].toString());
                }
            }
            // ユーザー設定名をクリア
            emit removeAttribute("selectedUserListFile");
        }

        if (bandId == LC70_BAND_FM_DEF) {
            bandId = LC70_BAND_FM;
        }

        if (bandId == LC70_BAND_CATV_UP_STREAM_DEF) {
            bandId = LC70_BAND_CATV_UP_STREAM;
        }

        // 地デジの場合は OFDM とする
        if (bandId == LC70_BAND_UHF) {
            _takeOver["selectedModulation"] = QString("OFDM");
        }

        _preservedChannelList.clear();

        for (int i = 0; i < _channelList.count(); ++i) {
            _preservedChannelList.append(_channelList[i]);
        }
    }

    // 保存していたスキャン後の有効チャンネルを破棄
    _model->clearScannedChannelList();

    // 局発周波数の表示切り替え
    if (_channelList[_currentChannelIndex].band == LC70_BAND_BS
            || _channelList[_currentChannelIndex].band == LC70_BAND_CS
            || _channelList[_currentChannelIndex].band == LC70_BAND_JCSAT3
            || _channelList[_currentChannelIndex].band == LC70_BAND_JCSAT4
            || _channelList[_currentChannelIndex].band == LC70_BAND_SCCB
            || _channelList[_currentChannelIndex].band == LC70_BAND_SCCC) {
        if (_constellation) {
            _constellation->setProperty("isLocalOscillatorVisible", true);
        }

        if (_delayProfile) {
            _delayProfile->setProperty("isLocalOscillatorVisible", true);
        }
    } else {
        if (_constellation) {
            _constellation->setProperty("isLocalOscillatorVisible", false);
        }

        if (_delayProfile) {
            _delayProfile->setProperty("isLocalOscillatorVisible", false);
        }
    }

    // ボタン表示設定
    // PILOT, LTE, FM 選択時はコンスタレーション、遅延プロファイル非表示
    if (_channelList[_currentChannelIndex].band == LC70_BAND_LTE
            || _channelList[_currentChannelIndex].band == LC70_BAND_PILOT
            || _channelList[_currentChannelIndex].band == LC70_BAND_FM
            || _channelList[_currentChannelIndex].band == LC70_BAND_CATV_UP_STREAM) {
        _buttonPattern = LimitedPattern2;
        _measurementButtons = SINGLE_MEASUREMENT_BUTTON_DATA;
        _buttonOperation->setButtonName(BUTTON_COLUMN_NUMBER, BUTTON_ROW_NUMBER_LIMITED2, _measurementButtons);
        if (_view) {
            _view->setProperty("buttonPattern", 2);
        }
    } else if (_channelList[_currentChannelIndex].band == LC70_BAND_UHF) {
        _buttonPattern = NormalPattern;
        _measurementButtons = SINGLE_MEASUREMENT_BUTTON_DATA;
        _buttonOperation->setButtonName(BUTTON_COLUMN_NUMBER, BUTTON_ROW_NUMBER_NORMAL, _measurementButtons);
        if (_view) {
            _view->setProperty("buttonPattern", 0);
        }
    } else {
        QString selectedModulation = _takeOver["selectedModulation"].toString();
        // CATV で変調方式が ISDBT(OFDM) なら遅延プロファイルを表示
        if ((_channelList[_currentChannelIndex].band == LC70_BAND_CATV_ALL ||
             _channelList[_currentChannelIndex].band == LC70_BAND_CATV_90_470MHZ ||
             _channelList[_currentChannelIndex].band == LC70_BAND_CATV_470_1002MHZ) && selectedModulation.contains("OFDM")) {
            _buttonPattern = NormalPattern;
            _measurementButtons = SINGLE_MEASUREMENT_BUTTON_DATA;
            _buttonOperation->setButtonName(BUTTON_COLUMN_NUMBER, BUTTON_ROW_NUMBER_NORMAL, _measurementButtons);
            if (_view) {
                _view->setProperty("buttonPattern", 0);
            }
        } else {
            _buttonPattern = LimitedPattern1;
            _measurementButtons = SINGLE_MEASUREMENT_BUTTON_DATA;
            _buttonOperation->setButtonName(BUTTON_COLUMN_NUMBER, BUTTON_ROW_NUMBER_LIMITED1, _measurementButtons);
            if (_view) {
                _view->setProperty("buttonPattern", 1);
            }
        }
    }

    // アニメーション設定
    // メーターの更新周期を取得
    _meterUpdateInterval = _model->applicationSetting()->meterUpdateInterval();
    // 入力無効時のメーター針移動倍率を取得
    _invalidValueMeterSpeed = _model->applicationSetting()->invalidValueMeterSpeed();

    _buttonOperation->setActiveObject(SINGLE_MEASUREMENT_BUTTON_DATA[_buttonPattern][0]);
    _activeButton = SINGLE_MEASUREMENT_BUTTON_DATA[_buttonPattern][0];

    // 画面を開いたときは常にシングル測定から
    if (_view) {
        setButtonActive(_view, _activeButton, false);
    }
    _selectedButtonIndex = 0;
    _activeButton = _measurementButtons[0][_selectedButtonIndex];
    _buttonOperation->setActiveObject(_activeButton);
    if (_view) {
        setButtonActive(_view, _activeButton, true);
    }
    setBoolProperty(DATA_BUTTON_NAME, "visible", true);

    // コンスタレーションの表示ポイント数の設定値を反映
    QVariant value;

    if (_model->getValue(VALUE_CONSTELLATION, value)) {
        bool ok = false;
        int constellationValue = value.toInt(&ok);

        if (ok) {
            _limitContellationPoints = static_cast<LC70_CONSTA_NUM_T>(constellationValue);
        }
    }

    // 閾値データを初期値に戻す
    _levelReceivableThreshold = float32_t(0);
    _mercnMerginHighThreshold = float32_t(0);
    _berMarginHighThreshold = float32_t(0);

    // グラフにカーソルがあたっているかのフラグを落とす
    _cursoredGraph = false;
    _delayTimeIndex = 0;
    if (_delayProfile) {
        _delayProfile->setProperty("cursoredGraph", _cursoredGraph);
    }

    // スキャンアイコン表示
    if (!scanned) {
        // なし
        _model->replaceMenuList(MenuListIconIndex::SCAN_ICON, "");
    } else {
        // あり
        _model->replaceMenuList(MenuListIconIndex::SCAN_ICON, "icon_scan_n.png");
    }

    // モデル側で保持しているScan結果をクリア
    _model->clearScanResult();

    // 給電
    _model->replaceMenuList(MenuListIconIndex::POWER_SUPPLY_ICON, "");
    _powerSupply = LC70_POWER_SUPPLY_OFF;

    // コンボボックス
    // 画面遷移時は必ず通常状態で遷移する
    _comboboxState = Normal;
    if (_view) {
        _view->setProperty("editFrequencyCombobox", false);
    }

    // 条件に当てはまらない場合はコンボボックスではなくラベルで表示する
    if (_view) {
        _view->setProperty("frequencyComboboxVisible", false);
        if (_combobox) {
            _combobox->setProperty("value", "");
        }
    }
    _iscomboboxVisible = false;
}

void SingleMeasurementViewModel::deactivate()
{
    AbstractMeasurementViewModel::deactivate();

    disconnect(this, SIGNAL(requestSetTakeOver(QVariant, QVariant)), _view, SLOT(backup(QVariant, QVariant)));
    disconnect(this, SIGNAL(resumeSingleMeasuring(LC70_CONSTA_NUM_T, LC70_MEAS_SINGLE_T, uint32_t)), _model,
               SLOT(onResumeSingleMeasuring(LC70_CONSTA_NUM_T, LC70_MEAS_SINGLE_T, uint32_t)));
    disconnect(_model, SIGNAL(singleMeasureResult(MeasureResult, QList<QVariant>)), this,
               SLOT(onSingleMeasureResult(MeasureResult, QList<QVariant>)));
    disconnect(_model, SIGNAL(delayProfileResult(QVector<QPointF>)), this, SLOT(onDelayProfileResult(QVector<QPointF>)));
    disconnect(_model, SIGNAL(notifyChannelChanging(bool)), this, SLOT(onNotifyChannelChanging(bool)));
    disconnect(_model, SIGNAL(scanningChannelResult(int, LC70_MEAS_MULTI_RESULT_T)), this, SLOT(onScanningChannelResult(int,
                                                                                                                        LC70_MEAS_MULTI_RESULT_T)));
    disconnect(_model, SIGNAL(scanningFinished(bool)), this, SLOT(onScanningFinished(bool)));
    disconnect(this, SIGNAL(changeCondition(LC70_MEAS_SINGLE_T)), _model,
               SIGNAL(singleMeasuringChangeCondition(LC70_MEAS_SINGLE_T)));
    disconnect(this, SIGNAL(requestToSaveData(SaveLocation, QString, uint16_t, MeasureResult)), _mainFrame,
               SLOT(onRequestToSaveData(SaveLocation, QString, uint16_t, MeasureResult)));
    disconnect(this, SIGNAL(requestToCapture()), _mainFrame, SLOT(onRequestToCapture()));
    disconnect(this, SIGNAL(requestToSaveImage(MeasureType, SaveLocation, QString)), _mainFrame, SLOT(onRequestToSaveImage(MeasureType, SaveLocation, QString)));
    disconnect(this, SIGNAL(requestToClearCaptureImage()), _mainFrame, SLOT(onRequestToClearCaptureImage()));
    disconnect(_mainFrame, SIGNAL(inputFileName(PopupId, SaveLocation, QString)), this, SLOT(onInputFileName(PopupId,
                                                                                                             SaveLocation, QString)));
    disconnect(_mainFrame, SIGNAL(requestToScan()), this, SLOT(onRequestToScan()));
    disconnect(_mainFrame, SIGNAL(requestToStopScanning()), this, SLOT(onRequestToStopScanning()));
    disconnect(_mainFrame, SIGNAL(notifyButtonInput(PopupId, ButtonId)), this, SLOT(onNotifyButtonInput(PopupId,
                                                                                                        ButtonId)));
    disconnect(this, SIGNAL(cursorPressed(QVariant)), _mainFrame, SIGNAL(cursorPressed(QVariant)));
    disconnect(this, SIGNAL(progress(double)), _mainFrame, SIGNAL(progress(double)));
    disconnect(this, SIGNAL(progressText(QString, bool)), _mainFrame, SIGNAL(progressText(QString, bool)));
    disconnect(this, SIGNAL(changeAttribute(QString, QVariant)), _mainFrame, SLOT(onChangeAttribute(QString, QVariant)));
    disconnect(this, SIGNAL(removeAttribute(QString)), _mainFrame, SLOT(onRemoveAttribute(QString)));
    disconnect(_mainFrame, SIGNAL(notifyPowerSupplyError(uint16_t, uint8_t, QString)), this, SLOT(detectPowerSupplyError(uint16_t, uint8_t, QString)));
    disconnect(this, SIGNAL(requestToReScan()), this, SLOT(onRequestToReScan()));
    disconnect(this, SIGNAL(notifyLevelMeter(QVariant,QVariant)), _view, SLOT(updateLevelMeterValue(QVariant, QVariant)));
    disconnect(this, SIGNAL(notifyMerMeter(QVariant,QVariant)), _view, SLOT(updateMerMeterValue(QVariant, QVariant)));
    disconnect(this, SIGNAL(notifyBerMeter(QVariant,QVariant)), _view, SLOT(updateBerMeterValue(QVariant, QVariant)));
    disconnect(this, SIGNAL(notifyLevelThreshold(QVariant,QVariant)), _view, SLOT(updateLevelThreshold(QVariant, QVariant)));
    disconnect(this, SIGNAL(notifyMerThreshold(QVariant,QVariant)), _view, SLOT(updateMerThreshold(QVariant, QVariant)));
    disconnect(this, SIGNAL(notifyBerThreshold(QVariant,QVariant)), _view, SLOT(updateBerThreshold(QVariant, QVariant)));
    disconnect(this, SIGNAL(requestToShowRegistScanResultPopup()), this, SLOT(onRequestToShowRegistScanResultPopup()));
    disconnect(this, SIGNAL(requestToSuspendSingleMeasuring()), _model, SLOT(onSuspendSingleMeasuring()));

    // メニューバーの情報をクリア
    setMenuBarVisible(false);

    // 給電
    _model->removeMenuList(MenuListIconIndex::POWER_SUPPLY_ICON);
    _powerSupply = LC70_POWER_SUPPLY_OFF;

    // スキャン
    _model->removeMenuList(MenuListIconIndex::SCAN_ICON);

    // ユーザー設定名をクリアー
    if (_view) {
        _view->setProperty("userMode", QString());
    }
}

bool SingleMeasurementViewModel::isValidChannel(int channleIndex)
{
    if (!_channelList[channleIndex].valid) {
        // 無効なチャンネルは対象外
        return false;
    }

    if (_model->acquisitionTarget() == DELAY_PROFILE) {
        if (!isEnableDelayProfileChannel(channleIndex)) {
            return false;
        }
    }

    if (_model->acquisitionTarget() == CONSTELLATION) {
        if (!isEnableConstellationBandId(_channelList[channleIndex].band)) {
            return false;
        }
    }

    return true;
}

void SingleMeasurementViewModel::changeByMenuBar()
{
    _model->writeLog(QString("シングル測定"), QString("[ENTER] %1").arg(__FUNCTION__));

    int barType = _menuBarType;
    int barIndex = _menuBarIndexMap[0];

    // メニューバーを閉じる
    closeAllMenuBar();

    // Menuアイコンの更新を実施
    changeMenuIcon();
    if (barType == 1) {
        _model->replaceMenuList(MenuListIconIndex::FUNCTION_ICON, "icon_menu_user.png");
    }

    // マルチ測定を選択している場合
    if (_takeOver["selectedMeasurement"].toString() == QString(MULTI_MEASURE_TYPE_NAME)) {

        // ユーザー設定からの遷移判定を初期値に戻す
        _takeOver["fromUserSetting"] = false;
        _fromUserSetting = false;

        // スレッドを停止(遷移先画面種別は測定画面以外に変更される)
        emit requestToSuspendSingleMeasuring();

        AbstractViewModel::suspend();

        // 一時停止済み
        _alreadySuspended = true;

        // 画面共有データの送信
        // [Memo] 測定情報、BAND情報は必ずデータが設定されている前提
        //        変調方式、偏波面、局発周波数は不要な場合は、未設定
        emit requestSetTakeOver("selectedMeasurement", _takeOver["selectedMeasurement"]);
        emit requestSetTakeOver("selectedBand", _takeOver["selectedBand"]);
        emit requestSetTakeOver("fromUserSetting", false);

        if (barType == 0) {
            // 変調方式、偏波面、局発周波数はnullではない場合に送信
            if (!_takeOver["selectedModulation"].isNull()) {
                emit requestSetTakeOver("selectedModulation", _takeOver["selectedModulation"]);
            }

            if (!_takeOver["selectedPolarization"].isNull()) {
                emit requestSetTakeOver("selectedPolarization", _takeOver["selectedPolarization"]);
            }

            if (!_takeOver["selectedLocalOscillator"].isNull()) {
                emit requestSetTakeOver("selectedLocalOscillator", _takeOver["selectedLocalOscillator"]);
            }
        } else if (barType == 1) {
            MainFrameViewModel *mainFrame = static_cast<MainFrameViewModel *>(_mainFrame);
            QVariantList vFileList = mainFrame->getUserFileList();
            QVariant vFileName = vFileList[barIndex];
            QString sFileName = vFileName.toString();
            _model->importDataLoggerUserSettingList(VALUE_USERSETTING_SETTINGLIST, sFileName);
            emit requestSetTakeOver("fromUserSetting", true);
            emit requestSetTakeOver("selectedUserListFile", sFileName);
        }

        // 画面遷移
        QList<ScreenId> history = QList<ScreenId>() << SCREEN_HOME << SCREEN_BAND;
        emit requestWithChangeHistory(SCREEN_MULTI_MEASUREMENT, history);
        return;
    }

    // アンテナ調整を選択している場合
    if (_takeOver["selectedBand"].toInt() == LC70_BAND_JCSAT3 * LC70_BAND_JCSAT4) {

        // 測定設定にアンテナ調整を追加
        _takeOver["selectedMeasurement"] = ANTENNA_MEASURE_TYPE_NAME;

        // ユーザー設定からの遷移判定を初期値に治す
        _takeOver["fromUserSetting"] = false;
        _fromUserSetting = false;

        // スレッドを停止(遷移先画面種別は測定画面以外に変更される)
        emit requestToSuspendSingleMeasuring();

        AbstractViewModel::suspend();

        // 一時停止
        _alreadySuspended = true;

        // 画面共有データの送信
        // [Memo] 測定情報、BAND情報は必ずデータが設定されている前提
        //        変調方式、偏波面、局発周波数は不要な場合は、未設定
        emit requestSetTakeOver("selectedMeasurement", _takeOver["selectedMeasurement"]);
        emit requestSetTakeOver("selectedBand", _takeOver["selectedBand"]);
        emit changeAttribute(QString("fromUserSetting"), false);

        // 変調方式、偏波面、局発周波数はnullではない場合に送信
        if (!_takeOver["selectedModulation"].isNull()) {
            emit requestSetTakeOver("selectedModulation", _takeOver["selectedModulation"]);
        }

        if (!_takeOver["selectedPolarization"].isNull()) {
            emit requestSetTakeOver("selectedPolarization", _takeOver["selectedPolarization"]);
        }

        if (!_takeOver["selectedLocalOscillator"].isNull()) {
            emit requestSetTakeOver("selectedLocalOscillator", _takeOver["selectedLocalOscillator"]);
        }

        // 画面遷移
        QList<ScreenId> history = QList<ScreenId>() << SCREEN_HOME << SCREEN_BAND;
        emit requestWithChangeHistory(SCREEN_ANTENNA_ADJUSTMENT, history);
        return;
    }

    // ユーザ設定からの遷移判定がTrueの場合
    if (_fromUserSetting || barType == 1) {
        // ユーザ設定からの遷移判定を初期値に戻す
        _takeOver["fromUserSetting"] = false;
        emit changeAttribute(QString("fromUserSetting"), false);
        // ユーザー設定名をクリア
        _takeOver["selectedUserListFile"] = "";
        emit removeAttribute("selectedUserListFile");
        if (_view) {
            _view->setProperty("userMode", "");
        }

        _fromUserSetting = false;

        // [Memo]
        // 画面の遷移履歴のみが変化する特別な画面遷移処理となるため、 requestWithChangeHistoryを行うが画面遷移は実施しない。
        // 遷移履歴を更新する
        QList<ScreenId> history = QList<ScreenId>() << SCREEN_HOME << SCREEN_BAND;
        emit requestWithChangeHistory(SCREEN_SINGLE_MEASUREMENT, history, false);

        // Menuボタン押下によるユーザ設定の場合
        if (barType == 1) {
            MainFrameViewModel *mainFrame = static_cast<MainFrameViewModel *>(_mainFrame);
            QVariantList vFileList = mainFrame->getUserFileList();
            QVariant vFileName = vFileList[barIndex];
            QString sFileName = vFileName.toString();
            _model->importDataLoggerUserSettingList(VALUE_USERSETTING_SETTINGLIST, sFileName);
            _takeOver["fromUserSetting"] = true;
            _takeOver["selectedUserListFile"] = sFileName;
        }
    }

    // 画面を初期状態に戻す
    resetScreen();

    _model->writeLog(QString("シングル測定"), QString("[LEAVE] %1").arg(__FUNCTION__));
}

void SingleMeasurementViewModel::resetScreen()
{
    // ブザー鳴動中である場合は停止する
    stopBuzzer();

    _model->changeAcquisitionTarget(SINGLE);

    // 給電を停止
    _powerSupply = LC70_POWER_SUPPLY_OFF;
    _model->setPowerSupply(_powerSupply);
    _model->removeMenuList(MenuListIconIndex::POWER_SUPPLY_ICON);

    // 画面遷移履歴のクリアー
    _changeHistory.clear();

    // 測定条件のクリアー
    memset(&_condition, 0x00, sizeof(LC70_MEAS_SINGLE_T));
    memset(&_previousCondition, 0x00, sizeof(LC70_MEAS_SINGLE_T));

    // チャンネル情報のクリアー
    _channelList.clear();

    // 局発周波数のリストをクリアー
    _localFrequencyList.clear();

    // ユーザー設定のクリアー
    _userSettings.clear();

    // ユーザー設定からの遷移判定を初期値に戻す
    _fromUserSetting = false;

    // 給電をヘッダーから取り除く
    _model->removeMenuList(MenuListIconIndex::POWER_SUPPLY_ICON);
    // ユーザー設定名をクリアー
    if (_view) {
        _view->setProperty("userMode", QString());
    }

    // スキャンをヘッダーから取り除く
    _model->removeMenuList(MenuListIconIndex::SCAN_ICON);

    // 閾値の設定をクリア
    _thresholdSet = false;
    if (_view) {
        _view->setProperty("levelRedZoneRatio", 0);
        _view->setProperty("merRedZoneRatio", 0);
        _view->setProperty("merYellowZoneRatio", 0);
        _view->setProperty("berRedZoneRatio", 0);
        _view->setProperty("berYellowZoneRatio", 0);
    }

    // 保持している測定データをクリア
    clearHoldMeasurementData();

    // 現在メーターを表示中でなければ、メーターに戻す必要がある
    int beforeMode = _selectedButtonIndex;

    // 初期値に戻す
    activate();

    // チャンネル変更(測定開始)
    changeChannel();

    if (beforeMode != MEASUREMENT_BUTTON_INDEX) {
        onRequest(MEASUREMENT_BUTTON_INDEX);
    }

    // メーターをアクティブに設定する
    changeActiveButton(MEASUREMENT_BUTTON_INDEX);
}

void SingleMeasurementViewModel::recover()
{
    if (_view) {
        _meter = _view->findChild<QObject *>("meter");
        _constellation = _view->findChild<QObject *>("constellation");
        _delayProfile = _view->findChild<QObject *>("delayProfile");
        if (_meter) {
            QObject * levelGauge = _meter->findChild<QObject *>("level");
            if (levelGauge) {
                if (_combobox == nullptr) {
                    _combobox = levelGauge->findChild<QObject *>("frequencyCombobox");
                }
            }
        }
    }
}

void SingleMeasurementViewModel::onKeyStatus(ButtonId id)
{
    if (isMenuBarVisible()) {
        // メニューバー表示中のボタン操作
        menuBarOperation(id);
        return;
    }

    if (_comboboxState == Edit) {
        // コンボボックス編集状態のボタン操作
        comboboxOperation(id);
        return;
    }

    // Home, Escape ボタン(偏波から遷移した場合を除く)はベースクラスで対応
    if (id == BUTTON_HOME) {
        qDebug() << QString("Go to HOME");

        // スレッドを停止
        emit requestToSuspendSingleMeasuring();

        AbstractViewModel::onKeyStatus(id);
    }

    if (id == BUTTON_ENTER) {

        if (_iscomboboxVisible) {
            // コンボボックスを表示している場合、コンボボックスの状態を編集状態にする
            _comboboxState = Edit;
            if (_view) {
                _view->setProperty("editFrequencyCombobox", true);
            }

            // ボタン操作情報の変更を行う
            QVector<QVector<QString>> buttonData = comboboxData(_channelList[_currentChannelIndex].band);
            _buttonOperation->setButtonName(comboboxColumnNumber(_channelList[_currentChannelIndex].band),
                                            comboboxRowNumber(_channelList[_currentChannelIndex].band),
                                            buttonData);
            _buttonOperation->setActiveObject(buttonData[0][0]);
            _activeButton = buttonData[0][0];
            if (_combobox) {
                setButtonActive(_combobox, _activeButton, true);
            }
            return;
        }

        if (_view == nullptr) {
            return;
        }
        QObject *target = _view->findChild<QObject *>(_activeButton);
        int index = -1;

        if (target) {
            bool ok = false;
            // 押下前の画像に更新
            target->setProperty("pressed", false);
            index = target->property("index").toInt(&ok);

            if (!ok) {
                return;
            }
        }

        if (index < 0) {
            return;
        }

        if (_selectedButtonIndex == index) {
            if (_selectedButtonIndex == DELAY_PROFILE_BUTTON_INDEX) {
                // グラフにカーソルを移動する
                _cursoredGraph = !_cursoredGraph;
                if (_delayProfile) {
                    _delayProfile->setProperty("cursoredGraph", _cursoredGraph);
                }
                if (_cursoredGraph) {
                    return;
                }
            } else {
                // nothing to do
            }
        }

        _selectedButtonIndex = index;
        onRequest(_selectedButtonIndex);
    } else if (id == BUTTON_SCAN) {
        // ユーザー設定から来た場合はスキャン不可
        if (!_fromUserSetting) {
            // FM, PILOT, LTE, CATV 上りはスキャン不可
            if (_takeOver["selectedBand"] == LC70_BAND_UNDEF || _takeOver["selectedBand"] == LC70_BAND_FM || _takeOver["selectedBand"] == LC70_BAND_PILOT || _takeOver["selectedBand"] == LC70_BAND_LTE || _takeOver["selectedBand"] == LC70_BAND_CATV_UP_STREAM) {
                emit requestToShowPopupWithMessage(POPUP_SCAN_NOT_APPLICABLE, QString());
                return;
            }
            int bandId = _takeOver["selectedBand"].toInt();
            emit requestToShowPopupScan(POPUP_SCAN_CONFIRMATION, bandId);
        }
    } else if (id == BUTTON_DATA_SAVE) {
        if (_selectedButtonIndex == MEASUREMENT_BUTTON_INDEX) {
            QString path = _model->applicationSetting()->resultDataPathSingle();
            int fileCount = countFilesRecursive(path);
            if (fileCount >= 1000) {
                emit requestToShowPopup(POPUP_DATA_SINGLE_MAX_SAVE_FILE);
                return;
            }
            // データ保存
            _saveFileType = SaveFileType::CSV;

            // DATAボタン押下時の測定データを保持する
            holdMeasurementData();

            // マニュアル測定時は保存先選択を表示、ユーザ設定からの場合は測定データのみ選択のためファイル名入力画面に遷移
            setBoolProperty(DATA_BUTTON_NAME, "pressed", false);

            // 測定データ保存
            emit requestToShowPopup(POPUP_SAVE_MEASUREMENT_DATA);
        }
    } else if (id == BUTTON_SCREEN_SAVE) {
        QString path = _model->applicationSetting()->resultDataPathSingle();
        int fileCount = countFilesRecursive(path);
        if (fileCount >= 1000) {
            emit requestToShowPopup(POPUP_DATA_SINGLE_MAX_SAVE_FILE);
            return;
        }

        // 画像保存
        _saveFileType = SaveFileType::PNG;

        // SCREENボタン押下時の画像を保持する
        emit requestToCapture();
        setBoolProperty(IMAGE_BUTTON_NAME, "pressed", false);

        emit requestToShowPopup(POPUP_INPUT_FILENAME);
    } else if (id == BUTTON_CHANNEL_UP) {
        _currentChannelIndex = nextChannel();

        if (_currentChannelIndex < 0) {
            return;
        }

        // チャンネル変更
        // CHANNEWL_UPボタン押下時は連打対策のため
        // チャンネル変更確定タイマーを動作させtimeOut時にチャンネル変更を確定する
        qDebug() << QString("channel up\n");
        changeChannel(false);
        // チャンネル変更確定タイマーを開始する
        _pressedChannelButtonId = BUTTON_CHANNEL_UP;
        startConfirmedChangeChannelTimer();
    } else if (id == BUTTON_CHANNEL_DOWN) {
        _currentChannelIndex = previousChannel();

        if (_currentChannelIndex < 0) {
            return;
        }

        // チャンネル変更
        // CHANNEWL_DOWNボタン押下時は連打対策のため
        // チャンネル変更確定タイマーを動作させtimeOut時にチャンネル変更を確定する
        qDebug() << QString("channel down\n");
        changeChannel(false);

        // チャンネル変更確定タイマーを開始する
        _pressedChannelButtonId = BUTTON_CHANNEL_DOWN;
        startConfirmedChangeChannelTimer();
    } else if (id == BUTTON_MENU) {
        if (isMenuBarVisible()) {
            return;
        }

        // メニューバーの設定を初期値に設定する
        AbstractViewModel::initializeMenuBarInformation();

        // メニューバーの表示
        _menuBarType = 1;
        emit requestToShowMenuBar(_menuBarType, _menuBarRank, _menuBarIndexMap.value(_menuBarRank));
        setMenuBarVisible(true);
    } else if (id == BUTTON_BAND) {
        if (isMenuBarVisible()) {
            return;
        }

        // メニューバーの表示
        emit requestToShowMenuBar(_menuBarType, _menuBarRank, _menuBarIndexMap.value(_menuBarRank));
        _menuBarRank = 1;  // 第 2 階層(Band)
        emit requestToShowMenuBar(_menuBarType, _menuBarRank, _menuBarIndexMap.value(_menuBarRank));
        setMenuBarVisible(true);

        // 測定設定をシングル測定
        _takeOver["selectedMeasurement"] = QString(SINGLE_MEASURE_TYPE_NAME);
        emit changeAttribute(QString("selectedMeasurement"), SINGLE_MEASURE_TYPE_NAME);

        // 画面遷移履歴の追加
        _changeHistory.clear();
        _changeHistory.append(SCREEN_HOME);

    } else if (id == BUTTON_USER) {
        // ユーザーデータロガー
#ifdef NEW_USER_SETTING
        emit requestDirect(SCREEN_USER_SETTING, true);
#else
        emit requestDirect(SCREEN_USER_DATA_LOGGER_MENU, true);
#endif
    } else if (id == BUTTON_POWER_SUPPLY) {
        if (_powerSupply == LC70_POWER_SUPPLY_OFF) {
            // バッテリーレベルが 5% 以下で未充電あるいは不定値のときはシャットダウン
            if (isLowBattery() && batteryStatus() <= BATTERY_STATUS_UNCHARGED) {
                // バッテリ低下状態中は給電開始させない
                emit requestToShowLowBatteryPopup();
                return;
            }

            // バッテリーレベルが 10% 以下なら給電不可
            if (batteryLevel() <= BATTERY_LEVEL2 && batteryStatus() <= BATTERY_STATUS_UNCHARGED) {
                qDebug() << QString("[Battery Level]:%1").arg(batteryLevel());
                emit requestToShowPopup(POPUP_DISABLE_POWER_SUPPLY);
                return;
            }

            // 給電確認
            if (_channelList[_currentChannelIndex].band == LC70_BAND_JCSAT3
                    || _channelList[_currentChannelIndex].band == LC70_BAND_JCSAT4
                    || _channelList[_currentChannelIndex].band == LC70_BAND_SCCB
                    || _channelList[_currentChannelIndex].band == LC70_BAND_SCCC) {
                // JCSAT, SCC は垂直11v水平15v, 15v固定, 給電しないを選択
                emit requestToShowPopup(POPUP_POWER_SUPPLY_SELECT3);
            } else {
                // その他は 15v, 給電しないを選択
                emit requestToShowPopup(POPUP_POWER_SUPPLY_SELECT2);
            }
        } else {
            // 給電停止
            _powerSupply = LC70_POWER_SUPPLY_OFF;
            _model->setPowerSupply(_powerSupply);
            // ヘッダーアイコン設定
            _model->removeMenuList(MenuListIconIndex::POWER_SUPPLY_ICON);
            // 給電中ではない
            _powerSupplied = false;
        }
    } else if (id == BUTTON_ESCAPE) {
        _model->removeMenuList(MenuListIconIndex::PREAMP_ICON);
        if (!_fromUserSetting) {
            // 画面共有情報の更新を実施
            emit changeAttribute("fromUserSetting", false);

            // ユーザ設定からの遷移以外はBAND選択画面に遷移する
            emit requestInHistory(SCREEN_BAND);
        } else {
            // 画面共有情報の更新を実施
            emit changeAttribute("fromUserSetting", false);

            // ユーザ設定からの遷移の場合は、ユーザ設定画面に遷移する
            AbstractViewModel::onKeyStatus(id);
        }
    } else {
        moveCursor(id);
    }
}

void SingleMeasurementViewModel::onButtonPress(ButtonId id)
{
    // メニューバー表示中
    if (isMenuBarVisible()) {
        if (id == BUTTON_ENTER) {
            if (_menuBarType == 1) {
                MainFrameViewModel *mainFrame = static_cast<MainFrameViewModel *>(_mainFrame);
                QVariantList vFileList = mainFrame->getUserFileList();
                qDebug() << "vFileList["  << _menuBarIndexMap.value(_menuBarRank) << "](fileName): " << vFileList[_menuBarIndexMap.value(_menuBarRank)].toString();
                qDebug() << "vFileList["  << _menuBarIndexMap.value(_menuBarRank) << "](isEmpty): " << vFileList[_menuBarIndexMap.value(_menuBarRank)].toString().isEmpty();
                if (vFileList[_menuBarIndexMap.value(_menuBarRank)].toString().isEmpty()) {
                    return;
                }
            }
            emit cursorPressed(_menuBarRank);
        }

        return;
    }

    if (id == BUTTON_ENTER) {
        setBoolProperty(_activeButton, "pressed", true);
    } else if (id == BUTTON_DATA_SAVE) {
        if (!_fromUserSetting) {
            if (_selectedButtonIndex == MEASUREMENT_BUTTON_INDEX) {
                setBoolProperty(DATA_BUTTON_NAME, "pressed", true);
            }
        }
    } else if (id == BUTTON_SCREEN_SAVE) {
        if (!_fromUserSetting) {
            setBoolProperty(IMAGE_BUTTON_NAME, "pressed", true);
        }
    }
}

void SingleMeasurementViewModel::resetSettings(int condition)
{
    _selectedButtonIndex = condition;

    // 表示クリア
    clear();

    // グラフからカーソルを外す
    _cursoredGraph = false;

    if (_delayProfile) {
        _delayProfile->setProperty("cursoredGraph", _cursoredGraph);
    }

    // 遅延プロファイルマーカー位置を初期位置へ移動
    moveProfileMarkerInitialPosition();
}

void SingleMeasurementViewModel::changeSettingToSingleMeasurement(bool isSuspend)
{
    if (!isSuspend) {
        // サスペンド時に設定を変更する際には、画面種別を変更しない
        // (遷移先での設定を邪魔するため)
        _model->changeDisplayType(LC70_DISP_TYPE_METER);
    }

    if (_meter) {
        _meter->setProperty("visible", true);
    }

    if (_constellation) {
        _constellation->setProperty("visible", false);
    }

    if (_delayProfile) {
        _delayProfile->setProperty("visible", false);
    }


    clearDelayProfileInfo();
}

void SingleMeasurementViewModel::onRequest(int condition)
{
    resetSettings(condition);

    // シングル測定、コンスタレーション、遅延プロファイルの表示を切り替える
    switch (condition) {
    case MEASUREMENT_BUTTON_INDEX:     // シングル測定
        // シングル測定結果の取得に設定
        _model->writeLog(QString("シングル測定"), QString("メーター表示"));

        changeSettingToSingleMeasurement();
        _model->changeAcquisitionTarget(SINGLE);

        // DATA2 表示
        setBoolProperty(DATA_BUTTON_NAME, "visible", true);

        // ロック待ち時間設定
        _lockWaitingTimer->setInterval(_model->applicationSetting()->waitingTimeForMeter());

        // コンスタレーションから移動してきたときは測定を止めない
        break;

    case CONSTELLATION_BUTTON_INDEX:    // コンスタレーション
        // コンスタレーションの取得に設定
        // ブザー鳴動中であればブザーを止める
        stopBuzzer();
        _model->writeLog(QString("シングル測定"), QString("コンスタレーション表示"));
        _model->changeDisplayType(LC70_DISP_TYPE_CONSTA);
        _model->changeAcquisitionTarget(CONSTELLATION);

        if (_meter) {
            _meter->setProperty("visible", false);
        }

        if (_constellation) {
            _constellation->setProperty("visible", true);
        }

        if (_delayProfile) {
            _delayProfile->setProperty("visible", false);
        }
        clearDelayProfileInfo();

        // DATA2 非表示
        setBoolProperty(DATA_BUTTON_NAME, "visible", false);

        // シングル測定から移動してきたときは測定を止めない

        // ロック待ち時間設定
        _lockWaitingTimer->setInterval(_model->applicationSetting()->waitingTimeForConstellation());

        break;

    case DELAY_PROFILE_BUTTON_INDEX:    // 遅延プロファイル
        // 遅延プロファイルの取得に変更
        // ブザー鳴動中であればブザーを止める
        stopBuzzer();
        _model->writeLog(QString("シングル測定"), QString("遅延プロファイル表示"));
        _model->changeDisplayType(LC70_DISP_TYPE_DLY_PRF);
        _model->changeAcquisitionTarget(DELAY_PROFILE);
        _isDelayProfileValid = true;

        if (_meter) {
            _meter->setProperty("visible", false);
        }

        if (_constellation) {
            _constellation->setProperty("visible", false);
        }

        if (_delayProfile) {
            _delayProfile->setProperty("visible", true);
        }

        // DATA2 非表示
        setBoolProperty(DATA_BUTTON_NAME, "visible", false);

        // ロック待ち時間設定
        _lockWaitingTimer->setInterval(_model->applicationSetting()->waitingTimeForDelayProfile());

        break;

    default:
        break;
    }
    // ロック待ち時間切れをリセット
    _waitforLockingTimeOut = false;
    _confirmedValue = false;

    // MER, BER 表示を "---" に
    emit notifyMerMeter(QString(INITIAL_MER), 0);
    emit notifyBerMeter(QString(INITIAL_BER), BER_METER_MINIMUM_POSITION);

    _lockWaitingTimer->setSingleShot(true);
    if (_lockWaitingTimer->isActive()) {
        _lockWaitingTimer->stop();
    }

    // 点滅時間設定
    if (_view) {
        _view->setProperty("blinkInterval", _model->applicationSetting()->blinkInterval());
        _view->setProperty("confirmedValue", _confirmedValue);
    }

    // ロック待ちタイマー開始
    _lockWaitingTimer->start();
}

void SingleMeasurementViewModel::onAccept(PopupId popupId, bool okOnly)
{
    Q_UNUSED(okOnly);

    AbstractViewModel::onAccept(popupId, okOnly);

    if (popupId == POPUP_POWER_SUPPLY_SELECT2) {
        if (!_fromUserSetting) {
            // 給電選択で 15v を選択
            _powerSupply = LC70_POWER_SUPPLY_15V;
        } else {
            // USER 設定の場合の給電設定
            selectPowerSupplyForUserSetting();
        }
        _model->setPowerSupply(_powerSupply);
        _model->replaceMenuList(MenuListIconIndex::POWER_SUPPLY_ICON, Correcter::powerSupplyIcon(_powerSupply));
        // 給電中
        _powerSupplied = true;

        return;
    } else if (popupId == POPUP_POWER_SUPPLY_ERROR_MANUAL_MEASURING) {
        // 給電異常音が鳴っていたら止める
        _powerSupplyErrorOccured = false;
        stopBuzzer();
        // (測定結果受領再開時に受信良好であれば再度受信良好音を鳴らす

#if 0
        // スレッド再開
        emit resumeSingleMeasuring(_limitContellationPoints, _condition,
                                   _model->applicationSetting()->pollingIntervalSingleMeasurement());
#endif
    } else if (popupId == POPUP_SELECT_SAVE_DATA) {
        // 保存先選択ダイアログで"ユーザ設定に保存"を選択した場合
        emit requestSetTakeOver("powerSupply", static_cast<int>(_powerSupply));
        emit requestSetTakeOver("dataloggerFunction", 6);
        emit requestSetTakeOver("selectedMeasurement", "SingleMeasurement");
        emit requestDirect(SCREEN_FILE_SELECTION, false);
        return;
    } else if (popupId == POPUP_SCAN_RESULT) {
        // SCAN結果保存確認ダイアログで"はい"を選択した場合
#if 1
        // オートパワーオフ設定を有効化
        _model->enablePowerOffSetting();
#endif
        emit requestSetTakeOver("powerSupply", static_cast<int>(_powerSupply));
        emit requestSetTakeOver("dataloggerFunction", 6);
#ifdef NEW_USER_SETTING
        // 保存先選択で USER データロガー設定を呼び出す
        emit requestSetTakeOver("userDataloggerFunction", CHANNEL_EDIT_SELECT_DESTINATION);
        emit requestDirect(SCREEN_USER_DATALOGGER_SETTING, false);
#else
        emit requestDirect(SCREEN_FILE_SELECTION, false);
#endif
        return;
    } else if (popupId == POPUP_POWER_SUPPLY_ERROR_USER_MEASURING) {
        // 給電異常音が鳴っていたら止める
        _powerSupplyErrorOccured = false;
        stopBuzzer();
        _powerSupplyErrorOccured = false;
        // (測定結果受領再開時に受信良好であれば再度受信良好音を鳴らす
    } else if (popupId == POPUP_FAILURE_READ_USER_SETTING_FILE) {
        // USER設定による測定開始前に戻る
        abortUserSettingMeasurement();
    } else if (popupId == POPUP_DATA_SINGLE_MAX_SAVE_FILE) {
        setBoolProperty(IMAGE_BUTTON_NAME, "pressed", false);
        setBoolProperty(DATA_BUTTON_NAME, "pressed", false);
    } else {
        // nothing to do
    }
}

void SingleMeasurementViewModel::onReject(PopupId popupId)
{
    if (popupId == POPUP_SCAN_RESULT) {
#if 1
        // オートパワーオフ設定を有効化
        _model->enablePowerOffSetting();
#endif
        _model->clearScannedChannelList();

        // 測定を開始する
        // スレッドを開始(方位角)
        emit resumeSingleMeasuring(_limitContellationPoints, _condition,
                                   _model->applicationSetting()->pollingIntervalSingleMeasurement());

        if (_view) {
            _view->setProperty("userMode", NO_FILE_NAME);
        }

        // チャンネル変更(測定開始)
        changeChannel();

        // 現在選択中の機能に切り替える
        onKeyStatus(BUTTON_ENTER);
        return;
    } else if (popupId == POPUP_INPUT_FILENAME) {
        // 保存するために保持した各データをクリアする
        emit requestToClearCaptureImage();
        clearHoldMeasurementData();
    }else if (popupId == POPUP_SELECT_SAVE_DATA) {
        // 保持している測定データをクリアする
        clearHoldMeasurementData();
    } else if (popupId == POPUP_FAILURE_READ_USER_SETTING_FILE) {
        // USER設定による測定開始前に戻る
        abortUserSettingMeasurement();
    }else {
        // nothing to do
    }
}

void SingleMeasurementViewModel::onRejectScanResult()
{
    // スキャンアイコンなし
    _model->removeMenuList(MenuListIconIndex::SCAN_ICON);

    if (_view) {
        _view->setProperty("userMode", "");
    }

    // スキャン後のチャンネル一覧をクリアー
    _model->clearScannedChannelList();

    // スキャン結果をクリアー
    _model->clearScanResult();

    // 全チャンネル再取得
    refreshAllChannels();

    // 先頭のチャンネルを指定
    _currentChannelIndex = 0;

    // 測定を開始
    onScanningFinished(false);
}

void SingleMeasurementViewModel::onSingleMeasureResult(MeasureResult result, QList<QVariant> constellations)
{
    if (_view == nullptr) {
        return;
    }

    //    _model->writeLog(QString("シングル測定"), QDateTime::currentDateTime().toString("hh:mm:ss.zzz") + QString(" [ENTER] 測定結果反映開始"), LOG_NOTICE);
    measuringTimer.restart();

    // チャンネル変更中か、チャンネル確定までのタイマーが動作中、またはチャンネル変更ボタン長押し中であれば届いた測定結果は表示しない
    if (_isChannelChanging
            || _confirmedChangeChannelTimer->isActive()
            || (_pressedChannelButtonId != BUTTON_UNKNOWN && AbstractViewModel::isKeyHolding(_pressedChannelButtonId))) {
        _model->writeLog(QString("シングル測定"), QString("%1 changing channel").arg(__FUNCTION__));
        return;
    }

    // 無効なインデックスの場合
    if (_currentChannelIndex < 0) {
        _model->writeLog(QString("シングル測定"), QString("%1 all invalid channels").arg(__FUNCTION__));
        return;
    }

    // チャンネル情報がなければ表示しない
    if (_channelList.count() < 1) {
        return;
    }

    // チャンネルが無効な場合
    if (!_channelList[_currentChannelIndex].valid) {
        _model->writeLog(QString("シングル測定"),
                         QString("%1 invalid channels(%2)").arg(__FUNCTION__).arg(_currentChannelIndex));
        return;
    }

    // SCAN中は測定結果を表示しない
    if (_isScanning) {
        return;
    }

    static int count = 0;
    static quint64 totalTime = 0;
    static int errorCount = 0;
    if (!result.Received) {
        ++errorCount;
    } else {
        errorCount = 0;
    }

    // 一定回数エラーが続いたらサブ CPU をリセットする
    if (errorCount == REPEAT_TIMES_TO_RESET) {
        errorCount = 0;
        _model->resetSubCpu();

        // 画面起動時の状態に戻す
        resetScreen();
        return;
    }

    timer.restart();

    _layer = result.Layer;
    if (Correcter::isSatelliteBroadCasting(_channelList[_currentChannelIndex].band)) {
        _layer = _channelList[_currentChannelIndex].polarization;
    }
    _timeStamp = result.Timestamp;

    _locked = result.Locked;
    if (result.Locked == LOCKED) {
        // ロック待ちのタイマーを止める
        if (_lockWaitingTimer->isActive()) {
            _lockWaitingTimer->stop();
        }
        _waitforLockingTimeOut = false;
        _confirmedValue = true; // 値確定

        // 点滅を止める
        if (_view) {
            _view->setProperty("blinkInterval", 0);
            _view->setProperty("confirmedValue", _confirmedValue);
        }

        //        // 点滅を止める
        //        if (_view) {
        //            _view->setProperty("blinkInterval", 0);
        //        }
        //        _waitforLockingTimeOut = false;
        //        _confirmedValue = true; // 値確定

        // 閾値の設定
        checkThresholdCondition(result.BroadcastSystem, result.Modulation);
        _model->writeLog(QString("シングル測定"), QDateTime::currentDateTime().toString("hh:mm:ss.zzz") + QString(" 閾値設定 更新 %1 ms").arg(measuringTimer.elapsed()));
    }

    if(_channelList[_currentChannelIndex].band== LC70_BAND_FM || _channelList[_currentChannelIndex].band == LC70_BAND_LTE ||
            _channelList[_currentChannelIndex].band == LC70_BAND_PILOT  || _channelList[_currentChannelIndex].band == LC70_BAND_CATV_UP_STREAM){


    }else{

        // 受信良好状態であった場合ブザーを連続鳴動させる
        if (_thresholdSet && _selectedButtonIndex == MEASUREMENT_BUTTON_INDEX) {
            if (checkGoodReception(result)) {
                _model->buzzer(LC70_BUZZ_VOL_CHG, LC70_BUZZ_MODE_REPEAT);
                // ブザー状態を変更
                _buzzerState = LC70_BUZZ_VOL_CHG;
                // GOOD アイコンを表示する
                if (_view) {
                    _view->setProperty("isGoodIconVisible", true);
                }
            } else {
                // 給電エラー発生中はブザーを止めないようにする
                if (!_powerSupplyErrorOccured) {
                    // GOOD アイコンを非表示にする。
                    stopBuzzer();
                }
            }
        }
    }

    bool isUpdateLevel = false;
    float32_t levelValue = -1;

    // 測定入力レベル
    // メータ中心部に表示する値は実データを表示する
    // 測定開始後一定時間経過後に更新する
    if (_displayLevel && result.Received) {
        isUpdateLevel = true;
        levelValue = result.Level;
    }

    // 遅延プロファイルである場合、ガードインターバルを表示する
    if (_model->acquisitionTarget() == DELAY_PROFILE) {
        if (result.Locked == LOCKED) {
            if (_view) {
                _view->setProperty("guardInterval", guardIntervalValue(result.GuardInterval));
            }
        }
    }

    // コンスタレーション表示中
    if (_model->acquisitionTarget() == CONSTELLATION) {
        static uint32_t totalConstellationPoints = 0;
        if (_view) {
            _view->setProperty("constellationData", constellations);
        }
        totalConstellationPoints += constellations.count();

        if (totalConstellationPoints >= _limitContellationPoints) {
            totalConstellationPoints = 0;
            _isConstellationDataClear = true;
        } else {
            _isConstellationDataClear = false;
        }

        if (result.Locked != LOCKED) {
            totalConstellationPoints = 0;
            _isConstellationDataClear = true;
        }

        if (_view) {
            _view->setProperty("isConstellationDataClear", _isConstellationDataClear);
        }
    }

    // 遅延プロファイル表示中
    if (_model->acquisitionTarget() == DELAY_PROFILE) {
        if (result.Locked != LOCKED) {
            clearDelayProfileInfo();
        }
    }

    if (_view) {
        _view->setProperty("locked", (result.Locked == LOCKED) ? true : false);
    }

    // 放送方式
    QString system = Correcter::displayBroadbandSystem(result.BroadcastSystem);
    if (result.Locked == LOCKED) {
        if (_view) {
            _view->setProperty("broadcastingSystem", system);
        }
    } else {
        QString value = QString("---");
        if (_waitforLockingTimeOut) {
            value = QString("---");
        }
        if (_view) {
            _view->setProperty("broadcastingSystem", value);
        }
    }

    if (_channelList[_currentChannelIndex].band == LC70_BAND_FM) {
        // FMの場合はFM固定
        if (_view) {
            _view->setProperty("displayModulationPrefixMode", DISPLAY_MODULATION_PREFIX_TOGETHER);
            _view->setProperty("modulation", FM);
            _view->setProperty("modulationPrefix", QString(""));
        }
    } else if (_channelList[_currentChannelIndex].band == LC70_BAND_PILOT
               || _channelList[_currentChannelIndex].band == LC70_BAND_CATV_UP_STREAM) {
        // PILOTの場合, CATV上りの場合はCW固定
        if (_view) {
            _view->setProperty("displayModulationPrefixMode", DISPLAY_MODULATION_PREFIX_TOGETHER);
            _view->setProperty("modulation", CW);
            _view->setProperty("modulationPrefix", QString(""));
        }
    } else {
        // 変調方式
        QString modulation = Correcter::displayModulation(result.Modulation);
        // 変調方式の接頭句
        QString modulationPrefix = Correcter::modulationPrefix(result.BroadcastSystem, modulation);
        // 変調方式と変調方式の接頭句をどのように表示するかを取得する
        int displayModulationMode = Correcter::displayModulationPrefixMode(result.BroadcastSystem);

        if (result.Locked == LOCKED) {
            _modulationInMeasureResult = result.Modulation;
            if (displayModulationMode == DISPLAY_MODULATION_PREFIX_TOGETHER) {
                if (modulationPrefix.isEmpty()) {
                    if (_view) {
                        _view->setProperty("modulation", modulation);
                    }
                } else {
                    if (_view) {
                        _view->setProperty("modulation", modulationPrefix + QString(" ") + modulation);
                    }
                }
                if (_view) {
                    _view->setProperty("modulationPrefix", QString(""));
                }
            } else if (displayModulationMode == DISPLAY_MODULATION_PREFIX_SEPARATE) {
                if (_view) {
                    _view->setProperty("modulation", modulation);
                    _view->setProperty("modulationPrefix", modulationPrefix);
                }
            } else {
                // nothing to do.
            }
            if (_view) {
                _view->setProperty("displayModulationPrefixMode", displayModulationMode);
            }
        } else {
            _modulationInMeasureResult = LC70_SETMOD_UNDEF;
            if (_view) {
                QString value = QString("---");
                if (_waitforLockingTimeOut) {
                    value = QString(UNLOCK_STRING);
                }

                _view->setProperty("displayModulationPrefixMode", DISPLAY_MODULATION_PREFIX_TOGETHER);
                _view->setProperty("modulation", value);
                _view->setProperty("modulationPrefix", QString(""));
            }
        }
        _model->writeLog(QString("シングル測定"),
                         QString("%1 layer: %2").arg(__FUNCTION__).arg(modulation));
    }
    _model->writeLog(QString("シングル測定"), QDateTime::currentDateTime().toString("hh:mm:ss.zzz") + QString(" 放送方式・変調方式 更新 %1 ms").arg(measuringTimer.elapsed()));

    // 階層
    // 放送方式が ISDBT のみ有効
    QString layer;

    if (result.BroadcastSystem == LC70_BD_SYS_ISDBT) {
        layer = Correcter::displayHierarchy(result.Layer);
        if (result.Locked == LOCKED) {
            if (_view) {
                _view->setProperty("hierarchy", layer);
            }
        } else {
            QString value = QString("---");
            if (_waitforLockingTimeOut) {
                value = QString("---");
            }
            if (_view) {
                _view->setProperty("hierarchy", value);
            }
        }
    } else {
        if (_view) {
            _view->setProperty("hierarchy", QString("---"));
        }
    }

    _model->writeLog(QString("シングル測定"),
                     QString("[シングル測定結果] 放送方式: %1 変調方式:%2 階層:%3").arg(result.BroadcastSystem).arg(result.Modulation).arg(result.Layer), LOG_INFO);


    if (!isUpdateLevel) {
        levelValue = _newestLevel;
    }

    _newestLevel = levelValue;
    _newestMer = updateMer(result);
    //    _model->writeLog(QString("シングル測定"), QDateTime::currentDateTime().toString("hh:mm:ss.zzz") + QString(" MER 更新 %1 ms").arg(measuringTimer.elapsed()), LOG_NOTICE);
    _newestBer = updateBer(result);
    //    _model->writeLog(QString("シングル測定"), QDateTime::currentDateTime().toString("hh:mm:ss.zzz") + QString(" BER 更新 %1 ms").arg(measuringTimer.elapsed()), LOG_NOTICE);

    if (!_isUpdateMeter) {
        onUpdateMeter();
        //        _model->writeLog(QString("シングル測定"), QDateTime::currentDateTime().toString("hh:mm:ss.zzz") + QString(" LEVEL 更新 %1 ms").arg(measuringTimer.elapsed()), LOG_NOTICE);
    }

    totalTime += timer.elapsed();
    ++count;

    if (count == 10) {
        _model->writeLog(QString("シングル測定"), QString("%1 %2 ms(average)").arg(__FUNCTION__).arg(totalTime / 10));
        count = 0;
        totalTime = 0;
    }

    //    _model->writeLog(QString("シングル測定"), QDateTime::currentDateTime().toString("hh:mm:ss.zzz") + QString(" [LEAVE] 測定結果反映完了 %1 ms").arg(measuringTimer.elapsed()), LOG_NOTICE);
}

void SingleMeasurementViewModel::onUpdateMeter()
{
    QString levelString;
    float32_t levelNeedle;
    updateLevelMeter(levelString, levelNeedle);

    QString merString;
    float32_t merNeedle;
    updateMerMeter(merString, merNeedle);

    QString berString;
    float32_t berNeedle;
    updateBerMeter(berString, berNeedle);

    // コンスタ、遅延プロファイルの場合は、ここで値を反映する
    if ((_selectedButtonIndex != MEASUREMENT_BUTTON_INDEX) || (_meterUpdateInterval == 0)) {
        _isUpdateMeter = false;
        return;
    }

    if ((_level != _newestLevel) || (_mer != _newestMer) || (_ber != _newestBer)) {
        _isUpdateMeter = true;
        QTimer::singleShot((_meterUpdateInterval), this, SLOT(onUpdateMeter()));
    } else {
        _isUpdateMeter = false;
    }
}

void SingleMeasurementViewModel::updateLevelMeter(QString &levelString, float32_t &levelNeedle) {
    // 現在値と最新値との差の大きさによって、針の移動量を決定するためのリスト
    static const QList<QPair<float32_t, float32_t>> levelResolutionList = {
        {20.0f, 16.0f},  // 例：差が20以上ある場合、針を動かす量は8ずつにする
        {10.0f, 8.0f},
        {5.0f,  2.0f},
        {3.0f,  1.2f},
        {2.0f,  0.8f},
        {1.0f,  0.4f},  // 例：差が1〜2の場合、針を動かす量は0.2ずつにする
        {0.0f,  0.2f},  // 例：それ未満は 0.1ずつにする
    };

    float32_t levelValue;
    bool isUndefinedValue = static_cast<int>(_newestLevel) == UNDEFINED_VALUE;
    float32_t newestValue = isUndefinedValue ? 0 : _newestLevel;

    if (newestValue != _level && (_selectedButtonIndex == MEASUREMENT_BUTTON_INDEX) && (_meterUpdateInterval != 0)) {
        float32_t difference = newestValue - _level;
        if (isUndefinedValue && (_invalidValueMeterSpeed == 0)) {
            // 無効時にアニメーションしない設定であれば、最新の値を採用する
            levelValue = newestValue;
        } else if (qIsNaN(difference) || qIsInf(difference)) {
            // 値の差がない場合は最新の値を採用する
            levelValue = newestValue;
        } else if ((0 < difference && difference < 1.0) ||
                   (-1.0 < difference && difference < 0)) {
            // 一目盛り分(±1.0 dBuV) のときは最新の値を採用する
            levelValue = newestValue;
            //_model->writeLog(QString("シングル測定"), QString("[NO ANIMATION] LEVEL %1 - %2 difference:%3").arg(newestValue).arg(_level).arg(difference), LOG_NOTICE);
        } else {
            // 差の大きさによって、一度に針を動かす量を決定する
            float32_t resolution = levelResolutionList.last().second;

            // 無効値が入力された場合用の特殊処理
            if (isUndefinedValue) {
                // 針の移動速度を、有効時と変え、倍率をかける
                resolution *= static_cast<float32_t>(_invalidValueMeterSpeed);
            }
            for (int i = 0 ; i < levelResolutionList.size(); ++i) {
                if (levelResolutionList.at(i).first < qAbs(difference)) {
                    resolution = levelResolutionList.at(i).second;
                    break;
                }
            }

            if (qAbs(difference) > resolution) {
                if (difference < 0) {
                    levelValue = _level - resolution;
                } else {
                    levelValue = _level + resolution;
                }
            } else {
                // 差が予定している針の移動量を下回る場合は、小さい方の値を採用する
                if (difference < 0) {
                    levelValue = _level - difference;
                } else {
                    levelValue = _level + difference;
                }
            }
            //_model->writeLog(QString("シングル測定"), QString("[ANIMATION] LEVEL %1 - %2 difference:%3").arg(newestValue).arg(_level).arg(difference), LOG_NOTICE);
        }
    } else {
        levelValue = newestValue;
    }

    _level = levelValue;

    // 現在のチャンネルから偏波面の取得を行う
    if (_channelList.count() == 0) {
        return;
    }
    LC70_BAND_T band = static_cast<LC70_BAND_T>(_channelList.at(_currentChannelIndex).band);
    QString polarize = Correcter::convertPolarizeNumToString(_channelList.at(_currentChannelIndex).polarization);
    BandTabIndex bandTab = Correcter::convertBandTabIndex(band, polarize);
    float32_t minimumLevel = 0;

    // 表示クリアー時のときは初期値設定
    if (isUndefinedValue) {
        levelValue = UNDEFINED_VALUE;
    }
    if (!Correcter::displayLevel(bandTab, levelValue, levelString)) {
        qDebug() << QString("displayLevel() failed");
    }

    if (!Correcter::minimumLevel(bandTab, minimumLevel)) {
        qDebug() << QString("minimumLevel() failed");
    }

    // 最大値は針の最大値から
    if (levelValue > minimumLevel && levelValue <= MAXIMUM_LEVEL) {
        levelNeedle = LEVEL_NEEDLE_OFFSET + ((levelValue - minimumLevel) * LEVEL_NEEDLE_STEP);
        if(minimumLevel == 30){
            levelNeedle = levelNeedle + (minimumLevel-6);
        }else if(minimumLevel == 40){
            levelNeedle = levelNeedle + (minimumLevel+11);
        }
    } else if (levelValue > MAXIMUM_LEVEL) {
#if 1	//20210705_dx
        levelNeedle = LEVEL_NEEDLE_OFFSET + ((MAXIMUM_LEVEL - 20) * LEVEL_NEEDLE_STEP);
#else
        levelNeedle = LEVEL_NEEDLE_OFFSET + ((MAXIMUM_LEVEL - minimumLevel) * LEVEL_NEEDLE_STEP);
#endif
    } else {
        if (minimumLevel == 30 || minimumLevel == 40) {
            // 20 はメーターの目盛りの最小値
            if (levelValue > 20) {
                levelNeedle = LEVEL_NEEDLE_OFFSET + ((levelValue - 20) * LEVEL_NEEDLE_STEP);
            } else {
                levelNeedle = LEVEL_METER_MINMUM_POSITION;
            }
        } else {
            levelNeedle = LEVEL_METER_MINMUM_POSITION;
        }
    }

    emit notifyLevelMeter(levelString, levelNeedle);
}

void SingleMeasurementViewModel::updateMerMeter(QString &merString, float32_t &merNeedle) {
    // 現在値と最新値との差の大きさによって、針の移動量を決定するためのリスト（LEVELと同じ）
    static const QList<QPair<float32_t, float32_t>> merResolutionList = {
        {10.0f, 8.0f},
        {5.0f,  4.0f},
        {2.5f,  2.5f},
        {1.5f,  1.2f},
        {1.0f,  0.8f},
        {0.5f,  0.4f},
        {0.0f,  0.2f},
    };

    if (_channelList.isEmpty()) {
        // 画面を離れるタイミングで、メータの更新が実施された場合は無視する
        // (画面を離れるタイミングでチャンネルリストを空にしている)
        return;
    }

    float32_t merValue;
    bool isUndefinedValue = static_cast<int>(_newestMer) == UNDEFINED_VALUE;
    bool isUnlockedValue = static_cast<int>(_newestMer) == UNLOCKED_VALUE;
    float32_t newestValue = isUndefinedValue ? 0.0f : _newestMer;

    if (newestValue != _mer && (_selectedButtonIndex == MEASUREMENT_BUTTON_INDEX) && (_meterUpdateInterval != 0)) {
        float32_t difference = newestValue - _mer;
        if (isUndefinedValue && (_invalidValueMeterSpeed == 0)) {
            // 無効時にアニメーションしない設定であれば、最新の値を採用する
            merValue = newestValue;
        } else if (qIsNaN(difference) || qIsInf(difference)) {
            // 値の差がない場合は最新の値を採用する
            merValue = newestValue;
        } else if ((0 < difference && difference < 0.5) ||
                   (-0.5 < difference && difference < 0)) {
            // 一目盛り分(±0.5 dBuV) のときは最新の値を採用する
            merValue = newestValue;
            //_model->writeLog(QString("シングル測定"), QString("[NO ANIMATION] MER %1 - %2 difference:%3").arg(newestValue).arg(_mer).arg(difference), LOG_NOTICE);
        } else {
            // 差の大きさによって、一度に針を動かす量を決定する
            float32_t resolution = merResolutionList.last().second;

            // 無効値が入力された場合用の特殊処理
            if (isUndefinedValue) {
                // 針の移動速度を、有効時と変え、倍率をかける
                resolution *= static_cast<float32_t>(_invalidValueMeterSpeed);
            }

            for (int i = 0 ; i < merResolutionList.size(); ++i) {
                if (merResolutionList.at(i).first < qAbs(difference)) {
                    resolution = merResolutionList.at(i).second;
                    break;
                }
            }

            if (qAbs(difference) > resolution) {
                if (difference < 0.0f) {
                    merValue = _mer - resolution;
                } else {
                    merValue = _mer + resolution;
                }
            } else {
                // 差が予定している針の移動量を下回る場合は、小さい方の値を採用する
                if (difference < 0.0f) {
                    merValue = _mer - difference;
                } else {
                    merValue = _mer + difference;
                }
            }

            // 値が十分小さくなったら0に落とす
            if (merValue < 1.0e-2) {
                merValue = 0.0f;
            }
            //_model->writeLog(QString("シングル測定"), QString("[ANIMATION] MER %1 - %2 difference:%3").arg(newestValue).arg(_mer).arg(difference), LOG_NOTICE);
        }
    } else {
        merValue = newestValue;
    }

    _mer = merValue;
    // ロック待ち時間のタイムアウト後
    if (_waitforLockingTimeOut) {
        _mer = -1;
    }

    if (merValue < static_cast<float>(_merRange.first)) {
        if (isUndefinedValue) {
            merString = QString(INITIAL_BER);
        } else if (isUnlockedValue) {
            merString = QString(INITIAL_MER);
        }
        // ロック待ち時間のタイムアウト後か値確定後
        if (_waitforLockingTimeOut || _confirmedValue) {
            if (_broadcastSystem == LC70_BD_SYS_ANNEXB || _broadcastSystem == LC70_BD_SYS_ANNEXC || _broadcastSystem == LC70_BD_SYS_OPTICAST){

                merString = "<20.0";

            }else{
                merString = QString(MER_TIMEOUT_VALUE);
            }

            //merString = Correcter::displayMer(_channelList[_currentChannelIndex].band, _broadcastSystem, _modulationInMeasureResult, merValue);
        }
    } else {
        merString = Correcter::displayMer(_channelList[_currentChannelIndex].band, _broadcastSystem, _modulationInMeasureResult, merValue);
#if 1
        /*if(merString == "<21.0") {
            merString = "<21";
        }else if(merString == "<27.0") {
            merString = "<27";
        }else if(merString == ">30.0"){
            merString= ">30";
        }else if(merString == ">35.0"){
            merString = ">35";
        }else if(merString == ">40.0"){
            merString = ">40";

        }*/

#endif
    }

    merNeedle = updateMerNeedle(merValue);
    emit notifyMerMeter(merString, merNeedle);
}

void SingleMeasurementViewModel::updateBerMeter(QString &berString, float32_t &berNeedle) {
    // 現在値と最新値との差の大きさによって、針の移動量を決定するためのリスト
    // LEVEL、MERと異なり、目盛り上の移動量について定義している
    static const QList<QPair<float32_t, float32_t>> berResolutionList = {
        {2.0f, 48.0f},  // 例：差が2目盛り以上ある場合、24度針を動かす
        {1.0f, 24.0f},
        {0.5f, 12.0f},
        {0.3f,  3.6f},
        {0.2f,  2.4f},
        {0.1f,  1.2f},
        {0.0f,  0.6f},
    };

    float32_t currentValue = (static_cast<int>(_ber) == UNDEFINED_VALUE) ? BER_METER_INVALID_VALUE : _ber;
    currentValue = (static_cast<int>(_ber) == UNLOCKED_VALUE) ? BER_METER_INVALID_VALUE : currentValue;
    float32_t currentNeedle = updateBerNeedle(currentValue);
    bool isUndefinedValue = static_cast<int>(_newestBer) == UNDEFINED_VALUE;
    bool isUnlockedValue = static_cast<int>(_newestBer) == UNLOCKED_VALUE;
    float32_t newestValue = isUndefinedValue ? BER_METER_INVALID_VALUE : _newestBer;
    newestValue = isUnlockedValue ? BER_METER_INVALID_VALUE : newestValue;
    float32_t newestNeedle = updateBerNeedle(newestValue);

    // 現在の針の位置と、最新の針の位置を比較し、移動した距離を求める
    float32_t move = (qAbs(qAbs(newestNeedle) - qAbs(currentNeedle)) / 30);
    float32_t difference = qAbs(newestNeedle) - qAbs(currentNeedle);

    float32_t berValue;
    float32_t resolution;
    if (isUndefinedValue && (_invalidValueMeterSpeed == 0)) {
        // 無効時にアニメーションしない設定であれば、最新の値を採用する
        berValue = newestValue;
        berNeedle = updateBerNeedle(berValue);
    } else if (qIsNaN(move) || qIsInf(move)) {
        berValue = newestValue;
        berNeedle = updateBerNeedle(berValue);
    } else if (qAbs(newestValue - currentValue) <= 1.0e-8 ) {
        // 差がない場合は最新値をそのまま採用する
        berValue = newestValue;
        berNeedle = updateBerNeedle(berValue);
    } else if (move <= 1.0e-8) {
        // 差がない場合は最新値をそのまま採用する
        berValue = newestValue;
        berNeedle = updateBerNeedle(berValue);
    } else if ((_selectedButtonIndex != MEASUREMENT_BUTTON_INDEX) || (_meterUpdateInterval == 0)) {
        // コンスタレーションや遅延プロファイルでは、最新値をそのまま採用する
        berValue = newestValue;
        berNeedle = updateBerNeedle(berValue);
    } else if ((0 < difference && difference < 6.0) ||
               (-6.0 < difference && difference < 0)) {
        // 一目盛り分(6度)未満のときは最新の値を採用する
        berValue = newestValue;
        berNeedle = updateBerNeedle(berValue);
        _model->writeLog(QString("シングル測定"), QString("[NO ANIMATION] BER %1 - %2 difference:%3").arg(qAbs(newestNeedle)).arg(qAbs(currentNeedle)).arg(difference), LOG_NOTICE);
    } else {
        // 差の大きさによって、一度に針を動かす量を決定する
        resolution = berResolutionList.last().second;

        for (int i = 0 ; i < berResolutionList.size(); ++i) {
            if (berResolutionList.at(i).first < qAbs(move)) {
                resolution = berResolutionList.at(i).second;
                break;
            }
        }

        // 無効値が入力された場合用の特殊処理
        if (isUndefinedValue) {
            // 針の移動速度を、有効時と変え、倍率をかける
            resolution *= static_cast<float32_t>(_invalidValueMeterSpeed);
        }

        // 現在の針の角度に、針の移動量を加え、表示したい針の角度を求める
        float32_t nextNeedle = currentNeedle;
        if (currentValue > newestValue) {
            nextNeedle -= resolution;
        } else {
            nextNeedle += resolution;
        }

        // 針の角度から BER の値を逆算する
        if (-180.0f > nextNeedle) {
            nextNeedle = -180.0f;
            berValue = 0;
        } else if (-180.0f <= nextNeedle && nextNeedle < -150.0f) {
            berValue = 1.0e-8;
            berValue += ((1.0e-7 - 1.0e-8) / 30.0f) * (180.0f - qAbs(nextNeedle));
        } else if (-150.0f <= nextNeedle && nextNeedle < -120.0f) {
            berValue = 1.0e-7;
            berValue += ((1.0e-6 - 1.0e-7) / 30.0f) * (150.0f - qAbs(nextNeedle));
        } else if (-120.0f <= nextNeedle && nextNeedle < -90.0f) {
            berValue = 1.0e-6;
            berValue += ((1.0e-5 - 1.0e-6) / 30.0f) * (120.0f - qAbs(nextNeedle));
        } else if (-90.0f <= nextNeedle && nextNeedle < -60.0f) {
            berValue = 1.0e-5;
            berValue += ((1.0e-4 - 1.0e-5) / 30.0f) * (90.0f - qAbs(nextNeedle));
        } else if (-60.0f <= nextNeedle && nextNeedle < -30.0f) {
            berValue = 1.0e-4;
            berValue += ((1.0e-3 - 1.0e-4) / 30.0f) * (60.0f - qAbs(nextNeedle));
        } else if (-30.0f <= nextNeedle && nextNeedle < 0.0f) {
            berValue = 1.0e-3;
            berValue += ((1.0e-2 - 1.0e-3) / 30.0f) * (30.0f - qAbs(nextNeedle));
        } else {
            nextNeedle = 0.0f;
            berValue = 1.0e-2;
        }
        berNeedle = nextNeedle;
        _model->writeLog(QString("シングル測定"), QString("[ANIMATION] BER %1 - %2 difference:%3").arg(qAbs(newestNeedle)).arg(qAbs(currentNeedle)).arg(difference), LOG_NOTICE);
    }

    _ber = berValue;
    // ロック待ち時間のタイムアウト後
    if (_waitforLockingTimeOut) {
        _ber = -1;
    }

    if ((qAbs(BER_METER_INVALID_VALUE - berValue) <= 1.0e-8)) {
        if (isUndefinedValue) {
            berString = QString(INITIAL_BER);
        } else if (isUnlockedValue) {
            berString = QString(INITIAL_BER);
        }
        // ロック待ち時間のタイムアウト後か値確定後
        if (_waitforLockingTimeOut || _confirmedValue) {
            berString = QString(BER_TIMEOUT_VALUE);
        }
    } else {
        berString = Correcter::displayBer(berValue);
    }

    emit notifyBerMeter(berString, berNeedle);
}

void SingleMeasurementViewModel::onInputFileName(PopupId popupId, SaveLocation saveLocation, QString fileName)
{
    Q_UNUSED(popupId);

    QString prefix;

    if (saveLocation == SAVE_MEASUREMENT_DATA) {
#if 1	//20210625_dx5
        prefix = "_" + QDateTime::currentDateTime().toString("yyyyMMddhhmmss") + "_" + _takeOver["selectedBand"].toString();
#else
        prefix = "_" + QDateTime::currentDateTime().toString("yyyyMMddhhmm") + "_" + _takeOver["selectedBand"].toString();
#endif
    } else {
        prefix = "_" + _takeOver["selectedBand"].toString();
    }

    switch (_saveFileType) {
    // 測定データ
    case CSV: {
        fileName = fileName + prefix + CSV_FILE_EXTENSION;
        emit requestToSaveData(saveLocation, fileName, _condition.usCondition, _holdMeasurementData);

        // 保持している測定データをクリアする
        clearHoldMeasurementData();

        break;
    }

        // 画像
    case PNG:
        fileName = fileName + prefix + PNG_FILE_EXTENSION;
        // キャプチャーした一時データを保存する
        emit requestToSaveImage(SINGLE_MEASUREMENT, saveLocation, fileName);
        break;

    default:
        break;
    }

    _saveFileType = SaveFileType::SAVE_FILE_COUNT;
}

void SingleMeasurementViewModel::onDelayProfileResult(QVector<QPointF> result)
{
    _model->writeLog(QString("シングル測定"),
                     QString("%1 delay profile received %2").arg(__FUNCTION__).arg(result.count()));

    if (!_isDelayProfileValid) {
        _model->writeLog(QString("シングル測定"),
                         QString("delay profile invalid data").arg(__FUNCTION__).arg(result.count()));
        return;
    }

    // 遅延プロファイルのグラフに反映
    if (_lineSeries) {
        _lineSeries->setLineSeries(result);
    }

    if (result.size() < LC70_DLY_PRF_NUM) {
        return;
    }

    if (_view) {
        // 遅延時間
        _view->setProperty("delayTime", getDelayTime());

        // レベル差
        _view->setProperty("levelDifference", result.at(_delayTimeIndex).y() - result.at(DELAY_PROFILE_ZERO_INDEX).y());
    }
}

void SingleMeasurementViewModel::onNotifyChannelChanging(bool changing)
{
    qDebug() << __FUNCTION__ << changing;
    _isChannelChanging = changing;

    if (_condition.ulFreq != _previousCondition.ulFreq ||
            _condition.ulLoFreq != _previousCondition.ulLoFreq ||
            _condition.usBdSys != _previousCondition.usBdSys ||
            _condition.usBand != _previousCondition.usBand ||
            _condition.usMod != _previousCondition.usMod ||
            _condition.usCondition != _previousCondition.usCondition ||
            QString::fromLocal8Bit((char *)_condition.ucStrChan) != QString::fromLocal8Bit((char *)_previousCondition.ucStrChan) ||
            _condition.ucOutputEnb != _previousCondition.ucOutputEnb ||
            _condition.powerSupply != _previousCondition.powerSupply) {
        qDebug() << QString("[Change condition again]");
        emit changeCondition(_condition);
        _previousCondition = _condition;
        return;
    }
    if (!changing) {
        // MER, BER 表示を "---" に
        emit notifyMerMeter(QString(INITIAL_MER), 0);
        emit notifyBerMeter(QString(INITIAL_BER), BER_METER_MINIMUM_POSITION);

        // ロック待ち時間切れをリセット
        _waitforLockingTimeOut = false;

        // 値未確定
        _confirmedValue = false;

        // ロック待ち時間設定
        switch (_selectedButtonIndex) {
        case MEASUREMENT_BUTTON_INDEX:     // シングル測定
            _lockWaitingTimer->setInterval(_model->applicationSetting()->waitingTimeForMeter());
            break;
        case CONSTELLATION_BUTTON_INDEX:    // コンスタレーション
            _lockWaitingTimer->setInterval(_model->applicationSetting()->waitingTimeForConstellation());
            break;
        case DELAY_PROFILE_BUTTON_INDEX:    // 遅延プロファイル
            _lockWaitingTimer->setInterval(_model->applicationSetting()->waitingTimeForDelayProfile());
            break;
        default:
            break;
        }

        // 点滅時間設定
        if (_view) {
            _view->setProperty("blinkInterval", _model->applicationSetting()->blinkInterval());
            _view->setProperty("confirmedValue", _confirmedValue);
        }

        _lockWaitingTimer->setSingleShot(true);
        if (_lockWaitingTimer->isActive()) {
            _lockWaitingTimer->stop();
        }

        // ロック待ちタイマー開始
        _lockWaitingTimer->start();
    }
}

void SingleMeasurementViewModel::onRequestToScan()
{
    // ユーザー設定は再スキャン不可
    if (_fromUserSetting) {
        return;
    }

    if (_view) {
        _view->setProperty("userMode", "");
    }

    // 保持したスキャン結果をクリアー
    _model->clearScanResult();

    // チャンネル情報を元に戻す
    _channelList.clear();

    for (int i = 0; i < _preservedChannelList.count(); ++i) {
        _channelList.append(_preservedChannelList[i]);
    }

    // 局発周波数
    uint32_t localFrequency = _takeOver["selectedLocalOscillator"].toUInt();

    // 変調方式
    unsigned int modulationValue = getModulationValue();
    QString modulation =  Correcter::displayModulation(modulationValue);
    uint16_t bdSys = Correcter::getBroadcastSystemCondition(_channelList[_currentChannelIndex].band, modulation);

    // 放送方式、変調方式の設定(定義ファイル 1.0.1 対応)
    uint16_t mod = Correcter::getModulationCondition(_channelList[_currentChannelIndex].band, bdSys, _channelList.at(_currentChannelIndex).polarization, modulationValue, _broadBandModulationSetting);

#if 1
    // オートパワーオフ設定を無効化
    _model->disablePowerOffSetting();
#endif
    // 表示クリア
    clear();

    // スキャンアイコンなし
    _model->removeMenuList(MenuListIconIndex::SCAN_ICON);

    // ユーザーモードクリア
    if (_view) {
        _view->setProperty("userMode", "");
    }

    // スキャンのときは測定画面以外にする
    _model->changeDisplayType(LC70_DISP_TYPE_NOT_MEAS);

    // スキャン開始
    _isScanning = true;
#if 1
    // 画面共有情報からBandを取得
    int bandId = _takeOver["selectedBand"].toInt();

    if (bandId == LC70_BAND_BS_CS) {
        _model->scan_bscs(_channelList, _model->acquisitionTarget(), bdSys, mod, true, _powerSupply);
    }
    else {
        _model->scan(_channelList, _model->acquisitionTarget(), localFrequency, bdSys, mod, true, _powerSupply);
    }
#else
    _model->scan(_channelList, _model->acquisitionTarget(), localFrequency, bdSys, mod, true, _powerSupply);
#endif

    // スキャン処理実行中は、シングル測定ができないため、
    // ブザー音が鳴動している場合は止める
    stopBuzzer();

    _model->writeLog(QString("シングル測定"),
                     QString("スキャン開始 変調方式:%1 局発周波数:%2").arg(modulation).arg(localFrequency));
}

void SingleMeasurementViewModel::onRequestToStopScanning()
{
    // ユーザー設定は再スキャン不可
    if (_fromUserSetting) {
        return;
    }

    // スキャン中止
    _model->stopChannelScan();
    _isScanning = false;
    _model->writeLog(QString("シングル測定"), QString("スキャン中止"));

    // スキャン結果を破棄
    _model->clearScanResult();

    // 全チャンネル再取得
    refreshAllChannels();

    // 遷移先画面種別を元に戻す
    switch (_selectedButtonIndex) {
    case MEASUREMENT_BUTTON_INDEX:     // シングル測定
        // シングル測定結果の取得に設定
        _model->changeDisplayType(LC70_DISP_TYPE_METER);
        break;

    case CONSTELLATION_BUTTON_INDEX:    // コンスタレーション
        // コンスタレーションの取得に設定
        _model->changeDisplayType(LC70_DISP_TYPE_CONSTA);
        break;

    case DELAY_PROFILE_BUTTON_INDEX:    // 遅延プロファイル
        // 遅延プロファイルの取得に変更
        _model->changeDisplayType(LC70_DISP_TYPE_DLY_PRF);
        break;

    default:
        break;
    }
}

void SingleMeasurementViewModel::onScanningChannelResult(int index, LC70_MEAS_MULTI_RESULT_T result)
{
//    static int count = 0;

    // チャンネルがない場合はスキャン終了
    if (_channelList.count() == 0) {
        onScanningFinished(true);
        return;
    }

    if ((result.ucLock == UNLOCKED) || (result.ulCnr <= 500)) {

        // 無効判定
        _channelList[index].valid = false;
    }

    // SCAN時設定した変調方式がOFDMかつ、
    // スキャン結果がOFDM以外の変調方式の場合
    // スキャン結果として登録する変調方式をOFDMに設定する
    int registModulation = result.ulModBitPtrn;
    if(getModulationValue() == LC70_SETMOD_OFDM && result.ulModBitPtrn != LC70_SETMOD_OFDM) {
        registModulation = LC70_SETMOD_OFDM;
    }

    // スキャン結果の保持
    int lockFlag = false;
    // ロックされ、MER が 5.0 以上であれば有効
    if (result.ucLock == LOCKED && result.ulCnr > 500) {
        MeasureResult scanResult;
        scanResult.Received = true;
        scanResult.Band = QString::number(_channelList[index].band);
        scanResult.Channel = _channelList[index].channelNumber;
        scanResult.Status = result.usStat;
        scanResult.Level = result.ulLevel;
        scanResult.Mer = 0;
        scanResult.Ber = 0;
        scanResult.BroadcastSystem = result.usBdSys;
        scanResult.Modulation = registModulation;
        scanResult.Layer = result.ucLayer;
        scanResult.Frequency = -1;
        scanResult.LocalOscillator = getLocalFrequency();
        _model->appendScanResult(scanResult);

        // スキャン時 有効チャンネルが見つかった場合 ブザーを1度鳴動させる(1 回だけ鳴動なのでブザー状態は変えない)
#if 1
        _model->buzzer(LC70_BUZZ_1CH_SCAN, LC70_BUZZ_MODE_ONESHOT);
#else
        _model->buzzer(LC70_BUZZ_VOL_CHG, LC70_BUZZ_MODE_ONESHOT);
#endif
        lockFlag = true;
    }

    emit progressText(_channelList[index].channelNumber, lockFlag);

    // プログレスバーの進捗を設定
    qDebug() << QString("total: %1 current: %2").arg(_channelList.count()).arg(index);
    double value = static_cast<double>(index + 1) / static_cast<double>(_channelList.count()) * 100;
    emit progress(value);

// 給電デバッグ用
//    ++count;
//    if (count == 1) {
//        dynamic_cast<MainFrameViewModel *>(_mainFrame)->test();
//    } else if (count == 4) {
//        dynamic_cast<MainFrameViewModel *>(_mainFrame)->test();
//        count = 0;
//    }
}

void SingleMeasurementViewModel::onScanningFinished(bool complete)
{
    _model->writeLog(QString("シングル測定"), QString("スキャン終了"));

    _isScanning = false;
    QList<MeasureResult> result = _model->allScanResult();

    for (int i = 0; i < result.count(); ++i) {
        QString log =
                QString("有効チャンネル Channel:%1 Status:%2 Broadcast:%3 Modulation:%4 Local:%5 Level:%6 MER:%7 BER:%8").arg(
                    result[i].Channel).arg(result[i].Status).arg(result[i].BroadcastSystem).arg(result[i].Modulation).arg(
                    result[i].LocalOscillator).arg(result[i].Level).arg(result[i].Mer).arg(result[i].Ber);
        _model->writeLog(QString("シングル測定"), log, LOG_INFO);
    }

    bool validChannelExists = false;

    // スキャンした結果の中に有効なチャンネルが存在するか
    for (int i = 0; i < _channelList.count(); ++i) {
        if (_channelList[i].valid) {
            validChannelExists = true;
            break;
        }
    }

    // スキャンした結果全て無効であれば全チャンネルを有効に戻す
    if (!validChannelExists) {
        for (int i = 0; i < _channelList.count(); ++i) {
            _channelList[i].valid = true;
        }
    }

    switch (_selectedButtonIndex) {
    case MEASUREMENT_BUTTON_INDEX:
        // シングル測定結果の取得に設定
        _model->changeDisplayType(LC70_DISP_TYPE_METER);
        _model->changeAcquisitionTarget(SINGLE);
        break;

    case CONSTELLATION_BUTTON_INDEX:
        // コンスタレーション測定結果の取得に設定
        _model->changeDisplayType(LC70_DISP_TYPE_CONSTA);
        _model->changeAcquisitionTarget(CONSTELLATION);
        break;

    case DELAY_PROFILE_BUTTON_INDEX:
        // 遅延プロファイル測定結果の取得に設定
        _model->changeDisplayType(LC70_DISP_TYPE_DLY_PRF);
        _model->changeAcquisitionTarget(DELAY_PROFILE);
        break;

    default:
        return;
    }

    // スレッドを開始する前に_conditionの設定を画面遷移時と同じ状態に戻すを初期化する
    memset(&_condition, 0x00, sizeof(LC70_MEAS_SINGLE_T));

    if (complete) {
        // スキャン完了時 ブザーを1度鳴動させる(1 回だけ鳴動なのでブザー状態は変えない)
        _model->buzzer(LC70_BUZZ_ALL_CH_SCAN, LC70_BUZZ_MODE_ONESHOT);
        emit requestToShowPopup(POPUP_SCAN_RESULT);

        // スキャンアイコンを表示する
        _model->replaceMenuList(MenuListIconIndex::SCAN_ICON, "icon_scan_n.png");
        if (_view) {
            _view->setProperty("userMode", NO_FILE_NAME);
        }

        // 有効なチャンネルがあればその先頭に移動
        // 有効なチャンネルの先頭に設定
        for (int i = 0; i < _channelList.count(); ++i) {
            if (isValidChannel(i)) {
                _currentChannelIndex = i;
                break;
            }
        }

        // スキャン後の有効なチャンネル一覧を保存
        _model->setScannedChannelList(_channelList, _takeOver["selectedPolarization"].toString());

        // 局初周波数を引き継ぐ
        emit requestSetTakeOver("selectedLocalOscillator", _takeOver["selectedLocalOscillator"]);
    } else {
#if 1
        // オートパワーオフ設定を有効化
        _model->enablePowerOffSetting();
#endif
        _model->removeMenuList(MenuListIconIndex::SCAN_ICON);
        // キャンセルした場合は、ここで測定を再開する
        // スレッドを開始(方位角)
        emit resumeSingleMeasuring(_limitContellationPoints, _condition,
                                   _model->applicationSetting()->pollingIntervalSingleMeasurement());

        // チャンネル変更(測定開始)
        changeChannel();
    }
}

void SingleMeasurementViewModel::onSelect(PopupId popupId, int index)
{
    if (popupId == POPUP_SCAN_RESULT) {
        // スキャン結果を採用する
        selectReactionScanResult(index);
        return;
    }

    // 現在のチャンネルから偏波面の取得を行う
    QString polarize = Correcter::convertPolarizeNumToString(_channelList.at(_currentChannelIndex).polarization);

    if (index == FIXED_15V) {       // 15v固定 給電
        _powerSupply = LC70_POWER_SUPPLY_15V;
        _model->setPowerSupply(_powerSupply);
        _model->replaceMenuList(MenuListIconIndex::POWER_SUPPLY_ICON, Correcter::powerSupplyIcon(_powerSupply));
        _powerSupplied = true;        
    } else if (index == H15V11) {   // 水平15v/垂直11v 給電
        if (_channelList[_currentChannelIndex].band == LC70_BAND_JCSAT4) {
            // 偏波による分岐
            if (polarize == HORIZONTALLY_POLARIZE) {
                _powerSupply = LC70_POWER_SUPPLY_15V_P;
                _model->setPowerSupply(_powerSupply);
                _model->replaceMenuList(MenuListIconIndex::POWER_SUPPLY_ICON, Correcter::powerSupplyIcon(_powerSupply));
            } else if (polarize == VERTICALLY_POLARIZE) {
                _powerSupply = LC70_POWER_SUPPLY_11V_P;
                _model->setPowerSupply(_powerSupply);
                _model->replaceMenuList(MenuListIconIndex::POWER_SUPPLY_ICON, Correcter::powerSupplyIcon(_powerSupply));
            }
        } else if (_channelList[_currentChannelIndex].band == LC70_BAND_JCSAT3 ||
                   _channelList[_currentChannelIndex].band == LC70_BAND_SCCB ||
                   _channelList[_currentChannelIndex].band == LC70_BAND_SCCC) {

            // 偏波による分岐
            if (polarize == HORIZONTALLY_POLARIZE) {
                _powerSupply = LC70_POWER_SUPPLY_15V;
                _model->setPowerSupply(_powerSupply);
                _model->replaceMenuList(MenuListIconIndex::POWER_SUPPLY_ICON, Correcter::powerSupplyIcon(_powerSupply));
            } else if (polarize == VERTICALLY_POLARIZE) {
                _powerSupply = LC70_POWER_SUPPLY_11V;
                _model->setPowerSupply(_powerSupply);
                _model->replaceMenuList(MenuListIconIndex::POWER_SUPPLY_ICON, Correcter::powerSupplyIcon(_powerSupply));
            }
        } else {
            _powerSupply = LC70_POWER_SUPPLY_15V;
            _model->setPowerSupply(_powerSupply);
            _model->replaceMenuList(MenuListIconIndex::POWER_SUPPLY_ICON, Correcter::powerSupplyIcon(_powerSupply));
        }
        _powerSupplied = true;
    }
}

void SingleMeasurementViewModel::onNotifyButtonInput(PopupId popupId, ButtonId buttonId)
{
    Q_UNUSED(popupId);

    // ポップアップ表示中に入力されたボタンの処理
    onKeyStatus(buttonId);
}

void SingleMeasurementViewModel::moveCursor(ButtonId buttonId)
{
    if (_lineSeries == nullptr) {
        return;
    }

    if ((buttonId == BUTTON_RIGHT) || (buttonId == BUTTON_LEFT)) {

        if (_comboboxState == Edit) {
            // コンボボックス操作
            QString nextButton;
            bool ret = _buttonOperation->nextActiveObject(buttonId, nextButton);

            if (ret) {
                if (_combobox) {
                    // inactive
                    setButtonActive(_combobox, _activeButton, false);

                    // active
                    setButtonActive(_combobox, nextButton, true);
                }
                _activeButton = nextButton;
                _buttonOperation->setActiveObject(_activeButton);
            }
            return;
        }

        if (_cursoredGraph) {
            // 遅延プロファイルのグラフ操作
            if (buttonId == BUTTON_RIGHT) {

                switch (_pressDownState) {
                case PRESS_DOWN_LEVEL_0:
                    ++_delayTimeIndex;
                    break;
                case PRESS_DOWN_LEVEL_1:
                    _delayTimeIndex += _model->applicationSetting()->delayProfileSkipIndexLow();
                    break;
                case PRESS_DOWN_LEVEL_2:
                    _delayTimeIndex += _model->applicationSetting()->delayProfileSkipIndexHigh();
                    break;
                default:
                    return;
                }

                if (_delayTimeIndex >= _lineSeries->count()) {
                    _delayTimeIndex = 0;
                }
            } else {

                switch (_pressDownState) {
                case PRESS_DOWN_LEVEL_0:
                    --_delayTimeIndex;
                    break;
                case PRESS_DOWN_LEVEL_1:
                    _delayTimeIndex -= _model->applicationSetting()->delayProfileSkipIndexLow();
                    break;
                case PRESS_DOWN_LEVEL_2:
                    _delayTimeIndex -= _model->applicationSetting()->delayProfileSkipIndexHigh();
                    break;
                default:
                    return;
                }

                if (_delayTimeIndex < 0) {
                    _delayTimeIndex = _lineSeries->count() - 1;
                }
            }

            // 線の移動
            int count = _lineSeries->count();
            qreal step = static_cast<qreal>(DELAY_PROFILE_XAXIS_LENGTH) / static_cast<qreal>(count);
            qreal x = step * _delayTimeIndex;
            if (_delayProfile) {
                _delayProfile->setProperty("markerX", x);
            }

            if (_view) {
                // 遅延時間
                _view->setProperty("delayTime", getDelayTime());
            }
            if (_view) {
                // レベル差
                _view->setProperty("levelDifference",
                                   _lineSeries->lineSeries(_delayTimeIndex).y() - _lineSeries->lineSeries(DELAY_PROFILE_ZERO_INDEX).y());
            }

            return;
        }

        // その他の操作
        QString nextButton;
        bool ret = _buttonOperation->nextActiveObject(buttonId, nextButton);

        if (ret) {
            if (_view) {
                // inactive
                setButtonActive(_view, _activeButton, false);
            }
            if (_view) {
                // active
                setButtonActive(_view, nextButton, true);
            }

            _activeButton = nextButton;
            _buttonOperation->setActiveObject(_activeButton);
        }
    }
}

void SingleMeasurementViewModel::comboboxOperation(ButtonId buttonId)
{
    if (_view == nullptr) {
        return;
    }

    if (buttonId == BUTTON_UP || buttonId == BUTTON_DOWN) {
        // 値を変更する
        editValue(buttonId);
        return;
    }

    if (buttonId == BUTTON_RIGHT || buttonId == BUTTON_LEFT) {
        // カーソルを移動する
        moveCursor(buttonId);
        return;
    }

    if (buttonId == BUTTON_ENTER) {
        // コンボボックスの値を反映する
        int currentChannelIndex = _currentChannelIndex;
        if(refrectFrequencyCombobox()) {
            // コンボボックスに正しく反映できれば、コンボボックスの状態を通常状態に戻す
            _comboboxState = Normal;
            if (_view) {
                _view->setProperty("editFrequencyCombobox", false);
            }
            // ボタンのフォーカスをコンボボックスから戻す
            if (_combobox) {
                setButtonActive(_combobox, _activeButton, false);
            }
            _buttonPattern = LimitedPattern2;
            _measurementButtons = SINGLE_MEASUREMENT_BUTTON_DATA;
            _buttonOperation->setButtonName(BUTTON_COLUMN_NUMBER, BUTTON_ROW_NUMBER_LIMITED2, _measurementButtons);
            _buttonOperation->setActiveObject(SINGLE_MEASUREMENT_BUTTON_DATA[_buttonPattern][0]);
            _activeButton = SINGLE_MEASUREMENT_BUTTON_DATA[_buttonPattern][0];
            if (_view) {
                setButtonActive(_view, _activeButton, true);
            }
            if (currentChannelIndex != _currentChannelIndex) {
                // 反映したチャンネル番号が前回と異なる場合
                // チャンネル変更処理を実施する
                changeChannel();
            }
            return;
        }
        // コンボボックスに値を反映することができていない場合
        emit requestToShowPopup(POPUP_FAILURE_SETTING_RANGE_WRONG);
    }

    if (buttonId == BUTTON_USER) {
        // ユーザデータロガー画面へ遷移する
#ifdef NEW_USER_SETTING
        emit requestDirect(SCREEN_USER_SETTING, true);
#else
        emit requestDirect(SCREEN_USER_DATA_LOGGER_MENU, true);
#endif
        return;
    }

    if (buttonId == BUTTON_HOME) {
        // ホーム画面へ遷移する
        emit requestDirect(SCREEN_HOME, true);
        return;
    }

    if (buttonId == BUTTON_ESCAPE) {
        // BAND選択画面に遷移する
        emit requestInHistory(SCREEN_BAND);
        return;
    }
}

void SingleMeasurementViewModel::changeChannel(bool isComfirmChangeChannel)
{
    if (_view == nullptr) {
        return;
    }

    if (_channelList.count() == 0) {
        _model->writeLog(QString("シングル測定"), QString("no channel list"), LOG_INFO);
        return;
    }

    if (_currentChannelIndex < 0) {
        _model->writeLog(QString("シングル測定"), QString("all invalid channels"), LOG_INFO);
        return;
    }

    timer.start();

    // ブザーが鳴動している場合は、ブザーを停止する
    stopBuzzer();

    // 表示クリア
    clear();

    // 地デジの場合は OFDM とする
    if (_channelList[_currentChannelIndex].band == LC70_BAND_UHF) {
        _takeOver["selectedModulation"] = QString("OFDM");
    }

    // user設定画面からの遷移かどうかで参照先が異なる値を設定
    unsigned int localFrequency;
    unsigned int modulationValue;

    // 画面共有情報からBandを取得
    int bandId = _takeOver["selectedBand"].toInt();

    if (bandId == LC70_BAND_BS_CS) {
        if(_channelList[_currentChannelIndex].polarization == 0) {      // 右旋
            localFrequency = 10678000;
        }
        else if(_channelList[_currentChannelIndex].polarization == 1) {	// 左旋
            localFrequency = 9505000;
        }
        modulationValue = 0;
    }
    else {
        localFrequency = getLocalFrequency();;
        modulationValue = getModulationValue();;
    }

    // Band アイコンの差し替え
    _model->replaceBandIcon(static_cast<LC70_BAND_T>(_channelList[_currentChannelIndex].band));

    // 変調方式から周波数を取得する。
    unsigned int centerFrequency = Correcter::centerFrequency(_channelList[_currentChannelIndex], modulationValue);

    unsigned int frequency = Correcter::frequency(centerFrequency, localFrequency);

    // 画面にチャンネル情報を反映する
    if (_view) {
        _view->setProperty("band", Correcter::displayBandName(_channelList[_currentChannelIndex].band));
        _view->setProperty("selectedModulation",  Correcter::displayModulation(getModulationValue()));
        _view->setProperty("isSatelliteBroadCasting",
                           Correcter::isSatelliteBroadCasting(_channelList[_currentChannelIndex].band));
        _view->setProperty("isDisplaylevelHierarchyPolarization",
                           Correcter::isDisplayLevelHierarchyPolarization(_channelList[_currentChannelIndex].band, modulationValue));
        _view->setProperty("channel", Correcter::displayChannelName(_channelList[_currentChannelIndex].channelNumber));
        _view->setProperty("frequency", Correcter::displayFrequency(_channelList[_currentChannelIndex].band, frequency));
        _view->setProperty("polarization", Correcter::displayPolarization(Correcter::convertWaveToPolarization(
                                                                              _channelList[_currentChannelIndex].polarization)));
        _view->setProperty("isModulationVisible", Correcter::isModulationVisible(_channelList[_currentChannelIndex].band));

        int valueMerCN = Correcter::selectCnrMer(static_cast<LC70_BAND_T>(_channelList[_currentChannelIndex].band), _cnrMerSetting);
        _view->setProperty("selectCnMer", valueMerCN);
    }

    // コンボボックスを表示している場合
    if (_iscomboboxVisible) {
        // コンボボックスの表示を更新する
        if (_combobox) {
            _combobox->setProperty("value",_channelList[_currentChannelIndex].channelNumber);
        }
    }

    // 前回選択していたバンド情報、変調方式を取得
    uint16_t previousBand = _condition.usBand;
    uint16_t previousModulation = _condition.usMod;

    memset(&_condition, 0x00, sizeof(LC70_MEAS_SINGLE_T));

    // チャンネル情報設定
    _condition.ulFreq = centerFrequency; // 実放送周波数
    _condition.usBand = _channelList[_currentChannelIndex].band; // BAND

    // USER測定において、地デジ13chで「ディレイ」にカーソルを置いたまま、BS-1にチャンネル変更した場合など、
    // 「ディレイ」を非表示にしたことでアクティブボタンがなくなってしまう問題に対応
    if (_activeButton == BUTTON_NAME_DELAY_PROFILE) {
        // カーソルが「ディレイ」上にある
        if (!isEnableDelayProfileChannel(_currentChannelIndex)) {
            // 変更後のチャンネルは遅延プロファイル測定に対応していない
            if (_model->acquisitionTarget() == CONSTELLATION) {
                // 変更後はコンスタレーション測定をするので「コンスタ」上にカーソルを移動
                changeActiveButton(CONSTELLATION_BUTTON_INDEX);
            } else {
                // そうでない場合は通常のシングル測定なので「メーター」上にカーソルを移動
                changeActiveButton(MEASUREMENT_BUTTON_INDEX);
            }
        }
    } else if (_activeButton == BUTTON_NAME_CONSTELLATION) {
        // カーソルが「コンスタ」上にある
        if (!isEnableConstellationBandId(_channelList[_currentChannelIndex].band)) {
            // 変更後のチャンネルはコンスタレーション測定に対応していない
            if (_model->acquisitionTarget() == SINGLE) {
                // 変更後は通常のシングル測定なので「メーター」上にカーソルを移動
                changeActiveButton(MEASUREMENT_BUTTON_INDEX);
            }
        }
    }

    // チャンネル番号文字列(NULL終端なし、8文字に足りない場合はスペースで埋める)
    memset(_condition.ucStrChan, ' ', CHANNEL_NUMBER_LENGTH);

    for (int i = 0; i < CHANNEL_NUMBER_LENGTH; ++i) {
        if (i <= _channelList[_currentChannelIndex].channelNumber.toLocal8Bit().length() - 1) {
            _condition.ucStrChan[i] = _channelList[_currentChannelIndex].channelNumber.toLocal8Bit()[i];
        }
    }

    int length = strnlen((char *)_condition.ucStrChan, sizeof _condition.ucStrChan);
    if (length < CHANNEL_NUMBER_LENGTH) {
        for (int j = length; j < CHANNEL_NUMBER_LENGTH; ++j) {
            _condition.ucStrChan[j] = ' ';
        }
    }

    // 局発周波数の連携
    _condition.ulLoFreq = localFrequency; // kHz
    // kHz => GHzに変換
    if (_view) {
        _view->setProperty("localFrequency", Correcter::displayLocalFrequency(localFrequency));
    }

    // 放送方式
    QString modulation = Correcter::displayModulation(getModulationValue());
    _condition.usBdSys = Correcter::getBroadcastSystemCondition(_channelList[_currentChannelIndex].band, modulation);

    // 放送方式、変調方式の設定(定義ファイル 1.0.1 対応)
    _condition.usMod = Correcter::getModulationCondition(_channelList[_currentChannelIndex].band, _condition.usBdSys, _channelList.at(_currentChannelIndex).polarization, modulationValue, _broadBandModulationSetting);

    //BS-7, BS-17の場合、usBdSysを ISDB-S3に設定する
    if ((bandId == LC70_BAND_BS) || (bandId == LC70_BAND_CS) || (bandId == LC70_BAND_BS_CS)) {
        if(_channelList[_currentChannelIndex].channelNumber == "BS-7") {
            _condition.usBdSys = LC70_BD_SYS_ISDBS3;
        }
        else if(_channelList[_currentChannelIndex].channelNumber == "BS-17") {
            _condition.usBdSys = LC70_BD_SYS_ISDBS3;
        }
    }
    // ユーザー設定から呼び出された場合の放送方式、変調方式(ユーザー設定値)の連携
    if (_userSettings.count() > _currentChannelIndex) {
        if ((_userSettings[_currentChannelIndex].ucBand == LC70_BAND_BS) || (_userSettings[_currentChannelIndex].ucBand == LC70_BAND_CS)) {
            if(QString::fromLocal8Bit(_userSettings[_currentChannelIndex].cStrChNo, LC70_CH_SET_CH_STRLEN) == "BS-7") {
                _condition.usBdSys = LC70_BD_SYS_ISDBS3;
            }
            else if(QString::fromLocal8Bit(_userSettings[_currentChannelIndex].cStrChNo, LC70_CH_SET_CH_STRLEN) == "BS-17") {
                _condition.usBdSys = LC70_BD_SYS_ISDBS3;
            }
        }
        _condition.usBdSys = _userSettings[_currentChannelIndex].ucSpec;
        _condition.usMod = _userSettings[_currentChannelIndex].ulModBit;
    }

    _modulation = _condition.usMod;
    _modulationInMeasureResult = _modulation;
    _broadcastSystem = _condition.usBdSys;

    // 測定条件の連携(コンスタレーション、遅延プロファイルはラッパー関数の呼び出しクラスで付加
    // LTE, PILOT, FM, CATV 上り は LEVEL のみ
    if (_fromUserSetting && ((_userSettings[_currentChannelIndex].ucBand == LC70_BAND_LTE) ||
                             (_userSettings[_currentChannelIndex].ucBand == LC70_BAND_PILOT) ||
                             (_userSettings[_currentChannelIndex].ucBand == LC70_BAND_CATV_UP_STREAM) ||
                             (_userSettings[_currentChannelIndex].ucBand == LC70_BAND_FM))) {
        _condition.usCondition = LC70_DATA_MSK_LEVEL;
    } else if (!_fromUserSetting &&
               (static_cast<LC70_BAND_T>(_channelList[_currentChannelIndex].band) == LC70_BAND_LTE ||
                static_cast<LC70_BAND_T>(_channelList[_currentChannelIndex].band) == LC70_BAND_PILOT ||
                static_cast<LC70_BAND_T>(_channelList[_currentChannelIndex].band) == LC70_BAND_CATV_UP_STREAM ||
                static_cast<LC70_BAND_T>(_channelList[_currentChannelIndex].band) == LC70_BAND_FM)) {

        _condition.usCondition = LC70_DATA_MSK_LEVEL;
    } else {
        if (_fromUserSetting){
            _condition.usCondition = LC70_DATA_MSK_LEVEL;
            // MER/CN の使い分け(CNR ビットは必ず立てる。MER のときは MER も立てる)
            if ((_userSettings[_currentChannelIndex].ucGetDataFlg & LC70_DATA_MSK_CNR) == LC70_DATA_MSK_CNR) {
                if (Correcter::selectCnrMer(static_cast<LC70_BAND_T>(_channelList[_currentChannelIndex].band), _cnrMerSetting) == LC70_MEAS_SET_SEL_CNR) {
                    _condition.usCondition |= LC70_DATA_MSK_CNR;
                } else if (Correcter::selectCnrMer(static_cast<LC70_BAND_T>(_channelList[_currentChannelIndex].band), _cnrMerSetting) == LC70_MEAS_SET_SEL_MER) {
                    _condition.usCondition |= (LC70_DATA_MSK_CNR | LC70_DATA_MSK_MER);
                }else {
                    // nothing to do
                }
            }
            if ((_userSettings[_currentChannelIndex].ucGetDataFlg & LC70_DATA_MSK_BER) == LC70_DATA_MSK_BER) {
                _condition.usCondition |= (LC70_DATA_MSK_BER);
            }
        } else {
            // MER/CN の使い分け(CNR ビットは必ず立てる。MER のときは MER も立てる)
            if (Correcter::selectCnrMer(static_cast<LC70_BAND_T>(_channelList[_currentChannelIndex].band), _cnrMerSetting) == LC70_MEAS_SET_SEL_CNR) {
                _condition.usCondition = LC70_DATA_MSK_LEVEL | LC70_DATA_MSK_CNR | LC70_DATA_MSK_BER;
            } else if (Correcter::selectCnrMer(static_cast<LC70_BAND_T>(_channelList[_currentChannelIndex].band), _cnrMerSetting) == LC70_MEAS_SET_SEL_MER) {
                _condition.usCondition = LC70_DATA_MSK_LEVEL | LC70_DATA_MSK_CNR | LC70_DATA_MSK_BER | LC70_DATA_MSK_MER;
            } else {
                _model->writeLog(QString("シングル測定"), QString("### ERROR CNR/MER NOT SELECTED"), LOG_INFO);
            }
        }
    }

    // 給電有効/無効 0:無効(マニュアル測定、スキャン、USER 測定) 1:有効(データロガーのみ)
    _condition.ucOutputEnb = 0;

    // ユーザー設定から遷移
    if (_fromUserSetting) {
        if (_powerSupply != LC70_POWER_SUPPLY_OFF) {
            // 給電設定 ON
            selectPowerSupplyForUserSetting();
        }
        if (_condition.powerSupply != _powerSupply) {
            _model->setPowerSupply(_powerSupply);
            _model->replaceMenuList(MenuListIconIndex::POWER_SUPPLY_ICON, Correcter::powerSupplyIcon(_powerSupply));
            _condition.powerSupply = _powerSupply;
        }
    }

    //  BAND、変調方式が変化している場合、ボタン操作情報の更新を行う。
    if (previousBand != _condition.usBand || previousModulation != _condition.usMod) {
        //  PILOT, LTE, CATV上り 選択時は遅延プロファイル非表示
        if (_condition.usBand == LC70_BAND_LTE
                || _condition.usBand == LC70_BAND_PILOT
                || _condition.usBand == LC70_BAND_FM
                || _condition.usBand == LC70_BAND_CATV_UP_STREAM) {
            _buttonPattern = LimitedPattern2;
            _measurementButtons = SINGLE_MEASUREMENT_BUTTON_DATA;
            _buttonOperation->setButtonName(BUTTON_COLUMN_NUMBER, BUTTON_ROW_NUMBER_LIMITED2, _measurementButtons);
            if (_view) {
                _view->setProperty("buttonPattern", 2);
            }
            // [Memo]
            //  選択できないボタンが_activeに設定されている場合、
            //  先頭のボタンをactiveに変更する。
            if (_activeButton == BUTTON_NAME_DELAY_PROFILE || _activeButton == BUTTON_NAME_CONSTELLATION) {
                if (_view) {
                    setButtonActive(_view, _activeButton, false);
                }
                _buttonOperation->setActiveObject(SINGLE_MEASUREMENT_BUTTON_DATA[_buttonPattern][0]);
                _activeButton = SINGLE_MEASUREMENT_BUTTON_DATA[_buttonPattern][0];
                if (_view) {
                    setButtonActive(_view, _activeButton, true);
                }
            }
        } else if (_channelList[_currentChannelIndex].band == LC70_BAND_UHF) {
            _buttonPattern = NormalPattern;
            _measurementButtons = SINGLE_MEASUREMENT_BUTTON_DATA;
            _buttonOperation->setButtonName(BUTTON_COLUMN_NUMBER, BUTTON_ROW_NUMBER_NORMAL, _measurementButtons);
            if (_view) {
                _view->setProperty("buttonPattern", 0);
            }
        } else {
            // CATV で変調方式が ISDBT(OFDM) なら遅延プロファイルを表示
            if ((_channelList[_currentChannelIndex].band == LC70_BAND_CATV_ALL ||
                 _channelList[_currentChannelIndex].band == LC70_BAND_CATV_90_470MHZ ||
                 _channelList[_currentChannelIndex].band == LC70_BAND_CATV_470_1002MHZ) && modulationValue == LC70_SETMOD_OFDM) {
                _buttonPattern = NormalPattern;
                _measurementButtons = SINGLE_MEASUREMENT_BUTTON_DATA;
                _buttonOperation->setButtonName(BUTTON_COLUMN_NUMBER, BUTTON_ROW_NUMBER_NORMAL, _measurementButtons);
                if (_view) {
                    _view->setProperty("buttonPattern", 0);
                }
            } else {
                _buttonPattern = LimitedPattern1;
                _measurementButtons = SINGLE_MEASUREMENT_BUTTON_DATA;
                _buttonOperation->setButtonName(BUTTON_COLUMN_NUMBER, BUTTON_ROW_NUMBER_LIMITED1, _measurementButtons);
                if (_view) {
                    _view->setProperty("buttonPattern", 1);
                }
            }
        }
    }

    _model->writeLog(QString("シングル測定"),
                     QString("next channel band:%1 frequency:%2 system:%3 modulation:%4 condition:%5 dc out:%6 power supply:%7").arg(_condition.usBand).arg(
                         _condition.ulFreq).arg(_condition.usBdSys).arg(_condition.usMod).arg(_condition.usCondition).arg(_condition.ucOutputEnb).arg(_condition.powerSupply), LOG_INFO);

    // チャンネル変更中の操作を禁止
    if (_isChannelChanging) {
        return;
    }

    _model->writeLog(QString("シングル測定"), QString("%1 %2 ms").arg(__FUNCTION__).arg(timer.elapsed()));

    if (isComfirmChangeChannel) {
        // チャンネル確定
        onConfirmedChangeChannelTimeout();
    }
}

void SingleMeasurementViewModel::clear()
{
    _newestLevel = -1;
    _newestMer = -1;
    _newestBer = -1;

    if (!_isUpdateMeter) {
        // MER, BER の値を未確定とする
        _confirmedValue = false;
        onUpdateMeter();
    }

    // 放送方式
    if (_view) {
        _view->setProperty("broadcastingSystem", QString("---"));

        // 変調方式
        _view->setProperty("modulationPrefixMode", DISPLAY_MODULATION_PREFIX_TOGETHER);
        _view->setProperty("modulation", QString("---"));
        _view->setProperty("modulationPrefix", QString(""));

        // 階層
        _view->setProperty("hierarchy", QString("---"));

        // コンスタレーションをクリア
        _view->setProperty("constellationData", QList<QVariant>());
        _isConstellationDataClear = true;
        _view->setProperty("isConstellationDataClear", _isConstellationDataClear);

        // 遅延プロファイルのグラフをクリア
        clearDelayProfileInfo();
    }

    qDebug() << QString("clear display.");
}

void SingleMeasurementViewModel::clearGraph()
{
    QVector<QPointF> points;

    double x = MINIMUM_DELAY_PROFILE_TIME + DELAY_PROFILE_INTERVAL;

    for (int i = 0; i < DELAY_PROFILE_DATA_COUNT; ++i) {
        points.append(QPointF(x, DELAY_PROFILE_INITIAL_DECIBEL));
        x += DELAY_PROFILE_INTERVAL;
    }

    if (_lineSeries) {
        _lineSeries->setLineSeries(points);
    }
}

void SingleMeasurementViewModel::refreshThreshold(const int &bandId)
{
    // 閾値の取得
    QVariant thresholdSetting;     // 実態は QVariantMap
    _model->getValue(VALUE_THRESHOLD, thresholdSetting);

    QVariantList thresholds;

    // 測定設定の変調方式がOFDMかつ、
    // 測定結果がOFDM以外の変調方式の場合スキャン結果として登録する変調方式をOFDMに設定する
    int modulation = _modulationInMeasureResult;
    if (getModulationValue() == LC70_SETMOD_OFDM && _modulationInMeasureResult != LC70_SETMOD_OFDM) {
        modulation = LC70_SETMOD_OFDM;
    }

    int thresholdIndex = Correcter::getThresholdIndex(bandId, _broadcastSystem, modulation);
    if (thresholdIndex < 0) {
        // CATV でアンテナケーブルの抜き差しで MER のメーターが全部緑になるのを避ける
        //useDefaultMerRange();
        _model->writeLog("SingleMeasurementViewModel", "there is no threshold data", LOG_INFO);
        return;
    }

    thresholds = thresholdSetting.toList().at(thresholdIndex).toList();

    for (int i = 0; i < thresholds.count(); ++i) {
        qDebug() << QString("### THRESHOLD %1").arg(thresholds[i].toString());
    }

    // 要素数が不足している場合
    if (thresholds.count() < THRESHOLD_ITEM_COUNT) {
        // CATV でアンテナケーブルの抜き差しで MER のメーターが全部緑になるのを避ける
        //useDefaultMerRange();
        return;
    }

    // MERの閾値の範囲を取得
    updateMerCnrRange(bandId, _broadcastSystem);

    // LEVEL の閾値(受信可の値)
    double levelLow = thresholds[1].toDouble() / 10;
    _levelReceivableThreshold = levelLow;
    double levelHigh = thresholds[2].toDouble() / 10;

    qDebug() << QString("### LEVEL: %1").arg(levelLow);
    double levelRedZone = 0;
    double levelYellowZone = 0;

    if (0 <= (levelLow - MINIMUM_LEVEL) / (MAXIMUM_LEVEL - MINIMUM_LEVEL)
            && (levelLow - MINIMUM_LEVEL) / (MAXIMUM_LEVEL - MINIMUM_LEVEL) <= 1.0) {
        levelRedZone = (levelLow - MINIMUM_LEVEL) / (MAXIMUM_LEVEL - MINIMUM_LEVEL);
    }

    if (0 <= (levelHigh - levelLow) / (MAXIMUM_LEVEL - MINIMUM_LEVEL)
            && (levelHigh - levelLow) / (MAXIMUM_LEVEL - MINIMUM_LEVEL) <= 1.0) {
        levelYellowZone = (levelHigh - levelLow) / (MAXIMUM_LEVEL - MINIMUM_LEVEL);
    }

    // Level の閾値を通知
    emit notifyLevelThreshold(levelRedZone, levelYellowZone);

    // MER の閾値
    double merLow = thresholds[3].toDouble() / 10;
    double merHigh = thresholds[4].toDouble() / 10;

    // MER余裕度高の閾値を保持する
    _mercnMerginHighThreshold = merHigh;

    qDebug() << QString("### MER LOW: %1 HIGH: %2").arg(merLow).arg(merHigh);

    double merRedZone = 0;
    double merYellowZone = 0;
    if (0 <= (merLow - _merRange.first) / (_merRange.second - _merRange.first) / _merRange.second
            && (merLow - _merRange.first) / (_merRange.second - _merRange.first) <= 1.0) {
        merRedZone = (merLow - _merRange.first) / (_merRange.second - _merRange.first);
    }

    if (0 <= (merHigh - _merRange.first) / (_merRange.second - _merRange.first)
            && (merHigh - _merRange.first) / (_merRange.second - _merRange.first) <= 1.0) {
        merYellowZone = (merHigh - _merRange.first) / (_merRange.second - _merRange.first);
    }

    // MER に閾値を通知
    emit notifyMerThreshold(merRedZone, merYellowZone);

    // BER の閾値
    // BER の値(100 倍されている)
    double berLow = thresholds[5].toDouble();
    double berHigh = thresholds[7].toDouble();

    // 100 倍されている値を調整してから最小値、最大値を求める
    int berLowExponent = qAbs(thresholds[6].toInt()) + MINIMUM_BER_EXPONENT;
    int berHighExponent = qAbs(thresholds[8].toInt()) + MINIMUM_BER_EXPONENT;

    // berの余裕度高の閾値を保持する
    _berMarginHighThreshold = berHigh * qPow(0.1, berHighExponent);

    // 閾値を変換
    double berLowStep = (berLow / 100.0) / pow(10.0, berLowExponent);
    double berHighStep = (berHigh / 100.0) / pow(10.0, berHighExponent);

    double berRedZone = calcBerZoneRatio(berHighStep);
    double berYellowZone = calcBerZoneRatio(berLowStep);

    // BER に閾値を通知
    emit notifyBerThreshold(berRedZone, berYellowZone);

    qDebug() << QString("[Leave]") << __FUNCTION__;
}

float32_t SingleMeasurementViewModel::calcBerZoneRatio(float32_t value)
{
    float32_t surplus = 0.0;
    float32_t berZoneRatio = 0.0;

    // 他のメーターと違い、指数の目盛り毎に値の単位が異なる
    // （E-2〜E-3の間は 0.001、E-3〜E-4の間は0.0001など）
    // また目盛りの画像には、各指数の間に5分割の線が描画してある。
    // 例えば E-2〜E-3の間では、目盛りが指している値は以下のようになる
    //   E-3          : 0.001
    //   E-3 目盛り中間 : 0.0019
    //   E-3 目盛り1   : 0.0028
    //   E-3 目盛り中間 : 0.0037
    //   E-3 目盛り2   : 0.0046
    //   E-3 目盛り中間 : 0.0055
    //   E-3 目盛り3   : 0.0064
    //   E-3 目盛り中間 : 0.0073
    //   E-3 目盛り4   : 0.0082
    //   E-3 目盛り中間 : 0.0091
    //   E-2          : 0.001
    // 目盛り線上に合う数値が感覚と異なるため注意が必要。
    // （中間点は0.005ではなく0.0055であるなど）
    if (1.0e-2 < value) {
        berZoneRatio = 1;
    } else if (1.0e-3 < value && value <= 1.0e-2) {
        berZoneRatio = BER_METER_STEP * 6;
        // 0.001
        surplus = (value - 1.0e-2) / ((1.0e-2 - 1.0e-3) / 10) * (BER_METER_STEP / 10);
    } else if (1.0e-4 < value && value <= 1.0e-3) {
        berZoneRatio = BER_METER_STEP * 5;
        // 0.0001
        surplus = (value - 1.0e-3) / ((1.0e-3 - 1.0e-4) / 10) * (BER_METER_STEP / 10);
    } else if (1.0e-5 < value && value <= 1.0e-4) {
        berZoneRatio = BER_METER_STEP * 4;
        // 0.00001
        surplus = (value - 1.0e-4) / ((1.0e-4 - 1.0e-5) / 10) * (BER_METER_STEP / 10);
    } else if (1.0e-6 < value && value <= 1.0e-5) {
        berZoneRatio = BER_METER_STEP * 3;
        // 0.000001
        surplus = (value - 1.0e-5) / ((1.0e-5 - 1.0e-6) / 10) * (BER_METER_STEP / 10);
    } else if (1.0e-7 < value && value <= 1.0e-6) {
        berZoneRatio = BER_METER_STEP * 2;
        // 0.0000001
        surplus = (value - 1.0e-6) / ((1.0e-6 - 1.0e-7) / 10) * (BER_METER_STEP / 10);
    } else if (1.0e-8 < value && value <= 1.0e-7) {
        berZoneRatio = BER_METER_STEP * 1;
        // 0.00000001
        surplus = (value - 1.0e-7) / ((1.0e-7 - 1.0e-8) / 10) * (BER_METER_STEP / 10);
    } else if (value <= 1.0e-8) {
        // nothing to do.
    } else if (value == 0) {
        // nothing to do.
    }

    berZoneRatio += surplus;

    return berZoneRatio;
}

float32_t SingleMeasurementViewModel::updateMer(const MeasureResult &result)
{
    // MER
    // LTE, PILOT, FM は LEVEL のみ更新
    if (_buttonPattern == LimitedPattern2) {
        return UNDEFINED_VALUE;
    }

    // USER設定でMER/CNの収集設定がOFFの場合は不定値を設定
    if(_userSettings.count() > _currentChannelIndex && (_userSettings[_currentChannelIndex].ucGetDataFlg & LC70_DATA_MSK_CNR) != LC70_DATA_MSK_CNR){
        return UNDEFINED_VALUE;
    }

    // 測定開始後一定時間経過後に更新する(ロックされていることを前提とする)
    if (_displayMer && result.Received && result.Locked == LOCKED) {
        if (qIsNaN(result.Mer) || qIsInf(result.Mer)) {
            return UNDEFINED_VALUE;
        }
        return  result.Mer;
    }
    return UNLOCKED_VALUE;
}

float32_t SingleMeasurementViewModel::updateMerNeedle(const float32_t &mer)
{
    // 針の位置は 0〜180 の 181 段階で、181 を閾値の最大値(35)から最小値(5)を引いた値(30)で割ったのが 1db あたりの針の移動量
    float32_t merNeedleStep = static_cast<float32_t>(180) / (_merRange.second - _merRange.first);
    float32_t merNeedle = 0;

    if (_merRange.first == 0 && _merRange.second == 0) {
        return merNeedle;
    }

    if (mer > _merRange.first && mer < _merRange.second) {
        merNeedle = (mer - _merRange.first) * merNeedleStep;
    } else if (mer >= _merRange.second) {
        merNeedle = 180;
    }
    _model->writeLog(QString("シングル測定"), QString("mer: %1 needle position: %2").arg(mer).arg(merNeedle));
    return merNeedle;
}

float32_t SingleMeasurementViewModel::updateBer(const MeasureResult &result)
{
    // BER
    // LTE, PILOT, FM は LEVEL のみ更新
    if (_buttonPattern == LimitedPattern2) {
        return UNDEFINED_VALUE;
    }

    // USER設定でMER/CNの収集設定がOFFの場合は不定値を設定
    if(_userSettings.count() > _currentChannelIndex && (_userSettings[_currentChannelIndex].ucGetDataFlg & LC70_DATA_MSK_BER) != LC70_DATA_MSK_BER){
        return UNDEFINED_VALUE;
    }

    // 測定開始後一定時間経過後に更新する(ロックされていることを前提とする)
    if (_displayBer && result.Received && result.Locked == LOCKED) {
        if (qIsNaN(result.Ber) || qIsInf(result.Ber)) {
            return UNDEFINED_VALUE;
        }
        return result.Ber;
    }
    return UNLOCKED_VALUE;
}

float32_t SingleMeasurementViewModel::updateBerNeedle(const float32_t &ber)
{
    static const QMap<BER_RANGE, float32_t> irregularBerValueMap = {
        {UNDER_E8, -180.0f},
        {OVER_E2,     0.0f},
    };

    static const QMap<BER_RANGE, QPair<QPair<float32_t, float32_t>, float32_t>> berValueMap = {
        // 針の位置は 0 〜 -180 の 181 段階
        // 1.0e-2 から負の指数が 1 増えるごとに -30度傾き、その指数で表す下限に近づくほど -180度に近づく
        // BERの範囲帯, { {範囲内の最低値, 値に対応した針の角度}, 範囲内で1度に値するBERの値}
        {E2_E3, {{1e-2,    0.0f}, 3.0e-4}},
        {E3_E4, {{1e-3,  -30.0f}, 3.0e-5}},
        {E4_E5, {{1e-4,  -60.0f}, 3.0e-6}},
        {E5_E6, {{1e-5,  -90.0f}, 3.0e-7}},
        {E6_E7, {{1e-6, -120.0f}, 3.0e-8}},
        {E7_E8, {{1e-7, -150.0f}, 3.0e-9}},
    };

    BER_RANGE range = getBerRange(ber);

    if (irregularBerValueMap.count(range) != 0) {
        return irregularBerValueMap.value(range);
    }
    if (berValueMap.count(range) == 0) {
        return -180.0f;
    }

    QPair<float32_t, float32_t> rangeUnderPoint = berValueMap.value(range).first;
    float32_t surplus = (ber - rangeUnderPoint.first) / berValueMap.value(range).second;
    float32_t berNeedle = rangeUnderPoint.second;

    berNeedle += surplus;

    _model->writeLog(QString("シングル測定"), QString("ber: %1 needle position: %2").arg(_ber).arg(berNeedle));
    return berNeedle;
}

BER_RANGE SingleMeasurementViewModel::getBerRange(const float32_t &value) {
    // 1.0e-2 から負の指数が 1 増えるごとに -30度傾き、その指数で表す下限に近づくほど -180度に近づく
    if (1.0e-2 < value) {
        return OVER_E2;
    } else if (1.0e-3 < value && value <= 1.0e-2) {
        // 0.001
        return E2_E3;
    } else if (1.0e-4 < value && value <= 1.0e-3) {
        // 0.0001
        return E3_E4;
    } else if (1.0e-5 < value && value <= 1.0e-4) {
        // 0.00001
        return E4_E5;
    } else if (1.0e-6 < value && value <= 1.0e-5) {
        // 0.000001
        return E5_E6;
    } else if (1.0e-7 < value && value <= 1.0e-6) {
        // 0.0000001
        return E6_E7;
    } else if (1.0e-8 < value && value <= 1.0e-7) {
        // 0.00000001
        return E7_E8;
    } else {
        return UNDER_E8;
    }
}

bool SingleMeasurementViewModel::isEnableDelayProfileChannel(int channelIndex) {
    // 遅延プロファイルは、地デジ、CATV(ISDBT)以外は対象外
    static const QMap<QPair<LC70_BAND_T, unsigned int>, bool> delayProfileBandMap = {
        {{LC70_BAND_UHF, LC70_SETMOD_OFDM}, true},
        {{LC70_BAND_CATV_ALL, LC70_SETMOD_OFDM}, true},
        {{LC70_BAND_CATV_90_470MHZ, LC70_SETMOD_OFDM}, true},
        {{LC70_BAND_CATV_470_1002MHZ, LC70_SETMOD_OFDM}, true},
    };

    unsigned int modulation;

    if (_fromUserSetting) {
        modulation = _userSettings.at(channelIndex).ulModBit;
    } else {
        modulation = Correcter::modulationValue(_takeOver["selectedModulation"].toString());
    }

    QPair<LC70_BAND_T, unsigned int> key = {static_cast<LC70_BAND_T>(_channelList[channelIndex].band), modulation};

    return delayProfileBandMap.value(key, false);
}

bool SingleMeasurementViewModel::isEnableConstellationBandId(int band) {
    // コンスタレーションを表示している場合は、PILOT、LTE、FMは対象外
    static const QMap<LC70_BAND_T, bool> constellationBandMap = {
        {LC70_BAND_UHF, true},
        {LC70_BAND_BS, true},
        {LC70_BAND_CS, true},
        {LC70_BAND_JCSAT3, true},
        {LC70_BAND_JCSAT4, true},
        {LC70_BAND_SCCB, true},
        {LC70_BAND_SCCC, true},
        {LC70_BAND_CATV_ALL, true},
        {LC70_BAND_CATV_90_470MHZ, true},
        {LC70_BAND_CATV_470_1002MHZ, true},
        {LC70_BAND_SKYP, true},
    };

    LC70_BAND_T bandId = static_cast<LC70_BAND_T>(band);

    return constellationBandMap.value(bandId, false);
}

void SingleMeasurementViewModel::startConfirmedChangeChannelTimer()
{
    // タイマーを開始する
    // 既にタイマーが開始している場合は、再度タイマーを開始する
    if (_confirmedChangeChannelTimer->isActive()) {
        _confirmedChangeChannelTimer->stop();
    } else {
        // 鳴動中のブザーを止める
        stopBuzzer();
    }

    _confirmedChangeChannelTimer->start(_model->applicationSetting()->channelChangeDelay());
}

void SingleMeasurementViewModel::checkThresholdCondition(uint16_t bdSys, uint32_t modulation)
{
    if (_condition.usBand != _previousCondition.usBand) {
        _model->writeLog(QString("シングル測定"), QDateTime::currentDateTime().toString("hh:mm:ss.zzz") + QString(" Band 変更 %1 to %2").arg(_previousCondition.usBand).arg(_condition.usBand), LOG_NOTICE);
        _thresholdSet = false;
    }

    if (_broadcastSystem != bdSys) {
        _thresholdSet = false;
        _model->writeLog(QString("シングル測定"), QDateTime::currentDateTime().toString("hh:mm:ss.zzz") + QString(" 放送方式変更 %1 to %2").arg(_broadcastSystem).arg(bdSys), LOG_NOTICE);
        _broadcastSystem = bdSys;
    }

    if (_modulationInMeasureResult != modulation) {
        _thresholdSet = false;
        // 測定結果の変調方式で測定条件の変調方式を上書きせず、測定結果の変調方式を上書きする
        //_modulation = modulation;
        _model->writeLog(QString("シングル測定"), QDateTime::currentDateTime().toString("hh:mm:ss.zzz") + QString(" 変調方式変更 %1 to %2").arg(_modulationInMeasureResult).arg(modulation), LOG_NOTICE);
        _modulationInMeasureResult = modulation;
    }

    if (!_thresholdSet) {
        // 閾値の反映
        refreshThreshold(_condition.usBand);
        _model->writeLog(QString("シングル測定"), QDateTime::currentDateTime().toString("hh:mm:ss.zzz") + QString(" 閾値変更 %1 ms").arg(measuringTimer.elapsed()), LOG_NOTICE);
        _model->writeLog(QString("シングル測定"), QString("%1 set threshold").arg(__FUNCTION__));
        _thresholdSet = true;
    }
}

void SingleMeasurementViewModel::onConfirmedChangeChannelTimeout()
{
    if (_pressedChannelButtonId != BUTTON_UNKNOWN && AbstractViewModel::isKeyHolding(_pressedChannelButtonId)) {
        // チャンネル変更ボタンを長押ししている間はチャンネル変更を確定しない
        // ボタンを離した際に開始されるタイマが発火した際に確定するため何もしない
        return;
    }

    // チャンネル変更が確定したら長押し判定用のIDをクリアする
    _pressedChannelButtonId = BUTTON_UNKNOWN;

    // Band、もしくは放送方式が変化する場合、閾値を更新する
    checkThresholdCondition(_condition.usBdSys, _condition.usMod);

    // 遅延プロファイル測定設定を初期状態に戻しておく
    resetSettings(_selectedButtonIndex);

    // 測定条件変更通  知を送信
    _isChannelChanging = true;
    emit changeCondition(_condition);
    _previousCondition = _condition;

    // 測定結果を表示に反映するまで待つためのタイマーを開始
    _displayLevel = false;
    _displayMer = false;
    _displayBer = false;

    if (_levelFixTimer->isActive()) {
        _levelFixTimer->stop();
        _model->writeLog(QString("シングル測定"), QString("測定結果から Level の表示を更新するまでの待ち時間を検知するタイマーを停止"), LOG_INFO);
    }
    _levelFixTimer->start();
    _model->writeLog(QString("シングル測定"), QString("測定結果から Level の表示を更新するまでの待ち時間を検知するタイマーを開始"), LOG_INFO);

    if (_merFixTimer->isActive()) {
        _merFixTimer->stop();
        _model->writeLog(QString("シングル測定"), QString("測定結果から MER の表示を更新するまでの待ち時間を検知するタイマーを停止"), LOG_INFO);
    }
    _merFixTimer->start();
    _model->writeLog(QString("シングル測定"), QString("測定結果から MER の表示を更新するまでの待ち時間を検知するタイマーを開始"), LOG_INFO);

    if (_berFixTimer->isActive()) {
        _berFixTimer->stop();
        _model->writeLog(QString("シングル測定"), QString("測定結果から BER の表示を更新するまでの待ち時間を検知するタイマーを停止"), LOG_INFO);
    }
    _berFixTimer->start();
    _model->writeLog(QString("シングル測定"), QString("測定結果から BER の表示を更新するまでの待ち時間を検知するタイマーを開始"), LOG_INFO);
}

void SingleMeasurementViewModel::stopBuzzer()
{
    // 既に止めているときは再度呼び出さないようにする
    if (_buzzerState == LC70_BUZZ_OFF) {
        return;
    }

    // ブザーを止めるとともにgoodIconの表示設定を非表示にする
    if (_view) {
        _view->setProperty("isGoodIconVisible", false);
    }
    _model->buzzer(LC70_BUZZ_OFF, LC70_BUZZ_MODE_STOP);
    _buzzerState = LC70_BUZZ_OFF;
}

void SingleMeasurementViewModel::onNotifyDelayProfileValid(bool valid)
{
    if (_isDelayProfileValid == valid) {
        return;
    }

    _isDelayProfileValid = valid;

    if (_model->acquisitionTarget() != DELAY_PROFILE) {
        return;
    }

    if (_isDelayProfileValid) {
        return;
    }

    // 遅延プロファイルのデータが無効なのでグラフ表示を消す
    clearGraph();
}

void SingleMeasurementViewModel::clearDelayProfileInfo()
{
    // グラフ表示クリア
    clearGraph();
    // ガードインターバルクリア
    if (_view) {
        _view->setProperty("guardInterval", -84);
    }

    // 遅延プロファイルマーカー位置を初期値へ移動
    moveProfileMarkerInitialPosition();
}

float32_t SingleMeasurementViewModel::guardIntervalValue(const uint8_t interval)
{
    static const QMap<LC70_ISDBT_GI_T, float32_t> intervalMap = {
        {LC70_ISDBT_GI_1_32, 31.3},
        {LC70_ISDBT_GI_1_16, 63.0},
        {LC70_ISDBT_GI_1_8, 126.0},
        {LC70_ISDBT_GI_1_4, 252.0},
    };

    return intervalMap.value(static_cast<LC70_ISDBT_GI_T>(interval), 0);
}

QString SingleMeasurementViewModel::getDelayTime()
{
    double delayTime = static_cast<double>(MINIMUM_DELAY_PROFILE_TIME) + (static_cast<double>
                                                                          (_delayTimeIndex) * DELAY_PROFILE_INTERVAL);
    return QString::number(delayTime, 'f', 1);
}

void SingleMeasurementViewModel::setBoolProperty(QString objectName, QString propertyName, bool value)
{
    if (_view) {
        QObject *target = _view->findChild<QObject *>(objectName);

        if (target) {
            // 押下前の画像に更新
            target->setProperty(propertyName.toUtf8().constData(), value);
        }
    }
}

unsigned int SingleMeasurementViewModel::getLocalFrequency()
{
    unsigned int localFreqency;

    if (_fromUserSetting) {
        localFreqency = _userSettings.at(_currentChannelIndex).ulLocalFreq;
    } else {
        localFreqency = _takeOver["selectedLocalOscillator"].toUInt();
    }

    return localFreqency;
}

unsigned int SingleMeasurementViewModel::getModulationValue()
{
    unsigned int modulationValue;

    if (_fromUserSetting) {
        modulationValue = _userSettings.at(_currentChannelIndex).ulModBit;
    } else {
        modulationValue = Correcter::modulationValue(_takeOver["selectedModulation"].toString());
    }

    return modulationValue;
}

bool SingleMeasurementViewModel::checkGoodReception(MeasureResult result)
{
    bool goodReception = true;

    // 給電異常時は良好としない
    if (_powerSupplyErrorOccured) {
        return false;
    }

    // 現在の測定条件から、LEVEL, MER, BERのどれを判定する値を決定する

    // LEVELのチェック
    if ((_condition.usCondition & LC70_DATA_MSK_LEVEL) == LC70_DATA_MSK_LEVEL) {
        // LEBEL、MER/CNは測定値が閾値を上回った場合
        // BERは測定値が閾値を下回った場合に受信良好状態となる
        if (_levelReceivableThreshold > result.Level) {
            goodReception = false;
        }
    }

    // MER,CNのチェック
    if (((_condition.usCondition & LC70_DATA_MSK_CNR) == LC70_DATA_MSK_CNR)
            || ((_condition.usCondition & LC70_DATA_MSK_MER) == LC70_DATA_MSK_MER)) {
        if (_mercnMerginHighThreshold > result.Mer) {
            goodReception = false;
        }
    }

    // BERのチェック
    if ((_condition.usCondition & LC70_DATA_MSK_BER) == LC70_DATA_MSK_BER) {
        if (qIsNaN(result.Ber) || qIsInf(result.Ber)) {
            // BERの測定結果の値は、場合によってNan, Infになる場合があるため考慮する必要有り
            // BERの値が、Nan及びInfになった場合は、受信良好状態をfalseとする
            goodReception = false;
        } else {
            if (_berMarginHighThreshold < result.Ber) {
                goodReception = false;
            }
        }
    }

    return goodReception;
}

void SingleMeasurementViewModel::detectPowerSupplyError(uint16_t code, uint8_t action, QString message) {
    _failureCode = code;
    _actionInFailure = action;
    _powerSupplyErrorOccured = true;

    // 受信良好音が鳴っていたら止める
    stopBuzzer();

#if 1
    // スキャン実行中の場合、スキャンを停止
    if(_isScanning == true) {
        onRequestToStopScanning();
        //        onScanningFinished(false);	//onRequestToStopScanningから呼び出されるため 不要
    }
#endif
    if (_takeOver["fromUserSetting"].toBool()) {
        // ユーザー測定： 測定を中断する
#if 1
        _powerSupply = LC70_POWER_SUPPLY_OFF;
        _model->setPowerSupply(_powerSupply);
#endif
        _model->removeMenuList(MenuListIconIndex::POWER_SUPPLY_ICON);

        if (_actionInFailure == LC70_FAIL_CTL_MSGDISP_CLR) {
            uint8_t addInfo[LC70_FAIL_ADD_INFO_NUM];
            memset(addInfo, 0x00, LC70_FAIL_ADD_INFO_NUM);
            //_model->setFaultInformation(_failureCode, LC70_FAIL_STAT_NORMAL, addInfo);     故障復旧処理不要
            _failureCode = 0;
            _actionInFailure = LC70_FAIL_CTL_NONE;
        }

#if 0
        // 測定停止
        emit requestToSuspendSingleMeasuring();
#endif

        // ポップアップ表示後にエラー音を鳴らす(音量 OFF でも実行する)
        _model->buzzer(LC70_BUZZ_ERR, LC70_BUZZ_MODE_ONESHOT, true);

        emit requestToShowPopupWithMessage(POPUP_POWER_SUPPLY_ERROR_USER_MEASURING, message);
    } else {
        // マニュアル測定： 給電を停止する
        _powerSupply = LC70_POWER_SUPPLY_OFF;
        _model->setPowerSupply(_powerSupply);
        _model->removeMenuList(MenuListIconIndex::POWER_SUPPLY_ICON);

        if (_actionInFailure == LC70_FAIL_CTL_MSGDISP_CLR) {
            uint8_t addInfo[LC70_FAIL_ADD_INFO_NUM];
            memset(addInfo, 0x00, LC70_FAIL_ADD_INFO_NUM);
            //_model->setFaultInformation(_failureCode, LC70_FAIL_STAT_NORMAL, addInfo);     故障復旧処理不要
            _failureCode = 0;
            _actionInFailure = LC70_FAIL_CTL_NONE;
        }

#if 0
        // 測定停止
        emit requestToSuspendSingleMeasuring();
#endif

        // ポップアップ表示後にエラー音を鳴らす(音量 OFF でも実行する)
        _model->buzzer(LC70_BUZZ_ERR, LC70_BUZZ_MODE_ONESHOT, true);

        emit requestToShowPopupWithMessage(POPUP_POWER_SUPPLY_ERROR_MANUAL_MEASURING, message);
    }

    if (_view) {
        _view->setProperty("isGoodIconVisible", false);
    }
}

void SingleMeasurementViewModel::selectReactionScanResult(const int index) {
    if (index == 0) {
        // スキャンアイコンを表示する
        _model->replaceMenuList(MenuListIconIndex::SCAN_ICON, "icon_scan_n.png");
    }
}

void SingleMeasurementViewModel::onRequestToReScan() {
    // スキャン中ポップアップを表示
    emit requestToShowPopup(POPUP_SCANNING);

    // 再スキャン
    onRequestToScan();
}

void SingleMeasurementViewModel::updateMerCnrRange(const int bandId, const int broadBandSystem) {

    // MERの変化単位の更新
    _merRange = Correcter::merRange(bandId, broadBandSystem, getModulationValue());

    // MERの目盛りに表示する文字列を更新する
    int range = _merRange.second - _merRange.first;
    int step = range / MER_STEP_VALUE;

    QStringList stepNumber = {};
    for (int i = 0; i <= step; ++i) {
        stepNumber.append(QString::number(_merRange.first+(MER_STEP_VALUE * i)));
    }
    if (_view) {
        _view->setProperty("merCnrGaugeStepValue", QVariant::fromValue(stepNumber));
    }
}

void SingleMeasurementViewModel::changeActiveButton(int buttonIndex) {
    static const QMap<int, QString> buttonNameMap = {
        {MEASUREMENT_BUTTON_INDEX,   BUTTON_NAME_SINGLE},
        {CONSTELLATION_BUTTON_INDEX, BUTTON_NAME_CONSTELLATION},
        {DELAY_PROFILE_BUTTON_INDEX, BUTTON_NAME_DELAY_PROFILE},
    };

    if (buttonNameMap.count(buttonIndex) == 0) {
        return;
    }

    QMap<int, QString>::const_iterator iterator;
    for (iterator = buttonNameMap.begin(); iterator != buttonNameMap.end(); ++iterator) {
        bool isActive = false;
        if (iterator.key() == buttonIndex) {
            isActive = true;
            _activeButton = iterator.value();
            _buttonOperation->setActiveObject(_activeButton);
        }
        if (_view) {
            setButtonActive(_view, iterator.value(), isActive);
        }
    }
}

void SingleMeasurementViewModel::moveProfileMarkerInitialPosition() {
    if (_lineSeries == nullptr) {
        return;
    }

    // 遅延プロファイルマーカー位置を初期位置へ移動
    _delayTimeIndex = DELAY_PROFILE_ZERO_INDEX;

    int count = 0;
    if (_lineSeries) {
        count = _lineSeries->count();
    }
    qreal step = static_cast<qreal>(DELAY_PROFILE_XAXIS_LENGTH) / static_cast<qreal>(count);
    qreal x = step * _delayTimeIndex;

    if (_delayProfile) {
        _delayProfile->setProperty("markerX",x);
    }

    if (_view) {
        // 遅延時間 初期値を表示
        _view->setProperty("delayTime", DELAY_TIME_INITIAL_VALUE);

        // レベル差 初期値を表示
        _view->setProperty("levelDifference", LEVEL_DEFFERENCE_INITIAL_VALUE);
    }
}

void SingleMeasurementViewModel::onRequestToShowRegistScanResultPopup() {
    emit requestToShowPopup(POPUP_CONFIRM_REGIST_SCAN_RESULT);
}

void SingleMeasurementViewModel::onNotifyStopMeasuring(AcquisitionTarget target)
{
    if (target != SINGLE) {
        return;
    }

    // シングル測定の停止を受信
}

void SingleMeasurementViewModel::useDefaultMerRange()
{
    // デフォルトの値を使用する
    updateMerCnrRange(LC70_BAND_UNDEF, LC70_BD_SYS_NONE);

    // MER に閾値を通知
    emit notifyMerThreshold(static_cast<double>(0), static_cast<double>(0));
}

void SingleMeasurementViewModel::holdMeasurementData()
{
    // 保持しているデータをクリアする
    clearHoldMeasurementData();

    // 中心周波数, 局発周波数を取得する
    unsigned int centerFrequency = Correcter::centerFrequency(_channelList[_currentChannelIndex], _modulation); // 測定条件の変調方式から
    unsigned int localFrequency = _takeOver["selectedLocalOscillator"].toUInt();

    // 現在のデータを保持する
#if 1	//210628_dx2
	// BSCS統合モードなどでも BAND、局発周波数を正しく表示するために修正
    _holdMeasurementData.Band = QString::number(static_cast<LC70_BAND_T>(_channelList[_currentChannelIndex].band));
    localFrequency = _condition.ulLoFreq;
#else
    _holdMeasurementData.Band = _takeOver["selectedBand"].toString();
#endif

#if 1	//210628_dx2
	// PILOT、FM、CATV上りでチャンネル情報を表示するために修正
    _holdMeasurementData.Channel = _channelList[_currentChannelIndex].channelNumber;
#else
    // BANDがPILOT,FM, CATV上りの場合は、チャンネル情報は保存しない
    if(_holdMeasurementData.Band == QString::number(LC70_BAND_PILOT)
            || _holdMeasurementData.Band == QString::number(LC70_BAND_FM)
            || _holdMeasurementData.Band == QString::number(LC70_BAND_CATV_UP_STREAM)) {
        _holdMeasurementData.Channel = "";
    } else {
        _holdMeasurementData.Channel = _channelList[_currentChannelIndex].channelNumber;
    }
#endif

    _holdMeasurementData.Frequency = Correcter::frequency(centerFrequency, localFrequency);
    _holdMeasurementData.LocalOscillator = localFrequency;
    _holdMeasurementData.BroadcastSystem = _broadcastSystem;
    _holdMeasurementData.Modulation = _modulationInMeasureResult;   // 測定結果の変調方式
    _holdMeasurementData.Layer = _layer;
    if (Correcter::isSatelliteBroadCasting(_channelList[_currentChannelIndex].band)) {
        _layer = _channelList[_currentChannelIndex].polarization;
    }

    _holdMeasurementData.Layer = _layer;


    _holdMeasurementData.Level = _level;
    _holdMeasurementData.Mer = _mer;
    _holdMeasurementData.Ber = _ber;
    _holdMeasurementData.Timestamp = _timeStamp;
    _holdMeasurementData.Locked = _locked;

}

void SingleMeasurementViewModel::clearHoldMeasurementData()
{
    MeasureResult initialData;
    _holdMeasurementData = initialData;
}

int SingleMeasurementViewModel::comboboxDisplayPattern(int bandId)
{
    QMap<LC70_BAND_T, int> comboboxDisplayPatternMap = {
        {LC70_BAND_FM,             FmPattern},
        {LC70_BAND_CATV_UP_STREAM, CatvUpStreamPattern}
    };

    return comboboxDisplayPatternMap.value(static_cast<LC70_BAND_T>(bandId), -1);
}

int SingleMeasurementViewModel::comboboxRowNumber(int bandId)
{
    // バンドIDとコンボボックスの行との関係マップ
    QMap<LC70_BAND_T, int> comboboxRowNumberMap = {
        {LC70_BAND_FM,             4},
        {LC70_BAND_CATV_UP_STREAM, 5}
    };

    return comboboxRowNumberMap.value(static_cast<LC70_BAND_T>(bandId), 0);
}

int SingleMeasurementViewModel::comboboxColumnNumber(int bandId)
{
    // BANDIDとコンボボックスの列との関係マップ
    QMap<LC70_BAND_T, int> comboboxColumnNumberMap = {
        {LC70_BAND_FM,             1},
        {LC70_BAND_CATV_UP_STREAM, 1}
    };

    return comboboxColumnNumberMap.value(static_cast<LC70_BAND_T>(bandId), 0);
}

QVector<QVector<QString>> SingleMeasurementViewModel::comboboxData(int bandId)
{
    // BANDIDとコンボボックスのデータとの関係マップ
    QMap <LC70_BAND_T, QVector<QVector<QString>>> comboboxDataMap = {
        {LC70_BAND_FM,             {{{"edit1"}, {"edit2"}, {""}, {"edit4"}}}            } ,
        {LC70_BAND_CATV_UP_STREAM, {{{"edit1"}, {"edit2"}, {""}, {"edit4"}, {"edit5"}}} }
    };

    return comboboxDataMap.value(static_cast<LC70_BAND_T>(bandId), {{{""}}});
}


void SingleMeasurementViewModel::editValue(ButtonId buttonId)
{
    QObject *changeTarget = _combobox->findChild<QObject *>(_activeButton);

    if (changeTarget) {
        int targetValue = changeTarget->property("message").toInt();

        if (buttonId == BUTTON_UP) {
            targetValue += comboboxChangeValue();
        } else if (buttonId == BUTTON_DOWN){
            targetValue -= comboboxChangeValue();
        } else {
            // nothing to do;
        }

        // コンボボックスの変化域を取得 循環して表示できるようにする
        QPair<int, int> changeRange = comboboxChangeRange(_channelList[_currentChannelIndex].band);
        if (targetValue < changeRange.first) {
            targetValue = changeRange.second;
        } else if(targetValue > changeRange.second) {
            targetValue = changeRange.first;
        } else {
            // nothing to do
        }
        // 変更した値をコンボボックスへ反映する
        changeTarget->setProperty("message", targetValue);
    }
}

int SingleMeasurementViewModel::comboboxChangeValue()
{
    // 各コンボボックスで、UP/DOWNボタンを押下した時の変化量を取得する
    // 5つ目のコンボボックスは、CATV上りの場合のみ表示され
    // CATV上りは0.05刻みでチャンネルが存在するため変化量も5ずつ変化するようにしている。
    QMap<QString, int> comboboxChangeValueMap = {
        {"edit1", 1},
        {"edit2", 1},
        {"edit4", 1},
        {"edit5", 5}
    };

    return comboboxChangeValueMap.value(_activeButton, 1);
}

QPair<int, int> SingleMeasurementViewModel::comboboxChangeRange(int bandId)
{
    // BAND FMの場合の各コンボボックスの変化範囲Map
    QMap<QString, QPair<int, int>> comboboxChangeRangeMapFM = {
        {"edit1", {7, 9}},
        {"edit2", {0, 9}},
        {"edit4", {0, 9}}
    };

    // BAND CATV上りの場合の各コンボボックスの変化範囲Map
    QMap<QString, QPair<int, int>> comboboxChangeRangeMapCATVUPSTREAM = {
        {"edit1", {1, 6}},
        {"edit2", {0, 9}},
        {"edit4", {0, 9}},
        {"edit5", {0, 5}}
    };

    if (static_cast<LC70_BAND_T>(bandId) == LC70_BAND_FM) {
        return comboboxChangeRangeMapFM.value(_activeButton, {0, 9});
    }

    if (static_cast<LC70_BAND_T>(bandId) == LC70_BAND_CATV_UP_STREAM) {
        return comboboxChangeRangeMapCATVUPSTREAM.value(_activeButton, {0,9});
    }

    return {0, 9};
}

bool SingleMeasurementViewModel::refrectFrequencyCombobox()
{
    qreal frequencyValue = 0;
    QVector<QVector<QString>> data = comboboxData(_channelList[_currentChannelIndex].band);

    for (int columnIndex = 0; columnIndex < comboboxColumnNumber(_channelList[_currentChannelIndex].band); ++columnIndex) {
        for(int rowIndex = 0; rowIndex < comboboxRowNumber(_channelList[_currentChannelIndex].band); ++rowIndex) {
            if (data[columnIndex][rowIndex].isEmpty()) {
                continue;
            }

            // コンボボックス毎に値を足し合わせ変更した周波数の値を取得する
            QObject *target = _combobox->findChild<QObject *>(data[columnIndex][rowIndex]);
            int targetValue = target->property("message").toInt();
            frequencyValue += targetValue * comboboxRadix(data[columnIndex][rowIndex]);
        }
    }

    int decimalPoint = 0;
    if (_channelList[_currentChannelIndex].band == LC70_BAND_FM) {
        decimalPoint = 1;
    } else if (_channelList[_currentChannelIndex].band == LC70_BAND_CATV_UP_STREAM) {
        decimalPoint = 2;
    }

    QString frequency = QString("%1").arg(static_cast<float32_t>(frequencyValue), 0, 'f', decimalPoint);

    if (_combobox) {
        _combobox->setProperty("value", frequency);
    }

    int channelIndex = -1;
    for (int i = 0; i < _channelList.size(); ++i) {
        if (_channelList[i].channelNumber == frequency) {
            channelIndex = i;
            break;
        }
    }

    if (channelIndex != -1) {
        // 変更したチャンネル情報が定義されている場合
        _currentChannelIndex = channelIndex;
        return true;
    }

    // 変更したチャンネル情報が定義されていない場合
    return false;
}

qreal SingleMeasurementViewModel::comboboxRadix(QString comboboxName)
{
    // 各コンボボックスの基数を取得する
    QMap<QString, qreal> comboboxRadixMap = {
        {"edit1", 10.00},
        {"edit2",  1.00},
        {"edit4",  0.10},
        {"edit5",  0.01}
    };

    return comboboxRadixMap.value(comboboxName, 0.0);
}

void SingleMeasurementViewModel::refreshAllChannels()
{
#if 1
    //bandId = _takeOver["selectedPolarization"].toString() になると思われるが既存実装への影響を懸念してBSCSのみ別処理とする。
#endif
    int bandId = _channelList[_currentChannelIndex].band;
    _channelList.clear();
    QString polarization = "";

    if (_takeOver.contains("selectedPolarization")) {
        polarization = _takeOver["selectedPolarization"].toString();
    }
#if 1
    // 画面共有情報からBandを取得
    int selectedBand = _takeOver["selectedBand"].toInt();

    if (selectedBand == LC70_BAND_BS_CS) {
        _model->selectChannelDefinitionList(LC70_BAND_BS_CS, polarization, _channelList);
    }
    else {
        _model->selectChannelDefinitionList(bandId, polarization, _channelList);
    }
#else
    _model->selectChannelDefinitionList(bandId, polarization, _channelList);
#endif
}

void SingleMeasurementViewModel::selectPowerSupplyForUserSetting()
{
    if (_userSettings.count() > _currentChannelIndex) {
        BandTabIndex bandTabIndex = Correcter::convertBandTabIndex(static_cast<LC70_BAND_T>(_userSettings[_currentChannelIndex].ucBand),
                                                                   Correcter::convertPolarizeNumToString(_channelList[_currentChannelIndex].polarization));

        // 給電 ON の場合に Band に応じて値を付け替える
        switch (bandTabIndex) {
        case BAND_TAB_JCSAT3A_H:
        case BAND_TAB_SCCB2_H:
        case BAND_TAB_SCCC2_H:
            _powerSupply = LC70_POWER_SUPPLY_15V;
            break;
        case BAND_TAB_JCSAT3A_V:
        case BAND_TAB_SCCB2_V:
        case BAND_TAB_SCCC2_V:
            _powerSupply = LC70_POWER_SUPPLY_11V;
            break;
        case BAND_TAB_JCSAT4B_H:
            _powerSupply = LC70_POWER_SUPPLY_15V_P;
            break;
        case BAND_TAB_JCSAT4B_V:
            _powerSupply = LC70_POWER_SUPPLY_11V_P;
            break;
        default:
            _powerSupply = LC70_POWER_SUPPLY_15V;
            break;
        }
    } else {
        qDebug() << QString("invalid user settings");
        return;
    }
}
