/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef SINGLEMEASUREMENTVIEWMODEL_H
#define SINGLEMEASUREMENTVIEWMODEL_H

#include "../abstractmeasurementviewmodel.h"
#include "buttonoperation.h"
#include "correcter.h"
#include "delayprofilelineseriesmodel.h"
#include <QTimer>

/**
 * @brief BER の値域の種類
 */
enum BER_RANGE {
    OVER_E2 = 0,    // 0.1       〜
    E2_E3,          // 0.01      〜 0.1
    E3_E4,          // 0.001     〜 0.01
    E4_E5,          // 0.0001    〜 0.001
    E5_E6,          // 0.00001   〜 0.0001
    E6_E7,          // 0.000001  〜 0.00001
    E7_E8,          // 0.0000001 〜 0.000001
    UNDER_E8,       //           〜 0.0000001
};

/**
 * @brief シングルチャンネル測定 ViewModel
 */
class SingleMeasurementViewModel : public AbstractMeasurementViewModel
{
    Q_OBJECT
public:
    /**
     * @brief コンストラクター
     * @param screenId 画面 ID
     * @param model Model クラスのポインター
     * @param parent 親オブジェクト
     */
    explicit SingleMeasurementViewModel(ScreenId screenId, LevelCheckerModel *&model, QObject *parent = nullptr);

    /**
     * @brief デストラクタ
     */
    virtual ~SingleMeasurementViewModel();

    /**
     * @brief 画面を開くときに呼び出すスレッドの開始、再開処理
     */
    virtual void resume(void) override;

    /**
     * @brief 画面を閉じるときに呼び出すスレッドの停止処理
     */
    virtual void suspend(void) override;

    /**
     * @brief 画面を開くときに呼び出す qml との接続処理
     * @param qml qml のオブジェクトのポインター
     * @param mainFrame MainFrameViewModel のポインター
     */
    virtual void createConnection(QObject *qml, QObject *mainFrame) override;

    /**
     * @brief 画面を開くときに呼び出す表示内容の復元処理
     */
    virtual void activate(void) override;

    /**
     * @brief 画面を閉じるときに呼び出す表示な用の保存処理
     */
    virtual void deactivate(void) override;

    /**
     * @brief 有効なチャンネルであるか判定する
     * @param channleIndex チャンネルインデックス
     * @return true:有効、false:無効
     */
    bool isValidChannel(int channleIndex) override;

    /**
     * @brief メニューバーによる設定変更
     * @note スレッドは止めずに変更した設定で表示し直す
     */
    void changeByMenuBar(void) override;

signals:

    /**
     * @brief 画面共通情報設定要求
     * @param key 画面共通情報キー
     * @param value 画面共通情報データ
     */
    void requestSetTakeOver(QVariant key, QVariant value);

    /**
     * @brief 測定結果取得開始、再開依頼を通知
     * @param constellationPoints コンスタレーション収集件数
     * @param condition チャンネル別測定条件
     * @param interval 測定結果取得間隔
     */
    void resumeSingleMeasuring(LC70_CONSTA_NUM_T constellationPoints, LC70_MEAS_SINGLE_T condition, uint32_t interval);

    /**
     * @brief チャンネル変更
     * @param condition チャンネル別測定条件
     */
    void changeCondition(LC70_MEAS_SINGLE_T condition);

    /**
     * @brief 画面のキャプチャー依頼
     */
    void requestToCapture(void);

    /**
     * @brief キャプチャーした画像の保存依頼
     * @param measureType 測定種類
     * @param saveLocation 保存先
     * @param fileName ファイル名
     */
    void requestToSaveImage(MeasureType measureType, SaveLocation saveLocation, QString fileName);

    /**
     * @brief キャプチャーした画像の削除依頼
     */
    void requestToClearCaptureImage(void);

    /**
     * @brief データ保存を依頼
     * @param saveLocation 保存先
     * @param fileName ファイル名
     * @param condition 測定条件
     * @param singleMeasureResult 測定結果
     */
    void requestToSaveData(SaveLocation saveLocation, QString fileName, uint16_t condition,
                           MeasureResult singleMeasureResult);

    /**
     * @brief メニューバーのカーソルのあるボタンを押した状態にする
     * @param rank 階層
     */
    void cursorPressed(QVariant rank);

    /**
     * @brief プログレスバーの進捗の値を通知
     * @param value 進捗の値
     */
    void progress(double value);

    /**
     * @brief プログレスを通知
     * @param current 現在、ロックされたチャンネル
     * @param flag ロックされた
     */
    void progressText(QString current, bool flag);

    /**
     * @brief 遅延プロファイル有効状態変更通知
     * @param valid 有効状態
     */
    void notifyDelayProfileValid(bool valid);

    /**
     * @brief 再スキャン要求
     */
    void requestToReScan(void);

    /**
     * @brief LEVEL メーター更新
     * @param level LEVEL の値
     * @param levelNeedlePosition LEVEL メーターの針の角度
     */
    void notifyLevelMeter(QVariant level, QVariant levelNeedlePosition);

    /**
     * @brief MER メーター更新
     * @param mer MER の値
     * @param merNeedlePosition MER メーターの針の角度
     */
    void notifyMerMeter(QVariant mer, QVariant merNeedlePosition);

    /**
     * @brief BER メーター更新
     * @param ber BER の値
     * @param berNeedlePosition BER メーターの針の角度
     */
    void notifyBerMeter(QVariant ber, QVariant berNeedlePosition);

    /**
     * @brief Level の閾値の通知
     * @param red 受信不可の割合
     * @param yellow 受信可の割合
     */
    void notifyLevelThreshold(QVariant red, QVariant yellow);

    /**
     * @brief MER の閾値の通知
     * @param red 受信不可の割合
     * @param yellow 受信可の割合
     */
    void notifyMerThreshold(QVariant red, QVariant yellow);

    /**
     * @brief BER の閾値の通知
     * @param red 受信不可の割合
     * @param yellow 受信可の割合
     */
    void notifyBerThreshold(QVariant red, QVariant yellow);

    /**
     * @brief スキャン結果保存確認ダイアログを表示する
     * @note コンテキストを切るためにシグナル/スロットで実装する
     */
    void requestToShowRegistScanResultPopup(void);

    /**
     * @brief シングル測定停止依頼
     */
    void requestToSuspendSingleMeasuring(void);

protected:
    /**
     * @brief 初期状態の画面表示に戻す
     */
    virtual void resetScreen(void) override;

    /**
     * @brief 必要に応じて View を指すポインターを付け替えた後の復帰処理を行う
     */
    virtual void recover(void) override;

protected slots:
    /**
     * @brief ボタン押下の状態を受信
     * @param id ボタン ID
     */
    virtual void onKeyStatus(ButtonId id) override;

    /**
     * @brief ボタン押下を受信
     * @param id ボタン ID
     */
    virtual void onButtonPress(ButtonId id) override;

    /**
     * @brief ポップアップから受諾の通知を受信
     * @param okOnly OK のみの場合 true
     */
    virtual void onAccept(PopupId popupId, bool okOnly) override;

    /**
     * @brief ポップアップから拒否通知を受信
     * @param popupId 表示中ポップアップのID
     */
    virtual void onReject(PopupId popupId) override;

    /**
     * @brief スキャン結果ポップアップから拒否通知を受信
     */
    virtual void onRejectScanResult() override;

    /**
     * @brief ポップアップから選択の通知を受信
     * @param index 選択されたインデックス
     */
    virtual void onSelect(PopupId popupId, int index) override;

private slots:
    /**
     * @brief onRequest
     * @param condition
     */
    virtual void onRequest(int condition) override;

    /**
     * @brief シングル測定結果を受信
     * @param resultれ 測定結果
     * @param constellations コンスタレーション
     */
    void onSingleMeasureResult(MeasureResult result, QList<QVariant> constellations);

    /**
     * @brief 入力されたファイル名を受信
     * @param popupId ポップアップの種類
     * @param saveLocation 保存先
     * @param fileName 入力されたファイル名
     */
    void onInputFileName(PopupId popupId, SaveLocation saveLocation, QString fileName);

    /**
     * @brief 遅延プロファイル測定結果を受信
     * @param result 遅延プロファイル測定結果
     */
    void onDelayProfileResult(QVector<QPointF> result);

    /**
     * @brief チャンネル変更中通知を受信
     * @param changing 変更中か否か
     */
    void onNotifyChannelChanging(bool changing);

    /**
     * @brief スキャンの実行依頼を受信
     * @note 呼び出し画面でスキャン処理(開始、結果取得、中止)を実施
     */
    void onRequestToScan(void);

    /**
     * @brief スキャン中止依頼を受信
     */
    virtual void onRequestToStopScanning(void) override;

    /**
     * @brief スキャン結果を受信
     * @param index スキャン対象のチャンネルリストのインデックス
     * @param result スキャン結果
     */
    void onScanningChannelResult(int index, LC70_MEAS_MULTI_RESULT_T result);

    /**
     * @brief スキャン終了通知を受信
     * @param complete 完了有無
     */
    void onScanningFinished(bool complete);

    /**
     * @brief ポップアップ表示中に入力されたボタンのボタン ID を受信
     * @param popupId ポップアップ ID
     * @param buttonId ボタン ID
     */
    void onNotifyButtonInput(PopupId popupId, ButtonId buttonId);

    /**
     * @brief チャンネル確定遅延タイマー タイムアウト受信時処理
     */
    void onConfirmedChangeChannelTimeout(void);

    /**
     * @brief 遅延プロファイル有効状態受信
     * @param valid 有効状態
     */
    void onNotifyDelayProfileValid(bool valid);

    /**
     * @brief 給電異常検知
     * @param 故障コード
     * @param action 故障時の動作
     * @param message 表示メッセージ
     */
    void detectPowerSupplyError(uint16_t code, uint8_t action, QString message);

    /**
     * @brief 再スキャン要求受信
     */
    void onRequestToReScan(void);

    /**
     * @brief メーター表示更新(アニメーション用)
     */
    void onUpdateMeter(void);

    /**
     * @brief スキャン結果保存確認ダイアログを表示する
     * @note コンテキストを切るためにシグナル/スロットで実装
     */
    void onRequestToShowRegistScanResultPopup(void);

    /**
     * @brief 測定停止を受信
     * @param target 測定の種類
     */
    void onNotifyStopMeasuring(AcquisitionTarget target);

private:
    /**
     * @brief 表示するボタンのパターン
     */
    enum ButtonPattern {
        NormalPattern = 0,  // 地デジ(測定、コンスタレーション、遅延プロファイル)
        LimitedPattern1,    // 衛星放送、CATV(測定、コンスタレーション)
        LimitedPattern2     // LTE, PILOT, FM(測定)
    };

    /**
     * @brief コンボボックスの操作パターン
     */
    enum ComboboxState {
        Normal = 0,          // 通常モード
        Edit                 // 操作中モード
    };

    /**
     * @brief コンボボックスのパターン
     */
    enum ComboboxPattern {
        FmPattern = 0,      // FMを表示するパターン
        CatvUpStreamPattern // CATV上りを表示するパターン
    };

    ButtonOperation *_buttonOperation;              //!< ボタン操作クラス
    QString _activeButton;                          //!< アクティブなボタンの objectName
    QObject *_meter;                                //!< シングル測定の QML
    QObject *_constellation;                        //!< コンスタレーションの QML
    QObject *_delayProfile;                         //!< 遅延プロファイルの QML
    QString _captureFileName;                       //!< キャプチャー画像ファイル名
    float32_t _level;                               //!< LEVEL
    float32_t _mer;                                 //!< MER
    float32_t _ber;                                 //!< BER
    LC70_CONSTA_NUM_T _limitContellationPoints;     //!< コンスタレーションで取得するデータ数の上限
    int _selectedButtonIndex;                       //!< 選択機能
    ButtonPattern _buttonPattern;                   //!< 表示するボタンのパターン
    QList<ChannelDefinitionItem>
    _preservedChannelList;                          //!< 保持しておくチャンネル定義情報のリスト(ユーザー設定を除く)
    LC70_MEAS_SINGLE_T _condition;                  //!< チャンネル別測定条件
    LC70_MEAS_SINGLE_T _previousCondition;          //!< 前回のチャンネル別測定条件
    uint32_t _modulation;                           //!< 測定条件の変調方式
    uint8_t _layer;                                 //!< 測定結果から取得した階層、偏波面
    QString _timeStamp;                             //!< 測定結果から取得した日時
    SaveFileType _saveFileType;                     //!< 保存するファイルの種類
    DelayProfileLineSeriesModel *_lineSeries;       //!< 遅延プロファイルに表示するデータ
    bool _isConstellationDataClear;                 //!< コンスタレーションの表示クリア有無
    QVector<LC70_LOGGER_SET_LOGGER_ROW_T> _userSettings;   //!< ユーザー設定
    bool _isChannelChanging;                        //!< チャンネル変更中
    bool _fromUserSetting;                          //!< ユーザー設定からの遷移
    bool _thresholdSet;                             //!< 閾値設定済みか否か
    uint16_t _broadcastSystem;                      //!< 測定条件の放送方式
    QVector<QVector<QString>> _measurementButtons;  //!< 選択ボタン
    bool _alreadySuspended;                         //!< 既に一時停止しているか否か(マルチ測定に遷移する場合に true にする)
    QTimer *_confirmedChangeChannelTimer;           //!< チャンネル確定遅延タイマー
    float32_t  _levelReceivableThreshold;           //!< 受信可能かどうかの閾値(Level)
    float32_t  _mercnMerginHighThreshold;           //!< 余裕度高かどうかの閾値(MER/CN)
    float32_t  _berMarginHighThreshold;             //!< 余裕度高かどうかの閾値(BER)
    bool _isDelayProfileValid;                      //!< 遅延プロファイル有効状態
    int _delayTimeIndex;                            //!< 遅延時間のインデックス
    bool _cursoredGraph;                            //!< グラフにカーソルが当たっているか true: 当たっている, false: 当たっていない
    QPair<int, int> _merRange;                      //!< MER/CNRの値域(first:最小値、second:最大値)
    QVariant _cnrMerSetting;                        //!< MER/CNRの切替設定
    QVariant _broadBandModulationSetting;           //!< マニュアル測定 放送方式, 変調方式選択
    float32_t _newestLevel;                         //!< 最新の値(LEVEL)
    float32_t _newestMer;                           //!< 最新の値(MER)
    float32_t _newestBer;                           //!< 最新の値(BER)
    bool _isUpdateMeter;                            //!< メーター更新稼働中フラグ
    int _meterUpdateInterval;                       //!< メーター更新周期
    int _invalidValueMeterSpeed;                    //!< 入力無効時のメーター針移動倍率
    QTimer *_levelFixTimer;                         //!< Level の表示反映用タイマー
    QTimer *_merFixTimer;                           //!< MER の表示反映用タイマー
    QTimer *_berFixTimer;                           //!< BER の表示反映用タイマー
    bool _displayLevel;                             //!< Level を表示
    bool _displayMer;                               //!< MER を表示
    bool _displayBer;                               //!< BER を表示
    MeasureResult _holdMeasurementData;             //!< 保存する測定データ (DATAボタン押下時の測定データ)
    QObject *_combobox;                             //!< コンボボックスのQML
    bool _iscomboboxVisible;                        //!< コンボボックスが表示されているか
    ComboboxState  _comboboxState;                  //!< コンボボックスの状態
    uint32_t _modulationInMeasureResult;            //!< 測定結果中の変調方式
    ButtonId _pressedChannelButtonId;               //!< 押下されたチャンネル変更ボタン
    LC70_BUZZ_T _buzzerState;                       //!< ブザー状態
    uint16_t _failureCode;                          //!< 故障コード
    uint8_t _actionInFailure;                       //!< 故障時の動作
    bool _powerSupplyErrorOccured;                  //!< 給電異常発生中か否か
    QTimer *_lockWaitingTimer;                      //!< ロック待ちのタイマー
    bool _waitforLockingTimeOut;                    //!< ロック待ち時間満了か否か
    bool _confirmedValue;                           //!< 確定した値か否か(ロックの取得あるいは待ち時間満了で確定とする)
    uint8_t _locked;

    /**
     * @brief 選択ボタンの移動
     * @param buttonId ボタン ID
     */
    void moveCursor(ButtonId buttonId);

    /**
     * @brief 周波数コンボボックの操作
     * @param buttonId ボタン ID
     */
    void comboboxOperation(ButtonId buttonId);

    /**
     * @brief チャンネル変更
     * @param isComfirmChangeChannel 処理中にチャンネル変更を確定させるか (true: 確定する, false: 確定しない)
     * @note CHANNEL_UP_BUTTON, CHANNEL_DOWN_BUTTONの押下時は、連打対策のためtimerのtimeoutを受けた時点でチャンネル変更を確定する
     */
    void changeChannel(bool isComfirmChangeChannel = true);

    /**
     * @brief 表示クリア
     */
    void clear(void);

    /**
     * @brief 閾値の反映
     * @param bandId Band の ID
     */
    void refreshThreshold(const int &bandId);

    /**
     * @brief MER の更新
     * @param result 測定結果
     * @return MER の値
     */
    float32_t updateMer(const MeasureResult &result);

    /**
     * @brief MER メーターの針の角度の更新
     * @param mer MER の値
     * @return MER メーターの針の角度
     */
    float32_t updateMerNeedle(const float32_t &mer);

    /**
     * @brief BER の更新
     * @param result 測定結果
     * @return BER の値
     */
    float32_t updateBer(const MeasureResult &result);

    /**
     * @brief BER の更新
     * @param ber BER の値
     * @return BER メーターの針の角度
     */
    float32_t updateBerNeedle(const float32_t &ber);

    /**
     * @brief 遅延プロファイル測定が有効な Band であるか判定する
     * @param channelIndex チャンネルのIndex
     * @return true：遅延プロファイル測定が有効なチャンネル、false：無効なチャンネル
     */
    bool isEnableDelayProfileChannel(int channelIndex);

    /**
     * @brief コンスタレーション測定が有効な Band であるか判定する
     * @param band Band の ID
     * @return true：コンスタレーション測定が有効な Band、false：無効な Band
     */
    bool isEnableConstellationBandId(int band);

    /**
     * @brief チャンネル変更確定タイマー開始処理
     */
    void startConfirmedChangeChannelTimer(void);

    /**
     * @brief 鳴動中のブザーを止める
     * @note  受信良好状態ではなくなった際に鳴動中のブザーを止める
     */
    void stopBuzzer(void);

    /**
     * @brief 閾値条件更新チェック
     * @param bdSys 放送方式
     * @param modulation 変調方式
     */
    void checkThresholdCondition(uint16_t bdSys, uint32_t modulation);

    /**
     * @brief 遅延プロファイルの表示データをクリア
     */
    void clearDelayProfileInfo(void);

    /**
     * @brief 遅延プロファイルグラフクリア
     */
    void clearGraph(void);

    /**
     * @brief ガードインターバル値取得
     * @param interval 計測結果のガードインターバル
     * @return インターバル値
     */
    float32_t guardIntervalValue(const uint8_t interval);

    /**
     * @brief 遅延時間取得
     * @return 遅延時間
     */
    QString getDelayTime(void);

    /**
     * @brief プロパティに Bool 値を設定
     * @param objectName オブジェクト名
     * @param propertyName プロパティ名
     * @param value 設定する Bool 値
     */
    void setBoolProperty(QString objectName, QString propertyName, bool value);

    /**
     * @brief 局発周波数の値を取得
     * @return 局発周波数の値
     */
    unsigned int getLocalFrequency(void);

    /**
     * @brief 変調方式の値を取得
     * @return 変調方式の値
     */
    unsigned int getModulationValue(void);

    /**
     * @brief BERゲージの比率を計算する
     * @param value BERの閾値の値
     * @return 比率
     */
    float32_t calcBerZoneRatio(float32_t value);

    /**
     * @brief 測定結果から受信良好状態かを調べる
     * @param result 測定結果
     * @return 受信良好状態かどうか True: 受信良好状態, False: 受信良好状態ではない
     */
    bool checkGoodReception(MeasureResult result);

    /**
     * @brief スキャン結果後の動作選択
     * @param index 選択結果
     */
    void selectReactionScanResult(const int index);

    /**
     * @brief LEVEL メーター更新
     * @param levelString LEVEL の表示値
     * @param levelNeedle LEVEL メーターの針の角度
     */
    void updateLevelMeter(QString &levelString, float32_t &levelNeedle);

    /**
     * @brief MER メーター更新
     * @param merString MER の表示値
     * @param merNeedle MER メーターの針の角度
     */
    void updateMerMeter(QString &merString, float32_t &merNeedle);

    /**
     * @brief BER メーター更新
     * @param berString BER の表示値
     * @param berNeedle BER メーターの針の角度
     */
    void updateBerMeter(QString &berString, float32_t &berNeedle);

    /**
     * @brief BER の値から値域の種類を取得
     * @param value BER の値
     * @return BER の値域の種類
     */
    BER_RANGE getBerRange(const float32_t &value);

    /**
     * @brief 設定リセット
     * @param condition 測定種別
     */
    void resetSettings(int condition);

    /**
     * @brief シングル測定状態への変更
     * @param isSuspend サスペンド時の呼び出しでは true
     */
    void changeSettingToSingleMeasurement(bool isSuspend = false);

    /**
     * @brief MER/CNRの範囲を更新する(最大値/最小値の更新)
     * @param bandId 測定するBAND (LC70_BAND_xxxの値)
     * @param broadBandSystem 放送方式 (LC70_BD_SYS_xxxの値)
     */
    void updateMerCnrRange(const int bandId, const int broadBandSystem);

    /**
     * @brief 測定モードに対応したボタン状態変更
     * @param buttonIndex ボタンインデックス
     */
    void changeActiveButton(int buttonIndex);

    /**
     * @brief 遅延プロファイルマーカー位置を初期値へ移動
     */
    void moveProfileMarkerInitialPosition(void);

    /**
     * @brief Merの範囲設定を初期値の状態に戻す
     */
    void useDefaultMerRange(void);

    /**
     * @brief 関数実行時の測定データを保持する
     * @note DATAボタンが押下されたときに使用する
     */
    void holdMeasurementData(void);

    /**
     * @brief 保持している測定データを初期化する
     */
    void clearHoldMeasurementData(void);

    /**
     * @brief コンボボックスの表示パターンを取得する
     * @param  bandId 測定BANDのID (LC70_BAND_Tの値)
     * @return コンボボックスの表示パターン
     */
    int comboboxDisplayPattern(int bandId);

    /**
     * @brief  コンボボックスの行数の取得
     * @param  bandId 測定BANDのID (LC70_BAND_Tの値)
     * @return コンボボックスの行数
     */
    int comboboxRowNumber(int bandId);

    /**
     * @brief  コンボボックスの列数の取得
     * @param  bandId 測定BANDのID (LC70_BAND_Tの値)
     * @return コンボボックスの列数
     */
    int comboboxColumnNumber(int bandId);

    /**
     * @brief  コンボボックスの情報を取得
     * @param  bandId 測定BANDのID (LC70_BAND_Tの値)
     * @return コンボボックスの情報
     */
    QVector<QVector<QString>> comboboxData(int bandId);

    /**
     * @brief 周波数の値の変更を行う
     * @param ButtonId 入力したボタンID
     */
    void editValue(ButtonId buttonId);

    /**
     * @brief  編集中のコンボボックスの変化量を取得する
     * @return 編集中コンボボックスの変化量
     */
    int comboboxChangeValue(void);

    /**
     * @brief  編集中のコンボボックスの変化域を取得する
     * @param  bandId バンドID (LC70_BAND_Tの値)
     * @return 編集中のコンボボックスの変化域
     */
    QPair<int, int> comboboxChangeRange(int bandId);

    /**
     * @brief  コンボボックスに対して変更した値を反映する
     * @return コンボボックスに対しての反映可否
     */
    bool refrectFrequencyCombobox(void);

    /**
     * @brief コンボボックスの基数を取得するし
     * @param comboboxName 対象のコンボボックス名
     * @return 引数で指定したコンボボックスの基数
     */
    qreal comboboxRadix(QString comboboxName);

    /**
     * @brief 全チャンネルを再取得する
     */
    void refreshAllChannels(void);

    /**
     * @brief USER 設定で給電 ON の場合の設定
     */
    void selectPowerSupplyForUserSetting(void);
};

#endif // SINGLEMEASUREMENTVIEWMODEL_H
