/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include "constellationsettingviewmodel.h"

QVector<QVector<QString>> constellationSettingButtonData = {
    {{"Constellation0"}, {"Constellation1"}, {"Constellation2"}}
};

const int ConstellationSettingViewModel::BUTTON_COLUMN_NUMBER = 1;
const int ConstellationSettingViewModel::BUTTON_ROW_NUMBER = 3;

ConstellationSettingViewModel::ConstellationSettingViewModel(ScreenId screenId, LevelCheckerModel *&model,
        QObject *parent)
    : AbstractViewModel(screenId, model, parent)
{
    _buttonOperation.setButtonName(BUTTON_COLUMN_NUMBER, BUTTON_ROW_NUMBER, constellationSettingButtonData);

    _activeButton = constellationSettingButtonData[0][0];
    _buttonOperation.setActiveObject(_activeButton);
}

ConstellationSettingViewModel::~ConstellationSettingViewModel()
{

}

void ConstellationSettingViewModel::activate()
{
    if (nullptr == _view) {
        qErrnoWarning("_view is null");
        return;
    }

    // ボタンActive設定
    QVariant value;
    int buttonIndex = 0;

    if (_model->getValue(VALUE_CONSTELLATION, value)) {
        buttonIndex = getButtonIndex(value.toInt());
    }

    // Active にするボタンを設定
    QString itemname;
    itemname.sprintf("Constellation%d", buttonIndex);

    QObject *item = nullptr;
    if (_view) {
        item = _view->findChild<QObject *>(itemname);
    }
    if (item) {
        item->setProperty("active", true);
    }
    _activeButton = itemname;
    _buttonOperation.setActiveObject(_activeButton);
}

void ConstellationSettingViewModel::deactivate()
{
    AbstractViewModel::deactivate();
    // 保存していなければ編集内容を破棄
    if (_model->isUpdate(VALUE_CONSTELLATION)) {
        _model->rollback(VALUE_CONSTELLATION);
    }
}

void ConstellationSettingViewModel::moveCursor(ButtonId buttonId)
{
    if ((buttonId == BUTTON_RIGHT) || (buttonId == BUTTON_LEFT)) {
        // UP, DOWN なし

        QString nextBtn;
        bool ret = _buttonOperation.nextActiveObject(buttonId, nextBtn);

        if (ret) {

            // inactive
            QObject *target = nullptr;
            if (_view) {
                target = _view->findChild<QObject *>(_activeButton);
            }
            if (target) {
                target->setProperty("active", false);
            }

            // active
            target = nullptr;
            if (_view) {
                target = _view->findChild<QObject *>(nextBtn);
            }
            if (target) {
                target->setProperty("active", true);
            }

            _activeButton = nextBtn;
            _buttonOperation.setActiveObject(_activeButton);

            QVariant value;

            if (_activeButton == "Constellation0") {
                value.setValue(LC70_MEAS_SET_CONSTA_NUM_1000);
            } else if (_activeButton == "Constellation1") {
                value.setValue(LC70_MEAS_SET_CONSTA_NUM_2000);
            } else if (_activeButton == "Constellation2") {
                value.setValue(LC70_MEAS_SET_CONSTA_NUM_4000);
            } else {

            }

            _model->setValue(VALUE_CONSTELLATION, value);
        }
    }
}

void ConstellationSettingViewModel::onKeyStatus(ButtonId id)
{
    if (id == BUTTON_ENTER) {
        QObject *target = nullptr;
        if (_view) {
            target = _view->findChild<QObject *>(_activeButton);
        }
        if (target) {
            target->setProperty("press", false);
        }
        // POPUP 表示
        if (_model->isUpdate(VALUE_CONSTELLATION)) {
            emit requestToShowPopup(POPUP_SAVE_CONFIRMATION); // 保存確認
        } else {
            // 変化がなければ前の画面に戻る
            emit back();
        }
    } else if (id == BUTTON_ESCAPE || id == BUTTON_HOME || id == BUTTON_USER) {
        if (_model->isUpdate(VALUE_CONSTELLATION)) {
            emit requestToShowPopup(POPUP_CANCEL_CONFIRMATION); // 破棄確認
        } else {
            onAccept(POPUP_CANCEL_CONFIRMATION, true);
        }
    } else {
        moveCursor(id);
    }
}

void ConstellationSettingViewModel::onAccept(PopupId popupId, bool okOnly)
{
    Q_UNUSED(okOnly)

    AbstractViewModel::onAccept(popupId, okOnly);

    if (popupId == POPUP_SAVE_CONFIRMATION) {
        _model->commit(VALUE_CONSTELLATION);

        // メニューに戻る
        emit back();

    } else if (popupId == POPUP_CANCEL_CONFIRMATION) {
        _model->rollback(VALUE_CONSTELLATION);

        // 押下されたボタンにより遷移先を変更する
        switch (_lastPressedButtonId) {
        case BUTTON_HOME:
            emit requestDirect(SCREEN_HOME, true);
            break;

        case BUTTON_ESCAPE:
            emit back();
            break;

        case BUTTON_USER:
#ifdef NEW_USER_SETTING
        emit requestDirect(SCREEN_USER_SETTING, true);
#else
        emit requestDirect(SCREEN_USER_DATA_LOGGER_MENU, true);
#endif
            break;

        default:
            break;
        }
    }
}

void ConstellationSettingViewModel::onButtonPress(ButtonId id)
{
    if (id == BUTTON_ENTER) {
        QObject *target = nullptr;
        if (_view) {
            target = _view->findChild<QObject *>(_activeButton);
        }
        if (target) {
            target->setProperty("press", true);
        }
    }
}

int ConstellationSettingViewModel::getButtonIndex(const int constallationValue)
{
    // コンスタレーションの設定値とボタンインデックスとのマップ
    static const QMap<int, int> ButtonMap = {
        {LC70_CONSTA_1000, 0},
        {LC70_CONSTA_2000, 1},
        {LC70_CONSTA_4000, 2}
    };

    // 値が得られなかった場合は、初期設定選択時のボタンインデックスを返す
    return ButtonMap.value(constallationValue, 1);
}
