/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include "localoscillatorviewmodel.h"

QVector<QVector<QString>> localOscillatorButtonData = {
    {{"oscillatorItem1"}, {"register"}},
    {{"oscillatorItem2"}, {""}},
    {{"oscillatorItem3"}, {""}},
    {{"oscillatorItem4"}, {""}},
};

QVector<QVector<QString>> localOscillatorEditButtonData = {
    {{"edit1"}, {"edit2"}, {""}, {"edit4"}, {"edit5"}, {"edit6"}}
};

const int LocalOscillatorViewModel::BUTTON_COLUMN_NUMBER = 4;
const int LocalOscillatorViewModel::BUTTON_ROW_NUMBER = 2;

const int LocalOscillatorViewModel::EDIT_BUTTON_COLUMN_NUMBER = 1;
const int LocalOscillatorViewModel::EDIT_BUTTON_ROW_NUMBER = 6;
const int LocalOscillatorViewModel::EDIT_LOCAL_FREQUENCY_INDEX = 7;

// 設定できる最大値、最小値 ( 注 : kHz単位で定義 )
const uint LocalOscillatorViewModel::LOCAL_FREQ_MAX = 12000000;
const uint LocalOscillatorViewModel::LOCAL_FREQ_MIN = 9000000;

LocalOscillatorViewModel::LocalOscillatorViewModel(ScreenId screenId, LevelCheckerModel *&model, QObject *parent)
    : AbstractViewModel(screenId, model, parent)
    , _screenState(SELECT)
{
    _buttonOperation.setButtonName(BUTTON_COLUMN_NUMBER, BUTTON_ROW_NUMBER, localOscillatorButtonData);
}

LocalOscillatorViewModel::~LocalOscillatorViewModel()
{

}

void LocalOscillatorViewModel::activate(void)
{
    if (_view == nullptr) {
        qErrnoWarning("_view is null");
        return;
    }

    connect(this, SIGNAL(requestSetTakeOver(QVariant, QVariant)), _view, SLOT(backup(QVariant, QVariant)));

    // 局部発振周波数設定値表示
    setLocalFrequencyData();

    // Active にするボタンを設定
    QString itemname("oscillatorItem1");
    QObject *item = nullptr;
    if (item) {
        item = _view->findChild<QObject *>(itemname);
    }
    if (item) {
        item->setProperty("active", true);
    }
    _selectItem = itemname;
    _buttonOperation.setActiveObject(_selectItem);

    // 画面遷移時は、リスト選択モード
    _screenState = SELECT;
}

void LocalOscillatorViewModel::deactivate(void)
{
    AbstractViewModel::deactivate();
}

void LocalOscillatorViewModel::moveCursor(ButtonId buttonId)
{
    if ((buttonId == BUTTON_RIGHT) || (buttonId == BUTTON_LEFT) || (buttonId == BUTTON_UP) || (buttonId == BUTTON_DOWN)) {
        if (_screenState == SELECT) {
            // 選択モードの場合
            QString nextButton;

            if ((buttonId == BUTTON_UP) || (buttonId == BUTTON_DOWN)) {
                // 右上を選択している場合 カーソル移動は実施しない
                if (_selectItem == localOscillatorButtonData[0][1]) {
                    return;
                }

                // 上下移動の場合はnextActiveObject関数で検索
                _buttonOperation.nextActiveObject(buttonId, nextButton);

            } else if ((buttonId == BUTTON_RIGHT) || (buttonId == BUTTON_LEFT)) {
                // 左右移動の場合 一番上のボタンに移動
                if (_selectItem == localOscillatorButtonData[0][1]) {
                    nextButton = localOscillatorButtonData[0][0];
                } else {
                    nextButton = localOscillatorButtonData[0][1];
                }
            }

            // inactivateオブジェクト (選択していたオブジェクト)に対しての操作
            QObject *target = nullptr;
            if (_view) {
                target = _view->findChild<QObject *>(_selectItem);
            }
            if (target) {
                target->setProperty("active", false);
            }

            // activeオブジェクト (選択するオブジェクト)に対しての操作
            target = nullptr;
            if (_view) {
                target = _view->findChild<QObject *>(nextButton);
            }
            if (target) {
                target->setProperty("active", true);
            }

            _selectItem = nextButton;
            _buttonOperation.setActiveObject(_selectItem);
        } else {
            // 編集モードの場合
            editNumber(buttonId);
        }
    }
}

void LocalOscillatorViewModel::editNumber(ButtonId buttonId)
{
    if (buttonId == BUTTON_LEFT || buttonId == BUTTON_RIGHT) {

        // 編集する数値の移動
        QString nextButton;
        bool ret = _editButtonOperation.nextActiveObject(buttonId, nextButton);

        if (ret) {
            // 選択中のリスト項目オブジェクト
            QObject *targetbase = nullptr;
            if (_view) {
                targetbase = _view->findChild<QObject *>(_selectItem);
            }
            // inactiveオブジェクト (編集していた数値)に対しての操作
            QObject *target = nullptr;
            if (targetbase) {
                target = targetbase->findChild<QObject *>(_activeEditButton);
            }
            // 非activeに変更
            if (target) {
                target->setProperty("active", false);
            }

            // activeオブジェクト (編集する数値)に対しての操作
            target = nullptr;
            if (targetbase) {
                target = targetbase->findChild<QObject *>(nextButton);
            }

            // activeに変更
            if (target) {
                target->setProperty("active", true);
            }

            _activeEditButton = nextButton;
            _editButtonOperation.setActiveObject(_activeEditButton);
        }
    } else if (buttonId == BUTTON_UP || buttonId == BUTTON_DOWN) {
        // 数値の増減
        int increase = 0;

        switch (buttonId) {
        case BUTTON_UP:
            increase = 1;
            break;

        case BUTTON_DOWN:
            increase = (-1);
            break;

        default:
            break;
        }

        // 数値更新実施
        QObject *targetbase = nullptr;
        if (_view) {
            targetbase = _view->findChild<QObject *> (_selectItem);
        }
        QObject *target = nullptr;
        if (targetbase) {
            target = targetbase->findChild<QObject *>(_activeEditButton);
        }
        if (target) {
            QVariant value = target->property("message");
            int result = value.toUInt();

            result += increase;

            // 0から9の間で循環表示をするように設定
            value = (result < 0 ? 9 : (result > 9 ? 0 : result));
            if (target) {
                target->setProperty("message", value);
            }
        }
    } else {
        // nothing to do
    }
}

void LocalOscillatorViewModel::onKeyStatus(ButtonId id)
{
    // 電源ボタン押下は、HOME画面と同様の処理を行う
    // HOME ボタン押下で本画面での設定を消去し、HOME画面に遷移する。
    // USER ボタン押下で本画面での設定を消去し、USER画面に遷移する。
    // ESCAPE ボタン押下で本画面での設定を消去し、測定設定画面(遷移元画面)に遷移する。
    if (id == BUTTON_ENTER) {
        onKeyEnter();
    } else if (id == BUTTON_ESCAPE || id == BUTTON_HOME || id == BUTTON_USER) {
        if (_model->isUpdate(VALUE_LOCAL_FREQUENCY)) {
            emit requestToShowPopup(POPUP_CANCEL_CONFIRMATION); // 変更破棄確認
        } else {
            onAccept(POPUP_CANCEL_CONFIRMATION, true);
        }
    } else {
        moveCursor(id);
    }
}

// acceptシグナル受信時
void LocalOscillatorViewModel::onAccept(PopupId popupId, bool okOnly)
{
    Q_UNUSED(okOnly)

    AbstractViewModel::onAccept(popupId, okOnly);

    if (popupId == POPUP_SAVE_CONFIRMATION) {
        // 設定値保存し、遷移元画面に遷移する。
        _model->commit(VALUE_LOCAL_FREQUENCY);
        emit back();
    } else if (popupId == POPUP_CANCEL_CONFIRMATION) {
        // 設定値破棄
        _model->rollback(VALUE_LOCAL_FREQUENCY);

        // 押下されたボタンにより遷移先を変更する
        switch (_lastPressedButtonId) {
        case BUTTON_HOME:
            emit requestDirect(SCREEN_HOME, true);
            break;

        case BUTTON_ESCAPE:
            emit back();
            break;

        case BUTTON_USER:
#ifdef NEW_USER_SETTING
        emit requestDirect(SCREEN_USER_SETTING, true);
#else
        emit requestDirect(SCREEN_USER_DATA_LOGGER_MENU, true);
#endif

        default:
            break;
        }
    } else if ( popupId == POPUP_WRONG_INPUT_DATA) {
        // nothing to do.
    }
}

// rejectシグナル受信時
void LocalOscillatorViewModel::onReject(PopupId popupId)
{
    Q_UNUSED(popupId);
}

// Enterキー押下時の処理
void LocalOscillatorViewModel::onKeyEnter()
{
    QObject *target = nullptr;
    if (_view) {
        target = _view->findChild<QObject *>(_selectItem);
    }
    if (target) {
        if (_selectItem == localOscillatorButtonData[0][1]) {
            // 保存ボタン選択時
            if (_model->isUpdate(VALUE_LOCAL_FREQUENCY)) {
                if (checkValue()) {
                    // 設定値チェックOK
                    emit requestToShowPopup(POPUP_SAVE_CONFIRMATION);
                } else {
                    // 設定値チェックNG
                    emit requestToShowPopup(POPUP_WRONG_LOCAL_OSCILLATOION_DATA);
                }
            } else {
                onAccept(POPUP_SAVE_CONFIRMATION, true);
            }
        } else {
            // リストアイテム選択時
            QVariant screenState = target->property("edit");

            if (screenState.toBool() == false) {
                // 編集モードへ移行
                _screenState = EDIT;
                target->setProperty("edit", true);

                // ボタンオペレーションの切替を実施
                _editButtonOperation.setButtonName(EDIT_BUTTON_COLUMN_NUMBER, EDIT_BUTTON_ROW_NUMBER, localOscillatorEditButtonData);

                _activeEditButton = "edit1";
                _editButtonOperation.setActiveObject(_activeEditButton);
            } else {
                // 選択モードへ移行
                _screenState = SELECT;
                target->setProperty("edit", false);

                // [Memo]
                //  以下、画面側でのデータ書き込み処理

                // 入力された数値の取得でmodel 設定値を更新
                QVariant targetValue;
                targetValue = target->property("value");

                // kHzからGHzに値を変更する
                int FrequencyValue = targetValue.toUInt() * 1000;

                // modelから変更するデータを取得
                QVariant value;
                _model->getValue(VALUE_LOCAL_FREQUENCY, value);
                QVariantList localFrequencyList =  value.toList();

                if (_selectItem == "oscillatorItem1") {
                    // BAND JCSAT-3Aの場合 偏波: 水平, 垂直方向の2方向のデータを更新する
                    QVariantList editTarget = localFrequencyList.at(LC70_MEAS_SET_LOCAL_FREQ_IDX_JCSAT3_V).toList();
                    editTarget.replace(EDIT_LOCAL_FREQUENCY_INDEX, FrequencyValue);
                    localFrequencyList.replace(LC70_MEAS_SET_LOCAL_FREQ_IDX_JCSAT3_V, editTarget);
                    editTarget = localFrequencyList.at(LC70_MEAS_SET_LOCAL_FREQ_IDX_JCSAT3_H).toList();
                    editTarget.replace(EDIT_LOCAL_FREQUENCY_INDEX, FrequencyValue);
                    localFrequencyList.replace(LC70_MEAS_SET_LOCAL_FREQ_IDX_JCSAT3_H, editTarget);

                } else if (_selectItem == "oscillatorItem2") {
                    // BAND JCSAT-4Bの場合 偏波: 水平, 垂直方向の2方向のデータを更新する
                    QVariantList editTarget = localFrequencyList.at(LC70_MEAS_SET_LOCAL_FREQ_IDX_JCSAT4_V).toList();
                    editTarget.replace(EDIT_LOCAL_FREQUENCY_INDEX, FrequencyValue);
                    localFrequencyList.replace(LC70_MEAS_SET_LOCAL_FREQ_IDX_JCSAT4_V, editTarget);
                    editTarget = localFrequencyList.at(LC70_MEAS_SET_LOCAL_FREQ_IDX_JCSAT4_H).toList();
                    editTarget.replace(EDIT_LOCAL_FREQUENCY_INDEX, FrequencyValue);
                    localFrequencyList.replace(LC70_MEAS_SET_LOCAL_FREQ_IDX_JCSAT4_H, editTarget);

                } else if (_selectItem == "oscillatorItem3") {
                    // BAND SCC-B2の場合 偏波: 水平, 垂直方向の2方向のデータを更新する
                    QVariantList editTarget = localFrequencyList.at(LC70_MEAS_SET_LOCAL_FREQ_IDX_SCCB_V).toList();
                    editTarget.replace(EDIT_LOCAL_FREQUENCY_INDEX, FrequencyValue);
                    localFrequencyList.replace(LC70_MEAS_SET_LOCAL_FREQ_IDX_SCCB_V, editTarget);
                    editTarget = localFrequencyList.at(LC70_MEAS_SET_LOCAL_FREQ_IDX_SCCB_H).toList();
                    editTarget.replace(EDIT_LOCAL_FREQUENCY_INDEX, FrequencyValue);
                    localFrequencyList.replace(LC70_MEAS_SET_LOCAL_FREQ_IDX_SCCB_H, editTarget);

                } else if (_selectItem == "oscillatorItem4") {
                    // BAND SCC-C2の場合 偏波: 水平, 垂直方向の2方向のデータを更新する
                    QVariantList editTarget = localFrequencyList.at(LC70_MEAS_SET_LOCAL_FREQ_IDX_SCCC_V).toList();
                    editTarget.replace(EDIT_LOCAL_FREQUENCY_INDEX, FrequencyValue);
                    localFrequencyList.replace(LC70_MEAS_SET_LOCAL_FREQ_IDX_SCCC_V, editTarget);
                    editTarget = localFrequencyList.at(LC70_MEAS_SET_LOCAL_FREQ_IDX_SCCC_H).toList();
                    editTarget.replace(EDIT_LOCAL_FREQUENCY_INDEX, FrequencyValue);
                    localFrequencyList.replace(LC70_MEAS_SET_LOCAL_FREQ_IDX_SCCC_H, editTarget);
                }

                // LevelCheckerModelを用いて値の更新を実施する。
                _model->setValue(VALUE_LOCAL_FREQUENCY, localFrequencyList);
            }
        }
    }
}

// 画面項目に対し、局部発振周波数を設定
void LocalOscillatorViewModel::setLocalFrequencyData()
{
    // 設定値をModelから取得
    QVariant value;
    _model->getValue(VALUE_LOCAL_FREQUENCY, value);

    QVariantList localFrequencyList = value.toList();
    QObject *target;
    QString showValue;

    // JCSAT-3A 局部発振周波数データ取得
    // 水平, 垂直どちらの偏波の場合でも同じ値が設定されている前提であるため 偏波: 水平から取得
    QVariantList jcsat3FreqList = localFrequencyList.at(
                                      LC70_MEAS_SET_LOCAL_FREQ_IDX_T::LC70_MEAS_SET_LOCAL_FREQ_IDX_JCSAT3_H).toList();
    // 表示値に変更
    int freqValue = jcsat3FreqList[EDIT_LOCAL_FREQUENCY_INDEX].toUInt() / 1000;
    showValue = QString::number(freqValue).rightJustified(5, '0');
    target = nullptr;
    if (_view) {
        target = _view->findChild<QObject *>("oscillatorItem1");
    }
    if (target) {
        target->setProperty("value", showValue);
    }
    // JCSAT-4B 局部発振周波数データ取得
    // 水平, 垂直どちらの偏波の場合でも同じ値が設定されている前提であるため 偏波: 水平から取得
    QVariantList jcsat4FreqList = localFrequencyList.at(
                                      LC70_MEAS_SET_LOCAL_FREQ_IDX_T::LC70_MEAS_SET_LOCAL_FREQ_IDX_JCSAT4_H).toList();
    // 表示値に変更
    freqValue = jcsat4FreqList[EDIT_LOCAL_FREQUENCY_INDEX].toUInt() / 1000;
    showValue = QString::number(freqValue).rightJustified(5, '0');
    target = nullptr;
    if (_view) {
        target = _view->findChild<QObject *>("oscillatorItem2");
    }
    if (target) {
        target->setProperty("value", showValue);
    }
    // SCC-B2 局部発振周波数データ取得
    // 水平, 垂直どちらの偏波の場合でも同じ値が設定されている前提であるため 偏波: 水平から取得
    QVariantList sccbFreqList = localFrequencyList.at(
                                    LC70_MEAS_SET_LOCAL_FREQ_IDX_T::LC70_MEAS_SET_LOCAL_FREQ_IDX_SCCB_H).toList();
    // 表示値に変更
    freqValue = sccbFreqList[EDIT_LOCAL_FREQUENCY_INDEX].toUInt() / 1000;
    showValue = QString::number(freqValue).rightJustified(5, '0');
    target = nullptr;
    if (_view) {
        target = _view->findChild<QObject *>("oscillatorItem3");
    }
    if (target) {
        target->setProperty("value", showValue);
    }
    // SCC-C2 局部発振周波数データ取得
    // 水平, 垂直どちらの偏波の場合でも同じ値が設定されている前提であるため 偏波: 水平から取得
    QVariantList scccFreqList = localFrequencyList.at(
                                    LC70_MEAS_SET_LOCAL_FREQ_IDX_T::LC70_MEAS_SET_LOCAL_FREQ_IDX_SCCC_H).toList();
    // 表示値に変更
    freqValue = scccFreqList[EDIT_LOCAL_FREQUENCY_INDEX].toUInt() / 1000;
    showValue = QString::number(freqValue).rightJustified(5, '0');
    target = nullptr;
    if (_view) {
        target = _view->findChild<QObject *>("oscillatorItem4");
    }
    if (_view) {
        target->setProperty("value", showValue);
    }
}

// 値チェック
bool LocalOscillatorViewModel::checkValue()
{
    QVariant value;
    int settingValue;

    // modelから設定値を取得
    _model->getValue(VALUE_LOCAL_FREQUENCY, value);
    QVariantList localFrequencyList = value.toList();

    // JCSAT-4B 局部発振周波数データ
    // 水平, 垂直どちらの偏波の場合でも同じ値が設定されている前提であるため 偏波: 水平から取得
    settingValue = localFrequencyList.at(LC70_MEAS_SET_LOCAL_FREQ_IDX_T::LC70_MEAS_SET_LOCAL_FREQ_IDX_JCSAT4_H).toList().at(
                       EDIT_LOCAL_FREQUENCY_INDEX).toUInt();

    if (!checkLocalFrequency(settingValue)) {
        return false;
    }

    // JCSAT-3A 局部発振周波数データ
    // 水平, 垂直どちらの偏波の場合でも同じ値が設定されている前提であるため 偏波: 水平から取得
    settingValue = localFrequencyList.at(LC70_MEAS_SET_LOCAL_FREQ_IDX_T::LC70_MEAS_SET_LOCAL_FREQ_IDX_JCSAT3_H).toList().at(
                       EDIT_LOCAL_FREQUENCY_INDEX).toUInt();

    if (!checkLocalFrequency(settingValue)) {
        return false;
    }

    // SCC-C2 局部発振周波数データ
    // 水平, 垂直どちらの偏波の場合でも同じ値が設定されている前提であるため 偏波: 水平から取得
    settingValue = localFrequencyList.at(LC70_MEAS_SET_LOCAL_FREQ_IDX_T::LC70_MEAS_SET_LOCAL_FREQ_IDX_SCCC_H).toList().at(
                       EDIT_LOCAL_FREQUENCY_INDEX).toUInt();

    if (!checkLocalFrequency(settingValue)) {
        return false;
    }

    // SCC-B2 局部発振周波数データ
    // 水平, 垂直どちらの偏波の場合でも同じ値が設定されている前提であるため 偏波: 水平から取得
    settingValue = localFrequencyList.at(LC70_MEAS_SET_LOCAL_FREQ_IDX_T::LC70_MEAS_SET_LOCAL_FREQ_IDX_SCCB_H).toList().at(
                       EDIT_LOCAL_FREQUENCY_INDEX).toUInt();

    if (!checkLocalFrequency(settingValue)) {
        return false;
    }

    return true;
}

// 局部発振周波数のチェック
bool LocalOscillatorViewModel::checkLocalFrequency(uint localFrequency)
{
    // 設定値が0の場合は、エラーにしない
    if (localFrequency == 0) {
        return true;
    }

    // 設定値が9よりも小さい、12よりも大きい値の場合エラーとする。
    if ( localFrequency < LOCAL_FREQ_MIN || LOCAL_FREQ_MAX < localFrequency) {
        return false;
    }

    return true;
}
