/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include "switchmercnviewmodel.h"

QVector<QVector<QString>> merCnButtonData = {
    {{"MERCN1"}, {"MERCN2"}}
};

const int SwitchMERCNViewModel::BUTTON_COLUMN_NUMBER = 1;
const int SwitchMERCNViewModel::BUTTON_ROW_NUMBER = 2;

const int SwitchMERCNViewModel::MER = 1;
const int SwitchMERCNViewModel::CN = 0;

SwitchMERCNViewModel::SwitchMERCNViewModel(ScreenId screenId, LevelCheckerModel *&model, QObject *parent)
    : AbstractViewModel(screenId, model, parent)
{
    _buttonOperation.setButtonName(BUTTON_COLUMN_NUMBER, BUTTON_ROW_NUMBER, merCnButtonData);
}

SwitchMERCNViewModel::~SwitchMERCNViewModel()
{

}

void SwitchMERCNViewModel::activate()
{
    if (nullptr == _view) {
        qErrnoWarning("_view is null");
        return;
    }

    // 設定値の取得
    QVariant value;
    int valueMerCN = -1;

    if (_model->getValue(VALUE_MER_CN, value)) {
        qDebug() << "MER_CN :" << value;
        valueMerCN = value.toInt();

        if (valueMerCN < 0 || valueMerCN > 1) {
            // 得られる値が範囲外(0, 1以外の値であった場合)
            // 初期値のMERを選択する
            valueMerCN = LC70_MEAS_SET_SEL_MER;
        }
    }

    // Active にするボタンを設定
    QString itemName;

    if (valueMerCN == MER) {
        // MERを選択している場合
        itemName = "MERCN1";
    } else {
        // C/Nを選択している場合
        itemName = "MERCN2";
    }

    QObject *item = nullptr;
    if (_view) {
        item = _view->findChild<QObject *>(itemName);
    }
    if (item) {
        item->setProperty("active", true);
    }
    _activeButton = itemName;
    _buttonOperation.setActiveObject(_activeButton);

}

void SwitchMERCNViewModel::moveCursor(ButtonId buttonId)
{
    if ((buttonId == BUTTON_RIGHT) || (buttonId == BUTTON_LEFT)) {
        // UP, DOWN なし

        QString nextBtn;
        bool ret = _buttonOperation.nextActiveObject(buttonId, nextBtn);

        if (ret) {

            // inactive
            QObject *target = nullptr;
            if (_view) {
                target = _view->findChild<QObject *>(_activeButton);
            }
            if (target) {
                target->setProperty("active", false);
            }

            // active
            target = nullptr;
            if (_view) {
                target = _view->findChild<QObject *>(nextBtn);
            }
            if (target) {
                target->setProperty("active", true);
            }

            _activeButton = nextBtn;
            _buttonOperation.setActiveObject(_activeButton);

            QVariant value;

            if (_activeButton == "MERCN1") {
                // MERを選択時
                value = MER;
            } else if (_activeButton == "MERCN2") {
                // C/Nを選択時
                value = CN;
            } else {
                // 何もしない
            }

            _model->setValue(VALUE_MER_CN, value);
        }
    }
}

void SwitchMERCNViewModel::onKeyStatus(ButtonId id)
{
    if (id == BUTTON_ENTER) {
        if (_model->isUpdate(VALUE_MER_CN)) {
            emit requestToShowPopup(POPUP_SAVE_CONFIRMATION); // 保存確認
        } else {
            // 変更が無い場合は遷移元画面に遷移する。
            emit back();
        }
    } else if (id == BUTTON_ESCAPE || id == BUTTON_HOME || id == BUTTON_USER) {
        if (_model->isUpdate(VALUE_MER_CN)) {
            emit requestToShowPopup(POPUP_CANCEL_CONFIRMATION); // 破棄確認
        } else {
            onAccept(POPUP_CANCEL_CONFIRMATION, true);
        }
    } else {
        moveCursor(id);
    }
}

void SwitchMERCNViewModel::onAccept(PopupId popupId, bool okOnly)
{
    Q_UNUSED(okOnly)

    AbstractViewModel::onAccept(popupId, okOnly);

    // MER/CN の設定
    if (popupId == POPUP_SAVE_CONFIRMATION) {
        _model->commit(VALUE_MER_CN);

        // メニューに戻る
        emit back();

    } else if (popupId == POPUP_CANCEL_CONFIRMATION) {
        _model->rollback(VALUE_MER_CN);

        // 押下されたボタンにより遷移先を変更する
        switch (_lastPressedButtonId) {
        case BUTTON_HOME:
            emit requestDirect(SCREEN_HOME, true);
            break;

        case BUTTON_ESCAPE:
            emit back();
            break;

        case BUTTON_USER:
#ifdef NEW_USER_SETTING
        emit requestDirect(SCREEN_USER_SETTING, true);
#else
        emit requestDirect(SCREEN_USER_DATA_LOGGER_MENU, true);
#endif
            break;

        default:
            break;
        }
    }
}
