/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include "switchopticalcoaxialcableviewmodel.h"

QVector<QVector<QString>> corexialOpticalButtonData = {
    {{"setting1"}, {"setting2"}}
};

const int SwitchOpticalCoaxialCableViewModel::BUTTON_COLUMN_NUMBER = 1;
const int SwitchOpticalCoaxialCableViewModel::BUTTON_ROW_NUMBER = 2;

const int SwitchOpticalCoaxialCableViewModel::COAX_CABLE_NUMBER = 0;
const int SwitchOpticalCoaxialCableViewModel::OPTICAL_CABLE_NUMBER = 1;

SwitchOpticalCoaxialCableViewModel::SwitchOpticalCoaxialCableViewModel(ScreenId screenId, LevelCheckerModel *&model,
        QObject *parent)
    : AbstractViewModel(screenId, model, parent)
{
    _buttonOperation.setButtonName(BUTTON_COLUMN_NUMBER, BUTTON_ROW_NUMBER, corexialOpticalButtonData);

    _activeButton = corexialOpticalButtonData[0][0];
    _buttonOperation.setActiveObject(_activeButton);
}

SwitchOpticalCoaxialCableViewModel::~SwitchOpticalCoaxialCableViewModel()
{

}

void SwitchOpticalCoaxialCableViewModel::activate()
{
    if (nullptr == _view) {
        qErrnoWarning("_view is null");
        return;
    }

    // ボタンActive設定
    QVariant value;
    int measInput = 0;
    int buttonIndex = 0;

    if (_model->getValue(VALUE_RF_OPTICAL, value)) {
        measInput = value.toInt();

        if (value.toInt() < 0 || value.toInt() > 1) {
            // 範囲チェック 値の範囲が0~1でなければ初期値である0をセットする。
            measInput = 0;
        }
    }

    buttonIndex = measInput + 1;

    // Active にするボタンを設定
    QString itemname("setting");
    itemname += QString().sprintf("%d", buttonIndex);

    QObject *item = nullptr;
    if (_view) {
        item = _view->findChild<QObject *>(itemname);
    }
    if (item) {
        item->setProperty("active", true);
    }
    _activeButton = itemname;
    _buttonOperation.setActiveObject(_activeButton);
}

void SwitchOpticalCoaxialCableViewModel::deactivate()
{
    AbstractViewModel::deactivate();
    // 保存していなければ編集内容を破棄
    if (_model->isUpdate(VALUE_RF_OPTICAL)) {
        _model->rollback(VALUE_RF_OPTICAL);
    }
}

void SwitchOpticalCoaxialCableViewModel::moveCursor(ButtonId buttonId)
{
    if ((buttonId == BUTTON_RIGHT) || (buttonId == BUTTON_LEFT)) {
        // UP, DOWN なし

        QString nextBtn;
        bool ret = _buttonOperation.nextActiveObject(buttonId, nextBtn);

        if (ret) {

            // inactive
            QObject *target = nullptr;
            if (_view) {
                target = _view->findChild<QObject *>(_activeButton);
            }
            if (target) {
                target->setProperty("active", false);
            }

            // active
            target = nullptr;
            if (_view) {
                target = _view->findChild<QObject *>(nextBtn);
            }
            if (target) {
                target->setProperty("active", true);
            }

            _activeButton = nextBtn;
            _buttonOperation.setActiveObject(_activeButton);

            QVariant value;

            if (_activeButton == "setting1") {
                value = 0;
            } else if (_activeButton == "setting2") {
                value = 1;
            } else {
                // 何もしない
            }

            _model->setValue(VALUE_RF_OPTICAL, value);
        }
    }
}

void SwitchOpticalCoaxialCableViewModel::onKeyStatus(ButtonId id)
{
    if (id == BUTTON_ENTER) {
        QObject *target = nullptr;
        if (_view) {
            target = _view->findChild<QObject *>(_activeButton);
        }
        if (target) {
            target->setProperty("press", false);
        }
        // POPUP 表示
        if (_model->isUpdate(VALUE_RF_OPTICAL)) {
            emit requestToShowPopup(POPUP_SAVE_CONFIRMATION); // 保存確認
        } else {
            // 同じ値の場合はメニューに戻る
            emit back();
        }
    } else if (id == BUTTON_ESCAPE || id == BUTTON_HOME || id == BUTTON_USER) {
        if (_model->isUpdate(VALUE_RF_OPTICAL)) {
            emit requestToShowPopup(POPUP_CANCEL_CONFIRMATION); // 破棄確認
        } else {
            onAccept(POPUP_CANCEL_CONFIRMATION, true);
        }
    } else {
        moveCursor(id);
    }
}

void SwitchOpticalCoaxialCableViewModel::onAccept(PopupId popupId, bool okOnly)
{
    Q_UNUSED(okOnly)

    AbstractViewModel::onAccept(popupId, okOnly);

    //  同軸／光の設定
    if (popupId == POPUP_SAVE_CONFIRMATION) {
        _model->commit(VALUE_RF_OPTICAL);

        // 同軸/光アイコンを更新する
        updateRfOpticalIcon();

        // メニューに戻る
        emit back();

    } else if (popupId == POPUP_CANCEL_CONFIRMATION) {
        _model->rollback(VALUE_RF_OPTICAL);

        // 押下されたボタンにより遷移先を変更する
        switch (_lastPressedButtonId) {
        case BUTTON_HOME:
            emit requestDirect(SCREEN_HOME, true);
            break;

        case BUTTON_ESCAPE:
            emit back();
            break;

        case BUTTON_USER:
#ifdef NEW_USER_SETTING
        emit requestDirect(SCREEN_USER_SETTING, true);
#else
        emit requestDirect(SCREEN_USER_DATA_LOGGER_MENU, true);
#endif
            break;

        default:
            break;
        }
    }
}

void SwitchOpticalCoaxialCableViewModel::updateRfOpticalIcon()
{
    // [Memo]
    //  設定に合わせて光・同軸アイコンの更新を実施する
    QVariant value;
    _model->getValue(VALUE_RF_OPTICAL, value);

    if (value.toInt() == COAX_CABLE_NUMBER) {
        _model->replaceMenuList(MenuListIconIndex::RF_OPTICAL_ICON, "icon_hikari_rf.png");
    } else if (value.toInt() == OPTICAL_CABLE_NUMBER) {
        _model->replaceMenuList(MenuListIconIndex::RF_OPTICAL_ICON, "icon_hikari_opt.png");
    } else {
        // 何もしない
    }
}

void SwitchOpticalCoaxialCableViewModel::onButtonPress(ButtonId id)
{
    if (id == BUTTON_ENTER) {
        QObject *target = nullptr;
        if (_view) {
            target = _view->findChild<QObject *>(_activeButton);
        }
        if (target) {
            target->setProperty("press", true);
        }
    }
}
