/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include "thresholdsettingviewmodel.h"
#include "correcter.h"

QVector<QVector<QString>> thresholdSettingButtonData = {
    { {"select1"}, {"select2"} },
    { {"select3"}, {"select4"} },
    { {"select5"}, {"select6"} },
    { {"save"}, {"initialize"}},
};

QVector<QVector<QString>> thresholdSettingButtonDataLevelOnly = {
    { {"select1"}, {"select2"} },
    { {"save"}, {"initialize"} },
};

QVector<QVector<QString>> thresholdEditButtonData = {
    { {"edit1"}, {"edit2"}, {"edit3"}, {""}, {"edit5"}}
};

QVector<QVector<QString>> thresholdEditExpButtonData = {
    { {"edit1"}, {""}, {"edit3"}, {""}, {"edit5"}}
};

QVector<QVector<QString>> thresholdEditFirstFixedButtonData = {
    { {""}, {"edit2"}, {"edit3"}, {""}, {"edit5"}}
};

const int ThresholdSettingViewModel::LEVELMIN = 200;
const int ThresholdSettingViewModel::LEVELMAX = 1200;
const int ThresholdSettingViewModel::MER_ADJUST = 10;
const double ThresholdSettingViewModel::BERMIN = 1.0e-2;
const double ThresholdSettingViewModel::BERMAX = 1.0e-8;

const int ThresholdSettingViewModel::BUTTON_COLUMN_NUMBER = 4;
const int ThresholdSettingViewModel::BUTTON_ROW_NUMBER = 2;

const int ThresholdSettingViewModel::BUTTON_COLUMN_NUMBER_LEVEL_ONLY = 2;
const int ThresholdSettingViewModel::BUTTON_ROW_NUMBER_LEVEL_ONLY = 2;

const int ThresholdSettingViewModel::EDIT_BUTTON_COLUMN_NUMBER = 1;
const int ThresholdSettingViewModel::EDIT_BUTTON_ROW_NUMBER = 5;


ThresholdSettingViewModel::ThresholdSettingViewModel(ScreenId screenId, LevelCheckerModel *&model, QObject *parent)
    : AbstractViewModel(screenId, model, parent)
    , _merRange(), _screenState(SELECT)
{
    _buttonOperation.setButtonName(BUTTON_COLUMN_NUMBER, BUTTON_ROW_NUMBER, thresholdSettingButtonData);

}

ThresholdSettingViewModel::~ThresholdSettingViewModel()
{

}

void ThresholdSettingViewModel::activate()
{
    if (nullptr == _view) {
        qErrnoWarning("_view is null");
        return;
    }

    connect(this, SIGNAL(requestSetTakeOver(QVariant, QVariant)), _view, SLOT(backup(QVariant, QVariant)));
    connect(this, SIGNAL(requestGetTakeOver()), _view, SLOT(getTakeOver()));
    emit requestGetTakeOver();

    // 前画面で指定された設定を取得
    QObject *target = nullptr;
    if (_view) {
        target = _view->findChild<QObject *>("takeOverData");
    }
    if (target) {
        _selectedBand = target->property("selectedBand").toInt();
        _selectedModulation = target->property("selectedModulation").toString();
        _selectedBroadcasting = target->property("selectedBroadcast").toString();
        int bandId = target->property("selectedBand").toInt();
        _merRange = Correcter::merRange(bandId, Correcter::broadcastSystemValue(_selectedBroadcasting), Correcter::modulationValue(_selectedModulation));
    }

    if (_selectedBand == static_cast<int>(LC70_BAND_PILOT) || _selectedBand == static_cast<int>(LC70_BAND_FM) || _selectedBand == static_cast<int>(LC70_BAND_CATV_UP_STREAM)) {    // MER/CN

        _view->setProperty("merbervis",false);
        _view->setProperty("levelpos",98);
    }else{
        _view->setProperty("merbervis",true);

    }
    // ボタン設定を反映
    setButtonOperation();

    // バンド種別表示
    setBandMessage();

    // 設定値表示
    setThresholdData();

    // Merの値域を設定
    if (_view) {
        _view->setProperty("merMinimum", _merRange.first);
    }
    if (_view) {
        _view->setProperty("merMaximum", _merRange.second);
    }

    // Active にするボタンを設定
    QString itemname("select1");

    QObject *item = nullptr;
    if (_view) {
        item = _view->findChild<QObject *>(itemname);
    }
    if (item) {
        item->setProperty("active", true);
    }
    _selectButton = itemname;
    _buttonOperation.setActiveObject(_selectButton);

    // 初期動作は閾値選択
    _screenState = SELECT;
}

void ThresholdSettingViewModel::deactivate()
{
    AbstractViewModel::deactivate();
    // 保存していなければ編集内容を破棄
    if (_model->isUpdate(VALUE_THRESHOLD)) {
        _model->rollback(VALUE_THRESHOLD);
    }
}

void ThresholdSettingViewModel::setButtonOperation()
{
    // [Memo]
    //  選択BANDが「PILOT」か「FM」,「CATV上り」の場合
    //  LEVELのみが変更可能　MER/CN, BERはdisableにする。
    if (_selectedBand == static_cast<int>(LC70_BAND_PILOT) || _selectedBand == static_cast<int>(LC70_BAND_FM) || _selectedBand == static_cast<int>(LC70_BAND_CATV_UP_STREAM)) {
        QObject *target;

        // _viewに対して送信する。
        target = nullptr;
        if (_view) {
            target = _view->findChild<QObject *>("select3");
        }
        if (target) {
            target->setProperty("disable", true);
        }

        target = nullptr;
        if (_view) {
            target = _view->findChild<QObject *>("select4");
        }
        if (target) {
            target->setProperty("disable", true);
        }

        target = nullptr;
        if (_view) {
            target = _view->findChild<QObject *>("select5");
        }
        if (target) {
            target->setProperty("disable", true);
        }

        target = nullptr;
        if (_view) {
            target = _view->findChild<QObject *>("select6");
        }
        if (target) {
            target->setProperty("disable", true);
        }

        _buttonOperation.setButtonName(BUTTON_ROW_NUMBER_LEVEL_ONLY, BUTTON_ROW_NUMBER_LEVEL_ONLY, thresholdSettingButtonDataLevelOnly);

    } else {
        _buttonOperation.setButtonName(BUTTON_COLUMN_NUMBER, BUTTON_ROW_NUMBER, thresholdSettingButtonData);
    }
}

void ThresholdSettingViewModel::moveCursor(ButtonId buttonId)
{
    if ((buttonId == BUTTON_RIGHT) || (buttonId == BUTTON_LEFT) || (buttonId == BUTTON_UP) || (buttonId == BUTTON_DOWN)) {
        switch (_screenState) {
        case SELECT: {
            // 閾値の移動
            QString nextBtn;
            bool ret = _buttonOperation.nextActiveObject(buttonId, nextBtn);

            if (ret) {

                // inactive
                QObject *target = nullptr;
                if (_view) {
                    target = _view->findChild<QObject *>(_selectButton);
                }
                if (target) {
                    target->setProperty("active", false);
                }

                // active
                target = nullptr;
                if (_view) {
                    target = _view->findChild<QObject *>(nextBtn);
                }
                if (target) {
                    target->setProperty("active", true);
                }

                _selectButton = nextBtn;
                _buttonOperation.setActiveObject(_selectButton);
            }

            break;
        }

        case EDIT: {
            editNumber(buttonId);
        }

        default:
            break;
        }
    }
}

void ThresholdSettingViewModel::onButtonPress(ButtonId id)
{
    if (id == BUTTON_ENTER) {
        QObject *target = nullptr;
        if (_view) {
            target = _view->findChild<QObject *>(_selectButton);
        }
        target->setProperty("press", true);
    }
}

void ThresholdSettingViewModel::editNumber(ButtonId buttonId)
{
    if (buttonId == BUTTON_LEFT || buttonId == BUTTON_RIGHT) {

        // 数値の移動
        QString nextBtn;
        bool ret = _editButtonOperation.nextActiveObject(buttonId, nextBtn);

        if (ret) {
            // 選択されている閾値オブジェクト
            QObject *targetbase = nullptr;
            if (_view) {
                targetbase = _view->findChild<QObject *>(_selectButton);
            }

            // inactive
            QObject *target = nullptr;
            if (targetbase) {
                target = targetbase->findChild<QObject *>(_activeEditButton);
            }
            if (target) {
                target->setProperty("active", false);
            }

            // active
            target = nullptr;
            if (targetbase) {
                target = targetbase->findChild<QObject *>(nextBtn);
            }
            if (target) {
                target->setProperty("active", true);
            }

            _activeEditButton = nextBtn;
            _editButtonOperation.setActiveObject(_activeEditButton);
        }
    } else if (buttonId == BUTTON_UP || buttonId == BUTTON_DOWN) {
        // 数値の増減
        int increase = 0;

        switch (buttonId) {
        case BUTTON_UP:
            increase = 1;
            break;

        case BUTTON_DOWN:
            increase = (-1);
            break;

        default:
            break;
        }


        // 選択されている閾値オブジェクト
        QObject *targetbase = nullptr;
        if (_view) {
            targetbase = _view->findChild<QObject *>(_selectButton);
        }
        QObject *target = nullptr;
        if (targetbase) {
            target = targetbase->findChild<QObject *>(_activeEditButton);
        }
        if (target) {
            QVariant value = target->property("message");
            int result = value.toInt();

            // 数値更新
            result += increase;

            // 0 から 9 の間で循環表示
            value = (result < 0 ? 9 : (result > 9 ? 0 : result));
            if (target) {
                target->setProperty("message", value);
            }
        }
    } else {
        // nothing to do
    }

}

void ThresholdSettingViewModel::onKeyStatus(ButtonId id)
{
    // 電源 ボタン押下は、HOME画面と同様の処理を行う
    // HOME ボタン押下で本画面を消去し、HOME画面に遷移する
    // USER ボタン押下で本画面を消去し、USER画面に遷移する
    // ESCAPE ボタン押下で本画面を消去し、HOME画面に戻る
    if (id == BUTTON_ENTER) {
        QObject *target = nullptr;
        if (_view) {
            target = _view->findChild<QObject *>(_selectButton);
        }
        if (target) {
            target->setProperty("press", false);
        }
        onKeyEnter();
    } else if (id == BUTTON_ESCAPE || id == BUTTON_HOME || id == BUTTON_USER) {
        if (_model->isUpdate(VALUE_THRESHOLD)) {
            emit requestToShowPopup(POPUP_CANCEL_CONFIRMATION); // 破棄確認
        } else {
            onAccept(POPUP_CANCEL_CONFIRMATION, true);
        }
    } else {
        moveCursor(id);
    }
}

void ThresholdSettingViewModel::onAccept(PopupId popupId, bool okOnly)
{
    Q_UNUSED(okOnly);

    AbstractViewModel::onAccept(popupId, okOnly);

    if (popupId == POPUP_SAVE_CONFIRMATION) {
        // 設定値保存
        _model->commit(VALUE_THRESHOLD);
        requestInHistory(SCREEN_MEASUREMENT_SETTING_MENU);
    } else if (popupId == POPUP_CANCEL_CONFIRMATION) {
        // 設定値破棄
        _model->rollback(VALUE_THRESHOLD);

        // 押下されたボタンにより遷移先を変更する
        switch (_lastPressedButtonId) {
        case BUTTON_HOME:
            emit requestDirect(SCREEN_HOME, true);
            break;

        case BUTTON_ESCAPE:
            emit requestInHistory(SCREEN_THRESHOLD);
            break;

        case BUTTON_USER:
#ifdef NEW_USER_SETTING
        emit requestDirect(SCREEN_USER_SETTING, true);
#else
        emit requestDirect(SCREEN_USER_DATA_LOGGER_MENU, true);
#endif
            break;

        default:
            break;
        }
    } else if (popupId == POPUP_WRONG_INPUT_DATA) {
        // nothing to do.
    }
}

void ThresholdSettingViewModel::onReject(PopupId popupId)
{
    Q_UNUSED(popupId);
}

void ThresholdSettingViewModel::onKeyEnter()
{
    QObject *target = nullptr;
    if (_view) {
        target = _view->findChild<QObject *>(_selectButton);
    }
    if (target) {
        if (_selectButton == "save") {
            // 保存ボタン押下時
            // 設定値保存
            if (_model->isUpdate(VALUE_THRESHOLD)) {
                if (checkValue()) {
                    // 範囲チェックOK
                    emit requestToShowPopup(POPUP_SAVE_CONFIRMATION);
                } else {
                    // 範囲チェックNG
                    emit requestToShowPopup(POPUP_WRONG_INPUT_DATA);
                }
            } else {
                onAccept(POPUP_SAVE_CONFIRMATION, true);
            }
        } else if (_selectButton == "initialize") {
            // 初期化ボタン押下時
            // 設定値表示を初期値に値を設定する
            QVariantList thresholdDataList = thresholdData(true);
            temporariliySaveThresholdData(thresholdDataList);
            setThresholdData();
        } else {
            // 保存ボタン, 初期化ボタン以外のボタン押下時
            QVariant isEditMode = target->property("edit");

            if (isEditMode.toBool() == false) {
                // 編集状態へ移行
                _screenState = EDIT;
                target->setProperty("edit", true);

                QVariant isExp = target->property("exp");

                if (isExp.toBool() == true) {
                    _editButtonOperation.setButtonName(EDIT_BUTTON_COLUMN_NUMBER, EDIT_BUTTON_ROW_NUMBER, thresholdEditExpButtonData);
                    _activeEditButton = "edit1";
                } else {
                    QVariant isFirstFiexd = target->property("isFirstFixed");

                    if (isFirstFiexd.toBool() == true) {
                        _editButtonOperation.setButtonName(EDIT_BUTTON_COLUMN_NUMBER, EDIT_BUTTON_ROW_NUMBER,
                                                           thresholdEditFirstFixedButtonData);
                        _activeEditButton = "edit2";
                    } else {
                        _editButtonOperation.setButtonName(EDIT_BUTTON_COLUMN_NUMBER, EDIT_BUTTON_ROW_NUMBER, thresholdEditButtonData);
                        _activeEditButton = "edit1";
                    }
                }

                _editButtonOperation.setActiveObject(_activeEditButton);


            } else {
                // 選択状態へ移行
                _screenState = SELECT;
                target->setProperty("edit", false);

                // 入力された数値の取得と model 設定値更新
                QVariant targetValue;
                targetValue = target->property("value");


                // model 設定値取得
                QVariantList thresholdDataList = thresholdData();

                if (_selectButton == "select1") {
                    int settingValue = targetValue.toInt();
                    // リストデータの更新を実施
                    thresholdDataList.replace(1, settingValue);
                } else if (_selectButton == "select2") {
                    int settingValue = targetValue.toInt();
                    // リストデータの更新を実施
                    thresholdDataList.replace(2, settingValue);
                } else if (_selectButton == "select3") {
                    int settingValue  = targetValue.toInt();
                    // リストデータの更新を実施
                    thresholdDataList.replace(3, settingValue);
                } else if (_selectButton == "select4") {
                    int settingValue  = targetValue.toInt();
                    // リストデータの更新を実施
                    thresholdDataList.replace(4, settingValue);
                } else if (_selectButton == "select5") {
                    // [Memo]
                    //  BER, MERの設定値の範囲は、
                    //  000 ~ 999であり必ず3桁の値となるため
                    //  3桁のうちの1,2桁は仮数部, 3桁は虚数部とすることができる。
                    QVariant strMantissa = targetValue.toString().left(2);
                    QVariant strExponent = targetValue.toString().right(1);

                    // 仮数部
                    // 表示している値の10倍の値が保持している値となるため10倍する。
                    int mantissa = strMantissa.toInt() * 10;

                    // 指数部
                    // 表示している値の -1倍し、-2を計算した値が保持している値となるため計算をする
                    int exponent = (strExponent.toInt() * (-1)) + (-2);

                    // リストデータの更新を実施
                    thresholdDataList.replace(5, mantissa);
                    thresholdDataList.replace(6, exponent);
                } else if (_selectButton == "select6") {
                    QVariant strMantissa = targetValue.toString().left(2);
                    QVariant strExponent = targetValue.toString().right(1);
                    // [Memo]
                    //  BER, MERの設定値の範囲は、
                    //  000 ~ 999であり必ず3桁の値となるため
                    //  3桁のうちの1,2桁は仮数部, 3桁は虚数部とすることができる。

                    // 仮数部
                    int mantissa = strMantissa.toInt() * 10;

                    // 指数部
                    int exponent = (strExponent.toInt() * (-1)) + (-2);

                    thresholdDataList.replace(7, mantissa);
                    thresholdDataList.replace(8, exponent);
                }

                // 一時保存処理を実施する
                temporariliySaveThresholdData(thresholdDataList);
            }
        }
    }

}

// 閾値データの一時保存処理
void ThresholdSettingViewModel::temporariliySaveThresholdData(QVariantList thresholdDataList)
{
    // 内部データを取得
    QVariant value;
    _model->getValue(VALUE_THRESHOLD, value);
    QVariantList settingData = value.toList();

    int thresholdIndex = Correcter::getThresholdIndex(_selectedBand, Correcter::broadcastSystemValue(_selectedBroadcasting), Correcter::modulationValue(_selectedModulation));

    //  内部データか選択中のBANDに対応したデータのreplaceを実行し、
    //  setValueを実施することで一時保存を実施
    settingData.replace(thresholdIndex, thresholdDataList);
    // LevelCheckerModelを用いて値の更新を実施する。
    _model->setValue(VALUE_THRESHOLD, settingData);
}

void ThresholdSettingViewModel::setBandMessage()
{
    QObject *bandName = nullptr;
    if (_view) {
        bandName = _view->findChild<QObject *>("bandName");
    }
    if (bandName) {
        QString bandLabel("");

        // [Memo]
        //  放送方式または、変調方式を設定している場合は、
        //  画面上部に　[BAND名] + " " (半角スペース) + [放送方式]or[変調方式]と表示
        //  放送方式、変調方式のどちらも設定していない場合は、
        //  画面上部に「BAND名」のみを表示する。

        if (_selectedBand == static_cast<int>(LC70_BAND_UHF)) {
            bandLabel = "地上\uff83\uff9e\uff7c\uff9e\uff80\uff99";
        } else if (_selectedBand == static_cast<int>(LC70_BAND_PILOT)) {
            bandLabel = "PILOT";
        } else if (_selectedBand == static_cast<int>(LC70_BAND_JCSAT3)) {
            bandLabel = "JCSAT-3A " + _selectedBroadcasting;
        } else if (_selectedBand == static_cast<int>(LC70_BAND_BS)) {
            bandLabel = "BS " + _selectedBroadcasting;
        } else if (_selectedBand == static_cast<int>(LC70_BAND_JCSAT4)) {
            bandLabel = "JCSAT-4B " + _selectedBroadcasting;
        } else if (_selectedBand == static_cast<int>(LC70_BAND_SCCB)) {
            bandLabel = "SCC-B " + _selectedBroadcasting;
        } else if (_selectedBand == static_cast<int>(LC70_BAND_SCCC)) {
            bandLabel = "SCC-C " + _selectedBroadcasting;
        } else if (_selectedBand == static_cast<int>(LC70_BAND_CS)) {
            bandLabel = "CS110° " + _selectedBroadcasting;
        } else if (_selectedBand == static_cast<int>(LC70_BAND_CATV_ALL)) { // ラッパー関数ライブラリ V2.00 対応
            bandLabel = "CATV " + _selectedModulation;
        } else if (_selectedBand == static_cast<int>(LC70_BAND_SKYP)) {
            bandLabel = "スカパー光 " + _selectedModulation;
        } else if (_selectedBand == static_cast<int>(LC70_BAND_CATV_UP_STREAM)) {
            bandLabel = "CATV 上り";
        } else if (_selectedBand == static_cast<int>(LC70_BAND_FM)) {
            bandLabel = "FM";
        } else {
            // nothing to do
        }
        bandName->setProperty("text", bandLabel);
    }

}

void ThresholdSettingViewModel::setThresholdData()
{
    QVariantList thresholdDataList = thresholdData();

    // Level および MER は値を 1/10 して設定する
    QString targetName;
    QObject *target = nullptr;

    // 閾値の設定を実施
    for (int i = 1; i <= 4; i++) {
        targetName = "select" + QString::number(i);
        if (_view) {
            target = _view->findChild<QObject *>(targetName);
        }
        if (target) {
            target->setProperty("value", thresholdDataList[i].toString());
        }
    }

    // BERとMERは仮数部と指数部を統合しView側へ送信する。
    // 例） 2.0E-4を表示しようとした場合は、204
    //     4.0E-6を表示しようとした場合は、406 をView側へ送信する。
    QString mantissa;     // 仮数部
    QString exponent;     // 虚数部
    QString settingValue; // 設定値

    // BER
    targetName = "select5";
    target = nullptr;
    if (_view) {
        target = _view->findChild<QObject *>(targetName);
    }
    mantissa = QString::number(thresholdDataList[5].toInt() /
                               10); // 仮数部は 10倍の値で保持しているため (1/10) した値
    exponent = QString::number((thresholdDataList[6].toInt() + 2) * (-1));  // 指数部は (-2) し (-1)を掛けた値
    settingValue = mantissa + exponent;
    if (target) {
        target->setProperty("value", settingValue);
    }
    // MER
    targetName = "select6";
    target = nullptr;
    if (_view) {
        target = _view->findChild<QObject *>(targetName);
    }
    mantissa = QString::number(thresholdDataList[7].toInt() /
                               10); // 仮数部は 10倍の値で保持しているため (1/10) した値
    exponent = QString::number((thresholdDataList[8].toInt() + 2) * (-1)); // 指数部は (-2) し (-1)を掛けた値
    settingValue = mantissa + exponent;
    if (target) {
        target->setProperty("value", settingValue);
    }
}

bool ThresholdSettingViewModel::checkValue()
{
    QVariantList thresholdDataList = thresholdData();


    // LEVEL
    if (LEVELMIN > thresholdDataList[1].toInt()) {
        // 最小値チェック
        // 最小値 > 閾値(左)の場合、エラーとする。
        return false;
    }

    if (LEVELMAX < thresholdDataList[2].toInt()) {
        // 最大値チェック
        // 最大値 < 閾値(右)の場合、エラーとする。
        return false;
    }

    if (thresholdDataList[1].toInt() > thresholdDataList[2].toInt()) {
        // 項目間チェック
        // 閾値(左) > 閾値(右)の場合、エラーとする。
        return false;
    }

    // [Memo]
    //  選択BANDがPILOTの場合は、LEVELのみの変更が可能であるため
    //  MER/CN, BERの入力値チェックは実施しない。
    if (_selectedBand != static_cast<int>(LC70_BAND_PILOT) && _selectedBand != static_cast<int>(LC70_BAND_FM) && _selectedBand != static_cast<int>(LC70_BAND_CATV_UP_STREAM)) {
        // MER/CN

        if ((_merRange.first * MER_ADJUST) > thresholdDataList[3].toInt()) {
            // 最小値チェック
            // 最小値 > 閾値(左)の場合エラーとする。
            return false;
        }

        if ((_merRange.second * MER_ADJUST) < thresholdDataList[4].toInt()) {
            // 最大値チェック
            // 最大値 < 閾値(右)の場合エラーとする。
            return false;
        }

        if (thresholdDataList[3].toInt() > thresholdDataList[4].toInt()) {
            // 項目間チェック
            // 閾値(左) > 閾値(右)の場合エラーとする。
            return false;
        }

        // BER
        double valueLeft = 0.0;
        double valueRight = 0.0;

        float lowmaintissa = thresholdDataList[5].toInt() / (float)100.0;
        int lowexponent = thresholdDataList[6].toInt() + 2;
        QString lowdoubleValue = QString::number(lowmaintissa) + "e" + QString::number(lowexponent);
        valueLeft = lowdoubleValue.toDouble();

        float highmaintissa = thresholdDataList[7].toInt() / (float)100.0;
        float highexponent = thresholdDataList[8].toInt() + 2;
        QString highdoubleValue = QString::number(highmaintissa) + "e" + QString::number(highexponent);
        valueRight = highdoubleValue.toDouble();

        if (BERMIN < valueLeft) {
            // 最大値チェック
            // BERの最大値 < BER(左)の場合エラーとする。
            return false;
        }

        if (valueRight < BERMAX) {
            // 最小値チェック
            // BERの最大値 < BER(右)の場合エラーとする。
            return false;
        }

        if (valueLeft < valueRight) {
            // 項目間チェック
            // BER(左) < BER(右)の場合エラーとする。
            return false;
        }
    }

    return true;

}

QVariantList ThresholdSettingViewModel::thresholdData(bool useDefaultValue)
{
    QVariant value;

    if (useDefaultValue) {
        _model->getThresholdInitialValue(value);
    } else {
        _model->getValue(VALUE_THRESHOLD, value);
    }

    // 閾値データのindex(LC70_MEAS_SET_THRD_IDX_xxxxの値)取得
    int thresholdIndex = Correcter::getThresholdIndex(_selectedBand, Correcter::broadcastSystemValue(_selectedBroadcasting), Correcter::modulationValue(_selectedModulation));

    if (thresholdIndex < 0) {
        _model->writeLog("thresholdSettingViewModel", "there is no threshold data", LOG_INFO);
    }
    return value.toList().at(thresholdIndex).toList();
}
