/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include "thresholdviewmodel.h"

QVector<QVector<QString>> thresholdButtonData = {
    { {"m1"},  {"m2"},  {"m3"},  {"m4"}, {"m5"} },
    { {""},    {"m7"},  {"m8"},  {"m9"}, {""} },
    { {"m6"}, {""}, {""}, {""}, {""} }
};

const int ThresholdViewModel::BUTTON_COLUMN_NUMBER = 3;
const int ThresholdViewModel::BUTTON_ROW_NUMBER    = 5;


ThresholdViewModel::ThresholdViewModel(ScreenId screenId, LevelCheckerModel *&model, QObject *parent)
    : AbstractViewModel(screenId, model, parent)
    , _selectedMeasurement("")
{
    setObjectName("ThresholdViewModel");
    _buttonOperation.setButtonName(BUTTON_COLUMN_NUMBER, BUTTON_ROW_NUMBER, thresholdButtonData);

    _activeButton = thresholdButtonData[0][0];
    _buttonOperation.setActiveObject(_activeButton);
}

ThresholdViewModel::~ThresholdViewModel()
{

}

void ThresholdViewModel::activate()
{
    connect(this, SIGNAL(requestSetTakeOver(QVariant, QVariant)), _view, SLOT(backup(QVariant, QVariant)));
    connect(this, SIGNAL(requestGetTakeOver()), _view, SLOT(getTakeOver()));
    connect(this, SIGNAL(removeAttribute(QString)), _mainFrame, SLOT(onRemoveAttribute(QString)));
    emit requestGetTakeOver();

    QObject *target = nullptr;
    if (_view) {
        target = _view->findChild<QObject *>("takeOverData");
    }
    if (target) {
        QVariant get = target->property("selectedMeasurement");
        _selectedMeasurement = get.toString();
    }

    // BAND(測定設定)画面遷移時は、必ずヘッダーBANDアイコンをクリアする
    _model->replaceMenuList(MenuListIconIndex::BAND_ICON, "");

    // BAND選択画面以降に設定する画面共有情報(選択BAND、変調方式、放送方式)情報をクリアする
    emit removeAttribute("selectedBand");
    emit removeAttribute("selectedModulation");
    emit removeAttribute("selectedBroadcast");

    // 閾値のBAND選択では表示しない (対象: アンテナ調整)
    target = nullptr;
    if (_view) {
        target = _view->findChild<QObject *>("m12");
    }
    if(target) {
            target->setProperty("disable",true);
    }

    target = nullptr;
    if (_view) {
        target = _view->findChild<QObject *>("m13");
    }
    if (target) {
        target->setProperty("disable", true);
    }



    // 閾値のBAND選択では表示しない (対象: LTE)
    target = nullptr;
    if (_view) {
        target = _view->findChild<QObject *>("m14");
    }
    if (target) {
        target->setProperty("disable", true);
    }

    // 先頭のボタンをアクティブにする
    resetToInitialButtonState(thresholdButtonData[0][0], _activeButton, _buttonOperation);

    setScreenName();
}

void ThresholdViewModel::deactivate()
{
    AbstractViewModel::deactivate();

    this->disconnect();
}

void ThresholdViewModel::setTakeOver()
{
    if (_activeButton == "m2") {
        emit requestSetTakeOver("selectedBand", LC70_BAND_BS);
    } else if (_activeButton == "m7") {
        emit requestSetTakeOver("selectedBand", LC70_BAND_CS);
    } else if (_activeButton == "m4") {
        emit requestSetTakeOver("selectedBand", LC70_BAND_SCCB);
    } else if (_activeButton == "m9") {
        emit requestSetTakeOver("selectedBand", LC70_BAND_SCCC);
    } else if (_activeButton == "m3") {
        emit requestSetTakeOver("selectedBand", LC70_BAND_JCSAT3);
    } else if (_activeButton == "m8") {
        emit requestSetTakeOver("selectedBand", LC70_BAND_JCSAT4);
    } else if (_activeButton == "m1") {
        emit requestSetTakeOver("selectedBand", LC70_BAND_UHF);
    } else if (_activeButton == "m5") {
        emit requestSetTakeOver("selectedBand", LC70_BAND_FM);
    } else {}
}

void ThresholdViewModel::moveCursor(ButtonId buttonId)
{
    if ((buttonId == BUTTON_UP) || (buttonId == BUTTON_DOWN) || (buttonId == BUTTON_RIGHT) || (buttonId == BUTTON_LEFT)) {

        QString nextButton;
        bool ret = _buttonOperation.nextActiveObject(buttonId, nextButton);

        if (ret) {

            // inactive
            QObject *target = nullptr;
            if (_view) {
                target = _view->findChild<QObject *>(_activeButton);
            }
            if (target) {
                target->setProperty("active", false);
            }

            // active
            target = nullptr;
            if (_view) {
                target = _view->findChild<QObject *>(nextButton);
            }
            if (target) {
                target->setProperty("active", true);
            }

            _activeButton = nextButton;
            _buttonOperation.setActiveObject(_activeButton);
        }
    }
}

void ThresholdViewModel::onKeyStatus(ButtonId id)
{
    // Home, Escape ボタンはベースクラスで対応
    AbstractViewModel::onKeyStatus(id);

    if (id == BUTTON_ENTER) {
        enterKey();
    } else if (id == BUTTON_HOME) {
        emit requestDirect(SCREEN_HOME, true);
    } else if (id == BUTTON_ESCAPE) {
        _model->removeMenuList(MenuListIconIndex::BAND_ICON);
        emit back();
    } else if (id == BUTTON_USER) {
#ifdef NEW_USER_SETTING
        emit requestDirect(SCREEN_USER_SETTING, true);
#else
        emit requestDirect(SCREEN_USER_DATA_LOGGER_MENU, true);
#endif
    } else {
        moveCursor(id);
    }
}

void ThresholdViewModel::onButtonPress(ButtonId id)
{
    if (id == BUTTON_ENTER) {
        QObject *target = nullptr;
        if (_view) {
            target = _view->findChild<QObject *>(_activeButton);
        }
        target->setProperty("press", true);
    }
}

void ThresholdViewModel::enterKey()
{
    int buttonIndex = _activeButton.mid(1).toInt();

    // 選択したBANDアイコンを追加する
    addSelectBandIcon(buttonIndex);

    switch (buttonIndex) {
    case 2:     // BS
    case 7:    // N-SAT110
        onRequest(1);     // 放送方式選択画面 : ISBDS、ISDBS3
        break;

    case 3:     // JCSAT3
    case 4:     // SCC-B
    case 8:     // JCSAT4
    case 9:     // SCC-C
        onRequest(1);     // 放送方式選択画面 : DVBS、DVBS2
        break;

    case 1:               // 地デジ
    case 5:               // FM
        onRequest(0);
        break;

    case 6:     // CATV
        onRequest(2);     // CATV BAND選択画面 : CATV
        break;

    default:
        break;
    }
}

void ThresholdViewModel::onRequest(int condition)
{
    emit request(_screenId, condition);
    return;
}

void ThresholdViewModel::setScreenName()
{
    QObject *target = nullptr;
    if (_view) {
        target = _view->findChild<QObject *>("screenName");
    }
    if (target) {
        target->setProperty("name", "インジケータ");
    }
}

void ThresholdViewModel::addSelectBandIcon(int selectButtonIndex)
{
    // メニューリストのサイズが2より上であれば、BANDアイコンの置き換えを実施する
    switch (selectButtonIndex) {
    case 1:  // 地デジ
        _model->replaceMenuList(MenuListIconIndex::BAND_ICON, "icon_band_chidigi.png");
        break;

    case 3:  // JCSAT3
        _model->replaceMenuList(MenuListIconIndex::BAND_ICON, "icon_band_3a.png");
        break;

    case 4:  // SCC-B
        _model->replaceMenuList(MenuListIconIndex::BAND_ICON, "icon_band_b.png");
        break;

    case 5:  // FM
        _model->replaceMenuList(MenuListIconIndex::BAND_ICON, "icon_band_fm.png");
        break;

    case 2:  // BS
        _model->replaceMenuList(MenuListIconIndex::BAND_ICON, "icon_band_bs.png");
        break;

    case 8:  // JCSAT4
        _model->replaceMenuList(MenuListIconIndex::BAND_ICON, "icon_band_4b.png");
        break;

    case 9: // SCC-C
        _model->replaceMenuList(MenuListIconIndex::BAND_ICON, "icon_band_c.png");
        break;

    case 7: // CS
        _model->replaceMenuList(MenuListIconIndex::BAND_ICON, "icon_band_cs.png");
        break;

    default:
        break;
    }
}
