/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include "compassviewmodel.h"
#include "mainframeviewmodel.h"

static constexpr uint16_t INVALID_AZUMUTH = 0xFFFF;         //!< 無効な方位角
static constexpr uint16_t STOP_AZUMUTH = 0xFFFE;            //!< 方位測定停止中

CompassViewModel::CompassViewModel(ScreenId screenId, LevelCheckerModel *&model,
        QObject *parent)
    : AbstractViewModel(screenId, model, parent)
    , _isEnabledCompass(true)
{

}

CompassViewModel::~CompassViewModel()
{

}

void CompassViewModel::resume()
{
    AbstractViewModel::resume();

    _model->startAzimuth();
}

void CompassViewModel::suspend()
{
    AbstractViewModel::suspend();

    _model->stopAzimuth();
}

void CompassViewModel::createConnection(QObject *qml, QObject *mainFrame)
{
    AbstractViewModel::createConnection(qml, mainFrame);

    connect(mainFrame, SIGNAL(notifyChangeCompassEnable(bool)), this, SLOT(onNotifyChangeCompassEnabled(bool)));
    connect(mainFrame, SIGNAL(notifyAzimuth(uint16_t)), this, SLOT(onNotifyAzimuth(uint16_t)));
}

void CompassViewModel::activate()
{
    AbstractViewModel::activate();

    if (nullptr == _view) {
        qErrnoWarning("_view is null");
        return;
    }

    // サブ CPU をスタンバイにする
#if 0
    if (_model->wakeUpRequested()) {
        _model->setWakeUpRequested(false);
        _model->requestStandBy(1);
    }
 #else
    _model->setWakeUpRequested(false);
    _model->requestStandBy(1);
 #endif


    _model->changeDeviceStatusInterval(true);

    _isEnabledCompass = true;
    if (_view) {
        _view->setProperty("isEnabledCompass", _isEnabledCompass);
    }
    MainFrameViewModel *mainFrame = static_cast<MainFrameViewModel *>(_mainFrame);
    onNotifyChangeCompassEnabled(mainFrame->isEnabledCompass());
}

void CompassViewModel::deactivate()
{
    AbstractViewModel::deactivate();

    _model->changeDeviceStatusInterval(false);

    disconnect(_mainFrame, SIGNAL(notifyChangeCompassEnable(bool)), this, SLOT(onNotifyChangeCompassEnabled(bool)));
    disconnect(_mainFrame, SIGNAL(notifyAzimuth(uint16_t)), this, SLOT(onNotifyAzimuth(uint16_t)));
}

void CompassViewModel::onKeyStatus(ButtonId id)
{
    if (!_isEnabledCompass) {
        if (id == BUTTON_ESCAPE) {
            // エラー発生時はESCキーのみ有効で、HOME画面に戻る
            emit requestDirect(SCREEN_HOME, true);
        }
        return;
    }
    AbstractViewModel::onKeyStatus(id);

    if (id == BUTTON_USER) {
#ifdef NEW_USER_SETTING
        emit requestDirect(SCREEN_USER_SETTING, true);
#else
        emit requestDirect(SCREEN_USER_DATA_LOGGER_MENU, true);
#endif
    }
}

void CompassViewModel::onNotifyChangeCompassEnabled(bool enabled)
{
    if (_isEnabledCompass == enabled) {
        return;
    }
    _isEnabledCompass = enabled;

    // コンパスのエラー有無を反映
    if (_view) {
        _view->setProperty("isErrorOccurred", !_isEnabledCompass);
    }
}

void CompassViewModel::onNotifyAzimuth(uint16_t azimuth)
{
    if (_view != nullptr) {
        // 無効値、測定停止中の場合は、コンパスに測定結果無効を通知
        if (azimuth == INVALID_AZUMUTH || azimuth == STOP_AZUMUTH) {
            if (_view) {
                _view->setProperty("isEnabledCompass", false);
            }
            return;
        }
        if (_view) {
            _view->setProperty("isEnabledCompass", true);
        }
        if (_view) {
            _view->setProperty("azimuth", QVariant::fromValue(azimuth / 10));
        }
    }
}
