/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include "dcmeasurementviewmodel.h"
#include "correcter.h"

DCMeasurementViewModel::DCMeasurementViewModel(ScreenId screenId, LevelCheckerModel *&model, QObject *parent)
    : AbstractViewModel(screenId, model, parent)
    , _voltage_msg(nullptr)
    , _timer(new QTimer())
{

}

DCMeasurementViewModel::~DCMeasurementViewModel()
{

}

void DCMeasurementViewModel::resume()
{
    AbstractViewModel::resume();

    // DC測定を開始
    if (!_model->startDCMeasurement()) {
        qDebug() << "DC Measurement start Error";
        return;
    }

    // タイマーとのコネクションを生成する
    connect(_timer, SIGNAL(timeout()), this, SLOT(onNotifyTimeOut()));

    timerStart();
}

void DCMeasurementViewModel::activate()
{
    if (nullptr == _view) {
        qErrnoWarning("_view is null");
        return;
    }

//#if 1
//    _model->requestStandBy(1);
//    _model->wakeUp(1);
//    _model->readyToMeasure();
//#endif

    // DC/光測定はスタンバイしてからウェイクアップしないと動作しない
    if (!_model->wakeUpRequested()) {
        _model->requestStandBy(1);
        _model->setWakeUpRequested(true);
        _model->wakeUp(1);
    }
    _model->readyToMeasure();

    // ヘッダメニュー表示
    _model->replaceMenuList(MenuListIconIndex::FUNCTION_ICON, "icon_menu_dc.png");

    // 電圧値のメッセージ部品を取得する.
    // シーケンスによると qml が生成されているのが確実なのは activate() だけ
    _voltage_msg = nullptr;
    if (_view) {
        _voltage_msg = _view->findChild<QObject *>("value");
    }
    if (nullptr == _voltage_msg) {
        qErrnoWarning("_voltage_msg is null");
        return;
    }
}

void DCMeasurementViewModel::deactivate()
{
    AbstractViewModel::deactivate();
    disconnect(_timer, SIGNAL(timeout()), this, SLOT(onNotifyTimeOut()));
}

void DCMeasurementViewModel::suspend()
{
    // タイマーを止める
    if (_timer->isActive()) {
        _timer->stop();
    }

    // DC測定を終了
    if (!_model->stopDCMeasurement()) {
        qDebug() << "DC Measurement stop Error";
    }

    // サブ CPU をスタンバイにする
    _model->setWakeUpRequested(false);
    _model->requestStandBy(1);

    AbstractViewModel::suspend();

    qDebug() << QString("[Leave] DCMeasurementViewmodel::suspend()");
}

void DCMeasurementViewModel::onKeyStatus(ButtonId id)
{
    switch (id) {
    case BUTTON_ESCAPE:
        emit back();
        break;

    case BUTTON_HOME:
        emit requestDirect(SCREEN_HOME, true);
        break;

    case BUTTON_USER:
#ifdef NEW_USER_SETTING
        emit requestDirect(SCREEN_USER_SETTING, true);
#else
        emit requestDirect(SCREEN_USER_DATA_LOGGER_MENU, true);
#endif
        break;

    default:
        break;
    }
}

void DCMeasurementViewModel::recover()
{
    if (_view) {
        _voltage_msg = _view->findChild<QObject *>("value");
    }
}

void DCMeasurementViewModel::timerStart()
{
    // DC測定を行う毎にTimeOutするタイマーを起動する
    _timer->setInterval(_model->applicationSetting()->pollingIntervalDCMeasurement());
    _timer->start();
}

void DCMeasurementViewModel::onNotifyTimeOut()
{
    // DC測定の結果を取得する
    QVariant measurementDCValue;

    if (!_model->getDCMeasurementResult(measurementDCValue)) {
        // エラー発生時
        qDebug() << "DC Measurement result get Error";
    } else {
        // 取得成功時
        if (_voltage_msg != nullptr) {
            // qmlからオブジェクトの取得ができていれば以降の処理を実施する
            _voltage_msg->setProperty("text", Correcter::displayVoltage(measurementDCValue.toInt()));
        }
    }

    // 再度タイマーを動作させる
    _timer->start();
}
