/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include "autopoweroffsettingviewmodel.h"

QVector<QVector<QString>> powerOFFMenuButtonData = {
    { {"timer1"}, {"timer2"},  {"timer3"} },
    { {"timer4"}, {"timer5"},  {"timer6"} },
};

const int AutoPowerOffSettingViewModel::BUTTON_COLUMN_NUMBER = 2;
const int AutoPowerOffSettingViewModel::BUTTON_ROW_NUMBER = 3;

AutoPowerOffSettingViewModel::AutoPowerOffSettingViewModel(ScreenId screenId, LevelCheckerModel *&model,
        QObject *parent)
    : AbstractViewModel(screenId, model, parent)
{
    _buttonOperation.setButtonName(BUTTON_COLUMN_NUMBER, BUTTON_ROW_NUMBER, powerOFFMenuButtonData);
}

AutoPowerOffSettingViewModel::~AutoPowerOffSettingViewModel()
{

}

void AutoPowerOffSettingViewModel::activate()
{
    if (nullptr == _view) {
        qErrnoWarning("_view is null");
        return;
    }

    // ボタンActive設定
    QVariant value;
    int buttonIndex = 0;

    if (_model->getValue(VALUE_AUTOPOWEROFF, value)) {
        qDebug() << "AUTOPOWEROFF" << value.toInt();

        if (value.toInt() == LC70_SYS_SET_POWOFF_OFF) {
            // オートパワーオフの設定値がOFFの場合
            buttonIndex = 1;
        } else if (value.toInt() == LC70_SYS_SET_POWOFF_5MIN) {
            // オートパワーオフの設定値が5MINの場合
            buttonIndex = 2;
        } else if (value.toInt() == LC70_SYS_SET_POWOFF_10MIN) {
            // オートパワーオフの設定値が10MINの場合
            buttonIndex = 3;
        } else if (value.toInt() == LC70_SYS_SET_POWOFF_15MIN) {
            // オートパワーオフの設定値が15MINの場合
            buttonIndex = 4;
        } else if (value.toInt() == LC70_SYS_SET_POWOFF_30MIN) {
            // オートパワーオフの設定値が30MINの場合
            buttonIndex = 5;
        } else if (value.toInt() == LC70_SYS_SET_POWOFF_60MIN) {
            // オートパワーオフの設定値が60MINの場合
            buttonIndex = 6;
        } else {
            // 上記以外の設定が入っていた場合
            // 初期値であるオートパワーオフの設定値が15を設定しているように設定する。
            buttonIndex = 4;
        }
    }

    // Active にするボタンを設定
    QString itemname("timer");
    itemname += QString().sprintf("%d", buttonIndex);

    QObject *item = nullptr;
    if (_view) {
        item = _view->findChild<QObject *>(itemname);
    }
    if (item) {
        item->setProperty("active", true);
    }
    _activeButton = itemname;
    _buttonOperation.setActiveObject(_activeButton);

}

void AutoPowerOffSettingViewModel::onRequest(int condition)
{
    qDebug("SystemSettingMenuViewModel::onRequest");

    emit request(_screenId, condition);
}

void AutoPowerOffSettingViewModel::moveCursor(ButtonId buttonId)
{
    if ((buttonId == BUTTON_RIGHT) || (buttonId == BUTTON_LEFT) || (buttonId == BUTTON_UP) || (buttonId == BUTTON_DOWN)) {
        // UP, DOWN なし

        QString nextBtn;
        bool ret = _buttonOperation.nextActiveObject(buttonId, nextBtn);

        if (ret) {

            // inactive
            QObject *target = nullptr;
            if (_view) {
                target = _view->findChild<QObject *>(_activeButton);
            }
            if (target) {
                target->setProperty("active", false);
            }

            // active
            target = nullptr;
            if (_view) {
                target = _view->findChild<QObject *>(nextBtn);
            }
            if (target) {
                target->setProperty("active", true);
            }

            _activeButton = nextBtn;
            _buttonOperation.setActiveObject(_activeButton);

            qDebug("auto poweroff set");
            QVariant value;

            if (_activeButton == "timer1") {
                // OFF 選択時
                value = LC70_SYS_SET_POWOFF_OFF;
            } else if (_activeButton == "timer2") {
                // 5 min選択時
                value = LC70_SYS_SET_POWOFF_5MIN;
            } else if (_activeButton == "timer3") {
                // 10 min選択時
                value = LC70_SYS_SET_POWOFF_10MIN;
            } else if (_activeButton == "timer4") {
                // 15 min選択時
                value = LC70_SYS_SET_POWOFF_15MIN;
            } else if (_activeButton == "timer5") {
                // 30 min選択時
                value = LC70_SYS_SET_POWOFF_30MIN;
            } else if (_activeButton == "timer6") {
                // 60 min選択時
                value = LC70_SYS_SET_POWOFF_60MIN;
            } else {
                // 何もしない
            }

            // バッファーに値を保持する
            _model->setValue(VALUE_AUTOPOWEROFF, value);
        }
    }
}

void AutoPowerOffSettingViewModel::onKeyStatus(ButtonId id)
{
    // 電源 ボタン押下は、HOME画面と同様の処理を行う
    // HOME ボタン押下で本画面を消去し、HOME画面に遷移する
    // USER ボタン押下で本画面を消去し、USER画面に遷移する
    // ESCAPE ボタン押下で本画面を消去し、HOME画面に戻る

    if (id == BUTTON_ENTER) {
        QObject *target = nullptr;
        if (_view) {
            target = _view->findChild<QObject *>(_activeButton);
        }
        if (target) {
            target->setProperty("press", false);
        }
        if (_model->isUpdate(VALUE_AUTOPOWEROFF)) {
            // 保存確認
            emit requestToShowPopup(POPUP_SAVE_CONFIRMATION);
        } else {
            // バッファーの値と現在保存されている値が同じ場合
            // ダイアログは表示せずメニューに戻る
            emit back();
        }
    } else if (id == BUTTON_ESCAPE || id == BUTTON_HOME || id == BUTTON_USER) {
        if (_model->isUpdate(VALUE_AUTOPOWEROFF)) {
            // 破棄確認
            emit requestToShowPopup(POPUP_CANCEL_CONFIRMATION);
        } else {
            // 修正がない場合は確認なしでメニューに戻る。
            onAccept(POPUP_CANCEL_CONFIRMATION, true);
        }
    } else {
        moveCursor(id);
    }
}

void AutoPowerOffSettingViewModel::onButtonPress(ButtonId id)
{
    if (id == BUTTON_ENTER) {
        QObject *target = nullptr;
        if (_view) {
            target = _view->findChild<QObject *>(_activeButton);
        }
        if (target) {
            target->setProperty("press", true);
        }
    }

    // スタンバイ、ウェイクアップでオートパワーオフ設定のフォーカス位置を元の設定値に戻す
    if (id == BUTTON_POWER) {
        _model->rollback(VALUE_AUTOPOWEROFF);
    }

}

void AutoPowerOffSettingViewModel::onAccept(PopupId popupId, bool okOnly)
{
    Q_UNUSED(okOnly)

    AbstractViewModel::onAccept(popupId, okOnly);

    // オートパワーオフ時間を設定
    if (popupId == POPUP_SAVE_CONFIRMATION) {
        _model->commit(VALUE_AUTOPOWEROFF);

        // メニューに戻る
        emit back();

    } else if (popupId == POPUP_CANCEL_CONFIRMATION) {
        _model->rollback(VALUE_AUTOPOWEROFF);

        // 押下されたボタンにより遷移先を変更する
        switch (_lastPressedButtonId) {
        case BUTTON_HOME:
            emit requestDirect(SCREEN_HOME, true);
            break;

        case BUTTON_USER:
#ifdef NEW_USER_SETTING
        emit requestDirect(SCREEN_USER_SETTING, true);
#else
        emit requestDirect(SCREEN_USER_DATA_LOGGER_MENU, true);
#endif
            break;

        case BUTTON_ESCAPE:
            emit back();
            break;

        default:
            break;
        }
    }
}

void AutoPowerOffSettingViewModel::onReject(PopupId popupId)
{
    Q_UNUSED(popupId)
    qDebug("nothing to do.");
}
