/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include "brightnesssettingviewmodel.h"


QVector<QVector<QString>> brightnessSettingMenuButtonData = {
    { {"brightness1"}, {"brightness2"},  {"brightness3"}},
    { {"register"}, {""}, {""}}
};

const int brightnessSettingViewModel::BUTTON_COLUMN_NUMBER = 2;
const int brightnessSettingViewModel::BUTTON_ROW_NUMBER = 3;
const int brightnessSettingViewModel::INITIAL_BRIGHTNESS_VALUE = 2;

brightnessSettingViewModel::brightnessSettingViewModel(ScreenId screenId, LevelCheckerModel *model, QObject *parent):
    AbstractViewModel(screenId, model, parent),
    _currentBrightness(2)
{
    _buttonOperation.setButtonName(BUTTON_COLUMN_NUMBER, BUTTON_ROW_NUMBER, brightnessSettingMenuButtonData);
}

brightnessSettingViewModel::~brightnessSettingViewModel()
{

}

void brightnessSettingViewModel::activate()
{
    if (nullptr == _view) {
        qErrnoWarning("_view is null");
        return;
    }

    // ボタンActive設定
    QVariant value;
    int buttonIndex = 0;

    if (_model->getValue(VALUE_BRIGHTNESS, value)) {
        int brightnessValue = value.toList().at(0).toInt();
        qDebug() << "BRIGHTNESS : " << brightnessValue;

        // [Memo]
        //  ラッパーから得られた値が範囲外の値であった場合は、
        //  初期値として設定する。
        if (brightnessValue < 1 || brightnessValue > 3) {
            brightnessValue = INITIAL_BRIGHTNESS_VALUE;
        }

        buttonIndex = brightnessValue - 1;
        _currentBrightness = brightnessValue;
    }

    // Active にするボタンを設定
    QString itemname = brightnessSettingMenuButtonData[0][buttonIndex];

    QObject *item = nullptr;
    if (_view) {
        item = _view->findChild<QObject *>(itemname);
    }
    if (item) {
        item->setProperty("active", true);
        item->setProperty("idlebg", "iconbtn.png");
    }
    _selectedBrightness = item;

    _activeButton = itemname;
    _buttonOperation.setActiveObject(_activeButton);
}

void brightnessSettingViewModel::moveCursor(ButtonId buttonId)
{

    QString nextBtn;

    if ((buttonId == BUTTON_RIGHT) || (buttonId == BUTTON_LEFT)) {
        if (_activeButton == brightnessSettingMenuButtonData[1][0]) {
            return;
        } else {
            _buttonOperation.nextActiveObject(buttonId, nextBtn);
        }
    } else if ((buttonId == BUTTON_UP) || (buttonId == BUTTON_DOWN)) {
        // 上下移動は左端のボタンに移動
        if (_activeButton == brightnessSettingMenuButtonData[1][0]) {
            nextBtn = brightnessSettingMenuButtonData[0][0];
        } else {
            nextBtn = brightnessSettingMenuButtonData[1][0];
        }
    } else {
        // 通らない
        return;
    }

    // inactive
    QObject *target = nullptr;
    if (_view) {
        target = _view->findChild<QObject *>(_activeButton);
    }
    if (target) {
        target->setProperty("active", false);
    }

    // active
    target = nullptr;
    if (_view) {
        target = _view->findChild<QObject *>(nextBtn);
    }
    if (target) {
        target->setProperty("active", true);
    }

    _activeButton = nextBtn;
    _buttonOperation.setActiveObject(_activeButton);
}

void brightnessSettingViewModel::onKeyStatus(ButtonId id)
{
    // 電源 ボタン押下は、HOME画面と同様の処理を行う
    // HOME ボタン押下で本画面を消去し、HOME画面に遷移する
    // USER ボタン押下で本画面を消去し、USER画面に遷移する
    // ESCAPE ボタン押下で本画面を消去し、HOME画面に戻る

    if (id == BUTTON_ENTER) {
        onKeyEnter();
    } else if (id == BUTTON_ESCAPE || id == BUTTON_HOME || id == BUTTON_USER) {
        if (_model->isUpdate(VALUE_BRIGHTNESS)) {
            emit requestToShowPopup(POPUP_CANCEL_CONFIRMATION); // 破棄確認
        } else {
            onAccept(POPUP_CANCEL_CONFIRMATION, true);
        }
    } else {
        moveCursor(id);
    }
}

void brightnessSettingViewModel::recover()
{
    QString itemname = _activeButton;

    QObject *item = nullptr;
    if (_view) {
        item = _view->findChild<QObject *>(itemname);
    }
    if (item) {
        item->setProperty("active", true);
        item->setProperty("idlebg", "iconbtn.png");
    }
    _selectedBrightness = item;
}

void brightnessSettingViewModel::onAccept(PopupId popupId, bool okOnly)
{
    Q_UNUSED(okOnly)

    AbstractViewModel::onAccept(popupId, okOnly);

    if (popupId == POPUP_SAVE_CONFIRMATION) {
        // 明るさを設定する。
        QVariant value;
        _model->commit(VALUE_BRIGHTNESS);

        // メニューに戻る
        emit back();

    } else if (popupId == POPUP_CANCEL_CONFIRMATION) {

        // 画面明るさ、バッファーの設定を画面遷移時に戻す
        _model->setScreenBrightness(_currentBrightness);
        _model->rollback(VALUE_BRIGHTNESS);

        // 押下されたボタンにより遷移先を変更する
        switch (_lastPressedButtonId) {
        case BUTTON_HOME:
            emit requestDirect(SCREEN_HOME, true);
            break;

        case BUTTON_ESCAPE:
            emit back();
            break;

        case BUTTON_USER:
#ifdef NEW_USER_SETTING
        emit requestDirect(SCREEN_USER_SETTING, true);
#else
        emit requestDirect(SCREEN_USER_DATA_LOGGER_MENU, true);
#endif
            break;

        default:
            break;
        }
    }
}

void brightnessSettingViewModel::onReject(PopupId popupId)
{
    Q_UNUSED(popupId)

    // Press 画像から元に戻す
    QObject *target = nullptr;
    if (_view) {
        target = _view->findChild<QObject *>(_activeButton);
    }
    if (target) {
        target->setProperty("press", false);
    }
}

void brightnessSettingViewModel::onKeyEnter()
{
    if (_activeButton == brightnessSettingMenuButtonData[1][0]) {
        // 保存確認
        emit requestToShowPopup(POPUP_SAVE_CONFIRMATION);
    } else {
        int brightness = 0;

        for (int i = 0; i < BUTTON_ROW_NUMBER; i++) {
            if (_activeButton == brightnessSettingMenuButtonData[0][i]) {
                brightness = i + 1;
                break;
            }
        }

        if (brightness != 0) {
            // お試し明るさ設定
            if (_model->setScreenBrightness(brightness) == false) {
                qErrnoWarning("brightness test fail.");
            }

            // バッファーに保存
            QVariantList valueList;
            valueList.append(brightness);

            if (!valueList.isEmpty()) {
                _model->setValue(VALUE_BRIGHTNESS, valueList);
            }
        }

        // Press 画像から元に戻す
        QObject *target = nullptr;
        if (_view) {
            target = _view->findChild<QObject *>(_activeButton);
        }
        if (target) {
            target->setProperty("press", false);
            // idle 画像を選択状態の画像に変更
            target->setProperty("idlebg", "iconbtn.png");
        }
        _selectedBrightness = target;
    }
}

void brightnessSettingViewModel::onRequest(int condition)
{
    emit request(_screenId, condition);
}

void brightnessSettingViewModel::onButtonPress(ButtonId id)
{
    if (id == BUTTON_ENTER) {
        // 選択中のボタン背景を通常に戻す
        if (_activeButton != brightnessSettingMenuButtonData[1][0]) {
            if (_selectedBrightness) {
                _selectedBrightness->setProperty("idlebg", "iconbtn.png");
            }

        }

        QObject *target = nullptr;
        if (_view) {
            target = _view->findChild<QObject *>(_activeButton);
        }
        if (target) {
            target->setProperty("press", true);
        }
    }
}
