/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include "modelinformationviewmodel.h"
#include <QTimer>

ModelInformationViewModel::ModelInformationViewModel(ScreenId screenId, LevelCheckerModel *&model, QObject *parent)
    : AbstractViewModel(screenId, model, parent)
    , _buttonOpration()
    , _updateButton(nullptr)
    , _acquiring(false)
{
    setObjectName("modelInformationViewModel");
    _buttonOpration.setActiveObject(QString("updateButton"));
}

ModelInformationViewModel::~ModelInformationViewModel()
{
}

void ModelInformationViewModel::createConnection(QObject *qml, QObject *mainFrame)
{
    AbstractViewModel::createConnection(qml, mainFrame);

    connect(_model, SIGNAL(notifyVersionUpReady(bool)), this, SLOT(onNotifyVersionUpReady(bool)));
    connect(_model, SIGNAL(notifySoftwareUpdateCompleted(bool)), this, SLOT(onNotifySoftwareUpdateCompleted(bool)));
    connect(this, SIGNAL(showIndicator(bool)), mainFrame, SLOT(onShowIndicator(bool)));

    if (_view) {
        _updateButton = _view->findChild<QObject *>("updateButton");
    }
}


void ModelInformationViewModel::activate()
{
    // サブを起動
    _model->wakeUp(1);

    if (_view == nullptr) {
        qErrnoWarning("_view is null");
        return;
    }

    QTimer *timer = new QTimer;
    uint32_t waitingTime = _model->applicationSetting()->waitingTimeForSub();
    timer->setInterval(waitingTime);
    timer->setSingleShot(true);
    QObject::connect(timer, &QTimer::timeout, [this, timer]() {
        if (_acquiring) {
            // 画面を開いてから一定時間待って機種情報を取得し、画面へ反映
            setModelInformation();
            timer->deleteLater();
        }
    });

    // バージョンアップ可なら最新にアップデートボタンをアクティブに
    if (_model->versionUpReady()) {
        if (_updateButton != nullptr) {
            _updateButton->setProperty("visible", true);
            _updateButton->setProperty("active", true);
        }
    }

    // サブが起動するまで待つためのタイマーを開始
    _acquiring = true;
    timer->start();
}

void ModelInformationViewModel::suspend()
{
    _acquiring = false;

    AbstractViewModel::suspend();
}

void ModelInformationViewModel::deactivate()
{
    AbstractViewModel::deactivate();

    disconnect(_model, SIGNAL(notifyVersionUpReady(bool)), this, SLOT(onNotifyVersionUpReady(bool)));
    disconnect(_model, SIGNAL(notifySoftwareUpdateCompleted(bool)), this, SLOT(onNotifySoftwareUpdateCompleted(bool)));
    disconnect(this, SIGNAL(showIndicator(bool)), _mainFrame, SLOT(onShowIndicator(bool)));
}

void ModelInformationViewModel::setModelInformation()
{
    // 機種情報の取得を行う
    LC70_PRODUCT_INFO_T productionInfo;
    memset(&productionInfo, 0, sizeof(LC70_PRODUCT_INFO_T));
    _model->getModelInformation(productionInfo);

    QString modelCode;
    for (int i = 0; i < LC70_MODELCODE_SIZE; ++i) {
        QChar code = QChar(productionInfo.cModelCode[i]);
        if (code == '\0') {
            break;
        }
        modelCode += code;
    }

    QString optionInfo;
    for (int i = 0; i < LC70_OPTINFO_SIZE; ++i) {
        QChar code = QChar(productionInfo.cOptInfo[i]);
        if (code == '\0') {
            break;
        }
        optionInfo += code;
    }
    QString serialNo;
    for (int i = 0; i < LC70_SERIALNO_SIZE; ++i) {
        QChar code = QChar(productionInfo.cSerialNo[i]);
        if (code == '\0') {
            break;
        }
        serialNo += code;
    }

    QString bootProgramVersion;
    for (int i = 0; i < LC70_VERSION_SIZE; ++i) {
        QChar code = QChar(productionInfo.cBootPrgVer[i]);
        if (code == '\0') {
            break;
        }
        bootProgramVersion += code;
    }

    QString mainProgramVersion;
    for (int i = 0; i < LC70_VERSION_SIZE; ++i) {
        QChar code = QChar(productionInfo.cMainPrgVer[i]);
        if (code == '\0') {
            break;
        }
        mainProgramVersion += code;
    }

    QString subProgramVersion;
    for (int i = 0; i < LC70_VERSION_SIZE; ++i) {
        QChar code = QChar(productionInfo.cSubPrgVer[i]);
        if (code == '\0') {
            break;
        }
        subProgramVersion += code;
    }

    QString displayProgramVersion;
    for (int i = 0; i < LC70_VERSION_SIZE; ++i) {
        QChar code = QChar(productionInfo.cQtVer[i]);
        if (code == '\0') {
            break;
        }
        displayProgramVersion += code;
    }

    QString productVersion;
    for (int i = 0; i < LC70_VERSION_SIZE; ++i) {
        QChar code = QChar(productionInfo.cProductVer[i]);
        if (code == '\0') {
            break;
        }
        productVersion += code;
    }

    if (_view) {
        _view->setProperty("modelCode", modelCode);
    }
    if (_view) {
        _view->setProperty("optionCode", optionInfo);
    }
    if (_view) {
        _view->setProperty("serialNo", serialNo);
    }
    if (_view) {
        _view->setProperty("bootProgramVersion", bootProgramVersion);
    }
    if (_view) {
        _view->setProperty("mainProgramVersion", mainProgramVersion);
    }
    if (_view) {
        _view->setProperty("subProgramVersion", subProgramVersion);
    }
    if (_view) {
        _view->setProperty("qtVersion", displayProgramVersion);
    }
    if (_view) {
        _view->setProperty("productVersion", productVersion);
    }
}

void ModelInformationViewModel::onKeyStatus(ButtonId id)
{
    if (id == BUTTON_ESCAPE) {
        emit back();
    } else if (id == BUTTON_HOME) {
        emit requestDirect(SCREEN_HOME, true);
    } else if (id == BUTTON_USER) {
#ifdef NEW_USER_SETTING
        emit requestDirect(SCREEN_USER_SETTING, true);
#else
        emit requestDirect(SCREEN_USER_DATA_LOGGER_MENU, true);
#endif
    } else if (id == BUTTON_ENTER) {
        QObject *target = nullptr;
        if (_view) {
            target = _view->findChild<QObject *>("updateButton");
        }
        if (target) {
            target->setProperty("press", false);
        }

        if (_model->versionUpReady()) {
            // アップデート確認メッセージを表示
            emit requestToShowPopup(POPUP_CONFIRM_UPDATE);
        }
    } else {
        // pass
    }
}

void ModelInformationViewModel::recover()
{
    if (_view) {
        _updateButton = _view->findChild<QObject *>("updateButton");
    }
}

void ModelInformationViewModel::onButtonPress(ButtonId id)
{
    if (id == BUTTON_ENTER) {
        QObject *target = nullptr;
        if (_view) {
            target = _view->findChild<QObject *>("updateButton");
        }
        if (target) {
            target->setProperty("press", true);
        }
    }
}

void ModelInformationViewModel::onAccept(PopupId popupId, bool okOnly)
{
    AbstractViewModel::onAccept(popupId, okOnly);

    AbstractViewModel::onAccept(popupId, okOnly);

    if (popupId == POPUP_CONFIRM_UPDATE) {
        // インジケーター表示
        emit showIndicator(true);

        // アップデート中のメッセージを表示
        emit requestToShowPopup(POPUP_UPDATING);

        // プログラムのアップデート
        _model->updateProgram();
    }
}

void ModelInformationViewModel::onNotifyVersionUpReady(bool versionUpReady)
{
    if (versionUpReady) {
        // ボタン有効
        if (_updateButton != nullptr) {
            _updateButton->setProperty("visible", true);
            _updateButton->setProperty("active", true);
        }
    } else {
        // ボタン無効
        if (_updateButton != nullptr) {
            _updateButton->setProperty("active", false);
            _updateButton->setProperty("visible", false);
        }
    }
}

void ModelInformationViewModel::onNotifySoftwareUpdateCompleted(bool success)
{
    Q_UNUSED(success);
    // インジケーター非表示
    emit showIndicator(false);
    // アップデート中のポップアップを閉じる
    emit requestPopupClose();
}
