/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include "systemsettingmenuviewmodel.h"

QVector<QVector<QString>> systemMenuButtonData = {
    { {"func1"}, {"func2"}, {"func3"} },
    { {"func4"}, {"func5"}, {"func6"} },
    { {"func7"}, {"func8"}, {"func9"} }
};
const int SystemSettingMenuViewModel::BUTTON_COLUMN_NUMBER = 3;
const int SystemSettingMenuViewModel::BUTTON_ROW_NUMBER    = 3;

SystemSettingMenuViewModel::SystemSettingMenuViewModel(ScreenId screenId, LevelCheckerModel *&model, QObject *parent)
    : AbstractViewModel(screenId, model, parent)
{
    setObjectName("systemSettingMenuViewModel");
    _buttonOperation.setButtonName(BUTTON_COLUMN_NUMBER, BUTTON_ROW_NUMBER, systemMenuButtonData);

    _activeButton = systemMenuButtonData[0][0];
    _buttonOperation.setActiveObject(_activeButton);
}

SystemSettingMenuViewModel::~SystemSettingMenuViewModel()
{

}

void SystemSettingMenuViewModel::createConnection(QObject *qml, QObject *mainFrame)
{
    AbstractViewModel::createConnection(qml, mainFrame);
}

void SystemSettingMenuViewModel::activate()
{
    AbstractViewModel::activate();

    if (nullptr == _view) {
        qErrnoWarning("_view is null");
        return;

    }

    // ヘッダメニュー表示
    _model->replaceMenuList(MenuListIconIndex::FUNCTION_ICON, "icon_menu_system.png");

    // 先頭のボタンをアクティブにする
    resetToInitialButtonState(systemMenuButtonData[0][0], _activeButton, _buttonOperation);
}

void SystemSettingMenuViewModel::deactivate()
{
    AbstractViewModel::deactivate();
}

void SystemSettingMenuViewModel::onRequest(int condition)
{
    emit request(_screenId, condition);
}

void SystemSettingMenuViewModel::moveCursor(ButtonId buttonId)
{
    if ((buttonId == BUTTON_RIGHT) || (buttonId == BUTTON_LEFT) || (buttonId == BUTTON_UP) || (buttonId == BUTTON_DOWN)) {
        // UP, DOWN なし

        QString nextBtn;
        bool ret = _buttonOperation.nextActiveObject(buttonId, nextBtn);

        if (ret) {

            // inactive
            QObject *target = nullptr;
            if (_view) {
                target = _view->findChild<QObject *>(_activeButton);
            }
            if (target) {
                target->setProperty("active", false);
            }

            // active
            target = nullptr;
            if (_view) {
                target = _view->findChild<QObject *>(nextBtn);
            }
            if (target) {
                target->setProperty("active", true);
            }

            _activeButton = nextBtn;
            _buttonOperation.setActiveObject(_activeButton);
        }
    }
}

void SystemSettingMenuViewModel::onKeyStatus(ButtonId id)
{
    // 電源 ボタン押下は、HOME画面と同様の処理を行う
    // HOME ボタン押下で本画面を消去し、HOME画面に遷移する
    // USER ボタン押下で本画面を消去し、USER画面に遷移する
    // ESCAPE ボタン押下で本画面を消去し、HOME画面に戻る

    if (id == BUTTON_ENTER) {

        QObject *target = nullptr;
        if (_view) {
            target = _view->findChild<QObject *>(_activeButton);
        }
        if (target) {
            target->setProperty("press", false);
        }
        if (isErrorUsbStatus()) {
            return;
        }
        // 決定ボタン押下で選択中の画面を表示する
        if(_activeButton == "func7"){
            emit requestToShowPopup(POPUP_USB_FORMAT_CONFIRMATION); // USBフォーマット実行確認
        }else{

            int button_index = _activeButton.mid(4).toInt() - 1; // ”func" 文字列を削除して数字を取り出す

            if(_activeButton == "func8" || _activeButton == "func9"){
                button_index = _activeButton.mid(4).toInt() - 2;
                onRequest((button_index));
            }else{
                onRequest(button_index);
            }

        }
    } else if (id == BUTTON_ESCAPE || id == BUTTON_HOME) {
        emit back();
    } else if (id == BUTTON_USER) {
#ifdef NEW_USER_SETTING
        emit requestDirect(SCREEN_USER_SETTING, true);
#else
        emit requestDirect(SCREEN_USER_DATA_LOGGER_MENU, true);
#endif
    } else {
        moveCursor(id);
    }
}

void SystemSettingMenuViewModel::onButtonPress(ButtonId id)
{
    if (id == BUTTON_ENTER) {
        QObject *target = nullptr;
        if (_view) {
            target = _view->findChild<QObject *>(_activeButton);
        }
        if (target) {
            target->setProperty("press", true);
        }
    }
}


//------------------------USBformat------------------------------------
void SystemSettingMenuViewModel::startUsbFormat()
{
    if (UsbOperation::format() != UsbErrorType::NON_ERROR) {
        // エラー
        emit requestPopupClose();
        QTimer::singleShot(100, this, SLOT(displayFormatErrorPopup()));
        return;
    }

    // フォーマット完了
    emit requestPopupClose();
}


bool SystemSettingMenuViewModel::isErrorUsbStatus()
{
    QMap<QString, QList<LC70_STRAGE_T>> enableUsbStatusMap = {\
        {"func7", {LC70_STRAGE_FORMAT, LC70_STRAGE_UNFORMAT}},
    };

    if (enableUsbStatusMap.count(_activeButton) == 0) {
        // USBに無関係のボタン
        return false;
    }

    if (enableUsbStatusMap.value(_activeButton).contains(_model->getDeviceStatus().usba)) {
        // 実行可能なUSB状態になっている
        return false;
    }

    // ボタンごとに定義している、押下可能なUSB状態になっていなければエラー表示
    setBoolProperty(_activeButton, "press", false);
    emit requestToShowPopup(POPUP_FAILURE_NO_USB_CONFIRMATION); // USB未装着エラー
    return true;
}

void SystemSettingMenuViewModel::setBoolProperty(QString objectName, QString propertyName, bool value)
{
    QObject *target = nullptr;
    if (_view) {
        target = _view->findChild<QObject *>(objectName);
    }
    if (target) {
        target->setProperty(propertyName.toUtf8().constData(), value);
    }

}

void SystemSettingMenuViewModel::onAccept(PopupId popupId, bool okOnly)
{
    AbstractViewModel::onAccept(popupId, okOnly);

    if (popupId == POPUP_USB_FORMAT_CONFIRMATION) {
        // USBのフォーマットを実施
        emit requestToShowPopup(POPUP_USB_FORMAT_RUNNING);
        QTimer::singleShot(300, this, SLOT(startUsbFormat()));
    }
}

//------------------------USBformat------------------------------------
