/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include "volumesettingviewmodel.h"

QVector<QVector<QString>> volumeSettingMenuButtonData = {
    { {"volume1"}, {"volume2"}, {"volume3"}, {"volume4"}},
    { {"register"}, {""}, {""}, {""}}
};

const int VolumeSettingViewModel::BUTTON_COLUMN_NUMBER = 2;
const int VolumeSettingViewModel::BUTTON_ROW_NUMBER = 4;

VolumeSettingViewModel::VolumeSettingViewModel(ScreenId screenId, LevelCheckerModel *&model, QObject *parent)
    : AbstractViewModel(screenId, model, parent),
      _currentVolume(0),
      _selectedVolume(nullptr)
{
    _buttonOperation.setButtonName(BUTTON_COLUMN_NUMBER, BUTTON_ROW_NUMBER, volumeSettingMenuButtonData);
}

VolumeSettingViewModel::~VolumeSettingViewModel()
{

}

void VolumeSettingViewModel::activate()
{
    if (nullptr == _view) {
        qErrnoWarning("_view is null");
        return;
    }

    // ボタンActive設定
    QVariant value;
    int buttonIndex = 0;

    if (_model->getValue(VALUE_SOUNDVOLUME, value)) {
        int settingVolume = value.toInt();
        qDebug() << "SOUNDVOLUME : " << settingVolume;

        // [Memo]
        //  モデルから得られたデータが範囲外であった場合
        //  音量の設定値を初期値として扱う。
        if (settingVolume < LC70_SYS_SET_VOLUME_OFF || settingVolume > LC70_SYS_SET_VOLUME_MAX) {
            settingVolume = LC70_SYS_SET_VOLUME_OFF;
        }

        buttonIndex = settingVolume;
        _currentVolume = settingVolume; // 初期音量を保持する
    }

    // Active にするボタンを設定
    QString itemname = volumeSettingMenuButtonData[0][buttonIndex];
    QObject *item = nullptr;
    if (_view) {
        item = _view->findChild<QObject *>(itemname);
    }
    if (item) {
        item->setProperty("active", true);
        item->setProperty("idlebg", "iconbtn.png");
    }
    _selectedVolume = item;

    _activeButton = itemname;
    _buttonOperation.setActiveObject(_activeButton);
}

void VolumeSettingViewModel::onRequest(int condition)
{
    emit request(_screenId, condition);
}

void VolumeSettingViewModel::moveCursor(ButtonId buttonId)
{

    QString nextBtn;

    if ((buttonId == BUTTON_RIGHT) || (buttonId == BUTTON_LEFT)) {
        if (_activeButton == volumeSettingMenuButtonData[1][0]) {
            return;
        } else {
            _buttonOperation.nextActiveObject(buttonId, nextBtn);
        }
    } else if ((buttonId == BUTTON_UP) || (buttonId == BUTTON_DOWN)) {
        // 上下移動は左端のボタンに移動
        if (_activeButton == volumeSettingMenuButtonData[1][0]) {
            nextBtn = volumeSettingMenuButtonData[0][0];
        } else {
            nextBtn = volumeSettingMenuButtonData[1][0];
        }
    } else {
        // 通らない
        return;
    }

    // inactive
    QObject *target = nullptr;
    if (_view) {
        target = _view->findChild<QObject *>(_activeButton);
    }
    if (target) {
        target->setProperty("active", false);
    }

    // active
    target = nullptr;
    if (_view) {
        target = _view->findChild<QObject *>(nextBtn);
    }
    if (target) {
        target->setProperty("active", true);
    }

    _activeButton = nextBtn;
    _buttonOperation.setActiveObject(_activeButton);
}

void VolumeSettingViewModel::onKeyStatus(ButtonId id)
{
    // 電源 ボタン押下は、HOME画面と同様の処理を行う
    // HOME ボタン押下で本画面を消去し、HOME画面に遷移する
    // USER ボタン押下で本画面を消去し、USER画面に遷移する
    // ESCAPE ボタン押下で本画面を消去し、HOME画面に戻る

    if (id == BUTTON_ENTER) {
        onKeyEnter();
    } else if (id == BUTTON_ESCAPE || id == BUTTON_HOME || id == BUTTON_USER) {
        if (_model->isUpdate(VALUE_SOUNDVOLUME)) {
            emit requestToShowPopup(POPUP_CANCEL_CONFIRMATION); // 破棄確認
        } else {
            onAccept(POPUP_CANCEL_CONFIRMATION, true);
        }
    } else {
        moveCursor(id);
    }
}

void VolumeSettingViewModel::onAccept(PopupId popupId, bool okOnly)
{
    Q_UNUSED(okOnly)

    // 音量を設定
    if (popupId == POPUP_SAVE_CONFIRMATION) {
        // 画面での設定を反映
        _model->commit(VALUE_SOUNDVOLUME);
        // メニューバーの音量アイコンを更新
        _model->changeSoundVolumeIcon();

        // メニューに戻る
        emit back();

    } else if (popupId == POPUP_CANCEL_CONFIRMATION) {
        // 設定、音量を画面遷移時の設定に戻す
        QVariant soundVolume;
        _model->rollback(VALUE_SOUNDVOLUME);
        _model->getValue(VALUE_SOUNDVOLUME, soundVolume);
        _model->fourceBuzzerVolume(soundVolume.toInt(), LC70_BUZZ_OFF);

        // 押下されたボタンにより遷移先を変更する
        switch (_lastPressedButtonId) {
        case BUTTON_HOME:
            emit requestDirect(SCREEN_HOME, true);
            break;

        case BUTTON_ESCAPE:
            emit back();
            break;

        case BUTTON_USER:
#ifdef NEW_USER_SETTING
        emit requestDirect(SCREEN_USER_SETTING, true);
#else
        emit requestDirect(SCREEN_USER_DATA_LOGGER_MENU, true);
#endif
            break;

        default:
            break;
        }
    }
}

void VolumeSettingViewModel::onReject(PopupId popupId)
{
    Q_UNUSED(popupId)

    // Press 画像から元に戻す
    QObject *target = nullptr;
    if (_view) {
        target = _view->findChild<QObject *>(_activeButton);
    }
    if (target) {
        target->setProperty("press", false);
    }
}

void VolumeSettingViewModel::onKeyEnter()
{
    if (_activeButton == volumeSettingMenuButtonData[1][0]) {
        // 保存確認
        emit requestToShowPopup(POPUP_SAVE_CONFIRMATION);
    } else {
        int volume = -1;

        for (int i = 0; i < BUTTON_ROW_NUMBER; i++) {
            if (_activeButton == volumeSettingMenuButtonData[0][i]) {
                volume = i;
                break;
            }
        }

        if (volume != -1) {
            if (volume != 0) {
                // お試し鳴動
                // [Memo]
                //  音量設定がOFF以外の時には、お試しでブザー音を鳴らす
                if (_model->fourceBuzzerVolume(volume, LC70_BUZZ_VOL_CHG) == false) {
                    qErrnoWarning("sound trial buzzer fail.");
                }
            }

            QVariant value = volume;

            if (value.isValid()) {
                _model->setValue(VALUE_SOUNDVOLUME, value);
            }
        }

        // Press 画像から元に戻す
        QObject *target = nullptr;
        if (_view) {
            target = _view->findChild<QObject *>(_activeButton);
        }
        if (target) {
            target->setProperty("press", false);
            // idle 画像を選択状態の画像に変更
            target->setProperty("idlebg", "iconbtn.png");
        }
        _selectedVolume = target;
    }
}

void VolumeSettingViewModel::onButtonPress(ButtonId id)
{
    if (id == BUTTON_ENTER) {
        // 選択中のボタン背景を通常に戻す
        if (_activeButton != volumeSettingMenuButtonData[1][0]) {
            if (_selectedVolume) {
                _selectedVolume->setProperty("idlebg", "iconbtn.png");
            }

        }

        QObject *target = nullptr;
        if (_view) {
            target = _view->findChild<QObject *>(_activeButton);
        }
        if (target) {
            target->setProperty("press", true);
        }
    }
}

void VolumeSettingViewModel::recover()
{
    QString itemname = _activeButton;
    QObject *item = nullptr;
    if (_view) {
        item = _view->findChild<QObject *>(itemname);
    }
    if (item) {
        item->setProperty("active", true);
        item->setProperty("idlebg", "iconbtn.png");
    }
    _selectedVolume = item;
}
