//
//      (c) 2020 DX ANTENNA CO.,LTD.
//

#include "autosettingviewmodel.h"
#include "mainframeviewmodel.h"

static QVector<QVector<QString>> _bandButtonData = {
    { {"m1"},  {"m2"}, {"m3"}, {"m4"},  {""} },
    { {"m6"},  {"" },  {"m8"}, {"m9"},  {""} },
    { {"m11"}, {""},   {""},   {"m14"}, {""} }
};
const int AutoSettingViewModel::BUTTON_COLUMN_NUMBER = 3;
const int AutoSettingViewModel::BUTTON_ROW_NUMBER    = 5;

// 給電 を表示するメニューリストのインデックス
static constexpr int POWER_SUPPLY_INDEX = 3;

// memo : スキャン関係の処理に修正が発生した際は
//        偏波面選択画面にも展開が必要か確認

AutoSettingViewModel::AutoSettingViewModel(ScreenId screenId, LevelCheckerModel *&model, QObject *parent)
    : AbstractViewModel(screenId, model, parent)
    , _selectedMeasurement("")
    , _bandId(0)
    , _channelList()
    , _broadBandModulationSetting()
    , _powerSupply(LC70_POWER_SUPPLY_OFF)
    , _scanConditionModulationValue(LC70_SETMOD_UNDEF)
{
    setObjectName("AutoSetting");
    _buttonOperation.setButtonName(BUTTON_COLUMN_NUMBER, BUTTON_ROW_NUMBER, _bandButtonData);

    _activeButton = _bandButtonData[0][0];
    _buttonOperation.setActiveObject(_activeButton);
}

AutoSettingViewModel::~AutoSettingViewModel()
{

}

void AutoSettingViewModel::createConnection(QObject *qml, QObject *mainFrame)
{
    AbstractViewModel::createConnection(qml, mainFrame);

    connect(_model, SIGNAL(scanningChannelResult(int, LC70_MEAS_MULTI_RESULT_T)),
            this, SLOT(onScanningChannelResult(int, LC70_MEAS_MULTI_RESULT_T)));
    connect(_model, SIGNAL(scanningFinished(bool)), this, SLOT(onScanningFinished(bool)));
    connect(mainFrame, SIGNAL(requestToScan()), this, SLOT(onRequestToScan()));
    connect(mainFrame, SIGNAL(requestToStopScanning()), this, SLOT(onRequestToStopScanning()));
    connect(mainFrame, SIGNAL(notifyPowerSupplyError(QString)), this, SLOT(detectPowerSupplyError(QString)));
    connect(this, SIGNAL(requestSetTakeOver(QVariant, QVariant)), _view, SLOT(backup(QVariant, QVariant)));
    connect(this, SIGNAL(progress(double)), mainFrame, SIGNAL(progress(double)));
    connect(this, SIGNAL(requestToReScan()), this, SLOT(onRequestToReScan()), Qt::QueuedConnection);
    connect(this, SIGNAL(requestToShowRegistScanResultPopup()), this, SLOT(onRequestToShowRegistScanResultPopup()), Qt::QueuedConnection);
}

void AutoSettingViewModel::activate()
{
    // マニュアル測定 放送方式, 変調方式選択の取得
    _broadBandModulationSetting.clear();
    _model->getValue(VALUE_BROADBANDSYSTEM_MODULATION, _broadBandModulationSetting);

    QObject *target = _view->findChild<QObject *>("takeOverData");

    if (target) {
        QVariant value = target->property("selectedMeasurement");
        _selectedMeasurement = value.toString();
    }

    // 前回開いたときのボタンをアクティブに復元
    target = _view->findChild<QObject *>(_activeButton);

    if (target) {
        target->setProperty("active", true);
    } else {
        // 初回あるいは Home 画面でリセットされたときは先頭のボタンをアクティブにする
        target = _view->findChild<QObject *>(_bandButtonData[0][0]);

        if (target == nullptr) {
            qDebug() << "undefined objectName " + _bandButtonData[0][0];
            return;
        }

        target->setProperty("active", true);
    }

    setScreenName();

    // ユーザ設定(自動)のBAND選択では表示しない (対象: アンテナ調整)
    target = _view->findChild<QObject *>("m13");

    if (target) {
        target->setProperty("disable", true);
    }

    // ユーザ設定(自動)のBAND選択では表示しない (対象: FM)
    target = _view->findChild<QObject *>("m5");
    target->setProperty("disable", true);

    // ユーザ設定(自動)のBAND選択では表示しない (対象: LTE)
    target = _view->findChild<QObject *>("m14");
    target->setProperty("disable", true);

    // TODO: その他の更新
    // ※シングル測定の場合は、JCSAT アンテナ調整のボタンを使用不可で表示する
    // ※光・同軸種別が光の場合は、スカパープレミアムグループの３つのボタンと
    //   その他グループの３つのボタンは使用不可（トーンダウン）で表示する

    //showHeaderArea();
}

void AutoSettingViewModel::deactivate()
{
    AbstractViewModel::deactivate();

    disconnect(_model, SIGNAL(scanningChannelResult(int, LC70_MEAS_MULTI_RESULT_T)),
               this, SLOT(onScanningChannelResult(int, LC70_MEAS_MULTI_RESULT_T)));
    disconnect(_model, SIGNAL(scanningFinished(bool)), this, SLOT(onScanningFinished(bool)));
    disconnect(_mainFrame, SIGNAL(requestToScan()), this, SLOT(onRequestToScan()));
    disconnect(_mainFrame, SIGNAL(requestToStopScanning()), this, SLOT(onRequestToStopScanning()));
    disconnect(_mainFrame, SIGNAL(notifyPowerSupplyError(QString)), this, SLOT(detectPowerSupplyError(QString)));
    disconnect(this, SIGNAL(requestSetTakeOver(QVariant, QVariant)), _view, SLOT(backup(QVariant, QVariant)));
    disconnect(this, SIGNAL(progress(double)), _mainFrame, SIGNAL(progress(double)));
    disconnect(this, SIGNAL(requestToReScan()), this, SLOT(onRequestToReScan()));
    disconnect(this, SIGNAL(requestToShowRegistScanResultPopup()), this, SLOT(onRequestToShowRegistScanResultPopup()));

    // 給電
    _model->removeMenuList(MenuListIconIndex::POWER_SUPPLY_ICON);

    // スキャン
    _model->removeMenuList(MenuListIconIndex::SCAN_ICON);
}

void AutoSettingViewModel::suspend()
{
    // 給電を停止
    _powerSupply = LC70_POWER_SUPPLY_OFF;
    _model->setPowerSupply(_powerSupply);
    _model->removeMenuList(MenuListIconIndex::POWER_SUPPLY_ICON);

    AbstractViewModel::suspend();
}

void AutoSettingViewModel::setTakeOver()
{
    // BAND は LC70_BAND_T のメンバーを値とする
    if (_activeButton == "m1") {
        emit requestSetTakeOver("selectedBand", LC70_BAND_UHF);
    } else if (_activeButton == "m6") {
        emit requestSetTakeOver("selectedBand", LC70_BAND_BS);
    } else if (_activeButton == "m11") {
        emit requestSetTakeOver("selectedBand", LC70_BAND_CS);
    } else if (_activeButton == "m3") {
        emit requestSetTakeOver("selectedBand", LC70_BAND_JCSAT3);
    } else if (_activeButton == "m8") {
        emit requestSetTakeOver("selectedBand", LC70_BAND_JCSAT4);
    } else if (_activeButton == "m4") {
        emit requestSetTakeOver("selectedBand", LC70_BAND_SCCB);
    } else if (_activeButton == "m9") {
        emit requestSetTakeOver("selectedBand", LC70_BAND_SCCC);
    } else if (_activeButton == "m14") {
        emit requestSetTakeOver("selectedBand", LC70_BAND_LTE);
    } else if (_activeButton == "m5") {
        emit requestSetTakeOver("selectedBand", LC70_BAND_FM);
    } else {}
}

int AutoSettingViewModel::getActiveBandId()
{
    QMap<QString, LC70_BAND_T> buttonBandMap = {
        {"m1", LC70_BAND_UHF    },
        {"m6", LC70_BAND_BS     },
        {"m11", LC70_BAND_CS    },
        {"m3", LC70_BAND_JCSAT3 },
        {"m8", LC70_BAND_JCSAT4 },
        {"m4", LC70_BAND_SCCB   },
        {"m9", LC70_BAND_SCCC   },
        {"m14", LC70_BAND_LTE   },
        {"m5", LC70_BAND_FM     }
    };

    return buttonBandMap.value(_activeButton, LC70_BAND_END);
}

void AutoSettingViewModel::moveCursor(ButtonId buttonId)
{
    if ((buttonId == BUTTON_UP) || (buttonId == BUTTON_DOWN) || (buttonId == BUTTON_RIGHT) || (buttonId == BUTTON_LEFT)) {

        QString nextButton;
        bool ret = _buttonOperation.nextActiveObject(buttonId, nextButton);

        if (ret) {

            // inactive
            QObject *target = _view->findChild<QObject *>(_activeButton);

            if (target) {
                target->setProperty("active", false);
            }

            // active
            target = _view->findChild<QObject *>(nextButton);

            if (target) {
                target->setProperty("active", true);
            }

            _activeButton = nextButton;
            _buttonOperation.setActiveObject(_activeButton);
        }
    }
}

void AutoSettingViewModel::onKeyStatus(ButtonId id)
{
    // Home, Escape ボタンはベースクラスで対応
    AbstractViewModel::onKeyStatus(id);

    if (id == BUTTON_ENTER) {
        // 給電/SCAN 実行確認
        QObject *target = _view->findChild<QObject *>(_activeButton);
        target->setProperty("press", false);

        if (_activeButton == "m2") {
            // CATVボタンを選択した場合、
            // CATVバンド選択ボタンへ遷移する。
            onRequest(3);
        } else {
            if ((getActiveBandId() == LC70_BAND_BS)            // BS
                    || (getActiveBandId() == LC70_BAND_CS)     // CS
                    || (getActiveBandId() == LC70_BAND_JCSAT3) // JCSAT3A
                    || (getActiveBandId() == LC70_BAND_JCSAT4) // JCSAT4B
                    || (getActiveBandId() == LC70_BAND_SCCB)   // SCC-B2
                    || (getActiveBandId() == LC70_BAND_SCCC)) { // SCC-C2
                // USER・データロガー設定  自動画面から偏波面選択画面への遷移
                // 偏波面選択画面で給電/SCAN
                onRequest(2);
            } else {
                if (isLowBattery()) {
                    // バッテリ低下状態中は給電開始させない
                    onRequestScanConfirm();
                    return;
                }
                // その他は [15v, 給電しない]
                emit requestToShowPopup(POPUP_POWER_SUPPLY_SELECT2);
            }
        }
    } else if (id == BUTTON_USER) {
#ifdef NEW_USER_SETTING
        emit requestDirect(SCREEN_USER_SETTING, true);
#else
        emit requestDirect(SCREEN_USER_DATA_LOGGER_MENU, true);
#endif
    } else {
        moveCursor(id);
    }
}

void AutoSettingViewModel::onButtonPress(ButtonId id)
{
    if (id == BUTTON_ENTER) {
        QObject *target = _view->findChild<QObject *>(_activeButton);
        target->setProperty("press", true);
    }
}

void AutoSettingViewModel::onAccept(PopupId popupId, bool okOnly)
{
    Q_UNUSED(okOnly);

    AbstractViewModel::onAccept(popupId, okOnly);

    if (popupId == POPUP_POWER_SUPPLY_SELECT2) {
        // 給電選択で 15v を選択
        _powerSupply = LC70_POWER_SUPPLY_15V;
        _model->setPowerSupply(_powerSupply);
        _model->replaceMenuList(MenuListIconIndex::POWER_SUPPLY_ICON, Correcter::powerSupplyIcon(_powerSupply));

        // スキャン開始
        // emit requestToShowPopup(POPUP_SCAN_CONFIRMATION);
        //   onReject 経路でのスキャン開始は処理を一度切る必要があった
        //   見えるタイミングがことなるため onReject 同じ処理とする
        QTimer::singleShot(100, this, SLOT(onRequestScanConfirm()));
    } else if (popupId == POPUP_POWER_SUPPLY_ERROR_MANUAL_MEASURING) {
        // 画面表示を継続するため何もしない
        return;
    } else if (popupId == POPUP_CONFIRM_REGIST_SCAN_RESULT) {
        // 保存先選択画面を表示
        onRequest(1);
    } else {
        // nothing to do
    }
}

void AutoSettingViewModel::onReject(PopupId popupId)
{
    if (popupId == POPUP_SCAN_CANCEL) {
    } else if (popupId == POPUP_POWER_SUPPLY_SELECT2) {
        // スキャン開始
        QTimer::singleShot(100, this, SLOT(onRequestScanConfirm()));
    }

    // 給電中だった場合は給電OFFにする
    if (_powerSupply != LC70_POWER_SUPPLY_OFF) {
        _powerSupply = LC70_POWER_SUPPLY_OFF;
        _model->setPowerSupply(_powerSupply);
        _model->replaceMenuList(MenuListIconIndex::POWER_SUPPLY_ICON, Correcter::powerSupplyIcon(_powerSupply));
    }

    // スキャン結果のアイコンを消去
    _model->removeMenuList(MenuListIconIndex::SCAN_ICON);

}

void AutoSettingViewModel::onRequest(int condition)
{
    if (condition == 2 || condition == 3) {
        emit requestSetTakeOver("autoSettingPolarization", true);
    }

    emit request(_screenId, condition);
    return;
}

void AutoSettingViewModel::onSelect(PopupId popupId, int index)
{
    if (popupId == POPUP_SCAN_RESULT) {
        // スキャン結果を採用する
        selectReactionScanResult(index);
        return;
    }
}

void AutoSettingViewModel::onLowBatteryStatusChanged(bool isLowBattery)
{
    if (isLowBattery && _powerSupply != LC70_POWER_SUPPLY_OFF) {
        // 給電を停止する
        _powerSupply = LC70_POWER_SUPPLY_OFF;
        _model->setPowerSupply(_powerSupply);
        _model->removeMenuList(MenuListIconIndex::POWER_SUPPLY_ICON);
        emit requestToShowLowBatteryPopup();
    }
}

void AutoSettingViewModel::onRequestToScan()
{
    // 指定BANDのチャンネルデータを取得
    _bandId = getActiveBandId();

    _model->getChannelDefinitionList(_bandId, _channelList);

    // 保持したスキャン結果をクリアー
    _model->clearScanResult();

    // 局発周波数
    // この画面からscan実行可能なBANDに局発周波数情報はないため 固定値を設定する
    uint32_t localFrequency = 0;

    // 変調方式

    // ローカルで保持している値を初期化する
    _scanConditionModulationValue = LC70_SETMOD_UNDEF;

    // 自動設定画面からスキャンを行うBANDの変調方式は、
    // 地デジの時には「OFDM」 他BANDの場合は「定義なし」と固定値となっている。
    if (_bandId == LC70_BAND_UHF){
        _scanConditionModulationValue = LC70_SETMOD_OFDM;
    } else {
        _scanConditionModulationValue = LC70_SETMOD_UNDEF;
    }

    QString modulation = Correcter::displayModulation(_scanConditionModulationValue);
    uint16_t bdSys = Correcter::getBroadcastSystemCondition(_bandId, modulation);

    // 放送方式、変調方式の設定(定義ファイル 1.0.1 対応)
    uint16_t mod = Correcter::getModulationCondition(_bandId, bdSys, 0, _scanConditionModulationValue, _broadBandModulationSetting);

    // スキャン実行を通知
    _model->writeLog(QString("USER・データロガー設定  自動"), QString("スキャン開始"));
    _model->scan(_channelList, AcquisitionTarget::NOT_ACQUISITION, localFrequency, bdSys, mod, false, _powerSupply);
}

void AutoSettingViewModel::onRequestToStopScanning()
{
    // スキャン中止
    _model->stopChannelScan();
    _model->writeLog(QString("USER・データロガー設定  自動"), QString("スキャン中止"));
}

void AutoSettingViewModel::onRequestScanConfirm()
{
    emit requestToShowPopup(POPUP_SCAN_CONFIRMATION);
}


void AutoSettingViewModel::onScanningChannelResult(int index, LC70_MEAS_MULTI_RESULT_T result)
{
    // ラッパー関数ライブラリ V2.00 対応
    if (result.ucLock == UNLOCKED) {
        // 無効判定
        _channelList[index].valid = false;
    }

    // SCAN時設定した変調方式がOFDMかつ、
    // スキャン結果がOFDM以外の変調方式の場合
    // スキャン結果として登録する変調方式をOFDMに設定する
    int registModulation = result.ulModBitPtrn;

    if (_scanConditionModulationValue == LC70_SETMOD_OFDM && result.ulModBitPtrn != LC70_SETMOD_OFDM){
        registModulation = LC70_SETMOD_OFDM;
    }

    // スキャン結果の保持
    if (result.ucLock == LOCKED) {
        MeasureResult scanResult;
        scanResult.Received = true;
        scanResult.Band = QString::number(_channelList[index].band);
        scanResult.Channel = _channelList[index].channelNumber;
        scanResult.Status = result.usStat;
        scanResult.Level = result.ulLevel;
        scanResult.Mer = 0;
        scanResult.Ber = 0;
        scanResult.BroadcastSystem = result.usBdSys;
        scanResult.Modulation = registModulation;
        scanResult.Layer = result.ucLayer;
        scanResult.Frequency = -1;
        scanResult.LocalOscillator = 0;
        _model->appendScanResult(scanResult);

        // スキャン結果で有効チャンネルが見つかった場合 ブザーを1度鳴動させる
        _model->buzzer(LC70_BUZZ_VOL_CHG, LC70_BUZZ_MODE_ONESHOT);
    } else {
        QString log =
            QString("無効チャンネル Channel:%1 Status:%2 Broadcast:%3 Modulation:%4").arg(
                _channelList[index].channelNumber).arg(result.usStat).arg(result.usBdSys).arg(result.ulModBitPtrn);
        _model->writeLog(QString("設定自動"), log, LOG_INFO);
    }

    // プログレスバーの進捗を設定
    qDebug() << QString("total: %1 current: %2").arg(_channelList.count()).arg(index);
    double value = static_cast<double>(index + 1) / static_cast<double>(_channelList.count()) * 100;
    emit progress(value);
}

void AutoSettingViewModel::onScanningFinished(bool complete)
{
    _model->writeLog(QString("USER・データロガー設定  自動"), QString("スキャン終了"));

    // 給電中だった場合は、一旦給電OFFにする
    if (_powerSupply != LC70_POWER_SUPPLY_OFF) {
        // 再スキャン時の給電有無を判断するため、_powerSupplyは更新しない
        _model->setPowerSupply(LC70_POWER_SUPPLY_OFF);
        _model->replaceMenuList(MenuListIconIndex::POWER_SUPPLY_ICON, Correcter::powerSupplyIcon(LC70_POWER_SUPPLY_OFF));
    }

    if (complete) {
        // スキャン完了時 ブザーを1度鳴動させる
        _model->buzzer(LC70_BUZZ_ALL_CH_SCAN, LC70_BUZZ_MODE_ONESHOT);

        emit requestToShowPopup(POPUP_SCAN_RESULT);
    } else {
        _model->removeMenuList(MenuListIconIndex::SCAN_ICON);
#ifdef NEW_USER_SETTING
        emit requestDirect(SCREEN_USER_SETTING, true);
#else
        emit requestDirect(SCREEN_USER_DATA_LOGGER_MENU, true);
#endif
    }
}

void AutoSettingViewModel::setScreenName()
{
    QObject *target = _view->findChild<QObject *>("screenName");

    if (target) {
        target->setProperty("name", "自動スキャン/編集");
    }

}

void AutoSettingViewModel::selectReactionScanResult(const int index) {
    if (index == 0) {
        // スキャンアイコン表示
        _model->replaceMenuList(MenuListIconIndex::SCAN_ICON, "icon_scan_n.png");

        // 保存先確認ダイアログを表示する
        emit requestToShowRegistScanResultPopup();
    } else {
        // 再スキャン
        emit requestToReScan();
    }
}

void AutoSettingViewModel::onRequestToReScan() {
    // スキャン中ポップアップを表示
    emit requestToShowPopup(POPUP_SCANNING);

    // スキャン開始時に給電が選択されていた場合は、再スキャン中も給電する
    if (_powerSupply != LC70_POWER_SUPPLY_OFF) {
        _model->setPowerSupply(_powerSupply);
        _model->replaceMenuList(MenuListIconIndex::POWER_SUPPLY_ICON, Correcter::powerSupplyIcon(_powerSupply));
    }

    // 再スキャン
    onRequestToScan();
}

void AutoSettingViewModel::detectPowerSupplyError(QString message) {
    // 給電を停止する
    _powerSupply = LC70_POWER_SUPPLY_OFF;
    _model->setPowerSupply(_powerSupply);
    _model->removeMenuList(MenuListIconIndex::POWER_SUPPLY_ICON);

    emit requestToShowPopupWithMessage(POPUP_POWER_SUPPLY_ERROR_MANUAL_MEASURING, message);
}

void AutoSettingViewModel::onRequestToShowRegistScanResultPopup() {
    emit requestToShowPopup(POPUP_CONFIRM_REGIST_SCAN_RESULT);
}
