//
//      (c) 2020 DX ANTENNA CO.,LTD.
//

#ifndef AUTOSETTINGVIEWMODEL_H
#define AUTOSETTINGVIEWMODEL_H

#include "abstractviewmodel.h"
#include "buttonoperation.h"
#include "correcter.h"

/**
 * @brief データロガー自動設定 ViewModel
 */
class AutoSettingViewModel : public AbstractViewModel
{
    Q_OBJECT
public:
    /**
     * @brief コンストラクター
     * @param screenId 画面 ID
     * @param model Model クラスのポインター
     * @param parent 親オブジェクト
     */
    explicit AutoSettingViewModel(ScreenId screenId, LevelCheckerModel *&model, QObject *parent = nullptr);

    /**
     * @brief デストラクタ
     */
    virtual ~AutoSettingViewModel();

    /**
     * @brief 画面を開くときに呼び出す表示内容の復元処理
     */
    virtual void activate(void) override;

    /**
     * @brief 画面を閉じるときに呼び出す表示な用の保存処理
     */
    virtual void deactivate(void) override;

    /**
     * @brief 画面を閉じるときに呼び出すスレッドの停止処理
     */
    virtual void suspend(void) override;

    /**
     * @brief 遷移先画面で参照する遷移前画面のデータ設定
     */
    virtual void setTakeOver() override;

    void createConnection(QObject *qml, QObject *mainFrame) override;

signals:
    void startChannelScanning(LC70_MEAS_MULTI_T *);
    void requestSetTakeOver(QVariant, QVariant);

    /**
     * @brief プログレスバーの進捗の値を通知
     * @param value 進捗の値
     */
    void progress(double value);

    /**
     * @brief 再スキャン要求
     */
    void requestToReScan(void);

    /**
     * @brief スキャン結果保存確認ダイアログを表示する
     * @note コンテキストを切るためにシグナル/スロットで実装する
     */
    void requestToShowRegistScanResultPopup();

private slots:
    virtual void onRequest(int condition) override;

    /**
     * @brief スキャンの実行依頼を受信
     * @note 呼び出し画面でスキャン処理(開始、結果取得、中止)を実施
     */
    void onRequestToScan(void);

    /**
     * @brief スキャン中止依頼を受信
     */
    void onRequestToStopScanning(void);

    /**
     * @brief スキャン結果を受信
     * @param index スキャン対象のチャンネルリストのインデックス
     * @param result スキャン結果
     */
    void onScanningChannelResult(int index, LC70_MEAS_MULTI_RESULT_T result);

    /**
     * @brief スキャン終了通知を受信
     * @param complete 完了有無
     */
    void onScanningFinished(bool complete);

    /**
     * @brief スキャン確認ダイアログを表示する
     */
    void onRequestScanConfirm();

    /**
     * @brief 再スキャン要求受信
     */
    void onRequestToReScan(void);

    /**
     * @brief 給電異常検知
     * @param message 表示メッセージ
     */
    void detectPowerSupplyError(QString message);

    /**
     * @brief スキャン結果保存確認ダイアログを表示する
     * @note コンテキストを１度切るためにシグナル/スロットで実装
     */
    void onRequestToShowRegistScanResultPopup(void);

protected:
    /**
     * @brief 押下ボタンを受信
     * @param id ...押下ボタンのID
     */
    virtual void onKeyStatus(ButtonId id) override;

    virtual void onButtonPress(ButtonId id) override;

protected slots:
    void onAccept(PopupId popupId, bool okOnly) override;
    void onReject(PopupId popupId) override;

    /**
     * @brief ポップアップから選択の通知を受信
     * @param index 選択されたインデックス
     */
    virtual void onSelect(PopupId popupId, int index) override;

    /**
     * @brief バッテリ低下状態変化通知
     * @param isLowBattery バッテリ低下状態かどうか
     */
    virtual void onLowBatteryStatusChanged(bool isLowBattery) override;

private:
    /**
     * @brief カーソル移動
     * @param buttonId 押下ボタンのID
     */
    void moveCursor(ButtonId buttonId);

    /**
     * @brief 画面名設定
     */
    void setScreenName();

    /**
     * @brief ActiveなBANDIDを取得する
     *        QML側で覚える処理が画面遷移が発生しないと動作しないので
     *        直接設定するメソッドを用意する
     *
     * @return バンドID
     */
    int getActiveBandId();

    /**
     * @brief スキャン結果後の動作選択
     * @param index 選択結果
     */
    void selectReactionScanResult(const int index);

private:
    QString         _selectedMeasurement;
    ButtonOperation _buttonOperation;
    QString         _activeButton;
    static const int BUTTON_COLUMN_NUMBER;
    static const int BUTTON_ROW_NUMBER;
    //・static QVector<QVector<QString>> _bandButtonData;

    int _bandId;    //< 設定されたバンドID
    QList<ChannelDefinitionItem> _channelList;      //!< チャンネル定義情報のリスト
    QVariant _broadBandModulationSetting;           //!< マニュアル測定 放送方式, 変調方式選択
    LC70_POWER_SUPPLY_T _powerSupply;               //!< 給電状態
    int _scanConditionModulationValue;              //!< スキャンする際に設定した変調方式の値
};

#endif // AUTOSETTINGVIEWMODEL_H
