/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include "fileselectionviewmodel.h"
#include "correcter.h"

const char *FileSelectionViewModel::EMPTY_VALUE = "-";

const int FileSelectionViewModel::USER_LIST_COLUMN_NUMBER = 7;
const int FileSelectionViewModel::USER_LIST_ROW_NUMBER = 1;

const int FileSelectionViewModel::CHANNEL_LIST_COLUMN_NUMBER = 6;
const int FileSelectionViewModel::CHANNEL_LIST_ROW_NUMBER = 1;

const int FileSelectionViewModel::ACTION_COLUMN_NUMBER = 1;
const int FileSelectionViewModel::ACTION_ROW_NUMBER = 1;

const QString FileSelectionViewModel::CANNOT_SELECTION_MESSAGE = "- 選択不可 -";

FileSelectionViewModel::FileSelectionViewModel(ScreenId screenId, LevelCheckerModel *&model, QObject *parent)
    : AbstractViewModel(screenId, model, parent),
      _fileSaveListCurrentPage(1),
      _displayInfoCurrentPage(1),
      _filemixState(0),
      _selectArea(USERLIST_AREA)
{
    setObjectName("fileSelectionViewModel");

    // ユーザリスト
    _dataLoggerFileListButtonData = {
        {"userdata0"},
        {"userdata1"},
        {"userdata2"},
        {"userdata3"},
        {"userdata4"},
        {"userdata5"},
        {"userdata6"},
    };

    _fileSaveListButtonOperation.setButtonName(
        USER_LIST_COLUMN_NUMBER,
        USER_LIST_ROW_NUMBER,
        _dataLoggerFileListButtonData);
    _fileSaveListButtonOperation.setActiveObject(_dataLoggerFileListButtonData[0][0]);
    _fileSaveListActiveButton = _dataLoggerFileListButtonData[0][0];

    // アクションボタン
    _dataLoggerActionButtonData = {
        {"execute1"},
    };

    _actionButtonOperation.setButtonName(
        ACTION_COLUMN_NUMBER,
        ACTION_ROW_NUMBER,
        _dataLoggerActionButtonData);

    _actionButtonOperation.setActiveObject(_dataLoggerActionButtonData[0][0]);
    _actionButtonActiveButton = _dataLoggerActionButtonData[0][0];
}

FileSelectionViewModel::~FileSelectionViewModel()
{
}

void FileSelectionViewModel::activate()
{
    if (nullptr == _view) {
        qErrnoWarning("_view is null");
        return;
    }

    connect(this, SIGNAL(requestSetTakeOver(QVariant, QVariant)), _view, SLOT(backup(QVariant, QVariant)));

    // 動作種別を取得する
    QObject *takeOverData = nullptr;
    if (_view) {
        takeOverData = _view->findChild<QObject *>("takeOverData");
    }
    if (takeOverData) {
        QVariant val;

        val = _takeOver["filemixState"];
        _filemixState = val.toInt();
    }


    // ページ数の初期化
    _fileSaveListCurrentPage = 1;
    _displayInfoCurrentPage = 1;

    // ユーザリストの初期表示
    _fileSaveListButtonOperation.setActiveObject(_dataLoggerFileListButtonData[0][0]);
    _fileSaveListActiveButton = _dataLoggerFileListButtonData[0][0];

    showUserFileList();
    showUserSettingList();
}

void FileSelectionViewModel::showUserFileList()
{
    QVariant value;

    QObject *target = nullptr;

    if (_model->getValue(VALUE_USERSETTING_FILELIST, value)) {
        QVariantList userlist = value.toList();
        int userlistSize = userlist.size();

        for (int i = 0; i < USER_LIST_COLUMN_NUMBER; i++) {
            target = nullptr;
            if (_view) {
                target = _view->findChild<QObject *>(_dataLoggerFileListButtonData[i][0]);
            }

            int listIndex = i + (_fileSaveListCurrentPage - 1) * USER_LIST_COLUMN_NUMBER;

            if (target) {
                target->setProperty("disable", false);

                if (listIndex < userlistSize) {
                    QString filename = userlist[listIndex].toList().at(2).toString();
                    target->setProperty("listItemNumber", QString::number(listIndex + 1));

                    if (filename.size() > 0) {
                        target->setProperty("message", filename);
                    } else {
                        target->setProperty("message", "");
                    }
                } else {
                    // ファイルが存在しない
                    target->setProperty("listItemNumer", QString::number(listIndex + 1));
                    target->setProperty("disable", true);
                    target->setProperty("message", CANNOT_SELECTION_MESSAGE);
                }
            }
        }
    }

    target = nullptr;
    if (_view) {
        target = _view->findChild<QObject *>("userfilelistpage");
    }

    if (target) {
        QString page;
        page.sprintf("ファイル保存先 %2d/20", _fileSaveListCurrentPage);
        target->setProperty("text", page);
    }

    // ユーザファイルリストの選択状態設定
    target = nullptr;
    if (_view) {
        target = _view->findChild<QObject *>(_fileSaveListActiveButton);
    }
    if (target) {
        target->setProperty("active", true);

        QString itemNumber = target->property("listItemNumber").toString();
        QString fileName = target->property("message").toString();


        // 選択中のfileListが- 選択不可 -の場合 決定ボタンはdisableとする。
        target = nullptr;
        if (_view) {
            target = _view->findChild<QObject *>("execute1");
        }
        if (target) {
            if (fileName != CANNOT_SELECTION_MESSAGE) {
                target->setProperty("disable", false);
            } else {
                target->setProperty("disable", true);
            }
        }

        int index = getSelectedUserListIndex();

        _model->getValue(VALUE_USERSETTING_FILELIST, value);
        QVariantList userList = value.toList();

        if (index >= userList.size()) {
            // [Memo]
            //  選択リストがuserListSizeを超えていた場合(- 選択不可 - の場合)
            //  USERファイルリストの詳細全てを「-」で表示する。
            showEmptyUserSettingList();

            target = nullptr;
            if (_view) {
                target = _view->findChild<QObject *>("channelListPage");
            }
            if (target) {
                target->setProperty("text", "");
            }
        } else {
            QString noFilename = itemNumber + " " + fileName;
            // 選択されているユーザのデータ読み込み
            getUserSettingList();

            // 対応するチャンネル情報の初期表示
            int listNum = showUserSettingList();

            target = nullptr;
            if (_view) {
                target = _view->findChild<QObject *>("channelListPage");
            }
            if (target) {
                if ((fileName.length() > 0 ) && (fileName != CANNOT_SELECTION_MESSAGE)) {
                    int maxPage = listNum / 6;

                    if ((listNum % 6) != 0) {
                        maxPage++;
                    }

                    QString title;
                    title.sprintf(" に登録されているCH：1/%d ページ", maxPage);
                    target->setProperty("text", noFilename + title);
                } else {
                    target->setProperty("text", "");
                }
            }
        }
    }
}

void FileSelectionViewModel::setTakeOver()
{
}

void FileSelectionViewModel::onRequest(int condition)
{
    emit request(_screenId, condition); // 0 origin のため
    return;
}

void FileSelectionViewModel::onKeyStatusUserListArea(ButtonId id)
{
    // EnterKey の動作はなし
    // 上下移動でユーザ設定データの表示切替を行う
    // パフォーマンス次第でEnterKeyで読み込みに変更かも

    moveCursorUserListArea(id);
}

void FileSelectionViewModel::moveCursorUserListArea(ButtonId id)
{
    if ((id == BUTTON_UP) || (id == BUTTON_DOWN)) {
        // 選択している USERLIST の切替

        QString nextBtn;
        bool ret = _fileSaveListButtonOperation.nextActiveObject(id, nextBtn);

        if (ret) {

            // inactive
            QObject *target = nullptr;
            if (_view) {
                target = _view->findChild<QObject *>(_fileSaveListActiveButton);
            }
            if (target) {
                target->setProperty("active", false);
            }

            // active
            target = nullptr;
            if (_view) {
                target = _view->findChild<QObject *>(nextBtn);
            }
            if (target) {
                target->setProperty("active", true);
            }

            _fileSaveListActiveButton = nextBtn;
            _fileSaveListButtonOperation.setActiveObject(_fileSaveListActiveButton);


            // ページ切替の確認
            switch (id) {
            case BUTTON_UP:
                if (nextBtn == _dataLoggerFileListButtonData[USER_LIST_COLUMN_NUMBER - 1][0]) {

                    const int maxPage = 20;

                    // 前ページに遷移
                    if (_fileSaveListCurrentPage == 1) {
                        _fileSaveListCurrentPage = maxPage;
                    } else {
                        _fileSaveListCurrentPage--;
                    }

                }

                break;

            case BUTTON_DOWN:
                if (nextBtn == _dataLoggerFileListButtonData[0][0]) {
                    const int maxPage = 20;

                    // 次ページに遷移
                    if (_fileSaveListCurrentPage == maxPage) {
                        _fileSaveListCurrentPage = 1;
                    } else {
                        _fileSaveListCurrentPage++;
                    }
                }

                break;

            default:
                break;
            }

            showUserFileList();
        }
    } else if (id == BUTTON_RIGHT) {
        // ファイル保存先のメッセージが- 選択不可 -以外の場合は決定ボタンに対して移動可能
        QObject *target = nullptr;
        if (_view) {
            target = _view->findChild<QObject *>(_fileSaveListActiveButton);
        }
        if (target) {
            QString fileName = target->property("message").toString();

            if (fileName != CANNOT_SELECTION_MESSAGE) {
                // 選択状態を表示
                _reserveUserListActivebg = target->property("activebg").toString();
                target->setProperty("activebg", "fileselection_userlist_btn_edit.png");

                _selectArea = ACTION_AREA;
                target = nullptr;
                if (_view) {
                    target = _view->findChild<QObject *>(_actionButtonActiveButton);
                }
                if (target) {
                    target->setProperty("active", true);
                }
            }
        }
    }
}

void FileSelectionViewModel::onKeyStatusActionButtonArea(ButtonId id)
{
    if (id == BUTTON_ENTER) {
        QObject *target = nullptr;
        if (_view) {
            target = _view->findChild<QObject *>(_actionButtonActiveButton);
        }
        if (target == nullptr) {
            return;
        }

        QVariant value;
        _model->getValue(VALUE_USERSETTING_FILELIST, value);

        QVariantList userFileList = value.toList();

        // 保存先インデックスを決める
        // 選択中のユーザリストのindex
        int index = getSelectedUserListIndex();

        // 0 は未設定あつかいとするため、1 ～ 設定
        index++;

        // 保存先のインデックスがUserListのサイズ範囲外の場合以降の処理は実行しない
        if (index > userFileList.size()) {
            _model->writeLog(QString("USER設定・ファイル保存先選択"),
                             QString("保存先インデックス範囲外"));
            return;
        }

        // 選択リストインデックスの保持
        emit requestSetTakeOver("fileselectionListIndex", index - 1);

        //　合成の場合 status 更新
        if (_filemixState == 1) {
            emit requestSetTakeOver("filemixState", 2);
        }

        moveCursorActionButtonArea(BUTTON_LEFT);

        emit requestSetTakeOver("userDataloggerFunction", 2);

        onRequest(0);
    } else {
        moveCursorActionButtonArea(id);
    }
}

void FileSelectionViewModel::moveCursorActionButtonArea(ButtonId id)
{
    if (id == BUTTON_LEFT) {
        QObject *target = nullptr;
        if (_view) {
            target = _view->findChild<QObject *>(_actionButtonActiveButton);
        }
        if (target) {
            target->setProperty("active", false);
        }

        target = nullptr;
        if (_view) {
            target = _view->findChild<QObject *>(_fileSaveListActiveButton);
        }
        if (target) {
            target->setProperty("active", true);
            target->setProperty("activebg", _reserveUserListActivebg);
        }

        _selectArea = USERLIST_AREA;
    }
}

void FileSelectionViewModel::getUserSettingList()
{
    // 選択中のユーザリストのindex
    int index = getSelectedUserListIndex();

    QVariant value;
    _model->getValue(VALUE_USERSETTING_FILELIST, value);

    QVariantList userList = value.toList();

    if (index < userList.size()) {
        QVariantList userData = userList[index].toList();

        // ファイル取得
        QString filename = userData.at(2).toString();

#if !Q_PROCESSOR_ARM

        if (!filename.contains(".csv")) {
            filename += ".csv";
        }

#endif
        _model->importDataLoggerUserSettingList(VALUE_USERSETTING_SETTINGLIST, filename);
    }
}

int FileSelectionViewModel::showUserSettingList()
{
    // チャンネル情報取得
    QVariant value;
    _model->getValue(VALUE_USERSETTING_SETTINGLIST, value);
    QVariantList userSettingList = value.toList();

    // 1.BAND種別
    // 2.放送規格
    // 3.チャンネル番号
    // 4.中心周波数
    // 5.変調方式
    // 6.局発周波数

    QVariantMap userSettingData;

    bool dataEmpty = false;

    for (int i = (_displayInfoCurrentPage - 1) * 6; i < (_displayInfoCurrentPage * 6); i++) {
        if (userSettingList.size() <= i) {
            // 当該行のデータが存在しない
            dataEmpty = true;
        } else {
            dataEmpty = false;
            userSettingData = userSettingList[i].toMap();

            if (userSettingData["enable"] == false) {
                //データが存在していない行や削除行
                dataEmpty = true;
            }
        }

        QString listItem = "list" + QString::number(i % 6);
        QObject *target = nullptr;
        if (_view) {
            target = _view->findChild<QObject *>(listItem);
        }
        if (target) {
            // 保存先選択画面用にリサイズ
            target->setProperty("bandWidth", 78);
            target->setProperty("broadbandSystemWidth", 78);
            target->setProperty("channelWidth", 78);
            target->setProperty("modulationWidth", 78);
            target->setProperty("leveloffsetWidth", 78);
            target->setProperty("mercnSwitchWidth", 0);
            target->setProperty("berSwitchWidth", 0);
            target->setProperty("powerSupplyWidth", 0);
            target->setProperty("trashWidth", 0);

            target->setProperty("no", i + 1);   // No.1 から開始
            target->setProperty("band", dataEmpty ? EMPTY_VALUE :  Correcter::displayBandName(userSettingData["band"].toInt()));
            target->setProperty("broadbandSystem",
                                dataEmpty ? EMPTY_VALUE : Correcter::displayBroadbandSystem(userSettingData["broadbandSystem"].toUInt()));
            target->setProperty("channel", dataEmpty ? EMPTY_VALUE : userSettingData["channel"]);
            target->setProperty("modulation",
                                dataEmpty ? EMPTY_VALUE : Correcter::displayModulation(userSettingData["modulationBit"].toUInt()));

            // 局発周波数
            if (static_cast<double>(userSettingData["localFreq"].toInt() != QVariant(0))) {
                // 局発周波数の値あり
                double leveloffset = static_cast<double>(userSettingData["localFreq"].toUInt()) / float(1000000);
                target->setProperty("leveloffset", dataEmpty ? EMPTY_VALUE : QString::number(leveloffset));
            } else {
                target->setProperty("leveloffset", EMPTY_VALUE);
            }
        }
    }

    return userSettingList.size();
}

void FileSelectionViewModel::showEmptyUserSettingList()
{
    for (int i = 0; i < 6; i++) {
        QString listItem = "list" + QString::number(i % 6);
        QObject *target = nullptr;
        if (_view) {
            target = _view->findChild<QObject *>(listItem);
        }
        if (target) {
            target->setProperty("no", i + 1);   // No.1 から開始
            target->setProperty("band", EMPTY_VALUE);
            target->setProperty("broadbandSystem", EMPTY_VALUE);
            target->setProperty("channel", EMPTY_VALUE);
            target->setProperty("modulation", EMPTY_VALUE);
            target->setProperty("leveloffset", EMPTY_VALUE);
            target->setProperty("isTrashVisible", false);
        }
    }
}

void FileSelectionViewModel::onKeyStatus(ButtonId id)
{
    if (id == BUTTON_ESCAPE || id == BUTTON_HOME || id == BUTTON_USER || id == BUTTON_MENU) {
        switch (id) {
        case BUTTON_HOME:
            emit requestDirect(SCREEN_HOME, true);
            break;

        case BUTTON_ESCAPE:
            // [Memo]
            //  ファイル合成画面に遷移するように filemixStateを1(FILEMIX_BEFORE)に設定する
            //  画面共有情報を変更する
            emit requestSetTakeOver("filemixState", 1);
            emit back();
            break;

        case BUTTON_USER:
#ifdef NEW_USER_SETTING
        emit requestDirect(SCREEN_USER_SETTING, true);
#else
        emit requestDirect(SCREEN_USER_DATA_LOGGER_MENU, true);
#endif
            break;

        default:
            break;
        }
    } else {
        switch (_selectArea) {
        case USERLIST_AREA:
            onKeyStatusUserListArea(id);
            break;

        case ACTION_AREA:
            onKeyStatusActionButtonArea(id);
            break;

        default:
            break;
        }
    }
}

void FileSelectionViewModel::onButtonPress(ButtonId id)
{
    if (id == BUTTON_ENTER) {
        QObject *target = nullptr;

        switch (_selectArea) {
        case USERLIST_AREA:
            // nothing to do
            break;

        case ACTION_AREA:
            target = nullptr;
            if (_view) {
                target = _view->findChild<QObject *>(_actionButtonActiveButton);
            }
            if (target) {
                target->setProperty("press", true);
            }

            break;

        default:
            break;
        }
    }
}

int FileSelectionViewModel::getSelectedUserListIndex()
{
    return _fileSaveListActiveButton.mid(8).toInt() + (_fileSaveListCurrentPage - 1) * 7;
}
