/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include "userdataloggermenuviewmodel.h"

// 選択可能ボタン(設定呼び出しシングル、設定呼び出しマルチ、呼び出しデータロガー、USER設定ファイル作成) USB接続中
QVector<QVector<QString>> userDataLoggerMenuButtonDataWithUsb = {
    {{"m0"}, {"m1"}, {"m3"}, {"m2"}},
};

const int UserDataLoggerMenuViewModel::BUTTON_COLUMN_NUMBER = 1;
const int UserDataLoggerMenuViewModel::BUTTON_ROW_NUMBER = 4;

UserDataLoggerMenuViewModel::UserDataLoggerMenuViewModel(ScreenId screenId, LevelCheckerModel *&model, QObject *parent)
    : AbstractViewModel(screenId, model, parent)
{
    setObjectName("userDataLoggerMenuViewModel");
    _buttonOperation.setButtonName(BUTTON_COLUMN_NUMBER, BUTTON_ROW_NUMBER, userDataLoggerMenuButtonDataWithUsb);
    _buttonOperation.setActiveObject(userDataLoggerMenuButtonDataWithUsb[0][0]);
    _activeButton = userDataLoggerMenuButtonDataWithUsb[0][0];
}

UserDataLoggerMenuViewModel::~UserDataLoggerMenuViewModel()
{

}

void UserDataLoggerMenuViewModel::activate()
{
    if (nullptr == _view) {
        qErrnoWarning("_view is null");
        return;
    }

    // サブに対するウェイクアップ要求が行われていなければ行う
    if (!_model->wakeUpRequested()) {
        _model->setWakeUpRequested(true);
        _model->wakeUp(1);
    }

    connect(this, SIGNAL(requestSetTakeOver(QVariant, QVariant)), _view, SLOT(backup(QVariant, QVariant)));
    connect(this, SIGNAL(removeAttribute(QString)), _mainFrame, SLOT(onRemoveAttribute(QString)));

    // [Memo]
    //  ユーザデータロガーメニュー画面は、ユーザボタンを押下した際にも遷移することが可能である
    //  ユーザデータロガーメニュー押下時は、メニューアイコンの３つ目以降のアイコンをクリアする
    _model->removeMenuList(MenuListIconIndex::BAND_ICON);         // バンドアイコンをクリア
    _model->removeMenuList(MenuListIconIndex::SCAN_ICON);         // スキャンアイコンをクリア
    _model->removeMenuList(MenuListIconIndex::POWER_SUPPLY_ICON); // 給電アイコンをクリア
    _model->replaceMenuList(MenuListIconIndex::FUNCTION_ICON, "icon_menu_user.png");

    // [Memo]
    // USERボタン押下時し、USERデータロガーメニューに遷移した場合、
    // 画面はHOME画面を経由していないため、
    // 画面共有情報が残り続けてしまうため不要な画面共有情報はクリアを実施する。
    emit removeAttribute("selectedMeasurement");
    emit removeAttribute("selectedBand");
    emit removeAttribute("selectedModulation");
    emit removeAttribute("selectedLocalOscillator");
    emit removeAttribute("selectedPolarization");
    emit removeAttribute("dataloggerFunction");
    emit removeAttribute("filemixFileName1");
    emit removeAttribute("filemixFileName2");
    emit removeAttribute("selectedUserSetting");
    emit removeAttribute("fileselectionListIndex");
    emit removeAttribute("filemixState");

    // 選択中のアイコンの設定
    setBoolProperty(_activeButton, "active", true);
}

void UserDataLoggerMenuViewModel::deactivate()
{
    AbstractViewModel::deactivate();
    disconnect(this, SIGNAL(requestSetTakeOver(QVariant, QVariant)), _view, SLOT(backup(QVariant, QVariant)));
    disconnect(this, SIGNAL(removeAttribute(QString)), _mainFrame, SLOT(onRemoveAttribute(QString)));
}

void UserDataLoggerMenuViewModel::setTakeOver()
{
    if (_activeButton == "m0") {
        emit requestSetTakeOver("dataloggerFunction", 1);
    } else if (_activeButton == "m1") {
        emit requestSetTakeOver("dataloggerFunction", 2);
    } else if (_activeButton == "m2") {
        emit requestSetTakeOver("dataloggerFunction", 3);
    } else if (_activeButton == "m3") {
        emit requestSetTakeOver("dataloggerFunction", 4);
    }
}

void UserDataLoggerMenuViewModel::onRequest(int condition)
{
    emit request(_screenId, condition); // 0 origin のため
    return;
}

void UserDataLoggerMenuViewModel::moveCursor(ButtonId id)
{
    if ((id == BUTTON_UP) || (id == BUTTON_DOWN) || (id == BUTTON_RIGHT) || (id == BUTTON_LEFT)) {

        QString nextBtn;
        bool ret = _buttonOperation.nextActiveObject(id, nextBtn);

        if (ret) {
            // inactive
            setBoolProperty(_activeButton, "active", false);

            // active
            setBoolProperty(nextBtn, "active", true);

            _activeButton = nextBtn;
            _buttonOperation.setActiveObject(_activeButton);
        }
    }
}

void UserDataLoggerMenuViewModel::onKeyStatus(ButtonId id)
{
    AbstractViewModel::onKeyStatus(id);

    if (id == BUTTON_ENTER) {
        if (isErrorUsbStatus()) {
            return;
        }
        int button_index = _activeButton.mid(1).toInt(); // 0 orign のため
        onRequest(button_index);
    } else {
        moveCursor(id);
    }
}

void UserDataLoggerMenuViewModel::onButtonPress(ButtonId id)
{
    if (id == BUTTON_ENTER) {
        setBoolProperty(_activeButton, "press", true);
    }
}

bool UserDataLoggerMenuViewModel::isErrorUsbStatus()
{
    QMap<QString, QList<LC70_STRAGE_T>> enableUsbStatusMap = {
        {"m3", {LC70_STRAGE_FORMAT}},
    };

    if (enableUsbStatusMap.count(_activeButton) == 0) {
        // USBに無関係のボタン
        return false;
    }

    if (enableUsbStatusMap.value(_activeButton).contains(_model->getDeviceStatus().usba)) {
        // 実行可能なUSB状態になっている
        return false;
    }

    // ボタンごとに定義している、押下可能なUSB状態になっていなければエラー表示
    setBoolProperty(_activeButton, "press", false);
    emit requestToShowPopup(POPUP_FAILURE_NO_USB_CONFIRMATION); // USB未装着エラー
    return true;
}

void UserDataLoggerMenuViewModel::setBoolProperty(QString objectName, QString propertyName, bool value)
{
    QObject *target = nullptr;
    if (_view) {
        target = _view->findChild<QObject *>(objectName);
    }
    if (target) {
        target->setProperty(propertyName.toUtf8().constData(), value);
    }
}
