/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef USERDATALOGGERSETTING_H
#define USERDATALOGGERSETTING_H

#include "abstractviewmodel.h"
#include "bandmodel.h"
#include "buttonoperation.h"

/**
 * @brief ユーザー設定/データロガー設定編集画面
 */
class UserDataLoggerSettingViewModel : public AbstractViewModel
{
    Q_OBJECT
public:
    /**
     * @brief コンストラクター
     * @param screenId 画面 ID
     * @param model Model クラスのポインター
     * @param parent 親オブジェクト
     */
    explicit UserDataLoggerSettingViewModel(ScreenId screenId, LevelCheckerModel *&model, QObject *parent = nullptr);

    /**
     * @brief デストラクタ
     */
    virtual ~UserDataLoggerSettingViewModel();

    /**
     * @brief 画面を開くときに呼び出すスレッドの開始、再開処理
     */
    virtual void resume(void) override;

    /**
     * @brief 画面を閉じるときに呼び出すスレッドの停止処理
     */
    virtual void suspend(void) override;

    /**
     * @brief 画面を開くときに呼び出す qml との接続処理
     * @param qml qml のオブジェクトのポインター
     * @param mainFrame MainFrameViewModel のポインター
     */
    virtual void createConnection(QObject *qml, QObject *mainFrame) override;

    /**
     * @brief 画面を開くときに呼び出す表示内容の復元処理
     */
    virtual void activate(void) override;

    /**
     * @brief 画面を閉じるときに呼び出す表示な用の保存処理
     */
    virtual void deactivate(void) override;

    /**
     * @brief 遷移先画面で参照する遷移前画面のデータ設定
     */
    virtual void setTakeOver();

signals:
    /**
     * @brief 画面共通情報設定要求
     * @param key 画面共通情報キー
     * @param value 画面共通情報データ
     */
    void requestSetTakeOver(QVariant, QVariant);

protected:
    /**
     * @brief USER データロガー設定チャンネル編集モードを返す
     * @return USER データロガー設定チャンネル編集モード
     */
    ChannelEditMode mode(void) const;

protected slots:
    /**
     * @brief ボタン押下の状態を受信
     * @param id ボタン ID
     */
    virtual void onKeyStatus(ButtonId id) override;

    /**
     * @brief ボタン押下を受信
     * @param id ボタン ID
     */
    virtual void onButtonPress(ButtonId id) override;

    /**
     * @brief ポップアップから受諾の通知を受信
     * @param okOnly OK のみの場合 true
     */
    virtual void onAccept(PopupId popupId, bool okOnly) override;

    /**
     * @brief ポップアップから拒否通知を受信
     * @param popupId 表示中ポップアップのID
     */
    virtual void onReject(PopupId popupId) override;

    /**
     * @brief ポップアップから選択の通知を受信
     * @param index 選択されたインデックス
     */
    virtual void onSelect(PopupId popupId, int index) override;

private slots:
    /**
     * @brief onRequest
     * @param condition
     */
    virtual void onRequest(int condition) override;

    /**
     * @brief 入力されたファイル名を受信
     * @param popupId ポップアップの種類
     * @param saveLocation 保存先
     * @param fileName 入力されたファイル名
     */
    void onInputFileName(PopupId popupId, SaveLocation saveLocation, QString fileName);

    /**
     * @brief USB挿抜変更通知を受信する
     * @param usbStatus USB挿抜状態 LC70_STRAGE_Tの値
     */
    void onChangeUsbInsertionRemoval(int usbStatus);

private:
    /**
     * @brief 操作対象のエリア定義
     */
    enum SelectArea {
        FILELIST_AREA = 1,  // 設定ファイルリスト
        CHANNEL_AREA  = 2,  // チャンネルリスト
        CHANNEL_EDIT  = 3,  // 設定値編集
        TARGET_AREA   = 4,  // 測定 ON/OFF リスト
        TARGET_EDIT   = 5,  // 測定 ON/OFF 編集
        ACTION_AREA   = 6,  // 動作ボタン部
        POPUP_AREA    = 7,  // 独自POPUP部
        TARGET_ALL      = 8,  // 測定 全てON/OFF
        TARGET_ALL_EDIT = 9,  // 測定 全てON/OFF 編集
    };

    /**
     * @brief データロガー内ポップアップ種別
     */
    enum DataloggerPopupType {
        TYPEUNKNOWN = (-1),     //!< 未定
        COLLECTCONFIRM = 0,     //!< 収集確認
        DATETIMESET  = 1,       //!< 取得日時期間設定
        CORRECTINTERVALSET = 2, //!< 取得間隔設定
        WAITING = 3,            //!< 収集待機
        CANCELCONFIRM = 4,      //!< 中止確認
        NODATA = 5              //!< データなし
    };

    /**
     * @brief データロガー内ポップアップ編集状態
     */
    enum PopupEditState {
        SELECT = 0,             //!< 選択
        EDIT = 1                //!< 編集
    };

    /**
     * @brief USER 設定のふるまい
     */
    enum UserSettingBehavior {
        ALLOW_SINGLE_BAND_ONLY = 0,     //!< 単一 Band のみ
        ALLOW_MULTIPLE_BANDS            //!< 複数 Band 設定可能
    };

    ChannelEditMode _mode;          //!< USER データロガー設定チャンネル編集モード
    SelectArea _selectArea;         //!< 選択している操作対象エリア
    QStringList _fileList;          //!< 設定ファイルのリスト
    int _currentFileListPage;       //!< 設定ファイルのリストの現在のページ
    int _currentFileIndex;          //!< 設定ファイルのリスト中の選択中のファイルのインデックス(0〜9まで)
    BandModel *_bandModel;          //!< Band タブのモデル
    int _currentTabIndex;           //!< 現在選択されている Band タブのインデックス
    QVariantList _channelList;      //!< 現在選択されている Band の測定内容に表示するデータ
    int _currentChannelIndex;       //!< チャンネルリスト中の選択中のチャンネルのインデックス(0〜4まで)
    int _currentEditPositon;        //!< 編集項目のリスト行での位置
    QVariantList _editItemList;     //!< 編集項目のリスト
    int _currentTargetIndex;        //!< 測定 ON/OFF リスト中の選択中のチャンネルのインデックス(0〜4まで)
    QString _temporaryFileName;     //!< 仮のファイル名
    int _currentActionIndex;        //!< 選択中の動作ボタン
    QString _oldFileName;           //!< Band 変更前のファイル名
    DataloggerPopupType _popupType; //!< データロガー内ポップアップ種別
    PopupEditState _popupState;     //!< データロガー内ポップアップ編集状態
    PopupEditState _popupRsvState;  //!< データロガー内ポップアップ編集状態
    int _destinationIndex;          //!< 同一周波数のチャンネルのインデックス
    int _destinationTab;            //!< 同一周波数のチャンネルがあるタブのインデックス
    UserSettingBehavior _userSetingBehavior;    //!< USER 設定のふるまい

    /**
     * @brief 設定ファイルのリストを表示する
     */
    void showSettingFileList(void);

#if 0
    /**
     * @brief チャンネルリストを表示する
     * @param noReadSettingFile 設定ファイルを読み込み反映しない
     * @note 通常は false だが既存データで Band を変更する場合は true
     * @note 旧設計
     */
    void showChannelList(bool noReadSettingFile = false);
#endif

    /**
     * @brief チャンネルリストを表示する
     * @param fileRead ファイルの読み込み有無
     * @note 新設計対応
     */
    void showChannelList(bool fileRead);

    /**
     * @brief 設定ファイルの有無
     */
    bool hasSettingFile(void);

    /**
     * @brief 設定ファイルの読み込み
     */
    void readSettingFile(void);

    /**
     * @brief ファイルリストエリアでのボタン操作
     * @param id ボタン ID
     */
    void keyStatusFileListArea(ButtonId id);

    /**
     * @brief ファイルリストエリアでのカーソル移動
     * @param id ボタン ID
     */
    void moveCursorFileListArea(ButtonId id);

    /**
     * @brief チャンネルリストエリアでのボタン操作
     * @param id ボタン ID
     */
    void keyStatusChannelListArea(ButtonId id);

    /**
     * @brief チャンネルリストエリアでのカーソル移動
     * @param id ボタン ID
     */
    void moveCursorChannelListArea(ButtonId id);

    /**
     * @brief チャンネル編集中のボタン操作
     * @param id ボタン ID
     */
    void keyStatusChannelEdit(ButtonId id);

    /**
     * @brief 測定 ON/OFF リストエリアでのカーソル移動
     * @param id ボタン ID
     */
    void keyStatusTargetListArea(ButtonId id);

    /**
     * @brief 測定 ON/OFF リストエリアでのカーソル移動
     * @param id ボタン ID
     */
    void moveCursorTargetListArea(ButtonId id);

    /**
     * @brief 測定 ON/OFF 編集中のボタン操作
     * @param id ボタン ID
     */
    void keyStatusTargetEdit(ButtonId id);

    /**
     * @brief 測定 全てON/OFF でのカーソル移動
     * @param id ボタン ID
     */
    void keyStatusTargetAllArea(ButtonId id);

    /**
     * @brief 測定 全てON/OFF エリアでのカーソル移動
     * @param id ボタン ID
     */
    void moveCursorTargetAllArea(ButtonId id);

    /**
     * @brief 測定 全てON/OFF 編集中のボタン操作
     * @param id ボタン ID
     */
    void keyStatusTargetAllEdit(ButtonId id);

    /**
     * @brief 測定 全てON/OFF 項目の編集
     */
    void targetAllEditItem(ButtonId id);

    /**
     * @brief 全チャンネルのユーザ設定の更新
     */
    void allUpdateSetting();

    /**
     * @brief 動作ボタンエリアでのボタン操作
     * @param id ボタン ID
     */
    void keyStatusActionArea(ButtonId id);

    /**
     * @brief 動作ボタンエリアでのカーソル移動
     * @param id ボタン ID
     */
    void moveCursorActionArea(ButtonId id);

    /**
     * @brief バンド切替
     * @param forward true: 右(後方)に移動 false: 左(前方)に移動
     */
    void changeBand(bool forward);

    /**
     * @brief タブ切り替え
     */
    void switchTab(void);

    /**
     * @brief エリア移動
     * @param destination 移動先エリア
     */
    void moveArea(SelectArea destination);

    /**
     * @brief 次の編集項目に移動
     * @return true:CHANNEL_AREA外の移動
     *         false:CHANNEL_AREA内の移動
     */
    bool moveNextEditItem();

    /**
     * @brief 前の編集項目に移動
     * @return true:CHANNEL_AREA外の移動
     *         false:CHANNEL_AREA内の移動
     */
    bool movePreviousEditItem();

    /**
     * @brief 編集項目リストを作成
     */
    void setEditItemList();

    /**
     * @brief 項目の編集
     */
    void editItem(ButtonId id);

    /**
     * @brief 設定の更新
     */
    void updateSetting();

    /**
     * @brief POPUP_CANCEL_CONFIRMATION の onAccept 処理
     */
    void acceptPopupCancelConfirmation();

    /**
     * @brief 設定内容の保存を可決
     * @note POPUP_SAVE_CONFIRMATION の onAccept 処理
     */
    void acceptPopupSaveConfirmation();

    /**
     * @brief 設定内容を保存するファイル名の確定
     * @note POPUP_DATA_LOGGER_KEYBOARD の onAccept 処理
     */
    void acceptPopupDataLoggerKeyboard();

    /**
     * @brief 同一ファイル名に対してファイル名を再入力の確定
     * @note POPUP_CONFIRM_OVERWRITE_USER_LOGGER_DATA の onAccept 処理
     */
    void acceptPopupConfirmOverwriteUserLoggerData();

    /**
     * @brief 設定内容の削除を可決
     * @note POPUP_CONFIRM_DELETE_SELECTED_FILE の onAccept 処理
     */
    void acceptPopupDeleteConfirmation();

    /**
     * @brief ファイル一括削除を可決
     * @note POPUP_CONFIRM_ALL_DELETE_FILE の onAccept 処理
     */
    void acceptPopupAllDeleteConfirmation();

    /**
     * @brief 既存データを履きし、Band の変更を承諾
     * @note POPUP_CONFIRM_CHANGE_BAND の onAccept 処理
     */
    void acceptPopupConfirmChangeBand();

    /**
     * @brief オートパワーオフでデータロガー実行
     */
    void acceptPopupConfirmExecuteWithAutoPowerOff();

    /**
     * @brief 引数の名前と同じファイル名が保持されているかを調べる
     * @param fileList 設定ファイルリスト
     * @param fileName 設定ファイル名
     * @return 引数の名前と同じ名前のファイル名が設定ファイルリストに含まれているか (frue: 含まれている false: 含まれていない)
     */
    bool checkSameFileName(QVariantList fileList, QString fileName);

    /**
     * @brief 選択されている Band のチャンネル定義一覧を返す
     * @param settingFile[in] 設定ファイルの内容
     * @param channels[out] 選択されている Band のチャンネル定義一覧
     */
    void retrieveChannels(QList<UserDataLoggerSettingFile> settingFile, QList<ChannelDefinitionItem> &channels);

    /**
     * @brief デフォルトの放送方式(放送規格)を返す
     * @param band Band ID
     * @return デフォルトの放送方式(放送規格)
     */
    QString defaultBroadcastSystem(const LC70_BAND_T &band);

    /**
     * @brief 測定内容を設定
     * @param band 選択されている Band の Band ID
     * @param channelDefinitionList チャンネル定義リスト
     * @param settingFile 選択されている Band の設定ファイルの内容
     */
    void populateChannels(const LC70_BAND_T &band, const QList<ChannelDefinitionItem> & channelDefinitionList, QList<UserDataLoggerSettingFile> settingFile);

    /**
     * @brief ファイル名にファイルグループを示す文字を追加
     * @param fileName ファイル名
     * @return 編集後のファイル名
     */
    QString addGroupPrefixToFileName(const QString& fileName);

    /**
     * @brief 現在編集中のチャンネルリストに測定ONのチャンネルが有るかどうか
     * @return true: 測定ONのチャンネルあり false: 全てのチャンネルが測定OFF
     */
    bool hasActiveChannel(void);

    /**
     * @brief 測定 ON のチャンネル数を返す
     * @return 測定 ON のチャンネル数
     */
    unsigned int activeChannel(void);

    /**
     * @brief 測定開始できるかどうか
     * @return true: 測定開始できる false: 測定開始できない
     */
    bool canStartMeasure();

    /**
     * @brief 現在選択中の設定ファイルの上書き保存、または新規設定ファイルの作成ができるかどうか
     * @return true: ファイル保存できる false: ファイル保存できない
     */
    bool canSave();

    /**
     * @brief 現在選択中の設定ファイルの削除ができるかどうか
     * @return true: 削除できる false: 削除できない
     */
    bool canDelete();

    /**
     * @brief 一括削除ができるかどうか
     * @return true: 削除できる false: 削除できない
     */
    bool canAllDelete();

    /**
     * @brief 現在選択されている Band と異なるファイルグループのプリフィクスが付いているか否か
     * @param fileName ファイル名
     * @return true: 現在選択されている Band と異なるファイルグループのプリフィクスが付いている false: 現在選択されている Band と同じファイルグループのプリフィクスが付いている
     */
    bool isDifferentFileGroup(const QString &fileName);

    /**
     * @brief ファイルグループのプリフィクスを除いたファイル名を返す
     * @param fileName ファイル名
     * @return ファイルグループのプリフィクスを除いたファイル名
     */
    QString removeFileGroupPrefix(const QString &fileName);

    /**
     * @brief プリセットの測定内容を使用するBandかどうか
     * @param band
     * @return
     */
    bool isPresetChannelBand(const LC70_BAND_T &band);

    /**
     * @brief 画面表示モードに応じた設定ファイルリストのValueIdを取得
     * @return ValueId
     */
    ValueId fileListValueId();

    /**
     * @brief 画面表示モードに応じた設定ファイルのValueIdを取得
     * @return ValueId
     */
    ValueId settingListValueId();

    /**
     * @brief チャンネルの測定内容の初期値を返す
     * @param band Band
     * @param channel チャンネル定義
     * @param defaultBroadcastValue デフォルトの放送規格
     * @param localFrequencyValue デフォルトの局発周波数
     * @param defaultPowerSupplyValue デフォルトの給電情報
     * @return チャンネルの測定内容の初期値
     */
    QVariantList initializeChannelData(const LC70_BAND_T &band, const ChannelDefinitionItem &channel, const QString &defaultBroadcastValue, const unsigned int &defaultLocalFrequencyValue, const unsigned int &defaultPowerSupplyValue);

    /**
     * @brief スキャン結果から設定ファイルに出力する内容を作成する
     */
    void createScannedChannelSetting(void);

    /**
     * @brief 測定 ON の行数を返す
     * @return 測定 ON の行数
     */
    int validChannelNumber(void);

    /**
     * @brief 編集中のデータロガー設定のチャンネル数を返す
     * @return 編集中のデータロガー設定のチャンネル数
     */
    int unregisteredDataLoggerChannelCount(void);

    /**
     * @brief 設定ファイル中の Band とチャンネル名に対応する局発周波数を返す
     * @param band Band
     * @param channelName チャンネル名
     * @param settingFile 設定ファイル
     * @return 設定ファイル中の Band とチャンネル名に対応する局発周波数
     */
    uint32_t localFrequencyForEachChannelInSettingFile(const LC70_BAND_T &band, const QString &channelName, const QList<LC70_LOGGER_SET_LOGGER_ROW_T> &settingFile);

    /**
     * @brief 編集中の設定ファイルに指定したチャンネルが存在するか否かを返す
     * @param band Band
     * @param channel チャンネル名
     * @param userSettingList 編集中の設定ファイルの内容
     * @return 編集中の設定ファイルに指定したチャンネルが存在するか否か
     */
    bool hasChannel(const LC70_BAND_T &band, const QVariant &channel, const QVariantList &userSettingList);

    /**
     * @brief 測定 ON/OFF リストの行を選択状態にする
     */
    void selectTargetRow(void);

    /**
     * @brief 同一の周波数が入力されているかいないかを返す
     * @return -1: 該当なし 0〜: 同一周波数の行のインデックス
     */
    int checkInputSameFrequency(void);

    /**
     * @brief USER 設定の場合に複数の Band のチャンネルを設定しているか否かを返す
     * @return USER 設定の場合に複数の Band のチャンネルを設定しているか否か
     */
    bool hasMultiBandsInUserSetting(void);

    /**
     * @brief 測定対象に CATV 上りのチャンネルを含むか否かを返す
     * @return CATV 上りのチャンネルを含むか否か
     */
    bool hasCATVUpstreamChannel(void);

    /**
     * @brief 光測定が選択されているか否かを返す
     * @return 光測定が選択されているか否か
     */
    bool isOpticalMeasurementSelected(void);

    /**
     * @brief チャンネルリストの色変更を取得
     * @param bandId Band の ID
     * @param mode チャンネル編集モード
     * @return 編集可否リスト
     */
    static QList<QVariant> getChannelListColor(int bandId, int mode);

    /**
     * @brief ファイル数の再帰的な取得
     * @param dirName ディレクトリ名
     */
    int countFilesRecursive(QString dirName);

    /**
     * @brief ファイルリストに戻ったときに変更前に戻す
     */
    void undo(void);

    //
    // dataloggerviewmodel から移植
    //
    // 独自POPUPの操作用
    QVector<QVector<QString>> _dataLoggerPopupType1ButtonData;
    QVector<QVector<QString>> _dataLoggerPopupType2ButtonData;
    QVector<QVector<QString>> _dataLoggerPopupType3ButtonData;
    QVector<QVector<QString>> _dataLoggerPopupType4ButtonData;
    QVector<QVector<QString>> _dataLoggerPopupType5ButtonData;
    ButtonOperation _popupButtonOperation;
    QString _popupActiveButton;

    // データ取得開始（日時設定）関連のメンバ
    int _yyyy;
    int _start_mm;
    int _start_dd;
    int _start_HH;
    int _start_MM;
    int _end_mm;
    int _end_dd;
    int _end_HH;
    int _end_MM;
    int _end_yyyy;
    QTimer *_reserveTimer;                  //!< データ取得開始（日時設定）実行タイマー
    bool _loggingStarted;                   //!< ロギングを開始したか否か
    int _dataloggerExecuteInterval; 		//!< 実行間隔
    int _dataloggerExecuteCycle;    		//!< 実行サイクル

    /**
     * @brief データロガーを実行する
     */
    void executeDataLogger();

    /**
     * @brief moveCursorPopupArea
     * @param id
     */
    void moveCursorPopupArea(ButtonId id);

    /**
     * @brief onKeyEnterDatetimeset
     */
    void onKeyEnterDatetimeset();

    /**
     * @brief onKeyStatusPopupButtonArea
     * @param id
     */
    void keyStatusPopupArea(ButtonId id);

    /**
     * @brief 開始日時（月）を更新する
     * @param id
     */
    void updateStartMonth(ButtonId id);

    /**
     * @brief 開始日時（日）を更新する
     * @param id
     */
    void updateStartDay(ButtonId id);

    /**
     * @brief 開始日時（時）を更新する
     * @param id
     */
    void updateStartHour(ButtonId id);

    /**
     * @brief 開始日時（分）を更新する
     * @param id
     */
    void updateStartMinutes(ButtonId id);

    /**
     * @brief 終了日時（月）を更新する
     * @param id
     */
    void updateEndMonth(ButtonId id);

    /**
     * @brief 終了日時（日）を更新する
     * @param id
     */
    void updateEndDay(ButtonId id);

    /**
     * @brief 終了日時（時）を更新する
     * @param id
     */
    void updateEndHour(ButtonId id);

    /**
     * @brief 終了日時（分）を更新する
     * @param id
     */
    void updateEndMinutes(ButtonId id);

    /**
     * @brief 初期表示設定
     */
    void initDatetimeset();

    /**
     * @brief Selectモードの画像設定
     */
    void setSelectModeImage();

    /**
     * @brief Editモードの画像設定
     */
    void setEditModeImage();

    /**
     * @brief データロガー測定リスト遷移
     */
    void transitionDataLoggerMeasurementList();

    /**
     * @brief Select/Edit操作中の画像表示非表示制御
     * @param id
     */
    void setVisibleSelectEditImage(ButtonId id);

    /**
     * @brief 設定リスト中のMER/BER の測定がONになっているかを返す
     * @return On の場合には true
     */
    bool isEnableMERorBER();

    /**
     * @brief 測定間隔を変更する
     * @param interval 測定間隔
     * @param id ボタンID
     * @return 更新後の測定間隔
     */
    int editInterval(int interval, ButtonId id);

    /**
     * @brief 測定サイクルを変更する
     * @param interval 測定間隔
     * @param id ボタンID
     * @return 更新後の測定間隔
     */
    int editCycle(int interval, ButtonId id);

    /**
     * @brief 開始可能判定
     * @param errorMessage
     */
    int isStartDataCollection(QString &errorMessage);
    /**
     * @brief 測定可能時間のチェック
     */
    bool isMeasurable();

    /**
     * @brief 測定可能サイズの取得
     */
    quint64 getAvailableSize();

    /**
     * @brief 推定使用サイズの取得
     */
    quint64 getEstimatedUseSize();

private slots:
    /**
     * @brief 測定予約
     */
    void reserveMeasurement();
};

#endif // USERDATALOGGERSETTING_H
