/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include "usersettingviewmodel.h"

const int UserSettingViewModel::BUTTON_COLUMN_NUMBER = 2;
const int UserSettingViewModel::BUTTON_ROW_NUMBER = 3;

QVector<QVector<QString>> userSettingButtonData = {
    { {"m1"}, {""}, {""} },
    { {"m4"}, {"m5"}, {""} },
};

QVector<QVector<QString>> readData1UserSettingButtonData = {
    { {"m1"}, {"m2"}, {""} },
    { {"m4"}, {"m5"}, {""} },
};

QVector<QVector<QString>> readData2UserSettingButtonData = {
    { {"m1"}, {""}, {"m3"} },
    { {"m4"}, {"m5"}, {""} },
};

QVector<QVector<QString>> allDataUserSettingButtonData = {
    { {"m1"}, {"m2"}, {"m3"} },
    { {"m4"}, {"m5"}, {""} },
};

UserSettingViewModel::UserSettingViewModel(ScreenId screenId, LevelCheckerModel *&model, QObject *parent)
    : AbstractViewModel(screenId, model, parent)
    , _buttonOperationMode(USER_SETTING)
{
    setObjectName("userSettingViewModel");
    _buttonOperation.setButtonName(BUTTON_COLUMN_NUMBER, BUTTON_ROW_NUMBER, userSettingButtonData);
    _buttonOperation.setActiveObject(userSettingButtonData[0][0]);
    _activeButton = userSettingButtonData[0][0];
}

UserSettingViewModel::~UserSettingViewModel()
{

}

void UserSettingViewModel::activate()
{
    if (nullptr == _view) {
        qErrnoWarning("_view is null");
        return;
    }

    // サブに対するウェイクアップ要求が行われていなければ行う
    if (!_model->wakeUpRequested()) {
        _model->setWakeUpRequested(true);
        _model->wakeUp(1);
    }

    connect(this, SIGNAL(requestSetTakeOver(QVariant, QVariant)), _view, SLOT(backup(QVariant, QVariant)));
    connect(this, SIGNAL(removeAttribute(QString)), _mainFrame, SLOT(onRemoveAttribute(QString)));

    // ヘッダーアイコンの設定
    _model->replaceMenuList(MenuListIconIndex::FUNCTION_ICON, "icon_menu_user.png");

    // バンドアイコンをクリア
    _model->removeMenuList(MenuListIconIndex::BAND_ICON);

    //プリアンプアイコンをクリア
    _model->removeMenuList(MenuListIconIndex::PREAMP_ICON);

    // 取り込みデータの有無でボタン操作・表示を設定
    buttonOperationVisibleSetting();
}

void UserSettingViewModel::deactivate()
{
    AbstractViewModel::deactivate();
    disconnect(this, SIGNAL(requestSetTakeOver(QVariant, QVariant)), _view, SLOT(backup(QVariant, QVariant)));
    disconnect(this, SIGNAL(removeAttribute(QString)), _mainFrame, SLOT(onRemoveAttribute(QString)));
}

void UserSettingViewModel::setTakeOver()
{
    if (_activeButton == "m1") {
        emit requestSetTakeOver("userDataloggerFunction", CHANNEL_EDIT_USER);
    } else if (_activeButton == "m2") {
        emit requestSetTakeOver("userDataloggerFunction", CHANNEL_EDIT_USER_USB1);
    } else if (_activeButton == "m3") {
        emit requestSetTakeOver("userDataloggerFunction", CHANNEL_EDIT_USER_USB2);
    }
}

void UserSettingViewModel::onKeyStatus(ButtonId id)
{
    AbstractViewModel::onKeyStatus(id);

    if (id == BUTTON_ENTER) {
        setBoolProperty(_activeButton, "press", false);

        if (isErrorUsbStatus()) {
            return;
        }

        if (_activeButton == "m4") {            // 本体⇛USB　一括コピー
            emit requestToShowPopup(POPUP_COPY_ALL_TO_USB_USER_SETTING_DATA_CONFIRMATION);
        } else if (_activeButton == "m5") {     // USB⇛本体　一括コピー
            emit requestToShowPopup(POPUP_COPY_ALL_TO_BODY_SELECT_USER_SETTING_DATA_CONFIRMATION);
        } else {
            int button_index = _activeButton.mid(4).toInt() - 1; // ”func" 文字列を削除して数字を取り出す
            onRequest(button_index);
        }
    } else {
        moveCursor(id);
    }
}

void UserSettingViewModel::onAccept(PopupId popupId, bool /*okOnly*/)
{
    if (popupId == POPUP_COPY_ALL_TO_USB_USER_SETTING_DATA_CONFIRMATION) {
        // USB メモリーが未装着ならボタンが押せたとしてもポップアップを表示して戻る
        if (_model->getDeviceStatus().usba != LC70_STRAGE_FORMAT) {
            emit requestToShowPopup(POPUP_FAILURE_NO_USB_CONFIRMATION);
            return;
        }
        // コピー(本体⇒USB)
        copyUserMeasurementSettingDataToUsb();
    } else {
        // nothing to do
    }
}

void UserSettingViewModel::onSelect(PopupId popupId, int index)
{
    if (popupId == POPUP_COPY_ALL_TO_BODY_SELECT_USER_SETTING_DATA_CONFIRMATION) {
        // コピー(USB⇒本体)
        copyUserMeasurementSettingDataToBody(index);

        // 取り込みデータの有無でボタン操作・表示を設定
        buttonOperationVisibleSetting();

        return;
    }
}

void UserSettingViewModel::onButtonPress(ButtonId id)
{
    Q_UNUSED(id);

    if (id == BUTTON_ENTER) {
        setBoolProperty(_activeButton, "press", true);
    }
}

void UserSettingViewModel::onRequest(int condition)
{
    emit request(_screenId, condition);
}

void UserSettingViewModel::moveCursor(ButtonId id)
{
    if ((id == BUTTON_UP) || (id == BUTTON_DOWN) || (id == BUTTON_RIGHT) || (id == BUTTON_LEFT)) {

        QString nextBtn;
        bool ret = _buttonOperation.nextActiveObject(id, nextBtn);

        if (ret) {
            // inactive
            setBoolProperty(_activeButton, "active", false);

            // active
            setBoolProperty(nextBtn, "active", true);

            _activeButton = nextBtn;
            _buttonOperation.setActiveObject(_activeButton);
        }
    }
}

void UserSettingViewModel::setBoolProperty(QString objectName, QString propertyName, bool value)
{
    QObject *target = nullptr;
    if (_view) {
        target = _view->findChild<QObject *>(objectName);
    }
    if (target) {
        target->setProperty(propertyName.toUtf8().constData(), value);
    }
}

void UserSettingViewModel::copyUserMeasurementSettingDataToUsb()
{
    UsbErrorType error = UsbErrorType::NON_ERROR;

    QString copyTarget = _model->applicationSetting()->settingDataPathUserData();

    QStringList copyTargetPathList = {};

    error = UsbOperation::checkExistsDirectory(copyTarget);

    if (error == UsbErrorType::NON_ERROR) {
        copyTargetPathList.append(copyTarget);
    }

    if (copyTargetPathList.isEmpty()) {
        // コピー対象ファイルが無い場合は以降の処理は実施しない
        return;
    }

    error = UsbErrorType::NON_ERROR;

    // コピー可能な状態かのチェックを行う
    error = UsbOperation::checkCopyable(copyTargetPathList);

    if (error == UsbErrorType::COPY_FILE_ERROR) {
        // コピー元ファイルがない
        emit requestToShowPopup(POPUP_NOTHING_COPY_FILE);
        return;
    } else if (error == UsbErrorType::SPACE_ERROR) {
        // USBの空き容量が足りない
        emit requestToShowPopup(POPUP_USB_NO_SPACE);
        return;
    } else {
        // nothing to do
    }

    error = UsbOperation::copyToUsb(UsbDataType::
                                    USER_MEASUREMENT_SETTING, copyTarget, true, _model->applicationSetting()->password());

    if (error == UsbErrorType::COPY_EXEC_ERROR) {
        // コピー失敗
        emit requestToShowPopup(POPUP_FAILURE_COPY_EXECUTE);
    }
}

void UserSettingViewModel::copyUserMeasurementSettingDataToBody(int index)
{
    UsbErrorType error = UsbErrorType::NON_ERROR;
    // USBから本体へフォルダコピー
    if(index == 0) {
        error = UsbOperation::copyToBody(UsbDataType::USER_MEASUREMENT_SETTING,
                                         _model->applicationSetting()->settingDataPathUserDataUsb1(), true, _model->applicationSetting()->password());
    } else if(index == 1) {
        error = UsbOperation::copyToBody(UsbDataType::USER_MEASUREMENT_SETTING,
                                         _model->applicationSetting()->settingDataPathUserDataUsb2(), true, _model->applicationSetting()->password());
    }

    if (error == UsbErrorType::COPY_EXEC_ERROR) {
        // コピー失敗
        emit requestToShowPopup(POPUP_FAILURE_COPY_EXECUTE);
    } else if (error == UsbErrorType::COPY_FILE_ERROR) {
        // コピー元ファイルがない
        emit requestToShowPopup(POPUP_NOTHING_COPY_FILE);
    } else {
        // コピーしたファイルを読み込み直す
        _model->initializeUserSettingFile();
    }
}

bool UserSettingViewModel::isErrorUsbStatus()
{
    QMap<QString, QList<LC70_STRAGE_T>> enableUsbStatusMap = {
        {"m4", {LC70_STRAGE_FORMAT}},
        {"m5", {LC70_STRAGE_FORMAT}},
    };

    if (enableUsbStatusMap.count(_activeButton) == 0) {
        // USBに無関係のボタン
        return false;
    }

    if (enableUsbStatusMap.value(_activeButton).contains(_model->getDeviceStatus().usba)) {
        // 実行可能なUSB状態になっている
        return false;
    }

    // ボタンごとに定義している、押下可能なUSB状態になっていなければエラー表示
    emit requestToShowPopup(POPUP_FAILURE_NO_USB_CONFIRMATION); // USB未装着エラー
    return true;
}

void UserSettingViewModel::buttonOperationVisibleSetting()
{
    bool readData1Mode = false;
    bool readData2Mode = false;
    UsbErrorType error = UsbErrorType::NON_ERROR;
    QString copyTarget = _model->applicationSetting()->settingDataPathUserDataUsb1();
    error = UsbOperation::checkExistsSettingFile(copyTarget);
    if (error == UsbErrorType::NON_ERROR) {
        if (_view) {
            _view->setProperty("readData1", true);
            readData1Mode = true;
        }
    } else {
        if (_view) {
            _view->setProperty("readData1", false);
            readData1Mode = false;
        }
    }
    error = UsbErrorType::NON_ERROR;
    copyTarget = _model->applicationSetting()->settingDataPathUserDataUsb2();
    error = UsbOperation::checkExistsSettingFile(copyTarget);
    if (error == UsbErrorType::NON_ERROR) {
        if (_view) {
            _view->setProperty("readData2", true);
            readData2Mode = true;
        }
    } else {
        if (_view) {
            _view->setProperty("readData2", false);
            readData2Mode = false;
        }
    }

    // 選択中のアイコンをリセット
    setBoolProperty(_activeButton, "active", false);

    if ( (readData1Mode == false) && (readData2Mode == false) ){
        _buttonOperation.setButtonName(BUTTON_COLUMN_NUMBER, BUTTON_ROW_NUMBER, userSettingButtonData);
        _buttonOperation.setActiveObject(userSettingButtonData[0][0]);
        _activeButton = userSettingButtonData[0][0];
    } else if ( (readData1Mode == true) && (readData2Mode == false) ){
        _buttonOperation.setButtonName(BUTTON_COLUMN_NUMBER, BUTTON_ROW_NUMBER, readData1UserSettingButtonData);
        _buttonOperation.setActiveObject(readData1UserSettingButtonData[0][0]);
        _activeButton = readData1UserSettingButtonData[0][0];
    } else if ( (readData1Mode == false) && (readData2Mode == true) ){
        _buttonOperation.setButtonName(BUTTON_COLUMN_NUMBER, BUTTON_ROW_NUMBER, readData2UserSettingButtonData);
        _buttonOperation.setActiveObject(readData2UserSettingButtonData[0][0]);
        _activeButton = readData2UserSettingButtonData[0][0];
    } else if ( (readData1Mode == true) && (readData2Mode == true) ){
        _buttonOperation.setButtonName(BUTTON_COLUMN_NUMBER, BUTTON_ROW_NUMBER, allDataUserSettingButtonData);
        _buttonOperation.setActiveObject(allDataUserSettingButtonData[0][0]);
        _activeButton = allDataUserSettingButtonData[0][0];
    }

    // 選択中のアイコンの設定
    setBoolProperty(_activeButton, "active", true);
}
