/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef USERSETTINGVIEWMODEL_H
#define USERSETTINGVIEWMODEL_H

#include "abstractviewmodel.h"
#include "buttonoperation.h"

enum BUTTON_USER_OPERATION_MODE {
    USER_SETTING = 0,
    READ_USER_1,
    READ_USER_2,
    ALL_USER,
};

/**
 * @brief ユーザ設定 ViewModel
 */
class UserSettingViewModel : public AbstractViewModel
{
    Q_OBJECT
public:
    /**
     * @brief コンストラクター
     * @param screenId 画面 ID
     * @param model Model クラスのポインター
     * @param parent 親オブジェクト
     */
    explicit UserSettingViewModel(ScreenId screenId, LevelCheckerModel *&model, QObject *parent = nullptr);

    /**
     * @brief デストラクタ
     */
    virtual ~UserSettingViewModel();

    /**
     * @brief 画面Active 時の処理
     */
    virtual void activate() override;

    /**
     * @brief 画面を閉じるときに呼び出す表示な用の保存処理
     */
    virtual void deactivate(void) override;

    /**
     * @brief 遷移先画面で参照する遷移前画面のデータ設定
     */
    virtual void setTakeOver() override;

signals:
    void requestSetTakeOver(QVariant, QVariant);

protected:
    ButtonOperation _buttonOperation;   //!< ボタン操作情報
    QString _activeButton;              //!< 活性ボタンID
    BUTTON_USER_OPERATION_MODE _buttonOperationMode;  //!< ボタンモード
    static const int BUTTON_COLUMN_NUMBER;   // ボタンの列数
    static const int BUTTON_ROW_NUMBER;      // ボタンの行数

    /**
     * @brief onKeyStatus
     * @param id
     */
    virtual void onKeyStatus(ButtonId id) override;

    /**
     * @brief onButtonPress
     * @param id
     */
    virtual void onButtonPress(ButtonId id) override;

    /**
     * @brief ダイアログからのAccept受信処理
     * @param popupId ダイアログID
     * @param okOnly 結果
     */
    void onAccept(PopupId popupId, bool okOnly) override;

    /**
     * @brief ポップアップから選択の通知を受信
     * @param popupId ポップアップのID
     * @param index 選択されたインデックス
     */
    virtual void onSelect(PopupId popupId, int index) override;

private slots:
    /**
     * @brief 画面遷移要求の受信時処理
     * @param condition 画面IDリストのインデックス
     */
    virtual void onRequest(int condition) override;

private:
    /**
     * @brief moveCursor
     * @param id
     */
    void moveCursor(ButtonId id);

    /**
     * @brief プロパティに Bool 値を設定
     * @param objectName オブジェクト名
     * @param propertyName プロパティ名
     * @param value 設定する Bool 値
     */
    void setBoolProperty(QString objectName, QString propertyName, bool value);

    /**
     * @brief ユーザ測定設定データを本体からUSBにコピーする
     */
    void copyUserMeasurementSettingDataToUsb(void);

    /**
     * @brief ユーザ測定設定データをUSBから本体にコピーする
     */
    void copyUserMeasurementSettingDataToBody(int index);

    /**
     * @brief USB状態エラー発生チェック
     * @return true: USB状態エラーが発生している、false: 発生していない
     */
    bool isErrorUsbStatus(void);

    /**
     * @brief 取り込みデータの有無でボタン操作・表示を設定
     */
    void buttonOperationVisibleSetting(void);
};

#endif // USERSETTINGVIEWMODEL_H
