/*
 * This file is part of pdisp. Copyright (c) 2021 DX ANTENNA CO.,LTD.
 *
 * pdisp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include "userdataloggersettingviewmodel.h"
#include "correcter.h"

#include <QDebug>
#include <QElapsedTimer>

static constexpr int FILE_LIST_ROWS = 10;                           //!< 設定ファイルのリスト行数
static constexpr int PRESET_CHANNEL_LENGTH = 12;                    //!< プリセットのチャンネル数
static constexpr int MAXIMUM_FILE_LIST_PAGE = 14;                   //!< 設定ファイルのリストの最大ページ数
static constexpr int CHANNEL_LIST_ROWS = 5;                         //!< 測定内容のリストの行数
static constexpr int FILE_NAME_PREFIX_LENGTH = 2;                   //!< ファイルグループのプリフィクスの長さ
static constexpr const char *CSV_FILE_EXTENSION = ".csv";           //!< 設定ファイルの拡張子(CSV)
static constexpr const char *BACKUP_FILE_EXTENSION = ".bak";        //!< バックアップファイルの拡張子
static constexpr const char *MEASURE_BUTTON = "startButton";        //!< 測定開始ボタン(qml ファイルの objectName プロパティと合わせておく)
static constexpr const char *SAVE_BUTTON = "saveButton";            //!< 保存ボタン(qml ファイルの objectName プロパティと合わせておく)
static constexpr const char *DELETE_BUTTON = "deleteButton";        //!< 削除ボタン(qml ファイルの objectName プロパティと合わせておく)
static constexpr const char *ALL_DELETE_BUTTON = "allDeleteButton"; //!< 一括削除ボタン(qml ファイルの objectName プロパティと合わせておく)
static constexpr const char *SCAN_SAVE_BUTTON = "determinationButton"; //!< SCAN保存ボタン(qml ファイルの objectName プロパティと合わせておく)
static constexpr int MEASUREMENT_1CH_DATA_SIZE = 62;                //!< 設定ファイルに定義された、1CHのデータ量
static constexpr const char *MEASURING_CATV_UPSTREAM_NOT_APPLICABLE = "光入力設定になっているため、\nCATV上りを測定できません。";
static constexpr int MEASUREMENT_CH_MAX = 50;                       //!< 測定チャンネル数上限
static constexpr int AN_HOUR = 3600000;                             //!< ms 単位で 1 時間

/**
 *  @brief Band タブごとの Band ID の定義
 */
static const QMap<int, LC70_BAND_T> TAB_BAND_MAP = {
    {BAND_TAB_UHF,              LC70_BAND_UHF},
    {BAND_TAB_BS_R,             LC70_BAND_BS},
    {BAND_TAB_BS_L,             LC70_BAND_BS},
    {BAND_TAB_CS_R,             LC70_BAND_CS},
    {BAND_TAB_CS_L,             LC70_BAND_CS},
    {BAND_TAB_CATV_ALL,         LC70_BAND_CATV_ALL},
    {BAND_TAB_SKY_PERFECTV,     LC70_BAND_SKYP},
    {BAND_TAB_CATV_UPSTREAM,    LC70_BAND_CATV_UP_STREAM},
    {BAND_TAB_JCSAT3A_H,        LC70_BAND_JCSAT3},
    {BAND_TAB_JCSAT3A_V,        LC70_BAND_JCSAT3},
    {BAND_TAB_JCSAT4B_H,        LC70_BAND_JCSAT4},
    {BAND_TAB_JCSAT4B_V,        LC70_BAND_JCSAT4},
    {BAND_TAB_SCCB2_H,          LC70_BAND_SCCB},
    {BAND_TAB_SCCB2_V,          LC70_BAND_SCCB},
    {BAND_TAB_SCCC2_H,          LC70_BAND_SCCC},
    {BAND_TAB_SCCC2_V,          LC70_BAND_SCCC},
    {BAND_TAB_LTE,              LC70_BAND_LTE},
    {BAND_TAB_FM,               LC70_BAND_FM},
    {BAND_TAB_PILOT,            LC70_BAND_PILOT},
    {BAND_TAB_COUNT,            LC70_BAND_UNDEF},
};

/**
 *  @brief Band タブごとの偏波の定義
 */
static const QMap<int, QString> TAB_POLARIZATION_MAP = {
    {BAND_TAB_UHF,              QString("")},
    {BAND_TAB_BS_R,             RIGHT_HANDED_POLARIZE},
    {BAND_TAB_BS_L,             LEFT_HANDED_POLARIZE},
    {BAND_TAB_CS_R,             RIGHT_HANDED_POLARIZE},
    {BAND_TAB_CS_L,             LEFT_HANDED_POLARIZE},
    {BAND_TAB_CATV_ALL,         QString("")},
    {BAND_TAB_SKY_PERFECTV,     QString("")},
    {BAND_TAB_CATV_UPSTREAM,    QString("")},
    {BAND_TAB_JCSAT3A_H,        HORIZONTALLY_POLARIZE},
    {BAND_TAB_JCSAT3A_V,        VERTICALLY_POLARIZE},
    {BAND_TAB_JCSAT4B_H,        HORIZONTALLY_POLARIZE},
    {BAND_TAB_JCSAT4B_V,        VERTICALLY_POLARIZE},
    {BAND_TAB_SCCB2_H,          HORIZONTALLY_POLARIZE},
    {BAND_TAB_SCCB2_V,          VERTICALLY_POLARIZE},
    {BAND_TAB_SCCC2_H,          HORIZONTALLY_POLARIZE},
    {BAND_TAB_SCCC2_V,          VERTICALLY_POLARIZE},
    {BAND_TAB_LTE,              QString("")},
    {BAND_TAB_FM,               QString("")},
    {BAND_TAB_PILOT,            QString("")},
    {BAND_TAB_COUNT,            QString("")},
};

static constexpr int EDIT_POSITION_CHANNEL = 0;         //!< チャンネル(USER 設定、データロガー設定とも編集不可)
static constexpr int EDIT_POSITION_FREQUENCY = 1;       //!< 周波数(USER 設定、データロガー設定とも FM, CATV 上りのみ)
static constexpr int EDIT_POSITION_BROADCAST = 2;       //!< 放送規格(USER 設定、データロガー設定とも CATV のみ)
static constexpr int EDIT_POSITION_LOCALFREQ = 3;       //!< 局発周波数(USER 設定、データロガー設定ともJCSAT-3A, JCSAT-4B, SCCB, SCCC のみ)
static constexpr int EDIT_POSITION_MER = 4;             //!< MER(USER 設定では編集不可、データロガー設定では全 Band)
static constexpr int EDIT_POSITION_BER = 5;             //!< BER(USER 設定では編集不可、データロガー設定では全 Band)
static constexpr int EDIT_POSITION_POWERSUPPLY = 6;     //!< 給電(USER 設定では編集不可、データロガー設定では全 Band)
static constexpr int EDIT_POSITION_MEASURE = 7;         //!< 測定 ON/OFF(全 Band)
static constexpr int EDIT_POSITION_ALL_SELECTION = 8;   //!< 測定 ON/OFF(全行更新)

static constexpr int ACTION_INDEX_NONE = 0;             //!< ボタン選択なし
static constexpr int ACTION_INDEX_ALL_DELETE = 1;       //!< 一括削除ボタン選択
static constexpr int ACTION_INDEX_SAVE = 2;             //!< 保存ボタン選択
static constexpr int ACTION_INDEX_DELETE = 3;           //!< 削除ボタン選択
static constexpr int ACTION_INDEX_DECISION = 4;         //!< 決定ボタン選択

/**
 *  @brief 給電設定リスト(JCSAT-3A, SCCB2, SCCC2 水平、その他用)
 */
static const QList<uint8_t> POWER_15VONLY = {
    LC70_MEAS_SET_DC_OUT_VAL_NONE,
    LC70_MEAS_SET_DC_OUT_VAL_15V,
};

/**
 *  @brief 給電設定リスト(JCSAT-3A, SCCB2, SCCC2 垂直用)
 */
static const QList<uint8_t> POWER_11VONLY = {
    LC70_MEAS_SET_DC_OUT_VAL_NONE,
    LC70_MEAS_SET_DC_OUT_VAL_11V
};
//static const QList<uint8_t> POWER_11V_15V = {
//    LC70_MEAS_SET_DC_OUT_VAL_NONE,
//    LC70_MEAS_SET_DC_OUT_VAL_11V,
//    LC70_MEAS_SET_DC_OUT_VAL_15V,
//};

/**
 *  @brief 給電設定リスト(JCSAT-4B 水平用)
 */
static const QList<uint8_t> POWER_15VPONLY = {
    LC70_MEAS_SET_DC_OUT_VAL_NONE,
    LC70_MEAS_SET_DC_OUT_VAL_15V_P
};
//static const QList<uint8_t> POWER_15VP_15V = {
//    LC70_MEAS_SET_DC_OUT_VAL_NONE,
//    LC70_MEAS_SET_DC_OUT_VAL_15V_P,
//    LC70_MEAS_SET_DC_OUT_VAL_15V
//};

/**
 *  @brief 給電設定リスト(JCSAT-4B 垂直用)
 */
static const QList<uint8_t> POWER_11VPONLY = {
    LC70_MEAS_SET_DC_OUT_VAL_NONE,
    LC70_MEAS_SET_DC_OUT_VAL_11V_P
};
//static const QList<uint8_t> POWER_11VP_15V = {
//    LC70_MEAS_SET_DC_OUT_VAL_NONE,
//    LC70_MEAS_SET_DC_OUT_VAL_11V_P,
//    LC70_MEAS_SET_DC_OUT_VAL_15V
//};

/**
 *  @brief Band、偏波別給電設定リスト
 */
static const QMap<QPair<LC70_BAND_T, int>, QList<uint8_t>> POWER_SUPPLY_SETTING_MAP = {
    {{LC70_BAND_JCSAT3, LC70_MEAS_SET_WAVE_H}, POWER_15VONLY},
    {{LC70_BAND_JCSAT3, LC70_MEAS_SET_WAVE_V}, POWER_11VONLY},
    {{LC70_BAND_JCSAT4, LC70_MEAS_SET_WAVE_H}, POWER_15VPONLY},
    {{LC70_BAND_JCSAT4, LC70_MEAS_SET_WAVE_V}, POWER_11VPONLY},
    {{LC70_BAND_SCCB,   LC70_MEAS_SET_WAVE_H}, POWER_15VONLY},
    {{LC70_BAND_SCCB,   LC70_MEAS_SET_WAVE_V}, POWER_11VONLY},
    {{LC70_BAND_SCCC,   LC70_MEAS_SET_WAVE_H}, POWER_15VONLY},
    {{LC70_BAND_SCCC,   LC70_MEAS_SET_WAVE_V}, POWER_11VONLY},
};

/**
  @brief 放送方式の保存値と表示内容
 */
static const QMap<int, QString> BROADCAST_SYSTEM_VALUE = {
    { LC70_BD_SYS_ISDBT, QString("ISDBT") },
    { LC70_BD_SYS_ANNEXC, QString("ISDBC") },
};

/**
  @brief 局発周波数の保存値と表示内容
 */
static const QMap<int, QString> LOCAL_FREQUENCY_VALUE = {
    { 10678000, QString("10.678") },
    { 11200000, QString("11.2") },
    { 11300000, QString("11.3") }
};

/**
  @brief 給電 ON/OFF の保存値と表示内容
 */
static const QMap<int, QString> POWER_SUPPLY_VALUE = {
    { 0, QString("OFF") },
    { 1, QString("ON") }
};

/**
  @brief 測定する/しないの保存値と表示内容
 */
static const QMap<int, QString> MEASURING_VALUE = {
    { 0, QString("しない") },
    { 1, QString("する") }
};

/**
  @brief CATV のチャンネルのうち設定により周波数の変更を行うチャンネル名とチャンネルのインデックス
 */
static const QMap<QString, int> REPLACE_FREQUENCY_CHANNEL_INDEX = {
    { "C24", 22 },
    { "C25", 23 },
    { "C26", 24 },
    { "C27", 25 },
};

UserDataLoggerSettingViewModel::UserDataLoggerSettingViewModel(ScreenId screenId, LevelCheckerModel *&model, QObject *parent)
    : AbstractViewModel(screenId, model, parent)
    , _mode(CHANNEL_EDIT_USER)
    , _selectArea(FILELIST_AREA)
    , _fileList()
    , _currentFileListPage(1)
    , _currentFileIndex(0)
    , _bandModel(nullptr)
    , _currentTabIndex(BAND_TAB_UHF)
    , _channelList()
    , _currentChannelIndex(0)
    , _currentEditPositon(0)
    , _editItemList()
    , _currentTargetIndex(0)
    , _temporaryFileName()
    , _currentActionIndex(0)
    , _oldFileName()
    , _popupType(TYPEUNKNOWN)
    , _popupState(SELECT)
    , _popupRsvState(SELECT)
    , _destinationIndex(-1)
    , _reserveTimer(nullptr)
    , _loggingStarted(false)
    , _dataloggerExecuteInterval(0)
    , _dataloggerExecuteCycle(0)
    , _userSetingBehavior(ALLOW_MULTIPLE_BANDS)
    , _destinationTab(BAND_TAB_COUNT)
{
    setObjectName("userDataLoggerSetting");

    _dataLoggerPopupType1ButtonData = {
        {{"m1", "m2"}}
    };

    _dataLoggerPopupType2ButtonData = {
        {{"intervaldata", "cycledata"}},
        {{"m1", "m1"}}
    };

    _dataLoggerPopupType3ButtonData = {
        {{"startmonthdata", "startdaydata", "starthourdata", "startminutesdata", ""}},
        {{"rsvintervaldata", "rsvcycledata", "", "", ""}},
        {{"m1", "m1", "", "", ""}}
    };

    _dataLoggerPopupType4ButtonData = {
        {{"m1"}}
    };

    _dataLoggerPopupType5ButtonData = {
        {{"m1", "m2"}}
    };

    _popupButtonOperation.setActiveObject(_dataLoggerPopupType1ButtonData[0][0]);
    _popupActiveButton = _dataLoggerPopupType1ButtonData[0][0];

    QDateTime currentDateTime = QDateTime::currentDateTime();
    _yyyy = currentDateTime.date().year();
    _start_mm = 1;
    _start_dd = 1;
    _start_HH = 0;
    _start_MM = 0;
    _end_mm = 1;
    _end_dd = 1;
    _end_HH = 0;
    _end_MM = 0;
    _end_yyyy = _yyyy;

    _reserveTimer = new QTimer(this);
    connect(_reserveTimer, SIGNAL(timeout()), this, SLOT(reserveMeasurement()));
    _reserveTimer->setSingleShot(true);
}

UserDataLoggerSettingViewModel::~UserDataLoggerSettingViewModel()
{

}

void UserDataLoggerSettingViewModel::resume()
{
    AbstractViewModel::resume();

}

void UserDataLoggerSettingViewModel::suspend()
{
    AbstractViewModel::suspend();

    disconnect(_model, SIGNAL(changeUsbInsertionRemoval(int)), this, SLOT(onChangeUsbInsertionRemoval(int)));
}

void UserDataLoggerSettingViewModel::createConnection(QObject *qml, QObject *mainFrame)
{
    AbstractViewModel::createConnection(qml, mainFrame);

    connect(_model, SIGNAL(changeUsbInsertionRemoval(int)), this, SLOT(onChangeUsbInsertionRemoval(int)));
}

void UserDataLoggerSettingViewModel::activate()
{
    if (nullptr == _view) {
        qErrnoWarning("_view is null");
        return;
    }

    connect(this, SIGNAL(requestSetTakeOver(QVariant, QVariant)), _view, SLOT(backup(QVariant, QVariant)));
    connect(this, SIGNAL(removeAttribute(QString)), _mainFrame, SLOT(onRemoveAttribute(QString)));

    // グラフ用の測定結果のモデル(BandModel の実体(objectName:bandModel)を QML から参照)
    if (_view) {
        _bandModel = dynamic_cast<BandModel *>(_view->findChild<QObject *>(QString("bandModel")));
    }

    _model->removeMenuList(MenuListIconIndex::PREAMP_ICON);

    // ユーザ設定ファイルを初期化
    _model->initializeUserSettingFile();

    // 画面表示時はファイルリスト選択から
    _selectArea = FILELIST_AREA;
    _popupType = TYPEUNKNOWN;
    if (_view) {
        _view->setProperty("activeArea", _selectArea);
    }
    // 地デジを選択
    _currentTabIndex = BAND_TAB_UHF;
    _bandModel->select(static_cast<BandTabIndex>(_currentTabIndex));

    // 前の画面の情報を引き継いで画面のモードを設定
    if (_view) {
        QObject *takeOver = _view->findChild<QObject *>("takeOverData");
        if (takeOver) {
            QVariant value;
            if (_takeOver.contains("userDataloggerFunction")) {
                value = _takeOver["userDataloggerFunction"];
                _mode = static_cast<ChannelEditMode>(value.toInt());
            } else {
                qDebug() << QString("mode not set.");
                return;
            }
        } else {
            qDebug() << QString("No take over.");
        }
    }
    if (_view) {
        _view->setProperty("mode", _mode);
    }

    // Menu, Band ボタンの長押し対応
    _reactToLongPress = true;

    // USER データロガー設定ファイルの一覧を取得し、画面に反映
    showSettingFileList();

	// USER データロガー測定完了時はポップアップ画面を表示する
    if (_takeOver.contains("completeDataLoggerMeasurement")) {
        if (_takeOver["completeDataLoggerMeasurement"].toBool()) {
            emit requestToShowPopup(POPUP_DATA_LOGGER_MESUREMENT_COMLETE);
        }
        emit removeAttribute("completeDataLoggerMeasurement");
        _takeOver["completeDataLoggerMeasurement"] = false;
    }

    // 測定する Band アイコンをクリアー
    _model->replaceMenuList(MenuListIconIndex::BAND_ICON, QString(""));
}

void UserDataLoggerSettingViewModel::deactivate()
{
    AbstractViewModel::deactivate();
    disconnect(this, SIGNAL(requestSetTakeOver(QVariant, QVariant)), _view, SLOT(backup(QVariant, QVariant)));
    disconnect(this, SIGNAL(removeAttribute(QString)), _mainFrame, SLOT(onRemoveAttribute(QString)));

    _fileList.clear();
    _channelList.clear();

    _currentFileListPage = 1;
    _currentFileIndex = 0;
    _currentTabIndex = BAND_TAB_UHF;
    _currentChannelIndex = 0;
    _currentEditPositon = 0;
    _currentTargetIndex = 0;
    _temporaryFileName.clear();
    _destinationIndex = -1;
    _destinationTab = BAND_TAB_COUNT;
    _reactToLongPress = false;
}

void UserDataLoggerSettingViewModel::setTakeOver()
{
    int index = _currentFileIndex + (_currentFileListPage - 1) * FILE_LIST_ROWS;
    if (_fileList.count() > 0) {
        emit requestSetTakeOver("selectedUserListFile", _fileList[index]);
        if ((_mode == CHANNEL_EDIT_USER) || (_mode == CHANNEL_EDIT_USER_USB1) || (_mode == CHANNEL_EDIT_USER_USB2)){
            emit requestSetTakeOver("selectedBand", TAB_BAND_MAP.value(_currentTabIndex, LC70_BAND_UNDEF));
            emit requestSetTakeOver("selectedPolarization", TAB_POLARIZATION_MAP.value(_currentTabIndex, QString("")));
        }
    } else {
        qDebug() << QString("No Setting File");
    }
    // 保存先選択から戻る場合は USER 設定からの呼び出しではない
    if (_mode == CHANNEL_EDIT_SELECT_DESTINATION) {
#if 1	//210628-1
		// SCAN後、保存先選択から ESCボタンを押して測定画面に戻ったときに USERファイル名が表示されてしまう不具合対策
		if (_lastPressedButtonId == BUTTON_ESCAPE) {
			emit requestSetTakeOver("selectedUserListFile", "");
		}
#endif
        emit requestSetTakeOver("fromUserSetting", false);
    }
}

ChannelEditMode UserDataLoggerSettingViewModel::mode() const
{
    return _mode;
}

void UserDataLoggerSettingViewModel::onKeyStatus(ButtonId id)
{
    qDebug() << QString("[Debug]") << QString("UserDataLoggerSettingViewModel::onKeyStatus() call");
    qDebug() << QString("[Debug]") << QString("_selectArea") << QString::number(_selectArea);
    qDebug() << QString("[Debug]") << QString("id") << QString::number(id);
    if (id == BUTTON_ESCAPE  || id == BUTTON_HOME || id == BUTTON_USER) {
        _lastPressedButtonId = id;

        // 保存先選択で HOME, USER ボタンを押下した場合はスキャン結果を保存しなかったものとする
        if (_mode == CHANNEL_EDIT_SELECT_DESTINATION) {
            if (id == BUTTON_HOME || id == BUTTON_USER) {
                _model->clearScannedChannelList();
            }
        }

        // 現在はこの条件が真になることはない
        if (_model->isUpdate(settingListValueId())) {
            emit requestToShowPopup(POPUP_CANCEL_CONFIRMATION);
        } else {
            if ( (_selectArea == POPUP_AREA) && (id == BUTTON_ESCAPE) ) {
	            QObject *popupItem = nullptr;
	            if (_view) {
	                popupItem = _view->findChild<QObject *>("dgPopup");
	            }
	            if (popupItem) {
                    popupItem->setProperty("visible", false);
                    if (_popupType != WAITING) {
                        _popupType = TYPEUNKNOWN;
                        _selectArea = FILELIST_AREA;
                    }
	            }
            } else {
                if ((id == BUTTON_ESCAPE)
                       ||  ((id == BUTTON_HOME || id == BUTTON_USER)
                       &&   ((_popupType != WAITING) && (_popupType != CANCELCONFIRM)))){
                    onAccept(POPUP_CANCEL_CONFIRMATION, true);
                }
	        }
	    }
    }

    switch (_selectArea) {
    case FILELIST_AREA:
        keyStatusFileListArea(id);
        break;
    case CHANNEL_AREA:
        keyStatusChannelListArea(id);
        break;
    case CHANNEL_EDIT:
        keyStatusChannelEdit(id);
        break;
    case TARGET_AREA:
        keyStatusTargetListArea(id);
        break;
    case TARGET_EDIT:
        keyStatusTargetEdit(id);
        break;
    case ACTION_AREA:
        keyStatusActionArea(id);
        break;
    case TARGET_ALL:
        keyStatusTargetAllArea(id);
        break;
    case TARGET_ALL_EDIT:
        keyStatusTargetAllEdit(id);
        break;
    case POPUP_AREA:
        keyStatusPopupArea(id);
        break;
    default:
        break;
    }
}

void UserDataLoggerSettingViewModel::onButtonPress(ButtonId id)
{
    if (id == BUTTON_ENTER) {
        QObject *target = nullptr;

        switch (_selectArea) {
        case CHANNEL_AREA:
        {
        }
            break;

        case TARGET_AREA:
        {
            int index = _currentChannelIndex % CHANNEL_LIST_ROWS;
            target = _model->search(QString("onOff%1").arg(index));
            if (target) {
                target->setProperty("press", true);
            }
        }
            break;

        case TARGET_ALL:
        {
            target = _model->search(QString("allOnOff"));
            if (target) {
                target->setProperty("press", true);
            }
        }
            break;

        case ACTION_AREA:
        {
            switch (_currentActionIndex) {
            case ACTION_INDEX_ALL_DELETE:
                if (_view) {
                    target = _view->findChild<QObject *>(ALL_DELETE_BUTTON);
                }
                break;
            case ACTION_INDEX_SAVE:
                if (_view) {
                    target = _view->findChild<QObject *>(SAVE_BUTTON);
                }
                break;
            case ACTION_INDEX_DELETE:
                if (_view) {
                    target = _view->findChild<QObject *>(DELETE_BUTTON);
                }
                break;
            default:
                break;
            }
            if (target) {
                target->setProperty("press", true);
            }
        }
            break;
        default:
            break;
        }
    }
}

void UserDataLoggerSettingViewModel::onAccept(PopupId popupId, bool okOnly)
{
    Q_UNUSED(okOnly);

    AbstractViewModel::onAccept(popupId, okOnly);

    if (popupId == POPUP_CANCEL_CONFIRMATION) {
        acceptPopupCancelConfirmation();
    } else if (popupId == POPUP_SAVE_CONFIRMATION || popupId == POPUP_SELECT_SAVE_CONFIRMATION) {
        acceptPopupSaveConfirmation();
        onReject(popupId);  // ボタンの押下状態を元に戻す
    } else if (popupId == POPUP_DATA_LOGGER_KEYBOARD) {
        acceptPopupDataLoggerKeyboard();
    } else if (popupId == POPUP_CONFIRM_OVERWRITE_USER_LOGGER_DATA) {
        acceptPopupConfirmOverwriteUserLoggerData();
    } else if (popupId == POPUP_CONFIRM_DELETE_SELECTED_FILE) {
        acceptPopupDeleteConfirmation();
        onReject(popupId);  // ボタンの押下状態を元に戻す
    } else if (popupId == POPUP_CONFIRM_ALL_DELETE_FILE) {
        acceptPopupAllDeleteConfirmation();
        onReject(popupId);  // ボタンの押下状態を元に戻す
    } else if (popupId == POPUP_CONFIRM_CHANGE_BAND) {
        acceptPopupConfirmChangeBand();
    } else if (popupId == POPUP_CONFIRM_EXECUTE_WITH_AUTO_POWER_OFF) {
        acceptPopupConfirmExecuteWithAutoPowerOff();
    } else if ( (popupId == POPUP_DATA_LOGGER_SETTING_EXCEED_UPPER_LIMIT) || (popupId == POPUP_USER_SETTING_EXCEED_UPPER_LIMIT) ) {
        // 更新したチャンネル(要素が配列のリスト)
        QVariantList editItem = _channelList[_currentChannelIndex].toList();

        // OFF に戻す
        editItem[EDIT_POSITION_MEASURE] = _editItemList[0];
        _channelList[_currentChannelIndex] = editItem;

        // 測定チャンネル数の上限を超えている場合は上限を表示
        unsigned int limit = LC70_CH_SET_CATV_ALL_IDX_MAX;
        if (_mode == CHANNEL_EDIT_DATA_LOGGER ||
            _mode == CHANNEL_EDIT_DATA_LOGGER_USB1 ||
            _mode == CHANNEL_EDIT_DATA_LOGGER_USB2) {
            limit = MEASUREMENT_CH_MAX;
        }
        if (_view) {
            // 測定チャンネル数を更新
            _view->setProperty("channelCount", limit);
        }

        if (_view) {
            // 測定内容を画面に反映
            _view->setProperty("channelList", QVariant::fromValue(_channelList));

            // チャンネルリストの文字色反映
            _view->setProperty("channelListColor", getChannelListColor(_currentTabIndex, _mode));
        }
        moveArea(TARGET_AREA);

        // チャンネル数を更新
        hasActiveChannel();
    } else if (popupId == POPUP_SAME_FREQUENCY_CHANNEL) {
        if (_destinationIndex >= 0) {
            // タブが FM, CATV 上り以外の場合にタブを切り替える
            if (_destinationTab != _currentTabIndex) {
                if (_destinationTab == BAND_TAB_CATV_UPSTREAM ||
                        _destinationTab == BAND_TAB_FM) {
                    _currentTabIndex = _destinationTab;
                    switchTab();
                }
            }

            _currentChannelIndex = _destinationIndex;
            selectTargetRow();
        }
        _destinationIndex = -1;
    }
}

void UserDataLoggerSettingViewModel::onReject(PopupId popupId)
{
    if (popupId == POPUP_SAVE_CONFIRMATION || popupId == POPUP_SELECT_SAVE_CONFIRMATION ||
        popupId == POPUP_CONFIRM_DELETE_SELECTED_FILE || popupId == POPUP_CONFIRM_ALL_DELETE_FILE) {
        QObject *target = nullptr;
        switch (_currentActionIndex) {
        case ACTION_INDEX_ALL_DELETE:
            if (_view) {
                target = _view->findChild<QObject *>(ALL_DELETE_BUTTON);
            }
            break;
        case ACTION_INDEX_SAVE:
            if (_view) {
                target = _view->findChild<QObject *>(SAVE_BUTTON);
            }
            break;
        case ACTION_INDEX_DELETE:
            if (_view) {
                target = _view->findChild<QObject *>(DELETE_BUTTON);
            }
            break;
        default:
            break;
        }
        if (target) {
            target->setProperty("press", false);
        }
    }
}

void UserDataLoggerSettingViewModel::onSelect(PopupId popupId, int index)
{
    Q_UNUSED(popupId);
    Q_UNUSED(index);
}

void UserDataLoggerSettingViewModel::onRequest(int condition)
{
    emit request(_screenId, condition);
}

void UserDataLoggerSettingViewModel::onInputFileName(PopupId popupId, SaveLocation saveLocation, QString fileName)
{
    Q_UNUSED(saveLocation);

    // ファイル名が空の場合は、後続処理は実施しない
    if (fileName.isEmpty()) {
        return;
    }

    // 選択中のユーザリストのindex
    int index = _currentFileIndex + (_currentFileListPage - 1) * FILE_LIST_ROWS;

    QVariant value;
    _model->getValue(fileListValueId(), value);

    QVariantList userList = value.toList();
    QVariantList userData = userList[index].toList();

    // 同一ファイル名の確認
    if (checkSameFileName(userList, fileName)) {
        // 仮のファイル名を設定
        _temporaryFileName = fileName;

        // 同じ名前のファイル名が存在する場合データ上書き保存確認ダイアログを表示する
        emit requestToShowPopup(POPUP_CONFIRM_OVERWRITE_USER_LOGGER_DATA);
        return;
    } else {
        // ファイルグループを示す文字をファイル名に付加
        fileName = addGroupPrefixToFileName(fileName);
    }

    // 旧ファイル名をリネーム
    if (!_oldFileName.isEmpty()) {
        if (_model->renameUserDataLoggerSettingFile(_mode, _oldFileName + QString(CSV_FILE_EXTENSION),
                                                    fileName + QString(CSV_FILE_EXTENSION))) {
            qDebug() << QString("rename %1 to %2").arg(_oldFileName + QString(CSV_FILE_EXTENSION)).arg(fileName + QString(CSV_FILE_EXTENSION));
        } else {
            emit requestToShowPopup(POPUP_WRITE_ERROR);
            return;
        }
        _oldFileName.clear();
    }

    // USER リストに保存する
    userData[2] = fileName;
    userList[index] = userData;
    _model->setValue(fileListValueId(), userList);
    _model->commit(fileListValueId());

    // 保存処理
    onAccept(popupId, true);
}

void UserDataLoggerSettingViewModel::onChangeUsbInsertionRemoval(int usbStatus)
{
    if (usbStatus != LC70_STRAGE_FORMAT) {
        if (_view) {
            // フォーマット済みの USB メモリー装着以外
            _view->setProperty("isUSBMemoryFound", false);
        }
    } else {
        if (_view) {
             // フォーマット済みの USB メモリー装着
            _view->setProperty("isUSBMemoryFound", true);
        }
    }
}

void UserDataLoggerSettingViewModel::showSettingFileList()
{
    QVariant value;
    bool ret = _model->getValue(fileListValueId(), value);

    bool fileExist = false;
    if (ret) {
        QVariantList fileList = value.toList();
        _fileList.clear();
        // 全ファイル取得
        for (QVariantList::iterator it = fileList.begin(); it != fileList.end(); ++it) {
            QString fileName = (*it).toList()[2].toString();
#if !Q_PROCESSOR_ARM
            if (fileName.contains(CSV_FILE_EXTENSION)) {
                fileName = fileName.left(fileName.length() - qstrlen(CSV_FILE_EXTENSION));
            }
#endif
            if (!fileName.isEmpty()) {
                fileExist = true;
            }
            _fileList.append(fileName);
        }

        // 設定ファイルがなければ一括削除を無効にする
        if (_view) {
            QObject *target = _view->findChild<QObject *>(ALL_DELETE_BUTTON);
            if (target) {
                target->setProperty("disable", !fileExist);
            }
        }

        // 全ファイルのリストから表示するページに対応するリストを作成
        QStringList files;
        for (int i = 0; i < FILE_LIST_ROWS; ++i) {
            int index = i + (_currentFileListPage - 1) * FILE_LIST_ROWS;
            if (_fileList.count() == 0) {
                files.append(QString("未設定"));
            } else {
                if(_fileList[index] == QString("")) {
                    files.append(QString("未設定"));
                } else {
                    files.append(_fileList[index]);
                }
            }
        }

        // リストを表示
        if (_view) {
            _view->setProperty("currentTopFileListIndex", 1 + (_currentFileListPage - 1) * FILE_LIST_ROWS);
        }
        if (_view) {
            _view->setProperty("fileList", QVariant::fromValue(files));
        }
        if (_view) {
            _view->setProperty("page", QString("ページ %1/%2").arg(_currentFileListPage).arg(MAXIMUM_FILE_LIST_PAGE));
        }
        if (_view) {
            // USER データロガー設定では先頭、保存先選択では選択されたインデックスをアクティブにする
            _view->setProperty("currentSelectedFileListIndex", _currentFileIndex);
        }
    }
    if (_view) {
        // ファイルリスト更新直後は未編集状態に戻す
        _view->setProperty("isFileEditing", false);
    }

    // ファイルリストの更新後は、チャンネルリストを表示
//    if (_mode == CHANNEL_EDIT_SELECT_DESTINATION) {
//        showChannelList(true);
//    } else {
        // 選択ファイルが変わるので件数をクリアー
        _bandModel->initialize();
        showChannelList(true);
//    }
}

#if 0
void UserDataLoggerSettingViewModel::showChannelList(bool noReadSettingFile)
{
    // USER 設定で単一 Band の許可のみの場合
    if (_userSetingBehavior == ALLOW_SINGLE_BAND_ONLY) {
        // USER 設定で Band を切り替える場合、件数をクリアーしておく
        if (_mode == CHANNEL_EDIT_USER || _mode == CHANNEL_EDIT_USER_USB1 || _mode == CHANNEL_EDIT_USER_USB2) {
            _bandModel->initialize();
        }
    }

    QList<LC70_LOGGER_SET_LOGGER_ROW_T> settingFileContents;
    LC70_BAND_T band = LC70_BAND_UNDEF;     // 最初の有効な Band
    QString channel;                        // 最初の有効なチャンネル

    // USER データロガー設定ファイルを初期化する
    retrieveSettingFile(settingFileContents, band, channel, noReadSettingFile);

    // 設定ファイルがある場合は 最初の有効なチャンネルの Band とチャンネル名から Band Tab を引き当て設定する
    if (hasSettingFile() && noReadSettingFile == false) {
        int candidate = identifyBandTab(band, channel);

        // Band が異なる場合はタブの表示切り替え
//        if (candidate != _currentTabIndex) {
            _currentTabIndex = candidate;
            _bandModel->select(static_cast<BandTabIndex>(_currentTabIndex));
//        }
        // 全 Band 未選択状態ではない
        _bandModel->unselectAll(false);

    } else {
    // 設定ファイルがないとき
        // USER 設定で単一 Band の許可のみの場合
        if (_userSetingBehavior == ALLOW_SINGLE_BAND_ONLY) {
            if (_mode == CHANNEL_EDIT_DATA_LOGGER || _mode == CHANNEL_EDIT_DATA_LOGGER_USB1 || _mode == CHANNEL_EDIT_DATA_LOGGER_USB2) {
                if (unregisteredDataLoggerChannelCount() == 0) {
                    // 全 Band 未選択状態
                    _bandModel->unselectAll(true);
                } else {
                    // 全 Band 未選択状態ではない
                    _bandModel->unselectAll(false);
                }
            } else {
                // 全 Band 未選択状態
                _bandModel->unselectAll(true);
            }
        } else {
            if (_mode != CHANNEL_EDIT_SELECT_DESTINATION) {
                if (unregisteredDataLoggerChannelCount() == 0) {
                    // 全 Band 未選択状態
                    _bandModel->unselectAll(true);
                } else {
                    // 全 Band 未選択状態ではない
                    _bandModel->unselectAll(false);
                }
            } else {
                // 全 Band 未選択状態
                _bandModel->unselectAll(true);
            }
        }
    }

    // 現在選択されている Band を特定
    band = TAB_BAND_MAP.value(_currentTabIndex, LC70_BAND_UNDEF);

    // 選択中のタブのチャンネル定義を取得
    QList<ChannelDefinitionItem> channelDefinitionList;
    retrieveChannels(channelDefinitionList);
    _forcedBandChange = false;

    // 測定内容のリストにチャンネルごとの測定内容を反映
    populateChannels(band, channelDefinitionList, settingFileContents);

    if (_view) {
        // 測定内容を画面に反映
        _view->setProperty("channelList", QVariant::fromValue(_channelList));

        // チャンネルリストの文字色反映
        _view->setProperty("channelListColor", getChannelListColor(_currentTabIndex, _mode));
    }
    if (_view) {
        _view->setProperty("channelTargetAll", false);
    }

    if (_channelList.size() <= _currentChannelIndex) {
        // Bandを変更した際、選択インデックスは維持する
        // 変更先のBandのチャンネル数が現在のBandより少なく、選択位置を維持できない場合は末尾を選択
        _currentChannelIndex = _channelList.size() - 1;
    }

    // チャンネルリストの表示位置を更新
    selectTargetRow();

    // チャンネル数を更新
    hasActiveChannel();
}
#endif

void UserDataLoggerSettingViewModel::showChannelList(bool fileRead)
{
    QString channel;
    QList<UserDataLoggerSettingFile> settingFile;

    if (fileRead) {
        QString fileName = _fileList[_currentFileIndex + (_currentFileListPage - 1) * FILE_LIST_ROWS];
        if (!fileName.contains(CSV_FILE_EXTENSION)) {
            fileName += QString(CSV_FILE_EXTENSION);
        }
        qDebug() << QString("index: %1 fileName: %2").arg(_currentFileIndex + (_currentFileListPage - 1) * FILE_LIST_ROWS).arg(fileName);

        // 既存の設定ファイルあるいはテンプレートファイルを読み込む
        if (!_model->importSettingFile(_mode, fileName)) {
            qDebug() << QString("Failed to import user datalogger setting file");
        }
        BandTabIndex bandTabIndex = (fileRead) ? BAND_TAB_UHF : static_cast<BandTabIndex>(_currentTabIndex);
        // 設定ファイル中の最初に測定するが定義されている Band を特定する
        _model->selectFirstValidChannel(bandTabIndex, channel);
        _currentTabIndex = bandTabIndex;
        settingFile = _model->selectEditingChannels(static_cast<BandTabIndex>(_currentTabIndex));
    } else {
        settingFile = _model->selectEditingChannels(static_cast<BandTabIndex>(_currentTabIndex));
        if (settingFile.count() > 0) {
            channel = settingFile[0].channel;
        }
    }

    LC70_BAND_T band = TAB_BAND_MAP.value(_currentTabIndex, LC70_BAND_UNDEF);

    int candidate = identifyBandTab(band, channel);

    // Band が異なる場合はタブの表示切り替え
    _currentTabIndex = candidate;
    _bandModel->select(static_cast<BandTabIndex>(_currentTabIndex));

    // 設定ファイルの有無で Band タブの表示を切り替える
    if (hasSettingFile()) {
        // 全 Band 未選択状態ではない
        _bandModel->unselectAll(false);
    } else {
        if (_mode != CHANNEL_EDIT_SELECT_DESTINATION) {
            if (unregisteredDataLoggerChannelCount() == 0) {
                // 全 Band 未選択状態
                _bandModel->unselectAll(true);
            } else {
                // 全 Band 未選択状態ではない
                _bandModel->unselectAll(false);
            }
        } else {
            // 全 Band 未選択状態
            _bandModel->unselectAll(true);
        }
    }

    // 選択中のタブのチャンネル定義を取得
    QList<ChannelDefinitionItem> channelDefinitionList;
    retrieveChannels(settingFile, channelDefinitionList);

    // 測定内容のリストにチャンネルごとの測定内容を反映
    populateChannels(band, channelDefinitionList, settingFile);

    if (_view) {
        // 測定内容を画面に反映
        _view->setProperty("channelList", QVariant::fromValue(_channelList));

        // チャンネルリストの文字色反映
        _view->setProperty("channelListColor", getChannelListColor(_currentTabIndex, _mode));
    }
    if (_view) {
        _view->setProperty("channelTargetAll", false);
    }

    if (_channelList.size() <= _currentChannelIndex) {
        // Bandを変更した際、選択インデックスは維持する
        // 変更先のBandのチャンネル数が現在のBandより少なく、選択位置を維持できない場合は末尾を選択
        _currentChannelIndex = _channelList.size() - 1;
    }

    // チャンネルリストの表示位置を更新
    selectTargetRow();

    // チャンネル数を更新
    hasActiveChannel();
}

bool UserDataLoggerSettingViewModel::hasSettingFile()
{
    if (_fileList.count() == 0) {
        return false;
    }
    QString fileName = _fileList[_currentFileIndex + (_currentFileListPage - 1) * FILE_LIST_ROWS];
    _model->writeLog(QString("USER データロガー設定"), QString("選択ファイル名: %1").arg(fileName), LOG_NOTICE);
    return !fileName.isEmpty();
}

void UserDataLoggerSettingViewModel::readSettingFile()
{
    if (_fileList.count() == 0) {
        return;
    }

    QString fileName = _fileList[_currentFileIndex + (_currentFileListPage - 1) * FILE_LIST_ROWS];
    if (!fileName.contains(CSV_FILE_EXTENSION)) {
        fileName += QString(CSV_FILE_EXTENSION);
    }
    _model->importSettingFile(_mode, fileName);
}

void UserDataLoggerSettingViewModel::keyStatusFileListArea(ButtonId id)
{
    // 上下ボタン移動で設定ファイルの表示切替を行い、右ボタンで測定内容の編集に移動、Band ボタンで Band タブを切り替え
    if (id == BUTTON_UP || id == BUTTON_DOWN) {
        moveCursorFileListArea(id);
    } else if (id == BUTTON_RIGHT) {
        if (_mode == CHANNEL_EDIT_SELECT_DESTINATION) {
            moveArea(ACTION_AREA);
        } else {
            // USER 設定では CATV と JCSAT 系、SCC 系のみ編集可能
            // データロガー設定では 全 Band 給電が編集可能、MER/CN, BER を測定する Band は MER/CN, BER を編集可能
            if (_currentTabIndex == BAND_TAB_UHF  ||
                _currentTabIndex == BAND_TAB_BS_R ||
                _currentTabIndex == BAND_TAB_BS_L ||
                _currentTabIndex == BAND_TAB_CS_R ||
                _currentTabIndex == BAND_TAB_CS_L ||
                _currentTabIndex == BAND_TAB_SKY_PERFECTV) {

                if (_mode == CHANNEL_EDIT_USER || _mode == CHANNEL_EDIT_USER_USB1 || _mode == CHANNEL_EDIT_USER_USB2) {
                    // 測定 ON/OFF のみ編集可能
                    _currentEditPositon = EDIT_POSITION_MEASURE;
                    moveArea(TARGET_AREA);
                } else if (_mode == CHANNEL_EDIT_DATA_LOGGER ||
                           _mode == CHANNEL_EDIT_DATA_LOGGER_USB1 ||
                           _mode == CHANNEL_EDIT_DATA_LOGGER_USB2) {
                    _currentEditPositon = EDIT_POSITION_POWERSUPPLY;
                    moveArea(CHANNEL_AREA);
                }
            } else if (_currentTabIndex == BAND_TAB_LTE || _currentTabIndex == BAND_TAB_PILOT) {
                if (_mode == CHANNEL_EDIT_USER || _mode == CHANNEL_EDIT_USER_USB1 || _mode == CHANNEL_EDIT_USER_USB2) {
                    // 測定 ON/OFF のみ編集可能
                    _currentEditPositon = EDIT_POSITION_MEASURE;
                    moveArea(TARGET_AREA);
                } else if (_mode == CHANNEL_EDIT_DATA_LOGGER ||
                           _mode == CHANNEL_EDIT_DATA_LOGGER_USB1 ||
                           _mode == CHANNEL_EDIT_DATA_LOGGER_USB2) {
                    _currentEditPositon = EDIT_POSITION_POWERSUPPLY;
                    moveArea(CHANNEL_AREA);
                }
            } else if (_currentTabIndex == BAND_TAB_CATV_ALL) {
                // 放送規格の編集
                _currentEditPositon = EDIT_POSITION_BROADCAST;
                moveArea(CHANNEL_AREA);
            } else if (_currentTabIndex == BAND_TAB_JCSAT3A_H ||
                        _currentTabIndex == BAND_TAB_JCSAT3A_V ||
                        _currentTabIndex == BAND_TAB_JCSAT4B_H ||
                        _currentTabIndex == BAND_TAB_JCSAT4B_V ||
                        _currentTabIndex == BAND_TAB_SCCB2_H ||
                        _currentTabIndex == BAND_TAB_SCCB2_V ||
                        _currentTabIndex == BAND_TAB_SCCC2_H ||
                        _currentTabIndex == BAND_TAB_SCCC2_V) {
                // 局発周波数の編集
                _currentEditPositon = EDIT_POSITION_LOCALFREQ;
                moveArea(CHANNEL_AREA);
            } else if (_currentTabIndex == BAND_TAB_FM || _currentTabIndex == BAND_TAB_CATV_UPSTREAM) {
                _currentEditPositon = EDIT_POSITION_FREQUENCY;
                moveArea(CHANNEL_AREA);
            }
        }
    } else if (id == BUTTON_BAND) {
        changeBand(true);
    } else if (id == BUTTON_MENU) {
        changeBand(false);
    } else if (id == BUTTON_ENTER) {
        // 選択中のファイルリストのインデックス
        int index = _currentFileIndex + (_currentFileListPage - 1) * FILE_LIST_ROWS;
        if(_fileList[index] == QString("")) {
            // BUTTON_RIGHT と同じ処理
            if (_mode == CHANNEL_EDIT_SELECT_DESTINATION) {
                moveArea(ACTION_AREA);
            } else {
                // USER 設定では CATV と JCSAT 系、SCC 系のみ編集可能
                // データロガー設定では 全 Band 給電が編集可能、MER/CN, BER を測定する Band は MER/CN, BER を編集可能
                if (_currentTabIndex == BAND_TAB_UHF  ||
                    _currentTabIndex == BAND_TAB_BS_R ||
                    _currentTabIndex == BAND_TAB_BS_L ||
                    _currentTabIndex == BAND_TAB_CS_R ||
                    _currentTabIndex == BAND_TAB_CS_L ||
                    _currentTabIndex == BAND_TAB_SKY_PERFECTV) {

                    if (_mode == CHANNEL_EDIT_USER || _mode == CHANNEL_EDIT_USER_USB1 || _mode == CHANNEL_EDIT_USER_USB2) {
                        // 測定 ON/OFF のみ編集可能
                        _currentEditPositon = EDIT_POSITION_MEASURE;
                        moveArea(TARGET_AREA);
                    } else if (_mode == CHANNEL_EDIT_DATA_LOGGER ||
                               _mode == CHANNEL_EDIT_DATA_LOGGER_USB1 ||
                               _mode == CHANNEL_EDIT_DATA_LOGGER_USB2) {
                        _currentEditPositon = EDIT_POSITION_POWERSUPPLY;
                        moveArea(CHANNEL_AREA);
                    }
                } else if (_currentTabIndex == BAND_TAB_LTE || _currentTabIndex == BAND_TAB_PILOT) {
                    if (_mode == CHANNEL_EDIT_USER || _mode == CHANNEL_EDIT_USER_USB1 || _mode == CHANNEL_EDIT_USER_USB2) {
                        // 測定 ON/OFF のみ編集可能
                        _currentEditPositon = EDIT_POSITION_MEASURE;
                        moveArea(TARGET_AREA);
                    } else if (_mode == CHANNEL_EDIT_DATA_LOGGER ||
                               _mode == CHANNEL_EDIT_DATA_LOGGER_USB1 ||
                               _mode == CHANNEL_EDIT_DATA_LOGGER_USB2) {
                        _currentEditPositon = EDIT_POSITION_POWERSUPPLY;
                        moveArea(CHANNEL_AREA);
                    }
                } else if (_currentTabIndex == BAND_TAB_CATV_ALL) {
                    // 放送規格の編集
                    _currentEditPositon = EDIT_POSITION_BROADCAST;
                    moveArea(CHANNEL_AREA);
                } else if (_currentTabIndex == BAND_TAB_JCSAT3A_H ||
                            _currentTabIndex == BAND_TAB_JCSAT3A_V ||
                            _currentTabIndex == BAND_TAB_JCSAT4B_H ||
                            _currentTabIndex == BAND_TAB_JCSAT4B_V ||
                            _currentTabIndex == BAND_TAB_SCCB2_H ||
                            _currentTabIndex == BAND_TAB_SCCB2_V ||
                            _currentTabIndex == BAND_TAB_SCCC2_H ||
                            _currentTabIndex == BAND_TAB_SCCC2_V) {
                    // 局発周波数の編集
                    _currentEditPositon = EDIT_POSITION_LOCALFREQ;
                    moveArea(CHANNEL_AREA);
                } else if (_currentTabIndex == BAND_TAB_FM || _currentTabIndex == BAND_TAB_CATV_UPSTREAM) {
                    _currentEditPositon = EDIT_POSITION_FREQUENCY;
                    moveArea(CHANNEL_AREA);
                }
            }
        } else {
// 2021.6.16 DX アンテナ検討中
//             光測定選択時に CATV 上りを含む設定は測定不可
//            if (hasCATVUpstreamChannel()) {
//                if (isOpticalMeasurementSelected()) {
//                    emit requestToShowPopupWithMessage(POPUP_MEASURE_CATV_UPSTREAM_NOT_APPLICABLE,
//                                                       QString(MEASURING_CATV_UPSTREAM_NOT_APPLICABLE));
//                    return;
//                }
//            }

            // 測定開始と同じ処理
            if (_mode == CHANNEL_EDIT_USER || _mode == CHANNEL_EDIT_USER_USB1 || _mode == CHANNEL_EDIT_USER_USB2) {
                readSettingFile();  // 編集時に FM, CATV 上りを 150 件の中から消しているので再読込
                setTakeOver();
                emit requestSetTakeOver("fromUserSetting", true);
                emit requestSetTakeOver("channelEditMode", _mode);
                emit requestSetTakeOver("userSettingFileName", _fileList[_currentFileIndex + (_currentFileListPage - 1) * FILE_LIST_ROWS]);

                // USER 設定で複数の Band を跨ぐ場合
                if (hasMultiBandsInUserSetting()) {
                    emit requestSetTakeOver("hasMultiBandsInUserSetting", true);
                } else {
                    emit requestSetTakeOver("hasMultiBandsInUserSetting", false);
                }

                onRequest(0);
            } else if (_mode == CHANNEL_EDIT_DATA_LOGGER || _mode == CHANNEL_EDIT_DATA_LOGGER_USB1 || _mode == CHANNEL_EDIT_DATA_LOGGER_USB2) {
                // オートパワーオフがオフでなければポップアップで実行確認
//                QVariant value;

//                if (_model->getValue(VALUE_AUTOPOWEROFF, value)) {
//                    if (value.toInt() != LC70_SYS_SET_POWOFF_OFF) {
//                        emit requestToShowPopup(POPUP_CONFIRM_EXECUTE_WITH_AUTO_POWER_OFF);
//                        return;
//                    }
//                }
                // データロガー実行
                executeDataLogger();
            } else if (_mode == CHANNEL_EDIT_SELECT_DESTINATION) {
                moveArea(ACTION_AREA);
            }
        }
    }
}

void UserDataLoggerSettingViewModel::moveCursorFileListArea(ButtonId id)
{
    // ファイルリストの選択行の移動
    if (id == BUTTON_UP) {
        --_currentFileIndex;
        if (_currentFileIndex < 0) {
            _currentFileIndex = FILE_LIST_ROWS - 1;

            --_currentFileListPage;
            if (_currentFileListPage < 1) {
                _currentFileListPage = MAXIMUM_FILE_LIST_PAGE;
            }
            showSettingFileList();
        }
    } else if (id == BUTTON_DOWN) {
        ++_currentFileIndex;
        if (_currentFileIndex >= FILE_LIST_ROWS) {
            _currentFileIndex = 0;

            ++_currentFileListPage;
            if (_currentFileListPage > MAXIMUM_FILE_LIST_PAGE) {
                _currentFileListPage = 1;
            }

            showSettingFileList();
        }
    }

    if (_view) {
        _view->setProperty("currentSelectedFileListIndex", _currentFileIndex);
    }

    // USER 設定で単一 Band の許可のみの場合
    if (_userSetingBehavior == ALLOW_SINGLE_BAND_ONLY) {
        // データロガー設定でファイルを変更した場合
        if (_mode == CHANNEL_EDIT_DATA_LOGGER || _mode == CHANNEL_EDIT_DATA_LOGGER_USB1 || _mode == CHANNEL_EDIT_DATA_LOGGER_USB2) {

            // 選択ファイルが変わるので件数をクリアー
            _bandModel->initialize();
        }
    } else {
        // USER 設定、データロガー設定でファイルを変更した場合
//        if (_mode != CHANNEL_EDIT_SELECT_DESTINATION) {

            // 選択ファイルが変わるので件数をクリアー
            _bandModel->initialize();
//        }
    }

    // 地デジを選択
    _currentTabIndex = BAND_TAB_UHF;
    _bandModel->select(static_cast<BandTabIndex>(_currentTabIndex));

    // 選択しているファイルが変更されたらチャンネルリストにファイルの内容を反映
    _currentChannelIndex = 0;

    // 選択しているファイルが変更されたらチャンネルリストにファイルの内容を反映
    showChannelList(true);

}

void UserDataLoggerSettingViewModel::keyStatusChannelListArea(ButtonId id)
{
    // チャンネルリストにフォーカスがある場合のボタン操作
    if (id == BUTTON_UP || id == BUTTON_DOWN) {
        moveCursorChannelListArea(id);
    } else if (id == BUTTON_LEFT) {
        if(movePreviousEditItem() == true) {
            // 測定内容を更新してファイルリストに戻る場合を考慮
            moveCursorChannelListArea(id);
        }
    } else if (id == BUTTON_RIGHT) {
        if(moveNextEditItem() == true) {
            moveArea(TARGET_AREA);
        }
    } else if (id == BUTTON_ENTER) {
        // 保存先選択時は使用不可
        if (_mode != CHANNEL_EDIT_SELECT_DESTINATION) {
            _useFrequencyInput = false;  // 長押し通常速度

            if (_currentTabIndex == BAND_TAB_FM || _currentTabIndex == BAND_TAB_CATV_UPSTREAM) {
                _useFrequencyInput = true;  // 長押し高速化
            }

            setEditItemList();
            moveArea(CHANNEL_EDIT);
        }
    } else if (id == BUTTON_BAND) {
        changeBand(true);
    } else if (id == BUTTON_MENU) {
        changeBand(false);
    }
}

void UserDataLoggerSettingViewModel::moveCursorChannelListArea(ButtonId id)
{
    // チャンネルリストの選択行の移動
    if (id == BUTTON_UP) {
        if (--_currentChannelIndex < 0) {
            _currentChannelIndex = _channelList.length() - 1;
        }
    } else if (id == BUTTON_DOWN) {
        if (++_currentChannelIndex >= _channelList.length()) {
            _currentChannelIndex = 0;
        }
    } else if (id == BUTTON_LEFT) {
        // データ編集されていた場合にはダイアログ表示(現在はこの条件が真になることはない)
        if (_model->isUpdate(settingListValueId())) {
            // データ破棄の確認POPUP
            emit requestToShowPopup(POPUP_CANCEL_CONFIRMATION);
        } else {
            // 現在はこの分岐しか通らない
            onAccept(POPUP_CANCEL_CONFIRMATION, true);
        }
    }

    selectTargetRow();
}

void UserDataLoggerSettingViewModel::keyStatusChannelEdit(ButtonId id)
{
    if (id == BUTTON_UP || id == BUTTON_DOWN) {
        editItem(id);
    } else if (id == BUTTON_LEFT) {
        movePreviousEditItem();
    } else if (id == BUTTON_RIGHT) {
        moveNextEditItem();
    } else if (id == BUTTON_ENTER) {
        _useFrequencyInput = false;  // 長押し通常速度
        updateSetting();
        moveArea(CHANNEL_AREA);
    }
}

void UserDataLoggerSettingViewModel::keyStatusTargetListArea(ButtonId id)
{
    // 測定 ON/OFF リストにフォーカスがある場合のボタン操作
    if (id == BUTTON_UP || id == BUTTON_DOWN) {
        moveCursorTargetListArea(id);
    } else if (id == BUTTON_LEFT) {
        if (_mode == CHANNEL_EDIT_SELECT_DESTINATION) {
            moveArea(CHANNEL_AREA);
        } else {
            // USER 設定では CATV と JCSAT 系、SCC 系のみ編集可能
            // データロガー設定では 全 Band 給電が編集可能、MER/CN, BER を測定する Band は MER/CN, BER を編集可能
            if (_currentTabIndex == BAND_TAB_UHF ||
                _currentTabIndex == BAND_TAB_BS_R ||
                _currentTabIndex == BAND_TAB_BS_L ||
                _currentTabIndex == BAND_TAB_CS_R ||
                _currentTabIndex == BAND_TAB_CS_L ||
                _currentTabIndex == BAND_TAB_SKY_PERFECTV) {

                if (_mode == CHANNEL_EDIT_USER || _mode == CHANNEL_EDIT_USER_USB1 || _mode == CHANNEL_EDIT_USER_USB2) {
                    undo();
                    // 測定 ON/OFF のみ編集可能
                    moveArea(FILELIST_AREA);
                } else if (_mode == CHANNEL_EDIT_DATA_LOGGER ||
                           _mode == CHANNEL_EDIT_DATA_LOGGER_USB1 ||
                           _mode == CHANNEL_EDIT_DATA_LOGGER_USB2) {
                    _currentEditPositon = EDIT_POSITION_POWERSUPPLY;
                    moveArea(CHANNEL_AREA);
                }
            } else if (_currentTabIndex == BAND_TAB_LTE || _currentTabIndex == BAND_TAB_PILOT) {
                if (_mode == CHANNEL_EDIT_USER || _mode == CHANNEL_EDIT_USER_USB1 || _mode == CHANNEL_EDIT_USER_USB2) {
                    undo();
                    // 測定 ON/OFF のみ編集可能
                    moveArea(FILELIST_AREA);
                } else if (_mode == CHANNEL_EDIT_DATA_LOGGER ||
                           _mode == CHANNEL_EDIT_DATA_LOGGER_USB1 ||
                           _mode == CHANNEL_EDIT_DATA_LOGGER_USB2) {
                    _currentEditPositon = EDIT_POSITION_POWERSUPPLY;
                    moveArea(CHANNEL_AREA);
                }
            } else if (_currentTabIndex == BAND_TAB_CATV_ALL) {
                if (_mode == CHANNEL_EDIT_USER || _mode == CHANNEL_EDIT_USER_USB1 || _mode == CHANNEL_EDIT_USER_USB2) {
                    // 放送方式の編集
                    _currentEditPositon = EDIT_POSITION_BROADCAST;
                    moveArea(CHANNEL_AREA);
                } else if (_mode == CHANNEL_EDIT_DATA_LOGGER ||
                           _mode == CHANNEL_EDIT_DATA_LOGGER_USB1 ||
                           _mode == CHANNEL_EDIT_DATA_LOGGER_USB2) {
                    _currentEditPositon = EDIT_POSITION_POWERSUPPLY;
                    moveArea(CHANNEL_AREA);
                }
            } else if (_currentTabIndex == BAND_TAB_JCSAT3A_H  ||
                        _currentTabIndex == BAND_TAB_JCSAT3A_V ||
                        _currentTabIndex == BAND_TAB_JCSAT4B_H ||
                        _currentTabIndex == BAND_TAB_JCSAT4B_V ||
                        _currentTabIndex == BAND_TAB_SCCB2_H   ||
                        _currentTabIndex == BAND_TAB_SCCB2_V   ||
                        _currentTabIndex == BAND_TAB_SCCC2_H   ||
                        _currentTabIndex == BAND_TAB_SCCC2_V) {
                if (_mode == CHANNEL_EDIT_USER || _mode == CHANNEL_EDIT_USER_USB1 || _mode == CHANNEL_EDIT_USER_USB2) {
                    // 局発周波数の編集
                    _currentEditPositon = EDIT_POSITION_LOCALFREQ;
                    moveArea(CHANNEL_AREA);
                } else if (_mode == CHANNEL_EDIT_DATA_LOGGER ||
                           _mode == CHANNEL_EDIT_DATA_LOGGER_USB1 ||
                           _mode == CHANNEL_EDIT_DATA_LOGGER_USB2) {
                    _currentEditPositon = EDIT_POSITION_POWERSUPPLY;
                    moveArea(CHANNEL_AREA);
                }
            } else if (_currentTabIndex == BAND_TAB_FM || _currentTabIndex == BAND_TAB_CATV_UPSTREAM) {
                if (_mode == CHANNEL_EDIT_USER || _mode == CHANNEL_EDIT_USER_USB1 || _mode == CHANNEL_EDIT_USER_USB2) {
                    // 周波数 のみ編集可能
                    _currentEditPositon = EDIT_POSITION_FREQUENCY;
                    moveArea(CHANNEL_AREA);
                } else if (_mode == CHANNEL_EDIT_DATA_LOGGER ||
                           _mode == CHANNEL_EDIT_DATA_LOGGER_USB1 ||
                           _mode == CHANNEL_EDIT_DATA_LOGGER_USB2) {
                    _currentEditPositon = EDIT_POSITION_POWERSUPPLY;
                    moveArea(CHANNEL_AREA);
                }
            }
        }
    } else if (id == BUTTON_RIGHT) {
        moveArea(ACTION_AREA);
    } else if (id == BUTTON_ENTER) {
        // 保存先選択時は使用不可
        if (_mode != CHANNEL_EDIT_SELECT_DESTINATION) {
            moveArea(TARGET_EDIT);
        }
    } else if (id == BUTTON_BAND) {
        changeBand(true);
    } else if (id == BUTTON_MENU) {
        changeBand(false);
    }
}

void UserDataLoggerSettingViewModel::moveCursorTargetListArea(ButtonId id)
{
    // 測定 ON/OFF リストの選択行の移動
    if (id == BUTTON_UP) {
        if (--_currentTargetIndex < 0) {
            if (_view) {
                _view->setProperty("currentSelectedTargetAllIndex", 1);
            }
            moveArea(TARGET_ALL);
            return;
        }
    } else if (id == BUTTON_DOWN) {
        if (++_currentTargetIndex >= _channelList.length()) {
            if (_view) {
                _view->setProperty("currentSelectedTargetAllIndex", 1);
            }
            moveArea(TARGET_ALL);
            return;
        }
    }
    // チャンネルリストと連動させる
    _currentChannelIndex = _currentTargetIndex;

    // 測定内容リストの何ページ目かを示すインデックス
    int page = (_currentChannelIndex + 1) / CHANNEL_LIST_ROWS;
    if ((_currentChannelIndex + 1) % CHANNEL_LIST_ROWS > 0) {
        ++page;
    }
    int channelListIndex = (page - 1) * CHANNEL_LIST_ROWS;
    int selectedIndex = _currentChannelIndex % CHANNEL_LIST_ROWS;
    if (_view) {
        _view->setProperty("currentTopChannelListIndex", channelListIndex);
    }
    if (_view) {
        _view->setProperty("currentSelectedTargetListIndex", selectedIndex);
    }
    // 測定 ON/OFF リストのインデックス
    if (_view) {
        _view->setProperty("currentTopTargetListIndex", channelListIndex);
    }
    if (_view) {
        _view->setProperty("currentSelectedChannelListIndex", selectedIndex);
    }
}

void UserDataLoggerSettingViewModel::keyStatusTargetEdit(ButtonId id)
{
    // 保存先選択時は使用不可
    if (_mode == CHANNEL_EDIT_SELECT_DESTINATION) {
        return;
    }

    // 測定 ON/OFF の編集
    _currentEditPositon = EDIT_POSITION_MEASURE;
    setEditItemList();

    if (id == BUTTON_UP || id == BUTTON_DOWN) {
        editItem(id);
    } else if (id == BUTTON_ENTER) {
        updateSetting();
        moveArea(TARGET_AREA);
    }
}

void UserDataLoggerSettingViewModel::keyStatusActionArea(ButtonId id)
{
    if (id == BUTTON_UP || id == BUTTON_DOWN) {
        moveCursorActionArea(id);
    } else if (id == BUTTON_LEFT) {
        if (_mode == CHANNEL_EDIT_SELECT_DESTINATION) {
            moveArea(FILELIST_AREA);
        } else {
            moveArea(TARGET_AREA);
        }
    } else if (id == BUTTON_BAND) {
        changeBand(true);
    } else if (id == BUTTON_MENU) {
        changeBand(false);
    } else if (id == BUTTON_ENTER) {
        QObject *target = nullptr;

        if(_selectArea == ACTION_AREA) {
            switch (_currentActionIndex) {
            case ACTION_INDEX_ALL_DELETE:
                if (_view) {
                    target = _view->findChild<QObject *>(ALL_DELETE_BUTTON);
                }
                break;
            case ACTION_INDEX_SAVE:
                if (_view) {
                    target = _view->findChild<QObject *>(SAVE_BUTTON);
                }
                break;
            case ACTION_INDEX_DELETE:
                if (_view) {
                    target = _view->findChild<QObject *>(DELETE_BUTTON);
                }
                break;
            case ACTION_INDEX_DECISION:
                if (_view) {
                    target = _view->findChild<QObject *>(SCAN_SAVE_BUTTON);
                }
                break;
            default:
                break;
            }
            if (target) {
                target->setProperty("press", false);
            }
        }
        if (_currentActionIndex == ACTION_INDEX_ALL_DELETE) {
            emit requestToShowPopup(POPUP_CONFIRM_ALL_DELETE_FILE);
        } else if (_currentActionIndex == ACTION_INDEX_SAVE) {
            emit requestToShowPopup(POPUP_SAVE_CONFIRMATION);
        } else if (_currentActionIndex == ACTION_INDEX_DELETE) {
            emit requestToShowPopup(POPUP_CONFIRM_DELETE_SELECTED_FILE);
        } else if (_currentActionIndex == ACTION_INDEX_DECISION) {
            if (hasSettingFile()) {
                emit requestToShowPopup(POPUP_SELECT_SAVE_CONFIRMATION);
            } else {
                onAccept(POPUP_SELECT_SAVE_CONFIRMATION, true);
            }
        }
    }
}

void UserDataLoggerSettingViewModel::moveCursorActionArea(ButtonId id)
{
    qDebug() << QString("moveCursorActionArea: %1").arg(id == BUTTON_UP ? "UP" : "DOWN");
    int nextAction = _currentActionIndex;
//    for (int i = nextAction; i < 2; id == BUTTON_UP ? i-- : i++) {
    do {
        nextAction = nextAction + (id == BUTTON_UP ? -1 : 1);
        qDebug() << QString("moveCursorActionArea: next=%1").arg(nextAction);
        if (nextAction < ACTION_INDEX_ALL_DELETE) {
            nextAction = ACTION_INDEX_DELETE;
        }
        if (nextAction > ACTION_INDEX_DELETE) {
            nextAction = ACTION_INDEX_ALL_DELETE;
        }
        if ((nextAction == ACTION_INDEX_ALL_DELETE && canAllDelete())
                || (nextAction == ACTION_INDEX_SAVE && canSave())
                || (nextAction == ACTION_INDEX_DELETE && canDelete())) {
            qDebug() << QString("moveCursorActionArea: %1 -> %2").arg(_currentActionIndex).arg(nextAction);
            _currentActionIndex = nextAction;
            if (_view) {
                _view->setProperty("currentActionIndex", _currentActionIndex);
            }
            return;
        }
        if (_currentActionIndex == ACTION_INDEX_DECISION) {
            return;
        }
    } while(nextAction != _currentActionIndex);
}


void UserDataLoggerSettingViewModel::keyStatusTargetAllArea(ButtonId id)
{
    // 測定 全てON/OFF にフォーカスがある場合のボタン操作
    if (id == BUTTON_UP || id == BUTTON_DOWN) {
        moveCursorTargetAllArea(id);
        moveArea(TARGET_AREA);
    } else if (id == BUTTON_LEFT) {
        // 測定ON/OFFのBUTTON_LEFTと同じ処理
        if (_mode == CHANNEL_EDIT_SELECT_DESTINATION) {
            moveArea(CHANNEL_AREA);
        } else {
            // USER 設定では CATV と JCSAT 系、SCC 系のみ編集可能
            // データロガー設定では 全 Band 給電が編集可能、MER/CN, BER を測定する Band は MER/CN, BER を編集可能
            if (_currentTabIndex == BAND_TAB_UHF ||
                _currentTabIndex == BAND_TAB_BS_R ||
                _currentTabIndex == BAND_TAB_BS_L ||
                _currentTabIndex == BAND_TAB_CS_R ||
                _currentTabIndex == BAND_TAB_CS_L ||
                _currentTabIndex == BAND_TAB_SKY_PERFECTV) {

                if (_mode == CHANNEL_EDIT_USER || _mode == CHANNEL_EDIT_USER_USB1 || _mode == CHANNEL_EDIT_USER_USB2) {
                    // 測定 ON/OFF のみ編集可能
                    moveArea(FILELIST_AREA);
                } else if (_mode == CHANNEL_EDIT_DATA_LOGGER ||
                           _mode == CHANNEL_EDIT_DATA_LOGGER_USB1 ||
                           _mode == CHANNEL_EDIT_DATA_LOGGER_USB2) {
                    _currentEditPositon = EDIT_POSITION_POWERSUPPLY;
                    moveArea(CHANNEL_AREA);
                }
            } else if (_currentTabIndex == BAND_TAB_LTE || _currentTabIndex == BAND_TAB_PILOT) {
                if (_mode == CHANNEL_EDIT_USER || _mode == CHANNEL_EDIT_USER_USB1 || _mode == CHANNEL_EDIT_USER_USB2) {
                    // 測定 ON/OFF のみ編集可能
                    moveArea(FILELIST_AREA);
                } else if (_mode == CHANNEL_EDIT_DATA_LOGGER ||
                           _mode == CHANNEL_EDIT_DATA_LOGGER_USB1 ||
                           _mode == CHANNEL_EDIT_DATA_LOGGER_USB2) {
                    _currentEditPositon = EDIT_POSITION_POWERSUPPLY;
                    moveArea(CHANNEL_AREA);
                }
            } else if (_currentTabIndex == BAND_TAB_CATV_ALL) {
                if (_mode == CHANNEL_EDIT_USER || _mode == CHANNEL_EDIT_USER_USB1 || _mode == CHANNEL_EDIT_USER_USB2) {
                    // 放送方式の編集
                    _currentEditPositon = EDIT_POSITION_BROADCAST;
                    moveArea(CHANNEL_AREA);
                } else if (_mode == CHANNEL_EDIT_DATA_LOGGER ||
                           _mode == CHANNEL_EDIT_DATA_LOGGER_USB1 ||
                           _mode == CHANNEL_EDIT_DATA_LOGGER_USB2) {
                    _currentEditPositon = EDIT_POSITION_POWERSUPPLY;
                    moveArea(CHANNEL_AREA);
                }
            } else if (_currentTabIndex == BAND_TAB_JCSAT3A_H  ||
                        _currentTabIndex == BAND_TAB_JCSAT3A_V ||
                        _currentTabIndex == BAND_TAB_JCSAT4B_H ||
                        _currentTabIndex == BAND_TAB_JCSAT4B_V ||
                        _currentTabIndex == BAND_TAB_SCCB2_H   ||
                        _currentTabIndex == BAND_TAB_SCCB2_V   ||
                        _currentTabIndex == BAND_TAB_SCCC2_H   ||
                        _currentTabIndex == BAND_TAB_SCCC2_V) {
                if (_mode == CHANNEL_EDIT_USER || _mode == CHANNEL_EDIT_USER_USB1 || _mode == CHANNEL_EDIT_USER_USB2) {
                    // 局発周波数の編集
                    _currentEditPositon = EDIT_POSITION_LOCALFREQ;
                    moveArea(CHANNEL_AREA);
                } else if (_mode == CHANNEL_EDIT_DATA_LOGGER ||
                           _mode == CHANNEL_EDIT_DATA_LOGGER_USB1 ||
                           _mode == CHANNEL_EDIT_DATA_LOGGER_USB2) {
                    _currentEditPositon = EDIT_POSITION_POWERSUPPLY;
                    moveArea(CHANNEL_AREA);
                }
            } else if (_currentTabIndex == BAND_TAB_FM || _currentTabIndex == BAND_TAB_CATV_UPSTREAM) {
                if (_mode == CHANNEL_EDIT_USER || _mode == CHANNEL_EDIT_USER_USB1 || _mode == CHANNEL_EDIT_USER_USB2) {
                    // 周波数 のみ編集可能
                    _currentEditPositon = EDIT_POSITION_FREQUENCY;
                    moveArea(CHANNEL_AREA);
                } else if (_mode == CHANNEL_EDIT_DATA_LOGGER ||
                           _mode == CHANNEL_EDIT_DATA_LOGGER_USB1 ||
                           _mode == CHANNEL_EDIT_DATA_LOGGER_USB2) {
                    _currentEditPositon = EDIT_POSITION_POWERSUPPLY;
                    moveArea(CHANNEL_AREA);
                }
            }
        }
    } else if (id == BUTTON_RIGHT) {
        moveArea(ACTION_AREA);
    } else if (id == BUTTON_ENTER) {
        moveArea(TARGET_ALL_EDIT);
    } else if (id == BUTTON_BAND) {
        changeBand(true);
    } else if (id == BUTTON_MENU) {
        changeBand(false);
    }
}

void UserDataLoggerSettingViewModel::moveCursorTargetAllArea(ButtonId id)
{
    // 測定 全てON/OFF の選択行の移動
    if (id == BUTTON_UP) {
            _currentTargetIndex = _channelList.length() - 1;
    } else if (id == BUTTON_DOWN) {
            _currentTargetIndex = 0;
    }
    // チャンネルリストと連動させる
    _currentChannelIndex = _currentTargetIndex;

    if (_view) {
        // 表示リストのインデックス
        _view->setProperty("currentSelectedChannelListIndex", _currentChannelIndex % CHANNEL_LIST_ROWS);
    }
    if (_view) {
        _view->setProperty("currentSelectedTargetListIndex", _currentTargetIndex % CHANNEL_LIST_ROWS);
    }
    if (_view) {
        // 表示リストの何ページ目かを示すインデックス
        _view->setProperty("currentTopChannelListIndex", _currentChannelIndex / CHANNEL_LIST_ROWS * CHANNEL_LIST_ROWS);
    }
    if (_view) {
        _view->setProperty("currentTopTargetListIndex", _currentTargetIndex / CHANNEL_LIST_ROWS * CHANNEL_LIST_ROWS);
    }
}

void UserDataLoggerSettingViewModel::keyStatusTargetAllEdit(ButtonId id)
{
    // 測定 ON/OFF の編集
    _currentEditPositon = EDIT_POSITION_MEASURE;
    setEditItemList();

    if (id == BUTTON_UP || id == BUTTON_DOWN) {
        targetAllEditItem(id);
    } else if (id == BUTTON_LEFT) {
        // nothing to do
    } else if (id == BUTTON_RIGHT) {
        // nothing to do
    } else if (id == BUTTON_ENTER) {
        allUpdateSetting();
        moveArea(TARGET_ALL);
    }
}

void UserDataLoggerSettingViewModel::targetAllEditItem(ButtonId)
{
    QVariant currentValue;
    unsigned count = 0;
    if (_view) {
        currentValue = _view->property("channelTargetAll");     // 編集対象項目の現在の値
    }

    // 測定するチャンネル数
    count = activeChannel();

    if(currentValue == true) {
        currentValue = false;           // 全てしない
    } else if(currentValue == false) {
        currentValue = true;            // 全てする
    }

    // USER 設定とデータロガー設定で上限が異なる
    unsigned int limit = LC70_CH_SET_CATV_ALL_IDX_MAX;
    if ( (_mode == CHANNEL_EDIT_DATA_LOGGER) ||
         (_mode == CHANNEL_EDIT_DATA_LOGGER_USB1) ||
         (_mode == CHANNEL_EDIT_DATA_LOGGER_USB2) ) {
        limit = MEASUREMENT_CH_MAX;
    }

    // 全てするのとき既に上限に達していたら何もしない
    if (currentValue == true && count >= limit) {
        return;
    }

    if (_view) {
        _view->setProperty("channelTargetAll", currentValue);
    }

    qDebug() << QString("全ON/OFF:(%1)").arg(currentValue.toString());
    // 測定 ON/OFF は boolean ではなく文字列に変更
    QVariant currentValueIndex = currentValue == true ? QString("する") : QString("しない");

    // 全てのチャンネルを更新(測定ON/OFFのみ更新)
    int candidate = 0;
    for (int i = 0; i < _channelList.count(); ++i) {
        QVariantList rowData = _channelList[i].toList();

        // 変更する値と同じであれば何もしない
        if (rowData[_currentEditPositon] == currentValueIndex) {
            continue;
        }

        rowData[_currentEditPositon] = currentValueIndex;
        // 一部項目のみ更新
        _channelList[i] = rowData;

        // 全てするの場合にチャンネル数の上限を超えないようにする
        if (currentValue.toBool()) {
            _model->editMeasuring(static_cast<BandTabIndex>(_currentTabIndex), rowData[EDIT_POSITION_CHANNEL].toString(), DO_MEASURING);
            ++candidate;
            if ((candidate + count) >= limit) {
                break;
            }
        } else {
            _model->editMeasuring(static_cast<BandTabIndex>(_currentTabIndex), rowData[EDIT_POSITION_CHANNEL].toString(), DO_NOT_MEASURE);
        }
    }

    if (_view) {
        _view->setProperty("channelList", QVariant::fromValue(_channelList));
    }
    // 測定対象のチャンネルの有無により Band Tab のテキスト色を変更する
    hasActiveChannel();
}

void UserDataLoggerSettingViewModel::allUpdateSetting()
{
    // 画面が参照できない場合は何もしない
    if (_view == nullptr) {
        return;
    }

    // 全てする/しないの値
    QVariant currentValue = _view->property("channelTargetAll");

    // USER 設定とデータロガー設定で上限が異なる
    unsigned int limit = LC70_CH_SET_CATV_ALL_IDX_MAX;
    if ( (_mode == CHANNEL_EDIT_DATA_LOGGER) ||
         (_mode == CHANNEL_EDIT_DATA_LOGGER_USB1) ||
         (_mode == CHANNEL_EDIT_DATA_LOGGER_USB2) ) {
        limit = MEASUREMENT_CH_MAX;
    }

    // 全てしないであれば既存の測定するの数ではなく上限からカウントする
    unsigned int count = (!currentValue.toBool()) ? limit : activeChannel();
    // 全てするを選択したときに既に上限に達していれば何もしない
    if (currentValue.toBool() && count >= limit) {
        return;
    }

    // 測定チャンネル数の上限に達するまでは測定する/しないに更新
    for (int i = 0; i < _channelList.count(); ++i) {
        QVariantList editItem = _channelList[i].toList();

        if (currentValue.toBool()) {
            _model->editMeasuring(static_cast<BandTabIndex>(_currentTabIndex), editItem[EDIT_POSITION_CHANNEL].toString(), DO_MEASURING);
            editItem[EDIT_POSITION_MEASURE] = QString("する");
            ++count;
        } else {
            _model->editMeasuring(static_cast<BandTabIndex>(_currentTabIndex), editItem[EDIT_POSITION_CHANNEL].toString(), DO_NOT_MEASURE);
            editItem[EDIT_POSITION_MEASURE] = QString("しない");
            --count;
        }
        _channelList[i] = editItem;

        // 上限に到達したら更新をやめる
        if (count >= limit) {
            break;
        }
    }

    if (_view) {
        // 測定内容を画面に反映
        _view->setProperty("channelList", QVariant::fromValue(_channelList));
    }

    // 測定対象のチャンネルの有無により Band Tab のテキスト色を変更する
    hasActiveChannel();
}

void UserDataLoggerSettingViewModel::changeBand(bool forward)
{
    // 保存先選択では Band 切り替え不可
//    if (_mode == CHANNEL_EDIT_SELECT_DESTINATION) {
//        return;
//    }

    if (forward) {
        if (static_cast<BandTabIndex>(++_currentTabIndex) >= BAND_TAB_COUNT) {
            _currentTabIndex = BAND_TAB_UHF;
        }
    } else {
        if (static_cast<BandTabIndex>(--_currentTabIndex) < BAND_TAB_UHF) {
            _currentTabIndex = BAND_TAB_PILOT;
        }
    }

    // タブの表示切り替え
    switchTab();
}

void UserDataLoggerSettingViewModel::switchTab()
{
    // タブの表示切り替え
    _bandModel->select(static_cast<BandTabIndex>(_currentTabIndex));
    // チャンネルリストの表示内容を更新(設定ファイルの内容を反映しないようにする)
    showChannelList(false);

    // 保存先選択以外のとき
    if (_mode != CHANNEL_EDIT_SELECT_DESTINATION) {
        // 先頭の行の測定する/しない列に移動
        _currentChannelIndex = 0;
        selectTargetRow();
        _currentEditPositon = EDIT_POSITION_MEASURE;
        moveArea(TARGET_AREA);
    }

//    // チャンネルリストにカーソルがある場合は、設定変更可能場所にカーソルを移動する
//    if(_selectArea == CHANNEL_AREA) {
//        if (_currentTabIndex == BAND_TAB_UHF  ||
//            _currentTabIndex == BAND_TAB_BS_R ||
//            _currentTabIndex == BAND_TAB_BS_L ||
//            _currentTabIndex == BAND_TAB_CS_R ||
//            _currentTabIndex == BAND_TAB_CS_L ||
//            _currentTabIndex == BAND_TAB_SKY_PERFECTV) {

//            if (_mode == CHANNEL_EDIT_USER || _mode == CHANNEL_EDIT_USER_USB1 || _mode == CHANNEL_EDIT_USER_USB2) {
//                // 測定 ON/OFF のみ編集可能
//                _currentEditPositon = EDIT_POSITION_MEASURE;
//                moveArea(TARGET_AREA);
//            } else if (_mode == CHANNEL_EDIT_DATA_LOGGER ||
//                       _mode == CHANNEL_EDIT_DATA_LOGGER_USB1 ||
//                       _mode == CHANNEL_EDIT_DATA_LOGGER_USB2) {
//                // 給電の編集
//                _currentEditPositon = EDIT_POSITION_POWERSUPPLY;
//                _view->setProperty("currentEditItemIndex", _currentEditPositon);
//            }
//        } else if (_currentTabIndex == BAND_TAB_LTE || _currentTabIndex == BAND_TAB_PILOT) {
//            if (_mode == CHANNEL_EDIT_USER || _mode == CHANNEL_EDIT_USER_USB1 || _mode == CHANNEL_EDIT_USER_USB2) {
//                // 測定 ON/OFF のみ編集可能
//                _currentEditPositon = EDIT_POSITION_MEASURE;
//                moveArea(TARGET_AREA);
//            } else if (_mode == CHANNEL_EDIT_DATA_LOGGER ||
//                       _mode == CHANNEL_EDIT_DATA_LOGGER_USB1 ||
//                       _mode == CHANNEL_EDIT_DATA_LOGGER_USB2) {
//                // 給電の編集
//                _currentEditPositon = EDIT_POSITION_POWERSUPPLY;
//                _view->setProperty("currentEditItemIndex", _currentEditPositon);
//            }
//        } else if (_currentTabIndex == BAND_TAB_CATV_ALL) {
//            // 放送規格の編集
//            _currentEditPositon = EDIT_POSITION_BROADCAST;
//            _view->setProperty("currentEditItemIndex", _currentEditPositon);
//        } else if ((_currentTabIndex == BAND_TAB_JCSAT3A_H ||
//                    _currentTabIndex == BAND_TAB_JCSAT3A_V ||
//                    _currentTabIndex == BAND_TAB_JCSAT4B_H ||
//                    _currentTabIndex == BAND_TAB_JCSAT4B_V ||
//                    _currentTabIndex == BAND_TAB_SCCB2_H ||
//                    _currentTabIndex == BAND_TAB_SCCB2_V ||
//                    _currentTabIndex == BAND_TAB_SCCC2_H ||
//                    _currentTabIndex == BAND_TAB_SCCC2_V)) {
//            // 局発周波数の編集
//            _currentEditPositon = EDIT_POSITION_LOCALFREQ;
//            _view->setProperty("currentEditItemIndex", _currentEditPositon);
//        } else if (_currentTabIndex == BAND_TAB_FM || _currentTabIndex == BAND_TAB_CATV_UPSTREAM) {
//            _currentEditPositon = EDIT_POSITION_FREQUENCY;
//            _view->setProperty("currentEditItemIndex", _currentEditPositon);
//        }
//    }
}

void UserDataLoggerSettingViewModel::moveArea(SelectArea destination)
{
    if (_selectArea == destination) {
        return;
    }

    if ((_selectArea == CHANNEL_AREA) && (destination != CHANNEL_EDIT)) {
        _currentEditPositon = 0;
        if (_view) {
            _view->setProperty("currentEditItemIndex", _currentEditPositon);
        }
        if (_view) {
            _view->setProperty("activeArea", _selectArea);
        }
        _editItemList.clear();
    }
    if (_selectArea == CHANNEL_EDIT || _selectArea == TARGET_EDIT || _selectArea == TARGET_ALL_EDIT) {
        if (_view) {
            // 編集後は、編集の有無を画面に反映する(現在は本画面で _model->isUpdate() が真になることはない)
            _view->setProperty("isFileEditing", _model->isUpdate(settingListValueId()));
        }
    }
    if (destination == ACTION_AREA) {
        if(canSave()) {
            _currentActionIndex = ACTION_INDEX_SAVE;
        } else if(canDelete()) {
            _currentActionIndex = ACTION_INDEX_DELETE;
        } else if (canAllDelete()) {
            _currentActionIndex = ACTION_INDEX_ALL_DELETE;
        } else if (_mode == CHANNEL_EDIT_SELECT_DESTINATION) {
            _currentActionIndex = ACTION_INDEX_DECISION;
        } else {
            // 動作できるボタンが無いのでモード変更させない
            return;
        }
        if (_view) {
            _view->setProperty("currentActionIndex", _currentActionIndex);
        }
    }

    _selectArea = destination;
    if (_view) {
        _view->setProperty("activeArea", _selectArea);
    }
    if ( (_selectArea == CHANNEL_EDIT) || (_selectArea == CHANNEL_AREA) ) {
        if (_view) {
            _view->setProperty("activeArea", _selectArea);
        }
        // USER 設定で単一 Band の許可のみの場合
        if (_userSetingBehavior == ALLOW_SINGLE_BAND_ONLY) {
            if ((_mode == CHANNEL_EDIT_USER || _mode == CHANNEL_EDIT_USER_USB1 || _mode == CHANNEL_EDIT_USER_USB2) &&
                    _currentTabIndex != BAND_TAB_CATV_ALL &&
                    _currentTabIndex != BAND_TAB_JCSAT3A_H &&
                    _currentTabIndex != BAND_TAB_JCSAT3A_V &&
                    _currentTabIndex != BAND_TAB_JCSAT4B_H &&
                    _currentTabIndex != BAND_TAB_JCSAT4B_V &&
                    _currentTabIndex != BAND_TAB_SCCB2_H &&
                    _currentTabIndex != BAND_TAB_SCCB2_V &&
                    _currentTabIndex != BAND_TAB_SCCC2_H &&
                    _currentTabIndex != BAND_TAB_SCCC2_V &&
                    _currentTabIndex != BAND_TAB_FM &&
                    _currentTabIndex != BAND_TAB_CATV_UPSTREAM) {
                _currentEditPositon = 0;
            }
        }
        if (_view) {
            _view->setProperty("currentEditItemIndex", _currentEditPositon);
        }
    }
}

bool UserDataLoggerSettingViewModel::moveNextEditItem()
{
    int srcEditPosition = _currentEditPositon;
    switch(_currentTabIndex) {
    // Band ごとに編集内容指定
    // データロガー設定では以下に加えて MER/CN, BER の編集も可能(FM, CATV 上りを除く)
    case BAND_TAB_UHF:
    case BAND_TAB_BS_R:
    case BAND_TAB_BS_L:
    case BAND_TAB_CS_R:
    case BAND_TAB_CS_L:
    case BAND_TAB_SKY_PERFECTV:
    {
        if (_mode == CHANNEL_EDIT_USER || _mode == CHANNEL_EDIT_USER_USB1 || _mode == CHANNEL_EDIT_USER_USB2) {
            // 測定 ON/OFF のみ編集可能
            return true;
        } else if (_mode == CHANNEL_EDIT_DATA_LOGGER ||
                   _mode == CHANNEL_EDIT_DATA_LOGGER_USB1 ||
                   _mode == CHANNEL_EDIT_DATA_LOGGER_USB2) {
            // データロガー設定では 給電、測定 ON/OFF が編集可能
            switch (_currentEditPositon) {
            case EDIT_POSITION_POWERSUPPLY:
                return true;
            default:
                _currentEditPositon = EDIT_POSITION_POWERSUPPLY;
                break;
            }
        }
    }
        break;
    case BAND_TAB_LTE:
    case BAND_TAB_PILOT:
    {
        if (_mode == CHANNEL_EDIT_USER || _mode == CHANNEL_EDIT_USER_USB1 || _mode == CHANNEL_EDIT_USER_USB2) {
            // 測定 ON/OFF のみ編集可能
            return true;
        } else if (_mode == CHANNEL_EDIT_DATA_LOGGER ||
                   _mode == CHANNEL_EDIT_DATA_LOGGER_USB1 ||
                   _mode == CHANNEL_EDIT_DATA_LOGGER_USB2) {
            // データロガー設定では給電、測定 ON/OFF が編集可能
            switch (_currentEditPositon) {
            case EDIT_POSITION_POWERSUPPLY:
                return true;
            default:
                _currentEditPositon = EDIT_POSITION_POWERSUPPLY;
                break;
            }
        }
    }
        break;
    case BAND_TAB_CATV_ALL:
    {
        if (_mode == CHANNEL_EDIT_USER || _mode == CHANNEL_EDIT_USER_USB1 || _mode == CHANNEL_EDIT_USER_USB2) {
            // USER 設定では放送規格、測定 ON/OFF が編集可能
            return true;
        } else if (_mode == CHANNEL_EDIT_DATA_LOGGER ||
                   _mode == CHANNEL_EDIT_DATA_LOGGER_USB1 ||
                   _mode == CHANNEL_EDIT_DATA_LOGGER_USB2) {
            // データロガー設定では放送規格、給電、測定 ON/OFF が編集可能
            switch (_currentEditPositon) {
            case EDIT_POSITION_BROADCAST:
                _currentEditPositon = EDIT_POSITION_POWERSUPPLY;
                break;
            case EDIT_POSITION_POWERSUPPLY:
                return true;
            default:
                _currentEditPositon = EDIT_POSITION_BROADCAST;
                break;
            }
        }
    }
        break;
    case BAND_TAB_CATV_UPSTREAM:
    case BAND_TAB_FM:
    {
        if (_mode == CHANNEL_EDIT_USER || _mode == CHANNEL_EDIT_USER_USB1 || _mode == CHANNEL_EDIT_USER_USB2) {
            // USER 設定では周波数、測定 ON/OFF が編集可能
            return true;
        } else if (_mode == CHANNEL_EDIT_DATA_LOGGER ||
                   _mode == CHANNEL_EDIT_DATA_LOGGER_USB1 ||
                   _mode == CHANNEL_EDIT_DATA_LOGGER_USB2) {
            // データロガー設定では周波数、給電、測定 ON/OFF が編集可能
            switch (_currentEditPositon) {
            case EDIT_POSITION_FREQUENCY:
                _currentEditPositon = EDIT_POSITION_POWERSUPPLY;
                break;
            case EDIT_POSITION_POWERSUPPLY:
                return true;
            default:
                _currentEditPositon = EDIT_POSITION_FREQUENCY;
                break;
            }
        }
    }
        break;
    case BAND_TAB_JCSAT3A_H:
    case BAND_TAB_JCSAT3A_V:
    case BAND_TAB_JCSAT4B_H:
    case BAND_TAB_JCSAT4B_V:
    case BAND_TAB_SCCB2_H:
    case BAND_TAB_SCCB2_V:
    case BAND_TAB_SCCC2_H:
    case BAND_TAB_SCCC2_V:
    {
        if (_mode == CHANNEL_EDIT_USER || _mode == CHANNEL_EDIT_USER_USB1 || _mode == CHANNEL_EDIT_USER_USB2) {
            // USER 設定では局発周波数、測定 ON/OFF が編集可能
            return true;
        } else if (_mode == CHANNEL_EDIT_DATA_LOGGER ||
                   _mode == CHANNEL_EDIT_DATA_LOGGER_USB1 ||
                   _mode == CHANNEL_EDIT_DATA_LOGGER_USB2) {
            // データロガー設定では局発周波数、給電、測定 ON/OFF が編集可能
            switch (_currentEditPositon) {
            case EDIT_POSITION_LOCALFREQ:
                _currentEditPositon = EDIT_POSITION_POWERSUPPLY;
                break;
            case EDIT_POSITION_POWERSUPPLY:
                return true;
            default:
                _currentEditPositon = EDIT_POSITION_LOCALFREQ;
                break;
            }
        }
    }
        break;
    default:
        break;
    }

    if (_currentEditPositon != srcEditPosition) {
        setEditItemList();
    }
    if (_view) {
        _view->setProperty("currentEditItemIndex", _currentEditPositon);
    }
    return false;
}

bool UserDataLoggerSettingViewModel::movePreviousEditItem()
{
    int srcEditPosition = _currentEditPositon;
    switch(_currentTabIndex) {
    // Band ごとに編集内容指定
    // データロガー設定では以下に加えて MER/CN, BER の編集も可能(FM, CATV 上りを除く)
    case BAND_TAB_UHF:
    case BAND_TAB_BS_R:
    case BAND_TAB_BS_L:
    case BAND_TAB_CS_R:
    case BAND_TAB_CS_L:
    case BAND_TAB_SKY_PERFECTV:
    {
        if (_mode == CHANNEL_EDIT_USER || _mode == CHANNEL_EDIT_USER_USB1 || _mode == CHANNEL_EDIT_USER_USB2) {
            // 測定 ON/OFF のみ編集可能
            return true;
        } else if (_mode == CHANNEL_EDIT_DATA_LOGGER ||
                   _mode == CHANNEL_EDIT_DATA_LOGGER_USB1 ||
                   _mode == CHANNEL_EDIT_DATA_LOGGER_USB2) {
            // データロガー設定では 給電、測定 ON/OFF が編集可能
            switch (_currentEditPositon) {
            case EDIT_POSITION_POWERSUPPLY:
                return true;
            default:
                _currentEditPositon = EDIT_POSITION_POWERSUPPLY;
                break;
            }
        }
    }
        break;
    case BAND_TAB_LTE:
    case BAND_TAB_PILOT:
    {
        if (_mode == CHANNEL_EDIT_USER || _mode == CHANNEL_EDIT_USER_USB1 || _mode == CHANNEL_EDIT_USER_USB2) {
            // 測定 ON/OFF のみ編集可能
            return true;
        } else if (_mode == CHANNEL_EDIT_DATA_LOGGER ||
                   _mode == CHANNEL_EDIT_DATA_LOGGER_USB1 ||
                   _mode == CHANNEL_EDIT_DATA_LOGGER_USB2) {
            // データロガー設定では給電、測定 ON/OFF が編集可能
            _currentEditPositon = EDIT_POSITION_POWERSUPPLY;
            switch (_currentEditPositon) {
            case EDIT_POSITION_POWERSUPPLY:
                return true;
            default:
                _currentEditPositon = EDIT_POSITION_POWERSUPPLY;
                break;
            }
        }
    }
        break;
    case BAND_TAB_CATV_ALL:
    {
        if (_mode == CHANNEL_EDIT_USER || _mode == CHANNEL_EDIT_USER_USB1 || _mode == CHANNEL_EDIT_USER_USB2) {
            // USER 設定では放送規格、測定 ON/OFF が編集可能
            return true;
        } else if (_mode == CHANNEL_EDIT_DATA_LOGGER ||
                   _mode == CHANNEL_EDIT_DATA_LOGGER_USB1 ||
                   _mode == CHANNEL_EDIT_DATA_LOGGER_USB2) {
            // データロガー設定では放送規格、給電、測定 ON/OFF が編集可能
            switch (_currentEditPositon) {
            case EDIT_POSITION_BROADCAST:
                return true;
            case EDIT_POSITION_POWERSUPPLY:
                _currentEditPositon = EDIT_POSITION_BROADCAST;
                break;
            default:
                _currentEditPositon = EDIT_POSITION_POWERSUPPLY;
                break;
            }
        }
    }
        break;
    case BAND_TAB_CATV_UPSTREAM:
    case BAND_TAB_FM:
    {
        if (_mode == CHANNEL_EDIT_USER || _mode == CHANNEL_EDIT_USER_USB1 || _mode == CHANNEL_EDIT_USER_USB2) {
            // USER 設定では周波数、測定 ON/OFF が編集可能
            return true;
        } else if (_mode == CHANNEL_EDIT_DATA_LOGGER ||
                   _mode == CHANNEL_EDIT_DATA_LOGGER_USB1 ||
                   _mode == CHANNEL_EDIT_DATA_LOGGER_USB2) {
            // データロガー設定では周波数、給電、測定 ON/OFF が編集可能
            switch (_currentEditPositon) {
            case EDIT_POSITION_FREQUENCY:
                return true;
            case EDIT_POSITION_POWERSUPPLY:
                _currentEditPositon = EDIT_POSITION_FREQUENCY;
                break;
            default:
                _currentEditPositon = EDIT_POSITION_POWERSUPPLY;
                break;
            }
        }
    }
        break;
    case BAND_TAB_JCSAT3A_H:
    case BAND_TAB_JCSAT3A_V:
    case BAND_TAB_JCSAT4B_H:
    case BAND_TAB_JCSAT4B_V:
    case BAND_TAB_SCCB2_H:
    case BAND_TAB_SCCB2_V:
    case BAND_TAB_SCCC2_H:
    case BAND_TAB_SCCC2_V:
    {
        if (_mode == CHANNEL_EDIT_USER || _mode == CHANNEL_EDIT_USER_USB1 || _mode == CHANNEL_EDIT_USER_USB2) {
            // USER 設定では局発周波数、測定 ON/OFF が編集可能
            return true;
        } else if (_mode == CHANNEL_EDIT_DATA_LOGGER ||
                   _mode == CHANNEL_EDIT_DATA_LOGGER_USB1 ||
                   _mode == CHANNEL_EDIT_DATA_LOGGER_USB2) {
            // データロガー設定では局発周波数、給電、測定 ON/OFF が編集可能
            switch (_currentEditPositon) {
            case EDIT_POSITION_LOCALFREQ:
                return true;
            case EDIT_POSITION_POWERSUPPLY:
                _currentEditPositon = EDIT_POSITION_LOCALFREQ;
                break;
            default:
                _currentEditPositon = EDIT_POSITION_POWERSUPPLY;
                break;
            }
        }
    }
        break;
    default:
        break;
    }
    if (_currentEditPositon != srcEditPosition) {
        setEditItemList();
    }
    if (_view) {
        _view->setProperty("currentEditItemIndex", _currentEditPositon);
    }
    return false;
}

void UserDataLoggerSettingViewModel::setEditItemList()
{
    _editItemList.clear();
    switch(_currentEditPositon) {
    case EDIT_POSITION_FREQUENCY:           // 周波数
    {
        QList<ChannelDefinitionItem> channelDefinitionList;

        if (_currentTabIndex == BAND_TAB_FM) {
            _model->selectChannelDefinitionList(LC70_BAND_FM, QString(""), channelDefinitionList);
        } else if (_currentTabIndex == BAND_TAB_CATV_UPSTREAM) {
            _model->selectChannelDefinitionList(LC70_BAND_CATV_UP_STREAM, QString(""), channelDefinitionList);
        }

        for (int i = 0; i < channelDefinitionList.length(); i++) {
            _editItemList.append(channelDefinitionList[i].channelNumber);
        }
    }
        break;
    case EDIT_POSITION_BROADCAST:           // 放送規格
    {
        // ISDBC(ANNEXC), ISDBT の二択
        _editItemList.append(QString("ISDBC"));
        _editItemList.append(QString("ISDBT"));
    }
        break;
    case EDIT_POSITION_LOCALFREQ:           // 局発周波数
    {
        QVariant localFrequencyValue;
        _model->getValue(VALUE_LOCAL_FREQUENCY, localFrequencyValue);

        LC70_BAND_T band = TAB_BAND_MAP.value(_currentTabIndex, LC70_BAND_UNDEF);
        QString polarization = TAB_POLARIZATION_MAP.value(_currentTabIndex, QString(""));

        QList<uint32_t> localFrequencyValueList = {};
        Correcter::getLocalOscillatorList(localFrequencyValue, band, polarization, localFrequencyValueList);

        for (int i = 0; i < localFrequencyValueList.count(); ++i) {
            _editItemList.append(static_cast<double>(localFrequencyValueList[i]) / 1000000);
        }
    }
        break;
    case EDIT_POSITION_MER:                 // MER/CN
    case EDIT_POSITION_BER:                 // BER
        _editItemList.append(QString("ON"));
        _editItemList.append(QString("OFF"));
        break;
    case EDIT_POSITION_POWERSUPPLY:         // 給電
    {
        LC70_BAND_T band = TAB_BAND_MAP.value(_currentTabIndex, LC70_BAND_UNDEF);

        QList<uint8_t> settingList;
        if (band != LC70_BAND_JCSAT3 && band != LC70_BAND_JCSAT4 &&
                band != LC70_BAND_SCCB && band != LC70_BAND_SCCC) {
            settingList = POWER_15VONLY;
        } else {
            QString channelName = _channelList[_currentChannelIndex].toList()[EDIT_POSITION_CHANNEL].toString();
            ChannelDefinitionItem channel;
            _model->getChannelDefinitionItem(band, channelName, channel);
             settingList = POWER_SUPPLY_SETTING_MAP.value({band, channel.polarization}, POWER_15VONLY);
        }

        for (int i = 0; i < settingList.count(); ++i) {
            _editItemList.append(Correcter::displayPowerSupply(settingList[i]));
        }
    }
        break;
    case EDIT_POSITION_MEASURE:             // 測定 ON/OFF
        _editItemList.append(QString("しない"));
        _editItemList.append(QString("する"));
        break;
    default:
        break;
    }
}

void UserDataLoggerSettingViewModel::editItem(ButtonId id)
{
    if (_channelList.count() == 0) {
        return;
    }

    QVariantList editRowData = _channelList[_currentChannelIndex].toList(); // 選択されている行のデータ
    QVariant currentValue = editRowData.at(_currentEditPositon);            // 編集対象項目の現在の値
    if (_currentEditPositon == EDIT_POSITION_MEASURE) {
        // 測定 ON/OFF は boolean ではなく文字列に変更
        currentValue = (editRowData[EDIT_POSITION_MEASURE].toString() == QString("する")) ? QString("する") : QString("しない");
    }

    int currentValueIndex = _editItemList.indexOf(currentValue);            // 編集対象項目の現在の値に対応するリストのインデックス
    int nextValueIndex = currentValueIndex;
    if (id == BUTTON_DOWN) {
        if (--nextValueIndex < 0) {
            nextValueIndex = _editItemList.length() - 1;
        }
    } else {
        if (++nextValueIndex >= _editItemList.length()) {
            nextValueIndex = 0;
        }
    }

    if (_editItemList.count() == 0) {
        qDebug() << QString("undefined edit list");
        return;
    }

    qDebug() << QString("editItem: %1 (%2 -> %3)").arg(_currentEditPositon).arg(currentValue.toString()).arg(_editItemList.at(nextValueIndex).toString());
    editRowData[_currentEditPositon] = _editItemList.at(nextValueIndex);

    LC70_BAND_T band = TAB_BAND_MAP.value(_currentTabIndex, LC70_BAND_UNDEF);

    // 局発周波数の変更を全チャンネルに反映する(USER 設定、データロガー設定共通)
    // データロガー設定の場合 MER, BER, 給電の変更を全チャンネルに反映する
    if (_currentEditPositon == EDIT_POSITION_LOCALFREQ ||
            ((_mode == CHANNEL_EDIT_DATA_LOGGER ||
              _mode == CHANNEL_EDIT_DATA_LOGGER_USB1 ||
              _mode == CHANNEL_EDIT_DATA_LOGGER_USB2) &&
             ( _currentEditPositon == EDIT_POSITION_MER ||
               _currentEditPositon == EDIT_POSITION_BER ||
               _currentEditPositon == EDIT_POSITION_POWERSUPPLY))) {
        QVariant newValue = editRowData[_currentEditPositon];

        ChannelDefinitionItem channelDefinitionItem;

        // 測定内容を表示するリストの全行に変更を反映
        for (int i = 0; i < _channelList.count(); ++i) {
            QVariantList rowData = _channelList[i].toList();
            rowData[_currentEditPositon] = newValue;

            _model->getChannelDefinitionItem(band, rowData[EDIT_POSITION_CHANNEL].toString(), channelDefinitionItem);

            // 局発周波数の変更時に周波数も変更する
            if (_currentEditPositon == EDIT_POSITION_LOCALFREQ) {

                QList<int> broadbandSystemList;
                _model->getBroadbandSystem(band, broadbandSystemList);
                int broadbandSystem = broadbandSystemList.at(0);
                unsigned int modulationBit = _model->getModulationBit(band, broadbandSystem);
                QList <unsigned int> bitList;
                Correcter::getModulationBitList(modulationBit, bitList);
                modulationBit = bitList.empty() ? LC70_SETMOD_UNDEF : bitList.at(0);

                // 実周波数(中心周波数 - 局発周波数)
                unsigned int centerFrequency =Correcter::centerFrequency(channelDefinitionItem, modulationBit);
                unsigned int frequency = Correcter::frequency(centerFrequency, rowData[EDIT_POSITION_LOCALFREQ].toFloat() * 1000 * 1000);
                rowData[EDIT_POSITION_FREQUENCY] = Correcter::displayFrequency(band, frequency);
            }

            _channelList[i] = rowData;
        }
    } else if (_currentEditPositon == EDIT_POSITION_BROADCAST) {
        // CATV ALL で放送方式を変更した場合に周波数も変更する
        ChannelDefinitionItem channelDefinitionItem;
        _model->getChannelDefinitionItem(band, editRowData[EDIT_POSITION_CHANNEL].toString(), channelDefinitionItem);
        // CATV チャンネル周波数の設定対応
        QList<UserDataLoggerSettingFile> channelList = _model->selectEditingChannels(BAND_TAB_CATV_ALL);

        for (auto channel : REPLACE_FREQUENCY_CHANNEL_INDEX.keys()) {
            if (channel != editRowData[EDIT_POSITION_CHANNEL].toString()) {
                continue;
            }
            int index = REPLACE_FREQUENCY_CHANNEL_INDEX.value(channel);
            QString frequency = QString::number(channelList[index].frequency);
            if (frequency.right(3) == QString("143")) {
                channelDefinitionItem.centerFrequency2 = channelList[index].frequency;
                channelDefinitionItem.centerFrequency1 = channelList[index].frequency - 143;
            } else {
                channelDefinitionItem.centerFrequency1 = channelList[index].frequency;
                channelDefinitionItem.centerFrequency2 = channelList[index].frequency + 143;
            }
            break;
        }

        // 実周波数(中心周波数 - 局発周波数)
        unsigned int centerFrequency =Correcter::centerFrequency(channelDefinitionItem, editRowData[EDIT_POSITION_BROADCAST].toString() == QString("ISDBC") ? LC70_SETMOD_64QAM : LC70_SETMOD_OFDM);
        unsigned int frequency = Correcter::frequency(centerFrequency, editRowData[EDIT_POSITION_LOCALFREQ].toFloat() * 1000 * 1000);
        editRowData[EDIT_POSITION_FREQUENCY] = Correcter::displayFrequency(band, frequency);

        // 選択されたチャンネルのみ更新(行の内容全て更新)
        _channelList[_currentChannelIndex] = editRowData;
    } else {
        // 選択されたチャンネルのみ更新(行の内容全て更新)
        _channelList[_currentChannelIndex] = editRowData;
    }

    if (_view) {
        // 測定内容を画面に反映
        _view->setProperty("channelList", QVariant::fromValue(_channelList));
    }
    // 選択行を指定
    selectTargetRow();

    // 測定対象のチャンネルの有無により Band Tab のテキスト色を変更する
    hasActiveChannel();
}

void UserDataLoggerSettingViewModel::updateSetting()
{
    // 更新したチャンネル(要素が配列のリスト)
    QVariantList editItem = _channelList[_currentChannelIndex].toList();

    unsigned int limit = LC70_CH_SET_CATV_ALL_IDX_MAX;
    if ( (_mode == CHANNEL_EDIT_DATA_LOGGER) ||
         (_mode == CHANNEL_EDIT_DATA_LOGGER_USB1) ||
         (_mode == CHANNEL_EDIT_DATA_LOGGER_USB2) ) {
        limit = MEASUREMENT_CH_MAX;
    }
    // User設定では150ch分、データロガー設定では50ch分登録している場合に、”測定”項目をONに変更できなくする
    if (_currentEditPositon == EDIT_POSITION_MEASURE &&
            editItem[EDIT_POSITION_MEASURE].toString() == QString("する")) {
        if (activeChannel() > limit) {
            if ( (_mode == CHANNEL_EDIT_DATA_LOGGER) ||
                 (_mode == CHANNEL_EDIT_DATA_LOGGER_USB1) ||
                 (_mode == CHANNEL_EDIT_DATA_LOGGER_USB2) ) {
                emit requestToShowPopup(POPUP_DATA_LOGGER_SETTING_EXCEED_UPPER_LIMIT);
            } else {
                emit requestToShowPopup(POPUP_USER_SETTING_EXCEED_UPPER_LIMIT);
            }
            return;
        }
    }

    // 測定 ON/OFF
    if (_currentEditPositon == EDIT_POSITION_FREQUENCY) {
        int value = editItem[EDIT_POSITION_FREQUENCY].toFloat() * 1000;
       _model->editFrequency(static_cast<BandTabIndex>(_currentTabIndex), editItem[EDIT_POSITION_CHANNEL].toString(), value);
    }

    // 放送規格の変更内容を反映する
    if (_currentEditPositon == EDIT_POSITION_BROADCAST) {
        int value = BROADCAST_SYSTEM_VALUE.key(editItem[EDIT_POSITION_BROADCAST].toString());
       _model->editBroadcastSystem(static_cast<BandTabIndex>(_currentTabIndex), editItem[EDIT_POSITION_CHANNEL].toString(), value);
    }

    // 局発周波数の変更内容を反映する(Band の全チャンネル)
    if (_currentEditPositon == EDIT_POSITION_LOCALFREQ) {
        int value = LOCAL_FREQUENCY_VALUE.key(editItem[EDIT_POSITION_LOCALFREQ].toString());
       _model->editLocalFrequency(static_cast<BandTabIndex>(_currentTabIndex), value);
    }

    // 給電 ON/OFF の変更内容を反映する(Band の全チャンネル)
    if (_currentEditPositon == EDIT_POSITION_POWERSUPPLY) {
        int value = POWER_SUPPLY_VALUE.key(editItem[EDIT_POSITION_POWERSUPPLY].toString());
        // 給電 ON の場合に Band に応じて値を付け替える
        if (value == POWER_SUPPLY_ON) {
            switch (_currentTabIndex) {
            case BAND_TAB_JCSAT3A_H:
            case BAND_TAB_SCCB2_H:
            case BAND_TAB_SCCC2_H:
                value = POWER_15VONLY[POWER_SUPPLY_ON];
                break;
            case BAND_TAB_JCSAT3A_V:
            case BAND_TAB_SCCB2_V:
            case BAND_TAB_SCCC2_V:
                value = POWER_11VONLY[POWER_SUPPLY_ON];
                break;
            case BAND_TAB_JCSAT4B_H:
                value = POWER_15VPONLY[POWER_SUPPLY_ON];
                break;
            case BAND_TAB_JCSAT4B_V:
                value = POWER_11VPONLY[POWER_SUPPLY_ON];
                break;
            default:
                value = POWER_15VONLY[POWER_SUPPLY_ON];
                break;
            }
        } else {
            // 給電 OFF
            value = LC70_MEAS_SET_DC_OUT_VAL_NONE;
        }
        _model->editPowerSupply(static_cast<BandTabIndex>(_currentTabIndex), value);
    }

    // 測定 ON/OFF
    if (_currentEditPositon == EDIT_POSITION_MEASURE) {
        int value = MEASURING_VALUE.key(editItem[EDIT_POSITION_MEASURE].toString());
       _model->editMeasuring(static_cast<BandTabIndex>(_currentTabIndex), editItem[EDIT_POSITION_CHANNEL].toString(), value);
    }

    // 測定対象のチャンネルの有無により Band Tab のテキスト色を変更する
    hasActiveChannel();
}

void UserDataLoggerSettingViewModel::acceptPopupCancelConfirmation()
{
    // キーごとに遷移先を特定
    switch (_lastPressedButtonId) {
    case BUTTON_LEFT:
    {
        if (_selectArea == CHANNEL_AREA) {
            // 変更の破棄を承諾したので測定内容を元に戻してファイルリストに戻る
            showChannelList(true);
            moveArea(FILELIST_AREA);
        }
    }
        break;
    case BUTTON_HOME:
        _reserveTimer->stop();
        emit requestDirect(SCREEN_HOME, true);
        break;

    case BUTTON_USER:
        _reserveTimer->stop();
        emit requestDirect(SCREEN_USER_SETTING, true);
        break;

    case BUTTON_ESCAPE:
        _reserveTimer->stop();
        emit back();
        break;

    default:
        break;
    }
}

void UserDataLoggerSettingViewModel::acceptPopupSaveConfirmation()
{
    _temporaryFileName.clear();
    _destinationTab = BAND_TAB_COUNT;
    _destinationIndex = -1;

    // FM, CATV 上りに同一周波数の入力が行われていないか調べる
    _destinationIndex = checkInputSameFrequency();

    // FM, CATV 上りに同一周波数のチャンネルあり
    if (_destinationIndex >= 0) {
        // ポップアップ表示
        emit requestToShowPopup(POPUP_SAME_FREQUENCY_CHANNEL);
        return;
    }

    // 選択中のファイルリストのインデックス
    int index = _currentFileIndex + (_currentFileListPage - 1) * FILE_LIST_ROWS;

    // ファイルリストの選択行の測定内容を取得
    QVariant value;
    _model->getValue(fileListValueId(), value);
    QVariantList settingFileList = value.toList();
    QVariantList measuringContents = settingFileList[index].toList();

    // 保存先選択の場合、スキャンした有効なチャンネルの Band を選択しておく
    if (_mode == CHANNEL_EDIT_SELECT_DESTINATION) {
        // チャンネル情報取得
        QList<MeasureResult> result = _model->allScanResult();

        if (result.count() > 0) {
            LC70_BAND_T band = static_cast<LC70_BAND_T>(result[0].Band.toInt());
            QString channel = result[0].Channel;
            _currentTabIndex = identifyBandTab(band, channel);
            _bandModel->select(static_cast<BandTabIndex>(_currentTabIndex));
        }
    }

    if (settingFileList[index].toList()[2].toString().isEmpty()) {
        // 新規ファイル
        // キーボード表示
        emit requestToShowPopup(POPUP_DATA_LOGGER_KEYBOARD);

        // ファイル名が取得できるようにシグナル接続をする
        connect(_mainFrame, SIGNAL(inputFileName(PopupId, SaveLocation, QString)),
                this, SLOT(onInputFileName(PopupId, SaveLocation, QString)));
    } else {
        QString fileName = measuringContents.at(2).toString();
#if !Q_PROCESSOR_ARM
        if (!fileName.contains(CSV_FILE_EXTENSION)) {
            fileName += QString(CSV_FILE_EXTENSION);
        }
#endif
        // 保存先選択のとき、スキャン結果を反映する
        if (_mode == CHANNEL_EDIT_SELECT_DESTINATION) {
            // スキャン結果保存
            createScannedChannelSetting();
            // 画面引き継ぎようにファイル名設定
            int index = _currentFileIndex + (_currentFileListPage - 1) * FILE_LIST_ROWS;
            _fileList[index] = fileName;
        }

        // Band 変更があった場合にキーボードを表示
        if (isDifferentFileGroup(fileName)) {
            _oldFileName = fileName;
            _temporaryFileName = removeFileGroupPrefix(fileName);

            // キーボード表示
            emit requestToShowPopupWithMessage(POPUP_DATA_LOGGER_KEYBOARD, _temporaryFileName);

            // ファイル名が取得できるようにシグナル接続をする
            connect(_mainFrame, SIGNAL(inputFileName(PopupId, SaveLocation, QString)),
                    this, SLOT(onInputFileName(PopupId, SaveLocation, QString)));
            return;
        }

        fileName = addGroupPrefixToFileName(removeFileGroupPrefix(fileName));

        // 既存ファイルに対する更新であればそのまま上書きする
        if (!_model->exportSettingFile(_mode, fileName)) {
            emit requestToShowPopup(POPUP_WRITE_ERROR);
        }

        // 保存先選択の場合はファイル出力後、前の画面に戻る
        if (_mode == CHANNEL_EDIT_SELECT_DESTINATION) {
            // スキャン結果の保存を連携
            emit requestSetTakeOver("saveScanData", true);
            emit back();
        }

        // ファイルリストにフォーカス設定
        // 一行前に戻っている
        moveArea(FILELIST_AREA);

        // 更新されたファイル名を反映するため設定ファイルリストを再表示
        showSettingFileList();
    }
}

void UserDataLoggerSettingViewModel::acceptPopupDataLoggerKeyboard()
{
    disconnect(_mainFrame, SIGNAL(inputFileName(PopupId, SaveLocation, QString)),
               this, SLOT(onInputFileName(PopupId, SaveLocation, QString)));

    // 選択中のファイルリストのインデックス
    int index = _currentFileIndex + (_currentFileListPage - 1) * FILE_LIST_ROWS;

    QVariant value;
    _model->getValue(fileListValueId(), value);

    // 設定ファイルの一覧から今回登録する行のデータを取得
    QVariantList userSettingFileList = value.toList();
    QVariantList registrationData = userSettingFileList[index].toList();

    // ファイル名取得
    QString fileName = registrationData.at(2).toString();

#if !Q_PROCESSOR_ARM
    if (!fileName.contains(CSV_FILE_EXTENSION)) {
        fileName += QString(CSV_FILE_EXTENSION);
    }
#endif

    // 保存先選択のとき、スキャン結果を反映する
    if (_mode == CHANNEL_EDIT_SELECT_DESTINATION) {
        // スキャン結果保存
        createScannedChannelSetting();
        // 画面引き継ぎようにファイル名設定
        int index = _currentFileIndex + (_currentFileListPage - 1) * FILE_LIST_ROWS;
        _fileList[index] = fileName;
    }

    // ファイル出力
    if (!_model->exportSettingFile(_mode, fileName)) {
        emit requestToShowPopup(POPUP_WRITE_ERROR);
    }

    // 保存先選択の場合はファイル出力後、前の画面に戻る
    if (_mode == CHANNEL_EDIT_SELECT_DESTINATION) {
        // スキャン結果の保存を連携
        emit requestSetTakeOver("saveScanData", true);
        emit back();
    }

    // ファイルリストにフォーカス設定
    // 一行前に戻っている
    moveArea(FILELIST_AREA);

    // 更新されたファイル名を反映するため設定ファイルリストを再表示
    showSettingFileList();

    // 保存先選択のとき、ファイル出力後、USER 設定に移行する
    if (_mode == CHANNEL_EDIT_SELECT_DESTINATION) {
        _mode = CHANNEL_EDIT_USER;
        if (_view) {
            _view->setProperty("mode", _mode);
        }
        // USER データロガー設定ファイルの一覧を取得し、画面に反映
        showSettingFileList();
    }
}

void UserDataLoggerSettingViewModel::acceptPopupConfirmOverwriteUserLoggerData()
{
    // 同一ファイル名の場合に再度キーボードを表示
    // キーボード表示
    emit requestToShowPopupWithMessage(POPUP_DATA_LOGGER_KEYBOARD, _temporaryFileName);

    // ファイル名が取得できるようにシグナル接続をする
    connect(_mainFrame, SIGNAL(inputFileName(PopupId, SaveLocation, QString)),
            this, SLOT(onInputFileName(PopupId, SaveLocation, QString)));

}

void UserDataLoggerSettingViewModel::acceptPopupDeleteConfirmation()
{
    // 選択中のファイルリストのインデックス
    int index = _currentFileIndex + (_currentFileListPage - 1) * FILE_LIST_ROWS;

    QVariant value;
    _model->getValue(fileListValueId(), value);

    QVariantList userList = value.toList();
    QVariantList userData = userList[index].toList();

    // ファイル取得
    QString fileName = userData.at(2).toString();

#if !Q_PROCESSOR_ARM
    if (!fileName.contains(CSV_FILE_EXTENSION)) {
        fileName += QString(CSV_FILE_EXTENSION);
    }
#endif
    // 削除できたら、USER リストを更新する
    QString backupFileName = userData[2].toString();
    userData[2] = "";
    userList[index] = userData;
    _model->setValue(fileListValueId(), userList);
    if (_model->commit(fileListValueId())) {
        // ファイル削除
        if(!_model->removeDataLoggerUserSettingList(settingListValueId(), fileName)) {
            userData[2] = backupFileName;
            userList[index] = userData;
            _model->setValue(fileListValueId(), userList);
            _model->commit(fileListValueId());
            emit requestToShowPopup(POPUP_FAILURE_DELETE_EXECUTE);
            return;
        }
        // バックアップの削除
        _model->removeUserDataLoggerBackupFile(_mode, fileName + QString(CSV_FILE_EXTENSION) + QString(BACKUP_FILE_EXTENSION));
    }

    // ファイルリストにフォーカス設定
    moveArea(FILELIST_AREA);

    // 削除されたファイルを反映するため設定ファイルリストを再表示
    showSettingFileList();
}

void UserDataLoggerSettingViewModel::acceptPopupAllDeleteConfirmation()
{
    // ファイル一括削除
    if(_model->allRemoveDataLoggerUserSettingList(settingListValueId())) {
        // バックアップファイルの削除
        _model->removeUserDataLoggerBackupFile(_mode, QString(""), true);
    }

    // ファイルリストにフォーカス設定
    moveArea(FILELIST_AREA);

    // ファイルを読み込み直す
    _model->initializeUserSettingFile();

    // 削除されたファイルを反映するため設定ファイルリストを再表示
    showSettingFileList();
}

void UserDataLoggerSettingViewModel::acceptPopupConfirmChangeBand()
{
    // USER 設定で単一 Band の許可のみの場合
    if (_userSetingBehavior == ALLOW_SINGLE_BAND_ONLY) {
        // USER 設定は Band を跨がらないので破棄してチャンネル数を 0 に更新
        if (_mode == CHANNEL_EDIT_USER || _mode == CHANNEL_EDIT_USER_USB1 || _mode == CHANNEL_EDIT_USER_USB2) {
            _bandModel->update(static_cast<BandTabIndex>(_currentTabIndex), 0, true);
        }
    }

    // Band を変更
    changeBand(true);
}

void UserDataLoggerSettingViewModel::acceptPopupConfirmExecuteWithAutoPowerOff()
{
    // オートパワーオフありでデータロガー実行
    executeDataLogger();
}

bool UserDataLoggerSettingViewModel::checkSameFileName(QVariantList fileList, QString fileName)
{
    bool exist = false;

    // プリフィクスを付与した文字列と比較
    fileName = addGroupPrefixToFileName(fileName);

    for (int index = 0; index < fileList.count(); ++index) {
        QString existingFile = fileList[index].toList().at(2).toString();
        if(existingFile == fileName){
            exist = true;
            break;
        }
    }

    return exist;
}

void UserDataLoggerSettingViewModel::retrieveChannels(QList<UserDataLoggerSettingFile> settingFile, QList<ChannelDefinitionItem> &channels)
{
    LC70_BAND_T band = TAB_BAND_MAP.value(_currentTabIndex, LC70_BAND_UNDEF);
    // CATV 上りと FM は設定ファイルからチャンネル一覧を作成する
    if (band == LC70_BAND_CATV_UP_STREAM) {
        QList<UserDataLoggerSettingFile> channelList = _model->selectEditingChannels(BAND_TAB_CATV_UPSTREAM);
        for (int i = 0; i < channelList.count(); ++i) {
            ChannelDefinitionItem channel;
            channel.band = band;
            channel.channelNumber = channelList[i].channel;
            channel.polarization = -1;
            channel.centerFrequency1 = channelList[i].frequency;
            channel.centerFrequency2 = 0;
            channel.bandWidth = -1;
            channel.valid = true;

            channels.append(channel);
        }
    } else if (band == LC70_BAND_FM) {
        QList<UserDataLoggerSettingFile> channelList = _model->selectEditingChannels(BAND_TAB_FM);
        for (int i = 0; i < channelList.count(); ++i) {
            ChannelDefinitionItem channel;
            channel.band = band;
            channel.channelNumber = channelList[i].channel;
            channel.polarization = -1;
            channel.centerFrequency1 = channelList[i].frequency;
            channel.centerFrequency2 = 0;
            channel.bandWidth = -1;
            channel.valid = true;

            channels.append(channel);
        }
    } else {
        QString polarization = TAB_POLARIZATION_MAP.value(_currentTabIndex, QString(""));
        _model->selectChannelDefinitionList(band, polarization, channels);

        // CATV の場合、チャンネル周波数の設定があるため、設定ファイルの周波数に付け替える
        if (band == LC70_BAND_CATV_ALL || band == LC70_BAND_CATV_90_470MHZ || band == LC70_BAND_CATV_470_1002MHZ) {

            // C24 のインデックスが 22 で C27 のインデックスが 25
            for (int i = 22; i < 26; ++i) {
                QString frequency = QString::number(settingFile[i].frequency);

                // 中心周波数を設定ファイルの内容から付け替える
                if (frequency.right(3) == QString("143")) {
                    frequency = QString::number(settingFile[i].frequency - 143);
                }
                channels[i].centerFrequency1 = frequency.toInt();
                channels[i].centerFrequency2 = channels[i].centerFrequency1 + 143;
            }
        }
    }
}

QString UserDataLoggerSettingViewModel::defaultBroadcastSystem(const LC70_BAND_T &band)
{
    QString defaultValueBroadcast = "---";
    switch(band) {
    case LC70_BAND_UHF:
        defaultValueBroadcast = QString("ISDBT");
        break;
    case LC70_BAND_BS:
    case LC70_BAND_CS:
        defaultValueBroadcast = QString("ISDBS/S3");
        break;
    case LC70_BAND_JCSAT3:
    case LC70_BAND_JCSAT4:
    case LC70_BAND_SCCB:
    case LC70_BAND_SCCC:
        defaultValueBroadcast = QString("DVBS/S2");
        break;
    case LC70_BAND_CATV_ALL:
        defaultValueBroadcast = QString("ISDBC");
        break;
    case LC70_BAND_PILOT:
        defaultValueBroadcast = QString("PILOT");
        break;
    case LC70_BAND_SKYP:
        defaultValueBroadcast = QString("OPTICAST");
        break;
    case LC70_BAND_LTE:
        defaultValueBroadcast = QString("LTE");
        break;
    case LC70_BAND_FM:
        defaultValueBroadcast = QString("FM");
        break;
    case LC70_BAND_CATV_UP_STREAM:
        defaultValueBroadcast = QString("CATV上り");
        break;
    default:
        break;
    }

    return defaultValueBroadcast;
}

void UserDataLoggerSettingViewModel::populateChannels(const LC70_BAND_T &band, const QList<ChannelDefinitionItem> &channelDefinitionList, QList<UserDataLoggerSettingFile> settingFile)
{
    // チャンネル定義がなければ何もしない
    if (channelDefinitionList.count() == 0) {
        return;
    }

    // 設定ファイルの内容を取得できていなければ何もしない
    if (settingFile.count() == 0) {
        return;
    }

    // チャンネル定義数と設定ファイルの行数が異なる場合は何もしない
    if (channelDefinitionList.count() != settingFile.count()) {
        qDebug() << QString("[CONFLICT] CHANNEL ROWS:%1 SETTING FILE ROWS:%2").arg(channelDefinitionList.count()).arg(settingFile.count());
        return;
    }

    // 現在の測定内容を破棄
    _channelList.clear();

    // デフォルトの放送規格
    QString defaultValueBroadcast = defaultBroadcastSystem(band);

    // 選択されている Band のチャンネル定義を全て反映
    for (int i = 0; i < channelDefinitionList.length(); i++) {
        // 測定内容の初期値を設定
        QVariantList channelData;
        QString channel = settingFile[i].channel;
        channel = channel.trimmed();

        // チャンネルごとの初期データの作成
        channelData = initializeChannelData(band, channelDefinitionList[i], defaultValueBroadcast, settingFile[i].localFrequency, settingFile[i].powerSupply);

        // CATV なら放送規格は選択内容を反映、その他はデフォルト表示
        if (band == LC70_BAND_CATV_ALL) {
            // ISDBC(ANNEXC) なら変更
            if (settingFile[i].broadcastsystem == LC70_BD_SYS_ANNEXC) {
                channelData[EDIT_POSITION_BROADCAST] = QString("ISDBC");
            } else {
                channelData[EDIT_POSITION_BROADCAST] = QString("ISDBT");
            }
        } else {
            channelData[EDIT_POSITION_BROADCAST] = defaultValueBroadcast;
        }

        // 実周波数(中心周波数 - 局発周波数)
        unsigned int modulation = LC70_SETMOD_OFDM;
        if (band == LC70_BAND_CATV_ALL) {
            if (settingFile[i].broadcastsystem == LC70_BD_SYS_ANNEXC) {
                modulation = LC70_SETMOD_64QAM;
            } else if (settingFile[i].broadcastsystem == LC70_BD_SYS_ISDBT) {
                modulation = LC70_SETMOD_OFDM;
            }
        } else if (band == LC70_BAND_SKYP) {
            modulation = LC70_SETMOD_256QAM;
        }
        unsigned int centerFrequency = Correcter::centerFrequency(channelDefinitionList[i], modulation);
        unsigned int frequency = Correcter::frequency(centerFrequency, settingFile[i].localFrequency);
        QString realFrequency = Correcter::displayFrequency(band, frequency);
        channelData[EDIT_POSITION_FREQUENCY] = realFrequency;

        if (settingFile[i].localFrequency != 0) {
            // kHz => GHz
            channelData[EDIT_POSITION_LOCALFREQ] = Correcter::displayLocalFrequency(settingFile[i].localFrequency);
        } else {
            channelData[EDIT_POSITION_LOCALFREQ] = QString("---");
        }
        // ON なら MER を測定する場合でも CNR のビットが立っている
        if (band == LC70_BAND_FM || band == LC70_BAND_CATV_UP_STREAM || band == LC70_BAND_PILOT || band == LC70_BAND_LTE) {
            channelData[EDIT_POSITION_MER] = QString("---");
            channelData[EDIT_POSITION_BER] = QString("---");
        } else {
            channelData[EDIT_POSITION_MER] = QString("ON");
            channelData[EDIT_POSITION_BER] = QString("ON");
        }

        channelData[EDIT_POSITION_POWERSUPPLY] = (settingFile[i].powerSupply == POWER_SUPPLY_OFF) ? QString("OFF") : QString("ON");
        channelData[EDIT_POSITION_MEASURE] = (settingFile[i].measure == DO_MEASURING) ? QString("する") : QString("しない");

        _channelList.append(QVariant::fromValue(channelData));
    }
}

QString UserDataLoggerSettingViewModel::addGroupPrefixToFileName(const QString &fileName)
{
    QString result;
    QString parts;
    QVector<QStringRef> words = fileName.splitRef("-");

    // 既にファイルグループが付加されていたら付け替える
//    if (words.count() <= 1) {
//        // ファイルグループなし
//        parts = fileName;
//    } else if (words.count() == 2) {
//        parts = words[1].toString();
//    } else {
//        // 複数 "-" で囲った文字列がある
//        parts = words[1].toString();
//        for (int i = 2; i < words.count(); ++i) {
//            parts += QString("-") + words[i].toString();
//        }
//    }

    parts = fileName;

//    // 既にプリフィックスが付いていたらそのまま返す
//    if (words.count() > 0) {
//        if (words[0] == QString("D") ||
//            words[0] == QString("M") ||
//            words[0] == QString("T") ||
//            words[0] == QString("S") ||
//            words[0] == QString("C") ||
//            words[0] == QString("J") ||
//            words[0] == QString("Z")) {
//            return fileName;
//        }
//    }

    // ファイルグループのプリフィクスを追加
    if (_mode == CHANNEL_EDIT_DATA_LOGGER ||
        _mode == CHANNEL_EDIT_DATA_LOGGER_USB1 ||
        _mode == CHANNEL_EDIT_DATA_LOGGER_USB2) {
        // データロガーは D
        result = QString("D-") + parts;
    } else {
        if (hasMultiBandsInUserSetting()) {
            // USER 設定で複数の Band を跨ぐ場合は M
            result = QString("M-") + parts;
        } else {
            BandTabIndex bandTabIndex = _model->selectGroupPrefix();

            // 保存先選択の場合にスキャンした Band のタブに付け替える
            if (_mode == CHANNEL_EDIT_SELECT_DESTINATION) {
                bandTabIndex = static_cast<BandTabIndex>(_currentTabIndex);
            }

            if (bandTabIndex == BAND_TAB_UHF) {
                // 地デジは T
                result = QString("T-") + parts;
            } else if (bandTabIndex == BAND_TAB_BS_L ||
                       bandTabIndex == BAND_TAB_BS_R ||
                       bandTabIndex == BAND_TAB_CS_L ||
                       bandTabIndex == BAND_TAB_CS_R) {
                // BS/CS は S
                result = QString("S-") + parts;
            } else if (bandTabIndex == BAND_TAB_CATV_ALL ||
                       bandTabIndex == BAND_TAB_SKY_PERFECTV ||
                       bandTabIndex == BAND_TAB_PILOT ||
                       bandTabIndex == BAND_TAB_CATV_UPSTREAM) {
                // CATV は C
                result = QString("C-") + parts;
            } else if (bandTabIndex == BAND_TAB_JCSAT3A_H ||
                       bandTabIndex == BAND_TAB_JCSAT3A_V ||
                       bandTabIndex == BAND_TAB_JCSAT4B_H ||
                       bandTabIndex == BAND_TAB_JCSAT4B_V) {
                // JCSAT は J
                result = QString("J-") + parts;
            } else {
                // SCC, FM, LTE は Z
                result = QString("Z-") + parts;
            }
        }
    }

    return result;
}

bool UserDataLoggerSettingViewModel::hasActiveChannel(void)
{
    unsigned count = activeChannel();

    // 保存先選択の場合にスキャン結果からチャンネル数を設定
//    if (_mode == CHANNEL_EDIT_SELECT_DESTINATION) {
//        // チャンネル情報取得
//        QList<MeasureResult> result = _model->allScanResult();
//        count = result.count();
//    }

    bool ret = (count > 0) ? true : false;

    if (ret) {
        // 全 Band 未選択状態ではない
        _bandModel->unselectAll(false);
    } else {
        if (unregisteredDataLoggerChannelCount() == 0) {
            // 全 Band 未選択状態
            _bandModel->unselectAll(true);
        } else {
            // 全 Band 未選択状態ではない
            _bandModel->unselectAll(false);
        }
    }

    // 測定チャンネル数の上限を超えている場合は上限を表示
    unsigned int limit = LC70_CH_SET_CATV_ALL_IDX_MAX;
    if (_mode == CHANNEL_EDIT_DATA_LOGGER ||
        _mode == CHANNEL_EDIT_DATA_LOGGER_USB1 ||
        _mode == CHANNEL_EDIT_DATA_LOGGER_USB2) {
        limit = MEASUREMENT_CH_MAX;
    }

    if (count > limit) {
        count = limit;
    }

    if (_view) {
        // 測定チャンネル数を更新
        _view->setProperty("channelCount", count);
    }
    // チャンネルリストを全てみて、測定ONのチャンネルがあればtrueなければfalseを返す
    return ret;
}

unsigned int UserDataLoggerSettingViewModel::activeChannel()
{
    unsigned count = 0;

//    if (_mode == CHANNEL_EDIT_SELECT_DESTINATION) {
//        for (int i = 0; i < _channelList.length(); i++) {
//            QVariantList channelListItem = _channelList[i].toList();
//            if (channelListItem[EDIT_POSITION_MEASURE].toString() == QString("ON")) {
//                ++count;
//            }
//        }
//    } else {
        for (int i = 0; i < BAND_TAB_COUNT; ++i) {
//            int channelCount = _model->selectMeasuringRows(static_cast<BandTabIndex>(i));
//            count += channelCount;
            int channelCount = 0;
            if (i != _currentTabIndex) {
                channelCount = _model->selectMeasuringRows(static_cast<BandTabIndex>(i));
                count += channelCount;
            } else {
                for (int j = 0; j < _channelList.count(); ++j) {
                    QVariantList channelListItem = _channelList[j].toList();
                    if (channelListItem[EDIT_POSITION_MEASURE].toString() == QString("ON") ||
                            channelListItem[EDIT_POSITION_MEASURE].toString() == QString("する")) {
                        ++count;
                        ++channelCount;
                    }
                }
            }
            // チャンネル数を更新
            _bandModel->update(static_cast<BandTabIndex>(i), channelCount, (i == _currentTabIndex) ? true : false);
        }
//    }

    return count;
}

bool UserDataLoggerSettingViewModel::canStartMeasure()
{
    // データロガー設定で USB メモリー未装着なら測定開始不可
    if (_mode == CHANNEL_EDIT_DATA_LOGGER ||
        _mode == CHANNEL_EDIT_DATA_LOGGER_USB1 ||
        _mode == CHANNEL_EDIT_DATA_LOGGER_USB2) {
        if (_model->getDeviceStatus().usba != LC70_STRAGE_FORMAT) {
           return false;
       }
    }
    // 現在は本画面で _model->isUpdate() が真になることはない
    return (_mode != CHANNEL_EDIT_SELECT_DESTINATION) && hasSettingFile() && !_model->isUpdate(settingListValueId());
}

bool UserDataLoggerSettingViewModel::canSave()
{
    return (_mode != CHANNEL_EDIT_SELECT_DESTINATION) && hasActiveChannel();
}

bool UserDataLoggerSettingViewModel::canDelete()
{
    return (_mode != CHANNEL_EDIT_SELECT_DESTINATION) && hasSettingFile();
}

bool UserDataLoggerSettingViewModel::canAllDelete()
{
    if (_mode == CHANNEL_EDIT_SELECT_DESTINATION) {
        return false;
    }

    // 保存済みの設定ファイルがあること
    for (int i = 0; i < _fileList.count(); ++i) {
        if (!_fileList[i].isEmpty()) {
            return true;
        }
    }

    return false;
}

bool UserDataLoggerSettingViewModel::isDifferentFileGroup(const QString &fileName)
{
    QString prefix;
    if (fileName.length() > FILE_NAME_PREFIX_LENGTH) {
        prefix = fileName.left(FILE_NAME_PREFIX_LENGTH);
    }

    // 設定ファイル中の最初に測定するが定義されている Band を特定する
    BandTabIndex bandTabIndex = BAND_TAB_COUNT;
    QString channel;
    _model->selectFirstValidChannel(bandTabIndex, channel);

    // ファイルグループのプリフィクスを追加
    if (_mode == CHANNEL_EDIT_DATA_LOGGER ||
        _mode == CHANNEL_EDIT_DATA_LOGGER_USB1 ||
        _mode == CHANNEL_EDIT_DATA_LOGGER_USB2) {
        // データロガーは D
        return (prefix != QString("D-"));
    } else {
        // USER 設定で複数 Band を跨る場合は下記と異なるプリフィクスを追加
        if (hasMultiBandsInUserSetting()) {
            // USER 設定で複数 Band を跨ぐ場合は M
            return (prefix != QString("M-"));
        } else if (bandTabIndex == BAND_TAB_UHF) {
            // 地デジは T
            return (prefix != QString("T-"));
        } else if (bandTabIndex == BAND_TAB_BS_L ||
                   bandTabIndex == BAND_TAB_BS_R ||
                   bandTabIndex == BAND_TAB_CS_L ||
                   bandTabIndex == BAND_TAB_CS_R) {
            // BS/CS は S
            return (prefix != QString("S-"));
        } else if (bandTabIndex == BAND_TAB_CATV_ALL ||
                   bandTabIndex == BAND_TAB_SKY_PERFECTV ||
                   bandTabIndex == BAND_TAB_PILOT ||
                   bandTabIndex == BAND_TAB_CATV_UPSTREAM) {
            // CATV は C
            return (prefix != QString("C-"));
        } else if (bandTabIndex == BAND_TAB_JCSAT3A_H ||
                   bandTabIndex == BAND_TAB_JCSAT3A_V ||
                   bandTabIndex == BAND_TAB_JCSAT4B_H ||
                   bandTabIndex == BAND_TAB_JCSAT4B_V) {
            // JCSAT は J
            return (prefix != QString("J-"));
        } else if (bandTabIndex == BAND_TAB_SCCB2_H ||
                   bandTabIndex == BAND_TAB_SCCB2_V ||
                   bandTabIndex == BAND_TAB_SCCC2_H ||
                   bandTabIndex == BAND_TAB_SCCC2_V ||
                   bandTabIndex == BAND_TAB_FM ||
                   bandTabIndex == BAND_TAB_LTE) {
            // SCC, FM, LTE は Z
            return (prefix != QString("Z-"));
        } else {
            // 全てプリフィクスが付くためここには来ない
        }
    }

    return false;
}

QString UserDataLoggerSettingViewModel::removeFileGroupPrefix(const QString &fileName)
{
    QString newName = fileName;

    // 拡張子が付いている場合は取り除く
    if (newName.contains(CSV_FILE_EXTENSION)) {
        newName = newName.left(newName.length() - qstrlen(CSV_FILE_EXTENSION));
    }

    if (newName.length() <= FILE_NAME_PREFIX_LENGTH) {
        return newName;
    }

     QString prefix = fileName.left(FILE_NAME_PREFIX_LENGTH);

    // ファイルグループのプリフィクスを追加
    if (prefix == QString("D-") ||
        prefix == QString("T-") ||
        prefix == QString("S-") ||
        prefix == QString("C-") ||
        prefix == QString("J-") ||
        prefix == QString("Z-") ||
        prefix == QString("M-")) {
        return newName.right(newName.length() - FILE_NAME_PREFIX_LENGTH);
    } else {
        return newName;
    }
}

bool UserDataLoggerSettingViewModel::isPresetChannelBand(const LC70_BAND_T &band)
{
    return band == LC70_BAND_FM || band == LC70_BAND_CATV_UP_STREAM;
}

ValueId UserDataLoggerSettingViewModel::fileListValueId()
{
    if (_mode == CHANNEL_EDIT_USER) {
        return VALUE_USERSETTING_FILELIST;
    } else if (_mode == CHANNEL_EDIT_USER_USB1) {
        return VALUE_USERSETTING_FILELIST_USB1;
    } else if (_mode == CHANNEL_EDIT_USER_USB2) {
        return VALUE_USERSETTING_FILELIST_USB2;
    } else if (_mode == CHANNEL_EDIT_DATA_LOGGER) {
        return VALUE_DATALOGGERSETTING_FILELIST;
    } else if (_mode == CHANNEL_EDIT_DATA_LOGGER_USB1) {
        return VALUE_DATALOGGERSETTING_FILELIST_USB1;
    } else if (_mode == CHANNEL_EDIT_DATA_LOGGER_USB2) {
        return VALUE_DATALOGGERSETTING_FILELIST_USB2;
    } else {
        // 保存先選択
        return VALUE_USERSETTING_FILELIST;
    }
}

ValueId UserDataLoggerSettingViewModel::settingListValueId()
{
    if (_mode == CHANNEL_EDIT_USER) {
        return VALUE_USERSETTING_SETTINGLIST;
    } else if (_mode == CHANNEL_EDIT_USER_USB1) {
        return VALUE_USERSETTING_SETTINGLIST_USB1;
    } else if (_mode == CHANNEL_EDIT_USER_USB2) {
        return VALUE_USERSETTING_SETTINGLIST_USB2;
    } else if (_mode == CHANNEL_EDIT_DATA_LOGGER) {
        return VALUE_DATALOGGERSETTING_SETTINGLIST;
    } else if (_mode == CHANNEL_EDIT_DATA_LOGGER_USB1) {
        return VALUE_DATALOGGERSETTING_SETTINGLIST_USB1;
    } else if (_mode == CHANNEL_EDIT_DATA_LOGGER_USB2) {
        return VALUE_DATALOGGERSETTING_SETTINGLIST_USB2;
    } else {
        // 保存先選択
        return VALUE_USERSETTING_SETTINGLIST;
    }
}

QVariantList UserDataLoggerSettingViewModel::initializeChannelData(const LC70_BAND_T &band, const ChannelDefinitionItem &channel, const QString &defaultBroadcastValue
                                                                   , const unsigned int &defaultLocalFrequencyValue, const unsigned int &defaultPowerSupplyValue)
{
    QVariantList channelData;

    // デフォルトの放送規格
    QString defaultValueBroadcast = defaultBroadcastValue;

    // デフォルトの局発周波数
    unsigned int defaultLocalFrequency = defaultLocalFrequencyValue;

    // デフォルトの MER, BER 測定
    QString defaultValueMer = "ON";
    QString defaultValueBer = "ON";
    if (band == LC70_BAND_PILOT || band == LC70_BAND_LTE || band == LC70_BAND_FM || band == LC70_BAND_CATV_UP_STREAM) {
        defaultValueMer = QString("---");
        defaultValueBer = QString("---");
    }

    // デフォルトの給電
    QString defaultValuePowerSupply =
            (defaultPowerSupplyValue == LC70_MEAS_SET_DC_OUT_VAL_11V) ? "ON" :
            (defaultPowerSupplyValue == LC70_MEAS_SET_DC_OUT_VAL_15V) ? "ON" :
            (defaultPowerSupplyValue == LC70_MEAS_SET_DC_OUT_VAL_11V_P) ? "ON" :
            (defaultPowerSupplyValue == LC70_MEAS_SET_DC_OUT_VAL_15V_P) ? "ON" : "OFF";
//    QString defaultValuePowerSupply =
//            (defaultPowerSupplyValue == LC70_MEAS_SET_DC_OUT_VAL_11V) ? "11V" :
//            (defaultPowerSupplyValue == LC70_MEAS_SET_DC_OUT_VAL_15V) ? "15V" :
//            (defaultPowerSupplyValue == LC70_MEAS_SET_DC_OUT_VAL_11V_P) ? "11V+P" :
//            (defaultPowerSupplyValue == LC70_MEAS_SET_DC_OUT_VAL_15V_P) ? "15V+P" : "OFF";

//    if (_mode != CHANNEL_EDIT_USER && band != LC70_BAND_FM && band != LC70_BAND_CATV_UP_STREAM) {
//        defaultValuePowerSupply = "15V";
//    }
//    // 全 Band で OFF にする
//    defaultValuePowerSupply = "OFF";

    // チャンネル番号
    channelData.append(channel.channelNumber);

    // 実周波数(中心周波数 - 局発周波数)
    unsigned int modulation = LC70_SETMOD_OFDM;
    if (band == LC70_BAND_CATV_ALL) {
        // CATV ALL はデフォルトが ISDBC で ISDBT も選択可能
        if (defaultBroadcastValue == QString("ISDBC")) {
            modulation = LC70_SETMOD_64QAM;
        }
    } else if (band == LC70_BAND_SKYP) {
        modulation = LC70_SETMOD_256QAM;
    }
    unsigned int centerFrequency = Correcter::centerFrequency(channel, modulation);
    unsigned int frequency = Correcter::frequency(centerFrequency, defaultLocalFrequency);
    QString realFrequency = Correcter::displayFrequency(band, frequency);
    channelData.append(realFrequency);

    // 放送規格
    channelData.append(defaultValueBroadcast);

    // 局発周波数
    channelData.append(defaultLocalFrequency > 0 ? Correcter::displayLocalFrequency(defaultLocalFrequency) : "---");

    // MER/CN
    channelData.append(defaultValueMer);

    // BER
    channelData.append(defaultValueBer);

    // 給電
    channelData.append(defaultValuePowerSupply);

    // 測定 OFF
    channelData.append(QString("しない"));

    return channelData;
}

void UserDataLoggerSettingViewModel::createScannedChannelSetting(void)
{
    // チャンネル情報取得
    QList<MeasureResult> result = _model->allScanResult();

    if (result.count() == 0) {
        return;
    }

    // テンプレートファイルを読み込み、既存データと合成しないようにする
    _model->initializeUserDataLoggerSettingFile(_mode);

    // スキャン結果の最初の有効なチャンネルのある Band に移動
    BandTabIndex bandTabIndex = identifyBandTab(static_cast<LC70_BAND_T>(result[0].Band.toInt()), result[0].Channel);
    _currentTabIndex = bandTabIndex;
    switchTab();

    QString modulation = _takeOver["selectedModulation"].toString();
    bool updateLocalFrequency = false;

    // スキャン結果を反映する(有効なチャンネルを「測定する」にする)
    for (int i = 0; i < result.count(); ++i) {
        LC70_BAND_T band = static_cast<LC70_BAND_T>(result[i].Band.toInt());
        // CATV ALL 以外の Band からのスキャン結果を受領した場合に CATV ALL に反映する
        if (band == LC70_BAND_CATV_90_470MHZ || band == LC70_BAND_CATV_470_1002MHZ) {
            band = LC70_BAND_CATV_ALL;
        }

        QString channel = result[i].Channel;

        // Band の特定
        BandTabIndex bandTabIndex = identifyBandTab(band, channel);

        // 局発周波数の更新
        if (bandTabIndex == BAND_TAB_JCSAT3A_H || bandTabIndex == BAND_TAB_JCSAT3A_V ||
                bandTabIndex == BAND_TAB_JCSAT4B_H || bandTabIndex == BAND_TAB_JCSAT4B_V ||
                bandTabIndex == BAND_TAB_SCCB2_H || bandTabIndex == BAND_TAB_SCCB2_V ||
                bandTabIndex == BAND_TAB_SCCC2_H || bandTabIndex == BAND_TAB_SCCC2_V) {
            if (!updateLocalFrequency) {
                _model->editLocalFrequency(bandTabIndex, result[i].LocalOscillator);
                updateLocalFrequency = true;
            }
        }

        // 測定する/しないの変更
        _model->editMeasuring(bandTabIndex, channel, DO_MEASURING);

        // CATV ALL のスキャン結果であれば変調方式を取得し、 OFDM なら周波数、放送方式を変更する(QAM ならデフォルト設定と合致するので何もしない）
        if (band == LC70_BAND_CATV_ALL) {
            if (modulation == QString("OFDM")) {
                _model->editBroadcastSystem(bandTabIndex, channel, LC70_BD_SYS_ISDBT, true);
            }
        }
    }
}

int UserDataLoggerSettingViewModel::validChannelNumber()
{
    int count = 0;
    // 設定ファイルのチャンネルリストを読み出す
    QVariant value;
    _model->getValue(settingListValueId(), value);
    // 選択されたチャンネルの測定条件のリスト
    QVariantList settingFileChanneList = value.toList();
    for (int i = 0; i < settingFileChanneList.count(); ++i) {
        QVariantMap settingFileChanneListItem = settingFileChanneList[i].toMap();

        if (settingFileChanneListItem["enable"].toUInt()) {
            count++;
        }
    }

    return count;
}

int UserDataLoggerSettingViewModel::unregisteredDataLoggerChannelCount()
{
    int count = 0;
    for (int i = 0; i < BAND_TAB_COUNT; ++i) {
        // 現在選択されているタブ以外の　Band は保存している内容から件数を取得し、現在選択されているタブの Band は画面の内容から件数を取得
        if (i != _currentTabIndex) {
            count +=_model->selectMeasuringRows(static_cast<BandTabIndex>(i));
        } else {
            for (int j = 0; j < _channelList.count(); ++j) {
                QVariantList channelListItem = _channelList[j].toList();
                if (channelListItem[EDIT_POSITION_MEASURE].toString() == QString("ON") ||
                        channelListItem[EDIT_POSITION_MEASURE].toString() == QString("する")) {
                    ++count;
                }
            }
        }
    }

    return count;
}

bool UserDataLoggerSettingViewModel::hasChannel(const LC70_BAND_T &band, const QVariant &channel, const QVariantList &userSettingList)
{
    for (int i = 0; i < userSettingList.count(); ++i) {
        QVariantMap userSettingData = userSettingList[i].toMap();
        // Band とチャンネルが合致したら削除対象
        if (band == userSettingData["band"].toUInt() && channel == userSettingData["channel"]) {
            return true;
        }
    }

    return false;
}

void UserDataLoggerSettingViewModel::selectTargetRow()
{
    // 測定 ON/OFF リストと連動させる
    _currentTargetIndex = _currentChannelIndex;

    // 測定内容リストの何ページ目かを示すインデックス
    int page = (_currentChannelIndex + 1) / CHANNEL_LIST_ROWS;
    if ((_currentChannelIndex + 1) % CHANNEL_LIST_ROWS > 0) {
        ++page;
    }
    int channelListIndex = (page - 1) * CHANNEL_LIST_ROWS;
    int selectedIndex = _currentChannelIndex % CHANNEL_LIST_ROWS;
    if (_view) {
        _view->setProperty("currentTopChannelListIndex", channelListIndex);
    }
    if (_view) {
        _view->setProperty("currentSelectedTargetListIndex", selectedIndex);
    }
    if (_view) {
        // 測定 ON/OFF リストのインデックス
        _view->setProperty("currentTopTargetListIndex", channelListIndex);
    }
    if (_view) {
        _view->setProperty("currentSelectedChannelListIndex", selectedIndex);
    }
}

int UserDataLoggerSettingViewModel::checkInputSameFrequency(void)
{
    // CATV 上りの同一チャンネルの入力チェック
    QList<UserDataLoggerSettingFile> contents = _model->selectEditingChannels(BAND_TAB_CATV_UPSTREAM);
    for (int i = 0; i < contents.count(); ++i) {
        // 測定 ON にしたときだけにする
        if (contents[i].measure == DO_MEASURING) {
            for (int j = i + 1; j < contents.count(); ++j) {
                if (contents[j].measure == DO_MEASURING) {
                    // 周波数が同じ
                    if (contents[i].frequency == contents[j].frequency) {
                        _destinationTab = BAND_TAB_CATV_UPSTREAM;
                        return j;
                    }
                }
            }
        }
    }

    // FM の同一チャンネルの入力チェック
    contents.clear();
    contents = _model->selectEditingChannels(BAND_TAB_FM);
    for (int i = 0; i < contents.count(); ++i) {
        // 測定 ON にしたときだけにする
        if (contents[i].measure == DO_MEASURING) {
            for (int j = i + 1; j < contents.count(); ++j) {
                if (contents[j].measure == DO_MEASURING) {
                    // 周波数が同じ
                    if (contents[i].frequency == contents[j].frequency) {
                        _destinationTab = BAND_TAB_FM;
                        return j;
                    }
                }
            }
        }
    }

    return -1;
}

bool UserDataLoggerSettingViewModel::hasMultiBandsInUserSetting()
{
    if (_mode != CHANNEL_EDIT_USER && _mode != CHANNEL_EDIT_USER_USB1 && _mode != CHANNEL_EDIT_USER_USB2 && _mode != CHANNEL_EDIT_SELECT_DESTINATION) {
        return false;
    }

    // BS/CS のスキャン結果の保存の場合
    if (_mode == CHANNEL_EDIT_SELECT_DESTINATION && _takeOver["selectedBand"] == LC70_BAND_BS_CS) {
        return true;
    }

    // USER 設定の場合に複数の Band で測定 ON になっているか
    int count = 0;
    for (int i = BAND_TAB_UHF; i < BAND_TAB_COUNT; ++i) {
        QList<UserDataLoggerSettingFile> contents = _model->selectEditingChannels(static_cast<BandTabIndex>(i));
        for (int j = 0; j < contents.count(); ++j) {
            // 測定 ON にしたときだけにする
            if (contents[j].measure == DO_MEASURING) {
                ++count;
                break;
            }
        }
    }

    return (count > 1) ? true : false;
}

bool UserDataLoggerSettingViewModel::hasCATVUpstreamChannel()
{
    return _model->selectMeasuringRows(BAND_TAB_CATV_UPSTREAM) > 0 ? true : false;
}

bool UserDataLoggerSettingViewModel::isOpticalMeasurementSelected()
{
    QVariant value;
    bool ok = false;
    _model->getValue(VALUE_RF_OPTICAL, value);

    uint32_t selection = value.toUInt(&ok);
    if (!ok) {
        return false;
    }

    // 同軸を選択
    if (selection == 0) {
        return false;
    }

    // 光を選択
    return true;
}

void UserDataLoggerSettingViewModel::executeDataLogger()
{
    QObject *popupItem = nullptr;
    if (_view) {
        popupItem = _view->findChild<QObject *>("dgPopup");
    }

    // データロガー設定ファイル名の引き継ぎ
    emit requestSetTakeOver("userSettingFileName", _fileList[_currentFileIndex + (_currentFileListPage - 1) * FILE_LIST_ROWS]);

    emit requestSetTakeOver("channelEditMode", _mode);

    // チャンネルデータを取得
    QList<UserDataLoggerSettingFile> channelList = _model->selectMeasuringChannels();

    if (channelList.size() < 1) {
        // 範囲外
        popupItem->setProperty("popupType", NODATA);
        popupItem->setProperty("visible", true);
        popupItem->setProperty("message", "選択チャンネルは無効です");
        popupItem->setProperty("leftButtonMessage", "戻る");

        _popupButtonOperation.setButtonName(1, 1, _dataLoggerPopupType4ButtonData);
        _popupButtonOperation.setActiveObject(_dataLoggerPopupType4ButtonData[0][0]);
        _popupActiveButton = _dataLoggerPopupType4ButtonData[0][0];

        QObject *target = popupItem->findChild<QObject *>(_popupActiveButton);

        if (target) {
            target->setProperty("active", true);
        }

        _selectArea = POPUP_AREA;
        _popupType = NODATA;
        return;
    }

    QString listname;
    listname.sprintf("%d: %s", _currentFileIndex + 1, _fileList[_currentFileIndex].toLocal8Bit().constData());

    popupItem->setProperty("popupType", COLLECTCONFIRM);
    popupItem->setProperty("visible", true);
    popupItem->setProperty("message", "");
    popupItem->setProperty("leftButtonMessage", "設定");
    popupItem->setProperty("rightButtonMessage", "予約設定");

    // ボタンのアクティブ・Editカーソル戻し.
    QObject* target = popupItem->findChild<QObject *>("m2");
    if (target) {
        target->setProperty("active", false);
    }

    _popupButtonOperation.setButtonName(1, 2, _dataLoggerPopupType1ButtonData);
    _popupButtonOperation.setActiveObject(_dataLoggerPopupType1ButtonData[0][0]);
    _popupActiveButton = _dataLoggerPopupType1ButtonData[0][0];

    target = popupItem->findChild<QObject *>(_popupActiveButton);

    if (target) {
        target->setProperty("active", true);
    }

    _selectArea = POPUP_AREA;
    _popupType = COLLECTCONFIRM;
}

void UserDataLoggerSettingViewModel::moveCursorPopupArea(ButtonId id)
{
    switch (_popupType) {
    case COLLECTCONFIRM:
    case CANCELCONFIRM: {
        if (id == BUTTON_LEFT || id == BUTTON_RIGHT) {
            QString nextBtn;
            bool ret = _popupButtonOperation.nextActiveObject(id, nextBtn);

            // active 表示の切替
            if (ret) {
                QObject *nextTarget = nullptr;

                if (_view) {
                    nextTarget = _view->findChild<QObject *>(nextBtn);
                }

                if (nextTarget) {
                    if (nextTarget->property("disable") == false) {
                        // inactive
                        QObject *target = nullptr;

                        if (_view) {
                            target = _view->findChild<QObject *>(_popupActiveButton);
                        }

                        if (target) {
                            target->setProperty("active", false);
                        }

                        // active
                        target = nullptr;
                        if (_view) {
                            target = _view->findChild<QObject *>(nextBtn);
                        }

                        if (target) {
                            target->setProperty("active", true);
                        }

                        _popupActiveButton = nextBtn;
                        _popupButtonOperation.setActiveObject(_popupActiveButton);
                    }
                }
            }
        }

        break;
    }

    case DATETIMESET: {
        QObject *popupItem = nullptr;

        if (_view) {
            popupItem = _view->findChild<QObject *>("dgPopup");
        }

        if (_popupState == EDIT) {
            if ((id == BUTTON_RIGHT) || (id == BUTTON_LEFT)) {
                if ((_popupActiveButton == "startmonthdata")
                || (_popupActiveButton == "startdaydata")
                || (_popupActiveButton == "starthourdata")
                || (_popupActiveButton == "startminutesdata")) {
	                QString nextBtn;
	                _popupButtonOperation.nextActiveObject(id, nextBtn);
	
	                // active 表示の切替
	                QObject *nextTarget = nullptr;
	                if (_view) {
	                    nextTarget = _view->findChild<QObject *>(nextBtn);
	                }
	
	                if (nextTarget) {
	                    if (nextTarget->property("disable") == false) {
	                        // inactive
	                        QObject *target = nullptr;
	                        if (_view) {
	                            target = _view->findChild<QObject *>(_popupActiveButton);
	                        }
	
	                        if (target) {
	                            target->setProperty("active", false);
	                        }
	
	                        // active
	                        target = nullptr;
	                        if (_view) {
	                            target = _view->findChild<QObject *>(nextBtn);
	                        }
	
	                        if (target) {
	                            target->setProperty("active", true);
	                        }
	
	                        _popupActiveButton = nextBtn;
	                        _popupButtonOperation.setActiveObject(_popupActiveButton);
	                    }
	                }
                }
            } else if ((id == BUTTON_UP) || (id == BUTTON_DOWN)) {
                // 日時の更新処理

                if (_popupActiveButton == "startmonthdata") {
                    updateStartMonth(id);
                }

                if (_popupActiveButton == "startdaydata") {
                    updateStartDay(id);
                }

                if (_popupActiveButton == "starthourdata") {
                    updateStartHour(id);
                }

                if (_popupActiveButton == "startminutesdata") {
                    updateStartMinutes(id);
                }

                if (_popupActiveButton == "endmonthdata") {
                    updateEndMonth(id);
                }

                if (_popupActiveButton == "enddaydata") {
                    updateEndDay(id);
                }

                if (_popupActiveButton == "endhourdata") {
                    updateEndHour(id);
                }

                if (_popupActiveButton == "endminutesdata") {
                    updateEndMinutes(id);
                }

                if (_popupActiveButton == "datetimeintervaldata") {
                    if (popupItem) {
                        _dataloggerExecuteInterval = editInterval(popupItem->property("interval").toInt(), id);
                    }
                    if (popupItem) {
                        popupItem->setProperty("interval", _dataloggerExecuteInterval);
                    }
                }
                if (_popupActiveButton == "rsvintervaldata") {
                    if (popupItem) {
                        _dataloggerExecuteInterval = editInterval(popupItem->property("rsvInterval").toInt(), id);
                        qDebug() << "[Debug] _dataloggerExecuteInterval: " << _dataloggerExecuteInterval;
                        popupItem->setProperty("rsvInterval", _dataloggerExecuteInterval);
                    }

                } 
                if (_popupActiveButton == "rsvcycledata") {
                    if (popupItem) {
                        _dataloggerExecuteCycle = editCycle(popupItem->property("rsvCycle").toInt(), id);
                        qDebug() << "[Debug] _dataloggerExecuteCycle: " << _dataloggerExecuteCycle;
                        popupItem->setProperty("rsvCycle", _dataloggerExecuteCycle);
                    }

                }
            }
        } else {
            QString nextBtn;

            if ((id == BUTTON_UP) || (id == BUTTON_DOWN)) {
                setVisibleSelectEditImage(id);

                // 上下移動は左端のボタンに移動
                if ((_popupActiveButton == _dataLoggerPopupType3ButtonData[0][0])
                        || _popupActiveButton == _dataLoggerPopupType3ButtonData[0][1]
                        || _popupActiveButton == _dataLoggerPopupType3ButtonData[0][2]
                        || _popupActiveButton == _dataLoggerPopupType3ButtonData[0][3]) {
                    if (id == BUTTON_UP) {
                        nextBtn = _dataLoggerPopupType3ButtonData[2][0];
                    } else {
                        nextBtn = _dataLoggerPopupType3ButtonData[1][0];
                    }
                }

                if (_popupActiveButton == _dataLoggerPopupType3ButtonData[1][0]
                        || _popupActiveButton == _dataLoggerPopupType3ButtonData[1][1]
                        || _popupActiveButton == _dataLoggerPopupType3ButtonData[1][2]
                        || _popupActiveButton == _dataLoggerPopupType3ButtonData[1][3]
                        || _popupActiveButton == _dataLoggerPopupType3ButtonData[1][4]) {
                    if (id == BUTTON_UP) {
                        nextBtn = _dataLoggerPopupType3ButtonData[0][0];
                    } else {
                        nextBtn = _dataLoggerPopupType3ButtonData[2][0];
                    }
                }

                if (_popupActiveButton == _dataLoggerPopupType3ButtonData[2][0]) {
                    if (id == BUTTON_UP) {
                        nextBtn = _dataLoggerPopupType3ButtonData[1][0];
                    } else {
                    	nextBtn = _dataLoggerPopupType3ButtonData[0][0];
                	}
                }
            } else if ((id == BUTTON_RIGHT) || (id == BUTTON_LEFT)){
                if (_popupActiveButton == _dataLoggerPopupType3ButtonData[1][0]) {
                    if (id == BUTTON_RIGHT) {
                        nextBtn = _dataLoggerPopupType3ButtonData[1][1];
                    } else {
                        nextBtn = _dataLoggerPopupType3ButtonData[1][1];
                    }
                }
                else if (_popupActiveButton == _dataLoggerPopupType3ButtonData[1][1]) {
                    if (id == BUTTON_LEFT) {
                        nextBtn = _dataLoggerPopupType3ButtonData[1][0];
            		} else {
                        nextBtn = _dataLoggerPopupType3ButtonData[1][0];
                    }
                }

            }

            // active 表示の切替
            QObject *nextTarget = nullptr;
            if (_view) {
                nextTarget = _view->findChild<QObject *>(nextBtn);
            }

            if (nextTarget) {
                if (nextTarget->property("disable") == false) {
                    // inactive
                    QObject *target = nullptr;
                    if (_view) {
                        target = _view->findChild<QObject *>(_popupActiveButton);
                    }
                    if (target) {
                        target->setProperty("active", false);
                    }

                    // active
                    target = nullptr;
                    if (_view) {
                        target = _view->findChild<QObject *>(nextBtn);
                    }
                    if (target) {
                        target->setProperty("active", true);
                    }

                    qDebug() << "nextBtn: " << nextBtn;
                    _popupActiveButton = nextBtn;
                    _popupButtonOperation.setActiveObject(_popupActiveButton);
                }
            }
        }

        break;
    }

    case CORRECTINTERVALSET: {

        if (_popupState == EDIT) {
            QObject *popupItem = nullptr;
            if (_view) {
                popupItem = _view->findChild<QObject *>("dgPopup");
            }
            if (popupItem) {
                // 更新間隔の変更
                if (id == BUTTON_UP || id == BUTTON_DOWN) {
                    if (_popupActiveButton == "intervaldata") {
                        _dataloggerExecuteInterval = editInterval(popupItem->property("interval").toInt(), id);

                        popupItem->setProperty("interval", _dataloggerExecuteInterval);
                    } else {
                        _dataloggerExecuteCycle = editCycle(popupItem->property("cycle").toInt(), id);

                        popupItem->setProperty("cycle", _dataloggerExecuteCycle);
                    }
                }
            }
        } else {
            if (id == BUTTON_UP || id == BUTTON_DOWN || id == BUTTON_RIGHT || id == BUTTON_LEFT) {
                QString nextBtn;
                bool ret = _popupButtonOperation.nextActiveObject(id, nextBtn);

                // active 表示の切替
                if (ret) {
                    QObject *nextTarget = nullptr;
                    if (_view) {
                        nextTarget = _view->findChild<QObject *>(nextBtn);
                    }

                    if (nextTarget) {
                        if (nextTarget->property("disable") == false) {
                            // inactive
                            QObject *target = nullptr;
                            if (_view) {
                                target = _view->findChild<QObject *>(_popupActiveButton);
                            }
                            if (target) {
                                target->setProperty("active", false);
                            }

                            // active
                            target = nullptr;
                            if (_view) {
                                target = _view->findChild<QObject *>(nextBtn);
                            }
                            if (target) {
                                target->setProperty("active", true);
                            }

                            _popupActiveButton = nextBtn;
                            _popupButtonOperation.setActiveObject(_popupActiveButton);
                        }
                    }
                }
            }
        }

        break;
    }

    case WAITING: {
        break;
    }

    default:
        break;
    }
}

// dataloggerviewmodel から移植
void UserDataLoggerSettingViewModel::keyStatusPopupArea(ButtonId id)
{
    QObject *target = nullptr;
    if (_view) {
        target = _view->findChild<QObject *>(_popupActiveButton);
    }
    if (target == nullptr) {
        return;
    }

    target->setProperty("press", false);

    if (id == BUTTON_ENTER) {
        QObject *popupItem = nullptr;
        if (_view) {
            popupItem = _view->findChild<QObject *>("dgPopup");
        }
        if (popupItem == nullptr) {
            return;
        }
        switch (_popupType) {
        case COLLECTCONFIRM: {
            if (_popupActiveButton == "m1") {
                // 取得間隔設定POPUPを表示する
                popupItem->setProperty("popupType", CORRECTINTERVALSET);
                popupItem->setProperty("visible", true);
                popupItem->setProperty("message", "");
                popupItem->setProperty("leftButtonMessage", "取得開始");
                _popupType = CORRECTINTERVALSET;
                _popupButtonOperation.setButtonName(2, 2, _dataLoggerPopupType2ButtonData);
                _popupButtonOperation.setActiveObject(_dataLoggerPopupType2ButtonData[0][0]);
                _popupActiveButton = _dataLoggerPopupType2ButtonData[0][0];
                _popupState = SELECT;
                // ボタンのアクティブ・Editカーソル戻し.
                QObject * target = popupItem->findChild<QObject *>("intervaldataCur");
                if (target) {
                    target->setProperty("source", "qrc:/resource/png/datalogger_interval_cycle_select.png");
                }
                target = popupItem->findChild<QObject *>("cycledata");
                if (target) {
                    target->setProperty("active", false);
                }
                target = popupItem->findChild<QObject *>("cycledataCur");
                if (target) {
                    target->setProperty("source", "qrc:/resource/png/datalogger_interval_cycle_select.png");
                }
            } else if (_popupActiveButton == "m2") {
                // 取得日時期間設定POPUPを表示する
                popupItem->setProperty("popupType", DATETIMESET);
                popupItem->setProperty("message", "");
                popupItem->setProperty("leftButtonMessage", "取得開始");

                _popupType = DATETIMESET;
                _popupButtonOperation.setButtonName(3, 5, _dataLoggerPopupType3ButtonData);
                _popupButtonOperation.setActiveObject(_dataLoggerPopupType3ButtonData[0][0]);
                _popupActiveButton = _dataLoggerPopupType3ButtonData[0][0];
            	_popupState = SELECT;
                initDatetimeset();
                setSelectModeImage();
                setVisibleSelectEditImage(BUTTON_UNKNOWN);
                // ボタンのアクティブ戻し.
                QObject *target = popupItem->findChild<QObject *>("startdaydata");
                if (target) {
                    target->setProperty("active", false);
                }
                target = popupItem->findChild<QObject *>("starthourdata");
                if (target) {
                    target->setProperty("active", false);
                }
                target = popupItem->findChild<QObject *>("startminutesdata");
                if (target) {
                    target->setProperty("active", false);
                }
                target = popupItem->findChild<QObject *>("rsvintervaldataCur");
                if (target) {
                    target->setProperty("source", "qrc:/resource/png/datalogger_interval_cycle_select.png");
                }
                target = popupItem->findChild<QObject *>("rsvcycledataCur");
                if (target) {
                    target->setProperty("source", "qrc:/resource/png/datalogger_interval_cycle_select.png");
                }
                target = popupItem->findChild<QObject *>("rsvintervaldata");
                if (target) {
                    target->setProperty("active", false);
                }
                target = popupItem->findChild<QObject *>("rsvcycledata");
                if (target) {
                    target->setProperty("active", false);
                }
                target = popupItem->findChild<QObject *>("m2");
                if (target) {
                    target->setProperty("active", false);
                }
            }

            // 間隔の初期値は連続(=0)
            popupItem->setProperty("interval", 0);
            popupItem->setProperty("cycle", 0);
            popupItem->setProperty("rsvInterval", 0);
            popupItem->setProperty("rsvCycle", 0);

            QObject *target = popupItem->findChild<QObject *>(_popupActiveButton);

            if (target) {
                target->setProperty("active", true);
            }

            break;
        }

        case DATETIMESET: {
            onKeyEnterDatetimeset();
            break;
        }

        case CORRECTINTERVALSET: {
            if (_popupActiveButton == "m1") {
                transitionDataLoggerMeasurementList();
            } else if (_popupActiveButton == "intervaldata") {
                // 編集・選択モードの切替
                _popupState = (_popupState == SELECT) ? EDIT : SELECT;

                QObject *target = nullptr;
                if (_view) {
                    target = _view->findChild<QObject *>("intervaldataCur");
                }

                if (target) {
                    if (_popupState == SELECT) {
                        target->setProperty("source", "qrc:/resource/png/datalogger_interval_btn_d.png");
                    } else {
                        target->setProperty("source", "qrc:/resource/png/datalogger_interval_btn_edit.png");
                    }
                }

            } else if (_popupActiveButton == "cycledata") {
                // 編集・選択モードの切替
                _popupState = (_popupState == SELECT) ? EDIT : SELECT;

                QObject *target = nullptr;
                if (_view) {
                    target = _view->findChild<QObject *>("cycledataCur");
                }

                if (target) {
                    if (_popupState == SELECT) {
                        target->setProperty("source", "qrc:/resource/png/datalogger_interval_btn_d.png");
                    } else {
                        target->setProperty("source", "qrc:/resource/png/datalogger_interval_btn_edit.png");
                    }
                }

            } else {
            }

            break;
        }

        case WAITING: {
            break;
        }

        case NODATA: {
            if (_popupActiveButton == "m1") {
                // POPUPを消去してロガーの画面に戻る
                _reserveTimer->stop();
                QObject *popupItem = nullptr;
                if (_view) {
                    popupItem = _view->findChild<QObject *>("dgPopup");
                }
                if (popupItem) {
                    popupItem->setProperty("visible", false);
                }
                _selectArea = FILELIST_AREA;
            }

            break;
        }
        case CANCELCONFIRM: {
            if (_popupActiveButton == "m1") {
                // POPUPを消去してロガーの画面に戻る
                _reserveTimer->stop();
                QObject *popupItem = nullptr;
                if (_view) {
                    popupItem = _view->findChild<QObject *>("dgPopup");
                }
                if (popupItem) {
                    popupItem->setProperty("visible", false);
                }
                _popupType = TYPEUNKNOWN;
                _selectArea = FILELIST_AREA;

                // オートパワーオフのタイマーを再開
                _model->enablePowerOffSetting();

            } else if (_popupActiveButton == "m2") {
                // 実行待ちPOPUPを表示する
                popupItem->setProperty("popupType", WAITING);
                _popupType = WAITING;
                QString message1("");
                QObject *target = popupItem->findChild<QObject *>("startmonthdata");
                if (target) {
                    message1 += target->property("message").toString();
                    message1 += " ";
                }
                target = popupItem->findChild<QObject *>("startdaydata");
                if (target) {
                    message1 += target->property("message").toString();
                    message1 += " ";
                }
                target = popupItem->findChild<QObject *>("starthourdata");
                if (target) {
                    QString hour = target->property("message").toString();
                    message1 += hour.rightJustified(2,'0');
                    message1 += ":";
                }
                target = popupItem->findChild<QObject *>("startminutesdata");
                if (target) {
                    QString min = target->property("message").toString();
                    message1 += min.rightJustified(2,'0');
                    message1 += " ";
                }

                popupItem->setProperty("visible", true);
                popupItem->setProperty("message", message1 +
                                     "にデータを取得開始します。\n\n※別画面に遷移した場合、予約設定がキャンセルされます。");
                popupItem->setProperty("leftButtonMessage", "");
            }

            break;
        }

        default:
            break;
        }
    } else {
        if ((_popupType == WAITING) && (id == BUTTON_ESCAPE)) {
            QObject *popupItem = nullptr;
            if (_view) {
                popupItem = _view->findChild<QObject *>("dgPopup");
            }
            if (popupItem) {
                popupItem->setProperty("popupType", CANCELCONFIRM);
                _popupType = CANCELCONFIRM;
                popupItem->setProperty("visible", true);
                popupItem->setProperty("message", "");
                popupItem->setProperty("leftButtonMessage", "はい");
                popupItem->setProperty("rightButtonMessage", "いいえ");
                popupItem->setProperty("visible", true);
                _popupButtonOperation.setButtonName(1, 2, _dataLoggerPopupType5ButtonData);
                _popupButtonOperation.setActiveObject(_dataLoggerPopupType5ButtonData[0][0]);
                _popupActiveButton = _dataLoggerPopupType5ButtonData[0][0];
                QObject* target = popupItem->findChild<QObject *>(_popupActiveButton);
                if (target) {
                    target->setProperty("active", true);
                }

                // ボタンのアクティブ・Editカーソル戻し.
                target = popupItem->findChild<QObject *>("m2");
                if (target) {
                    target->setProperty("active", false);
                }


            }

        }
        moveCursorPopupArea(id);
    }
}

void UserDataLoggerSettingViewModel::updateStartMonth(ButtonId id)
{
    QDate date(_yyyy, _start_mm, _start_dd);

    if (id == BUTTON_UP) {
        date = date.addMonths(1);
    } else if (id == BUTTON_DOWN) {
        date = date.addMonths(-1);
    } else {
        // nothing to do
    }

    _start_mm = date.month();
    // 月末日付の調整
    int day = date.day();

    if (_start_dd != day) {
        _start_dd = day;
    }

    QObject *btn1 = nullptr;
    if (_view) {
        btn1 = _view->findChild<QObject *>("startmonthdata");
    }
    if (btn1) {
        btn1->setProperty("message", QString().sprintf("%d月", _start_mm));
    }

    QObject *btn2 = nullptr;
    if (_view) {
        btn2 = _view->findChild<QObject *>("startdaydata");
    }
    if (btn2) {
        btn2->setProperty("message", QString().sprintf("%d日", _start_dd));
    }
}

void UserDataLoggerSettingViewModel::updateStartDay(ButtonId id)
{
    QDate date(_yyyy, _start_mm, _start_dd);

    if (id == BUTTON_UP) {
        _start_dd++;
    } else if (id == BUTTON_DOWN) {
        _start_dd--;
    } else {
        // nothing to do
    }

    // 月末日付の調整
    int daysInMonth = date.daysInMonth();
    _start_dd = (_start_dd + daysInMonth) % daysInMonth;

    if (_start_dd == 0) {
        _start_dd = daysInMonth;
    }

    QObject *btn = nullptr;
    if (_view) {
        btn = _view->findChild<QObject *>("startdaydata");
    }
    if (btn) {
        btn->setProperty("message", QString().sprintf("%d日", _start_dd));
    }
}

void UserDataLoggerSettingViewModel::updateStartHour(ButtonId id)
{
    if (id == BUTTON_UP) {
        _start_HH++;
    } else if (id == BUTTON_DOWN) {
        _start_HH--;
    } else {
        // nothing to do.
    }

    _start_HH = (_start_HH + 24) % 24;
    QObject *btn = nullptr;
    if (_view) {
        btn = _view->findChild<QObject *>("starthourdata");
    }
    if (btn) {
#if 1
        btn->setProperty("message", QString().sprintf("%d", _start_HH).rightJustified(2, '0'));
#else
        btn->setProperty("message", QString().sprintf("%d", _start_HH));
#endif
    }
}

void UserDataLoggerSettingViewModel::updateStartMinutes(ButtonId id)
{
    if (id == BUTTON_UP) {
        _start_MM++;
    } else if (id == BUTTON_DOWN) {
        _start_MM--;
    } else {
        // nothing to do.
    }

    _start_MM = (_start_MM + 60) % 60;
    QObject *btn = nullptr;
    if (_view) {
        btn = _view->findChild<QObject *>("startminutesdata");
    }
    if (btn) {
#if 1
        btn->setProperty("message", QString().sprintf("%d", _start_MM).rightJustified(2, '0'));
#else
        btn->setProperty("message", QString().sprintf("%d", _start_MM));
#endif
    }
}

void UserDataLoggerSettingViewModel::updateEndMonth(ButtonId id)
{
    QDate date(_yyyy, _end_mm, _end_dd);

    if (id == BUTTON_UP) {
        date = date.addMonths(1);
    } else if (id == BUTTON_DOWN) {
        date = date.addMonths(-1);
    } else {
        // nothing to do
    }

    _end_mm = date.month();
    // 月末日付の調整
    int day = date.day();

    if (_end_dd != day) {
        _end_dd = day;
    }

    QObject *btn1 = nullptr;
    if (_view) {
        btn1= _view->findChild<QObject *>("endmonthdata");
    }
    if (btn1) {
        btn1->setProperty("message", QString().sprintf("%d月", _end_mm));
    }

    QObject *btn2 = nullptr;
    if (_view) {
        btn2 = _view->findChild<QObject *>("enddaydata");
    }
    if (btn2) {
        btn2->setProperty("message", QString().sprintf("%d日", _end_dd));
    }
}

void UserDataLoggerSettingViewModel::updateEndDay(ButtonId id)
{
    QDate date(_yyyy, _end_mm, _end_dd);

    if (id == BUTTON_UP) {
        _end_dd++;
    } else if (id == BUTTON_DOWN) {
        _end_dd--;
    } else {
        // nothing to do
    }

    // 月末日付の調整
    int daysInMonth = date.daysInMonth();
    _end_dd = (_end_dd + daysInMonth) % daysInMonth;

    if (_end_dd == 0) {
        _end_dd = daysInMonth;
    }

    QObject *btn = nullptr;
    if (_view) {
        btn = _view->findChild<QObject *>("enddaydata");
    }
    if (btn) {
        btn->setProperty("message", QString().sprintf("%d日", _end_dd));
    }
}

void UserDataLoggerSettingViewModel::updateEndHour(ButtonId id)
{
    if (id == BUTTON_UP) {
        _end_HH++;
    } else if (id == BUTTON_DOWN) {
        _end_HH--;
    } else {
        // nothing to do.
    }

    _end_HH = (_end_HH + 24) % 24;
    QObject *btn = nullptr;
    if (_view) {
        btn = _view->findChild<QObject *>("endhourdata");
    }

    if (btn) {
        btn->setProperty("message", QString().sprintf("%d", _end_HH));
    }
}

void UserDataLoggerSettingViewModel::updateEndMinutes(ButtonId id)
{
    if (id == BUTTON_UP) {
        _end_MM++;
    } else if (id == BUTTON_DOWN) {
        _end_MM--;
    } else {
        // nothing to do.
    }

    _end_MM = (_end_MM + 60) % 60;
    QObject *btn = nullptr;
    if (_view) {
        btn = _view->findChild<QObject *>("endminutesdata");
    }

    if (btn) {
        btn->setProperty("message", QString().sprintf("%d", _end_MM));
    }
}

void UserDataLoggerSettingViewModel::initDatetimeset()
{
    // 現在時刻を取得
    QDateTime currentdatetime = QDateTime::currentDateTime();

    QString val;
    val = currentdatetime.toString("yyyy");
    _yyyy = val.toInt();

    val = currentdatetime.toString("MM");
    _start_mm = val.toInt();
    _end_mm = val.toInt();

    val = currentdatetime.toString("dd");
    _start_dd = val.toInt();
    _end_dd = val.toInt();

    val = currentdatetime.toString("HH");
    _start_HH = val.toInt();
    _end_HH = val.toInt();

    val = currentdatetime.toString("mm");
#if 0
    //5分先の時刻を表示する
    _start_MM = val.toInt() + 5;
#else
    _start_MM = val.toInt();
#endif
    _end_MM = val.toInt();

    // 初期表示
    updateStartMonth(BUTTON_UNKNOWN);
    updateStartDay(BUTTON_UNKNOWN);
    updateStartHour(BUTTON_UNKNOWN);
    updateStartMinutes(BUTTON_UNKNOWN);

    updateEndMonth(BUTTON_UNKNOWN);
    updateEndDay(BUTTON_UNKNOWN);
    updateEndHour(BUTTON_UNKNOWN);
    updateEndMinutes(BUTTON_UNKNOWN);

    // 測定可能時間を表示する
    QObject *target = nullptr;
    if (_view) {
        target = _view->findChild<QObject *>("measurableTime");
    }
    if (target) {
        if (_popupType == DATETIMESET) {
            target->setProperty("text", "開始日時");
        } else {
            target->setProperty("text", "測定可能時間は約 24 時間");
        }
    }
}

void UserDataLoggerSettingViewModel::setSelectModeImage()
{
    // startDatetimeBackground
    // endDatetimeBackground
    // startmonthdata
    // endmonthdata
    QString object1("startDatetimeBackground");
    QObject *startTimeImage = nullptr;
    if (_view) {
        startTimeImage = _view->findChild<QObject *>(object1);
    }
    if (startTimeImage) {
        startTimeImage->setProperty("startDatetimeBackgroundImage",
                                    "qrc:/resource/png/datalogger_datetimeset_starttime_cur.png");
    }

    QString object2("endDatetimeBackground");
    QObject *endTimeImage = nullptr;
    if (_view) {
        endTimeImage = _view->findChild<QObject *>(object2);
    }
    if (endTimeImage) {
        endTimeImage->setProperty("endDatetimeBackgroundImage", "qrc:/resource/png/datalogger_datetimeset_endtime_cur.png");
    }

    QString object3("startmonthdata");
    QObject *startmonthdataImage = nullptr;
    if (_view) {
        startmonthdataImage = _view->findChild<QObject *>(object3);
    }
    if (startmonthdataImage) {
        startmonthdataImage->setProperty("activebgImage", "");
    }

    QString object4("endmonthdata");
    QObject *endmonthdataImage = nullptr;
    if (_view) {
        endmonthdataImage = _view->findChild<QObject *>(object4);
    }
    if (endmonthdataImage) {
        endmonthdataImage->setProperty("activebgImage", "");
    }
}

void UserDataLoggerSettingViewModel::setEditModeImage()
{
    // startDatetimeBackground
    // endDatetimeBackground
    // startmonthdata
    // endmonthdata
    QString object1("startDatetimeBackground");
    QObject *startTimeImage = nullptr;
    if (_view) {
        startTimeImage = _view->findChild<QObject *>(object1);
    }
    if (startTimeImage) {
        startTimeImage->setProperty("startDatetimeBackgroundImage",
                                    "qrc:/resource/png/datalogger_datetimeset_starttime_cur_edit.png");
    }

    QString object2("startmonthdata");
    QObject *startmonthdataImage = nullptr;
    if (_view) {
        startmonthdataImage = _view->findChild<QObject *>(object2);
    }
    if (startmonthdataImage) {
        startmonthdataImage->setProperty("activebgImage", "datetime_btn.png");
    }
}

void UserDataLoggerSettingViewModel::transitionDataLoggerMeasurementList()
{
    //取得開始
    // ダイアログを消去して収集画面に遷移する
    QObject *popupItem = nullptr;
    if (_view) {
        popupItem = _view->findChild<QObject *>("dgPopup");
    }
    if (popupItem) {
        popupItem->setProperty("visible", false);
    }
    _selectArea = ACTION_AREA;

    // データロガー測定ファイル数が1000件以上の場合は測定を開始せずに、エラーポップアップを表示する。
    QString path = _model->applicationSetting()->resultDataPathLogger();
    int fileCount = countFilesRecursive(path);
    if (fileCount >= 1000) {
        emit requestToShowPopup(POPUP_DATA_LOGGER_MAX_SAVE_FILE);
        _selectArea = FILELIST_AREA;
        return;
    }

    // 収集間隔を取得
    // データ収集実行結果画面を表示

    // 選択ユーザリストの保存
    QVariant value;
    _model->getValue(VALUE_DATALOGGERSETTING_FILELIST, value);

    QVariantList userList = value.toList();

    if (_currentFileIndex < userList.size()) {
        QVariantList userData = userList[_currentFileIndex].toList();

        QString filename = userData.at(2).toString();
        // ".csv" なしで保存
        emit requestSetTakeOver("selectedUserListFile", filename);
    }

    QObject *target = nullptr;
    if (_view) {
        target = _view->findChild<QObject *>("dgPopup");
    }
    if (target) {
        value = target->property("interval");
    }
    emit requestSetTakeOver("getInterval", value.toInt());
    if (target) {
        value = target->property("cycle");
    }
    emit requestSetTakeOver("getCycle", value.toInt());
    emit requestSetTakeOver("enddatetimeyyyy", _end_yyyy);
    emit requestSetTakeOver("enddatetimemm", _end_mm);
    emit requestSetTakeOver("enddatetimedd", _end_dd);
    emit requestSetTakeOver("enddatetimeHH", _end_HH);
    emit requestSetTakeOver("enddatetimeMM", _end_MM);


    onRequest(1);
}

void UserDataLoggerSettingViewModel::setVisibleSelectEditImage(ButtonId id)
{
    // 画像表示非表示切り替え
    // startmonthdata
    // endmonthdata
    // m1

    bool visibleStart = false;

    if (id == BUTTON_UNKNOWN) {
        // 初期表示時で
        visibleStart = true;
    }

    // 上下移動は左端のボタンに移動
    if ((_popupActiveButton == "startmonthdata")
            || (_popupActiveButton == "startmonthdata")
            || _popupActiveButton == "starthourdata"
            || _popupActiveButton == "startminutesdata") {
        if (id == BUTTON_UP || id == BUTTON_DOWN) {
            visibleStart = false;
        }
    }

    if (_popupActiveButton == _dataLoggerPopupType3ButtonData[1][0]
            || _popupActiveButton == _dataLoggerPopupType3ButtonData[1][1]
            || _popupActiveButton == _dataLoggerPopupType3ButtonData[1][2]
            || _popupActiveButton == _dataLoggerPopupType3ButtonData[1][3]
            || _popupActiveButton == _dataLoggerPopupType3ButtonData[1][4]) {
        if (id == BUTTON_UP) {
            visibleStart = true;
        } else {
            visibleStart = false;
        }
    }

    if (_popupActiveButton == _dataLoggerPopupType3ButtonData[2][0]) {
        if (id == BUTTON_UP) {
            visibleStart = false;
        } else {
        	visibleStart = true;
		}
    }

    QString object1("startDatetimeBackground");
    QObject *startTimeImage = nullptr;
    if (_view) {
        startTimeImage = _view->findChild<QObject *>(object1);
    }
    if (startTimeImage) {
        startTimeImage->setProperty("visible", visibleStart);
    }
}

bool UserDataLoggerSettingViewModel::isEnableMERorBER()
{
    QVariant value;
    QVariantList list;
    _model->getValue(VALUE_USERSETTING_SETTINGLIST, value);
    list = value.toList();

    foreach (const QVariant &item, list) {
        QVariantMap data = item.toMap();

        if (data["MERCN"].toBool()) {
            return true;
        }

        if (data["BER"].toBool()) {
            return true;
        }
    }

    return false;
}

int UserDataLoggerSettingViewModel::editInterval(int interval, ButtonId id)
{
    if (id == BUTTON_UP) {
        interval++;

        // MER/BER が ON の設定がある場合には 5分間隔は選択肢から外す
        if (interval == 1 && isEnableMERorBER()) {
            interval++;
        }
    } else if (id == BUTTON_DOWN) {
        interval--;

        // MER/BER が ON の設定がある場合には 5分間隔は選択肢から外す
        if (interval == 1 && isEnableMERorBER()) {
            interval--;
        }
    }

    // 下限・上限チェック
    if (interval < 0) {
        interval = 4;
    } else if (interval  > 4) {
        interval = 0;
    }

    return interval;
}

int UserDataLoggerSettingViewModel::editCycle(int interval, ButtonId id)
{
    if (id == BUTTON_UP) {
        interval++;

    } else if (id == BUTTON_DOWN) {
        interval--;

    }

    // 下限・上限チェック
    if (interval < 0) {
        interval = 9;
    } else if (interval  > 9) {
        interval = 0;
    }
    
    return interval;
}

int UserDataLoggerSettingViewModel::isStartDataCollection(QString &errorMessage)
{
    // 1．開始日時の未来日時チェック
    //   開始日時＜＝現在日時＋1分の場合、エラーとする
    //
    // 3．測定可能時間チェック
    //   終了日時－開始日時＞測定可能時間の場合、エラーとする

    // 注意:
    //   開始日時＞終了日時の場合、年を扱っていないため、終了日時は翌年の指定として処理する

    // errorcode
    // 0x01:開始日過去扱い
    // 0x02:

    int errorStart = 1;
    int errorSize  = 2;

    // 現在時刻を取得
    QDateTime currentdatetime = QDateTime::currentDateTime();

    QString val;
    val = currentdatetime.toString("yyyy");
    int current_yyyy = val.toInt();

    val = currentdatetime.toString("MM");
    int current_mm = val.toInt();

    val = currentdatetime.toString("dd");
    int current_dd = val.toInt();

    val = currentdatetime.toString("HH");
    int current_HH = val.toInt();

    val = currentdatetime.toString("mm");
    int current_MM = val.toInt();


    QDate currentDay(current_yyyy, current_mm, current_dd);
#if 0
    // 年は同じ
    QDate startDay(current_yyyy, _start_mm, _start_dd);
#else
    // 未来か確認
    // (現在月の-1)月までを過去と判定
    QDate beforeDay = currentDay.addMonths(-1);     // この月日(年無視)以前は未来扱い
    QDate startDay(beforeDay.year(), _start_mm, _start_dd);
    _yyyy = beforeDay.year();

    qint64 diffDay = beforeDay.daysTo(startDay);

    // 未来なら年を足しておく
    if (diffDay < 0) {
        _yyyy = _yyyy + 1;
    }

#endif

//    qint64 diffDay = currentDay.daysTo(startDay);

//    if (diffDay > 0) {
//        // 未来が設定されている
//    } else if (diffDay < 0) {
//        // 過去が設定されている
//        errorMessage = "開始月日の指定が過去になっています";
//        return errorStart;
//    } else {
//         // 現在時刻プラス１分より前の場合、エラーとする
//         if ((_start_HH * 60 + _start_MM) <= (current_HH * 60 + current_MM + 1)) {
//             errorMessage = "開始の分設定は現在より2分以上\n未来を設定してください";
//             return errorStart;
//         }
//    }

// _endは未使用？
    // 翌年か確認する _end_yyyy を設定する
    // サイズチェックまで処理を動かして、収まらないNGとする
    QDate endDay(current_yyyy, _end_mm, _end_dd);

    diffDay = startDay.daysTo(endDay);

    if (diffDay < 0) {
        _end_yyyy = current_yyyy + 1;
    }

    // データロガー測定ファイル数が1000件以上の場合は測定を開始せずに、エラーポップアップを表示する。
    QString path = _model->applicationSetting()->resultDataPathLogger();
    int fileCount = countFilesRecursive(path);
    if (fileCount >= 1000) {
        emit requestToShowPopup(POPUP_DATA_LOGGER_MAX_SAVE_FILE);
        return -1;
    }

#if 0 // USBの残量チェック
    // 測定可能時間のチェック
    if (isMeasurable()) {
        return 0;
    } else {
        errorMessage = QString("USBの保存領域が不足しています。\n不要なデータを消去してください。");
        return errorSize;
    }
#endif
    return 0;
}

bool UserDataLoggerSettingViewModel::isMeasurable()
{
    quint64 availableSize = getAvailableSize();

    if (availableSize == 0) {
        return false;
    }

    quint64 estimatedUseSize = getEstimatedUseSize();

    if (estimatedUseSize == 0) {
        return false;
    }

    if (estimatedUseSize < availableSize) {
        return true;
    } else {
        return false;
    }
}

quint64 UserDataLoggerSettingViewModel::getAvailableSize()
{
    // 1. 空きがあるか
    //bool isNoSpace = UsbOperation::freeSpace();

    bool isNoSpace = false;

    if (isNoSpace) {
        return 0;
    }

    // 2. 空きサイズはどれくらいか
    quint64 udbFreeSpace = UsbOperation::getFreeSpaceSize();
    return udbFreeSpace;
}

quint64 UserDataLoggerSettingViewModel::getEstimatedUseSize()
{
    // 測定可能時間＝USBの空き容量－（※1×リストのCH数×※2）
    // ※1：設定ファイルに定義された、1CHのデータ量
    // ※2：時間内に計測できる回数＝（終了日時－開始日時）／間隔

    QDateTime enddatetime(QDate(_end_yyyy, _end_mm, _end_dd), QTime(_end_HH, _end_MM, 0));
    QDateTime startdatetime(QDate(_yyyy, _start_mm, _start_dd), QTime(_start_HH, _start_MM, 0));
    qint64 msec = startdatetime.msecsTo(enddatetime);

    quint64 sec = msec / 1000;
    quint64 numberOfMeasurements = -1;

    // マルチ測定取得間隔(ms)
    quint64 interval = _model->applicationSetting()->pollingIntervalMultiMeasurement();
    // 1 秒間の測定取得回数
    int expectedCount = 1000 / interval;
    if (1000 % interval) {
        ++expectedCount;
    }

    switch (_dataloggerExecuteInterval) {
    case 0:// 0 : 連続
        numberOfMeasurements = sec / 1 * expectedCount;
        break;

    case 1:// 1 :  5min
        numberOfMeasurements = sec / (5 * 60) * expectedCount;
        break;

    case 2:// 2 : 15min
        numberOfMeasurements = sec / (15 * 60) * expectedCount;
        break;

    case 3:// 3 : 30min
        numberOfMeasurements = sec / (30 * 60) * expectedCount;
        break;

    case 4:// 4 : 60min
        numberOfMeasurements = sec / (60 * 60) * expectedCount;
        break;

    default:
        break;
    }

    if (numberOfMeasurements < 1) {
        return 0;
    }

    // 設定ファイルの有効行から概算で容量を見積もる
    quint64 needSize = MEASUREMENT_1CH_DATA_SIZE * validChannelNumber() * numberOfMeasurements;
    return needSize;
}

void UserDataLoggerSettingViewModel::onKeyEnterDatetimeset()
{
    QObject *popupItem = nullptr;
    if (_view) {
        popupItem = _view->findChild<QObject *>("dgPopup");
    }

    if (_popupActiveButton == "m1") {
        QString errorMessage;
        int errorCode = isStartDataCollection(errorMessage);
        if (errorCode == -1) { // 測定ファイル上限超過
            // ダイアログを消去して収集画面に遷移する
            QObject *popupItem = nullptr;
            if (_view) {
                popupItem = _view->findChild<QObject *>("dgPopup");
            }
            if (popupItem) {
                popupItem->setProperty("visible", false);
            }
            _selectArea = FILELIST_AREA;
            return;
        }
        if (errorCode == 0) {
            // 現在時刻を取得
            QDateTime currentdatetime = QDateTime::currentDateTime();
            QDateTime startdatetime(QDate(_yyyy, _start_mm, _start_dd), QTime(_start_HH, _start_MM, 0));
            qint64 msec = currentdatetime.msecsTo(startdatetime);

            if (msec > 3000) {
                // 現在時刻より3秒を越える場合はタイマー発火測定

                if (popupItem) {

                    // オートパワーオフを無効化
                    _model->disablePowerOffSetting();

                    // 実行待ちPOPUPを表示する
                    popupItem->setProperty("popupType", WAITING);
                    _popupType = WAITING;

                    QString message1("");

                    QObject *target = popupItem->findChild<QObject *>("startmonthdata");

                    if (target) {
                        message1 += target->property("message").toString();
                        message1 += " ";
                    }

                    target = popupItem->findChild<QObject *>("startdaydata");

                    if (target) {
                        message1 += target->property("message").toString();
                        message1 += " ";
                    }

                    target = popupItem->findChild<QObject *>("starthourdata");

                    if (target) {
                        QString hour = target->property("message").toString();
                        message1 += hour.rightJustified(2,'0');
                        message1 += ":";
                    }

                    target = popupItem->findChild<QObject *>("startminutesdata");

                    if (target) {
                        QString min = target->property("message").toString();
                        message1 += min.rightJustified(2,'0');
                        message1 += " ";
                    }

                    popupItem->setProperty("visible", true);
                    popupItem->setProperty("message", message1 +
                                         "にデータを取得開始します。\n\n※別画面に遷移した場合、予約設定がキャンセルされます。");
                    popupItem->setProperty("leftButtonMessage", "");
                }

                // ロギング開始日時を検知するタイマーを開始
                reserveMeasurement();

            } else {
                // 現在時刻より3秒以内の場合は即時測定を開始
                transitionDataLoggerMeasurementList();
            }
        } else {
            // 測定データの登録なし
            QObject *popupItem = nullptr;
            if (_view) {
                popupItem = _view->findChild<QObject *>("dgPopup");
            }
            if (popupItem) {
                popupItem->setProperty("popupType", NODATA);
                popupItem->setProperty("visible", true);
                popupItem->setProperty("message", errorMessage);
                popupItem->setProperty("leftButtonMessage", "戻る");

                _popupButtonOperation.setButtonName(1, 1, _dataLoggerPopupType4ButtonData);
                _popupButtonOperation.setActiveObject(_dataLoggerPopupType4ButtonData[0][0]);
                _popupActiveButton = _dataLoggerPopupType4ButtonData[0][0];

                QObject *target = popupItem->findChild<QObject *>(_popupActiveButton);

                if (target) {
                    target->setProperty("active", true);
                }
            }
            _selectArea = POPUP_AREA;
            _popupType = NODATA;
        }
    } else {
        _popupState = (_popupState  == EDIT) ? SELECT : EDIT;

        if (_popupState == SELECT) {
            setSelectModeImage();

            if ((_popupActiveButton == "startmonthdata") || (_popupActiveButton == "endmonthdata")) {
                // 何もしない
            } else if ((_popupActiveButton == "startdaydata") || (_popupActiveButton == "starthourdata")
                       || (_popupActiveButton == "startminutesdata")) {
                QObject *target = nullptr;
                if (_view) {
                    // active change.
                    target = _view->findChild<QObject *>(_popupActiveButton);
                }
                if (target) {
                    target->setProperty("active", false);
                }

                target = nullptr;
                if (_view) {
                    target = _view->findChild<QObject *>("startmonthdata");
                }

                if (target) {
                    target->setProperty("active", true);
                }

                _popupActiveButton = "startmonthdata";
                _popupButtonOperation.setActiveObject("startmonthdata");
            }
            else if (_popupActiveButton == "rsvintervaldata") {
                 QObject *target = nullptr;
                 if (_view) {
                     target = _view->findChild<QObject *>("rsvintervaldataCur");
                 }
                 if (target) {
                     target->setProperty("source", "qrc:/resource/png/datalogger_interval_btn_d.png");
                 }

             } else if (_popupActiveButton == "rsvcycledata") {
                 QObject *target = nullptr;
                 if (_view) {
                     target = _view->findChild<QObject *>("rsvcycledataCur");
                 }

                 if (target) {
                     target->setProperty("source", "qrc:/resource/png/datalogger_interval_btn_d.png");
                 }
            }


            else {
                // active change.
                QObject *target = nullptr;
                if (_view) {
                    target = _view->findChild<QObject *>(_popupActiveButton);
                }

                if (target) {
                    target->setProperty("active", false);
                }

                target = nullptr;
                if (_view) {
                    target = _view->findChild<QObject *>("endmonthdata");
                }

                if (target) {
                    target->setProperty("active", true);
                }

                _popupActiveButton = "endmonthdata";
            }
        } else {
            setEditModeImage();
            if (_popupActiveButton == "rsvintervaldata") {
                 QObject *target = nullptr;
                 if (_view) {
                     target = _view->findChild<QObject *>("rsvintervaldataCur");
                 }
                 if (target) {
                     target->setProperty("source", "qrc:/resource/png/datalogger_interval_btn_edit.png");
                 }

             } else if (_popupActiveButton == "rsvcycledata") {
                 QObject *target = nullptr;
                 if (_view) {
                     target = _view->findChild<QObject *>("rsvcycledataCur");
                 }
                 if (target) {
                     target->setProperty("source", "qrc:/resource/png/datalogger_interval_btn_edit.png");
                 }
            }
        }
    }
}

void UserDataLoggerSettingViewModel::reserveMeasurement()
{
    // RTC と同期
    _model->initDateTime();

    QDateTime currentdatetime = QDateTime::currentDateTime();
    QDateTime startdatetime(QDate(_yyyy, _start_mm, _start_dd), QTime(_start_HH, _start_MM, 0));
    qint64 msec = currentdatetime.msecsTo(startdatetime);

    qDebug() << QString("[TIMER] %1 ms remains.").arg(msec);

    // 開始日時の到来
    if (msec <= 0) {
        qDebug() << QString("TRANSIT DATA LOGGER");
        emit requestSetTakeOver("enddatetimeSet", true);
        transitionDataLoggerMeasurementList();

        _loggingStarted = !_loggingStarted;
        return;
    }

    // 残り時間があれば再度タイマーを開始
    if (msec > AN_HOUR) {
        qDebug() << QString("[TIMER] set %1 ms interval").arg(AN_HOUR);
        // 1 時間より大きければ 1 時間で設定
        _reserveTimer->start(AN_HOUR);
    } else {
        qDebug() << QString("[TIMER] set %1 ms interval").arg(msec);
        // 1 時間以内なら残り時間を設定
        _reserveTimer->start(msec);
    }
}

QList<QVariant> UserDataLoggerSettingViewModel::getChannelListColor(int bandId, int mode)
{
    /**
     *  @brief チャンネル毎のチャンネルリストの文字色
     *         true:白色 false:グレー
     *         [CH],[周波数],[放送規格],[局発],[MER/CN],[BER],[給電],[測定ON/OFF]
     *         CH:ここでの変更不可。常にfalseとすること
     *         測定ON/OFF:ここでの変更不可。常にfalseとすること
     */

    if(mode == CHANNEL_EDIT_DATA_LOGGER ||
       mode == CHANNEL_EDIT_DATA_LOGGER_USB1 ||
       mode == CHANNEL_EDIT_DATA_LOGGER_USB2) {
        // データロガー
        static const QMap <int, QList<QVariant>> dataloggerChannelListColorMap = {
                                  // CH      周波数   放送規格   局発     MER/CN  BER    給電    測定ON/OFF
            {BAND_TAB_UHF,          {false,  false,  false,   false,  false,  false,  true,  false}},
            {BAND_TAB_BS_R,         {false,  false,  false,   false,  false,  false,  true,  false}},
            {BAND_TAB_BS_L,         {false,  false,  false,   false,  false,  false,  true,  false}},
            {BAND_TAB_CS_R,         {false,  false,  false,   false,  false,  false,  true,  false}},
            {BAND_TAB_CS_L,         {false,  false,  false,   false,  false,  false,  true,  false}},
            {BAND_TAB_CATV_ALL,     {false,  false,  true,    false,  false,  false,  true,  false}},
            {BAND_TAB_SKY_PERFECTV, {false,  false,  false,   false,  false,  false,  true,  false}},
            {BAND_TAB_CATV_UPSTREAM,{false,  true,   false,   false,  false,  false,  true,  false}},
            {BAND_TAB_JCSAT3A_H,    {false,  false,  false,   true,   false,  false,  true,  false}},
            {BAND_TAB_JCSAT3A_V,    {false,  false,  false,   true,   false,  false,  true,  false}},
            {BAND_TAB_JCSAT4B_H,    {false,  false,  false,   true,   false,  false,  true,  false}},
            {BAND_TAB_JCSAT4B_V,    {false,  false,  false,   true,   false,  false,  true,  false}},
            {BAND_TAB_SCCB2_H,      {false,  false,  false,   true,   false,  false,  true,  false}},
            {BAND_TAB_SCCB2_V,      {false,  false,  false,   true,   false,  false,  true,  false}},
            {BAND_TAB_SCCC2_H,      {false,  false,  false,   true,   false,  false,  true,  false}},
            {BAND_TAB_SCCC2_V,      {false,  false,  false,   true,   false,  false,  true,  false}},
            {BAND_TAB_LTE,          {false,  false,  false,   false,  false,  false,  true,  false}},
            {BAND_TAB_FM,           {false,  true,   false,   false,  false,  false,  true,  false}},
            {BAND_TAB_PILOT,        {false,  false,  false,   false,  false,  false,  true,  false}},
        };
        return dataloggerChannelListColorMap.value(static_cast<int>(bandId), {false, false, false, false, false, false, false, false});
    } else {
        // USER
        static const QMap <int, QList<QVariant>> userChannelListColorMap = {
                                  // CH      周波数  放送規格   局発     MER/CN  BER     給電     測定ON/OFF
            {BAND_TAB_UHF,          {false,  false,  false,   false,  false,  false,  false,  false}},
            {BAND_TAB_BS_R,         {false,  false,  false,   false,  false,  false,  false,  false}},
            {BAND_TAB_BS_L,         {false,  false,  false,   false,  false,  false,  false,  false}},
            {BAND_TAB_CS_R,         {false,  false,  false,   false,  false,  false,  false,  false}},
            {BAND_TAB_CS_L,         {false,  false,  false,   false,  false,  false,  false,  false}},
            {BAND_TAB_CATV_ALL,     {false,  false,  true,    false,  false,  false,  false,  false}},
            {BAND_TAB_SKY_PERFECTV, {false,  false,  false,   false,  false,  false,  false,  false}},
            {BAND_TAB_CATV_UPSTREAM,{false,  true,   false,   false,  false,  false,  false,  false}},
            {BAND_TAB_JCSAT3A_H,    {false,  false,  false,   true,   false,  false,  false,  false}},
            {BAND_TAB_JCSAT3A_V,    {false,  false,  false,   true,   false,  false,  false,  false}},
            {BAND_TAB_JCSAT4B_H,    {false,  false,  false,   true,   false,  false,  false,  false}},
            {BAND_TAB_JCSAT4B_V,    {false,  false,  false,   true,   false,  false,  false,  false}},
            {BAND_TAB_SCCB2_H,      {false,  false,  false,   true,   false,  false,  false,  false}},
            {BAND_TAB_SCCB2_V,      {false,  false,  false,   true,   false,  false,  false,  false}},
            {BAND_TAB_SCCC2_H,      {false,  false,  false,   true,   false,  false,  false,  false}},
            {BAND_TAB_SCCC2_V,      {false,  false,  false,   true,   false,  false,  false,  false}},
            {BAND_TAB_LTE,          {false,  false,  false,   false,  false,  false,  false,  false}},
            {BAND_TAB_FM,           {false,  true,   false,   false,  false,  false,  false,  false}},
            {BAND_TAB_PILOT,        {false,  false,  false,   false,  false,  false,  false,  false}},
        };

        return userChannelListColorMap.value(static_cast<int>(bandId), {false, false, false, false, false, false, false, false});
    }
}

int UserDataLoggerSettingViewModel::countFilesRecursive(QString dirName)
{
    int count = 0;
    QDirIterator it(dirName, QDir::Files, QDirIterator::Subdirectories);
    while(it.hasNext()) {
        if (it.next() > 0) {
            count++;
        }
    }

    return count;
}

void UserDataLoggerSettingViewModel::undo()
{
    // 編集前の表示に戻す
    _bandModel->initialize();
    showChannelList(true);
}
